@@ -28,6 +28,9 @@
 
 #include <qmlprivategate.h>
 
+#include <QtQuick/private/qquickitem_p.h>
+#include <QtQuick/private/qquickshadereffectsource_p.h>
+
 #include <QQmlProperty>
 #include <QQmlExpression>
 #include <QQuickView>
@@ -41,6 +44,7 @@ namespace QmlDesigner {
 namespace Internal {
 
 bool QuickItemNodeInstance::s_createEffectItem = false;
+bool QuickItemNodeInstance::s_unifiedRenderPath = false;
 
 QuickItemNodeInstance::QuickItemNodeInstance(QQuickItem *item)
    : ObjectNodeInstance(item),
@@ -58,8 +62,15 @@ QuickItemNodeInstance::QuickItemNodeInstance(QQuickItem *item)
 
 QuickItemNodeInstance::~QuickItemNodeInstance()
 {
-    if (quickItem())
-        designerSupport()->derefFromEffectItem(quickItem());
+}
+
+void QuickItemNodeInstance::handleObjectDeletion(QObject *object)
+{
+    auto item = qobject_cast<QQuickItem *>(object);
+    if (item && checkIfRefFromEffect(instanceId()))
+        designerSupport()->derefFromEffectItem(item);
+
+    ObjectNodeInstance::handleObjectDeletion(object);
 }
 
 static bool isContentItem(QQuickItem *item, NodeInstanceServer *nodeInstanceServer)
@@ -95,7 +106,7 @@ QTransform QuickItemNodeInstance::transform() const
 QObject *QuickItemNodeInstance::parent() const
 {
     if (!quickItem() || !quickItem()->parentItem())
-        return 0;
+        return nullptr;
 
     return quickItem()->parentItem();
 }
@@ -156,22 +167,40 @@ void QuickItemNodeInstance::createEffectItem(bool createEffectItem)
     s_createEffectItem = createEffectItem;
 }
 
-void QuickItemNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNodeInstance)
+void QuickItemNodeInstance::enableUnifiedRenderPath(bool unifiedRenderPath)
+{
+    s_unifiedRenderPath = unifiedRenderPath;
+}
+
+bool QuickItemNodeInstance::checkIfRefFromEffect(qint32 id)
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    if (s_unifiedRenderPath)
+        return false;
+
+    return (s_createEffectItem || id == 0);
+#else
+    Q_UNUSED(id)
+    return false;
+#endif
+}
+
+void QuickItemNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNodeInstance,
+                                       InstanceContainer::NodeFlags flags)
 {
 
-    if (instanceId() == 0) {
-        DesignerSupport::setRootItem(nodeInstanceServer()->quickView(), quickItem());
-    } else {
-        quickItem()->setParentItem(qobject_cast<QQuickItem*>(nodeInstanceServer()->quickView()->rootObject()));
+    if (instanceId() == 0)
+        nodeInstanceServer()->setRootItem(quickItem());
+    else
+        quickItem()->setParentItem(nodeInstanceServer()->rootItem());
+
+    if (quickItem()->window() && checkIfRefFromEffect(instanceId())) {
+        designerSupport()->refFromEffectItem(quickItem(),
+                                             !flags.testFlag(
+                                                 InstanceContainer::ParentTakesOverRendering));
     }
 
-    if (quickItem()->window()) {
-        if (s_createEffectItem || instanceId() == 0)
-            designerSupport()->refFromEffectItem(quickItem());
-    }
-
-    ObjectNodeInstance::initialize(objectNodeInstance);
-    quickItem()->update();
+    ObjectNodeInstance::initialize(objectNodeInstance, flags);
 }
 
 QQuickItem *QuickItemNodeInstance::contentItem() const
@@ -191,14 +220,14 @@ void QuickItemNodeInstance::doComponentComplete()
 {
     ObjectNodeInstance::doComponentComplete();
 
+    QmlPrivateGate::disableTextCursor(quickItem());
+
+    QmlPrivateGate::emitComponentComplete(quickItem());
+
     QQmlProperty contentItemProperty(quickItem(), "contentItem", engine());
     if (contentItemProperty.isValid())
         m_contentItem = contentItemProperty.read().value<QQuickItem*>();
 
-    QmlPrivateGate::disableTextCursor(quickItem());
-
-    DesignerSupport::emitComponentCompleteSignalForAttachedProperty(quickItem());
-
     quickItem()->update();
 }
 
@@ -230,6 +259,66 @@ QList<QQuickItem *> QuickItemNodeInstance::allItemsRecursive() const
     return itemList;
 }
 
+QStringList QuickItemNodeInstance::allStates() const
+{
+    QStringList list;
+
+    QList<QObject*> stateList = DesignerSupport::statesForItem(quickItem());
+    for (QObject *state : stateList) {
+        QQmlProperty property(state, "name");
+        if (property.isValid())
+            list.append(property.read().toString());
+    }
+
+    return list;
+}
+
+void QuickItemNodeInstance::updateDirtyNode(QQuickItem *item)
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    if (s_unifiedRenderPath)
+        return;
+    DesignerSupport::updateDirtyNode(item);
+#else
+    Q_UNUSED(item)
+#endif
+}
+
+bool QuickItemNodeInstance::unifiedRenderPath()
+{
+    return s_unifiedRenderPath;
+}
+
+bool QuickItemNodeInstance::unifiedRenderPathOrQt6()
+{
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    return true;
+#else
+    return s_unifiedRenderPath;
+#endif
+}
+
+void QuickItemNodeInstance::setHiddenInEditor(bool hide)
+{
+    ObjectNodeInstance::setHiddenInEditor(hide);
+    if (s_unifiedRenderPath && !nodeInstanceServer()->isInformationServer()) {
+        QQmlProperty property(object(), "visible", context());
+
+        if (!property.isValid())
+            return;
+
+        bool visible = property.read().toBool();
+
+        if (hide && visible) {
+            setPropertyVariant("visible", false);
+            m_hidden = true;
+        } else if (!hide && !visible && m_hidden) {
+            setPropertyVariant("visible", true);
+            m_hidden = false;
+        }
+    }
+}
+
 QRectF QuickItemNodeInstance::contentItemBoundingBox() const
 {
     if (contentItem()) {
@@ -240,11 +329,23 @@ QRectF QuickItemNodeInstance::contentItemBoundingBox() const
     return QRectF();
 }
 
+static bool layerEnabledAndEffect(QQuickItem *item)
+{
+    QQuickItemPrivate *pItem = QQuickItemPrivate::get(item);
+
+    if (pItem && pItem->layer() && pItem->layer()->enabled() && pItem->layer()->effect())
+        return true;
+
+    return false;
+}
+
 QRectF QuickItemNodeInstance::boundingRect() const
 {
     if (quickItem()) {
         if (quickItem()->clip()) {
             return quickItem()->boundingRect();
+        } else if (layerEnabledAndEffect(quickItem())) {
+            return ServerNodeInstance::effectAdjustedBoundingRect(quickItem());
         } else {
             return boundingRectWithStepChilds(quickItem());
         }
@@ -362,17 +463,44 @@ double QuickItemNodeInstance::y() const
 
 QImage QuickItemNodeInstance::renderImage() const
 {
+    if (s_unifiedRenderPath && !isRootNodeInstance())
+        return {};
+
     updateDirtyNodesRecursive(quickItem());
 
     QRectF renderBoundingRect = boundingRect();
+    QImage renderImage;
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QSize size = renderBoundingRect.size().toSize();
     static double devicePixelRatio = qgetenv("FORMEDITOR_DEVICE_PIXEL_RATIO").toDouble();
+    if (size.width() * size.height() > 4000 * 4000)
+        size = QSize(0,0);
     size *= devicePixelRatio;
 
-    QImage renderImage = designerSupport()->renderImageForItem(quickItem(), renderBoundingRect, size);
+    if (s_unifiedRenderPath) {
+        renderImage = nodeInstanceServer()->quickWindow()->grabWindow();
+    } else {
+        // Fake render loop signaling to update things like QML items as 3D textures
+        nodeInstanceServer()->quickWindow()->beforeSynchronizing();
+        nodeInstanceServer()->quickWindow()->beforeRendering();
 
+        renderImage = designerSupport()->renderImageForItem(quickItem(), renderBoundingRect, size);
+
+        nodeInstanceServer()->quickWindow()->afterRendering();
+    }
     renderImage.setDevicePixelRatio(devicePixelRatio);
+#else
+    if (s_unifiedRenderPath) {
+        renderImage = nodeInstanceServer()->grabWindow();
+        renderImage = renderImage.copy(renderBoundingRect.toRect());
+    } else {
+        renderImage = nodeInstanceServer()->grabItem(quickItem());
+    }
+
+    /* When grabbing an offscren window the device pixel ratio is 1 */
+    renderImage.setDevicePixelRatio(1);
+#endif
 
     return renderImage;
 }
@@ -382,10 +510,34 @@ QImage QuickItemNodeInstance::renderPreviewImage(const QSize &previewImageSize)
     QRectF previewItemBoundingRect = boundingRect();
 
     if (previewItemBoundingRect.isValid() && quickItem()) {
+        static double devicePixelRatio = qgetenv("FORMEDITOR_DEVICE_PIXEL_RATIO").toDouble();
+        const QSize size = previewImageSize * devicePixelRatio;
         if (quickItem()->isVisible()) {
-            return designerSupport()->renderImageForItem(quickItem(), previewItemBoundingRect, previewImageSize);
+            QImage image;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+            if (s_unifiedRenderPath) {
+                image = nodeInstanceServer()->quickWindow()->grabWindow();
+            } else {
+                // Fake render loop signaling to update things like QML items as 3D textures
+                nodeInstanceServer()->quickWindow()->beforeSynchronizing();
+                nodeInstanceServer()->quickWindow()->beforeRendering();
+
+                image = designerSupport()->renderImageForItem(quickItem(),
+                                                              previewItemBoundingRect,
+                                                              size);
+
+                nodeInstanceServer()->quickWindow()->afterRendering();
+            }
+#else
+            image = nodeInstanceServer()->grabWindow();
+            image = image.copy(previewItemBoundingRect.toRect());
+#endif
+
+            image = image.scaledToWidth(size.width());
+
+            return image;
         } else {
-            QImage transparentImage(previewImageSize, QImage::Format_ARGB32_Premultiplied);
+            QImage transparentImage(size, QImage::Format_ARGB32_Premultiplied);
             transparentImage.fill(Qt::transparent);
             return transparentImage;
         }
@@ -394,6 +546,11 @@ QImage QuickItemNodeInstance::renderPreviewImage(const QSize &previewImageSize)
     return QImage();
 }
 
+QSharedPointer<QQuickItemGrabResult> QuickItemNodeInstance::createGrabResult() const
+{
+    return quickItem()->grabToImage(size().toSize());
+}
+
 void QuickItemNodeInstance::updateAllDirtyNodesRecursive()
 {
     updateAllDirtyNodesRecursive(quickItem());
@@ -453,15 +610,30 @@ void QuickItemNodeInstance::updateDirtyNodesRecursive(QQuickItem *parentItem) co
     }
 
     QmlPrivateGate::disableNativeTextRendering(parentItem);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     DesignerSupport::updateDirtyNode(parentItem);
+#endif
 }
 
 void QuickItemNodeInstance::updateAllDirtyNodesRecursive(QQuickItem *parentItem) const
 {
-    foreach (QQuickItem *childItem, parentItem->childItems())
-            updateAllDirtyNodesRecursive(childItem);
+    const QList<QQuickItem *> children = parentItem->childItems();
+    for (QQuickItem *childItem : children)
+        updateAllDirtyNodesRecursive(childItem);
 
-    DesignerSupport::updateDirtyNode(parentItem);
+    updateDirtyNode(parentItem);
+}
+
+void QuickItemNodeInstance::setAllNodesDirtyRecursive(QQuickItem *parentItem) const
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    Q_UNUSED(parentItem)
+#else
+    const QList<QQuickItem *> children = parentItem->childItems();
+    for (QQuickItem *childItem : children)
+        setAllNodesDirtyRecursive(childItem);
+    DesignerSupport::addDirty(parentItem, QQuickDesignerSupport::Content);
+#endif
 }
 
 static inline bool isRectangleSane(const QRectF &rect)
@@ -469,20 +641,52 @@ static inline bool isRectangleSane(const QRectF &rect)
     return rect.isValid() && (rect.width() < 10000) && (rect.height() < 10000);
 }
 
+static bool isEffectItem(QQuickItem *item)
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    Q_UNUSED(item)
+    return false;
+#else
+    if (qobject_cast<QQuickShaderEffectSource *>(item))
+        return true;
+
+    const auto propName = "source";
+
+    QQmlProperty prop(item, QString::fromLatin1(propName));
+    if (!prop.isValid())
+        return false;
+
+    QQuickShaderEffectSource *source = prop.read().value<QQuickShaderEffectSource *>();
+
+    if (source && source->sourceItem()) {
+        QQuickItemPrivate *pItem = QQuickItemPrivate::get(source->sourceItem());
+
+        if (pItem && pItem->layer() && pItem->layer()->enabled() && pItem->layer()->effect())
+            return true;
+    }
+
+    return false;
+#endif
+}
+
 QRectF QuickItemNodeInstance::boundingRectWithStepChilds(QQuickItem *parentItem) const
 {
     QRectF boundingRect = parentItem->boundingRect();
 
     boundingRect = boundingRect.united(QRectF(QPointF(0, 0), size()));
 
-    foreach (QQuickItem *childItem, parentItem->childItems()) {
-        if (!nodeInstanceServer()->hasInstanceForObject(childItem)) {
-            QRectF transformedRect = childItem->mapRectToItem(parentItem, boundingRectWithStepChilds(childItem));
+    for (QQuickItem *childItem : parentItem->childItems()) {
+        if (!nodeInstanceServer()->hasInstanceForObject(childItem) && !isEffectItem(childItem)) {
+            QRectF transformedRect = childItem->mapRectToItem(parentItem,
+                                                              boundingRectWithStepChilds(childItem));
             if (isRectangleSane(transformedRect))
                 boundingRect = boundingRect.united(transformedRect);
         }
     }
 
+    if (boundingRect.isEmpty())
+        QRectF{0, 0, 640, 480};
+
     return boundingRect;
 }
 
@@ -575,8 +779,13 @@ void QuickItemNodeInstance::reparent(const ObjectNodeInstance::Pointer &oldParen
         setMovable(true);
     }
 
+    markRepeaterParentDirty();
+
     ObjectNodeInstance::reparent(oldParentInstance, oldParentProperty, newParentInstance, newParentProperty);
 
+    if (!newParentInstance)
+        quickItem()->setParentItem(nullptr);
+
     if (instanceIsValidLayoutable(newParentInstance, newParentProperty)) {
         setInLayoutable(true);
         setMovable(false);
@@ -592,7 +801,8 @@ void QuickItemNodeInstance::reparent(const ObjectNodeInstance::Pointer &oldParen
 
     if (quickItem()->parentItem()) {
         refresh();
-        DesignerSupport::updateDirtyNode(quickItem());
+
+        updateDirtyNode(quickItem());
 
         if (instanceIsValidLayoutable(oldParentInstance, oldParentProperty))
             oldParentInstance->refreshLayoutable();
@@ -607,8 +817,8 @@ void QuickItemNodeInstance::setPropertyVariant(const PropertyName &name, const Q
     if (ignoredProperties().contains(name))
         return;
 
-    if (name == "state")
-        return; // states are only set by us
+    if (name == "state" && isRootNodeInstance())
+        return; // states on the root item are only set by us
 
     if (name == "height") {
         m_height = value.toDouble();
@@ -632,6 +842,11 @@ void QuickItemNodeInstance::setPropertyVariant(const PropertyName &name, const Q
     if (name == "y")
         m_y = value.toDouble();
 
+    if (name == "layer.enabled" || name == "layer.effect")
+        setAllNodesDirtyRecursive(quickItem());
+
+    markRepeaterParentDirty();
+
     ObjectNodeInstance::setPropertyVariant(name, value);
 
     refresh();
@@ -645,12 +860,14 @@ void QuickItemNodeInstance::setPropertyBinding(const PropertyName &name, const Q
     if (ignoredProperties().contains(name))
         return;
 
-    if (name == "state")
-        return; // states are only set by us
+    if (name == "state" && isRootNodeInstance())
+        return; // states on the root item are only set by us
 
     if (name.startsWith("anchors.") && isRootNodeInstance())
         return;
 
+    markRepeaterParentDirty();
+
     ObjectNodeInstance::setPropertyBinding(name, expression);
 
     refresh();
@@ -701,6 +918,9 @@ void QuickItemNodeInstance::resetProperty(const PropertyName &name)
     if (name == "y")
         m_y = 0.0;
 
+    if (name == "layer.enabled" || name == "layer.effect")
+        setAllNodesDirtyRecursive(quickItem());
+
     DesignerSupport::resetAnchor(quickItem(), QString::fromUtf8(name));
 
     if (name == "anchors.fill") {
@@ -725,6 +945,8 @@ void QuickItemNodeInstance::resetProperty(const PropertyName &name)
         resetVertical();
     }
 
+    markRepeaterParentDirty();
+
     ObjectNodeInstance::resetProperty(name);
 
     if (isInLayoutable())
@@ -795,12 +1017,41 @@ bool QuickItemNodeInstance::isAnchoredBySibling() const
 
 QQuickItem *QuickItemNodeInstance::quickItem() const
 {
-    if (object() == 0)
-        return 0;
+    if (object() == nullptr)
+        return nullptr;
 
     return static_cast<QQuickItem*>(object());
 }
 
+void QuickItemNodeInstance::markRepeaterParentDirty() const
+{
+    const qint32 id = instanceId();
+    if (id <= 0 && !isValid())
+        return;
+
+    QQuickItem *item = quickItem();
+    if (!item)
+        return;
+
+    QQuickItem *parentItem = item->parentItem();
+    if (!parentItem)
+        return;
+
+    // If a Repeater instance was changed in any way, the parent must be marked dirty to rerender it
+    const QByteArray type("QQuickRepeater");
+    if (ServerNodeInstance::isSubclassOf(item, type))
+        DesignerSupport::addDirty(parentItem, QQuickDesignerSupport::Content);
+
+    // Repeater's parent must also be dirtied when a child of a repeater was changed
+    if (ServerNodeInstance::isSubclassOf(parentItem, type)) {
+        QQuickItem *parentsParent = parentItem->parentItem();
+        if (parentsParent)
+            DesignerSupport::addDirty(parentsParent, QQuickDesignerSupport::Content);
+    }
+}
+
+
+
 } // namespace Internal
 } // namespace QmlDesigner
 
