@@ -135,6 +135,13 @@ bool ServerNodeInstance::isSubclassOf(QObject *object, const QByteArray &superTy
     return  Internal::QmlPrivateGate::isSubclassOf(object, superTypeName);
 }
 
+QRectF ServerNodeInstance::effectAdjustedBoundingRect(QQuickItem *item)
+{
+    if (item)
+        return item->boundingRect().adjusted(-40, -40, 40, 40);
+    return {};
+}
+
 void ServerNodeInstance::setModifiedFlag(bool b)
 {
     m_nodeInstance->setModifiedFlag(b);
@@ -150,6 +157,11 @@ bool ServerNodeInstance::holdsGraphical() const
     return m_nodeInstance->isQuickItem();
 }
 
+bool ServerNodeInstance::isComponentWrap() const
+{
+    return m_nodeInstance->isComponentWrap();
+}
+
 void ServerNodeInstance::updateDirtyNodeRecursive()
 {
     m_nodeInstance->updateAllDirtyNodesRecursive();
@@ -219,6 +231,35 @@ QString static getErrorString(QQmlEngine *engine, const QString &componentPath)
     return s;
 }
 
+bool isInPathList(const QStringList &pathList, const QString &componentPath)
+{
+    if (componentPath.indexOf("qml/QtQuick/Controls") > 0)
+        return true;
+
+    return std::any_of(pathList.begin(), pathList.end(), [&](auto &&path) {
+        return componentPath.startsWith(path);
+    });
+}
+
+bool canBeCreatedAsPrimitive(const QStringList &pathList,
+                             const InstanceContainer &instanceContainer,
+                             QQmlContext *context,
+                             QObject *&object)
+{
+    if (isInPathList(pathList, instanceContainer.componentPath())) {
+        object = Internal::ObjectNodeInstance::createPrimitive(QString::fromUtf8(
+                                                                   instanceContainer.type()),
+                                                               instanceContainer.majorNumber(),
+                                                               instanceContainer.minorNumber(),
+                                                               context);
+
+        if (object)
+            return true;
+
+    }
+    return false;
+}
+
 ServerNodeInstance ServerNodeInstance::create(NodeInstanceServer *nodeInstanceServer,
                                               const InstanceContainer &instanceContainer,
                                               ComponentWrap componentWrap)
@@ -233,7 +274,9 @@ ServerNodeInstance ServerNodeInstance::create(NodeInstanceServer *nodeInstanceSe
         object = Internal::ObjectNodeInstance::createCustomParserObject(instanceContainer.nodeSource(), nodeInstanceServer->importCode(), nodeInstanceServer->context());
         if (object == nullptr)
             nodeInstanceServer->sendDebugOutput(DebugOutputCommand::ErrorType, QLatin1String("Custom parser object could not be created."), instanceContainer.instanceId());
-    } else if (!instanceContainer.componentPath().isEmpty()) {
+    } else if (!instanceContainer.componentPath().isEmpty()
+               && !canBeCreatedAsPrimitive(nodeInstanceServer->engine()->importPathList(),
+                                instanceContainer, nodeInstanceServer->context(), object)) {
         object = Internal::ObjectNodeInstance::createComponent(instanceContainer.componentPath(), nodeInstanceServer->context());
         if (object == nullptr) {
             object = Internal::ObjectNodeInstance::createPrimitive(QString::fromUtf8(instanceContainer.type()), instanceContainer.majorNumber(), instanceContainer.minorNumber(), nodeInstanceServer->context());
@@ -243,7 +286,7 @@ ServerNodeInstance ServerNodeInstance::create(NodeInstanceServer *nodeInstanceSe
                 nodeInstanceServer->sendDebugOutput(DebugOutputCommand::ErrorType, message + errors, instanceContainer.instanceId());
             }
         }
-    } else {
+    } else if (!object) {
         object = Internal::ObjectNodeInstance::createPrimitive(QString::fromUtf8(instanceContainer.type()), instanceContainer.majorNumber(), instanceContainer.minorNumber(), nodeInstanceServer->context());
         if (object == nullptr)
             nodeInstanceServer->sendDebugOutput(DebugOutputCommand::ErrorType, QLatin1String("Item could not be created."), instanceContainer.instanceId());
@@ -268,6 +311,8 @@ ServerNodeInstance ServerNodeInstance::create(NodeInstanceServer *nodeInstanceSe
 
     instance.internalInstance()->setInstanceId(instanceContainer.instanceId());
 
+    instance.internalInstance()->setComponentWrap(componentWrap == WrapAsComponent);
+
     instance.internalInstance()->initialize(instance.m_nodeInstance, instanceContainer.metaFlags());
 
     // Handle hidden state to initialize pickable state
@@ -479,7 +524,7 @@ QDebug operator <<(QDebug debug, const ServerNodeInstance &instance)
     return debug.space();
 }
 
-uint qHash(const ServerNodeInstance &instance)
+ServerNodeInstance::QHashValueType qHash(const ServerNodeInstance &instance)
 {
     return ::qHash(instance.instanceId());
 }
