@@ -25,7 +25,6 @@
 
 #include "qmlprivategate.h"
 
-#include "metaobject.h"
 #include "designercustomobjectdata.h"
 
 #include <objectnodeinstance.h>
@@ -34,6 +33,7 @@
 #include <QQuickItem>
 #include <QQmlComponent>
 #include <QFileInfo>
+#include <QProcessEnvironment>
 
 #include <private/qabstractfileengine_p.h>
 #include <private/qfsfileengine_p.h>
@@ -47,6 +47,24 @@
 #include <private/qqmldata_p.h>
 #include <private/qqmlcomponentattached_p.h>
 
+#include <private/qabstractanimation_p.h>
+#include <private/qobject_p.h>
+#include <private/qquickbehavior_p.h>
+#include <private/qquicktext_p.h>
+#include <private/qquicktextinput_p.h>
+#include <private/qquicktextedit_p.h>
+#include <private/qquicktransition_p.h>
+#include <private/qquickloader_p.h>
+
+#include <private/qquickanimation_p.h>
+#include <private/qqmlmetatype_p.h>
+#include <private/qqmltimer_p.h>
+
+#ifdef QUICK3D_MODULE
+#include <private/qquick3dobject_p.h>
+#include <private/qquick3drepeater_p.h>
+#endif
+
 namespace QmlDesigner {
 
 namespace Internal {
@@ -58,18 +76,98 @@ bool isPropertyBlackListed(const QmlDesigner::PropertyName &propertyName)
     return QQuickDesignerSupportProperties::isPropertyBlackListed(propertyName);
 }
 
-PropertyNameList allPropertyNames(QObject *object,
-                                  const PropertyName &baseName,
-                                  QObjectList *inspectedObjects)
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0)) && (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
+
+static void addToPropertyNameListIfNotBlackListed(
+    PropertyNameList *propertyNameList, const QQuickDesignerSupport::PropertyName &propertyName)
 {
-    return QQuickDesignerSupportProperties::allPropertyNames(object, baseName, inspectedObjects);
+    if (!QQuickDesignerSupportProperties::isPropertyBlackListed(propertyName))
+        propertyNameList->append(propertyName);
 }
 
-PropertyNameList propertyNameListForWritableProperties(QObject *object,
-                                                       const PropertyName &baseName,
-                                                       QObjectList *inspectedObjects)
+PropertyNameList allPropertyNamesInline(QObject *object,
+                                        const PropertyName &baseName = {},
+                                        QObjectList *inspectedObjects = nullptr,
+                                        int depth = 0)
 {
-    return QQuickDesignerSupportProperties::propertyNameListForWritableProperties(object, baseName, inspectedObjects);
+    QQuickDesignerSupport::PropertyNameList propertyNameList;
+
+    QObjectList localObjectList;
+
+    if (inspectedObjects == nullptr)
+        inspectedObjects = &localObjectList;
+
+    if (depth > 2)
+        return propertyNameList;
+
+    if (!inspectedObjects->contains(object))
+        inspectedObjects->append(object);
+
+    const QMetaObject *metaObject = object->metaObject();
+
+    QStringList deferredPropertyNames;
+    const int namesIndex = metaObject->indexOfClassInfo("DeferredPropertyNames");
+    if (namesIndex != -1) {
+        QMetaClassInfo classInfo = metaObject->classInfo(namesIndex);
+        deferredPropertyNames = QString::fromUtf8(classInfo.value()).split(QLatin1Char(','));
+    }
+
+    for (int index = 0; index < metaObject->propertyCount(); ++index) {
+        QMetaProperty metaProperty = metaObject->property(index);
+        QQmlProperty declarativeProperty(object, QString::fromUtf8(metaProperty.name()));
+        if (declarativeProperty.isValid()
+            && declarativeProperty.propertyTypeCategory() == QQmlProperty::Object) {
+            if (declarativeProperty.name() != QLatin1String("parent")
+                && !deferredPropertyNames.contains(declarativeProperty.name())) {
+                QObject *childObject = QQmlMetaType::toQObject(declarativeProperty.read());
+                if (childObject)
+                    propertyNameList.append(
+                        allPropertyNamesInline(childObject,
+                                               baseName
+                                                   + QQuickDesignerSupport::PropertyName(
+                                                       metaProperty.name())
+                                                   + '.',
+                                               inspectedObjects,
+                                               depth + 1));
+            }
+        } else if (QQmlGadgetPtrWrapper *valueType
+                   = QQmlGadgetPtrWrapper::instance(qmlEngine(object), metaProperty.userType())) {
+            valueType->setValue(metaProperty.read(object));
+            propertyNameList.append(baseName
+                                    + QQuickDesignerSupport::PropertyName(metaProperty.name()));
+            propertyNameList.append(
+                allPropertyNamesInline(valueType,
+                                       baseName
+                                           + QQuickDesignerSupport::PropertyName(metaProperty.name())
+                                           + '.',
+                                       inspectedObjects,
+                                       depth + 1));
+        } else {
+            addToPropertyNameListIfNotBlackListed(&propertyNameList,
+                                                  baseName
+                                                      + QQuickDesignerSupport::PropertyName(
+                                                          metaProperty.name()));
+        }
+    }
+
+    return propertyNameList;
+}
+#endif
+
+PropertyNameList allPropertyNames(QObject *object)
+{
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    return QQuickDesignerSupportProperties::allPropertyNames(object);
+#elif QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+    return allPropertyNamesInline(object);
+#else
+    return QQuickDesignerSupportProperties::allPropertyNames(object);
+#endif
+}
+
+PropertyNameList propertyNameListForWritableProperties(QObject *object)
+{
+    return QQuickDesignerSupportProperties::propertyNameListForWritableProperties(object);
 }
 
 void tweakObjects(QObject *object)
@@ -87,9 +185,38 @@ void registerNodeInstanceMetaObject(QObject *object, QQmlEngine *engine)
     QQuickDesignerSupportProperties::registerNodeInstanceMetaObject(object, engine);
 }
 
+static bool isQuickStyleItemMetaObject(const QMetaObject *metaObject)
+{
+    if (metaObject) {
+        if (metaObject->className() == QByteArrayLiteral("QQuickStyleItem"))
+            return true;
+
+        return isQuickStyleItemMetaObject(metaObject->superClass());
+    }
+
+    return false;
+}
+
+static bool isQuickStyleItem(QObject *object)
+{
+    if (object)
+        return isQuickStyleItemMetaObject(object->metaObject());
+
+    return false;
+}
+
+// This is used in share/qtcreator/qml/qmlpuppet/qml2puppet/instances/objectnodeinstance.cpp
 QObject *createPrimitive(const QString &typeName, int majorNumber, int minorNumber, QQmlContext *context)
 {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+
+    QTypeRevision revision = QTypeRevision::zero();
+    if (majorNumber > 0)
+        revision = QTypeRevision::fromVersion(majorNumber, minorNumber);
+    return QQuickDesignerSupportItems::createPrimitive(typeName, revision, context);
+#else
     return QQuickDesignerSupportItems::createPrimitive(typeName, majorNumber, minorNumber, context);
+#endif
 }
 
 static QString qmlDesignerRCPath()
@@ -225,13 +352,21 @@ void emitComponentComplete(QObject *item)
 
     QQmlData *data = QQmlData::get(item);
     if (data && data->context) {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         QQmlComponentAttached *componentAttached = data->context->componentAttached;
+#else
+        QQmlComponentAttached *componentAttached = data->context->componentAttacheds();
+#endif
         while (componentAttached) {
             if (componentAttached->parent())
                 if (componentAttached->parent() == item)
                     emit componentAttached->completed();
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
             componentAttached = componentAttached->next;
+#else
+            componentAttached = componentAttached->next();
+#endif
         }
     }
 }
@@ -243,6 +378,11 @@ void doComponentCompleteRecursive(QObject *object, NodeInstanceServer *nodeInsta
 
         if (item && DesignerSupport::isComponentComplete(item))
             return;
+#ifdef QUICK3D_MODULE
+        auto obj3d = qobject_cast<QQuick3DRepeater *>(object);
+        if (obj3d && QQuick3DObjectPrivate::get(obj3d)->componentComplete)
+            return;
+#endif
 
         if (!nodeInstanceServer->hasInstanceForObject(item))
             emitComponentComplete(object);
@@ -260,12 +400,21 @@ void doComponentCompleteRecursive(QObject *object, NodeInstanceServer *nodeInsta
                 doComponentCompleteRecursive(child, nodeInstanceServer);
         }
 
-        if (item) {
-            static_cast<QQmlParserStatus*>(item)->componentComplete();
-        } else {
-            QQmlParserStatus *qmlParserStatus = dynamic_cast< QQmlParserStatus*>(object);
-            if (qmlParserStatus)
-                qmlParserStatus->componentComplete();
+        if (!isQuickStyleItem(item)) {
+            if (item) {
+                static_cast<QQmlParserStatus *>(item)->componentComplete();
+            } else {
+                QQmlParserStatus *qmlParserStatus = dynamic_cast<QQmlParserStatus *>(object);
+                if (qmlParserStatus) {
+                    qmlParserStatus->componentComplete();
+                    auto *anim = dynamic_cast<QQuickAbstractAnimation *>(object);
+                    if (anim && ViewConfig::isParticleViewMode()) {
+                        nodeInstanceServer->addAnimation(anim);
+                        anim->setEnableUserControl();
+                        anim->stop();
+                    }
+                }
+            }
         }
     }
 }
@@ -428,7 +577,7 @@ ComponentCompleteDisabler::~ComponentCompleteDisabler()
 class QrcEngineHandler : public QAbstractFileEngineHandler
 {
 public:
-    QAbstractFileEngine *create(const QString &fileName) const;
+    QAbstractFileEngine *create(const QString &fileName) const final;
 };
 
 QAbstractFileEngine *QrcEngineHandler::create(const QString &fileName) const
