@@ -64,8 +64,8 @@ Lexer::Lexer(TranslationUnit *unit)
 }
 
 Lexer::Lexer(const char *firstChar, const char *lastChar)
-    : _translationUnit(0),
-      _control(0),
+    : _translationUnit(nullptr),
+      _control(nullptr),
       _state(0),
       _flags(0),
       _currentLine(1)
@@ -211,7 +211,7 @@ void Lexer::scan_helper(Token *tok)
         _state = 0;
         scanCppComment(originalKind);
         return;
-    } else if (isRawStringLiteral(s._tokenKind)) {
+    } else if (!control() && isRawStringLiteral(s._tokenKind)) {
         tok->f.kind = s._tokenKind;
         if (scanUntilRawStringLiteralEndSimple())
             _state = 0;
@@ -747,20 +747,25 @@ void Lexer::scanRawStringLiteral(Token *tok, unsigned char hint)
     const char *yytext = _currentChar;
 
     int delimLength = -1;
-    const char *closingDelimCandidate = 0;
+    const char *closingDelimCandidate = nullptr;
+    bool closed = false;
     while (_yychar) {
         if (_yychar == '(' && delimLength == -1) {
             delimLength = _currentChar - yytext;
             yyinp();
         } else if (_yychar == ')') {
             yyinp();
-            if (delimLength == -1)
-                break;
+            if (delimLength == -1) {
+                tok->f.kind = T_ERROR;
+                return;
+            }
             closingDelimCandidate = _currentChar;
         } else {
             if (delimLength == -1) {
-                if (_yychar == '\\' || std::isspace(_yychar))
-                    break;
+                if (_yychar == '\\' || std::isspace(_yychar)) {
+                    tok->f.kind = T_ERROR;
+                    return;
+                }
                 yyinp();
             } else {
                 if (!closingDelimCandidate) {
@@ -769,13 +774,14 @@ void Lexer::scanRawStringLiteral(Token *tok, unsigned char hint)
                     if (_yychar == '"') {
                         if (delimLength == _currentChar - closingDelimCandidate) {
                             // Got a matching closing delimiter.
+                            closed = true;
                             break;
                         }
                     }
 
                     // Make sure this continues to be a valid candidate.
                     if (_yychar != *(yytext + (_currentChar - closingDelimCandidate)))
-                        closingDelimCandidate = 0;
+                        closingDelimCandidate = nullptr;
 
                     yyinp();
                 }
@@ -802,7 +808,7 @@ void Lexer::scanRawStringLiteral(Token *tok, unsigned char hint)
     else
         tok->f.kind = T_RAW_STRING_LITERAL;
 
-    if (!_yychar)
+    if (!control() && !closed)
         s._tokenKind = tok->f.kind;
 }
 
@@ -920,11 +926,15 @@ bool Lexer::scanOptionalIntegerSuffix(bool allowU)
         yyinp();
         if (_yychar == 'l')
             yyinp();
+        if (_yychar == 'u' || _yychar == 'U')
+            yyinp();
         return true;
     case 'L':
         yyinp();
         if (_yychar == 'L')
             yyinp();
+        if (_yychar == 'u' || _yychar == 'U')
+            yyinp();
         return true;
     default:
         return false;
@@ -933,7 +943,7 @@ bool Lexer::scanOptionalIntegerSuffix(bool allowU)
 
 void Lexer::scanOptionalUserDefinedLiteral(Token *tok)
 {
-    if (_languageFeatures.cxx11Enabled && _yychar == '_') {
+    if (_languageFeatures.cxx11Enabled && (_yychar == '_' || std::isalpha(_yychar))) {
         tok->f.userDefinedLiteral = true;
         while (std::isalnum(_yychar) || _yychar == '_' || isByteOfMultiByteCodePoint(_yychar))
             yyinp();
@@ -948,7 +958,8 @@ void Lexer::scanNumericLiteral(Token *tok)
             yyinp();
             while (std::isdigit(_yychar) ||
                    (_yychar >= 'a' && _yychar <= 'f') ||
-                   (_yychar >= 'A' && _yychar <= 'F')) {
+                   (_yychar >= 'A' && _yychar <= 'F') ||
+                   ((_yychar == '\'') && _languageFeatures.cxx14Enabled)) {
                 yyinp();
             }
             if (!scanOptionalIntegerSuffix())
@@ -956,7 +967,8 @@ void Lexer::scanNumericLiteral(Token *tok)
             goto theEnd;
         } else if (_yychar == 'b' || _yychar == 'B') { // see n3472
             yyinp();
-            while (_yychar == '0' || _yychar == '1')
+            while (_yychar == '0' || _yychar == '1' ||
+                   ((_yychar == '\'') && _languageFeatures.cxx14Enabled))
                 yyinp();
             if (!scanOptionalIntegerSuffix())
                 scanOptionalUserDefinedLiteral(tok);
@@ -964,7 +976,8 @@ void Lexer::scanNumericLiteral(Token *tok)
         } else if (_yychar >= '0' && _yychar <= '7') {
             do {
                 yyinp();
-            } while (_yychar >= '0' && _yychar <= '7');
+            } while ((_yychar >= '0' && _yychar <= '7') ||
+                     ((_yychar == '\'') && _languageFeatures.cxx14Enabled));
             if (!scanOptionalIntegerSuffix())
                 scanOptionalUserDefinedLiteral(tok);
             goto theEnd;
@@ -983,7 +996,8 @@ void Lexer::scanNumericLiteral(Token *tok)
             if (scanExponentPart() && !scanOptionalFloatingSuffix())
                 scanOptionalUserDefinedLiteral(tok);
             break;
-        } else if (std::isdigit(_yychar)) {
+        } else if (std::isdigit(_yychar) ||
+                   ((_yychar == '\'') && _languageFeatures.cxx14Enabled)) {
             yyinp();
         } else {
             if (!scanOptionalIntegerSuffix())
@@ -1020,7 +1034,7 @@ void Lexer::scanPreprocessorNumber(Token *tok, bool dotAlreadySkipped)
             yyinp();
             if (_yychar == '+' || _yychar == '-')
                 yyinp();
-        } else if (std::isalnum(_yychar) || _yychar == '_' || _yychar == '.') {
+        } else if (std::isalnum(_yychar) || (_yychar == '\'') || _yychar == '_' || _yychar == '.') {
             yyinp();
         } else {
             scanOptionalUserDefinedLiteral(tok);
@@ -1042,10 +1056,17 @@ void Lexer::scanIdentifier(Token *tok, unsigned extraProcessedChars)
         yyinp();
     }
     int yylen = _currentChar - yytext;
-    if (f._scanKeywords)
+    if (f._scanKeywords) {
         tok->f.kind = classify(yytext, yylen, _languageFeatures);
-    else
+
+        if (tok->f.kind == T_FALSE || tok->f.kind == T_TRUE) {
+            if (control()) {
+                tok->number = control()->numericLiteral(yytext, yylen);
+            }
+        }
+    } else {
         tok->f.kind = T_IDENTIFIER;
+    }
 
     if (tok->f.kind == T_IDENTIFIER) {
         tok->f.kind = classifyOperator(yytext, yylen);
