@@ -10,8 +10,8 @@
 #include "definition_p.h"
 #include "ksyntaxhighlighting_logging.h"
 #include "rule_p.h"
-#include "xml_p.h"
 #include "worddelimiters_p.h"
+#include "xml_p.h"
 
 #include <QString>
 #include <QXmlStreamReader>
@@ -31,39 +31,55 @@ static bool isOctalChar(QChar c)
 
 static bool isHexChar(QChar c)
 {
-    return isDigit(c)
-        || (c <= QLatin1Char('f') && QLatin1Char('a') <= c)
-        || (c <= QLatin1Char('F') && QLatin1Char('A') <= c);
+    return isDigit(c) || (c <= QLatin1Char('f') && QLatin1Char('a') <= c) || (c <= QLatin1Char('F') && QLatin1Char('A') <= c);
 }
 
-static int matchEscapedChar(const QString &text, int offset)
+static int matchEscapedChar(QStringView text, int offset)
 {
-    if (text.at(offset) != QLatin1Char('\\') || text.size() < offset + 2)
+    if (text.at(offset) != QLatin1Char('\\') || text.size() < offset + 2) {
         return offset;
+    }
 
     const auto c = text.at(offset + 1);
     switch (c.unicode()) {
     // control chars
-    case 'a': case 'b': case 'e': case 'f':
-    case 'n': case 'r': case 't': case 'v':
-    case '"': case '\'': case '?': case '\\':
+    case 'a':
+    case 'b':
+    case 'e':
+    case 'f':
+    case 'n':
+    case 'r':
+    case 't':
+    case 'v':
+    case '"':
+    case '\'':
+    case '?':
+    case '\\':
         return offset + 2;
 
     // hex encoded character
     case 'x':
         if (offset + 2 < text.size() && isHexChar(text.at(offset + 2))) {
-            if (offset + 3 < text.size() && isHexChar(text.at(offset + 3)))
+            if (offset + 3 < text.size() && isHexChar(text.at(offset + 3))) {
                 return offset + 4;
+            }
             return offset + 3;
         }
         return offset;
 
     // octal encoding, simple \0 is OK, too, unlike simple \x above
-    case '0': case '1': case '2': case '3':
-    case '4': case '5': case '6': case '7':
+    case '0':
+    case '1':
+    case '2':
+    case '3':
+    case '4':
+    case '5':
+    case '6':
+    case '7':
         if (offset + 2 < text.size() && isOctalChar(text.at(offset + 2))) {
-            if (offset + 3 < text.size() && isOctalChar(text.at(offset + 3)))
+            if (offset + 3 < text.size() && isOctalChar(text.at(offset + 3))) {
                 return offset + 4;
+            }
             return offset + 3;
         }
         return offset + 2;
@@ -81,6 +97,13 @@ static QString replaceCaptures(const QString &pattern, const QStringList &captur
     return result;
 }
 
+Rule::~Rule()
+{
+    if (!m_additionalDeliminator.isEmpty() || !m_weakDeliminator.isEmpty()) {
+        delete m_wordDelimiters;
+    }
+}
+
 Definition Rule::definition() const
 {
     return m_def.definition();
@@ -96,26 +119,31 @@ bool Rule::load(QXmlStreamReader &reader)
     Q_ASSERT(reader.tokenType() == QXmlStreamReader::StartElement);
 
     m_attribute = reader.attributes().value(QLatin1String("attribute")).toString();
-    if (reader.name() != QLatin1String("IncludeRules")) // IncludeRules uses this with a different semantic
+    if (reader.name() != QLatin1String("IncludeRules")) { // IncludeRules uses this with a different semantic
         m_context.parse(reader.attributes().value(QLatin1String("context")));
+    }
     m_firstNonSpace = Xml::attrToBool(reader.attributes().value(QLatin1String("firstNonSpace")));
     m_lookAhead = Xml::attrToBool(reader.attributes().value(QLatin1String("lookAhead")));
     bool colOk = false;
     m_column = reader.attributes().value(QLatin1String("column")).toInt(&colOk);
-    if (!colOk)
+    if (!colOk) {
         m_column = -1;
+    }
 
     auto regionName = reader.attributes().value(QLatin1String("beginRegion"));
-    if (!regionName.isEmpty())
+    if (!regionName.isEmpty()) {
         m_beginRegion = FoldingRegion(FoldingRegion::Begin, DefinitionData::get(m_def.definition())->foldingRegionId(regionName.toString()));
+    }
     regionName = reader.attributes().value(QLatin1String("endRegion"));
-    if (!regionName.isEmpty())
+    if (!regionName.isEmpty()) {
         m_endRegion = FoldingRegion(FoldingRegion::End, DefinitionData::get(m_def.definition())->foldingRegionId(regionName.toString()));
+    }
 
     auto result = doLoad(reader);
 
-    if (m_lookAhead && m_context.isStay())
+    if (m_lookAhead && m_context.isStay()) {
         result = false;
+    }
 
     // be done with this rule, skip all subelements, e.g. no longer supported sub-rules
     reader.skipCurrentElement();
@@ -124,12 +152,17 @@ bool Rule::load(QXmlStreamReader &reader)
 
 void Rule::resolveContext()
 {
-    auto const& def = m_def.definition();
+    auto const &def = m_def.definition();
 
     m_context.resolve(def);
 
     // cache for DefinitionData::wordDelimiters, is accessed VERY often
     m_wordDelimiters = &DefinitionData::get(def)->wordDelimiters;
+    if (!m_additionalDeliminator.isEmpty() || !m_weakDeliminator.isEmpty()) {
+        m_wordDelimiters = new WordDelimiters(*m_wordDelimiters);
+        m_wordDelimiters->append(m_additionalDeliminator);
+        m_wordDelimiters->remove(m_weakDeliminator);
+    }
 }
 
 void Rule::resolveAttributeFormat(Context *lookupContext)
@@ -151,44 +184,68 @@ bool Rule::doLoad(QXmlStreamReader &reader)
     return true;
 }
 
-Rule::Ptr Rule::create(const QStringView &name)
+void Rule::loadAdditionalWordDelimiters(QXmlStreamReader &reader)
 {
-    if (name == QLatin1String("AnyChar"))
+    m_additionalDeliminator = reader.attributes().value(QLatin1String("additionalDeliminator")).toString();
+    m_weakDeliminator = reader.attributes().value(QLatin1String("weakDeliminator")).toString();
+}
+
+Rule::Ptr Rule::create(QStringView name)
+{
+    if (name == QLatin1String("AnyChar")) {
         return std::make_shared<AnyChar>();
-    if (name == QLatin1String("DetectChar"))
+    }
+    if (name == QLatin1String("DetectChar")) {
         return std::make_shared<DetectChar>();
-    if (name == QLatin1String("Detect2Chars"))
+    }
+    if (name == QLatin1String("Detect2Chars")) {
         return std::make_shared<Detect2Char>();
-    if (name == QLatin1String("DetectIdentifier"))
+    }
+    if (name == QLatin1String("DetectIdentifier")) {
         return std::make_shared<DetectIdentifier>();
-    if (name == QLatin1String("DetectSpaces"))
+    }
+    if (name == QLatin1String("DetectSpaces")) {
         return std::make_shared<DetectSpaces>();
-    if (name == QLatin1String("Float"))
+    }
+    if (name == QLatin1String("Float")) {
         return std::make_shared<Float>();
-    if (name == QLatin1String("Int"))
+    }
+    if (name == QLatin1String("Int")) {
         return std::make_shared<Int>();
-    if (name == QLatin1String("HlCChar"))
+    }
+    if (name == QLatin1String("HlCChar")) {
         return std::make_shared<HlCChar>();
-    if (name == QLatin1String("HlCHex"))
+    }
+    if (name == QLatin1String("HlCHex")) {
         return std::make_shared<HlCHex>();
-    if (name == QLatin1String("HlCOct"))
+    }
+    if (name == QLatin1String("HlCOct")) {
         return std::make_shared<HlCOct>();
-    if (name == QLatin1String("HlCStringChar"))
+    }
+    if (name == QLatin1String("HlCStringChar")) {
         return std::make_shared<HlCStringChar>();
-    if (name == QLatin1String("IncludeRules"))
+    }
+    if (name == QLatin1String("IncludeRules")) {
         return std::make_shared<IncludeRules>();
-    if (name == QLatin1String("keyword"))
+    }
+    if (name == QLatin1String("keyword")) {
         return std::make_shared<KeywordListRule>();
-    if (name == QLatin1String("LineContinue"))
+    }
+    if (name == QLatin1String("LineContinue")) {
         return std::make_shared<LineContinue>();
-    if (name == QLatin1String("RangeDetect"))
+    }
+    if (name == QLatin1String("RangeDetect")) {
         return std::make_shared<RangeDetect>();
-    if (name == QLatin1String("RegExpr"))
+    }
+    if (name == QLatin1String("RegExpr")) {
         return std::make_shared<RegExpr>();
-    if (name == QLatin1String("StringDetect"))
+    }
+    if (name == QLatin1String("StringDetect")) {
         return std::make_shared<StringDetect>();
-    if (name == QLatin1String("WordDetect"))
+    }
+    if (name == QLatin1String("WordDetect")) {
         return std::make_shared<WordDetect>();
+    }
 
     qCWarning(Log) << "Unknown rule type:" << name;
     return Ptr(nullptr);
@@ -202,23 +259,26 @@ bool Rule::isWordDelimiter(QChar c) const
 bool AnyChar::doLoad(QXmlStreamReader &reader)
 {
     m_chars = reader.attributes().value(QLatin1String("String")).toString();
-    if (m_chars.size() == 1)
+    if (m_chars.size() == 1) {
         qCDebug(Log) << "AnyChar rule with just one char: use DetectChar instead.";
+    }
     return !m_chars.isEmpty();
 }
 
-MatchResult AnyChar::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult AnyChar::doMatch(QStringView text, int offset, const QStringList &) const
 {
-    if (m_chars.contains(text.at(offset)))
+    if (m_chars.contains(text.at(offset))) {
         return offset + 1;
+    }
     return offset;
 }
 
 bool DetectChar::doLoad(QXmlStreamReader &reader)
 {
     const auto s = reader.attributes().value(QLatin1String("char"));
-    if (s.isEmpty())
+    if (s.isEmpty()) {
         return false;
+    }
     m_char = s.at(0);
     m_dynamic = Xml::attrToBool(reader.attributes().value(QLatin1String("dynamic")));
     if (m_dynamic) {
@@ -227,18 +287,21 @@ bool DetectChar::doLoad(QXmlStreamReader &reader)
     return true;
 }
 
-MatchResult DetectChar::doMatch(const QString &text, int offset, const QStringList &captures) const
+MatchResult DetectChar::doMatch(QStringView text, int offset, const QStringList &captures) const
 {
     if (m_dynamic) {
-        if (m_captureIndex == 0 || captures.size() <= m_captureIndex || captures.at(m_captureIndex).isEmpty())
+        if (m_captureIndex == 0 || captures.size() <= m_captureIndex || captures.at(m_captureIndex).isEmpty()) {
             return offset;
-        if (text.at(offset) == captures.at(m_captureIndex).at(0))
+        }
+        if (text.at(offset) == captures.at(m_captureIndex).at(0)) {
             return offset + 1;
+        }
         return offset;
     }
 
-    if (text.at(offset) == m_char)
+    if (text.at(offset) == m_char) {
         return offset + 1;
+    }
     return offset;
 }
 
@@ -246,149 +309,196 @@ bool Detect2Char::doLoad(QXmlStreamReader &reader)
 {
     const auto s1 = reader.attributes().value(QLatin1String("char"));
     const auto s2 = reader.attributes().value(QLatin1String("char1"));
-    if (s1.isEmpty() || s2.isEmpty())
+    if (s1.isEmpty() || s2.isEmpty()) {
         return false;
+    }
     m_char1 = s1.at(0);
     m_char2 = s2.at(0);
     return true;
 }
 
-MatchResult Detect2Char::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult Detect2Char::doMatch(QStringView text, int offset, const QStringList &) const
 {
-    if (text.size() - offset < 2)
+    if (text.size() - offset < 2) {
         return offset;
-    if (text.at(offset) == m_char1 && text.at(offset + 1) == m_char2)
+    }
+    if (text.at(offset) == m_char1 && text.at(offset + 1) == m_char2) {
         return offset + 2;
+    }
     return offset;
 }
 
-MatchResult DetectIdentifier::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult DetectIdentifier::doMatch(QStringView text, int offset, const QStringList &) const
 {
-    if (!text.at(offset).isLetter() && text.at(offset) != QLatin1Char('_'))
+    if (!text.at(offset).isLetter() && text.at(offset) != QLatin1Char('_')) {
         return offset;
+    }
 
     for (int i = offset + 1; i < text.size(); ++i) {
         const auto c = text.at(i);
-        if (!c.isLetterOrNumber() && c != QLatin1Char('_'))
+        if (!c.isLetterOrNumber() && c != QLatin1Char('_')) {
             return i;
+        }
     }
 
     return text.size();
 }
 
-MatchResult DetectSpaces::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult DetectSpaces::doMatch(QStringView text, int offset, const QStringList &) const
 {
-    while (offset < text.size() && text.at(offset).isSpace())
+    while (offset < text.size() && text.at(offset).isSpace()) {
         ++offset;
+    }
     return offset;
 }
 
-MatchResult Float::doMatch(const QString &text, int offset, const QStringList &) const
+bool Float::doLoad(QXmlStreamReader &reader)
 {
-    if (offset > 0 && !isWordDelimiter(text.at(offset - 1)))
+    loadAdditionalWordDelimiters(reader);
+    return true;
+}
+
+MatchResult Float::doMatch(QStringView text, int offset, const QStringList &) const
+{
+    if (offset > 0 && !isWordDelimiter(text.at(offset - 1))) {
         return offset;
+    }
 
     auto newOffset = offset;
-    while (newOffset < text.size() && isDigit(text.at(newOffset)))
+    while (newOffset < text.size() && isDigit(text.at(newOffset))) {
         ++newOffset;
+    }
 
-    if (newOffset >= text.size() || text.at(newOffset) != QLatin1Char('.'))
+    if (newOffset >= text.size() || text.at(newOffset) != QLatin1Char('.')) {
         return offset;
+    }
     ++newOffset;
 
-    while (newOffset < text.size() && isDigit(text.at(newOffset)))
+    while (newOffset < text.size() && isDigit(text.at(newOffset))) {
         ++newOffset;
+    }
 
-    if (newOffset == offset + 1) // we only found a decimal point
+    if (newOffset == offset + 1) { // we only found a decimal point
         return offset;
+    }
 
     auto expOffset = newOffset;
-    if (expOffset >= text.size() || (text.at(expOffset) != QLatin1Char('e') && text.at(expOffset) != QLatin1Char('E')))
+    if (expOffset >= text.size() || (text.at(expOffset) != QLatin1Char('e') && text.at(expOffset) != QLatin1Char('E'))) {
         return newOffset;
+    }
     ++expOffset;
 
-    if (expOffset < text.size() && (text.at(expOffset) == QLatin1Char('+') || text.at(expOffset) == QLatin1Char('-')))
+    if (expOffset < text.size() && (text.at(expOffset) == QLatin1Char('+') || text.at(expOffset) == QLatin1Char('-'))) {
         ++expOffset;
+    }
     bool foundExpDigit = false;
     while (expOffset < text.size() && isDigit(text.at(expOffset))) {
         ++expOffset;
         foundExpDigit = true;
     }
 
-    if (!foundExpDigit)
+    if (!foundExpDigit) {
         return newOffset;
+    }
     return expOffset;
 }
 
-MatchResult HlCChar::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult HlCChar::doMatch(QStringView text, int offset, const QStringList &) const
 {
-    if (text.size() < offset + 3)
+    if (text.size() < offset + 3) {
         return offset;
+    }
 
-    if (text.at(offset) != QLatin1Char('\'') || text.at(offset + 1) == QLatin1Char('\''))
+    if (text.at(offset) != QLatin1Char('\'') || text.at(offset + 1) == QLatin1Char('\'')) {
         return offset;
+    }
 
     auto newOffset = matchEscapedChar(text, offset + 1);
     if (newOffset == offset + 1) {
-        if (text.at(newOffset) == QLatin1Char('\\'))
+        if (text.at(newOffset) == QLatin1Char('\\')) {
             return offset;
-        else
+        } else {
             ++newOffset;
+        }
     }
-    if (newOffset >= text.size())
+    if (newOffset >= text.size()) {
         return offset;
+    }
 
-    if (text.at(newOffset) == QLatin1Char('\''))
+    if (text.at(newOffset) == QLatin1Char('\'')) {
         return newOffset + 1;
+    }
 
     return offset;
 }
 
-MatchResult HlCHex::doMatch(const QString &text, int offset, const QStringList &) const
+bool HlCHex::doLoad(QXmlStreamReader &reader)
 {
-    if (offset > 0 && !isWordDelimiter(text.at(offset - 1)))
-        return offset;
+    loadAdditionalWordDelimiters(reader);
+    return true;
+}
 
-    if (text.size() < offset + 3)
+MatchResult HlCHex::doMatch(QStringView text, int offset, const QStringList &) const
+{
+    if (offset > 0 && !isWordDelimiter(text.at(offset - 1))) {
         return offset;
+    }
 
-    if (text.at(offset) != QLatin1Char('0') || (text.at(offset + 1) != QLatin1Char('x') && text.at(offset + 1) != QLatin1Char('X')))
+    if (text.size() < offset + 3) {
         return offset;
+    }
 
-    if (!isHexChar(text.at(offset + 2)))
+    if (text.at(offset) != QLatin1Char('0') || (text.at(offset + 1) != QLatin1Char('x') && text.at(offset + 1) != QLatin1Char('X'))) {
         return offset;
+    }
+
+    if (!isHexChar(text.at(offset + 2))) {
+        return offset;
+    }
 
     offset += 3;
-    while (offset < text.size() && isHexChar(text.at(offset)))
+    while (offset < text.size() && isHexChar(text.at(offset))) {
         ++offset;
+    }
 
     // TODO Kate matches U/L suffix, QtC does not?
 
     return offset;
 }
 
-MatchResult HlCOct::doMatch(const QString &text, int offset, const QStringList &) const
+bool HlCOct::doLoad(QXmlStreamReader &reader)
 {
-    if (offset > 0 && !isWordDelimiter(text.at(offset - 1)))
-        return offset;
+    loadAdditionalWordDelimiters(reader);
+    return true;
+}
 
-    if (text.size() < offset + 2)
+MatchResult HlCOct::doMatch(QStringView text, int offset, const QStringList &) const
+{
+    if (offset > 0 && !isWordDelimiter(text.at(offset - 1))) {
         return offset;
+    }
 
-    if (text.at(offset) != QLatin1Char('0'))
+    if (text.size() < offset + 2) {
         return offset;
+    }
 
-    if (!isOctalChar(text.at(offset + 1)))
+    if (text.at(offset) != QLatin1Char('0')) {
         return offset;
+    }
+
+    if (!isOctalChar(text.at(offset + 1))) {
+        return offset;
+    }
 
     offset += 2;
-    while (offset < text.size() && isOctalChar(text.at(offset)))
+    while (offset < text.size() && isOctalChar(text.at(offset))) {
         ++offset;
+    }
 
     return offset;
 }
 
-MatchResult HlCStringChar::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult HlCStringChar::doMatch(QStringView text, int offset, const QStringList &) const
 {
     return matchEscapedChar(text, offset);
 }
@@ -411,35 +521,41 @@ bool IncludeRules::includeAttribute() const
 bool IncludeRules::doLoad(QXmlStreamReader &reader)
 {
     const auto s = reader.attributes().value(QLatin1String("context"));
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-    const auto split = s.split(QLatin1String("##"), QString::KeepEmptyParts);
-#else
     const auto split = s.split(QString::fromLatin1("##"), Qt::KeepEmptyParts);
-#endif
-    if (split.isEmpty())
+    if (split.isEmpty()) {
         return false;
+    }
     m_contextName = split.at(0).toString();
-    if (split.size() > 1)
+    if (split.size() > 1) {
         m_defName = split.at(1).toString();
+    }
     m_includeAttribute = Xml::attrToBool(reader.attributes().value(QLatin1String("includeAttrib")));
 
     return !m_contextName.isEmpty() || !m_defName.isEmpty();
 }
 
-MatchResult IncludeRules::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult IncludeRules::doMatch(QStringView text, int offset, const QStringList &) const
 {
     Q_UNUSED(text);
     qCWarning(Log) << "Unresolved include rule for" << m_contextName << "##" << m_defName;
     return offset;
 }
 
-MatchResult Int::doMatch(const QString &text, int offset, const QStringList &) const
+bool Int::doLoad(QXmlStreamReader &reader)
 {
-    if (offset > 0 && !isWordDelimiter(text.at(offset - 1)))
-        return offset;
+    loadAdditionalWordDelimiters(reader);
+    return true;
+}
 
-    while (offset < text.size() && isDigit(text.at(offset)))
+MatchResult Int::doMatch(QStringView text, int offset, const QStringList &) const
+{
+    if (offset > 0 && !isWordDelimiter(text.at(offset - 1))) {
+        return offset;
+    }
+
+    while (offset < text.size() && isDigit(text.at(offset))) {
         ++offset;
+    }
     return offset;
 }
 
@@ -466,24 +582,29 @@ bool KeywordListRule::doLoad(QXmlStreamReader &reader)
         m_hasCaseSensitivityOverride = false;
     }
 
+    loadAdditionalWordDelimiters(reader);
+
     return !m_keywordList->isEmpty();
 }
 
-MatchResult KeywordListRule::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult KeywordListRule::doMatch(QStringView text, int offset, const QStringList &) const
 {
     auto newOffset = offset;
-    while (text.size() > newOffset && !isWordDelimiter(text.at(newOffset)))
+    while (text.size() > newOffset && !isWordDelimiter(text.at(newOffset))) {
         ++newOffset;
-    if (newOffset == offset)
+    }
+    if (newOffset == offset) {
         return offset;
+    }
 
     if (m_hasCaseSensitivityOverride) {
-        if (m_keywordList->contains(QStringView(text).mid(offset, newOffset - offset),
-                                    m_caseSensitivityOverride))
+        if (m_keywordList->contains(text.mid(offset, newOffset - offset), m_caseSensitivityOverride)) {
             return newOffset;
+        }
     } else {
-        if (m_keywordList->contains(QStringView(text).mid(offset, newOffset - offset)))
+        if (m_keywordList->contains(text.mid(offset, newOffset - offset))) {
             return newOffset;
+        }
     }
 
     // we don't match, but we can skip until newOffset as we can't start a keyword in-between
@@ -493,17 +614,19 @@ MatchResult KeywordListRule::doMatch(const QString &text, int offset, const QStr
 bool LineContinue::doLoad(QXmlStreamReader &reader)
 {
     const auto s = reader.attributes().value(QLatin1String("char"));
-    if (s.isEmpty())
+    if (s.isEmpty()) {
         m_char = QLatin1Char('\\');
-    else
+    } else {
         m_char = s.at(0);
+    }
     return true;
 }
 
-MatchResult LineContinue::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult LineContinue::doMatch(QStringView text, int offset, const QStringList &) const
 {
-    if (offset == text.size() - 1 && text.at(offset) == m_char)
+    if (offset == text.size() - 1 && text.at(offset) == m_char) {
         return offset + 1;
+    }
     return offset;
 }
 
@@ -511,24 +634,28 @@ bool RangeDetect::doLoad(QXmlStreamReader &reader)
 {
     const auto s1 = reader.attributes().value(QLatin1String("char"));
     const auto s2 = reader.attributes().value(QLatin1String("char1"));
-    if (s1.isEmpty() || s2.isEmpty())
+    if (s1.isEmpty() || s2.isEmpty()) {
         return false;
+    }
     m_begin = s1.at(0);
     m_end = s2.at(0);
     return true;
 }
 
-MatchResult RangeDetect::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult RangeDetect::doMatch(QStringView text, int offset, const QStringList &) const
 {
-    if (text.size() - offset < 2)
+    if (text.size() - offset < 2) {
         return offset;
-    if (text.at(offset) != m_begin)
+    }
+    if (text.at(offset) != m_begin) {
         return offset;
+    }
 
     auto newOffset = offset + 1;
     while (newOffset < text.size()) {
-        if (text.at(newOffset) == m_end)
+        if (text.at(newOffset) == m_end) {
             return newOffset + 1;
+        }
         ++newOffset;
     }
     return offset;
@@ -540,24 +667,58 @@ bool RegExpr::doLoad(QXmlStreamReader &reader)
 
     const auto isMinimal = Xml::attrToBool(reader.attributes().value(QLatin1String("minimal")));
     const auto isCaseInsensitive = Xml::attrToBool(reader.attributes().value(QLatin1String("insensitive")));
-    m_regexp.setPatternOptions((isMinimal ? QRegularExpression::InvertedGreedinessOption : QRegularExpression::NoPatternOption) | (isCaseInsensitive ? QRegularExpression::CaseInsensitiveOption : QRegularExpression::NoPatternOption));
+    m_regexp.setPatternOptions((isMinimal ? QRegularExpression::InvertedGreedinessOption : QRegularExpression::NoPatternOption)
+                               | (isCaseInsensitive ? QRegularExpression::CaseInsensitiveOption : QRegularExpression::NoPatternOption)
+                               // DontCaptureOption is removed by resolvePostProcessing() when necessary
+                               | QRegularExpression::DontCaptureOption
+                               // ensure Unicode support is enabled
+                               | QRegularExpression::UseUnicodePropertiesOption);
+
+    m_dynamic = Xml::attrToBool(reader.attributes().value(QLatin1String("dynamic")));
+
+    return !m_regexp.pattern().isEmpty();
+}
+
+void KSyntaxHighlighting::RegExpr::resolvePostProcessing()
+{
+    if (m_isResolved) {
+        return;
+    }
+
+    m_isResolved = true;
+    bool hasCapture = false;
+
+    // disable DontCaptureOption when reference a context with dynamic rule
+    if (auto *ctx = context().context()) {
+        for (const Rule::Ptr &rule : ctx->rules()) {
+            if (rule->isDynamic()) {
+                hasCapture = true;
+                m_regexp.setPatternOptions(m_regexp.patternOptions() & ~QRegularExpression::DontCaptureOption);
+                break;
+            }
+        }
+    }
 
     // optimize the pattern for the non-dynamic case, we use them OFTEN
-    m_dynamic = Xml::attrToBool(reader.attributes().value(QLatin1String("dynamic")));
     if (!m_dynamic) {
         m_regexp.optimize();
     }
 
-    // always using m_regexp.isValid() would be better, but parses the regexp and thus is way too expensive for release builds
+    bool isValid = m_regexp.isValid();
+    if (!isValid) {
+        // DontCaptureOption with back reference capture is an error, remove this option then try again
+        if (!hasCapture) {
+            m_regexp.setPatternOptions(m_regexp.patternOptions() & ~QRegularExpression::DontCaptureOption);
+            isValid = m_regexp.isValid();
+        }
 
-    if (Log().isDebugEnabled()) {
-        if (!m_regexp.isValid())
+        if (!isValid) {
             qCDebug(Log) << "Invalid regexp:" << m_regexp.pattern();
+        }
     }
-    return !m_regexp.pattern().isEmpty();
 }
 
-MatchResult RegExpr::doMatch(const QString &text, int offset, const QStringList &captures) const
+MatchResult RegExpr::doMatch(QStringView text, int offset, const QStringList &captures) const
 {
     /**
      * for dynamic case: create new pattern with right instantiation
@@ -567,7 +728,11 @@ MatchResult RegExpr::doMatch(const QString &text, int offset, const QStringList
     /**
      * match the pattern
      */
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 2)
+    const auto result = regexp.match(text.toString(), offset, QRegularExpression::NormalMatch, QRegularExpression::DontCheckSubjectStringMatchOption);
+#else
     const auto result = regexp.match(text, offset, QRegularExpression::NormalMatch, QRegularExpression::DontCheckSubjectStringMatchOption);
+#endif
     if (result.capturedStart() == offset) {
         /**
          * we only need to compute the captured texts if we have real capture groups
@@ -599,16 +764,16 @@ bool StringDetect::doLoad(QXmlStreamReader &reader)
     return !m_string.isEmpty();
 }
 
-MatchResult StringDetect::doMatch(const QString &text, int offset, const QStringList &captures) const
+MatchResult StringDetect::doMatch(QStringView text, int offset, const QStringList &captures) const
 {
     /**
      * for dynamic case: create new pattern with right instantiation
      */
     const auto &pattern = m_dynamic ? replaceCaptures(m_string, captures, false) : m_string;
 
-    if (offset + pattern.size() <= text.size()
-        && QStringView(text).mid(offset, pattern.size()).compare(pattern, m_caseSensitivity) == 0)
+    if (offset + pattern.size() <= text.size() && text.mid(offset, pattern.size()).compare(pattern, m_caseSensitivity) == 0) {
         return offset + pattern.size();
+    }
     return offset;
 }
 
@@ -616,26 +781,31 @@ bool WordDetect::doLoad(QXmlStreamReader &reader)
 {
     m_word = reader.attributes().value(QLatin1String("String")).toString();
     m_caseSensitivity = Xml::attrToBool(reader.attributes().value(QLatin1String("insensitive"))) ? Qt::CaseInsensitive : Qt::CaseSensitive;
+    loadAdditionalWordDelimiters(reader);
     return !m_word.isEmpty();
 }
 
-MatchResult WordDetect::doMatch(const QString &text, int offset, const QStringList &) const
+MatchResult WordDetect::doMatch(QStringView text, int offset, const QStringList &) const
 {
-    if (text.size() - offset < m_word.size())
+    if (text.size() - offset < m_word.size()) {
         return offset;
+    }
 
     /**
      * detect delimiter characters on the inner and outer boundaries of the string
      * NOTE: m_word isn't empty
      */
-    if (offset > 0 && !isWordDelimiter(text.at(offset - 1)) && !isWordDelimiter(text.at(offset)))
+    if (offset > 0 && !isWordDelimiter(text.at(offset - 1)) && !isWordDelimiter(text.at(offset))) {
         return offset;
+    }
 
-    if (QStringView(text).mid(offset, m_word.size()).compare(m_word, m_caseSensitivity) != 0)
+    if (text.mid(offset, m_word.size()).compare(m_word, m_caseSensitivity) != 0) {
         return offset;
+    }
 
-    if (text.size() == offset + m_word.size() || isWordDelimiter(text.at(offset + m_word.size())) || isWordDelimiter(text.at(offset + m_word.size() - 1)))
+    if (text.size() == offset + m_word.size() || isWordDelimiter(text.at(offset + m_word.size())) || isWordDelimiter(text.at(offset + m_word.size() - 1))) {
         return offset + m_word.size();
+    }
 
     return offset;
 }
