@@ -562,7 +562,7 @@ static const char* windowsMessageString(int messageId)
                 this,
                 &FloatingDockContainer::close);
     #else
-        setWindowFlags(Qt::Window | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint);
+        setWindowFlags(Qt::Window | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint | Qt::Tool);
         QBoxLayout *boxLayout = new QBoxLayout(QBoxLayout::TopToBottom);
         boxLayout->setContentsMargins(0, 0, 0, 0);
         boxLayout->setSpacing(0);
@@ -878,13 +878,8 @@ bool FloatingDockContainer::event(QEvent *event)
         // is clicked.
         // It is really great to work around the whole NonClientMouseArea
         // bugs
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 12, 2))
         if (event->type() == QEvent::NonClientAreaMouseButtonPress
                 /*&& QGuiApplication::mouseButtons().testFlag(Qt::LeftButton)*/)
-#else
-        if (event->type() == QEvent::NonClientAreaMouseButtonPress
-                && QGuiApplication::mouseButtons().testFlag(Qt::LeftButton))
-#endif
         {
             qCInfo(adsLog) << Q_FUNC_INFO << "QEvent::NonClientAreaMouseButtonPress"
                                           << event->type();
