@@ -28,6 +28,7 @@
 #include "clangcodemodelserverproxy.h"
 #include "lineprefixer.h"
 #include "processcreator.h"
+#include "processhandle.h"
 
 #include <QLocalServer>
 #include <QLocalSocket>
@@ -37,9 +38,7 @@
 #include <future>
 #include <memory>
 
-QT_BEGIN_NAMESPACE
-class QProcess;
-QT_END_NAMESPACE
+namespace Utils { class QtcProcess; }
 
 class Utf8String;
 class Utf8StringVector;
@@ -77,7 +76,7 @@ public:
     bool waitForEcho();
     bool waitForConnected();
 
-    QProcess *processForTestOnly();
+    Utils::QtcProcess *processForTestOnly();
 
 signals:
     void connectedToLocalSocket();
@@ -98,27 +97,27 @@ protected:
     virtual QString outputName() const = 0;
 
     QString connectionName() const;
-    bool event(QEvent* event);
+    bool event(QEvent* event) override;
 
-    virtual void newConnectedServer(QIODevice *ioDevice) = 0;
+    virtual void newConnectedServer(QLocalSocket *localSocket) = 0;
 
 private:
-    static bool isProcessRunning(QProcess *process);
+    static bool isProcessRunning(Utils::QtcProcess *process);
     void finishProcess(QProcessUniquePointer &&process);
-    void endProcess(QProcess *process);
-    void terminateProcess(QProcess *process);
-    void killProcess(QProcess *process);
+    void endProcess(Utils::QtcProcess *process);
+    void terminateProcess(Utils::QtcProcess *process);
+    void killProcess(Utils::QtcProcess *process);
     void finishConnection();
     void printLocalSocketError(QLocalSocket::LocalSocketError socketError);
     void printStandardOutput();
     void printStandardError();
-    void initializeProcess(QProcess *process);
+    void initializeProcess(Utils::QtcProcess *process);
 
     void resetTemporaryDirectory();
 
     void connectLocalSocketDisconnected();
     void disconnectLocalSocketDisconnected();
-    void connectStandardOutputAndError(QProcess *process) const;
+    void connectStandardOutputAndError(Utils::QtcProcess *process) const;
     void connectLocalSocketError() const;
     void connectAliveTimer();
     void connectNewConnection();
