@@ -47,5 +47,17 @@ QDebug operator<<(QDebug debug, const DiagnosticContainer &container)
     return debug;
 }
 
+QDebug operator<<(QDebug debug, const QVector<DiagnosticContainer> &containers)
+{
+    debug.nospace() << "{";
+    for (int i = 0; i < containers.size(); i++) {
+        debug.nospace() << containers[i];
+        if (i < containers.size() - 1)
+            debug.nospace() << ", ";
+    }
+    debug.nospace() << "}";
+    return debug;
+}
+
 } // namespace ClangBackEnd
 
