@@ -389,18 +389,22 @@ void TypePrettyPrinter::visit(Function *type)
         for (Scope *s = type->enclosingScope(); s && i >= 0; s = s->enclosingScope()) {
             if (Template *templ = s->asTemplate()) {
                 QString &n = nameParts[i];
-                n += '<';
-                for (int index = 0; index < templ->templateParameterCount(); ++index) {
-                    if (index)
-                        n += QLatin1String(", ");
-                    QString arg = _overview->prettyName(templ->templateParameterAt(index)->name());
-                    if (arg.isEmpty()) {
-                        arg += 'T';
-                        arg += QString::number(index + 1);
+                const int paramCount = templ->templateParameterCount();
+                if (paramCount > 0) {
+                    n += '<';
+                    for (int index = 0; index < paramCount; ++index) {
+                        if (index)
+                            n += QLatin1String(", ");
+                        QString arg = _overview->prettyName(
+                                    templ->templateParameterAt(index)->name());
+                        if (arg.isEmpty()) {
+                            arg += 'T';
+                            arg += QString::number(index + 1);
+                        }
+                        n += arg;
                     }
-                    n += arg;
+                    n += '>';
                 }
-                n += '>';
             }
             if (s->identifier())
                 --i;
@@ -423,8 +427,15 @@ void TypePrettyPrinter::visit(Function *type)
         _name.clear();
     }
 
+    Overview retAndArgOverview;
+    retAndArgOverview.starBindFlags = _overview->starBindFlags;
+    retAndArgOverview.showReturnTypes = true;
+    retAndArgOverview.showArgumentNames = false;
+    retAndArgOverview.showFunctionSignatures = true;
+    retAndArgOverview.showTemplateParameters = true;
+
     if (_overview->showReturnTypes) {
-        const QString returnType = _overview->prettyType(type->returnType());
+        const QString returnType = retAndArgOverview.prettyType(type->returnType());
         if (!returnType.isEmpty()) {
             if (!endsWithPtrOrRef(returnType) || !(_overview->starBindFlags & Overview::BindToIdentifier))
                 _text.prepend(QLatin1Char(' '));
@@ -435,7 +446,8 @@ void TypePrettyPrinter::visit(Function *type)
     if (_overview->showEnclosingTemplate) {
         if (Template *templ = type->enclosingTemplate()) {
             QString templateScope = "template<";
-            for (int i = 0, total = templ->templateParameterCount(); i < total; ++i) {
+            const int paramCount = templ->templateParameterCount();
+            for (int i = 0; i < paramCount; ++i) {
                 if (Symbol *param = templ->templateParameterAt(i)) {
                     if (i > 0)
                         templateScope.append(", ");
@@ -452,17 +464,12 @@ void TypePrettyPrinter::visit(Function *type)
                     }
                 }
             }
-            _text.prepend(templateScope + ">\n");
+            if (paramCount > 0)
+                _text.prepend(templateScope + ">\n");
         }
     }
 
     if (_overview->showFunctionSignatures) {
-        Overview argumentText;
-        argumentText.starBindFlags = _overview->starBindFlags;
-        argumentText.showReturnTypes = true;
-        argumentText.showArgumentNames = false;
-        argumentText.showFunctionSignatures = true;
-
         _text += QLatin1Char('(');
 
         for (int index = 0, argc = type->argumentCount(); index < argc; ++index) {
@@ -478,7 +485,7 @@ void TypePrettyPrinter::visit(Function *type)
                 if (_overview->showArgumentNames)
                     name = arg->name();
 
-                _text += argumentText.prettyType(arg->type(), name);
+                _text += retAndArgOverview.prettyType(arg->type(), name);
 
                 if (_overview->showDefaultArguments) {
                     if (const StringLiteral *initializer = arg->initializer()) {
