@@ -27,6 +27,7 @@
 
 #include "jsonkeys.h"
 #include "lsptypes.h"
+#include "semantictokens.h"
 
 namespace LanguageServerProtocol {
 
@@ -38,9 +39,84 @@ public:
     Utils::optional<bool> dynamicRegistration() const { return optionalValue<bool>(dynamicRegistrationKey); }
     void setDynamicRegistration(bool dynamicRegistration) { insert(dynamicRegistrationKey, dynamicRegistration); }
     void clearDynamicRegistration() { remove(dynamicRegistrationKey); }
+};
 
-    bool isValid(ErrorHierarchy *error) const override
-    { return checkOptional<bool>(error, dynamicRegistrationKey); }
+class LANGUAGESERVERPROTOCOL_EXPORT FullSemanticTokenOptions : public JsonObject
+{
+public:
+    using JsonObject::JsonObject;
+
+    /**
+      * The client will send the `textDocument/semanticTokens/full/delta`
+      * request if the server provides a corresponding handler.
+      */
+    Utils::optional<bool> delta() const { return optionalValue<bool>(deltaKey); }
+    void setDelta(bool delta) { insert(deltaKey, delta); }
+    void clearDelta() { remove(deltaKey); }
+};
+
+class LANGUAGESERVERPROTOCOL_EXPORT SemanticTokensClientCapabilities : public DynamicRegistrationCapabilities
+{
+public:
+    using DynamicRegistrationCapabilities::DynamicRegistrationCapabilities;
+    class LANGUAGESERVERPROTOCOL_EXPORT Requests : public JsonObject
+    {
+        /**
+         * Which requests the client supports and might send to the server
+         * depending on the server's capability. Please note that clients might not
+         * show semantic tokens or degrade some of the user experience if a range
+         * or full request is advertised by the client but not provided by the
+         * server. If for example the client capability `requests.full` and
+         * `request.range` are both set to true but the server only provides a
+         * range provider the client might not render a minimap correctly or might
+         * even decide to not show any semantic tokens at all.
+         */
+    public:
+        using JsonObject::JsonObject;
+
+        /**
+         * The client will send the `textDocument/semanticTokens/range` request
+         * if the server provides a corresponding handler.
+         */
+        Utils::optional<Utils::variant<bool, QJsonObject>> range() const;
+        void setRange(const Utils::variant<bool, QJsonObject> &range);
+        void clearRange() { remove(rangeKey); }
+
+        /**
+         * The client will send the `textDocument/semanticTokens/full` request
+         * if the server provides a corresponding handler.
+         */
+        Utils::optional<Utils::variant<bool, FullSemanticTokenOptions>> full() const;
+        void setFull(const Utils::variant<bool, FullSemanticTokenOptions> &full);
+        void clearFull() { remove(fullKey); }
+    };
+
+    Requests requests() const { return typedValue<Requests>(requestsKey); }
+    void setRequests(const Requests &requests) { insert(requestsKey, requests); }
+
+    /// The token types that the client supports.
+    QList<QString> tokenTypes() const { return array<QString>(tokenTypesKey); }
+    void setTokenTypes(const QList<QString> &value) { insertArray(tokenTypesKey, value); }
+
+    /// The token modifiers that the client supports.
+    QList<QString> tokenModifiers() const { return array<QString>(tokenModifiersKey); }
+    void setTokenModifiers(const QList<QString> &value) { insertArray(tokenModifiersKey, value); }
+
+    /// The formats the clients supports.
+    QList<QString> formats() const { return array<QString>(formatsKey); }
+    void setFormats(const QList<QString> &value) { insertArray(formatsKey, value); }
+
+    /// Whether the client supports tokens that can overlap each other.
+    Utils::optional<bool> overlappingTokenSupport() const { return optionalValue<bool>(overlappingTokenSupportKey); }
+    void setOverlappingTokenSupport(bool overlappingTokenSupport) { insert(overlappingTokenSupportKey, overlappingTokenSupport); }
+    void clearOverlappingTokenSupport() { remove(overlappingTokenSupportKey); }
+
+    /// Whether the client supports tokens that can span multiple lines.
+    Utils::optional<bool> multiLineTokenSupport() const { return optionalValue<bool>(multiLineTokenSupportKey); }
+    void setMultiLineTokenSupport(bool multiLineTokenSupport) { insert(multiLineTokenSupportKey, multiLineTokenSupport); }
+    void clearMultiLineTokenSupport() { remove(multiLineTokenSupportKey); }
+
+    bool isValid() const override;
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT SymbolCapabilities : public DynamicRegistrationCapabilities
@@ -66,9 +142,6 @@ public:
         Utils::optional<QList<SymbolKind>> valueSet() const;
         void setValueSet(const QList<SymbolKind> &valueSet);
         void clearValueSet() { remove(valueSetKey); }
-
-        bool isValid(ErrorHierarchy *error) const override
-        { return checkOptionalArray<int>(error, valueSetKey); }
     };
 
     // Specific capabilities for the `SymbolKind` in the `workspace/symbol` request.
@@ -77,7 +150,11 @@ public:
     void setSymbolKind(const SymbolKindCapabilities &symbolKind) { insert(symbolKindKey, symbolKind); }
     void clearSymbolKind() { remove(symbolKindKey); }
 
-    bool isValid(ErrorHierarchy *error) const override;
+    Utils::optional<bool> hierarchicalDocumentSymbolSupport() const
+    { return optionalValue<bool>(hierarchicalDocumentSymbolSupportKey); }
+    void setHierarchicalDocumentSymbolSupport(bool hierarchicalDocumentSymbolSupport)
+    { insert(hierarchicalDocumentSymbolSupportKey, hierarchicalDocumentSymbolSupport); }
+    void clearHierachicalDocumentSymbolSupport() { remove(hierarchicalDocumentSymbolSupportKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT TextDocumentClientCapabilities : public JsonObject
@@ -110,8 +187,6 @@ public:
         Utils::optional<bool> didSave() const { return optionalValue<bool>(didSaveKey); }
         void setDidSave(bool didSave) { insert(didSaveKey, didSave); }
         void clearDidSave() { remove(didSaveKey); }
-
-        bool isValid(ErrorHierarchy *error) const override;
     };
 
     Utils::optional<SynchronizationCapabilities> synchronization() const
@@ -129,8 +204,7 @@ public:
         void setSemanticHighlighting(bool semanticHighlighting)
         { insert(semanticHighlightingKey, semanticHighlighting); }
 
-        bool isValid(ErrorHierarchy *error) const override
-        { return check<bool>(error, semanticHighlightingKey); }
+        bool isValid() const override { return contains(semanticHighlightingKey); }
     };
 
     Utils::optional<SemanticHighlightingCapabilities> semanticHighlightingCapabilities() const
@@ -178,13 +252,6 @@ public:
             Utils::optional<QList<MarkupKind>> documentationFormat() const;
             void setDocumentationFormat(const QList<MarkupKind> &documentationFormat);
             void clearDocumentationFormat() { remove(documentationFormatKey); }
-
-            bool isValid(ErrorHierarchy *error) const override
-            {
-                return checkOptional<bool>(error, snippetSupportKey)
-                        && checkOptional<bool>(error, commitCharacterSupportKey)
-                        && checkOptionalArray<int>(error, documentationFormatKey);
-            }
         };
 
         // The client supports the following `CompletionItem` specific capabilities.
@@ -212,9 +279,6 @@ public:
             Utils::optional<QList<CompletionItemKind::Kind>> valueSet() const;
             void setValueSet(const QList<CompletionItemKind::Kind> &valueSet);
             void clearValueSet() { remove(valueSetKey); }
-
-            bool isValid(ErrorHierarchy *error) const override
-            { return checkOptionalArray<int>(error, valueSetKey); }
         };
 
         Utils::optional<CompletionItemKindCapabilities> completionItemKind() const
@@ -230,8 +294,6 @@ public:
         Utils::optional<bool> contextSupport() const { return optionalValue<bool>(contextSupportKey); }
         void setContextSupport(bool contextSupport) { insert(contextSupportKey, contextSupport); }
         void clearContextSupport() { remove(contextSupportKey); }
-
-        bool isValid(ErrorHierarchy *error) const override;
     };
 
     // Capabilities specific to the `textDocument/completion`
@@ -252,8 +314,6 @@ public:
         Utils::optional<QList<MarkupKind>> contentFormat() const;
         void setContentFormat(const QList<MarkupKind> &contentFormat);
         void clearContentFormat() { remove(contentFormatKey); }
-
-        bool isValid(ErrorHierarchy *error) const override;
     };
 
     Utils::optional<HoverCapabilities> hover() const { return optionalValue<HoverCapabilities>(hoverKey); }
@@ -277,18 +337,19 @@ public:
             void setDocumentationFormat(const QList<MarkupKind> &documentationFormat);
             void clearDocumentationFormat() { remove(documentationFormatKey); }
 
-            bool isValid(ErrorHierarchy *error) const override
-            { return checkOptionalArray<int>(error, documentationFormatKey); }
+            Utils::optional<bool> activeParameterSupport() const
+            { return optionalValue<bool>(activeParameterSupportKey); }
+            void setActiveParameterSupport(bool activeParameterSupport)
+            { insert(activeParameterSupportKey, activeParameterSupport); }
+            void clearActiveParameterSupport() { remove(activeParameterSupportKey); }
         };
 
         // The client supports the following `SignatureInformation` specific properties.
-        Utils::optional<SignatureHelpCapabilities> signatureInformation() const
-        { return optionalValue<SignatureHelpCapabilities>(signatureInformationKey); }
-        void setSignatureInformation(const SignatureHelpCapabilities &signatureInformation)
+        Utils::optional<SignatureInformationCapabilities> signatureInformation() const
+        { return optionalValue<SignatureInformationCapabilities>(signatureInformationKey); }
+        void setSignatureInformation(const SignatureInformationCapabilities &signatureInformation)
         { insert(signatureInformationKey, signatureInformation); }
         void clearSignatureInformation() { remove(signatureInformationKey); }
-
-        bool isValid(ErrorHierarchy *error) const override;
     };
 
     // Capabilities specific to the `textDocument/signatureHelp`
@@ -390,8 +451,7 @@ public:
                 void setValueSet(const QList<QString> &valueSet)
                 { insertArray(valueSetKey, valueSet); }
 
-                bool isValid(ErrorHierarchy *errorHierarchy) const override
-                { return checkArray<QString>(errorHierarchy, valueSetKey); }
+                bool isValid() const override { return contains(valueSetKey); }
             };
 
             CodeActionKind codeActionKind() const
@@ -399,8 +459,7 @@ public:
             void setCodeActionKind(const CodeActionKind &codeActionKind)
             { insert(codeActionKindKey, codeActionKind); }
 
-            bool isValid(ErrorHierarchy *errorHierarchy) const override
-            { return check<CodeActionKind>(errorHierarchy, codeActionKindKey); }
+            bool isValid() const override { return contains(codeActionKindKey); }
         };
 
         Utils::optional<CodeActionLiteralSupport> codeActionLiteralSupport() const
@@ -408,8 +467,6 @@ public:
         void setCodeActionLiteralSupport(const CodeActionLiteralSupport &codeActionLiteralSupport)
         { insert(codeActionLiteralSupportKey, codeActionLiteralSupport); }
         void clearCodeActionLiteralSupport() { remove(codeActionLiteralSupportKey); }
-
-        bool isValid(ErrorHierarchy *errorHierarchy) const override;
     };
 
     // Whether code action supports dynamic registration.
@@ -458,8 +515,6 @@ public:
         Utils::optional<bool> prepareSupport() const { return optionalValue<bool>(prepareSupportKey); }
         void setPrepareSupport(bool prepareSupport) { insert(prepareSupportKey, prepareSupport); }
         void clearPrepareSupport() { remove(prepareSupportKey); }
-
-        bool isValid(ErrorHierarchy *error) const override;
     };
 
     // Whether rename supports dynamic registration.
@@ -469,7 +524,27 @@ public:
     { insert(renameKey, rename); }
     void clearRename() { remove(renameKey); }
 
-    bool isValid(ErrorHierarchy *error) const override;
+    Utils::optional<SemanticTokensClientCapabilities> semanticTokens() const;
+    void setSemanticTokens(const SemanticTokensClientCapabilities &semanticTokens);
+    void clearSemanticTokens() { remove(semanticTokensKey); }
+};
+
+class LANGUAGESERVERPROTOCOL_EXPORT SemanticTokensWorkspaceClientCapabilities : public JsonObject
+{
+public:
+    using JsonObject::JsonObject;
+    /**
+     * Whether the client implementation supports a refresh request sent from
+     * the server to the client.
+     *
+     * Note that this event is global and will force the client to refresh all
+     * semantic tokens currently shown. It should be used with absolute care
+     * and is useful for situation where a server for example detect a project
+     * wide change that requires such a calculation.
+     */
+    Utils::optional<bool> refreshSupport() const { return optionalValue<bool>(refreshSupportKey); }
+    void setRefreshSupport(bool refreshSupport) { insert(refreshSupportKey, refreshSupport); }
+    void clearRefreshSupport() { remove(refreshSupportKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT WorkspaceClientCapabilities : public JsonObject
@@ -497,9 +572,6 @@ public:
         void setDocumentChanges(bool documentChanges)
         { insert(documentChangesKey, documentChanges); }
         void clearDocumentChanges() { remove(documentChangesKey); }
-
-        bool isValid(ErrorHierarchy *error) const override
-        { return checkOptional<bool>(error, documentChangesKey); }
     };
 
     // Capabilities specific to `WorkspaceEdit`s
@@ -548,7 +620,32 @@ public:
     void setConfiguration(bool configuration) { insert(configurationKey, configuration); }
     void clearConfiguration() { remove(configurationKey); }
 
-    bool isValid(ErrorHierarchy *error) const override;
+    Utils::optional<SemanticTokensWorkspaceClientCapabilities> semanticTokens() const
+    { return optionalValue<SemanticTokensWorkspaceClientCapabilities>(semanticTokensKey); }
+    void setSemanticTokens(const SemanticTokensWorkspaceClientCapabilities &semanticTokens)
+    { insert(semanticTokensKey, semanticTokens); }
+    void clearSemanticTokens() { remove(semanticTokensKey); }
+};
+
+class WindowClientClientCapabilities : public JsonObject
+{
+public:
+    using JsonObject::JsonObject;
+
+    /**
+      * Whether client supports handling progress notifications.
+      * If set, servers are allowed to report in `workDoneProgress` property
+      * in the request specific server capabilities.
+      *
+      */
+    Utils::optional<bool> workDoneProgress() const
+    { return optionalValue<bool>(workDoneProgressKey); }
+    void setWorkDoneProgress(bool workDoneProgress)
+    { insert(workDoneProgressKey, workDoneProgress); }
+    void clearWorkDoneProgress() { remove(workDoneProgressKey); }
+
+private:
+    constexpr static const char workDoneProgressKey[] = "workDoneProgress";
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT ClientCapabilities : public JsonObject
@@ -570,12 +667,17 @@ public:
     { insert(textDocumentKey, textDocument); }
     void clearTextDocument() { remove(textDocumentKey); }
 
+    // Window specific client capabilities.
+    Utils::optional<WindowClientClientCapabilities> window() const
+    { return optionalValue<WindowClientClientCapabilities>(windowKey); }
+    void setWindow(const WindowClientClientCapabilities &window)
+    { insert(windowKey, window); }
+    void clearWindow() { remove(windowKey); }
+
     // Experimental client capabilities.
     QJsonValue experimental() const { return value(experimentalKey); }
     void setExperimental(const QJsonValue &experimental) { insert(experimentalKey, experimental); }
     void clearExperimental() { remove(experimentalKey); }
-
-    bool isValid(ErrorHierarchy *error) const override;
 };
 
-}
+} // namespace LanguageServerProtocol
