@@ -62,19 +62,7 @@ Utils::optional<Diagnostic::Code> Diagnostic::code() const
 
 void Diagnostic::setCode(const Diagnostic::Code &code)
 {
-    if (auto val = Utils::get_if<int>(&code))
-        insert(codeKey, *val);
-    else if (auto val = Utils::get_if<QString>(&code))
-        insert(codeKey, *val);
-}
-
-bool Diagnostic::isValid(ErrorHierarchy *error) const
-{
-    return check<Range>(error, rangeKey)
-            && checkOptional<int>(error, severityKey)
-            && (checkOptional<int>(error, codeKey) || checkOptional<QString>(error, codeKey))
-            && checkOptional<QString>(error, sourceKey)
-            && check<QString>(error, messageKey);
+    insertVariant<int, QString>(codeKey, code);
 }
 
 Utils::optional<WorkspaceEdit::Changes> WorkspaceEdit::changes() const
@@ -126,23 +114,11 @@ MarkupOrString::MarkupOrString(const QJsonValue &val)
         emplace<QString>(val.toString());
     } else {
         MarkupContent markupContent(val.toObject());
-        if (markupContent.isValid(nullptr))
+        if (markupContent.isValid())
             emplace<MarkupContent>(MarkupContent(val.toObject()));
     }
 }
 
-bool MarkupOrString::isValid(ErrorHierarchy *error) const
-{
-    if (Utils::holds_alternative<MarkupContent>(*this) || Utils::holds_alternative<QString>(*this))
-        return true;
-    if (error) {
-        error->setError(
-            QCoreApplication::translate("LanguageServerProtocoll::MarkupOrString",
-                                        "Expected a string or MarkupContent in MarkupOrString."));
-    }
-    return false;
-}
-
 QJsonValue MarkupOrString::toJson() const
 {
     if (Utils::holds_alternative<QString>(*this))
@@ -152,28 +128,6 @@ QJsonValue MarkupOrString::toJson() const
     return {};
 }
 
-bool SymbolInformation::isValid(ErrorHierarchy *error) const
-{
-    return check<QString>(error, nameKey)
-            && check<int>(error, kindKey)
-            && check<Location>(error, locationKey)
-            && checkOptional<QString>(error, containerNameKey);
-}
-
-bool TextDocumentEdit::isValid(ErrorHierarchy *error) const
-{
-    return check<VersionedTextDocumentIdentifier>(error, textDocumentKey)
-            && checkArray<TextEdit>(error, editsKey);
-}
-
-bool TextDocumentItem::isValid(ErrorHierarchy *error) const
-{
-    return check<QString>(error, uriKey)
-            && check<QString>(error, languageIdKey)
-            && check<int>(error, versionKey)
-            && check<QString>(error, textKey);
-}
-
 static QHash<Utils::MimeType, QString> mimeTypeLanguageIdMap()
 {
     static QHash<Utils::MimeType, QString> hash;
@@ -284,6 +238,11 @@ QMap<QString, QString> languageIds()
     return languages;
 }
 
+bool TextDocumentItem::isValid() const
+{
+    return contains(uriKey) && contains(languageIdKey) && contains(versionKey) && contains(textKey);
+}
+
 QString TextDocumentItem::mimeTypeToLanguageId(const Utils::MimeType &mimeType)
 {
     return mimeTypeLanguageIdMap().value(mimeType);
@@ -307,12 +266,6 @@ TextDocumentPositionParams::TextDocumentPositionParams(
     setPosition(position);
 }
 
-bool TextDocumentPositionParams::isValid(ErrorHierarchy *error) const
-{
-    return check<TextDocumentIdentifier>(error, textDocumentKey)
-            && check<Position>(error, positionKey);
-}
-
 Position::Position(int line, int character)
 {
     setLine(line);
@@ -323,7 +276,7 @@ Position::Position(const QTextCursor &cursor)
     : Position(cursor.blockNumber(), cursor.positionInBlock())
 { }
 
-int Position::toPositionInDocument(QTextDocument *doc) const
+int Position::toPositionInDocument(const QTextDocument *doc) const
 {
     const QTextBlock block = doc->findBlockByNumber(line());
     if (!block.isValid())
@@ -340,6 +293,14 @@ QTextCursor Position::toTextCursor(QTextDocument *doc) const
     return cursor;
 }
 
+Position Position::withOffset(int offset, const QTextDocument *doc) const
+{
+    int line;
+    int character;
+    Utils::Text::convertPosition(doc, toPositionInDocument(doc) + offset, &line, &character);
+    return Position(line - 1, character - 1);
+}
+
 Range::Range(const Position &start, const Position &end)
 {
     setStart(start);
@@ -359,24 +320,40 @@ Range::Range(const QTextCursor &cursor)
     setEnd(Position(line - 1, character - 1));
 }
 
+bool Range::contains(const Range &other) const
+{
+    if (start() > other.start())
+        return false;
+    if (end() < other.end())
+        return false;
+    return true;
+}
+
 bool Range::overlaps(const Range &range) const
 {
-    return contains(range.start()) || contains(range.end());
+    return !isLeftOf(range) && !range.isLeftOf(*this);
+}
+
+QString expressionForGlob(QString globPattern)
+{
+    const QString anySubDir("qtc_anysubdir_id");
+    globPattern.replace("**/", anySubDir);
+    QString regexp = QRegularExpression::wildcardToRegularExpression(globPattern);
+    regexp.replace(anySubDir,"(.*[/\\\\])*");
+    regexp.replace("\\{", "(");
+    regexp.replace("\\}", ")");
+    regexp.replace(",", "|");
+    return regexp;
 }
 
 bool DocumentFilter::applies(const Utils::FilePath &fileName, const Utils::MimeType &mimeType) const
 {
-    if (Utils::optional<QString> _scheme = scheme()) {
-        if (_scheme.value() == fileName.toString())
-            return true;
-    }
     if (Utils::optional<QString> _pattern = pattern()) {
         QRegularExpression::PatternOption option = QRegularExpression::NoPatternOption;
-        if (Utils::HostOsInfo::fileNameCaseSensitivity() == Qt::CaseInsensitive)
+        if (fileName.caseSensitivity() == Qt::CaseInsensitive)
             option = QRegularExpression::CaseInsensitiveOption;
-        QRegularExpression regexp(QRegularExpression::wildcardToRegularExpression(_pattern.value()),
-                                  option);
-        if (regexp.match(fileName.toString()).hasMatch())
+        const QRegularExpression regexp(expressionForGlob(_pattern.value()), option);
+        if (regexp.isValid() && regexp.match(fileName.toString()).hasMatch())
             return true;
     }
     if (Utils::optional<QString> _lang = language()) {
@@ -391,16 +368,9 @@ bool DocumentFilter::applies(const Utils::FilePath &fileName, const Utils::MimeT
     return !contains(schemeKey) && !contains(languageKey) && !contains(patternKey);
 }
 
-bool DocumentFilter::isValid(ErrorHierarchy *error) const
-{
-    return Utils::allOf(QStringList{languageKey, schemeKey, patternKey}, [this, &error](auto key){
-        return this->checkOptional<QString>(error, key);
-    });
-}
-
 Utils::Link Location::toLink() const
 {
-    if (!isValid(nullptr))
+    if (!isValid())
         return Utils::Link();
 
     // Ensure %xx like %20 are really decoded using fromPercentEncoding
@@ -411,7 +381,9 @@ Utils::Link Location::toLink() const
     // fromPercentEncoding convert %xx encoding to raw values and then interpret
     // the result as utf-8, so toUtf8() must be used here.
     file = QUrl::fromPercentEncoding(file.toUtf8());
-    return Utils::Link(file, range().start().line() + 1, range().start().character());
+    return Utils::Link(Utils::FilePath::fromString(file),
+                       range().start().line() + 1,
+                       range().start().character());
 }
 
 DocumentUri::DocumentUri(const QString &other)
