@@ -26,9 +26,20 @@
 #pragma once
 
 #include "lsptypes.h"
+#include "semantictokens.h"
 
 namespace LanguageServerProtocol {
 
+class LANGUAGESERVERPROTOCOL_EXPORT WorkDoneProgressOptions : public JsonObject
+{
+public:
+    using JsonObject::JsonObject;
+
+    Utils::optional<bool> workDoneProgress() const { return optionalValue<bool>(workDoneProgressKey); }
+    void setWorkDoneProgress(bool workDoneProgress) { insert(workDoneProgressKey, workDoneProgress); }
+    void clearWorkDoneProgress() { remove(workDoneProgressKey); }
+};
+
 class LANGUAGESERVERPROTOCOL_EXPORT ResolveProviderOption : public JsonObject
 {
 public:
@@ -37,9 +48,6 @@ public:
     Utils::optional<bool> resolveProvider() const { return optionalValue<bool>(resolveProviderKey); }
     void setResolveProvider(bool resolveProvider) { insert(resolveProviderKey, resolveProvider); }
     void clearResolveProvider() { remove(resolveProviderKey); }
-
-    bool isValid(ErrorHierarchy *error) const override
-    { return checkOptional<bool>(error, resolveProviderKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT TextDocumentRegistrationOptions : public JsonObject
@@ -55,8 +63,7 @@ public:
     bool filterApplies(const Utils::FilePath &fileName,
                        const Utils::MimeType &mimeType = Utils::MimeType()) const;
 
-    bool isValid(ErrorHierarchy *error) const override
-    { return checkArray<DocumentFilter>(error, documentSelectorKey); }
+    bool isValid() const override { return contains(documentSelectorKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT SaveOptions : public JsonObject
@@ -68,9 +75,6 @@ public:
     Utils::optional<bool> includeText() const { return optionalValue<bool>(includeTextKey); }
     void setIncludeText(bool includeText) { insert(includeTextKey, includeText); }
     void clearIncludeText() { remove(includeTextKey); }
-
-    bool isValid(ErrorHierarchy *error) const override
-    { return checkOptional<bool>(error, includeTextKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT TextDocumentSyncOptions : public JsonObject
@@ -105,8 +109,6 @@ public:
     Utils::optional<SaveOptions> save() const { return optionalValue<SaveOptions>(saveKey); }
     void setSave(const SaveOptions &save) { insert(saveKey, save); }
     void clearSave() { remove(saveKey); }
-
-    bool isValid(ErrorHierarchy *error) const override;
 };
 
 enum class TextDocumentSyncKind
@@ -120,17 +122,59 @@ enum class TextDocumentSyncKind
     Incremental = 2
 };
 
-class LANGUAGESERVERPROTOCOL_EXPORT CodeActionOptions : public JsonObject
+class LANGUAGESERVERPROTOCOL_EXPORT CodeActionOptions : public WorkDoneProgressOptions
 {
 public:
-    using JsonObject::JsonObject;
+    using WorkDoneProgressOptions::WorkDoneProgressOptions;
 
     QList<QString> codeActionKinds() const { return array<QString>(codeActionKindsKey); }
     void setCodeActionKinds(const QList<QString> &codeActionKinds)
     { insertArray(codeActionKindsKey, codeActionKinds); }
 
-    bool isValid(ErrorHierarchy *error) const override
-    { return checkArray<QString>(error, codeActionKindsKey); }
+    bool isValid() const override;
+};
+
+enum class SemanticRequestType {
+    None = 0x0,
+    Full = 0x1,
+    FullDelta = 0x2,
+    Range = 0x4
+};
+Q_DECLARE_FLAGS(SemanticRequestTypes, SemanticRequestType)
+
+class LANGUAGESERVERPROTOCOL_EXPORT SemanticTokensOptions : public WorkDoneProgressOptions
+{
+public:
+    using WorkDoneProgressOptions::WorkDoneProgressOptions;
+
+    /// The legend used by the server
+    SemanticTokensLegend legend() const { return typedValue<SemanticTokensLegend>(legendKey); }
+    void setLegend(const SemanticTokensLegend &legend) { insert(legendKey, legend); }
+
+    /// Server supports providing semantic tokens for a specific range of a document.
+    Utils::optional<Utils::variant<bool, QJsonObject>> range() const;
+    void setRange(const Utils::variant<bool, QJsonObject> &range);
+    void clearRange() { remove(rangeKey); }
+
+    class FullSemanticTokenOptions : public JsonObject
+    {
+    public:
+        using JsonObject::JsonObject;
+
+        /// The server supports deltas for full documents.
+        Utils::optional<bool> delta() const { return optionalValue<bool>(deltaKey); }
+        void setDelta(bool delta) { insert(deltaKey, delta); }
+        void clearDelta() { remove(deltaKey); }
+    };
+
+    /// Server supports providing semantic tokens for a full document.
+    Utils::optional<Utils::variant<bool, FullSemanticTokenOptions>> full() const;
+    void setFull(const Utils::variant<bool, FullSemanticTokenOptions> &full);
+    void clearFull() { remove(fullKey); }
+
+    bool isValid() const override { return contains(legendKey); }
+
+    SemanticRequestTypes supportedRequests() const;
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT ServerCapabilities : public JsonObject
@@ -140,10 +184,10 @@ public:
 
     // Defines how the host (editor) should sync document changes to the language server.
 
-    class LANGUAGESERVERPROTOCOL_EXPORT CompletionOptions : public ResolveProviderOption
+    class LANGUAGESERVERPROTOCOL_EXPORT CompletionOptions : public WorkDoneProgressOptions
     {
     public:
-        using ResolveProviderOption::ResolveProviderOption;
+        using WorkDoneProgressOptions::WorkDoneProgressOptions;
 
         // The characters that trigger completion automatically.
         Utils::optional<QList<QString>> triggerCharacters() const
@@ -152,14 +196,15 @@ public:
         { insertArray(triggerCharactersKey, triggerCharacters); }
         void clearTriggerCharacters() { remove(triggerCharactersKey); }
 
-        bool isValid(ErrorHierarchy *error) const override
-        { return checkOptionalArray<QString>(error, triggerCharactersKey); }
+        Utils::optional<bool> resolveProvider() const { return optionalValue<bool>(resolveProviderKey); }
+        void setResolveProvider(bool resolveProvider) { insert(resolveProviderKey, resolveProvider); }
+        void clearResolveProvider() { remove(resolveProviderKey); }
     };
 
-    class LANGUAGESERVERPROTOCOL_EXPORT SignatureHelpOptions : public JsonObject
+    class LANGUAGESERVERPROTOCOL_EXPORT SignatureHelpOptions : public WorkDoneProgressOptions
     {
     public:
-        using JsonObject::JsonObject;
+        using WorkDoneProgressOptions::WorkDoneProgressOptions;
 
         // The characters that trigger signature help automatically.
         Utils::optional<QList<QString>> triggerCharacters() const
@@ -188,25 +233,20 @@ public:
         { insertArray(moreTriggerCharacterKey, moreTriggerCharacter); }
         void clearMoreTriggerCharacter() { remove(moreTriggerCharacterKey); }
 
-        bool isValid(ErrorHierarchy *error) const override
-        {
-            return check<QString>(error, firstTriggerCharacterKey)
-                    && checkOptionalArray<QString>(error, moreTriggerCharacterKey);
-        }
+        bool isValid() const override { return contains(firstTriggerCharacterKey); }
     };
 
     using DocumentLinkOptions = ResolveProviderOption;
 
-    class LANGUAGESERVERPROTOCOL_EXPORT ExecuteCommandOptions : public JsonObject
+    class LANGUAGESERVERPROTOCOL_EXPORT ExecuteCommandOptions : public WorkDoneProgressOptions
     {
     public:
-        using JsonObject::JsonObject;
+        using WorkDoneProgressOptions::WorkDoneProgressOptions;
 
         QList<QString> commands() const { return array<QString>(commandsKey); }
         void setCommands(const QList<QString> &commands) { insertArray(commandsKey, commands); }
 
-        bool isValid(ErrorHierarchy *error) const override
-        { return checkArray<QString>(error, commandsKey); }
+        bool isValid() const override;
     };
 
     using ColorProviderOptions = JsonObject;
@@ -231,7 +271,7 @@ public:
         Utils::optional<QList<QList<QString>>> scopes() const;
         void setScopes(const QList<QList<QString>> &scopes);
 
-        bool isValid(ErrorHierarchy *) const override;
+        bool isValid() const override;
     };
 
     // Defines how text documents are synced. Is either a detailed structure defining each
@@ -244,8 +284,8 @@ public:
     TextDocumentSyncKind textDocumentSyncKindHelper();
 
     // The server provides hover support.
-    Utils::optional<bool> hoverProvider() const { return optionalValue<bool>(hoverProviderKey); }
-    void setHoverProvider(bool hoverProvider) { insert(hoverProviderKey, hoverProvider); }
+    Utils::optional<Utils::variant<bool, WorkDoneProgressOptions>> hoverProvider() const;
+    void setHoverProvider(const Utils::variant<bool, WorkDoneProgressOptions> &hoverProvider);
     void clearHoverProvider() { remove(hoverProviderKey); }
 
     // The server provides completion support.
@@ -288,8 +328,7 @@ public:
         void setId(const QString &id) { insert(idKey, id); }
         void clearId() { remove(idKey); }
 
-        bool isValid(ErrorHierarchy *error) const override
-        { return checkArray<DocumentFilter>(error, documentSelectorKey) && checkOptional<bool>(error, idKey); }
+        bool isValid() const override { return contains(documentSelectorKey); }
     };
 
     // The server provides Goto Type Definition support.
@@ -303,29 +342,28 @@ public:
     void clearImplementationProvider() { remove(implementationProviderKey); }
 
     // The server provides find references support.
-    Utils::optional<bool> referencesProvider() const { return optionalValue<bool>(referencesProviderKey); }
-    void setReferencesProvider(bool referenceProvider) { insert(referencesProviderKey, referenceProvider); }
+    Utils::optional<Utils::variant<bool, WorkDoneProgressOptions>> referencesProvider() const;
+    void setReferencesProvider(const Utils::variant<bool, WorkDoneProgressOptions> &referencesProvider);
     void clearReferencesProvider() { remove(referencesProviderKey); }
 
     // The server provides document highlight support.
-    Utils::optional<bool> documentHighlightProvider() const
-    { return optionalValue<bool>(documentHighlightProviderKey); }
-    void setDocumentHighlightProvider(bool documentHighlightProvider)
-    { insert(documentHighlightProviderKey, documentHighlightProvider); }
+    Utils::optional<Utils::variant<bool, WorkDoneProgressOptions>> documentHighlightProvider() const;
+    void setDocumentHighlightProvider(
+        const Utils::variant<bool, WorkDoneProgressOptions> &documentHighlightProvider);
     void clearDocumentHighlightProvider() { remove(documentHighlightProviderKey); }
 
     // The server provides document symbol support.
-    Utils::optional<bool> documentSymbolProvider() const
-    { return optionalValue<bool>(documentSymbolProviderKey); }
-    void setDocumentSymbolProvider(bool documentSymbolProvider)
-    { insert(documentSymbolProviderKey, documentSymbolProvider); }
+    Utils::optional<Utils::variant<bool, WorkDoneProgressOptions>> documentSymbolProvider() const;
+    void setDocumentSymbolProvider(Utils::variant<bool, WorkDoneProgressOptions> documentSymbolProvider);
     void clearDocumentSymbolProvider() { remove(documentSymbolProviderKey); }
 
+    Utils::optional<SemanticTokensOptions> semanticTokensProvider() const;
+    void setSemanticTokensProvider(const SemanticTokensOptions &semanticTokensProvider);
+    void clearSemanticTokensProvider() { remove(semanticTokensProviderKey); }
+
     // The server provides workspace symbol support.
-    Utils::optional<bool> workspaceSymbolProvider() const
-    { return optionalValue<bool>(workspaceSymbolProviderKey); }
-    void setWorkspaceSymbolProvider(bool workspaceSymbolProvider)
-    { insert(workspaceSymbolProviderKey, workspaceSymbolProvider); }
+    Utils::optional<Utils::variant<bool, WorkDoneProgressOptions>> workspaceSymbolProvider() const;
+    void setWorkspaceSymbolProvider(Utils::variant<bool, WorkDoneProgressOptions> workspaceSymbolProvider);
     void clearWorkspaceSymbolProvider() { remove(workspaceSymbolProviderKey); }
 
     // The server provides code actions.
@@ -344,31 +382,25 @@ public:
     void clearCodeLensProvider() { remove(codeLensProviderKey); }
 
     // The server provides document formatting.
-    Utils::optional<bool> documentFormattingProvider() const
-    { return optionalValue<bool>(documentFormattingProviderKey); }
-    void setDocumentFormattingProvider(bool documentFormattingProvider)
-    { insert(documentFormattingProviderKey, documentFormattingProvider); }
+    Utils::optional<Utils::variant<bool, WorkDoneProgressOptions>> documentFormattingProvider() const;
+    void setDocumentFormattingProvider(
+        const Utils::variant<bool, WorkDoneProgressOptions> &documentFormattingProvider);
     void clearDocumentFormattingProvider() { remove(documentFormattingProviderKey); }
 
     // The server provides document formatting on typing.
-    Utils::optional<bool> documentRangeFormattingProvider() const
-    { return optionalValue<bool>(documentRangeFormattingProviderKey); }
-    void setDocumentRangeFormattingProvider(bool documentRangeFormattingProvider)
-    { insert(documentRangeFormattingProviderKey, documentRangeFormattingProvider); }
+    Utils::optional<Utils::variant<bool, WorkDoneProgressOptions>> documentRangeFormattingProvider() const;
+    void setDocumentRangeFormattingProvider(Utils::variant<bool, WorkDoneProgressOptions> documentRangeFormattingProvider);
     void clearDocumentRangeFormattingProvider() { remove(documentRangeFormattingProviderKey); }
 
-    class RenameOptions : public JsonObject
+    class LANGUAGESERVERPROTOCOL_EXPORT RenameOptions : public WorkDoneProgressOptions
     {
     public:
-        using JsonObject::JsonObject;
+        using WorkDoneProgressOptions::WorkDoneProgressOptions;
 
         // Renames should be checked and tested before being executed.
         Utils::optional<bool> prepareProvider() const { return optionalValue<bool>(prepareProviderKey); }
         void setPrepareProvider(bool prepareProvider) { insert(prepareProviderKey, prepareProvider); }
         void clearPrepareProvider() { remove(prepareProviderKey); }
-
-        bool isValid(ErrorHierarchy * error) const override
-        { return checkOptional<bool>(error, prepareProviderKey); }
     };
 
     // The server provides rename support.
@@ -413,8 +445,6 @@ public:
             Utils::optional<Utils::variant<QString, bool>> changeNotifications() const;
             void setChangeNotifications(Utils::variant<QString, bool> changeNotifications);
             void clearChangeNotifications() { remove(changeNotificationsKey); }
-
-            bool isValid(ErrorHierarchy *error) const override;
         };
 
         Utils::optional<WorkspaceFoldersCapabilities> workspaceFolders() const
@@ -439,8 +469,6 @@ public:
     void setSemanticHighlighting(const SemanticHighlightingServerCapabilities &semanticHighlighting)
     { insert(semanticHighlightingKey, semanticHighlighting); }
     void clearSemanticHighlighting() { remove(semanticHighlightingKey); }
-
-    bool isValid(ErrorHierarchy *error) const override;
 };
 
 } // namespace LanguageClient
