@@ -53,6 +53,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT WorkspaceFoldersChangeEvent : public JsonObj
 {
 public:
     using JsonObject::JsonObject;
+    WorkspaceFoldersChangeEvent();
 
     QList<WorkSpaceFolder> added() const { return array<WorkSpaceFolder>(addedKey); }
     void setAdded(const QList<WorkSpaceFolder> &added) { insertArray(addedKey, added); }
@@ -60,7 +61,7 @@ public:
     QList<WorkSpaceFolder> removed() const { return array<WorkSpaceFolder>(removedKey); }
     void setRemoved(const QList<WorkSpaceFolder> &removed) { insertArray(removedKey, removed); }
 
-    bool isValid(ErrorHierarchy *error) const override;
+    bool isValid() const override { return contains(addedKey) && contains(removedKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT DidChangeWorkspaceFoldersParams : public JsonObject
@@ -72,8 +73,7 @@ public:
     { return typedValue<WorkspaceFoldersChangeEvent>(eventKey); }
     void setEvent(const WorkspaceFoldersChangeEvent &event) { insert(eventKey, event); }
 
-    bool isValid(ErrorHierarchy *error) const override
-    { return check<WorkspaceFoldersChangeEvent>(error, eventKey); }
+    bool isValid() const override { return contains(eventKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT DidChangeWorkspaceFoldersNotification : public Notification<
@@ -90,10 +90,10 @@ class LANGUAGESERVERPROTOCOL_EXPORT DidChangeConfigurationParams : public JsonOb
 public:
     using JsonObject::JsonObject;
 
-    QJsonValue settings() const { return typedValue<QJsonValue>(settingsKey); }
+    QJsonValue settings() const { return value(settingsKey); }
     void setSettings(QJsonValue settings) { insert(settingsKey, settings); }
 
-    bool isValid(ErrorHierarchy *error) const override;
+    bool isValid() const override { return contains(settingsKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT DidChangeConfigurationNotification : public Notification<
@@ -109,7 +109,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT ConfigurationParams : public JsonObject
 {
 public:
     using JsonObject::JsonObject;
-    class ConfigureationItem : public JsonObject
+    class LANGUAGESERVERPROTOCOL_EXPORT ConfigurationItem : public JsonObject
     {
     public:
         using JsonObject::JsonObject;
@@ -122,14 +122,13 @@ public:
         void setSection(const QString &section) { insert(sectionKey, section); }
         void clearSection() { remove(sectionKey); }
 
-        bool isValid(ErrorHierarchy *error) const override;
+        bool isValid() const override { return contains(scopeUriKey); }
     };
 
-    QList<ConfigureationItem> items() const { return array<ConfigureationItem>(itemsKey); }
-    void setItems(const QList<ConfigureationItem> &items) { insertArray(itemsKey, items); }
+    QList<ConfigurationItem> items() const { return array<ConfigurationItem>(itemsKey); }
+    void setItems(const QList<ConfigurationItem> &items) { insertArray(itemsKey, items); }
 
-    bool isValid(ErrorHierarchy *error) const override
-    { return checkArray<ConfigureationItem>(error, itemsKey); }
+    bool isValid() const override { return contains(itemsKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT ConfigurationRequest : public Request<
@@ -163,15 +162,13 @@ public:
             Deleted = 3
         };
 
-        bool isValid(ErrorHierarchy *error) const override
-        { return check<QString>(error, uriKey) && check<int>(error, typeKey); }
+        bool isValid() const override { return contains(uriKey) && contains(typeKey); }
     };
 
     QList<FileEvent> changes() const { return array<FileEvent>(changesKey); }
     void setChanges(const QList<FileEvent> &changes) { insertArray(changesKey, changes); }
 
-    bool isValid(ErrorHierarchy *error) const override
-    { return checkArray<FileEvent>(error, changesKey); }
+    bool isValid() const override { return contains(changesKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT DidChangeWatchedFilesNotification : public Notification<
@@ -191,7 +188,9 @@ public:
     QString query() const { return typedValue<QString>(queryKey); }
     void setQuery(const QString &query) { insert(queryKey, query); }
 
-    bool isValid(ErrorHierarchy *error) const override { return check<QString>(error, queryKey); }
+    void setLimit(int limit) { insert("limit", limit); } // clangd extension
+
+    bool isValid() const override { return contains(queryKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT WorkspaceSymbolRequest : public Request<
@@ -218,11 +217,7 @@ public:
     void setArguments(const QJsonArray &arguments) { insert(argumentsKey, arguments); }
     void clearArguments() { remove(argumentsKey); }
 
-    bool isValid(ErrorHierarchy *error) const override
-    {
-        return check<QString>(error, commandKey)
-                && checkOptionalArray<QJsonValue>(error, argumentsKey);
-    }
+    bool isValid() const override { return contains(commandKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT ExecuteCommandRequest : public Request<
@@ -246,8 +241,7 @@ public:
     WorkspaceEdit edit() const { return typedValue<WorkspaceEdit>(editKey); }
     void setEdit(const WorkspaceEdit &edit) { insert(editKey, edit); }
 
-    bool isValid(ErrorHierarchy *error) const override
-    { return check<WorkspaceEdit>(error, editKey) && checkOptional<QString>(error, labelKey); }
+    bool isValid() const override { return contains(editKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT ApplyWorkspaceEditResponse : public JsonObject
@@ -258,7 +252,7 @@ public:
     bool applied() const { return typedValue<bool>(appliedKey); }
     void setApplied(bool applied) { insert(appliedKey, applied); }
 
-    bool isValid(ErrorHierarchy *error) const override { return check<bool>(error, appliedKey); }
+    bool isValid() const override { return contains(appliedKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT ApplyWorkspaceEditRequest : public Request<
