@@ -24,7 +24,10 @@
 ****************************************************************************/
 
 #include "qmlenginecontrolclient.h"
-#include "utils/qtcassert.h"
+#include "qpacketprotocol.h"
+#include <utils/qtcassert.h>
+
+#include <functional>
 
 namespace QmlDebug {
 
@@ -64,11 +67,16 @@ void QmlEngineControlClient::releaseEngine(int engineId)
     }
 }
 
+QList<int> QmlEngineControlClient::blockedEngines() const
+{
+    return m_blockedEngines.keys();
+}
+
 void QmlEngineControlClient::messageReceived(const QByteArray &data)
 {
-    QDataStream stream(data);
-    int message;
-    int id;
+    QPacket stream(dataStreamVersion(), data);
+    qint32 message;
+    qint32 id;
     QString name;
 
     stream >> message >> id;
@@ -76,38 +84,43 @@ void QmlEngineControlClient::messageReceived(const QByteArray &data)
     if (!stream.atEnd())
         stream >> name;
 
-    EngineState &state = m_blockedEngines[id];
-    QTC_ASSERT(state.blockers == 0 && state.releaseCommand == InvalidCommand, /**/);
+    auto handleWaiting = [&](CommandType command, std::function<void()> emitter) {
+        EngineState &state = m_blockedEngines[id];
+        QTC_CHECK(state.blockers == 0);
+        QTC_CHECK(state.releaseCommand == InvalidCommand);
+        state.releaseCommand = command;
+        emitter();
+        if (state.blockers == 0) {
+            sendCommand(state.releaseCommand, id);
+            m_blockedEngines.remove(id);
+        }
+    };
 
     switch (message) {
     case EngineAboutToBeAdded:
-        state.releaseCommand = StartWaitingEngine;
-        emit engineAboutToBeAdded(id, name);
+        handleWaiting(StartWaitingEngine, [&](){
+            emit engineAboutToBeAdded(id, name);
+        });
         break;
     case EngineAdded:
         emit engineAdded(id, name);
         break;
     case EngineAboutToBeRemoved:
-        state.releaseCommand = StopWaitingEngine;
-        emit engineAboutToBeRemoved(id, name);
+        handleWaiting(StopWaitingEngine, [&](){
+            emit engineAboutToBeRemoved(id, name);
+        });
         break;
     case EngineRemoved:
         emit engineRemoved(id, name);
         break;
     }
-
-    if (state.blockers == 0 && state.releaseCommand != InvalidCommand) {
-        sendCommand(state.releaseCommand, id);
-        m_blockedEngines.remove(id);
-    }
 }
 
 void QmlEngineControlClient::sendCommand(QmlEngineControlClient::CommandType command, int engineId)
 {
-    QByteArray data;
-    QDataStream stream(&data, QIODevice::WriteOnly);
-    stream << int(command) << engineId;
-    QmlDebugClient::sendMessage(data);
+    QPacket stream(dataStreamVersion());
+    stream << static_cast<qint32>(command) << engineId;
+    QmlDebugClient::sendMessage(stream.data());
 }
 
 }
