@@ -152,16 +152,34 @@ ObjectValue *Bind::bindObject(UiQualifiedId *qualifiedTypeNameId, UiObjectInitia
     }
 
     parentObjectValue = switchObjectValue(objectValue);
-
-    if (parentObjectValue) {
-        objectValue->setMember(QLatin1String("parent"), parentObjectValue);
-    } else if (!_rootObjectValue) {
+    ObjectValue *nextRoot = _rootObjectValue;
+    QString parentComponentName = _currentComponentName;
+    if (!_rootObjectValue) {
         _rootObjectValue = objectValue;
-        _rootObjectValue->setClassName(_doc->componentName());
+        _inlineComponents[_currentComponentName] = objectValue;
+        if (!_currentComponentName.isEmpty()) {
+            if (_currentComponentName.contains('.')) {
+                parentComponentName = _currentComponentName.mid(0,_currentComponentName.lastIndexOf('.'));
+                nextRoot = _inlineComponents.value(parentComponentName);
+            } else {
+                parentComponentName = "";
+                nextRoot = _rootObjectValue;
+            }
+            // we add the inline component inside its parent
+            nextRoot->setMember(_currentComponentName.mid(_currentComponentName.lastIndexOf('.') + 1), objectValue);
+            _rootObjectValue->setClassName(_doc->componentName() + "." + _currentComponentName); // use :: instead of .?
+        } else {
+            nextRoot = _rootObjectValue;
+            _rootObjectValue->setClassName(_doc->componentName());
+        }
+    } else if (parentObjectValue) {
+        objectValue->setMember(QLatin1String("parent"), parentObjectValue);
     }
 
     accept(initializer);
 
+    _rootObjectValue = nextRoot;
+    _currentComponentName = parentComponentName;
     return switchObjectValue(parentObjectValue);
 }
 
@@ -205,19 +223,21 @@ bool Bind::visit(UiImport *ast)
     if (ast->version)
         version = ComponentVersion(ast->version->majorVersion, ast->version->minorVersion);
 
-    if (ast->importUri) {
-        QVersionNumber qtVersion = QLibraryInfo::version();
+    if (auto importUri = ast->importUri) {
+        QVersionNumber qtVersion;
+        QString uri = toString(importUri);
         if (ModelManagerInterface *model = ModelManagerInterface::instance()) {
             ModelManagerInterface::ProjectInfo pInfo = model->projectInfoForPath(_doc->fileName());
             qtVersion = QVersionNumber::fromString(pInfo.qtVersionString);
+            uri = pInfo.moduleMappings.value(uri, uri);
         }
-        if (!version.isValid() && qtVersion.majorVersion() < 6) {
+        if (!version.isValid() && (!qtVersion.isNull() && qtVersion.majorVersion() < 6)) {
             _diagnosticMessages->append(
                         errorMessage(ast, tr("package import requires a version number")));
         }
         const QString importId = ast->importId.toString();
-        ImportInfo import = ImportInfo::moduleImport(toString(ast->importUri), version,
-                                                     importId, ast);
+
+        ImportInfo import = ImportInfo::moduleImport(uri, version, importId, ast);
         if (_doc->language() == Dialect::Qml) {
             const QString importStr = import.name() + importId;
             if (ModelManagerInterface::instance()) {
@@ -318,6 +338,18 @@ bool Bind::visit(UiArrayBinding *)
     return true;
 }
 
+bool Bind::visit(UiInlineComponent *ast)
+{
+    if (!_currentComponentName.isEmpty()) {
+        _currentComponentName += ".";
+        _diagnosticMessages->append(
+            errorMessage(ast, tr("Nested inline components are not supported")));
+    }
+    _currentComponentName += ast->name.toString();
+    _rootObjectValue = nullptr;
+    return true;
+}
+
 bool Bind::visit(PatternElement *ast)
 {
     if (ast->bindingIdentifier.isEmpty() || !ast->isVariableDeclaration())
