@@ -65,6 +65,7 @@ protected:
     void postVisit(AST::Node *ast) override;
 
     bool visit(AST::UiProgram *ast) override;
+    bool visit(AST::UiImport *ast) override;
     bool visit(AST::UiObjectDefinition *ast) override;
     bool visit(AST::UiObjectBinding *ast) override;
     bool visit(AST::UiScriptBinding *ast) override;
@@ -84,7 +85,6 @@ protected:
     bool visit(AST::ExpressionStatement *ast) override;
     bool visit(AST::IfStatement *ast) override;
     bool visit(AST::ForStatement *ast) override;
-    bool visit(AST::LocalForStatement *ast) override;
     bool visit(AST::WhileStatement *ast) override;
     bool visit(AST::DoWhileStatement *ast) override;
     bool visit(AST::CaseBlock *ast) override;
@@ -99,12 +99,13 @@ protected:
 
     void endVisit(QmlJS::AST::UiObjectInitializer *) override;
 
+    void throwRecursionDepthError() override;
 private:
     void visitQmlObject(AST::Node *ast, AST::UiQualifiedId *typeId,
                         AST::UiObjectInitializer *initializer);
     const Value *checkScopeObjectMember(const AST::UiQualifiedId *id);
     void checkAssignInCondition(AST::ExpressionNode *condition);
-    void checkCaseFallthrough(AST::StatementList *statements, AST::SourceLocation errorLoc, AST::SourceLocation nextLoc);
+    void checkCaseFallthrough(AST::StatementList *statements, SourceLocation errorLoc, SourceLocation nextLoc);
     void checkProperty(QmlJS::AST::UiQualifiedId *);
     void checkNewExpression(AST::ExpressionNode *node);
     void checkBindingRhs(AST::Statement *statement);
@@ -112,7 +113,7 @@ private:
 
     void addMessages(const QList<StaticAnalysis::Message> &messages);
     void addMessage(const StaticAnalysis::Message &message);
-    void addMessage(StaticAnalysis::Type type, const AST::SourceLocation &location,
+    void addMessage(StaticAnalysis::Type type, const SourceLocation &location,
                     const QString &arg1 = QString(), const QString &arg2 = QString());
 
     void scanCommentsForAnnotations();
@@ -121,6 +122,9 @@ private:
     bool isQtQuick2() const;
     bool isQtQuick2Ui() const;
 
+    bool isCaseOrDefault(AST::Node *n);
+    bool hasVarStatement(AST::Block *b) const;
+
     AST::Node *parent(int distance = 0);
 
     Document::Ptr _doc;
@@ -137,20 +141,26 @@ private:
     QStack<StringSet> m_propertyStack;
     QStack<QString> m_typeStack;
 
+    using ShortImportInfo = QPair<QString, LanguageUtils::ComponentVersion>;
+    QList<ShortImportInfo> m_importInfo;
+
     class MessageTypeAndSuppression
     {
     public:
-        AST::SourceLocation suppressionSource;
+        SourceLocation suppressionSource;
         StaticAnalysis::Type type;
         bool wasSuppressed;
     };
 
+    enum TranslationFunction { qsTr, qsTrId, qsTranslate, noTranslationfunction };
+
     QHash< int, QList<MessageTypeAndSuppression> > m_disabledMessageTypesByLine;
 
     bool _importsOk;
     bool _inStatementBinding;
+    int _componentChildCount = 0;
     const Imports *_imports;
-    bool _isQtQuick2;
+    TranslationFunction lastTransLationfunction = noTranslationfunction;
 };
 
 } // namespace QmlJS
