@@ -32,6 +32,7 @@
 #include "qmljsmodelmanagerinterface.h"
 #include "qmljsconstants.h"
 
+#include <utils/porting.h>
 #include <utils/qrcparser.h>
 
 #include <QDir>
@@ -54,7 +55,7 @@ public:
     {}
 
 private:
-    friend uint qHash(const ImportCacheKey &);
+    friend Utils::QHashValueType qHash(const ImportCacheKey &);
     friend bool operator==(const ImportCacheKey &, const ImportCacheKey &);
 
     int m_type;
@@ -63,7 +64,7 @@ private:
     int m_minorVersion;
 };
 
-uint qHash(const ImportCacheKey &info)
+Utils::QHashValueType qHash(const ImportCacheKey &info)
 {
     return ::qHash(info.m_type) ^ ::qHash(info.m_path) ^
             ::qHash(info.m_majorVersion) ^ ::qHash(info.m_minorVersion);
@@ -150,7 +151,9 @@ Link::Link(const Snapshot &snapshot, const ViewerContext &vContext, const Librar
     d->diagnosticMessages = nullptr;
     d->allDiagnosticMessages = nullptr;
 
-    ModelManagerInterface *modelManager = ModelManagerInterface::instance();
+    QFutureInterface<void> fi;
+    fi.reportStarted();
+    ModelManagerInterface *modelManager = ModelManagerInterface::instanceForFuture(fi.future());
     if (modelManager) {
         const ModelManagerInterface::CppDataHash cppDataHash = modelManager->cppData();
         {
@@ -176,6 +179,7 @@ Link::Link(const Snapshot &snapshot, const ViewerContext &vContext, const Librar
         }
         d->m_valueOwner->cppQmlTypes().setCppContextProperties(cppContextProperties);
     }
+    fi.reportFinished();
 }
 
 ContextPtr Link::operator()(QHash<QString, QList<DiagnosticMessage> > *messages)
@@ -410,8 +414,13 @@ Import LinkPrivate::importNonFile(const Document::Ptr &doc, const ImportInfo &im
     const QString packageName = importInfo.name();
     const ComponentVersion version = importInfo.version();
 
-    QString libraryPath = modulePath(packageName, version.toString(), m_importPaths);
-    bool importFound = !libraryPath.isEmpty() && importLibrary(doc, libraryPath, &import, import.object);
+    QStringList libraryPaths = modulePaths(packageName, version.toString(), m_importPaths);
+    bool importFound = false;
+    for (const QString &libPath : libraryPaths) {
+        importFound = !libPath.isEmpty() && importLibrary(doc, libPath, &import, import.object);
+        if (importFound)
+            break;
+    }
 
     if (!importFound) {
         for (const QString &dir : qAsConst(m_applicationDirectories)) {
@@ -501,7 +510,10 @@ bool LinkPrivate::importLibrary(const Document::Ptr &doc,
         Import subImport;
         subImport.valid = true;
         subImport.info = ImportInfo::moduleImport(importName, vNow, importInfo.as(), importInfo.ast());
-        subImport.libraryPath = modulePath(importName, vNow.toString(), m_importPaths);
+
+        const QStringList libraryPaths = modulePaths(importName, vNow.toString(), m_importPaths);
+        subImport.libraryPath = libraryPaths.value(0); // first is the best match
+
         bool subImportFound = importLibrary(doc, subImport.libraryPath, &subImport, targetObject, importPath, true);
 
         if (!subImportFound && errorLoc.isValid()) {
