@@ -34,6 +34,7 @@
 #include <utils/filesystemwatcher.h>
 #include <utils/fileutils.h>
 #include <utils/hostosinfo.h>
+#include <utils/qtcprocess.h>
 #include <utils/runextensions.h>
 
 #include <QDir>
@@ -41,7 +42,9 @@
 #include <QRegularExpression>
 
 using namespace LanguageUtils;
-using namespace QmlJS;
+using namespace Utils;
+
+namespace QmlJS {
 
 PluginDumper::PluginDumper(ModelManagerInterface *modelManager)
     : QObject(modelManager)
@@ -86,8 +89,7 @@ void PluginDumper::onLoadBuiltinTypes(const QmlJS::ModelManagerInterface::Projec
     if (info.qmlDumpPath.isEmpty() || info.qtQmlPath.isEmpty())
         return;
 
-    const QString importsPath = QDir::cleanPath(info.qtQmlPath);
-    if (m_runningQmldumps.values().contains(importsPath))
+    if (m_runningQmldumps.values().contains(info.qmlDumpPath))
         return;
 
     LibraryInfo builtinInfo;
@@ -101,26 +103,19 @@ void PluginDumper::onLoadBuiltinTypes(const QmlJS::ModelManagerInterface::Projec
     m_modelManager->updateLibraryInfo(info.qtQmlPath, builtinInfo);
 
     // prefer QTDIR/qml/builtins.qmltypes if available
-    const QString builtinQmltypesPath = info.qtQmlPath + QLatin1String("/builtins.qmltypes");
-    if (QFile::exists(builtinQmltypesPath)) {
-        loadQmltypesFile(QStringList(builtinQmltypesPath), info.qtQmlPath, builtinInfo);
+    const FilePath builtinQmltypesPath = info.qtQmlPath / "builtins.qmltypes";
+    if (builtinQmltypesPath.exists()) {
+        loadQmltypesFile({builtinQmltypesPath}, info.qtQmlPath, builtinInfo);
         return;
     }
 
     runQmlDump(info, QStringList(QLatin1String("--builtins")), info.qtQmlPath);
-    m_qtToInfo.insert(info.qtQmlPath, info);
-}
-
-static QString makeAbsolute(const QString &path, const QString &base)
-{
-    if (QFileInfo(path).isAbsolute())
-        return path;
-    return QString::fromLatin1("%1/%3").arg(base, path);
+    m_qtToInfo.insert(info.qtQmlPath.toString(), info);
 }
 
 void PluginDumper::onLoadPluginTypes(const QString &libraryPath, const QString &importPath, const QString &importUri, const QString &importVersion)
 {
-    const QString canonicalLibraryPath = QDir::cleanPath(libraryPath);
+    const FilePath canonicalLibraryPath = FilePath::fromUserInput(libraryPath).cleanPath();
     if (m_runningQmldumps.values().contains(canonicalLibraryPath))
         return;
     const Snapshot snapshot = m_modelManager->snapshot();
@@ -131,7 +126,7 @@ void PluginDumper::onLoadPluginTypes(const QString &libraryPath, const QString &
     // avoid inserting the same plugin twice
     int index;
     for (index = 0; index < m_plugins.size(); ++index) {
-        if (m_plugins.at(index).qmldirPath == libraryPath)
+        if (m_plugins.at(index).qmldirPath == canonicalLibraryPath)
             break;
     }
     if (index == m_plugins.size())
@@ -144,40 +139,40 @@ void PluginDumper::onLoadPluginTypes(const QString &libraryPath, const QString &
     plugin.importVersion = importVersion;
 
     // add default qmltypes file if it exists
-    QDirIterator it(canonicalLibraryPath, QStringList { "*.qmltypes" }, QDir::Files);
+    QDirIterator it(canonicalLibraryPath.toString(), QStringList { "*.qmltypes" }, QDir::Files);
 
     while (it.hasNext()) {
-        const QString defaultQmltypesPath = makeAbsolute(it.next(), canonicalLibraryPath);
+        const FilePath defaultQmltypesPath = canonicalLibraryPath.resolvePath(it.next());
 
         if (!plugin.typeInfoPaths.contains(defaultQmltypesPath))
             plugin.typeInfoPaths += defaultQmltypesPath;
     }
 
     // add typeinfo files listed in qmldir
-    foreach (const QmlDirParser::TypeInfo &typeInfo, libraryInfo.typeInfos()) {
-        QString pathNow = makeAbsolute(typeInfo.fileName, canonicalLibraryPath);
-        if (!plugin.typeInfoPaths.contains(pathNow) && QFile::exists(pathNow))
+    foreach (const QString &typeInfo, libraryInfo.typeInfos()) {
+        const FilePath pathNow = canonicalLibraryPath.resolvePath(typeInfo);
+        if (!plugin.typeInfoPaths.contains(pathNow) && pathNow.exists())
             plugin.typeInfoPaths += pathNow;
     }
 
     // watch plugin libraries
     foreach (const QmlDirParser::Plugin &plugin, snapshot.libraryInfo(canonicalLibraryPath).plugins()) {
-        const QString pluginLibrary = resolvePlugin(canonicalLibraryPath, plugin.path, plugin.name);
+        const QString pluginLibrary = resolvePlugin(canonicalLibraryPath.toString(), plugin.path, plugin.name);
         if (!pluginLibrary.isEmpty()) {
             if (!pluginWatcher()->watchesFile(pluginLibrary))
-                pluginWatcher()->addFile(pluginLibrary, Utils::FileSystemWatcher::WatchModifiedDate);
+                pluginWatcher()->addFile(pluginLibrary, FileSystemWatcher::WatchModifiedDate);
             m_libraryToPluginIndex.insert(pluginLibrary, index);
         }
     }
 
     // watch library qmltypes file
     if (!plugin.typeInfoPaths.isEmpty()) {
-        foreach (const QString &path, plugin.typeInfoPaths) {
-            if (!QFile::exists(path))
+        for (const FilePath &path : qAsConst(plugin.typeInfoPaths)) {
+            if (!path.exists())
                 continue;
-            if (!pluginWatcher()->watchesFile(path))
-                pluginWatcher()->addFile(path, Utils::FileSystemWatcher::WatchModifiedDate);
-            m_libraryToPluginIndex.insert(path, index);
+            if (!pluginWatcher()->watchesFile(path.toString()))
+                pluginWatcher()->addFile(path.toString(), FileSystemWatcher::WatchModifiedDate);
+            m_libraryToPluginIndex.insert(path.toString(), index);
         }
     }
 
@@ -191,26 +186,25 @@ void PluginDumper::dumpAllPlugins()
     }
 }
 
-static QString noTypeinfoError(const QString &libraryPath)
+static QString noTypeinfoError(const FilePath &libraryPath)
 {
     return PluginDumper::tr("QML module does not contain information about components contained in plugins.\n\n"
                             "Module path: %1\n"
                             "See \"Using QML Modules with Plugins\" in the documentation.").arg(
-                libraryPath);
+                libraryPath.toUserOutput());
 }
 
-static QString qmldumpErrorMessage(const QString &libraryPath, const QString &error)
+static QString qmldumpErrorMessage(const FilePath &libraryPath, const QString &error)
 {
-    return noTypeinfoError(libraryPath) + QLatin1String("\n\n") +
+    return noTypeinfoError(libraryPath) + "\n\n" +
             PluginDumper::tr("Automatic type dump of QML module failed.\nErrors:\n%1").
             arg(error) + QLatin1Char('\n');
 }
 
-static QString qmldumpFailedMessage(const QString &libraryPath, const QString &error)
+static QString qmldumpFailedMessage(const FilePath &libraryPath, const QString &error)
 {
-    QString firstLines =
-            QStringList(error.split(QLatin1Char('\n')).mid(0, 10)).join(QLatin1Char('\n'));
-    return noTypeinfoError(libraryPath) + QLatin1String("\n\n") +
+    QString firstLines = QStringList(error.split('\n').mid(0, 10)).join('\n');
+    return noTypeinfoError(libraryPath) + "\n\n" +
             PluginDumper::tr("Automatic type dump of QML module failed.\n"
                              "First 10 lines or errors:\n"
                              "\n"
@@ -220,17 +214,17 @@ static QString qmldumpFailedMessage(const QString &libraryPath, const QString &e
                              ).arg(firstLines);
 }
 
-static void printParseWarnings(const QString &libraryPath, const QString &warning)
+static void printParseWarnings(const FilePath &libraryPath, const QString &warning)
 {
     ModelManagerInterface::writeWarning(
                 PluginDumper::tr("Warnings while parsing QML type information of %1:\n"
-                                 "%2").arg(libraryPath, warning));
+                                 "%2").arg(libraryPath.toUserOutput(), warning));
 }
 
-static QString qmlPluginDumpErrorMessage(QProcess *process)
+static QString qmlPluginDumpErrorMessage(QtcProcess *process)
 {
     QString errorMessage;
-    const QString binary = QDir::toNativeSeparators(process->program());
+    const QString binary = process->commandLine().executable().toUserOutput();
     switch (process->error()) {
     case QProcess::FailedToStart:
         errorMessage = PluginDumper::tr("\"%1\" failed to start: %2").arg(binary, process->errorString());
@@ -250,7 +244,7 @@ static QString qmlPluginDumpErrorMessage(QProcess *process)
             errorMessage = PluginDumper::tr("\"%1\" returned exit code %2.").arg(binary).arg(process->exitCode());
         break;
     }
-    errorMessage += QLatin1Char('\n') + PluginDumper::tr("Arguments: %1").arg(process->arguments().join(QLatin1Char(' ')));
+    errorMessage += '\n' + PluginDumper::tr("Arguments: %1").arg(process->commandLine().arguments());
     if (process->error() != QProcess::FailedToStart) {
         const QString stdErr = QString::fromLocal8Bit(process->readAllStandardError());
         if (!stdErr.isEmpty()) {
@@ -261,21 +255,18 @@ static QString qmlPluginDumpErrorMessage(QProcess *process)
     return errorMessage;
 }
 
-void PluginDumper::qmlPluginTypeDumpDone(int exitCode)
+void PluginDumper::qmlPluginTypeDumpDone(QtcProcess *process)
 {
-    QProcess *process = qobject_cast<QProcess *>(sender());
-    if (!process)
-        return;
     process->deleteLater();
 
-    const QString libraryPath = m_runningQmldumps.take(process);
+    const FilePath libraryPath = m_runningQmldumps.take(process);
     if (libraryPath.isEmpty())
         return;
     const Snapshot snapshot = m_modelManager->snapshot();
     LibraryInfo libraryInfo = snapshot.libraryInfo(libraryPath);
     bool privatePlugin = libraryPath.endsWith(QLatin1String("private"));
 
-    if (exitCode != 0) {
+    if (process->exitCode() != 0) {
         const QString errorMessages = qmlPluginDumpErrorMessage(process);
         if (!privatePlugin)
             ModelManagerInterface::writeWarning(qmldumpErrorMessage(libraryPath, errorMessages));
@@ -297,12 +288,13 @@ void PluginDumper::qmlPluginTypeDumpDone(int exitCode)
             CppQmlTypesInfo infos;
             CppQmlTypesLoader::parseQmlTypeDescriptions(output, &infos.objectsList, &infos.moduleApis, &infos.dependencies,
                                                         &infos.error, &infos.warning,
-                                                        QLatin1String("<dump of ") + libraryPath + QLatin1Char('>'));
+                                                        "<dump of " + libraryPath.toUserOutput() + '>');
             future.reportFinished(&infos);
         });
+        m_modelManager->addFuture(future);
 
-        auto finalFuture = Utils::onFinished(future, this,
-                [this, libraryInfo, privatePlugin, libraryPath] (const QFuture<CppQmlTypesInfo>& future) {
+        Utils::onFinished(future, this, [this, libraryInfo, privatePlugin, libraryPath]
+                          (const QFuture<CppQmlTypesInfo>& future) {
             CppQmlTypesInfo infos = future.result();
 
             LibraryInfo libInfo = libraryInfo;
@@ -325,7 +317,6 @@ void PluginDumper::qmlPluginTypeDumpDone(int exitCode)
 
             m_modelManager->updateLibraryInfo(libraryPath, libInfo);
         });
-        m_modelManager->addFuture(finalFuture);
     } else {
         libraryInfo.setPluginTypeInfoStatus(LibraryInfo::DumpDone);
         libraryInfo.updateFingerprint();
@@ -333,20 +324,17 @@ void PluginDumper::qmlPluginTypeDumpDone(int exitCode)
     }
 }
 
-void PluginDumper::qmlPluginTypeDumpError(QProcess::ProcessError)
+void PluginDumper::qmlPluginTypeDumpError(QtcProcess *process)
 {
-    QProcess *process = qobject_cast<QProcess *>(sender());
-    if (!process)
-        return;
     process->deleteLater();
 
-    const QString libraryPath = m_runningQmldumps.take(process);
+    const FilePath libraryPath = m_runningQmldumps.take(process);
     if (libraryPath.isEmpty())
         return;
     const QString errorMessages = qmlPluginDumpErrorMessage(process);
     const Snapshot snapshot = m_modelManager->snapshot();
     LibraryInfo libraryInfo = snapshot.libraryInfo(libraryPath);
-    if (!libraryPath.endsWith(QLatin1String("private"), Qt::CaseInsensitive))
+    if (!libraryPath.path().endsWith(QLatin1String("private"), Qt::CaseInsensitive))
         ModelManagerInterface::writeWarning(qmldumpErrorMessage(libraryPath, errorMessages));
     libraryInfo.setPluginTypeInfoStatus(LibraryInfo::DumpError, qmldumpFailedMessage(libraryPath, errorMessages));
     libraryInfo.updateFingerprint();
@@ -363,12 +351,13 @@ void PluginDumper::pluginChanged(const QString &pluginLibrary)
     dump(plugin);
 }
 
-QFuture<PluginDumper::QmlTypeDescription> PluginDumper::loadQmlTypeDescription(const QStringList &paths) const {
+QFuture<PluginDumper::QmlTypeDescription> PluginDumper::loadQmlTypeDescription(const FilePaths &paths) const
+{
     auto future = Utils::runAsync([=](QFutureInterface<PluginDumper::QmlTypeDescription> &future)
     {
         PluginDumper::QmlTypeDescription result;
 
-        for (const QString &p: paths) {
+        for (const FilePath &p: paths) {
             Utils::FileReader reader;
             if (!reader.fetch(p, QFile::Text)) {
                 result.errors += reader.errorString();
@@ -380,9 +369,9 @@ QFuture<PluginDumper::QmlTypeDescription> PluginDumper::loadQmlTypeDescription(c
             QList<ModuleApiInfo> apis;
             QStringList deps;
             CppQmlTypesLoader::parseQmlTypeDescriptions(reader.data(), &objs, &apis, &deps,
-                                                        &error, &warning, p);
+                                                        &error, &warning, p.toString());
             if (!error.isEmpty()) {
-                result.errors += tr("Failed to parse \"%1\".\nError: %2").arg(p, error);
+                result.errors += tr("Failed to parse \"%1\".\nError: %2").arg(p.toUserOutput(), error);
             } else {
                 result.objects += objs.values();
                 result.moduleApis += apis;
@@ -395,6 +384,7 @@ QFuture<PluginDumper::QmlTypeDescription> PluginDumper::loadQmlTypeDescription(c
 
         future.reportFinished(&result);
     });
+    m_modelManager->addFuture(future);
 
     return future;
 }
@@ -421,16 +411,17 @@ QString PluginDumper::buildQmltypesPath(const QString &name) const
         version = m.captured("major") + QLatin1Char('.') + m.captured("minor");
     }
 
-    const QString path = modulePath(qualifiedName, version, m_modelManager->importPathsNames());
+    const QStringList paths = modulePaths(qualifiedName, version, m_modelManager->importPathsNames());
 
-    if (path.isEmpty())
+    if (paths.isEmpty())
         return QString();
 
-    QDirIterator it(path, QStringList { "*.qmltypes" }, QDir::Files);
-
-    if (it.hasNext())
-        return it.next();
+    for (const QString &path : paths) {
+        QDirIterator it(path, QStringList { "*.qmltypes" }, QDir::Files);
 
+        if (it.hasNext())
+            return it.next();
+    }
     return QString();
 }
 
@@ -444,27 +435,26 @@ QString PluginDumper::buildQmltypesPath(const QString &name) const
  * Recursively load type descriptions of dependencies, collecting results
  * in \a objects.
  */
-QFuture<PluginDumper::DependencyInfo> PluginDumper::loadDependencies(const QStringList &dependencies,
-                                                                     QSharedPointer<QSet<QString>> visited) const
+QFuture<PluginDumper::DependencyInfo> PluginDumper::loadDependencies(const FilePaths &dependencies,
+                                                                     QSharedPointer<QSet<FilePath>> visited) const
 {
     auto iface = QSharedPointer<QFutureInterface<PluginDumper::DependencyInfo>>(new QFutureInterface<PluginDumper::DependencyInfo>);
 
-    if (visited.isNull()) {
-        visited = QSharedPointer<QSet<QString>>(new QSet<QString>());
-    }
+    if (visited.isNull())
+        visited = QSharedPointer<QSet<FilePath>>(new QSet<FilePath>());
 
-    QStringList dependenciesPaths;
+    FilePaths dependenciesPaths;
     QString path;
-    for (const QString &name: dependencies) {
-        path = buildQmltypesPath(name);
+    for (const FilePath &name : dependencies) {
+        path = buildQmltypesPath(name.toString());
         if (!path.isNull())
-            dependenciesPaths << path;
+            dependenciesPaths << FilePath::fromString(path);
         visited->insert(name);
     }
 
     Utils::onFinished(loadQmlTypeDescription(dependenciesPaths), const_cast<PluginDumper*>(this), [=] (const QFuture<PluginDumper::QmlTypeDescription> &typesFuture) {
         PluginDumper::QmlTypeDescription typesResult = typesFuture.result();
-        QStringList newDependencies = typesResult.dependencies;
+        FilePaths newDependencies = Utils::transform(typesResult.dependencies, &FilePath::fromString);
         newDependencies = Utils::toList(Utils::toSet(newDependencies) - *visited.data());
         if (!newDependencies.isEmpty()) {
             Utils::onFinished(loadDependencies(newDependencies, visited),
@@ -564,7 +554,7 @@ static void applyQt515MissingImportWorkaround(const QString &path, LibraryInfo &
 }
 
 void PluginDumper::prepareLibraryInfo(LibraryInfo &libInfo,
-                                      const QString &libraryPath,
+                                      const FilePath &libraryPath,
                                       const QStringList &deps,
                                       const QStringList &errors,
                                       const QStringList &warnings,
@@ -588,21 +578,22 @@ void PluginDumper::prepareLibraryInfo(LibraryInfo &libInfo,
     if (!warnings.isEmpty())
         printParseWarnings(libraryPath, warnings.join(QLatin1String("\n")));
 
-    applyQt515MissingImportWorkaround(libraryPath, libInfo);
+    applyQt515MissingImportWorkaround(libraryPath.toString(), libInfo);
 
     libInfo.updateFingerprint();
 }
 
-void PluginDumper::loadQmltypesFile(const QStringList &qmltypesFilePaths,
-                                    const QString &libraryPath,
+void PluginDumper::loadQmltypesFile(const FilePaths &qmltypesFilePaths,
+                                    const FilePath &libraryPath,
                                     QmlJS::LibraryInfo libraryInfo)
 {
-    auto future = Utils::onFinished(loadQmlTypeDescription(qmltypesFilePaths), this, [=](const QFuture<PluginDumper::QmlTypeDescription> &typesFuture)
+    Utils::onFinished(loadQmlTypeDescription(qmltypesFilePaths), this, [=](const QFuture<PluginDumper::QmlTypeDescription> &typesFuture)
     {
         PluginDumper::QmlTypeDescription typesResult = typesFuture.result();
         if (!typesResult.dependencies.isEmpty())
         {
-            auto depFuture = Utils::onFinished(loadDependencies(typesResult.dependencies, QSharedPointer<QSet<QString>>()), this,
+            Utils::onFinished(loadDependencies(Utils::transform(typesResult.dependencies, &FilePath::fromString),
+                                               QSharedPointer<QSet<FilePath>>()), this,
                               [typesResult, libraryInfo, libraryPath, this] (const QFuture<PluginDumper::DependencyInfo> &loadFuture)
             {
                 PluginDumper::DependencyInfo loadResult = loadFuture.result();
@@ -620,7 +611,6 @@ void PluginDumper::loadQmltypesFile(const QStringList &qmltypesFilePaths,
                                    typesResult.moduleApis, objects);
                 m_modelManager->updateLibraryInfo(libraryPath, libInfo);
             });
-            m_modelManager->addFuture(depFuture);
         } else {
             QmlJS::LibraryInfo libInfo = libraryInfo;
             prepareLibraryInfo(libInfo, libraryPath, typesResult.dependencies,
@@ -629,22 +619,18 @@ void PluginDumper::loadQmltypesFile(const QStringList &qmltypesFilePaths,
             m_modelManager->updateLibraryInfo(libraryPath, libInfo);
         }
     });
-    m_modelManager->addFuture(future);
 }
 
-void PluginDumper::runQmlDump(const QmlJS::ModelManagerInterface::ProjectInfo &info,
-    const QStringList &arguments, const QString &importPath)
+void PluginDumper::runQmlDump(const ModelManagerInterface::ProjectInfo &info,
+    const QStringList &arguments, const FilePath &importPath)
 {
-    QDir wd = QDir(importPath);
-    wd.cdUp();
-    QProcess *process = new QProcess(this);
-    process->setEnvironment(info.qmlDumpEnvironment.toStringList());
-    QString workingDir = wd.canonicalPath();
-    process->setWorkingDirectory(workingDir);
-    connect(process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
-            this, &PluginDumper::qmlPluginTypeDumpDone);
-    connect(process, &QProcess::errorOccurred, this, &PluginDumper::qmlPluginTypeDumpError);
-    process->start(info.qmlDumpPath, arguments);
+    auto process = new QtcProcess(this);
+    process->setEnvironment(info.qmlDumpEnvironment);
+    process->setWorkingDirectory(importPath);
+    process->setCommand({info.qmlDumpPath, arguments});
+    connect(process, &QtcProcess::finished, this, [this, process] { qmlPluginTypeDumpDone(process); });
+    connect(process, &QtcProcess::errorOccurred, this, [this, process] { qmlPluginTypeDumpError(process); });
+    process->start();
     m_runningQmldumps.insert(process, importPath);
 }
 
@@ -792,3 +778,5 @@ QString PluginDumper::resolvePlugin(const QDir &qmldirPath, const QString &qmldi
     }
     return resolvePlugin(qmldirPath, qmldirPluginPath, baseName, validSuffixList, prefix);
 }
+
+} // QmlJS
