@@ -239,6 +239,8 @@ StaticAnalysisMessages::StaticAnalysisMessages()
             tr("States are only supported in the root item in a Qt Quick UI form."));
     newMsg(ErrReferenceToParentItemNotSupportedInQmlUi, Error,
             tr("Referencing the parent of the root item is not supported in a Qt Quick UI form."));
+    newMsg(ErrDoNotMixTranslationFunctionsInQmlUi, Error,
+           tr("Do not mix translation functions in a Qt Quick UI form."));
     newMsg(StateCannotHaveChildItem, Error,
             tr("A State cannot have a child item (%1)."), 1);
     newMsg(WarnDuplicateImport, Warning,
@@ -247,6 +249,12 @@ StaticAnalysisMessages::StaticAnalysisMessages()
            tr("Hit maximum recursion limit when visiting AST."));
     newMsg(ErrTypeIsInstantiatedRecursively, Error,
             tr("Type cannot be instantiated recursively (%1)."), 1);
+    newMsg(WarnLogicalValueDoesNotDependOnValues, Warning,
+           tr("Logical value does not depend on actual values"));
+    newMsg(ErrToManyComponentChildren, Error,
+           tr("Components are only allowed to have a single child element."));
+    newMsg(WarnComponentRequiresChildren, Warning,
+           tr("Components require a child element."));
 }
 
 } // anonymous namespace
