@@ -41,6 +41,8 @@ class Database;
 class SQLITE_EXPORT DatabaseBackend
 {
 public:
+    using BusyHandler = std::function<bool(int count)>;
+
     DatabaseBackend(Database &database);
     ~DatabaseBackend();
 
@@ -66,6 +68,9 @@ public:
     void setJournalMode(JournalMode journalMode);
     JournalMode journalMode();
 
+    void setLockingMode(LockingMode lockingMode);
+    LockingMode lockingMode() const;
+
     Utils::SmallStringVector columnNames(Utils::SmallStringView tableName);
 
     int changesCount() const;
@@ -76,8 +81,8 @@ public:
 
     void execute(Utils::SmallStringView sqlStatement);
 
-    template <typename Type>
-    Type toValue(Utils::SmallStringView sqlStatement);
+    template<typename Type>
+    Type toValue(Utils::SmallStringView sqlStatement) const;
 
     static int openMode(OpenMode);
 
@@ -90,15 +95,15 @@ public:
         void (*callback)(void *object, int, char const *database, char const *, long long rowId));
     void resetUpdateHook();
 
+    void setBusyHandler(BusyHandler &&busyHandler);
+
+    void registerBusyHandler();
+
 protected:
     bool databaseIsOpen() const;
 
     void setPragmaValue(Utils::SmallStringView pragma, Utils::SmallStringView value);
-    Utils::SmallString pragmaValue(Utils::SmallStringView pragma);
-
-    void registerBusyHandler();
-    void registerRankingFunction();
-    static int busyHandlerCallback(void*, int counter);
+    Utils::SmallString pragmaValue(Utils::SmallStringView pragma) const;
 
     void checkForOpenDatabaseWhichCanBeClosed();
     void checkDatabaseClosing(int resultCode);
@@ -126,6 +131,7 @@ protected:
 private:
     Database &m_database;
     sqlite3 *m_databaseHandle;
+    BusyHandler m_busyHandler;
 };
 
 } // namespace Sqlite
