@@ -28,18 +28,31 @@
 #include "sqlitebasestatement.h"
 
 namespace Sqlite {
-
-class SQLITE_EXPORT WriteStatement : protected StatementImplementation<BaseStatement>
+template<int BindParameterCount = 0>
+class WriteStatement : protected StatementImplementation<BaseStatement, -1, BindParameterCount>
 {
-public:
-    explicit WriteStatement(Utils::SmallStringView sqlStatement, Database &database);
+    using Base = StatementImplementation<BaseStatement, -1, BindParameterCount>;
 
-    using StatementImplementation::execute;
-    using StatementImplementation::database;
-    using StatementImplementation::write;
+public:
+    WriteStatement(Utils::SmallStringView sqlStatement, Database &database)
+        : Base(sqlStatement, database)
+    {
+        checkIsWritableStatement();
+        Base::checkBindingParameterCount(BindParameterCount);
+        Base::checkColumnCount(0);
+    }
+
+    using Base::database;
+    using Base::execute;
+    using Base::write;
 
 protected:
-    void checkIsWritableStatement();
+    void checkIsWritableStatement()
+    {
+        if (Base::isReadOnlyStatement())
+            throw NotWriteSqlStatement(
+                "SqliteStatement::SqliteWriteStatement: is not a writable statement!");
+    }
 };
 
 } // namespace Sqlite
