@@ -32,44 +32,40 @@
 
 #include <QCoreApplication>
 #include <QList>
-#include <QMutex>
-#include <QMutexLocker>
 #include <QObject>
 #include <QThread>
 #include <QTimer>
 
 namespace QSsh {
 namespace Internal {
-class UnaquiredConnection {
+
+class UnacquiredConnection {
 public:
-    UnaquiredConnection(SshConnection *conn) : connection(conn), scheduledForRemoval(false) {}
+    UnacquiredConnection(SshConnection *conn) : connection(conn), scheduledForRemoval(false) {}
 
     SshConnection *connection;
     bool scheduledForRemoval;
 };
-bool operator==(const UnaquiredConnection &c1, const UnaquiredConnection &c2) {
+bool operator==(const UnacquiredConnection &c1, const UnacquiredConnection &c2) {
     return c1.connection == c2.connection;
 }
-bool operator!=(const UnaquiredConnection &c1, const UnaquiredConnection &c2) {
+bool operator!=(const UnacquiredConnection &c1, const UnacquiredConnection &c2) {
     return !(c1 == c2);
 }
 
-class SshConnectionManager : public QObject
+class SshConnectionManagerPrivate : public QObject
 {
-    Q_OBJECT
-
 public:
-    SshConnectionManager()
+    SshConnectionManagerPrivate()
     {
-        moveToThread(QCoreApplication::instance()->thread());
         connect(&m_removalTimer, &QTimer::timeout,
-                this, &SshConnectionManager::removeInactiveConnections);
+                this, &SshConnectionManagerPrivate::removeInactiveConnections);
         m_removalTimer.start(SshSettings::connectionSharingTimeout() * 1000 * 60 / 2);
     }
 
-    ~SshConnectionManager()
+    ~SshConnectionManagerPrivate() override
     {
-        foreach (const UnaquiredConnection &connection, m_unacquiredConnections) {
+        for (const UnacquiredConnection &connection : qAsConst(m_unacquiredConnections)) {
             disconnect(connection.connection, nullptr, this, nullptr);
             delete connection.connection;
         }
@@ -80,16 +76,11 @@ public:
 
     SshConnection *acquireConnection(const SshConnectionParameters &sshParams)
     {
-        QMutexLocker locker(&m_listMutex);
-
         // Check in-use connections:
-        foreach (SshConnection * const connection, m_acquiredConnections) {
+        for (SshConnection * const connection : qAsConst(m_acquiredConnections)) {
             if (connection->connectionParameters() != sshParams)
                 continue;
 
-            if (connection->thread() != QThread::currentThread())
-                continue;
-
             if (connection->sharingEnabled() != SshSettings::connectionSharingEnabled())
                 continue;
 
@@ -101,20 +92,13 @@ public:
         }
 
         // Check cached open connections:
-        foreach (const UnaquiredConnection &c, m_unacquiredConnections) {
-            SshConnection * const connection = c.connection;
+        for (int i = 0; i < m_unacquiredConnections.count(); ++i) {
+            SshConnection * const connection = m_unacquiredConnections.at(i).connection;
             if (connection->state() != SshConnection::Connected
                     || connection->connectionParameters() != sshParams)
                 continue;
 
-            if (connection->thread() != QThread::currentThread()) {
-                QMetaObject::invokeMethod(this, "switchToCallerThread",
-                    Qt::BlockingQueuedConnection,
-                    Q_ARG(SshConnection *, connection),
-                    Q_ARG(QObject *, QThread::currentThread()));
-            }
-
-            m_unacquiredConnections.removeOne(c);
+            m_unacquiredConnections.removeAt(i);
             m_acquiredConnections.append(connection);
             return connection;
         }
@@ -122,43 +106,30 @@ public:
         // create a new connection:
         SshConnection * const connection = new SshConnection(sshParams);
         connect(connection, &SshConnection::disconnected,
-                this, &SshConnectionManager::cleanup);
-        m_acquiredConnections.append(connection);
+                this, &SshConnectionManagerPrivate::cleanup);
+        if (SshSettings::connectionSharingEnabled())
+            m_acquiredConnections.append(connection);
 
         return connection;
     }
 
     void releaseConnection(SshConnection *connection)
     {
-        QMutexLocker locker(&m_listMutex);
-
-        const bool wasAquired = m_acquiredConnections.removeOne(connection);
-        QTC_ASSERT(wasAquired, return);
+        const bool wasAcquired = m_acquiredConnections.removeOne(connection);
+        QTC_ASSERT(wasAcquired == connection->sharingEnabled(), return);
         if (m_acquiredConnections.contains(connection))
             return;
 
         bool doDelete = false;
-        connection->moveToThread(QCoreApplication::instance()->thread());
-        if (m_deprecatedConnections.removeOne(connection)
+        if (!connection->sharingEnabled()) {
+            doDelete = true;
+        } else if (m_deprecatedConnections.removeOne(connection)
                 || connection->state() != SshConnection::Connected) {
             doDelete = true;
         } else {
-            QTC_ASSERT(!m_unacquiredConnections.contains(UnaquiredConnection(connection)), return);
-
-            // It can happen that two or more connections with the same parameters were acquired
-            // if the clients were running in different threads. Only keep one of them in
-            // such a case.
-            bool haveConnection = false;
-            foreach (const UnaquiredConnection &c, m_unacquiredConnections) {
-                if (c.connection->connectionParameters() == connection->connectionParameters()) {
-                    haveConnection = true;
-                    break;
-                }
-            }
-            if (!haveConnection)
-                m_unacquiredConnections.append(UnaquiredConnection(connection));
-            else
-                doDelete = true;
+            UnacquiredConnection uc(connection);
+            QTC_ASSERT(!m_unacquiredConnections.contains(uc), return);
+            m_unacquiredConnections.append(uc);
         }
 
         if (doDelete) {
@@ -170,8 +141,6 @@ public:
 
     void forceNewConnection(const SshConnectionParameters &sshParams)
     {
-        QMutexLocker locker(&m_listMutex);
-
         for (int i = 0; i < m_unacquiredConnections.count(); ++i) {
             SshConnection * const connection = m_unacquiredConnections.at(i).connection;
             if (connection->connectionParameters() == sshParams) {
@@ -182,7 +151,7 @@ public:
             }
         }
 
-        foreach (SshConnection * const connection, m_acquiredConnections) {
+        for (SshConnection * const connection : qAsConst(m_acquiredConnections)) {
             if (connection->connectionParameters() == sshParams) {
                 if (!m_deprecatedConnections.contains(connection))
                     m_deprecatedConnections.append(connection);
@@ -191,20 +160,13 @@ public:
     }
 
 private:
-    Q_INVOKABLE void switchToCallerThread(SshConnection *connection, QObject *threadObj)
-    {
-        connection->moveToThread(qobject_cast<QThread *>(threadObj));
-    }
-
     void cleanup()
     {
-        QMutexLocker locker(&m_listMutex);
-
         SshConnection *currentConnection = qobject_cast<SshConnection *>(sender());
         if (!currentConnection)
             return;
 
-        if (m_unacquiredConnections.removeOne(UnaquiredConnection(currentConnection))) {
+        if (m_unacquiredConnections.removeOne(UnacquiredConnection(currentConnection))) {
             disconnect(currentConnection, nullptr, this, nullptr);
             currentConnection->deleteLater();
         }
@@ -212,9 +174,8 @@ private:
 
     void removeInactiveConnections()
     {
-        QMutexLocker locker(&m_listMutex);
         for (int i = m_unacquiredConnections.count() - 1; i >= 0; --i) {
-            UnaquiredConnection &c = m_unacquiredConnections[i];
+            UnacquiredConnection &c = m_unacquiredConnections[i];
             if (c.scheduledForRemoval) {
                 disconnect(c.connection, nullptr, this, nullptr);
                 c.connection->deleteLater();
@@ -229,44 +190,41 @@ private:
     // We expect the number of concurrently open connections to be small.
     // If that turns out to not be the case, we can still use a data
     // structure with faster access.
-    QList<UnaquiredConnection> m_unacquiredConnections;
+    QList<UnacquiredConnection> m_unacquiredConnections;
 
     // Can contain the same connection more than once; this acts as a reference count.
     QList<SshConnection *> m_acquiredConnections;
 
     QList<SshConnection *> m_deprecatedConnections;
-    QMutex m_listMutex;
     QTimer m_removalTimer;
 };
 
 } // namespace Internal
 
-static QMutex instanceMutex;
-
-static Internal::SshConnectionManager &instance()
+SshConnectionManager::SshConnectionManager()
+    : d(new Internal::SshConnectionManagerPrivate())
 {
-    static Internal::SshConnectionManager manager;
-    return manager;
+    QTC_CHECK(QThread::currentThread() == qApp->thread());
 }
 
-SshConnection *acquireConnection(const SshConnectionParameters &sshParams)
+SshConnectionManager::~SshConnectionManager()
 {
-    QMutexLocker locker(&instanceMutex);
-    return instance().acquireConnection(sshParams);
+    delete d;
 }
 
-void releaseConnection(SshConnection *connection)
+SshConnection *SshConnectionManager::acquireConnection(const SshConnectionParameters &sshParams)
 {
-    QMutexLocker locker(&instanceMutex);
-    instance().releaseConnection(connection);
+    return instance()->d->acquireConnection(sshParams);
 }
 
-void forceNewConnection(const SshConnectionParameters &sshParams)
+void SshConnectionManager::releaseConnection(SshConnection *connection)
 {
-    QMutexLocker locker(&instanceMutex);
-    instance().forceNewConnection(sshParams);
+    instance()->d->releaseConnection(connection);
+}
+
+void SshConnectionManager::forceNewConnection(const SshConnectionParameters &sshParams)
+{
+    instance()->d->forceNewConnection(sshParams);
 }
 
 } // namespace QSsh
-
-#include "sshconnectionmanager.moc"
