@@ -28,6 +28,7 @@
 #include <utils/algorithm.h>
 #include <utils/environment.h>
 #include <utils/hostosinfo.h>
+#include <utils/commandline.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
 #include <utils/winutils.h>
@@ -82,7 +83,7 @@ public:
 
     static QString m_defaultConsoleProcess;
     ConsoleProcess::Mode m_mode = ConsoleProcess::Run;
-    QString m_workingDir;
+    FilePath m_workingDir;
     Environment m_environment;
     qint64 m_appPid = 0;
     int m_appCode;
@@ -94,6 +95,7 @@ public:
     QProcess::ProcessError m_error = QProcess::UnknownError;
     QString m_errorString;
     bool m_abortOnMetaChars = true;
+    bool m_runAsRoot = false;
     QSettings *m_settings = nullptr;
 
     // Used on Unix only
@@ -231,11 +233,11 @@ TerminalCommand ConsoleProcess::terminalEmulator(const QSettings *settings)
             const QString value = settings->value("General/TerminalEmulator").toString().trimmed();
             if (!value.isEmpty()) {
                 // split off command and options
-                const QStringList splitCommand = QtcProcess::splitArgs(value);
+                const QStringList splitCommand = ProcessArgs::splitArgs(value);
                 if (QTC_GUARD(!splitCommand.isEmpty())) {
                     const QString command = splitCommand.first();
                     const QStringList quotedArgs = Utils::transform(splitCommand.mid(1),
-                                                                    &QtcProcess::quoteArgUnix);
+                                                                    &ProcessArgs::quoteArgUnix);
                     const QString options = quotedArgs.join(' ');
                     return {command, "", options};
                 }
@@ -357,7 +359,7 @@ bool ConsoleProcess::startTerminalEmulator(QSettings *settings, const QString &w
     const TerminalCommand term = terminalEmulator(settings);
     QProcess process;
     process.setProgram(term.command);
-    process.setArguments(QtcProcess::splitArgs(term.openArgs));
+    process.setArguments(ProcessArgs::splitArgs(term.openArgs));
     process.setProcessEnvironment(env.toProcessEnvironment());
     process.setWorkingDirectory(workingDir);
 
@@ -393,11 +395,9 @@ bool ConsoleProcess::start()
         pcmd = d->m_commandLine.executable().toString();
         pargs = d->m_commandLine.arguments();
     } else {
-        QtcProcess::Arguments outArgs;
-        QtcProcess::prepareCommand(d->m_commandLine.executable().toString(),
-                                   d->m_commandLine.arguments(),
-                                   &pcmd, &outArgs, OsTypeWindows,
-                                   &d->m_environment, &d->m_workingDir);
+        ProcessArgs outArgs;
+        ProcessArgs::prepareCommand(d->m_commandLine, &pcmd, &outArgs,
+                                    &d->m_environment, &d->m_workingDir);
         pargs = outArgs.toWindowsArgs();
     }
 
@@ -451,7 +451,7 @@ bool ConsoleProcess::start()
             return false;
         }
         d->m_tempFile->flush();
-}
+    }
 
     STARTUPINFO si;
     ZeroMemory(&si, sizeof(si));
@@ -460,7 +460,7 @@ bool ConsoleProcess::start()
     d->m_pid = new PROCESS_INFORMATION;
     ZeroMemory(d->m_pid, sizeof(PROCESS_INFORMATION));
 
-    QString workDir = QDir::toNativeSeparators(workingDirectory());
+    QString workDir = workingDirectory().toUserOutput();
     if (!workDir.isEmpty() && !workDir.endsWith(QLatin1Char('\\')))
         workDir.append(QLatin1Char('\\'));
 
@@ -496,19 +496,19 @@ bool ConsoleProcess::start()
 
 #else
 
-    QtcProcess::SplitError perr;
-    QtcProcess::Arguments pargs = QtcProcess::prepareArgs(d->m_commandLine.arguments(),
-                                                          &perr,
-                                                          HostOsInfo::hostOs(),
-                                                          &d->m_environment,
-                                                          &d->m_workingDir,
-                                                          d->m_abortOnMetaChars);
+    ProcessArgs::SplitError perr;
+    ProcessArgs pargs = ProcessArgs::prepareArgs(d->m_commandLine.arguments(),
+                                                 &perr,
+                                                 HostOsInfo::hostOs(),
+                                                 &d->m_environment,
+                                                 &d->m_workingDir,
+                                                 d->m_abortOnMetaChars);
 
     QString pcmd;
-    if (perr == QtcProcess::SplitOk) {
+    if (perr == ProcessArgs::SplitOk) {
         pcmd = d->m_commandLine.executable().toString();
     } else {
-        if (perr != QtcProcess::FoundMeta) {
+        if (perr != ProcessArgs::FoundMeta) {
             emitError(QProcess::FailedToStart, tr("Quoting error in command."));
             return false;
         }
@@ -519,20 +519,20 @@ bool ConsoleProcess::start()
             return false;
         }
         pcmd = qEnvironmentVariable("SHELL", "/bin/sh");
-        pargs = QtcProcess::Arguments::createUnixArgs(
-                        {"-c", (QtcProcess::quoteArg(d->m_commandLine.executable().toString())
+        pargs = ProcessArgs::createUnixArgs(
+                        {"-c", (ProcessArgs::quoteArg(d->m_commandLine.executable().toString())
                          + ' ' + d->m_commandLine.arguments())});
     }
 
-    QtcProcess::SplitError qerr;
+    ProcessArgs::SplitError qerr;
     const TerminalCommand terminal = terminalEmulator(d->m_settings);
-    const QtcProcess::Arguments terminalArgs = QtcProcess::prepareArgs(terminal.executeArgs,
-                                                                       &qerr,
-                                                                       HostOsInfo::hostOs(),
-                                                                       &d->m_environment,
-                                                                       &d->m_workingDir);
-    if (qerr != QtcProcess::SplitOk) {
-        emitError(QProcess::FailedToStart, qerr == QtcProcess::BadQuoting
+    const ProcessArgs terminalArgs = ProcessArgs::prepareArgs(terminal.executeArgs,
+                                                              &qerr,
+                                                              HostOsInfo::hostOs(),
+                                                              &d->m_environment,
+                                                              &d->m_workingDir);
+    if (qerr != ProcessArgs::SplitOk) {
+        emitError(QProcess::FailedToStart, qerr == ProcessArgs::BadQuoting
                           ? tr("Quoting error in terminal command.")
                           : tr("Terminal command may not be a shell command."));
         return false;
@@ -545,6 +545,7 @@ bool ConsoleProcess::start()
     }
 
     d->m_environment.unset(QLatin1String("TERM"));
+
     const QStringList env = d->m_environment.toStringList();
     if (!env.isEmpty()) {
         d->m_tempFile = new QTemporaryFile();
@@ -570,21 +571,26 @@ bool ConsoleProcess::start()
     }
 
     const QString stubPath = QCoreApplication::applicationDirPath()
-            + QLatin1String("/" QTC_REL_TOOLS_PATH "/qtcreator_process_stub");
-    QStringList allArgs = terminalArgs.toUnixArgs()
-                          << stubPath
-                          << modeOption(d->m_mode)
-                          << d->m_stubServer.fullServerName()
-                          << msgPromptToClose()
-                          << workingDirectory()
-                          << (d->m_tempFile ? d->m_tempFile->fileName() : QString())
-                          << QString::number(getpid())
-                          << pcmd
-                          << pargs.toUnixArgs();
+            + QLatin1String("/" RELATIVE_LIBEXEC_PATH "/qtcreator_process_stub");
+
+    QStringList allArgs = terminalArgs.toUnixArgs();
+    if (d->m_runAsRoot)
+        allArgs << "sudo" << "-A";
+
+    allArgs << stubPath
+            << modeOption(d->m_mode)
+            << d->m_stubServer.fullServerName()
+            << msgPromptToClose()
+            << workingDirectory().path()
+            << (d->m_tempFile ? d->m_tempFile->fileName() : QString())
+            << QString::number(getpid())
+            << pcmd
+            << pargs.toUnixArgs();
 
     if (terminal.needsQuotes)
-        allArgs = QStringList { QtcProcess::joinArgs(allArgs) };
+        allArgs = QStringList { ProcessArgs::joinArgs(allArgs) };
 
+    d->m_process.setEnvironment(env);
     d->m_process.start(terminal.command, allArgs);
     if (!d->m_process.waitForStarted()) {
         stubServerShutdown();
@@ -604,6 +610,30 @@ bool ConsoleProcess::start()
     return true;
 }
 
+void Utils::ConsoleProcess::kickoffProcess()
+{
+#ifdef Q_OS_WIN
+    // Not used.
+#else
+    if (d->m_stubSocket && d->m_stubSocket->isWritable()) {
+        d->m_stubSocket->write("c", 1);
+        d->m_stubSocket->flush();
+    }
+#endif
+}
+
+void ConsoleProcess::interruptProcess()
+{
+#ifdef Q_OS_WIN
+    // Not used.
+#else
+    if (d->m_stubSocket && d->m_stubSocket->isWritable()) {
+        d->m_stubSocket->write("i", 1);
+        d->m_stubSocket->flush();
+    }
+#endif
+}
+
 void ConsoleProcess::killProcess()
 {
 #ifdef Q_OS_WIN
@@ -926,12 +956,12 @@ QProcess::ExitStatus ConsoleProcess::exitStatus() const
     return d->m_appStatus;
 }
 
-void ConsoleProcess::setWorkingDirectory(const QString &dir)
+void ConsoleProcess::setWorkingDirectory(const FilePath &dir)
 {
     d->m_workingDir = dir;
 }
 
-QString ConsoleProcess::workingDirectory() const
+FilePath ConsoleProcess::workingDirectory() const
 {
     return d->m_workingDir;
 }
@@ -946,6 +976,11 @@ Environment ConsoleProcess::environment() const
     return d->m_environment;
 }
 
+void Utils::ConsoleProcess::setRunAsRoot(bool on)
+{
+    d->m_runAsRoot = on;
+}
+
 QProcess::ProcessError ConsoleProcess::error() const
 {
     return d->m_error;
@@ -988,9 +1023,9 @@ QString ConsoleProcess::msgUnexpectedOutput(const QByteArray &what)
     return tr("Unexpected output from helper program (%1).").arg(QString::fromLatin1(what));
 }
 
-QString ConsoleProcess::msgCannotChangeToWorkDir(const QString & dir, const QString &why)
+QString ConsoleProcess::msgCannotChangeToWorkDir(const FilePath &dir, const QString &why)
 {
-    return tr("Cannot change to working directory \"%1\": %2").arg(dir, why);
+    return tr("Cannot change to working directory \"%1\": %2").arg(dir.toString(), why);
 }
 
 QString ConsoleProcess::msgCannotExecute(const QString & p, const QString &why)
