@@ -78,7 +78,7 @@ static int commonOverlap(const QString &text1, const QString &text2)
     const int text2Count = text2.count();
     const int maxCount = qMin(text1Count, text2Count);
     while (i < maxCount) {
-        if (text1.midRef(text1Count - maxCount + i) == text2.leftRef(maxCount - i))
+        if (QStringView(text1).mid(text1Count - maxCount + i) == QStringView(text2).left(maxCount - i))
             return maxCount - i;
         i++;
     }
@@ -89,14 +89,13 @@ static QList<Diff> decode(const QList<Diff> &diffList, const QStringList &lines)
 {
     QList<Diff> newDiffList;
     newDiffList.reserve(diffList.count());
-    for (Diff diff : diffList) {
+    for (const Diff &diff : diffList) {
         QString text;
         for (QChar c : diff.text) {
             const int idx = static_cast<ushort>(c.unicode());
             text += lines.value(idx);
         }
-        diff.text = text;
-        newDiffList.append(diff);
+        newDiffList.append({diff.command, text});
     }
     return newDiffList;
 }
@@ -159,7 +158,7 @@ static QList<Diff> cleanupOverlaps(const QList<Diff> &diffList)
         Diff thisDiff = diffList.at(i);
         Diff nextDiff = i < diffList.count() - 1
                 ? diffList.at(i + 1)
-                : Diff(Diff::Equal, QString());
+                : Diff(Diff::Equal);
         if (thisDiff.command == Diff::Delete
                 && nextDiff.command == Diff::Insert) {
             const int delInsOverlap = commonOverlap(thisDiff.text, nextDiff.text);
@@ -206,7 +205,7 @@ static int cleanupSemanticsScore(const QString &text1, const QString &text2)
     const QRegularExpression blankLineStart("^\\r?\\n\\r?\\n");
     const QRegularExpression sentenceEnd("\\. $");
 
-    if (!text1.count() || !text2.count()) // Edges
+    if (text1.isEmpty() || text2.isEmpty()) // Edges
         return 6;
 
     const QChar char1 = text1[text1.count() - 1];
@@ -298,7 +297,7 @@ QList<Diff> Differ::moveWhitespaceIntoEqualities(const QList<Diff> &input)
                     }
                     if (j > 0) {
                         // diff starts with j whitespaces, move them to the previous diff
-                        previousDiff.text.append(diff.text.leftRef(j));
+                        previousDiff.text.append(diff.text.left(j));
                         diff.text = diff.text.mid(j);
                     }
                 }
@@ -551,13 +550,12 @@ static QString encodeExpandedWhitespace(const QString &leftEquality,
             return QString(); // equalities broken
         }
 
-        if ((leftWhitespaces.count() && !rightWhitespaces.count())
-                || (!leftWhitespaces.count() && rightWhitespaces.count())) {
+        if (leftWhitespaces.isEmpty() ^ rightWhitespaces.isEmpty()) {
             // there must be at least 1 corresponding whitespace, equalities broken
             return QString();
         }
 
-        if (leftWhitespaces.count() && rightWhitespaces.count()) {
+        if (!leftWhitespaces.isEmpty() && !rightWhitespaces.isEmpty()) {
             const int replacementPosition = output.count();
             const int replacementSize = qMax(leftWhitespaces.count(), rightWhitespaces.count());
             const QString replacement(replacementSize, ' ');
@@ -698,7 +696,7 @@ static bool diffWithWhitespaceExpandedInEqualities(const QList<Diff> &leftInput,
     }
 
     Differ differ;
-    const QList<Diff> &diffList = differ.cleanupSemantics(
+    const QList<Diff> &diffList = Differ::cleanupSemantics(
                 differ.diff(leftText, rightText));
 
     QList<Diff> leftDiffList;
@@ -723,10 +721,10 @@ static void appendWithEqualitiesSquashed(const QList<Diff> &leftInput,
                              QList<Diff> *leftOutput,
                              QList<Diff> *rightOutput)
 {
-    if (leftInput.count()
-            && rightInput.count()
-            && leftOutput->count()
-            && rightOutput->count()
+    if (!leftInput.isEmpty()
+            && !rightInput.isEmpty()
+            && !leftOutput->isEmpty()
+            && !rightOutput->isEmpty()
             && leftInput.first().command == Diff::Equal
             && rightInput.first().command == Diff::Equal
             && leftOutput->last().command == Diff::Equal
@@ -883,7 +881,7 @@ void Differ::diffBetweenEqualities(const QList<Diff> &leftInput,
                     && previousRightDiff.command == Diff::Insert) {
                 Differ differ;
                 differ.setDiffMode(Differ::CharMode);
-                const QList<Diff> commonOutput = differ.cleanupSemantics(
+                const QList<Diff> commonOutput = cleanupSemantics(
                             differ.diff(previousLeftDiff.text, previousRightDiff.text));
 
                 QList<Diff> outputLeftDiffList;
@@ -927,12 +925,6 @@ void Differ::diffBetweenEqualities(const QList<Diff> &leftInput,
 
 ///////////////
 
-
-Diff::Diff() :
-    command(Diff::Equal)
-{
-}
-
 Diff::Diff(Command com, const QString &txt) :
     command(com),
     text(txt)
@@ -1245,7 +1237,7 @@ QList<Diff> Differ::diffNonCharMode(const QString &text1, const QString &text2)
         } else if (diffItem.command == Diff::Insert) {
             lastInsert += diffItem.text;
         } else { // Diff::Equal
-            if (lastDelete.count() || lastInsert.count()) {
+            if (!(lastDelete.isEmpty() && lastInsert.isEmpty())) {
                 // Rediff here on char basis
                 newDiffList += preprocess1AndDiff(lastDelete, lastInsert);
 
@@ -1334,7 +1326,7 @@ QList<Diff> Differ::merge(const QList<Diff> &diffList)
         } else if (diff.command == Diff::Insert) {
             lastInsert += diff.text;
         } else { // Diff::Equal
-            if (lastDelete.count() || lastInsert.count()) {
+            if (!(lastDelete.isEmpty() && lastInsert.isEmpty())) {
 
                 // common prefix
                 const int prefixCount = commonPrefix(lastDelete, lastInsert);
@@ -1343,7 +1335,7 @@ QList<Diff> Differ::merge(const QList<Diff> &diffList)
                     lastDelete = lastDelete.mid(prefixCount);
                     lastInsert = lastInsert.mid(prefixCount);
 
-                    if (newDiffList.count()
+                    if (!newDiffList.isEmpty()
                             && newDiffList.last().command == Diff::Equal) {
                         newDiffList.last().text += prefix;
                     } else {
@@ -1362,20 +1354,20 @@ QList<Diff> Differ::merge(const QList<Diff> &diffList)
                 }
 
                 // append delete / insert / equal
-                if (lastDelete.count())
+                if (!lastDelete.isEmpty())
                     newDiffList.append(Diff(Diff::Delete, lastDelete));
-                if (lastInsert.count())
+                if (!lastInsert.isEmpty())
                     newDiffList.append(Diff(Diff::Insert, lastInsert));
-                if (diff.text.count())
+                if (!diff.text.isEmpty())
                     newDiffList.append(diff);
                 lastDelete.clear();
                 lastInsert.clear();
             } else { // join with last equal diff
-                if (newDiffList.count()
+                if (!newDiffList.isEmpty()
                         && newDiffList.last().command == Diff::Equal) {
                     newDiffList.last().text += diff.text;
                 } else {
-                    if (diff.text.count())
+                    if (!diff.text.isEmpty())
                         newDiffList.append(diff);
                 }
             }
