@@ -27,6 +27,8 @@
 
 #include "utils_global.h"
 
+#include "filepath.h"
+
 #include <QObject>
 #include <QMimeData>
 #include <QPoint>
@@ -45,28 +47,28 @@ class QTCREATOR_UTILS_EXPORT DropSupport : public QObject
     Q_OBJECT
 public:
     struct FileSpec {
-        FileSpec(const QString &path, int r = -1, int c = -1) : filePath(path), line(r), column(c) {}
-        QString filePath;
+        FileSpec(const FilePath &path, int r = -1, int c = -1) : filePath(path), line(r), column(c) {}
+        FilePath filePath;
         int line;
         int column;
     };
     // returns true if the event should be accepted
-    typedef std::function<bool(QDropEvent*,DropSupport*)> DropFilterFunction;
+    using DropFilterFunction = std::function<bool(QDropEvent*, DropSupport*)>;
 
     DropSupport(QWidget *parentWidget, const DropFilterFunction &filterFunction = DropFilterFunction());
 
     static QStringList mimeTypesForFilePaths();
 
 signals:
-    void filesDropped(const QList<Utils::DropSupport::FileSpec> &files);
+    void filesDropped(const QList<Utils::DropSupport::FileSpec> &files, const QPoint &dropPos);
     void valuesDropped(const QList<QVariant> &values, const QPoint &dropPos);
 
 public:
-    bool isFileDrop(QDropEvent *event) const;
-    bool isValueDrop(QDropEvent *event) const;
+    static bool isFileDrop(QDropEvent *event);
+    static bool isValueDrop(QDropEvent *event);
 
 protected:
-    bool eventFilter(QObject *obj, QEvent *event);
+    bool eventFilter(QObject *obj, QEvent *event) override;
 
 private:
     void emitFilesDropped();
@@ -89,7 +91,7 @@ public:
     Qt::DropAction overrideFileDropAction() const;
     bool isOverridingFileDropAction() const;
 
-    void addFile(const QString &filePath, int line = -1, int column = -1);
+    void addFile(const FilePath &filePath, int line = -1, int column = -1);
     QList<DropSupport::FileSpec> files() const;
 
     void addValue(const QVariant &value);
