@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -33,12 +28,13 @@
 
 #include <QDir>
 #include <QDebug>
+#include <QtEndian>
 
 #include <new> // std::bad_alloc
 
 namespace Utils {
 
-quint16 getHalfWord(const unsigned char *&s, const ElfData &context)
+static quint16 getHalfWord(const unsigned char *&s, const ElfData &context)
 {
     quint16 res;
     if (context.endian == Elf_ELFDATA2MSB)
@@ -49,7 +45,7 @@ quint16 getHalfWord(const unsigned char *&s, const ElfData &context)
     return res;
 }
 
-quint32 getWord(const unsigned char *&s, const ElfData &context)
+static quint32 getWord(const unsigned char *&s, const ElfData &context)
 {
     quint32 res;
     if (context.endian == Elf_ELFDATA2MSB)
@@ -60,7 +56,7 @@ quint32 getWord(const unsigned char *&s, const ElfData &context)
     return res;
 }
 
-quint64 getAddress(const unsigned char *&s, const ElfData &context)
+static quint64 getAddress(const unsigned char *&s, const ElfData &context)
 {
     quint64 res;
     if (context.elfclass == Elf_ELFCLASS32) {
@@ -79,7 +75,7 @@ quint64 getAddress(const unsigned char *&s, const ElfData &context)
     return res;
 }
 
-quint64 getOffset(const unsigned char *&s, const ElfData &context)
+static quint64 getOffset(const unsigned char *&s, const ElfData &context)
 {
     return getAddress(s, context);
 }
@@ -104,16 +100,26 @@ static void parseProgramHeader(const uchar *s, ElfProgramHeader *sh, const ElfDa
     sh->memsz = getWord(s, context);
 }
 
-ElfMapper::ElfMapper(const ElfReader *reader) : file(reader->m_binary) {}
+ElfMapper::ElfMapper(const ElfReader *reader)
+    : binary(reader->m_binary)
+{}
 
 bool ElfMapper::map()
 {
+    if (binary.needsDevice()) {
+        raw = binary.fileContents();
+        start = raw.constData();
+        fdlen = raw.size();
+        return fdlen > 0;
+    }
+
+    file.setFileName(binary.path());
     if (!file.open(QIODevice::ReadOnly))
         return false;
 
     fdlen = file.size();
     ustart = file.map(0, fdlen);
-    if (ustart == 0) {
+    if (ustart == nullptr) {
         // Try reading the data into memory instead.
         try {
             raw = file.readAll();
@@ -126,7 +132,7 @@ bool ElfMapper::map()
     return true;
 }
 
-ElfReader::ElfReader(const QString &binary)
+ElfReader::ElfReader(const FilePath &binary)
     : m_binary(binary)
 {
 }
@@ -137,10 +143,10 @@ ElfData ElfReader::readHeaders()
     return m_elfData;
 }
 
-static inline QString msgInvalidElfObject(const QString &binary, const QString &why)
+static QString msgInvalidElfObject(const FilePath &binary, const QString &why)
 {
     return ElfReader::tr("\"%1\" is an invalid ELF object (%2)")
-           .arg(QDir::toNativeSeparators(binary), why);
+           .arg(binary.toUserOutput(), why);
 }
 
 ElfReader::Result ElfReader::readIt()
@@ -157,12 +163,12 @@ ElfReader::Result ElfReader::readIt()
     const quint64 fdlen = mapper.fdlen;
 
     if (fdlen < 64) {
-        m_errorString = tr("\"%1\" is not an ELF object (file too small)").arg(QDir::toNativeSeparators(m_binary));
+        m_errorString = tr("\"%1\" is not an ELF object (file too small)").arg(m_binary.toUserOutput());
         return NotElf;
     }
 
     if (strncmp(mapper.start, "\177ELF", 4) != 0) {
-        m_errorString = tr("\"%1\" is not an ELF object").arg(QDir::toNativeSeparators(m_binary));
+        m_errorString = tr("\"%1\" is not an ELF object").arg(m_binary.toUserOutput());
         return NotElf;
     }
 
@@ -226,7 +232,7 @@ ElfReader::Result ElfReader::readIt()
     QTC_CHECK(data == mapper.ustart + (is64Bit ? 64 : 52));
 
     if (quint64(e_shnum) * e_shentsize > fdlen) {
-        const QString reason = tr("announced %n sections, each %1 bytes, exceed file size", 0, e_shnum)
+        const QString reason = tr("announced %n sections, each %1 bytes, exceed file size", nullptr, e_shnum)
                                .arg(e_shentsize);
         m_errorString = msgInvalidElfObject(m_binary, reason);
         return Corrupt;
