@@ -130,35 +130,15 @@ Environment Environment::systemEnvironment()
     return *staticSystemEnvironment();
 }
 
-const char lcMessages[] = "LC_MESSAGES";
-const char language[] = "LANGUAGE";
-const char englishLocale[] = "en_US.utf8";
-const char languageEnglishLocales[] = "en_US:en";
-
-void Environment::setupEnglishOutput(Environment *environment)
+void Environment::setupEnglishOutput()
 {
-    QTC_ASSERT(environment, return);
-    environment->set(lcMessages, englishLocale);
-    environment->set(language, languageEnglishLocales);
+    set("LC_MESSAGES", "en_US.utf8");
+    set("LANGUAGE", "en_US:en");
 }
 
-void Environment::setupEnglishOutput(QProcessEnvironment *environment)
-{
-    QTC_ASSERT(environment, return);
-    environment->insert(lcMessages, englishLocale);
-    environment->insert(language, languageEnglishLocales);
-}
-
-void Environment::setupEnglishOutput(QStringList *environment)
-{
-    QTC_ASSERT(environment, return);
-    Environment env(*environment);
-    setupEnglishOutput(&env);
-    *environment = env.toStringList();
-}
-
-FilePath Environment::searchInDirectory(const QStringList &execs, const FilePath &directory,
-                                        QSet<FilePath> &alreadyChecked) const
+static FilePath searchInDirectory(const QStringList &execs,
+                                  const FilePath &directory,
+                                  QSet<FilePath> &alreadyChecked)
 {
     const int checkedCount = alreadyChecked.count();
     alreadyChecked.insert(directory);
@@ -204,7 +184,9 @@ bool Environment::isSameExecutable(const QString &exe1, const QString &exe2) con
             const FilePath f2 = FilePath::fromString(i2);
             if (f1 == f2)
                 return true;
-            if (FileUtils::resolveSymlinks(f1) == FileUtils::resolveSymlinks(f2))
+            if (f1.needsDevice() != f2.needsDevice() || f1.scheme() != f2.scheme())
+                return false;
+            if (f1.resolveSymlinks() == f2.resolveSymlinks())
                 return true;
             if (FileUtils::fileId(f1) == FileUtils::fileId(f2))
                 return true;
@@ -218,17 +200,19 @@ QString Environment::expandedValueForKey(const QString &key) const
     return expandVariables(value(key));
 }
 
-FilePath Environment::searchInPath(const QString &executable,
-                                   const FilePaths &additionalDirs,
-                                   const PathFilter &func) const
+static FilePath searchInDirectoriesHelper(const Environment &env,
+                                          const QString &executable,
+                                          const FilePaths &dirs,
+                                          const Environment::PathFilter &func,
+                                          bool usePath)
 {
     if (executable.isEmpty())
         return FilePath();
 
-    const QString exec = QDir::cleanPath(expandVariables(executable));
+    const QString exec = QDir::cleanPath(env.expandVariables(executable));
     const QFileInfo fi(exec);
 
-    const QStringList execs = appendExeExtensions(exec);
+    const QStringList execs = env.appendExeExtensions(exec);
 
     if (fi.isAbsolute()) {
         for (const QString &path : execs) {
@@ -240,23 +224,38 @@ FilePath Environment::searchInPath(const QString &executable,
     }
 
     QSet<FilePath> alreadyChecked;
-    for (const FilePath &dir : additionalDirs) {
+    for (const FilePath &dir : dirs) {
         FilePath tmp = searchInDirectory(execs, dir, alreadyChecked);
         if (!tmp.isEmpty() && (!func || func(tmp)))
             return tmp;
     }
 
-    if (executable.contains('/'))
-        return FilePath();
+    if (usePath) {
+        if (executable.contains('/'))
+            return FilePath();
 
-    for (const FilePath &p : path()) {
-        FilePath tmp = searchInDirectory(execs, p, alreadyChecked);
-        if (!tmp.isEmpty() && (!func || func(tmp)))
-            return tmp;
+        for (const FilePath &p : env.path()) {
+            FilePath tmp = searchInDirectory(execs, p, alreadyChecked);
+            if (!tmp.isEmpty() && (!func || func(tmp)))
+                return tmp;
+        }
     }
     return FilePath();
 }
 
+FilePath Environment::searchInDirectories(const QString &executable,
+                                          const FilePaths &dirs) const
+{
+    return searchInDirectoriesHelper(*this, executable, dirs, {}, false);
+}
+
+FilePath Environment::searchInPath(const QString &executable,
+                                   const FilePaths &additionalDirs,
+                                   const PathFilter &func) const
+{
+    return searchInDirectoriesHelper(*this, executable, additionalDirs, func, true);
+}
+
 FilePaths Environment::findAllInPath(const QString &executable,
                                         const FilePaths &additionalDirs,
                                         const Environment::PathFilter &func) const
@@ -313,6 +312,11 @@ void Environment::modifySystemEnvironment(const EnvironmentItems &list)
     staticSystemEnvironment->modify(list);
 }
 
+void Environment::setSystemEnvironment(const Environment &environment)
+{
+    *staticSystemEnvironment = environment;
+}
+
 /** Expand environment variables in a string.
  *
  * Environment variables are accepted in the following forms:
@@ -417,4 +421,46 @@ optional<EnvironmentProvider> EnvironmentProvider::provider(const QByteArray &id
     return nullopt;
 }
 
+void EnvironmentChange::addSetValue(const QString &key, const QString &value)
+{
+    m_changeItems.append([key, value](Environment &env) { env.set(key, value); });
+}
+
+void EnvironmentChange::addUnsetValue(const QString &key)
+{
+    m_changeItems.append([key](Environment &env) { env.unset(key); });
+}
+
+void EnvironmentChange::addPrependToPath(const QStringList &values)
+{
+    for (int i = values.size(); --i >= 0; ) {
+        const QString value = values.at(i);
+        m_changeItems.append([value](Environment &env) { env.prependOrSetPath(value); });
+    }
+}
+
+void EnvironmentChange::addAppendToPath(const QStringList &values)
+{
+    for (const QString &value : values)
+        m_changeItems.append([value](Environment &env) { env.appendOrSetPath(value); });
+}
+
+void EnvironmentChange::addModify(const NameValueItems &items)
+{
+    m_changeItems.append([items](Environment &env) { env.modify(items); });
+}
+
+EnvironmentChange EnvironmentChange::fromFixedEnvironment(const Environment &fixedEnv)
+{
+    EnvironmentChange change;
+    change.m_changeItems.append([fixedEnv](Environment &env) { env = fixedEnv; });
+    return change;
+}
+
+void EnvironmentChange::applyToEnvironment(Environment &env) const
+{
+    for (const Item &item : m_changeItems)
+        item(env);
+}
+
 } // namespace Utils
