@@ -26,19 +26,23 @@
 #include "macroexpander.h"
 
 #include "algorithm.h"
+#include "fileutils.h"
+#include "commandline.h"
 #include "qtcassert.h"
-#include "qtcprocess.h"
 #include "stringutils.h"
 
 #include <QCoreApplication>
 #include <QDebug>
 #include <QDir>
 #include <QFileInfo>
+#include <QLoggingCategory>
 #include <QMap>
 
 namespace Utils {
 namespace Internal {
 
+static Q_LOGGING_CATEGORY(expanderLog, "qtc.utils.macroexpander", QtWarningMsg)
+
 const char kFilePathPostfix[] = ":FilePath";
 const char kPathPostfix[] = ":Path";
 const char kNativeFilePathPostfix[] = ":NativeFilePath";
@@ -82,7 +86,7 @@ public:
         return this == globalMacroExpander()->d ? false : globalMacroExpander()->d->resolveMacro(name, ret, seen);
     }
 
-    QString value(const QByteArray &variable, bool *found)
+    QString value(const QByteArray &variable, bool *found) const
     {
         MacroExpander::StringFunction sf = m_map.value(variable);
         if (sf) {
@@ -292,7 +296,11 @@ QString MacroExpander::expand(const QString &stringWithVariables) const
 
 FilePath MacroExpander::expand(const FilePath &fileNameWithVariables) const
 {
-    return FilePath::fromString(expand(fileNameWithVariables.toString()));
+    FilePath result = fileNameWithVariables;
+    result.setPath(expand(result.path()));
+    result.setHost(expand(result.host()));
+    result.setScheme(expand(result.scheme()));
+    return result;
 }
 
 QByteArray MacroExpander::expand(const QByteArray &stringWithVariables) const
@@ -322,7 +330,10 @@ QVariant MacroExpander::expandVariant(const QVariant &v) const
 
 QString MacroExpander::expandProcessArgs(const QString &argsWithVariables) const
 {
-    return QtcProcess::expandMacros(argsWithVariables, d);
+    QString result = argsWithVariables;
+    const bool ok = ProcessArgs::expandMacros(&result, d);
+    QTC_ASSERT(ok, qCDebug(expanderLog) << "Expanding failed: " << argsWithVariables);
+    return result;
 }
 
 static QByteArray fullPrefix(const QByteArray &prefix)
@@ -390,22 +401,22 @@ void MacroExpander::registerIntVariable(const QByteArray &variable,
  * \sa registerVariable(), registerIntVariable(), registerPrefix()
  */
 void MacroExpander::registerFileVariables(const QByteArray &prefix,
-    const QString &heading, const StringFunction &base, bool visibleInChooser)
+    const QString &heading, const FileFunction &base, bool visibleInChooser)
 {
     registerVariable(prefix + kFilePathPostfix,
          tr("%1: Full path including file name.").arg(heading),
-         [base]() -> QString { QString tmp = base(); return tmp.isEmpty() ? QString() : QFileInfo(tmp).filePath(); },
+         [base]() -> QString { QString tmp = base().toString(); return tmp.isEmpty() ? QString() : QFileInfo(tmp).filePath(); },
          visibleInChooser);
 
     registerVariable(prefix + kPathPostfix,
          tr("%1: Full path excluding file name.").arg(heading),
-         [base]() -> QString { QString tmp = base(); return tmp.isEmpty() ? QString() : QFileInfo(tmp).path(); },
+         [base]() -> QString { QString tmp = base().toString(); return tmp.isEmpty() ? QString() : QFileInfo(tmp).path(); },
          visibleInChooser);
 
     registerVariable(prefix + kNativeFilePathPostfix,
          tr("%1: Full path including file name, with native path separator (backslash on Windows).").arg(heading),
          [base]() -> QString {
-             QString tmp = base();
+             QString tmp = base().toString();
              return tmp.isEmpty() ? QString() : QDir::toNativeSeparators(QFileInfo(tmp).filePath());
          },
          visibleInChooser);
@@ -413,19 +424,19 @@ void MacroExpander::registerFileVariables(const QByteArray &prefix,
     registerVariable(prefix + kNativePathPostfix,
          tr("%1: Full path excluding file name, with native path separator (backslash on Windows).").arg(heading),
          [base]() -> QString {
-             QString tmp = base();
+             QString tmp = base().toString();
              return tmp.isEmpty() ? QString() : QDir::toNativeSeparators(QFileInfo(tmp).path());
          },
          visibleInChooser);
 
     registerVariable(prefix + kFileNamePostfix,
          tr("%1: File name without path.").arg(heading),
-         [base]() -> QString { QString tmp = base(); return tmp.isEmpty() ? QString() : FilePath::fromString(tmp).fileName(); },
+         [base]() -> QString { QString tmp = base().toString(); return tmp.isEmpty() ? QString() : FilePath::fromString(tmp).fileName(); },
          visibleInChooser);
 
     registerVariable(prefix + kFileBaseNamePostfix,
          tr("%1: File base name without path and suffix.").arg(heading),
-         [base]() -> QString { QString tmp = base(); return tmp.isEmpty() ? QString() : QFileInfo(tmp).baseName(); },
+         [base]() -> QString { QString tmp = base().toString(); return tmp.isEmpty() ? QString() : QFileInfo(tmp).baseName(); },
          visibleInChooser);
 }
 
