@@ -25,12 +25,12 @@
 
 #include "pathchooser.h"
 
+#include "commandline.h"
 #include "environment.h"
-#include "qtcassert.h"
-#include "macroexpander.h"
-
-#include "synchronousprocess.h"
 #include "hostosinfo.h"
+#include "macroexpander.h"
+#include "qtcassert.h"
+#include "qtcprocess.h"
 #include "theme/theme.h"
 
 #include <QDebug>
@@ -70,14 +70,13 @@
 
 namespace Utils {
 
-static QString appBundleExpandedPath(const QString &path)
+static FilePath appBundleExpandedPath(const FilePath &path)
 {
-    if (HostOsInfo::hostOs() == OsTypeMac && path.endsWith(".app")) {
+    if (path.osType() == OsTypeMac && path.endsWith(".app")) {
         // possibly expand to Foo.app/Contents/MacOS/Foo
-        QFileInfo info(path);
-        if (info.isDir()) {
-            QString exePath = path + "/Contents/MacOS/" + info.completeBaseName();
-            if (QFileInfo::exists(exePath))
+        if (path.isDir()) {
+            const FilePath exePath = path / "Contents/MacOS" / path.completeBaseName();
+            if (exePath.exists())
                 return exePath;
         }
     }
@@ -127,7 +126,7 @@ bool BinaryVersionToolTipEventFilter::eventFilter(QObject *o, QEvent *e)
     const QString binary = le->text();
     if (!binary.isEmpty()) {
         const QString version = BinaryVersionToolTipEventFilter::toolVersion(
-                    CommandLine(QDir::cleanPath(binary), m_arguments));
+                    CommandLine(FilePath::fromString(QDir::cleanPath(binary)), m_arguments));
         if (!version.isEmpty()) {
             // Concatenate tooltips.
             QString tooltip = "<html><head/><body>";
@@ -150,12 +149,13 @@ QString BinaryVersionToolTipEventFilter::toolVersion(const CommandLine &cmd)
 {
     if (cmd.executable().isEmpty())
         return QString();
-    SynchronousProcess proc;
+    QtcProcess proc;
     proc.setTimeoutS(1);
-    SynchronousProcessResponse response = proc.runBlocking(cmd);
-    if (response.result != SynchronousProcessResponse::Finished)
+    proc.setCommand(cmd);
+    proc.runBlocking();
+    if (proc.result() != QtcProcess::FinishedWithSuccess)
         return QString();
-    return response.allOutput();
+    return proc.allOutput();
 }
 
 // Extends BinaryVersionToolTipEventFilter to prepend the existing pathchooser
@@ -167,8 +167,7 @@ public:
         BinaryVersionToolTipEventFilter(pe->lineEdit()), m_pathChooser(pe) {}
 
 private:
-    QString defaultToolTip() const override
-        { return m_pathChooser->errorMessage(); }
+    QString defaultToolTip() const override { return m_pathChooser->errorMessage(); }
 
     const PathChooser *m_pathChooser = nullptr;
 };
@@ -180,7 +179,7 @@ class PathChooserPrivate
 public:
     PathChooserPrivate();
 
-    QString expandedPath(const QString &path) const;
+    FilePath expandedPath(const QString &path) const;
 
     QHBoxLayout *m_hLayout = nullptr;
     FancyLineEdit *m_lineEdit = nullptr;
@@ -188,17 +187,14 @@ public:
     PathChooser::Kind m_acceptingKind = PathChooser::ExistingDirectory;
     QString m_dialogTitleOverride;
     QString m_dialogFilter;
-    QString m_initialBrowsePathOverride;
+    FilePath m_initialBrowsePathOverride;
     QString m_defaultValue;
     FilePath m_baseDirectory;
-    Environment m_environment;
+    EnvironmentChange m_environmentChange;
     BinaryVersionToolTipEventFilter *m_binaryVersionToolTipEventFilter = nullptr;
     QList<QAbstractButton *> m_buttons;
     MacroExpander *m_macroExpander = globalMacroExpander();
     std::function<void()> m_openTerminal;
-
-    bool m_isReadOnly = false;
-    bool m_isFileDialogOnly = false;
 };
 
 PathChooserPrivate::PathChooserPrivate()
@@ -207,24 +203,28 @@ PathChooserPrivate::PathChooserPrivate()
 {
 }
 
-QString PathChooserPrivate::expandedPath(const QString &input) const
+FilePath PathChooserPrivate::expandedPath(const QString &input) const
 {
     if (input.isEmpty())
-        return input;
+        return {};
+
+    FilePath path = FilePath::fromUserInput(input);
+
+    Environment env = path.deviceEnvironment();
+    m_environmentChange.applyToEnvironment(env);
+    path = env.expandVariables(path);
 
-    QString expandedInput = m_environment.expandVariables(input);
     if (m_macroExpander)
-        expandedInput = m_macroExpander->expand(expandedInput);
+        path = m_macroExpander->expand(path);
 
-    const QString path = FilePath::fromUserInput(expandedInput).toString();
     if (path.isEmpty())
         return path;
 
     switch (m_acceptingKind) {
     case PathChooser::Command:
     case PathChooser::ExistingCommand: {
-        const FilePath expanded = m_environment.searchInPath(path, {m_baseDirectory});
-        return expanded.isEmpty() ? path : expanded.toString();
+        const FilePath expanded = path.searchInPath({m_baseDirectory});
+        return expanded.isEmpty() ? path : expanded;
     }
     case PathChooser::Any:
         break;
@@ -232,8 +232,19 @@ QString PathChooserPrivate::expandedPath(const QString &input) const
     case PathChooser::ExistingDirectory:
     case PathChooser::File:
     case PathChooser::SaveFile:
-        if (!m_baseDirectory.isEmpty() && QFileInfo(path).isRelative())
-            return m_baseDirectory.pathAppended(path).toFileInfo().absoluteFilePath();
+        if (!m_baseDirectory.isEmpty()) {
+            Utils::FilePath fp = m_baseDirectory.resolvePath(path.path()).absoluteFilePath();
+            // FIXME bad hotfix for manually editing PathChooser (invalid paths, jumping cursor)
+            // examples: have an absolute path and try to change the device letter by typing the new
+            // letter and removing the original afterwards ends up in
+            // D:\\dev\\project\\cD:\\dev\\build-project (before trying to remove the original)
+            // as 'cD:\\dev\\build-project' is considered is handled as being relative
+            // input = "cD:\\dev\build-project"; // prepended 'c' to change the device letter
+            // m_baseDirectory = "D:\\dev\\project"
+            if (!fp.needsDevice() && HostOsInfo::isWindowsHost() && fp.toString().count(':') > 1)
+                return path;
+            return fp;
+        }
         break;
     }
     return path;
@@ -245,10 +256,9 @@ PathChooser::PathChooser(QWidget *parent) :
 {
     d->m_hLayout->setContentsMargins(0, 0, 0, 0);
 
-    setContextMenuPolicy(Qt::CustomContextMenu);
-    d->m_lineEdit->setContextMenuPolicy(Qt::NoContextMenu);
+    d->m_lineEdit->setContextMenuPolicy(Qt::CustomContextMenu);
 
-    connect(this,
+    connect(d->m_lineEdit,
             &FancyLineEdit::customContextMenuRequested,
             this,
             &PathChooser::contextMenuRequested);
@@ -257,7 +267,11 @@ PathChooser::PathChooser(QWidget *parent) :
             [this] { emit rawPathChanged(rawPath()); });
     connect(d->m_lineEdit, &FancyLineEdit::validChanged, this, &PathChooser::validChanged);
     connect(d->m_lineEdit, &QLineEdit::editingFinished, this, &PathChooser::editingFinished);
-    connect(d->m_lineEdit, &QLineEdit::textChanged, this, [this] { emit pathChanged(d->m_lineEdit->text()); });
+    connect(d->m_lineEdit, &QLineEdit::textChanged, this, [this] {
+        const QString text = d->m_lineEdit->text();
+        emit pathChanged(text);
+        emit filePathChanged(FilePath::fromUserInput(text));
+    });
 
     d->m_lineEdit->setMinimumWidth(120);
     d->m_hLayout->addWidget(d->m_lineEdit);
@@ -268,7 +282,6 @@ PathChooser::PathChooser(QWidget *parent) :
     setLayout(d->m_hLayout);
     setFocusProxy(d->m_lineEdit);
     setFocusPolicy(d->m_lineEdit->focusPolicy());
-    setEnvironment(Environment::systemEnvironment());
 
     d->m_lineEdit->setValidationFunction(defaultValidationFunction());
 }
@@ -294,7 +307,6 @@ void PathChooser::insertButton(int index, const QString &text, QObject *context,
     connect(button, &QAbstractButton::clicked, context, callback);
     d->m_hLayout->insertWidget(index + 1/*line edit*/, button);
     d->m_buttons.insert(index, button);
-    updateReadOnlyStateOfSubwidgets();
 }
 
 QString PathChooser::browseButtonLabel()
@@ -320,10 +332,10 @@ FilePath PathChooser::baseDirectory() const
     return d->m_baseDirectory;
 }
 
-void PathChooser::setEnvironment(const Environment &env)
+void PathChooser::setEnvironmentChange(const EnvironmentChange &env)
 {
     QString oldExpand = filePath().toString();
-    d->m_environment = env;
+    d->m_environmentChange = env;
     if (filePath().toString() != oldExpand) {
         triggerChanged();
         emit rawPathChanged(rawPath());
@@ -337,12 +349,17 @@ QString PathChooser::rawPath() const
 
 FilePath PathChooser::rawFilePath() const
 {
-    return FilePath::fromString(QDir::fromNativeSeparators(d->m_lineEdit->text()));
+    return FilePath::fromUserInput(d->m_lineEdit->text());
 }
 
 FilePath PathChooser::filePath() const
 {
-    return FilePath::fromUserInput(d->expandedPath(rawFilePath().toString()));
+    return d->expandedPath(rawFilePath().toString());
+}
+
+FilePath PathChooser::absoluteFilePath() const
+{
+    return d->m_baseDirectory.resolvePath(filePath());
 }
 
 // FIXME: try to remove again
@@ -371,87 +388,69 @@ void PathChooser::setFilePath(const FilePath &fn)
 
 bool PathChooser::isReadOnly() const
 {
-    return d->m_isReadOnly;
+    return d->m_lineEdit->isReadOnly();
 }
 
 void PathChooser::setReadOnly(bool b)
 {
-    d->m_isReadOnly = b;
-    updateReadOnlyStateOfSubwidgets();
-}
-
-bool PathChooser::isFileDialogOnly() const
-{
-    return d->m_isFileDialogOnly;
-}
-
-void PathChooser::setFileDialogOnly(bool b)
-{
-    d->m_isFileDialogOnly = b;
-    updateReadOnlyStateOfSubwidgets();
+    d->m_lineEdit->setReadOnly(b);
+    const auto buttons = d->m_buttons;
+    for (QAbstractButton *button : buttons)
+        button->setEnabled(!b);
 }
 
 void PathChooser::slotBrowse()
 {
     emit beforeBrowsing();
 
-    QString predefined = filePath().toString();
-    QFileInfo fi(predefined);
+    FilePath predefined = filePath();
 
-    if (!predefined.isEmpty() && !fi.isDir()) {
-        predefined = fi.path();
-        fi.setFile(predefined);
+    if (!predefined.isEmpty() && !predefined.isDir()) {
+        predefined = predefined.parentDir();
     }
 
-    if ((predefined.isEmpty() || !fi.isDir())
-            && !d->m_initialBrowsePathOverride.isNull()) {
+    if ((predefined.isEmpty() || !predefined.isDir()) && !d->m_initialBrowsePathOverride.isEmpty()) {
         predefined = d->m_initialBrowsePathOverride;
-        fi.setFile(predefined);
-        if (!fi.isDir()) {
+        if (!predefined.isDir())
             predefined.clear();
-            fi.setFile(QString());
-        }
     }
 
     // Prompt for a file/dir
-    QString newPath;
+    FilePath newPath;
     switch (d->m_acceptingKind) {
     case PathChooser::Directory:
     case PathChooser::ExistingDirectory:
-        newPath = QFileDialog::getExistingDirectory(this,
-                makeDialogTitle(tr("Choose Directory")), predefined);
+        newPath = FileUtils::getExistingDirectory(this,
+                        makeDialogTitle(tr("Choose Directory")), predefined);
         break;
     case PathChooser::ExistingCommand:
     case PathChooser::Command:
-        newPath = QFileDialog::getOpenFileName(this,
-                makeDialogTitle(tr("Choose Executable")), predefined,
-                d->m_dialogFilter);
+        newPath = FileUtils::getOpenFilePath(this,
+                        makeDialogTitle(tr("Choose Executable")), predefined, d->m_dialogFilter);
         newPath = appBundleExpandedPath(newPath);
         break;
     case PathChooser::File: // fall through
-        newPath = QFileDialog::getOpenFileName(this,
-                makeDialogTitle(tr("Choose File")), predefined,
-                d->m_dialogFilter);
+        newPath = FileUtils::getOpenFilePath(this,
+                        makeDialogTitle(tr("Choose File")), predefined, d->m_dialogFilter);
         newPath = appBundleExpandedPath(newPath);
         break;
     case PathChooser::SaveFile:
-        newPath = QFileDialog::getSaveFileName(this,
-                makeDialogTitle(tr("Choose File")), predefined,
-                d->m_dialogFilter);
+        newPath = FileUtils::getSaveFilePath(this,
+                        makeDialogTitle(tr("Choose File")), predefined, d->m_dialogFilter);
         break;
     case PathChooser::Any: {
         QFileDialog dialog(this);
         dialog.setFileMode(QFileDialog::AnyFile);
         dialog.setWindowTitle(makeDialogTitle(tr("Choose File")));
-        if (fi.exists())
-            dialog.setDirectory(fi.absolutePath());
+        if (predefined.exists())
+            dialog.setDirectory(predefined.absolutePath().toDir());
         // FIXME: fix QFileDialog so that it filters properly: lib*.a
         dialog.setNameFilter(d->m_dialogFilter);
         if (dialog.exec() == QDialog::Accepted) {
             // probably loop here until the *.framework dir match
             QStringList paths = dialog.selectedFiles();
             if (!paths.isEmpty())
-                newPath = paths.at(0);
+                newPath = FilePath::fromString(paths.at(0));
         }
         break;
         }
@@ -464,12 +463,11 @@ void PathChooser::slotBrowse()
     window()->raise();
     window()->activateWindow();
 
-    // Delete trailing slashes unless it is "/"|"\\", only
+    // Delete trailing slashes unless it is "/" only.
     if (!newPath.isEmpty()) {
-        newPath = QDir::toNativeSeparators(newPath);
-        if (newPath.size() > 1 && newPath.endsWith(QDir::separator()))
-            newPath.truncate(newPath.size() - 1);
-        setPath(newPath);
+        if (newPath.endsWith("/") && newPath.path().size() > 1)
+            newPath = newPath.withNewPath(newPath.path().chopped(1));
+        setFilePath(newPath);
     }
 
     emit browsingFinished();
@@ -478,29 +476,14 @@ void PathChooser::slotBrowse()
 
 void PathChooser::contextMenuRequested(const QPoint &pos)
 {
-    if (!d->m_lineEdit->rect().contains(pos))
-        return;
-    QMenu *menu = d->m_lineEdit->createStandardContextMenu();
-    if (!menu)
-        menu = new QMenu;
-    if (s_aboutToShowContextMenuHandler)
-        s_aboutToShowContextMenuHandler(this, menu);
-    if (!menu->actions().isEmpty()) {
+    if (QMenu *menu = d->m_lineEdit->createStandardContextMenu()) {
         menu->setAttribute(Qt::WA_DeleteOnClose);
-        menu->popup(mapToGlobal(pos));
-    } else {
-        delete menu;
-    }
-}
 
-void PathChooser::updateReadOnlyStateOfSubwidgets()
-{
-    const bool readOnlyLineEdit = d->m_isReadOnly || d->m_isFileDialogOnly;
-    d->m_lineEdit->setEnabled(!readOnlyLineEdit);
-    d->m_lineEdit->setReadOnly(readOnlyLineEdit);
-    setFocusPolicy(d->m_lineEdit->focusPolicy());
-    for (QAbstractButton *button : qAsConst(d->m_buttons))
-        button->setEnabled(!d->m_isReadOnly);
+        if (s_aboutToShowContextMenuHandler)
+            s_aboutToShowContextMenuHandler(this, menu);
+
+        menu->popup(d->m_lineEdit->mapToGlobal(pos));
+    }
 }
 
 bool PathChooser::isValid() const
@@ -559,75 +542,80 @@ bool PathChooser::validatePath(FancyLineEdit *edit, QString *errorMessage) const
         }
     }
 
-    const QString expandedPath = d->expandedPath(path);
-    if (expandedPath.isEmpty()) {
+    const FilePath filePath = d->expandedPath(path);
+    if (filePath.isEmpty()) {
         if (errorMessage)
             *errorMessage = tr("The path \"%1\" expanded to an empty string.").arg(QDir::toNativeSeparators(path));
         return false;
     }
-    const QFileInfo fi(expandedPath);
 
     // Check if existing
     switch (d->m_acceptingKind) {
     case PathChooser::ExistingDirectory:
-        if (!fi.exists()) {
+        if (!filePath.exists()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" does not exist.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput());
             return false;
         }
-        if (!fi.isDir()) {
+        if (!filePath.isDir()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a directory.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" is not a directory.").arg(filePath.toUserOutput());
             return false;
         }
         break;
     case PathChooser::File:
-        if (!fi.exists()) {
+        if (!filePath.exists()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" does not exist.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput());
             return false;
         }
-        if (!fi.isFile()) {
+        if (!filePath.isFile()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a file.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" is not a file.").arg(filePath.toUserOutput());
             return false;
         }
         break;
     case PathChooser::SaveFile:
-        if (!fi.absoluteDir().exists()) {
+        if (!filePath.parentDir().exists()) {
             if (errorMessage)
-                *errorMessage = tr("The directory \"%1\" does not exist.").arg(QDir::toNativeSeparators(fi.absolutePath()));
+                *errorMessage = tr("The directory \"%1\" does not exist.").arg(filePath.toUserOutput());
             return false;
         }
-        if (fi.exists() && fi.isDir()) {
+        if (filePath.exists() && filePath.isDir()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a file.").arg(QDir::toNativeSeparators(fi.absolutePath()));
+                *errorMessage = tr("The path \"%1\" is not a file.").arg(filePath.toUserOutput());
             return false;
         }
         break;
     case PathChooser::ExistingCommand:
-        if (!fi.exists()) {
+        if (!filePath.exists()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" does not exist.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput());
             return false;
         }
-        if (!fi.isFile() || !fi.isExecutable()) {
+        if (!filePath.isExecutableFile()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not an executable file.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" is not an executable file.").arg(filePath.toUserOutput());
             return false;
         }
         break;
     case PathChooser::Directory:
-        if (fi.exists() && !fi.isDir()) {
+        if (filePath.exists() && !filePath.isDir()) {
             if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a directory.").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("The path \"%1\" is not a directory.").arg(filePath.toUserOutput());
+            return false;
+        }
+        if (HostOsInfo::isWindowsHost() && !filePath.startsWithDriveLetter()
+                && !filePath.startsWith("\\\\") && !filePath.startsWith("//")) {
+            if (errorMessage)
+                *errorMessage = tr("Invalid path \"%1\".").arg(filePath.toUserOutput());
             return false;
         }
         break;
     case PathChooser::Command:
-        if (fi.exists() && !fi.isExecutable()) {
+        if (filePath.exists() && !filePath.isExecutableFile()) {
             if (errorMessage)
-                *errorMessage = tr("Cannot execute \"%1\".").arg(QDir::toNativeSeparators(expandedPath));
+                *errorMessage = tr("Cannot execute \"%1\".").arg(filePath.toUserOutput());
             return false;
         }
         break;
@@ -637,7 +625,7 @@ bool PathChooser::validatePath(FancyLineEdit *edit, QString *errorMessage) const
     }
 
     if (errorMessage)
-        *errorMessage = tr("Full path: \"%1\"").arg(QDir::toNativeSeparators(expandedPath));
+        *errorMessage = tr("Full path: \"%1\"").arg(filePath.toUserOutput());
     return true;
 }
 
@@ -651,14 +639,14 @@ QString PathChooser::label()
     return tr("Path:");
 }
 
-QString PathChooser::homePath()
+FilePath PathChooser::homePath()
 {
     // Return 'users/<name>/Documents' on Windows, since Windows explorer
     // does not let people actually display the contents of their home
     // directory. Alternatively, create a QtCreator-specific directory?
     if (HostOsInfo::isWindowsHost())
-        return QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
-    return QDir::homePath();
+        return FilePath::fromString(QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation));
+    return FilePath::fromString(QDir::homePath());
 }
 
 /*!
@@ -708,7 +696,7 @@ QString PathChooser::promptDialogFilter() const
     return d->m_dialogFilter;
 }
 
-void PathChooser::setInitialBrowsePathBackup(const QString &path)
+void PathChooser::setInitialBrowsePathBackup(const FilePath &path)
 {
     d->m_initialBrowsePathOverride = path;
 }
@@ -729,9 +717,9 @@ FancyLineEdit *PathChooser::lineEdit() const
     return d->m_lineEdit;
 }
 
-QString PathChooser::toolVersion(const QString &binary, const QStringList &arguments)
+QString PathChooser::toolVersion(const CommandLine &cmd)
 {
-    return BinaryVersionToolTipEventFilter::toolVersion({binary, arguments});
+    return BinaryVersionToolTipEventFilter::toolVersion(cmd);
 }
 
 void PathChooser::installLineEditVersionToolTip(QLineEdit *le, const QStringList &arguments)
