@@ -38,32 +38,31 @@ QT_END_NAMESPACE
 
 namespace Utils {
 
+class CommandLine;
 class FancyLineEdit;
+class MacroExpander;
 class Environment;
+class EnvironmentChange;
 class PathChooserPrivate;
 
 class QTCREATOR_UTILS_EXPORT PathChooser : public QWidget
 {
     Q_OBJECT
-    Q_ENUMS(Kind)
     Q_PROPERTY(QString path READ path WRITE setPath NOTIFY pathChanged DESIGNABLE true)
     Q_PROPERTY(QString promptDialogTitle READ promptDialogTitle WRITE setPromptDialogTitle DESIGNABLE true)
     Q_PROPERTY(QString promptDialogFilter READ promptDialogFilter WRITE setPromptDialogFilter DESIGNABLE true)
     Q_PROPERTY(Kind expectedKind READ expectedKind WRITE setExpectedKind DESIGNABLE true)
-    Q_PROPERTY(QString baseDirectory READ baseDirectory WRITE setBaseDirectory DESIGNABLE true)
+    Q_PROPERTY(Utils::FilePath baseDirectory READ baseDirectory WRITE setBaseDirectory DESIGNABLE true)
     Q_PROPERTY(QStringList commandVersionArguments READ commandVersionArguments WRITE setCommandVersionArguments)
     Q_PROPERTY(bool readOnly READ isReadOnly WRITE setReadOnly DESIGNABLE true)
     // Designer does not know this type, so force designable to false:
-    Q_PROPERTY(Utils::FileName fileName READ fileName WRITE setFileName DESIGNABLE false)
-    Q_PROPERTY(Utils::FileName baseFileName READ baseFileName WRITE setBaseFileName DESIGNABLE false)
-    Q_PROPERTY(QColor errorColor READ errorColor WRITE setErrorColor DESIGNABLE true)
-    Q_PROPERTY(QColor okColor READ okColor WRITE setOkColor DESIGNABLE true)
+    Q_PROPERTY(Utils::FilePath filePath READ filePath WRITE setFilePath DESIGNABLE false)
 
 public:
     static QString browseButtonLabel();
 
     explicit PathChooser(QWidget *parent = nullptr);
-    virtual ~PathChooser();
+    ~PathChooser() override;
 
     enum Kind {
         ExistingDirectory,
@@ -74,6 +73,7 @@ public:
         Command, // A command that may or may not exist at the time of selection (e.g. result of a build)
         Any
     };
+    Q_ENUM(Kind)
 
     // Default is <Directory>
     void setExpectedKind(Kind expected);
@@ -85,26 +85,24 @@ public:
     void setPromptDialogFilter(const QString &filter);
     QString promptDialogFilter() const;
 
-    void setInitialBrowsePathBackup(const QString &path);
+    void setInitialBrowsePathBackup(const FilePath &path);
 
     bool isValid() const;
     QString errorMessage() const;
 
-    QString path() const;
-    QString rawPath() const; // The raw unexpanded input.
-    FileName rawFileName() const; // The raw unexpanded input.
-    FileName fileName() const;
+    FilePath filePath() const; // Close to what's in the line edit.
+    FilePath absoluteFilePath() const; // Relative paths resolved wrt the specified base dir.
 
-    static QString expandedDirectory(const QString &input, const Utils::Environment &env,
+    QString rawPath() const; // The raw unexpanded input.
+    FilePath rawFilePath() const; // The raw unexpanded input as FilePath.
+
+    static QString expandedDirectory(const QString &input, const Environment &env,
                                      const QString &baseDir);
 
-    QString baseDirectory() const;
-    void setBaseDirectory(const QString &directory);
+    FilePath baseDirectory() const;
+    void setBaseDirectory(const FilePath &base);
 
-    FileName baseFileName() const;
-    void setBaseFileName(const FileName &base);
-
-    void setEnvironment(const Environment &env);
+    void setEnvironmentChange(const EnvironmentChange &change);
 
     /** Returns the suggested label title when used in a form layout. */
     static QString label();
@@ -113,7 +111,7 @@ public:
     void setValidationFunction(const FancyLineEdit::ValidationFunction &fn);
 
     /** Return the home directory, which needs some fixing under Windows. */
-    static QString homePath();
+    static FilePath homePath();
 
     void addButton(const QString &text, QObject *context, const std::function<void()> &callback);
     void insertButton(int index, const QString &text, QObject *context, const std::function<void()> &callback);
@@ -128,13 +126,18 @@ public:
     void setCommandVersionArguments(const QStringList &arguments);
 
     // Utility to run a tool and return its stdout.
-    static QString toolVersion(const QString &binary, const QStringList &arguments);
+    static QString toolVersion(const Utils::CommandLine &cmd);
     // Install a tooltip on lineedits used for binaries showing the version.
     static void installLineEditVersionToolTip(QLineEdit *le, const QStringList &arguments);
 
     // Enable a history completer with a history of entries.
     void setHistoryCompleter(const QString &historyKey, bool restoreLastItemFromHistory = false);
 
+    // Sets a macro expander that is used when producing path and fileName.
+    // By default, the global expander is used.
+    // nullptr can be passed to disable macro expansion.
+    void setMacroExpander(MacroExpander *macroExpander);
+
     bool isReadOnly() const;
     void setReadOnly(bool b);
 
@@ -142,12 +145,19 @@ public:
 
     // global handler for adding context menus to ALL pathchooser
     // used by the coreplugin to add "Open in Terminal" and "Open in Explorer" context menu actions
-    using AboutToShowContextMenuHandler = std::function<void (Utils::PathChooser *, QMenu *)>;
+    using AboutToShowContextMenuHandler = std::function<void (PathChooser *, QMenu *)>;
     static void setAboutToShowContextMenuHandler(AboutToShowContextMenuHandler handler);
 
-    QColor errorColor() const;
-    QColor okColor() const;
+    void setOpenTerminalHandler(const std::function<void()> &openTerminal);
+    std::function<void()> openTerminalHandler() const;
 
+    // Deprecated. Use filePath().toString() or better suitable conversions.
+    QString path() const { return filePath().toString(); }
+
+    // this sets the placeHolderText to defaultValue and enables to use this as
+    // input value during validation if the real value is empty
+    // setting an empty QString will disable this and clear the placeHolderText
+    void setDefaultValue(const QString &defaultValue);
 private:
     bool validatePath(FancyLineEdit *edit, QString *errorMessage) const;
     // Returns overridden title or the one from <title>
@@ -159,6 +169,7 @@ signals:
     void validChanged(bool validState);
     void rawPathChanged(const QString &text);
     void pathChanged(const QString &path);
+    void filePathChanged(const FilePath &path);
     void editingFinished();
     void beforeBrowsing();
     void browsingFinished();
@@ -166,10 +177,7 @@ signals:
 
 public slots:
     void setPath(const QString &);
-    void setFileName(const Utils::FileName &);
-
-    void setErrorColor(const QColor &errorColor);
-    void setOkColor(const QColor &okColor);
+    void setFilePath(const FilePath &);
 
 private:
     PathChooserPrivate *d = nullptr;
