@@ -183,7 +183,7 @@ void ParseValueStackEntry::addChild(const QString &key, const QVariant &v)
 class ParseContext : public Context
 {
 public:
-    QVariantMap parse(QFile &file);
+    QVariantMap parse(const FilePath &file);
 
 private:
     enum Element { QtCreatorElement, DataElement, VariableElement,
@@ -204,9 +204,9 @@ private:
     QString m_currentVariableName;
 };
 
-QVariantMap ParseContext::parse(QFile &file)
+QVariantMap ParseContext::parse(const FilePath &file)
 {
-    QXmlStreamReader r(&file);
+    QXmlStreamReader r(file.fileContents());
 
     m_result.clear();
     m_currentVariableName.clear();
@@ -354,15 +354,11 @@ bool PersistentSettingsReader::load(const FilePath &fileName)
 {
     m_valueMap.clear();
 
-    QFile file(fileName.toString());
-    if (file.size() == 0) // skip empty files
+    if (fileName.fileSize() == 0) // skip empty files
         return false;
 
-    if (!file.open(QIODevice::ReadOnly|QIODevice::Text))
-        return false;
     ParseContext ctx;
-    m_valueMap = ctx.parse(file);
-    file.close();
+    m_valueMap = ctx.parse(fileName);
     return true;
 }
 
@@ -427,11 +423,6 @@ PersistentSettingsWriter::PersistentSettingsWriter(const FilePath &fileName, con
     m_fileName(fileName), m_docType(docType)
 { }
 
-PersistentSettingsWriter::~PersistentSettingsWriter()
-{
-    write(m_savedData, nullptr);
-}
-
 bool PersistentSettingsWriter::save(const QVariantMap &data, QString *errorString) const
 {
     if (data == m_savedData)
@@ -463,9 +454,8 @@ void PersistentSettingsWriter::setContents(const QVariantMap &data)
 
 bool PersistentSettingsWriter::write(const QVariantMap &data, QString *errorString) const
 {
-    QDir tmp;
-    tmp.mkpath(m_fileName.toFileInfo().path());
-    FileSaver saver(m_fileName.toString(), QIODevice::Text);
+    m_fileName.parentDir().ensureWritableDir();
+    FileSaver saver(m_fileName, QIODevice::Text);
     if (!saver.hasError()) {
         const Context ctx;
         QXmlStreamWriter w(saver.file());
