@@ -49,8 +49,8 @@ namespace Utils {
 QMessageBox::StandardButtons SettingsAccessor::Issue::allButtons() const
 {
     QMessageBox::StandardButtons result = QMessageBox::NoButton;
-    for (const QMessageBox::StandardButton &b : buttons.keys())
-        result |= b;
+    for (auto it = buttons.cbegin(); it != buttons.cend(); ++it)
+        result |= it.key();
     return result;
 }
 
@@ -176,7 +176,7 @@ SettingsAccessor::writeFile(const FilePath &path, const QVariantMap &data) const
 
 SettingsAccessor::ProceedInfo
 SettingsAccessor::reportIssues(const SettingsAccessor::Issue &issue, const FilePath &path,
-                               QWidget *parent) const
+                               QWidget *parent)
 {
     if (!path.exists())
         return Continue;
@@ -218,12 +218,10 @@ QVariantMap SettingsAccessor::prepareToWriteSettings(const QVariantMap &data) co
 
 FilePaths BackUpStrategy::readFileCandidates(const FilePath &baseFileName) const
 {
+    const QStringList filter(baseFileName.fileName() + '*');
+    const FilePath baseFileDir = baseFileName.parentDir();
 
-    const QFileInfo pfi = baseFileName.toFileInfo();
-    const QStringList filter(pfi.fileName() + '*');
-    const QFileInfoList list = QDir(pfi.dir()).entryInfoList(filter, QDir::Files | QDir::Hidden | QDir::System);
-
-    return Utils::transform(list, [](const QFileInfo &fi) { return FilePath::fromString(fi.absoluteFilePath()); });
+    return baseFileDir.dirEntries(filter, QDir::Files | QDir::Hidden | QDir::System);
 }
 
 int BackUpStrategy::compare(const SettingsAccessor::RestoreData &data1,
@@ -329,10 +327,8 @@ void BackingUpSettingsAccessor::backupFile(const FilePath &path, const QVariantM
         return;
 
     // Do we need to do a backup?
-    const QString origName = path.toString();
-    optional<FilePath> backupFileName = m_strategy->backupName(oldSettings.data, path, data);
-    if (backupFileName)
-        QFile::copy(origName, backupFileName.value().toString());
+    if (optional<FilePath> backupFileName = m_strategy->backupName(oldSettings.data, path, data))
+        path.copyFile(backupFileName.value());
 }
 
 // --------------------------------------------------------------------
@@ -697,7 +693,7 @@ MergingSettingsAccessor::mergeSettings(const SettingsAccessor::RestoreData &main
 /*!
  * Returns true for housekeeping related keys.
  */
-bool MergingSettingsAccessor::isHouseKeepingKey(const QString &key) const
+bool MergingSettingsAccessor::isHouseKeepingKey(const QString &key)
 {
     return key == VERSION_KEY || key == ORIGINAL_VERSION_KEY || key == SETTINGS_ID_KEY;
 }
