@@ -27,16 +27,11 @@
 
 #include "utils_global.h"
 
-#include "synchronousprocess.h"
-
-#include <QObject>
-
-#include <functional>
+#include "qtcprocess.h"
 
 QT_BEGIN_NAMESPACE
 class QMutex;
 class QVariant;
-class QProcessEnvironment;
 template <typename T>
 class QFutureInterface;
 template <typename T>
@@ -45,7 +40,6 @@ QT_END_NAMESPACE
 
 namespace Utils {
 
-class CommandLine;
 namespace Internal { class ShellCommandPrivate; }
 
 class QTCREATOR_UTILS_EXPORT ProgressParser
@@ -66,22 +60,6 @@ private:
     friend class ShellCommand;
 };
 
-// Users of this class can either be in the GUI thread or in other threads.
-// Use Qt::AutoConnection to always append in the GUI thread (directly or queued)
-class QTCREATOR_UTILS_EXPORT OutputProxy : public QObject
-{
-    Q_OBJECT
-
-    friend class ShellCommand;
-
-signals:
-    void append(const QString &text);
-    void appendSilently(const QString &text);
-    void appendError(const QString &text);
-    void appendCommand(const QString &workingDirectory, const Utils::CommandLine &command);
-    void appendMessage(const QString &text);
-};
-
 class QTCREATOR_UTILS_EXPORT ShellCommand : public QObject
 {
     Q_OBJECT
@@ -104,25 +82,25 @@ public:
     };
 
 
-    ShellCommand(const QString &workingDirectory, const QProcessEnvironment &environment);
+    ShellCommand(const FilePath &workingDirectory, const Environment &environment);
     ~ShellCommand() override;
 
     QString displayName() const;
     void setDisplayName(const QString &name);
 
     void addJob(const CommandLine &command,
-                const QString &workingDirectory = QString(),
-                const ExitCodeInterpreter &interpreter = defaultExitCodeInterpreter);
+                const FilePath &workingDirectory = {},
+                const ExitCodeInterpreter &interpreter = {});
     void addJob(const CommandLine &command, int timeoutS,
-                const QString &workingDirectory = QString(),
-                const ExitCodeInterpreter &interpreter = defaultExitCodeInterpreter);
+                const FilePath &workingDirectory = {},
+                const ExitCodeInterpreter &interpreter = {});
     void execute(); // Execute tasks asynchronously!
     void abort();
     bool lastExecutionSuccess() const;
     int lastExecutionExitCode() const;
 
-    const QString &defaultWorkingDirectory() const;
-    virtual const QProcessEnvironment processEnvironment() const;
+    const FilePath &defaultWorkingDirectory() const;
+    virtual const Environment processEnvironment() const;
 
     int defaultTimeoutS() const;
     void setDefaultTimeoutS(int timeout);
@@ -140,15 +118,14 @@ public:
     bool hasProgressParser() const;
     void setProgressiveOutput(bool progressive);
 
-    void setOutputProxyFactory(const std::function<OutputProxy *()> &factory);
+    void setDisableUnixTerminal();
 
     // This is called once per job in a thread.
     // When called from the UI thread it will execute fully synchronously, so no signals will
     // be triggered!
-    virtual SynchronousProcessResponse runCommand(const CommandLine &command,
-                                                  int timeoutS,
-                                                  const QString &workingDirectory = QString(),
-                                                  const ExitCodeInterpreter &interpreter = defaultExitCodeInterpreter);
+    virtual void runCommand(QtcProcess &process,
+                            const CommandLine &command,
+                            const FilePath &workingDirectory = {});
 
     void cancel();
 
@@ -161,25 +138,24 @@ signals:
 
     void terminate(); // Internal
 
+    void append(const QString &text);
+    void appendSilently(const QString &text);
+    void appendError(const QString &text);
+    void appendCommand(const Utils::FilePath &workingDirectory, const Utils::CommandLine &command);
+    void appendMessage(const QString &text);
+
 protected:
-    virtual unsigned processFlags() const;
     virtual void addTask(QFuture<void> &future);
     int timeoutS() const;
-    QString workDirectory(const QString &wd) const;
+    FilePath workDirectory(const FilePath &wd) const;
 
 private:
     void run(QFutureInterface<void> &future);
 
     // Run without a event loop in fully blocking mode. No signals will be delivered.
-    SynchronousProcessResponse runFullySynchronous(const CommandLine &cmd,
-                                                   QSharedPointer<OutputProxy> proxy,
-                                                   int timeoutS, const QString &workingDirectory,
-                                                   const ExitCodeInterpreter &interpreter = defaultExitCodeInterpreter);
+    void runFullySynchronous(QtcProcess &proc, const FilePath &workingDirectory);
     // Run with an event loop. Signals will be delivered.
-    SynchronousProcessResponse runSynchronous(const CommandLine &cmd,
-                                              QSharedPointer<OutputProxy> proxy,
-                                              int timeoutS, const QString &workingDirectory,
-                                              const ExitCodeInterpreter &interpreter = defaultExitCodeInterpreter);
+    void runSynchronous(QtcProcess &proc, const FilePath &workingDirectory);
 
     class Internal::ShellCommandPrivate *const d;
 };
