@@ -39,7 +39,9 @@ namespace Utils {
 using std::get;
 using std::get_if;
 using std::holds_alternative;
+using std::monostate;
 using std::variant;
+using std::variant_alternative_t;
 using std::visit;
 } // namespace Utils
 
@@ -50,7 +52,9 @@ namespace Utils {
 using mpark::get;
 using mpark::get_if;
 using mpark::holds_alternative;
+using mpark::monostate;
 using mpark::variant;
+using mpark::variant_alternative_t;
 using mpark::visit;
 } // namespace Utils
 
