@@ -22,14 +22,19 @@
 ** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
+
 #include "androidavdmanager.h"
 
+#include "avdmanageroutputparser.h"
+
 #include <coreplugin/icore.h>
+
 #include <projectexplorer/projectexplorerconstants.h>
+
 #include <utils/algorithm.h>
 #include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
 #include <utils/runextensions.h>
-#include <utils/synchronousprocess.h>
 
 #include <QApplication>
 #include <QFileInfo>
@@ -51,17 +56,6 @@ namespace Internal {
 
 using namespace std;
 
-// Avd list keys to parse avd
-const char avdInfoNameKey[] = "Name:";
-const char avdInfoPathKey[] = "Path:";
-const char avdInfoAbiKey[] = "abi.type";
-const char avdInfoTargetKey[] = "target";
-const char avdInfoErrorKey[] = "Error:";
-const char avdInfoSdcardKey[] = "Sdcard";
-const char avdInfoTargetTypeKey[] = "Target";
-const char avdInfoDeviceKey[] = "Device";
-const char avdInfoSkinKey[] = "Skin";
-
 const int avdCreateTimeoutMs = 30000;
 
 /*!
@@ -72,29 +66,15 @@ const int avdCreateTimeoutMs = 30000;
 bool AndroidAvdManager::avdManagerCommand(const AndroidConfig &config, const QStringList &args, QString *output)
 {
     CommandLine cmd(config.avdManagerToolPath(), args);
-    Utils::SynchronousProcess proc;
-    auto env = AndroidConfigurations::toolsEnvironment(config).toStringList();
+    Utils::QtcProcess proc;
+    Environment env = AndroidConfigurations::toolsEnvironment(config);
     proc.setEnvironment(env);
     qCDebug(avdManagerLog) << "Running AVD Manager command:" << cmd.toUserOutput();
-    SynchronousProcessResponse response = proc.runBlocking(cmd);
-    if (response.result == Utils::SynchronousProcessResponse::Finished) {
+    proc.setCommand(cmd);
+    proc.runBlocking();
+    if (proc.result() == Utils::QtcProcess::FinishedWithSuccess) {
         if (output)
-            *output = response.allOutput();
-        return true;
-    }
-    return false;
-}
-
-/*!
-    Parses the \a line for a [spaces]key[spaces]value[spaces] pattern and returns
-    \c true if the key is found, \c false otherwise. The value is copied into \a value.
- */
-static bool valueForKey(QString key, const QString &line, QString *value = nullptr)
-{
-    auto trimmedInput = line.trimmed();
-    if (trimmedInput.startsWith(key)) {
-        if (value)
-            *value = trimmedInput.section(key, 1, 1).trimmed();
+            *output = proc.allOutput();
         return true;
     }
     return false;
@@ -135,15 +115,16 @@ static CreateAvdInfo createAvdCommand(const AndroidConfig &config, const CreateA
     if (result.overwrite)
         arguments << "-f";
 
-    const QString avdManagerTool = config.avdManagerToolPath().toString();
+    const FilePath avdManagerTool = config.avdManagerToolPath();
     qCDebug(avdManagerLog)
             << "Running AVD Manager command:" << CommandLine(avdManagerTool, arguments).toUserOutput();
     QProcess proc;
-    proc.start(avdManagerTool, arguments);
+    proc.setEnvironment(AndroidConfigurations::toolsEnvironment(config).toStringList());
+    proc.start(avdManagerTool.toString(), arguments);
     if (!proc.waitForStarted()) {
         result.error = QApplication::translate("AndroidAvdManager",
                                                "Could not start process \"%1 %2\"")
-                .arg(avdManagerTool, arguments.join(' '));
+                .arg(avdManagerTool.toString(), arguments.join(' '));
         return result;
     }
     QTC_CHECK(proc.state() == QProcess::Running);
@@ -203,25 +184,8 @@ static void avdProcessFinished(int exitCode, QProcess *p)
     p->deleteLater();
 }
 
-/*!
-    \class AvdManagerOutputParser
-    \brief The AvdManagerOutputParser class is a helper class to parse the output of the avdmanager
-    commands.
- */
-class AvdManagerOutputParser
-{
-public:
-    AndroidDeviceInfoList listVirtualDevices(const AndroidConfig &config);
-    AndroidDeviceInfoList parseAvdList(const QString &output);
-
-private:
-    bool parseAvd(const QStringList &deviceInfo, AndroidDeviceInfo *avd);
-};
-
-
-AndroidAvdManager::AndroidAvdManager(const AndroidConfig &config):
-    m_config(config),
-    m_parser(new AvdManagerOutputParser)
+AndroidAvdManager::AndroidAvdManager(const AndroidConfig &config)
+    : m_config(config)
 {
 
 }
@@ -237,15 +201,77 @@ bool AndroidAvdManager::removeAvd(const QString &name) const
 {
     const CommandLine command(m_config.avdManagerToolPath(), {"delete", "avd", "-n", name});
     qCDebug(avdManagerLog) << "Running command (removeAvd):" << command.toUserOutput();
-    Utils::SynchronousProcess proc;
+    QtcProcess proc;
     proc.setTimeoutS(5);
-    const Utils::SynchronousProcessResponse response = proc.runBlocking(command);
-    return response.result == Utils::SynchronousProcessResponse::Finished && response.exitCode == 0;
+    proc.setEnvironment(AndroidConfigurations::toolsEnvironment(m_config));
+    proc.setCommand(command);
+    proc.runBlocking();
+    return proc.result() == QtcProcess::FinishedWithSuccess;
+}
+
+static void avdConfigEditManufacturerTag(const QString &avdPathStr, bool recoverMode = false)
+{
+    const Utils::FilePath avdPath = Utils::FilePath::fromString(avdPathStr);
+    if (avdPath.exists()) {
+        const QString configFilePath = avdPath.pathAppended("config.ini").toString();
+        QFile configFile(configFilePath);
+        if (configFile.open(QIODevice::ReadWrite | QIODevice::Text)) {
+            QString newContent;
+            QTextStream textStream(&configFile);
+            while (!textStream.atEnd()) {
+                QString line = textStream.readLine();
+                if (!line.contains("hw.device.manufacturer"))
+                    newContent.append(line + "\n");
+                else if (recoverMode)
+                    newContent.append(line.replace("#", "") + "\n");
+                else
+                    newContent.append("#" + line + "\n");
+            }
+            configFile.resize(0);
+            textStream << newContent;
+            configFile.close();
+        }
+    }
+}
+
+static AndroidDeviceInfoList listVirtualDevices(const AndroidConfig &config)
+{
+    QString output;
+    AndroidDeviceInfoList avdList;
+    /*
+        Currenly avdmanager tool fails to parse some AVDs because the correct
+        device definitions at devices.xml does not have some of the newest devices.
+        Particularly, failing because of tag "hw.device.manufacturer", thus removing
+        it would make paring successful. However, it has to be returned afterwards,
+        otherwise, Android Studio would give an error during parsing also. So this fix
+        aim to keep support for Qt Creator and Android Studio.
+    */
+    QStringList allAvdErrorPaths;
+    QStringList avdErrorPaths;
+
+    do {
+        if (!AndroidAvdManager::avdManagerCommand(config, {"list", "avd"}, &output)) {
+            qCDebug(avdManagerLog)
+                << "Avd list command failed" << output << config.sdkToolsVersion();
+            return {};
+        }
+
+        avdErrorPaths.clear();
+        avdList = parseAvdList(output, &avdErrorPaths);
+        allAvdErrorPaths << avdErrorPaths;
+        for (const QString &avdPathStr : qAsConst(avdErrorPaths))
+            avdConfigEditManufacturerTag(avdPathStr); // comment out manufacturer tag
+    } while (!avdErrorPaths.isEmpty());               // try again
+
+    for (const QString &avdPathStr : qAsConst(allAvdErrorPaths))
+        avdConfigEditManufacturerTag(avdPathStr, true); // re-add manufacturer tag
+
+    return avdList;
 }
 
 QFuture<AndroidDeviceInfoList> AndroidAvdManager::avdList() const
 {
-    return Utils::runAsync(&AvdManagerOutputParser::listVirtualDevices, m_parser.get(), m_config);
+    return Utils::runAsync(listVirtualDevices, m_config);
 }
 
 QString AndroidAvdManager::startAvd(const QString &name) const
@@ -294,7 +320,7 @@ QString AndroidAvdManager::findAvd(const QString &avdName) const
 {
     QVector<AndroidDeviceInfo> devices = m_config.connectedDevices();
     foreach (AndroidDeviceInfo device, devices) {
-        if (device.type != AndroidDeviceInfo::Emulator)
+        if (device.type != ProjectExplorer::IDevice::Emulator)
             continue;
         if (device.avdname == avdName)
             return device.serialNumber;
@@ -309,7 +335,7 @@ QString AndroidAvdManager::waitForAvd(const QString &avdName,
     // 60 rounds of 2s sleeping, two minutes for the avd to start
     QString serialNumber;
     for (int i = 0; i < 60; ++i) {
-        if (cancelChecker())
+        if (cancelChecker && cancelChecker())
             return QString();
         serialNumber = findAvd(avdName);
         if (!serialNumber.isEmpty())
@@ -326,12 +352,13 @@ bool AndroidAvdManager::isAvdBooted(const QString &device) const
 
     const CommandLine command({m_config.adbToolPath(), arguments});
     qCDebug(avdManagerLog) << "Running command (isAvdBooted):" << command.toUserOutput();
-    SynchronousProcess adbProc;
+    QtcProcess adbProc;
     adbProc.setTimeoutS(10);
-    const SynchronousProcessResponse response = adbProc.runBlocking(command);
-    if (response.result != Utils::SynchronousProcessResponse::Finished)
+    adbProc.setCommand(command);
+    adbProc.runBlocking();
+    if (adbProc.result() != QtcProcess::FinishedWithSuccess)
         return false;
-    QString value = response.allOutput().trimmed();
+    QString value = adbProc.allOutput().trimmed();
     return value == "stopped";
 }
 
@@ -340,7 +367,7 @@ bool AndroidAvdManager::waitForBooted(const QString &serialNumber,
 {
     // found a serial number, now wait until it's done booting...
     for (int i = 0; i < 60; ++i) {
-        if (cancelChecker())
+        if (cancelChecker && cancelChecker())
             return false;
         if (isAvdBooted(serialNumber)) {
             return true;
@@ -353,147 +380,5 @@ bool AndroidAvdManager::waitForBooted(const QString &serialNumber,
     return false;
 }
 
-/* Currenly avdmanager tool fails to parse some AVDs because the correct
- * device definitions at devices.xml does not have some of the newest devices.
- * Particularly, failing because of tag "hw.device.manufacturer", thus removing
- * it would make paring successful. However, it has to be returned afterwards,
- * otherwise, Android Studio would give an error during parsing also. So this fix
- * aim to keep support for Qt Creator and Android Studio.
- */
-static const QString avdManufacturerError = "no longer exists as a device";
-static QStringList avdErrorPaths;
-
-static void AvdConfigEditManufacturerTag(const QString &avdPathStr, bool recoverMode = false)
-{
-    const Utils::FilePath avdPath = Utils::FilePath::fromString(avdPathStr);
-    if (avdPath.exists()) {
-        const QString configFilePath = avdPath.pathAppended("config.ini").toString();
-        QFile configFile(configFilePath);
-        if (configFile.open(QIODevice::ReadWrite | QIODevice::Text)) {
-            QString newContent;
-            QTextStream textStream(&configFile);
-            while (!textStream.atEnd()) {
-                QString line = textStream.readLine();
-                if (!line.contains("hw.device.manufacturer"))
-                    newContent.append(line + "\n");
-                else if (recoverMode)
-                    newContent.append(line.replace("#", "") + "\n");
-                else
-                    newContent.append("#" + line + "\n");
-            }
-            configFile.resize(0);
-            textStream << newContent;
-            configFile.close();
-        }
-    }
-}
-
-AndroidDeviceInfoList AvdManagerOutputParser::listVirtualDevices(const AndroidConfig &config)
-{
-    QString output;
-    AndroidDeviceInfoList avdList;
-
-    do {
-        if (!AndroidAvdManager::avdManagerCommand(config, {"list", "avd"}, &output)) {
-            qCDebug(avdManagerLog)
-                << "Avd list command failed" << output << config.sdkToolsVersion();
-            return {};
-        }
-
-        avdList = parseAvdList(output);
-    } while (output.contains(avdManufacturerError));
-
-    for (const QString &avdPathStr : avdErrorPaths)
-        AvdConfigEditManufacturerTag(avdPathStr, true);
-
-    return avdList;
-}
-
-AndroidDeviceInfoList AvdManagerOutputParser::parseAvdList(const QString &output)
-{
-    AndroidDeviceInfoList avdList;
-    QStringList avdInfo;
-    auto parseAvdInfo = [&avdInfo, &avdList, this] () {
-        AndroidDeviceInfo avd;
-        if (!avdInfo.filter(avdManufacturerError).isEmpty()) {
-            for (const QString &line : avdInfo) {
-                QString value;
-                if (valueForKey(avdInfoPathKey, line, &value)) {
-                    avdErrorPaths.append(value);
-                    AvdConfigEditManufacturerTag(value);
-                }
-            }
-        } else if (parseAvd(avdInfo, &avd)) {
-            // armeabi-v7a devices can also run armeabi code
-            if (avd.cpuAbi.contains(ProjectExplorer::Constants::ANDROID_ABI_ARMEABI_V7A))
-                avd.cpuAbi << ProjectExplorer::Constants::ANDROID_ABI_ARMEABI;
-            avd.state = AndroidDeviceInfo::OkState;
-            avd.type = AndroidDeviceInfo::Emulator;
-            avdList << avd;
-        } else {
-            qCDebug(avdManagerLog) << "Avd Parsing: Parsing failed: " << avdInfo;
-        }
-        avdInfo.clear();
-    };
-
-    for (const QString &line : output.split('\n')) {
-        if (line.startsWith("---------") || line.isEmpty())
-            parseAvdInfo();
-        else
-            avdInfo << line;
-    }
-
-    Utils::sort(avdList);
-
-    return avdList;
-}
-
-bool AvdManagerOutputParser::parseAvd(const QStringList &deviceInfo, AndroidDeviceInfo *avd)
-{
-    QTC_ASSERT(avd, return false);
-    for (const QString &line : deviceInfo) {
-        QString value;
-        if (valueForKey(avdInfoErrorKey, line)) {
-            qCDebug(avdManagerLog) << "Avd Parsing: Skip avd device. Error key found:" << line;
-            return false;
-        } else if (valueForKey(avdInfoNameKey, line, &value)) {
-            avd->avdname = value;
-        } else if (valueForKey(avdInfoPathKey, line, &value)) {
-            const Utils::FilePath avdPath = Utils::FilePath::fromString(value);
-            if (avdPath.exists())
-            {
-                // Get ABI.
-                const Utils::FilePath configFile = avdPath.pathAppended("config.ini");
-                QSettings config(configFile.toString(), QSettings::IniFormat);
-                value = config.value(avdInfoAbiKey).toString();
-                if (!value.isEmpty())
-                    avd->cpuAbi << value;
-                else
-                   qCDebug(avdManagerLog) << "Avd Parsing: Cannot find ABI:" << configFile;
-
-                // Get Target
-                const QString avdInfoFileName = avd->avdname + ".ini";
-                const Utils::FilePath
-                        avdInfoFile = avdPath.parentDir().pathAppended(avdInfoFileName);
-                QSettings avdInfo(avdInfoFile.toString(), QSettings::IniFormat);
-                value = avdInfo.value(avdInfoTargetKey).toString();
-                if (!value.isEmpty())
-                    avd->sdk = value.section('-', -1).toInt();
-                else
-                   qCDebug(avdManagerLog) << "Avd Parsing: Cannot find sdk API:" << avdInfoFile.toString();
-            }
-        } else if (valueForKey(avdInfoDeviceKey, line, &value)) {
-            avd->avdDevice = value.remove(0, 2);
-        } else if (valueForKey(avdInfoTargetTypeKey, line, &value)) {
-            avd->avdTarget = value.remove(0, 2);
-        } else if (valueForKey(avdInfoSkinKey, line, &value)) {
-            avd->avdSkin = value.remove(0, 2);
-        } else if (valueForKey(avdInfoSdcardKey, line, &value)) {
-            avd->avdSdcardSize = value.remove(0, 2);
-        }
-    }
-    return true;
-}
-
 } // namespace Internal
 } // namespace Android
