@@ -27,7 +27,7 @@
 #include "androidconfigurations.h"
 #include "ui_androidcreatekeystorecertificate.h"
 
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 
 #include <QFileDialog>
 #include <QRegularExpression>
@@ -169,9 +169,9 @@ void AndroidCreateKeystoreCertificate::buttonBoxAccepted()
     if (!validateUserInput())
         return;
 
-    m_keystoreFilePath = Utils::FilePath::fromString(QFileDialog::getSaveFileName(this, tr("Keystore Filename"),
-                                                                                  QDir::homePath() + QLatin1String("/android_release.keystore"),
-                                                                                  tr("Keystore files (*.keystore *.jks)")));
+    m_keystoreFilePath = FileUtils::getSaveFilePath(this, tr("Keystore Filename"),
+                                                    FileUtils::homePath() / "android_release.keystore",
+                                                    tr("Keystore files (*.keystore *.jks)"));
     if (m_keystoreFilePath.isEmpty())
         return;
     QString distinguishedNames(QString::fromLatin1("CN=%1, O=%2, L=%3, C=%4")
@@ -196,13 +196,15 @@ void AndroidCreateKeystoreCertificate::buttonBoxAccepted()
                               "-keypass", certificatePassword(),
                               "-dname", distinguishedNames});
 
-    SynchronousProcess genKeyCertProc;
+    QtcProcess genKeyCertProc;
     genKeyCertProc.setTimeoutS(15);
-    SynchronousProcessResponse response = genKeyCertProc.run(command);
+    genKeyCertProc.setCommand(command);
+    genKeyCertProc.setProcessUserEventWhileRunning();
+    genKeyCertProc.runBlocking();
 
-    if (response.result != Utils::SynchronousProcessResponse::Finished || response.exitCode != 0) {
+    if (genKeyCertProc.result() != QtcProcess::FinishedWithSuccess) {
         QMessageBox::critical(this, tr("Error"),
-                              response.exitMessage(command.executable().toString(), 15) + '\n' + response.allOutput());
+                              genKeyCertProc.exitMessage() + '\n' + genKeyCertProc.allOutput());
         return;
     }
     accept();
