@@ -32,26 +32,30 @@
 #include "androidconstants.h"
 #include "androidglobal.h"
 #include "androidavdmanager.h"
+#include "androidqtversion.h"
+#include "androiddevice.h"
 
 #include <coreplugin/fileutils.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/messagemanager.h>
 
 #include <projectexplorer/buildconfiguration.h>
+#include <projectexplorer/buildsystem.h>
 #include <projectexplorer/buildsteplist.h>
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectnodes.h>
 #include <projectexplorer/runconfiguration.h>
 #include <projectexplorer/target.h>
+#include <projectexplorer/taskhub.h>
 #include <projectexplorer/toolchain.h>
 
 #include <qtsupport/qtkitinformation.h>
 
 #include <utils/algorithm.h>
+#include <utils/layoutbuilder.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
-#include <utils/synchronousprocess.h>
 
 #include <QCheckBox>
 #include <QFileDialog>
@@ -69,7 +73,7 @@ namespace Android {
 namespace Internal {
 
 namespace {
-Q_LOGGING_CATEGORY(deployStepLog, "qtc.android.build.androiddeployqtstep", QtWarningMsg)
+static Q_LOGGING_CATEGORY(deployStepLog, "qtc.android.build.androiddeployqtstep", QtWarningMsg)
 }
 
 const QLatin1String UninstallPreviousPackageKey("UninstallPreviousPackage");
@@ -77,189 +81,184 @@ const QLatin1String InstallFailedInconsistentCertificatesString("INSTALL_PARSE_F
 const QLatin1String InstallFailedUpdateIncompatible("INSTALL_FAILED_UPDATE_INCOMPATIBLE");
 const QLatin1String InstallFailedPermissionModelDowngrade("INSTALL_FAILED_PERMISSION_MODEL_DOWNGRADE");
 const QLatin1String InstallFailedVersionDowngrade("INSTALL_FAILED_VERSION_DOWNGRADE");
-static const char *qmlProjectRunConfigIdName = "QmlProjectManager.QmlRunConfiguration";
-
-
-// AndroidDeployQtStepFactory
-
-AndroidDeployQtStepFactory::AndroidDeployQtStepFactory()
-{
-    registerStep<AndroidDeployQtStep>(AndroidDeployQtStep::stepId());
-    setSupportedStepList(ProjectExplorer::Constants::BUILDSTEPS_DEPLOY);
-    setSupportedDeviceType(Constants::ANDROID_DEVICE_TYPE);
-    setRepeatable(false);
-    setDisplayName(AndroidDeployQtStep::tr("Deploy to Android device or emulator"));
-}
-
-// AndroidDeployQtWidget
-
-class AndroidDeployQtWidget : public BuildStepConfigWidget
-{
-public:
-    AndroidDeployQtWidget(AndroidDeployQtStep *step)
-        : ProjectExplorer::BuildStepConfigWidget(step)
-    {
-        setDisplayName(QString("<b>%1</b>").arg(step->displayName()));
-        setSummaryText(displayName());
-
-        auto uninstallPreviousPackage = new QCheckBox(this);
-        uninstallPreviousPackage->setText(AndroidDeployQtStep::tr("Uninstall previous package"));
-        uninstallPreviousPackage->setChecked(step->uninstallPreviousPackage() > AndroidDeployQtStep::Keep);
-        uninstallPreviousPackage->setEnabled(step->uninstallPreviousPackage() != AndroidDeployQtStep::ForceUnintall);
-
-        auto resetDefaultDevices = new QPushButton(this);
-        resetDefaultDevices->setText(AndroidDeployQtStep::tr("Reset Default Devices"));
-
-        auto installMinistroButton = new QPushButton(this);
-        installMinistroButton->setText(AndroidDeployQtStep::tr("Install Ministro from APK"));
-
-        connect(installMinistroButton, &QAbstractButton::clicked, this, [this, step] {
-            QString packagePath =
-                    QFileDialog::getOpenFileName(this,
-                                                 AndroidDeployQtStep::tr("Qt Android Smart Installer"),
-                                                 QDir::homePath(),
-                                                 AndroidDeployQtStep::tr("Android package (*.apk)"));
-            if (!packagePath.isEmpty())
-                AndroidManager::installQASIPackage(step->target(), packagePath);
-        });
-
-        connect(resetDefaultDevices, &QAbstractButton::clicked, this, [step] {
-            AndroidConfigurations::clearDefaultDevices(step->project());
-        });
-
-        connect(uninstallPreviousPackage, &QAbstractButton::toggled,
-                step, &AndroidDeployQtStep::setUninstallPreviousPackage);
-
-        auto layout = new QVBoxLayout(this);
-        layout->addWidget(uninstallPreviousPackage);
-        layout->addWidget(resetDefaultDevices);
-        layout->addWidget(installMinistroButton);
-    }
-};
 
 // AndroidDeployQtStep
 
-AndroidDeployQtStep::AndroidDeployQtStep(ProjectExplorer::BuildStepList *parent)
-    : ProjectExplorer::BuildStep(parent, stepId())
+AndroidDeployQtStep::AndroidDeployQtStep(BuildStepList *parent, Utils::Id id)
+    : BuildStep(parent, id)
 {
     setImmutable(true);
-    m_uninstallPreviousPackage = QtSupport::QtKitAspect::qtVersion(target()->kit())->qtVersion() < QtSupport::QtVersionNumber(5, 4, 0);
+    setUserExpanded(true);
 
-    //: AndroidDeployQtStep default display name
-    setDefaultDisplayName(tr("Deploy to Android device"));
+    m_uninstallPreviousPackage = addAspect<BoolAspect>();
+    m_uninstallPreviousPackage->setSettingsKey(UninstallPreviousPackageKey);
+    m_uninstallPreviousPackage->setLabel(tr("Uninstall the existing app before deployment"),
+                                         BoolAspect::LabelPlacement::AtCheckBox);
+    m_uninstallPreviousPackage->setValue(false);
+
+    const QtSupport::BaseQtVersion * const qt = QtSupport::QtKitAspect::qtVersion(kit());
+    const bool forced = qt && qt->qtVersion() < QtSupport::QtVersionNumber(5, 4, 0);
+    if (forced) {
+        m_uninstallPreviousPackage->setValue(true);
+        m_uninstallPreviousPackage->setEnabled(false);
+    }
 
     connect(this, &AndroidDeployQtStep::askForUninstall,
             this, &AndroidDeployQtStep::slotAskForUninstall,
             Qt::BlockingQueuedConnection);
-
-    connect(this, &AndroidDeployQtStep::setSerialNumber,
-            this, &AndroidDeployQtStep::slotSetSerialNumber);
-}
-
-Core::Id AndroidDeployQtStep::stepId()
-{
-    return "Qt4ProjectManager.AndroidDeployQtStep";
 }
 
 bool AndroidDeployQtStep::init()
 {
+    QtSupport::BaseQtVersion *version = QtSupport::QtKitAspect::qtVersion(kit());
+    if (!version) {
+        reportWarningOrError(tr("The Qt version for kit %1 is invalid.").arg(kit()->displayName()),
+                             Task::Error);
+        return false;
+    }
+
     m_androiddeployqtArgs = CommandLine();
 
     m_androidABIs = AndroidManager::applicationAbis(target());
     if (m_androidABIs.isEmpty()) {
-        emit addOutput(tr("No Android arch set by the .pro file."), OutputFormat::Stderr);
+        reportWarningOrError(tr("No Android architecture (ABI) is set by the project."),
+                             Task::Error);
         return false;
     }
 
-    emit addOutput(tr("Initializing deployment to Android device/simulator"), OutputFormat::Stdout);
+    emit addOutput(tr("Initializing deployment to Android device/simulator"),
+                   OutputFormat::NormalMessage);
 
     RunConfiguration *rc = target()->activeRunConfiguration();
-    QTC_ASSERT(rc, return false);
-    const bool deployQtLive = rc->id().name().startsWith(qmlProjectRunConfigIdName);
-    ProjectExplorer::BuildConfiguration *bc = buildConfiguration();
-    QTC_ASSERT(deployQtLive || bc, return false);
+    QTC_ASSERT(rc, reportWarningOrError(tr("The kit's run configuration is invalid."), Task::Error);
+            return false);
+    BuildConfiguration *bc = target()->activeBuildConfiguration();
+    QTC_ASSERT(bc, reportWarningOrError(tr("The kit's build configuration is invalid."),
+                                        Task::Error);
+            return false);
 
-    auto androidBuildApkStep = AndroidBuildApkStep::findInBuild(bc);
-    int minTargetApi = AndroidManager::minimumSDK(target());
+    auto androidBuildApkStep = bc->buildSteps()->firstOfType<AndroidBuildApkStep>();
+    const int minTargetApi = AndroidManager::minimumSDK(target());
     qCDebug(deployStepLog) << "Target architecture:" << m_androidABIs
                            << "Min target API" << minTargetApi;
 
     // Try to re-use user-provided information from an earlier step of the same type.
     BuildStepList *bsl = stepList();
-    QTC_ASSERT(bsl, return false);
+    QTC_ASSERT(bsl, reportWarningOrError(tr("The kit's build steps list is invalid."), Task::Error);
+            return false);
     auto androidDeployQtStep = bsl->firstOfType<AndroidDeployQtStep>();
-    QTC_ASSERT(androidDeployQtStep, return false);
+    QTC_ASSERT(androidDeployQtStep,
+               reportWarningOrError(tr("The kit's deploy configuration is invalid."), Task::Error);
+            return false);
     AndroidDeviceInfo info;
     if (androidDeployQtStep != this)
         info = androidDeployQtStep->m_deviceInfo;
 
+    const BuildSystem *bs = buildSystem();
+    auto selectedAbis = bs->property(Constants::AndroidAbis).toStringList();
+
+    const QString buildKey = target()->activeBuildKey();
+    if (selectedAbis.isEmpty())
+        selectedAbis = bs->extraData(buildKey, Constants::AndroidAbis).toStringList();
+
+    if (selectedAbis.isEmpty())
+        selectedAbis.append(bs->extraData(buildKey, Constants::AndroidAbi).toString());
+
+    // TODO: use AndroidDevice directly instead of AndroidDeviceInfo.
     if (!info.isValid()) {
-        info = AndroidConfigurations::showDeviceDialog(project(), minTargetApi, m_androidABIs);
+        const IDevice *dev = DeviceKitAspect::device(kit()).data();
+        if (!dev) {
+            reportWarningOrError(tr("The deployment device \"%1\" is invalid.").arg(""),
+                                 Task::Error);
+            return false;
+        }
+
+        info = AndroidDevice::androidDeviceInfoFromIDevice(dev);
         m_deviceInfo = info; // Keep around for later steps
+
+        if (!info.isValid()) {
+            reportWarningOrError(tr("The deployment device \"%1\" is invalid.")
+                                 .arg(dev->displayName()), Task::Error);
+            return false;
+        }
+
+        const AndroidDevice *androidDev = static_cast<const AndroidDevice *>(dev);
+        if (androidDev && !androidDev->canSupportAbis(selectedAbis)) {
+            const QString error = tr("The deployment device \"%1\" does not support the "
+                                     "architectures used by the kit.\n"
+                                     "The kit supports \"%2\", but the device uses \"%3\".")
+                                      .arg(dev->displayName()).arg(selectedAbis.join(", "))
+                                      .arg(androidDev->supportedAbis().join(", "));
+            reportWarningOrError(error, Task::Error);
+            return false;
+        }
+
+        if (androidDev && !androidDev->canHandleDeployments()) {
+            reportWarningOrError(tr("The deployment device \"%1\" is disconnected.")
+                                 .arg(dev->displayName()), Task::Error);
+            return false;
+        }
     }
 
-    if (!info.isValid()) // aborted
-        return false;
+    const QtSupport::BaseQtVersion * const qt = QtSupport::QtKitAspect::qtVersion(kit());
+    if (qt && qt->supportsMultipleQtAbis() && !selectedAbis.contains(info.cpuAbi.first())) {
+        TaskHub::addTask(DeploymentTask(Task::Warning,
+            tr("Android: The main ABI of the deployment device (%1) is not selected. The app "
+               "execution or debugging might not work properly. Add it from Projects > Build > "
+               "Build Steps > qmake > ABIs.")
+                .arg(info.cpuAbi.first())));
+    }
 
     m_avdName = info.avdname;
     m_serialNumber = info.serialNumber;
-    qCDebug(deployStepLog) << "Selected Device:" << info;
+    qCDebug(deployStepLog) << "Selected device info:" << info;
 
-    if (!deployQtLive)
-        gatherFilesToPull();
+    gatherFilesToPull();
 
     AndroidManager::setDeviceSerialNumber(target(), m_serialNumber);
     AndroidManager::setDeviceApiLevel(target(), info.sdk);
     AndroidManager::setDeviceAbis(target(), info.cpuAbi);
 
-    emit addOutput(tr("Deploying to %1").arg(m_serialNumber), OutputFormat::Stdout);
+    emit addOutput(tr("Deploying to %1").arg(m_serialNumber), OutputFormat::NormalMessage);
 
-    QtSupport::BaseQtVersion *version = QtSupport::QtKitAspect::qtVersion(target()->kit());
-    if (!version)
-        return false;
-
-    m_uninstallPreviousPackageRun = m_uninstallPreviousPackage;
+    m_uninstallPreviousPackageRun = m_uninstallPreviousPackage->value();
     if (m_uninstallPreviousPackageRun)
         m_manifestName = AndroidManager::manifestPath(target());
 
-    m_useAndroiddeployqt = !deployQtLive &&
-            version->qtVersion() >= QtSupport::QtVersionNumber(5, 4, 0);
-
+    m_useAndroiddeployqt = version->qtVersion() >= QtSupport::QtVersionNumber(5, 4, 0);
     if (m_useAndroiddeployqt) {
-        const ProjectNode *node = target()->project()->findNodeForBuildKey(rc->buildKey());
-        if (!node)
+        const QString buildKey = target()->activeBuildKey();
+        const ProjectNode *node = target()->project()->findNodeForBuildKey(buildKey);
+        if (!node) {
+            reportWarningOrError(tr("The deployment step's project node is invalid."), Task::Error);
             return false;
+        }
         m_apkPath = Utils::FilePath::fromString(node->data(Constants::AndroidApk).toString());
         if (!m_apkPath.isEmpty()) {
             m_manifestName = Utils::FilePath::fromString(node->data(Constants::AndroidManifest).toString());
             m_command = AndroidConfigurations::currentConfig().adbToolPath();
             AndroidManager::setManifestPath(target(), m_manifestName);
         } else {
-            QString jsonFile = node->data(Constants::AndroidDeploySettingsFile).toString();
+            QString jsonFile = AndroidQtVersion::androidDeploymentSettings(target()).toString();
             if (jsonFile.isEmpty()) {
-                emit addOutput(tr("Cannot find the androiddeploy Json file."), OutputFormat::Stderr);
+                reportWarningOrError(tr("Cannot find the androiddeployqt input JSON file."),
+                                     Task::Error);
                 return false;
             }
             m_command = version->hostBinPath();
             if (m_command.isEmpty()) {
-                emit addOutput(tr("Cannot find the androiddeployqt tool."), OutputFormat::Stderr);
+                reportWarningOrError(tr("Cannot find the androiddeployqt tool."), Task::Error);
                 return false;
             }
-            qCDebug(deployStepLog) << "Using androiddeployqt";
-            m_command = m_command.pathAppended(HostOsInfo::withExecutableSuffix("androiddeployqt"));
+            m_command = m_command.pathAppended("androiddeployqt").withExecutableSuffix();
 
-            m_workingDirectory = bc->buildDirectory().pathAppended(Constants::ANDROID_BUILDDIRECTORY);
+            m_workingDirectory = AndroidManager::androidBuildDirectory(target());
 
             m_androiddeployqtArgs.addArgs({"--verbose",
                                            "--output", m_workingDirectory.toString(),
                                            "--no-build",
                                            "--input", jsonFile});
 
-            if (androidBuildApkStep && androidBuildApkStep->useMinistro()) {
-                qCDebug(deployStepLog) << "Using ministro";
-                m_androiddeployqtArgs.addArgs({"--deployment", "ministro"});
-            }
-
             m_androiddeployqtArgs.addArg("--gradle");
 
             if (androidBuildApkStep && androidBuildApkStep->signPackage()) {
@@ -273,9 +272,8 @@ bool AndroidDeployQtStep::init()
     } else {
         m_uninstallPreviousPackageRun = true;
         m_command = AndroidConfigurations::currentConfig().adbToolPath();
-        const AndroidConfig &config = AndroidConfigurations::currentConfig();
-        m_apkPath = deployQtLive ? config.qtLiveApkPath() : AndroidManager::apkPath(target());
-        m_workingDirectory = bc ? bc->buildDirectory() : FilePath();
+        m_apkPath = AndroidManager::apkPath(target());
+        m_workingDirectory = bc ? AndroidManager::buildDirectory(target()): FilePath();
     }
     m_environment = bc ? bc->environment() : Utils::Environment();
 
@@ -304,40 +302,19 @@ AndroidDeployQtStep::DeployErrorCode AndroidDeployQtStep::runDeploy()
     } else {
         RunConfiguration *rc = target()->activeRunConfiguration();
         QTC_ASSERT(rc, return DeployErrorCode::Failure);
-        const bool deployQtLive = rc->id().name().startsWith(qmlProjectRunConfigIdName);
         QString packageName;
-        int packageVersion = -1;
-        if (deployQtLive) {
-            // Do not install Qt live if apk is already installed or the same version is
-            // being installed.
-            AndroidManager::apkInfo(m_apkPath, &packageName, &packageVersion);
-            if (AndroidManager::packageInstalled(m_serialNumber, packageName)) {
-                int installedVersion = AndroidManager::packageVersionCode(m_serialNumber,
-                                                                          packageName);
-                if (installedVersion == packageVersion) {
-                    qCDebug(deployStepLog) << "Qt live APK already installed. APK version:"
-                                           << packageVersion << "Installed version:"
-                                           << installedVersion;
-                    return DeployErrorCode::NoError;
-                } else {
-                    qCDebug(deployStepLog) << "Re-installing Qt live APK. Version mismatch."
-                                           << "APK version:" << packageVersion
-                                           << "Installed version:" << installedVersion;
-                }
-            } else {
-                qCDebug(deployStepLog) << "Installing Qt live APK. APK version:" << packageVersion;
-            }
-        }
 
         if (m_uninstallPreviousPackageRun) {
-            if (!deployQtLive)
-                packageName = AndroidManager::packageName(m_manifestName);
+            packageName = AndroidManager::packageName(m_manifestName);
             if (packageName.isEmpty()) {
-                emit addOutput(tr("Cannot find the package name."), OutputFormat::Stderr);
+                reportWarningOrError(tr("Cannot find the package name from the Android Manifest "
+                                        "file \"%1\".").arg(m_manifestName.toUserOutput()),
+                                     Task::Error);
                 return Failure;
             }
-            qCDebug(deployStepLog) << "Uninstalling previous package";
-            emit addOutput(tr("Uninstall previous package %1.").arg(packageName), OutputFormat::NormalMessage);
+            const QString msg = tr("Uninstalling the previous package \"%1\".").arg(packageName);
+            qCDebug(deployStepLog) << msg;
+            emit addOutput(msg, OutputFormat::NormalMessage);
             runCommand({m_adbPath,
                        AndroidDeviceInfo::adbSelector(m_serialNumber)
                        << "uninstall" << packageName});
@@ -347,24 +324,26 @@ AndroidDeployQtStep::DeployErrorCode AndroidDeployQtStep::runDeploy()
         cmd.addArgs({"install", "-r", m_apkPath.toString()});
     }
 
-    m_process = new Utils::QtcProcess;
+    m_process = new QtcProcess;
     m_process->setCommand(cmd);
-    m_process->setWorkingDirectory(m_workingDirectory.toString());
+    m_process->setWorkingDirectory(m_workingDirectory);
     m_process->setEnvironment(m_environment);
-
-    if (Utils::HostOsInfo::isWindowsHost())
-        m_process->setUseCtrlCStub(true);
+    m_process->setUseCtrlCStub(true);
 
     DeployErrorCode deployError = NoError;
-    connect(m_process, &Utils::QtcProcess::readyReadStandardOutput,
-            std::bind(&AndroidDeployQtStep::processReadyReadStdOutput, this, std::ref(deployError)));
-    connect(m_process, &Utils::QtcProcess::readyReadStandardError,
-            std::bind(&AndroidDeployQtStep::processReadyReadStdError, this, std::ref(deployError)));
+
+    m_process->setStdOutLineCallback([this, &deployError](const QString &line) {
+        deployError |= parseDeployErrors(line);
+        stdOutput(line);
+    });
+    m_process->setStdErrLineCallback([this, &deployError](const QString &line) {
+        deployError |= parseDeployErrors(line);
+        stdError(line);
+    });
 
     m_process->start();
 
-    emit addOutput(tr("Starting: \"%1\"").arg(cmd.toUserOutput()),
-                   BuildStep::OutputFormat::NormalMessage);
+    emit addOutput(tr("Starting: \"%1\"").arg(cmd.toUserOutput()), OutputFormat::NormalMessage);
 
     while (!m_process->waitForFinished(200)) {
         if (m_process->state() == QProcess::NotRunning)
@@ -395,22 +374,27 @@ AndroidDeployQtStep::DeployErrorCode AndroidDeployQtStep::runDeploy()
 
     if (exitStatus == QProcess::NormalExit && exitCode == 0) {
         emit addOutput(tr("The process \"%1\" exited normally.").arg(m_command.toUserOutput()),
-                       BuildStep::OutputFormat::NormalMessage);
+                       OutputFormat::NormalMessage);
     } else if (exitStatus == QProcess::NormalExit) {
-        emit addOutput(tr("The process \"%1\" exited with code %2.")
-                       .arg(m_command.toUserOutput(), QString::number(exitCode)),
-                       BuildStep::OutputFormat::ErrorMessage);
+        const QString error = tr("The process \"%1\" exited with code %2.")
+                                  .arg(m_command.toUserOutput(), QString::number(exitCode));
+        reportWarningOrError(error, Task::Error);
     } else {
-        emit addOutput(tr("The process \"%1\" crashed.").arg(m_command.toUserOutput()),
-                       BuildStep::OutputFormat::ErrorMessage);
+        const QString error = tr("The process \"%1\" crashed.").arg(m_command.toUserOutput());
+        reportWarningOrError(error, Task::Error);
     }
 
     if (deployError != NoError) {
-        if (m_uninstallPreviousPackageRun)
+        if (m_uninstallPreviousPackageRun) {
             deployError = Failure; // Even re-install failed. Set to Failure.
+            reportWarningOrError(
+                        tr("Installing the app failed even after uninstalling the previous one."),
+                        Task::Error);
+        }
     } else if (exitCode != 0 || exitStatus != QProcess::NormalExit) {
         // Set the deployError to Failure when no deployError code was detected
         // but the adb tool failed otherwise relay the detected deployError.
+        reportWarningOrError(tr("Installing the app failed with an unknown error."), Task::Error);
         deployError = Failure;
     }
 
@@ -445,27 +429,26 @@ void AndroidDeployQtStep::slotAskForUninstall(DeployErrorCode errorCode)
       mask <<= 1;
     }
 
-    uninstallMsg.append(tr("\nUninstalling the installed package may solve the issue.\nDo you want to uninstall the existing package?"));
+    uninstallMsg.append(tr("\nUninstalling the installed package may solve the issue.\n"
+                           "Do you want to uninstall the existing package?"));
     int button = QMessageBox::critical(nullptr, tr("Install failed"), uninstallMsg,
                                        QMessageBox::Yes, QMessageBox::No);
     m_askForUninstall = button == QMessageBox::Yes;
 }
 
-void AndroidDeployQtStep::slotSetSerialNumber(const QString &serialNumber)
-{
-    qCDebug(deployStepLog) << "Target device serial number change:" << serialNumber;
-    AndroidManager::setDeviceSerialNumber(target(), serialNumber);
-}
-
 bool AndroidDeployQtStep::runImpl()
 {
     if (!m_avdName.isEmpty()) {
         QString serialNumber = AndroidAvdManager().waitForAvd(m_avdName, cancelChecker());
         qCDebug(deployStepLog) << "Deploying to AVD:" << m_avdName << serialNumber;
-        if (serialNumber.isEmpty())
+        if (serialNumber.isEmpty()) {
+            reportWarningOrError(tr("The deployment AVD \"%1\" cannot be started.")
+                                 .arg(m_avdName), Task::Error);
             return false;
+        }
         m_serialNumber = serialNumber;
-        emit setSerialNumber(serialNumber);
+        qCDebug(deployStepLog) << "Deployment device serial number changed:" << serialNumber;
+        AndroidManager::setDeviceSerialNumber(target(), serialNumber);
     }
 
     DeployErrorCode returnValue = runDeploy();
@@ -480,15 +463,21 @@ bool AndroidDeployQtStep::runImpl()
     if (!m_filesToPull.isEmpty())
         emit addOutput(tr("Pulling files necessary for debugging."), OutputFormat::NormalMessage);
 
+    // Note that values are not necessarily unique, e.g. app_process is looked up in several
+    // directories
     for (auto itr = m_filesToPull.constBegin(); itr != m_filesToPull.constEnd(); ++itr) {
         QFile::remove(itr.value());
+    }
+
+    for (auto itr = m_filesToPull.constBegin(); itr != m_filesToPull.constEnd(); ++itr) {
         runCommand({m_adbPath,
                    AndroidDeviceInfo::adbSelector(m_serialNumber)
                    << "pull" << itr.key() << itr.value()});
         if (!QFileInfo::exists(itr.value())) {
-            emit addOutput(tr("Package deploy: Failed to pull \"%1\" to \"%2\".")
-                           .arg(itr.key())
-                           .arg(itr.value()), OutputFormat::ErrorMessage);
+            const QString error = tr("Package deploy: Failed to pull \"%1\" to \"%2\".")
+                                      .arg(itr.key())
+                                      .arg(itr.value());
+            reportWarningOrError(error, Task::Error);
         }
     }
 
@@ -498,9 +487,8 @@ bool AndroidDeployQtStep::runImpl()
 void AndroidDeployQtStep::gatherFilesToPull()
 {
     m_filesToPull.clear();
-    ProjectExplorer::BuildConfiguration *bc = buildConfiguration();
-    QString buildDir = bc ? bc->buildDirectory().toString() : QString();
-    if (bc && !buildDir.endsWith("/")) {
+    QString buildDir = AndroidManager::buildDirectory(target()).toString();
+    if (!buildDir.endsWith("/")) {
         buildDir += "/";
     }
 
@@ -510,7 +498,8 @@ void AndroidDeployQtStep::gatherFilesToPull()
     QString linkerName("linker");
     QString libDirName("lib");
     auto preferreABI = AndroidManager::apkDevicePreferredAbi(target());
-    if (preferreABI == "arm64-v8a" || preferreABI == "x86_64") {
+    if (preferreABI == ProjectExplorer::Constants::ANDROID_ABI_ARM64_V8A
+            || preferreABI == ProjectExplorer::Constants::ANDROID_ABI_X86_64) {
         m_filesToPull["/system/bin/app_process64"] = buildDir + "app_process";
         libDirName = "lib64";
         linkerName = "linker64";
@@ -522,9 +511,8 @@ void AndroidDeployQtStep::gatherFilesToPull()
     m_filesToPull["/system/bin/" + linkerName] = buildDir + linkerName;
     m_filesToPull["/system/" + libDirName + "/libc.so"] = buildDir + "libc.so";
 
-    qCDebug(deployStepLog) << "Files to pull from device:";
     for (auto itr = m_filesToPull.constBegin(); itr != m_filesToPull.constEnd(); ++itr)
-        qCDebug(deployStepLog) << itr.key() << "to" << itr.value();
+        qCDebug(deployStepLog) << "Pulling file from device:" << itr.key() << "to:" << itr.value();
 }
 
 void AndroidDeployQtStep::doRun()
@@ -534,29 +522,40 @@ void AndroidDeployQtStep::doRun()
 
 void AndroidDeployQtStep::runCommand(const CommandLine &command)
 {
-    SynchronousProcess buildProc;
+    QtcProcess buildProc;
     buildProc.setTimeoutS(2 * 60);
     emit addOutput(tr("Package deploy: Running command \"%1\".").arg(command.toUserOutput()),
                    OutputFormat::NormalMessage);
-    SynchronousProcessResponse response = buildProc.run(command);
-    if (response.result != SynchronousProcessResponse::Finished || response.exitCode != 0)
-        emit addOutput(response.exitMessage(command.executable().toString(), 2 * 60),
-                       OutputFormat::ErrorMessage);
+
+    buildProc.setCommand(command);
+    buildProc.setProcessUserEventWhileRunning();
+    buildProc.runBlocking();
+    if (buildProc.result() != QtcProcess::FinishedWithSuccess)
+        reportWarningOrError(buildProc.exitMessage(), Task::Error);
 }
 
-ProjectExplorer::BuildStepConfigWidget *AndroidDeployQtStep::createConfigWidget()
+QWidget *AndroidDeployQtStep::createConfigWidget()
 {
-    return new AndroidDeployQtWidget(this);
-}
+    auto widget = new QWidget;
+    auto installCustomApkButton = new QPushButton(widget);
+    installCustomApkButton->setText(tr("Install an APK File"));
 
-void AndroidDeployQtStep::processReadyReadStdOutput(DeployErrorCode &errorCode)
-{
-    m_process->setReadChannel(QProcess::StandardOutput);
-    while (m_process->canReadLine()) {
-        QString line = QString::fromLocal8Bit(m_process->readLine());
-        errorCode |= parseDeployErrors(line);
-        stdOutput(line);
-    }
+    connect(installCustomApkButton, &QAbstractButton::clicked, this, [this, widget] {
+        const FilePath packagePath
+                = FileUtils::getOpenFilePath(widget,
+                                             tr("Qt Android Installer"),
+                                             FileUtils::homePath(),
+                                             tr("Android package (*.apk)"));
+        if (!packagePath.isEmpty())
+            AndroidManager::installQASIPackage(target(), packagePath);
+    });
+
+    Layouting::Form builder;
+    builder.addRow(m_uninstallPreviousPackage);
+    builder.addRow(installCustomApkButton);
+    builder.attachTo(widget);
+
+    return widget;
 }
 
 void AndroidDeployQtStep::stdOutput(const QString &line)
@@ -564,22 +563,27 @@ void AndroidDeployQtStep::stdOutput(const QString &line)
     emit addOutput(line, BuildStep::OutputFormat::Stdout, BuildStep::DontAppendNewline);
 }
 
-void AndroidDeployQtStep::processReadyReadStdError(DeployErrorCode &errorCode)
-{
-    m_process->setReadChannel(QProcess::StandardError);
-    while (m_process->canReadLine()) {
-        QString line = QString::fromLocal8Bit(m_process->readLine());
-        errorCode |= parseDeployErrors(line);
-        stdError(line);
-    }
-}
-
 void AndroidDeployQtStep::stdError(const QString &line)
 {
     emit addOutput(line, BuildStep::OutputFormat::Stderr, BuildStep::DontAppendNewline);
+
+    QString newOutput = line;
+    newOutput.remove(QRegularExpression("^(\\n)+"));
+
+    if (newOutput.isEmpty())
+        return;
+
+    if (newOutput.startsWith("warning", Qt::CaseInsensitive)
+        || newOutput.startsWith("note", Qt::CaseInsensitive)
+        || newOutput.startsWith(QLatin1String("All files should be loaded."))) {
+        TaskHub::addTask(DeploymentTask(Task::Warning, newOutput));
+    } else {
+        TaskHub::addTask(DeploymentTask(Task::Error, newOutput));
+    }
 }
 
-AndroidDeployQtStep::DeployErrorCode AndroidDeployQtStep::parseDeployErrors(QString &deployOutputLine) const
+AndroidDeployQtStep::DeployErrorCode AndroidDeployQtStep::parseDeployErrors(
+        const QString &deployOutputLine) const
 {
     DeployErrorCode errorCode = NoError;
 
@@ -595,29 +599,22 @@ AndroidDeployQtStep::DeployErrorCode AndroidDeployQtStep::parseDeployErrors(QStr
     return errorCode;
 }
 
-bool AndroidDeployQtStep::fromMap(const QVariantMap &map)
+void AndroidDeployQtStep::reportWarningOrError(const QString &message, Task::TaskType type)
 {
-    m_uninstallPreviousPackage = map.value(UninstallPreviousPackageKey, m_uninstallPreviousPackage).toBool();
-    return ProjectExplorer::BuildStep::fromMap(map);
+    qCDebug(deployStepLog) << message;
+    emit addOutput(message, OutputFormat::ErrorMessage);
+    TaskHub::addTask(DeploymentTask(type, message));
 }
 
-QVariantMap AndroidDeployQtStep::toMap() const
-{
-    QVariantMap map = ProjectExplorer::BuildStep::toMap();
-    map.insert(UninstallPreviousPackageKey, m_uninstallPreviousPackage);
-    return map;
-}
+// AndroidDeployQtStepFactory
 
-void AndroidDeployQtStep::setUninstallPreviousPackage(bool uninstall)
+AndroidDeployQtStepFactory::AndroidDeployQtStepFactory()
 {
-    m_uninstallPreviousPackage = uninstall;
-}
-
-AndroidDeployQtStep::UninstallType AndroidDeployQtStep::uninstallPreviousPackage()
-{
-    if (QtSupport::QtKitAspect::qtVersion(target()->kit())->qtVersion() < QtSupport::QtVersionNumber(5, 4, 0))
-        return ForceUnintall;
-    return m_uninstallPreviousPackage ? Uninstall : Keep;
+    registerStep<AndroidDeployQtStep>(Constants::ANDROID_DEPLOY_QT_ID);
+    setSupportedStepList(ProjectExplorer::Constants::BUILDSTEPS_DEPLOY);
+    setSupportedDeviceType(Constants::ANDROID_DEVICE_TYPE);
+    setRepeatable(false);
+    setDisplayName(AndroidDeployQtStep::tr("Deploy to Android device"));
 }
 
 } // Internal
