@@ -31,7 +31,7 @@
 #include "androidmanifestdocument.h"
 #include "androidmanager.h"
 #include "androidservicewidget.h"
-#include "splashiconcontainerwidget.h"
+#include "splashscreencontainerwidget.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/editormanager/ieditor.h>
@@ -404,12 +404,6 @@ QGroupBox *Android::Internal::AndroidManifestEditorWidget::createApplicationGrou
     m_activityNameLineEdit = new QLineEdit(applicationGroupBox);
     formLayout->addRow(tr("Activity name:"), m_activityNameLineEdit);
 
-    m_targetLineEdit = new QComboBox(applicationGroupBox);
-    m_targetLineEdit->setEditable(true);
-    m_targetLineEdit->setDuplicatesEnabled(true);
-    m_targetLineEdit->installEventFilter(this);
-    formLayout->addRow(tr("Run:"), m_targetLineEdit);
-
     m_styleExtractMethod = new QComboBox(applicationGroupBox);
     formLayout->addRow(tr("Style extraction:"), m_styleExtractMethod);
     const QList<QStringList> styleMethodsMap = {
@@ -467,8 +461,6 @@ QGroupBox *Android::Internal::AndroidManifestEditorWidget::createApplicationGrou
             this, [this]() { setDirty(); });
     connect(m_activityNameLineEdit, &QLineEdit::textEdited,
             this, [this]() { setDirty(); });
-    connect(m_targetLineEdit, &QComboBox::currentTextChanged,
-            this, [this]() { setDirty(); });
     connect(m_styleExtractMethod,
             QOverload<int>::of(&QComboBox::currentIndexChanged),
             this, [this]() { setDirty(); });
@@ -492,7 +484,8 @@ QGroupBox *AndroidManifestEditorWidget::createAdvancedGroupBox(QWidget *parent)
     m_services = new AndroidServiceWidget(otherGroupBox);
     m_advanvedTabWidget->addTab(m_services, tr("Android services"));
 
-    m_splashButtons = new SplashIconContainerWidget(otherGroupBox, m_textEditorWidget);
+    m_splashButtons = new SplashScreenContainerWidget(otherGroupBox,
+                                                      m_textEditorWidget);
     m_advanvedTabWidget->addTab(m_splashButtons, tr("Splash screen"));
 
     connect(m_services, &AndroidServiceWidget::servicesModified, this, [this]() { setDirty(); });
@@ -500,7 +493,7 @@ QGroupBox *AndroidManifestEditorWidget::createAdvancedGroupBox(QWidget *parent)
             this, &AndroidManifestEditorWidget::clearInvalidServiceInfo);
     connect(m_services, &AndroidServiceWidget::servicesInvalid,
             this, &AndroidManifestEditorWidget::setInvalidServiceInfo);
-    connect(m_splashButtons, &SplashIconContainerWidget::splashScreensModified,
+    connect(m_splashButtons, &SplashScreenContainerWidget::splashScreensModified,
             this, [this]() { setDirty(); });
     connect(m_iconButtons, &AndroidManifestEditorIconContainerWidget::iconsModified,
             this, [this]() { setDirty(); });
@@ -531,16 +524,6 @@ void AndroidManifestEditorWidget::initializePage()
     insertWidget(Source, m_textEditorWidget);
 }
 
-bool AndroidManifestEditorWidget::eventFilter(QObject *obj, QEvent *event)
-{
-    if (obj == m_targetLineEdit) {
-        if (event->type() == QEvent::FocusIn)
-            QTimer::singleShot(0, this, &AndroidManifestEditorWidget::updateTargetComboBox);
-    }
-
-    return QWidget::eventFilter(obj, event);
-}
-
 void AndroidManifestEditorWidget::focusInEvent(QFocusEvent *event)
 {
     if (currentWidget()) {
@@ -551,30 +534,6 @@ void AndroidManifestEditorWidget::focusInEvent(QFocusEvent *event)
     }
 }
 
-void AndroidManifestEditorWidget::updateTargetComboBox()
-{
-    QStringList items;
-    if (Target *target = androidTarget(m_textEditorWidget->textDocument()->filePath())) {
-        ProjectNode *root = target->project()->rootProjectNode();
-        root->forEachProjectNode([&items](const ProjectNode *projectNode) {
-            items << projectNode->targetApplications();
-        });
-        items.sort();
-    }
-
-    // QComboBox randomly resets what the user has entered
-    // if all rows are removed, thus we ensure that the current text
-    // is not removed by first adding it and then removing all old rows
-    // and then adding the new rows
-    QString text = m_targetLineEdit->currentText();
-    m_targetLineEdit->addItem(text);
-    while (m_targetLineEdit->count() > 1)
-        m_targetLineEdit->removeItem(0);
-    items.removeDuplicates();
-    items.removeAll(text);
-    m_targetLineEdit->addItems(items);
-}
-
 void AndroidManifestEditorWidget::updateAfterFileLoad()
 {
     QString error;
@@ -761,18 +720,21 @@ void AndroidManifestEditorWidget::updateInfoBar()
 
 void AndroidManifestEditorWidget::updateSdkVersions()
 {
-    QPair<int, int> apiLevels = AndroidManager::apiLevelRange();
-    for (int i = apiLevels.first; i < apiLevels.second + 1; ++i)
-        m_androidMinSdkVersion->addItem(tr("API %1: %2")
-                                        .arg(i)
-                                        .arg(AndroidManager::androidNameForApiLevel(i)),
-                                        i);
+    static const QPair<int, int> sdkPair = qMakePair(16, 31);
+    int minSdk = sdkPair.first;
+    const int targetSdk = sdkPair.second;
+    const Target *target = androidTarget(m_textEditorWidget->textDocument()->filePath());
+    if (target) {
+        const QtSupport::BaseQtVersion *qt = QtSupport::QtKitAspect::qtVersion(target->kit());
+        minSdk = AndroidManager::defaultMinimumSDK(qt);
+    }
 
-    for (int i = apiLevels.first; i < apiLevels.second + 1; ++i)
-        m_androidTargetSdkVersion->addItem(tr("API %1: %2")
-                                           .arg(i)
-                                           .arg(AndroidManager::androidNameForApiLevel(i)),
-                                           i);
+    for (int i = minSdk; i <= targetSdk; ++i) {
+        const QString apiStr = tr("API %1: %2").arg(i)
+                                               .arg(AndroidManager::androidNameForApiLevel(i));
+        m_androidMinSdkVersion->addItem(apiStr, i);
+        m_androidTargetSdkVersion->addItem(apiStr, i);
+    }
 }
 
 void AndroidManifestEditorWidget::updateInfoBar(const QString &errorMessage, int line, int column)
@@ -876,11 +838,7 @@ void AndroidManifestEditorWidget::syncToWidgets(const QDomDocument &doc)
     enum SplashImageParseGuard {splashNone = 0, splash = 1, portraitSplash = 2, landscapeSplash = 4, splashDone = 8};
     int splashParseGuard = SplashImageParseGuard::splashNone;
     while (!metadataElem.isNull()) {
-        if (metadataElem.attribute(QLatin1String("android:name")) == QLatin1String("android.app.lib_name")
-                && !(activityParseGuard & ActivityParseGuard::libName)) {
-            m_targetLineEdit->setEditText(metadataElem.attribute(QLatin1String("android:value")));
-            activityParseGuard |= ActivityParseGuard::libName;
-        } else if (metadataElem.attribute(QLatin1String("android:name"))
+        if (metadataElem.attribute(QLatin1String("android:name"))
                    == QLatin1String("android.app.extract_android_style")
                    && !(activityParseGuard & ActivityParseGuard::styleExtract)) {
             m_styleExtractMethod->setCurrentText(
@@ -890,7 +848,8 @@ void AndroidManifestEditorWidget::syncToWidgets(const QDomDocument &doc)
                    == QLatin1String("android.app.splash_screen_sticky")
                    && !(activityParseGuard & ActivityParseGuard::stickySplash)) {
             QString sticky = metadataElem.attribute(QLatin1String("android:value"));
-            m_splashButtons->setSticky(sticky == QLatin1String("true"));
+            m_currentsplashSticky = (sticky == QLatin1String("true"));
+            m_splashButtons->setSticky(m_currentsplashSticky);
             activityParseGuard |= ActivityParseGuard::stickySplash;
         } else if (metadataElem.attribute(QLatin1String("android:name"))
                    .startsWith(QLatin1String("android.app.splash_screen_drawable"))
@@ -900,17 +859,20 @@ void AndroidManifestEditorWidget::syncToWidgets(const QDomDocument &doc)
             QLatin1String drawable = QLatin1String("@drawable/");
             QString splashImageValue = metadataElem.attribute(QLatin1String("android:resource"));
             QString splashImageName;
-            if (splashImageValue.startsWith(drawable)) {
-                splashImageName = splashImageValue.mid(drawable.size());
-            }
+            if (!splashImageValue.startsWith(drawable))
+                continue;
+            splashImageName = splashImageValue.mid(drawable.size());
             if (attrName == QLatin1String("android.app.splash_screen_drawable")) {
-                m_splashButtons->setImageFileName(splashImageName);
+                m_splashButtons->checkSplashscreenImage(splashImageName);
+                m_currentsplashImageName[0] = splashImageName;
                 splashParseGuard |= SplashImageParseGuard::splash;
             } else if (attrName == QLatin1String("android.app.splash_screen_drawable_portrait")) {
-                    m_splashButtons->setPortraitImageFileName(splashImageName);
-                    splashParseGuard |= SplashImageParseGuard::portraitSplash;
+                m_splashButtons->checkSplashscreenImage(splashImageName);
+                m_currentsplashImageName[1] = splashImageName;
+                splashParseGuard |= SplashImageParseGuard::portraitSplash;
             } else if (attrName == QLatin1String("android.app.splash_screen_drawable_landscape")) {
-                m_splashButtons->setLandscapeImageFileName(splashImageName);
+                m_splashButtons->checkSplashscreenImage(splashImageName);
+                m_currentsplashImageName[2] = splashImageName;
                 splashParseGuard |= SplashImageParseGuard::landscapeSplash;
             }
             if (splashParseGuard & SplashImageParseGuard::splashDone)
@@ -1201,19 +1163,36 @@ static void writeMetadataElement(const char *name,
 
 void AndroidManifestEditorWidget::parseSplashScreen(QXmlStreamWriter &writer)
 {
-    if (m_splashButtons->hasImages())
+    QString splashImageName[3];
+    bool splashSticky;
+
+    if (m_splashButtons->isSplashscreenEnabled()) {
+        if (m_splashButtons->hasImages())
+            splashImageName[0] = m_splashButtons->imageName();
+        if (m_splashButtons->hasPortraitImages())
+            splashImageName[1] = m_splashButtons->portraitImageName();
+        if (m_splashButtons->hasLandscapeImages())
+            splashImageName[2] = m_splashButtons->landscapeImageName();
+        splashSticky = m_splashButtons->isSticky();
+    } else {
+        for (int i = 0; i < 3; i++)
+            splashImageName[i] = m_currentsplashImageName[i];
+        splashSticky = m_currentsplashSticky;
+    }
+
+    if (!splashImageName[0].isEmpty())
         writeMetadataElement("android.app.splash_screen_drawable",
-                             "android:resource", QLatin1String("@drawable/") + m_splashButtons->imageFileName(),
+                             "android:resource", QLatin1String("@drawable/%1").arg(splashImageName[0]),
                              writer);
-    if (m_splashButtons->hasPortraitImages())
+    if (!splashImageName[1].isEmpty())
         writeMetadataElement("android.app.splash_screen_drawable_portrait",
-                             "android:resource", QLatin1String("@drawable/") + m_splashButtons->portraitImageFileName(),
+                             "android:resource", QLatin1String("@drawable/%1").arg(splashImageName[1]),
                              writer);
-    if (m_splashButtons->hasLandscapeImages())
+    if (!splashImageName[2].isEmpty())
         writeMetadataElement("android.app.splash_screen_drawable_landscape",
-                             "android:resource", QLatin1String("@drawable/") + m_splashButtons->landscapeImageFileName(),
+                             "android:resource", QLatin1String("@drawable/%1").arg(splashImageName[2]),
                              writer);
-    if (m_splashButtons->isSticky())
+    if (splashSticky)
         writeMetadataElement("android.app.splash_screen_sticky",
                              "android:value", "true",
                              writer);
@@ -1353,7 +1332,14 @@ void AndroidManifestEditorWidget::parseActivity(QXmlStreamReader &reader, QXmlSt
     QXmlStreamAttributes attributes = reader.attributes();
     QStringList keys = { QLatin1String("android:label"), QLatin1String("android:screenOrientation") };
     QStringList values = { m_activityNameLineEdit->text(), m_screenOrientation->currentText() };
-    QXmlStreamAttributes result = modifyXmlStreamAttributes(attributes, keys, values);
+    QStringList removes;
+    if (m_splashButtons->hasImages() || m_splashButtons->hasPortraitImages() || m_splashButtons->hasLandscapeImages()) {
+        keys << QLatin1String("android:theme");
+        values << QLatin1String("@style/splashScreenTheme");
+    } else {
+        removes << QLatin1String("android:theme");
+    }
+    QXmlStreamAttributes result = modifyXmlStreamAttributes(attributes, keys, values, removes);
     writer.writeAttributes(result);
 
     reader.readNext();
@@ -1363,13 +1349,6 @@ void AndroidManifestEditorWidget::parseActivity(QXmlStreamReader &reader, QXmlSt
     while (!reader.atEnd()) {
         if (reader.isEndElement()) {
             parseSplashScreen(writer);
-            if (!found) {
-                writer.writeEmptyElement(QLatin1String("meta-data"));
-                writer.writeAttribute(QLatin1String("android:name"),
-                                      QLatin1String("android.app.lib_name"));
-                writer.writeAttribute(QLatin1String("android:value"),
-                                      m_targetLineEdit->currentText());
-            }
             writer.writeCurrentToken(reader);
             return;
         } else if (reader.isStartElement()) {
@@ -1400,12 +1379,7 @@ bool AndroidManifestEditorWidget::parseMetaData(QXmlStreamReader &reader, QXmlSt
     QStringList keys;
     QStringList values;
 
-    if (attributes.value(QLatin1String("android:name")) == QLatin1String("android.app.lib_name")) {
-        keys = QStringList("android:value");
-        values = QStringList(m_targetLineEdit->currentText());
-        result = modifyXmlStreamAttributes(attributes, keys, values);
-        found = true;
-    } else if (attributes.value(QLatin1String("android:name"))
+    if (attributes.value(QLatin1String("android:name"))
                == QLatin1String("android.app.extract_android_style")) {
         keys = QStringList("android:value");
         values = QStringList(m_styleExtractMethod->currentText());
