@@ -68,16 +68,13 @@ bool AndroidPotentialKit::isEnabled() const
 {
     QList<ProjectExplorer::Kit *> kits = ProjectExplorer::KitManager::kits();
     foreach (ProjectExplorer::Kit *kit, kits) {
-        Utils::Id deviceId = ProjectExplorer::DeviceKitAspect::deviceId(kit);
-        if (kit->isAutoDetected()
-                && deviceId == Utils::Id(Constants::ANDROID_DEVICE_ID)
-                && !kit->isSdkProvided()) {
+        if (kit->isAutoDetected() && !kit->isSdkProvided()) {
             return false;
         }
     }
 
     return QtSupport::QtVersionManager::version([](const QtSupport::BaseQtVersion *v) {
-        return v->isValid() && v->type() == QString::fromLatin1(Constants::ANDROIDQT);
+        return v->isValid() && v->type() == QString::fromLatin1(Constants::ANDROID_QT_TYPE);
     });
 }
 
@@ -120,10 +117,7 @@ void AndroidPotentialKitWidget::recheck()
 {
     QList<ProjectExplorer::Kit *> kits = ProjectExplorer::KitManager::kits();
     foreach (ProjectExplorer::Kit *kit, kits) {
-        Utils::Id deviceId = ProjectExplorer::DeviceKitAspect::deviceId(kit);
-        if (kit->isAutoDetected()
-                && deviceId == Utils::Id(Constants::ANDROID_DEVICE_ID)
-                && !kit->isSdkProvided()) {
+        if (kit->isAutoDetected() && !kit->isSdkProvided()) {
             setVisible(false);
             return;
         }
