@@ -35,11 +35,14 @@
 #include <qtsupport/qtsupportconstants.h>
 #include <qtsupport/qtversionmanager.h>
 
+#include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectnodes.h>
 #include <projectexplorer/target.h>
 #include <projectexplorer/kit.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projecttree.h>
+#include <projectexplorer/buildsystem.h>
 
 #include <proparser/profileevaluator.h>
 
@@ -143,12 +146,10 @@ void AndroidQtVersion::addToEnvironment(const Kit *k, Utils::Environment &env) c
             config.bestNdkPlatformMatch(qMax(minimumNDK(), AndroidManager::minimumSDK(k)), this));
 }
 
-Utils::Environment AndroidQtVersion::qmakeRunEnvironment() const
+void AndroidQtVersion::setupQmakeRunEnvironment(Utils::Environment &env) const
 {
-    Utils::Environment env = Utils::Environment::systemEnvironment();
     env.set(QLatin1String("ANDROID_NDK_ROOT"),
             AndroidConfigurations::currentConfig().ndkLocation(this).toUserOutput());
-    return env;
 }
 
 QString AndroidQtVersion::description() const
@@ -169,11 +170,34 @@ int AndroidQtVersion::minimumNDK() const
     return m_minNdk;
 }
 
+Utils::FilePath AndroidQtVersion::androidDeploymentSettings(const Target *target)
+{
+    // Try to fetch the file name from node data as provided by qmake and Qbs
+    QString buildKey = target->activeBuildKey();
+    const ProjectNode *node = target->project()->findNodeForBuildKey(buildKey);
+    if (node) {
+        const QString nameFromData = node->data(Constants::AndroidDeploySettingsFile).toString();
+        if (!nameFromData.isEmpty())
+            return Utils::FilePath::fromUserInput(nameFromData);
+    }
+
+    // If unavailable, construct the name by ourselves (CMake)
+    const BuildSystem *bs = target->buildSystem();
+    if (!bs)
+        return {};
+    const QString displayName = bs->buildTarget(buildKey).displayName;
+    return AndroidManager::buildDirectory(target).pathAppended(
+                AndroidManager::isQt5CmakeProject(target)
+                ? QLatin1String("android_deployment_settings.json")
+                : QString::fromLatin1("android-%1-deployment-settings.json")
+                  .arg(displayName));
+}
+
 void AndroidQtVersion::parseMkSpec(ProFileEvaluator *evaluator) const
 {
     m_androidAbis = evaluator->values("ALL_ANDROID_ABIS");
     if (m_androidAbis.isEmpty())
-        m_androidAbis = QStringList{evaluator->value("ANDROID_TARGET_ARCH")};
+        m_androidAbis = QStringList{evaluator->value(Constants::ANDROID_TARGET_ARCH)};
     const QString androidPlatform = evaluator->value("ANDROID_PLATFORM");
     if (!androidPlatform.isEmpty()) {
         const QRegularExpression regex("android-(\\d+)");
@@ -208,7 +232,7 @@ QSet<Utils::Id> AndroidQtVersion::targetDeviceTypes() const
 AndroidQtVersionFactory::AndroidQtVersionFactory()
 {
     setQtVersionCreator([] { return new AndroidQtVersion; });
-    setSupportedType(Constants::ANDROIDQT);
+    setSupportedType(Constants::ANDROID_QT_TYPE);
     setPriority(90);
 
     setRestrictionChecker([](const SetupData &setup) {
