@@ -47,7 +47,7 @@ public:
     ProjectExplorer::Abis detectQtAbis() const override;
 
     void addToEnvironment(const ProjectExplorer::Kit *k, Utils::Environment &env) const override;
-    Utils::Environment qmakeRunEnvironment() const override;
+    void setupQmakeRunEnvironment(Utils::Environment &env) const override;
 
     QSet<Utils::Id> availableFeatures() const override;
     QSet<Utils::Id> targetDeviceTypes() const override;
@@ -56,6 +56,8 @@ public:
     const QStringList &androidAbis() const;
     int minimumNDK() const;
 
+    static Utils::FilePath androidDeploymentSettings(const ProjectExplorer::Target *target);
+
 protected:
     void parseMkSpec(ProFileEvaluator *) const override;
 private:
