@@ -34,12 +34,13 @@ namespace Internal {
 
 class AndroidRunner;
 
-class AndroidRunSupport : public AndroidRunner
+class AndroidRunSupport final : public AndroidRunner
 {
     Q_OBJECT
 
 public:
-    explicit AndroidRunSupport(ProjectExplorer::RunControl *runControl);
+    explicit AndroidRunSupport(ProjectExplorer::RunControl *runControl,
+                               const QString &intentName = QString());
     ~AndroidRunSupport() override;
 
     void start() override;
