@@ -34,7 +34,6 @@
 #include <utils/qtcprocess.h>
 #include <utils/runextensions.h>
 #include <utils/stringutils.h>
-#include <utils/synchronousprocess.h>
 
 #include <QFutureWatcher>
 #include <QLoggingCategory>
@@ -42,6 +41,11 @@
 #include <QRegularExpression>
 #include <QSettings>
 
+#ifdef WITH_TESTS
+#   include <QTest>
+#   include "androidplugin.h"
+#endif // WITH_TESTS
+
 namespace {
 static Q_LOGGING_CATEGORY(sdkManagerLog, "qtc.android.sdkManager", QtWarningMsg)
 }
@@ -57,33 +61,14 @@ const char commonArgsKey[] = "Common Arguments:";
 const int sdkManagerCmdTimeoutS = 60;
 const int sdkManagerOperationTimeoutS = 600;
 
-const QRegularExpression assertionReg("(\\(\\s*y\\s*[\\/\\\\]\\s*n\\s*\\)\\s*)(?<mark>[\\:\\?])",
-                                      QRegularExpression::CaseInsensitiveOption |
-                                      QRegularExpression::MultilineOption);
+Q_GLOBAL_STATIC_WITH_ARGS(QRegularExpression, assertionReg,
+                          ("(\\(\\s*y\\s*[\\/\\\\]\\s*n\\s*\\)\\s*)(?<mark>[\\:\\?])",
+                           QRegularExpression::CaseInsensitiveOption
+                           | QRegularExpression::MultilineOption))
 
 using namespace Utils;
 using SdkCmdFutureInterface = QFutureInterface<AndroidSdkManager::OperationOutput>;
 
-int platformNameToApiLevel(const QString &platformName)
-{
-    int apiLevel = -1;
-    QRegularExpression re("(android-)(?<apiLevel>[0-9A-Z]{1,})",
-                          QRegularExpression::CaseInsensitiveOption);
-    QRegularExpressionMatch match = re.match(platformName);
-    if (match.hasMatch()) {
-        QString apiLevelStr = match.captured("apiLevel");
-        bool isUInt;
-        apiLevel = apiLevelStr.toUInt(&isUInt);
-        if (!isUInt) {
-            if (apiLevelStr == 'Q')
-                apiLevel = 29;
-            else if (apiLevelStr == 'R')
-                apiLevel = 30;
-        }
-    }
-    return apiLevel;
-}
-
 /*!
     Parses the \a line for a [spaces]key[spaces]value[spaces] pattern and returns
     \c true if \a key is found, false otherwise. Result is copied into \a value.
@@ -114,7 +99,7 @@ int parseProgress(const QString &out, bool &foundAssertion)
                 progress = -1;
         }
         if (!foundAssertion)
-            foundAssertion = assertionReg.match(line).hasMatch();
+            foundAssertion = assertionReg->match(line).hasMatch();
     }
     return progress;
 }
@@ -146,14 +131,16 @@ static bool sdkManagerCommand(const AndroidConfig &config, const QStringList &ar
     qCDebug(sdkManagerLog) << "Running SDK Manager command (sync):"
                            << CommandLine(config.sdkManagerToolPath(), newArgs)
                                   .toUserOutput();
-    SynchronousProcess proc;
-    proc.setProcessEnvironment(AndroidConfigurations::toolsEnvironment(config));
+    QtcProcess proc;
+    proc.setEnvironment(AndroidConfigurations::toolsEnvironment(config));
     proc.setTimeoutS(timeout);
     proc.setTimeOutMessageBoxEnabled(true);
-    SynchronousProcessResponse response = proc.run({config.sdkManagerToolPath(), newArgs});
+    proc.setCommand({config.sdkManagerToolPath(), newArgs});
+    proc.setProcessUserEventWhileRunning();
+    proc.runBlocking();
     if (output)
-        *output = response.allOutput();
-    return response.result == SynchronousProcessResponse::Finished;
+        *output = proc.allOutput();
+    return proc.result() == QtcProcess::FinishedWithSuccess;
 }
 
 /*!
@@ -172,36 +159,35 @@ static void sdkManagerCommand(const AndroidConfig &config, const QStringList &ar
     qCDebug(sdkManagerLog) << "Running SDK Manager command (async):"
                            << CommandLine(config.sdkManagerToolPath(), newArgs).toUserOutput();
     int offset = fi.progressValue();
-    SynchronousProcess proc;
-    proc.setProcessEnvironment(AndroidConfigurations::toolsEnvironment(config));
+    QtcProcess proc;
+    proc.setEnvironment(AndroidConfigurations::toolsEnvironment(config));
     bool assertionFound = false;
-    proc.setStdErrBufferedSignalsEnabled(true);
-    proc.setStdOutBufferedSignalsEnabled(true);
     proc.setTimeoutS(timeout);
-    QObject::connect(&proc, &SynchronousProcess::stdOutBuffered,
-                     [offset, progressQuota, &proc, &assertionFound, &fi](const QString &out) {
+    proc.setStdOutCallback([offset, progressQuota, &proc, &assertionFound, &fi](const QString &out) {
         int progressPercent = parseProgress(out, assertionFound);
         if (assertionFound)
-            proc.terminate();
+            proc.stopProcess();
         if (progressPercent != -1)
             fi.setProgressValue(offset + qRound((progressPercent / 100.0) * progressQuota));
     });
-    QObject::connect(&proc, &SynchronousProcess::stdErrBuffered, [&output](const QString &err) {
+    proc.setStdErrCallback([&output](const QString &err) {
         output.stdError = err;
     });
     if (interruptible) {
         QObject::connect(&sdkManager, &AndroidSdkManager::cancelActiveOperations,
-                         &proc, &SynchronousProcess::terminate);
+                         &proc, &QtcProcess::stopProcess);
     }
-    SynchronousProcessResponse response = proc.run({config.sdkManagerToolPath(), newArgs});
+    proc.setCommand({config.sdkManagerToolPath(), newArgs});
+    proc.setProcessUserEventWhileRunning();
+    proc.runBlocking();
     if (assertionFound) {
         output.success = false;
-        output.stdOutput = response.stdOut();
+        output.stdOutput = proc.stdOut();
         output.stdError = QCoreApplication::translate("Android::Internal::AndroidSdkManager",
                                                       "The operation requires user interaction. "
                                                       "Use the \"sdkmanager\" command-line tool.");
     } else {
-        output.success = response.result == SynchronousProcessResponse::Finished;
+        output.success = proc.result() == QtcProcess::FinishedWithSuccess;
     }
 }
 
@@ -315,20 +301,21 @@ private:
     QHash<AndroidSdkPackage *, int> m_systemImages;
 };
 
-const std::map<SdkManagerOutputParser::MarkerTag, const char *> markerTags {
-    {SdkManagerOutputParser::MarkerTag::InstalledPackagesMarker,    "Installed packages:"},
-    {SdkManagerOutputParser::MarkerTag::AvailablePackagesMarkers,   "Available Packages:"},
-    {SdkManagerOutputParser::MarkerTag::AvailableUpdatesMarker,     "Available Updates:"},
-    {SdkManagerOutputParser::MarkerTag::PlatformMarker,             "platforms"},
-    {SdkManagerOutputParser::MarkerTag::SystemImageMarker,          "system-images"},
-    {SdkManagerOutputParser::MarkerTag::BuildToolsMarker,           "build-tools"},
-    {SdkManagerOutputParser::MarkerTag::SdkToolsMarker,             "tools"},
-    {SdkManagerOutputParser::MarkerTag::CmdlineSdkToolsMarker,      "cmdline-tools"},
-    {SdkManagerOutputParser::MarkerTag::PlatformToolsMarker,        "platform-tools"},
-    {SdkManagerOutputParser::MarkerTag::EmulatorToolsMarker,        "emulator"},
-    {SdkManagerOutputParser::MarkerTag::NdkMarker,                  "ndk"},
-    {SdkManagerOutputParser::MarkerTag::ExtrasMarker,               "extras"}
-};
+using MarkerTagsType = std::map<SdkManagerOutputParser::MarkerTag, const char *>;
+Q_GLOBAL_STATIC_WITH_ARGS(MarkerTagsType, markerTags, ({
+        {SdkManagerOutputParser::MarkerTag::InstalledPackagesMarker,    "Installed packages:"},
+        {SdkManagerOutputParser::MarkerTag::AvailablePackagesMarkers,   "Available Packages:"},
+        {SdkManagerOutputParser::MarkerTag::AvailableUpdatesMarker,     "Available Updates:"},
+        {SdkManagerOutputParser::MarkerTag::PlatformMarker,             "platforms"},
+        {SdkManagerOutputParser::MarkerTag::SystemImageMarker,          "system-images"},
+        {SdkManagerOutputParser::MarkerTag::BuildToolsMarker,           "build-tools"},
+        {SdkManagerOutputParser::MarkerTag::SdkToolsMarker,             "tools"},
+        {SdkManagerOutputParser::MarkerTag::CmdlineSdkToolsMarker,      "cmdline-tools"},
+        {SdkManagerOutputParser::MarkerTag::PlatformToolsMarker,        "platform-tools"},
+        {SdkManagerOutputParser::MarkerTag::EmulatorToolsMarker,        "emulator"},
+        {SdkManagerOutputParser::MarkerTag::NdkMarker,                  "ndk"},
+        {SdkManagerOutputParser::MarkerTag::ExtrasMarker,               "extras"}
+}));
 
 AndroidSdkManager::AndroidSdkManager(const AndroidConfig &config):
     m_d(new AndroidSdkManagerPrivate(*this, config))
@@ -491,7 +478,8 @@ void SdkManagerOutputParser::parsePackageListing(const QString &output)
     };
 
     QRegularExpression delimiters("[\\n\\r]");
-    for (const QString &outputLine : output.split(delimiters)) {
+    const auto lines = output.split(delimiters);
+    for (const QString &outputLine : lines) {
 
         // NOTE: we don't want to parse Dependencies part as it does not add value
         if (outputLine.startsWith("        "))
@@ -566,7 +554,7 @@ void SdkManagerOutputParser::compilePackageAssociations()
     deleteAlreadyInstalled(images);
 
     // Associate the system images with sdk platforms.
-    for (AndroidSdkPackage *image : images) {
+    for (AndroidSdkPackage *image : qAsConst(images)) {
         int imageApi = m_systemImages[image];
         auto itr = std::find_if(m_packages.begin(), m_packages.end(),
                                 [imageApi](const AndroidSdkPackage *p) {
@@ -641,7 +629,7 @@ void SdkManagerOutputParser::parsePackageData(MarkerTag packageMarker, const QSt
         break;
 
     default:
-        qCDebug(sdkManagerLog) << "Unhandled package: " << markerTags.at(packageMarker);
+        qCDebug(sdkManagerLog) << "Unhandled package: " << markerTags->at(packageMarker);
         break;
     }
 
@@ -655,7 +643,7 @@ void SdkManagerOutputParser::parsePackageData(MarkerTag packageMarker, const QSt
             package->setState(AndroidSdkPackage::Available);
             break;
         default:
-            qCDebug(sdkManagerLog) << "Invalid section marker: " << markerTags.at(m_currentSection);
+            qCDebug(sdkManagerLog) << "Invalid section marker: " << markerTags->at(m_currentSection);
             break;
         }
     }
@@ -684,7 +672,7 @@ bool SdkManagerOutputParser::parseAbstractData(SdkManagerOutputParser::GenericPa
         for (const auto &key: qAsConst(keys)) {
             if (valueForKey(key, line, &value)) {
                 if (key == installLocationKey)
-                    output.installedLocation = Utils::FilePath::fromString(value);
+                    output.installedLocation = Utils::FilePath::fromUserInput(value);
                 else if (key == revisionKey)
                     output.revision = QVersionNumber::fromString(value);
                 else if (key == descriptionKey)
@@ -704,7 +692,7 @@ AndroidSdkPackage *SdkManagerOutputParser::parsePlatform(const QStringList &data
     SdkPlatform *platform = nullptr;
     GenericPackageData packageData;
     if (parseAbstractData(packageData, data, 2, "Platform")) {
-        int apiLevel = platformNameToApiLevel(packageData.headerParts.at(1));
+        const int apiLevel = AndroidConfig::platformNameToApiLevel(packageData.headerParts.at(1));
         if (apiLevel == -1) {
             qCDebug(sdkManagerLog) << "Platform: Cannot parse api level:"<< data;
             return nullptr;
@@ -724,7 +712,7 @@ QPair<SystemImage *, int> SdkManagerOutputParser::parseSystemImage(const QString
     QPair <SystemImage *, int> result(nullptr, -1);
     GenericPackageData packageData;
     if (parseAbstractData(packageData, data, 4, "System-image")) {
-        int apiLevel = platformNameToApiLevel(packageData.headerParts.at(1));
+        const int apiLevel = AndroidConfig::platformNameToApiLevel(packageData.headerParts.at(1));
         if (apiLevel == -1) {
             qCDebug(sdkManagerLog) << "System-image: Cannot parse api level:"<< data;
             return result;
@@ -859,7 +847,7 @@ SdkManagerOutputParser::MarkerTag SdkManagerOutputParser::parseMarkers(const QSt
     if (line.isEmpty())
         return EmptyMarker;
 
-    for (auto pair: markerTags) {
+    for (auto pair : *markerTags) {
         if (line.startsWith(QLatin1String(pair.second)))
             return pair.first;
     }
@@ -1014,10 +1002,12 @@ void AndroidSdkManagerPrivate::checkPendingLicense(SdkCmdFutureInterface &fi)
     AndroidSdkManager::OperationOutput result;
     result.type = AndroidSdkManager::LicenseCheck;
     const QStringList args = {"--licenses", sdkRootArg(m_config)};
-    if (!fi.isCanceled())
-        sdkManagerCommand(m_config, args, m_sdkManager, fi, result, 100.0);
-    else
+    if (!fi.isCanceled()) {
+        const int timeOutS = 4; // Short timeout as workaround for QTCREATORBUG-25667
+        sdkManagerCommand(m_config, args, m_sdkManager, fi, result, 100.0, true, timeOutS);
+    } else {
         qCDebug(sdkManagerLog) << "Update: Operation cancelled before start";
+    }
 
     fi.reportResult(result);
     fi.setProgressValue(100);
@@ -1029,12 +1019,11 @@ void AndroidSdkManagerPrivate::getPendingLicense(SdkCmdFutureInterface &fi)
     fi.setProgressValue(0);
     AndroidSdkManager::OperationOutput result;
     result.type = AndroidSdkManager::LicenseWorkflow;
-    QtcProcess licenseCommand;
-    licenseCommand.setProcessEnvironment(AndroidConfigurations::toolsEnvironment(m_config));
+    QtcProcess licenseCommand(ProcessMode::Writer);
+    licenseCommand.setEnvironment(AndroidConfigurations::toolsEnvironment(m_config));
     bool reviewingLicenses = false;
     licenseCommand.setCommand(CommandLine(m_config.sdkManagerToolPath(), {"--licenses", sdkRootArg(m_config)}));
-    if (Utils::HostOsInfo::isWindowsHost())
-        licenseCommand.setUseCtrlCStub(true);
+    licenseCommand.setUseCtrlCStub(true);
     licenseCommand.start();
     QTextCodec *codec = QTextCodec::codecForLocale();
     int inputCounter = 0, steps = -1;
@@ -1076,7 +1065,7 @@ void AndroidSdkManagerPrivate::getPendingLicense(SdkCmdFutureInterface &fi)
     }
 
     m_licenseTextCache.clear();
-    result.success = licenseCommand.exitStatus() == QtcProcess::NormalExit;
+    result.success = licenseCommand.exitStatus() == QProcess::NormalExit;
     if (!result.success) {
         result.stdError = QCoreApplication::translate("Android::Internal::AndroidSdkManager",
                                                       "License command failed.\n\n");
@@ -1108,7 +1097,7 @@ bool AndroidSdkManagerPrivate::onLicenseStdOut(const QString &output, bool notif
                                                SdkCmdFutureInterface &fi)
 {
     m_licenseTextCache.append(output);
-    QRegularExpressionMatch assertionMatch = assertionReg.match(m_licenseTextCache);
+    QRegularExpressionMatch assertionMatch = assertionReg->match(m_licenseTextCache);
     if (assertionMatch.hasMatch()) {
         if (notify) {
             result.stdOutput = m_licenseTextCache;
@@ -1136,7 +1125,8 @@ void AndroidSdkManagerPrivate::parseCommonArguments(QFutureInterface<QString> &f
     QString output;
     sdkManagerCommand(m_config, QStringList("--help"), &output);
     bool foundTag = false;
-    for (const QString& line : output.split('\n')) {
+    const auto lines = output.split('\n');
+    for (const QString& line : lines) {
         if (fi.isCanceled())
             break;
         if (foundTag)
@@ -1151,10 +1141,59 @@ void AndroidSdkManagerPrivate::parseCommonArguments(QFutureInterface<QString> &f
 
 void AndroidSdkManagerPrivate::clearPackages()
 {
-    for (AndroidSdkPackage *p : m_allPackages)
+    for (AndroidSdkPackage *p : qAsConst(m_allPackages))
         delete p;
     m_allPackages.clear();
 }
 
+#ifdef WITH_TESTS
+void AndroidPlugin::testAndroidSdkManagerProgressParser_data()
+{
+    QTest::addColumn<QString>("output");
+    QTest::addColumn<int>("progress");
+    QTest::addColumn<bool>("foundAssertion");
+
+    // Output of "sdkmanager --licenses", Android SDK Tools version 4.0
+    QTest::newRow("Loading local repository")
+            << "Loading local repository...                                                     \r"
+            << -1
+            << false;
+
+    QTest::newRow("Fetch progress (single line)")
+            << "[=============                          ] 34% Fetch remote repository...        \r"
+            << 34
+            << false;
+
+    QTest::newRow("Fetch progress (multi line)")
+            << "[=============================          ] 73% Fetch remote repository...        \r"
+               "[=============================          ] 75% Fetch remote repository...        \r"
+            << 75
+            << false;
+
+    QTest::newRow("Some SDK package licenses not accepted")
+            << "7 of 7 SDK package licenses not accepted.\n"
+            << -1
+            << false;
+
+    QTest::newRow("Unaccepted licenses assertion")
+            << "\nReview licenses that have not been accepted (y/N)? "
+            << -1
+            << true;
+}
+
+void AndroidPlugin::testAndroidSdkManagerProgressParser()
+{
+    QFETCH(QString, output);
+    QFETCH(int, progress);
+    QFETCH(bool, foundAssertion);
+
+    bool actualFoundAssertion = false;
+    const int actualProgress = parseProgress(output, actualFoundAssertion);
+
+    QCOMPARE(progress, actualProgress);
+    QCOMPARE(foundAssertion, actualFoundAssertion);
+}
+#endif // WITH_TESTS
+
 } // namespace Internal
 } // namespace Android
