@@ -41,7 +41,6 @@
 #include <QLineEdit>
 #include <QLoggingCategory>
 #include <QMessageBox>
-#include <QProcess>
 #include <QSortFilterProxyModel>
 
 namespace {
@@ -59,10 +58,12 @@ public:
     PackageFilterModel(AndroidSdkModel* sdkModel);
 
     void setAcceptedPackageState(AndroidSdkPackage::PackageState state);
+    void setAcceptedSearchPackage(const QString &text);
     bool filterAcceptsRow(int source_row, const QModelIndex &sourceParent) const override;
 
 private:
     AndroidSdkPackage::PackageState m_packageState =  AndroidSdkPackage::AnyValidState;
+    QString m_searchText;
 };
 
 AndroidSdkManagerWidget::AndroidSdkManagerWidget(AndroidConfig &config,
@@ -96,12 +97,10 @@ AndroidSdkManagerWidget::AndroidSdkManagerWidget(AndroidConfig &config,
 
     auto proxyModel = new PackageFilterModel(m_sdkModel);
     m_ui->packagesView->setModel(proxyModel);
+    m_ui->packagesView->header()->setSectionResizeMode(QHeaderView::ResizeToContents);
     m_ui->packagesView->header()->setSectionResizeMode(AndroidSdkModel::packageNameColumn,
-                                                       QHeaderView::ResizeToContents);
-    m_ui->packagesView->header()->setSectionResizeMode(AndroidSdkModel::apiLevelColumn,
-                                                       QHeaderView::ResizeToContents);
-    m_ui->packagesView->header()->setSectionResizeMode(AndroidSdkModel::packageRevisionColumn,
-                                                       QHeaderView::ResizeToContents);
+                                                       QHeaderView::Stretch);
+    m_ui->packagesView->header()->setStretchLastSection(false);
     connect(m_ui->expandCheck, &QCheckBox::stateChanged, [this](int state) {
        if (state == Qt::Checked)
            m_ui->packagesView->expandAll();
@@ -129,6 +128,14 @@ AndroidSdkManagerWidget::AndroidSdkManagerWidget(AndroidConfig &config,
         }
     });
 
+    m_ui->searchField->setPlaceholderText("Filter");
+    connect(m_ui->searchField, &QLineEdit::textChanged, [this, proxyModel](const QString &text) {
+        proxyModel->setAcceptedSearchPackage(text);
+        m_sdkModel->resetSelection();
+        // It is more convenient to expand the view with the results
+        m_ui->expandCheck->setChecked(!text.isEmpty());
+    });
+
     connect(m_ui->applySelectionButton, &QPushButton::clicked,
             this, &AndroidSdkManagerWidget::onApplyButton);
     connect(m_ui->cancelButton, &QPushButton::clicked, this,
@@ -160,7 +167,7 @@ void AndroidSdkManagerWidget::installEssentials()
     if (!m_sdkModel->missingEssentials().isEmpty()) {
         QMessageBox::warning(this,
                              tr("Android SDK Changes"),
-                             tr("%1 couldn't find the following essential packages: \"%2\".\n"
+                             tr("%1 cannot find the following essential packages: \"%2\".\n"
                                 "Install them manually after the current operation is done.\n")
                                  .arg(Core::Constants::IDE_DISPLAY_NAME)
                                  .arg(m_sdkModel->missingEssentials().join("\", \"")));
@@ -269,7 +276,7 @@ void AndroidSdkManagerWidget::onLicenseCheckResult(const AndroidSdkManager::Oper
     } else {
         // Assertion was found. Provide user workflow to accept licenses.
         QString warningMessage = tr("Review Android SDK package licenses that have not been "
-                                    "accepted?\nPlease note that the installation and use of "
+                                    "accepted?\nNote that the installation and use of "
                                     "Android SDK packages may fail if respective licenses are not "
                                     "accepted.");
         int userSelection = QMessageBox::question(this, tr("Android SDK Licenses"), warningMessage,
@@ -469,6 +476,12 @@ void PackageFilterModel::setAcceptedPackageState(AndroidSdkPackage::PackageState
     invalidateFilter();
 }
 
+void PackageFilterModel::setAcceptedSearchPackage(const QString &name)
+{
+    m_searchText = name;
+    invalidateFilter();
+}
+
 bool PackageFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
 {
     QModelIndex srcIndex = sourceModel()->index(sourceRow, 0, sourceParent);
@@ -479,19 +492,24 @@ bool PackageFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sour
       return (AndroidSdkPackage::PackageState)i.data(AndroidSdkModel::PackageStateRole).toInt();
     };
 
+    auto packageFound = [this](const QModelIndex& i) {
+        return i.data(AndroidSdkModel::packageNameColumn).toString()
+                .contains(m_searchText, Qt::CaseInsensitive);
+    };
+
     bool showTopLevel = false;
     if (!sourceParent.isValid()) {
         // Top Level items
         for (int row = 0; row < sourceModel()->rowCount(srcIndex); ++row) {
             QModelIndex childIndex = sourceModel()->index(row, 0, srcIndex);
-            if (m_packageState & packageState(childIndex)) {
+            if ((m_packageState & packageState(childIndex) && packageFound(childIndex))) {
                 showTopLevel = true;
                 break;
             }
         }
     }
 
-    return showTopLevel || (packageState(srcIndex) & m_packageState);
+    return showTopLevel || ((packageState(srcIndex) & m_packageState) && packageFound(srcIndex));
 }
 
 OptionsDialog::OptionsDialog(AndroidSdkManager *sdkManager, const QStringList &args,
