@@ -145,8 +145,8 @@ public:
     ChooseDirectoryPage(CreateAndroidManifestWizard *wizard);
 
 private:
-    void initializePage();
-    bool isComplete() const;
+    void initializePage() final;
+    bool isComplete() const final;
     void checkPackageSourceDir();
 
     CreateAndroidManifestWizard *m_wizard;
@@ -177,7 +177,7 @@ ChooseDirectoryPage::ChooseDirectoryPage(CreateAndroidManifestWizard *wizard)
     m_sourceDirectoryWarning->setWordWrap(true);
     m_layout->addRow(m_sourceDirectoryWarning);
 
-    connect(m_androidPackageSourceDir, &PathChooser::pathChanged,
+    connect(m_androidPackageSourceDir, &PathChooser::filePathChanged,
             m_wizard, &CreateAndroidManifestWizard::setDirectory);
 
     if (wizard->copyGradle()) {
@@ -217,28 +217,31 @@ void ChooseDirectoryPage::initializePage()
     const Target *target = m_wizard->buildSystem()->target();
     const QString buildKey = m_wizard->buildKey();
     const BuildTargetInfo bti = target->buildTarget(buildKey);
-    const QString projectDir = bti.projectFilePath.toFileInfo().absolutePath();
 
-    QString androidPackageDir;
+    FilePath androidPackageDir;
     if (const ProjectNode *node = target->project()->findNodeForBuildKey(buildKey))
-        androidPackageDir = node->data(Android::Constants::AndroidPackageSourceDir).toString();
+        androidPackageDir = FilePath::fromVariant(node->data(Android::Constants::AndroidPackageSourceDir));
 
     if (androidPackageDir.isEmpty()) {
         m_label->setText(tr("Select the Android package source directory.\n\n"
                           "The files in the Android package source directory are copied to the build directory's "
                           "Android directory and the default files are overwritten."));
 
-        m_androidPackageSourceDir->setPath(projectDir + "/android");
+        const FilePath projectPath = bti.projectFilePath.isFile()
+                ? bti.projectFilePath.parentDir() : bti.projectFilePath;
+
+        m_androidPackageSourceDir->setFilePath(projectPath / "android");
         connect(m_androidPackageSourceDir, &PathChooser::rawPathChanged,
                 this, &ChooseDirectoryPage::checkPackageSourceDir);
     } else {
-        m_label->setText(tr("The Android template files will be created in the ANDROID_PACKAGE_SOURCE_DIR set in the .pro file."));
-        m_androidPackageSourceDir->setPath(androidPackageDir);
+        m_label->setText(tr("The Android template files will be created in the %1 set in the .pro "
+                            "file.").arg(QLatin1String(Constants::ANDROID_PACKAGE_SOURCE_DIR)));
+        m_androidPackageSourceDir->setFilePath(androidPackageDir);
         m_androidPackageSourceDir->setReadOnly(true);
     }
 
 
-    m_wizard->setDirectory(m_androidPackageSourceDir->filePath().toString());
+    m_wizard->setDirectory(m_androidPackageSourceDir->filePath());
 }
 
 //
@@ -275,7 +278,7 @@ void CreateAndroidManifestWizard::setBuildKey(const QString &buildKey)
     m_buildKey = buildKey;
 }
 
-void CreateAndroidManifestWizard::setDirectory(const QString &directory)
+void CreateAndroidManifestWizard::setDirectory(const FilePath &directory)
 {
     m_directory = directory;
 }
@@ -301,25 +304,21 @@ void CreateAndroidManifestWizard::createAndroidTemplateFiles()
     if (!version)
         return;
     if (version->qtVersion() < QtSupport::QtVersionNumber(5, 4, 0)) {
-        const QString src = version->prefix().toString() + "/src/android/java/AndroidManifest.xml";
-        FileUtils::copyRecursively(FilePath::fromString(src),
-                                   FilePath::fromString(m_directory
-                                                        + QLatin1String("/AndroidManifest.xml")),
+        FileUtils::copyRecursively(version->prefix() / "src/android/java/AndroidManifest.xml",
+                                   m_directory / "AndroidManifest.xml",
                                    nullptr,
                                    copy);
     } else {
-        const QString src = version->prefix().toString() + "/src/android/templates";
-
-        FileUtils::copyRecursively(FilePath::fromString(src),
-                                   FilePath::fromString(m_directory),
+        FileUtils::copyRecursively(version->prefix() / "src/android/templates",
+                                   m_directory,
                                    nullptr,
                                    copy);
 
         if (m_copyGradle) {
-            FilePath gradlePath = version->prefix().pathAppended("src/3rdparty/gradle");
+            FilePath gradlePath = version->prefix() / "src/3rdparty/gradle";
             if (!gradlePath.exists())
-                gradlePath = AndroidConfigurations::currentConfig().sdkLocation().pathAppended("/tools/templates/gradle/wrapper");
-            FileUtils::copyRecursively(gradlePath, FilePath::fromString(m_directory), nullptr, copy);
+                gradlePath = AndroidConfigurations::currentConfig().sdkLocation() / "tools/templates/gradle/wrapper";
+            FileUtils::copyRecursively(gradlePath, m_directory, nullptr, copy);
         }
 
         AndroidManager::updateGradleProperties(target, m_buildKey);
@@ -337,7 +336,7 @@ void CreateAndroidManifestWizard::createAndroidTemplateFiles()
             const BuildTargetInfo bti = target->buildTarget(m_buildKey);
             const QString value = "$$PWD/"
                                   + bti.projectFilePath.toFileInfo().absoluteDir().relativeFilePath(
-                                      m_directory);
+                                      m_directory.toString());
             bool result = node->setData(Android::Constants::AndroidPackageSourceDir, value);
 
             if (!result) {
@@ -348,7 +347,7 @@ void CreateAndroidManifestWizard::createAndroidTemplateFiles()
             }
         }
     }
-    Core::EditorManager::openEditor(m_directory + QLatin1String("/AndroidManifest.xml"));
+    Core::EditorManager::openEditor(m_directory / "AndroidManifest.xml");
 }
 
 BuildSystem *CreateAndroidManifestWizard::buildSystem() const
