@@ -24,58 +24,62 @@
 ****************************************************************************/
 
 #include "autotestplugin.h"
+
 #include "autotestconstants.h"
 #include "autotesticons.h"
 #include "projectsettingswidget.h"
 #include "testcodeparser.h"
 #include "testframeworkmanager.h"
+#include "testnavigationwidget.h"
 #include "testprojectsettings.h"
+#include "testresultspane.h"
 #include "testrunner.h"
 #include "testsettings.h"
 #include "testsettingspage.h"
 #include "testtreeitem.h"
-#include "testtreeview.h"
 #include "testtreemodel.h"
-#include "testresultspane.h"
-#include "testnavigationwidget.h"
+#include "testtreeview.h"
 
-#include "qtest/qttestframework.h"
-#include "quick/quicktestframework.h"
-#include "gtest/gtestframework.h"
 #include "boost/boosttestframework.h"
 #include "catch/catchframework.h"
+#include "ctest/ctesttool.h"
+#include "gtest/gtestframework.h"
+#include "qtest/qttestframework.h"
+#include "quick/quicktestframework.h"
 
-#include <coreplugin/icore.h>
-#include <coreplugin/icontext.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/coreconstants.h>
+#include <coreplugin/icontext.h>
+#include <coreplugin/icore.h>
 #include <coreplugin/messagemanager.h>
 #include <cppeditor/cppeditorconstants.h>
 #include <extensionsystem/pluginmanager.h>
 #include <projectexplorer/buildmanager.h>
+#include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorericons.h>
 #include <projectexplorer/projectpanelfactory.h>
-#include <projectexplorer/runconfiguration.h>
+#include <projectexplorer/runcontrol.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
-#include <texteditor/texteditor.h>
 #include <texteditor/textdocument.h>
+#include <texteditor/texteditor.h>
 #include <utils/textutils.h>
 #include <utils/utilsicons.h>
 
 #include <QAction>
 #include <QList>
-#include <QMap>
-#include <QMessageBox>
 #include <QMainWindow>
+#include <QMap>
 #include <QMenu>
+#include <QMessageBox>
 #include <QTextCursor>
 
 #ifdef WITH_TESTS
 #include "autotestunittests.h"
+#include "loadprojectscenario.h"
 #endif
 
 using namespace Core;
@@ -90,13 +94,13 @@ public:
     AutotestPluginPrivate();
     ~AutotestPluginPrivate() override;
 
-    TestNavigationWidgetFactory *m_navigationWidgetFactory = nullptr;
+    TestNavigationWidgetFactory m_navigationWidgetFactory;
     TestResultsPane *m_resultsPane = nullptr;
     QMap<QString, ChoicePair> m_runconfigCache;
 
     void initializeMenuEntries();
-    void onRunAllTriggered();
-    void onRunSelectedTriggered();
+    void onRunAllTriggered(TestRunMode mode);
+    void onRunSelectedTriggered(TestRunMode mode);
     void onRunFailedTriggered();
     void onRunFileTriggered();
     void onRunUnderCursorTriggered(TestRunMode mode);
@@ -108,6 +112,9 @@ public:
     TestTreeModel m_testTreeModel{&m_testCodeParser};
     TestRunner m_testRunner;
     TestFrameworkManager m_frameworkManager;
+#ifdef WITH_TESTS
+    LoadProjectScenario m_loadProjectScenario{&m_testTreeModel};
+#endif
 };
 
 static AutotestPluginPrivate *dd = nullptr;
@@ -139,8 +146,9 @@ AutotestPluginPrivate::AutotestPluginPrivate()
     m_frameworkManager.registerTestFramework(new BoostTestFramework);
     m_frameworkManager.registerTestFramework(new CatchFramework);
 
+    m_frameworkManager.registerTestTool(new CTestTool);
+
     m_frameworkManager.synchronizeSettings(ICore::settings());
-    m_navigationWidgetFactory = new TestNavigationWidgetFactory;
     m_resultsPane = TestResultsPane::instance();
 
     auto panelFactory = new ProjectExplorer::ProjectPanelFactory();
@@ -152,8 +160,9 @@ AutotestPluginPrivate::AutotestPluginPrivate()
     });
     ProjectExplorer::ProjectPanelFactory::registerFactory(panelFactory);
 
-    m_frameworkManager.activateFrameworksFromSettings(&m_settings);
+    TestFrameworkManager::activateFrameworksAndToolsFromSettings(&m_settings);
     m_testTreeModel.synchronizeTestFrameworks();
+    m_testTreeModel.synchronizeTestTools();
 
     auto sessionManager = ProjectExplorer::SessionManager::instance();
     connect(sessionManager, &ProjectExplorer::SessionManager::startupProjectChanged,
@@ -176,7 +185,6 @@ AutotestPluginPrivate::~AutotestPluginPrivate()
         s_projectSettings.clear();
     }
 
-    delete m_navigationWidgetFactory;
     delete m_resultsPane;
 }
 
@@ -201,33 +209,51 @@ void AutotestPluginPrivate::initializeMenuEntries()
     menu->setOnAllDisabledBehavior(ActionContainer::Show);
 
     QAction *action = new QAction(tr("Run &All Tests"), this);
-    action->setIcon(Utils::Icons::RUN_SMALL_TOOLBAR.icon());
+    action->setIcon(Utils::Icons::RUN_SMALL.icon());
     action->setToolTip(tr("Run All Tests"));
     Command *command = ActionManager::registerAction(action, Constants::ACTION_RUN_ALL_ID);
     command->setDefaultKeySequence(
         QKeySequence(useMacShortcuts ? tr("Ctrl+Meta+T, Ctrl+Meta+A") : tr("Alt+Shift+T,Alt+A")));
-    connect(action, &QAction::triggered, this, &AutotestPluginPrivate::onRunAllTriggered);
+    connect(action, &QAction::triggered,
+            std::bind(&AutotestPluginPrivate::onRunAllTriggered, this, TestRunMode::Run));
+    action->setEnabled(false);
+    menu->addAction(command);
+
+    action = new QAction(tr("Run All Tests Without Deployment"), this);
+    action->setIcon(Utils::Icons::RUN_SMALL.icon());
+    action->setToolTip(tr("Run All Tests Without Deployment"));
+    command = ActionManager::registerAction(action, Constants::ACTION_RUN_ALL_NODEPLOY_ID);
+    command->setDefaultKeySequence(
+                QKeySequence(useMacShortcuts ? tr("Ctrl+Meta+T, Ctrl+Meta+E") : tr("Alt+Shift+T,Alt+E")));
+    connect(action, &QAction::triggered,
+            std::bind(&AutotestPluginPrivate::onRunAllTriggered, this, TestRunMode::RunWithoutDeploy));
     action->setEnabled(false);
     menu->addAction(command);
 
     action = new QAction(tr("&Run Selected Tests"), this);
-    Utils::Icon runSelectedIcon = Utils::Icons::RUN_SMALL_TOOLBAR;
-    for (const Utils::IconMaskAndColor &maskAndColor : Icons::RUN_SELECTED_OVERLAY)
-        runSelectedIcon.append(maskAndColor);
-    action->setIcon(runSelectedIcon.icon());
+    action->setIcon(Utils::Icons::RUN_SELECTED.icon());
     action->setToolTip(tr("Run Selected Tests"));
     command = ActionManager::registerAction(action, Constants::ACTION_RUN_SELECTED_ID);
     command->setDefaultKeySequence(
         QKeySequence(useMacShortcuts ? tr("Ctrl+Meta+T, Ctrl+Meta+R") : tr("Alt+Shift+T,Alt+R")));
-    connect(action, &QAction::triggered, this, &AutotestPluginPrivate::onRunSelectedTriggered);
+    connect(action, &QAction::triggered,
+            std::bind(&AutotestPluginPrivate::onRunSelectedTriggered, this, TestRunMode::Run));
+    action->setEnabled(false);
+    menu->addAction(command);
+
+    action = new QAction(tr("&Run Selected Tests Without Deployment"), this);
+    action->setIcon(Utils::Icons::RUN_SELECTED.icon());
+    action->setToolTip(tr("Run Selected Tests"));
+    command = ActionManager::registerAction(action, Constants::ACTION_RUN_SELECTED_NODEPLOY_ID);
+    command->setDefaultKeySequence(
+        QKeySequence(useMacShortcuts ? tr("Ctrl+Meta+T, Ctrl+Meta+W") : tr("Alt+Shift+T,Alt+W")));
+    connect(action, &QAction::triggered,
+            std::bind(&AutotestPluginPrivate::onRunSelectedTriggered, this, TestRunMode::RunWithoutDeploy));
     action->setEnabled(false);
     menu->addAction(command);
 
     action = new QAction(tr("Run &Failed Tests"),  this);
-    Utils::Icon runFailedIcon = Utils::Icons::RUN_SMALL_TOOLBAR;
-    for (const Utils::IconMaskAndColor &maskAndColor: Icons::RUN_FAILED_OVERLAY)
-        runFailedIcon.append(maskAndColor);
-    action->setIcon(runFailedIcon.icon());
+    action->setIcon(Icons::RUN_FAILED.icon());
     action->setToolTip(tr("Run Failed Tests"));
     command = ActionManager::registerAction(action, Constants::ACTION_RUN_FAILED_ID);
     command->setDefaultKeySequence(
@@ -237,10 +263,7 @@ void AutotestPluginPrivate::initializeMenuEntries()
     menu->addAction(command);
 
     action = new QAction(tr("Run Tests for &Current File"), this);
-    Utils::Icon runFileIcon = Utils::Icons::RUN_SMALL_TOOLBAR;
-    for (const Utils::IconMaskAndColor &maskAndColor : Icons::RUN_FILE_OVERLAY)
-        runFileIcon.append(maskAndColor);
-    action->setIcon(runFileIcon.icon());
+    action->setIcon(Utils::Icons::RUN_FILE.icon());
     action->setToolTip(tr("Run Tests for Current File"));
     command = ActionManager::registerAction(action, Constants::ACTION_RUN_FILE_ID);
     command->setDefaultKeySequence(
@@ -276,6 +299,12 @@ bool AutotestPlugin::initialize(const QStringList &arguments, QString *errorStri
     Q_UNUSED(errorString)
 
     dd = new AutotestPluginPrivate;
+#ifdef WITH_TESTS
+    ExtensionSystem::PluginManager::registerScenario("TestStringTable",
+                   [] { return dd->m_loadProjectScenario(); });
+    ExtensionSystem::PluginManager::registerScenario("TestModelManagerInterface",
+                   [] { return dd->m_loadProjectScenario(); });
+#endif
     return true;
 }
 
@@ -295,6 +324,15 @@ void AutotestPlugin::extensionsInitialized()
     contextMenu->addSeparator();
     contextMenu->addAction(command);
 
+    action = new QAction(tr("Run Test Under Cursor Without Deployment"), this);
+    action->setEnabled(false);
+    action->setIcon(Utils::Icons::RUN_SMALL.icon());
+
+    command = ActionManager::registerAction(action, Constants::ACTION_RUN_UCURSOR_NODEPLOY);
+    connect(action, &QAction::triggered,
+            std::bind(&AutotestPluginPrivate::onRunUnderCursorTriggered, dd, TestRunMode::RunWithoutDeploy));
+    contextMenu->addAction(command);
+
     action = new QAction(tr("&Debug Test Under Cursor"), this);
     action->setEnabled(false);
     action->setIcon(ProjectExplorer::Icons::DEBUG_START_SMALL.icon());
@@ -303,6 +341,15 @@ void AutotestPlugin::extensionsInitialized()
     connect(action, &QAction::triggered,
             std::bind(&AutotestPluginPrivate::onRunUnderCursorTriggered, dd, TestRunMode::Debug));
     contextMenu->addAction(command);
+
+    action = new QAction(tr("Debug Test Under Cursor Without Deployment"), this);
+    action->setEnabled(false);
+    action->setIcon(ProjectExplorer::Icons::DEBUG_START_SMALL.icon());
+
+    command = ActionManager::registerAction(action, Constants::ACTION_RUN_DBG_UCURSOR_NODEPLOY);
+    connect(action, &QAction::triggered,
+            std::bind(&AutotestPluginPrivate::onRunUnderCursorTriggered, dd, TestRunMode::DebugWithoutDeploy));
+    contextMenu->addAction(command);
     contextMenu->addSeparator();
 }
 
@@ -313,21 +360,21 @@ ExtensionSystem::IPlugin::ShutdownFlag AutotestPlugin::aboutToShutdown()
     return SynchronousShutdown;
 }
 
-void AutotestPluginPrivate::onRunAllTriggered()
+void AutotestPluginPrivate::onRunAllTriggered(TestRunMode mode)
 {
     m_testRunner.setSelectedTests(m_testTreeModel.getAllTestCases());
-    m_testRunner.prepareToRunTests(TestRunMode::Run);
+    m_testRunner.prepareToRunTests(mode);
 }
 
-void AutotestPluginPrivate::onRunSelectedTriggered()
+void AutotestPluginPrivate::onRunSelectedTriggered(TestRunMode mode)
 {
     m_testRunner.setSelectedTests(m_testTreeModel.getSelectedTests());
-    m_testRunner.prepareToRunTests(TestRunMode::Run);
+    m_testRunner.prepareToRunTests(mode);
 }
 
 void AutotestPluginPrivate::onRunFailedTriggered()
 {
-    const QList<TestConfiguration *> failed = m_testTreeModel.getFailedTests();
+    const QList<ITestConfiguration *> failed = m_testTreeModel.getFailedTests();
     if (failed.isEmpty()) // the framework might not be able to provide them
         return;
     m_testRunner.setSelectedTests(failed);
@@ -344,7 +391,7 @@ void AutotestPluginPrivate::onRunFileTriggered()
     if (fileName.isEmpty())
         return;
 
-    const QList<TestConfiguration *> tests = m_testTreeModel.getTestsForFile(fileName);
+    const QList<ITestConfiguration *> tests = m_testTreeModel.getTestsForFile(fileName);
     if (tests.isEmpty())
         return;
 
@@ -352,12 +399,12 @@ void AutotestPluginPrivate::onRunFileTriggered()
     m_testRunner.prepareToRunTests(TestRunMode::Run);
 }
 
-static QList<TestConfiguration *> testItemsToTestConfigurations(const QList<TestTreeItem *> &items,
+static QList<ITestConfiguration *> testItemsToTestConfigurations(const QList<ITestTreeItem *> &items,
                                                                 TestRunMode mode)
 {
-    QList<TestConfiguration *> configs;
-    for (const TestTreeItem * item : items) {
-        if (TestConfiguration *currentConfig = item->asConfiguration(mode))
+    QList<ITestConfiguration *> configs;
+    for (const ITestTreeItem * item : items) {
+        if (ITestConfiguration *currentConfig = item->asConfiguration(mode))
             configs << currentConfig;
     }
     return configs;
@@ -372,22 +419,22 @@ void AutotestPluginPrivate::onRunUnderCursorTriggered(TestRunMode mode)
     if (text.isEmpty())
         return; // Do not trigger when no name under cursor
 
-    const QList<TestTreeItem *> testsItems = m_testTreeModel.testItemsByName(text);
+    const QList<ITestTreeItem *> testsItems = m_testTreeModel.testItemsByName(text);
     if (testsItems.isEmpty())
         return; // Wrong location triggered
 
     // check whether we have been triggered on a test function definition
     const int line = currentEditor->currentLine();
-    const QString &filePath = currentEditor->textDocument()->filePath().toString();
-    const QList<TestTreeItem *> filteredItems = Utils::filtered(testsItems, [&](TestTreeItem *it){
+    const Utils::FilePath &filePath = currentEditor->textDocument()->filePath();
+    const QList<ITestTreeItem *> filteredItems = Utils::filtered(testsItems, [&](ITestTreeItem *it){
         return it->line() == line && it->filePath() == filePath;
     });
 
-    const QList<TestConfiguration *> testsToRun = testItemsToTestConfigurations(
+    const QList<ITestConfiguration *> testsToRun = testItemsToTestConfigurations(
                 filteredItems.size() == 1 ? filteredItems : testsItems, mode);
 
     if (testsToRun.isEmpty()) {
-        MessageManager::write(tr("Selected test was not found (%1).").arg(text), MessageManager::Flash);
+        MessageManager::writeFlashing(tr("Selected test was not found (%1).").arg(text));
         return;
     }
 
@@ -411,6 +458,8 @@ void AutotestPlugin::updateMenuItemsEnabledState()
 
     ActionManager::command(Constants::ACTION_RUN_ALL_ID)->action()->setEnabled(canRun);
     ActionManager::command(Constants::ACTION_RUN_SELECTED_ID)->action()->setEnabled(canRun);
+    ActionManager::command(Constants::ACTION_RUN_ALL_NODEPLOY_ID)->action()->setEnabled(canRun);
+    ActionManager::command(Constants::ACTION_RUN_SELECTED_NODEPLOY_ID)->action()->setEnabled(canRun);
     ActionManager::command(Constants::ACTION_RUN_FAILED_ID)->action()->setEnabled(canRunFailed);
     ActionManager::command(Constants::ACTION_RUN_FILE_ID)->action()->setEnabled(canRun);
     ActionManager::command(Constants::ACTION_SCAN_ID)->action()->setEnabled(canScan);
@@ -420,7 +469,9 @@ void AutotestPlugin::updateMenuItemsEnabledState()
         return; // When no context menu, actions do not exists
 
     ActionManager::command(Constants::ACTION_RUN_UCURSOR)->action()->setEnabled(canRun);
+    ActionManager::command(Constants::ACTION_RUN_UCURSOR_NODEPLOY)->action()->setEnabled(canRun);
     ActionManager::command(Constants::ACTION_RUN_DBG_UCURSOR)->action()->setEnabled(canRun);
+    ActionManager::command(Constants::ACTION_RUN_DBG_UCURSOR_NODEPLOY)->action()->setEnabled(canRun);
 }
 
 void AutotestPlugin::cacheRunConfigChoice(const QString &buildTargetKey, const ChoicePair &choice)
@@ -457,7 +508,7 @@ QVector<QObject *> AutotestPlugin::createTestObjects() const
 
 bool ChoicePair::matches(const ProjectExplorer::RunConfiguration *rc) const
 {
-    return rc && rc->displayName() == displayName && rc->runnable().executable.toString() == executable;
+    return rc && rc->displayName() == displayName && rc->runnable().command.executable().toString() == executable;
 }
 
 } // Internal
