@@ -55,28 +55,29 @@ QStringList filterInterfering(const QStringList &provided, QStringList *omitted)
                                                          "--gtest_stream_result_to=",
                                                          "--gtest_break_on_failure",
                                                          "--gtest_throw_on_failure",
+                                                         "--gtest_catch_exceptions=",
                                                          "--gtest_print_time="
                                                          };
 
-    QSet<QString> allowed = Utils::filtered(Utils::toSet(provided), [] (const QString &arg) {
+    QStringList allowed = Utils::filtered(provided, [] (const QString &arg) {
         return Utils::allOf(knownInterferingOptions, [&arg] (const QString &interfering) {
             return !arg.startsWith(interfering);
         });
     });
 
-    if (omitted) {
+    if (omitted && allowed.size() < provided.size()) {
         QSet<QString> providedSet = Utils::toSet(provided);
-        providedSet.subtract(allowed);
+        providedSet.subtract(Utils::toSet(allowed));
         omitted->append(Utils::toList(providedSet));
     }
-    return Utils::toList(allowed);
+    return allowed;
 }
 
 QStringList GTestConfiguration::argumentsForTestRunner(QStringList *omitted) const
 {
     QStringList arguments;
     if (AutotestPlugin::settings()->processArgs) {
-        arguments << filterInterfering(runnable().commandLineArguments.split(
+        arguments << filterInterfering(runnable().command.arguments().split(
                                            ' ', Qt::SkipEmptyParts), omitted);
     }
 
@@ -84,22 +85,23 @@ QStringList GTestConfiguration::argumentsForTestRunner(QStringList *omitted) con
     if (!testSets.isEmpty())
         arguments << "--gtest_filter=" + testSets.join(':');
 
-    auto gSettings = dynamic_cast<GTestSettings *>(framework()->frameworkSettings());
+    auto gSettings = static_cast<GTestSettings *>(framework()->testSettings());
     if (!gSettings)
         return arguments;
 
-    if (gSettings->runDisabled)
+    if (gSettings->runDisabled.value())
         arguments << "--gtest_also_run_disabled_tests";
-    if (gSettings->repeat)
-        arguments << QString("--gtest_repeat=%1").arg(gSettings->iterations);
-    if (gSettings->shuffle)
-        arguments << "--gtest_shuffle" << QString("--gtest_random_seed=%1").arg(gSettings->seed);
-    if (gSettings->throwOnFailure)
+    if (gSettings->repeat.value())
+        arguments << QString("--gtest_repeat=%1").arg(gSettings->iterations.value());
+    if (gSettings->shuffle.value())
+        arguments << "--gtest_shuffle" << QString("--gtest_random_seed=%1").arg(gSettings->seed.value());
+    if (gSettings->throwOnFailure.value())
         arguments << "--gtest_throw_on_failure";
 
     if (isDebugRunMode()) {
-        if (gSettings->breakOnFailure)
+        if (gSettings->breakOnFailure.value())
             arguments << "--gtest_break_on_failure";
+        arguments << "--gtest_catch_exceptions=0";
     }
     return arguments;
 }
