@@ -125,14 +125,11 @@ static QString constructBenchmarkInformation(const QString &metric, double value
             .arg(iterations);
 }
 
-static QString constructSourceFilePath(const QString &path, const QString &filePath)
-{
-    return QFileInfo(path, filePath).canonicalFilePath();
-}
-
 QtTestOutputReader::QtTestOutputReader(const QFutureInterface<TestResultPtr> &futureInterface,
-                                       QProcess *testApplication, const QString &buildDirectory,
-                                       const QString &projectFile, OutputMode mode, TestType type)
+                                       QProcess *testApplication,
+                                       const Utils::FilePath &buildDirectory,
+                                       const Utils::FilePath &projectFile,
+                                       OutputMode mode, TestType type)
     : TestOutputReader(futureInterface, testApplication, buildDirectory)
     , m_projectFile(projectFile)
     , m_mode(mode)
@@ -199,6 +196,8 @@ void QtTestOutputReader::processXMLOutput(const QByteArray &outputLine)
             break;
         }
     }
+    if (m_cdataMode == Description)
+        m_xmlReader.addData("\n");
     m_xmlReader.addData(QString::fromUtf8(outputLine));
     while (!m_xmlReader.atEnd()) {
         if (m_futureInterface.isCanceled())
@@ -237,9 +236,8 @@ void QtTestOutputReader::processXMLOutput(const QByteArray &outputLine)
                 const QXmlStreamAttributes &attributes = m_xmlReader.attributes();
                 m_result = TestResult::resultFromString(
                             attributes.value(QStringLiteral("type")).toString());
-                m_file = decode(attributes.value(QStringLiteral("file")).toString());
-                if (!m_file.isEmpty())
-                    m_file = constructSourceFilePath(m_buildDir, m_file);
+                const QString file = decode(attributes.value(QStringLiteral("file")).toString());
+                m_file = constructSourceFilePath(m_buildDir, file);
                 m_lineNumber = attributes.value(QStringLiteral("line")).toInt();
             } else if (currentTag == QStringLiteral("BenchmarkResult")) {
                 const QXmlStreamAttributes &attributes = m_xmlReader.attributes();
@@ -359,8 +357,8 @@ void QtTestOutputReader::processPlainTextOutput(const QByteArray &outputLine)
     static const QRegularExpression start("^[*]{9} Start testing of (.*) [*]{9}$");
     static const QRegularExpression config("^Config: Using QtTest library (.*), "
                                            "(Qt (\\d+(\\.\\d+){2}) \\(.*\\))$");
-    static const QRegularExpression summary("^Totals: \\d+ passed, \\d+ failed, "
-                                            "\\d+ skipped(, \\d+ blacklisted)?$");
+    static const QRegularExpression summary("^Totals: (\\d+) passed, (\\d+) failed, "
+                                            "(\\d+) skipped(, (\\d+) blacklisted)?(, \\d+ms)?$");
     static const QRegularExpression finish("^[*]{9} Finished testing of (.*) [*]{9}$");
 
     static const QRegularExpression result("^(PASS   |FAIL!  |XFAIL  |XPASS  |SKIP   |RESULT "
@@ -397,7 +395,15 @@ void QtTestOutputReader::processPlainTextOutput(const QByteArray &outputLine)
         m_className = match.captured(1);
         QTC_CHECK(!m_className.isEmpty());
         sendStartMessage(false);
-    } else if (summary.match(line).hasMatch() || finish.match(line).hasMatch()) {
+    } else if (hasMatch(summary)) {
+        m_summary[ResultType::Pass] = match.captured(1).toInt();
+        m_summary[ResultType::Fail] = match.captured(2).toInt();
+        m_summary[ResultType::Skip] = match.captured(3).toInt();
+        // BlacklistedXYZ is wrong here, but we use it for convenience (avoids another enum value)
+        if (int blacklisted = match.captured(5).toInt())
+            m_summary[ResultType::BlacklistedPass] = blacklisted;
+        processSummaryFinishOutput();
+    } else if (finish.match(line).hasMatch()) {
         processSummaryFinishOutput();
     } else { // we have some plain output, but we cannot say where for sure it belongs to..
         if (!m_description.isEmpty())
@@ -479,10 +485,10 @@ void QtTestOutputReader::sendCompleteInformation()
         testResult->setFileName(m_file);
         testResult->setLine(m_lineNumber);
     } else {
-        const TestTreeItem *testItem = testResult->findTestTreeItem();
+        const ITestTreeItem *testItem = testResult->findTestTreeItem();
         if (testItem && testItem->line()) {
             testResult->setFileName(testItem->filePath());
-            testResult->setLine(static_cast<int>(testItem->line()));
+            testResult->setLine(testItem->line());
         }
     }
     testResult->setDescription(m_description);
@@ -503,10 +509,10 @@ void QtTestOutputReader::sendStartMessage(bool isFunction)
     testResult->setResult(ResultType::TestStart);
     testResult->setDescription(isFunction ? tr("Executing test function %1").arg(m_testCase)
                                           : tr("Executing test case %1").arg(m_className));
-    const TestTreeItem *testItem = testResult->findTestTreeItem();
+    const ITestTreeItem *testItem = testResult->findTestTreeItem();
     if (testItem && testItem->line()) {
         testResult->setFileName(testItem->filePath());
-        testResult->setLine(static_cast<int>(testItem->line()));
+        testResult->setLine(testItem->line());
     }
     reportResult(testResult);
 }
