@@ -27,6 +27,7 @@
 #include "quicktestparser.h"
 #include "quicktesttreeitem.h"
 
+#include "../autotestconstants.h"
 #include "../testframeworkmanager.h"
 #include "../qtest/qttestconstants.h"
 
@@ -38,10 +39,10 @@ ITestParser *QuickTestFramework::createTestParser()
     return new QuickTestParser(this);
 }
 
-TestTreeItem *QuickTestFramework::createRootNode()
+ITestTreeItem *QuickTestFramework::createRootNode()
 {
     return new QuickTestTreeItem(this, QCoreApplication::translate("QuickTestFramework", "Quick Test"),
-                                 QString(), TestTreeItem::Root);
+                                 Utils::FilePath(), ITestTreeItem::Root);
 }
 
 const char *QuickTestFramework::name() const
@@ -54,12 +55,12 @@ unsigned QuickTestFramework::priority() const
     return 5;
 }
 
-IFrameworkSettings *QuickTestFramework::frameworkSettings()
+ITestSettings *QuickTestFramework::testSettings()
 {
     static const Utils::Id id
             = Utils::Id(Constants::FRAMEWORK_PREFIX).withSuffix(QtTest::Constants::FRAMEWORK_NAME);
     ITestFramework *qtTestFramework = TestFrameworkManager::frameworkForId(id);
-    return qtTestFramework->frameworkSettings();
+    return qtTestFramework->testSettings();
 }
 
 } // namespace Internal
