@@ -23,17 +23,18 @@
 **
 ****************************************************************************/
 
-#include "autotestplugin.h"
-#include "autotesticons.h"
 #include "testresultspane.h"
-#include "testresultmodel.h"
+
+#include "autotesticons.h"
+#include "autotestplugin.h"
+#include "itestframework.h"
+#include "testeditormark.h"
 #include "testresultdelegate.h"
+#include "testresultmodel.h"
+#include "testresultmodel.h"
 #include "testrunner.h"
 #include "testsettings.h"
 #include "testtreemodel.h"
-#include "testcodeparser.h"
-#include "testeditormark.h"
-#include "testoutputreader.h"
 
 #include <aggregation/aggregate.h>
 #include <coreplugin/actionmanager/actionmanager.h>
@@ -48,6 +49,7 @@
 #include <texteditor/fontsettings.h>
 #include <texteditor/texteditor.h>
 #include <texteditor/texteditorsettings.h>
+#include <utils/proxyaction.h>
 #include <utils/qtcassert.h>
 #include <utils/stylehelper.h>
 #include <utils/theme/theme.h>
@@ -66,6 +68,7 @@
 #include <QVBoxLayout>
 
 using namespace Core;
+using namespace Utils;
 
 namespace Autotest {
 namespace Internal {
@@ -74,6 +77,7 @@ ResultsTreeView::ResultsTreeView(QWidget *parent)
     : Utils::TreeView(parent)
 {
     setAttribute(Qt::WA_MacShowFocusRect, false);
+    setFrameStyle(NoFrame);
 }
 
 void ResultsTreeView::keyPressEvent(QKeyEvent *event)
@@ -163,6 +167,7 @@ TestResultsPane::TestResultsPane(QObject *parent) :
             this, &TestResultsPane::addTestResult);
     connect(TestRunner::instance(), &TestRunner::hadDisabledTests,
             m_model, &TestResultModel::raiseDisabledTests);
+    visualOutputWidget->installEventFilter(this);
 }
 
 void TestResultsPane::createToolButtons()
@@ -180,15 +185,27 @@ void TestResultsPane::createToolButtons()
     });
 
     m_runAll = new QToolButton(m_treeView);
-    m_runAll->setDefaultAction(ActionManager::command(Constants::ACTION_RUN_ALL_ID)->action());
+    m_runAll->setDefaultAction(
+                Utils::ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_ALL_ID)->action(),
+                    Utils::Icons::RUN_SMALL_TOOLBAR.icon()));
 
     m_runSelected = new QToolButton(m_treeView);
-    m_runSelected->setDefaultAction(ActionManager::command(Constants::ACTION_RUN_SELECTED_ID)->action());
+    m_runSelected->setDefaultAction(
+                Utils::ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_SELECTED_ID)->action(),
+                    Utils::Icons::RUN_SELECTED_TOOLBAR.icon()));
 
     m_runFailed = new QToolButton(m_treeView);
-    m_runFailed->setDefaultAction(ActionManager::command(Constants::ACTION_RUN_FAILED_ID)->action());
+    m_runFailed->setDefaultAction(
+                Utils::ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_FAILED_ID)->action(),
+                    Icons::RUN_FAILED_TOOLBAR.icon()));
     m_runFile = new QToolButton(m_treeView);
-    m_runFile->setDefaultAction(ActionManager::command(Constants::ACTION_RUN_FILE_ID)->action());
+    m_runFile->setDefaultAction(
+                Utils::ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_FILE_ID)->action(),
+                    Utils::Icons::RUN_FILE_TOOLBAR.icon()));
 
     m_stopTestRun = new QToolButton(m_treeView);
     m_stopTestRun->setIcon(Utils::Icons::STOP_SMALL_TOOLBAR.icon());
@@ -235,6 +252,7 @@ void TestResultsPane::addTestResult(const TestResultPtr &result)
     m_atEnd = scrollBar ? scrollBar->value() == scrollBar->maximum() : true;
 
     m_model->addTestResult(result, m_expandCollapse->isChecked());
+    m_filterModel->invalidate();
     setIconBadgeNumber(m_model->resultTypeCount(ResultType::Fail)
                        + m_model->resultTypeCount(ResultType::MessageFatal)
                        + m_model->resultTypeCount(ResultType::UnexpectedPass));
@@ -280,7 +298,7 @@ void TestResultsPane::addOutputLine(const QByteArray &outputLine, OutputChannel
 
     const Utils::FormattedText formattedText
             = Utils::FormattedText{QString::fromUtf8(outputLine), m_defaultFormat};
-    QList<Utils::FormattedText> formatted = channel == OutputChannel::StdOut
+    const QList<Utils::FormattedText> formatted = channel == OutputChannel::StdOut
             ? m_stdOutHandler.parseText(formattedText)
             : m_stdErrHandler.parseText(formattedText);
 
@@ -459,7 +477,7 @@ void TestResultsPane::onItemActivated(const QModelIndex &index)
 
     const TestResult *testResult = m_filterModel->testResult(index);
     if (testResult && !testResult->fileName().isEmpty())
-        EditorManager::openEditorAt(testResult->fileName(), testResult->line(), 0);
+        EditorManager::openEditorAt(Utils::Link{testResult->fileName(), testResult->line(), 0});
 }
 
 void TestResultsPane::onRunAllTriggered()
@@ -493,9 +511,10 @@ void TestResultsPane::initializeFilterMenu()
     textAndType.insert(ResultType::MessageDebug, tr("Debug Messages"));
     textAndType.insert(ResultType::MessageWarn, tr("Warning Messages"));
     textAndType.insert(ResultType::MessageInternal, tr("Internal Messages"));
-    for (ResultType result : textAndType.keys()) {
+    for (auto it = textAndType.cbegin(); it != textAndType.cend(); ++it) {
+        const ResultType &result = it.key();
         QAction *action = new QAction(m_filterMenu);
-        action->setText(textAndType.value(result));
+        action->setText(it.value());
         action->setCheckable(true);
         action->setChecked(result != ResultType::MessageInternal || !omitIntern);
         action->setData(int(result));
@@ -559,6 +578,14 @@ void TestResultsPane::filterMenuTriggered(QAction *action)
     navigateStateChanged();
 }
 
+bool TestResultsPane::eventFilter(QObject *object, QEvent *event)
+{
+    QTC_ASSERT(m_outputWidget, return false);
+    if (event->type() == QEvent::Resize && object->parent() == m_outputWidget)
+        static_cast<TestResultDelegate *>(m_treeView->itemDelegate())->clearCache();
+    return false;
+}
+
 void TestResultsPane::onTestRunStarted()
 {
     m_testRunning = true;
@@ -631,13 +658,34 @@ void TestResultsPane::onCustomContextMenuRequested(const QPoint &pos)
     });
     menu.addAction(action);
 
+    action = new QAction(tr("Run This Test Without Deployment"), &menu);
+    action->setEnabled(correlatingItem && correlatingItem->canProvideTestConfiguration());
+    connect(action, &QAction::triggered, this, [this, clicked] {
+        onRunThisTestTriggered(TestRunMode::RunWithoutDeploy, clicked);
+    });
+    menu.addAction(action);
+
     action = new QAction(tr("Debug This Test"), &menu);
-    action->setEnabled(correlatingItem && correlatingItem->canProvideDebugConfiguration());
+    bool debugEnabled = false;
+    if (correlatingItem) {
+        if (correlatingItem->testBase()->type() == ITestBase::Framework) {
+            auto testTreeItem = static_cast<const TestTreeItem *>(correlatingItem);
+            debugEnabled = testTreeItem && testTreeItem->canProvideDebugConfiguration();
+        }
+    }
+    action->setEnabled(debugEnabled);
     connect(action, &QAction::triggered, this, [this, clicked] {
         onRunThisTestTriggered(TestRunMode::Debug, clicked);
     });
     menu.addAction(action);
 
+    action = new QAction(tr("Debug This Test Without Deployment"), &menu);
+    action->setEnabled(debugEnabled);
+    connect(action, &QAction::triggered, this, [this, clicked] {
+        onRunThisTestTriggered(TestRunMode::DebugWithoutDeploy, clicked);
+    });
+    menu.addAction(action);
+
     menu.exec(m_treeView->mapToGlobal(pos));
 }
 
@@ -665,15 +713,14 @@ void TestResultsPane::onCopyWholeTriggered()
 
 void TestResultsPane::onSaveWholeTriggered()
 {
-    const QString fileName = QFileDialog::getSaveFileName(ICore::dialogParent(),
-                                                          tr("Save Output To"));
-    if (fileName.isEmpty())
+    const FilePath filePath = FileUtils::getSaveFilePath(nullptr, tr("Save Output To"));
+    if (filePath.isEmpty())
         return;
 
-    Utils::FileSaver saver(fileName, QIODevice::Text);
+    FileSaver saver(filePath, QIODevice::Text);
     if (!saver.write(getWholeOutput().toUtf8()) || !saver.finalize()) {
         QMessageBox::critical(ICore::dialogParent(), tr("Error"),
-                              tr("Failed to write \"%1\".\n\n%2").arg(fileName)
+                              tr("Failed to write \"%1\".\n\n%2").arg(filePath.toUserOutput())
                               .arg(saver.errorString()));
     }
 }
@@ -682,7 +729,7 @@ void TestResultsPane::onRunThisTestTriggered(TestRunMode runMode, const TestResu
 {
     QTC_ASSERT(result, return);
 
-    const TestTreeItem *item = result->findTestTreeItem();
+    const ITestTreeItem *item = result->findTestTreeItem();
 
     if (item)
         TestRunner::instance()->runTest(runMode, item);
@@ -728,8 +775,7 @@ void TestResultsPane::createMarks(const QModelIndex &parent)
         bool isLocationItem = result->result() == ResultType::MessageLocation;
         if (interested.contains(result->result())
                 || (isLocationItem && interested.contains(parentType))) {
-            const Utils::FilePath fileName = Utils::FilePath::fromString(result->fileName());
-            TestEditorMark *mark = new TestEditorMark(index, fileName, result->line());
+            TestEditorMark *mark = new TestEditorMark(index, result->fileName(), result->line());
             mark->setIcon(index.data(Qt::DecorationRole).value<QIcon>());
             mark->setColor(Utils::Theme::OutputPanes_TestFailTextColor);
             mark->setPriority(TextEditor::TextMark::NormalPriority);
