@@ -25,12 +25,13 @@
 
 #pragma once
 
+#include <utils/fileutils.h>
 #include <utils/treemodel.h>
 
 #include <QList>
+#include <QMetaType>
 #include <QSet>
 #include <QString>
-#include <QMetaType>
 
 namespace {
     enum ItemRole {
@@ -42,17 +43,15 @@ namespace {
     };
 }
 
-namespace CppTools { class CppModelManager; }
-namespace Utils { class FilePath; }
-
 namespace Autotest {
 
+class ITestBase;
 class ITestFramework;
-class TestConfiguration;
+class ITestConfiguration;
 class TestParseResult;
 enum class TestRunMode;
 
-class TestTreeItem : public Utils::TypedTreeItem<TestTreeItem>
+class ITestTreeItem : public Utils::TypedTreeItem<ITestTreeItem>
 {
 public:
 
@@ -73,56 +72,87 @@ public:
         Naturally
     };
 
-    explicit TestTreeItem(ITestFramework *framework,
+    explicit ITestTreeItem(ITestBase *testBase,
+                           const QString &name = QString(),
+                           const Utils::FilePath &filePath = Utils::FilePath(),
+                           Type type = Root);
+
+    virtual QVariant data(int column, int role) const override;
+    virtual bool setData(int column, const QVariant &data, int role) override;
+    virtual Qt::ItemFlags flags(int column) const override;
+
+    virtual Qt::CheckState checked() const;
+    virtual bool canProvideTestConfiguration() const { return false; }
+    virtual ITestConfiguration *testConfiguration() const { return nullptr; }
+    virtual ITestConfiguration *asConfiguration(TestRunMode mode) const;
+
+    virtual QList<ITestConfiguration *> getAllTestConfigurations() const { return {}; }
+    virtual QList<ITestConfiguration *> getSelectedTestConfigurations() const { return {}; };
+    virtual QList<ITestConfiguration *> getFailedTestConfigurations() const { return {}; }
+
+    const QString name() const { return m_name; }
+    void setName(const QString &name) { m_name = name; }
+    const Utils::FilePath filePath() const { return m_filePath; }
+    void setFilePath(const Utils::FilePath &filePath) { m_filePath = filePath; }
+    Type type() const { return m_type; }
+    int line() const { return m_line; }
+    void setLine(int line) { m_line = line;}
+    ITestBase *testBase() const { return m_testBase; }
+
+    virtual bool lessThan(const ITestTreeItem *other, SortMode mode) const;
+    QString cacheName() const { return m_filePath.toString() + ':' + m_name; }
+
+protected:
+    void setType(Type type) { m_type = type; }
+    Qt::CheckState m_checked = Qt::Checked;
+
+private:
+    ITestBase *m_testBase = nullptr; // not owned
+    QString m_name;
+    Utils::FilePath m_filePath;
+    Type m_type;
+    int m_line = 0;
+    bool m_failed = false;
+};
+
+class TestTreeItem : public ITestTreeItem
+{
+public:
+    explicit TestTreeItem(ITestFramework *testFramework,
                           const QString &name = QString(),
-                          const QString &filePath = QString(),
+                          const Utils::FilePath &filePath = Utils::FilePath(),
                           Type type = Root);
 
     virtual TestTreeItem *copyWithoutChildren() = 0;
     virtual QVariant data(int column, int role) const override;
-    virtual bool setData(int column, const QVariant &data, int role) override;
-    virtual Qt::ItemFlags flags(int column) const override;
     bool modifyTestCaseOrSuiteContent(const TestParseResult *result);
     bool modifyTestFunctionContent(const TestParseResult *result);
     bool modifyDataTagContent(const TestParseResult *result);
     bool modifyLineAndColumn(const TestParseResult *result);
 
     ITestFramework *framework() const;
-    const QString name() const { return m_name; }
-    void setName(const QString &name) { m_name = name; }
-    const QString filePath() const { return m_filePath; }
-    void setFilePath(const QString &filePath) { m_filePath = filePath; }
-    void setLine(int line) { m_line = line;}
-    int line() const { return m_line; }
     void setColumn(int column) { m_column = column; }
     int column() const { return m_column; }
-    QString proFile() const { return m_proFile; }
-    void setProFile(const QString &proFile) { m_proFile = proFile; }
-    virtual Qt::CheckState checked() const;
-    Type type() const { return m_type; }
+    Utils::FilePath proFile() const { return m_proFile; }
+    void setProFile(const Utils::FilePath &proFile) { m_proFile = proFile; }
     void markForRemoval(bool mark);
     void markForRemovalRecursively(bool mark);
-    virtual void markForRemovalRecursively(const QString &filePath);
-    virtual bool removeOnSweepIfEmpty() const { return m_type == GroupNode; }
+    virtual void markForRemovalRecursively(const Utils::FilePath &filepath);
+    virtual bool removeOnSweepIfEmpty() const { return type() == GroupNode; }
     bool markedForRemoval() const { return m_status == MarkedForRemoval; }
     bool newlyAdded() const { return m_status == NewlyAdded; }
+    TestTreeItem *childItem(int at) const;
     TestTreeItem *parentItem() const;
 
     TestTreeItem *findChildByName(const QString &name);
-    TestTreeItem *findChildByFile(const QString &filePath);
-    TestTreeItem *findChildByFileAndType(const QString &filePath, Type type);
-    TestTreeItem *findChildByNameAndFile(const QString &name, const QString &filePath);
+    TestTreeItem *findChildByFile(const Utils::FilePath &filePath);
+    TestTreeItem *findChildByFileAndType(const Utils::FilePath &filePath, Type type);
+    TestTreeItem *findChildByNameAndFile(const QString &name, const Utils::FilePath &filePath);
 
-    virtual bool canProvideTestConfiguration() const { return false; }
+    virtual ITestConfiguration *debugConfiguration() const { return nullptr; }
     virtual bool canProvideDebugConfiguration() const { return false; }
-    virtual TestConfiguration *testConfiguration() const { return nullptr; }
-    virtual TestConfiguration *debugConfiguration() const { return nullptr; }
-    TestConfiguration *asConfiguration(TestRunMode mode) const;
-    virtual QList<TestConfiguration *> getAllTestConfigurations() const;
-    virtual QList<TestConfiguration *> getSelectedTestConfigurations() const;
-    virtual QList<TestConfiguration *> getFailedTestConfigurations() const;
-    virtual QList<TestConfiguration *> getTestConfigurationsForFile(const Utils::FilePath &fileName) const;
-    virtual bool lessThan(const TestTreeItem *other, SortMode mode) const;
+    ITestConfiguration *asConfiguration(TestRunMode mode) const final;
+    virtual QList<ITestConfiguration *> getTestConfigurationsForFile(const Utils::FilePath &fileName) const;
     virtual TestTreeItem *find(const TestParseResult *result) = 0;
     virtual TestTreeItem *findChild(const TestTreeItem *other) = 0;
     virtual bool modify(const TestParseResult *result) = 0;
@@ -134,17 +164,16 @@ public:
     virtual TestTreeItem *applyFilters() { return nullptr; }
     // decide whether an item should still be added to the treemodel
     virtual bool shouldBeAddedAfterFiltering() const { return true; }
-    virtual QSet<QString> internalTargets() const;
 
-    QString cacheName() const { return m_filePath + ':' + m_name; }
+    void forAllChildItems(const std::function<void(TestTreeItem *)> &pred) const;
+    void forFirstLevelChildItems(const std::function<void(TestTreeItem *)> &pred) const;
+    TestTreeItem *findFirstLevelChildItem(const std::function<bool(TestTreeItem *)> &pred) const;
 protected:
     void copyBasicDataFrom(const TestTreeItem *other);
     typedef std::function<bool(const TestTreeItem *)> CompareFunction;
-    static QSet<QString> dependingInternalTargets(CppTools::CppModelManager *cppMM,
-                                                  const QString &file);
 
 private:
-    bool modifyFilePath(const QString &filePath);
+    bool modifyFilePath(const Utils::FilePath &filepath);
     bool modifyName(const QString &name);
 
     enum Status
@@ -154,15 +183,8 @@ private:
         Cleared
     };
 
-    ITestFramework *m_framework = nullptr;
-    QString m_name;
-    QString m_filePath;
-    Qt::CheckState m_checked;
-    bool m_failed = false;
-    Type m_type;
-    int m_line = 0;
     int m_column = 0;
-    QString m_proFile;
+    Utils::FilePath m_proFile;
     Status m_status = NewlyAdded;
 
     friend class TestTreeModel; // grant access to (protected) findChildBy()
@@ -171,7 +193,8 @@ private:
 class TestCodeLocationAndType
 {
 public:
-    QString m_name;     // tag name for m_type == TestDataTag, file name for other values
+    QString m_name;
+    Utils::FilePath m_filePath;
     int m_line = 0;
     int m_column = 0;
     TestTreeItem::Type m_type = TestTreeItem::Root;
