@@ -38,7 +38,6 @@
 #include <utils/pathchooser.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
-#include <utils/synchronousprocess.h>
 
 #include <QDebug>
 #include <QDir>
@@ -58,9 +57,7 @@ namespace Internal {
 
 // Helpers:
 
-static const char compilerCommandKeyC[] = "CompilerPath";
 static const char compilerPlatformCodeGenFlagsKeyC[] = "PlatformCodeGenFlags";
-static const char targetAbiKeyC[] = "TargetAbi";
 
 static bool compilerExists(const FilePath &compilerPath)
 {
@@ -70,7 +67,7 @@ static bool compilerExists(const FilePath &compilerPath)
 
 static QString cppLanguageOption(const FilePath &compiler)
 {
-    const QString baseName = compiler.toFileInfo().baseName();
+    const QString baseName = compiler.baseName();
     if (baseName == "iccarm" || baseName == "iccrl78"
             || baseName == "iccrh850" || baseName == "iccrx"
             || baseName == "iccriscv") {
@@ -89,7 +86,7 @@ static QString cppLanguageOption(const FilePath &compiler)
 }
 
 static Macros dumpPredefinedMacros(const FilePath &compiler, const QStringList &extraArgs,
-                                   const Utils::Id languageId, const QStringList &env)
+                                   const Id languageId, const Environment &env)
 {
     if (compiler.isEmpty() || !compiler.toFileInfo().isExecutable())
         return {};
@@ -103,7 +100,7 @@ static Macros dumpPredefinedMacros(const FilePath &compiler, const QStringList &
 
     const QString outpath = fakeIn.fileName() + ".tmp";
 
-    SynchronousProcess cpp;
+    QtcProcess cpp;
     cpp.setEnvironment(env);
     cpp.setTimeoutS(10);
 
@@ -114,10 +111,10 @@ static Macros dumpPredefinedMacros(const FilePath &compiler, const QStringList &
     cmd.addArg("--predef_macros");
     cmd.addArg(outpath);
 
-    const SynchronousProcessResponse response = cpp.runBlocking(cmd);
-    if (response.result != SynchronousProcessResponse::Finished
-            || response.exitCode != 0) {
-        qWarning() << response.exitMessage(cmd.toUserOutput(), 10);
+    cpp.setCommand(cmd);
+    cpp.runBlocking();
+    if (cpp.result() != QtcProcess::FinishedWithSuccess) {
+        qWarning() << cpp.exitMessage();
         return {};
     }
 
@@ -130,8 +127,8 @@ static Macros dumpPredefinedMacros(const FilePath &compiler, const QStringList &
     return Macro::toMacros(output);
 }
 
-static HeaderPaths dumpHeaderPaths(const FilePath &compiler, const Utils::Id languageId,
-                                   const QStringList &env)
+static HeaderPaths dumpHeaderPaths(const FilePath &compiler, const Id languageId,
+                                   const Environment &env)
 {
     if (!compiler.exists())
         return {};
@@ -149,22 +146,21 @@ static HeaderPaths dumpHeaderPaths(const FilePath &compiler, const Utils::Id lan
         return {};
     fakeIn.close();
 
-    SynchronousProcess cpp;
-    cpp.setEnvironment(env);
-    cpp.setTimeoutS(10);
-
     CommandLine cmd(compiler, {fakeIn.fileName()});
     if (languageId == ProjectExplorer::Constants::CXX_LANGUAGE_ID)
         cmd.addArg(cppLanguageOption(compiler));
     cmd.addArg("--preinclude");
     cmd.addArg(".");
 
-    // Note: Response should retutn an error, just don't check on errors.
-    const SynchronousProcessResponse response = cpp.runBlocking(cmd);
+    QtcProcess cpp;
+    cpp.setEnvironment(env);
+    cpp.setTimeoutS(10);
+    cpp.setCommand(cmd);
+    cpp.runBlocking();
 
     HeaderPaths headerPaths;
 
-    const QByteArray output = response.allOutput().toUtf8();
+    const QByteArray output = cpp.allOutput().toUtf8();
     for (auto pos = 0; pos < output.size(); ++pos) {
         const int searchIndex = output.indexOf("searched:", pos);
         if (searchIndex == -1)
@@ -185,7 +181,7 @@ static HeaderPaths dumpHeaderPaths(const FilePath &compiler, const Utils::Id lan
 
         // Ignore the QtC binary directory path.
         if (headerPath != QCoreApplication::applicationDirPath())
-            headerPaths.append({headerPath, HeaderPathType::BuiltIn});
+            headerPaths.append(HeaderPath::makeBuiltIn(headerPath));
 
         pos = endQuoteIndex + 1;
     }
@@ -294,24 +290,8 @@ IarToolChain::IarToolChain() :
     ToolChain(Constants::IAREW_TOOLCHAIN_TYPEID)
 {
     setTypeDisplayName(Internal::IarToolChain::tr("IAREW"));
-}
-
-void IarToolChain::setTargetAbi(const Abi &abi)
-{
-    if (abi == m_targetAbi)
-        return;
-    m_targetAbi = abi;
-    toolChainUpdated();
-}
-
-Abi IarToolChain::targetAbi() const
-{
-    return m_targetAbi;
-}
-
-bool IarToolChain::isValid() const
-{
-    return true;
+    setTargetAbiKey("TargetAbi");
+    setCompilerCommandKey("CompilerPath");
 }
 
 ToolChain::MacroInspectionRunner IarToolChain::createMacroInspectionRunner() const
@@ -319,16 +299,16 @@ ToolChain::MacroInspectionRunner IarToolChain::createMacroInspectionRunner() con
     Environment env = Environment::systemEnvironment();
     addToEnvironment(env);
 
-    const Utils::FilePath compilerCommand = m_compilerCommand;
-    const Utils::Id languageId = language();
+    const FilePath compiler = compilerCommand();
+    const Id languageId = language();
     const QStringList extraArgs = m_extraCodeModelFlags;
     MacrosCache macrosCache = predefinedMacrosCache();
 
-    return [env, compilerCommand, extraArgs, macrosCache, languageId]
+    return [env, compiler, extraArgs, macrosCache, languageId]
             (const QStringList &flags) {
         Q_UNUSED(flags)
 
-        Macros macros = dumpPredefinedMacros(compilerCommand, extraArgs, languageId, env.toStringList());
+        Macros macros = dumpPredefinedMacros(compiler, extraArgs, languageId, env);
         macros.append({"__intrinsic", "", MacroType::Define});
         macros.append({"__nounwind", "", MacroType::Define});
         macros.append({"__noreturn", "", MacroType::Define});
@@ -361,18 +341,18 @@ ToolChain::BuiltInHeaderPathsRunner IarToolChain::createBuiltInHeaderPathsRunner
     Environment env = Environment::systemEnvironment();
     addToEnvironment(env);
 
-    const Utils::FilePath compilerCommand = m_compilerCommand;
-    const Utils::Id languageId = language();
+    const FilePath compiler = compilerCommand();
+    const Id languageId = language();
 
     HeaderPathsCache headerPaths = headerPathsCache();
 
-    return [env, compilerCommand, headerPaths, languageId](const QStringList &flags,
-                                                                const QString &fileName,
-                                                                const QString &) {
+    return [env, compiler, headerPaths, languageId](const QStringList &flags,
+                                                    const QString &fileName,
+                                                    const QString &) {
         Q_UNUSED(flags)
         Q_UNUSED(fileName)
 
-        const HeaderPaths paths = dumpHeaderPaths(compilerCommand, languageId, env.toStringList());
+        const HeaderPaths paths = dumpHeaderPaths(compiler, languageId, env);
         headerPaths->insert({}, paths);
 
         return paths;
@@ -381,8 +361,8 @@ ToolChain::BuiltInHeaderPathsRunner IarToolChain::createBuiltInHeaderPathsRunner
 
 void IarToolChain::addToEnvironment(Environment &env) const
 {
-    if (!m_compilerCommand.isEmpty()) {
-        const FilePath path = m_compilerCommand.parentDir();
+    if (!compilerCommand().isEmpty()) {
+        const FilePath path = compilerCommand().parentDir();
         env.prependOrSetPath(path.toString());
     }
 }
@@ -395,9 +375,7 @@ QList<Utils::OutputLineParser *> IarToolChain::createOutputParsers() const
 QVariantMap IarToolChain::toMap() const
 {
     QVariantMap data = ToolChain::toMap();
-    data.insert(compilerCommandKeyC, m_compilerCommand.toString());
     data.insert(compilerPlatformCodeGenFlagsKeyC, m_extraCodeModelFlags);
-    data.insert(targetAbiKeyC, m_targetAbi.toString());
     return data;
 }
 
@@ -405,9 +383,7 @@ bool IarToolChain::fromMap(const QVariantMap &data)
 {
     if (!ToolChain::fromMap(data))
         return false;
-    m_compilerCommand = FilePath::fromString(data.value(compilerCommandKeyC).toString());
     m_extraCodeModelFlags = data.value(compilerPlatformCodeGenFlagsKeyC).toStringList();
-    m_targetAbi = Abi::fromString(data.value(targetAbiKeyC).toString());
     return true;
 }
 
@@ -422,23 +398,8 @@ bool IarToolChain::operator==(const ToolChain &other) const
         return false;
 
     const auto customTc = static_cast<const IarToolChain *>(&other);
-    return m_compilerCommand == customTc->m_compilerCommand
-            && m_targetAbi == customTc->m_targetAbi
-            && m_extraCodeModelFlags == customTc->m_extraCodeModelFlags
-            ;
-}
-
-void IarToolChain::setCompilerCommand(const FilePath &file)
-{
-    if (file == m_compilerCommand)
-        return;
-    m_compilerCommand = file;
-    toolChainUpdated();
-}
-
-FilePath IarToolChain::compilerCommand() const
-{
-    return m_compilerCommand;
+    return compilerCommand() == customTc->compilerCommand()
+            && m_extraCodeModelFlags == customTc->m_extraCodeModelFlags;
 }
 
 void IarToolChain::setExtraCodeModelFlags(const QStringList &flags)
@@ -472,8 +433,10 @@ IarToolChainFactory::IarToolChainFactory()
     setUserCreatable(true);
 }
 
-QList<ToolChain *> IarToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown)
+QList<ToolChain *> IarToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown,
+                                                   const IDevice::Ptr &device)
 {
+    Q_UNUSED(device);
     Candidates candidates;
 
 #ifdef Q_OS_WIN
@@ -574,8 +537,7 @@ QList<ToolChain *> IarToolChainFactory::autoDetectToolchain(
         const Candidate &candidate, Utils::Id languageId) const
 {
     const auto env = Environment::systemEnvironment();
-    const Macros macros = dumpPredefinedMacros(candidate.compilerPath, {}, languageId,
-                                               env.toStringList());
+    const Macros macros = dumpPredefinedMacros(candidate.compilerPath, {}, languageId, env);
     if (macros.isEmpty())
         return {};
     const Abi abi = guessAbi(macros);
@@ -604,7 +566,7 @@ IarToolChainConfigWidget::IarToolChainConfigWidget(IarToolChain *tc) :
     m_compilerCommand->setHistoryCompleter("PE.IAREW.Command.History");
     m_mainLayout->addRow(tr("&Compiler path:"), m_compilerCommand);
     m_platformCodeGenFlagsLineEdit = new QLineEdit(this);
-    m_platformCodeGenFlagsLineEdit->setText(QtcProcess::joinArgs(tc->extraCodeModelFlags()));
+    m_platformCodeGenFlagsLineEdit->setText(ProcessArgs::joinArgs(tc->extraCodeModelFlags()));
     m_mainLayout->addRow(tr("Platform codegen flags:"), m_platformCodeGenFlagsLineEdit);
     m_mainLayout->addRow(tr("&ABI:"), m_abiWidget);
 
@@ -646,7 +608,7 @@ bool IarToolChainConfigWidget::isDirtyImpl() const
 {
     const auto tc = static_cast<IarToolChain *>(toolChain());
     return m_compilerCommand->filePath() != tc->compilerCommand()
-            || m_platformCodeGenFlagsLineEdit->text() != QtcProcess::joinArgs(tc->extraCodeModelFlags())
+            || m_platformCodeGenFlagsLineEdit->text() != ProcessArgs::joinArgs(tc->extraCodeModelFlags())
             || m_abiWidget->currentAbi() != tc->targetAbi()
             ;
 }
@@ -663,7 +625,7 @@ void IarToolChainConfigWidget::setFromToolchain()
     const QSignalBlocker blocker(this);
     const auto tc = static_cast<IarToolChain *>(toolChain());
     m_compilerCommand->setFilePath(tc->compilerCommand());
-    m_platformCodeGenFlagsLineEdit->setText(QtcProcess::joinArgs(tc->extraCodeModelFlags()));
+    m_platformCodeGenFlagsLineEdit->setText(ProcessArgs::joinArgs(tc->extraCodeModelFlags()));
     m_abiWidget->setAbis({}, tc->targetAbi());
     const bool haveCompiler = compilerExists(m_compilerCommand->filePath());
     m_abiWidget->setEnabled(haveCompiler && !tc->isAutoDetected());
@@ -676,9 +638,8 @@ void IarToolChainConfigWidget::handleCompilerCommandChange()
     if (haveCompiler) {
         const auto env = Environment::systemEnvironment();
         const QStringList extraArgs = splitString(m_platformCodeGenFlagsLineEdit->text());
-        const auto languageId = toolChain()->language();
-        m_macros = dumpPredefinedMacros(compilerPath, extraArgs, languageId,
-                                        env.toStringList());
+        const Id languageId = toolChain()->language();
+        m_macros = dumpPredefinedMacros(compilerPath, extraArgs, languageId, env);
         const Abi guessed = guessAbi(m_macros);
         m_abiWidget->setAbis({}, guessed);
     }
@@ -690,7 +651,7 @@ void IarToolChainConfigWidget::handleCompilerCommandChange()
 void IarToolChainConfigWidget::handlePlatformCodeGenFlagsChange()
 {
     const QString str1 = m_platformCodeGenFlagsLineEdit->text();
-    const QString str2 = QtcProcess::joinArgs(splitString(str1));
+    const QString str2 = ProcessArgs::joinArgs(splitString(str1));
     if (str1 != str2)
         m_platformCodeGenFlagsLineEdit->setText(str2);
     else
