@@ -38,7 +38,6 @@
 #include <utils/pathchooser.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
-#include <utils/synchronousprocess.h>
 
 #include <QDebug>
 #include <QDir>
@@ -60,9 +59,7 @@ namespace Internal {
 
 // Helpers:
 
-static const char compilerCommandKeyC[] = "CompilerPath";
 static const char compilerPlatformCodeGenFlagsKeyC[] = "PlatformCodeGenFlags";
-static const char targetAbiKeyC[] = "TargetAbi";
 
 static bool compilerExists(const FilePath &compilerPath)
 {
@@ -85,7 +82,7 @@ static Abi::Architecture guessArchitecture(const FilePath &compilerPath)
     return Abi::Architecture::UnknownArchitecture;
 }
 
-static Macros dumpMcsPredefinedMacros(const FilePath &compiler, const QStringList &env)
+static Macros dumpMcsPredefinedMacros(const FilePath &compiler, const Environment &env)
 {
     // Note: The KEIL C51 or C251 compiler does not support the predefined
     // macros dumping. So, we do it with the following trick, where we try
@@ -138,13 +135,13 @@ static Macros dumpMcsPredefinedMacros(const FilePath &compiler, const QStringLis
 
     fakeIn.close();
 
-    SynchronousProcess cpp;
+    QtcProcess cpp;
     cpp.setEnvironment(env);
     cpp.setTimeoutS(10);
+    cpp.setCommand({compiler, {fakeIn.fileName()}});
 
-    const CommandLine cmd(compiler, {fakeIn.fileName()});
-    const SynchronousProcessResponse response = cpp.runBlocking(cmd);
-    QString output = response.allOutput();
+    cpp.runBlocking();
+    QString output = cpp.allOutput();
     Macros macros;
     QTextStream stream(&output);
     QString line;
@@ -158,7 +155,7 @@ static Macros dumpMcsPredefinedMacros(const FilePath &compiler, const QStringLis
     return macros;
 }
 
-static Macros dumpC166PredefinedMacros(const FilePath &compiler, const QStringList &env)
+static Macros dumpC166PredefinedMacros(const FilePath &compiler, const Environment &env)
 {
     // Note: The KEIL C166 compiler does not support the predefined
     // macros dumping. Also, it does not support the '#pragma' and
@@ -215,7 +212,7 @@ static Macros dumpC166PredefinedMacros(const FilePath &compiler, const QStringLi
 
     fakeIn.close();
 
-    SynchronousProcess cpp;
+    QtcProcess cpp;
     cpp.setEnvironment(env);
     cpp.setTimeoutS(10);
 
@@ -271,32 +268,31 @@ static Macros dumpC166PredefinedMacros(const FilePath &compiler, const QStringLi
         }
     };
 
-    const CommandLine cmd(compiler, {fakeIn.fileName()});
-    const SynchronousProcessResponse response = cpp.runBlocking(cmd);
-    const QString output = response.allOutput();
+    cpp.setCommand({compiler, {fakeIn.fileName()}});
+    cpp.runBlocking();
+    const QString output = cpp.allOutput();
     extractMacros(output);
     return macros;
 }
 
-static Macros dumpArmPredefinedMacros(const FilePath &compiler, const QStringList &extraArgs, const QStringList &env)
+static Macros dumpArmPredefinedMacros(const FilePath &compiler, const QStringList &extraArgs, const Environment &env)
 {
-    SynchronousProcess cpp;
+    QtcProcess cpp;
     cpp.setEnvironment(env);
     cpp.setTimeoutS(10);
 
     QStringList args = extraArgs;
     args.push_back("-E");
     args.push_back("--list-macros");
-    const CommandLine cmd(compiler, args);
+    cpp.setCommand({compiler, args});
 
-    const SynchronousProcessResponse response = cpp.runBlocking(cmd);
-    if (response.result != SynchronousProcessResponse::Finished
-            || response.exitCode != 0) {
-        qWarning() << response.exitMessage(compiler.toString(), 10);
+    cpp.runBlocking();
+    if (cpp.result() != QtcProcess::FinishedWithSuccess) {
+        qWarning() << cpp.exitMessage();
         return {};
     }
 
-    const QByteArray output = response.allOutput().toUtf8();
+    const QByteArray output = cpp.allOutput().toUtf8();
     return Macro::toMacros(output);
 }
 
@@ -316,7 +312,7 @@ static bool isArmArchitecture(Abi::Architecture arch)
     return arch == Abi::Architecture::ArmArchitecture;
 }
 
-static Macros dumpPredefinedMacros(const FilePath &compiler, const QStringList &args, const QStringList &env)
+static Macros dumpPredefinedMacros(const FilePath &compiler, const QStringList &args, const Environment &env)
 {
     if (compiler.isEmpty() || !compiler.toFileInfo().isExecutable())
         return {};
@@ -346,11 +342,11 @@ static HeaderPaths dumpHeaderPaths(const FilePath &compiler)
     if (isMcsArchitecture(arch) || isC166Architecture(arch)) {
         QDir includeDir(toolkitDir);
         if (includeDir.cd("inc"))
-            headerPaths.push_back({includeDir.canonicalPath(), HeaderPathType::BuiltIn});
+            headerPaths.push_back(HeaderPath::makeBuiltIn(includeDir.canonicalPath()));
     } else if (isArmArchitecture(arch)) {
         QDir includeDir(toolkitDir);
         if (includeDir.cd("include"))
-            headerPaths.push_back({includeDir.canonicalPath(), HeaderPathType::BuiltIn});
+            headerPaths.push_back(HeaderPath::makeBuiltIn(includeDir.canonicalPath()));
     }
 
     return headerPaths;
@@ -433,24 +429,8 @@ KeilToolChain::KeilToolChain() :
     ToolChain(Constants::KEIL_TOOLCHAIN_TYPEID)
 {
     setTypeDisplayName(tr("KEIL"));
-}
-
-void KeilToolChain::setTargetAbi(const Abi &abi)
-{
-    if (abi == m_targetAbi)
-        return;
-    m_targetAbi = abi;
-    toolChainUpdated();
-}
-
-Abi KeilToolChain::targetAbi() const
-{
-    return m_targetAbi;
-}
-
-bool KeilToolChain::isValid() const
-{
-    return true;
+    setTargetAbiKey("TargetAbi");
+    setCompilerCommandKey("CompilerPath");
 }
 
 ToolChain::MacroInspectionRunner KeilToolChain::createMacroInspectionRunner() const
@@ -458,17 +438,16 @@ ToolChain::MacroInspectionRunner KeilToolChain::createMacroInspectionRunner() co
     Environment env = Environment::systemEnvironment();
     addToEnvironment(env);
 
-    const Utils::FilePath compilerCommand = m_compilerCommand;
-    const Utils::Id lang = language();
+    const FilePath compiler = compilerCommand();
+    const Id lang = language();
 
     MacrosCache macroCache = predefinedMacrosCache();
     const QStringList extraArgs = m_extraCodeModelFlags;
 
-    return [env, compilerCommand, extraArgs, macroCache, lang]
-            (const QStringList &flags) {
+    return [env, compiler, extraArgs, macroCache, lang](const QStringList &flags) {
         Q_UNUSED(flags)
 
-        const Macros macros = dumpPredefinedMacros(compilerCommand, extraArgs, env.toStringList());
+        const Macros macros = dumpPredefinedMacros(compiler, extraArgs, env);
         const auto report = MacroInspectionReport{macros, languageVersion(lang, macros)};
         macroCache->insert({}, report);
 
@@ -490,16 +469,15 @@ WarningFlags KeilToolChain::warningFlags(const QStringList &cxxflags) const
 ToolChain::BuiltInHeaderPathsRunner KeilToolChain::createBuiltInHeaderPathsRunner(
         const Environment &) const
 {
-    const Utils::FilePath compilerCommand = m_compilerCommand;
+    const FilePath compiler = compilerCommand();
+    const HeaderPathsCache headerPaths = headerPathsCache();
 
-    HeaderPathsCache headerPaths = headerPathsCache();
-
-    return [compilerCommand,
+    return [compiler,
             headerPaths](const QStringList &flags, const QString &fileName, const QString &) {
         Q_UNUSED(flags)
         Q_UNUSED(fileName)
 
-        const HeaderPaths paths = dumpHeaderPaths(compilerCommand);
+        const HeaderPaths paths = dumpHeaderPaths(compiler);
         headerPaths->insert({}, paths);
 
         return paths;
@@ -508,8 +486,8 @@ ToolChain::BuiltInHeaderPathsRunner KeilToolChain::createBuiltInHeaderPathsRunne
 
 void KeilToolChain::addToEnvironment(Environment &env) const
 {
-    if (!m_compilerCommand.isEmpty()) {
-        const FilePath path = m_compilerCommand.parentDir();
+    if (!compilerCommand().isEmpty()) {
+        const FilePath path = compilerCommand().parentDir();
         env.prependOrSetPath(path.toString());
     }
 }
@@ -522,9 +500,7 @@ QList<OutputLineParser *> KeilToolChain::createOutputParsers() const
 QVariantMap KeilToolChain::toMap() const
 {
     QVariantMap data = ToolChain::toMap();
-    data.insert(compilerCommandKeyC, m_compilerCommand.toString());
     data.insert(compilerPlatformCodeGenFlagsKeyC, m_extraCodeModelFlags);
-    data.insert(targetAbiKeyC, m_targetAbi.toString());
     return data;
 }
 
@@ -532,9 +508,7 @@ bool KeilToolChain::fromMap(const QVariantMap &data)
 {
     if (!ToolChain::fromMap(data))
         return false;
-    m_compilerCommand = FilePath::fromString(data.value(compilerCommandKeyC).toString());
     m_extraCodeModelFlags = data.value(compilerPlatformCodeGenFlagsKeyC).toStringList();
-    m_targetAbi = Abi::fromString(data.value(targetAbiKeyC).toString());
     return true;
 }
 
@@ -549,23 +523,9 @@ bool KeilToolChain::operator ==(const ToolChain &other) const
         return false;
 
     const auto customTc = static_cast<const KeilToolChain *>(&other);
-    return m_compilerCommand == customTc->m_compilerCommand
-            && m_targetAbi == customTc->m_targetAbi
-            && m_extraCodeModelFlags == customTc->m_extraCodeModelFlags
-            ;
-}
-
-void KeilToolChain::setCompilerCommand(const FilePath &file)
-{
-    if (file == m_compilerCommand)
-        return;
-    m_compilerCommand = file;
-    toolChainUpdated();
-}
-
-FilePath KeilToolChain::compilerCommand() const
-{
-    return m_compilerCommand;
+    return compilerCommand() == customTc->compilerCommand()
+            && targetAbi() == customTc->targetAbi()
+            && m_extraCodeModelFlags == customTc->m_extraCodeModelFlags;
 }
 
 void KeilToolChain::setExtraCodeModelFlags(const QStringList &flags)
@@ -643,8 +603,10 @@ static QString extractVersion(const QString &toolsFile, const QString &section)
     return {};
 }
 
-QList<ToolChain *> KeilToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown)
+QList<ToolChain *> KeilToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown,
+                                                    const IDevice::Ptr &device)
 {
+    Q_UNUSED(device)
 #ifdef Q_OS_WIN64
     static const char kRegistryNode[] = "HKEY_LOCAL_MACHINE\\SOFTWARE\\WOW6432Node\\Microsoft\\" \
                                         "Windows\\CurrentVersion\\Uninstall\\Keil µVision4";
@@ -728,7 +690,7 @@ QList<ToolChain *> KeilToolChainFactory::autoDetectToolchain(
 
     QStringList extraArgs;
     addDefaultCpuArgs(candidate.compilerPath, extraArgs);
-    const Macros macros = dumpPredefinedMacros(candidate.compilerPath, extraArgs, env.toStringList());
+    const Macros macros = dumpPredefinedMacros(candidate.compilerPath, extraArgs, env);
     if (macros.isEmpty())
         return {};
 
@@ -764,7 +726,7 @@ KeilToolChainConfigWidget::KeilToolChainConfigWidget(KeilToolChain *tc) :
     m_compilerCommand->setHistoryCompleter("PE.KEIL.Command.History");
     m_mainLayout->addRow(tr("&Compiler path:"), m_compilerCommand);
     m_platformCodeGenFlagsLineEdit = new QLineEdit(this);
-    m_platformCodeGenFlagsLineEdit->setText(QtcProcess::joinArgs(tc->extraCodeModelFlags()));
+    m_platformCodeGenFlagsLineEdit->setText(ProcessArgs::joinArgs(tc->extraCodeModelFlags()));
     m_mainLayout->addRow(tr("Platform codegen flags:"), m_platformCodeGenFlagsLineEdit);
     m_mainLayout->addRow(tr("&ABI:"), m_abiWidget);
 
@@ -806,7 +768,7 @@ bool KeilToolChainConfigWidget::isDirtyImpl() const
 {
     const auto tc = static_cast<KeilToolChain *>(toolChain());
     return m_compilerCommand->filePath() != tc->compilerCommand()
-            || m_platformCodeGenFlagsLineEdit->text() != QtcProcess::joinArgs(tc->extraCodeModelFlags())
+            || m_platformCodeGenFlagsLineEdit->text() != ProcessArgs::joinArgs(tc->extraCodeModelFlags())
             || m_abiWidget->currentAbi() != tc->targetAbi()
             ;
 }
@@ -823,7 +785,7 @@ void KeilToolChainConfigWidget::setFromToolChain()
     const QSignalBlocker blocker(this);
     const auto tc = static_cast<KeilToolChain *>(toolChain());
     m_compilerCommand->setFilePath(tc->compilerCommand());
-    m_platformCodeGenFlagsLineEdit->setText(QtcProcess::joinArgs(tc->extraCodeModelFlags()));
+    m_platformCodeGenFlagsLineEdit->setText(ProcessArgs::joinArgs(tc->extraCodeModelFlags()));
     m_abiWidget->setAbis({}, tc->targetAbi());
     const bool haveCompiler = compilerExists(m_compilerCommand->filePath());
     m_abiWidget->setEnabled(haveCompiler && !tc->isAutoDetected());
@@ -839,8 +801,8 @@ void KeilToolChainConfigWidget::handleCompilerCommandChange()
         QStringList newExtraArgs = prevExtraArgs;
         addDefaultCpuArgs(compilerPath, newExtraArgs);
         if (prevExtraArgs != newExtraArgs)
-            m_platformCodeGenFlagsLineEdit->setText(QtcProcess::joinArgs(newExtraArgs));
-        m_macros = dumpPredefinedMacros(compilerPath, newExtraArgs, env.toStringList());
+            m_platformCodeGenFlagsLineEdit->setText(ProcessArgs::joinArgs(newExtraArgs));
+        m_macros = dumpPredefinedMacros(compilerPath, newExtraArgs, env);
         const Abi guessed = guessAbi(m_macros);
         m_abiWidget->setAbis({}, guessed);
     }
@@ -852,7 +814,7 @@ void KeilToolChainConfigWidget::handleCompilerCommandChange()
 void KeilToolChainConfigWidget::handlePlatformCodeGenFlagsChange()
 {
     const QString str1 = m_platformCodeGenFlagsLineEdit->text();
-    const QString str2 = QtcProcess::joinArgs(splitString(str1));
+    const QString str2 = ProcessArgs::joinArgs(splitString(str1));
     if (str1 != str2)
         m_platformCodeGenFlagsLineEdit->setText(str2);
     else
