@@ -25,23 +25,32 @@
 
 #pragma once
 
+#include <coreplugin/dialogs/ioptionspage.h>
+
 #include <vcsbase/vcsbaseclientsettings.h>
 
 namespace Bazaar {
 namespace Internal {
 
-class BazaarSettings : public VcsBase::VcsBaseClientSettings
+class BazaarSettings final : public VcsBase::VcsBaseSettings
+{
+    Q_DECLARE_TR_FUNCTIONS(Bazaar::Internal::BazaarSettings)
+
+public:
+    BazaarSettings();
+
+    Utils::BoolAspect diffIgnoreWhiteSpace;
+    Utils::BoolAspect diffIgnoreBlankLines;
+    Utils::BoolAspect logVerbose;
+    Utils::BoolAspect logForward;
+    Utils::BoolAspect logIncludeMerges;
+    Utils::StringAspect logFormat;
+};
+
+class BazaarSettingsPage final : public Core::IOptionsPage
 {
 public:
-    static const QLatin1String diffIgnoreWhiteSpaceKey;
-    static const QLatin1String diffIgnoreBlankLinesKey;
-    static const QLatin1String logVerboseKey;
-    static const QLatin1String logForwardKey;
-    static const QLatin1String logIncludeMergesKey;
-    static const QLatin1String logFormatKey;
-
-    BazaarSettings();
-    bool sameUserId(const BazaarSettings &other) const;
+    explicit BazaarSettingsPage(BazaarSettings *settings);
 };
 
 } // namespace Internal
