@@ -40,13 +40,16 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/idocument.h>
-#include <cppeditor/cppeditorconstants.h>
+
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projecttree.h>
+
 #include <texteditor/formattexteditor.h>
+
 #include <utils/fileutils.h>
 #include <utils/hostosinfo.h>
+#include <utils/stringutils.h>
 
 #include <QAction>
 #include <QMenu>
@@ -55,29 +58,25 @@ using namespace TextEditor;
 
 namespace Beautifier {
 namespace Internal {
-namespace ArtisticStyle {
 
 ArtisticStyle::ArtisticStyle()
 {
-    Core::ActionContainer *menu = Core::ActionManager::createMenu(Constants::ArtisticStyle::MENU_ID);
+    Core::ActionContainer *menu = Core::ActionManager::createMenu("ArtisticStyle.Menu");
     menu->menu()->setTitle(tr("&Artistic Style"));
 
     m_formatFile = new QAction(BeautifierPlugin::msgFormatCurrentFile(), this);
-    menu->addAction(Core::ActionManager::registerAction(m_formatFile,
-                                                        Constants::ArtisticStyle::ACTION_FORMATFILE));
+    menu->addAction(Core::ActionManager::registerAction(m_formatFile, "ArtisticStyle.FormatFile"));
     connect(m_formatFile, &QAction::triggered, this, &ArtisticStyle::formatFile);
 
     Core::ActionManager::actionContainer(Constants::MENU_ID)->addMenu(menu);
 
     connect(&m_settings, &ArtisticStyleSettings::supportedMimeTypesChanged,
             [this] { updateActions(Core::EditorManager::currentEditor()); });
-
-    new ArtisticStyleOptionsPage(&m_settings, this);
 }
 
 QString ArtisticStyle::id() const
 {
-    return QLatin1String(Constants::ArtisticStyle::DISPLAY_NAME);
+    return QLatin1String(Constants::ARTISTICSTYLE_DISPLAY_NAME);
 }
 
 void ArtisticStyle::updateActions(Core::IEditor *editor)
@@ -90,7 +89,7 @@ void ArtisticStyle::formatFile()
     const QString cfgFileName = configurationFile();
     if (cfgFileName.isEmpty()) {
         BeautifierPlugin::showError(BeautifierPlugin::msgCannotGetConfigurationFile(
-                                        tr(Constants::ArtisticStyle::DISPLAY_NAME)));
+                                        tr(Constants::ARTISTICSTYLE_DISPLAY_NAME)));
     } else {
         formatCurrentFile(command(cfgFileName));
     }
@@ -104,7 +103,7 @@ QString ArtisticStyle::configurationFile() const
     if (m_settings.useOtherFiles()) {
         if (const ProjectExplorer::Project *project
                 = ProjectExplorer::ProjectTree::currentProject()) {
-            const Utils::FilePathList astyleRcfiles = project->files(
+            const Utils::FilePaths astyleRcfiles = project->files(
                 [](const ProjectExplorer::Node *n) { return n->filePath().endsWith(".astylerc"); });
             for (const Utils::FilePath &file : astyleRcfiles) {
                 const QFileInfo fi = file.toFileInfo();
@@ -165,6 +164,5 @@ Command ArtisticStyle::command(const QString &cfgFile) const
     return command;
 }
 
-} // namespace ArtisticStyle
 } // namespace Internal
 } // namespace Beautifier
