@@ -53,8 +53,8 @@
 #include <utils/fileutils.h>
 #include <utils/mimetypes/mimedatabase.h>
 #include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
 #include <utils/runextensions.h>
-#include <utils/synchronousprocess.h>
 #include <utils/temporarydirectory.h>
 #include <utils/textutils.h>
 
@@ -62,7 +62,6 @@
 #include <QFileInfo>
 #include <QMenu>
 #include <QPlainTextEdit>
-#include <QProcess>
 #include <QScrollBar>
 #include <QTextBlock>
 
@@ -133,6 +132,13 @@ void BeautifierPlugin::extensionsInitialized()
     dd = new BeautifierPluginPrivate;
 }
 
+ExtensionSystem::IPlugin::ShutdownFlag BeautifierPlugin::aboutToShutdown()
+{
+    delete dd;
+    dd = nullptr;
+    return SynchronousShutdown;
+}
+
 BeautifierPluginPrivate::BeautifierPluginPrivate()
 {
     updateActions();
@@ -191,7 +197,7 @@ void BeautifierPluginPrivate::autoFormatOnSave(Core::IDocument *document)
 
 void BeautifierPlugin::showError(const QString &error)
 {
-    Core::MessageManager::write(tr("Error in Beautifier: %1").arg(error.trimmed()));
+    Core::MessageManager::writeFlashing(tr("Error in Beautifier: %1").arg(error.trimmed()));
 }
 
 QString BeautifierPlugin::msgCannotGetConfigurationFile(const QString &command)
