@@ -31,7 +31,7 @@
 
 #include <coreplugin/icore.h>
 
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 
 #include <QDateTime>
 #include <QFile>
@@ -41,9 +41,7 @@
 
 namespace Beautifier {
 namespace Internal {
-namespace Uncrustify {
 
-namespace {
 const char USE_OTHER_FILES[]               = "useOtherFiles";
 const char USE_HOME_FILE[]                 = "useHomeFile";
 const char USE_SPECIFIC_CONFIG_FILE_PATH[] = "useSpecificConfigFile";
@@ -51,12 +49,12 @@ const char SPECIFIC_CONFIG_FILE_PATH[]     = "specificConfigFile";
 const char USE_CUSTOM_STYLE[]              = "useCustomStyle";
 const char CUSTOM_STYLE[]                  = "customStyle";
 const char FORMAT_ENTIRE_FILE_FALLBACK[]   = "formatEntireFileFallback";
-}
+const char SETTINGS_NAME[]                 = "uncrustify";
 
 UncrustifySettings::UncrustifySettings() :
-    AbstractSettings(Constants::Uncrustify::SETTINGS_NAME, ".cfg")
+    AbstractSettings(SETTINGS_NAME, ".cfg")
 {
-    connect(&m_versionProcess, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
+    connect(&m_versionProcess, &Utils::QtcProcess::finished,
             this, &UncrustifySettings::parseVersionProcessResult);
 
     setCommand("uncrustify");
@@ -142,18 +140,19 @@ void UncrustifySettings::setFormatEntireFileFallback(bool formatEntireFileFallba
 
 QString UncrustifySettings::documentationFilePath() const
 {
-    return Core::ICore::userResourcePath() + '/' + Beautifier::Constants::SETTINGS_DIRNAME + '/'
-            + Beautifier::Constants::DOCUMENTATION_DIRNAME + '/'
-            + Constants::Uncrustify::SETTINGS_NAME + ".xml";
+    return (Core::ICore::userResourcePath() / Beautifier::Constants::SETTINGS_DIRNAME
+                / Beautifier::Constants::DOCUMENTATION_DIRNAME / SETTINGS_NAME
+            + ".xml")
+        .toString();
 }
 
 void UncrustifySettings::createDocumentationFile() const
 {
-    Utils::SynchronousProcess process;
+    Utils::QtcProcess process;
     process.setTimeoutS(2);
-    Utils::SynchronousProcessResponse response
-            = process.runBlocking({command(), {"--show-config"}});
-    if (response.result != Utils::SynchronousProcessResponse::Finished)
+    process.setCommand({command(), {"--show-config"}});
+    process.runBlocking();
+    if (process.result() != Utils::QtcProcess::FinishedWithSuccess)
         return;
 
     QFile file(documentationFilePath());
@@ -170,7 +169,7 @@ void UncrustifySettings::createDocumentationFile() const
     stream.writeComment("Created " + QDateTime::currentDateTime().toString(Qt::ISODate));
     stream.writeStartElement(Constants::DOCUMENTATION_XMLROOT);
 
-    const QStringList lines = response.allOutput().split(QLatin1Char('\n'));
+    const QStringList lines = process.allOutput().split(QLatin1Char('\n'));
     const int totalLines = lines.count();
     for (int i = 0; i < totalLines; ++i) {
         const QString &line = lines.at(i);
@@ -230,19 +229,18 @@ void UncrustifySettings::updateVersion()
         m_versionProcess.kill();
         m_versionProcess.waitForFinished();
     }
-    m_versionProcess.start(command().toString(), {"--version"});
+    m_versionProcess.setCommand({ command(), { "--version" } });
+    m_versionProcess.start();
 }
 
-void UncrustifySettings::parseVersionProcessResult(int exitCode, QProcess::ExitStatus exitStatus)
+void UncrustifySettings::parseVersionProcessResult()
 {
-    Q_UNUSED(exitCode)
-    if (exitStatus != QProcess::NormalExit)
+    if (m_versionProcess.exitStatus() != QProcess::NormalExit)
         return;
 
     if (!parseVersion(QString::fromUtf8(m_versionProcess.readAllStandardOutput()), m_version))
         parseVersion(QString::fromUtf8(m_versionProcess.readAllStandardError()), m_version);
 }
 
-} // namespace Uncrustify
 } // namespace Internal
 } // namespace Beautifier
