@@ -230,61 +230,62 @@ public:
         return type == TypeRemoved ? BehaviorSilent : IDocument::reloadBehavior(state, type);
     }
 
-    bool save(QString *errorString, const QString &fn, bool autoSave) override
+    bool save(QString *errorString, const Utils::FilePath &filePath, bool autoSave) override
     {
         QTC_ASSERT(!autoSave, return true); // bineditor does not support autosave - it would be a bit expensive
-        const FilePath fileNameToUse = fn.isEmpty() ? filePath() : FilePath::fromString(fn);
-        if (m_widget->save(errorString, filePath().toString(), fileNameToUse.toString())) {
+        const FilePath &fileNameToUse = filePath.isEmpty() ? this->filePath() : filePath;
+        if (m_widget->save(errorString, this->filePath(), fileNameToUse)) {
             setFilePath(fileNameToUse);
             return true;
-        } else {
-            return false;
         }
+        return false;
     }
 
-    OpenResult open(QString *errorString, const QString &fileName,
-                    const QString &realFileName) override
+    OpenResult open(QString *errorString, const FilePath &filePath,
+                    const FilePath &realFilePath) override
     {
-        QTC_CHECK(fileName == realFileName); // The bineditor can do no autosaving
-        return openImpl(errorString, fileName);
+        QTC_CHECK(filePath == realFilePath); // The bineditor can do no autosaving
+        return openImpl(errorString, filePath);
     }
 
-    OpenResult openImpl(QString *errorString, const QString &fileName, quint64 offset = 0)
+    OpenResult openImpl(QString *errorString, const FilePath &filePath, quint64 offset = 0)
     {
-        QFile file(fileName);
-        if (file.open(QIODevice::ReadOnly)) {
-            file.close();
-            quint64 size = static_cast<quint64>(file.size());
-            if (size == 0) {
-                QString msg = tr("The Binary Editor cannot open empty files.");
-                if (errorString)
-                    *errorString = msg;
-                else
-                    QMessageBox::critical(ICore::dialogParent(), tr("File Error"), msg);
-                return OpenResult::CannotHandle;
-            }
-            if (size / 16 >= qint64(1) << 31) {
-                // The limit is 2^31 lines (due to QText* interfaces) * 16 bytes per line.
-                QString msg = tr("The file is too big for the Binary Editor (max. 32GB).");
-                if (errorString)
-                    *errorString = msg;
-                else
-                    QMessageBox::critical(ICore::dialogParent(), tr("File Error"), msg);
-                return OpenResult::CannotHandle;
-            }
-            if (offset >= size)
-                return OpenResult::CannotHandle;
-            setFilePath(FilePath::fromString(fileName));
-            m_widget->setSizes(offset, file.size());
-            return OpenResult::Success;
+        const qint64 size = filePath.fileSize();
+        if (size < 0) {
+            QString msg = tr("Cannot open %1: %2").arg(filePath.toUserOutput(), tr("File Error"));
+            // FIXME: Was: file.errorString(), but we don't have a file anymore.
+            if (errorString)
+                *errorString = msg;
+            else
+                QMessageBox::critical(ICore::dialogParent(), tr("File Error"), msg);
+            return OpenResult::ReadError;
         }
-        QString errStr = tr("Cannot open %1: %2").arg(
-                QDir::toNativeSeparators(fileName), file.errorString());
-        if (errorString)
-            *errorString = errStr;
-        else
-            QMessageBox::critical(ICore::dialogParent(), tr("File Error"), errStr);
-        return OpenResult::ReadError;
+
+        if (size == 0) {
+            QString msg = tr("The Binary Editor cannot open empty files.");
+            if (errorString)
+                *errorString = msg;
+            else
+                QMessageBox::critical(ICore::dialogParent(), tr("File Error"), msg);
+            return OpenResult::CannotHandle;
+        }
+
+        if (size / 16 >= qint64(1) << 31) {
+            // The limit is 2^31 lines (due to QText* interfaces) * 16 bytes per line.
+            QString msg = tr("The file is too big for the Binary Editor (max. 32GB).");
+            if (errorString)
+                *errorString = msg;
+            else
+                QMessageBox::critical(ICore::dialogParent(), tr("File Error"), msg);
+            return OpenResult::CannotHandle;
+        }
+
+        if (offset >= quint64(size))
+            return OpenResult::CannotHandle;
+
+        setFilePath(filePath);
+        m_widget->setSizes(offset, size);
+        return OpenResult::Success;
     }
 
     void provideData(quint64 address)
@@ -292,27 +293,21 @@ public:
         const FilePath fn = filePath();
         if (fn.isEmpty())
             return;
-        QFile file(fn.toString());
-        if (file.open(QIODevice::ReadOnly)) {
-            int blockSize = m_widget->dataBlockSize();
-            file.seek(address);
-            QByteArray data = file.read(blockSize);
-            file.close();
-            const int dataSize = data.size();
-            if (dataSize != blockSize)
-                data += QByteArray(blockSize - dataSize, 0);
-            m_widget->addData(address, data);
-        } else {
-            QMessageBox::critical(ICore::dialogParent(), tr("File Error"),
-                                  tr("Cannot open %1: %2").arg(
-                                        fn.toUserOutput(), file.errorString()));
-        }
+        const int blockSize = m_widget->dataBlockSize();
+        QByteArray data = fn.fileContents(blockSize, address);
+        const int dataSize = data.size();
+        if (dataSize != blockSize)
+            data += QByteArray(blockSize - dataSize, 0);
+        m_widget->addData(address, data);
+//       QMessageBox::critical(ICore::dialogParent(), tr("File Error"),
+//                             tr("Cannot open %1: %2").arg(
+//                                   fn.toUserOutput(), file.errorString()));
     }
 
     void provideNewRange(quint64 offset)
     {
         if (filePath().exists())
-            openImpl(nullptr, filePath().toString(), offset);
+            openImpl(nullptr, filePath(), offset);
     }
 
 public:
@@ -322,31 +317,20 @@ public:
                                                  : m_widget->isModified();
     }
 
-    bool isFileReadOnly() const override {
-        const FilePath fn = filePath();
-        if (fn.isEmpty())
-            return false;
-        return !fn.toFileInfo().isWritable();
-    }
-
     bool isSaveAsAllowed() const override { return true; }
 
     bool reload(QString *errorString, ReloadFlag flag, ChangeType type) override
     {
+        Q_UNUSED(type)
         if (flag == FlagIgnore)
             return true;
-        if (type == TypePermissions) {
-            emit changed();
-        } else {
-            emit aboutToReload();
-            int cPos = m_widget->cursorPosition();
-            m_widget->clear();
-            const bool success = (openImpl(errorString, filePath().toString()) == OpenResult::Success);
-            m_widget->setCursorPosition(cPos);
-            emit reloadFinished(success);
-            return success;
-        }
-        return true;
+        emit aboutToReload();
+        int cPos = m_widget->cursorPosition();
+        m_widget->clear();
+        const bool success = (openImpl(errorString, filePath()) == OpenResult::Success);
+        m_widget->setCursorPosition(cPos);
+        emit reloadFinished(success);
+        return success;
     }
 
 private:
