@@ -30,16 +30,16 @@
 namespace ClangCodeModel {
 namespace Internal {
 
-ClangCompletionAssistInterface::ClangCompletionAssistInterface(
-        BackendCommunicator &communicator,
+ClangCompletionAssistInterface::ClangCompletionAssistInterface(BackendCommunicator &communicator, CompletionType type,
         const TextEditor::TextEditorWidget *textEditorWidget,
         int position,
-        const QString &fileName,
+        const Utils::FilePath &fileName,
         TextEditor::AssistReason reason,
-        const CppTools::ProjectPartHeaderPaths &headerPaths,
+        const ProjectExplorer::HeaderPaths &headerPaths,
         const CPlusPlus::LanguageFeatures &features)
     : AssistInterface(textEditorWidget->document(), position, fileName, reason)
     , m_communicator(communicator)
+    , m_type(type)
     , m_headerPaths(headerPaths)
     , m_languageFeatures(features)
     , m_textEditorWidget(textEditorWidget)
@@ -51,7 +51,7 @@ bool ClangCompletionAssistInterface::objcEnabled() const
     return true; // TODO:
 }
 
-const CppTools::ProjectPartHeaderPaths &ClangCompletionAssistInterface::headerPaths() const
+const ProjectExplorer::HeaderPaths &ClangCompletionAssistInterface::headerPaths() const
 {
     return m_headerPaths;
 }
@@ -61,7 +61,7 @@ CPlusPlus::LanguageFeatures ClangCompletionAssistInterface::languageFeatures() c
     return m_languageFeatures;
 }
 
-void ClangCompletionAssistInterface::setHeaderPaths(const CppTools::ProjectPartHeaderPaths &headerPaths)
+void ClangCompletionAssistInterface::setHeaderPaths(const ProjectExplorer::HeaderPaths &headerPaths)
 {
     m_headerPaths = headerPaths;
 }
