@@ -26,37 +26,38 @@
 #include "clangassistproposalitem.h"
 
 #include "clangactivationsequenceprocessor.h"
-#include "clangassistproposal.h"
 #include "clangassistproposalmodel.h"
 #include "clangcompletionassistprocessor.h"
 #include "clangcompletioncontextanalyzer.h"
+#include "clangfixitoperation.h"
 #include "clangfunctionhintmodel.h"
 #include "clangcompletionchunkstotextconverter.h"
 #include "clangpreprocessorassistproposalitem.h"
 
-#include <cpptools/cppdoxygen.h>
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpptoolsbridge.h>
-#include <cpptools/editordocumenthandle.h>
+#include <cppeditor/cppdoxygen.h>
+#include <cppeditor/cppmodelmanager.h>
+#include <cppeditor/cpptoolsreuse.h>
+#include <cppeditor/editordocumenthandle.h>
 
 #include <texteditor/codeassist/assistproposalitem.h>
 #include <texteditor/codeassist/functionhintproposal.h>
+#include <texteditor/codeassist/genericproposal.h>
 #include <texteditor/codeassist/ifunctionhintproposalmodel.h>
+#include <texteditor/texteditorsettings.h>
 
-#include <cplusplus/BackwardsScanner.h>
-#include <cplusplus/ExpressionUnderCursor.h>
 #include <cplusplus/Icons.h>
-#include <cplusplus/SimpleLexer.h>
 
 #include <clangsupport/filecontainer.h>
 
 #include <utils/algorithm.h>
-#include <utils/textutils.h>
 #include <utils/mimetypes/mimedatabase.h>
+#include <utils/optional.h>
+#include <utils/porting.h>
 #include <utils/qtcassert.h>
-#include <utils/qtcfallthrough.h>
+#include <utils/textutils.h>
 
 #include <QDirIterator>
+#include <QPair>
 #include <QTextDocument>
 
 namespace ClangCodeModel {
@@ -65,52 +66,113 @@ namespace Internal {
 using ClangBackEnd::CodeCompletion;
 using TextEditor::AssistProposalItemInterface;
 
-namespace {
-
-QList<AssistProposalItemInterface *> toAssistProposalItems(const CodeCompletions &completions)
+static void addAssistProposalItem(QList<AssistProposalItemInterface *> &items,
+                                  const CodeCompletion &codeCompletion,
+                                  const QString &name)
 {
+    auto item = new ClangAssistProposalItem;
+    items.push_back(item);
 
-    bool signalCompletion = false; // TODO
-    bool slotCompletion = false; // TODO
+    item->setText(name);
+    item->setOrder(int(codeCompletion.priority));
+    item->appendCodeCompletion(codeCompletion);
+}
 
-    QHash<QString, ClangAssistProposalItem *> items;
-    foreach (const CodeCompletion &codeCompletion, completions) {
-        if (codeCompletion.text().isEmpty()) // TODO: Make isValid()?
-            continue;
-        if (signalCompletion && codeCompletion.completionKind() != CodeCompletion::SignalCompletionKind)
-            continue;
-        if (slotCompletion && codeCompletion.completionKind() != CodeCompletion::SlotCompletionKind)
+// Add the next CXXMethod or CXXConstructor which is the overload for another existing item.
+static void addFunctionOverloadAssistProposalItem(QList<AssistProposalItemInterface *> &items,
+                                                  AssistProposalItemInterface *sameItem,
+                                                  const ClangCompletionAssistInterface *interface,
+                                                  const CodeCompletion &codeCompletion,
+                                                  const QString &name)
+{
+    auto *item = static_cast<ClangAssistProposalItem *>(sameItem);
+    item->setHasOverloadsWithParameters(codeCompletion.hasParameters);
+    if (codeCompletion.completionKind == CodeCompletion::ConstructorCompletionKind) {
+        // It's the constructor, currently constructor definitions do not lead here.
+        // CLANG-UPGRADE-CHECK: Can we get here with constructor definition?
+        item->appendCodeCompletion(codeCompletion);
+        return;
+    }
+
+    QTextCursor cursor = interface->textEditorWidget()->textCursor();
+    cursor.setPosition(interface->position());
+    cursor.movePosition(QTextCursor::StartOfWord);
+
+    const ClangBackEnd::CodeCompletionChunk resultType = codeCompletion.chunks.first();
+    if (matchPreviousWord(*interface->textEditorWidget(), cursor, resultType.text.toString())) {
+        // Function definition completion - do not merge completions together.
+        addAssistProposalItem(items, codeCompletion, name);
+    } else {
+        item->appendCodeCompletion(codeCompletion);
+    }
+}
+
+// Check if they are both CXXMethod or CXXConstructor.
+static bool isTheSameFunctionOverload(const CodeCompletion &completion,
+                                      const QString &name,
+                                      ClangAssistProposalItem *lastItem)
+{
+    return completion.completionKind == lastItem->firstCodeCompletion().completionKind
+            && lastItem->text() == name;
+}
+
+QList<AssistProposalItemInterface *> ClangCompletionAssistProcessor::toAssistProposalItems(
+        const CodeCompletions &completions) const
+{
+    // TODO: Handle Qt4's SIGNAL/SLOT
+    //   Possibly check for m_completionOperator == T_SIGNAL
+    //   Possibly check for codeCompletion.completionKind == CodeCompletion::SignalCompletionKind
+
+    QList<AssistProposalItemInterface *> items;
+    items.reserve(completions.size());
+
+    // If there are signals among the candidates, we employ the built-in code model to find out
+    // whether the cursor was on the second argument of a (dis)connect() call.
+    // If so, we offer only signals, as nothing else makes sense in that context.
+    bool considerOnlySignals = false;
+    if (m_position != -1 && Utils::anyOf(completions, [](const CodeCompletion &c) {
+        return c.completionKind == CodeCompletion::SignalCompletionKind;
+    })) {
+        considerOnlySignals = CppEditor::CppModelManager::instance()
+                ->positionRequiresSignal(m_interface->filePath().toString(), m_content, m_position);
+    }
+    for (const CodeCompletion &codeCompletion : completions) {
+        if (codeCompletion.text.isEmpty())
+            continue; // It's an OverloadCandidate which has text but no typedText.
+
+        if (considerOnlySignals
+                && codeCompletion.completionKind != CodeCompletion::ClassCompletionKind
+                && codeCompletion.completionKind != CodeCompletion::NamespaceCompletionKind
+                && codeCompletion.completionKind != CodeCompletion::SignalCompletionKind) {
+                continue;
+        }
+
+        // Don't offer symbols that are not accessible here.
+        if (codeCompletion.availability == CodeCompletion::NotAvailable
+                || codeCompletion.availability == CodeCompletion::NotAccessible) {
             continue;
+        }
 
-        QString name;
-        if (codeCompletion.completionKind() == CodeCompletion::KeywordCompletionKind)
-            name = CompletionChunksToTextConverter::convertToName(codeCompletion.chunks());
-        else
-            name = codeCompletion.text().toString();
+        const QString name = codeCompletion.completionKind == CodeCompletion::KeywordCompletionKind
+                ? CompletionChunksToTextConverter::convertToName(codeCompletion.chunks)
+                : codeCompletion.text.toString();
 
-        ClangAssistProposalItem *item = items.value(name, 0);
-        if (item) {
-            if (codeCompletion.hasParameters())
-                item->setHasOverloadsWithParameters(true);
+        if (items.empty()) {
+            addAssistProposalItem(items, codeCompletion, name);
         } else {
-            item = new ClangAssistProposalItem;
-            items.insert(name, item);
-
-            item->setText(name);
-            item->setOrder(int(codeCompletion.priority()));
-            item->setCodeCompletion(codeCompletion);
+            auto *lastItem = static_cast<ClangAssistProposalItem *>(items.last());
+            if (isTheSameFunctionOverload(codeCompletion, name, lastItem)) {
+                addFunctionOverloadAssistProposalItem(items, items.back(), m_interface.data(),
+                                                      codeCompletion, name);
+            } else {
+                addAssistProposalItem(items, codeCompletion, name);
+            }
         }
     }
 
-    QList<AssistProposalItemInterface *> results;
-    results.reserve(items.size());
-    std::copy(items.cbegin(), items.cend(), std::back_inserter(results));
-
-    return results;
+    return items;
 }
 
-} // Anonymous
-
 using namespace CPlusPlus;
 using namespace TextEditor;
 
@@ -120,48 +182,83 @@ ClangCompletionAssistProcessor::ClangCompletionAssistProcessor()
 {
 }
 
-ClangCompletionAssistProcessor::~ClangCompletionAssistProcessor()
-{
-}
+ClangCompletionAssistProcessor::~ClangCompletionAssistProcessor() = default;
 
 IAssistProposal *ClangCompletionAssistProcessor::perform(const AssistInterface *interface)
 {
     m_interface.reset(static_cast<const ClangCompletionAssistInterface *>(interface));
 
     if (interface->reason() != ExplicitlyInvoked && !accepts()) {
-        setPerformWasApplicable(false);
-        return 0;
+        m_requestSent = false;
+        return nullptr;
     }
 
     return startCompletionHelper(); // == 0 if results are calculated asynchronously
 }
 
-static CodeCompletions filterFunctionSignatures(const CodeCompletions &completions)
+// All completions require fix-it, apply this fix-it now.
+CodeCompletions ClangCompletionAssistProcessor::applyCompletionFixIt(const CodeCompletions &completions)
 {
-    return ::Utils::filtered(completions, [](const CodeCompletion &completion) {
-        return completion.completionKind() == CodeCompletion::FunctionOverloadCompletionKind;
-    });
+    // CLANG-UPGRADE-CHECK: Currently we rely on fact that there are only 2 possible fix-it types:
+    // 'dot to arrow' and 'arrow to dot' and they can't appear for the same item.
+    // However if we get multiple options which trigger for the same code we need to improve this
+    // algorithm. Check comments to FixIts field of CodeCompletionResult and which fix-its are used
+    // to construct results in SemaCodeComplete.cpp.
+    const CodeCompletion &completion = completions.front();
+    const ClangBackEnd::FixItContainer &fixIt = completion.requiredFixIts.front();
+
+    ClangFixItOperation fixItOperation(Utf8String(), completion.requiredFixIts);
+    fixItOperation.perform();
+
+    const int fixItLength = fixIt.range.end.column - fixIt.range.start.column;
+    const QString fixItText = fixIt.text.toString();
+    m_positionForProposal += fixItText.length() - fixItLength;
+
+    CodeCompletions completionsWithoutFixIts;
+    completionsWithoutFixIts.reserve(completions.size());
+    for (const CodeCompletion &completion : completions) {
+        CodeCompletion completionCopy = completion;
+        completionCopy.requiredFixIts.clear();
+        completionsWithoutFixIts.push_back(completionCopy);
+    }
+
+    return completionsWithoutFixIts;
 }
 
-void ClangCompletionAssistProcessor::handleAvailableCompletions(
-        const CodeCompletions &completions,
-        CompletionCorrection neededCorrection)
+void ClangCompletionAssistProcessor::handleAvailableCompletions(const CodeCompletions &completions)
 {
     QTC_CHECK(m_completions.isEmpty());
 
-    if (m_sentRequestType == NormalCompletion) {
+    if (m_sentRequestType == FunctionHintCompletion && !completions.isEmpty()) {
+        const CodeCompletion &firstCompletion = completions.front();
+        if (firstCompletion.completionKind == CodeCompletion::FunctionOverloadCompletionKind) {
+            setAsyncProposalAvailable(createFunctionHintProposal(completions));
+            return;
+        }
+
+        if (!m_fallbackToNormalCompletion) {
+            // We must report back to the code assistant under all circumstances
+            setAsyncProposalAvailable(nullptr);
+            return;
+        }
+        // else: Proceed with a normal completion in case:
+        // 1) it was not a function call, but e.g. a function declaration like "void f("
+        // 2) '{' meant not a constructor call.
+    }
+
+    //m_sentRequestType == NormalCompletion or function signatures were empty
+
+    // Completions are sorted the way that all items with fix-its come after all items without them
+    // therefore it's enough to check only the first one.
+    if (!completions.isEmpty() && !completions.front().requiredFixIts.isEmpty())
+        m_completions = toAssistProposalItems(applyCompletionFixIt(completions));
+    else
         m_completions = toAssistProposalItems(completions);
 
-        if (m_addSnippets && !m_completions.isEmpty())
-            addSnippets();
+    if (m_addSnippets && !m_completions.isEmpty())
+        addSnippets();
 
-        setAsyncProposalAvailable(createProposal(neededCorrection));
-    } else {
-        const CodeCompletions functionSignatures = filterFunctionSignatures(completions);
-        if (!functionSignatures.isEmpty())
-            setAsyncProposalAvailable(createFunctionHintProposal(functionSignatures));
-        // else: Not a function call, but e.g. a function declaration like "void f("
-    }
+    setAsyncProposalAvailable(createProposal());
 }
 
 const TextEditorWidget *ClangCompletionAssistProcessor::textEditorWidget() const
@@ -216,12 +313,21 @@ static QByteArray modifyInput(QTextDocument *doc, int endOfExpression) {
     return modifiedInput;
 }
 
+static QChar lastPrecedingNonWhitespaceChar(const ClangCompletionAssistInterface *interface)
+{
+    int pos = interface->position();
+    while (pos >= 0 && interface->characterAt(pos).isSpace())
+        --pos;
+    return pos >= 0 ? interface->characterAt(pos) : QChar::Null;
+}
+
 IAssistProposal *ClangCompletionAssistProcessor::startCompletionHelper()
 {
     ClangCompletionContextAnalyzer analyzer(m_interface.data(), m_interface->languageFeatures());
     analyzer.analyze();
     m_completionOperator = analyzer.completionOperator();
     m_positionForProposal = analyzer.positionForProposal();
+    m_addSnippets = analyzer.addSnippets();
 
     QByteArray modifiedFileContent;
 
@@ -232,7 +338,9 @@ IAssistProposal *ClangCompletionAssistProcessor::startCompletionHelper()
             return createProposal();
         break;
     case ClangCompletionContextAnalyzer::CompleteIncludePath:
-        if (completeInclude(analyzer.positionEndOfExpression()))
+        m_completions = completeInclude(analyzer.positionEndOfExpression(), m_completionOperator,
+                                        m_interface.data(), m_interface->headerPaths());
+        if (!m_completions.isEmpty())
             return createProposal();
         break;
     case ClangCompletionContextAnalyzer::CompletePreprocessorDirective:
@@ -245,25 +353,25 @@ IAssistProposal *ClangCompletionAssistProcessor::startCompletionHelper()
                                           analyzer.positionEndOfExpression());
         Q_FALLTHROUGH();
     case ClangCompletionContextAnalyzer::PassThroughToLibClang: {
-        m_addSnippets = m_completionOperator == T_EOF_SYMBOL;
         m_sentRequestType = NormalCompletion;
-        const bool requestSent = sendCompletionRequest(analyzer.positionForClang(),
-                                                       modifiedFileContent);
-        setPerformWasApplicable(requestSent);
+        m_requestSent = sendCompletionRequest(analyzer.positionForClang(),
+                                              modifiedFileContent);
         break;
     }
     case ClangCompletionContextAnalyzer::PassThroughToLibClangAfterLeftParen: {
         m_sentRequestType = FunctionHintCompletion;
-        const bool requestSent = sendCompletionRequest(analyzer.positionForClang(), QByteArray(),
-                                                       analyzer.functionNameStart());
-        setPerformWasApplicable(requestSent);
+        if (lastPrecedingNonWhitespaceChar(m_interface.data()) == ',')
+            m_fallbackToNormalCompletion = false;
+        m_requestSent = sendCompletionRequest(analyzer.positionForClang(), QByteArray(),
+                                              analyzer.functionNameStart());
         break;
     }
+    case ClangCompletionContextAnalyzer::CompleteNone:
     default:
         break;
     }
 
-    return 0;
+    return nullptr;
 }
 
 int ClangCompletionAssistProcessor::startOfOperator(int positionInDocument,
@@ -316,44 +424,15 @@ bool ClangCompletionAssistProcessor::accepts() const
 
         return true;
     } else {
-        // Trigger completion after three characters of a name have been typed, when not editing an existing name
+        // Trigger completion after n characters of a name have been typed, when not editing an existing name
         QChar characterUnderCursor = m_interface->characterAt(pos);
         if (!characterUnderCursor.isLetterOrNumber() && characterUnderCursor != QLatin1Char('_')) {
             const int startOfName = findStartOfName(pos);
-            if (pos - startOfName >= 3) {
+            if (pos - startOfName >= TextEditorSettings::completionSettings().m_characterThreshold) {
                 const QChar firstCharacter = m_interface->characterAt(startOfName);
                 if (firstCharacter.isLetter() || firstCharacter == QLatin1Char('_')) {
-                    // Finally check that we're not inside a comment or string (code copied from startOfOperator)
-                    QTextCursor tc(m_interface->textDocument());
-                    tc.setPosition(pos);
-
-                    SimpleLexer tokenize;
-                    LanguageFeatures lf = tokenize.languageFeatures();
-                    lf.qtMocRunEnabled = true;
-                    lf.objCEnabled = true;
-                    tokenize.setLanguageFeatures(lf);
-                    tokenize.setSkipComments(false);
-                    const Tokens &tokens = tokenize(tc.block().text(), BackwardsScanner::previousBlockState(tc.block()));
-                    const int tokenIdx = SimpleLexer::tokenBefore(tokens, qMax(0, tc.positionInBlock() - 1));
-                    const Token tk = (tokenIdx == -1) ? Token() : tokens.at(tokenIdx);
-
-                    if (!tk.isComment() && !tk.isLiteral()) {
-                        return true;
-                    } else if (tk.isLiteral()
-                               && tokens.size() == 3
-                               && tokens.at(0).kind() == T_POUND
-                               && tokens.at(1).kind() == T_IDENTIFIER) {
-                        const QString &line = tc.block().text();
-                        const Token &idToken = tokens.at(1);
-                        const QStringRef &identifier =
-                                line.midRef(idToken.bytesBegin(),
-                                            idToken.bytesEnd() - idToken.bytesBegin());
-                        if (identifier == QLatin1String("include")
-                                || identifier == QLatin1String("include_next")
-                                || (m_interface->objcEnabled() && identifier == QLatin1String("import"))) {
-                            return true;
-                        }
-                    }
+                    return !CppEditor::isInCommentOrString(m_interface.data(),
+                                                           m_interface->languageFeatures());
                 }
             }
         }
@@ -364,66 +443,81 @@ bool ClangCompletionAssistProcessor::accepts() const
 
 /**
  * @brief Creates completion proposals for #include and given cursor
- * @param cursor - cursor placed after opening bracked or quote
- * @return false if completions list is empty
+ * @param position - cursor placed after opening bracked or quote
+ * @param completionOperator - the type of token
+ * @param interface - relevant document data
+ * @param headerPaths - the include paths
+ * @return the list of completion items
  */
-bool ClangCompletionAssistProcessor::completeInclude(const QTextCursor &cursor)
+QList<AssistProposalItemInterface *> ClangCompletionAssistProcessor::completeInclude(
+        int position, unsigned completionOperator, const TextEditor::AssistInterface *interface,
+        const ProjectExplorer::HeaderPaths &headerPaths)
 {
+    QTextCursor cursor(interface->textDocument());
+    cursor.setPosition(position);
     QString directoryPrefix;
-    if (m_completionOperator == T_SLASH) {
+    if (completionOperator == T_SLASH) {
         QTextCursor c = cursor;
         c.movePosition(QTextCursor::StartOfLine, QTextCursor::KeepAnchor);
         QString sel = c.selectedText();
         int startCharPos = sel.indexOf(QLatin1Char('"'));
         if (startCharPos == -1) {
             startCharPos = sel.indexOf(QLatin1Char('<'));
-            m_completionOperator = T_ANGLE_STRING_LITERAL;
+            completionOperator = T_ANGLE_STRING_LITERAL;
         } else {
-            m_completionOperator = T_STRING_LITERAL;
+            completionOperator = T_STRING_LITERAL;
         }
         if (startCharPos != -1)
             directoryPrefix = sel.mid(startCharPos + 1, sel.length() - 1);
     }
 
     // Make completion for all relevant includes
-    CppTools::ProjectPartHeaderPaths headerPaths = m_interface->headerPaths();
-    const CppTools::ProjectPartHeaderPath currentFilePath(QFileInfo(m_interface->fileName()).path(),
-                                                          CppTools::ProjectPartHeaderPath::IncludePath);
-    if (!headerPaths.contains(currentFilePath))
-        headerPaths.append(currentFilePath);
+    ProjectExplorer::HeaderPaths allHeaderPaths = headerPaths;
+    const auto currentFilePath = ProjectExplorer::HeaderPath::makeUser(
+                interface->filePath().toFileInfo().path());
+    if (!allHeaderPaths.contains(currentFilePath))
+        allHeaderPaths.append(currentFilePath);
 
     const ::Utils::MimeType mimeType = ::Utils::mimeTypeForName("text/x-c++hdr");
     const QStringList suffixes = mimeType.suffixes();
 
-    foreach (const CppTools::ProjectPartHeaderPath &headerPath, headerPaths) {
+    QList<AssistProposalItemInterface *> completions;
+    foreach (const ProjectExplorer::HeaderPath &headerPath, allHeaderPaths) {
         QString realPath = headerPath.path;
         if (!directoryPrefix.isEmpty()) {
             realPath += QLatin1Char('/');
             realPath += directoryPrefix;
-            if (headerPath.isFrameworkPath())
+            if (headerPath.type == ProjectExplorer::HeaderPathType::Framework)
                 realPath += QLatin1String(".framework/Headers");
         }
-        completeIncludePath(realPath, suffixes);
+        completions << completeIncludePath(realPath, suffixes, completionOperator);
     }
 
-    return !m_completions.isEmpty();
-}
+    QList<QPair<AssistProposalItemInterface *, QString>> completionsForSorting;
+    for (AssistProposalItemInterface * const item : qAsConst(completions)) {
+        QString s = item->text();
+        s.replace('/', QChar(0)); // The dir separator should compare less than anything else.
+        completionsForSorting << qMakePair(item, s);
+    }
+    Utils::sort(completionsForSorting, [](const auto &left, const auto &right) {
+        return left.second < right.second;
+    });
+    for (int i = 0; i < completionsForSorting.count(); ++i)
+        completions[i] = completionsForSorting[i].first;
 
-bool ClangCompletionAssistProcessor::completeInclude(int position)
-{
-    QTextCursor textCursor(m_interface->textDocument()); // TODO: Simplify, move into function
-    textCursor.setPosition(position);
-    return completeInclude(textCursor);
+    return completions;
 }
 
 /**
- * @brief Adds #include completion proposals using given include path
+ * @brief Finds #include completion proposals using given include path
  * @param realPath - one of directories where compiler searches includes
  * @param suffixes - file suffixes for C/C++ header files
+ * @return a list of matching completion items
  */
-void ClangCompletionAssistProcessor::completeIncludePath(const QString &realPath,
-                                                         const QStringList &suffixes)
+QList<AssistProposalItemInterface *> ClangCompletionAssistProcessor::completeIncludePath(
+        const QString &realPath, const QStringList &suffixes, unsigned completionOperator)
 {
+    QList<AssistProposalItemInterface *> completions;
     QDirIterator i(realPath, QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot);
     //: Parent folder for proposed #include completion
     const QString hint = tr("Location: %1").arg(QDir::toNativeSeparators(QDir::cleanPath(realPath)));
@@ -439,30 +533,31 @@ void ClangCompletionAssistProcessor::completeIncludePath(const QString &realPath
             auto *item = new ClangPreprocessorAssistProposalItem;
             item->setText(text);
             item->setDetail(hint);
-            item->setIcon(Icons::keywordIcon());
-            item->setCompletionOperator(m_completionOperator);
-            m_completions.append(item);
+            item->setIcon(CPlusPlus::Icons::keywordIcon());
+            item->setCompletionOperator(completionOperator);
+            completions.append(item);
         }
     }
+    return completions;
 }
 
 bool ClangCompletionAssistProcessor::completePreprocessorDirectives()
 {
-    foreach (const QString &preprocessorCompletion, m_preprocessorCompletions)
+    foreach (const QString &preprocessorCompletion, preprocessorCompletions())
         addCompletionItem(preprocessorCompletion,
-                          Icons::iconForType(Icons::MacroIconType));
+                          Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::Macro));
 
     if (m_interface->objcEnabled())
         addCompletionItem(QLatin1String("import"),
-                          Icons::iconForType(Icons::MacroIconType));
+                          Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::Macro));
 
     return !m_completions.isEmpty();
 }
 
 bool ClangCompletionAssistProcessor::completeDoxygenKeywords()
 {
-    for (int i = 1; i < CppTools::T_DOXY_LAST_TAG; ++i)
-        addCompletionItem(QString::fromLatin1(CppTools::doxygenTagSpell(i)), Icons::keywordIcon());
+    for (int i = 1; i < CppEditor::T_DOXY_LAST_TAG; ++i)
+        addCompletionItem(QString::fromLatin1(CppEditor::doxygenTagSpell(i)), CPlusPlus::Icons::keywordIcon());
     return !m_completions.isEmpty();
 }
 
@@ -497,17 +592,16 @@ void ClangCompletionAssistProcessor::sendFileContent(const QByteArray &customFil
     const UnsavedFileContentInfo info = unsavedFileContent(customFileContent);
 
     BackendCommunicator &communicator = m_interface->communicator();
-    communicator.updateTranslationUnitsForEditor({{m_interface->fileName(),
-                                                   Utf8String(),
-                                                   Utf8String::fromByteArray(info.unsavedContent),
-                                                   info.isDocumentModified,
-                                                   uint(m_interface->textDocument()->revision())}});
+    communicator.documentsChanged({{m_interface->filePath().toString(),
+                                    Utf8String::fromByteArray(info.unsavedContent),
+                                    info.isDocumentModified,
+                                    uint(m_interface->textDocument()->revision())}});
 }
 namespace {
 bool shouldSendDocumentForCompletion(const QString &filePath,
                                      int completionPosition)
 {
-    CppTools::CppEditorDocumentHandle *document = ClangCodeModel::Utils::cppDocument(filePath);
+    CppEditor::CppEditorDocumentHandle *document = cppDocument(filePath);
 
     if (document) {
         auto &sendTracker = document->sendTracker();
@@ -521,7 +615,7 @@ bool shouldSendDocumentForCompletion(const QString &filePath,
 bool shouldSendCodeCompletion(const QString &filePath,
                               int completionPosition)
 {
-    CppTools::CppEditorDocumentHandle *document = ClangCodeModel::Utils::cppDocument(filePath);
+    CppEditor::CppEditorDocumentHandle *document = cppDocument(filePath);
 
     if (document) {
         auto &sendTracker = document->sendTracker();
@@ -533,7 +627,7 @@ bool shouldSendCodeCompletion(const QString &filePath,
 
 void setLastDocumentRevision(const QString &filePath)
 {
-    CppTools::CppEditorDocumentHandle *document = ClangCodeModel::Utils::cppDocument(filePath);
+    CppEditor::CppEditorDocumentHandle *document = cppDocument(filePath);
 
     if (document)
         document->sendTracker().setLastSentRevision(int(document->revision()));
@@ -542,7 +636,7 @@ void setLastDocumentRevision(const QString &filePath)
 void setLastCompletionPosition(const QString &filePath,
                                int completionPosition)
 {
-    CppTools::CppEditorDocumentHandle *document = ClangCodeModel::Utils::cppDocument(filePath);
+    CppEditor::CppEditorDocumentHandle *document = cppDocument(filePath);
 
     if (document)
         document->sendTracker().setLastCompletionPosition(completionPosition);
@@ -557,9 +651,9 @@ ClangCompletionAssistProcessor::extractLineColumn(int position)
         return {-1, -1};
 
     int line = -1, column = -1;
-    ::Utils::Text::convertPosition(m_interface->textDocument(), position, &line, &column);
+    Utils::Text::convertPosition(m_interface->textDocument(), position, &line, &column);
 
-    column = Utils::clangColumn(m_interface->textDocument()->findBlock(position), column);
+    column = clangColumn(m_interface->textDocument()->findBlock(position), column);
     return {line, column};
 }
 
@@ -567,7 +661,7 @@ bool ClangCompletionAssistProcessor::sendCompletionRequest(int position,
                                                            const QByteArray &customFileContent,
                                                            int functionNameStartPosition)
 {
-    const QString filePath = m_interface->fileName();
+    const QString filePath = m_interface->filePath().toString();
 
     auto &communicator = m_interface->communicator();
 
@@ -579,33 +673,46 @@ bool ClangCompletionAssistProcessor::sendCompletionRequest(int position,
 
         const Position cursorPosition = extractLineColumn(position);
         const Position functionNameStart = extractLineColumn(functionNameStartPosition);
-        const QString projectPartId = CppTools::CppToolsBridge::projectPartIdForFile(filePath);
-        communicator.completeCode(this, filePath, uint(cursorPosition.line),
-                                  uint(cursorPosition.column), projectPartId,
-                                  functionNameStart.line, functionNameStart.column);
+        communicator.requestCompletions(this,
+                                        filePath,
+                                        uint(cursorPosition.line),
+                                        uint(cursorPosition.column),
+                                        functionNameStart.line,
+                                        functionNameStart.column);
         setLastCompletionPosition(filePath, position);
+        if (m_sentRequestType == NormalCompletion) {
+            if (!customFileContent.isEmpty())
+                m_content = customFileContent;
+            else if (const CppEditor::CppEditorDocumentHandle * const doc = cppDocument(filePath))
+                m_content = doc->contents();
+            m_position = position;
+        }
         return true;
     }
 
     return false;
 }
 
-TextEditor::IAssistProposal *ClangCompletionAssistProcessor::createProposal(
-        CompletionCorrection neededCorrection) const
+TextEditor::IAssistProposal *ClangCompletionAssistProcessor::createProposal()
 {
-    ClangAssistProposalModel *model = new ClangAssistProposalModel(neededCorrection);
+    m_requestSent = false;
+    TextEditor::GenericProposalModelPtr model(new ClangAssistProposalModel());
     model->loadContent(m_completions);
-    return new ClangAssistProposal(m_positionForProposal, model);
+    return new GenericProposal(m_positionForProposal, model);
 }
 
 IAssistProposal *ClangCompletionAssistProcessor::createFunctionHintProposal(
-        const ClangBackEnd::CodeCompletions &completions) const
+        const ClangBackEnd::CodeCompletions &completions)
 {
-    auto *model = new ClangFunctionHintModel(completions);
-    auto *proposal = new FunctionHintProposal(m_positionForProposal, model);
-    return proposal;
+    m_requestSent = false;
+    TextEditor::FunctionHintProposalModelPtr model(new ClangFunctionHintModel(completions));
+    return new FunctionHintProposal(m_positionForProposal, model);
+}
+
+void ClangCompletionAssistProcessor::cancel()
+{
+    m_interface->communicator().cancelCompletions(this);
 }
 
 } // namespace Internal
 } // namespace ClangCodeModel
-
