@@ -26,26 +26,27 @@
 #include "clangeditordocumentprocessor.h"
 
 #include "clangbackendcommunicator.h"
+#include "clangprojectsettings.h"
 #include "clangdiagnostictooltipwidget.h"
 #include "clangfixitoperation.h"
 #include "clangfixitoperationsextractor.h"
 #include "clangmodelmanagersupport.h"
 #include "clanghighlightingresultreporter.h"
-#include "clangprojectsettings.h"
 #include "clangutils.h"
 
 #include <diagnosticcontainer.h>
 #include <sourcelocationcontainer.h>
 
-#include <cpptools/builtincursorinfo.h>
-#include <cpptools/clangdiagnosticconfigsmodel.h>
-#include <cpptools/compileroptionsbuilder.h>
-#include <cpptools/cppcodemodelsettings.h>
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpptoolsbridge.h>
-#include <cpptools/cpptoolsreuse.h>
-#include <cpptools/cppworkingcopy.h>
-#include <cpptools/editordocumenthandle.h>
+#include <clangsupport/tokeninfocontainer.h>
+
+#include <cppeditor/builtincursorinfo.h>
+#include <cppeditor/clangdiagnosticconfigsmodel.h>
+#include <cppeditor/compileroptionsbuilder.h>
+#include <cppeditor/cppcodemodelsettings.h>
+#include <cppeditor/cppmodelmanager.h>
+#include <cppeditor/cpptoolsreuse.h>
+#include <cppeditor/cppworkingcopy.h>
+#include <cppeditor/editordocumenthandle.h>
 
 #include <texteditor/fontsettings.h>
 #include <texteditor/texteditor.h>
@@ -66,12 +67,6 @@
 namespace ClangCodeModel {
 namespace Internal {
 
-static ClangProjectSettings &getProjectSettings(ProjectExplorer::Project *project)
-{
-    QTC_CHECK(project);
-    return ClangModelManagerSupport::instance()->projectSettings(project);
-}
-
 ClangEditorDocumentProcessor::ClangEditorDocumentProcessor(
         BackendCommunicator &communicator,
         TextEditor::TextDocument *document)
@@ -94,25 +89,24 @@ ClangEditorDocumentProcessor::ClangEditorDocumentProcessor(
 
     // Forwarding the semantic info from the builtin processor enables us to provide all
     // editor (widget) related features that are not yet implemented by the clang plugin.
-    connect(&m_builtinProcessor, &CppTools::BuiltinEditorDocumentProcessor::cppDocumentUpdated,
+    connect(&m_builtinProcessor, &CppEditor::BuiltinEditorDocumentProcessor::cppDocumentUpdated,
             this, &ClangEditorDocumentProcessor::cppDocumentUpdated);
-    connect(&m_builtinProcessor, &CppTools::BuiltinEditorDocumentProcessor::semanticInfoUpdated,
+    connect(&m_builtinProcessor, &CppEditor::BuiltinEditorDocumentProcessor::semanticInfoUpdated,
             this, &ClangEditorDocumentProcessor::semanticInfoUpdated);
+
+    m_parserSynchronizer.setCancelOnWait(true);
 }
 
 ClangEditorDocumentProcessor::~ClangEditorDocumentProcessor()
 {
     m_updateBackendDocumentTimer.stop();
 
-    m_parserWatcher.cancel();
-    m_parserWatcher.waitForFinished();
-
     if (m_projectPart)
         closeBackendDocument();
 }
 
 void ClangEditorDocumentProcessor::runImpl(
-        const CppTools::BaseEditorDocumentParser::UpdateParams &updateParams)
+        const CppEditor::BaseEditorDocumentParser::UpdateParams &updateParams)
 {
     m_updateBackendDocumentTimer.start();
 
@@ -127,6 +121,7 @@ void ClangEditorDocumentProcessor::runImpl(
             this, &ClangEditorDocumentProcessor::onParserFinished);
     const QFuture<void> future = ::Utils::runAsync(&runParser, parser(), updateParams);
     m_parserWatcher.setFuture(future);
+    m_parserSynchronizer.addFuture(future);
 
     // Run builtin processor
     m_builtinProcessor.runImpl(updateParams);
@@ -144,18 +139,20 @@ void ClangEditorDocumentProcessor::semanticRehighlight()
     };
     if (!Utils::contains(Core::EditorManager::visibleEditors(), matchesEditor))
         return;
+    if (ClangModelManagerSupport::instance()->clientForFile(m_document.filePath()))
+        return;
 
     m_semanticHighlighter.updateFormatMapFromFontSettings();
     if (m_projectPart)
         requestAnnotationsFromBackend();
 }
 
-CppTools::SemanticInfo ClangEditorDocumentProcessor::recalculateSemanticInfo()
+CppEditor::SemanticInfo ClangEditorDocumentProcessor::recalculateSemanticInfo()
 {
     return m_builtinProcessor.recalculateSemanticInfo();
 }
 
-CppTools::BaseEditorDocumentParser::Ptr ClangEditorDocumentProcessor::parser()
+CppEditor::BaseEditorDocumentParser::Ptr ClangEditorDocumentProcessor::parser()
 {
     return m_parser;
 }
@@ -175,7 +172,7 @@ bool ClangEditorDocumentProcessor::hasProjectPart() const
     return !m_projectPart.isNull();
 }
 
-CppTools::ProjectPart::Ptr ClangEditorDocumentProcessor::projectPart() const
+CppEditor::ProjectPart::ConstPtr ClangEditorDocumentProcessor::projectPart() const
 {
     return m_projectPart;
 }
@@ -195,6 +192,9 @@ void ClangEditorDocumentProcessor::updateCodeWarnings(
         const ClangBackEnd::DiagnosticContainer &firstHeaderErrorDiagnostic,
         uint documentRevision)
 {
+    if (ClangModelManagerSupport::instance()->clientForFile(m_document.filePath()))
+        return;
+
     if (documentRevision == revision()) {
         if (m_invalidationState == InvalidationState::Scheduled)
             m_invalidationState = InvalidationState::Canceled;
@@ -253,20 +253,27 @@ void ClangEditorDocumentProcessor::generateTaskHubIssues()
     m_diagnosticManager.generateTaskHubIssues();
 }
 
+void ClangEditorDocumentProcessor::clearTextMarks(const Utils::FilePath &filePath)
+{
+    if (ClangEditorDocumentProcessor * const proc = get(filePath.toString())) {
+        proc->m_diagnosticManager.cleanMarks();
+        emit proc->codeWarningsUpdated(proc->revision(), {}, {}, {});
+    }
+}
+
 void ClangEditorDocumentProcessor::updateHighlighting(
         const QVector<ClangBackEnd::TokenInfoContainer> &tokenInfos,
         const QVector<ClangBackEnd::SourceRangeContainer> &skippedPreprocessorRanges,
         uint documentRevision)
 {
+    if (ClangModelManagerSupport::instance()->clientForFile(m_document.filePath()))
+        return;
     if (documentRevision == revision()) {
         const auto skippedPreprocessorBlocks = toTextEditorBlocks(textDocument(), skippedPreprocessorRanges);
         emit ifdefedOutBlocksUpdated(documentRevision, skippedPreprocessorBlocks);
 
         m_semanticHighlighter.setHighlightingRunner(
-            [tokenInfos]() {
-                auto *reporter = new HighlightingResultReporter(tokenInfos);
-                return reporter->start();
-            });
+            [tokenInfos]() { return highlightResults(tokenInfos); });
         m_semanticHighlighter.run();
     }
 }
@@ -317,7 +324,7 @@ TextEditor::TextMarks ClangEditorDocumentProcessor::diagnosticTextMarksAt(uint l
 }
 
 void ClangEditorDocumentProcessor::setParserConfig(
-        const CppTools::BaseEditorDocumentParser::Configuration &config)
+        const CppEditor::BaseEditorDocumentParser::Configuration &config)
 {
     m_parser->setConfiguration(config);
     m_builtinProcessor.parser()->setConfiguration(config);
@@ -326,13 +333,13 @@ void ClangEditorDocumentProcessor::setParserConfig(
 static bool isCursorOnIdentifier(const QTextCursor &textCursor)
 {
     QTextDocument *document = textCursor.document();
-    return CppTools::isValidIdentifierChar(document->characterAt(textCursor.position()));
+    return CppEditor::isValidIdentifierChar(document->characterAt(textCursor.position()));
 }
 
-static QFuture<CppTools::CursorInfo> defaultCursorInfoFuture()
+static QFuture<CppEditor::CursorInfo> defaultCursorInfoFuture()
 {
-    QFutureInterface<CppTools::CursorInfo> futureInterface;
-    futureInterface.reportResult(CppTools::CursorInfo());
+    QFutureInterface<CppEditor::CursorInfo> futureInterface;
+    futureInterface.reportResult(CppEditor::CursorInfo());
     futureInterface.reportFinished();
 
     return futureInterface.future();
@@ -348,8 +355,8 @@ static bool convertPosition(const QTextCursor &textCursor, int *line, int *colum
     return converted;
 }
 
-QFuture<CppTools::CursorInfo>
-ClangEditorDocumentProcessor::cursorInfo(const CppTools::CursorInfoParams &params)
+QFuture<CppEditor::CursorInfo>
+ClangEditorDocumentProcessor::cursorInfo(const CppEditor::CursorInfoParams &params)
 {
     int line, column;
     convertPosition(params.textCursor, &line, &column);
@@ -358,8 +365,8 @@ ClangEditorDocumentProcessor::cursorInfo(const CppTools::CursorInfoParams &param
         return defaultCursorInfoFuture();
 
     column = clangColumn(params.textCursor.document()->findBlockByNumber(line - 1), column);
-    const CppTools::SemanticInfo::LocalUseMap localUses
-        = CppTools::BuiltinCursorInfo::findLocalUses(params.semanticInfo.doc, line, column);
+    const CppEditor::SemanticInfo::LocalUseMap localUses
+        = CppEditor::BuiltinCursorInfo::findLocalUses(params.semanticInfo.doc, line, column);
 
     return m_communicator.requestReferences(simpleFileContainer(),
                                             static_cast<quint32>(line),
@@ -367,7 +374,7 @@ ClangEditorDocumentProcessor::cursorInfo(const CppTools::CursorInfoParams &param
                                             localUses);
 }
 
-QFuture<CppTools::CursorInfo> ClangEditorDocumentProcessor::requestLocalReferences(
+QFuture<CppEditor::CursorInfo> ClangEditorDocumentProcessor::requestLocalReferences(
         const QTextCursor &cursor)
 {
     int line, column;
@@ -383,7 +390,7 @@ QFuture<CppTools::CursorInfo> ClangEditorDocumentProcessor::requestLocalReferenc
                                                  static_cast<quint32>(column));
 }
 
-QFuture<CppTools::SymbolInfo>
+QFuture<CppEditor::SymbolInfo>
 ClangEditorDocumentProcessor::requestFollowSymbol(int line, int column)
 {
     return m_communicator.requestFollowSymbol(simpleFileContainer(),
@@ -391,7 +398,7 @@ ClangEditorDocumentProcessor::requestFollowSymbol(int line, int column)
                                               static_cast<quint32>(column));
 }
 
-QFuture<CppTools::ToolTipInfo> ClangEditorDocumentProcessor::toolTipInfo(const QByteArray &codecName,
+QFuture<CppEditor::ToolTipInfo> ClangEditorDocumentProcessor::toolTipInfo(const QByteArray &codecName,
                                                                          int line,
                                                                          int column)
 {
@@ -407,12 +414,11 @@ void ClangEditorDocumentProcessor::clearDiagnosticsWithFixIts()
 
 ClangEditorDocumentProcessor *ClangEditorDocumentProcessor::get(const QString &filePath)
 {
-    auto *processor = CppTools::CppToolsBridge::baseEditorDocumentProcessor(filePath);
-
-    return qobject_cast<ClangEditorDocumentProcessor*>(processor);
+    return qobject_cast<ClangEditorDocumentProcessor*>(
+                CppEditor::CppModelManager::cppEditorDocumentProcessor(filePath));
 }
 
-static bool isProjectPartLoadedOrIsFallback(CppTools::ProjectPart::Ptr projectPart)
+static bool isProjectPartLoadedOrIsFallback(CppEditor::ProjectPart::ConstPtr projectPart)
 {
     return projectPart
         && (projectPart->id().isEmpty() || isProjectPartLoaded(projectPart));
@@ -420,14 +426,14 @@ static bool isProjectPartLoadedOrIsFallback(CppTools::ProjectPart::Ptr projectPa
 
 void ClangEditorDocumentProcessor::updateBackendProjectPartAndDocument()
 {
-    const CppTools::ProjectPart::Ptr projectPart = m_parser->projectPartInfo().projectPart;
+    const CppEditor::ProjectPart::ConstPtr projectPart = m_parser->projectPartInfo().projectPart;
 
     if (isProjectPartLoadedOrIsFallback(projectPart)) {
         updateBackendDocument(*projectPart.data());
 
         m_projectPart = projectPart;
         m_isProjectFile = m_parser->projectPartInfo().hints
-                & CppTools::ProjectPartInfo::IsFromProjectMatch;
+                & CppEditor::ProjectPartInfo::IsFromProjectMatch;
     }
 }
 
@@ -439,124 +445,7 @@ void ClangEditorDocumentProcessor::onParserFinished()
     updateBackendProjectPartAndDocument();
 }
 
-namespace {
-// TODO: Can we marry this with CompilerOptionsBuilder?
-class FileOptionsBuilder
-{
-public:
-    FileOptionsBuilder(const QString &filePath, CppTools::ProjectPart &projectPart)
-        : m_filePath(filePath)
-        , m_projectPart(projectPart)
-        , m_builder(projectPart)
-    {
-        // Determine the driver mode from toolchain and flags.
-        m_builder.evaluateCompilerFlags();
-        m_isClMode = m_builder.isClStyle();
-
-        addLanguageOptions();
-        addGlobalDiagnosticOptions(); // Before addDiagnosticOptions() so users still can overwrite.
-        addDiagnosticOptions();
-        addGlobalOptions();
-        addPrecompiledHeaderOptions();
-    }
-
-    const QStringList &options() const { return m_options; }
-    const ::Utils::Id &diagnosticConfigId() const { return m_diagnosticConfigId; }
-    CppTools::UseBuildSystemWarnings useBuildSystemWarnings() const
-    {
-        return m_useBuildSystemWarnings;
-    }
-
-private:
-    void addLanguageOptions()
-    {
-        // Determine file kind with respect to ambiguous headers.
-        CppTools::ProjectFile::Kind fileKind = CppTools::ProjectFile::classify(m_filePath);
-        if (fileKind == CppTools::ProjectFile::AmbiguousHeader) {
-            fileKind = m_projectPart.languageVersion <= ::Utils::LanguageVersion::LatestC
-                 ? CppTools::ProjectFile::CHeader
-                 : CppTools::ProjectFile::CXXHeader;
-        }
-
-        m_builder.reset();
-        m_builder.updateFileLanguage(fileKind);
-
-        m_options.append(m_builder.options());
-    }
-
-    void addDiagnosticOptions()
-    {
-        if (m_projectPart.project) {
-            ClangProjectSettings &projectSettings = getProjectSettings(m_projectPart.project);
-            if (!projectSettings.useGlobalConfig()) {
-                const ::Utils::Id warningConfigId = projectSettings.warningConfigId();
-                const CppTools::ClangDiagnosticConfigsModel configsModel
-                    = CppTools::diagnosticConfigsModel();
-                if (configsModel.hasConfigWithId(warningConfigId)) {
-                    addDiagnosticOptionsForConfig(configsModel.configWithId(warningConfigId));
-                    return;
-                }
-            }
-        }
-
-        addDiagnosticOptionsForConfig(CppTools::codeModelSettings()->clangDiagnosticConfig());
-    }
-
-    void addDiagnosticOptionsForConfig(const CppTools::ClangDiagnosticConfig &diagnosticConfig)
-    {
-        m_diagnosticConfigId = diagnosticConfig.id();
-        m_useBuildSystemWarnings = diagnosticConfig.useBuildSystemWarnings()
-                                       ? CppTools::UseBuildSystemWarnings::Yes
-                                       : CppTools::UseBuildSystemWarnings::No;
-
-        const QStringList options = m_isClMode
-                                        ? CppTools::clangArgsForCl(diagnosticConfig.clangOptions())
-                                        : diagnosticConfig.clangOptions();
-        m_options.append(options);
-    }
-
-    void addGlobalDiagnosticOptions()
-    {
-        m_options += CppTools::ClangDiagnosticConfigsModel::globalDiagnosticOptions();
-    }
-
-    void addGlobalOptions()
-    {
-        if (!m_projectPart.project)
-            m_options.append(ClangProjectSettings::globalCommandLineOptions());
-        else
-            m_options.append(getProjectSettings(m_projectPart.project).commandLineOptions());
-    }
-
-    void addPrecompiledHeaderOptions()
-    {
-        using namespace CppTools;
-
-        if (getPchUsage() == UsePrecompiledHeaders::No)
-            return;
-
-        if (m_projectPart.precompiledHeaders.contains(m_filePath))
-            return;
-
-        m_builder.reset();
-        m_builder.addPrecompiledHeaderOptions(UsePrecompiledHeaders::Yes);
-
-        m_options.append(m_builder.options());
-    }
-
-private:
-    const QString &m_filePath;
-    const CppTools::ProjectPart &m_projectPart;
-
-    ::Utils::Id m_diagnosticConfigId;
-    CppTools::UseBuildSystemWarnings m_useBuildSystemWarnings = CppTools::UseBuildSystemWarnings::No;
-    CppTools::CompilerOptionsBuilder m_builder;
-    bool m_isClMode = false;
-    QStringList m_options;
-};
-} // namespace
-
-void ClangEditorDocumentProcessor::updateBackendDocument(CppTools::ProjectPart &projectPart)
+void ClangEditorDocumentProcessor::updateBackendDocument(const CppEditor::ProjectPart &projectPart)
 {
     // On registration we send the document content immediately as an unsaved
     // file, because
@@ -572,17 +461,14 @@ void ClangEditorDocumentProcessor::updateBackendDocument(CppTools::ProjectPart &
             return;
     }
 
-    const FileOptionsBuilder fileOptions(filePath(), projectPart);
-    m_diagnosticConfigId = fileOptions.diagnosticConfigId();
-
-    const QStringList projectPartOptions = createClangOptions(
-        projectPart, fileOptions.useBuildSystemWarnings(),
-        CppTools::ProjectFile::Unsupported); // No language option as FileOptionsBuilder adds it.
-
-    const QStringList compilationArguments = projectPartOptions + fileOptions.options();
+    ProjectExplorer::Project * const project = CppEditor::projectForProjectPart(projectPart);
+    const CppEditor::ClangDiagnosticConfig config = warningsConfigForProject(project);
+    const QStringList clangOptions = createClangOptions(projectPart, filePath(), config,
+                                                        optionsForProject(project));
+    m_diagnosticConfigId = config.id();
 
     m_communicator.documentsOpened(
-        {fileContainerWithOptionsAndDocumentContent(compilationArguments, projectPart.headerPaths)});
+        {fileContainerWithOptionsAndDocumentContent(clangOptions, projectPart.headerPaths)});
     setLastSentDocumentRevision(filePath(), revision());
 }
 
@@ -606,12 +492,12 @@ void ClangEditorDocumentProcessor::requestAnnotationsFromBackend()
     m_communicator.requestAnnotations(fileContainer);
 }
 
-CppTools::BaseEditorDocumentProcessor::HeaderErrorDiagnosticWidgetCreator
+CppEditor::BaseEditorDocumentProcessor::HeaderErrorDiagnosticWidgetCreator
 ClangEditorDocumentProcessor::creatorForHeaderErrorDiagnosticWidget(
         const ClangBackEnd::DiagnosticContainer &firstHeaderErrorDiagnostic)
 {
     if (firstHeaderErrorDiagnostic.text.isEmpty())
-        return CppTools::BaseEditorDocumentProcessor::HeaderErrorDiagnosticWidgetCreator();
+        return CppEditor::BaseEditorDocumentProcessor::HeaderErrorDiagnosticWidgetCreator();
 
     return [firstHeaderErrorDiagnostic]() {
         auto vbox = new QVBoxLayout;
@@ -619,7 +505,8 @@ ClangEditorDocumentProcessor::creatorForHeaderErrorDiagnosticWidget(
         vbox->setSpacing(2);
 
         vbox->addWidget(ClangDiagnosticWidget::createWidget({firstHeaderErrorDiagnostic},
-                                                            ClangDiagnosticWidget::InfoBar));
+                                                            ClangDiagnosticWidget::InfoBar, {},
+                                                            "libclang"));
 
         auto widget = new QWidget;
         widget->setLayout(vbox);
