@@ -51,11 +51,11 @@ int ClangFunctionHintModel::size() const
 
 QString ClangFunctionHintModel::text(int index) const
 {
-    const ClangBackEnd::CodeCompletionChunks chunks = m_functionSymbols.at(index).chunks();
+    const ClangBackEnd::CodeCompletionChunks &chunks = m_functionSymbols.at(index).chunks;
     const QString signatureWithEmphasizedCurrentParameter
             = CompletionChunksToTextConverter::convertToFunctionSignatureWithHtml(
                 chunks,
-                m_functionSymbols.at(index).completionKind(),
+                m_functionSymbols.at(index).completionKind,
                 m_currentArgument + 1);
 
     return signatureWithEmphasizedCurrentParameter;
@@ -64,8 +64,8 @@ QString ClangFunctionHintModel::text(int index) const
 QString ClangFunctionHintModel::id(int index) const
 {
     QString chunks;
-    for (const ClangBackEnd::CodeCompletionChunk &chunk : m_functionSymbols.at(index).chunks())
-        chunks += chunk.text();
+    for (const ClangBackEnd::CodeCompletionChunk &chunk : m_functionSymbols.at(index).chunks)
+        chunks += chunk.text;
 
     return chunks;
 }
