@@ -27,23 +27,26 @@
 
 #include "clangeditordocumentprocessor.h"
 #include "clangmodelmanagersupport.h"
+#include "clangprojectsettings.h"
 
 #include <clangsupport/tokeninfocontainer.h>
 
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
-#include <cpptools/baseeditordocumentparser.h>
-#include <cpptools/compileroptionsbuilder.h>
-#include <cpptools/cppcodemodelsettings.h>
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpptoolsreuse.h>
-#include <cpptools/editordocumenthandle.h>
-#include <cpptools/projectpart.h>
+#include <cppeditor/baseeditordocumentparser.h>
+#include <cppeditor/compileroptionsbuilder.h>
+#include <cppeditor/cppcodemodelsettings.h>
+#include <cppeditor/cppmodelmanager.h>
+#include <cppeditor/cpptoolsreuse.h>
+#include <cppeditor/editordocumenthandle.h>
+#include <cppeditor/projectpart.h>
 #include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/target.h>
+#include <texteditor/codeassist/textdocumentmanipulatorinterface.h>
 
+#include <cplusplus/SimpleLexer.h>
 #include <utils/algorithm.h>
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
@@ -56,13 +59,13 @@
 #include <QStringList>
 #include <QTextBlock>
 
-using namespace ClangCodeModel;
-using namespace ClangCodeModel::Internal;
 using namespace Core;
-using namespace CppTools;
+using namespace CppEditor;
+using namespace ProjectExplorer;
+using namespace Utils;
 
 namespace ClangCodeModel {
-namespace Utils {
+namespace Internal {
 
 class LibClangOptionsBuilder final : public CompilerOptionsBuilder
 {
@@ -75,7 +78,7 @@ public:
                                  UseLanguageDefines::No,
                                  useBuildSystemWarnings,
                                  QString(CLANG_VERSION),
-                                 QString(CLANG_RESOURCE_DIR))
+                                 FilePath(CLANG_INCLUDE_DIR))
     {
     }
 
@@ -99,43 +102,37 @@ private:
     void addDummyUiHeaderOnDiskIncludePath()
     {
         const QString path = ClangModelManagerSupport::instance()->dummyUiHeaderOnDiskDirPath();
-        if (!path.isEmpty())
-            add({"-I", QDir::toNativeSeparators(path)});
+        if (!path.isEmpty()) {
+            prepend(QDir::toNativeSeparators(path));
+            prepend("-I");
+        }
     }
 };
 
-QStringList createClangOptions(const ProjectPart &projectPart,
-                               UseBuildSystemWarnings useBuildSystemWarnings,
-                               ProjectFile::Kind fileKind)
+ProjectPart::ConstPtr projectPartForFile(const QString &filePath)
 {
-    return LibClangOptionsBuilder(projectPart, useBuildSystemWarnings)
-        .build(fileKind, UsePrecompiledHeaders::No);
-}
-
-ProjectPart::Ptr projectPartForFile(const QString &filePath)
-{
-    if (const auto parser = CppTools::BaseEditorDocumentParser::get(filePath))
+    if (const auto parser = CppEditor::BaseEditorDocumentParser::get(filePath))
         return parser->projectPartInfo().projectPart;
-    return ProjectPart::Ptr();
+    return ProjectPart::ConstPtr();
 }
 
-ProjectPart::Ptr projectPartForFileBasedOnProcessor(const QString &filePath)
+ProjectPart::ConstPtr projectPartForFileBasedOnProcessor(const QString &filePath)
 {
     if (const auto processor = ClangEditorDocumentProcessor::get(filePath))
         return processor->projectPart();
-    return ProjectPart::Ptr();
+    return ProjectPart::ConstPtr();
 }
 
-bool isProjectPartLoaded(const ProjectPart::Ptr projectPart)
+bool isProjectPartLoaded(const ProjectPart::ConstPtr projectPart)
 {
     if (projectPart)
-        return CppModelManager::instance()->projectPartForId(projectPart->id());
+        return !CppModelManager::instance()->projectPartForId(projectPart->id()).isNull();
     return false;
 }
 
 QString projectPartIdForFile(const QString &filePath)
 {
-    const ProjectPart::Ptr projectPart = projectPartForFile(filePath);
+    const ProjectPart::ConstPtr projectPart = projectPartForFile(filePath);
 
     if (isProjectPartLoaded(projectPart))
         return projectPart->id(); // OK, Project Part is still loaded
@@ -144,7 +141,7 @@ QString projectPartIdForFile(const QString &filePath)
 
 CppEditorDocumentHandle *cppDocument(const QString &filePath)
 {
-    return CppTools::CppModelManager::instance()->cppEditorDocument(filePath);
+    return CppEditor::CppModelManager::instance()->cppEditorDocument(filePath);
 }
 
 void setLastSentDocumentRevision(const QString &filePath, uint revision)
@@ -173,52 +170,52 @@ int cppEditorColumn(const QTextBlock &line, int clangColumn)
     return QString::fromUtf8(line.text().toUtf8().left(clangColumn - 1)).size() + 1;
 }
 
-::Utils::CodeModelIcon::Type iconTypeForToken(const ClangBackEnd::TokenInfoContainer &token)
+CodeModelIcon::Type iconTypeForToken(const ClangBackEnd::TokenInfoContainer &token)
 {
     const ClangBackEnd::ExtraInfo &extraInfo = token.extraInfo;
     if (extraInfo.signal)
-        return ::Utils::CodeModelIcon::Signal;
+        return CodeModelIcon::Signal;
 
     ClangBackEnd::AccessSpecifier access = extraInfo.accessSpecifier;
     if (extraInfo.slot) {
         switch (access) {
         case ClangBackEnd::AccessSpecifier::Public:
         case ClangBackEnd::AccessSpecifier::Invalid:
-            return ::Utils::CodeModelIcon::SlotPublic;
+            return CodeModelIcon::SlotPublic;
         case ClangBackEnd::AccessSpecifier::Protected:
-            return ::Utils::CodeModelIcon::SlotProtected;
+            return CodeModelIcon::SlotProtected;
         case ClangBackEnd::AccessSpecifier::Private:
-            return ::Utils::CodeModelIcon::SlotPrivate;
+            return CodeModelIcon::SlotPrivate;
         }
     }
 
     ClangBackEnd::HighlightingType mainType = token.types.mainHighlightingType;
 
     if (mainType == ClangBackEnd::HighlightingType::QtProperty)
-        return ::Utils::CodeModelIcon::Property;
+        return CodeModelIcon::Property;
 
     if (mainType == ClangBackEnd::HighlightingType::PreprocessorExpansion
             || mainType == ClangBackEnd::HighlightingType::PreprocessorDefinition) {
-        return ::Utils::CodeModelIcon::Macro;
+        return CodeModelIcon::Macro;
     }
 
     if (mainType == ClangBackEnd::HighlightingType::Enumeration)
-        return ::Utils::CodeModelIcon::Enumerator;
+        return CodeModelIcon::Enumerator;
 
     if (mainType == ClangBackEnd::HighlightingType::Type
             || mainType == ClangBackEnd::HighlightingType::Keyword) {
         const ClangBackEnd::MixinHighlightingTypes &types = token.types.mixinHighlightingTypes;
         if (types.contains(ClangBackEnd::HighlightingType::Enum))
-            return ::Utils::CodeModelIcon::Enum;
+            return CodeModelIcon::Enum;
         if (types.contains(ClangBackEnd::HighlightingType::Struct))
-            return ::Utils::CodeModelIcon::Struct;
+            return CodeModelIcon::Struct;
         if (types.contains(ClangBackEnd::HighlightingType::Namespace))
-            return ::Utils::CodeModelIcon::Namespace;
+            return CodeModelIcon::Namespace;
         if (types.contains(ClangBackEnd::HighlightingType::Class))
-            return ::Utils::CodeModelIcon::Class;
+            return CodeModelIcon::Class;
         if (mainType == ClangBackEnd::HighlightingType::Keyword)
-            return ::Utils::CodeModelIcon::Keyword;
-        return ::Utils::CodeModelIcon::Class;
+            return CodeModelIcon::Keyword;
+        return CodeModelIcon::Class;
     }
 
     ClangBackEnd::StorageClass storageClass = extraInfo.storageClass;
@@ -230,21 +227,21 @@ int cppEditorColumn(const QTextBlock &line, int clangColumn)
             switch (access) {
             case ClangBackEnd::AccessSpecifier::Public:
             case ClangBackEnd::AccessSpecifier::Invalid:
-                return ::Utils::CodeModelIcon::FuncPublic;
+                return CodeModelIcon::FuncPublic;
             case ClangBackEnd::AccessSpecifier::Protected:
-                return ::Utils::CodeModelIcon::FuncProtected;
+                return CodeModelIcon::FuncProtected;
             case ClangBackEnd::AccessSpecifier::Private:
-                return ::Utils::CodeModelIcon::FuncPrivate;
+                return CodeModelIcon::FuncPrivate;
             }
         } else {
             switch (access) {
             case ClangBackEnd::AccessSpecifier::Public:
             case ClangBackEnd::AccessSpecifier::Invalid:
-                return ::Utils::CodeModelIcon::FuncPublicStatic;
+                return CodeModelIcon::FuncPublicStatic;
             case ClangBackEnd::AccessSpecifier::Protected:
-                return ::Utils::CodeModelIcon::FuncProtectedStatic;
+                return CodeModelIcon::FuncProtectedStatic;
             case ClangBackEnd::AccessSpecifier::Private:
-                return ::Utils::CodeModelIcon::FuncPrivateStatic;
+                return CodeModelIcon::FuncPrivateStatic;
             }
         }
     }
@@ -254,26 +251,26 @@ int cppEditorColumn(const QTextBlock &line, int clangColumn)
             switch (access) {
             case ClangBackEnd::AccessSpecifier::Public:
             case ClangBackEnd::AccessSpecifier::Invalid:
-                return ::Utils::CodeModelIcon::VarPublic;
+                return CodeModelIcon::VarPublic;
             case ClangBackEnd::AccessSpecifier::Protected:
-                return ::Utils::CodeModelIcon::VarProtected;
+                return CodeModelIcon::VarProtected;
             case ClangBackEnd::AccessSpecifier::Private:
-                return ::Utils::CodeModelIcon::VarPrivate;
+                return CodeModelIcon::VarPrivate;
             }
         } else {
             switch (access) {
             case ClangBackEnd::AccessSpecifier::Public:
             case ClangBackEnd::AccessSpecifier::Invalid:
-                return ::Utils::CodeModelIcon::VarPublicStatic;
+                return CodeModelIcon::VarPublicStatic;
             case ClangBackEnd::AccessSpecifier::Protected:
-                return ::Utils::CodeModelIcon::VarProtectedStatic;
+                return CodeModelIcon::VarProtectedStatic;
             case ClangBackEnd::AccessSpecifier::Private:
-                return ::Utils::CodeModelIcon::VarPrivateStatic;
+                return CodeModelIcon::VarPrivateStatic;
             }
         }
     }
 
-    return ::Utils::CodeModelIcon::Unknown;
+    return CodeModelIcon::Unknown;
 }
 
 QString diagnosticCategoryPrefixRemoved(const QString &text)
@@ -301,31 +298,10 @@ QString diagnosticCategoryPrefixRemoved(const QString &text)
     return text;
 }
 
-static ::Utils::FilePath compilerPath(const CppTools::ProjectPart &projectPart)
-{
-    ProjectExplorer::Target *target = projectPart.project->activeTarget();
-    if (!target)
-        return ::Utils::FilePath();
-
-    ProjectExplorer::ToolChain *toolchain = ProjectExplorer::ToolChainKitAspect::toolChain(
-        target->kit(), ProjectExplorer::Constants::CXX_LANGUAGE_ID);
-
-    return toolchain->compilerCommand();
-}
-
-static ::Utils::FilePath buildDirectory(const ProjectExplorer::Project &project)
-{
-    if (auto *target = project.activeTarget()) {
-        if (auto *bc = target->activeBuildConfiguration())
-            return bc->buildDirectory();
-    }
-    return {};
-}
-
 static QStringList projectPartArguments(const ProjectPart &projectPart)
 {
     QStringList args;
-    args << compilerPath(projectPart).toString();
+    args << projectPart.compilerFilePath.toString();
     args << "-c";
     if (projectPart.toolchainType != ProjectExplorer::Constants::MSVC_TOOLCHAIN_TYPEID) {
         args << "--target=" + projectPart.toolChainTargetTriple;
@@ -352,46 +328,59 @@ static QStringList projectPartArguments(const ProjectPart &projectPart)
     return args;
 }
 
-static QJsonObject createFileObject(const ::Utils::FilePath &buildDir,
+static QJsonObject createFileObject(const FilePath &buildDir,
                                     const QStringList &arguments,
                                     const ProjectPart &projectPart,
-                                    const ProjectFile &projFile)
+                                    const ProjectFile &projFile,
+                                    CompilationDbPurpose purpose,
+                                    const ClangDiagnosticConfig &warningsConfig,
+                                    const QStringList &projectOptions)
 {
     QJsonObject fileObject;
     fileObject["file"] = projFile.path;
-    QJsonArray args = QJsonArray::fromStringList(arguments);
+    QJsonArray args;
 
-    const ProjectFile::Kind kind = ProjectFile::classify(projFile.path);
-    if (projectPart.toolchainType == ProjectExplorer::Constants::MSVC_TOOLCHAIN_TYPEID
-        || projectPart.toolchainType == ProjectExplorer::Constants::CLANG_CL_TOOLCHAIN_TYPEID) {
-        if (ProjectFile::isC(kind))
-            args.append("/TC");
-        else if (ProjectFile::isCxx(kind))
-            args.append("/TP");
+    if (purpose == CompilationDbPurpose::Project) {
+        args = QJsonArray::fromStringList(arguments);
+
+        const ProjectFile::Kind kind = ProjectFile::classify(projFile.path);
+        if (projectPart.toolchainType == ProjectExplorer::Constants::MSVC_TOOLCHAIN_TYPEID
+                || projectPart.toolchainType == ProjectExplorer::Constants::CLANG_CL_TOOLCHAIN_TYPEID) {
+            if (ProjectFile::isC(kind))
+                args.append("/TC");
+            else if (ProjectFile::isCxx(kind))
+                args.append("/TP");
+        } else {
+            QStringList langOption
+                    = createLanguageOptionGcc(kind,
+                                              projectPart.languageExtensions
+                                              & LanguageExtension::ObjectiveC);
+            for (const QString &langOptionPart : langOption)
+                args.append(langOptionPart);
+        }
     } else {
-        QStringList langOption
-            = createLanguageOptionGcc(kind,
-                                      projectPart.languageExtensions
-                                          & ::Utils::LanguageExtension::ObjectiveC);
-        for (const QString &langOptionPart : langOption)
-            args.append(langOptionPart);
+        // TODO: Do we really need to re-calculate the project part options per source file?
+        args = QJsonArray::fromStringList(createClangOptions(projectPart, projFile.path,
+                                                             warningsConfig, projectOptions));
+        args.prepend("clang"); // TODO: clang-cl for MSVC targets? Does it matter at all what we put here?
     }
+
     args.append(QDir::toNativeSeparators(projFile.path));
     fileObject["arguments"] = args;
     fileObject["directory"] = buildDir.toString();
     return fileObject;
 }
 
-GenerateCompilationDbResult generateCompilationDB(CppTools::ProjectInfo projectInfo)
+GenerateCompilationDbResult generateCompilationDB(const CppEditor::ProjectInfo::ConstPtr projectInfo,
+                                                  const Utils::FilePath &baseDir,
+                                                  CompilationDbPurpose purpose,
+                                                  const ClangDiagnosticConfig &warningsConfig,
+                                                  const QStringList &projectOptions)
 {
-    const ::Utils::FilePath buildDir = buildDirectory(*projectInfo.project());
-    QTC_ASSERT(!buildDir.isEmpty(), return GenerateCompilationDbResult(QString(),
+    QTC_ASSERT(!baseDir.isEmpty(), return GenerateCompilationDbResult(QString(),
         QCoreApplication::translate("ClangUtils", "Could not retrieve build directory.")));
-
-    QDir dir(buildDir.toString());
-    if (!dir.exists())
-        dir.mkpath(dir.path());
-    QFile compileCommandsFile(buildDir.toString() + "/compile_commands.json");
+    QTC_CHECK(baseDir.ensureWritableDir());
+    QFile compileCommandsFile(baseDir.toString() + "/compile_commands.json");
     const bool fileOpened = compileCommandsFile.open(QIODevice::WriteOnly | QIODevice::Truncate);
     if (!fileOpened) {
         return GenerateCompilationDbResult(QString(),
@@ -400,10 +389,13 @@ GenerateCompilationDbResult generateCompilationDB(CppTools::ProjectInfo projectI
     }
     compileCommandsFile.write("[");
 
-    for (ProjectPart::Ptr projectPart : projectInfo.projectParts()) {
-        const QStringList args = projectPartArguments(*projectPart);
+    for (ProjectPart::ConstPtr projectPart : projectInfo->projectParts()) {
+        QStringList args;
+        if (purpose == CompilationDbPurpose::Project)
+            args = projectPartArguments(*projectPart);
         for (const ProjectFile &projFile : projectPart->files) {
-            const QJsonObject json = createFileObject(buildDir, args, *projectPart, projFile);
+            const QJsonObject json = createFileObject(baseDir, args, *projectPart, projFile,
+                                                      purpose, warningsConfig, projectOptions);
             if (compileCommandsFile.size() > 1)
                 compileCommandsFile.write(",");
             compileCommandsFile.write('\n' + QJsonDocument(json).toJson().trimmed());
@@ -420,7 +412,7 @@ QString currentCppEditorDocumentFilePath()
     QString filePath;
 
     const auto currentEditor = Core::EditorManager::currentEditor();
-    if (currentEditor && CppTools::CppModelManager::isCppEditor(currentEditor)) {
+    if (currentEditor && CppEditor::CppModelManager::isCppEditor(currentEditor)) {
         const auto currentDocument = currentEditor->document();
         if (currentDocument)
             filePath = currentDocument->filePath().toString();
@@ -475,5 +467,189 @@ QString DiagnosticTextInfo::clazyCheckName(const QString &option)
     return option;
 }
 
-} // namespace Utils
+
+namespace {
+static ClangProjectSettings &getProjectSettings(ProjectExplorer::Project *project)
+{
+    QTC_CHECK(project);
+    return ClangModelManagerSupport::instance()->projectSettings(project);
+}
+
+// TODO: Can we marry this with CompilerOptionsBuilder?
+class FileOptionsBuilder
+{
+public:
+    FileOptionsBuilder(const QString &filePath, const CppEditor::ProjectPart &projectPart,
+                       const ClangDiagnosticConfig &warningsConfig,
+                       const QStringList &projectOptions)
+        : m_filePath(filePath)
+        , m_projectPart(projectPart)
+        , m_warningsConfig(warningsConfig)
+        , m_builder(projectPart)
+    {
+        // Determine the driver mode from toolchain and flags.
+        m_builder.evaluateCompilerFlags();
+        m_isClMode = m_builder.isClStyle();
+
+        addLanguageOptions();
+        addGlobalDiagnosticOptions(); // Before addDiagnosticOptions() so users still can overwrite.
+        addDiagnosticOptions();
+        m_options.append(projectOptions);
+        addPrecompiledHeaderOptions();
+    }
+
+    const QStringList &options() const { return m_options; }
+    CppEditor::UseBuildSystemWarnings useBuildSystemWarnings() const
+    {
+        return m_useBuildSystemWarnings;
+    }
+
+private:
+    void addLanguageOptions()
+    {
+        // Determine file kind with respect to ambiguous headers.
+        CppEditor::ProjectFile::Kind fileKind = CppEditor::ProjectFile::Unclassified;
+        if (!m_filePath.isEmpty())
+            fileKind = CppEditor::ProjectFile::classify(m_filePath);
+        if (fileKind == CppEditor::ProjectFile::AmbiguousHeader) {
+            fileKind = m_projectPart.languageVersion <= ::Utils::LanguageVersion::LatestC
+                 ? CppEditor::ProjectFile::CHeader
+                 : CppEditor::ProjectFile::CXXHeader;
+        }
+
+        m_builder.reset();
+        m_builder.updateFileLanguage(fileKind);
+
+        m_options.append(m_builder.options());
+    }
+
+    void addDiagnosticOptions()
+    {
+        addDiagnosticOptionsForConfig(m_warningsConfig);
+    }
+
+    void addDiagnosticOptionsForConfig(const CppEditor::ClangDiagnosticConfig &diagnosticConfig)
+    {
+        m_useBuildSystemWarnings = diagnosticConfig.useBuildSystemWarnings()
+                                       ? CppEditor::UseBuildSystemWarnings::Yes
+                                       : CppEditor::UseBuildSystemWarnings::No;
+
+        const QStringList options = m_isClMode
+                                        ? CppEditor::clangArgsForCl(diagnosticConfig.clangOptions())
+                                        : diagnosticConfig.clangOptions();
+        m_options.append(options);
+    }
+
+    void addGlobalDiagnosticOptions()
+    {
+        m_options += CppEditor::ClangDiagnosticConfigsModel::globalDiagnosticOptions();
+    }
+
+    void addPrecompiledHeaderOptions()
+    {
+        using namespace CppEditor;
+
+        if (getPchUsage() == UsePrecompiledHeaders::No)
+            return;
+
+        if (m_projectPart.precompiledHeaders.contains(m_filePath))
+            return;
+
+        m_builder.reset();
+        m_builder.addPrecompiledHeaderOptions(UsePrecompiledHeaders::Yes);
+
+        m_options.append(m_builder.options());
+    }
+
+private:
+    const QString &m_filePath;
+    const CppEditor::ProjectPart &m_projectPart;
+    const ClangDiagnosticConfig &m_warningsConfig;
+
+    CppEditor::UseBuildSystemWarnings m_useBuildSystemWarnings = CppEditor::UseBuildSystemWarnings::No;
+    CppEditor::CompilerOptionsBuilder m_builder;
+    bool m_isClMode = false;
+    QStringList m_options;
+};
+} // namespace
+
+QStringList createClangOptions(const ProjectPart &projectPart, const QString &filePath,
+                               const ClangDiagnosticConfig &warningsConfig,
+                               const QStringList &projectOptions)
+{
+    const FileOptionsBuilder fileOptions(filePath, projectPart, warningsConfig, projectOptions);
+    LibClangOptionsBuilder optionsBuilder(projectPart, fileOptions.useBuildSystemWarnings());
+    const QStringList projectPartOptions = optionsBuilder.build(CppEditor::ProjectFile::Unsupported,
+                                                                UsePrecompiledHeaders::No);
+
+    // FIXME: Sanitize FileOptionsBuilder instead.
+    QStringList fileArgs = fileOptions.options();
+    if (projectPartOptions.contains(QLatin1String("-TP")))
+        fileArgs.removeAll(QLatin1String("/TP"));
+    if (projectPartOptions.contains(QLatin1String("-TC")))
+        fileArgs.removeAll(QLatin1String("/TC"));
+
+    return projectPartOptions + fileArgs;
+}
+
+ClangDiagnosticConfig warningsConfigForProject(Project *project)
+{
+    if (project) {
+        ClangProjectSettings &projectSettings = ClangModelManagerSupport::instance()
+                ->projectSettings(project);
+        if (!projectSettings.useGlobalConfig()) {
+            const Utils::Id warningConfigId = projectSettings.warningConfigId();
+            const CppEditor::ClangDiagnosticConfigsModel configsModel
+                    = CppEditor::diagnosticConfigsModel();
+            if (configsModel.hasConfigWithId(warningConfigId))
+                return configsModel.configWithId(warningConfigId);
+        }
+    }
+    return CppEditor::codeModelSettings()->clangDiagnosticConfig();
+}
+
+const QStringList optionsForProject(ProjectExplorer::Project *project)
+{
+    if (project)
+        return getProjectSettings(project).commandLineOptions();
+    return ClangProjectSettings::globalCommandLineOptions();
+}
+
+// 7.3.3: using typename(opt) nested-name-specifier unqualified-id ;
+bool isAtUsingDeclaration(TextEditor::TextDocumentManipulatorInterface &manipulator,
+                          int basePosition)
+{
+    using namespace CPlusPlus;
+    SimpleLexer lexer;
+    lexer.setLanguageFeatures(LanguageFeatures::defaultFeatures());
+    const QString textToLex = textUntilPreviousStatement(manipulator, basePosition);
+    const Tokens tokens = lexer(textToLex);
+    if (tokens.empty())
+        return false;
+
+    // The nested-name-specifier always ends with "::", so check for this first.
+    const Token lastToken = tokens[tokens.size() - 1];
+    if (lastToken.kind() != T_COLON_COLON)
+        return false;
+
+    return contains(tokens, [](const Token &token) { return token.kind() == T_USING; });
+}
+
+QString textUntilPreviousStatement(TextEditor::TextDocumentManipulatorInterface &manipulator,
+                                   int startPosition)
+{
+    static const QString stopCharacters(";{}#");
+
+    int endPosition = 0;
+    for (int i = startPosition; i >= 0 ; --i) {
+        if (stopCharacters.contains(manipulator.characterAt(i))) {
+            endPosition = i + 1;
+            break;
+        }
+    }
+
+    return manipulator.textAt(endPosition, startPosition - endPosition);
+}
+
+} // namespace Internal
 } // namespace Clang
