@@ -27,35 +27,43 @@
 
 #include <cplusplus/Icons.h>
 
-#include <cpptools/projectpart.h>
-#include <cpptools/compileroptionsbuilder.h>
+#include <cppeditor/projectinfo.h>
+#include <cppeditor/compileroptionsbuilder.h>
 
+#include <QPair>
 #include <QTextCursor>
 
 QT_BEGIN_NAMESPACE
 class QTextBlock;
 QT_END_NAMESPACE
 
-namespace CppTools {
+namespace CppEditor {
+class ClangDiagnosticConfig;
 class CppEditorDocumentHandle;
-class ProjectInfo;
 }
 
+namespace TextEditor { class TextDocumentManipulatorInterface; }
+
 namespace ClangBackEnd { class TokenInfoContainer; }
 
+namespace ProjectExplorer { class Project; }
+
 namespace ClangCodeModel {
 namespace Internal {
 
-CppTools::CppEditorDocumentHandle *cppDocument(const QString &filePath);
+CppEditor::CppEditorDocumentHandle *cppDocument(const QString &filePath);
 void setLastSentDocumentRevision(const QString &filePath, uint revision);
 
-QStringList createClangOptions(const CppTools::ProjectPart &projectPart,
-                               CppTools::UseBuildSystemWarnings useBuildSystemWarnings,
-                               CppTools::ProjectFile::Kind fileKind);
+CppEditor::ClangDiagnosticConfig warningsConfigForProject(ProjectExplorer::Project *project);
+const QStringList optionsForProject(ProjectExplorer::Project *project);
 
-CppTools::ProjectPart::Ptr projectPartForFile(const QString &filePath);
-CppTools::ProjectPart::Ptr projectPartForFileBasedOnProcessor(const QString &filePath);
-bool isProjectPartLoaded(const CppTools::ProjectPart::Ptr projectPart);
+QStringList createClangOptions(const CppEditor::ProjectPart &projectPart, const QString &filePath,
+                               const CppEditor::ClangDiagnosticConfig &warningsConfig,
+                               const QStringList &projectOptions);
+
+CppEditor::ProjectPart::ConstPtr projectPartForFile(const QString &filePath);
+CppEditor::ProjectPart::ConstPtr projectPartForFileBasedOnProcessor(const QString &filePath);
+bool isProjectPartLoaded(const CppEditor::ProjectPart::ConstPtr projectPart);
 QString projectPartIdForFile(const QString &filePath);
 int clangColumn(const QTextBlock &line, int cppEditorColumn);
 int cppEditorColumn(const QTextBlock &line, int clangColumn);
@@ -78,7 +86,10 @@ public:
     QString error;
 };
 
-GenerateCompilationDbResult generateCompilationDB(CppTools::ProjectInfo projectInfo);
+enum class CompilationDbPurpose { Project, CodeModel };
+GenerateCompilationDbResult generateCompilationDB(const CppEditor::ProjectInfo::ConstPtr projectInfo,
+        const Utils::FilePath &baseDir, CompilationDbPurpose purpose,
+        const CppEditor::ClangDiagnosticConfig &warningsConfig, const QStringList &projectOptions);
 
 class DiagnosticTextInfo
 {
@@ -98,7 +109,7 @@ private:
 };
 
 template <class CharacterProvider>
-void moveToPreviousChar(CharacterProvider &provider, QTextCursor &cursor)
+void moveToPreviousChar(const CharacterProvider &provider, QTextCursor &cursor)
 {
     cursor.movePosition(QTextCursor::PreviousCharacter);
     while (provider.characterAt(cursor.position()).isSpace())
@@ -114,7 +125,7 @@ void moveToPreviousWord(CharacterProvider &provider, QTextCursor &cursor)
 }
 
 template <class CharacterProvider>
-bool matchPreviousWord(CharacterProvider &provider, QTextCursor cursor, QString pattern)
+bool matchPreviousWord(const CharacterProvider &provider, QTextCursor cursor, QString pattern)
 {
     cursor.movePosition(QTextCursor::PreviousWord);
     while (provider.characterAt(cursor.position()) == ':')
@@ -142,5 +153,11 @@ bool matchPreviousWord(CharacterProvider &provider, QTextCursor cursor, QString
     return pattern.isEmpty();
 }
 
+QString textUntilPreviousStatement(TextEditor::TextDocumentManipulatorInterface &manipulator,
+                                   int startPosition);
+
+bool isAtUsingDeclaration(TextEditor::TextDocumentManipulatorInterface &manipulator,
+                          int basePosition);
+
 } // namespace Internal
 } // namespace Clang
