@@ -61,8 +61,9 @@ public:
         bool gotResults = false;
 
         processor->setAsyncCompletionAvailableHandler(
-                    [this, &gotResults] (TextEditor::IAssistProposal *proposal) {
+                    [this, processor, &gotResults] (TextEditor::IAssistProposal *proposal) {
             QTC_ASSERT(proposal, return);
+            QTC_CHECK(!processor->running());
             proposalModel = proposal->model();
             delete proposal;
             gotResults = true;
@@ -94,14 +95,6 @@ public:
     TextEditor::ProposalModelPtr proposalModel;
 };
 
-static const ProjectExplorer::HeaderPaths toHeaderPaths(const QStringList &paths)
-{
-    ProjectExplorer::HeaderPaths result;
-    foreach (const QString &path, paths)
-        result.push_back({path, ProjectExplorer::HeaderPathType::User});
-    return result;
-}
-
 TextEditor::ProposalModelPtr completionResults(TextEditor::BaseTextEditor *textEditor,
                                                const QStringList &includePaths,
                                                int timeOutInMs)
@@ -115,7 +108,7 @@ TextEditor::ProposalModelPtr completionResults(TextEditor::BaseTextEditor *textE
     QTC_ASSERT(assistInterface, return TextEditor::ProposalModelPtr());
     if (!includePaths.isEmpty()) {
         auto clangAssistInterface = static_cast<ClangCompletionAssistInterface *>(assistInterface);
-        clangAssistInterface->setHeaderPaths(toHeaderPaths(includePaths));
+        clangAssistInterface->setHeaderPaths(ProjectExplorer::toUserHeaderPaths(includePaths));
     }
 
     CompletionAssistProvider *assistProvider
@@ -126,7 +119,7 @@ TextEditor::ProposalModelPtr completionResults(TextEditor::BaseTextEditor *textE
     QTC_ASSERT(assistProvider->runType() == IAssistProvider::Asynchronous,
                return TextEditor::ProposalModelPtr());
 
-    QScopedPointer<IAssistProcessor> processor(assistProvider->createProcessor());
+    QScopedPointer<IAssistProcessor> processor(assistProvider->createProcessor(assistInterface));
     QTC_ASSERT(processor, return TextEditor::ProposalModelPtr());
 
     WaitForAsyncCompletions waitForCompletions;
@@ -138,5 +131,10 @@ TextEditor::ProposalModelPtr completionResults(TextEditor::BaseTextEditor *textE
     return waitForCompletions.proposalModel;
 }
 
+QString qrcPath(const QByteArray &relativeFilePath)
+{
+    return QLatin1String(":/unittests/ClangCodeModel/") + QString::fromUtf8(relativeFilePath);
+}
+
 } // namespace Internal
 } // namespace ClangCodeModel
