@@ -32,10 +32,10 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/icore.h>
-#include <cpptools/cpptoolsreuse.h>
-#include <cpptools/cpptoolstestcase.h>
-#include <cpptools/modelmanagertesthelper.h>
-#include <cpptools/projectinfo.h>
+#include <cppeditor/cpptoolsreuse.h>
+#include <cppeditor/cpptoolstestcase.h>
+#include <cppeditor/modelmanagertesthelper.h>
+#include <cppeditor/projectinfo.h>
 #include <projectexplorer/projectexplorer.h>
 #include <texteditor/codeassist/assistinterface.h>
 #include <texteditor/codeassist/assistproposalitem.h>
@@ -58,10 +58,11 @@
 #include <QThread>
 
 using namespace ClangBackEnd;
-using namespace ClangCodeModel;
-using namespace ClangCodeModel::Internal;
 using namespace ProjectExplorer;
 
+namespace ClangCodeModel {
+namespace Internal {
+
 static Q_LOGGING_CATEGORY(debug, "qtc.clangcodemodel.batch", QtWarningMsg);
 
 static int timeOutFromEnvironmentVariable()
@@ -78,7 +79,7 @@ static int timeOutFromEnvironmentVariable()
     return intervalAsInt;
 }
 
-static int timeOutInMs()
+int timeOutInMs()
 {
     static int timeOut = timeOutFromEnvironmentVariable();
     return timeOut;
@@ -231,13 +232,13 @@ bool OpenProjectCommand::run()
     qCDebug(debug) << "line" << context().lineNumber << "OpenProjectCommand" << m_projectFilePath;
 
     const ProjectExplorerPlugin::OpenProjectResult openProjectSucceeded
-            = ProjectExplorerPlugin::openProject(m_projectFilePath);
+            = ProjectExplorerPlugin::openProject(Utils::FilePath::fromString(m_projectFilePath));
     QTC_ASSERT(openProjectSucceeded, return false);
 
     Project *project = openProjectSucceeded.project();
-    project->configureAsExampleProject();
+    project->configureAsExampleProject(nullptr);
 
-    return CppTools::Tests::TestCase::waitUntilProjectIsFullyOpened(project, timeOutInMs());
+    return CppEditor::Tests::TestCase::waitUntilProjectIsFullyOpened(project, timeOutInMs());
 }
 
 Command::Ptr OpenProjectCommand::parse(BatchFileLineTokenizer &arguments,
@@ -279,8 +280,6 @@ OpenDocumentCommand::OpenDocumentCommand(const CommandContext &context,
 
 class WaitForUpdatedCodeWarnings : public QObject
 {
-    Q_OBJECT
-
 public:
     WaitForUpdatedCodeWarnings(ClangEditorDocumentProcessor *processor);
 
@@ -469,7 +468,7 @@ bool CompleteCommand::run()
     auto *processor = ClangEditorDocumentProcessor::get(documentFilePath);
     QTC_ASSERT(processor, return false);
 
-    return completionResults(editor, QStringList(), timeOutInMs());
+    return !completionResults(editor, QStringList(), timeOutInMs()).isNull();
 }
 
 Command::Ptr CompleteCommand::parse(BatchFileLineTokenizer &arguments,
@@ -747,9 +746,6 @@ bool BatchFileParser::parseLine(const QString &line)
 
 } // anonymous namespace
 
-namespace ClangCodeModel {
-namespace Internal {
-
 static QString applySubstitutions(const QString &filePath, const QString &text)
 {
     const QString dirPath = QFileInfo(filePath).absolutePath();
@@ -796,5 +792,3 @@ bool runClangBatchFile(const QString &filePath)
 
 } // namespace Internal
 } // namespace ClangCodeModel
-
-#include "clangbatchfileprocessor.moc"
