@@ -44,9 +44,9 @@
 
 #include <cppeditor/cppeditorconstants.h>
 
-#include <cpptools/cppcodestylepreferencesfactory.h>
-#include <cpptools/cpptoolsconstants.h>
-#include <cpptools/cppmodelmanager.h>
+#include <cppeditor/cppcodestylepreferencesfactory.h>
+#include <cppeditor/cppeditorconstants.h>
+#include <cppeditor/cppmodelmanager.h>
 
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/project.h>
@@ -59,6 +59,7 @@
 #include <clang/Format/Format.h>
 
 #include <utils/algorithm.h>
+#include <utils/infobar.h>
 
 #include <QAction>
 #include <QDebug>
@@ -66,16 +67,17 @@
 #include <QMessageBox>
 #include <QMenu>
 
+using namespace Core;
 using namespace ProjectExplorer;
+using namespace Utils;
 
 namespace ClangFormat {
 
-#ifdef KEEP_LINE_BREAKS_FOR_NON_EMPTY_LINES_BACKPORTED
-class ClangFormatStyleFactory : public CppTools::CppCodeStylePreferencesFactory
+class ClangFormatStyleFactory : public CppEditor::CppCodeStylePreferencesFactory
 {
 public:
     TextEditor::CodeStyleEditorWidget *createCodeStyleEditor(
-        TextEditor::ICodeStylePreferences *preferences, QWidget *parent = nullptr) override
+        TextEditor::ICodeStylePreferences *preferences, QWidget *parent) override
     {
         Q_UNUSED(preferences);
         if (!parent)
@@ -97,73 +99,70 @@ public:
 static void replaceCppCodeStyle()
 {
     using namespace TextEditor;
-    TextEditorSettings::unregisterCodeStyleFactory(CppTools::Constants::CPP_SETTINGS_ID);
-    ICodeStylePreferencesFactory *factory = new ClangFormatStyleFactory();
-    TextEditorSettings::registerCodeStyleFactory(factory);
+    TextEditorSettings::unregisterCodeStyleFactory(CppEditor::Constants::CPP_SETTINGS_ID);
+    TextEditorSettings::registerCodeStyleFactory(new ClangFormatStyleFactory);
 }
-#endif
 
 bool ClangFormatPlugin::initialize(const QStringList &arguments, QString *errorString)
 {
     Q_UNUSED(arguments)
     Q_UNUSED(errorString)
-#ifdef KEEP_LINE_BREAKS_FOR_NON_EMPTY_LINES_BACKPORTED
     replaceCppCodeStyle();
 
-    Core::ActionContainer *contextMenu = Core::ActionManager::actionContainer(
-        CppEditor::Constants::M_CONTEXT);
+    ActionContainer *contextMenu = ActionManager::actionContainer(CppEditor::Constants::M_CONTEXT);
     if (contextMenu) {
         auto openClangFormatConfigAction
             = new QAction(tr("Open Used .clang-format Configuration File"), this);
-        Core::Command *command
-            = Core::ActionManager::registerAction(openClangFormatConfigAction,
-                                                  Constants::OPEN_CURRENT_CONFIG_ID);
+        Command *command = ActionManager::registerAction(openClangFormatConfigAction,
+                                                         Constants::OPEN_CURRENT_CONFIG_ID);
         contextMenu->addSeparator();
         contextMenu->addAction(command);
 
-        if (Core::EditorManager::currentEditor()) {
-            const Core::IDocument *doc = Core::EditorManager::currentEditor()->document();
-            if (doc)
-                openClangFormatConfigAction->setData(doc->filePath().toString());
+        if (EditorManager::currentEditor()) {
+            if (const IDocument *doc = EditorManager::currentEditor()->document())
+                openClangFormatConfigAction->setData(doc->filePath().toVariant());
         }
 
         connect(openClangFormatConfigAction,
                 &QAction::triggered,
                 this,
                 [openClangFormatConfigAction]() {
-                    const QString fileName = openClangFormatConfigAction->data().toString();
-                    if (!fileName.isEmpty()) {
-                        const QString clangFormatConfigPath = configForFile(
-                            Utils::FilePath::fromString(fileName));
-                        Core::EditorManager::openEditor(clangFormatConfigPath);
-                    }
+                    const FilePath fileName = FilePath::fromVariant(openClangFormatConfigAction->data());
+                    if (!fileName.isEmpty())
+                        EditorManager::openEditor(configForFile(fileName));
                 });
 
-        connect(Core::EditorManager::instance(),
-                &Core::EditorManager::currentEditorChanged,
+        connect(EditorManager::instance(),
+                &EditorManager::currentEditorChanged,
                 this,
-                [openClangFormatConfigAction](Core::IEditor *editor) {
+                [openClangFormatConfigAction](IEditor *editor) {
                     if (!editor)
                         return;
 
-                    const Core::IDocument *doc = editor->document();
-                    if (doc)
-                        openClangFormatConfigAction->setData(doc->filePath().toString());
+                    if (const IDocument *doc = editor->document())
+                        openClangFormatConfigAction->setData(doc->filePath().toVariant());
                 });
     }
-    return true;
-#else
+#ifndef KEEP_LINE_BREAKS_FOR_NON_EMPTY_LINES_BACKPORTED
 #ifdef _MSC_VER
 #pragma message( \
-    "ClangFormat: building dummy plugin due to unmodified Clang, see README.md for more info")
+    "ClangFormat: building against unmodified Clang, see README.md for more info")
 #else
-#warning ClangFormat: building dummy plugin due to unmodified Clang, see README.md for more info
+#warning ClangFormat: building against unmodified Clang, see README.md for more info
 #endif
-    *errorString = "Disabling ClangFormat plugin as it has not been built against a modified Clang's libFormat."
-                   "For more information see the Qt Creator README at "
-                   "https://code.qt.io/cgit/qt-creator/qt-creator.git/tree/README.md";
-    return false;
+    static const Id clangFormatFormatWarningKey = "ClangFormatFormatWarning";
+    if (!ICore::infoBar()->canInfoBeAdded(clangFormatFormatWarningKey))
+        return true;
+    InfoBarEntry
+        info(clangFormatFormatWarningKey,
+             tr("The ClangFormat plugin has been built against an unmodified Clang. "
+                "You might experience formatting glitches in certain circumstances. "
+                "See https://code.qt.io/cgit/qt-creator/qt-creator.git/tree/README.md for more "
+                 "information."),
+             InfoBarEntry::GlobalSuppression::Enabled);
+    ICore::infoBar()->addInfo(info);
 #endif
+    return true;
 }
 
 } // namespace ClangFormat
