@@ -29,7 +29,7 @@
 #include "clangformatsettings.h"
 
 #include <coreplugin/icore.h>
-#include <cpptools/cppcodestylesettings.h>
+#include <cppeditor/cppcodestylesettings.h>
 #include <texteditor/tabsettings.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/session.h>
@@ -39,9 +39,10 @@
 using namespace clang;
 using namespace format;
 using namespace llvm;
-using namespace CppTools;
+using namespace CppEditor;
 using namespace ProjectExplorer;
 using namespace TextEditor;
+using namespace Utils;
 
 namespace ClangFormat {
 
@@ -51,8 +52,13 @@ static clang::format::FormatStyle qtcStyle()
     style.Language = FormatStyle::LK_Cpp;
     style.AccessModifierOffset = -4;
     style.AlignAfterOpenBracket = FormatStyle::BAS_Align;
+#if LLVM_VERSION_MAJOR >= 12
+    style.AlignConsecutiveAssignments = FormatStyle::ACS_None;
+    style.AlignConsecutiveDeclarations = FormatStyle::ACS_None;
+#else
     style.AlignConsecutiveAssignments = false;
     style.AlignConsecutiveDeclarations = false;
+#endif
     style.AlignEscapedNewlines = FormatStyle::ENAS_DontAlign;
 #if LLVM_VERSION_MAJOR >= 11
     style.AlignOperands = FormatStyle::OAS_Align;
@@ -115,7 +121,11 @@ static clang::format::FormatStyle qtcStyle()
     style.ExperimentalAutoDetectBinPacking = false;
     style.FixNamespaceComments = true;
     style.ForEachMacros = {"forever", "foreach", "Q_FOREACH", "BOOST_FOREACH"};
+#if LLVM_VERSION_MAJOR >= 12
+    style.IncludeStyle.IncludeCategories = {{"^<Q.*", 200, 200, true}};
+#else
     style.IncludeStyle.IncludeCategories = {{"^<Q.*", 200, 200}};
+#endif
     style.IncludeStyle.IncludeIsMainRegex = "(Test)?$";
     style.IndentCaseLabels = false;
     style.IndentWidth = 4;
@@ -140,7 +150,11 @@ static clang::format::FormatStyle qtcStyle()
     style.PenaltyReturnTypeOnItsOwnLine = 300;
     style.PointerAlignment = FormatStyle::PAS_Right;
     style.ReflowComments = false;
+#if LLVM_VERSION_MAJOR >= 13
+    style.SortIncludes = FormatStyle::SI_CaseSensitive;
+#else
     style.SortIncludes = true;
+#endif
     style.SortUsingDeclarations = true;
     style.SpaceAfterCStyleCast = true;
     style.SpaceAfterTemplateKeyword = false;
@@ -148,11 +162,18 @@ static clang::format::FormatStyle qtcStyle()
     style.SpaceBeforeParens = FormatStyle::SBPO_ControlStatements;
     style.SpaceInEmptyParentheses = false;
     style.SpacesBeforeTrailingComments = 1;
+#if LLVM_VERSION_MAJOR >= 13
+    style.SpacesInAngles = FormatStyle::SIAS_Never;
+#else
     style.SpacesInAngles = false;
+#endif
     style.SpacesInContainerLiterals = false;
     style.SpacesInCStyleCastParentheses = false;
     style.SpacesInParentheses = false;
     style.SpacesInSquareBrackets = false;
+    style.StatementMacros.emplace_back("Q_OBJECT");
+    style.StatementMacros.emplace_back("QT_BEGIN_NAMESPACE");
+    style.StatementMacros.emplace_back("QT_END_NAMESPACE");
     style.Standard = FormatStyle::LS_Cpp11;
     style.TabWidth = 4;
     style.UseTab = FormatStyle::UT_Never;
@@ -183,7 +204,7 @@ static bool useProjectOverriddenSettings()
 
 static Utils::FilePath globalPath()
 {
-    return Utils::FilePath::fromString(Core::ICore::userResourcePath());
+    return Core::ICore::userResourcePath();
 }
 
 static Utils::FilePath projectPath()
@@ -259,25 +280,25 @@ static clang::format::FormatStyle constructStyle(const QByteArray &baseStyle = Q
 
 void createStyleFileIfNeeded(bool isGlobal)
 {
-    const Utils::FilePath path = isGlobal ? globalPath() : projectPath();
-    const QString configFile = path.pathAppended(Constants::SETTINGS_FILE_NAME).toString();
+    const FilePath path = isGlobal ? globalPath() : projectPath();
+    const FilePath configFile = path / Constants::SETTINGS_FILE_NAME;
 
-    if (QFile::exists(configFile))
+    if (configFile.exists())
         return;
 
     QDir().mkpath(path.toString());
     if (!isGlobal) {
         const Project *project = SessionManager::startupProject();
-        Utils::FilePath possibleProjectConfig = project->rootProjectDirectory().pathAppended(
-            Constants::SETTINGS_FILE_NAME);
+        FilePath possibleProjectConfig = project->rootProjectDirectory()
+                / Constants::SETTINGS_FILE_NAME;
         if (possibleProjectConfig.exists()) {
             // Just copy th .clang-format if current project has one.
-            QFile::copy(possibleProjectConfig.toString(), configFile);
+            possibleProjectConfig.copyFile(configFile);
             return;
         }
     }
 
-    std::fstream newStyleFile(configFile.toStdString(), std::fstream::out);
+    std::fstream newStyleFile(configFile.toString().toStdString(), std::fstream::out);
     if (newStyleFile.is_open()) {
         newStyleFile << clang::format::configurationAsText(constructStyle());
         newStyleFile.close();
