@@ -26,9 +26,9 @@
 #include "clangfixitsrefactoringchanges.h"
 
 #include <coreplugin/editormanager/editormanager.h>
-#include <cpptools/cppcodestylesettings.h>
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpptoolsconstants.h>
+#include <cppeditor/cppcodestylesettings.h>
+#include <cppeditor/cppeditorconstants.h>
+#include <cppeditor/cppmodelmanager.h>
 
 #include <texteditor/icodestylepreferencesfactory.h>
 #include <texteditor/indenter.h>
@@ -36,11 +36,11 @@
 #include <texteditor/texteditorsettings.h>
 
 #include <QDebug>
-#include <QFileInfo>
 #include <QLoggingCategory>
 #include <QTextBlock>
 #include <QTextCursor>
 
+#include <utils/filepath.h>
 #include <utils/qtcassert.h>
 
 #include <algorithm>
@@ -79,41 +79,45 @@ bool FixitsRefactoringFile::apply()
         return false; // Error nothing to apply TODO: Is this correct to return?
 
     ICodeStylePreferencesFactory *factory = TextEditorSettings::codeStyleFactory(
-        CppTools::Constants::CPP_SETTINGS_ID);
+        CppEditor::Constants::CPP_SETTINGS_ID);
 
-    // Apply changes
-    std::unique_ptr<TextEditor::Indenter> indenter;
-    QString lastFilename;
-    ReplacementOperations operationsForFile;
+    QHash<FilePath, QPair<ReplacementOperations, int>> operationsByFile;
 
-    for (int i=0; i < m_replacementOperations.size(); ++i) {
+    for (int i = 0; i < m_replacementOperations.size(); ++i) {
         ReplacementOperation &op = *m_replacementOperations[i];
-        if (op.apply) {
-            // Check for permissions
-            if (!QFileInfo(op.fileName).isWritable())
-                return false; // Error file not writable
+        if (!op.apply)
+            continue;
 
-            qCDebug(fixitsLog) << " " << i << "Applying" << op;
+        const FilePath filePath = FilePath::fromString(op.fileName);
 
-            // Shift subsequent operations that are affected
-            shiftAffectedReplacements(op, i + 1);
+        // Check for permissions
+        if (!filePath.isWritableFile())
+            return false;
 
-            // Apply
-            QTextDocument *doc = document(op.fileName);
-            if (lastFilename != op.fileName) {
-                if (indenter)
-                    format(*indenter, doc, operationsForFile, i);
-                operationsForFile.clear();
-                indenter = std::unique_ptr<TextEditor::Indenter>(factory->createIndenter(doc));
-                indenter->setFileName(Utils::FilePath::fromString(op.fileName));
-            }
+        qCDebug(fixitsLog) << " " << i << "Applying" << op;
 
-            QTextCursor cursor(doc);
-            cursor.setPosition(op.pos);
-            cursor.setPosition(op.pos + op.length, QTextCursor::KeepAnchor);
-            cursor.insertText(op.text);
-            operationsForFile.push_back(&op);
-        }
+        // Shift subsequent operations that are affected
+        shiftAffectedReplacements(op, i + 1);
+
+        // Apply
+        QTextDocument * const doc = document(op.fileName);
+        QTextCursor cursor(doc);
+        cursor.setPosition(op.pos);
+        cursor.setPosition(op.pos + op.length, QTextCursor::KeepAnchor);
+        cursor.insertText(op.text);
+        auto &opsForFile = operationsByFile[filePath];
+        opsForFile.first.push_back(&op);
+        opsForFile.second = i;
+    }
+
+    // Format
+    for (auto it = operationsByFile.cbegin(); it != operationsByFile.cend(); ++it) {
+        QTextDocument * const doc = document(it.key().toString());
+        const std::unique_ptr<TextEditor::Indenter> indenter(factory->createIndenter(doc));
+        if (!indenter)
+            continue;
+        indenter->setFileName(it.key());
+        format(*indenter, doc, it.value().first, it.value().second);
     }
 
     // Write file
@@ -122,7 +126,9 @@ bool FixitsRefactoringFile::apply()
 
     QString error;
     for (auto it = m_documents.begin(); it != m_documents.end(); ++it) {
-        if (!m_textFileFormat.writeFile(it.key(), it.value()->toPlainText(), &error)) {
+        if (!m_textFileFormat.writeFile(FilePath::fromString(it.key()),
+                                        it.value()->toPlainText(),
+                                        &error)) {
             qCDebug(fixitsLog) << "ERROR: Could not write file" << it.key() << ":" << error;
             return false; // Error writing file
         }
@@ -163,10 +169,12 @@ QTextDocument *FixitsRefactoringFile::document(const QString &filePath) const
         if (!filePath.isEmpty()) {
             QString error;
             QTextCodec *defaultCodec = Core::EditorManager::defaultTextCodec();
-            TextFileFormat::ReadResult result = TextFileFormat::readFile(
-                        filePath, defaultCodec,
-                        &fileContents, &m_textFileFormat,
-                        &error);
+            TextFileFormat::ReadResult result = TextFileFormat::readFile(FilePath::fromString(
+                                                                             filePath),
+                                                                         defaultCodec,
+                                                                         &fileContents,
+                                                                         &m_textFileFormat,
+                                                                         &error);
             if (result != TextFileFormat::ReadSuccess) {
                 qCDebug(fixitsLog) << "ERROR: Could not read " << filePath << ":" << error;
                 m_textFileFormat.codec = nullptr;
