@@ -30,7 +30,6 @@
 #include <utils/environment.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
-#include <utils/synchronousprocess.h>
 #include <utils/temporaryfile.h>
 
 #include <QDebug>
@@ -40,6 +39,8 @@
 
 static Q_LOGGING_CATEGORY(LOG, "qtc.clangtools.runner", QtWarningMsg)
 
+using namespace Utils;
+
 namespace ClangTools {
 namespace Internal {
 
@@ -59,7 +60,7 @@ static QString finishedWithBadExitCode(const QString &name, int exitCode)
 }
 
 ClangToolRunner::ClangToolRunner(QObject *parent)
-    : QObject(parent), m_process(new QProcess)
+    : QObject(parent), m_process(new Utils::QtcProcess)
 {}
 
 ClangToolRunner::~ClangToolRunner()
@@ -70,26 +71,22 @@ ClangToolRunner::~ClangToolRunner()
             m_process->kill();
             m_process->waitForFinished(100);
         } else {
-            Utils::SynchronousProcess::stopProcess(*m_process);
+            m_process->stopProcess();
         }
     }
 
     m_process->deleteLater();
 }
 
-void ClangToolRunner::init(const QString &outputDirPath,
-                           const Utils::Environment &environment)
+void ClangToolRunner::init(const FilePath &outputDirPath, const Environment &environment)
 {
     m_outputDirPath = outputDirPath;
     QTC_CHECK(!m_outputDirPath.isEmpty());
 
-    m_process->setProcessChannelMode(QProcess::MergedChannels);
-    m_process->setProcessEnvironment(environment.toProcessEnvironment());
+    m_process->setEnvironment(environment);
     m_process->setWorkingDirectory(m_outputDirPath); // Current clang-cl puts log file into working dir.
-    connect(m_process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
-            this, &ClangToolRunner::onProcessFinished);
-    connect(m_process, &QProcess::errorOccurred, this, &ClangToolRunner::onProcessError);
-    connect(m_process, &QProcess::readyRead, this, &ClangToolRunner::onProcessOutput);
+    connect(m_process, &QtcProcess::finished, this, &ClangToolRunner::onProcessFinished);
+    connect(m_process, &QtcProcess::errorOccurred, this, &ClangToolRunner::onProcessError);
 }
 
 QStringList ClangToolRunner::mainToolArguments() const
@@ -104,26 +101,25 @@ QStringList ClangToolRunner::mainToolArguments() const
 
 bool ClangToolRunner::supportsVFSOverlay() const
 {
-    static QMap<QString, bool> vfsCapabilities;
+    static QMap<FilePath, bool> vfsCapabilities;
     auto it = vfsCapabilities.find(m_executable);
     if (it == vfsCapabilities.end()) {
-        Utils::SynchronousProcess p;
-        Utils::SynchronousProcessResponse response = p.runBlocking(
-            Utils::CommandLine(m_executable, {"--help"}));
-        it = vfsCapabilities.insert(m_executable, response.allOutput().contains("vfsoverlay"));
+        QtcProcess p;
+        p.setCommand({m_executable, {"--help"}});
+        p.runBlocking();
+        it = vfsCapabilities.insert(m_executable, p.allOutput().contains("vfsoverlay"));
     }
     return it.value();
 }
 
-static QString createOutputFilePath(const QString &dirPath, const QString &fileToAnalyze)
+static QString createOutputFilePath(const FilePath &dirPath, const QString &fileToAnalyze)
 {
     const QString fileName = QFileInfo(fileToAnalyze).fileName();
-    const QString fileTemplate = dirPath
-            + QLatin1String("/report-") + fileName + QLatin1String("-XXXXXX");
+    const FilePath fileTemplate = dirPath.pathAppended("report-" + fileName + "-XXXXXX");
 
-    Utils::TemporaryFile temporaryFile("clangtools");
+    TemporaryFile temporaryFile("clangtools");
     temporaryFile.setAutoRemove(false);
-    temporaryFile.setFileTemplate(fileTemplate);
+    temporaryFile.setFileTemplate(fileTemplate.path());
     if (temporaryFile.open()) {
         temporaryFile.close();
         return temporaryFile.fileName();
@@ -138,29 +134,25 @@ bool ClangToolRunner::run(const QString &fileToAnalyze, const QStringList &compi
     QTC_CHECK(!compilerOptions.contains(fileToAnalyze));
 
     m_fileToAnalyze = fileToAnalyze;
-    m_processOutput.clear();
 
     m_outputFilePath = createOutputFilePath(m_outputDirPath, fileToAnalyze);
     QTC_ASSERT(!m_outputFilePath.isEmpty(), return false);
-    const QStringList arguments = m_argsCreator(compilerOptions);
-    m_commandLine = Utils::QtcProcess::joinArgs(QStringList(m_executable) + arguments);
+    m_commandLine = {m_executable, m_argsCreator(compilerOptions)};
 
-    qCDebug(LOG).noquote() << "Starting" << m_commandLine;
-    m_process->start(m_executable, arguments);
+    qCDebug(LOG).noquote() << "Starting" << m_commandLine.toUserOutput();
+    m_process->setCommand(m_commandLine);
+    m_process->start();
     return true;
 }
 
-void ClangToolRunner::onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus)
+void ClangToolRunner::onProcessFinished()
 {
-    if (exitStatus == QProcess::NormalExit) {
-        if (exitCode == 0) {
-            qCDebug(LOG).noquote() << "Output:\n" << Utils::SynchronousProcess::normalizeNewlines(
-                                                        QString::fromLocal8Bit(m_processOutput));
-            emit finishedWithSuccess(m_fileToAnalyze);
-        } else {
-            emit finishedWithFailure(finishedWithBadExitCode(m_name, exitCode),
-                                     commandlineAndOutput());
-        }
+    if (m_process->result() == QtcProcess::FinishedWithSuccess) {
+        qCDebug(LOG).noquote() << "Output:\n" << m_process->stdOut();
+        emit finishedWithSuccess(m_fileToAnalyze);
+    } else if (m_process->result() == QtcProcess::FinishedWithError) {
+        emit finishedWithFailure(finishedWithBadExitCode(m_name, m_process->exitCode()),
+                                 commandlineAndOutput());
     } else { // == QProcess::CrashExit
         emit finishedWithFailure(finishedDueToCrash(m_name), commandlineAndOutput());
     }
@@ -174,19 +166,14 @@ void ClangToolRunner::onProcessError(QProcess::ProcessError error)
     emit finishedWithFailure(generalProcessError(m_name), commandlineAndOutput());
 }
 
-void ClangToolRunner::onProcessOutput()
-{
-    m_processOutput.append(m_process->readAll());
-}
-
 QString ClangToolRunner::commandlineAndOutput() const
 {
     return tr("Command line: %1\n"
               "Process Error: %2\n"
               "Output:\n%3")
-        .arg(m_commandLine,
-             QString::number(m_process->error()),
-             Utils::SynchronousProcess::normalizeNewlines(QString::fromLocal8Bit(m_processOutput)));
+        .arg(m_commandLine.toUserOutput())
+        .arg(m_process->error())
+        .arg(m_process->stdOut());
 }
 
 } // namespace Internal
