@@ -50,16 +50,15 @@
 #include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
 
-#include <cpptools/cpptoolsconstants.h>
-#include <cpptools/cppmodelmanager.h>
+#include <cppeditor/cppeditorconstants.h>
+#include <cppeditor/cppmodelmanager.h>
 
 #include <texteditor/texteditor.h>
 
-#include <cppeditor/cppeditorconstants.h>
-
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/projectpanelfactory.h>
 #include <projectexplorer/target.h>
+#include <projectexplorer/taskhub.h>
 
 #include <QAction>
 #include <QDebug>
@@ -91,7 +90,7 @@ public:
 
     DocumentClangToolRunner *runnerForFilePath(const Utils::FilePath &filePath)
     {
-        for (DocumentClangToolRunner *runner : documentRunners) {
+        for (DocumentClangToolRunner *runner : qAsConst(documentRunners)) {
             if (runner->filePath() == filePath)
                 return runner;
         }
@@ -114,7 +113,9 @@ bool ClangToolsPlugin::initialize(const QStringList &arguments, QString *errorSt
     Q_UNUSED(arguments)
     Q_UNUSED(errorString)
 
-    // Import tidy/clazy diagnostic configs from CppTools now
+    TaskHub::addCategory(taskCategory(), tr("Clang Tools"));
+
+    // Import tidy/clazy diagnostic configs from CppEditor now
     // instead of at opening time of the settings page
     ClangToolsSettings::instance();
 
@@ -156,7 +157,7 @@ void ClangToolsPlugin::registerAnalyzeActions()
     ActionManager::registerAction(d->clangTool.startAction(), Constants::RUN_ON_PROJECT);
     Command *cmd = ActionManager::registerAction(d->clangTool.startOnCurrentFileAction(),
                                                  Constants::RUN_ON_CURRENT_FILE);
-    ActionContainer *mtoolscpp = ActionManager::actionContainer(CppTools::Constants::M_TOOLS_CPP);
+    ActionContainer *mtoolscpp = ActionManager::actionContainer(CppEditor::Constants::M_TOOLS_CPP);
     if (mtoolscpp)
         mtoolscpp->addAction(cmd);
 
