@@ -29,28 +29,29 @@
 #include "clangtoolsutils.h"
 
 #include <coreplugin/icore.h>
-#include <cpptools/clangdiagnosticconfig.h>
-#include <cpptools/cppcodemodelsettings.h>
-#include <cpptools/cpptoolsreuse.h>
+#include <cppeditor/clangdiagnosticconfig.h>
+#include <cppeditor/cppcodemodelsettings.h>
+#include <cppeditor/cpptoolsreuse.h>
 
 #include <utils/algorithm.h>
 
 #include <QThread>
 
-static const char clangTidyExecutableKey[] = "ClangTidyExecutable";
-static const char clazyStandaloneExecutableKey[] = "ClazyStandaloneExecutable";
-
-static const char parallelJobsKey[] = "ParallelJobs";
-static const char buildBeforeAnalysisKey[] = "BuildBeforeAnalysis";
-static const char analyzeOpenFilesKey[] = "AnalyzeOpenFiles";
-static const char oldDiagnosticConfigIdKey[] = "diagnosticConfigId";
-
-using namespace CppTools;
+using namespace CppEditor;
+using namespace Utils;
 
 namespace ClangTools {
 namespace Internal {
 
-static Utils::Id defaultDiagnosticId()
+const char clangTidyExecutableKey[] = "ClangTidyExecutable";
+const char clazyStandaloneExecutableKey[] = "ClazyStandaloneExecutable";
+
+const char parallelJobsKey[] = "ParallelJobs";
+const char buildBeforeAnalysisKey[] = "BuildBeforeAnalysis";
+const char analyzeOpenFilesKey[] = "AnalyzeOpenFiles";
+const char oldDiagnosticConfigIdKey[] = "diagnosticConfigId";
+
+static Id defaultDiagnosticId()
 {
     return ClangTools::Constants::DIAG_CONFIG_TIDY_AND_CLAZY;
 }
@@ -63,7 +64,7 @@ RunSettings::RunSettings()
 
 void RunSettings::fromMap(const QVariantMap &map, const QString &prefix)
 {
-    m_diagnosticConfigId = Utils::Id::fromSetting(map.value(prefix + diagnosticConfigIdKey));
+    m_diagnosticConfigId = Id::fromSetting(map.value(prefix + diagnosticConfigIdKey));
     m_parallelJobs = map.value(prefix + parallelJobsKey).toInt();
     m_buildBeforeAnalysis = map.value(prefix + buildBeforeAnalysisKey).toBool();
     m_analyzeOpenFiles = map.value(prefix + analyzeOpenFilesKey).toBool();
@@ -77,7 +78,7 @@ void RunSettings::toMap(QVariantMap &map, const QString &prefix) const
     map.insert(prefix + analyzeOpenFilesKey, m_analyzeOpenFiles);
 }
 
-Utils::Id RunSettings::diagnosticConfigId() const
+Id RunSettings::diagnosticConfigId() const
 {
     if (!diagnosticConfigsModel().hasConfigWithId(m_diagnosticConfigId))
         return defaultDiagnosticId();
@@ -151,8 +152,8 @@ void ClangToolsSettings::readSettings()
 
     QSettings *s = Core::ICore::settings();
     s->beginGroup(Constants::SETTINGS_ID);
-    m_clangTidyExecutable = s->value(clangTidyExecutableKey).toString();
-    m_clazyStandaloneExecutable = s->value(clazyStandaloneExecutableKey).toString();
+    m_clangTidyExecutable = FilePath::fromVariant(s->value(clangTidyExecutableKey));
+    m_clazyStandaloneExecutable = FilePath::fromVariant(s->value(clazyStandaloneExecutableKey));
     m_diagnosticConfigs.append(diagnosticConfigsFromSettings(s));
 
     QVariantMap map;
@@ -184,8 +185,8 @@ void ClangToolsSettings::writeSettings()
     QSettings *s = Core::ICore::settings();
     s->beginGroup(Constants::SETTINGS_ID);
 
-    s->setValue(clangTidyExecutableKey, m_clangTidyExecutable);
-    s->setValue(clazyStandaloneExecutableKey, m_clazyStandaloneExecutable);
+    s->setValue(clangTidyExecutableKey, m_clangTidyExecutable.toVariant());
+    s->setValue(clazyStandaloneExecutableKey, m_clazyStandaloneExecutable.toVariant());
     diagnosticConfigsToSettings(s, m_diagnosticConfigs);
 
     QVariantMap map;
@@ -198,5 +199,34 @@ void ClangToolsSettings::writeSettings()
     emit changed();
 }
 
+void ClangToolsSettings::setClangTidyExecutable(const FilePath &path)
+{
+    m_clangTidyExecutable = path;
+    m_clangTidyVersion = {};
+}
+
+void ClangToolsSettings::setClazyStandaloneExecutable(const FilePath &path)
+{
+    m_clazyStandaloneExecutable = path;
+    m_clazyVersion = {};
+}
+
+static QVersionNumber getVersionNumber(QVersionNumber &version, const FilePath &toolFilePath)
+{
+    if (version.isNull() && !toolFilePath.isEmpty())
+        version = QVersionNumber::fromString(queryVersion(toolFilePath, QueryFailMode::Silent));
+    return version;
+}
+
+QVersionNumber ClangToolsSettings::clangTidyVersion()
+{
+    return getVersionNumber(instance()->m_clangTidyVersion, Internal::clangTidyExecutable());
+}
+
+QVersionNumber ClangToolsSettings::clazyVersion()
+{
+    return ClazyStandaloneInfo::getInfo(Internal::clazyStandaloneExecutable()).version;
+}
+
 } // namespace Internal
 } // namespace ClangTools
