@@ -25,22 +25,23 @@
 
 #pragma once
 
-#include <cpptools/clangdiagnosticconfig.h>
+#include <cppeditor/clangdiagnosticconfig.h>
 
+#include <utils/id.h>
 #include <utils/optional.h>
 
 #include <QtGlobal>
 
-QT_BEGIN_NAMESPACE
-class QString;
-QT_END_NAMESPACE
-
-namespace CppTools { class ClangDiagnosticConfigsModel; }
+namespace CppEditor { class ClangDiagnosticConfigsModel; }
 namespace Debugger { class DiagnosticLocation; }
+namespace Utils { class FilePath; }
 
 namespace ClangTools {
 namespace Internal {
 
+QString clangTidyDocUrl(const QString &check);
+QString clazyDocUrl(const QString &check);
+
 class Diagnostic;
 
 enum class FixitStatus {
@@ -57,33 +58,35 @@ QString createDiagnosticToolTipString(
     Utils::optional<FixitStatus> status = Utils::nullopt,
     bool showSteps = true);
 
+CppEditor::ClangDiagnosticConfig builtinConfig();
+
 QString createFullLocationString(const Debugger::DiagnosticLocation &location);
 
 QString hintAboutBuildBeforeAnalysis();
 void showHintAboutBuildBeforeAnalysis();
 
-bool isFileExecutable(const QString &filePath);
+Utils::FilePath shippedClazyStandaloneExecutable();
+Utils::FilePath clazyStandaloneExecutable();
+Utils::FilePath clazyStandaloneFallbackExecutable();
 
-QString shippedClazyStandaloneExecutable();
-QString clazyStandaloneExecutable();
-QString clazyStandaloneFallbackExecutable();
+Utils::FilePath shippedClangTidyExecutable();
+Utils::FilePath clangTidyExecutable();
+Utils::FilePath clangTidyFallbackExecutable();
 
-QString shippedClangTidyExecutable();
-QString clangTidyExecutable();
-QString clangTidyFallbackExecutable();
-
-QString fullPath(const QString &executable);
+Utils::FilePath fullPath(const Utils::FilePath &executable);
 
 QString documentationUrl(const QString &checkName);
 
-CppTools::ClangDiagnosticConfigsModel diagnosticConfigsModel();
-CppTools::ClangDiagnosticConfigsModel diagnosticConfigsModel(
-    const CppTools::ClangDiagnosticConfigs &customConfigs);
+CppEditor::ClangDiagnosticConfigsModel diagnosticConfigsModel();
+CppEditor::ClangDiagnosticConfigsModel diagnosticConfigsModel(
+    const CppEditor::ClangDiagnosticConfigs &customConfigs);
 
-CppTools::ClangDiagnosticConfig diagnosticConfig(const Utils::Id &diagConfigId);
+CppEditor::ClangDiagnosticConfig diagnosticConfig(const Utils::Id &diagConfigId);
 
 QStringList extraClangToolsPrependOptions();
 QStringList extraClangToolsAppendOptions();
 
+inline Utils::Id taskCategory() { return "ClangTools"; }
+
 } // namespace Internal
 } // namespace ClangTools
