@@ -30,27 +30,29 @@
 #include "clangtoolsconstants.h"
 #include "clangtoolsutils.h"
 
-#include <cpptools/clangdiagnosticconfigsmodel.h>
-#include <cpptools/clangdiagnosticconfigsselectionwidget.h>
+#include <cppeditor/clangdiagnosticconfigsmodel.h>
+#include <cppeditor/clangdiagnosticconfigsselectionwidget.h>
 
 #include <debugger/analyzer/analyzericons.h>
 
 #include <utils/optional.h>
 
+using namespace Utils;
+
 namespace ClangTools {
 namespace Internal {
 
 static SettingsWidget *m_instance = nullptr;
 
-static void setupPathChooser(Utils::PathChooser *const chooser,
+static void setupPathChooser(PathChooser *const chooser,
                              const QString &promptDiaglogTitle,
                              const QString &placeHolderText,
-                             const QString &pathFromSettings,
+                             const FilePath &pathFromSettings,
                              const QString &historyCompleterId)
 {
     chooser->setPromptDialogTitle(promptDiaglogTitle);
     chooser->setDefaultValue(placeHolderText);
-    chooser->setPath(pathFromSettings);
+    chooser->setFilePath(pathFromSettings);
     chooser->setExpectedKind(Utils::PathChooser::ExistingCommand);
     chooser->setHistoryCompleter(historyCompleterId);
 }
@@ -71,8 +73,8 @@ SettingsWidget::SettingsWidget()
     // Group box "Executables"
     //
 
-    QString placeHolderText = shippedClangTidyExecutable();
-    QString path = m_settings->clangTidyExecutable();
+    QString placeHolderText = shippedClangTidyExecutable().toUserOutput();
+    FilePath path = m_settings->clangTidyExecutable();
     if (path.isEmpty() && placeHolderText.isEmpty())
         path = Constants::CLANG_TIDY_EXECUTABLE_NAME;
     setupPathChooser(m_ui->clangTidyPathChooser,
@@ -81,7 +83,7 @@ SettingsWidget::SettingsWidget()
                      path,
                      "ClangTools.ClangTidyExecutable.History");
 
-    placeHolderText = shippedClazyStandaloneExecutable();
+    placeHolderText = shippedClazyStandaloneExecutable().toUserOutput();
     path = m_settings->clazyStandaloneExecutable();
     if (path.isEmpty() && placeHolderText.isEmpty())
         path = Constants::CLAZY_STANDALONE_EXECUTABLE_NAME;
@@ -101,14 +103,14 @@ SettingsWidget::SettingsWidget()
 void SettingsWidget::apply()
 {
     // Executables
-    m_settings->setClangTidyExecutable(m_ui->clangTidyPathChooser->rawPath());
-    m_settings->setClazyStandaloneExecutable(m_ui->clazyStandalonePathChooser->rawPath());
+    m_settings->setClangTidyExecutable(clangTidyPath());
+    m_settings->setClazyStandaloneExecutable(clazyStandalonePath());
 
     // Run options
     m_settings->setRunSettings(m_ui->runSettingsWidget->toSettings());
 
     // Custom configs
-    const CppTools::ClangDiagnosticConfigs customConfigs
+    const CppEditor::ClangDiagnosticConfigs customConfigs
         = m_ui->runSettingsWidget->diagnosticSelectionWidget()->customConfigs();
     m_settings->setDiagnosticConfigs(customConfigs);
 
@@ -120,14 +122,14 @@ SettingsWidget::~SettingsWidget()
     m_instance = nullptr;
 }
 
-QString SettingsWidget::clangTidyPath() const
+FilePath SettingsWidget::clangTidyPath() const
 {
-    return m_ui->clangTidyPathChooser->rawPath();
+    return m_ui->clangTidyPathChooser->rawFilePath();
 }
 
-QString SettingsWidget::clazyStandalonePath() const
+FilePath SettingsWidget::clazyStandalonePath() const
 {
-    return m_ui->clazyStandalonePathChooser->rawPath();
+    return m_ui->clazyStandalonePathChooser->rawFilePath();
 }
 
 // ClangToolsOptionsPage
