@@ -63,17 +63,6 @@ namespace Internal {
     true if plugin becames visible, otherwise it returns false.
 */
 
-/*!
-    \fn void NavigationWidget::requestGotoLocation(const QString &name,
-                                                   int line,
-                                                   int column)
-
-    Emits a signal that requests to open the file with \a name at \a line
-    and \a column.
-
-   \sa Manager::gotoLocation
-*/
-
 /*!
     \fn void NavigationWidget::requestGotoLocations(const QList<QVariant> &locations)
 
@@ -83,14 +72,6 @@ namespace Internal {
    \sa Manager::gotoLocations
 */
 
-/*!
-    \fn void NavigationWidget::requestTreeDataUpdate()
-
-    Emits a signal that the widget wants to receive the latest tree info.
-
-   \sa Manager::onRequestTreeDataUpdate
-*/
-
 NavigationWidget::NavigationWidget(QWidget *parent) :
     QWidget(parent)
 {
@@ -124,17 +105,11 @@ NavigationWidget::NavigationWidget(QWidget *parent) :
     connect(this, &NavigationWidget::visibilityChanged,
             manager, &Manager::onWidgetVisibilityIsChanged);
 
-    connect(this, &NavigationWidget::requestGotoLocation,
-            manager, &Manager::gotoLocation);
-
     connect(this, &NavigationWidget::requestGotoLocations,
             manager, &Manager::gotoLocations);
 
     connect(manager, &Manager::treeDataUpdate,
             this, &NavigationWidget::onDataUpdate);
-
-    connect(this, &NavigationWidget::requestTreeDataUpdate,
-            manager, &Manager::onRequestTreeDataUpdate);
 }
 
 NavigationWidget::~NavigationWidget() = default;
@@ -149,9 +124,6 @@ void NavigationWidget::showEvent(QShowEvent *event)
 {
     emit visibilityChanged(true);
 
-    // request to update to the current state - to be sure
-    emit requestTreeDataUpdate();
-
     QWidget::showEvent(event);
 }
 
@@ -170,7 +142,7 @@ QList<QToolButton *> NavigationWidget::createToolButtons()
     // full projects mode
     if (!fullProjectsModeButton) {
         // create a button
-        fullProjectsModeButton = new QToolButton();
+        fullProjectsModeButton = new QToolButton(this);
         fullProjectsModeButton->setIcon(
                     ::Utils::CodeModelIcon::iconForType(::Utils::CodeModelIcon::Class));
         fullProjectsModeButton->setCheckable(true);
@@ -315,8 +287,8 @@ void NavigationWidget::fetchExpandedItems(QStandardItem *item, const QStandardIt
         QStandardItem *itemChild = item->child(itemIndex);
         const QStandardItem *targetChild = target->child(targetIndex);
 
-        const SymbolInformation &itemInf = Utils::symbolInformationFromItem(itemChild);
-        const SymbolInformation &targetInf = Utils::symbolInformationFromItem(targetChild);
+        const SymbolInformation &itemInf = Internal::symbolInformationFromItem(itemChild);
+        const SymbolInformation &targetInf = Internal::symbolInformationFromItem(targetChild);
 
         if (itemInf < targetInf) {
             ++itemIndex;
