@@ -35,6 +35,8 @@
 #include <QFile>
 #include <QIODevice>
 
+using namespace Utils;
+
 namespace CMakeProjectManager {
 
 // --------------------------------------------------------------------
@@ -53,19 +55,28 @@ CMakeConfigItem::CMakeConfigItem(const QByteArray &k, const QByteArray &v) :
     key(k), value(v)
 { }
 
-QByteArray CMakeConfigItem::valueOf(const QByteArray &key, const QList<CMakeConfigItem> &input)
+QByteArray CMakeConfig::valueOf(const QByteArray &key) const
 {
-    for (auto it = input.constBegin(); it != input.constEnd(); ++it) {
+    for (auto it = constBegin(); it != constEnd(); ++it) {
         if (it->key == key)
             return it->value;
     }
     return QByteArray();
 }
 
-QString CMakeConfigItem::expandedValueOf(const ProjectExplorer::Kit *k, const QByteArray &key,
-                                         const QList<CMakeConfigItem> &input)
+QString CMakeConfig::stringValueOf(const QByteArray &key) const
 {
-    for (auto it = input.constBegin(); it != input.constEnd(); ++it) {
+    return QString::fromUtf8(valueOf(key));
+}
+
+FilePath CMakeConfig::filePathValueOf(const QByteArray &key) const
+{
+    return FilePath::fromUtf8(valueOf(key));
+}
+
+QString CMakeConfig::expandedValueOf(const ProjectExplorer::Kit *k, const QByteArray &key) const
+{
+    for (auto it = constBegin(); it != constEnd(); ++it) {
         if (it->key == key)
             return it->expandedValue(k);
     }
@@ -145,9 +156,10 @@ CMakeConfigItem::Type CMakeConfigItem::typeStringToType(const QByteArray &type)
         return CMakeConfigItem::PATH;
     if (type == "STATIC")
         return CMakeConfigItem::STATIC;
+    if (type == "INTERNAL")
+        return CMakeConfigItem::INTERNAL;
 
-    QTC_CHECK(type == "INTERNAL" || type == "UNINITIALIZED");
-    return CMakeConfigItem::INTERNAL;
+    return CMakeConfigItem::UNINITIALIZED;
 }
 
 QString CMakeConfigItem::typeToTypeString(const CMakeConfigItem::Type t)
@@ -163,18 +175,20 @@ QString CMakeConfigItem::typeToTypeString(const CMakeConfigItem::Type t)
         return {"INTERNAL"};
     case CMakeProjectManager::CMakeConfigItem::STATIC:
         return {"STATIC"};
-    case CMakeConfigItem::BOOL:
+    case CMakeProjectManager::CMakeConfigItem::BOOL:
         return {"BOOL"};
+    case CMakeProjectManager::CMakeConfigItem::UNINITIALIZED:
+        return {"UNINITIALIZED"};
     }
     QTC_CHECK(false);
     return {};
 }
 
-Utils::optional<bool> CMakeConfigItem::toBool(const QByteArray &value)
+Utils::optional<bool> CMakeConfigItem::toBool(const QString &value)
 {
     // Taken from CMakes if(<constant>) documentation:
     // "Named boolean constants are case-insensitive."
-    const QString v = QString::fromUtf8(value).toUpper();
+    const QString v = value.toUpper();
 
     bool isInt = false;
     v.toInt(&isInt);
@@ -196,12 +210,12 @@ QString CMakeConfigItem::expandedValue(const ProjectExplorer::Kit *k) const
 
 QString CMakeConfigItem::expandedValue(const Utils::MacroExpander *expander) const
 {
-    return expander->expand(QString::fromUtf8(value));
+    return expander ? expander->expand(QString::fromUtf8(value)) : QString::fromUtf8(value);
 }
 
-std::function<bool (const CMakeConfigItem &a, const CMakeConfigItem &b)> CMakeConfigItem::sortOperator()
+bool CMakeConfigItem::less(const CMakeConfigItem &a, const CMakeConfigItem &b)
 {
-    return [](const CMakeConfigItem &a, const CMakeConfigItem &b) { return a.key < b.key; };
+    return a.key < b.key;
 }
 
 CMakeConfigItem CMakeConfigItem::fromString(const QString &s)
@@ -296,7 +310,7 @@ static CMakeConfigItem unsetItemFromString(const QString &input)
     return item;
 }
 
-QList<CMakeConfigItem> CMakeConfigItem::itemsFromArguments(const QStringList &list)
+CMakeConfig CMakeConfig::fromArguments(const QStringList &list)
 {
     CMakeConfig result;
     bool inSet = false;
@@ -332,7 +346,7 @@ QList<CMakeConfigItem> CMakeConfigItem::itemsFromArguments(const QStringList &li
     return result;
 }
 
-QList<CMakeConfigItem> CMakeConfigItem::itemsFromFile(const Utils::FilePath &cacheFile, QString *errorMessage)
+CMakeConfig CMakeConfig::fromFile(const Utils::FilePath &cacheFile, QString *errorMessage)
 {
     CMakeConfig result;
     QFile cache(cacheFile.toString());
@@ -389,7 +403,7 @@ QList<CMakeConfigItem> CMakeConfigItem::itemsFromFile(const Utils::FilePath &cac
         }
     }
 
-    Utils::sort(result, CMakeConfigItem::sortOperator());
+    Utils::sort(result, &CMakeConfigItem::less);
 
     return result;
 
@@ -418,6 +432,9 @@ QString CMakeConfigItem::toString(const Utils::MacroExpander *expander) const
     case CMakeProjectManager::CMakeConfigItem::INTERNAL:
         typeStr = QLatin1String("INTERNAL");
         break;
+    case CMakeProjectManager::CMakeConfigItem::UNINITIALIZED:
+        typeStr = QLatin1String("UNINITIALIZED");
+        break;
     case CMakeProjectManager::CMakeConfigItem::STRING:
     default:
         typeStr = QLatin1String("STRING");
@@ -454,6 +471,11 @@ bool CMakeConfigItem::operator==(const CMakeConfigItem &o) const
     return o.key == key && o.value == value && o.isUnset == isUnset;
 }
 
+Utils::QHashValueType qHash(const CMakeConfigItem &it)
+{
+    return ::qHash(it.key) ^ ::qHash(it.value) ^ ::qHash(it.isUnset);
+}
+
 #if WITH_TESTS
 
 } // namespace CMakeProjectManager
