@@ -31,21 +31,27 @@
 #include "cmaketool.h"
 #include "cmaketoolmanager.h"
 
+#include <app/app_version.h>
+
 #include <coreplugin/icore.h>
+
+#include <ios/iosconstants.h>
+
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/projectexplorersettings.h>
 #include <projectexplorer/task.h>
 #include <projectexplorer/toolchain.h>
+#include <projectexplorer/devicesupport/idevice.h>
+
 #include <qtsupport/baseqtversion.h>
 #include <qtsupport/qtkitinformation.h>
-#include <projectexplorer/projectexplorerconstants.h>
-
-#include <app/app_version.h>
 
 #include <utils/algorithm.h>
 #include <utils/elidinglabel.h>
 #include <utils/environment.h>
+#include <utils/layoutbuilder.h>
 #include <utils/macroexpander.h>
 #include <utils/qtcassert.h>
 #include <utils/variablechooser.h>
@@ -60,27 +66,35 @@
 #include <QPushButton>
 
 using namespace ProjectExplorer;
+using namespace Utils;
 
 namespace CMakeProjectManager {
 // --------------------------------------------------------------------
 // CMakeKitAspect:
 // --------------------------------------------------------------------
 
-static Utils::Id defaultCMakeToolId()
+static bool isIos(const Kit *k)
 {
-    CMakeTool *defaultTool = CMakeToolManager::defaultCMakeTool();
-    return defaultTool ? defaultTool->id() : Utils::Id();
+    const Id deviceType = DeviceTypeKitAspect::deviceTypeId(k);
+    return deviceType == Ios::Constants::IOS_DEVICE_TYPE
+           || deviceType == Ios::Constants::IOS_SIMULATOR_TYPE;
 }
 
-static const char TOOL_ID[] = "CMakeProjectManager.CMakeKitInformation";
+static Id defaultCMakeToolId()
+{
+    CMakeTool *defaultTool = CMakeToolManager::defaultCMakeTool();
+    return defaultTool ? defaultTool->id() : Id();
+}
 
-class CMakeKitAspectWidget : public KitAspectWidget
+const char TOOL_ID[] = "CMakeProjectManager.CMakeKitInformation";
+
+class CMakeKitAspectWidget final : public KitAspectWidget
 {
     Q_DECLARE_TR_FUNCTIONS(CMakeProjectManager::Internal::CMakeKitAspect)
 public:
     CMakeKitAspectWidget(Kit *kit, const KitAspect *ki) : KitAspectWidget(kit, ki),
-        m_comboBox(new QComboBox),
-        m_manageButton(new QPushButton(KitAspectWidget::msgManage()))
+        m_comboBox(createSubWidget<QComboBox>()),
+        m_manageButton(createManageButton(Constants::CMAKE_SETTINGS_PAGE_ID))
     {
         m_comboBox->setSizePolicy(QSizePolicy::Ignored, m_comboBox->sizePolicy().verticalPolicy());
         m_comboBox->setEnabled(false);
@@ -94,10 +108,6 @@ public:
         connect(m_comboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
                 this, &CMakeKitAspectWidget::currentCMakeToolChanged);
 
-        m_manageButton->setContentsMargins(0, 0, 0, 0);
-        connect(m_manageButton, &QPushButton::clicked,
-                this, &CMakeKitAspectWidget::manageCMakeTools);
-
         CMakeToolManager *cmakeMgr = CMakeToolManager::instance();
         connect(cmakeMgr, &CMakeToolManager::cmakeAdded,
                 this, &CMakeKitAspectWidget::cmakeToolAdded);
@@ -116,8 +126,13 @@ public:
 private:
     // KitAspectWidget interface
     void makeReadOnly() override { m_comboBox->setEnabled(false); }
-    QWidget *mainWidget() const override { return m_comboBox; }
-    QWidget *buttonWidget() const override { return m_manageButton; }
+
+    void addToLayout(LayoutBuilder &builder) override
+    {
+        addMutableAction(m_comboBox);
+        builder.addItem(m_comboBox);
+        builder.addItem(m_manageButton);
+    }
 
     void refresh() override
     {
@@ -125,10 +140,10 @@ private:
         m_comboBox->setCurrentIndex(tool ? indexOf(tool->id()) : -1);
     }
 
-    int indexOf(const Utils::Id &id)
+    int indexOf(Id id)
     {
         for (int i = 0; i < m_comboBox->count(); ++i) {
-            if (id == Utils::Id::fromSetting(m_comboBox->itemData(i)))
+            if (id == Id::fromSetting(m_comboBox->itemData(i)))
                 return i;
         }
         return -1;
@@ -137,20 +152,19 @@ private:
     void updateComboBox()
     {
         // remove unavailable cmake tool:
-        int pos = indexOf(Utils::Id());
+        int pos = indexOf(Id());
         if (pos >= 0)
             m_comboBox->removeItem(pos);
 
         if (m_comboBox->count() == 0) {
-            m_comboBox->addItem(tr("<No CMake Tool available>"),
-                                Utils::Id().toSetting());
+            m_comboBox->addItem(tr("<No CMake Tool available>"), Id().toSetting());
             m_comboBox->setEnabled(false);
         } else {
             m_comboBox->setEnabled(true);
         }
     }
 
-    void cmakeToolAdded(const Utils::Id &id)
+    void cmakeToolAdded(Id id)
     {
         const CMakeTool *tool = CMakeToolManager::findById(id);
         QTC_ASSERT(tool, return);
@@ -160,7 +174,7 @@ private:
         refresh();
     }
 
-    void cmakeToolUpdated(const Utils::Id &id)
+    void cmakeToolUpdated(Id id)
     {
         const int pos = indexOf(id);
         QTC_ASSERT(pos >= 0, return);
@@ -171,7 +185,7 @@ private:
         m_comboBox->setItemText(pos, tool->displayName());
     }
 
-    void cmakeToolRemoved(const Utils::Id &id)
+    void cmakeToolRemoved(Id id)
     {
         const int pos = indexOf(id);
         QTC_ASSERT(pos >= 0, return);
@@ -191,18 +205,13 @@ private:
         if (m_removingItem)
             return;
 
-        const Utils::Id id = Utils::Id::fromSetting(m_comboBox->itemData(index));
+        const Id id = Id::fromSetting(m_comboBox->itemData(index));
         CMakeKitAspect::setCMakeTool(m_kit, id);
     }
 
-    void manageCMakeTools()
-    {
-        Core::ICore::showOptionsDialog(Constants::CMAKE_SETTINGS_PAGE_ID, buttonWidget());
-    }
-
     bool m_removingItem = false;
     QComboBox *m_comboBox;
-    QPushButton *m_manageButton;
+    QWidget *m_manageButton;
 };
 
 CMakeKitAspect::CMakeKitAspect()
@@ -216,23 +225,23 @@ CMakeKitAspect::CMakeKitAspect()
 
     //make sure the default value is set if a selected CMake is removed
     connect(CMakeToolManager::instance(), &CMakeToolManager::cmakeRemoved,
-            [this]() { foreach (Kit *k, KitManager::kits()) fix(k); });
+            [this] { for (Kit *k : KitManager::kits()) fix(k); });
 
     //make sure the default value is set if a new default CMake is set
     connect(CMakeToolManager::instance(), &CMakeToolManager::defaultCMakeChanged,
-            [this]() { foreach (Kit *k, KitManager::kits()) fix(k); });
+            [this] { for (Kit *k : KitManager::kits()) fix(k); });
 }
 
-Utils::Id CMakeKitAspect::id()
+Id CMakeKitAspect::id()
 {
     return TOOL_ID;
 }
 
-Utils::Id CMakeKitAspect::cmakeToolId(const Kit *k)
+Id CMakeKitAspect::cmakeToolId(const Kit *k)
 {
     if (!k)
         return {};
-    return Utils::Id::fromSetting(k->value(TOOL_ID));
+    return Id::fromSetting(k->value(TOOL_ID));
 }
 
 CMakeTool *CMakeKitAspect::cmakeTool(const Kit *k)
@@ -240,9 +249,9 @@ CMakeTool *CMakeKitAspect::cmakeTool(const Kit *k)
     return CMakeToolManager::findById(cmakeToolId(k));
 }
 
-void CMakeKitAspect::setCMakeTool(Kit *k, const Utils::Id id)
+void CMakeKitAspect::setCMakeTool(Kit *k, const Id id)
 {
-    const Utils::Id toSet = id.isValid() ? id : defaultCMakeToolId();
+    const Id toSet = id.isValid() ? id : defaultCMakeToolId();
     QTC_ASSERT(!id.isValid() || CMakeToolManager::findById(toSet), return);
     if (k)
         k->setValue(TOOL_ID, toSet.toSetting());
@@ -264,8 +273,20 @@ Tasks CMakeKitAspect::validate(const Kit *k) const
 void CMakeKitAspect::setup(Kit *k)
 {
     CMakeTool *tool = CMakeKitAspect::cmakeTool(k);
-    if (!tool)
-        setCMakeTool(k, defaultCMakeToolId());
+    if (tool)
+        return;
+
+    // Look for a suitable auto-detected one:
+    const QString kitSource = k->autoDetectionSource();
+    for (CMakeTool *tool : CMakeToolManager::cmakeTools()) {
+        const QString toolSource = tool->detectionSource();
+        if (!toolSource.isEmpty() && toolSource == kitSource) {
+            setCMakeTool(k, tool->id());
+            return;
+        }
+    }
+
+    setCMakeTool(k, defaultCMakeToolId());
 }
 
 void CMakeKitAspect::fix(Kit *k)
@@ -285,17 +306,17 @@ KitAspectWidget *CMakeKitAspect::createConfigWidget(Kit *k) const
     return new CMakeKitAspectWidget(k, this);
 }
 
-void CMakeKitAspect::addToMacroExpander(Kit *k, Utils::MacroExpander *expander) const
+void CMakeKitAspect::addToMacroExpander(Kit *k, MacroExpander *expander) const
 {
     QTC_ASSERT(k, return);
     expander->registerFileVariables("CMake:Executable", tr("Path to the cmake executable"),
-                                    [k]() -> QString {
-                                        CMakeTool *tool = CMakeKitAspect::cmakeTool(k);
-                                        return tool ? tool->cmakeExecutable().toString() : QString();
-    });
+        [k] {
+            CMakeTool *tool = CMakeKitAspect::cmakeTool(k);
+            return tool ? tool->cmakeExecutable() : FilePath();
+        });
 }
 
-QSet<Utils::Id> CMakeKitAspect::availableFeatures(const Kit *k) const
+QSet<Id> CMakeKitAspect::availableFeatures(const Kit *k) const
 {
     if (cmakeTool(k))
         return { CMakeProjectManager::Constants::CMAKE_FEATURE_ID };
@@ -313,21 +334,22 @@ QString CMakeKitAspect::msgUnsupportedVersion(const QByteArray &versionString)
 // CMakeGeneratorKitAspect:
 // --------------------------------------------------------------------
 
-static const char GENERATOR_ID[] = "CMake.GeneratorKitInformation";
+const char GENERATOR_ID[] = "CMake.GeneratorKitInformation";
 
-static const char GENERATOR_KEY[] = "Generator";
-static const char EXTRA_GENERATOR_KEY[] = "ExtraGenerator";
-static const char PLATFORM_KEY[] = "Platform";
-static const char TOOLSET_KEY[] = "Toolset";
+const char GENERATOR_KEY[] = "Generator";
+const char EXTRA_GENERATOR_KEY[] = "ExtraGenerator";
+const char PLATFORM_KEY[] = "Platform";
+const char TOOLSET_KEY[] = "Toolset";
 
-class CMakeGeneratorKitAspectWidget : public KitAspectWidget
+class CMakeGeneratorKitAspectWidget final : public KitAspectWidget
 {
     Q_DECLARE_TR_FUNCTIONS(CMakeProjectManager::Internal::CMakeGeneratorKitAspect)
+
 public:
-    CMakeGeneratorKitAspectWidget(Kit *kit, const ::KitAspect *ki)
+    CMakeGeneratorKitAspectWidget(Kit *kit, const KitAspect *ki)
         : KitAspectWidget(kit, ki),
-          m_label(new Utils::ElidingLabel),
-          m_changeButton(new QPushButton)
+          m_label(createSubWidget<ElidingLabel>()),
+          m_changeButton(createSubWidget<QPushButton>())
     {
         m_label->setToolTip(ki->description());
         m_changeButton->setText(tr("Change..."));
@@ -345,8 +367,13 @@ public:
 private:
     // KitAspectWidget interface
     void makeReadOnly() override { m_changeButton->setEnabled(false); }
-    QWidget *mainWidget() const override { return m_label; }
-    QWidget *buttonWidget() const override { return m_changeButton; }
+
+    void addToLayout(LayoutBuilder &builder) override
+    {
+        addMutableAction(m_label);
+        builder.addItem(m_label);
+        builder.addItem(m_changeButton);
+    }
 
     void refresh() override
     {
@@ -382,16 +409,16 @@ private:
 
         changeDialog->setWindowTitle(tr("CMake Generator"));
 
-        auto *layout = new QGridLayout(changeDialog);
+        auto layout = new QGridLayout(changeDialog);
         layout->setSizeConstraint(QLayout::SetFixedSize);
 
-        auto *cmakeLabel = new QLabel;
+        auto cmakeLabel = new QLabel;
         cmakeLabel->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
 
-        auto *generatorCombo = new QComboBox;
-        auto *extraGeneratorCombo = new QComboBox;
-        auto *platformEdit = new QLineEdit;
-        auto *toolsetEdit = new QLineEdit;
+        auto generatorCombo = new QComboBox;
+        auto extraGeneratorCombo = new QComboBox;
+        auto platformEdit = new QLineEdit;
+        auto toolsetEdit = new QLineEdit;
 
         int row = 0;
         layout->addWidget(new QLabel(QLatin1String("Executable:")));
@@ -414,7 +441,7 @@ private:
         layout->addWidget(toolsetEdit, row, 1);
 
         ++row;
-        auto *bb = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
+        auto bb = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
         layout->addWidget(bb, row, 0, 1, 2);
 
         connect(bb, &QDialogButtonBox::accepted, changeDialog.data(), &QDialog::accept);
@@ -430,14 +457,14 @@ private:
 
         auto updateDialog = [&generatorList, generatorCombo, extraGeneratorCombo,
                 platformEdit, toolsetEdit](const QString &name) {
-            auto it = std::find_if(generatorList.constBegin(), generatorList.constEnd(),
+            const auto it = std::find_if(generatorList.constBegin(), generatorList.constEnd(),
                                    [name](const CMakeTool::Generator &g) { return g.name == name; });
             QTC_ASSERT(it != generatorList.constEnd(), return);
             generatorCombo->setCurrentText(name);
 
             extraGeneratorCombo->clear();
             extraGeneratorCombo->addItem(tr("<none>"), QString());
-            foreach (const QString &eg, it->extraGenerators)
+            for (const QString &eg : qAsConst(it->extraGenerators))
                 extraGeneratorCombo->addItem(eg, eg);
             extraGeneratorCombo->setEnabled(extraGeneratorCombo->count() > 1);
 
@@ -449,8 +476,8 @@ private:
 
         generatorCombo->setCurrentText(CMakeGeneratorKitAspect::generator(kit()));
         extraGeneratorCombo->setCurrentText(CMakeGeneratorKitAspect::extraGenerator(kit()));
-        platformEdit->setText(platformEdit->isEnabled() ? CMakeGeneratorKitAspect::platform(kit()) : QLatin1String("<unsupported>"));
-        toolsetEdit->setText(toolsetEdit->isEnabled() ? CMakeGeneratorKitAspect::toolset(kit()) : QLatin1String("<unsupported>"));
+        platformEdit->setText(platformEdit->isEnabled() ? CMakeGeneratorKitAspect::platform(kit()) : QString());
+        toolsetEdit->setText(toolsetEdit->isEnabled() ? CMakeGeneratorKitAspect::toolset(kit()) : QString());
 
         connect(generatorCombo, &QComboBox::currentTextChanged, updateDialog);
 
@@ -466,7 +493,7 @@ private:
     }
 
     bool m_ignoreChange = false;
-    Utils::ElidingLabel *m_label;
+    ElidingLabel *m_label;
     QPushButton *m_changeButton;
     CMakeTool *m_currentTool = nullptr;
 };
@@ -619,6 +646,14 @@ QStringList CMakeGeneratorKitAspect::generatorArguments(const Kit *k)
     return result;
 }
 
+bool CMakeGeneratorKitAspect::isMultiConfigGenerator(const Kit *k)
+{
+    const QString generator = CMakeGeneratorKitAspect::generator(k);
+    return generator.indexOf("Visual Studio") != -1 ||
+           generator == "Xcode" ||
+           generator == "Ninja Multi-Config";
+}
+
 QVariant CMakeGeneratorKitAspect::defaultValue(const Kit *k) const
 {
     QTC_ASSERT(k, return QVariant());
@@ -627,6 +662,9 @@ QVariant CMakeGeneratorKitAspect::defaultValue(const Kit *k) const
     if (!tool)
         return QVariant();
 
+    if (isIos(k))
+        return GeneratorInfo("Xcode").toVariant();
+
     const QList<CMakeTool::Generator> known = tool->supportedGenerators();
     auto it = std::find_if(known.constBegin(), known.constEnd(), [](const CMakeTool::Generator &g) {
         return g.matches("Ninja");
@@ -636,9 +674,8 @@ QVariant CMakeGeneratorKitAspect::defaultValue(const Kit *k) const
             Internal::CMakeSpecificSettings *settings
                 = Internal::CMakeProjectPlugin::projectTypeSpecificSettings();
 
-            if (settings->ninjaPath().isEmpty()) {
-                Utils::Environment env = Utils::Environment::systemEnvironment();
-                k->addToEnvironment(env);
+            if (settings->ninjaPath.filePath().isEmpty()) {
+                Environment env = k->buildEnvironment();
                 return !env.searchInPath("ninja").isEmpty();
             }
             return true;
@@ -648,7 +685,7 @@ QVariant CMakeGeneratorKitAspect::defaultValue(const Kit *k) const
             return GeneratorInfo("Ninja").toVariant();
     }
 
-    if (Utils::HostOsInfo::isWindowsHost()) {
+    if (tool->filePath().osType() == OsTypeWindows) {
         // *sigh* Windows with its zoo of incompatible stuff again...
         ToolChain *tc = ToolChainKitAspect::cxxToolChain(k);
         if (tc && tc->typeId() == ProjectExplorer::Constants::MINGW_TOOLCHAIN_TYPEID) {
@@ -757,7 +794,7 @@ void CMakeGeneratorKitAspect::fix(Kit *k)
         dv.fromVariant(defaultValue(k));
         setGeneratorInfo(k, dv);
     } else {
-        const GeneratorInfo dv(info.generator,
+        const GeneratorInfo dv(isIos(k) ? QString("Xcode") : info.generator,
                                info.extraGenerator,
                                it->supportsPlatform ? info.platform : QString(),
                                it->supportsToolset ? info.toolset : QString());
@@ -805,13 +842,14 @@ KitAspectWidget *CMakeGeneratorKitAspect::createConfigWidget(Kit *k) const
     return new CMakeGeneratorKitAspectWidget(k, this);
 }
 
-void CMakeGeneratorKitAspect::addToEnvironment(const Kit *k, Utils::Environment &env) const
+void CMakeGeneratorKitAspect::addToBuildEnvironment(const Kit *k, Environment &env) const
 {
     GeneratorInfo info = generatorInfo(k);
     if (info.generator == "NMake Makefiles JOM") {
         if (env.searchInPath("jom.exe").exists())
             return;
-        env.appendOrSetPath(QCoreApplication::applicationDirPath());
+        env.appendOrSetPath(Core::ICore::libexecPath().toUserOutput());
+        env.appendOrSetPath(Core::ICore::libexecPath("jom").toUserOutput());
     }
 }
 
@@ -819,21 +857,22 @@ void CMakeGeneratorKitAspect::addToEnvironment(const Kit *k, Utils::Environment
 // CMakeConfigurationKitAspect:
 // --------------------------------------------------------------------
 
-static const char CONFIGURATION_ID[] = "CMake.ConfigurationKitInformation";
+const char CONFIGURATION_ID[] = "CMake.ConfigurationKitInformation";
 
-static const char CMAKE_C_TOOLCHAIN_KEY[] = "CMAKE_C_COMPILER";
-static const char CMAKE_CXX_TOOLCHAIN_KEY[] = "CMAKE_CXX_COMPILER";
-static const char CMAKE_QMAKE_KEY[] = "QT_QMAKE_EXECUTABLE";
-static const char CMAKE_PREFIX_PATH_KEY[] = "CMAKE_PREFIX_PATH";
+const char CMAKE_C_TOOLCHAIN_KEY[] = "CMAKE_C_COMPILER";
+const char CMAKE_CXX_TOOLCHAIN_KEY[] = "CMAKE_CXX_COMPILER";
+const char CMAKE_QMAKE_KEY[] = "QT_QMAKE_EXECUTABLE";
+const char CMAKE_PREFIX_PATH_KEY[] = "CMAKE_PREFIX_PATH";
 
-class CMakeConfigurationKitAspectWidget : public KitAspectWidget
+class CMakeConfigurationKitAspectWidget final : public KitAspectWidget
 {
     Q_DECLARE_TR_FUNCTIONS(CMakeProjectManager::Internal::CMakeConfigurationKitAspect)
+
 public:
     CMakeConfigurationKitAspectWidget(Kit *kit, const KitAspect *ki)
         : KitAspectWidget(kit, ki),
-          m_summaryLabel(new Utils::ElidingLabel),
-          m_manageButton(new QPushButton)
+          m_summaryLabel(createSubWidget<ElidingLabel>()),
+          m_manageButton(createSubWidget<QPushButton>())
     {
         refresh();
         m_manageButton->setText(tr("Change..."));
@@ -843,8 +882,12 @@ public:
 
 private:
     // KitAspectWidget interface
-    QWidget *mainWidget() const override { return m_summaryLabel; }
-    QWidget *buttonWidget() const override { return m_manageButton; }
+    void addToLayout(LayoutBuilder &builder) override
+    {
+        addMutableAction(m_summaryLabel);
+        builder.addItem(m_summaryLabel);
+        builder.addItem(m_manageButton);
+    }
 
     void makeReadOnly() override
     {
@@ -881,7 +924,7 @@ private:
                                 "You may provide a type hint by adding \":TYPE\" before the \"=\"."));
         m_editor->setMinimumSize(800, 200);
 
-        auto chooser = new Utils::VariableChooser(m_dialog);
+        auto chooser = new VariableChooser(m_dialog);
         chooser->addSupportedWidget(m_editor);
         chooser->addMacroExpanderProvider([this]() { return kit()->macroExpander(); });
 
@@ -953,15 +996,15 @@ void CMakeConfigurationKitAspect::setConfiguration(Kit *k, const CMakeConfig &co
 {
     if (!k)
         return;
-    const QStringList tmp = Utils::transform(config, [](const CMakeConfigItem &i) { return i.toString(); });
+    const QStringList tmp = Utils::transform(config.toList(),
+                                             [](const CMakeConfigItem &i) { return i.toString(); });
     k->setValue(CONFIGURATION_ID, tmp);
 }
 
 QStringList CMakeConfigurationKitAspect::toStringList(const Kit *k)
 {
-    QStringList current
-            = Utils::transform(CMakeConfigurationKitAspect::configuration(k),
-                               [](const CMakeConfigItem &i) { return i.toString(); });
+    QStringList current = Utils::transform(CMakeConfigurationKitAspect::configuration(k).toList(),
+                                           [](const CMakeConfigItem &i) { return i.toString(); });
     current = Utils::filtered(current, [](const QString &s) { return !s.isEmpty(); });
     Utils::sort(current);
     return current;
@@ -970,7 +1013,7 @@ QStringList CMakeConfigurationKitAspect::toStringList(const Kit *k)
 void CMakeConfigurationKitAspect::fromStringList(Kit *k, const QStringList &in)
 {
     CMakeConfig result;
-    foreach (const QString &s, in) {
+    for (const QString &s : in) {
         const CMakeConfigItem item = CMakeConfigItem::fromString(s);
         if (!item.key.isEmpty())
             result << item;
@@ -980,7 +1023,7 @@ void CMakeConfigurationKitAspect::fromStringList(Kit *k, const QStringList &in)
 
 QStringList CMakeConfigurationKitAspect::toArgumentsList(const Kit *k)
 {
-    return Utils::transform(CMakeConfigurationKitAspect::configuration(k),
+    return Utils::transform(CMakeConfigurationKitAspect::configuration(k).toList(),
                             [](const CMakeConfigItem &i) { return i.toArgument(nullptr); });
 }
 
@@ -1001,12 +1044,10 @@ CMakeConfig CMakeConfigurationKitAspect::defaultConfiguration(const Kit *k)
 
 QVariant CMakeConfigurationKitAspect::defaultValue(const Kit *k) const
 {
-    Q_UNUSED(k)
-
     // FIXME: Convert preload scripts
     CMakeConfig config = defaultConfiguration(k);
-    const QStringList tmp
-            = Utils::transform(config, [](const CMakeConfigItem &i) { return i.toString(); });
+    const QStringList tmp = Utils::transform(config.toList(),
+                                             [](const CMakeConfigItem &i) { return i.toString(); });
     return tmp;
 }
 
@@ -1020,14 +1061,14 @@ Tasks CMakeConfigurationKitAspect::validate(const Kit *k) const
     const CMakeConfig config = configuration(k);
 
     const bool isQt4 = version && version->qtVersion() < QtSupport::QtVersionNumber(5, 0, 0);
-    Utils::FilePath qmakePath;
-    QStringList qtInstallDirs;
-    Utils::FilePath tcCPath;
-    Utils::FilePath tcCxxPath;
-    foreach (const CMakeConfigItem &i, config) {
+    FilePath qmakePath; // This is relative to the cmake used for building.
+    QStringList qtInstallDirs; // This is relativ to the cmake used for building.
+    FilePath tcCPath;
+    FilePath tcCxxPath;
+    for (const CMakeConfigItem &i : config) {
         // Do not use expand(QByteArray) as we cannot be sure the input is latin1
-        const Utils::FilePath expandedValue
-            = Utils::FilePath::fromString(k->macroExpander()->expand(QString::fromUtf8(i.value)));
+        const FilePath expandedValue
+            = FilePath::fromString(k->macroExpander()->expand(QString::fromUtf8(i.value)));
         if (i.key == CMAKE_QMAKE_KEY)
             qmakePath = expandedValue;
         else if (i.key == CMAKE_C_TOOLCHAIN_KEY)
@@ -1035,7 +1076,7 @@ Tasks CMakeConfigurationKitAspect::validate(const Kit *k) const
         else if (i.key == CMAKE_CXX_TOOLCHAIN_KEY)
             tcCxxPath = expandedValue;
         else if (i.key == CMAKE_PREFIX_PATH_KEY)
-            qtInstallDirs = CMakeConfigItem::cmakeSplitValue(expandedValue.toString());
+            qtInstallDirs = CMakeConfigItem::cmakeSplitValue(expandedValue.path());
     }
 
     Tasks result;
@@ -1053,13 +1094,13 @@ Tasks CMakeConfigurationKitAspect::validate(const Kit *k) const
         if (!version || !version->isValid()) {
             addWarning(tr("CMake configuration has a path to a qmake binary set, "
                           "even though the kit has no valid Qt version."));
-        } else if (qmakePath != version->qmakeCommand() && isQt4) {
+        } else if (qmakePath != version->qmakeFilePath() && isQt4) {
             addWarning(tr("CMake configuration has a path to a qmake binary set "
                           "that does not match the qmake binary path "
                           "configured in the Qt version."));
         }
     }
-    if (version && !qtInstallDirs.contains(version->prefix().toString()) && !isQt4) {
+    if (version && !qtInstallDirs.contains(version->prefix().path()) && !isQt4) {
         if (version->isValid()) {
             addWarning(tr("CMake configuration has no CMAKE_PREFIX_PATH set "
                           "that points to the kit Qt version."));
