@@ -41,7 +41,8 @@
 namespace CMakeProjectManager {
 namespace Internal {
 
-class CMakeProcess : public QObject {
+class CMakeProcess : public QObject
+{
     Q_OBJECT
 
 public:
@@ -58,16 +59,14 @@ public:
     void reportFinished(); // None of the progress related functions will work after this!
     void setProgressValue(int p);
 
-    // Process stdout/stderr:
-    void processStandardOutput();
-    void processStandardError();
+    int lastExitCode() const { return m_lastExitCode; }
 
 signals:
     void started();
-    void finished(int exitCode, QProcess::ExitStatus exitStatus);
+    void finished();
 
 private:
-    void handleProcessFinished(int code, QProcess::ExitStatus status);
+    void handleProcessFinished();
     void checkForCancelled();
 
     std::unique_ptr<Utils::QtcProcess> m_process;
@@ -76,6 +75,7 @@ private:
     bool m_processWasCanceled = false;
     QTimer m_cancelTimer;
     QElapsedTimer m_elapsed;
+    int m_lastExitCode = 0;
 };
 
 } // namespace Internal
