@@ -25,7 +25,9 @@
 
 #include "cmakeproject.h"
 
+#include "cmakebuildconfiguration.h"
 #include "cmakebuildstep.h"
+#include "cmakebuildsystem.h"
 #include "cmakekitinformation.h"
 #include "cmakeprojectconstants.h"
 #include "cmakeprojectimporter.h"
@@ -46,11 +48,6 @@ namespace CMakeProjectManager {
 
 using namespace Internal;
 
-// QtCreator CMake Generator wishlist:
-// Which make targets we need to build to get all executables
-// What is the actual compiler executable
-// DEFINES
-
 /*!
   \class CMakeProject
 */
@@ -61,7 +58,6 @@ CMakeProject::CMakeProject(const FilePath &fileName)
     setProjectLanguages(Core::Context(ProjectExplorer::Constants::CXX_LANGUAGE_ID));
     setDisplayName(projectDirectory().fileName());
     setCanBuildProducts();
-    setKnowsAllBuildExecutables(false);
     setHasMakeInstallEquivalent(true);
 }
 
@@ -79,6 +75,8 @@ Tasks CMakeProject::projectIssues(const Kit *k) const
     if (ToolChainKitAspect::toolChains(k).isEmpty())
         result.append(createProjectTask(Task::TaskType::Warning, tr("No compilers set in kit.")));
 
+    result.append(m_issues);
+
     return result;
 }
 
@@ -90,6 +88,16 @@ ProjectImporter *CMakeProject::projectImporter() const
     return m_projectImporter;
 }
 
+void CMakeProject::addIssue(IssueType type, const QString &text)
+{
+    m_issues.append(createProjectTask(type, text));
+}
+
+void CMakeProject::clearIssues()
+{
+    m_issues.clear();
+}
+
 bool CMakeProject::setupTarget(Target *t)
 {
     t->updateDefaultBuildConfigurations();
@@ -119,7 +127,26 @@ MakeInstallCommand CMakeProject::makeInstallCommand(const Target *target,
                 cmd.command = tool->cmakeExecutable();
         }
     }
-    cmd.arguments << "--build" << "." << "--target" << "install";
+
+    QString installTarget = "install";
+    QStringList config;
+
+    auto bs = qobject_cast<CMakeBuildSystem*>(target->buildSystem());
+    auto bc = qobject_cast<CMakeBuildConfiguration*>(target->activeBuildConfiguration());
+    if (bs && bc) {
+        if (bs->usesAllCapsTargets())
+            installTarget = "INSTALL";
+        if (bs->isMultiConfig())
+            config << "--config" << bc->cmakeBuildType();
+    }
+
+    FilePath buildDirectory = ".";
+    if (bc)
+        buildDirectory = bc->buildDirectory();
+
+    cmd.arguments << "--build" << buildDirectory.onDevice(cmd.command).path()
+                  << "--target" << installTarget << config;
+
     cmd.environment.set("DESTDIR", QDir::toNativeSeparators(installRoot));
     return cmd;
 }
