@@ -27,34 +27,16 @@
 
 #include "cmake_global.h"
 
-#include "builddirmanager.h"
-#include "cmakebuildsystem.h"
-#include "cmakebuildtarget.h"
-#include "cmakeprojectimporter.h"
-
-#include <projectexplorer/buildsystem.h>
 #include <projectexplorer/project.h>
 
-#include <utils/fileutils.h>
-
-#include <QFuture>
-#include <QHash>
-#include <QTimer>
-
-#include <memory>
-
-namespace CppTools { class CppProjectUpdater; }
-namespace ProjectExplorer { class FileNode; }
-
 namespace CMakeProjectManager {
 
 namespace Internal {
-class CMakeBuildConfiguration;
-class CMakeBuildSettingsWidget;
-class CMakeProjectNode;
-} // namespace Internal
+class CMakeProjectImporter;
+class CMakeBuildSystem;
+}
 
-class CMAKE_EXPORT CMakeProject : public ProjectExplorer::Project
+class CMAKE_EXPORT CMakeProject final : public ProjectExplorer::Project
 {
     Q_OBJECT
 
@@ -64,31 +46,25 @@ public:
 
     ProjectExplorer::Tasks projectIssues(const ProjectExplorer::Kit *k) const final;
 
-    void runCMake();
-    void runCMakeAndScanProjectTree();
-
-    // Context menu actions:
-    void buildCMakeTarget(const QString &buildTarget);
-
     ProjectExplorer::ProjectImporter *projectImporter() const final;
 
-    bool persistCMakeState();
-    void clearCMakeCache();
-    bool mustUpdateCMakeStateBeforeBuild() const;
+    using IssueType = ProjectExplorer::Task::TaskType;
+    void addIssue(IssueType type, const QString &text);
+    void clearIssues();
 
 protected:
     bool setupTarget(ProjectExplorer::Target *t) final;
 
 private:
-    QStringList filesGeneratedFrom(const QString &sourceFile) const final;
-
     ProjectExplorer::DeploymentKnowledge deploymentKnowledge() const override;
     ProjectExplorer::MakeInstallCommand makeInstallCommand(const ProjectExplorer::Target *target,
                                                            const QString &installRoot) override;
 
-    mutable std::unique_ptr<Internal::CMakeProjectImporter> m_projectImporter;
+    mutable Internal::CMakeProjectImporter *m_projectImporter = nullptr;
 
-    friend class CMakeBuildSystem;
+    friend class Internal::CMakeBuildSystem;
+
+    ProjectExplorer::Tasks m_issues;
 };
 
 } // namespace CMakeProjectManager
