@@ -30,6 +30,7 @@
 #include <utils/algorithm.h>
 #include <utils/environment.h>
 #include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
 
 #include <QDir>
 #include <QJsonDocument>
@@ -40,6 +41,8 @@
 
 #include <memory>
 
+using namespace Utils;
+
 namespace CMakeProjectManager {
 
 const char CMAKE_INFORMATION_ID[] = "Id";
@@ -47,8 +50,10 @@ const char CMAKE_INFORMATION_COMMAND[] = "Binary";
 const char CMAKE_INFORMATION_DISPLAYNAME[] = "DisplayName";
 const char CMAKE_INFORMATION_AUTORUN[] = "AutoRun";
 const char CMAKE_INFORMATION_QCH_FILE_PATH[] = "QchFile";
+// obsolete since Qt Creator 5. Kept for backward compatibility
 const char CMAKE_INFORMATION_AUTO_CREATE_BUILD_DIRECTORY[] = "AutoCreateBuildDirectory";
 const char CMAKE_INFORMATION_AUTODETECTED[] = "AutoDetected";
+const char CMAKE_INFORMATION_DETECTIONSOURCE[] = "DetectionSource";
 const char CMAKE_INFORMATION_READERTYPE[] = "ReaderType";
 
 bool CMakeTool::Generator::matches(const QString &n, const QString &ex) const
@@ -107,17 +112,17 @@ public:
 ///////////////////////////
 // CMakeTool
 ///////////////////////////
-CMakeTool::CMakeTool(Detection d, const Utils::Id &id)
+CMakeTool::CMakeTool(Detection d, const Id &id)
     : m_id(id)
     , m_isAutoDetected(d == AutoDetection)
     , m_introspection(std::make_unique<Internal::IntrospectionData>())
 {
-    QTC_ASSERT(m_id.isValid(), m_id = Utils::Id::fromString(QUuid::createUuid().toString()));
+    QTC_ASSERT(m_id.isValid(), m_id = Id::fromString(QUuid::createUuid().toString()));
 }
 
 CMakeTool::CMakeTool(const QVariantMap &map, bool fromSdk) :
     CMakeTool(fromSdk ? CMakeTool::AutoDetection : CMakeTool::ManualDetection,
-              Utils::Id::fromSetting(map.value(CMAKE_INFORMATION_ID)))
+              Id::fromSetting(map.value(CMAKE_INFORMATION_ID)))
 {
     m_displayName = map.value(CMAKE_INFORMATION_DISPLAYNAME).toString();
     m_isAutoRun = map.value(CMAKE_INFORMATION_AUTORUN, true).toBool();
@@ -128,10 +133,11 @@ CMakeTool::CMakeTool(const QVariantMap &map, bool fromSdk) :
     //loading a CMakeTool from SDK is always autodetection
     if (!fromSdk)
         m_isAutoDetected = map.value(CMAKE_INFORMATION_AUTODETECTED, false).toBool();
+    m_detectionSource = map.value(CMAKE_INFORMATION_DETECTIONSOURCE).toString();
 
-    setFilePath(Utils::FilePath::fromString(map.value(CMAKE_INFORMATION_COMMAND).toString()));
+    setFilePath(FilePath::fromString(map.value(CMAKE_INFORMATION_COMMAND).toString()));
 
-    m_qchFilePath = Utils::FilePath::fromVariant(map.value(CMAKE_INFORMATION_QCH_FILE_PATH));
+    m_qchFilePath = FilePath::fromVariant(map.value(CMAKE_INFORMATION_QCH_FILE_PATH));
 
     if (m_qchFilePath.isEmpty())
         m_qchFilePath = searchQchFile(m_executable);
@@ -139,12 +145,12 @@ CMakeTool::CMakeTool(const QVariantMap &map, bool fromSdk) :
 
 CMakeTool::~CMakeTool() = default;
 
-Utils::Id CMakeTool::createId()
+Id CMakeTool::createId()
 {
-    return Utils::Id::fromString(QUuid::createUuid().toString());
+    return Id::fromString(QUuid::createUuid().toString());
 }
 
-void CMakeTool::setFilePath(const Utils::FilePath &executable)
+void CMakeTool::setFilePath(const FilePath &executable)
 {
     if (m_executable == executable)
         return;
@@ -155,7 +161,7 @@ void CMakeTool::setFilePath(const Utils::FilePath &executable)
     CMakeToolManager::notifyAboutUpdate(this);
 }
 
-Utils::FilePath CMakeTool::filePath() const
+FilePath CMakeTool::filePath() const
 {
     return m_executable;
 }
@@ -169,15 +175,6 @@ void CMakeTool::setAutorun(bool autoRun)
     CMakeToolManager::notifyAboutUpdate(this);
 }
 
-void CMakeTool::setAutoCreateBuildDirectory(bool autoBuildDir)
-{
-    if (m_autoCreateBuildDirectory == autoBuildDir)
-        return;
-
-    m_autoCreateBuildDirectory = autoBuildDir;
-    CMakeToolManager::notifyAboutUpdate(this);
-}
-
 bool CMakeTool::isValid() const
 {
     if (!m_id.isValid() || !m_introspection)
@@ -189,17 +186,16 @@ bool CMakeTool::isValid() const
     return m_introspection->m_didRun && !m_introspection->m_fileApis.isEmpty();
 }
 
-Utils::SynchronousProcessResponse CMakeTool::run(const QStringList &args, int timeoutS) const
+void CMakeTool::runCMake(QtcProcess &cmake, const QStringList &args, int timeoutS) const
 {
-    Utils::SynchronousProcess cmake;
     cmake.setTimeoutS(timeoutS);
-    cmake.setFlags(Utils::SynchronousProcess::UnixTerminalDisabled);
-    Utils::Environment env = Utils::Environment::systemEnvironment();
-    Utils::Environment::setupEnglishOutput(&env);
-    cmake.setProcessEnvironment(env.toProcessEnvironment());
+    cmake.setDisableUnixTerminal();
+    Environment env = Environment::systemEnvironment();
+    env.setupEnglishOutput();
+    cmake.setEnvironment(env);
     cmake.setTimeOutMessageBoxEnabled(false);
-
-    return cmake.runBlocking({cmakeExecutable(), args});
+    cmake.setCommand({cmakeExecutable(), args});
+    cmake.runBlocking();
 }
 
 QVariantMap CMakeTool::toMap() const
@@ -215,27 +211,28 @@ QVariantMap CMakeTool::toMap() const
         data.insert(CMAKE_INFORMATION_READERTYPE,
                     Internal::readerTypeToString(m_readerType.value()));
     data.insert(CMAKE_INFORMATION_AUTODETECTED, m_isAutoDetected);
+    data.insert(CMAKE_INFORMATION_DETECTIONSOURCE, m_detectionSource);
     return data;
 }
 
-Utils::FilePath CMakeTool::cmakeExecutable() const
+FilePath CMakeTool::cmakeExecutable() const
 {
     return cmakeExecutable(m_executable);
 }
 
-void CMakeTool::setQchFilePath(const Utils::FilePath &path)
+void CMakeTool::setQchFilePath(const FilePath &path)
 {
     m_qchFilePath = path;
 }
 
-Utils::FilePath CMakeTool::qchFilePath() const
+FilePath CMakeTool::qchFilePath() const
 {
     return m_qchFilePath;
 }
 
-Utils::FilePath CMakeTool::cmakeExecutable(const Utils::FilePath &path)
+FilePath CMakeTool::cmakeExecutable(const FilePath &path)
 {
-    if (Utils::HostOsInfo::isMacHost()) {
+    if (HostOsInfo::isMacHost()) {
         const QString executableString = path.toString();
         const int appIndex = executableString.lastIndexOf(".app");
         const int appCutIndex = appIndex + 4;
@@ -243,17 +240,16 @@ Utils::FilePath CMakeTool::cmakeExecutable(const Utils::FilePath &path)
         const bool containsApp = appIndex >= 0 && !endsWithApp
                                  && executableString.at(appCutIndex) == '/';
         if (endsWithApp || containsApp) {
-            const Utils::FilePath toTest = Utils::FilePath::fromString(
-                                               executableString.left(appCutIndex))
-                                               .pathAppended("Contents/bin/cmake");
+            const FilePath toTest = FilePath::fromString(executableString.left(appCutIndex))
+                    .pathAppended("Contents/bin/cmake");
             if (toTest.exists())
                 return toTest.canonicalPath();
         }
     }
 
-    const Utils::FilePath resolvedPath = path.canonicalPath();
+    const FilePath resolvedPath = path.canonicalPath();
     // Evil hack to make snap-packages of CMake work. See QTCREATORBUG-23376
-    if (Utils::HostOsInfo::isLinuxHost() && resolvedPath.fileName() == "snap")
+    if (HostOsInfo::isLinuxHost() && resolvedPath.fileName() == "snap")
         return path;
 
     return resolvedPath;
@@ -264,11 +260,6 @@ bool CMakeTool::isAutoRun() const
     return m_isAutoRun;
 }
 
-bool CMakeTool::autoCreateBuildDirectory() const
-{
-    return m_autoCreateBuildDirectory;
-}
-
 QList<CMakeTool::Generator> CMakeTool::supportedGenerators() const
 {
     return isValid() ? m_introspection->m_generators : QList<CMakeTool::Generator>();
@@ -280,22 +271,22 @@ TextEditor::Keywords CMakeTool::keywords()
         return {};
 
     if (m_introspection->m_functions.isEmpty() && m_introspection->m_didRun) {
-        Utils::SynchronousProcessResponse response;
-        response = run({"--help-command-list"}, 5);
-        if (response.result == Utils::SynchronousProcessResponse::Finished)
-            m_introspection->m_functions = response.stdOut().split('\n');
+        QtcProcess proc;
+        runCMake(proc, {"--help-command-list"}, 5);
+        if (proc.result() == QtcProcess::FinishedWithSuccess)
+            m_introspection->m_functions = proc.stdOut().split('\n');
 
-        response = run({"--help-commands"}, 5);
-        if (response.result == Utils::SynchronousProcessResponse::Finished)
-            parseFunctionDetailsOutput(response.stdOut());
+        runCMake(proc, {"--help-commands"}, 5);
+        if (proc.result() == QtcProcess::FinishedWithSuccess)
+            parseFunctionDetailsOutput(proc.stdOut());
 
-        response = run({"--help-property-list"}, 5);
-        if (response.result == Utils::SynchronousProcessResponse::Finished)
-            m_introspection->m_variables = parseVariableOutput(response.stdOut());
+        runCMake(proc, {"--help-property-list"}, 5);
+        if (proc.result() == QtcProcess::FinishedWithSuccess)
+            m_introspection->m_variables = parseVariableOutput(proc.stdOut());
 
-        response = run({"--help-variable-list"}, 5);
-        if (response.result == Utils::SynchronousProcessResponse::Finished) {
-            m_introspection->m_variables.append(parseVariableOutput(response.stdOut()));
+        runCMake(proc, {"--help-variable-list"}, 5);
+        if (proc.result() == QtcProcess::FinishedWithSuccess) {
+            m_introspection->m_variables.append(parseVariableOutput(proc.stdOut()));
             m_introspection->m_variables = Utils::filteredUnique(m_introspection->m_variables);
             Utils::sort(m_introspection->m_variables);
         }
@@ -311,16 +302,23 @@ bool CMakeTool::hasFileApi() const
     return isValid() ? !m_introspection->m_fileApis.isEmpty() : false;
 }
 
-QVector<std::pair<QString, int>> CMakeTool::supportedFileApiObjects() const
-{
-    return isValid() ? Utils::transform(m_introspection->m_fileApis, [](const Internal::FileApi &api) { return std::make_pair(api.kind, api.version.first); }) : QVector<std::pair<QString, int>>();
-}
-
 CMakeTool::Version CMakeTool::version() const
 {
     return m_introspection ? m_introspection->m_version : CMakeTool::Version();
 }
 
+QString CMakeTool::versionDisplay() const
+{
+    if (!m_introspection)
+        return CMakeToolManager::tr("Version not parseable");
+
+    const Version &version = m_introspection->m_version;
+    if (version.fullVersion.isEmpty())
+        return QString::fromUtf8(version.fullVersion);
+
+    return QString("%1.%2.%3").arg(version.major).arg(version.minor).arg(version.patch);
+}
+
 bool CMakeTool::isAutoDetected() const
 {
     return m_isAutoDetected;
@@ -346,7 +344,7 @@ CMakeTool::PathMapper CMakeTool::pathMapper() const
 {
     if (m_pathMapper)
         return m_pathMapper;
-    return [](const Utils::FilePath &fn) { return fn; };
+    return [](const FilePath &fn) { return fn; };
 }
 
 Utils::optional<CMakeTool::ReaderType> CMakeTool::readerType() const
@@ -360,12 +358,12 @@ Utils::optional<CMakeTool::ReaderType> CMakeTool::readerType() const
     return {};
 }
 
-Utils::FilePath CMakeTool::searchQchFile(const Utils::FilePath &executable)
+FilePath CMakeTool::searchQchFile(const FilePath &executable)
 {
-    if (executable.isEmpty())
+    if (executable.isEmpty() || executable.needsDevice()) // do not register docs from devices
         return {};
 
-    Utils::FilePath prefixDir = executable.parentDir().parentDir();
+    FilePath prefixDir = executable.parentDir().parentDir();
     QDir docDir{prefixDir.pathAppended("doc/cmake").toString()};
     if (!docDir.exists())
         docDir.setPath(prefixDir.pathAppended("share/doc/cmake").toString());
@@ -375,7 +373,7 @@ Utils::FilePath CMakeTool::searchQchFile(const Utils::FilePath &executable)
     const QStringList files = docDir.entryList(QStringList("*.qch"));
     for (const QString &docFile : files) {
         if (docFile.startsWith("cmake", Qt::CaseInsensitive)) {
-            return Utils::FilePath::fromString(docDir.absoluteFilePath(docFile));
+            return FilePath::fromString(docDir.absoluteFilePath(docFile));
         }
     }
 
@@ -486,11 +484,12 @@ QStringList CMakeTool::parseVariableOutput(const QString &output)
 
 void CMakeTool::fetchFromCapabilities() const
 {
-    Utils::SynchronousProcessResponse response = run({"-E", "capabilities"});
+    QtcProcess cmake;
+    runCMake(cmake, {"-E", "capabilities"});
 
-    if (response.result == Utils::SynchronousProcessResponse::Finished) {
+    if (cmake.result() == QtcProcess::FinishedWithSuccess) {
         m_introspection->m_didRun = true;
-        parseFromCapabilities(response.stdOut());
+        parseFromCapabilities(cmake.stdOut());
     } else {
         m_introspection->m_didRun = false;
     }
