@@ -28,11 +28,16 @@
 #include "fileapiparser.h"
 #include "projecttreehelper.h"
 
+#include <cppeditor/cppeditorconstants.h>
+
 #include <utils/algorithm.h>
+#include <utils/mimetypes/mimedatabase.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
 #include <utils/utilsicons.h>
 
+#include <projectexplorer/projecttree.h>
+
 #include <QDir>
 
 using namespace ProjectExplorer;
@@ -51,7 +56,7 @@ using namespace CMakeProjectManager::Internal::FileApiDetails;
 class CMakeFileResult
 {
 public:
-    QSet<FilePath> cmakeFiles;
+    QSet<CMakeFileInfo> cmakeFiles;
 
     std::vector<std::unique_ptr<ProjectExplorer::FileNode>> cmakeNodesSource;
     std::vector<std::unique_ptr<ProjectExplorer::FileNode>> cmakeNodesBuild;
@@ -59,20 +64,18 @@ public:
     std::vector<std::unique_ptr<ProjectExplorer::FileNode>> cmakeListNodes;
 };
 
-CMakeFileResult extractCMakeFilesData(const std::vector<FileApiDetails::CMakeFileInfo> &cmakefiles,
+CMakeFileResult extractCMakeFilesData(const std::vector<CMakeFileInfo> &cmakefiles,
                                       const FilePath &sourceDirectory,
                                       const FilePath &buildDirectory)
 {
     CMakeFileResult result;
 
-    QDir sourceDir(sourceDirectory.toString());
-    QDir buildDir(buildDirectory.toString());
-
     for (const CMakeFileInfo &info : cmakefiles) {
-        const FilePath sfn = FilePath::fromString(
-            QDir::cleanPath(sourceDir.absoluteFilePath(info.path)));
+        const FilePath sfn = sourceDirectory.resolvePath(info.path);
         const int oldCount = result.cmakeFiles.count();
-        result.cmakeFiles.insert(sfn);
+        CMakeFileInfo absolute(info);
+        absolute.path = sfn;
+        result.cmakeFiles.insert(absolute);
         if (oldCount < result.cmakeFiles.count()) {
             if (info.isCMake && !info.isCMakeListsDotTxt) {
                 // Skip files that cmake considers to be part of the installation -- but include
@@ -89,9 +92,9 @@ CMakeFileResult extractCMakeFilesData(const std::vector<FileApiDetails::CMakeFil
 
             if (info.isCMakeListsDotTxt) {
                 result.cmakeListNodes.emplace_back(std::move(node));
-            } else if (sfn.isChildOf(sourceDir)) {
+            } else if (sfn.isChildOf(sourceDirectory)) {
                 result.cmakeNodesSource.emplace_back(std::move(node));
-            } else if (sfn.isChildOf(buildDir)) {
+            } else if (sfn.isChildOf(buildDirectory)) {
                 result.cmakeNodesBuild.emplace_back(std::move(node));
             } else {
                 result.cmakeNodesOther.emplace_back(std::move(node));
@@ -102,28 +105,12 @@ CMakeFileResult extractCMakeFilesData(const std::vector<FileApiDetails::CMakeFil
     return result;
 }
 
-Configuration extractConfiguration(std::vector<Configuration> &codemodel, QString &errorMessage)
-{
-    if (codemodel.size() == 0) {
-        qWarning() << "No configuration found!";
-        errorMessage = "No configuration found!";
-        return {};
-    }
-    if (codemodel.size() > 1)
-        qWarning() << "Multi-configuration generator found, ignoring all but first configuration";
-
-    Configuration result = std::move(codemodel[0]);
-    codemodel.clear();
-
-    return result;
-}
-
 class PreprocessedData
 {
 public:
     CMakeProjectManager::CMakeConfig cache;
 
-    QSet<FilePath> cmakeFiles;
+    QSet<CMakeFileInfo> cmakeFiles;
 
     std::vector<std::unique_ptr<ProjectExplorer::FileNode>> cmakeNodesSource;
     std::vector<std::unique_ptr<ProjectExplorer::FileNode>> cmakeNodesBuild;
@@ -139,16 +126,13 @@ PreprocessedData preprocess(FileApiData &data,
                             const FilePath &buildDirectory,
                             QString &errorMessage)
 {
+    Q_UNUSED(errorMessage)
+
     PreprocessedData result;
 
     result.cache = std::move(data.cache); // Make sure this is available, even when nothing else is
 
-    // Simplify to only one configuration:
-    result.codemodel = extractConfiguration(data.codemodel, errorMessage);
-
-    if (!errorMessage.isEmpty()) {
-        return result;
-    }
+    result.codemodel = std::move(data.codemodel);
 
     CMakeFileResult cmakeFileResult = extractCMakeFilesData(data.cmakeFiles,
                                                             sourceDirectory,
@@ -198,23 +182,30 @@ QVector<FolderNode::LocationInfo> extractBacktraceInformation(const BacktraceInf
     return info;
 }
 
+static bool isChildOf(const FilePath &path, const QStringList &prefixes)
+{
+    for (const QString &prefix : prefixes)
+        if (path.isChildOf(FilePath::fromString(prefix)))
+            return true;
+    return false;
+}
+
 QList<CMakeBuildTarget> generateBuildTargets(const PreprocessedData &input,
                                              const FilePath &sourceDirectory,
-                                             const FilePath &buildDirectory)
+                                             const FilePath &buildDirectory,
+                                             bool haveLibrariesRelativeToBuildDirectory)
 {
     QDir sourceDir(sourceDirectory.toString());
-    QDir buildDir(buildDirectory.toString());
 
-    const QList<CMakeBuildTarget> result = transform<QList>(
-        input.targetDetails, [&sourceDir, &buildDir](const TargetDetails &t) -> CMakeBuildTarget {
-            const auto currentBuildDir = QDir(buildDir.absoluteFilePath(t.buildDir.toString()));
+    const QList<CMakeBuildTarget> result = transform<QList>(input.targetDetails,
+        [&sourceDir, &sourceDirectory, &buildDirectory,
+         &haveLibrariesRelativeToBuildDirectory](const TargetDetails &t) {
+            const FilePath currentBuildDir = buildDirectory.resolvePath(t.buildDir);
 
             CMakeBuildTarget ct;
             ct.title = t.name;
-            ct.executable = t.artifacts.isEmpty()
-                                ? FilePath()
-                                : FilePath::fromString(QDir::cleanPath(
-                                    buildDir.absoluteFilePath(t.artifacts.at(0).toString())));
+            if (!t.artifacts.isEmpty())
+                ct.executable = buildDirectory.resolvePath(t.artifacts.at(0));
             TargetType type = UtilityType;
             if (t.type == "EXECUTABLE")
                 type = ExecutableType;
@@ -228,10 +219,9 @@ QList<CMakeBuildTarget> generateBuildTargets(const PreprocessedData &input,
                 type = UtilityType;
             ct.targetType = type;
             ct.workingDirectory = ct.executable.isEmpty()
-                                      ? FilePath::fromString(currentBuildDir.absolutePath())
+                                      ? currentBuildDir.absolutePath()
                                       : ct.executable.parentDir();
-            ct.sourceDirectory = FilePath::fromString(
-                QDir::cleanPath(sourceDir.absoluteFilePath(t.sourceDir.toString())));
+            ct.sourceDirectory = sourceDirectory.resolvePath(t.sourceDir);
 
             ct.backtrace = extractBacktraceInformation(t.backtraceGraph, sourceDir, t.backtrace, 0);
 
@@ -254,7 +244,7 @@ QList<CMakeBuildTarget> generateBuildTargets(const PreprocessedData &input,
                 }
             }
             for (const InstallDestination &id : t.installDestination) {
-                ct.includeDefinitions.append(
+                ct.installDefinitions.append(
                     extractBacktraceInformation(t.backtraceGraph, sourceDir, id.backtrace, 500));
             }
 
@@ -269,35 +259,49 @@ QList<CMakeBuildTarget> generateBuildTargets(const PreprocessedData &input,
                                                                  || f.fragment.contains("Qt6Gui"));
                                                   });
 
+                ct.qtcRunnable = t.folderTargetProperty == "qtc_runnable";
+
                 // Extract library directories for executables:
                 for (const FragmentInfo &f : t.link.value().fragments) {
                     if (f.role == "flags") // ignore all flags fragments
                         continue;
 
                     // CMake sometimes mixes several shell-escaped pieces into one fragment. Disentangle that again:
-                    const QStringList parts = QtcProcess::splitArgs(f.fragment);
+                    const QStringList parts = ProcessArgs::splitArgs(f.fragment);
                     for (const QString &part : parts) {
                         // Some projects abuse linking to libraries to pass random flags to the linker, so ignore
                         // flags mixed into a fragment
                         if (part.startsWith("-"))
                             continue;
 
-                        FilePath tmp = FilePath::fromString(
-                            currentBuildDir.absoluteFilePath(QDir::fromNativeSeparators(part)));
+                        const FilePath buildDir = haveLibrariesRelativeToBuildDirectory ? buildDirectory : currentBuildDir;
+                        FilePath tmp = buildDir.resolvePath(FilePath::fromUserInput(part));
 
                         if (f.role == "libraries")
                             tmp = tmp.parentDir();
 
-                        if (!tmp.isEmpty()
-                            && tmp.isDir()) { // f.role is libraryPath or frameworkPath
-                            librarySeachPaths.append(tmp);
-                            // Libraries often have their import libs in ../lib and the
-                            // actual dll files in ../bin on windows. Qt is one example of that.
-                            if (tmp.fileName() == "lib" && HostOsInfo::isWindowsHost()) {
-                                const FilePath path = tmp.parentDir().pathAppended("bin");
+                        if (!tmp.isEmpty() && tmp.isDir()) {
+                            // f.role is libraryPath or frameworkPath
+                            // On Linux, exclude sub-paths from "/lib(64)", "/usr/lib(64)" and
+                            // "/usr/local/lib" since these are usually in the standard search
+                            // paths. There probably are more, but the naming schemes are arbitrary
+                            // so we'd need to ask the linker ("ld --verbose | grep SEARCH_DIR").
+                            if (!HostOsInfo::isLinuxHost()
+                                || !isChildOf(tmp,
+                                              {"/lib",
+                                               "/lib64",
+                                               "/usr/lib",
+                                               "/usr/lib64",
+                                               "/usr/local/lib"})) {
+                                librarySeachPaths.append(tmp);
+                                // Libraries often have their import libs in ../lib and the
+                                // actual dll files in ../bin on windows. Qt is one example of that.
+                                if (tmp.fileName() == "lib" && HostOsInfo::isWindowsHost()) {
+                                    const FilePath path = tmp.parentDir().pathAppended("bin");
 
-                                if (path.isDir())
-                                    librarySeachPaths.append(path);
+                                    if (path.isDir())
+                                        librarySeachPaths.append(path);
+                                }
                             }
                         }
                     }
@@ -314,20 +318,25 @@ static QStringList splitFragments(const QStringList &fragments)
 {
     QStringList result;
     for (const QString &f : fragments) {
-        result += QtcProcess::splitArgs(f);
+        result += ProcessArgs::splitArgs(f);
     }
     return result;
 }
 
+bool isPchFile(const FilePath &buildDirectory, const FilePath &path)
+{
+    return path.isChildOf(buildDirectory) && path.fileName().startsWith("cmake_pch");
+}
+
 RawProjectParts generateRawProjectParts(const PreprocessedData &input,
-                                        const FilePath &sourceDirectory)
+                                        const FilePath &sourceDirectory,
+                                        const FilePath &buildDirectory)
 {
     RawProjectParts rpps;
 
     int counter = 0;
     for (const TargetDetails &t : input.targetDetails) {
         QDir sourceDir(sourceDirectory.toString());
-
         bool needPostfix = t.compileGroups.size() > 1;
         int count = 1;
         for (const CompileInfo &ci : t.compileGroups) {
@@ -348,10 +357,15 @@ RawProjectParts generateRawProjectParts(const PreprocessedData &input,
             }
 
             QString ending;
-            if (ci.language == "C")
+            QString qtcPchFile;
+            if (ci.language == "C") {
                 ending = "/cmake_pch.h";
-            else if (ci.language == "CXX")
+                qtcPchFile = "qtc_cmake_pch.h";
+            }
+            else if (ci.language == "CXX") {
                 ending = "/cmake_pch.hxx";
+                qtcPchFile = "qtc_cmake_pch.hxx";
+            }
 
             ++counter;
             RawProjectPart rpp;
@@ -362,30 +376,84 @@ RawProjectParts generateRawProjectParts(const PreprocessedData &input,
             rpp.setMacros(transform<QVector>(ci.defines, &DefineInfo::define));
             rpp.setHeaderPaths(transform<QVector>(ci.includes, &IncludeInfo::path));
 
-            RawProjectPartFlags cProjectFlags;
-            cProjectFlags.commandLineFlags = splitFragments(ci.fragments);
-            rpp.setFlagsForC(cProjectFlags);
+            QStringList fragments = splitFragments(ci.fragments);
 
-            RawProjectPartFlags cxxProjectFlags;
-            cxxProjectFlags.commandLineFlags = cProjectFlags.commandLineFlags;
-            rpp.setFlagsForCxx(cxxProjectFlags);
+            // Get all sources from the compiler group, except generated sources
+            QStringList sources;
+            for (auto idx: ci.sources) {
+                SourceInfo si = t.sources.at(idx);
+                if (si.isGenerated)
+                    continue;
+                sources.push_back(sourceDir.absoluteFilePath(si.path));
+            }
+
+            // If we are not in a pch compiler group, add all the headers that are not generated
+            const bool hasPchSource = anyOf(sources, [buildDirectory](const QString &path) {
+                return isPchFile(buildDirectory, FilePath::fromString(path));
+            });
+            if (!hasPchSource) {
+                QString headerMimeType;
+                if (ci.language == "C")
+                    headerMimeType = CppEditor::Constants::C_HEADER_MIMETYPE;
+                else if (ci.language == "CXX")
+                    headerMimeType = CppEditor::Constants::CPP_HEADER_MIMETYPE;
+
+                for (const SourceInfo &si : t.sources) {
+                    if (si.isGenerated)
+                        continue;
+                    const auto mimeTypes = Utils::mimeTypesForFileName(si.path);
+                    for (auto mime : mimeTypes)
+                        if (mime.name() == headerMimeType)
+                            sources.push_back(sourceDir.absoluteFilePath(si.path));
+                }
+            }
+
+            // Set project files except pch files
+            rpp.setFiles(Utils::filtered(sources, [buildDirectory](const QString &path) {
+                return !isPchFile(buildDirectory, FilePath::fromString(path));
+            }));
 
             FilePath precompiled_header
                 = FilePath::fromString(findOrDefault(t.sources, [&ending](const SourceInfo &si) {
                                            return si.path.endsWith(ending);
                                        }).path);
-
-            rpp.setFiles(transform<QList>(ci.sources, [&t, &sourceDir](const int si) {
-                return sourceDir.absoluteFilePath(t.sources[static_cast<size_t>(si)].path);
-            }));
             if (!precompiled_header.isEmpty()) {
                 if (precompiled_header.toFileInfo().isRelative()) {
                     const FilePath parentDir = FilePath::fromString(sourceDir.absolutePath());
                     precompiled_header = parentDir.pathAppended(precompiled_header.toString());
                 }
-                rpp.setPreCompiledHeaders({precompiled_header.toString()});
+
+                // Remove the CMake PCH usage command line options in order to avoid the case
+                // when the build system would produce a .pch/.gch file that would be treated
+                // by the Clang code model as its own and fail.
+                auto remove = [&](const QStringList &args) {
+                    auto foundPos = std::search(fragments.begin(), fragments.end(),
+                                                args.begin(), args.end());
+                    if (foundPos != fragments.end())
+                        fragments.erase(foundPos, std::next(foundPos, args.size()));
+                };
+
+                remove({"-Xclang", "-include-pch", "-Xclang", precompiled_header.toString() + ".gch"});
+                remove({"-Xclang", "-include-pch", "-Xclang", precompiled_header.toString() + ".pch"});
+                remove({"-Xclang", "-include", "-Xclang", precompiled_header.toString()});
+                remove({"-include", precompiled_header.toString()});
+                remove({"/FI", precompiled_header.toString()});
+
+                // Make a copy of the CMake PCH header and use it instead
+                FilePath qtc_precompiled_header = precompiled_header.parentDir().pathAppended(qtcPchFile);
+                FileUtils::copyIfDifferent(precompiled_header, qtc_precompiled_header);
+
+                rpp.setPreCompiledHeaders({qtc_precompiled_header.toString()});
             }
 
+            RawProjectPartFlags cProjectFlags;
+            cProjectFlags.commandLineFlags = fragments;
+            rpp.setFlagsForC(cProjectFlags);
+
+            RawProjectPartFlags cxxProjectFlags;
+            cxxProjectFlags.commandLineFlags = cProjectFlags.commandLineFlags;
+            rpp.setFlagsForCxx(cxxProjectFlags);
+
             const bool isExecutable = t.type == "EXECUTABLE";
             rpp.setBuildTargetType(isExecutable ? ProjectExplorer::BuildTargetType::Executable
                                                 : ProjectExplorer::BuildTargetType::Library);
@@ -397,27 +465,25 @@ RawProjectParts generateRawProjectParts(const PreprocessedData &input,
     return rpps;
 }
 
-FilePath directorySourceDir(const Configuration &c, const QDir &sourceDir, int directoryIndex)
+FilePath directorySourceDir(const Configuration &c, const FilePath &sourceDir, int directoryIndex)
 {
     const size_t di = static_cast<size_t>(directoryIndex);
     QTC_ASSERT(di < c.directories.size(), return FilePath());
 
-    return FilePath::fromString(
-        QDir::cleanPath(sourceDir.absoluteFilePath(c.directories[di].sourcePath)));
+    return sourceDir.resolvePath(c.directories[di].sourcePath).cleanPath();
 }
 
-FilePath directoryBuildDir(const Configuration &c, const QDir &buildDir, int directoryIndex)
+FilePath directoryBuildDir(const Configuration &c, const FilePath &buildDir, int directoryIndex)
 {
     const size_t di = static_cast<size_t>(directoryIndex);
     QTC_ASSERT(di < c.directories.size(), return FilePath());
 
-    return FilePath::fromString(
-        QDir::cleanPath(buildDir.absoluteFilePath(c.directories[di].buildPath)));
+    return buildDir.resolvePath(c.directories[di].buildPath).cleanPath();
 }
 
 void addProjects(const QHash<Utils::FilePath, ProjectNode *> &cmakeListsNodes,
                  const Configuration &config,
-                 const QDir &sourceDir)
+                 const FilePath &sourceDir)
 {
     for (const FileApiDetails::Project &p : config.projects) {
         if (p.parent == -1)
@@ -445,7 +511,8 @@ FolderNode *createSourceGroupNode(const QString &sourceGroupName,
             if (!existingNode) {
                 auto node = createCMakeVFolder(sourceDirectory, Node::DefaultFolderPriority + 5, p);
                 node->setListInProject(false);
-                node->setIcon(QIcon::fromTheme("edit-copy", ::Utils::Icons::COPY.icon()));
+                node->setIcon(
+                    [] { return QIcon::fromTheme("edit-copy", ::Utils::Icons::COPY.icon()); });
 
                 existingNode = node.get();
 
@@ -462,8 +529,7 @@ void addCompileGroups(ProjectNode *targetRoot,
                       const Utils::FilePath &topSourceDirectory,
                       const Utils::FilePath &sourceDirectory,
                       const Utils::FilePath &buildDirectory,
-                      const TargetDetails &td,
-                      QSet<FilePath> &knownHeaderNodes)
+                      const TargetDetails &td)
 {
     const bool inSourceBuild = (sourceDirectory == buildDirectory);
 
@@ -474,15 +540,12 @@ void addCompileGroups(ProjectNode *targetRoot,
     targetRoot->forEachGenericNode(
         [&alreadyListed](const Node *n) { alreadyListed.insert(n->filePath()); });
 
-    const QDir topSourceDir(topSourceDirectory.toString());
-
     std::vector<std::unique_ptr<FileNode>> buildFileNodes;
     std::vector<std::unique_ptr<FileNode>> otherFileNodes;
     std::vector<std::vector<std::unique_ptr<FileNode>>> sourceGroupFileNodes{td.sourceGroups.size()};
 
     for (const SourceInfo &si : td.sources) {
-        const FilePath sourcePath = FilePath::fromString(
-            QDir::cleanPath(topSourceDir.absoluteFilePath(si.path)));
+        const FilePath sourcePath = topSourceDirectory.resolvePath(si.path).cleanPath();
 
         // Filter out already known files:
         const int count = alreadyListed.count();
@@ -494,9 +557,10 @@ void addCompileGroups(ProjectNode *targetRoot,
         auto node = std::make_unique<FileNode>(sourcePath, Node::fileTypeForFileName(sourcePath));
         node->setIsGenerated(si.isGenerated);
 
-        // Register headers:
-        if (node->fileType() == FileType::Header)
-            knownHeaderNodes.insert(node->filePath());
+        // CMake pch files are generated at configured time, but not marked as generated
+        // so that a "clean" step won't remove them and at a subsequent build they won't exist.
+        if (isPchFile(buildDirectory, sourcePath))
+            node->setIsGenerated(true);
 
         // Where does the file node need to go?
         if (sourcePath.isChildOf(buildDirectory) && !inSourceBuild) {
@@ -544,14 +608,23 @@ void addCompileGroups(ProjectNode *targetRoot,
 void addTargets(const QHash<Utils::FilePath, ProjectExplorer::ProjectNode *> &cmakeListsNodes,
                 const Configuration &config,
                 const std::vector<TargetDetails> &targetDetails,
-                const FilePath &topSourceDir,
-                const QDir &sourceDir,
-                const QDir &buildDir,
-                QSet<FilePath> &knownHeaderNodes)
+                const FilePath &sourceDir,
+                const FilePath &buildDir)
 {
+    QHash<QString, const TargetDetails *> targetDetailsHash;
+    for (const TargetDetails &t : targetDetails)
+        targetDetailsHash.insert(t.id, &t);
+    const TargetDetails defaultTargetDetails;
+    auto getTargetDetails = [&targetDetailsHash, &defaultTargetDetails](const QString &id)
+            -> const TargetDetails & {
+        auto it = targetDetailsHash.constFind(id);
+        if (it != targetDetailsHash.constEnd())
+            return *it.value();
+        return defaultTargetDetails;
+    };
+
     for (const FileApiDetails::Target &t : config.targets) {
-        const TargetDetails &td = Utils::findOrDefault(targetDetails,
-                                                       Utils::equal(&TargetDetails::id, t.id));
+        const TargetDetails &td = getTargetDetails(t.id);
 
         const FilePath dir = directorySourceDir(config, sourceDir, t.directory);
 
@@ -561,46 +634,37 @@ void addTargets(const QHash<Utils::FilePath, ProjectExplorer::ProjectNode *> &cm
         tNode->setTargetInformation(td.artifacts, td.type);
         tNode->setBuildDirectory(directoryBuildDir(config, buildDir, t.directory));
 
-        addCompileGroups(tNode, topSourceDir, dir, tNode->buildDirectory(), td, knownHeaderNodes);
+        addCompileGroups(tNode, sourceDir, dir, tNode->buildDirectory(), td);
     }
 }
 
-std::pair<std::unique_ptr<CMakeProjectNode>, QSet<FilePath>> generateRootProjectNode(
+std::unique_ptr<CMakeProjectNode> generateRootProjectNode(
     PreprocessedData &data, const FilePath &sourceDirectory, const FilePath &buildDirectory)
 {
-    std::pair<std::unique_ptr<CMakeProjectNode>, QSet<FilePath>> result;
-    result.first = std::make_unique<CMakeProjectNode>(sourceDirectory);
-
-    const QDir sourceDir(sourceDirectory.toString());
-    const QDir buildDir(buildDirectory.toString());
+    std::unique_ptr<CMakeProjectNode> result = std::make_unique<CMakeProjectNode>(sourceDirectory);
 
     const FileApiDetails::Project topLevelProject
         = findOrDefault(data.codemodel.projects, equal(&FileApiDetails::Project::parent, -1));
     if (!topLevelProject.name.isEmpty())
-        result.first->setDisplayName(topLevelProject.name);
+        result->setDisplayName(topLevelProject.name);
     else
-        result.first->setDisplayName(sourceDirectory.fileName());
+        result->setDisplayName(sourceDirectory.fileName());
 
-    QHash<FilePath, ProjectNode *> cmakeListsNodes = addCMakeLists(result.first.get(),
+    QHash<FilePath, ProjectNode *> cmakeListsNodes = addCMakeLists(result.get(),
                                                                    std::move(data.cmakeListNodes));
     data.cmakeListNodes.clear(); // Remove all the nullptr in the vector...
 
-    QSet<FilePath> knownHeaders;
-    addProjects(cmakeListsNodes, data.codemodel, sourceDir);
+    addProjects(cmakeListsNodes, data.codemodel, sourceDirectory);
 
     addTargets(cmakeListsNodes,
                data.codemodel,
                data.targetDetails,
                sourceDirectory,
-               sourceDir,
-               buildDir,
-               knownHeaders);
-
-    // addHeaderNodes(root.get(), knownHeaders, allFiles);
+               buildDirectory);
 
     if (!data.cmakeNodesSource.empty() || !data.cmakeNodesBuild.empty()
         || !data.cmakeNodesOther.empty())
-        addCMakeInputs(result.first.get(),
+        addCMakeInputs(result.get(),
                        sourceDirectory,
                        buildDirectory,
                        std::move(data.cmakeNodesSource),
@@ -611,16 +675,21 @@ std::pair<std::unique_ptr<CMakeProjectNode>, QSet<FilePath>> generateRootProject
     data.cmakeNodesBuild.clear();  // Remove all the nullptr in the vector...
     data.cmakeNodesOther.clear();  // Remove all the nullptr in the vector...
 
-    result.second = knownHeaders;
-
     return result;
 }
 
 void setupLocationInfoForTargets(CMakeProjectNode *rootNode, const QList<CMakeBuildTarget> &targets)
 {
+    const QSet<QString> titles = Utils::transform<QSet>(targets, &CMakeBuildTarget::title);
+    QHash<QString, FolderNode *> buildKeyToNode;
+    rootNode->forEachGenericNode([&buildKeyToNode, &titles](Node *node) {
+        FolderNode *folderNode = node->asFolderNode();
+        const QString &buildKey = node->buildKey();
+        if (folderNode && titles.contains(buildKey))
+            buildKeyToNode.insert(buildKey, folderNode);
+    });
     for (const CMakeBuildTarget &t : targets) {
-        FolderNode *folderNode = static_cast<FolderNode *>(
-            rootNode->findNode(Utils::equal(&Node::buildKey, t.title)));
+        FolderNode *folderNode = buildKeyToNode.value(t.title);
         if (folderNode) {
             QSet<std::pair<FilePath, int>> locations;
             auto dedup = [&locations](const Backtrace &bt) {
@@ -678,16 +747,26 @@ FileApiQtcData extractData(FileApiData &input,
         return {};
     }
 
-    result.buildTargets = generateBuildTargets(data, sourceDirectory, buildDirectory);
-    result.cmakeFiles = std::move(data.cmakeFiles);
-    result.projectParts = generateRawProjectParts(data, sourceDirectory);
+    // Ninja generator from CMake version 3.20.5 has libraries relative to build directory
+    const bool haveLibrariesRelativeToBuildDirectory =
+            input.replyFile.generator.startsWith("Ninja")
+         && input.replyFile.cmakeVersion >= QVersionNumber(3, 20, 5);
 
-    auto pair = generateRootProjectNode(data, sourceDirectory, buildDirectory);
-    result.rootProjectNode = std::move(pair.first);
-    result.knownHeaders = std::move(pair.second);
+    result.buildTargets = generateBuildTargets(data, sourceDirectory, buildDirectory, haveLibrariesRelativeToBuildDirectory);
+    result.cmakeFiles = std::move(data.cmakeFiles);
+    result.projectParts = generateRawProjectParts(data, sourceDirectory, buildDirectory);
+
+    auto rootProjectNode = generateRootProjectNode(data, sourceDirectory, buildDirectory);
+    ProjectTree::applyTreeManager(rootProjectNode.get(), ProjectTree::AsyncPhase); // QRC nodes
+    result.rootProjectNode = std::move(rootProjectNode);
 
     setupLocationInfoForTargets(result.rootProjectNode.get(), result.buildTargets);
 
+    result.ctestPath = input.replyFile.ctestExecutable;
+    result.isMultiConfig = input.replyFile.isMultiConfig;
+    if (input.replyFile.isMultiConfig && input.replyFile.generator != "Ninja Multi-Config")
+        result.usesAllCapsTargets = true;
+
     return result;
 }
 
