@@ -61,10 +61,11 @@ static FilePath cmakeReplyDirectory(const FilePath &buildDirectory)
 
 static void reportFileApiSetupFailure()
 {
-    Core::MessageManager::write(QCoreApplication::translate(
-        "CMakeProjectManager::Internal",
-        "Failed to set up CMake file API support. %1 cannot "
-        "extract project information.").arg(Core::Constants::IDE_DISPLAY_NAME));
+    Core::MessageManager::writeFlashing(
+        QCoreApplication::translate("CMakeProjectManager::Internal",
+                                    "Failed to set up CMake file API support. %1 cannot "
+                                    "extract project information.")
+            .arg(Core::Constants::IDE_DISPLAY_NAME));
 }
 
 static std::pair<int, int> cmakeVersion(const QJsonObject &obj)
@@ -90,13 +91,12 @@ static std::pair<QString, QString> nameValue(const QJsonObject &obj)
     return std::make_pair(obj.value("name").toString(), obj.value("value").toString());
 }
 
-static QJsonDocument readJsonFile(const QString &path)
+static QJsonDocument readJsonFile(const FilePath &filePath)
 {
-    qCDebug(cmakeFileApi) << "readJsonFile:" << path;
+    qCDebug(cmakeFileApi) << "readJsonFile:" << filePath;
+    QTC_ASSERT(!filePath.isEmpty(), return {});
 
-    QFile file(path);
-    file.open(QIODevice::ReadOnly | QIODevice::Text);
-    const QJsonDocument doc = QJsonDocument::fromJson(file.readAll());
+    const QJsonDocument doc = QJsonDocument::fromJson(filePath.fileContents());
 
     return doc;
 }
@@ -115,9 +115,9 @@ std::vector<int> indexList(const QJsonValue &v)
 
 // Reply file:
 
-static ReplyFileContents readReplyFile(const QFileInfo &fi, QString &errorMessage)
+static ReplyFileContents readReplyFile(const FilePath &filePath, QString &errorMessage)
 {
-    const QJsonDocument document = readJsonFile(fi.filePath());
+    const QJsonDocument document = readJsonFile(filePath);
     static const QString msg = QCoreApplication::translate("CMakeProjectManager::Internal",
                                                            "Invalid reply file created by CMake.");
 
@@ -135,11 +135,20 @@ static ReplyFileContents readReplyFile(const QFileInfo &fi, QString &errorMessag
             const QJsonObject paths = cmakeObject.value("paths").toObject();
             {
                 result.cmakeExecutable = paths.value("cmake").toString();
+                result.ctestExecutable = paths.value("ctest").toString();
                 result.cmakeRoot = paths.value("root").toString();
             }
             const QJsonObject generator = cmakeObject.value("generator").toObject();
             {
                 result.generator = generator.value("name").toString();
+                result.isMultiConfig = generator.value("multiConfig").toBool();
+            }
+            const QJsonObject version = cmakeObject.value("version").toObject();
+            {
+                int major = version.value("major").toInt();
+                int minor = version.value("minor").toInt();
+                int patch = version.value("patch").toInt();
+                result.cmakeVersion = QVersionNumber(major, minor, patch);
             }
         }
     }
@@ -173,7 +182,7 @@ static ReplyFileContents readReplyFile(const QFileInfo &fi, QString &errorMessag
 
 // Cache file:
 
-static CMakeConfig readCacheFile(const QString &cacheFile, QString &errorMessage)
+static CMakeConfig readCacheFile(const FilePath &cacheFile, QString &errorMessage)
 {
     CMakeConfig result;
 
@@ -203,7 +212,7 @@ static CMakeConfig readCacheFile(const QString &cacheFile, QString &errorMessage
                 const QJsonObject prop = v.toObject();
                 auto nv = nameValue(prop);
                 if (nv.first == "ADVANCED") {
-                    const auto boolValue = CMakeConfigItem::toBool(nv.second.toUtf8());
+                    const auto boolValue = CMakeConfigItem::toBool(nv.second);
                     item.isAdvanced = boolValue.has_value() && boolValue.value();
                 } else if (nv.first == "HELPSTRING") {
                     item.documentation = nv.second.toUtf8();
@@ -219,7 +228,7 @@ static CMakeConfig readCacheFile(const QString &cacheFile, QString &errorMessage
 
 // CMake Files:
 
-std::vector<CMakeFileInfo> readCMakeFilesFile(const QString &cmakeFilesFile, QString &errorMessage)
+static std::vector<CMakeFileInfo> readCMakeFilesFile(const FilePath &cmakeFilesFile, QString &errorMessage)
 {
     std::vector<CMakeFileInfo> result;
 
@@ -236,10 +245,10 @@ std::vector<CMakeFileInfo> readCMakeFilesFile(const QString &cmakeFilesFile, QSt
     for (const QJsonValue &v : inputs) {
         CMakeFileInfo info;
         const QJsonObject input = v.toObject();
-        info.path = input.value("path").toString();
+        info.path = cmakeFilesFile.withNewPath(input.value("path").toString());
 
         info.isCMake = input.value("isCMake").toBool();
-        const QString filename = FilePath::fromString(info.path).fileName();
+        const QString filename = info.path.fileName();
         info.isCMakeListsDotTxt = (filename.compare("CMakeLists.txt",
                                                     HostOsInfo::fileNameCaseSensitivity())
                                    == 0);
@@ -328,13 +337,6 @@ static std::vector<Project> extractProjects(const QJsonArray &projects, QString
 
 static std::vector<Target> extractTargets(const QJsonArray &targets, QString &errorMessage)
 {
-    if (targets.isEmpty()) {
-        errorMessage
-            = QCoreApplication::translate("CMakeProjectManager::Internal",
-                                          "Invalid codemodel file generated by CMake: No targets.");
-        return {};
-    }
-
     std::vector<Target> result;
     for (const QJsonValue &v : targets) {
         const QJsonObject obj = v.toObject();
@@ -489,7 +491,7 @@ static std::vector<Configuration> extractConfigurations(const QJsonArray &config
     return result;
 }
 
-static std::vector<Configuration> readCodemodelFile(const QString &codemodelFile,
+static std::vector<Configuration> readCodemodelFile(const FilePath &codemodelFile,
                                                     QString &errorMessage)
 {
     const QJsonDocument doc = readJsonFile(codemodelFile);
@@ -506,7 +508,7 @@ static std::vector<Configuration> readCodemodelFile(const QString &codemodelFile
 
 // TargetDetails:
 
-std::vector<FileApiDetails::FragmentInfo> extractFragments(const QJsonObject &obj)
+static std::vector<FileApiDetails::FragmentInfo> extractFragments(const QJsonObject &obj)
 {
     const QJsonArray fragments = obj.value("commandFragments").toArray();
     return transform<std::vector>(fragments, [](const QJsonValue &v) {
@@ -516,7 +518,7 @@ std::vector<FileApiDetails::FragmentInfo> extractFragments(const QJsonObject &ob
     });
 }
 
-TargetDetails extractTargetDetails(const QJsonObject &root, QString &errorMessage)
+static TargetDetails extractTargetDetails(const QJsonObject &root, QString &errorMessage)
 {
     TargetDetails t;
     t.name = root.value("name").toString();
@@ -673,7 +675,7 @@ TargetDetails extractTargetDetails(const QJsonObject &root, QString &errorMessag
     return t;
 }
 
-int validateBacktraceGraph(const TargetDetails &t)
+static int validateBacktraceGraph(const TargetDetails &t)
 {
     const int backtraceFilesCount = static_cast<int>(t.backtraceGraph.files.size());
     const int backtraceCommandsCount = static_cast<int>(t.backtraceGraph.commands.size());
@@ -707,7 +709,7 @@ int validateBacktraceGraph(const TargetDetails &t)
     return backtraceNodeCount;
 }
 
-bool validateTargetDetails(const TargetDetails &t)
+static bool validateTargetDetails(const TargetDetails &t)
 {
     // The part filled in by the codemodel file has already been covered!
 
@@ -786,7 +788,7 @@ bool validateTargetDetails(const TargetDetails &t)
     return true;
 }
 
-TargetDetails readTargetFile(const QString &targetFile, QString &errorMessage)
+static TargetDetails readTargetFile(const FilePath &targetFile, QString &errorMessage)
 {
     const QJsonDocument doc = readJsonFile(targetFile);
     const QJsonObject root = doc.object();
@@ -804,13 +806,13 @@ TargetDetails readTargetFile(const QString &targetFile, QString &errorMessage)
 // ReplyFileContents:
 // --------------------------------------------------------------------
 
-QString FileApiDetails::ReplyFileContents::jsonFile(const QString &kind, const QDir &replyDir) const
+FilePath FileApiDetails::ReplyFileContents::jsonFile(const QString &kind, const FilePath &replyDir) const
 {
     const auto ro = findOrDefault(replies, equal(&ReplyObject::kind, kind));
     if (ro.file.isEmpty())
-        return QString();
+        return {};
     else
-        return replyDir.absoluteFilePath(ro.file);
+        return (replyDir / ro.file).absoluteFilePath();
 }
 
 // --------------------------------------------------------------------
@@ -819,75 +821,115 @@ QString FileApiDetails::ReplyFileContents::jsonFile(const QString &kind, const Q
 
 bool FileApiParser::setupCMakeFileApi(const FilePath &buildDirectory, Utils::FileSystemWatcher &watcher)
 {
-    const QDir buildDir = QDir(buildDirectory.toString());
-    buildDir.mkpath(
-        QString::fromLatin1(CMAKE_RELATIVE_REPLY_PATH)); // So that we have a directory to watch!
+    // So that we have a directory to watch.
+    buildDirectory.pathAppended(CMAKE_RELATIVE_REPLY_PATH).ensureWritableDir();
 
-    const QString relativeQueryPath = QString::fromLatin1(CMAKE_RELATIVE_QUERY_PATH);
-    buildDir.mkpath(relativeQueryPath);
-
-    QDir queryDir = buildDir;
-    queryDir.cd(relativeQueryPath);
+    FilePath queryDir = buildDirectory.pathAppended(CMAKE_RELATIVE_QUERY_PATH);
+    queryDir.ensureWritableDir();
 
     if (!queryDir.exists()) {
         reportFileApiSetupFailure();
         return false;
     }
-    QTC_ASSERT(queryDir.exists(), );
+    QTC_ASSERT(queryDir.exists(), return false);
 
     bool failedBefore = false;
-    for (const QString &filePath : cmakeQueryFilePaths(buildDirectory)) {
-        QFile f(filePath);
-        if (!f.exists()) {
-            f.open(QFile::WriteOnly);
-            f.close();
-        }
-
-        if (!f.exists() && !failedBefore) {
+    for (const FilePath &filePath : cmakeQueryFilePaths(buildDirectory)) {
+        const bool success = filePath.ensureExistingFile();
+        if (!success && !failedBefore) {
             failedBefore = true;
             reportFileApiSetupFailure();
         }
     }
 
-    watcher.addDirectory(cmakeReplyDirectory(buildDirectory).toString(), FileSystemWatcher::WatchAllChanges);
+    watcher.addDirectory(cmakeReplyDirectory(buildDirectory).path(), FileSystemWatcher::WatchAllChanges);
     return true;
 }
 
-static QStringList uniqueTargetFiles(const std::vector<Configuration> &configs)
+static QStringList uniqueTargetFiles(const Configuration &config)
 {
     QSet<QString> knownIds;
     QStringList files;
-    for (const Configuration &config : configs) {
-        for (const Target &t : config.targets) {
-            const int knownCount = knownIds.count();
-            knownIds.insert(t.id);
-            if (knownIds.count() > knownCount) {
-                files.append(t.jsonFile);
-            }
+    for (const Target &t : config.targets) {
+        const int knownCount = knownIds.count();
+        knownIds.insert(t.id);
+        if (knownIds.count() > knownCount) {
+            files.append(t.jsonFile);
         }
     }
     return files;
 }
 
-FileApiData FileApiParser::parseData(const QFileInfo &replyFileInfo, QString &errorMessage)
+FileApiData FileApiParser::parseData(QFutureInterface<std::shared_ptr<FileApiQtcData>> &fi,
+                                     const FilePath &replyFilePath,
+                                     const QString &cmakeBuildType,
+                                     QString &errorMessage)
 {
     QTC_CHECK(errorMessage.isEmpty());
-    const QDir replyDir = replyFileInfo.dir();
+    QTC_CHECK(!replyFilePath.needsDevice());
+    const FilePath replyDir = replyFilePath.parentDir();
 
     FileApiData result;
 
-    result.replyFile = readReplyFile(replyFileInfo, errorMessage);
+    const auto cancelCheck = [&fi, &errorMessage]() -> bool {
+        if (fi.isCanceled()) {
+            errorMessage = FileApiParser::tr("CMake parsing was cancelled.");
+            return true;
+        }
+        return false;
+    };
+
+    result.replyFile = readReplyFile(replyFilePath, errorMessage);
+    if (cancelCheck())
+        return {};
     result.cache = readCacheFile(result.replyFile.jsonFile("cache", replyDir), errorMessage);
+    if (cancelCheck())
+        return {};
     result.cmakeFiles = readCMakeFilesFile(result.replyFile.jsonFile("cmakeFiles", replyDir),
                                            errorMessage);
-    result.codemodel = readCodemodelFile(result.replyFile.jsonFile("codemodel", replyDir),
+    if (cancelCheck())
+        return {};
+    auto codeModels = readCodemodelFile(result.replyFile.jsonFile("codemodel", replyDir),
                                          errorMessage);
 
+    if (codeModels.size() == 0) {
+        errorMessage = "No CMake configuration found!";
+        return result;
+    }
+
+    auto it = std::find_if(codeModels.cbegin(), codeModels.cend(),
+                           [cmakeBuildType](const Configuration& cfg) {
+                               return QString::compare(cfg.name, cmakeBuildType, Qt::CaseInsensitive) == 0;
+                           });
+    if (it == codeModels.cend()) {
+        QStringList buildTypes;
+        for (const Configuration &cfg: codeModels)
+            buildTypes << cfg.name;
+
+        if (result.replyFile.isMultiConfig) {
+            errorMessage = tr("No \"%1\" CMake configuration found. Available configurations: \"%2\".\n"
+                              "Make sure that CMAKE_CONFIGURATION_TYPES variable contains the \"Build type\" field.")
+                           .arg(cmakeBuildType)
+                           .arg(buildTypes.join(", "));
+        } else {
+            errorMessage = tr("No \"%1\" CMake configuration found. Available configuration: \"%2\".\n"
+                              "Make sure that CMAKE_BUILD_TYPE variable matches the \"Build type\" field.")
+                           .arg(cmakeBuildType)
+                           .arg(buildTypes.join(", "));
+        }
+        return result;
+    }
+    result.codemodel = std::move(*it);
+    if (cancelCheck())
+        return {};
+
     const QStringList targetFiles = uniqueTargetFiles(result.codemodel);
 
     for (const QString &targetFile : targetFiles) {
+        if (cancelCheck())
+            return {};
         QString targetErrorMessage;
-        TargetDetails td = readTargetFile(replyDir.absoluteFilePath(targetFile), targetErrorMessage);
+        TargetDetails td = readTargetFile((replyDir / targetFile).absoluteFilePath(), targetErrorMessage);
         if (targetErrorMessage.isEmpty()) {
             result.targetDetails.emplace_back(std::move(td));
         } else {
@@ -900,23 +942,22 @@ FileApiData FileApiParser::parseData(const QFileInfo &replyFileInfo, QString &er
     return result;
 }
 
-QFileInfo FileApiParser::scanForCMakeReplyFile(const FilePath &buildDirectory)
+FilePath FileApiParser::scanForCMakeReplyFile(const FilePath &buildDirectory)
 {
-    QDir replyDir(cmakeReplyDirectory(buildDirectory).toString());
+    const FilePath replyDir = cmakeReplyDirectory(buildDirectory);
     if (!replyDir.exists())
         return {};
 
-    const QFileInfoList fis = replyDir.entryInfoList(QStringList("index-*.json"),
-                                                     QDir::Files,
-                                                     QDir::Name);
-    return fis.isEmpty() ? QFileInfo() : fis.last();
+    const FilePaths entries = replyDir.dirEntries({"index-*.json"}, QDir::Files, QDir::Name);
+    return entries.isEmpty() ? FilePath() : entries.first();
 }
 
-QStringList FileApiParser::cmakeQueryFilePaths(const Utils::FilePath &buildDirectory)
+FilePaths FileApiParser::cmakeQueryFilePaths(const FilePath &buildDirectory)
 {
-    QDir queryDir(QDir::cleanPath(buildDirectory.pathAppended(CMAKE_RELATIVE_QUERY_PATH).toString()));
-    return transform(CMAKE_QUERY_FILENAMES,
-                     [&queryDir](const QString &name) { return queryDir.absoluteFilePath(name); });
+    FilePath queryDir = buildDirectory / CMAKE_RELATIVE_QUERY_PATH;
+    return transform(CMAKE_QUERY_FILENAMES, [&queryDir](const QString &name) {
+        return queryDir.resolvePath(FilePath::fromString(name));
+    });
 }
 
 } // namespace Internal
