@@ -27,14 +27,14 @@
 
 #include "fileapidataextractor.h"
 #include "fileapiparser.h"
-#include "projecttreehelper.h"
+
+#include <coreplugin/messagemanager.h>
 
 #include <projectexplorer/projectexplorer.h>
 
 #include <utils/algorithm.h>
 #include <utils/runextensions.h>
 
-#include <QDateTime>
 #include <QLoggingCategory>
 
 using namespace ProjectExplorer;
@@ -62,8 +62,6 @@ FileApiReader::FileApiReader()
 
 FileApiReader::~FileApiReader()
 {
-    if (isParsing())
-        emit errorOccurred(tr("Parsing has been canceled."));
     stop();
     resetData();
 }
@@ -75,13 +73,12 @@ void FileApiReader::setParameters(const BuildDirParameters &p)
 
     // Update:
     m_parameters = p;
-    qCDebug(cmakeFileApiMode) << "Work directory:" << m_parameters.workDirectory.toUserOutput();
+    qCDebug(cmakeFileApiMode) << "Work directory:" << m_parameters.buildDirectory.toUserOutput();
 
     // Reset watcher:
-    m_watcher.removeFiles(m_watcher.files());
-    m_watcher.removeDirectories(m_watcher.directories());
+    m_watcher.clear();
 
-    FileApiParser::setupCMakeFileApi(m_parameters.workDirectory, m_watcher);
+    FileApiParser::setupCMakeFileApi(m_parameters.buildDirectory, m_watcher);
 
     resetData();
 }
@@ -89,14 +86,17 @@ void FileApiReader::setParameters(const BuildDirParameters &p)
 void FileApiReader::resetData()
 {
     m_cmakeFiles.clear();
-    if (!m_parameters.sourceDirectory.isEmpty())
-        m_cmakeFiles.insert(m_parameters.sourceDirectory.pathAppended("CMakeLists.txt"));
+    if (!m_parameters.sourceDirectory.isEmpty()) {
+        CMakeFileInfo cmakeListsTxt;
+        cmakeListsTxt.path = m_parameters.sourceDirectory.pathAppended("CMakeLists.txt");
+        cmakeListsTxt.isCMakeListsDotTxt = true;
+        m_cmakeFiles.insert(cmakeListsTxt);
+    }
 
     m_cache.clear();
     m_buildTargets.clear();
     m_projectParts.clear();
     m_rootProjectNode.reset();
-    m_knownHeaders.clear();
 }
 
 void FileApiReader::parse(bool forceCMakeRun,
@@ -114,7 +114,7 @@ void FileApiReader::parse(bool forceCMakeRun,
                                                         : QStringList());
     qCDebug(cmakeFileApiMode) << "Parameters request these CMake arguments:" << args;
 
-    const QFileInfo replyFi = FileApiParser::scanForCMakeReplyFile(m_parameters.workDirectory);
+    const FilePath replyFile = FileApiParser::scanForCMakeReplyFile(m_parameters.buildDirectory);
     // Only need to update when one of the following conditions is met:
     //  * The user forces the cmake run,
     //  * The user provided arguments,
@@ -123,17 +123,15 @@ void FileApiReader::parse(bool forceCMakeRun,
     //    for creator to run CMake as needed,
     //  * A query file is newer than the reply file
     const bool hasArguments = !args.isEmpty();
-    const bool replyFileMissing = !replyFi.exists();
+    const bool replyFileMissing = !replyFile.exists();
     const bool cmakeFilesChanged = m_parameters.cmakeTool() && m_parameters.cmakeTool()->isAutoRun()
-                                   && anyOf(m_cmakeFiles, [&replyFi](const FilePath &f) {
-                                          return f.toFileInfo().lastModified()
-                                                 > replyFi.lastModified();
+                                   && anyOf(m_cmakeFiles, [&replyFile](const CMakeFileInfo &info) {
+                                          return !info.isGenerated
+                                                 && info.path.lastModified() > replyFile.lastModified();
                                       });
-    const bool queryFileChanged = anyOf(FileApiParser::cmakeQueryFilePaths(
-                                            m_parameters.workDirectory),
-                                        [&replyFi](const QString &qf) {
-                                            return QFileInfo(qf).lastModified()
-                                                   > replyFi.lastModified();
+    const bool queryFileChanged = anyOf(FileApiParser::cmakeQueryFilePaths(m_parameters.buildDirectory),
+                                        [&replyFile](const FilePath &qf) {
+                                            return qf.lastModified() > replyFile.lastModified();
                                         });
 
     const bool mustUpdate = forceCMakeRun || hasArguments || replyFileMissing || cmakeFilesChanged
@@ -154,7 +152,7 @@ void FileApiReader::parse(bool forceCMakeRun,
                                          .arg(args.join("\", \""));
         startCMakeState(args);
     } else {
-        endState(replyFi);
+        endState(replyFile, false);
     }
 }
 
@@ -163,6 +161,13 @@ void FileApiReader::stop()
     if (m_cmakeProcess)
         disconnect(m_cmakeProcess.get(), nullptr, this, nullptr);
     m_cmakeProcess.reset();
+
+    if (m_future) {
+        m_future->cancel();
+        m_future->waitForFinished();
+    }
+    m_future = {};
+    m_isParsing = false;
 }
 
 bool FileApiReader::isParsing() const
@@ -172,42 +177,47 @@ bool FileApiReader::isParsing() const
 
 QSet<FilePath> FileApiReader::projectFilesToWatch() const
 {
-    return m_cmakeFiles;
+    return Utils::transform(
+                Utils::filtered(m_cmakeFiles,
+                                [](const CMakeFileInfo &info) { return !info.isGenerated; }),
+                [](const CMakeFileInfo &info) { return info.path;});
 }
 
 QList<CMakeBuildTarget> FileApiReader::takeBuildTargets(QString &errorMessage){
     Q_UNUSED(errorMessage)
 
-    auto result = std::move(m_buildTargets);
-    m_buildTargets.clear();
-    return result;
+    return std::exchange(m_buildTargets, {});
 }
 
 CMakeConfig FileApiReader::takeParsedConfiguration(QString &errorMessage)
 {
-    Q_UNUSED(errorMessage)
+    if (m_lastCMakeExitCode != 0)
+        errorMessage = tr("CMake returned error code: %1").arg(m_lastCMakeExitCode);
 
-    CMakeConfig cache = m_cache;
-    m_cache.clear();
-    return cache;
+    return std::exchange(m_cache, {});
 }
 
-std::unique_ptr<CMakeProjectNode> FileApiReader::generateProjectTree(
-    const QList<const FileNode *> &allFiles, QString &errorMessage)
+QString FileApiReader::ctestPath() const
 {
-    Q_UNUSED(errorMessage)
+    // if we failed to run cmake we should not offer ctest information either
+    return m_lastCMakeExitCode == 0 ? m_ctestPath : QString();
+}
 
-    addHeaderNodes(m_rootProjectNode.get(), m_knownHeaders, allFiles);
-    return std::move(m_rootProjectNode);
+bool FileApiReader::isMultiConfig() const
+{
+    return m_isMultiConfig;
+}
+
+bool FileApiReader::usesAllCapsTargets() const
+{
+    return m_usesAllCapsTargets;
 }
 
 RawProjectParts FileApiReader::createRawProjectParts(QString &errorMessage)
 {
     Q_UNUSED(errorMessage)
 
-    RawProjectParts result = std::move(m_projectParts);
-    m_projectParts.clear();
-    return result;
+    return std::exchange(m_projectParts, {});
 }
 
 void FileApiReader::startState()
@@ -222,51 +232,117 @@ void FileApiReader::startState()
     emit configurationStarted();
 }
 
-void FileApiReader::endState(const QFileInfo &replyFi)
+void FileApiReader::endState(const FilePath &replyFilePath, bool restoredFromBackup)
 {
     qCDebug(cmakeFileApiMode) << "FileApiReader: END STATE.";
     QTC_ASSERT(m_isParsing, return );
     QTC_ASSERT(!m_future.has_value(), return );
 
     const FilePath sourceDirectory = m_parameters.sourceDirectory;
-    const FilePath buildDirectory = m_parameters.workDirectory;
+    const FilePath buildDirectory = m_parameters.buildDirectory;
+    const QString cmakeBuildType = m_parameters.cmakeBuildType == "Build" ? "" : m_parameters.cmakeBuildType;
 
-    m_lastReplyTimestamp = replyFi.lastModified();
+    QTC_CHECK(!replyFilePath.needsDevice());
+    m_lastReplyTimestamp = replyFilePath.lastModified();
 
     m_future = runAsync(ProjectExplorerPlugin::sharedThreadPool(),
-                        [replyFi, sourceDirectory, buildDirectory]() {
-                            auto result = std::make_unique<FileApiQtcData>();
-                            FileApiData data = FileApiParser::parseData(replyFi,
+                        [replyFilePath, sourceDirectory, buildDirectory, cmakeBuildType](
+                            QFutureInterface<std::shared_ptr<FileApiQtcData>> &fi) {
+                            auto result = std::make_shared<FileApiQtcData>();
+                            FileApiData data = FileApiParser::parseData(fi,
+                                                                        replyFilePath,
+                                                                        cmakeBuildType,
                                                                         result->errorMessage);
-                            if (!result->errorMessage.isEmpty()) {
+                            if (result->errorMessage.isEmpty())
+                                *result = extractData(data, sourceDirectory, buildDirectory);
+                            else
                                 qWarning() << result->errorMessage;
-                                return result.release();
-                            }
-                            *result = extractData(data, sourceDirectory, buildDirectory);
-                            if (!result->errorMessage.isEmpty()) {
-                                qWarning() << result->errorMessage;
-                            }
 
-                            return result.release();
+                            fi.reportResult(result);
                         });
-    onFinished(m_future.value(), this, [this](const QFuture<FileApiQtcData *> &f) {
-        std::unique_ptr<FileApiQtcData> value(f.result()); // Adopt the pointer again:-)
+    onResultReady(m_future.value(),
+                  this,
+                  [this, sourceDirectory, buildDirectory, restoredFromBackup](
+                      const std::shared_ptr<FileApiQtcData> &value) {
+                      m_isParsing = false;
+                      m_cache = std::move(value->cache);
+                      m_cmakeFiles = std::move(value->cmakeFiles);
+                      m_buildTargets = std::move(value->buildTargets);
+                      m_projectParts = std::move(value->projectParts);
+                      m_rootProjectNode = std::move(value->rootProjectNode);
+                      m_ctestPath = std::move(value->ctestPath);
+                      m_isMultiConfig = std::move(value->isMultiConfig);
+                      m_usesAllCapsTargets = std::move(value->usesAllCapsTargets);
 
-        m_future = {};
-        m_isParsing = false;
-        m_cache = std::move(value->cache);
-        m_cmakeFiles = std::move(value->cmakeFiles);
-        m_buildTargets = std::move(value->buildTargets);
-        m_projectParts = std::move(value->projectParts);
-        m_rootProjectNode = std::move(value->rootProjectNode);
-        m_knownHeaders = std::move(value->knownHeaders);
+                      if (value->errorMessage.isEmpty()) {
+                          emit this->dataAvailable(restoredFromBackup);
+                      } else {
+                          emit this->errorOccurred(value->errorMessage);
+                      }
+                      m_future = {};
+                  });
+}
 
-        if (value->errorMessage.isEmpty()) {
-            emit this->dataAvailable();
-        } else {
-            emit this->errorOccurred(value->errorMessage);
-        }
-    });
+void FileApiReader::makeBackupConfiguration(bool store)
+{
+    FilePath reply = m_parameters.buildDirectory.pathAppended(".cmake/api/v1/reply");
+    FilePath replyPrev = m_parameters.buildDirectory.pathAppended(".cmake/api/v1/reply.prev");
+    if (!store)
+        std::swap(reply, replyPrev);
+
+    if (reply.exists()) {
+        if (replyPrev.exists())
+            replyPrev.removeRecursively();
+        QTC_CHECK(!replyPrev.exists());
+        if (!reply.renameFile(replyPrev))
+            Core::MessageManager::writeFlashing(tr("Failed to rename %1 to %2.")
+                                                .arg(reply.toString(), replyPrev.toString()));
+    }
+
+    FilePath cmakeCacheTxt = m_parameters.buildDirectory.pathAppended("CMakeCache.txt");
+    FilePath cmakeCacheTxtPrev = m_parameters.buildDirectory.pathAppended("CMakeCache.txt.prev");
+    if (!store)
+        std::swap(cmakeCacheTxt, cmakeCacheTxtPrev);
+
+    if (cmakeCacheTxt.exists())
+        if (!FileUtils::copyIfDifferent(cmakeCacheTxt, cmakeCacheTxtPrev))
+            Core::MessageManager::writeFlashing(tr("Failed to copy %1 to %2.")
+                                                .arg(cmakeCacheTxt.toString(), cmakeCacheTxtPrev.toString()));
+
+}
+
+void FileApiReader::writeConfigurationIntoBuildDirectory(const QStringList &configurationArguments)
+{
+    const FilePath buildDir = m_parameters.buildDirectory;
+    QTC_CHECK(buildDir.ensureWritableDir());
+
+    QByteArray contents;
+    contents.append("# This file is managed by Qt Creator, do not edit!\n\n");
+    contents.append(
+        transform(CMakeConfig::fromArguments(configurationArguments).toList(),
+            [](const CMakeConfigItem &item) {
+                return item.toCMakeSetLine(nullptr);
+            })
+            .join('\n')
+            .toUtf8());
+
+    const FilePath settingsFile = buildDir / "qtcsettings.cmake";
+    QTC_CHECK(settingsFile.writeFileContents(contents));
+}
+
+std::unique_ptr<CMakeProjectNode> FileApiReader::rootProjectNode()
+{
+    return std::exchange(m_rootProjectNode, {});
+}
+
+FilePath FileApiReader::topCmakeFile() const
+{
+    return m_cmakeFiles.size() == 1 ? (*m_cmakeFiles.begin()).path : FilePath{};
+}
+
+int FileApiReader::lastCMakeExitCode() const
+{
+    return m_lastCMakeExitCode;
 }
 
 void FileApiReader::startCMakeState(const QStringList &configurationArguments)
@@ -279,19 +355,29 @@ void FileApiReader::startCMakeState(const QStringList &configurationArguments)
     connect(m_cmakeProcess.get(), &CMakeProcess::finished, this, &FileApiReader::cmakeFinishedState);
 
     qCDebug(cmakeFileApiMode) << ">>>>>> Running cmake with arguments:" << configurationArguments;
+    // Reset watcher:
+    m_watcher.removeFiles(m_watcher.files());
+    m_watcher.removeDirectories(m_watcher.directories());
+
+    makeBackupConfiguration(true);
+    writeConfigurationIntoBuildDirectory(configurationArguments);
     m_cmakeProcess->run(m_parameters, configurationArguments);
 }
 
-void FileApiReader::cmakeFinishedState(int code, QProcess::ExitStatus status)
+void FileApiReader::cmakeFinishedState()
 {
     qCDebug(cmakeFileApiMode) << "FileApiReader: CMAKE FINISHED STATE.";
 
-    Q_UNUSED(code)
-    Q_UNUSED(status)
-
+    m_lastCMakeExitCode = m_cmakeProcess->lastExitCode();
     m_cmakeProcess.release()->deleteLater();
 
-    endState(FileApiParser::scanForCMakeReplyFile(m_parameters.workDirectory));
+    if (m_lastCMakeExitCode != 0)
+        makeBackupConfiguration(false);
+
+    FileApiParser::setupCMakeFileApi(m_parameters.buildDirectory, m_watcher);
+
+    endState(FileApiParser::scanForCMakeReplyFile(m_parameters.buildDirectory),
+             m_lastCMakeExitCode != 0);
 }
 
 void FileApiReader::replyDirectoryHasChanged(const QString &directory) const
@@ -299,13 +385,14 @@ void FileApiReader::replyDirectoryHasChanged(const QString &directory) const
     if (m_isParsing)
         return; // This has been triggered by ourselves, ignore.
 
-    const QFileInfo fi = FileApiParser::scanForCMakeReplyFile(m_parameters.workDirectory);
-    const QString dir = fi.absolutePath();
+    const FilePath reply = FileApiParser::scanForCMakeReplyFile(m_parameters.buildDirectory);
+    const FilePath dir = reply.absolutePath();
     if (dir.isEmpty())
         return; // CMake started to fill the result dir, but has not written a result file yet
-    QTC_ASSERT(dir == directory, return);
+    QTC_CHECK(!dir.needsDevice());
+    QTC_ASSERT(dir.path() == directory, return);
 
-    if (m_lastReplyTimestamp.isValid() && fi.lastModified() > m_lastReplyTimestamp)
+    if (m_lastReplyTimestamp.isValid() && reply.lastModified() > m_lastReplyTimestamp)
         emit dirty();
 }
 
