@@ -29,8 +29,8 @@
 #include "compilationdbparser.h"
 
 #include <coreplugin/icontext.h>
-#include <cpptools/cppprojectupdater.h>
-#include <cpptools/projectinfo.h>
+#include <cppeditor/cppprojectupdater.h>
+#include <cppeditor/projectinfo.h>
 #include <projectexplorer/buildinfo.h>
 #include <projectexplorer/buildsteplist.h>
 #include <projectexplorer/buildtargetinfo.h>
@@ -54,7 +54,7 @@
 #include <QFileDialog>
 
 #ifdef Q_OS_WIN
-#include <Windows.h>
+#include <windows.h>
 #endif
 
 using namespace ProjectExplorer;
@@ -176,7 +176,7 @@ RawProjectPart makeRawProjectPart(const Utils::FilePath &projectFile,
 {
     HeaderPaths headerPaths;
     Macros macros;
-    CppTools::ProjectFile::Kind fileKind = CppTools::ProjectFile::Unclassified;
+    CppEditor::ProjectFile::Kind fileKind = CppEditor::ProjectFile::Unclassified;
 
     const QStringList originalFlags = flags;
     filteredFlags(fileName.fileName(),
@@ -195,15 +195,15 @@ RawProjectPart makeRawProjectPart(const Utils::FilePath &projectFile,
     rpp.setHeaderPaths(headerPaths);
     rpp.setMacros(macros);
 
-    if (fileKind == CppTools::ProjectFile::Kind::CHeader
-            || fileKind == CppTools::ProjectFile::Kind::CSource) {
+    if (fileKind == CppEditor::ProjectFile::Kind::CHeader
+            || fileKind == CppEditor::ProjectFile::Kind::CSource) {
         if (!kitInfo.cToolChain) {
             kitInfo.cToolChain = toolchainFromFlags(kit,
                                                     originalFlags,
                                                     ProjectExplorer::Constants::C_LANGUAGE_ID);
         }
         addDriverModeFlagIfNeeded(kitInfo.cToolChain, flags, originalFlags);
-        rpp.setFlagsForC({kitInfo.cToolChain, flags});
+        rpp.setFlagsForC({kitInfo.cToolChain, flags, workingDir});
     } else {
         if (!kitInfo.cxxToolChain) {
             kitInfo.cxxToolChain = toolchainFromFlags(kit,
@@ -211,7 +211,7 @@ RawProjectPart makeRawProjectPart(const Utils::FilePath &projectFile,
                                                       ProjectExplorer::Constants::CXX_LANGUAGE_ID);
         }
         addDriverModeFlagIfNeeded(kitInfo.cxxToolChain, flags, originalFlags);
-        rpp.setFlagsForCxx({kitInfo.cxxToolChain, flags});
+        rpp.setFlagsForCxx({kitInfo.cxxToolChain, flags, workingDir});
     }
 
     return rpp;
@@ -266,8 +266,8 @@ FolderNode *createFoldersIfNeeded(FolderNode *root, const Utils::FilePath &folde
 
 FileType fileTypeForName(const QString &fileName)
 {
-    CppTools::ProjectFile::Kind fileKind = CppTools::ProjectFile::classify(fileName);
-    if (CppTools::ProjectFile::isHeader(fileKind))
+    CppEditor::ProjectFile::Kind fileKind = CppEditor::ProjectFile::classify(fileName);
+    if (CppEditor::ProjectFile::isHeader(fileKind))
         return FileType::Header;
     return FileType::Source;
 }
@@ -335,7 +335,7 @@ void createTree(std::unique_ptr<ProjectNode> &root,
 
 CompilationDatabaseBuildSystem::CompilationDatabaseBuildSystem(Target *target)
     : BuildSystem(target)
-    , m_cppCodeModelUpdater(std::make_unique<CppTools::CppProjectUpdater>())
+    , m_cppCodeModelUpdater(std::make_unique<CppEditor::CppProjectUpdater>())
     , m_deployFileWatcher(new FileSystemWatcher(this))
 {
     connect(target->project(), &CompilationDatabaseProject::rootProjectDirectoryChanged,
@@ -437,12 +437,14 @@ CompilationDatabaseProject::CompilationDatabaseProject(const Utils::FilePath &pr
 
 Utils::FilePath CompilationDatabaseProject::rootPathFromSettings() const
 {
-#ifdef WITH_TESTS
-    return Utils::FilePath::fromString(projectDirectory().fileName());
-#else
-    return Utils::FilePath::fromString(
+    Utils::FilePath rootPath;
+#ifndef WITH_TESTS
+    rootPath = Utils::FilePath::fromString(
         namedSettings(ProjectExplorer::Constants::PROJECT_ROOT_PATH_KEY).toString());
 #endif
+    if (rootPath.isEmpty())
+        rootPath = projectDirectory();
+    return rootPath;
 }
 
 void CompilationDatabaseProject::configureAsExampleProject(Kit *kit)
@@ -485,7 +487,7 @@ void CompilationDatabaseBuildSystem::updateDeploymentData()
                                               projectDirectory().toString());
     setDeploymentData(deploymentData);
     if (m_deployFileWatcher->files() != QStringList(deploymentFilePath.toString())) {
-        m_deployFileWatcher->removeFiles(m_deployFileWatcher->files());
+        m_deployFileWatcher->clear();
         m_deployFileWatcher->addFile(deploymentFilePath.toString(),
                                      FileSystemWatcher::WatchModifiedDate);
     }
