@@ -398,7 +398,7 @@ TouchBar *ActionContainerPrivate::touchBar() const
     return nullptr;
 }
 
-bool ActionContainerPrivate::canAddAction(Command *action) const
+bool ActionContainerPrivate::canAddAction(Command *action)
 {
     return action && action->action();
 }
@@ -408,7 +408,7 @@ void ActionContainerPrivate::scheduleUpdate()
     if (m_updateRequested)
         return;
     m_updateRequested = true;
-    QTimer::singleShot(0, this, &ActionContainerPrivate::update);
+    QMetaObject::invokeMethod(this, &ActionContainerPrivate::update, Qt::QueuedConnection);
 }
 
 void ActionContainerPrivate::update()
@@ -481,7 +481,7 @@ bool MenuActionContainer::updateInternal()
     bool hasitems = false;
     QList<QAction *> actions = m_menu->actions();
 
-    for (const Group &group : m_groups) {
+    for (const Group &group : qAsConst(m_groups)) {
         foreach (QObject *item, group.items) {
             if (auto container = qobject_cast<ActionContainerPrivate*>(item)) {
                 actions.removeAll(container->menu()->menuAction());
