@@ -29,18 +29,16 @@
 #include <coreplugin/idocument.h>
 #include <coreplugin/modemanager.h>
 #include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/coreicons.h>
 
-#include <coreplugin/editormanager/ieditor.h>
+#include <extensionsystem/pluginmanager.h>
 
-#include <QPointer>
-#include <QStringList>
 #include <QDebug>
-
+#include <QPointer>
 #include <QStackedWidget>
-
-static Core::DesignMode *m_instance = 0;
+#include <QStringList>
 
 namespace Core {
 
@@ -60,17 +58,14 @@ public:
 
 public:
     QPointer<IEditor> m_currentEditor;
-    bool m_isActive;
-    bool m_isRequired;
+    bool m_isActive = false;
     QList<DesignEditorInfo*> m_editors;
     QStackedWidget *m_stackWidget;
     Context m_activeContext;
 };
 
 DesignModePrivate::DesignModePrivate()
-    : m_isActive(false),
-      m_isRequired(false),
-      m_stackWidget(new QStackedWidget)
+    : m_stackWidget(new QStackedWidget)
 {}
 
 DesignModePrivate::~DesignModePrivate()
@@ -78,13 +73,13 @@ DesignModePrivate::~DesignModePrivate()
     delete m_stackWidget;
 }
 
-DesignMode::DesignMode()
-    : d(new DesignModePrivate)
-{
-    m_instance = this;
+static DesignMode *m_instance = nullptr;
+static DesignModePrivate *d = nullptr;
 
+DesignMode::DesignMode()
+{
     ICore::addPreCloseListener([]() -> bool {
-        m_instance->currentEditorChanged(0);
+        m_instance->currentEditorChanged(nullptr);
         return true;
     });
 
@@ -108,7 +103,6 @@ DesignMode::DesignMode()
 DesignMode::~DesignMode()
 {
     qDeleteAll(d->m_editors);
-    delete d;
 }
 
 DesignMode *DesignMode::instance()
@@ -118,20 +112,9 @@ DesignMode *DesignMode::instance()
 
 void DesignMode::setDesignModeIsRequired()
 {
-    d->m_isRequired = true;
-}
-
-bool DesignMode::designModeIsRequired() const
-{
-    return d->m_isRequired;
-}
-
-QStringList DesignMode::registeredMimeTypes() const
-{
-    QStringList rc;
-    foreach (const DesignEditorInfo *i, d->m_editors)
-        rc += i->mimeTypes;
-    return rc;
+    // d != nullptr indicates "isRequired".
+    if (!d)
+        d = new DesignModePrivate;
 }
 
 /**
@@ -146,7 +129,7 @@ void DesignMode::registerDesignWidget(QWidget *widget,
     setDesignModeIsRequired();
     int index = d->m_stackWidget->addWidget(widget);
 
-    DesignEditorInfo *info = new DesignEditorInfo;
+    auto info = new DesignEditorInfo;
     info->mimeTypes = mimeTypes;
     info->context = context;
     info->widgetIndex = index;
@@ -197,10 +180,10 @@ void DesignMode::currentEditorChanged(IEditor *editor)
 
     if (!mimeEditorAvailable) {
         setActiveContext(Context());
-        if (ModeManager::currentMode() == id())
+        if (ModeManager::currentModeId() == id())
             ModeManager::activateMode(Constants::MODE_EDIT);
         setEnabled(false);
-        d->m_currentEditor = 0;
+        d->m_currentEditor = nullptr;
         emit actionsUpdated(d->m_currentEditor.data());
     } else {
         d->m_currentEditor = editor;
@@ -217,7 +200,7 @@ void DesignMode::updateActions()
     emit actionsUpdated(d->m_currentEditor.data());
 }
 
-void DesignMode::updateContext(Id newMode, Id oldMode)
+void DesignMode::updateContext(Utils::Id newMode, Utils::Id oldMode)
 {
     if (newMode == id())
         ICore::addAdditionalContext(d->m_activeContext);
@@ -230,10 +213,27 @@ void DesignMode::setActiveContext(const Context &context)
     if (d->m_activeContext == context)
         return;
 
-    if (ModeManager::currentMode() == id())
+    if (ModeManager::currentModeId() == id())
         ICore::updateAdditionalContexts(d->m_activeContext, context);
 
     d->m_activeContext = context;
 }
 
+void DesignMode::createModeIfRequired()
+{
+    if (d) {
+        m_instance = new DesignMode;
+        ExtensionSystem::PluginManager::addObject(m_instance);
+    }
+}
+
+void DesignMode::destroyModeIfRequired()
+{
+    if (m_instance) {
+        ExtensionSystem::PluginManager::removeObject(m_instance);
+        delete m_instance;
+    }
+    delete d;
+}
+
 } // namespace Core
