@@ -88,7 +88,7 @@ public:
     void setTools(const QMap<QString, QList<ExternalTool *> > &tools);
     QMap<QString, QList<ExternalTool *> > tools() const { return m_tools; }
 
-    ExternalTool *toolForIndex(const QModelIndex &modelIndex) const;
+    static ExternalTool *toolForIndex(const QModelIndex &modelIndex);
     QString categoryForIndex(const QModelIndex &modelIndex, bool *found) const;
     void revertTool(const QModelIndex &modelIndex);
     QModelIndex addCategory();
@@ -97,8 +97,8 @@ public:
     Qt::DropActions supportedDropActions() const override { return Qt::MoveAction; }
 
 private:
-    QVariant data(ExternalTool *tool, int role = Qt::DisplayRole) const;
-    QVariant data(const QString &category, int role = Qt::DisplayRole) const;
+    static QVariant data(ExternalTool *tool, int role = Qt::DisplayRole);
+    static QVariant data(const QString &category, int role = Qt::DisplayRole);
 
     QMap<QString, QList<ExternalTool *> > m_tools;
 };
@@ -129,7 +129,7 @@ QVariant ExternalToolModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
-QVariant ExternalToolModel::data(ExternalTool *tool, int role) const
+QVariant ExternalToolModel::data(ExternalTool *tool, int role)
 {
     switch (role) {
     case Qt::DisplayRole:
@@ -141,7 +141,7 @@ QVariant ExternalToolModel::data(ExternalTool *tool, int role) const
     return QVariant();
 }
 
-QVariant ExternalToolModel::data(const QString &category, int role) const
+QVariant ExternalToolModel::data(const QString &category, int role)
 {
     switch (role) {
     case Qt::DisplayRole:
@@ -315,7 +315,7 @@ void ExternalToolModel::setTools(const QMap<QString, QList<ExternalTool *> > &to
     endResetModel();
 }
 
-ExternalTool *ExternalToolModel::toolForIndex(const QModelIndex &index) const
+ExternalTool *ExternalToolModel::toolForIndex(const QModelIndex &index)
 {
     return static_cast<ExternalTool *>(index.internalPointer());
 }
@@ -547,7 +547,7 @@ void ExternalToolConfig::handleCurrentChanged(const QModelIndex &now, const QMod
 
 void ExternalToolConfig::updateButtons(const QModelIndex &index)
 {
-    ExternalTool *tool = m_model.toolForIndex(index);
+    const ExternalTool *tool = ExternalToolModel::toolForIndex(index);
     if (!tool) {
         m_ui.removeButton->setEnabled(false);
         m_ui.revertButton->setEnabled(false);
@@ -571,18 +571,18 @@ void ExternalToolConfig::updateCurrentItem()
 
 void ExternalToolConfig::updateItem(const QModelIndex &index)
 {
-    ExternalTool *tool = m_model.toolForIndex(index);
+    ExternalTool *tool = ExternalToolModel::toolForIndex(index);
     if (!tool)
         return;
     tool->setDescription(m_ui.description->text());
-    QStringList executables = tool->executables();
+    FilePaths executables = tool->executables();
     if (executables.size() > 0)
-        executables[0] = m_ui.executable->rawPath();
+        executables[0] = m_ui.executable->rawFilePath();
     else
-        executables << m_ui.executable->rawPath();
+        executables << m_ui.executable->rawFilePath();
     tool->setExecutables(executables);
     tool->setArguments(m_ui.arguments->text());
-    tool->setWorkingDirectory(m_ui.workingDirectory->rawPath());
+    tool->setWorkingDirectory(m_ui.workingDirectory->rawFilePath());
     tool->setBaseEnvironmentProviderId(Id::fromSetting(m_ui.baseEnvironment->currentData()));
     tool->setEnvironmentUserChanges(m_environment);
     tool->setOutputHandling(ExternalTool::OutputHandling(m_ui.outputBehavior->currentIndex()));
@@ -594,12 +594,12 @@ void ExternalToolConfig::updateItem(const QModelIndex &index)
 void ExternalToolConfig::showInfoForItem(const QModelIndex &index)
 {
     updateButtons(index);
-    ExternalTool *tool = m_model.toolForIndex(index);
+    const ExternalTool *tool = ExternalToolModel::toolForIndex(index);
     if (!tool) {
         m_ui.description->clear();
-        m_ui.executable->setPath(QString());
+        m_ui.executable->setFilePath({});
         m_ui.arguments->clear();
-        m_ui.workingDirectory->setPath(QString());
+        m_ui.workingDirectory->setFilePath({});
         m_ui.inputText->clear();
         m_ui.infoWidget->setEnabled(false);
         m_environment.clear();
@@ -607,10 +607,10 @@ void ExternalToolConfig::showInfoForItem(const QModelIndex &index)
     }
     m_ui.infoWidget->setEnabled(true);
     m_ui.description->setText(tool->description());
-    m_ui.executable->setPath(tool->executables().isEmpty() ? QString()
-                                                          : tool->executables().constFirst());
+    m_ui.executable->setFilePath(tool->executables().isEmpty() ? FilePath()
+                                                               : tool->executables().constFirst());
     m_ui.arguments->setText(tool->arguments());
-    m_ui.workingDirectory->setPath(tool->workingDirectory());
+    m_ui.workingDirectory->setFilePath(tool->workingDirectory());
     m_ui.outputBehavior->setCurrentIndex(int(tool->outputHandling()));
     m_ui.errorOutputBehavior->setCurrentIndex(int(tool->errorHandling()));
     m_ui.modifiesDocumentCheckbox->setChecked(tool->modifiesCurrentDocument());
@@ -630,20 +630,19 @@ void ExternalToolConfig::showInfoForItem(const QModelIndex &index)
     updateEffectiveArguments();
 }
 
-static QString getUserFilePath(const QString &proposalFileName)
+static FilePath getUserFilePath(const QString &proposalFileName)
 {
-    const QDir resourceDir(ICore::userResourcePath());
+    const QDir resourceDir(ICore::userResourcePath().toDir());
     if (!resourceDir.exists(QLatin1String("externaltools")))
         resourceDir.mkpath(QLatin1String("externaltools"));
     const QFileInfo fi(proposalFileName);
     const QString &suffix = QLatin1Char('.') + fi.completeSuffix();
-    const QString &newFilePath = ICore::userResourcePath()
-            + QLatin1String("/externaltools/") + fi.baseName();
+    const FilePath newFilePath = ICore::userResourcePath("externaltools") / fi.baseName();
     int count = 0;
-    QString tryPath = newFilePath + suffix;
-    while (QFile::exists(tryPath)) {
+    FilePath tryPath = newFilePath + suffix;
+    while (tryPath.exists()) {
         if (++count > 15)
-            return QString();
+            return {};
         // add random number
         const int number = QRandomGenerator::global()->generate() % 1000;
         tryPath = newFilePath + QString::number(number) + suffix;
@@ -710,8 +709,8 @@ void ExternalToolConfig::apply()
                     if (tool->preset() && (*tool) != (*(tool->preset()))) {
                         // check if we need to choose a new file name
                         if (tool->preset()->fileName() == tool->fileName()) {
-                            const QString &fileName = FilePath::fromString(tool->preset()->fileName()).fileName();
-                            const QString &newFilePath = getUserFilePath(fileName);
+                            const QString &fileName = tool->preset()->fileName().fileName();
+                            const FilePath &newFilePath = getUserFilePath(fileName);
                             // TODO error handling if newFilePath.isEmpty() (i.e. failed to find a unused name)
                             tool->setFileName(newFilePath);
                         }
@@ -721,9 +720,9 @@ void ExternalToolConfig::apply()
                     } else if (tool->preset() && (*tool) == (*(tool->preset()))) {
                         // check if we need to delete the changed description
                         if (originalTool->fileName() != tool->preset()->fileName()
-                                && QFile::exists(originalTool->fileName())) {
+                                && originalTool->fileName().exists()) {
                             // TODO error handling
-                            QFile::remove(originalTool->fileName());
+                            originalTool->fileName().removeFile();
                         }
                         tool->setFileName(tool->preset()->fileName());
                         // no need to save, it's the same as the preset
@@ -756,7 +755,7 @@ void ExternalToolConfig::apply()
     foreach (ExternalTool *tool, originalTools) {
         QTC_ASSERT(!tool->preset(), continue);
         // TODO error handling
-        QFile::remove(tool->fileName());
+        tool->fileName().removeFile();
     }
 
     ExternalToolManager::setToolsByCategory(resultMap);
