@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2020 The Qt Company Ltd.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
@@ -29,6 +29,10 @@
 #include "../idocument.h"
 
 #include "documentmodel.h"
+#include "ieditor.h"
+
+#include "utils/link.h"
+#include "utils/textfileformat.h"
 
 #include <QList>
 #include <QWidget>
@@ -37,27 +41,21 @@
 
 QT_FORWARD_DECLARE_CLASS(QMenu)
 
-namespace Utils { class MimeType; }
+namespace Utils {
+class MimeType;
+}
 
 namespace Core {
 
-class IEditor;
 class IDocument;
 class SearchResultItem;
 
-enum MakeWritableResult {
-    OpenedWithVersionControl,
-    MadeWritable,
-    SavedAs,
-    Failed
-};
-
 namespace Internal {
 class EditorManagerPrivate;
 class MainWindow;
 } // namespace Internal
 
-class CORE_EXPORT EditorManagerPlaceHolder : public QWidget
+class CORE_EXPORT EditorManagerPlaceHolder final : public QWidget
 {
     Q_OBJECT
 public:
@@ -82,38 +80,45 @@ public:
         DoNotChangeCurrentEditor = 1,
         IgnoreNavigationHistory = 2,
         DoNotMakeVisible = 4,
-        CanContainLineAndColumnNumber = 8,
-        OpenInOtherSplit = 16,
-        DoNotSwitchToDesignMode = 32,
-        DoNotSwitchToEditMode = 64,
-        SwitchSplitIfAlreadyVisible = 128
+        OpenInOtherSplit = 8,
+        DoNotSwitchToDesignMode = 16,
+        DoNotSwitchToEditMode = 32,
+        SwitchSplitIfAlreadyVisible = 64,
+        DoNotRaise = 128
     };
     Q_DECLARE_FLAGS(OpenEditorFlags, OpenEditorFlag)
 
-    struct FilePathInfo {
-        QString filePath; // file path without line/column suffix
-        QString postfix; // line/column suffix as string, e.g. ":10:1"
-        int lineNumber; // extracted line number, -1 if none
-        int columnNumber; // extracted column number, -1 if none
-    };
-    static FilePathInfo splitLineAndColumnNumber(const QString &filePath);
-    static IEditor *openEditor(const QString &fileName, Id editorId = Id(),
-        OpenEditorFlags flags = NoFlags, bool *newEditor = nullptr);
-    static IEditor *openEditorAt(const QString &fileName,  int line, int column = 0,
-                                 Id editorId = Id(), OpenEditorFlags flags = NoFlags,
+    static IEditor *openEditor(const Utils::FilePath &filePath,
+                               Utils::Id editorId = {},
+                               OpenEditorFlags flags = NoFlags,
+                               bool *newEditor = nullptr);
+    static IEditor *openEditorAt(const Utils::Link &link,
+                                 Utils::Id editorId = {},
+                                 OpenEditorFlags flags = NoFlags,
                                  bool *newEditor = nullptr);
-    static void openEditorAtSearchResult(const SearchResultItem &item, OpenEditorFlags flags = NoFlags);
-    static IEditor *openEditorWithContents(Id editorId, QString *titlePattern = nullptr,
+
+    // Kept for a while for transition.
+    static IEditor *openEditor(const QString &fileName, Utils::Id editorId = {},
+        OpenEditorFlags flags = NoFlags, bool *newEditor = nullptr); // FIXME: Remove overload
+    static IEditor *openEditorAt(const QString &fileName,  int line, int column = 0,
+                                 Utils::Id editorId = {}, OpenEditorFlags flags = NoFlags,
+                                 bool *newEditor = nullptr); // FIXME: Remove overload
+
+    static void openEditorAtSearchResult(const SearchResultItem &item,
+                                         Utils::Id editorId = {},
+                                         OpenEditorFlags flags = NoFlags,
+                                         bool *newEditor = nullptr);
+    static IEditor *openEditorWithContents(Utils::Id editorId, QString *titlePattern = nullptr,
                                            const QByteArray &contents = QByteArray(),
                                            const QString &uniqueId = QString(),
                                            OpenEditorFlags flags = NoFlags);
-    static bool skipOpeningBigTextFile(const QString &filePath);
+    static bool skipOpeningBigTextFile(const Utils::FilePath &filePath);
     static void clearUniqueId(IDocument *document);
 
-    static bool openExternalEditor(const QString &fileName, Id editorId);
+    static bool openExternalEditor(const Utils::FilePath &filePath, Utils::Id editorId);
     static void addCloseEditorListener(const std::function<bool(IEditor *)> &listener);
 
-    static QStringList getOpenFileNames();
+    static Utils::FilePaths getOpenFilePaths();
 
     static IDocument *currentDocument();
     static IEditor *currentEditor();
@@ -123,20 +128,18 @@ public:
     static void activateEditorForEntry(DocumentModel::Entry *entry, OpenEditorFlags flags = NoFlags);
     static IEditor *activateEditorForDocument(IDocument *document, OpenEditorFlags flags = NoFlags);
 
-    static bool closeDocument(IDocument *document, bool askAboutModifiedEditors = true);
     static bool closeDocuments(const QList<IDocument *> &documents, bool askAboutModifiedEditors = true);
-    static void closeDocument(DocumentModel::Entry *entry);
     static bool closeDocuments(const QList<DocumentModel::Entry *> &entries);
     static void closeOtherDocuments(IDocument *document);
     static bool closeAllDocuments();
 
     static void addCurrentPositionToNavigationHistory(const QByteArray &saveState = QByteArray());
+    static void setLastEditLocation(const IEditor *editor);
     static void cutForwardNavigationHistory();
 
     static bool saveDocument(IDocument *document);
 
     static bool closeEditors(const QList<IEditor *> &editorsToClose, bool askAboutModifiedEditors = true);
-    static void closeEditor(IEditor *editor, bool askAboutModifiedEditors = true);
 
     static QByteArray saveState();
     static bool restoreState(const QByteArray &state);
@@ -146,13 +149,17 @@ public:
                                     const QString &infoText,
                                     const QString &buttonText = QString(),
                                     QObject *object = nullptr,
-                                    const std::function<void()> &function = nullptr);
+                                    const std::function<void()> &function = {});
     static void hideEditorStatusBar(const QString &id);
 
     static bool isAutoSaveFile(const QString &fileName);
 
+    static bool autoSaveAfterRefactoring();
+
     static QTextCodec *defaultTextCodec();
 
+    static Utils::TextFileFormat::LineTerminationMode defaultLineEnding();
+
     static qint64 maxTextFileSize();
 
     static void setWindowTitleAdditionHandler(WindowTitleHandler handler);
@@ -163,7 +170,7 @@ public:
                                              IEditor *editor = nullptr);
     static void addPinEditorActions(QMenu *contextMenu, DocumentModel::Entry *entry);
     static void addNativeDirAndOpenWithActions(QMenu *contextMenu, DocumentModel::Entry *entry);
-    static void populateOpenWithMenu(QMenu *menu, const QString &fileName);
+    static void populateOpenWithMenu(QMenu *menu, const Utils::FilePath &filePath);
 
 public: // for tests
     static IDocument::ReloadSetting reloadSetting();
@@ -186,6 +193,10 @@ signals:
     void autoSaved();
     void currentEditorAboutToChange(Core::IEditor *editor);
 
+#ifdef WITH_TESTS
+    void linkOpened();
+#endif
+
 public slots:
     static void saveDocument();
     static void saveDocumentAs();
