@@ -101,7 +101,7 @@ ExternalTool::ExternalTool(const ExternalTool *other)
       m_outputHandling(other->m_outputHandling),
       m_errorHandling(other->m_errorHandling),
       m_modifiesCurrentDocument(other->m_modifiesCurrentDocument),
-      m_fileName(other->m_fileName),
+      m_filePath(other->m_filePath),
       m_presetTool(other->m_presetTool)
 {
 }
@@ -121,7 +121,7 @@ ExternalTool &ExternalTool::operator=(const ExternalTool &other)
     m_outputHandling = other.m_outputHandling;
     m_errorHandling = other.m_errorHandling;
     m_modifiesCurrentDocument = other.m_modifiesCurrentDocument;
-    m_fileName = other.m_fileName;
+    m_filePath = other.m_filePath;
     m_presetFileName = other.m_presetFileName;
     m_presetTool = other.m_presetTool;
     return *this;
@@ -154,7 +154,7 @@ int ExternalTool::order() const
     return m_order;
 }
 
-QStringList ExternalTool::executables() const
+FilePaths ExternalTool::executables() const
 {
     return m_executables;
 }
@@ -169,7 +169,7 @@ QString ExternalTool::input() const
     return m_input;
 }
 
-QString ExternalTool::workingDirectory() const
+FilePath ExternalTool::workingDirectory() const
 {
     return m_workingDirectory;
 }
@@ -210,9 +210,9 @@ bool ExternalTool::modifiesCurrentDocument() const
     return m_modifiesCurrentDocument;
 }
 
-void ExternalTool::setFileName(const QString &fileName)
+void ExternalTool::setFileName(const Utils::FilePath &fileName)
 {
-    m_fileName = fileName;
+    m_filePath = fileName;
 }
 
 void ExternalTool::setPreset(QSharedPointer<ExternalTool> preset)
@@ -220,9 +220,9 @@ void ExternalTool::setPreset(QSharedPointer<ExternalTool> preset)
     m_presetTool = preset;
 }
 
-QString ExternalTool::fileName() const
+Utils::FilePath ExternalTool::fileName() const
 {
-    return m_fileName;
+    return m_filePath;
 }
 
 QSharedPointer<ExternalTool> ExternalTool::preset() const
@@ -250,44 +250,37 @@ void ExternalTool::setDescription(const QString &description)
     m_description = description;
 }
 
-
 void ExternalTool::setOutputHandling(OutputHandling handling)
 {
     m_outputHandling = handling;
 }
 
-
 void ExternalTool::setErrorHandling(OutputHandling handling)
 {
     m_errorHandling = handling;
 }
 
-
 void ExternalTool::setModifiesCurrentDocument(bool modifies)
 {
     m_modifiesCurrentDocument = modifies;
 }
 
-
-void ExternalTool::setExecutables(const QStringList &executables)
+void ExternalTool::setExecutables(const FilePaths &executables)
 {
     m_executables = executables;
 }
 
-
 void ExternalTool::setArguments(const QString &arguments)
 {
     m_arguments = arguments;
 }
 
-
 void ExternalTool::setInput(const QString &input)
 {
     m_input = input;
 }
 
-
-void ExternalTool::setWorkingDirectory(const QString &workingDirectory)
+void ExternalTool::setWorkingDirectory(const FilePath &workingDirectory)
 {
     m_workingDirectory = workingDirectory;
 }
@@ -417,7 +410,7 @@ ExternalTool * ExternalTool::createFromXml(const QByteArray &xml, QString *error
             }
             while (reader.readNextStartElement()) {
                 if (reader.name() == QLatin1String(kPath)) {
-                    tool->m_executables.append(reader.readElementText());
+                    tool->m_executables.append(FilePath::fromString(reader.readElementText()));
                 } else if (reader.name() == QLatin1String(kArguments)) {
                     if (!tool->m_arguments.isEmpty()) {
                         reader.raiseError("only one <arguments> element allowed");
@@ -435,7 +428,7 @@ ExternalTool * ExternalTool::createFromXml(const QByteArray &xml, QString *error
                         reader.raiseError("only one <workingdirectory> element allowed");
                         break;
                     }
-                    tool->m_workingDirectory = reader.readElementText();
+                    tool->m_workingDirectory = FilePath::fromString(reader.readElementText());
                 } else if (reader.name() == QLatin1String(kBaseEnvironmentId)) {
                     if (tool->m_baseEnvironmentProviderId.isValid()) {
                         reader.raiseError("only one <baseEnvironmentId> element allowed");
@@ -470,16 +463,16 @@ ExternalTool * ExternalTool::createFromXml(const QByteArray &xml, QString *error
     return tool;
 }
 
-ExternalTool * ExternalTool::createFromFile(const QString &fileName, QString *errorMessage, const QString &locale)
+ExternalTool * ExternalTool::createFromFile(const Utils::FilePath &fileName, QString *errorMessage, const QString &locale)
 {
-    QString absFileName = QFileInfo(fileName).absoluteFilePath();
+    Utils::FilePath absFileName = fileName.absoluteFilePath();
     FileReader reader;
     if (!reader.fetch(absFileName, errorMessage))
         return nullptr;
     ExternalTool *tool = ExternalTool::createFromXml(reader.data(), errorMessage, locale);
     if (!tool)
         return nullptr;
-    tool->m_fileName = absFileName;
+    tool->m_filePath = absFileName;
     return tool;
 }
 
@@ -498,9 +491,9 @@ static QString stringForOutputHandling(ExternalTool::OutputHandling handling)
 
 bool ExternalTool::save(QString *errorMessage) const
 {
-    if (m_fileName.isEmpty())
+    if (m_filePath.isEmpty())
         return false;
-    FileSaver saver(m_fileName);
+    FileSaver saver(m_filePath);
     if (!saver.hasError()) {
         QXmlStreamWriter out(saver.file());
         out.setAutoFormatting(true);
@@ -519,14 +512,14 @@ bool ExternalTool::save(QString *errorMessage) const
         out.writeAttribute(kOutput, stringForOutputHandling(m_outputHandling));
         out.writeAttribute(kError, stringForOutputHandling(m_errorHandling));
         out.writeAttribute(kModifiesDocument, QLatin1String(m_modifiesCurrentDocument ? kYes : kNo));
-        foreach (const QString &executable, m_executables)
-            out.writeTextElement(kPath, executable);
+        for (const FilePath &executable : m_executables)
+            out.writeTextElement(kPath, executable.toString());
         if (!m_arguments.isEmpty())
             out.writeTextElement(kArguments, m_arguments);
         if (!m_input.isEmpty())
             out.writeTextElement(kInput, m_input);
         if (!m_workingDirectory.isEmpty())
-            out.writeTextElement(kWorkingDirectory, m_workingDirectory);
+            out.writeTextElement(kWorkingDirectory, m_workingDirectory.toString());
         if (m_baseEnvironmentProviderId.isValid())
             out.writeTextElement(kBaseEnvironmentId, m_baseEnvironmentProviderId.toString());
         if (!m_environment.isEmpty()) {
@@ -560,7 +553,7 @@ bool ExternalTool::operator==(const ExternalTool &other) const
             && m_outputHandling == other.m_outputHandling
             && m_modifiesCurrentDocument == other.m_modifiesCurrentDocument
             && m_errorHandling == other.m_errorHandling
-            && m_fileName == other.m_fileName;
+            && m_filePath == other.m_filePath;
 }
 
 // #pragma mark -- ExternalToolRunner
@@ -608,11 +601,12 @@ bool ExternalToolRunner::resolve()
 
     {
         // executable
-        QStringList expandedExecutables; /* for error message */
-        foreach (const QString &executable, m_tool->executables()) {
-            QString expanded = expander->expand(executable);
+        FilePaths expandedExecutables; /* for error message */
+        const FilePaths executables = m_tool->executables();
+        for (const FilePath &executable : executables) {
+            FilePath expanded = expander->expand(executable);
             expandedExecutables.append(expanded);
-            m_resolvedExecutable = m_resolvedEnvironment.searchInPath(expanded);
+            m_resolvedExecutable = m_resolvedEnvironment.searchInPath(expanded.path());
             if (!m_resolvedExecutable.isEmpty())
                 break;
         }
@@ -620,7 +614,8 @@ bool ExternalToolRunner::resolve()
             m_hasError = true;
             for (int i = 0; i < expandedExecutables.size(); ++i) {
                 m_errorString += tr("Could not find executable for \"%1\" (expanded \"%2\")")
-                        .arg(m_tool->executables().at(i), expandedExecutables.at(i));
+                        .arg(m_tool->executables().at(i).toUserOutput(),
+                             expandedExecutables.at(i).toUserOutput());
                 m_errorString += QLatin1Char('\n');
             }
             if (!m_errorString.isEmpty())
@@ -644,58 +639,55 @@ void ExternalToolRunner::run()
     }
     if (m_tool->modifiesCurrentDocument()) {
         if (IDocument *document = EditorManager::currentDocument()) {
-            m_expectedFileName = document->filePath().toString();
+            m_expectedFilePath = document->filePath();
             if (!DocumentManager::saveModifiedDocument(document)) {
                 deleteLater();
                 return;
             }
-            DocumentManager::expectFileChange(m_expectedFileName);
+            DocumentManager::expectFileChange(m_expectedFilePath);
         }
     }
     m_process = new QtcProcess(this);
-    connect(m_process, &QProcess::started, this, &ExternalToolRunner::started);
-    connect(m_process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
-            this, &ExternalToolRunner::finished);
-    connect(m_process, &QProcess::errorOccurred, this, &ExternalToolRunner::error);
-    connect(m_process, &QProcess::readyReadStandardOutput,
+    connect(m_process, &QtcProcess::finished, this, &ExternalToolRunner::finished);
+    connect(m_process, &QtcProcess::errorOccurred, this, &ExternalToolRunner::error);
+    connect(m_process, &QtcProcess::readyReadStandardOutput,
             this, &ExternalToolRunner::readStandardOutput);
-    connect(m_process, &QProcess::readyReadStandardError,
+    connect(m_process, &QtcProcess::readyReadStandardError,
             this, &ExternalToolRunner::readStandardError);
     if (!m_resolvedWorkingDirectory.isEmpty())
         m_process->setWorkingDirectory(m_resolvedWorkingDirectory);
     const CommandLine cmd{m_resolvedExecutable, m_resolvedArguments, CommandLine::Raw};
     m_process->setCommand(cmd);
     m_process->setEnvironment(m_resolvedEnvironment);
-    MessageManager::writeWithTime(tr("Starting external tool \"%1\"")
-                                  .arg(cmd.toUserOutput()), MessageManager::Silent);
+    const auto write = m_tool->outputHandling() == ExternalTool::ShowInPane
+                           ? QOverload<const QString &>::of(MessageManager::writeDisrupting)
+                           : QOverload<const QString &>::of(MessageManager::writeSilently);
+    write(tr("Starting external tool \"%1\"").arg(cmd.toUserOutput()));
+    if (!m_resolvedInput.isEmpty())
+        m_process->setWriteData(m_resolvedInput.toLocal8Bit());
     m_process->start();
 }
 
-void ExternalToolRunner::started()
+void ExternalToolRunner::finished()
 {
-    if (!m_resolvedInput.isEmpty())
-        m_process->write(m_resolvedInput.toLocal8Bit());
-    m_process->closeWriteChannel();
-}
-
-void ExternalToolRunner::finished(int exitCode, QProcess::ExitStatus status)
-{
-    if (status == QProcess::NormalExit && exitCode == 0
+    if (m_process->result() == QtcProcess::FinishedWithSuccess
             &&  (m_tool->outputHandling() == ExternalTool::ReplaceSelection
                  || m_tool->errorHandling() == ExternalTool::ReplaceSelection)) {
         ExternalToolManager::emitReplaceSelectionRequested(m_processOutput);
     }
     if (m_tool->modifiesCurrentDocument())
-        DocumentManager::unexpectFileChange(m_expectedFileName);
-    MessageManager::writeWithTime(tr("\"%1\" finished")
-                                  .arg(m_resolvedExecutable.toUserOutput()), MessageManager::Silent);
+        DocumentManager::unexpectFileChange(m_expectedFilePath);
+    const auto write = m_tool->outputHandling() == ExternalTool::ShowInPane
+                           ? QOverload<const QString &>::of(MessageManager::writeFlashing)
+                           : QOverload<const QString &>::of(MessageManager::writeSilently);
+    write(tr("\"%1\" finished").arg(m_resolvedExecutable.toUserOutput()));
     deleteLater();
 }
 
 void ExternalToolRunner::error(QProcess::ProcessError error)
 {
     if (m_tool->modifiesCurrentDocument())
-        DocumentManager::unexpectFileChange(m_expectedFileName);
+        DocumentManager::unexpectFileChange(m_expectedFilePath);
     // TODO inform about errors
     Q_UNUSED(error)
     deleteLater();
@@ -705,10 +697,12 @@ void ExternalToolRunner::readStandardOutput()
 {
     if (m_tool->outputHandling() == ExternalTool::Ignore)
         return;
-    QByteArray data = m_process->readAllStandardOutput();
-    QString output = m_outputCodec->toUnicode(data.constData(), data.length(), &m_outputCodecState);
+    const QByteArray data = m_process->readAllStandardOutput();
+    const QString output = m_outputCodec->toUnicode(data.constData(),
+                                                    data.length(),
+                                                    &m_outputCodecState);
     if (m_tool->outputHandling() == ExternalTool::ShowInPane)
-        MessageManager::write(output);
+        MessageManager::writeSilently(output);
     else if (m_tool->outputHandling() == ExternalTool::ReplaceSelection)
         m_processOutput.append(output);
 }
@@ -717,10 +711,12 @@ void ExternalToolRunner::readStandardError()
 {
     if (m_tool->errorHandling() == ExternalTool::Ignore)
         return;
-    QByteArray data = m_process->readAllStandardError();
-    QString output = m_outputCodec->toUnicode(data.constData(), data.length(), &m_errorCodecState);
+    const QByteArray data = m_process->readAllStandardError();
+    const QString output = m_outputCodec->toUnicode(data.constData(),
+                                                    data.length(),
+                                                    &m_errorCodecState);
     if (m_tool->errorHandling() == ExternalTool::ShowInPane)
-        MessageManager::write(output);
+        MessageManager::writeSilently(output);
     else if (m_tool->errorHandling() == ExternalTool::ReplaceSelection)
         m_processOutput.append(output);
 }
