@@ -25,8 +25,9 @@
 
 #pragma once
 
-#include <utils/fileutils.h>
 #include <utils/environment.h>
+#include <utils/fileutils.h>
+#include <utils/id.h>
 
 #include <QObject>
 #include <QStringList>
@@ -63,20 +64,22 @@ public:
     OutputHandling errorHandling() const;
     bool modifiesCurrentDocument() const;
 
-    QStringList executables() const;
+    Utils::FilePaths executables() const;
     QString arguments() const;
     QString input() const;
-    QString workingDirectory() const;
-    QList<Utils::EnvironmentItem> environment() const;
+    Utils::FilePath workingDirectory() const;
+    Utils::Id baseEnvironmentProviderId() const;
+    Utils::Environment baseEnvironment() const;
+    Utils::EnvironmentItems environmentUserChanges() const;
 
-    void setFileName(const QString &fileName);
+    void setFileName(const Utils::FilePath &fileName);
     void setPreset(QSharedPointer<ExternalTool> preset);
-    QString fileName() const;
+    Utils::FilePath fileName() const;
     // all tools that are preset (changed or unchanged) have the original value here:
     QSharedPointer<ExternalTool> preset() const;
 
     static ExternalTool *createFromXml(const QByteArray &xml, QString *errorMessage = nullptr, const QString &locale = QString());
-    static ExternalTool *createFromFile(const QString &fileName, QString *errorMessage = nullptr,
+    static ExternalTool *createFromFile(const Utils::FilePath &fileName, QString *errorMessage = nullptr,
                                         const QString &locale = QString());
 
     bool save(QString *errorMessage = nullptr) const;
@@ -92,11 +95,12 @@ public:
     void setOutputHandling(OutputHandling handling);
     void setErrorHandling(OutputHandling handling);
     void setModifiesCurrentDocument(bool modifies);
-    void setExecutables(const QStringList &executables);
+    void setExecutables(const Utils::FilePaths &executables);
     void setArguments(const QString &arguments);
     void setInput(const QString &input);
-    void setWorkingDirectory(const QString &workingDirectory);
-    void setEnvironment(const QList<Utils::EnvironmentItem> &items);
+    void setWorkingDirectory(const Utils::FilePath &workingDirectory);
+    void setBaseEnvironmentProviderId(Utils::Id id);
+    void setEnvironmentUserChanges(const Utils::EnvironmentItems &items);
 
 private:
     QString m_id;
@@ -104,17 +108,18 @@ private:
     QString m_displayName;
     QString m_displayCategory;
     int m_order = -1;
-    QStringList m_executables;
+    Utils::FilePaths m_executables;
     QString m_arguments;
     QString m_input;
-    QString m_workingDirectory;
-    QList<Utils::EnvironmentItem> m_environment;
+    Utils::FilePath m_workingDirectory;
+    Utils::Id m_baseEnvironmentProviderId;
+    Utils::EnvironmentItems m_environment;
     OutputHandling m_outputHandling = ShowInPane;
     OutputHandling m_errorHandling = ShowInPane;
     bool m_modifiesCurrentDocument = false;
 
-    QString m_fileName;
-    QString m_presetFileName;
+    Utils::FilePath m_filePath;
+    Utils::FilePath m_presetFileName;
     QSharedPointer<ExternalTool> m_presetTool;
 };
 
@@ -129,8 +134,7 @@ public:
     QString errorString() const;
 
 private:
-    void started();
-    void finished(int exitCode, QProcess::ExitStatus status);
+    void finished();
     void error(QProcess::ProcessError error);
     void readStandardOutput();
     void readStandardError();
@@ -139,17 +143,17 @@ private:
     bool resolve();
 
     const ExternalTool *m_tool; // is a copy of the tool that was passed in
-    Utils::FileName m_resolvedExecutable;
+    Utils::FilePath m_resolvedExecutable;
     QString m_resolvedArguments;
     QString m_resolvedInput;
-    QString m_resolvedWorkingDirectory;
+    Utils::FilePath m_resolvedWorkingDirectory;
     Utils::Environment m_resolvedEnvironment;
     Utils::QtcProcess *m_process;
     QTextCodec *m_outputCodec;
     QTextCodec::ConverterState m_outputCodecState;
     QTextCodec::ConverterState m_errorCodecState;
     QString m_processOutput;
-    QString m_expectedFileName;
+    Utils::FilePath m_expectedFilePath;
     bool m_hasError;
     QString m_errorString;
 };
