@@ -34,9 +34,10 @@
 #include <utils/consoleprocess.h>
 #include <utils/environment.h>
 #include <utils/hostosinfo.h>
-#include <utils/qtcprocess.h>
+#include <utils/commandline.h>
 #include <utils/textfileformat.h>
 #include <utils/unixutils.h>
+#include <utils/qtcprocess.h>
 
 #include <QApplication>
 #include <QDir>
@@ -70,9 +71,9 @@ static void showGraphicalShellError(QWidget *parent, const QString &app, const Q
         ICore::showOptionsDialog(Constants::SETTINGS_ID_INTERFACE, parent);
 }
 
-void FileUtils::showInGraphicalShell(QWidget *parent, const QString &pathIn)
+void FileUtils::showInGraphicalShell(QWidget *parent, const FilePath &pathIn)
 {
-    const QFileInfo fileInfo(pathIn);
+    const QFileInfo fileInfo = pathIn.toFileInfo();
     // Mac, Windows support folder or file.
     if (HostOsInfo::isWindowsHost()) {
         const FilePath explorer = Environment::systemEnvironment().searchInPath(QLatin1String("explorer.exe"));
@@ -85,25 +86,28 @@ void FileUtils::showInGraphicalShell(QWidget *parent, const QString &pathIn)
             return;
         }
         QStringList param;
-        if (!fileInfo.isDir())
+        if (!pathIn.isDir())
             param += QLatin1String("/select,");
         param += QDir::toNativeSeparators(fileInfo.canonicalFilePath());
-        QProcess::startDetached(explorer.toString(), param);
+        QtcProcess::startDetached({explorer, param});
     } else if (HostOsInfo::isMacHost()) {
         QStringList scriptArgs;
         scriptArgs << QLatin1String("-e")
                    << QString::fromLatin1("tell application \"Finder\" to reveal POSIX file \"%1\"")
                                          .arg(fileInfo.canonicalFilePath());
-        QProcess::execute(QLatin1String("/usr/bin/osascript"), scriptArgs);
+        QtcProcess osascriptProcess;
+        osascriptProcess.setCommand({"/usr/bin/osascript", scriptArgs});
+        osascriptProcess.runBlocking();
         scriptArgs.clear();
         scriptArgs << QLatin1String("-e")
                    << QLatin1String("tell application \"Finder\" to activate");
-        QProcess::execute(QLatin1String("/usr/bin/osascript"), scriptArgs);
+        osascriptProcess.setCommand({"/usr/bin/osascript", scriptArgs});
+        osascriptProcess.runBlocking();
     } else {
         // we cannot select a file here, because no file browser really supports it...
         const QString folder = fileInfo.isDir() ? fileInfo.absoluteFilePath() : fileInfo.filePath();
         const QString app = UnixUtils::fileBrowser(ICore::settings());
-        QStringList browserArgs = Utils::QtcProcess::splitArgs(
+        QStringList browserArgs = ProcessArgs::splitArgs(
                     UnixUtils::substituteFileBrowserParameters(app, folder));
         QString error;
         if (browserArgs.isEmpty()) {
@@ -124,14 +128,14 @@ void FileUtils::showInGraphicalShell(QWidget *parent, const QString &pathIn)
     }
 }
 
-void FileUtils::openTerminal(const QString &path)
+void FileUtils::openTerminal(const FilePath &path)
 {
     openTerminal(path, Environment::systemEnvironment());
 }
 
-void FileUtils::openTerminal(const QString &path, const Environment &env)
+void FileUtils::openTerminal(const FilePath &path, const Environment &env)
 {
-    const QFileInfo fileInfo(path);
+    const QFileInfo fileInfo = path.toFileInfo();
     const QString pwd = QDir::toNativeSeparators(fileInfo.isDir() ?
                                                  fileInfo.absoluteFilePath() :
                                                  fileInfo.absolutePath());
@@ -143,6 +147,11 @@ QString FileUtils::msgFindInDirectory()
     return QApplication::translate("Core::Internal", "Find in This Directory...");
 }
 
+QString FileUtils::msgFileSystemAction()
+{
+    return QApplication::translate("Core::Internal", "Show in File System View");
+}
+
 QString FileUtils::msgGraphicalShellAction()
 {
     if (HostOsInfo::isWindowsHost())
@@ -168,11 +177,6 @@ QString FileUtils::msgTerminalWithAction()
                         "Opens a submenu for choosing an environment, such as \"Run Environment\"");
 }
 
-void FileUtils::removeFile(const QString &filePath, bool deleteFromFS)
-{
-    removeFiles({FilePath::fromString(filePath)}, deleteFromFS);
-}
-
 void FileUtils::removeFiles(const FilePaths &filePaths, bool deleteFromFS)
 {
     // remove from version control
@@ -187,49 +191,40 @@ void FileUtils::removeFiles(const FilePaths &filePaths, bool deleteFromFS)
         if (!file.exists()) // could have been deleted by vc
             continue;
         if (!file.remove()) {
-            MessageManager::write(QCoreApplication::translate(
-                                      "Core::Internal",
-                                      "Failed to remove file \"%1\")1.").
-                                  arg(fp.toUserOutput()), MessageManager::ModeSwitch);
+            MessageManager::writeDisrupting(
+                QCoreApplication::translate("Core::Internal", "Failed to remove file \"%1\".")
+                    .arg(fp.toUserOutput()));
         }
     }
 }
 
-static inline bool fileSystemRenameFile(const QString &orgFilePath,
-                                        const QString &newFilePath)
-{
-    // QTBUG-3570 is also valid for Qt 5 but QAbstractFileEngine is now in a private header file and
-    // the symbol is not exported.
-    return QFile::rename(orgFilePath, newFilePath);
-}
-
-bool FileUtils::renameFile(const QString &orgFilePath, const QString &newFilePath,
+bool FileUtils::renameFile(const FilePath &orgFilePath, const FilePath &newFilePath,
                            HandleIncludeGuards handleGuards)
 {
     if (orgFilePath == newFilePath)
         return false;
 
-    QString dir = QFileInfo(orgFilePath).absolutePath();
+    FilePath dir = orgFilePath.absolutePath();
     IVersionControl *vc = VcsManager::findVersionControlForDirectory(dir);
 
     bool result = false;
     if (vc && vc->supportsOperation(IVersionControl::MoveOperation))
         result = vc->vcsMove(orgFilePath, newFilePath);
     if (!result) // The moving via vcs failed or the vcs does not support moving, fall back
-        result = fileSystemRenameFile(orgFilePath, newFilePath);
+        result = orgFilePath.renameFile(newFilePath);
     if (result) {
         // yeah we moved, tell the filemanager about it
         DocumentManager::renamedFile(orgFilePath, newFilePath);
     }
 
     if (result && handleGuards == HandleIncludeGuards::Yes) {
-        QFileInfo fi(orgFilePath);
-        bool headerUpdateSuccess = updateHeaderFileGuardAfterRename(newFilePath, fi.baseName());
+        bool headerUpdateSuccess = updateHeaderFileGuardAfterRename(newFilePath.toString(),
+                                                                    orgFilePath.baseName());
         if (!headerUpdateSuccess) {
-            Core::MessageManager::write(QCoreApplication::translate(
-                                            "Core::FileUtils",
+            Core::MessageManager::writeDisrupting(
+                QCoreApplication::translate("Core::FileUtils",
                                             "Failed to rename the include guard in file \"%1\".")
-                                        .arg(newFilePath));
+                    .arg(newFilePath.toUserOutput()));
         }
     }
 
