@@ -38,7 +38,6 @@
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
-#include <coreplugin/id.h>
 #include <coreplugin/coreplugin.h>
 
 #include <extensionsystem/pluginmanager.h>
@@ -48,28 +47,133 @@
 
 #include <QMenu>
 #include <QStringListModel>
+#include <QVector>
 #include <QAction>
-
-#include <QtPlugin>
 #include <QSettings>
 
-/*!
-    \namespace Core::Internal
-    \internal
-*/
 /*!
     \namespace Core::Internal::ItemDataRoles
     \internal
 */
 
+/*!
+    \class Core::Find
+    \inmodule QtCreator
+    \internal
+*/
+
 Q_DECLARE_METATYPE(Core::IFindFilter*)
 
+using namespace Qt;
+using namespace Utils;
+
 namespace {
     const int MAX_COMPLETIONS = 50;
 }
 
 namespace Core {
 
+struct CompletionEntry
+{
+    QString text;
+    FindFlags findFlags;
+};
+
+QDebug operator<<(QDebug d, const CompletionEntry &e)
+{
+    QDebugStateSaver saver(d);
+    d.noquote();
+    d.nospace();
+    d << "CompletionEntry(\"" << e.text << "\", flags="
+      << "0x" << QString::number(e.findFlags, 16) << ')';
+    return d;
+}
+
+class CompletionModel : public QAbstractListModel
+{
+public:
+    explicit CompletionModel(QObject *p = nullptr) : QAbstractListModel(p) {}
+
+    int rowCount(const QModelIndex & = QModelIndex()) const override { return m_entries.size(); }
+
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
+
+    void writeSettings(QSettings *settings) const;
+    void readSettings(QSettings *settings);
+
+    void updateCompletion(const QString &text, FindFlags f);
+
+private:
+    QVector<CompletionEntry> m_entries;
+};
+
+QVariant CompletionModel::data(const QModelIndex &index, int role) const
+{
+    if (index.isValid()) {
+        const CompletionEntry &entry = m_entries.at(index.row());
+        switch (role) {
+        case Qt::DisplayRole:
+        case Qt::EditRole:
+            return QVariant(entry.text);
+        case Find::CompletionModelFindFlagsRole:
+            return QVariant(int(entry.findFlags));
+        default:
+            break;
+        }
+    }
+    return QVariant();
+}
+
+static inline QString completionSettingsArrayPrefix() { return QStringLiteral("FindCompletions"); }
+static inline QString completionSettingsTextKey() { return QStringLiteral("Text"); }
+static inline QString completionSettingsFlagsKey() { return QStringLiteral("Flags"); }
+
+void CompletionModel::writeSettings(QSettings *settings) const
+{
+    if (m_entries.isEmpty()) {
+        settings->remove(completionSettingsArrayPrefix());
+    } else {
+        const int size = m_entries.size();
+        settings->beginWriteArray(completionSettingsArrayPrefix(), size);
+        for (int i = 0; i < size; ++i) {
+            settings->setArrayIndex(i);
+            settings->setValue(completionSettingsTextKey(), m_entries.at(i).text);
+            settings->setValue(completionSettingsFlagsKey(), int(m_entries.at(i).findFlags));
+        }
+        settings->endArray();
+    }
+}
+
+void CompletionModel::readSettings(QSettings *settings)
+{
+    beginResetModel();
+    const int size = settings->beginReadArray(completionSettingsArrayPrefix());
+    m_entries.clear();
+    m_entries.reserve(size);
+    for (int i = 0; i < size; ++i) {
+        settings->setArrayIndex(i);
+        CompletionEntry entry;
+        entry.text = settings->value(completionSettingsTextKey()).toString();
+        entry.findFlags = FindFlags(settings->value(completionSettingsFlagsKey(), 0).toInt());
+        if (!entry.text.isEmpty())
+            m_entries.append(entry);
+    }
+    settings->endArray();
+    endResetModel();
+}
+
+void CompletionModel::updateCompletion(const QString &text, FindFlags f)
+{
+    if (text.isEmpty())
+         return;
+     beginResetModel();
+     Utils::erase(m_entries, Utils::equal(&CompletionEntry::text, text));
+     m_entries.prepend({text, f});
+     while (m_entries.size() > MAX_COMPLETIONS)
+         m_entries.removeLast();
+     endResetModel();
+}
+
 class FindPrivate : public QObject
 {
     Q_DECLARE_TR_FUNCTIONS(Core::Find)
@@ -78,30 +182,30 @@ public:
     bool isAnyFilterEnabled() const;
     void writeSettings();
     void setFindFlag(Core::FindFlag flag, bool enabled);
-    void updateCompletion(const QString &text, QStringList &completions, QStringListModel *model);
+    static void updateCompletion(const QString &text, QStringList &completions,
+                                 QStringListModel *model);
     void setupMenu();
     void setupFilterMenuItems();
     void readSettings();
 
-    Internal::CurrentDocumentFind *m_currentDocumentFind = 0;
-    Internal::FindToolBar *m_findToolBar = 0;
-    Internal::FindToolWindow *m_findDialog = 0;
-    SearchResultWindow *m_searchResultWindow = 0;
+    Internal::CurrentDocumentFind *m_currentDocumentFind = nullptr;
+    Internal::FindToolBar *m_findToolBar = nullptr;
+    Internal::FindToolWindow *m_findDialog = nullptr;
+    SearchResultWindow *m_searchResultWindow = nullptr;
     FindFlags m_findFlags;
-    QStringListModel m_findCompletionModel;
+    CompletionModel m_findCompletionModel;
     QStringListModel m_replaceCompletionModel;
-    QStringList m_findCompletions;
     QStringList m_replaceCompletions;
-    QAction *m_openFindDialog = 0;
+    QAction *m_openFindDialog = nullptr;
 };
 
-Find *m_instance = 0;
-FindPrivate *d = 0;
+Find *m_instance = nullptr;
+FindPrivate *d = nullptr;
 
 void Find::destroy()
 {
     delete m_instance;
-    m_instance = 0;
+    m_instance = nullptr;
     if (d) {
         delete d->m_currentDocumentFind;
         delete d->m_findToolBar;
@@ -148,7 +252,7 @@ void Find::extensionsInitialized()
 void Find::aboutToShutdown()
 {
     d->m_findToolBar->setVisible(false);
-    d->m_findToolBar->setParent(0);
+    d->m_findToolBar->setParent(nullptr);
     d->m_currentDocumentFind->removeConnections();
 }
 
@@ -202,13 +306,12 @@ static QString filterActionName(const IFindFilter *filter)
 
 void FindPrivate::setupFilterMenuItems()
 {
-    QList<IFindFilter*> findInterfaces = ExtensionSystem::PluginManager::getObjects<IFindFilter>();
     Command *cmd;
 
     ActionContainer *mfindadvanced = ActionManager::actionContainer(Constants::M_FIND_ADVANCED);
     bool haveEnabledFilters = false;
     const Id base("FindFilter.");
-    QList<IFindFilter *> sortedFilters = findInterfaces;
+    QList<IFindFilter *> sortedFilters = IFindFilter::allFindFilters();
     Utils::sort(sortedFilters, &IFindFilter::displayName);
     foreach (IFindFilter *filter, sortedFilters) {
         QAction *action = new QAction(filterActionName(filter), this);
@@ -282,15 +385,17 @@ bool Find::hasFindFlag(FindFlag flag)
 
 void FindPrivate::writeSettings()
 {
-    QSettings *settings = ICore::settings();
+    QtcSettings *settings = ICore::settings();
     settings->beginGroup(QLatin1String("Find"));
-    settings->setValue(QLatin1String("Backward"), bool(m_findFlags & FindBackward));
-    settings->setValue(QLatin1String("CaseSensitively"), bool(m_findFlags & FindCaseSensitively));
-    settings->setValue(QLatin1String("WholeWords"), bool(m_findFlags & FindWholeWords));
-    settings->setValue(QLatin1String("RegularExpression"), bool(m_findFlags & FindRegularExpression));
-    settings->setValue(QLatin1String("PreserveCase"), bool(m_findFlags & FindPreserveCase));
-    settings->setValue(QLatin1String("FindStrings"), m_findCompletions);
-    settings->setValue(QLatin1String("ReplaceStrings"), m_replaceCompletions);
+    settings->setValueWithDefault("Backward", bool(m_findFlags & FindBackward), false);
+    settings->setValueWithDefault("CaseSensitively", bool(m_findFlags & FindCaseSensitively), false);
+    settings->setValueWithDefault("WholeWords", bool(m_findFlags & FindWholeWords), false);
+    settings->setValueWithDefault("RegularExpression",
+                                  bool(m_findFlags & FindRegularExpression),
+                                  false);
+    settings->setValueWithDefault("PreserveCase", bool(m_findFlags & FindPreserveCase), false);
+    m_findCompletionModel.writeSettings(settings);
+    settings->setValueWithDefault("ReplaceStrings", m_replaceCompletions);
     settings->endGroup();
     m_findToolBar->writeSettings();
     m_findDialog->writeSettings();
@@ -301,16 +406,16 @@ void FindPrivate::readSettings()
 {
     QSettings *settings = ICore::settings();
     settings->beginGroup(QLatin1String("Find"));
-    bool block = m_instance->blockSignals(true);
-    Find::setBackward(settings->value(QLatin1String("Backward"), false).toBool());
-    Find::setCaseSensitive(settings->value(QLatin1String("CaseSensitively"), false).toBool());
-    Find::setWholeWord(settings->value(QLatin1String("WholeWords"), false).toBool());
-    Find::setRegularExpression(settings->value(QLatin1String("RegularExpression"), false).toBool());
-    Find::setPreserveCase(settings->value(QLatin1String("PreserveCase"), false).toBool());
-    m_instance->blockSignals(block);
-    m_findCompletions = settings->value(QLatin1String("FindStrings")).toStringList();
+    {
+        QSignalBlocker blocker(m_instance);
+        Find::setBackward(settings->value(QLatin1String("Backward"), false).toBool());
+        Find::setCaseSensitive(settings->value(QLatin1String("CaseSensitively"), false).toBool());
+        Find::setWholeWord(settings->value(QLatin1String("WholeWords"), false).toBool());
+        Find::setRegularExpression(settings->value(QLatin1String("RegularExpression"), false).toBool());
+        Find::setPreserveCase(settings->value(QLatin1String("PreserveCase"), false).toBool());
+    }
+    m_findCompletionModel.readSettings(settings);
     m_replaceCompletions = settings->value(QLatin1String("ReplaceStrings")).toStringList();
-    m_findCompletionModel.setStringList(m_findCompletions);
     m_replaceCompletionModel.setStringList(m_replaceCompletions);
     settings->endGroup();
     m_findToolBar->readSettings();
@@ -318,14 +423,14 @@ void FindPrivate::readSettings()
     emit m_instance->findFlagsChanged(); // would have been done in the setXXX methods above
 }
 
-void Find::updateFindCompletion(const QString &text)
+void Find::updateFindCompletion(const QString &text, FindFlags flags)
 {
-    d->updateCompletion(text, d->m_findCompletions, &d->m_findCompletionModel);
+    d->m_findCompletionModel.updateCompletion(text, flags);
 }
 
 void Find::updateReplaceCompletion(const QString &text)
 {
-    d->updateCompletion(text, d->m_replaceCompletions, &d->m_replaceCompletionModel);
+    FindPrivate::updateCompletion(text, d->m_replaceCompletions, &d->m_replaceCompletionModel);
 }
 
 void FindPrivate::updateCompletion(const QString &text, QStringList &completions, QStringListModel *model)
@@ -353,7 +458,7 @@ void Find::openFindToolBar(FindDirection direction)
     }
 }
 
-QStringListModel *Find::findCompletionModel()
+QAbstractListModel *Find::findCompletionModel()
 {
     return &(d->m_findCompletionModel);
 }
