@@ -733,7 +733,7 @@ void FindToolBar::hideAndResetFocus()
     hide();
 }
 
-FindToolBarPlaceHolder *FindToolBar::findToolBarPlaceHolder() const
+FindToolBarPlaceHolder *FindToolBar::findToolBarPlaceHolder()
 {
     const QList<FindToolBarPlaceHolder*> placeholders = FindToolBarPlaceHolder::allFindToolbarPlaceHolders();
     QWidget *candidate = QApplication::focusWidget();
@@ -889,19 +889,25 @@ bool FindToolBar::focusNextPrevChild(bool next)
 void FindToolBar::resizeEvent(QResizeEvent *event)
 {
     Q_UNUSED(event)
-    QTimer::singleShot(0, this, &FindToolBar::updateToolBar);
+    QMetaObject::invokeMethod(this, &FindToolBar::updateToolBar, Qt::QueuedConnection);
 }
 
 void FindToolBar::writeSettings()
 {
-    QSettings *settings = ICore::settings();
-    settings->beginGroup(QLatin1String("Find"));
-    settings->beginGroup(QLatin1String("FindToolBar"));
-    settings->setValue(QLatin1String("Backward"), QVariant((m_findFlags & FindBackward) != 0));
-    settings->setValue(QLatin1String("CaseSensitively"), QVariant((m_findFlags & FindCaseSensitively) != 0));
-    settings->setValue(QLatin1String("WholeWords"), QVariant((m_findFlags & FindWholeWords) != 0));
-    settings->setValue(QLatin1String("RegularExpression"), QVariant((m_findFlags & FindRegularExpression) != 0));
-    settings->setValue(QLatin1String("PreserveCase"), QVariant((m_findFlags & FindPreserveCase) != 0));
+    Utils::QtcSettings *settings = ICore::settings();
+    settings->beginGroup("Find");
+    settings->beginGroup("FindToolBar");
+    settings->setValueWithDefault("Backward", bool((m_findFlags & FindBackward) != 0), false);
+    settings->setValueWithDefault("CaseSensitively",
+                                  bool((m_findFlags & FindCaseSensitively) != 0),
+                                  false);
+    settings->setValueWithDefault("WholeWords", bool((m_findFlags & FindWholeWords) != 0), false);
+    settings->setValueWithDefault("RegularExpression",
+                                  bool((m_findFlags & FindRegularExpression) != 0),
+                                  false);
+    settings->setValueWithDefault("PreserveCase",
+                                  bool((m_findFlags & FindPreserveCase) != 0),
+                                  false);
     settings->endGroup();
     settings->endGroup();
 }
