@@ -32,6 +32,7 @@
 #include <extensionsystem/pluginmanager.h>
 
 #include <utils/qtcassert.h>
+#include <utils/algorithm.h>
 
 #include <QApplication>
 #include <QDebug>
@@ -141,6 +142,7 @@
     from the focus object as well as the additional context.
 */
 
+#include "dialogs/newdialogwidget.h"
 #include "dialogs/newdialog.h"
 #include "iwizardfactory.h"
 #include "mainwindow.h"
@@ -154,7 +156,6 @@
 #include <QMessageBox>
 #include <QPushButton>
 #include <QStatusBar>
-#include <QTimer>
 
 using namespace Core::Internal;
 using namespace ExtensionSystem;
@@ -166,6 +167,13 @@ namespace Core {
 static ICore *m_instance = nullptr;
 static MainWindow *m_mainwindow = nullptr;
 
+static NewDialog *defaultDialogFactory(QWidget *parent)
+{
+    return new NewDialogWidget(parent);
+}
+
+static std::function<NewDialog *(QWidget *)> m_newDialogFactory = defaultDialogFactory;
+
 /*!
     Returns the pointer to the instance. Only use for connecting to signals.
 */
@@ -211,6 +219,12 @@ ICore::ICore(MainWindow *mainwindow)
             qWarning("Test run was not successful: %d test(s) failed.", failedTests);
         QCoreApplication::exit(failedTests);
     });
+    connect(PluginManager::instance(), &PluginManager::scenarioFinished, [this] (int exitCode) {
+        emit coreAboutToClose();
+        QCoreApplication::exit(exitCode);
+    });
+
+    FileUtils::setDialogParentGetter(&ICore::dialogParent);
 }
 
 /*!
@@ -240,12 +254,28 @@ ICore::~ICore()
 */
 void ICore::showNewItemDialog(const QString &title,
                               const QList<IWizardFactory *> &factories,
-                              const QString &defaultLocation,
+                              const FilePath &defaultLocation,
                               const QVariantMap &extraVariables)
 {
     QTC_ASSERT(!isNewItemDialogRunning(), return);
-    auto newDialog = new NewDialog(dialogParent());
-    connect(newDialog, &QObject::destroyed, m_instance, &ICore::updateNewItemDialogState);
+
+    /* This is a workaround for QDS: In QDS, we currently have a "New Project" dialog box but we do
+     * not also have a "New file" dialog box (yet). Therefore, when requested to add a new file, we
+     * need to use QtCreator's dialog box. In QDS, if `factories` contains project wizard factories
+     * (even though it may contain file wizard factories as well), then we consider it to be a
+     * request for "New Project". Otherwise, if we only have file wizard factories, we defer to
+     * QtCreator's dialog and request "New File"
+     */
+    auto dialogFactory = m_newDialogFactory;
+    bool haveProjectWizards = Utils::anyOf(factories, [](IWizardFactory *f) {
+        return f->kind() == IWizardFactory::ProjectWizard;
+    });
+
+    if (!haveProjectWizards)
+        dialogFactory = defaultDialogFactory;
+
+    NewDialog *newDialog = dialogFactory(dialogParent());
+    connect(newDialog->widget(), &QObject::destroyed, m_instance, &ICore::updateNewItemDialogState);
     newDialog->setWizardFactories(factories, defaultLocation, extraVariables);
     newDialog->setWindowTitle(title);
     newDialog->showDialog();
@@ -343,7 +373,7 @@ bool ICore::showWarningWithOptions(const QString &title, const QString &text,
 
     \sa settingsDatabase()
 */
-QSettings *ICore::settings(QSettings::Scope scope)
+QtcSettings *ICore::settings(QSettings::Scope scope)
 {
     if (scope == QSettings::UserScope)
         return PluginManager::settings();
@@ -387,8 +417,16 @@ QString ICore::userInterfaceLanguage()
     return qApp->property("qtc_locale").toString();
 }
 
+static QString pathHelper(const QString &rel)
+{
+    if (rel.isEmpty())
+        return rel;
+    if (rel.startsWith('/'))
+        return rel;
+    return '/' + rel;
+}
 /*!
-    Returns the absolute path that is used for resources like
+    Returns the absolute path for the relative path \a rel that is used for resources like
     project templates and the debugger macros.
 
     This abstraction is needed to avoid platform-specific code all over
@@ -397,13 +435,15 @@ QString ICore::userInterfaceLanguage()
 
     \sa userResourcePath()
 */
-QString ICore::resourcePath()
+FilePath ICore::resourcePath(const QString &rel)
 {
-    return QDir::cleanPath(QCoreApplication::applicationDirPath() + '/' + RELATIVE_DATA_PATH);
+    return FilePath::fromString(
+               QDir::cleanPath(QCoreApplication::applicationDirPath() + '/' + RELATIVE_DATA_PATH))
+           / rel;
 }
 
 /*!
-    Returns the absolute path in the users directory that is used for
+    Returns the absolute path for the relative path \a rel in the users directory that is used for
     resources like project templates.
 
     Use this function for finding the place for resources that the user may
@@ -412,7 +452,7 @@ QString ICore::resourcePath()
     \sa resourcePath()
 */
 
-QString ICore::userResourcePath()
+FilePath ICore::userResourcePath(const QString &rel)
 {
     // Create qtcreator dir if it doesn't yet exist
     const QString configDir = QFileInfo(settings(QSettings::UserScope)->fileName()).path();
@@ -424,24 +464,26 @@ QString ICore::userResourcePath()
             qWarning() << "could not create" << urp;
     }
 
-    return urp;
+    return FilePath::fromString(urp + pathHelper(rel));
 }
 
 /*!
-    Returns a writable path that can be used for persistent cache files.
+    Returns a writable path for the relative path \a rel that can be used for persistent cache files.
 */
-QString ICore::cacheResourcePath()
+FilePath ICore::cacheResourcePath(const QString &rel)
 {
-    return QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
+    return FilePath::fromString(QStandardPaths::writableLocation(QStandardPaths::CacheLocation)
+                                + pathHelper(rel));
 }
 
 /*!
-    Returns the path to resources written by the installer, for example
-    pre-defined kits and toolchains.
+    Returns the path, based on the relative path \a rel, to resources written by the installer,
+    for example pre-defined kits and toolchains.
 */
-QString ICore::installerResourcePath()
+FilePath ICore::installerResourcePath(const QString &rel)
 {
-    return QFileInfo(settings(QSettings::SystemScope)->fileName()).path() + '/' + Constants::IDE_ID;
+    return FilePath::fromString(settings(QSettings::SystemScope)->fileName()).parentDir()
+                                / Constants::IDE_ID / rel;
 }
 
 /*!
@@ -474,12 +516,27 @@ QString ICore::userPluginPath()
 }
 
 /*!
-    Returns the path to the command line tools that are included in the \QC
-    installation.
+    Returns the path, based on the relative path \a rel, to the command line tools that are
+    included in the \QC installation.
  */
-QString ICore::libexecPath()
+FilePath ICore::libexecPath(const QString &rel)
 {
-    return QDir::cleanPath(QApplication::applicationDirPath() + '/' + RELATIVE_LIBEXEC_PATH);
+    return FilePath::fromString(QDir::cleanPath(QApplication::applicationDirPath()
+                                                + pathHelper(RELATIVE_LIBEXEC_PATH)))
+           / rel;
+}
+
+FilePath ICore::crashReportsPath()
+{
+    if (Utils::HostOsInfo::isMacHost())
+        return libexecPath("crashpad_reports/completed");
+    else
+        return libexecPath("crashpad_reports/reports");
+}
+
+QString ICore::ideDisplayName()
+{
+    return Constants::IDE_DISPLAY_NAME;
 }
 
 static QString clangIncludePath(const QString &clangVersion)
@@ -490,31 +547,31 @@ static QString clangIncludePath(const QString &clangVersion)
 /*!
     \internal
 */
-QString ICore::clangIncludeDirectory(const QString &clangVersion,
-                                     const QString &clangFallbackIncludeDir)
+FilePath ICore::clangIncludeDirectory(const QString &clangVersion,
+                                      const FilePath &clangFallbackIncludeDir)
 {
-    QDir dir(libexecPath() + "/clang" + clangIncludePath(clangVersion));
-    if (!dir.exists() || !QFileInfo(dir, "stdint.h").exists())
-        dir = QDir(clangFallbackIncludeDir);
-    return QDir::toNativeSeparators(dir.canonicalPath());
+    FilePath dir = libexecPath("clang" + clangIncludePath(clangVersion));
+    if (!dir.exists() || !dir.pathAppended("stdint.h").exists())
+        dir = clangFallbackIncludeDir;
+    return dir.canonicalPath();
 }
 
 /*!
     \internal
 */
-static QString clangBinary(const QString &binaryBaseName, const QString &clangBinDirectory)
+static FilePath clangBinary(const QString &binaryBaseName, const FilePath &clangBinDirectory)
 {
-    const QString hostExeSuffix(QTC_HOST_EXE_SUFFIX);
-    QFileInfo executable(ICore::libexecPath() + "/clang/bin/" + binaryBaseName + hostExeSuffix);
+    FilePath executable =
+        ICore::libexecPath("clang/bin").pathAppended(binaryBaseName).withExecutableSuffix();
     if (!executable.exists())
-        executable = QFileInfo(clangBinDirectory + "/" + binaryBaseName + hostExeSuffix);
-    return QDir::toNativeSeparators(executable.canonicalFilePath());
+        executable = clangBinDirectory.pathAppended(binaryBaseName).withExecutableSuffix();
+    return executable.canonicalPath();
 }
 
 /*!
     \internal
 */
-QString ICore::clangExecutable(const QString &clangBinDirectory)
+FilePath ICore::clangExecutable(const FilePath &clangBinDirectory)
 {
     return clangBinary("clang", clangBinDirectory);
 }
@@ -522,7 +579,15 @@ QString ICore::clangExecutable(const QString &clangBinDirectory)
 /*!
     \internal
 */
-QString ICore::clangTidyExecutable(const QString &clangBinDirectory)
+FilePath ICore::clangdExecutable(const FilePath &clangBinDirectory)
+{
+    return clangBinary("clangd", clangBinDirectory);
+}
+
+/*!
+    \internal
+*/
+FilePath ICore::clangTidyExecutable(const FilePath &clangBinDirectory)
 {
     return clangBinary("clang-tidy", clangBinDirectory);
 }
@@ -530,7 +595,7 @@ QString ICore::clangTidyExecutable(const QString &clangBinDirectory)
 /*!
     \internal
 */
-QString ICore::clazyStandaloneExecutable(const QString &clangBinDirectory)
+FilePath ICore::clazyStandaloneExecutable(const FilePath &clangBinDirectory)
 {
     return clangBinary("clazy-standalone", clangBinDirectory);
 }
@@ -538,12 +603,14 @@ QString ICore::clazyStandaloneExecutable(const QString &clangBinDirectory)
 static QString compilerString()
 {
 #if defined(Q_CC_CLANG) // must be before GNU, because clang claims to be GNU too
-    QString isAppleString;
+    QString platformSpecific;
 #if defined(__apple_build_version__) // Apple clang has other version numbers
-    isAppleString = QLatin1String(" (Apple)");
+    platformSpecific = QLatin1String(" (Apple)");
+#elif defined(Q_CC_MSVC)
+    platformSpecific = QLatin1String(" (clang-cl)");
 #endif
     return QLatin1String("Clang " ) + QString::number(__clang_major__) + QLatin1Char('.')
-            + QString::number(__clang_minor__) + isAppleString;
+            + QString::number(__clang_minor__) + platformSpecific;
 #elif defined(Q_CC_GNU)
     return QLatin1String("GCC " ) + QLatin1String(__VERSION__);
 #elif defined(Q_CC_MSVC)
@@ -762,14 +829,14 @@ void ICore::registerWindow(QWidget *window, const Context &context)
 }
 
 /*!
-    Opens files using \a arguments and \a flags like it would be
+    Opens files using \a filePaths and \a flags like it would be
     done if they were given to \QC on the command line, or
     they were opened via \uicontrol File > \uicontrol Open.
 */
 
-void ICore::openFiles(const QStringList &arguments, ICore::OpenFilesFlags flags)
+void ICore::openFiles(const FilePaths &filePaths, ICore::OpenFilesFlags flags)
 {
-    m_mainwindow->openFiles(arguments, flags);
+    MainWindow::openFiles(filePaths, flags);
 }
 
 /*!
@@ -790,7 +857,7 @@ void ICore::addPreCloseListener(const std::function<bool ()> &listener)
 */
 QString ICore::systemInformation()
 {
-    QString result = PluginManager::instance()->systemInformation() + '\n';
+    QString result = PluginManager::systemInformation() + '\n';
     result += versionString() + '\n';
     result += buildCompatibilityString() + '\n';
 #ifdef IDE_REVISION
@@ -821,7 +888,7 @@ public:
     {
         QTC_ASSERT(watched == m_widget, return false);
         if (event->type() == QEvent::Show)
-            QTimer::singleShot(0, this, &ScreenShooter::helper);
+            QMetaObject::invokeMethod(this, &ScreenShooter::helper, Qt::QueuedConnection);
         return false;
     }
 
@@ -902,4 +969,12 @@ void ICore::updateNewItemDialogState()
     emit instance()->newItemDialogStateChanged();
 }
 
+/*!
+    \internal
+*/
+void ICore::setNewDialogFactory(const std::function<NewDialog *(QWidget *)> &newFactory)
+{
+    m_newDialogFactory = newFactory;
+}
+
 } // namespace Core
