@@ -29,27 +29,26 @@
 
 #include <utils/id.h>
 
-#include <QObject>
 #include <QStringList>
 
 #include <functional>
 
+namespace Utils { class FilePath; }
+
 namespace Core {
 
 class IDocument;
 
-class CORE_EXPORT IDocumentFactory : public QObject
+class CORE_EXPORT IDocumentFactory
 {
-    Q_OBJECT
-
 public:
-    IDocumentFactory(QObject *parent = nullptr);
-    ~IDocumentFactory() override;
+    IDocumentFactory();
+    ~IDocumentFactory();
 
     static const QList<IDocumentFactory *> allDocumentFactories();
 
-    using Opener = std::function<IDocument *(const QString &fileName)>;
-    IDocument *open(const QString &filename);
+    using Opener = std::function<IDocument *(const Utils::FilePath &filePath)>;
+    IDocument *open(const Utils::FilePath &filePath);
 
     QStringList mimeTypes() const { return m_mimeTypes; }
 
@@ -59,7 +58,6 @@ public:
     void addMimeType(const QString &mimeType) { m_mimeTypes.append(mimeType); }
 
 private:
-    Utils::Id m_id;
     Opener m_opener;
     QStringList m_mimeTypes;
     QString m_displayName;
