@@ -34,10 +34,12 @@
 #include <extensionsystem/pluginmanager.h>
 
 #include <utils/fileutils.h>
+#include <utils/icon.h>
 #include <utils/qtcassert.h>
 #include <utils/wizard.h>
 
 #include <QAction>
+#include <QPainter>
 
 /*!
     \class Core::IWizardFactory
@@ -141,7 +143,7 @@ class NewItemDialogData
 {
 public:
     void setData(const QString &t, const QList<IWizardFactory *> &f,
-                 const QString &dl, const QVariantMap &ev)
+                 const FilePath &dl, const QVariantMap &ev)
     {
         QTC_ASSERT(!hasData(), return);
 
@@ -173,7 +175,7 @@ public:
 private:
     QString title;
     QList<IWizardFactory *> factories;
-    QString defaultLocation;
+    FilePath defaultLocation;
     QVariantMap extraVariables;
 };
 
@@ -213,7 +215,7 @@ QList<IWizardFactory*> IWizardFactory::allWizardFactories()
 
                 connect(newFactory->m_action, &QAction::triggered, newFactory, [newFactory]() {
                     if (!ICore::isNewItemDialogRunning()) {
-                        QString path = newFactory->runPath(QString());
+                        FilePath path = newFactory->runPath({});
                         newFactory->runWizard(path, ICore::dialogParent(), Id(), QVariantMap());
                     }
                 });
@@ -227,9 +229,9 @@ QList<IWizardFactory*> IWizardFactory::allWizardFactories()
     return s_allFactories;
 }
 
-QString IWizardFactory::runPath(const QString &defaultPath)
+FilePath IWizardFactory::runPath(const FilePath &defaultPath) const
 {
-    QString path = defaultPath;
+    FilePath path = defaultPath;
     if (path.isEmpty()) {
         switch (kind()) {
         case IWizardFactory::ProjectWizard:
@@ -237,7 +239,7 @@ QString IWizardFactory::runPath(const QString &defaultPath)
             // use last visited directory of file dialog. Never start
             // at current.
             path = DocumentManager::useProjectsDirectory()
-                       ? DocumentManager::projectsDirectory().toString()
+                       ? DocumentManager::projectsDirectory()
                        : DocumentManager::fileDialogLastVisitedDirectory();
             break;
         default:
@@ -257,14 +259,17 @@ QString IWizardFactory::runPath(const QString &defaultPath)
     created. The wizard should fill this in its path selection elements as a
     default path.
 */
-Utils::Wizard *IWizardFactory::runWizard(const QString &path, QWidget *parent, Id platform, const QVariantMap &variables)
+Wizard *IWizardFactory::runWizard(const FilePath &path, QWidget *parent, Id platform,
+                                  const QVariantMap &variables,
+                                  bool showWizard)
 {
     QTC_ASSERT(!s_isWizardRunning, return nullptr);
 
     s_isWizardRunning = true;
     ICore::updateNewItemDialogState();
 
-    Utils::Wizard *wizard = runWizardImpl(path, parent, platform, variables);
+    Utils::Wizard *wizard = runWizardImpl(path, parent, platform, variables, showWizard);
+
 
     if (wizard) {
         s_currentWizard = wizard;
@@ -286,8 +291,10 @@ Utils::Wizard *IWizardFactory::runWizard(const QString &path, QWidget *parent, I
             s_reopenData.reopen();
         });
         s_inspectWizardAction->setEnabled(true);
-        wizard->show();
-        Core::ICore::registerWindow(wizard, Core::Context("Core.NewWizard"));
+        if (showWizard) {
+            wizard->show();
+            Core::ICore::registerWindow(wizard, Core::Context("Core.NewWizard"));
+        }
     } else {
         s_isWizardRunning = false;
         ICore::updateNewItemDialogState();
@@ -358,12 +365,17 @@ QWidget *IWizardFactory::currentWizard()
 
 void IWizardFactory::requestNewItemDialog(const QString &title,
                                           const QList<IWizardFactory *> &factories,
-                                          const QString &defaultLocation,
+                                          const FilePath &defaultLocation,
                                           const QVariantMap &extraVariables)
 {
     s_reopenData.setData(title, factories, defaultLocation, extraVariables);
 }
 
+QIcon IWizardFactory::themedIcon(const Utils::FilePath &iconMaskPath)
+{
+    return Utils::Icon({{iconMaskPath, Theme::PanelTextColorDark}}, Icon::Tint).icon();
+}
+
 void IWizardFactory::destroyFeatureProvider()
 {
     qDeleteAll(s_providerList);
@@ -381,7 +393,7 @@ void IWizardFactory::clearWizardFactories()
     s_areFactoriesLoaded = false;
 }
 
-QSet<Id> IWizardFactory::pluginFeatures() const
+QSet<Id> IWizardFactory::pluginFeatures()
 {
     static QSet<Id> plugins;
     if (plugins.isEmpty()) {
@@ -394,7 +406,7 @@ QSet<Id> IWizardFactory::pluginFeatures() const
     return plugins;
 }
 
-QSet<Id> IWizardFactory::availableFeatures(Id platformId) const
+QSet<Id> IWizardFactory::availableFeatures(Id platformId)
 {
     QSet<Id> availableFeatures;
 
@@ -418,3 +430,57 @@ void IWizardFactory::initialize()
     s_inspectWizardAction = new QAction(tr("Inspect Wizard State"), ActionManager::instance());
     ActionManager::registerAction(s_inspectWizardAction, "Wizard.Inspect");
 }
+
+static QIcon iconWithText(const QIcon &icon, const QString &text)
+{
+    if (icon.isNull()) {
+        static const QIcon fallBack =
+                IWizardFactory::themedIcon(":/utils/images/wizardicon-file.png");
+        return iconWithText(fallBack, text);
+    }
+
+    if (text.isEmpty())
+        return icon;
+
+    QIcon iconWithText;
+    for (const QSize &pixmapSize : icon.availableSizes()) {
+        QPixmap pixmap = icon.pixmap(pixmapSize);
+        const qreal originalPixmapDpr = pixmap.devicePixelRatio();
+        pixmap.setDevicePixelRatio(1); // Hack for QTCREATORBUG-26315
+        const int fontSize = pixmap.height() / 4;
+        const int margin = pixmap.height() / 8;
+        QFont font;
+        font.setPixelSize(fontSize);
+        font.setStretch(85);
+        QPainter p(&pixmap);
+        p.setPen(Utils::creatorTheme()->color(Theme::PanelTextColorDark));
+        p.setFont(font);
+        QTextOption textOption(Qt::AlignHCenter | Qt::AlignBottom);
+        textOption.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
+        p.drawText(pixmap.rect().adjusted(margin, margin, -margin, -margin), text, textOption);
+        p.end();
+        pixmap.setDevicePixelRatio(originalPixmapDpr);
+        iconWithText.addPixmap(pixmap);
+    }
+    return iconWithText;
+}
+
+void IWizardFactory::setIcon(const QIcon &icon, const QString &iconText)
+{
+    m_icon = iconWithText(icon, iconText);
+}
+
+void IWizardFactory::setDetailsPageQmlPath(const QString &filePath)
+{
+    if (filePath.isEmpty())
+        return;
+
+    if (filePath.startsWith(':')) {
+        m_detailsPageQmlPath.setScheme(QLatin1String("qrc"));
+        QString path = filePath;
+        path.remove(0, 1);
+        m_detailsPageQmlPath.setPath(path);
+    } else {
+        m_detailsPageQmlPath = QUrl::fromLocalFile(filePath);
+    }
+}
