@@ -28,11 +28,12 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <utils/algorithm.h>
 #include <utils/fileutils.h>
+#include <utils/linecolumn.h>
+#include <utils/link.h>
 #include <utils/qtcassert.h>
 
 #include <QDir>
 #include <QRegularExpression>
-#include <QTimer>
 
 using namespace Utils;
 
@@ -50,7 +51,7 @@ public:
     }
 
     QSharedPointer<BaseFileFilter::Iterator> iterator;
-    FilePathList previousResultPaths;
+    FilePaths previousResultPaths;
     bool forceNewSearchList;
     QString previousEntry;
 };
@@ -64,8 +65,31 @@ public:
 
 } // Internal
 
+/*!
+    \class Core::BaseFileFilter
+    \inheaderfile coreplugin/locator/basefilefilter.h
+    \inmodule QtCreator
+
+    \brief The BaseFileFilter class is a base class for locator filter classes.
+*/
+
+/*!
+    \class Core::BaseFileFilter::Iterator
+    \inmodule QtCreator
+    \internal
+*/
+
+/*!
+    \class Core::BaseFileFilter::ListIterator
+    \inmodule QtCreator
+    \internal
+*/
+
 BaseFileFilter::Iterator::~Iterator() = default;
 
+/*!
+    \internal
+*/
 BaseFileFilter::BaseFileFilter()
   : d(new Internal::BaseFileFilterPrivate)
 {
@@ -73,23 +97,26 @@ BaseFileFilter::BaseFileFilter()
     setFileIterator(new ListIterator({}));
 }
 
+/*!
+    \internal
+*/
 BaseFileFilter::~BaseFileFilter()
 {
     delete d;
 }
 
+/*!
+    \reimp
+*/
 void BaseFileFilter::prepareSearch(const QString &entry)
 {
     Q_UNUSED(entry)
-    d->m_current.iterator = d->m_data.iterator;
-    d->m_current.previousResultPaths = d->m_data.previousResultPaths;
-    d->m_current.forceNewSearchList = d->m_data.forceNewSearchList;
-    d->m_current.previousEntry = d->m_data.previousEntry;
+    d->m_current = d->m_data;
     d->m_data.forceNewSearchList = false;
 }
 
 ILocatorFilter::MatchLevel BaseFileFilter::matchLevelFor(const QRegularExpressionMatch &match,
-                                                         const QString &matchText) const
+                                                         const QString &matchText)
 {
     const int consecutivePos = match.capturedStart(1);
     if (consecutivePos == 0)
@@ -104,14 +131,18 @@ ILocatorFilter::MatchLevel BaseFileFilter::matchLevelFor(const QRegularExpressio
     return MatchLevel::Normal;
 }
 
+/*!
+    \reimp
+*/
 QList<LocatorFilterEntry> BaseFileFilter::matchesFor(QFutureInterface<LocatorFilterEntry> &future, const QString &origEntry)
 {
     QList<LocatorFilterEntry> entries[int(MatchLevel::Count)];
     // If search string contains spaces, treat them as wildcard '*' and search in full path
     const QString entry = QDir::fromNativeSeparators(origEntry).replace(' ', '*');
-    const EditorManager::FilePathInfo fp = EditorManager::splitLineAndColumnNumber(entry);
+    QString postfix;
+    Link link = Link::fromString(entry, true, &postfix);
 
-    const QRegularExpression regexp = createRegExp(fp.filePath);
+    const QRegularExpression regexp = createRegExp(link.targetFilePath.toString());
     if (!regexp.isValid()) {
         d->m_current.clear(); // free memory
         return {};
@@ -120,9 +151,9 @@ QList<LocatorFilterEntry> BaseFileFilter::matchesFor(QFutureInterface<LocatorFil
         return candidate.contains('/') || candidate.contains('*');
     };
 
-    const bool hasPathSeparator = containsPathSeparator(fp.filePath);
+    const bool hasPathSeparator = containsPathSeparator(link.targetFilePath.toString());
     const bool containsPreviousEntry = !d->m_current.previousEntry.isEmpty()
-            && fp.filePath.contains(d->m_current.previousEntry);
+            && link.targetFilePath.toString().contains(d->m_current.previousEntry);
     const bool pathSeparatorAdded = !containsPathSeparator(d->m_current.previousEntry)
             && hasPathSeparator;
     const bool searchInPreviousResults = !d->m_current.forceNewSearchList && containsPreviousEntry
@@ -132,7 +163,7 @@ QList<LocatorFilterEntry> BaseFileFilter::matchesFor(QFutureInterface<LocatorFil
 
     QTC_ASSERT(d->m_current.iterator.data(), return QList<LocatorFilterEntry>());
     d->m_current.previousResultPaths.clear();
-    d->m_current.previousEntry = fp.filePath;
+    d->m_current.previousEntry = link.targetFilePath.toString();
     d->m_current.iterator->toFront();
     bool canceled = false;
     while (d->m_current.iterator->hasNext()) {
@@ -147,10 +178,9 @@ QList<LocatorFilterEntry> BaseFileFilter::matchesFor(QFutureInterface<LocatorFil
         QRegularExpressionMatch match = regexp.match(matchText);
 
         if (match.hasMatch()) {
-            QFileInfo fi(path.toString());
-            LocatorFilterEntry filterEntry(this, fi.fileName(), QString(path.toString() + fp.postfix));
-            filterEntry.fileName = path.toString();
-            filterEntry.extraInfo = FilePath::fromFileInfo(fi).shortNativePath();
+            LocatorFilterEntry filterEntry(this, path.fileName(), QString(path.toString() + postfix));
+            filterEntry.filePath = path;
+            filterEntry.extraInfo = path.shortNativePath();
 
             const MatchLevel matchLevel = matchLevelFor(match, matchText);
             if (hasPathSeparator) {
@@ -173,7 +203,8 @@ QList<LocatorFilterEntry> BaseFileFilter::matchesFor(QFutureInterface<LocatorFil
         d->m_current.clear(); // free memory
     } else {
         d->m_current.iterator.clear();
-        QTimer::singleShot(0, this, &BaseFileFilter::updatePreviousResultData);
+        QMetaObject::invokeMethod(this, &BaseFileFilter::updatePreviousResultData,
+                                  Qt::QueuedConnection);
     }
 
     for (auto &entry : entries) {
@@ -184,14 +215,33 @@ QList<LocatorFilterEntry> BaseFileFilter::matchesFor(QFutureInterface<LocatorFil
     return std::accumulate(std::begin(entries), std::end(entries), QList<LocatorFilterEntry>());
 }
 
+/*!
+    \reimp
+*/
 void BaseFileFilter::accept(LocatorFilterEntry selection,
                             QString *newText, int *selectionStart, int *selectionLength) const
 {
     Q_UNUSED(newText)
     Q_UNUSED(selectionStart)
     Q_UNUSED(selectionLength)
-    EditorManager::openEditor(selection.internalData.toString(), Id(),
-                              EditorManager::CanContainLineAndColumnNumber);
+    openEditorAt(selection);
+}
+
+void BaseFileFilter::openEditorAt(const LocatorFilterEntry& selection)
+{
+    const FilePath locatorText = FilePath::fromVariant(selection.internalData);
+    const int postfixLength = locatorText.fileName().length() - selection.filePath.fileName().length();
+    if (postfixLength > 0) {
+        const QString postfix = selection.internalData.toString().right(postfixLength);
+        int postfixPos = -1;
+        const LineColumn lineColumn = LineColumn::extractFromFileName(postfix, postfixPos);
+        if (postfixPos >= 0) {
+            const Link link(selection.filePath, lineColumn.line, lineColumn.column);
+            EditorManager::openEditorAt(link);
+            return;
+        }
+    }
+    EditorManager::openEditor(selection.filePath);
 }
 
 /*!
@@ -206,6 +256,9 @@ void BaseFileFilter::setFileIterator(BaseFileFilter::Iterator *iterator)
     d->m_data.iterator.reset(iterator);
 }
 
+/*!
+    Returns the file iterator.
+*/
 QSharedPointer<BaseFileFilter::Iterator> BaseFileFilter::fileIterator()
 {
     return d->m_data.iterator;
@@ -220,7 +273,7 @@ void BaseFileFilter::updatePreviousResultData()
     // forceNewSearchList was already reset in prepareSearch
 }
 
-BaseFileFilter::ListIterator::ListIterator(const FilePathList  &filePaths)
+BaseFileFilter::ListIterator::ListIterator(const FilePaths  &filePaths)
 {
     m_filePaths = filePaths;
     toFront();
