@@ -37,7 +37,9 @@ ExternalToolsFilter::ExternalToolsFilter()
 {
     setId("Run external tool");
     setDisplayName(tr("Run External Tool"));
-    setShortcutString("x");
+    setDescription(tr("Runs an external tool that you have set up in the options (Environment > "
+                      "External Tools)."));
+    setDefaultShortcutString("x");
     setPriority(Medium);
 }
 
@@ -58,11 +60,7 @@ void ExternalToolsFilter::accept(LocatorFilterEntry selection,
 
     auto runner = new ExternalToolRunner(tool);
     if (runner->hasError())
-        MessageManager::write(runner->errorString());
-}
-
-void ExternalToolsFilter::refresh(QFutureInterface<void> &)
-{
+        MessageManager::writeFlashing(runner->errorString());
 }
 
 void ExternalToolsFilter::prepareSearch(const QString &entry)
