@@ -34,12 +34,19 @@
 #include <extensionsystem/pluginmanager.h>
 #include <utils/qtcassert.h>
 
+#include <QApplication>
+
 using namespace Core::Internal;
 
 namespace Core {
 
-LocatorManager::LocatorManager(QObject *parent)
-  : QObject(parent)
+/*!
+    \class Core::LocatorManager
+    \inmodule QtCreator
+    \internal
+*/
+
+LocatorManager::LocatorManager()
 {
 }
 
@@ -47,6 +54,9 @@ static LocatorWidget *locatorWidget()
 {
     static QPointer<LocatorPopup> popup;
     QWidget *window = ICore::dialogParent()->window();
+    // if that is a popup, try to find a better one
+    if (window->windowFlags() & Qt::Popup && window->parentWidget())
+        window = window->parentWidget()->window();
     if (auto *widget = Aggregation::query<LocatorWidget>(window)) {
         if (popup)
             popup->close();
@@ -67,14 +77,15 @@ void LocatorManager::showFilter(ILocatorFilter *filter)
     // add shortcut string at front or replace existing shortcut string
     if (!currentText.isEmpty()) {
         searchText = currentText;
-        foreach (ILocatorFilter *otherfilter, Locator::filters()) {
-            if (currentText.startsWith(otherfilter->shortcutString() + QLatin1Char(' '))) {
+        const QList<ILocatorFilter *> allFilters = Locator::filters();
+        for (ILocatorFilter *otherfilter : allFilters) {
+            if (currentText.startsWith(otherfilter->shortcutString() + ' ')) {
                 searchText = currentText.mid(otherfilter->shortcutString().length() + 1);
                 break;
             }
         }
     }
-    show(filter->shortcutString() + QLatin1Char(' ') + searchText,
+    show(filter->shortcutString() + ' ' + searchText,
          filter->shortcutString().length() + 1,
          searchText.length());
 }
@@ -95,4 +106,15 @@ QWidget *LocatorManager::createLocatorInputWidget(QWidget *window)
     return locatorWidget;
 }
 
+bool LocatorManager::locatorHasFocus()
+{
+    QWidget *w = qApp->focusWidget();
+    while (w) {
+        if (qobject_cast<LocatorWidget *>(w))
+            return true;
+        w = w->parentWidget();
+    }
+    return false;
+}
+
 } // namespace Core
