@@ -55,8 +55,9 @@
 #include <coreplugin/actionmanager/actionmanager_p.h>
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/dialogs/externaltoolconfig.h>
-#include <coreplugin/dialogs/newdialog.h>
+#include <coreplugin/iwizardfactory.h>
 #include <coreplugin/dialogs/shortcutsettings.h>
+#include <coreplugin/editormanager/documentmodel_p.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/editormanager_p.h>
 #include <coreplugin/editormanager/ieditor.h>
@@ -89,9 +90,9 @@
 #include <QSettings>
 #include <QStatusBar>
 #include <QStyleFactory>
-#include <QTimer>
 #include <QToolButton>
 #include <QUrl>
+#include <QWindow>
 
 using namespace ExtensionSystem;
 using namespace Utils;
@@ -101,6 +102,14 @@ namespace Internal {
 
 enum { debugMainWindow = 0 };
 
+static bool isQtDesignStudio()
+{
+    QSettings *settings = Core::ICore::settings();
+    const QString qdsStandaloneEntry = "QML/Designer/StandAloneMode"; //entry from qml settings
+
+    return settings->value(qdsStandaloneEntry, false).toBool();
+}
+
 MainWindow::MainWindow()
     : AppMainWindow()
     , m_coreImpl(new ICore(this))
@@ -148,7 +157,7 @@ MainWindow::MainWindow()
 
     QApplication::setStyle(new ManhattanStyle(baseName));
     m_generalSettings->setShowShortcutsInContextMenu(
-        m_generalSettings->showShortcutsInContextMenu());
+        GeneralSettings::showShortcutsInContextMenu());
 
     setDockNestingEnabled(true);
 
@@ -299,6 +308,7 @@ void MainWindow::init()
 {
     m_progressManager->init(); // needs the status bar manager
     MessageManager::init();
+    OutputPaneManager::create();
 }
 
 void MainWindow::extensionsInitialized()
@@ -307,8 +317,8 @@ void MainWindow::extensionsInitialized()
     MimeTypeSettings::restoreSettings();
     m_windowSupport = new WindowSupport(this, Context("Core.MainWindow"));
     m_windowSupport->setCloseActionEnabled(false);
-    OutputPaneManager::create();
-    m_vcsManager->extensionsInitialized();
+    OutputPaneManager::initialize();
+    VcsManager::extensionsInitialized();
     m_leftNavigationWidget->setFactories(INavigationWidgetFactory::allNavigationFactories());
     m_rightNavigationWidget->setFactories(INavigationWidgetFactory::allNavigationFactories());
 
@@ -319,8 +329,8 @@ void MainWindow::extensionsInitialized()
 
     emit m_coreImpl->coreAboutToOpen();
     // Delay restoreWindowState, since it is overridden by LayoutRequest event
-    QTimer::singleShot(0, this, &MainWindow::restoreWindowState);
-    QTimer::singleShot(0, m_coreImpl, &ICore::coreOpened);
+    QMetaObject::invokeMethod(this, &MainWindow::restoreWindowState, Qt::QueuedConnection);
+    QMetaObject::invokeMethod(m_coreImpl, &ICore::coreOpened, Qt::QueuedConnection);
 }
 
 static void setRestart(bool restart)
@@ -388,7 +398,7 @@ void MainWindow::closeEvent(QCloseEvent *event)
 void MainWindow::openDroppedFiles(const QList<DropSupport::FileSpec> &files)
 {
     raiseWindow();
-    QStringList filePaths = Utils::transform(files, &DropSupport::FileSpec::filePath);
+    const FilePaths filePaths = Utils::transform(files, &DropSupport::FileSpec::filePath);
     openFiles(filePaths, ICore::SwitchMode);
 }
 
@@ -517,19 +527,40 @@ void MainWindow::registerDefaultActions()
 
     // New File Action
     QIcon icon = QIcon::fromTheme(QLatin1String("document-new"), Utils::Icons::NEWFILE.icon());
-    m_newAction = new QAction(icon, tr("&New File or Project..."), this);
+
+    const bool isQDS = isQtDesignStudio();
+    const QString newActionText = isQDS ? tr("&New Project...") : tr("&New File or Project...");
+    m_newAction = new QAction(icon, newActionText, this);
     cmd = ActionManager::registerAction(m_newAction, Constants::NEW);
     cmd->setDefaultKeySequence(QKeySequence::New);
     mfile->addAction(cmd, Constants::G_FILE_NEW);
     connect(m_newAction, &QAction::triggered, this, []() {
         if (!ICore::isNewItemDialogRunning()) {
             ICore::showNewItemDialog(tr("New File or Project", "Title of dialog"),
-                                     IWizardFactory::allWizardFactories(), QString());
+                                     IWizardFactory::allWizardFactories(), FilePath());
         } else {
             ICore::raiseWindow(ICore::newItemDialog());
         }
     });
 
+    if (isQDS) {
+        auto action = new QAction(icon,  tr("New File..."), this);
+        cmd = ActionManager::registerAction(action, Constants::NEW_FILE);
+        mfile->addAction(cmd, Constants::G_FILE_NEW);
+        connect(action, &QAction::triggered, this, []() {
+            if (!ICore::isNewItemDialogRunning()) {
+                ICore::showNewItemDialog(
+                    tr("New File", "Title of dialog"),
+                    Utils::filtered(Core::IWizardFactory::allWizardFactories(),
+                                    Utils::equal(&Core::IWizardFactory::kind,
+                                                 Core::IWizardFactory::FileWizard)),
+                    FilePath());
+            } else {
+                ICore::raiseWindow(ICore::newItemDialog());
+            }
+        });
+    }
+
     // Open Action
     icon = QIcon::fromTheme(QLatin1String("document-open"), Utils::Icons::OPENFILE.icon());
     m_openAction = new QAction(icon, tr("&Open File or Project..."), this);
@@ -796,6 +827,14 @@ void MainWindow::registerDefaultActions()
 //    mhelp->addAction(cmd, Constants::G_HELP_ABOUT);
 //    tmpaction->setEnabled(true);
 //    connect(tmpaction, &QAction::triggered, qApp, &QApplication::aboutQt);
+
+    // Contact
+    tmpaction = new QAction(tr("Contact..."), this);
+    cmd = ActionManager::registerAction(tmpaction, "QtCreator.Contact");
+    mhelp->addAction(cmd, Constants::G_HELP_ABOUT);
+    tmpaction->setEnabled(true);
+    connect(tmpaction, &QAction::triggered, this, &MainWindow::contact);
+
     // About sep
     if (!HostOsInfo::isMacHost()) { // doesn't have the "About" actions in the Help menu
         tmpaction = new QAction(this);
@@ -843,13 +882,13 @@ void MainWindow::registerModeSelectorStyleActions()
 
 void MainWindow::openFile()
 {
-    openFiles(EditorManager::getOpenFileNames(), ICore::SwitchMode);
+    openFiles(EditorManager::getOpenFilePaths(), ICore::SwitchMode);
 }
 
 static IDocumentFactory *findDocumentFactory(const QList<IDocumentFactory*> &fileFactories,
-                                     const QFileInfo &fi)
+                                             const FilePath &filePath)
 {
-    const QString typeName = Utils::mimeTypeForFile(fi).name();
+    const QString typeName = Utils::mimeTypeForFile(filePath).name();
     return Utils::findOrDefault(fileFactories, [typeName](IDocumentFactory *f) {
         return f->mimeTypes().contains(typeName);
     });
@@ -857,7 +896,7 @@ static IDocumentFactory *findDocumentFactory(const QList<IDocumentFactory*> &fil
 
 /*!
  * \internal
- * Either opens \a fileNames with editors or loads a project.
+ * Either opens \a filePaths with editors or loads a project.
  *
  *  \a flags can be used to stop on first failure, indicate that a file name
  *  might include line numbers and/or switch mode to edit mode.
@@ -870,18 +909,24 @@ static IDocumentFactory *findDocumentFactory(const QList<IDocumentFactory*> &fil
  *
  *  \sa IPlugin::remoteArguments()
  */
-IDocument *MainWindow::openFiles(const QStringList &fileNames,
+IDocument *MainWindow::openFiles(const FilePaths &filePaths,
                                  ICore::OpenFilesFlags flags,
                                  const QString &workingDirectory)
 {
     const QList<IDocumentFactory*> documentFactories = IDocumentFactory::allDocumentFactories();
     IDocument *res = nullptr;
 
-    foreach (const QString &fileName, fileNames) {
-        const QDir workingDir(workingDirectory.isEmpty() ? QDir::currentPath() : workingDirectory);
-        const QFileInfo fi(workingDir, fileName);
-        const QString absoluteFilePath = fi.absoluteFilePath();
-        if (IDocumentFactory *documentFactory = findDocumentFactory(documentFactories, fi)) {
+    const QString workingDirBase = workingDirectory.isEmpty() ? QDir::currentPath() : workingDirectory;
+    for (const FilePath &filePath : filePaths) {
+        const FilePath workingDir = filePath.withNewPath(workingDirBase);
+        FilePath absoluteFilePath;
+        if (filePath.isAbsolutePath()) {
+            absoluteFilePath = filePath;
+        } else {
+            QTC_CHECK(!filePath.needsDevice());
+            absoluteFilePath = FilePath::fromString(workingDirBase).resolvePath(filePath.path());
+        }
+        if (IDocumentFactory *documentFactory = findDocumentFactory(documentFactories, filePath)) {
             IDocument *document = documentFactory->open(absoluteFilePath);
             if (!document) {
                 if (flags & ICore::StopOnLoadFail)
@@ -892,19 +937,28 @@ IDocument *MainWindow::openFiles(const QStringList &fileNames,
                 if (flags & ICore::SwitchMode)
                     ModeManager::activateMode(Id(Constants::MODE_EDIT));
             }
-        } else {
+        } else if (flags & (ICore::SwitchSplitIfAlreadyVisible | ICore::CanContainLineAndColumnNumbers)
+                   || !res) {
             QFlags<EditorManager::OpenEditorFlag> emFlags;
-            if (flags & ICore::CanContainLineAndColumnNumbers)
-                emFlags |=  EditorManager::CanContainLineAndColumnNumber;
             if (flags & ICore::SwitchSplitIfAlreadyVisible)
                 emFlags |= EditorManager::SwitchSplitIfAlreadyVisible;
-            IEditor *editor = EditorManager::openEditor(absoluteFilePath, Id(), emFlags);
+            IEditor *editor = nullptr;
+            if (flags & ICore::CanContainLineAndColumnNumbers) {
+                const Link &link = Link::fromFilePath(absoluteFilePath, true);
+                editor = EditorManager::openEditorAt(link, {}, emFlags);
+            } else {
+                editor = EditorManager::openEditor(absoluteFilePath, {}, emFlags);
+            }
             if (!editor) {
                 if (flags & ICore::StopOnLoadFail)
                     return res;
             } else if (!res) {
                 res = editor->document();
             }
+        } else {
+            auto factory = IEditorFactory::preferredEditorFactories(absoluteFilePath).value(0);
+            DocumentModelPrivate::addSuspendedDocument(absoluteFilePath, {},
+                                                       factory ? factory->id() : Id());
         }
     }
     return res;
@@ -915,6 +969,20 @@ void MainWindow::setFocusToEditor()
     EditorManagerPrivate::doEscapeKeyFocusMoveMagic();
 }
 
+static void acceptModalDialogs()
+{
+    const QWidgetList topLevels = QApplication::topLevelWidgets();
+    QList<QDialog *> dialogsToClose;
+    for (QWidget *topLevel : topLevels) {
+        if (auto dialog = qobject_cast<QDialog *>(topLevel)) {
+            if (dialog->isModal())
+                dialogsToClose.append(dialog);
+        }
+    }
+    for (QDialog *dialog : dialogsToClose)
+        dialog->accept();
+}
+
 void MainWindow::exit()
 {
     // this function is most likely called from a user action
@@ -922,20 +990,29 @@ void MainWindow::exit()
     // since on close we are going to delete everything
     // so to prevent the deleting of that object we
     // just append it
-    QTimer::singleShot(0, this,  &QWidget::close);
+    QMetaObject::invokeMethod(
+        this,
+        [this] {
+            // Modal dialogs block the close event. So close them, in case this was triggered from
+            // a RestartDialog in the settings dialog.
+            acceptModalDialogs();
+            close();
+        },
+        Qt::QueuedConnection);
 }
 
 void MainWindow::openFileWith()
 {
-    foreach (const QString &fileName, EditorManager::getOpenFileNames()) {
+    const FilePaths filePaths = EditorManager::getOpenFilePaths();
+    for (const FilePath &filePath : filePaths) {
         bool isExternal;
-        const Id editorId = EditorManagerPrivate::getOpenWithEditorId(fileName, &isExternal);
+        const Id editorId = EditorManagerPrivate::getOpenWithEditorId(filePath, &isExternal);
         if (!editorId.isValid())
             continue;
         if (isExternal)
-            EditorManager::openExternalEditor(fileName, editorId);
+            EditorManager::openExternalEditor(filePath, editorId);
         else
-            EditorManagerPrivate::openEditorWith(fileName, editorId);
+            EditorManagerPrivate::openEditorWith(filePath, editorId);
     }
 }
 
@@ -1029,6 +1106,8 @@ static const char windowGeometryKey[] = "WindowGeometry";
 static const char windowStateKey[] = "WindowState";
 static const char modeSelectorLayoutKey[] = "ModeSelectorLayout";
 
+static const bool askBeforeExitDefault = false;
+
 void MainWindow::readSettings()
 {
     QSettings *settings = PluginManager::settings();
@@ -1043,7 +1122,7 @@ void MainWindow::readSettings()
                                   QColor(StyleHelper::DEFAULT_BASE_COLOR)).value<QColor>());
     }
 
-    m_askConfirmationBeforeExit = settings->value(askBeforeExitKey, false).toBool();
+    m_askConfirmationBeforeExit = settings->value(askBeforeExitKey, askBeforeExitDefault).toBool();
 
     {
         ModeManager::Style modeStyle =
@@ -1070,13 +1149,17 @@ void MainWindow::readSettings()
 
 void MainWindow::saveSettings()
 {
-    QSettings *settings = PluginManager::settings();
+    QtcSettings *settings = PluginManager::settings();
     settings->beginGroup(QLatin1String(settingsGroup));
 
     if (!(m_overrideColor.isValid() && StyleHelper::baseColor() == m_overrideColor))
-        settings->setValue(QLatin1String(colorKey), StyleHelper::requestedBaseColor());
+        settings->setValueWithDefault(colorKey,
+                                      StyleHelper::requestedBaseColor(),
+                                      QColor(StyleHelper::DEFAULT_BASE_COLOR));
 
-    settings->setValue(askBeforeExitKey, m_askConfirmationBeforeExit);
+    settings->setValueWithDefault(askBeforeExitKey,
+                                  m_askConfirmationBeforeExit,
+                                  askBeforeExitDefault);
 
     settings->endGroup();
 
@@ -1175,8 +1258,7 @@ void MainWindow::aboutToShowRecentFiles()
     for (int i = 0; i < recentFiles.count(); ++i) {
         const DocumentManager::RecentFile file = recentFiles[i];
 
-        const QString filePath
-                = Utils::quoteAmpersands(QDir::toNativeSeparators(withTildeHomePath(file.first)));
+        const QString filePath = Utils::quoteAmpersands(file.first.shortNativePath());
         const QString actionText = ActionManager::withNumberAccelerator(filePath, i + 1);
         QAction *action = menu->addAction(actionText);
         connect(action, &QAction::triggered, this, [file] {
@@ -1224,6 +1306,33 @@ void MainWindow::aboutPlugins()
     dialog.exec();
 }
 
+void MainWindow::contact()
+{
+    QMessageBox dlg(QMessageBox::Information, tr("Contact"),
+           tr("<p>Qt Creator developers can be reached at the Qt Creator mailing list:</p>"
+              "%1"
+              "<p>or the #qt-creator channel on Libera.Chat IRC:</p>"
+              "%2"
+              "<p>Our bug tracker is located at %3.</p>"
+              "<p>Please use %4 for bigger chunks of text.</p>")
+                    .arg("<p>&nbsp;&nbsp;&nbsp;&nbsp;"
+                            "<a href=\"https://lists.qt-project.org/listinfo/qt-creator\">"
+                            "mailto:qt-creator@qt-project.org"
+                         "</a></p>")
+                    .arg("<p>&nbsp;&nbsp;&nbsp;&nbsp;"
+                            "<a href=\"https://web.libera.chat/#qt-creator\">"
+                            "https://web.libera.chat/#qt-creator"
+                         "</a></p>")
+                    .arg("<a href=\"https://bugreports.qt.io/projects/QTCREATORBUG\">"
+                            "https://bugreports.qt.io"
+                         "</a>")
+                    .arg("<a href=\"https://pastebin.com\">"
+                            "https://pastebin.com"
+                         "</a>"),
+           QMessageBox::Ok, this);
+    dlg.exec();
+}
+
 QPrinter *MainWindow::printer() const
 {
     if (!m_printer)
