@@ -89,9 +89,9 @@ public:
     void addContextObject(IContext *context);
     void removeContextObject(IContext *context);
 
-    IDocument *openFiles(const QStringList &fileNames,
-                         ICore::OpenFilesFlags flags,
-                         const QString &workingDirectory = QString());
+    static IDocument *openFiles(const Utils::FilePaths &filePaths,
+                                ICore::OpenFilesFlags flags,
+                                const QString &workingDirectory = QString());
 
     inline SettingsDatabase *settingsDatabase() const { return m_settingsDatabase; }
     virtual QPrinter *printer() const;
@@ -117,18 +117,19 @@ public:
     void restart();
 
 public slots:
-    void openFileWith();
+    static void openFileWith();
     void exit();
 
 protected:
     void closeEvent(QCloseEvent *event) override;
 
 private:
-    void openFile();
+    static void openFile();
     void aboutToShowRecentFiles();
-    void setFocusToEditor();
+    static void setFocusToEditor();
     void aboutQtCreator();
     void aboutPlugins();
+    void contact();
     void updateFocusWidget(QWidget *old, QWidget *now);
     NavigationWidget *navigationWidget(Side side) const;
     void setSidebarVisible(bool visible, Side side);
