@@ -37,6 +37,7 @@
 #include <utils/qtcassert.h>
 
 #include <QApplication>
+#include <QCheckBox>
 #include <QComboBox>
 #include <QDockWidget>
 #include <QFormLayout>
@@ -47,6 +48,7 @@
 #include <QPainter>
 #include <QPainterPath>
 #include <QPixmap>
+#include <QSpinBox>
 #include <QStatusBar>
 #include <QStyleFactory>
 #include <QStyleOption>
@@ -71,14 +73,23 @@ bool styleEnabled(const QWidget *widget)
     return true;
 }
 
+static bool isInUnstyledDialogOrPopup(const QWidget *widget)
+{
+    // Do not style contents of dialogs or popups without "panelwidget" property
+    const QWidget *window = widget->window();
+    if (window->property("panelwidget").toBool())
+        return false;
+    const Qt::WindowType windowType = window->windowType();
+    return (windowType == Qt::Dialog || windowType == Qt::Popup);
+}
+
 // Consider making this a QStyle state
 bool panelWidget(const QWidget *widget)
 {
     if (!widget)
         return false;
 
-    // Do not style dialogs or explicitly ignored widgets
-    if ((widget->window()->windowFlags() & Qt::WindowType_Mask) == Qt::Dialog)
+    if (isInUnstyledDialogOrPopup(widget))
         return false;
 
     if (qobject_cast<const FancyMainWindow *>(widget))
@@ -105,8 +116,7 @@ bool lightColored(const QWidget *widget)
     if (!widget)
         return false;
 
-    // Don't style dialogs or explicitly ignored widgets
-    if ((widget->window()->windowFlags() & Qt::WindowType_Mask) == Qt::Dialog)
+    if (isInUnstyledDialogOrPopup(widget))
         return false;
 
     const QWidget *p = widget;
@@ -282,15 +292,18 @@ void ManhattanStyle::polish(QWidget *widget)
             widget->setContentsMargins(0, 0, 0, 0);
 
         widget->setAttribute(Qt::WA_LayoutUsesWidgetRect, true);
+        // So that text isn't cutoff in line-edits, comboboxes... etc.
+        const int height = qMax(StyleHelper::navigationWidgetHeight(), QApplication::fontMetrics().height());
         if (qobject_cast<QToolButton*>(widget) || qobject_cast<QLineEdit*>(widget)) {
             widget->setAttribute(Qt::WA_Hover);
-            widget->setMaximumHeight(StyleHelper::navigationWidgetHeight() - 2);
-        } else if (qobject_cast<QLabel*>(widget)) {
+            widget->setMaximumHeight(height - 2);
+        } else if (qobject_cast<QLabel*>(widget) || qobject_cast<QSpinBox*>(widget)
+                   || qobject_cast<QCheckBox*>(widget)) {
             widget->setPalette(panelPalette(widget->palette(), lightColored(widget)));
         } else if (widget->property("panelwidget_singlerow").toBool()) {
-            widget->setFixedHeight(StyleHelper::navigationWidgetHeight());
+            widget->setFixedHeight(height);
         } else if (qobject_cast<QStatusBar*>(widget)) {
-            widget->setFixedHeight(StyleHelper::navigationWidgetHeight() + 2);
+            widget->setFixedHeight(height + 2);
         } else if (qobject_cast<QComboBox*>(widget)) {
             const bool isLightColored = lightColored(widget);
             QPalette palette = panelPalette(widget->palette(), isLightColored);
@@ -298,7 +311,7 @@ void ManhattanStyle::polish(QWidget *widget)
                 palette.setBrush(QPalette::All, QPalette::WindowText,
                                  creatorTheme()->color(Theme::ComboBoxTextColor));
             widget->setPalette(palette);
-            widget->setMaximumHeight(StyleHelper::navigationWidgetHeight() - 2);
+            widget->setMaximumHeight(height - 2);
             widget->setAttribute(Qt::WA_Hover);
         }
     }
@@ -490,7 +503,7 @@ static void drawPrimitiveTweakedForDarkTheme(QStyle::PrimitiveElement element,
             QPen outline(indicatorColor, 1, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin);
             painter->setPen(outline);
             QColor fill(frameColor);
-            fill.setAlphaF(0.8);
+            fill.setAlphaF(0.8f);
             painter->setBrush(fill);
             const double o = 3.5;
             painter->drawRect(QRectF(option->rect).adjusted(o, o, -o, -o));
@@ -673,9 +686,9 @@ void ManhattanStyle::drawPrimitive(PrimitiveElement element, const QStyleOption
                 }
                 if (option->state & State_HasFocus && (option->state & State_KeyboardFocusChange)) {
                     QColor highlight = option->palette.highlight().color();
-                    highlight.setAlphaF(0.4);
+                    highlight.setAlphaF(0.4f);
                     painter->setPen(QPen(highlight.lighter(), 1));
-                    highlight.setAlphaF(0.3);
+                    highlight.setAlphaF(0.3f);
                     painter->setBrush(highlight);
                     painter->setRenderHint(QPainter::Antialiasing);
                     const QRectF rect = option->rect;
@@ -741,10 +754,10 @@ void ManhattanStyle::drawPrimitive(PrimitiveElement element, const QStyleOption
 
             painter->setPen(Qt::NoPen);
             QColor dark = StyleHelper::borderColor();
-            dark.setAlphaF(0.4);
+            dark.setAlphaF(0.4f);
 
             QColor light = StyleHelper::baseColor();
-            light.setAlphaF(0.4);
+            light.setAlphaF(0.4f);
 
             painter->fillPath(path, light);
             painter->save();
@@ -906,7 +919,7 @@ void ManhattanStyle::drawControl(ControlElement element, const QStyleOption *opt
     case CE_SizeGrip: {
             painter->save();
             QColor dark = Qt::white;
-            dark.setAlphaF(0.1);
+            dark.setAlphaF(0.1f);
             int x, y, w, h;
             option->rect.getRect(&x, &y, &w, &h);
             int sw = qMin(h, w);
@@ -1174,7 +1187,7 @@ void ManhattanStyle::drawComplexControl(ComplexControl control, const QStyleOpti
     }
 }
 
-void ManhattanStyle::drawButtonSeparator(QPainter *painter, const QRect &rect, bool reverse) const
+void ManhattanStyle::drawButtonSeparator(QPainter *painter, const QRect &rect, bool reverse)
 {
     const QRectF borderRect = QRectF(rect).adjusted(0.5, 0.5, -0.5, -0.5);
     if (creatorTheme()->flag(Theme::FlatToolBars)) {
