@@ -27,9 +27,7 @@
 #include "outputwindow.h"
 #include "icontext.h"
 #include "coreconstants.h"
-#include "find/basetextfind.h"
 
-#include <aggregation/aggregate.h>
 #include <coreplugin/icore.h>
 #include <utils/utilsicons.h>
 
@@ -45,14 +43,6 @@ MessageOutputWindow::MessageOutputWindow()
 {
     m_widget = new OutputWindow(Context(Constants::C_GENERAL_OUTPUT_PANE), zoomSettingsKey);
     m_widget->setReadOnly(true);
-    // Let selected text be colored as if the text edit was editable,
-    // otherwise the highlight for searching is too light
-    QPalette p = m_widget->palette();
-    QColor activeHighlight = p.color(QPalette::Active, QPalette::Highlight);
-    p.setColor(QPalette::Highlight, activeHighlight);
-    QColor activeHighlightedText = p.color(QPalette::Active, QPalette::HighlightedText);
-    p.setColor(QPalette::HighlightedText, activeHighlightedText);
-    m_widget->setPalette(p);
 
     connect(this, &IOutputPane::zoomInRequested, m_widget, &Core::OutputWindow::zoomIn);
     connect(this, &IOutputPane::zoomOutRequested, m_widget, &Core::OutputWindow::zoomOut);
@@ -60,10 +50,6 @@ MessageOutputWindow::MessageOutputWindow()
     connect(this, &IOutputPane::fontChanged, m_widget, &OutputWindow::setBaseFont);
     connect(this, &IOutputPane::wheelZoomEnabledChanged, m_widget, &OutputWindow::setWheelZoomEnabled);
 
-    auto agg = new Aggregation::Aggregate;
-    agg->add(m_widget);
-    agg->add(new BaseTextFind(m_widget));
-
     setupFilterUi("MessageOutputPane.Filter");
     setFilteringEnabled(true);
     setupContext(Constants::C_GENERAL_OUTPUT_PANE, m_widget);
@@ -107,7 +93,7 @@ QString MessageOutputWindow::displayName() const
 
 void MessageOutputWindow::append(const QString &text)
 {
-    m_widget->appendMessage(text, Utils::DebugFormat);
+    m_widget->appendMessage(text, Utils::GeneralMessageFormat);
 }
 
 int MessageOutputWindow::priorityInStatusBar() const
