@@ -26,11 +26,13 @@
 #include "outputwindow.h"
 
 #include "actionmanager/actionmanager.h"
-#include "editormanager/editormanager.h"
 #include "coreconstants.h"
 #include "coreplugin.h"
+#include "editormanager/editormanager.h"
+#include "find/basetextfind.h"
 #include "icore.h"
 
+#include <aggregation/aggregate.h>
 #include <utils/outputformatter.h>
 #include <utils/qtcassert.h>
 
@@ -145,17 +147,20 @@ OutputWindow::OutputWindow(Context context, const QString &settingsKey, QWidget
     connect(this, &QPlainTextEdit::copyAvailable, copyAction, &QAction::setEnabled);
     connect(Core::ICore::instance(), &Core::ICore::saveSettingsRequested, this, [this] {
         if (!d->settingsKey.isEmpty())
-            Core::ICore::settings()->setValue(d->settingsKey, fontZoom());
+            Core::ICore::settings()->setValueWithDefault(d->settingsKey, fontZoom(), 0.f);
     });
 
-    connect(outputFormatter(), &OutputFormatter::openInEditorRequested, this,
-            [](const Utils::FilePath &fp, int line, int column) {
-        EditorManager::openEditorAt(fp.toString(), line, column);
+    connect(outputFormatter(), &OutputFormatter::openInEditorRequested, this, [](const Link &link) {
+        EditorManager::openEditorAt(link);
     });
 
-    connect(verticalScrollBar(), &QAbstractSlider::sliderMoved,
+    connect(verticalScrollBar(), &QAbstractSlider::actionTriggered,
             this, &OutputWindow::updateAutoScroll);
 
+    // For when "Find" changes the position; see QTCREATORBUG-26100.
+    connect(this, &QPlainTextEdit::selectionChanged, this, &OutputWindow::updateAutoScroll,
+            Qt::QueuedConnection);
+
     undoAction->setEnabled(false);
     redoAction->setEnabled(false);
     cutAction->setEnabled(false);
@@ -173,6 +178,19 @@ OutputWindow::OutputWindow(Context context, const QString &settingsKey, QWidget
         float zoom = Core::ICore::settings()->value(d->settingsKey).toFloat();
         setFontZoom(zoom);
     }
+
+    // Let selected text be colored as if the text edit was editable,
+    // otherwise the highlight for searching is too light
+    QPalette p = palette();
+    QColor activeHighlight = p.color(QPalette::Active, QPalette::Highlight);
+    p.setColor(QPalette::Highlight, activeHighlight);
+    QColor activeHighlightedText = p.color(QPalette::Active, QPalette::HighlightedText);
+    p.setColor(QPalette::HighlightedText, activeHighlightedText);
+    setPalette(p);
+
+    auto agg = new Aggregation::Aggregate;
+    agg->add(this);
+    agg->add(new BaseTextFind(this));
 }
 
 OutputWindow::~OutputWindow()
@@ -253,7 +271,7 @@ void OutputWindow::showEvent(QShowEvent *e)
 {
     QPlainTextEdit::showEvent(e);
     if (d->scrollToBottom)
-        verticalScrollBar()->setValue(verticalScrollBar()->maximum());
+        scrollToBottom();
 }
 
 void OutputWindow::wheelEvent(QWheelEvent *e)
@@ -449,7 +467,7 @@ void OutputWindow::handleOutputChunk(const QString &output, OutputFormat format)
 
 void OutputWindow::updateAutoScroll()
 {
-    d->scrollToBottom = isScrollbarAtBottom();
+    d->scrollToBottom = verticalScrollBar()->sliderPosition() >= verticalScrollBar()->maximum() - 1;
 }
 
 void OutputWindow::setMaxCharCount(int count)
@@ -473,11 +491,6 @@ void OutputWindow::appendMessage(const QString &output, OutputFormat format)
         d->queueTimer.start();
 }
 
-bool OutputWindow::isScrollbarAtBottom() const
-{
-    return verticalScrollBar()->value() == verticalScrollBar()->maximum();
-}
-
 QMimeData *OutputWindow::createMimeDataFromSelection() const
 {
     const auto mimeData = new QMimeData;
@@ -520,7 +533,7 @@ void OutputWindow::flush()
         return;
     }
     d->queueTimer.stop();
-    for (const auto &chunk : d->queuedOutput)
+    for (const auto &chunk : qAsConst(d->queuedOutput))
         handleOutputChunk(chunk.first, chunk.second);
     d->queuedOutput.clear();
     d->formatter.flush();
@@ -531,6 +544,7 @@ void OutputWindow::reset()
     flush();
     d->queueTimer.stop();
     d->formatter.reset();
+    d->scrollToBottom = true;
     if (!d->queuedOutput.isEmpty()) {
         d->queuedOutput.clear();
         d->formatter.appendMessage(tr("[Discarding excessive amount of pending output.]\n"),
