@@ -79,7 +79,7 @@ public:
         QString topLevel;
     };
 
-    Utils::optional<VcsInfo> findInCache(const QString &dir)
+    Utils::optional<VcsInfo> findInCache(const QString &dir) const
     {
         QTC_ASSERT(QDir(dir).isAbsolute(), return Utils::nullopt);
         QTC_ASSERT(!dir.endsWith(QLatin1Char('/')), return Utils::nullopt);
@@ -173,8 +173,11 @@ void VcsManager::extensionsInitialized()
 {
     // Change signal connections
     foreach (IVersionControl *versionControl, versionControls()) {
-        connect(versionControl, &IVersionControl::filesChanged,
-                DocumentManager::instance(), &DocumentManager::filesChangedInternally);
+        connect(versionControl, &IVersionControl::filesChanged, DocumentManager::instance(),
+                [](const QStringList fileNames) {
+                    DocumentManager::notifyFilesChangedInternally(
+                        Utils::transform(fileNames, &Utils::FilePath::fromString));
+                });
         connect(versionControl, &IVersionControl::repositoryChanged,
                 m_instance, &VcsManager::repositoryChanged);
         connect(versionControl, &IVersionControl::configurationChanged,
@@ -200,17 +203,17 @@ static QString absoluteWithNoTrailingSlash(const QString &directory)
     return res;
 }
 
-void VcsManager::resetVersionControlForDirectory(const QString &inputDirectory)
+void VcsManager::resetVersionControlForDirectory(const FilePath &inputDirectory)
 {
     if (inputDirectory.isEmpty())
         return;
 
-    const QString directory = absoluteWithNoTrailingSlash(inputDirectory);
+    const QString directory = absoluteWithNoTrailingSlash(inputDirectory.toString());
     d->resetCache(directory);
-    emit m_instance->repositoryChanged(directory);
+    emit m_instance->repositoryChanged(FilePath::fromString(directory));
 }
 
-IVersionControl* VcsManager::findVersionControlForDirectory(const QString &inputDirectory,
+IVersionControl* VcsManager::findVersionControlForDirectory(const FilePath &inputDirectory,
                                                             QString *topLevelDirectory)
 {
     using StringVersionControlPair = QPair<QString, IVersionControl *>;
@@ -222,7 +225,7 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
     }
 
     // Make sure we an absolute path:
-    QString directory = absoluteWithNoTrailingSlash(inputDirectory);
+    QString directory = absoluteWithNoTrailingSlash(inputDirectory.toString());
 #ifdef WITH_TESTS
     if (directory[0].isLetter() && directory.indexOf(QLatin1Char(':') + QLatin1String(TEST_PREFIX)) == 1)
         directory = directory.mid(2);
@@ -238,9 +241,9 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
     StringVersionControlPairs allThatCanManage;
 
     foreach (IVersionControl * versionControl, versionControls()) {
-        QString topLevel;
-        if (versionControl->managesDirectory(directory, &topLevel))
-            allThatCanManage.push_back(StringVersionControlPair(topLevel, versionControl));
+        FilePath topLevel;
+        if (versionControl->managesDirectory(FilePath::fromString(directory), &topLevel))
+            allThatCanManage.push_back(StringVersionControlPair(topLevel.toString(), versionControl));
     }
 
     // To properly find a nested repository (say, git checkout inside SVN),
@@ -318,11 +321,11 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
     return versionControl;
 }
 
-QString VcsManager::findTopLevelForDirectory(const QString &directory)
+FilePath VcsManager::findTopLevelForDirectory(const FilePath &directory)
 {
     QString result;
     findVersionControlForDirectory(directory, &result);
-    return result;
+    return FilePath::fromString(result);
 }
 
 QStringList VcsManager::repositories(const IVersionControl *vc)
@@ -345,15 +348,13 @@ FilePaths VcsManager::promptToDelete(const FilePaths &filePaths)
     // Categorize files by their parent directory, so we won't call
     // findVersionControlForDirectory() more often than necessary.
     QMap<FilePath, FilePaths> filesByParentDir;
-    for (const FilePath &fp : filePaths) {
-        filesByParentDir[FilePath::fromString(QDir::cleanPath(fp.toFileInfo().absolutePath()))]
-                .append(fp);
-    }
+    for (const FilePath &fp : filePaths)
+        filesByParentDir[fp.absolutePath()].append(fp);
 
     // Categorize by version control system.
     QHash<IVersionControl *, FilePaths> filesByVersionControl;
     for (auto it = filesByParentDir.cbegin(); it != filesByParentDir.cend(); ++it) {
-        IVersionControl * const vc = findVersionControlForDirectory(it.key().toString());
+        IVersionControl * const vc = findVersionControlForDirectory(it.key());
         if (vc)
             filesByVersionControl[vc] << it.value();
     }
@@ -376,7 +377,7 @@ FilePaths VcsManager::promptToDelete(IVersionControl *vc, const FilePaths &fileP
         return fp.toUserOutput();
     }).join("</li><li>") + "</li></ul>";
     const QString title = tr("Version Control");
-    const QString msg = tr("Remove the following files from from the version control system (%2)?"
+    const QString msg = tr("Remove the following files from the version control system (%2)?"
                            "%1Note: This might remove the local file.").arg(fileListForUi, vc->displayName());
     const QMessageBox::StandardButton button =
         QMessageBox::question(ICore::dialogParent(), title, msg, QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes);
@@ -385,7 +386,7 @@ FilePaths VcsManager::promptToDelete(IVersionControl *vc, const FilePaths &fileP
 
     FilePaths failedFiles;
     for (const FilePath &fp : filePaths) {
-        if (!vc->vcsDelete(fp.toString()))
+        if (!vc->vcsDelete(fp))
             failedFiles << fp;
     }
     return failedFiles;
@@ -430,13 +431,13 @@ QStringList VcsManager::additionalToolsPath()
     return d->m_cachedAdditionalToolsPaths;
 }
 
-void VcsManager::promptToAdd(const QString &directory, const QStringList &fileNames)
+void VcsManager::promptToAdd(const FilePath &directory, const FilePaths &filePaths)
 {
     IVersionControl *vc = findVersionControlForDirectory(directory);
     if (!vc || !vc->supportsOperation(IVersionControl::AddOperation))
         return;
 
-    const QStringList unmanagedFiles = vc->unmanagedFiles(fileNames);
+    const FilePaths unmanagedFiles = vc->unmanagedFiles(filePaths);
     if (unmanagedFiles.isEmpty())
         return;
 
@@ -444,9 +445,9 @@ void VcsManager::promptToAdd(const QString &directory, const QStringList &fileNa
                                  unmanagedFiles, vc->displayName());
     if (dlg.exec() == QDialog::Accepted) {
         QStringList notAddedToVc;
-        foreach (const QString &file, unmanagedFiles) {
-            if (!vc->vcsAdd(QDir(directory).filePath(file)))
-                notAddedToVc << file;
+        for (const FilePath &file : unmanagedFiles) {
+            if (!vc->vcsAdd(directory.resolvePath(file)))
+                notAddedToVc << file.toUserOutput();
         }
 
         if (!notAddedToVc.isEmpty()) {
@@ -457,7 +458,7 @@ void VcsManager::promptToAdd(const QString &directory, const QStringList &fileNa
     }
 }
 
-void VcsManager::emitRepositoryChanged(const QString &repository)
+void VcsManager::emitRepositoryChanged(const FilePath &repository)
 {
     emit m_instance->repositoryChanged(repository);
 }
@@ -467,7 +468,7 @@ void VcsManager::clearVersionControlCache()
     QStringList repoList = d->m_cachedMatches.keys();
     d->clearCache();
     foreach (const QString &repo, repoList)
-        emit m_instance->repositoryChanged(repo);
+        emit m_instance->repositoryChanged(FilePath::fromString(repo));
 }
 
 void VcsManager::handleConfigurationChanges()
@@ -494,16 +495,16 @@ namespace Internal {
 const char ID_VCS_A[] = "A";
 const char ID_VCS_B[] = "B";
 
-using FileHash = QHash<QString, QString>;
+using FileHash = QHash<FilePath, FilePath>;
 
 static FileHash makeHash(const QStringList &list)
 {
     FileHash result;
-    foreach (const QString &i, list) {
+    for (const QString &i : list) {
         QStringList parts = i.split(QLatin1Char(':'));
         QTC_ASSERT(parts.count() == 2, continue);
-        result.insert(QString::fromLatin1(TEST_PREFIX) + parts.at(0),
-                      QString::fromLatin1(TEST_PREFIX) + parts.at(1));
+        result.insert(FilePath::fromString(QString::fromLatin1(TEST_PREFIX) + parts.at(0)),
+                      FilePath::fromString(QString::fromLatin1(TEST_PREFIX) + parts.at(1)));
     }
     return result;
 }
@@ -605,7 +606,8 @@ void CorePlugin::testVcsManager()
             ++expectedCount;
 
         IVersionControl *vcs;
-        vcs = VcsManager::findVersionControlForDirectory(makeString(directory), &realTopLevel);
+        vcs = VcsManager::findVersionControlForDirectory(
+            FilePath::fromString(makeString(directory)), &realTopLevel);
         QCOMPARE(realTopLevel, makeString(topLevel));
         if (vcs)
             QCOMPARE(vcs->id().toString(), vcsId);
