@@ -27,7 +27,7 @@
 
 #include <utils/networkaccessmanager.h>
 
-#include <cpptools/cpptoolsconstants.h>
+#include <cppeditor/cppeditorconstants.h>
 #include <designer/designerconstants.h>
 #include <glsleditor/glsleditorconstants.h>
 #include <qmljstools/qmljstoolsconstants.h>
@@ -55,9 +55,7 @@ Protocol::Protocol()
 {
 }
 
-Protocol::~Protocol()
-{
-}
+Protocol::~Protocol() = default;
 
 bool Protocol::hasSettings() const
 {
@@ -71,7 +69,7 @@ bool Protocol::checkConfiguration(QString *)
 
 Core::IOptionsPage *Protocol::settingsPage() const
 {
-    return 0;
+    return nullptr;
 }
 
 void Protocol::list()
@@ -81,20 +79,21 @@ void Protocol::list()
 
 Protocol::ContentType Protocol::contentType(const QString &mt)
 {
-    if (mt == QLatin1String(CppTools::Constants::C_SOURCE_MIMETYPE)
-        || mt == QLatin1String(CppTools::Constants::C_HEADER_MIMETYPE)
+    if (mt == QLatin1String(CppEditor::Constants::C_SOURCE_MIMETYPE)
+        || mt == QLatin1String(CppEditor::Constants::C_HEADER_MIMETYPE)
         || mt == QLatin1String(GlslEditor::Constants::GLSL_MIMETYPE)
         || mt == QLatin1String(GlslEditor::Constants::GLSL_MIMETYPE_VERT)
         || mt == QLatin1String(GlslEditor::Constants::GLSL_MIMETYPE_FRAG)
         || mt == QLatin1String(GlslEditor::Constants::GLSL_MIMETYPE_VERT_ES)
         || mt == QLatin1String(GlslEditor::Constants::GLSL_MIMETYPE_FRAG_ES))
         return C;
-    if (mt == QLatin1String(CppTools::Constants::CPP_SOURCE_MIMETYPE)
-        || mt == QLatin1String(CppTools::Constants::CPP_HEADER_MIMETYPE)
-        || mt == QLatin1String(CppTools::Constants::OBJECTIVE_C_SOURCE_MIMETYPE)
-        || mt == QLatin1String(CppTools::Constants::OBJECTIVE_CPP_SOURCE_MIMETYPE))
+    if (mt == QLatin1String(CppEditor::Constants::CPP_SOURCE_MIMETYPE)
+        || mt == QLatin1String(CppEditor::Constants::CPP_HEADER_MIMETYPE)
+        || mt == QLatin1String(CppEditor::Constants::OBJECTIVE_C_SOURCE_MIMETYPE)
+        || mt == QLatin1String(CppEditor::Constants::OBJECTIVE_CPP_SOURCE_MIMETYPE))
         return Cpp;
     if (mt == QLatin1String(QmlJSTools::Constants::QML_MIMETYPE)
+        || mt == QLatin1String(QmlJSTools::Constants::QMLUI_MIMETYPE)
         || mt == QLatin1String(QmlJSTools::Constants::QMLPROJECT_MIMETYPE)
         || mt == QLatin1String(QmlJSTools::Constants::QBS_MIMETYPE)
         || mt == QLatin1String(QmlJSTools::Constants::JS_MIMETYPE)
@@ -160,7 +159,7 @@ bool Protocol::showConfigurationError(const Protocol *p,
         showConfig = false;
 
     if (!parent)
-        parent = Core::ICore::mainWindow();
+        parent = Core::ICore::dialogParent();
     const QString title = tr("%1 - Configuration Error").arg(p->name());
     QMessageBox mb(QMessageBox::Warning, title, message, QMessageBox::Cancel, parent);
     QPushButton *settingsButton = nullptr;
@@ -204,9 +203,7 @@ QNetworkReply *NetworkProtocol::httpPost(const QString &link, const QByteArray &
     return Utils::NetworkAccessManager::instance()->post(r, data);
 }
 
-NetworkProtocol::~NetworkProtocol()
-{
-}
+NetworkProtocol::~NetworkProtocol() = default;
 
 bool NetworkProtocol::httpStatus(QString url, QString *errorMessage, bool useHttps)
 {
@@ -223,7 +220,7 @@ bool NetworkProtocol::httpStatus(QString url, QString *errorMessage, bool useHtt
                     tr("Checking connection"),
                     tr("Connecting to %1...").arg(url),
                     QMessageBox::Cancel,
-                    Core::ICore::mainWindow());
+                    Core::ICore::dialogParent());
     connect(reply.data(), &QNetworkReply::finished, &box, &QWidget::close);
     QApplication::setOverrideCursor(Qt::WaitCursor);
     box.exec();
