@@ -43,19 +43,22 @@ CppcheckRunner::CppcheckRunner(CppcheckTool &tool) :
 {
     if (Utils::HostOsInfo::hostOs() == Utils::OsTypeLinux) {
         QProcess getConf;
-        getConf.start("getconf ARG_MAX");
+        getConf.start("getconf", {"ARG_MAX"});
         getConf.waitForFinished(2000);
         const QByteArray argMax = getConf.readAllStandardOutput().replace("\n", "");
         m_maxArgumentsLength = std::max(argMax.toInt(), m_maxArgumentsLength);
     }
 
-    connect(m_process, &QProcess::readyReadStandardOutput,
-            this, &CppcheckRunner::readOutput);
-    connect(m_process, &QProcess::readyReadStandardOutput,
-            this, &CppcheckRunner::readError);
-    connect(m_process, &QProcess::started,
+    m_process->setStdOutLineCallback([this](const QString &line) {
+        m_tool.parseOutputLine(line);
+    });
+    m_process->setStdErrLineCallback([this](const QString &line) {
+       m_tool.parseErrorLine(line);
+    });
+
+    connect(m_process, &QtcProcess::started,
             this, &CppcheckRunner::handleStarted);
-    connect(m_process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
+    connect(m_process, &QtcProcess::finished,
             this, &CppcheckRunner::handleFinished);
 
     m_queueTimer.setSingleShot(true);
@@ -71,16 +74,16 @@ CppcheckRunner::~CppcheckRunner()
     m_queueTimer.stop();
 }
 
-void CppcheckRunner::reconfigure(const QString &binary, const QString &arguments)
+void CppcheckRunner::reconfigure(const FilePath &binary, const QString &arguments)
 {
     m_binary = binary;
     m_arguments = arguments;
 }
 
-void CppcheckRunner::addToQueue(const Utils::FilePathList &files,
+void CppcheckRunner::addToQueue(const Utils::FilePaths &files,
                                 const QString &additionalArguments)
 {
-    Utils::FilePathList &existing = m_queue[additionalArguments];
+    Utils::FilePaths &existing = m_queue[additionalArguments];
     if (existing.isEmpty()) {
         existing = files;
     } else {
@@ -96,7 +99,7 @@ void CppcheckRunner::addToQueue(const Utils::FilePathList &files,
     m_queueTimer.start();
 }
 
-void CppcheckRunner::stop(const Utils::FilePathList &files)
+void CppcheckRunner::stop(const Utils::FilePaths &files)
 {
     if (!m_isRunning)
         return;
@@ -105,7 +108,7 @@ void CppcheckRunner::stop(const Utils::FilePathList &files)
         m_process->kill();
 }
 
-void CppcheckRunner::removeFromQueue(const Utils::FilePathList &files)
+void CppcheckRunner::removeFromQueue(const Utils::FilePaths &files)
 {
     if (m_queue.isEmpty())
         return;
@@ -121,15 +124,14 @@ void CppcheckRunner::removeFromQueue(const Utils::FilePathList &files)
     }
 }
 
-const Utils::FilePathList &CppcheckRunner::currentFiles() const
+const Utils::FilePaths &CppcheckRunner::currentFiles() const
 {
     return m_currentFiles;
 }
 
 QString CppcheckRunner::currentCommand() const
 {
-    return m_process->program() + ' ' +
-            m_process->arguments().join(' ');
+    return m_process->commandLine().toUserOutput();
 }
 
 void CppcheckRunner::checkQueued()
@@ -137,7 +139,7 @@ void CppcheckRunner::checkQueued()
     if (m_queue.isEmpty() || m_binary.isEmpty())
         return;
 
-    Utils::FilePathList files = m_queue.begin().value();
+    Utils::FilePaths files = m_queue.begin().value();
     QString arguments = m_arguments + ' ' + m_queue.begin().key();
     m_currentFiles.clear();
     int argumentsLength = arguments.length();
@@ -155,40 +157,10 @@ void CppcheckRunner::checkQueued()
     else
         m_queue.begin().value() = files;
 
-    m_process->setCommand(CommandLine(FilePath::fromString(m_binary), arguments, CommandLine::Raw));
+    m_process->setCommand(CommandLine(m_binary, arguments, CommandLine::Raw));
     m_process->start();
 }
 
-void CppcheckRunner::readOutput()
-{
-    if (!m_isRunning) // workaround for QTBUG-30929
-        handleStarted();
-
-    m_process->setReadChannel(QProcess::StandardOutput);
-
-    while (!m_process->atEnd() && m_process->canReadLine()) {
-        QString line = QString::fromUtf8(m_process->readLine());
-        if (line.endsWith('\n'))
-            line.chop(1);
-        m_tool.parseOutputLine(line);
-    }
-}
-
-void CppcheckRunner::readError()
-{
-    if (!m_isRunning) // workaround for QTBUG-30929
-        handleStarted();
-
-    m_process->setReadChannel(QProcess::StandardError);
-
-    while (!m_process->atEnd() && m_process->canReadLine()) {
-        QString line = QString::fromUtf8(m_process->readLine());
-        if (line.endsWith('\n'))
-            line.chop(1);
-        m_tool.parseErrorLine(line);
-    }
-}
-
 void CppcheckRunner::handleStarted()
 {
     if (m_isRunning)
@@ -198,15 +170,13 @@ void CppcheckRunner::handleStarted()
     m_tool.startParsing();
 }
 
-void CppcheckRunner::handleFinished(int)
+void CppcheckRunner::handleFinished()
 {
     if (m_process->error() != QProcess::FailedToStart) {
-        readOutput();
-        readError();
         m_tool.finishParsing();
     } else {
         const QString message = tr("Cppcheck failed to start: \"%1\".").arg(currentCommand());
-        Core::MessageManager::write(message, Core::MessageManager::Silent);
+        Core::MessageManager::writeSilently(message);
     }
     m_currentFiles.clear();
     m_process->close();
