@@ -33,7 +33,7 @@
 #include <coreplugin/progressmanager/futureprogress.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 
-#include <cpptools/cppmodelmanager.h>
+#include <cppeditor/cppmodelmanager.h>
 
 #include <utils/algorithm.h>
 #include <utils/macroexpander.h>
@@ -123,7 +123,7 @@ void CppcheckTool::updateArguments()
     m_runner->reconfigure(m_options.binary, arguments.join(' '));
 }
 
-QStringList CppcheckTool::additionalArguments(const CppTools::ProjectPart &part) const
+QStringList CppcheckTool::additionalArguments(const CppEditor::ProjectPart &part) const
 {
     QStringList result;
 
@@ -164,7 +164,8 @@ QStringList CppcheckTool::additionalArguments(const CppTools::ProjectPart &part)
         break;
     case Version::CXX98:
     case Version::CXX17:
-    case Version::CXX2a:
+    case Version::CXX20:
+    case Version::CXX2b:
         result.push_back("--language=c++");
         break;
     case Version::None:
@@ -203,19 +204,22 @@ void CppcheckTool::check(const Utils::FilePaths &files)
     if (filtered.isEmpty())
         return;
 
-    const CppTools::ProjectInfo info = CppTools::CppModelManager::instance()->projectInfo(m_project);
-    const QVector<CppTools::ProjectPart::Ptr> parts = info.projectParts();
+    const CppEditor::ProjectInfo::ConstPtr info
+            = CppEditor::CppModelManager::instance()->projectInfo(m_project);
+    if (!info)
+        return;
+    const QVector<CppEditor::ProjectPart::ConstPtr> parts = info->projectParts();
     if (parts.size() == 1) {
         QTC_ASSERT(parts.first(), return);
         addToQueue(filtered, *parts.first());
         return;
     }
 
-    std::map<CppTools::ProjectPart::Ptr, Utils::FilePaths> groups;
+    std::map<CppEditor::ProjectPart::ConstPtr, Utils::FilePaths> groups;
     for (const Utils::FilePath &file : qAsConst(filtered)) {
         const QString stringed = file.toString();
-        for (const CppTools::ProjectPart::Ptr &part : parts) {
-            using CppTools::ProjectFile;
+        for (const CppEditor::ProjectPart::ConstPtr &part : parts) {
+            using CppEditor::ProjectFile;
             QTC_ASSERT(part, continue);
             const auto match = [stringed](const ProjectFile &pFile){return pFile.path == stringed;};
             if (Utils::contains(part->files, match))
@@ -227,7 +231,7 @@ void CppcheckTool::check(const Utils::FilePaths &files)
         addToQueue(group.second, *group.first);
 }
 
-void CppcheckTool::addToQueue(const Utils::FilePaths &files, CppTools::ProjectPart &part)
+void CppcheckTool::addToQueue(const Utils::FilePaths &files, const CppEditor::ProjectPart &part)
 {
     const QString key = part.id();
     if (!m_cachedAdditionalArguments.contains(key))
@@ -245,7 +249,7 @@ void CppcheckTool::startParsing()
 {
     if (m_options.showOutput) {
         const QString message = tr("Cppcheck started: \"%1\".").arg(m_runner->currentCommand());
-        Core::MessageManager::write(message, Core::MessageManager::Silent);
+        Core::MessageManager::writeSilently(message);
     }
 
     m_progress = std::make_unique<QFutureInterface<void>>();
@@ -263,7 +267,7 @@ void CppcheckTool::parseOutputLine(const QString &line)
         return;
 
     if (m_options.showOutput)
-        Core::MessageManager::write(line, Core::MessageManager::Silent);
+        Core::MessageManager::writeSilently(line);
 
     enum Matches { Percentage = 1 };
     const QRegularExpressionMatch match = m_progressRegexp.match(line);
@@ -294,7 +298,7 @@ void CppcheckTool::parseErrorLine(const QString &line)
         return;
 
     if (m_options.showOutput)
-        Core::MessageManager::write(line, Core::MessageManager::Silent);
+        Core::MessageManager::writeSilently(line);
 
     enum Matches { File = 1, Line, Severity, Id, Message };
     const QRegularExpressionMatch match = m_messageRegexp.match(line);
@@ -319,7 +323,7 @@ void CppcheckTool::parseErrorLine(const QString &line)
 void CppcheckTool::finishParsing()
 {
     if (m_options.showOutput)
-        Core::MessageManager::write(tr("Cppcheck finished."), Core::MessageManager::Silent);
+        Core::MessageManager::writeSilently(tr("Cppcheck finished."));
 
     QTC_ASSERT(m_progress, return);
     m_progress->reportFinished();
