@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,39 +9,34 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "cppcanonicalsymbol.h"
 
-#include <cpptools/cpptoolsreuse.h>
-#include <texteditor/convenience.h>
+#include "cpptoolsreuse.h"
 
 #include <cplusplus/ExpressionUnderCursor.h>
 
+#include <utils/textutils.h>
+
 #include <QTextCursor>
 #include <QTextDocument>
 
 using namespace CPlusPlus;
 
-namespace CppEditor {
-namespace Internal {
+namespace CppEditor::Internal {
 
 CanonicalSymbol::CanonicalSymbol(const Document::Ptr &document,
                                  const Snapshot &snapshot)
@@ -60,26 +55,25 @@ const LookupContext &CanonicalSymbol::context() const
 Scope *CanonicalSymbol::getScopeAndExpression(const QTextCursor &cursor, QString *code)
 {
     if (!m_document)
-        return 0;
+        return nullptr;
 
     QTextCursor tc = cursor;
     int line, column;
-    TextEditor::Convenience::convertPosition(cursor.document(), tc.position(), &line, &column);
-    ++column; // 1-based line and 1-based column
+    Utils::Text::convertPosition(cursor.document(), tc.position(), &line, &column);
 
     int pos = tc.position();
     QTextDocument *textDocument = cursor.document();
-    if (!CppTools::isValidIdentifierChar(textDocument->characterAt(pos)))
-        if (!(pos > 0 && CppTools::isValidIdentifierChar(textDocument->characterAt(pos - 1))))
-            return 0;
+    if (!isValidIdentifierChar(textDocument->characterAt(pos)))
+        if (!(pos > 0 && isValidIdentifierChar(textDocument->characterAt(pos - 1))))
+            return nullptr;
 
-    while (CppTools::isValidIdentifierChar(textDocument->characterAt(pos)))
+    while (isValidIdentifierChar(textDocument->characterAt(pos)))
         ++pos;
     tc.setPosition(pos);
 
     ExpressionUnderCursor expressionUnderCursor(m_document->languageFeatures());
     *code = expressionUnderCursor(tc);
-    return m_document->scopeAt(line, column);
+    return m_document->scopeAt(line, column - 1);
 }
 
 Symbol *CanonicalSymbol::operator()(const QTextCursor &cursor)
@@ -89,7 +83,7 @@ Symbol *CanonicalSymbol::operator()(const QTextCursor &cursor)
     if (Scope *scope = getScopeAndExpression(cursor, &code))
         return operator()(scope, code);
 
-    return 0;
+    return nullptr;
 }
 
 Symbol *CanonicalSymbol::operator()(Scope *scope, const QString &code)
@@ -124,15 +118,12 @@ Symbol *CanonicalSymbol::canonicalSymbol(Scope *scope, const QString &code,
         }
     }
 
-    for (int i = 0; i < results.size(); ++i) {
-        const LookupItem &r = results.at(i);
-
-        if (r.declaration())
+    for (const auto &r : results) {
+         if (r.declaration())
             return r.declaration();
     }
 
-    return 0;
+    return nullptr;
 }
 
-} // namespace Internal
-} // namespace CppEditor
+} // namespace CppEditor::Internal
