@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,54 +9,48 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-
 #include "cppfollowsymbolundercursor.h"
-#include "cppeditor.h"
+
+#include "cppeditorwidget.h"
+#include "cppmodelmanager.h"
+#include "cpptoolsreuse.h"
 #include "cppvirtualfunctionassistprovider.h"
+#include "functionutils.h"
+#include "symbolfinder.h"
 
 #include <cplusplus/ASTPath.h>
 #include <cplusplus/BackwardsScanner.h>
 #include <cplusplus/ExpressionUnderCursor.h>
 #include <cplusplus/ResolveExpression.h>
 #include <cplusplus/SimpleLexer.h>
-#include <cplusplus/Templates.h>
 #include <cplusplus/TypeOfExpression.h>
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/functionutils.h>
-#include <cpptools/cpptoolsreuse.h>
-#include <cpptools/symbolfinder.h>
 #include <texteditor/textdocumentlayout.h>
+#include <utils/textutils.h>
 #include <utils/qtcassert.h>
 
 #include <QList>
 #include <QSet>
 
 using namespace CPlusPlus;
-using namespace CppTools;
-using namespace CppEditor;
-using namespace CppEditor::Internal;
 using namespace TextEditor;
 
-typedef TextEditorWidget::Link Link;
+using Link = Utils::Link;
+
+namespace CppEditor {
 
 namespace {
 
@@ -67,6 +61,7 @@ public:
                           const Document::Ptr &document,
                           const Snapshot &snapshot,
                           SymbolFinder *symbolFinder);
+    VirtualFunctionHelper() = delete;
 
     bool canLookupVirtualFunctionOverrides(Function *function);
 
@@ -75,7 +70,6 @@ public:
     { return m_staticClassOfFunctionCallExpression; }
 
 private:
-    VirtualFunctionHelper();
     Q_DISABLE_COPY(VirtualFunctionHelper)
 
     Class *staticClassOfFunctionCallExpression_internal() const;
@@ -90,10 +84,10 @@ private:
     SymbolFinder *m_finder;
 
     // Determined
-    ExpressionAST *m_baseExpressionAST;
-    Function *m_function;
-    int m_accessTokenKind;
-    Class *m_staticClassOfFunctionCallExpression; // Output
+    ExpressionAST *m_baseExpressionAST = nullptr;
+    Function *m_function = nullptr;
+    int m_accessTokenKind = 0;
+    Class *m_staticClassOfFunctionCallExpression = nullptr; // Output
 };
 
 VirtualFunctionHelper::VirtualFunctionHelper(TypeOfExpression &typeOfExpression,
@@ -107,10 +101,6 @@ VirtualFunctionHelper::VirtualFunctionHelper(TypeOfExpression &typeOfExpression,
     , m_snapshot(snapshot)
     , m_typeOfExpression(typeOfExpression)
     , m_finder(finder)
-    , m_baseExpressionAST(0)
-    , m_function(0)
-    , m_accessTokenKind(0)
-    , m_staticClassOfFunctionCallExpression(0)
 {
     if (ExpressionAST *expressionAST = typeOfExpression.expressionAST()) {
         if (CallAST *callAST = expressionAST->asCall()) {
@@ -123,8 +113,17 @@ VirtualFunctionHelper::VirtualFunctionHelper(TypeOfExpression &typeOfExpression,
 bool VirtualFunctionHelper::canLookupVirtualFunctionOverrides(Function *function)
 {
     m_function = function;
-    if (!m_function || !m_baseExpressionAST || !m_expressionDocument || !m_document || !m_scope
-            || m_scope->isClass() || m_scope->isFunction() || m_snapshot.isEmpty()) {
+
+    if (!m_document || m_snapshot.isEmpty() || !m_function || !m_scope)
+        return false;
+
+    if (m_scope->isClass() && m_function->isPureVirtual()) {
+        m_staticClassOfFunctionCallExpression = m_scope->asClass();
+        return true;
+    }
+
+    if (!m_baseExpressionAST || !m_expressionDocument
+            || m_scope->isClass() || m_scope->isFunction()) {
         return false;
     }
 
@@ -133,12 +132,12 @@ bool VirtualFunctionHelper::canLookupVirtualFunctionOverrides(Function *function
     if (IdExpressionAST *idExpressionAST = m_baseExpressionAST->asIdExpression()) {
         NameAST *name = idExpressionAST->name;
         const bool nameIsQualified = name && name->asQualifiedName();
-        result = !nameIsQualified && FunctionUtils::isVirtualFunction(
+        result = !nameIsQualified && Internal::FunctionUtils::isVirtualFunction(
                     function, LookupContext(m_document, m_snapshot));
     } else if (MemberAccessAST *memberAccessAST = m_baseExpressionAST->asMemberAccess()) {
         NameAST *name = memberAccessAST->member_name;
         const bool nameIsQualified = name && name->asQualifiedName();
-        if (!nameIsQualified && FunctionUtils::isVirtualFunction(
+        if (!nameIsQualified && Internal::FunctionUtils::isVirtualFunction(
                     function, LookupContext(m_document, m_snapshot))) {
             TranslationUnit *unit = m_expressionDocument->translationUnit();
             QTC_ASSERT(unit, return false);
@@ -168,9 +167,9 @@ bool VirtualFunctionHelper::canLookupVirtualFunctionOverrides(Function *function
 Class *VirtualFunctionHelper::staticClassOfFunctionCallExpression_internal() const
 {
     if (!m_finder)
-        return 0;
+        return nullptr;
 
-    Class *result = 0;
+    Class *result = nullptr;
 
     if (m_baseExpressionAST->asIdExpression()) {
         for (Scope *s = m_scope; s ; s = s->enclosingScope()) {
@@ -184,7 +183,7 @@ Class *VirtualFunctionHelper::staticClassOfFunctionCallExpression_internal() con
         const QList<LookupItem> items = m_typeOfExpression(memberAccessAST->base_expression,
                                                            m_expressionDocument, m_scope);
         ResolveExpression resolveExpression(m_typeOfExpression.context());
-        LookupScope *binding = resolveExpression.baseExpression(items, m_accessTokenKind);
+        ClassOrNamespace *binding = resolveExpression.baseExpression(items, m_accessTokenKind);
         if (binding) {
             if (Class *klass = binding->rootClass()) {
                 result = klass;
@@ -211,7 +210,7 @@ Link findMacroLink_helper(const QByteArray &name, Document::Ptr doc, const Snaps
         foreach (const Macro &macro, doc->definedMacros()) {
             if (macro.name() == name) {
                 Link link;
-                link.targetFileName = macro.fileName();
+                link.targetFilePath = Utils::FilePath::fromString(macro.fileName());
                 link.targetLine = macro.line();
                 return link;
             }
@@ -304,9 +303,9 @@ inline LookupItem skipForwardDeclarations(const QList<LookupItem> &resolvedSymbo
     return result;
 }
 
-CppEditorWidget::Link attemptFuncDeclDef(const QTextCursor &cursor,
-    CppEditorWidget *, Snapshot snapshot, const Document::Ptr &document,
-    SymbolFinder *symbolFinder)
+Link attemptDeclDef(const QTextCursor &cursor, Snapshot snapshot,
+                    const Document::Ptr &document,
+                    SymbolFinder *symbolFinder)
 {
     Link result;
     QTC_ASSERT(document, return result);
@@ -331,41 +330,42 @@ CppEditorWidget::Link attemptFuncDeclDef(const QTextCursor &cursor,
     for (int i = path.size() - 1; i != -1; --i) {
         AST *node = path.at(i);
 
-        if (node->asParameterDeclaration() != 0)
+        if (node->asParameterDeclaration() != nullptr)
             return result;
     }
 
-    AST *declParent = 0;
-    DeclaratorAST *decl = 0;
+    AST *declParent = nullptr;
+    DeclaratorAST *decl = nullptr;
     for (int i = path.size() - 2; i > 0; --i) {
-        if ((decl = path.at(i)->asDeclarator()) != 0) {
+        if ((decl = path.at(i)->asDeclarator()) != nullptr) {
             declParent = path.at(i - 1);
             break;
         }
     }
     if (!decl || !declParent)
         return result;
-    if (!decl->postfix_declarator_list || !decl->postfix_declarator_list->value)
-        return result;
-    FunctionDeclaratorAST *funcDecl = decl->postfix_declarator_list->value->asFunctionDeclarator();
-    if (!funcDecl)
-        return result;
 
-    Symbol *target = 0;
+    Symbol *target = nullptr;
     if (FunctionDefinitionAST *funDef = declParent->asFunctionDefinition()) {
         QList<Declaration *> candidates =
                 symbolFinder->findMatchingDeclaration(LookupContext(document, snapshot),
                                                         funDef->symbol);
         if (!candidates.isEmpty()) // TODO: improve disambiguation
             target = candidates.first();
-    } else if (declParent->asSimpleDeclaration()) {
-        target = symbolFinder->findMatchingDefinition(funcDecl->symbol, snapshot);
+    } else if (const SimpleDeclarationAST * const simpleDecl = declParent->asSimpleDeclaration()) {
+        FunctionDeclaratorAST *funcDecl = nullptr;
+        if (decl->postfix_declarator_list && decl->postfix_declarator_list->value)
+            funcDecl = decl->postfix_declarator_list->value->asFunctionDeclarator();
+        if (funcDecl)
+            target = symbolFinder->findMatchingDefinition(funcDecl->symbol, snapshot);
+        else if (simpleDecl->symbols)
+            target = symbolFinder->findMatchingVarDefinition(simpleDecl->symbols->value, snapshot);
     }
 
     if (target) {
-        result = CppTools::linkToSymbol(target);
+        result = target->toLink();
 
-        unsigned startLine, startColumn, endLine, endColumn;
+        int startLine, startColumn, endLine, endColumn;
         document->translationUnit()->getTokenStartPosition(name->firstToken(), &startLine,
                                                            &startColumn);
         document->translationUnit()->getTokenEndPosition(name->lastToken() - 1, &endLine,
@@ -384,10 +384,10 @@ CppEditorWidget::Link attemptFuncDeclDef(const QTextCursor &cursor,
 Symbol *findDefinition(Symbol *symbol, const Snapshot &snapshot, SymbolFinder *symbolFinder)
 {
     if (symbol->isFunction())
-        return 0; // symbol is a function definition.
+        return nullptr; // symbol is a function definition.
 
-    else if (!symbol->type()->isFunctionType())
-        return 0; // not a function declaration
+    if (!symbol->type()->isFunctionType())
+        return nullptr; // not a function declaration
 
     return symbolFinder->findMatchingDefinition(symbol, snapshot);
 }
@@ -417,17 +417,18 @@ bool maybeAppendArgumentOrParameterList(QString *expression, const QTextCursor &
 
 bool isCursorOnTrailingReturnType(const QList<AST *> &astPath)
 {
-    for (auto it = astPath.cend() - 1, begin = astPath.cbegin(); it >= begin; --it) {
+    if (astPath.size() < 3)
+        return false;
+    for (auto it = astPath.cend() - 3, begin = astPath.cbegin(); it >= begin; --it) {
+        if (!(*it)->asTrailingReturnType())
+            continue;
         const auto nextIt = it + 1;
         const auto nextNextIt = nextIt + 1;
-        if (nextNextIt != astPath.cend() && (*it)->asTrailingReturnType()) {
-            return (*nextIt)->asNamedTypeSpecifier()
-                    && ((*nextNextIt)->asSimpleName()
-                        || (*nextNextIt)->asQualifiedName()
-                        || (*nextNextIt)->asTemplateId());
-        }
+        return (*nextIt)->asNamedTypeSpecifier()
+               && ((*nextNextIt)->asSimpleName()
+                   || (*nextNextIt)->asQualifiedName()
+                   || (*nextNextIt)->asTemplateId());
     }
-
     return false;
 }
 
@@ -466,17 +467,11 @@ QString expressionUnderCursorAsString(const QTextCursor &textCursor,
 
 } // anonymous namespace
 
-FollowSymbolUnderCursor::FollowSymbolUnderCursor(CppEditorWidget *widget)
-    : m_widget(widget)
-    , m_virtualFunctionAssistProvider(new VirtualFunctionAssistProvider)
+FollowSymbolUnderCursor::FollowSymbolUnderCursor()
+    : m_virtualFunctionAssistProvider(new VirtualFunctionAssistProvider)
 {
 }
 
-FollowSymbolUnderCursor::~FollowSymbolUnderCursor()
-{
-    delete m_virtualFunctionAssistProvider;
-}
-
 static int skipMatchingParentheses(const Tokens &tokens, int idx, int initialDepth)
 {
     int j = idx;
@@ -494,19 +489,33 @@ static int skipMatchingParentheses(const Tokens &tokens, int idx, int initialDep
     return j;
 }
 
-TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &cursor,
-    bool resolveTarget, const Snapshot &theSnapshot, const Document::Ptr &documentFromSemanticInfo,
-    SymbolFinder *symbolFinder, bool inNextSplit)
+void FollowSymbolUnderCursor::findLink(
+        const CursorInEditor &data,
+        Utils::ProcessLinkCallback &&processLinkCallback,
+        bool resolveTarget,
+        const Snapshot &theSnapshot,
+        const Document::Ptr &documentFromSemanticInfo,
+        SymbolFinder *symbolFinder,
+        bool inNextSplit)
 {
     Link link;
 
+    QTextCursor cursor = data.cursor();
+    QTextDocument *document = cursor.document();
+    if (!document)
+        return processLinkCallback(link);
+
+    int line = 0;
+    int column = 0;
+    Utils::Text::convertPosition(document, cursor.position(), &line, &column);
+    const int positionInBlock = column - 1;
+
     Snapshot snapshot = theSnapshot;
 
     // Move to end of identifier
     QTextCursor tc = cursor;
-    QTextDocument *document = m_widget->document();
     QChar ch = document->characterAt(tc.position());
-    while (CppTools::isValidIdentifierChar(ch)) {
+    while (isValidIdentifierChar(ch)) {
         tc.movePosition(QTextCursor::NextCharacter);
         ch = document->characterAt(tc.position());
     }
@@ -518,19 +527,14 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
         int pos = tc.position();
         while (document->characterAt(pos).isSpace())
             ++pos;
-        if (document->characterAt(pos) == QLatin1Char('(')) {
-            link = attemptFuncDeclDef(cursor, m_widget, snapshot, documentFromSemanticInfo,
-                                      symbolFinder);
+        const QChar ch = document->characterAt(pos);
+        if (ch == '(' || ch == ';') {
+            link = attemptDeclDef(cursor, snapshot, documentFromSemanticInfo, symbolFinder);
             if (link.hasValidLinkText())
-                return link;
+                return processLinkCallback(link);
         }
     }
 
-    int lineNumber = 0, positionInBlock = 0;
-    m_widget->convertPosition(cursor.position(), &lineNumber, &positionInBlock);
-    const unsigned line = lineNumber;
-    const unsigned column = positionInBlock + 1;
-
     // Try to find a signal or slot inside SIGNAL() or SLOT()
     int beginOfToken = 0;
     int endOfToken = 0;
@@ -549,8 +553,7 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
     for (int i = 0; i < tokens.size(); ++i) {
         const Token &tk = tokens.at(i);
 
-        if (((unsigned) positionInBlock) >= tk.utf16charsBegin()
-                && ((unsigned) positionInBlock) < tk.utf16charsEnd()) {
+        if (positionInBlock >= tk.utf16charsBegin() && positionInBlock < tk.utf16charsEnd()) {
             int closingParenthesisPos = tokens.size();
             if (i >= 2 && tokens.at(i).is(T_IDENTIFIER) && tokens.at(i - 1).is(T_LPAREN)
                 && (tokens.at(i - 2).is(T_SIGNAL) || tokens.at(i - 2).is(T_SLOT))) {
@@ -592,22 +595,20 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
 
             // In this case we want to look at one token before the current position to recognize
             // an operator if the cursor is inside the actual operator: operator[$]
-            if (unsigned(positionInBlock) >= tk.utf16charsBegin()
-                    && unsigned(positionInBlock) <= tk.utf16charsEnd()) {
+            if (positionInBlock >= tk.utf16charsBegin() && positionInBlock <= tk.utf16charsEnd()) {
                 cursorRegionReached = true;
                 if (tk.is(T_OPERATOR)) {
-                    link = attemptFuncDeclDef(cursor, m_widget, theSnapshot,
-                                              documentFromSemanticInfo, symbolFinder);
+                    link = attemptDeclDef(cursor, theSnapshot,
+                                          documentFromSemanticInfo, symbolFinder);
                     if (link.hasValidLinkText())
-                        return link;
-                } else if (tk.isOperator() && i > 0 && tokens.at(i - 1).is(T_OPERATOR)) {
+                        return processLinkCallback(link);
+                } else if (tk.isPunctuationOrOperator() && i > 0 && tokens.at(i - 1).is(T_OPERATOR)) {
                     QTextCursor c = cursor;
                     c.movePosition(QTextCursor::Left, QTextCursor::MoveAnchor,
                                    positionInBlock - tokens.at(i - 1).utf16charsBegin());
-                    link = attemptFuncDeclDef(c, m_widget, theSnapshot, documentFromSemanticInfo,
-                                              symbolFinder);
+                    link = attemptDeclDef(c, theSnapshot, documentFromSemanticInfo, symbolFinder);
                     if (link.hasValidLinkText())
-                        return link;
+                        return processLinkCallback(link);
                 }
             } else if (cursorRegionReached) {
                 break;
@@ -615,12 +616,16 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
         }
     }
 
+    CppEditorWidget *editorWidget = data.editorWidget();
+    if (!editorWidget)
+        return processLinkCallback(link);
+
     // Now we prefer the doc from the snapshot with macros expanded.
-    Document::Ptr doc = snapshot.document(m_widget->textDocument()->filePath());
+    Document::Ptr doc = snapshot.document(data.filePath());
     if (!doc) {
         doc = documentFromSemanticInfo;
         if (!doc)
-            return link;
+            return processLinkCallback(link);
     }
 
     if (!recognizedQtMethod) {
@@ -638,19 +643,20 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
 
         // Handle include directives
         if (tk.is(T_STRING_LITERAL) || tk.is(T_ANGLE_STRING_LITERAL)) {
-            const unsigned lineno = cursor.blockNumber() + 1;
+            const int lineno = cursor.blockNumber() + 1;
             foreach (const Document::Include &incl, doc->resolvedIncludes()) {
                 if (incl.line() == lineno) {
-                    link.targetFileName = incl.resolvedFileName();
+                    link.targetFilePath = Utils::FilePath::fromString(incl.resolvedFileName());
                     link.linkTextStart = beginOfToken + 1;
                     link.linkTextEnd = endOfToken - 1;
-                    return link;
+                    processLinkCallback(link);
+                    return;
                 }
             }
         }
 
         if (tk.isNot(T_IDENTIFIER) && !tk.isQtKeyword())
-            return link;
+            return processLinkCallback(link);
 
         tc.setPosition(endOfToken);
     }
@@ -659,27 +665,28 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
     const Macro *macro = doc->findMacroDefinitionAt(line);
     if (macro) {
         QTextCursor macroCursor = cursor;
-        const QByteArray name = CppTools::identifierUnderCursor(&macroCursor).toUtf8();
+        const QByteArray name = identifierUnderCursor(&macroCursor).toUtf8();
         if (macro->name() == name)
-            return link;    //already on definition!
+            return processLinkCallback(link); //already on definition!
     } else if (const Document::MacroUse *use = doc->findMacroUseAt(endOfToken - 1)) {
         const QString fileName = use->macro().fileName();
         if (fileName == CppModelManager::editorConfigurationFileName()) {
-            m_widget->showPreProcessorWidget();
+            editorWidget->showPreProcessorWidget();
         } else if (fileName != CppModelManager::configurationFileName()) {
             const Macro &macro = use->macro();
-            link.targetFileName = macro.fileName();
+            link.targetFilePath = Utils::FilePath::fromString(macro.fileName());
             link.targetLine = macro.line();
             link.linkTextStart = use->utf16charsBegin();
             link.linkTextEnd = use->utf16charsEnd();
         }
-        return link;
+        processLinkCallback(link);
+        return;
     }
 
     // Find the last symbol up to the cursor position
-    Scope *scope = doc->scopeAt(line, column);
+    Scope *scope = doc->scopeAt(line, positionInBlock);
     if (!scope)
-        return link;
+        return processLinkCallback(link);
 
     // Evaluate the type of the expression under the cursor
     QTC_CHECK(document == tc.document());
@@ -699,20 +706,21 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
             if (Symbol *d = r.declaration()) {
                 if (d->isDeclaration() || d->isFunction()) {
                     const QString fileName = QString::fromUtf8(d->fileName(), d->fileNameLength());
-                    if (m_widget->textDocument()->filePath().toString() == fileName) {
-                        if (unsigned(lineNumber) == d->line()
-                            && unsigned(positionInBlock) >= d->column()) { // TODO: check the end
+                    if (data.filePath().toString() == fileName) {
+                        if (line == d->line() && positionInBlock >= d->column()) {
+                            // TODO: check the end
                             result = r; // take the symbol under cursor.
                             break;
                         }
                     }
                 } else if (d->isUsingDeclaration()) {
-                    int tokenBeginLineNumber = 0, tokenBeginColumnNumber = 0;
-                    m_widget->convertPosition(beginOfToken, &tokenBeginLineNumber,
-                                              &tokenBeginColumnNumber);
-                    if (unsigned(tokenBeginLineNumber) > d->line()
-                            || (unsigned(tokenBeginLineNumber) == d->line()
-                                && unsigned(tokenBeginColumnNumber) > d->column())) {
+                    int tokenBeginLineNumber = 0;
+                    int tokenBeginColumnNumber = 0;
+                    Utils::Text::convertPosition(document, beginOfToken, &tokenBeginLineNumber,
+                                                 &tokenBeginColumnNumber);
+                    if (tokenBeginLineNumber > d->line()
+                            || (tokenBeginLineNumber == d->line()
+                                && tokenBeginColumnNumber >= d->column())) {
                         result = r; // take the symbol under cursor.
                         break;
                     }
@@ -721,7 +729,7 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
         }
 
         if (Symbol *symbol = result.declaration()) {
-            Symbol *def = 0;
+            Symbol *def = nullptr;
 
             if (resolveTarget) {
                 // Consider to show a pop-up displaying overrides for the function
@@ -738,7 +746,8 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
                     params.openInNextSplit = inNextSplit;
 
                     if (m_virtualFunctionAssistProvider->configure(params)) {
-                        m_widget->invokeAssist(FollowSymbol, m_virtualFunctionAssistProvider);
+                        editorWidget->invokeTextEditorWidgetAssist(
+                                    FollowSymbol,m_virtualFunctionAssistProvider.data());
                         m_virtualFunctionAssistProvider->clearParams();
                     }
 
@@ -746,7 +755,8 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
                     Link link;
                     link.linkTextStart = beginOfToken;
                     link.linkTextEnd = endOfToken;
-                    return link;
+                    processLinkCallback(link);
+                    return;
                 }
 
                 Symbol *lastVisibleSymbol = doc->lastVisibleSymbolAt(line, column);
@@ -754,7 +764,7 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
                 def = findDefinition(symbol, snapshot, symbolFinder);
 
                 if (def == lastVisibleSymbol)
-                    def = 0; // jump to declaration then.
+                    def = nullptr; // jump to declaration then.
 
                 if (symbol->isForwardClassDeclaration()) {
                     def = symbolFinder->findMatchingClassDeclaration(symbol, snapshot);
@@ -767,32 +777,114 @@ TextEditorWidget::Link FollowSymbolUnderCursor::findLink(const QTextCursor &curs
 
             }
 
-            link = CppTools::linkToSymbol(def ? def : symbol);
+            link = (def ? def : symbol)->toLink();
             link.linkTextStart = beginOfToken;
             link.linkTextEnd = endOfToken;
-            return link;
+            processLinkCallback(link);
+            return;
         }
     }
 
     // Handle macro uses
     QTextCursor macroCursor = cursor;
-    const QByteArray name = CppTools::identifierUnderCursor(&macroCursor).toUtf8();
+    const QByteArray name = identifierUnderCursor(&macroCursor).toUtf8();
     link = findMacroLink(name, documentFromSemanticInfo);
     if (link.hasValidTarget()) {
         link.linkTextStart = macroCursor.selectionStart();
         link.linkTextEnd = macroCursor.selectionEnd();
-        return link;
+        processLinkCallback(link);
+        return;
     }
 
-    return Link();
+    processLinkCallback(Link());
 }
 
-VirtualFunctionAssistProvider *FollowSymbolUnderCursor::virtualFunctionAssistProvider()
+void FollowSymbolUnderCursor::switchDeclDef(
+        const CursorInEditor &data,
+        Utils::ProcessLinkCallback &&processLinkCallback,
+        const CPlusPlus::Snapshot &snapshot,
+        const CPlusPlus::Document::Ptr &documentFromSemanticInfo,
+        SymbolFinder *symbolFinder)
+{
+    if (!documentFromSemanticInfo) {
+        processLinkCallback({});
+        return;
+    }
+
+    // Find function declaration or definition under cursor
+    Function *functionDefinitionSymbol = nullptr;
+    Symbol *functionDeclarationSymbol = nullptr;
+    Symbol *declarationSymbol = nullptr;
+
+    ASTPath astPathFinder(documentFromSemanticInfo);
+    const QList<AST *> astPath = astPathFinder(data.cursor());
+
+    for (AST *ast : astPath) {
+        if (FunctionDefinitionAST *functionDefinitionAST = ast->asFunctionDefinition()) {
+            if ((functionDefinitionSymbol = functionDefinitionAST->symbol))
+                break; // Function definition found!
+        } else if (SimpleDeclarationAST *simpleDeclaration = ast->asSimpleDeclaration()) {
+            if (List<Symbol *> *symbols = simpleDeclaration->symbols) {
+                if (Symbol *symbol = symbols->value) {
+                    if (symbol->isDeclaration()) {
+                        declarationSymbol = symbol;
+                        if (symbol->type()->isFunctionType()) {
+                            functionDeclarationSymbol = symbol;
+                            break; // Function declaration found!
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    // Link to function definition/declaration
+    Utils::Link symbolLink;
+    if (functionDeclarationSymbol) {
+        Symbol *symbol = symbolFinder->findMatchingDefinition(functionDeclarationSymbol, snapshot);
+        if (symbol)
+            symbolLink = symbol->toLink();
+    } else if (declarationSymbol) {
+        Symbol *symbol = symbolFinder->findMatchingVarDefinition(declarationSymbol, snapshot);
+        if (symbol)
+            symbolLink = symbol->toLink();
+    } else if (functionDefinitionSymbol) {
+        LookupContext context(documentFromSemanticInfo, snapshot);
+        ClassOrNamespace *binding = context.lookupType(functionDefinitionSymbol);
+        const QList<LookupItem> declarations
+            = context.lookup(functionDefinitionSymbol->name(),
+                             functionDefinitionSymbol->enclosingScope());
+
+        QList<Symbol *> best;
+        foreach (const LookupItem &r, declarations) {
+            if (Symbol *decl = r.declaration()) {
+                if (Function *funTy = decl->type()->asFunctionType()) {
+                    if (funTy->match(functionDefinitionSymbol)) {
+                        if (decl != functionDefinitionSymbol && binding == r.binding())
+                            best.prepend(decl);
+                        else
+                            best.append(decl);
+                    }
+                }
+            }
+        }
+
+        if (best.isEmpty())
+            return;
+        symbolLink = best.first()->toLink();
+    }
+    processLinkCallback(symbolLink);
+}
+
+QSharedPointer<VirtualFunctionAssistProvider> FollowSymbolUnderCursor::virtualFunctionAssistProvider()
 {
     return m_virtualFunctionAssistProvider;
 }
 
-void FollowSymbolUnderCursor::setVirtualFunctionAssistProvider(VirtualFunctionAssistProvider *provider)
+void FollowSymbolUnderCursor::setVirtualFunctionAssistProvider(
+        const QSharedPointer<VirtualFunctionAssistProvider> &provider)
 {
     m_virtualFunctionAssistProvider = provider;
 }
+
+} // namespace CppEditor
