@@ -25,15 +25,15 @@
 
 #include "cppfunctiondecldeflink.h"
 
+#include "cppcodestylesettings.h"
+#include "cppeditorconstants.h"
 #include "cppeditorwidget.h"
+#include "cpplocalsymbols.h"
 #include "cppquickfixassistant.h"
+#include "symbolfinder.h"
 
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/command.h>
-#include <cpptools/cppcodestylesettings.h>
-#include <cpptools/cpplocalsymbols.h>
-#include <cpptools/cpptoolsconstants.h>
-#include <cpptools/symbolfinder.h>
 #include <texteditor/refactoroverlay.h>
 #include <texteditor/texteditorconstants.h>
 
@@ -51,7 +51,6 @@
 #include <QVarLengthArray>
 
 using namespace CPlusPlus;
-using namespace CppTools;
 using namespace TextEditor;
 using namespace Utils;
 
@@ -172,7 +171,8 @@ static QSharedPointer<FunctionDeclDefLink> findLinkHelper(QSharedPointer<Functio
     // parse the target file to get the linked decl/def
     const QString targetFileName = QString::fromUtf8(
                 target->fileName(), target->fileNameLength());
-    CppRefactoringFileConstPtr targetFile = changes.fileNoEditor(targetFileName);
+    CppRefactoringFileConstPtr targetFile = changes.fileNoEditor(
+        Utils::FilePath::fromString(targetFileName));
     if (!targetFile->isValid())
         return noResult;
 
@@ -220,7 +220,8 @@ void FunctionDeclDefLinkFinder::startFindLinkAt(
 
     // find the start/end offsets
     CppRefactoringChanges refactoringChanges(snapshot);
-    CppRefactoringFilePtr sourceFile = refactoringChanges.file(doc->fileName());
+    CppRefactoringFilePtr sourceFile = refactoringChanges.file(
+        Utils::FilePath::fromString(doc->fileName()));
     sourceFile->setCppDocument(doc);
     int start, end;
     declDefLinkStartEnd(sourceFile, parent, funcDecl, &start, &end);
@@ -280,7 +281,7 @@ void FunctionDeclDefLink::apply(CppEditorWidget *editor, bool jumpToMatch)
 
     // first verify the interesting region of the target file is unchanged
     CppRefactoringChanges refactoringChanges(snapshot);
-    CppRefactoringFilePtr newTargetFile = refactoringChanges.file(targetFile->fileName());
+    CppRefactoringFilePtr newTargetFile = refactoringChanges.file(targetFile->filePath());
     if (!newTargetFile->isValid())
         return;
     const int targetStart = newTargetFile->position(targetLine, targetColumn);
@@ -304,7 +305,7 @@ void FunctionDeclDefLink::hideMarker(CppEditorWidget *editor)
     if (!hasMarker)
         return;
     editor->setRefactorMarkers(RefactorMarker::filterOutType(
-        editor->refactorMarkers(), CppTools::Constants::CPP_FUNCTION_DECL_DEF_LINK_MARKER_ID));
+        editor->refactorMarkers(), Constants::CPP_FUNCTION_DECL_DEF_LINK_MARKER_ID));
     hasMarker = false;
 }
 
@@ -314,7 +315,7 @@ void FunctionDeclDefLink::showMarker(CppEditorWidget *editor)
         return;
 
     QList<RefactorMarker> markers = RefactorMarker::filterOutType(
-        editor->refactorMarkers(), CppTools::Constants::CPP_FUNCTION_DECL_DEF_LINK_MARKER_ID);
+        editor->refactorMarkers(), Constants::CPP_FUNCTION_DECL_DEF_LINK_MARKER_ID);
     RefactorMarker marker;
 
     // show the marker at the end of the linked area, with a special case
@@ -339,7 +340,7 @@ void FunctionDeclDefLink::showMarker(CppEditorWidget *editor)
         message = ProxyAction::stringWithAppendedShortcut(message, quickfixCommand->keySequence());
 
     marker.tooltip = message;
-    marker.type = CppTools::Constants::CPP_FUNCTION_DECL_DEF_LINK_MARKER_ID;
+    marker.type = Constants::CPP_FUNCTION_DECL_DEF_LINK_MARKER_ID;
     marker.callback = [](TextEditor::TextEditorWidget *widget) {
         if (auto cppEditor = qobject_cast<CppEditorWidget *>(widget))
             cppEditor->applyDeclDefLinkChanges(true);
