@@ -24,10 +24,11 @@
 ****************************************************************************/
 
 #include "cpphighlighter.h"
-#include "cppeditorenums.h"
 
-#include <cpptools/cppdoxygen.h>
-#include <cpptools/cpptoolsreuse.h>
+#include "cppdoxygen.h"
+#include "cppmodelmanager.h"
+#include "cpptoolsreuse.h"
+
 #include <texteditor/textdocumentlayout.h>
 
 #include <cplusplus/SimpleLexer.h>
@@ -90,11 +91,15 @@ void CppHighlighter::highlightBlock(const QString &text)
 
     const int firstNonSpace = tokens.first().utf16charsBegin();
 
-    Parentheses parentheses;
+    // Keep "semantic parentheses".
+    Parentheses parentheses = Utils::filtered(TextDocumentLayout::parentheses(currentBlock()),
+            [](const Parenthesis &p) { return p.source.isValid(); });
+    const auto insertParen = [&parentheses](const Parenthesis &p) { insertSorted(parentheses, p); };
     parentheses.reserve(5);
 
     bool expectPreprocessorKeyword = false;
     bool onlyHighlightComments = false;
+    bool blockHasPreprocessorDirective = false;
 
     for (int i = 0; i < tokens.size(); ++i) {
         const Token &tk = tokens.at(i);
@@ -114,7 +119,7 @@ void CppHighlighter::highlightBlock(const QString &text)
 
         if (tk.is(T_LPAREN) || tk.is(T_LBRACE) || tk.is(T_LBRACKET)) {
             const QChar c = text.at(tk.utf16charsBegin());
-            parentheses.append(Parenthesis(Parenthesis::Opened, c, tk.utf16charsBegin()));
+            insertParen({Parenthesis::Opened, c, tk.utf16charsBegin()});
             if (tk.is(T_LBRACE)) {
                 ++braceDepth;
 
@@ -127,7 +132,7 @@ void CppHighlighter::highlightBlock(const QString &text)
             }
         } else if (tk.is(T_RPAREN) || tk.is(T_RBRACE) || tk.is(T_RBRACKET)) {
             const QChar c = text.at(tk.utf16charsBegin());
-            parentheses.append(Parenthesis(Parenthesis::Closed, c, tk.utf16charsBegin()));
+            insertParen({Parenthesis::Closed, c, tk.utf16charsBegin()});
             if (tk.is(T_RBRACE)) {
                 --braceDepth;
                 if (braceDepth < foldingIndent) {
@@ -152,6 +157,7 @@ void CppHighlighter::highlightBlock(const QString &text)
             setFormatWithSpaces(text, tk.utf16charsBegin(), tk.utf16chars(),
                           formatForCategory(C_PREPROCESSOR));
             expectPreprocessorKeyword = true;
+            blockHasPreprocessorDirective = true;
         } else if (highlightCurrentWordAsPreprocessor && (tk.isKeyword() || tk.is(T_IDENTIFIER))
                    && isPPKeyword(Utils::midView(text, tk.utf16charsBegin(), tk.utf16chars()))) {
             setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(C_PREPROCESSOR));
@@ -165,7 +171,22 @@ void CppHighlighter::highlightBlock(const QString &text)
         } else if (tk.is(T_NUMERIC_LITERAL)) {
             setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(C_NUMBER));
         } else if (tk.isStringLiteral() || tk.isCharLiteral()) {
-            if (!highlightRawStringLiteral(text, tk)) {
+            // Our highlighting is broken for multi-line raw string literals, so if a superior
+            // option is available, don't do anything.
+            // Note that this does not just save unneeded work, but can actually be required,
+            // because mis-detected strings are not necessarily overwritten by the semantic
+            // highlighter. Example:
+            // const char *s = R"delim(
+            //    line1
+            //    "line)"  // <- is misdeteced by SimpleLexer as end of raw string literal
+            //    line3    // <- erroneously not formatted by us, but that would be ok;
+            //             //    the semantic highlighter does it for us later
+            //    )delim"; // <- end quote is erroneously interpreted as *start* of a string,
+            //             //    and because clangd does not include punctuation in its semantic
+            //             //    tokens, the semicolon would stay formatted as a string even
+            //             //    after the semantic highlighter has run.
+            if ((!CppModelManager::instance()->isClangCodeModelActive()
+                 || blockHasPreprocessorDirective) && !highlightRawStringLiteral(text, tk)) {
                 setFormatWithSpaces(text, tk.utf16charsBegin(), tk.utf16chars(),
                                     formatForCategory(C_STRING));
             }
@@ -183,7 +204,8 @@ void CppHighlighter::highlightBlock(const QString &text)
             //  - the line starts in a C Comment (initalState != 0)
             //  - the first token of the line is a T_COMMENT (i == 0 && tk.is(T_COMMENT))
             //  - is not a continuation line (tokens.size() > 1 || !state)
-            if (initialLexerState && i == 0 && (tokens.size() > 1 || !lexerState)) {
+            if (initialLexerState && i == 0 && (tk.is(T_COMMENT) || tk.is(T_DOXY_COMMENT))
+                && (tokens.size() > 1 || !lexerState)) {
                 --braceDepth;
                 // unless we are at the end of the block, we reduce the folding indent
                 if (i == tokens.size()-1)
@@ -191,7 +213,7 @@ void CppHighlighter::highlightBlock(const QString &text)
                 else
                     foldingIndent = qMin(braceDepth, foldingIndent);
                 const int tokenEnd = tk.utf16charsBegin() + tk.utf16chars() - 1;
-                parentheses.append(Parenthesis(Parenthesis::Closed, QLatin1Char('-'), tokenEnd));
+                insertParen({Parenthesis::Closed, QLatin1Char('-'), tokenEnd});
 
                 // clear the initial state.
                 initialLexerState = 0;
@@ -199,7 +221,7 @@ void CppHighlighter::highlightBlock(const QString &text)
 
         } else if (tk.isKeyword()
                    || (m_languageFeatures.qtKeywordsEnabled
-                       && CppTools::isQtKeyword(
+                       && isQtKeyword(
                            QStringView{text}.mid(tk.utf16charsBegin(), tk.utf16chars())))
                    || (m_languageFeatures.objCEnabled && tk.isObjCAtKeyword())) {
             setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(C_KEYWORD));
@@ -227,8 +249,7 @@ void CppHighlighter::highlightBlock(const QString &text)
     if (!initialLexerState && lexerState && !tokens.isEmpty()) {
         const Token &lastToken = tokens.last();
         if (lastToken.is(T_COMMENT) || lastToken.is(T_DOXY_COMMENT)) {
-            parentheses.append(Parenthesis(Parenthesis::Opened, QLatin1Char('+'),
-                                           lastToken.utf16charsBegin()));
+            insertParen({Parenthesis::Opened, QLatin1Char('+'), lastToken.utf16charsBegin()});
             ++braceDepth;
         }
     }
@@ -277,7 +298,7 @@ void CppHighlighter::setLanguageFeatures(const LanguageFeatures &languageFeature
     }
 }
 
-bool CppHighlighter::isPPKeyword(const QStringView &text) const
+bool CppHighlighter::isPPKeyword(QStringView text) const
 {
     switch (text.length())
     {
@@ -369,7 +390,7 @@ void CppHighlighter::highlightWord(QStringView word, int position, int length)
     }
 }
 
-bool CppHighlighter::highlightRawStringLiteral(const QStringView &_text, const Token &tk)
+bool CppHighlighter::highlightRawStringLiteral(QStringView _text, const Token &tk)
 {
     // Step one: Does the lexer think this is a raw string literal?
     switch (tk.kind()) {
@@ -433,11 +454,11 @@ void CppHighlighter::highlightDoxygenComment(const QString &text, int position,
             ++it;
 
             const QChar *start = it;
-            while (CppTools::isValidAsciiIdentifierChar(*it))
+            while (isValidAsciiIdentifierChar(*it))
                 ++it;
 
-            int k = CppTools::classifyDoxygenTag(start, it - start);
-            if (k != CppTools::T_DOXY_IDENTIFIER) {
+            int k = classifyDoxygenTag(start, it - start);
+            if (k != T_DOXY_IDENTIFIER) {
                 setFormatWithSpaces(text, initial, start - uc - initial, format);
                 setFormat(start - uc - 1, it - start + 1, kwFormat);
                 initial = it - uc;
