@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -75,7 +70,9 @@ CppLocalRenaming::CppLocalRenaming(TextEditor::TextEditorWidget *editorWidget)
 void CppLocalRenaming::updateSelectionsForVariableUnderCursor(
         const QList<QTextEdit::ExtraSelection> &selections)
 {
-    QTC_ASSERT(!isActive(), return);
+    if (isActive())
+        return;
+
     m_selections = selections;
 }
 
@@ -110,7 +107,7 @@ bool CppLocalRenaming::handleCut()
         return false;
 
     startRenameChange();
-    m_editorWidget->TextEditorWidget::paste();
+    m_editorWidget->TextEditorWidget::cut();
     finishRenameChange();
     return true;
 }
@@ -196,16 +193,22 @@ bool CppLocalRenaming::handleKeyPressEvent(QKeyEvent *e)
     startRenameChange();
 
     const bool wantEditBlock = isWithinRenameSelection(cursorPosition);
+    const int undoSizeBeforeEdit = m_editorWidget->document()->availableUndoSteps();
     if (wantEditBlock) {
         if (m_firstRenameChangeExpected) // Change inside rename selection
             cursor.beginEditBlock();
         else
             cursor.joinPreviousEditBlock();
-        m_firstRenameChangeExpected = false;
     }
     emit processKeyPressNormally(e);
-    if (wantEditBlock)
+    if (wantEditBlock) {
         cursor.endEditBlock();
+        if (m_firstRenameChangeExpected
+                // QTCREATORBUG-16350
+                && m_editorWidget->document()->availableUndoSteps() != undoSizeBeforeEdit) {
+            m_firstRenameChangeExpected = false;
+        }
+    }
     finishRenameChange();
     return true;
 }
@@ -240,16 +243,31 @@ void CppLocalRenaming::forgetRenamingSelection()
     m_renameSelectionIndex = -1;
 }
 
+bool CppLocalRenaming::isWithinSelection(const QTextEdit::ExtraSelection &selection, int position)
+{
+    return selection.cursor.selectionStart() <= position
+           && position <= selection.cursor.selectionEnd();
+}
+
 bool CppLocalRenaming::isWithinRenameSelection(int position)
 {
-    return renameSelectionBegin() <= position && position <= renameSelectionEnd();
+    return isWithinSelection(renameSelection(), position);
+}
+
+bool CppLocalRenaming::isSameSelection(int cursorPosition) const
+{
+    if (!isActive())
+        return false;
+
+    const QTextEdit::ExtraSelection &sel = m_selections[m_renameSelectionIndex];
+    return isWithinSelection(sel, cursorPosition);
 }
 
 bool CppLocalRenaming::findRenameSelection(int cursorPosition)
 {
     for (int i = 0, total = m_selections.size(); i < total; ++i) {
         const QTextEdit::ExtraSelection &sel = m_selections.at(i);
-        if (sel.cursor.position() <= cursorPosition && cursorPosition <= sel.cursor.anchor()) {
+        if (isWithinSelection(sel, cursorPosition)) {
             m_renameSelectionIndex = i;
             return true;
         }
