@@ -27,10 +27,9 @@
 
 #include "cppeditorconstants.h"
 #include "cppeditorwidget.h"
+#include "cppmodelmanager.h"
 #include "cppquickfixes.h"
-
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpprefactoringchanges.h>
+#include "cpprefactoringchanges.h"
 
 #include <texteditor/codeassist/genericproposal.h>
 #include <texteditor/codeassist/iassistprocessor.h>
@@ -41,9 +40,8 @@
 #include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 
-using namespace TextEditor;
-using namespace CppTools;
 using namespace CPlusPlus;
+using namespace TextEditor;
 
 namespace CppEditor {
 namespace Internal {
@@ -57,8 +55,6 @@ class CppQuickFixAssistProcessor : public IAssistProcessor
     {
         QSharedPointer<const AssistInterface> assistInterface(interface);
         auto cppInterface = assistInterface.staticCast<const CppQuickFixInterface>();
-        if (cppInterface->path().isEmpty())
-            return nullptr;
 
         QuickFixOperations quickFixes;
         for (CppQuickFixFactory *factory : CppQuickFixFactory::cppQuickFixFactories())
@@ -76,7 +72,7 @@ IAssistProvider::RunType CppQuickFixAssistProvider::runType() const
     return Synchronous;
 }
 
-IAssistProcessor *CppQuickFixAssistProvider::createProcessor() const
+IAssistProcessor *CppQuickFixAssistProvider::createProcessor(const AssistInterface *) const
 {
     return new CppQuickFixAssistProcessor;
 }
