@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,44 +9,35 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-
 #include "cppvirtualfunctionassistprovider.h"
 
-#include "cppeditorconstants.h"
 #include "cppvirtualfunctionproposalitem.h"
 
+#include "cpptoolsreuse.h"
+#include "functionutils.h"
+#include "symbolfinder.h"
+
 #include <cplusplus/Icons.h>
 #include <cplusplus/Overview.h>
 
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/command.h>
 
-#include <cpptools/cpptoolsreuse.h>
-#include <cpptools/functionutils.h>
-#include <cpptools/symbolfinder.h>
-#include <cpptools/typehierarchybuilder.h>
-
 #include <texteditor/codeassist/genericproposalmodel.h>
-#include <texteditor/codeassist/genericproposal.h>
 #include <texteditor/codeassist/genericproposalwidget.h>
 #include <texteditor/codeassist/assistinterface.h>
 #include <texteditor/codeassist/iassistprocessor.h>
@@ -56,10 +47,10 @@
 #include <utils/qtcassert.h>
 
 using namespace CPlusPlus;
-using namespace CppEditor::Internal;
-using namespace CppTools;
 using namespace TextEditor;
 
+namespace CppEditor {
+
 /// Activate current item with the same shortcut that is configured for Follow Symbol Under Cursor.
 /// This is limited to single-key shortcuts without modifiers.
 class VirtualFunctionProposalWidget : public GenericProposalWidget
@@ -75,10 +66,10 @@ public:
     }
 
 protected:
-    bool eventFilter(QObject *o, QEvent *e)
+    bool eventFilter(QObject *o, QEvent *e) override
     {
         if (e->type() == QEvent::ShortcutOverride && m_sequence.count() == 1) {
-            QKeyEvent *ke = static_cast<QKeyEvent *>(e);
+            auto ke = static_cast<const QKeyEvent*>(e);
             const QKeySequence seq(ke->key());
             if (seq == m_sequence) {
                 activateCurrentProposalItem();
@@ -91,11 +82,15 @@ protected:
 
     void showProposal(const QString &prefix) override
     {
-        GenericProposalModel *proposalModel = model();
+        GenericProposalModelPtr proposalModel = model();
         if (proposalModel && proposalModel->size() == 1) {
-            emit proposalItemActivated(proposalModel->proposalItem(0));
-            deleteLater();
-            return;
+            const auto item = dynamic_cast<VirtualFunctionProposalItem *>(
+                        proposalModel->proposalItem(0));
+            if (item && item->link().hasValidTarget()) {
+                emit proposalItemActivated(proposalModel->proposalItem(0));
+                deleteLater();
+                return;
+            }
         }
         GenericProposalWidget::showProposal(prefix);
     }
@@ -104,22 +99,7 @@ private:
     QKeySequence m_sequence;
 };
 
-class VirtualFunctionProposal : public GenericProposal
-{
-public:
-    VirtualFunctionProposal(int cursorPos, const QList<AssistProposalItem *> &items, bool openInSplit)
-        : GenericProposal(cursorPos, items)
-        , m_openInSplit(openInSplit)
-    {}
 
-    bool isFragile() const override { return true; }
-
-    IAssistProposalWidget *createWidget() const override
-    { return new VirtualFunctionProposalWidget(m_openInSplit); }
-
-private:
-    bool m_openInSplit;
-};
 
 class VirtualFunctionAssistProcessor : public IAssistProcessor
 {
@@ -130,15 +110,14 @@ public:
 
     IAssistProposal *immediateProposal(const AssistInterface *) override
     {
-        QTC_ASSERT(m_params.function, return 0);
+        QTC_ASSERT(m_params.function, return nullptr);
 
-        AssistProposalItem *hintItem
-                = new VirtualFunctionProposalItem(TextEditorWidget::Link());
+        auto *hintItem = new VirtualFunctionProposalItem(Utils::Link());
         hintItem->setText(QCoreApplication::translate("VirtualFunctionsAssistProcessor",
-                                                      "...searching overrides"));
+                                                      "collecting overrides ..."));
         hintItem->setOrder(-1000);
 
-        QList<AssistProposalItem *> items;
+        QList<AssistProposalItemInterface *> items;
         items << itemFromFunction(m_params.function);
         items << hintItem;
         return new VirtualFunctionProposal(m_params.cursorPosition, items, m_params.openInNextSplit);
@@ -148,21 +127,21 @@ public:
     {
         delete assistInterface;
 
-        QTC_ASSERT(m_params.function, return 0);
-        QTC_ASSERT(m_params.staticClass, return 0);
-        QTC_ASSERT(!m_params.snapshot.isEmpty(), return 0);
+        QTC_ASSERT(m_params.function, return nullptr);
+        QTC_ASSERT(m_params.staticClass, return nullptr);
+        QTC_ASSERT(!m_params.snapshot.isEmpty(), return nullptr);
 
-        Class *functionsClass = m_finder.findMatchingClassDeclaration(
-                    m_params.function, m_params.snapshot, &m_params.typeOfExpression->context());
+        Class *functionsClass = m_finder.findMatchingClassDeclaration(m_params.function,
+                                                                      m_params.snapshot);
         if (!functionsClass)
-            return 0;
+            return nullptr;
 
-        const QList<Function *> overrides = FunctionUtils::overrides(
+        const QList<Function *> overrides = Internal::FunctionUtils::overrides(
             m_params.function, functionsClass, m_params.staticClass, m_params.snapshot);
         if (overrides.isEmpty())
-            return 0;
+            return nullptr;
 
-        QList<AssistProposalItem *> items;
+        QList<AssistProposalItemInterface *> items;
         foreach (Function *func, overrides)
             items << itemFromFunction(func);
         items.first()->setOrder(1000); // Ensure top position for function of static type
@@ -178,29 +157,26 @@ private:
         return func;
     }
 
-    AssistProposalItem *itemFromFunction(Function *func) const
+    VirtualFunctionProposalItem *itemFromFunction(Function *func) const
     {
-        const TextEditorWidget::Link link = CppTools::linkToSymbol(maybeDefinitionFor(func));
+        const Utils::Link link = maybeDefinitionFor(func)->toLink();
         QString text = m_overview.prettyName(LookupContext::fullyQualifiedName(func));
         if (func->isPureVirtual())
             text += QLatin1String(" = 0");
 
-        AssistProposalItem *item = new VirtualFunctionProposalItem(link, m_params.openInNextSplit);
+        auto *item = new VirtualFunctionProposalItem(link, m_params.openInNextSplit);
         item->setText(text);
-        item->setIcon(m_icons.iconForSymbol(func));
+        item->setIcon(Icons::iconForSymbol(func));
 
         return item;
     }
 
     VirtualFunctionAssistProvider::Parameters m_params;
     Overview m_overview;
-    Icons m_icons;
     mutable SymbolFinder m_finder;
 };
 
-VirtualFunctionAssistProvider::VirtualFunctionAssistProvider()
-{
-}
+VirtualFunctionAssistProvider::VirtualFunctionAssistProvider() = default;
 
 bool VirtualFunctionAssistProvider::configure(const Parameters &parameters)
 {
@@ -213,12 +189,21 @@ IAssistProvider::RunType VirtualFunctionAssistProvider::runType() const
     return AsynchronousWithThread;
 }
 
-bool VirtualFunctionAssistProvider::supportsEditor(Core::Id editorId) const
-{
-    return editorId == Constants::CPPEDITOR_ID;
-}
-
-IAssistProcessor *VirtualFunctionAssistProvider::createProcessor() const
+IAssistProcessor *VirtualFunctionAssistProvider::createProcessor(const AssistInterface *) const
 {
     return new VirtualFunctionAssistProcessor(m_params);
 }
+
+VirtualFunctionProposal::VirtualFunctionProposal(
+        int cursorPos, const QList<AssistProposalItemInterface *> &items, bool openInSplit)
+    : GenericProposal(cursorPos, items), m_openInSplit(openInSplit)
+{
+    setFragile(true);
+}
+
+IAssistProposalWidget *VirtualFunctionProposal::createWidget() const
+{
+    return new VirtualFunctionProposalWidget(m_openInSplit);
+}
+
+} // namespace CppEditor
