@@ -23,23 +23,25 @@
 **
 ****************************************************************************/
 
-#include "cppeditor.h"
-#include "cppeditorwidget.h"
+#include "fileandtokenactions_test.h"
+
 #include "cppeditorplugin.h"
-#include "cppeditortestcase.h"
+#include "cppeditorwidget.h"
 #include "cppquickfix.h"
 #include "cppquickfixassistant.h"
 #include "cppquickfixes.h"
 #include "cppinsertvirtualmethods.h"
+#include "cppmodelmanager.h"
+#include "cpptoolsreuse.h"
+#include "cpptoolstestcase.h"
+#include "cppworkingcopy.h"
+#include "projectinfo.h"
 
 #include <coreplugin/editormanager/editormanager.h>
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpptoolsreuse.h>
-#include <cpptools/cppworkingcopy.h>
-#include <cpptools/projectinfo.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
 #include <texteditor/textdocument.h>
+#include <texteditor/texteditor.h>
 
 #include <cplusplus/CppDocument.h>
 #include <cplusplus/TranslationUnit.h>
@@ -69,15 +71,13 @@
 
 using namespace Core;
 using namespace CPlusPlus;
-using namespace CppTools;
 using namespace TextEditor;
 
-namespace CppEditor {
-namespace Internal {
+namespace CppEditor::Internal::Tests {
 
 namespace {
 
-class TestActionsTestCase : public Tests::TestCase
+class TestActionsTestCase : public CppEditor::Tests::TestCase
 {
 public:
     class AbstractAction
@@ -111,7 +111,7 @@ private:
     /// Move word camel case wise from current cursor position until given token (not included)
     /// and execute the tokenActions for each new position.
     static void moveWordCamelCaseToToken(TranslationUnit *translationUnit, const Token &token,
-                                         CppEditor *editor, const Actions &tokenActions);
+                                         BaseTextEditor *editor, const Actions &tokenActions);
 
     static void undoAllChangesAndCloseAllEditors();
 };
@@ -141,7 +141,7 @@ static bool waitUntilAProjectIsLoaded(int timeOutInMs = 30000)
 }
 
 TestActionsTestCase::TestActionsTestCase(const Actions &tokenActions, const Actions &fileActions)
-    : Tests::TestCase(/*runGarbageCollector=*/false)
+    : CppEditor::Tests::TestCase(/*runGarbageCollector=*/false)
 {
     QVERIFY(succeededSoFar());
 
@@ -152,15 +152,12 @@ TestActionsTestCase::TestActionsTestCase(const Actions &tokenActions, const Acti
     // Collect files to process
     QStringList filesToOpen;
     QList<QPointer<ProjectExplorer::Project> > projects;
-    const QList<ProjectInfo> projectInfos = m_modelManager->projectInfos();
+    const QList<ProjectInfo::ConstPtr> projectInfos = m_modelManager->projectInfos();
 
-    foreach (const ProjectInfo &info, projectInfos) {
-        QPointer<ProjectExplorer::Project> project = info.project();
-        if (!projects.contains(project))
-            projects << project;
-        qDebug() << "Project" << info.project()->displayName() << "- files to process:"
-                 << info.sourceFiles().size();
-        foreach (const QString &sourceFile, info.sourceFiles())
+    foreach (const ProjectInfo::ConstPtr &info, projectInfos) {
+        qDebug() << "Project" << info->projectFilePath().toUserOutput() << "- files to process:"
+                 << info->sourceFiles().size();
+        foreach (const QString &sourceFile, info->sourceFiles())
             filesToOpen << sourceFile;
     }
 
@@ -179,7 +176,7 @@ TestActionsTestCase::TestActionsTestCase(const Actions &tokenActions, const Acti
 
         // Open editor
         QCOMPARE(DocumentModel::openedDocuments().size(), 0);
-        CppEditor *editor;
+        BaseTextEditor *editor;
         CppEditorWidget *editorWidget;
         QVERIFY(openCppEditor(filePath, &editor, &editorWidget));
 
@@ -198,7 +195,7 @@ TestActionsTestCase::TestActionsTestCase(const Actions &tokenActions, const Acti
 
         const Snapshot snapshot = globalSnapshot();
         Document::Ptr document = snapshot.preprocessedDocument(
-            editorWidget->document()->toPlainText().toUtf8(), filePath);
+            editorWidget->document()->toPlainText().toUtf8(), Utils::FilePath::fromString(filePath));
         QVERIFY(document);
         document->parse();
         TranslationUnit *translationUnit = document->translationUnit();
@@ -279,7 +276,7 @@ void TestActionsTestCase::executeActionsOnEditorWidget(CppEditorWidget *editorWi
 
 void TestActionsTestCase::moveWordCamelCaseToToken(TranslationUnit *translationUnit,
                                                    const Token &token,
-                                                   CppEditor *editor,
+                                                   BaseTextEditor *editor,
                                                    const Actions &tokenActions)
 {
     QVERIFY(translationUnit);
@@ -484,7 +481,7 @@ void SwitchHeaderSourceFileAction::run(CppEditorWidget *)
 {
     // Switch Header/Source
     IEditor *editorBefore = EditorManager::currentEditor();
-    CppTools::switchHeaderSource();
+    switchHeaderSource();
     QApplication::processEvents();
 
     // Go back
@@ -496,56 +493,55 @@ void SwitchHeaderSourceFileAction::run(CppEditorWidget *)
 
 } // anonymous namespace
 
-void CppEditorPlugin::test_openEachFile()
+void FileAndTokenActionsTest::testOpenEachFile()
 {
     TestActionsTestCase();
 }
 
-void CppEditorPlugin::test_switchHeaderSourceOnEachFile()
+void FileAndTokenActionsTest::testSwitchHeaderSourceOnEachFile()
 {
     TestActionsTestCase(Actions(), singleAction(ActionPointer(new SwitchHeaderSourceFileAction)));
 }
 
-void CppEditorPlugin::test_moveTokenWiseThroughEveryFile()
+void FileAndTokenActionsTest::testMoveTokenWiseThroughEveryFile()
 {
     TestActionsTestCase(singleAction(ActionPointer(new NoOpTokenAction)));
 }
 
 /// May block if file does not exist (e.g. a not generated ui_* file).
-void CppEditorPlugin::test_moveTokenWiseThroughEveryFileAndFollowSymbol()
+void FileAndTokenActionsTest::testMoveTokenWiseThroughEveryFileAndFollowSymbol()
 {
     TestActionsTestCase(singleAction(ActionPointer(new FollowSymbolUnderCursorTokenAction)));
 }
 
-void CppEditorPlugin::test_moveTokenWiseThroughEveryFileAndSwitchDeclarationDefinition()
+void FileAndTokenActionsTest::testMoveTokenWiseThroughEveryFileAndSwitchDeclarationDefinition()
 {
     TestActionsTestCase(singleAction(ActionPointer(new SwitchDeclarationDefinitionTokenAction)));
 }
 
-void CppEditorPlugin::test_moveTokenWiseThroughEveryFileAndFindUsages()
+void FileAndTokenActionsTest::testMoveTokenWiseThroughEveryFileAndFindUsages()
 {
     TestActionsTestCase(singleAction(ActionPointer(new FindUsagesTokenAction)));
 }
 
-void CppEditorPlugin::test_moveTokenWiseThroughEveryFileAndRenameUsages()
+void FileAndTokenActionsTest::testMoveTokenWiseThroughEveryFileAndRenameUsages()
 {
     TestActionsTestCase(singleAction(ActionPointer(new RenameSymbolUnderCursorTokenAction)));
 }
 
-void CppEditorPlugin::test_moveTokenWiseThroughEveryFileAndOpenTypeHierarchy()
+void FileAndTokenActionsTest::testMoveTokenWiseThroughEveryFileAndOpenTypeHierarchy()
 {
     TestActionsTestCase(singleAction(ActionPointer(new OpenTypeHierarchyTokenAction)));
 }
 
-void CppEditorPlugin::test_moveTokenWiseThroughEveryFileAndInvokeCompletion()
+void FileAndTokenActionsTest::testMoveTokenWiseThroughEveryFileAndInvokeCompletion()
 {
     TestActionsTestCase(singleAction(ActionPointer(new InvokeCompletionTokenAction)));
 }
 
-void CppEditorPlugin::test_moveTokenWiseThroughEveryFileAndTriggerQuickFixes()
+void FileAndTokenActionsTest::testMoveTokenWiseThroughEveryFileAndTriggerQuickFixes()
 {
     TestActionsTestCase(singleAction(ActionPointer(new RunAllQuickFixesTokenAction)));
 }
 
-} // namespace Internal
-} // namespace CppEditor
+} // namespace CppEditor::Internal::Tests
