@@ -53,7 +53,7 @@ public:
     bool canHandleToolTip(const DebuggerToolTipContext &context) const override;
 
     void setupEngine() override;
-    void runEngine() override;
+    void runEngine();
     void shutdownInferior() override;
     void shutdownEngine() override;
     void abortDebuggerProcess() override;
@@ -78,9 +78,7 @@ public:
     void activateFrame(int index) override;
     void selectThread(const Thread &thread) override;
 
-    bool stateAcceptsBreakpointChanges() const override;
     bool acceptsBreakpoint(const BreakpointParameters &params) const override;
-
     void insertBreakpoint(const Breakpoint &bp) override;
     void removeBreakpoint(const Breakpoint &bp) override;
     void updateBreakpoint(const Breakpoint &bp) override;
@@ -196,6 +194,7 @@ private:
     int elapsedLogTime();
     unsigned parseStackTrace(const GdbMi &data, bool sourceStepInto);
     void mergeStartParametersSourcePathMap();
+    void checkQtSdkPdbFiles(const QString &module);
 
     const QString m_tokenPrefix;
     void handleSetupFailure(const QString &errorMessage);
