@@ -62,7 +62,6 @@ enum DebuggerState
 
     EngineSetupRequested,      // Engine starts
     EngineSetupFailed,
-    EngineSetupOk,
 
     EngineRunRequested,
     EngineRunFailed,
@@ -150,7 +149,7 @@ public:
     // Used by Android to avoid false positives on warnOnRelease
     bool skipExecutableValidation = false;
     bool useTargetAsync = false;
-    Utils::FilePathList additionalSearchDirectories;
+    Utils::FilePaths additionalSearchDirectories;
 
     // Used by iOS.
     QString platform;
@@ -159,7 +158,7 @@ public:
     Utils::FilePath sysRoot;
 
     // Used by general core file debugging. Public access requested in QTCREATORBUG-17158.
-    QString coreFile;
+    Utils::FilePath coreFile;
 
     // Macro-expanded and passed to debugger startup.
     QString additionalStartupCommands;
@@ -170,18 +169,19 @@ public:
     bool breakOnMain = false;
     bool multiProcess = false; // Whether to set detach-on-fork off.
     bool useTerminal = false;
+    bool runAsRoot = false;
 
     ProjectExplorer::Runnable debugger;
-    QString overrideStartScript; // Used in attach to core and remote debugging
+    Utils::FilePath overrideStartScript; // Used in attach to core and remote debugging
     QString startMessage; // First status message shown.
-    QString debugInfoLocation; // Gdb "set-debug-file-directory".
+    Utils::FilePath debugInfoLocation; // Gdb "set-debug-file-directory".
     QStringList debugSourceLocation; // Gdb "directory"
     QString qtPackageSourceLocation;
     bool isSnapshot = false; // Set if created internally.
     ProjectExplorer::Abi toolChainAbi;
 
     Utils::FilePath projectSourceDirectory;
-    Utils::FilePathList projectSourceFiles;
+    Utils::FilePaths projectSourceFiles;
 
     // Used by Script debugging
     QString interpreter;
@@ -195,12 +195,17 @@ public:
     bool isCppDebugging() const;
     bool isNativeMixedDebugging() const;
 
-    Utils::MacroExpander *macroExpander = nullptr;
+    const Utils::MacroExpander *macroExpander = nullptr;
+
+    Utils::optional<int> exitCode = {};
 
     // For Debugger testing.
     int testCase = 0;
 
     QStringList validationErrors;
+
+    Utils::FilePath dumperPath;
+    int fallbackQtVersion = 0x50200;
 };
 
 class UpdateParameters
@@ -225,17 +230,17 @@ class Location
 public:
     Location() = default;
     Location(quint64 address) { m_address = address; }
-    Location(const QString &file) { m_fileName = file; }
-    Location(const QString &file, int line, bool marker = true)
+    Location(const Utils::FilePath &file) { m_fileName = file; }
+    Location(const Utils::FilePath &file, int line, bool marker = true)
         { m_lineNumber = line; m_fileName = file; m_needsMarker = marker; }
     Location(const StackFrame &frame, bool marker = true);
-    QString fileName() const { return m_fileName; }
+    Utils::FilePath fileName() const { return m_fileName; }
     QString functionName() const { return m_functionName; }
     QString from() const { return m_from; }
     int lineNumber() const { return m_lineNumber; }
     void setNeedsRaise(bool on) { m_needsRaise = on; }
     void setNeedsMarker(bool on) { m_needsMarker = on; }
-    void setFileName(const QString &fileName) { m_fileName = fileName; }
+    void setFileName(const Utils::FilePath &fileName) { m_fileName = fileName; }
     void setUseAssembler(bool on) { m_hasDebugInfo = !on; }
     bool needsRaise() const { return m_needsRaise; }
     bool needsMarker() const { return m_needsMarker; }
@@ -249,7 +254,7 @@ private:
     bool m_needsRaise = true;
     bool m_hasDebugInfo = true;
     int m_lineNumber = -1;
-    QString m_fileName;
+    Utils::FilePath m_fileName;
     QString m_functionName;
     QString m_from;
     quint64 m_address = 0;
@@ -342,7 +347,6 @@ public:
     virtual Core::Context languageContext() const { return {}; }
     QString displayName() const;
 
-    virtual bool stateAcceptsBreakpointChanges() const { return true; }
     virtual bool acceptsBreakpoint(const BreakpointParameters &bp) const = 0;
     virtual void insertBreakpoint(const Breakpoint &bp) = 0;
     virtual void removeBreakpoint(const Breakpoint &bp) = 0;
@@ -382,6 +386,7 @@ public:
 
     static QString stateName(int s);
 
+    void notifyExitCode(int code);
     void notifyInferiorPid(const Utils::ProcessHandle &pid);
     qint64 inferiorPid() const;
 
@@ -392,9 +397,8 @@ public:
     void handleReverseDirection(bool);
 
     // Convenience
-    Q_SLOT virtual void showMessage(const QString &msg, int channel = LogDebug,
-        int timeout = -1) const;
-    Q_SLOT void showStatusMessage(const QString &msg, int timeout = -1) const;
+    void showMessage(const QString &msg, int channel = LogDebug, int timeout = -1) const;
+    void showStatusMessage(const QString &msg, int timeout = -1) const;
 
     virtual void resetLocation();
     virtual void gotoLocation(const Internal::Location &location);
@@ -510,7 +514,6 @@ protected:
     void notifyEngineIll();
 
     virtual void setupEngine() = 0;
-    virtual void runEngine() = 0;
     virtual void shutdownInferior() = 0;
     virtual void shutdownEngine() = 0;
     virtual void resetInferior() {}
