@@ -35,13 +35,12 @@
 #include <utils/hostosinfo.h>
 #include <utils/macroexpander.h>
 #include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
 #include <utils/stringutils.h>
-#include <utils/synchronousprocess.h>
 #include <utils/utilsicons.h>
 #include <utils/winutils.h>
 
 #include <QFileInfo>
-#include <QProcess>
 #include <QUuid>
 
 #ifdef WITH_TESTS
@@ -53,11 +52,14 @@ using namespace Debugger::Internal;
 using namespace ProjectExplorer;
 using namespace Utils;
 
+namespace Debugger {
+
 const char DEBUGGER_INFORMATION_COMMAND[] = "Binary";
 const char DEBUGGER_INFORMATION_DISPLAYNAME[] = "DisplayName";
 const char DEBUGGER_INFORMATION_ID[] = "Id";
 const char DEBUGGER_INFORMATION_ENGINETYPE[] = "EngineType";
-const char DEBUGGER_INFORMATION_AUTODETECTED[] = "AutoDetected";
+const char DEBUGGER_INFORMATION_AUTODETECTED[] = "AutoDetected"; // FIXME: Merge into DetectionSource
+const char DEBUGGER_INFORMATION_DETECTION_SOURCE[] = "DetectionSource";
 const char DEBUGGER_INFORMATION_VERSION[] = "Version";
 const char DEBUGGER_INFORMATION_ABIS[] = "Abis";
 const char DEBUGGER_INFORMATION_LASTMODIFIED[] = "LastModified";
@@ -66,13 +68,14 @@ const char DEBUGGER_INFORMATION_WORKINGDIRECTORY[] = "WorkingDirectory";
 
 //! Return the configuration of gdb as a list of --key=value
 //! \note That the list will also contain some output not in this format.
-static QString getConfigurationOfGdbCommand(const FilePath &command)
+static QString getGdbConfiguration(const FilePath &command, const Environment &sysEnv)
 {
     // run gdb with the --configuration opion
-    Utils::SynchronousProcess gdbConfigurationCall;
-    Utils::SynchronousProcessResponse output =
-            gdbConfigurationCall.runBlocking({command, {"--configuration"}});
-    return output.allOutput();
+    QtcProcess proc;
+    proc.setEnvironment(sysEnv);
+    proc.setCommand({command, {"--configuration"}});
+    proc.runBlocking();
+    return proc.allOutput();
 }
 
 //! Extract the target ABI identifier from GDB output
@@ -92,8 +95,6 @@ static QString extractGdbTargetAbiStringFromGdbOutput(const QString &gdbOutput)
 }
 
 
-namespace Debugger {
-
 // --------------------------------------------------------------------------
 // DebuggerItem
 // --------------------------------------------------------------------------
@@ -112,6 +113,7 @@ DebuggerItem::DebuggerItem(const QVariantMap &data)
     m_workingDirectory = FilePath::fromVariant(data.value(DEBUGGER_INFORMATION_WORKINGDIRECTORY));
     m_unexpandedDisplayName = data.value(DEBUGGER_INFORMATION_DISPLAYNAME).toString();
     m_isAutoDetected = data.value(DEBUGGER_INFORMATION_AUTODETECTED, false).toBool();
+    m_detectionSource = data.value(DEBUGGER_INFORMATION_DETECTION_SOURCE).toString();
     m_version = data.value(DEBUGGER_INFORMATION_VERSION).toString();
     m_engineType = DebuggerEngineType(data.value(DEBUGGER_INFORMATION_ENGINETYPE,
                                                  static_cast<int>(NoEngineType)).toInt());
@@ -140,57 +142,56 @@ void DebuggerItem::createId()
     m_id = QUuid::createUuid().toString();
 }
 
-static bool isUVisionExecutable(const QFileInfo &fileInfo)
-{
-    if (!HostOsInfo::isWindowsHost())
-        return false;
-    const QString baseName = fileInfo.baseName();
-    return baseName == "UV4";
-}
-
-void DebuggerItem::reinitializeFromFile()
+void DebuggerItem::reinitializeFromFile(const Environment &sysEnv, QString *error)
 {
     // CDB only understands the single-dash -version, whereas GDB and LLDB are
     // happy with both -version and --version. So use the "working" -version
     // except for the experimental LLDB-MI which insists on --version.
     QString version = "-version";
-    const QFileInfo fileInfo = m_command.toFileInfo();
-    m_lastModified = fileInfo.lastModified();
-    if (fileInfo.baseName().toLower().contains("lldb-mi"))
+    m_lastModified = m_command.lastModified();
+    if (m_command.baseName().toLower().contains("lldb-mi")
+            || m_command.baseName().startsWith("LLDBFrontend")) // Comes with Android Studio
         version = "--version";
 
     // We don't need to start the uVision executable to
     // determine its version.
-    if (isUVisionExecutable(fileInfo)) {
+    if (HostOsInfo::isWindowsHost() && m_command.baseName() == "UV4") {
         QString errorMessage;
         m_version = winGetDLLVersion(WinDLLFileVersion,
-                                     fileInfo.absoluteFilePath(),
+                                     m_command.absoluteFilePath().path(),
                                      &errorMessage);
         m_engineType = UvscEngineType;
         m_abis.clear();
         return;
     }
 
+    Environment env = sysEnv.size() == 0 ? Environment::systemEnvironment() : sysEnv;
     // Prevent calling lldb on Windows because the lldb from the llvm package is linked against
     // python but does not contain a python dll.
-    if (HostOsInfo::isWindowsHost() && m_command.fileName().startsWith("lldb")) {
+    const bool isAndroidNdkLldb = DebuggerItem::addAndroidLldbPythonEnv(m_command, env);
+    if (HostOsInfo::isWindowsHost() && m_command.fileName().startsWith("lldb")
+            && !isAndroidNdkLldb) {
         QString errorMessage;
         m_version = winGetDLLVersion(WinDLLFileVersion,
-                                     fileInfo.absoluteFilePath(),
+                                     m_command.absoluteFilePath().path(),
                                      &errorMessage);
         m_engineType = LldbEngineType;
         m_abis = Abi::abisOfBinary(m_command);
         return;
     }
 
-    SynchronousProcess proc;
-    SynchronousProcessResponse response = proc.runBlocking({m_command, {version}});
-    if (response.result != SynchronousProcessResponse::Finished) {
+    QtcProcess proc;
+    proc.setEnvironment(env);
+    proc.setCommand({m_command, {version}});
+    proc.runBlocking();
+    const QString output = proc.allOutput().trimmed();
+    if (proc.result() != QtcProcess::FinishedWithSuccess) {
+        if (error)
+            *error = output;
         m_engineType = NoEngineType;
         return;
     }
     m_abis.clear();
-    const QString output = response.allOutput().trimmed();
     if (output.contains("gdb")) {
         m_engineType = GdbEngineType;
 
@@ -207,8 +208,8 @@ void DebuggerItem::reinitializeFromFile()
         const bool unableToFindAVersion = (0 == version);
         const bool gdbSupportsConfigurationFlag = (version >= 70700);
         if (gdbSupportsConfigurationFlag || unableToFindAVersion) {
-            const auto gdbConfiguration = getConfigurationOfGdbCommand(m_command);
-            const auto gdbTargetAbiString =
+            const QString gdbConfiguration = getGdbConfiguration(m_command, sysEnv);
+            const QString gdbTargetAbiString =
                     extractGdbTargetAbiStringFromGdbOutput(gdbConfiguration);
             if (!gdbTargetAbiString.isEmpty()) {
                 m_abis.append(Abi::abiFromTargetTriplet(gdbTargetAbiString));
@@ -227,19 +228,20 @@ void DebuggerItem::reinitializeFromFile()
             return;
         }
 
-        qWarning() << "Unable to determine gdb target ABI";
+        qWarning() << "Unable to determine gdb target ABI via" << proc.commandLine().toUserOutput();
         //! \note If unable to determine the GDB ABI, no ABI is appended to m_abis here.
         return;
     }
-    if (output.startsWith("lldb") || output.startsWith("LLDB")) {
+    if (output.contains("lldb") || output.startsWith("LLDB")) {
         m_engineType = LldbEngineType;
         m_abis = Abi::abisOfBinary(m_command);
 
         // Version
         // Self-build binaries also emit clang and llvm revision.
         const QString line = output.split('\n')[0];
-        if (line.startsWith(("lldb version "))) { // Linux typically.
-            int pos1 = int(strlen("lldb version "));
+        const QString nonMacOSPrefix = "lldb version ";
+        if (line.contains(nonMacOSPrefix)) { // Linux typically, or some Windows builds.
+            int pos1 = line.indexOf(nonMacOSPrefix) + nonMacOSPrefix.length();
             int pos2 = line.indexOf(' ', pos1);
             m_version = line.mid(pos1, pos2 - pos1);
         } else if (line.startsWith("lldb-") || line.startsWith("LLDB-")) { // Mac typically.
@@ -258,9 +260,27 @@ void DebuggerItem::reinitializeFromFile()
         m_engineType = PdbEngineType;
         return;
     }
+    if (error)
+        *error = output;
     m_engineType = NoEngineType;
 }
 
+bool DebuggerItem::addAndroidLldbPythonEnv(const Utils::FilePath &lldbCmd, Utils::Environment &env)
+{
+    if (lldbCmd.baseName().contains("lldb") &&
+            (lldbCmd.path().contains("/ndk/") || lldbCmd.path().contains("/ndk-bundle/"))) {
+        const FilePath pythonDir = lldbCmd.parentDir().parentDir().pathAppended("python3");
+        const FilePath pythonBinDir =
+                HostOsInfo::isAnyUnixHost() ? pythonDir.pathAppended("bin") : pythonDir;
+        if (pythonBinDir.exists()) {
+            env.set("PYTHONHOME", pythonDir.toUserOutput());
+            env.prependOrSetPath(pythonBinDir.path());
+            return true;
+        }
+    }
+    return false;
+}
+
 QString DebuggerItem::engineTypeName() const
 {
     switch (m_engineType) {
@@ -295,11 +315,11 @@ QDateTime DebuggerItem::lastModified() const
 QIcon DebuggerItem::decoration() const
 {
     if (m_engineType == NoEngineType)
-        return Utils::Icons::CRITICAL.icon();
-    if (!m_command.toFileInfo().isExecutable())
-        return Utils::Icons::WARNING.icon();
+        return Icons::CRITICAL.icon();
+    if (!m_command.isExecutableFile())
+        return Icons::WARNING.icon();
     if (!m_workingDirectory.isEmpty() && !m_workingDirectory.isDir())
-        return Utils::Icons::WARNING.icon();
+        return Icons::WARNING.icon();
     return QIcon();
 }
 
@@ -315,6 +335,7 @@ bool DebuggerItem::operator==(const DebuggerItem &other) const
     return m_id == other.m_id
             && m_unexpandedDisplayName == other.m_unexpandedDisplayName
             && m_isAutoDetected == other.m_isAutoDetected
+            && m_detectionSource == other.m_detectionSource
             && m_command == other.m_command
             && m_workingDirectory == other.m_workingDirectory;
 }
@@ -328,6 +349,7 @@ QVariantMap DebuggerItem::toMap() const
     data.insert(DEBUGGER_INFORMATION_WORKINGDIRECTORY, m_workingDirectory.toVariant());
     data.insert(DEBUGGER_INFORMATION_ENGINETYPE, int(m_engineType));
     data.insert(DEBUGGER_INFORMATION_AUTODETECTED, m_isAutoDetected);
+    data.insert(DEBUGGER_INFORMATION_DETECTION_SOURCE, m_detectionSource);
     data.insert(DEBUGGER_INFORMATION_VERSION, m_version);
     data.insert(DEBUGGER_INFORMATION_ABIS, abiNames());
     data.insert(DEBUGGER_INFORMATION_LASTMODIFIED, m_lastModified);
