@@ -52,6 +52,12 @@ public:
     static QVariant registerDebugger(const DebuggerItem &item);
     static void deregisterDebugger(const QVariant &id);
 
+    static void autoDetectDebuggersForDevice(const Utils::FilePath &deviceRoot,
+                                             const QString &detectionSource,
+                                             QString *logMessage);
+    static void removeDetectedDebuggers(const QString &detectionSource, QString *logMessage);
+    static void listDetectedDebuggers(const QString &detectionSource, QString *logMessage);
+
     static const DebuggerItem *findByCommand(const Utils::FilePath &command);
     static const DebuggerItem *findById(const QVariant &id);
     static const DebuggerItem *findByEngineType(DebuggerEngineType engineType);
