@@ -32,7 +32,7 @@
 #include <QJsonObject>
 #include <QVector>
 
-#include <functional>
+#include <utils/fileutils.h>
 
 namespace Utils { class ProcessHandle; }
 
@@ -45,9 +45,9 @@ class DebuggerResponse;
 class DebuggerCommand
 {
 public:
-    typedef std::function<void(const DebuggerResponse &)> Callback;
+    using Callback = std::function<void (const DebuggerResponse &)>;
 
-    DebuggerCommand() {}
+    DebuggerCommand() = default;
     DebuggerCommand(const QString &f) : function(f) {}
     DebuggerCommand(const QString &f, const QJsonValue &a) : function(f), args(a) {}
     DebuggerCommand(const QString &f, int fl) : function(f), flags(fl) {}
@@ -64,6 +64,7 @@ public:
     void arg(const char *name, const QList<int> &list);
     void arg(const char *name, const QStringList &list); // Note: Hex-encodes.
     void arg(const char *name, const QJsonValue &value);
+    void arg(const char *name, const Utils::FilePath &filePath);
 
     QString argsToPython() const;
     QString argsToString() const;
@@ -84,8 +85,6 @@ public:
         ExitRequest = 32,
         // Auto-set inferior shutdown related states.
         LosesChild = 64,
-        // Trigger breakpoint model rebuild when no such commands are pending anymore.
-        RebuildBreakpointModel = 128,
         // This is a native (non-Python) command that's handled directly by the backend.
         NativeCommand = 256,
         // This is a command that needs to be wrapped into -interpreter-exec console
@@ -111,7 +110,7 @@ private:
 class DebuggerCommandSequence
 {
 public:
-    DebuggerCommandSequence() {}
+    DebuggerCommandSequence() = default;
     bool isEmpty() const { return m_commands.isEmpty(); }
     bool wantContinue() const { return m_continue; }
     const QList<DebuggerCommand> &commands() const { return m_commands; }
@@ -125,19 +124,48 @@ public:
     bool m_continue = false;
 };
 
-// FIXME: rename into GdbMiValue
+class DebuggerOutputParser
+{
+public:
+    explicit DebuggerOutputParser(const QString &output);
+
+    QChar current() const { return *from; }
+    bool isCurrent(QChar c) const { return *from == c; }
+    bool isAtEnd() const { return from >= to; }
+
+    void advance() { ++from; }
+    void advance(int n) { from += n; }
+    QChar lookAhead(int offset) const { return from[offset]; }
+
+    int readInt();
+    QChar readChar();
+    QString readCString();
+    QString readString(const std::function<bool(char)> &isValidChar);
+
+    QString buffer() const { return QString(from, to - from); }
+    int remainingChars() const { return int(to - from); }
+
+    void skipCommas();
+    void skipSpaces();
+
+private:
+    const QChar *from = nullptr;
+    const QChar *to = nullptr;
+};
+
 class GdbMi
 {
 public:
-    GdbMi() : m_type(Invalid) {}
+    GdbMi() = default;
 
     QString m_name;
     QString m_data;
-    QVector<GdbMi> m_children;
 
+    using Children = QVector<GdbMi>;
     enum Type { Invalid, Const, Tuple, List };
+    Type m_type = Invalid;
 
-    Type m_type;
+    void addChild(const GdbMi &child) { m_children.push_back(child); }
 
     Type type() const { return m_type; }
     const QString &name() const { return m_name; }
@@ -147,7 +175,8 @@ public:
     bool isList() const { return m_type == List; }
 
     const QString &data() const { return m_data; }
-    const QVector<GdbMi> &children() const { return m_children; }
+    Children::const_iterator begin() const { return m_children.begin(); }
+    Children::const_iterator end() const { return m_children.end(); }
     int childCount() const { return int(m_children.size()); }
 
     const GdbMi &childAt(int index) const { return m_children[index]; }
@@ -161,16 +190,16 @@ public:
     void fromString(const QString &str);
     void fromStringMultiple(const QString &str);
 
-    static QString parseCString(const QChar *&from, const QChar *to);
     static QString escapeCString(const QString &ba);
-    void parseResultOrValue(const QChar *&from, const QChar *to);
-    void parseValue(const QChar *&from, const QChar *to);
-    void parseTuple(const QChar *&from, const QChar *to);
-    void parseTuple_helper(const QChar *&from, const QChar *to);
-    void parseList(const QChar *&from, const QChar *to);
+    void parseResultOrValue(DebuggerOutputParser &state);
+    void parseValue(DebuggerOutputParser &state);
+    void parseTuple(DebuggerOutputParser &state);
+    void parseTuple_helper(DebuggerOutputParser &state);
+    void parseList(DebuggerOutputParser &state);
 
 private:
     void dumpChildren(QString *str, bool multiline, int indent) const;
+    Children m_children;
 };
 
 QString fromHex(const QString &str);
@@ -191,12 +220,12 @@ enum ResultClass
 class DebuggerResponse
 {
 public:
-    DebuggerResponse() : token(-1), resultClass(ResultUnknown) {}
+    DebuggerResponse() = default;
     QString toString() const;
     static QString stringFromResultClass(ResultClass resultClass);
 
-    int         token;
-    ResultClass resultClass;
+    int         token = -1;
+    ResultClass resultClass = ResultUnknown;
     GdbMi       data;
     QString     logStreamOutput;
     QString     consoleStreamOutput;
@@ -226,7 +255,7 @@ public:
         DateTimeInternal,
     };
 
-    DebuggerEncoding() {}
+    DebuggerEncoding() = default;
     explicit DebuggerEncoding(const QString &data);
     QString toString() const;
 
@@ -303,6 +332,20 @@ const char DisplayImageFile[]    = "imagefile:separate";
 const char DisplayPlotData[]     = "plotdata:separate";
 const char DisplayArrayData[]    = "arraydata:separate";
 
+enum LocationType { UnknownLocation, LocationByFile, LocationByAddress };
+
+class ContextData
+{
+public:
+    bool isValid() const { return type != UnknownLocation; }
+
+public:
+    LocationType type = UnknownLocation;
+    Utils::FilePath fileName;
+    int lineNumber = 0;
+    quint64 address = 0;
+};
+
 } // namespace Internal
 } // namespace Debugger
 
