@@ -42,9 +42,9 @@
 #include <texteditor/textdocument.h>
 #include <texteditor/texteditor.h>
 
+#include <utils/aspects.h>
 #include <utils/mimetypes/mimedatabase.h>
 #include <utils/qtcassert.h>
-#include <utils/savedaction.h>
 
 #include <QTextBlock>
 #include <QDir>
@@ -181,7 +181,7 @@ int DisassemblerAgentPrivate::lineForAddress(quint64 address) const
 DisassemblerAgent::DisassemblerAgent(DebuggerEngine *engine)
     : d(new DisassemblerAgentPrivate(engine))
 {
-    connect(action(IntelFlavor), &Utils::SavedAction::valueChanged,
+    connect(&debuggerSettings()->intelFlavor, &Utils::BaseAspect::changed,
             this, &DisassemblerAgent::reload);
 }
 
@@ -368,7 +368,7 @@ void DisassemblerAgent::removeBreakpointMarker(const Breakpoint &bp)
     if (!d->document)
         return;
 
-    for (DisassemblerBreakpointMarker *marker : d->breakpointMarks) {
+    for (DisassemblerBreakpointMarker *marker : qAsConst(d->breakpointMarks)) {
         if (marker->m_bp == bp) {
             d->breakpointMarks.removeOne(marker);
             d->document->removeMark(marker);
