@@ -50,7 +50,6 @@
 #include <coreplugin/icore.h>
 
 #include <utils/qtcassert.h>
-#include <utils/savedaction.h>
 #include <utils/qtcprocess.h>
 
 #include <QApplication>
@@ -83,28 +82,30 @@ static int &currentToken()
 ///////////////////////////////////////////////////////////////////////
 
 LldbEngine::LldbEngine()
+    : m_lldbProc(ProcessMode::Writer)
 {
     setObjectName("LldbEngine");
     setDebuggerName("LLDB");
 
-    connect(action(AutoDerefPointers), &SavedAction::valueChanged,
+    DebuggerSettings &ds = *debuggerSettings();
+    connect(&ds.autoDerefPointers, &BaseAspect::changed,
             this, &LldbEngine::updateLocals);
-    connect(action(CreateFullBacktrace)->action(), &QAction::triggered,
+    connect(ds.createFullBacktrace.action(), &QAction::triggered,
             this, &LldbEngine::fetchFullBacktrace);
-    connect(action(UseDebuggingHelpers), &SavedAction::valueChanged,
+    connect(&ds.useDebuggingHelpers, &BaseAspect::changed,
             this, &LldbEngine::updateLocals);
-    connect(action(UseDynamicType), &SavedAction::valueChanged,
+    connect(&ds.useDynamicType, &BaseAspect::changed,
             this, &LldbEngine::updateLocals);
-    connect(action(IntelFlavor), &SavedAction::valueChanged,
+    connect(&ds.intelFlavor, &BaseAspect::changed,
             this, &LldbEngine::updateAll);
 
-    connect(&m_lldbProc, &QProcess::errorOccurred,
+    connect(&m_lldbProc, &QtcProcess::errorOccurred,
             this, &LldbEngine::handleLldbError);
-    connect(&m_lldbProc, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
+    connect(&m_lldbProc, &QtcProcess::finished,
             this, &LldbEngine::handleLldbFinished);
-    connect(&m_lldbProc, &QProcess::readyReadStandardOutput,
+    connect(&m_lldbProc, &QtcProcess::readyReadStandardOutput,
             this, &LldbEngine::readLldbStandardOutput);
-    connect(&m_lldbProc, &QProcess::readyReadStandardError,
+    connect(&m_lldbProc, &QtcProcess::readyReadStandardError,
             this, &LldbEngine::readLldbStandardError);
 
     connect(this, &LldbEngine::outputReady,
@@ -207,16 +208,22 @@ void LldbEngine::setupEngine()
 {
     QTC_ASSERT(state() == EngineSetupRequested, qDebug() << state());
 
-    const FilePath lldbCmd = runParameters().debugger.executable;
+    const FilePath lldbCmd = runParameters().debugger.command.executable();
 
     showMessage("STARTING LLDB: " + lldbCmd.toUserOutput());
     Environment environment = runParameters().debugger.environment;
     environment.appendOrSet("PYTHONUNBUFFERED", "1");  // avoid flushing problem on macOS
+    DebuggerItem::addAndroidLldbPythonEnv(lldbCmd, environment);
     m_lldbProc.setEnvironment(environment);
-    if (QFileInfo(runParameters().debugger.workingDirectory).isDir())
+
+    if (runParameters().debugger.workingDirectory.isDir())
         m_lldbProc.setWorkingDirectory(runParameters().debugger.workingDirectory);
 
-    m_lldbProc.setCommand(CommandLine(lldbCmd));
+    if (HostOsInfo::isRunningUnderRosetta())
+        m_lldbProc.setCommand(CommandLine("/usr/bin/arch", {"-arm64", lldbCmd.toString()}));
+    else
+        m_lldbProc.setCommand(CommandLine(lldbCmd));
+
     m_lldbProc.start();
 
     if (!m_lldbProc.waitForStarted()) {
@@ -232,10 +239,9 @@ void LldbEngine::setupEngine()
 
     showStatusMessage(tr("Setting up inferior..."));
 
-    const QByteArray dumperSourcePath =
-        ICore::resourcePath().toLocal8Bit() + "/debugger/";
+    const DebuggerRunParameters &rp = runParameters();
 
-    executeCommand("script sys.path.insert(1, '" + dumperSourcePath + "')");
+    executeCommand("script sys.path.insert(1, '" + rp.dumperPath.path().toLocal8Bit() + "')");
     // This triggers reportState("enginesetupok") or "enginesetupfailed":
     executeCommand("script from lldbbridge import *");
 
@@ -244,14 +250,14 @@ void LldbEngine::setupEngine()
         executeCommand(commands.toLocal8Bit());
 
 
-    const QString path = stringSetting(ExtraDumperFile);
+    const QString path = debuggerSettings()->extraDumperFile.value();
     if (!path.isEmpty() && QFileInfo(path).isReadable()) {
         DebuggerCommand cmd("addDumperModule");
         cmd.arg("path", path);
         runCommand(cmd);
     }
 
-    commands = stringSetting(ExtraDumperCommands);
+    commands = debuggerSettings()->extraDumperCommands.value();
     if (!commands.isEmpty()) {
         DebuggerCommand cmd("executeDebuggerCommand");
         cmd.arg("command", commands);
@@ -264,11 +270,8 @@ void LldbEngine::setupEngine()
     };
     runCommand(cmd1);
 
-    const DebuggerRunParameters &rp = runParameters();
-
     const SourcePathMap sourcePathMap =
-        DebuggerSourcePathMappingWidget::mergePlatformQtPath(rp,
-                Internal::globalDebuggerOptions()->sourcePathMap);
+            mergePlatformQtPath(rp, debuggerSettings()->sourcePathMap.value());
     for (auto it = sourcePathMap.constBegin(), cend = sourcePathMap.constEnd();
          it != cend;
          ++it) {
@@ -277,14 +280,14 @@ void LldbEngine::setupEngine()
     }
 
     DebuggerCommand cmd2("setupInferior");
-    cmd2.arg("executable", rp.inferior.executable.toString());
+    cmd2.arg("executable", rp.inferior.command.executable().toString());
     cmd2.arg("breakonmain", rp.breakOnMain);
     cmd2.arg("useterminal", bool(terminal()));
     cmd2.arg("startmode", rp.startMode);
     cmd2.arg("nativemixed", isNativeMixedActive());
     cmd2.arg("workingdirectory", rp.inferior.workingDirectory);
     cmd2.arg("environment", rp.inferior.environment.toStringList());
-    cmd2.arg("processargs", toHex(QtcProcess::splitArgs(rp.inferior.commandLineArguments).join(QChar(0))));
+    cmd2.arg("processargs", toHex(ProcessArgs::splitArgs(rp.inferior.command.arguments()).join(QChar(0))));
     cmd2.arg("platform", rp.platform);
     cmd2.arg("symbolfile", rp.symbolFile);
 
@@ -302,8 +305,8 @@ void LldbEngine::setupEngine()
         cmd2.arg("startmode", rp.startMode);
         // it is better not to check the start mode on the python sid (as we would have to duplicate the
         // enum values), and thus we assume that if the rp.attachPID is valid we really have to attach
-        QTC_CHECK(!rp.attachPID.isValid() || (rp.startMode == AttachCrashedExternal
-                                              || rp.startMode == AttachExternal));
+        QTC_CHECK(!rp.attachPID.isValid() || (rp.startMode == AttachToCrashedProcess
+                                              || rp.startMode == AttachToLocalProcess));
         cmd2.arg("attachpid", rp.attachPID.pid());
         cmd2.arg("sysroot", rp.deviceSymbolsRoot.isEmpty() ? rp.sysRoot.toString()
                                                            : rp.deviceSymbolsRoot);
@@ -312,7 +315,7 @@ void LldbEngine::setupEngine()
                                   ? rp.remoteChannel : QString()));
         cmd2.arg("platform", rp.platform);
         QTC_CHECK(!rp.continueAfterAttach || (rp.startMode == AttachToRemoteProcess
-                                              || rp.startMode == AttachExternal
+                                              || rp.startMode == AttachToLocalProcess
                                               || rp.startMode == AttachToRemoteServer));
         m_continueAtNextSpontaneousStop = false;
     }
@@ -324,7 +327,10 @@ void LldbEngine::setupEngine()
             // Some extra roundtrip to make sure we end up behind all commands triggered
             // from claimBreakpointsForEngine().
             DebuggerCommand cmd3("executeRoundtrip");
-            cmd3.callback = [this](const DebuggerResponse &) { notifyEngineSetupOk(); };
+            cmd3.callback = [this](const DebuggerResponse &) {
+                notifyEngineSetupOk();
+                runEngine();
+            };
             runCommand(cmd3);
         } else {
             notifyEngineSetupFailed();
@@ -333,6 +339,10 @@ void LldbEngine::setupEngine()
 
     cmd2.flags = Silent;
     runCommand(cmd2);
+
+    DebuggerCommand cmd0("setFallbackQtVersion");
+    cmd0.arg("version", rp.fallbackQtVersion);
+    runCommand(cmd0);
 }
 
 void LldbEngine::runEngine()
@@ -341,7 +351,7 @@ void LldbEngine::runEngine()
     QTC_ASSERT(state() == EngineRunRequested, qDebug() << state(); return);
     showStatusMessage(tr("Running requested..."), 5000);
     DebuggerCommand cmd("runEngine");
-    if (rp.startMode == AttachCore)
+    if (rp.startMode == AttachToCore)
         cmd.arg("coreFile", rp.coreFile);
     runCommand(cmd);
 }
@@ -477,28 +487,14 @@ void LldbEngine::selectThread(const Thread &thread)
     DebuggerCommand cmd("selectThread");
     cmd.arg("id", thread->id());
     cmd.callback = [this](const DebuggerResponse &) {
-        fetchStack(action(MaximalStackDepth)->value().toInt());
+        fetchStack(debuggerSettings()->maximalStackDepth.value());
     };
     runCommand(cmd);
 }
 
-bool LldbEngine::stateAcceptsBreakpointChanges() const
-{
-    switch (state()) {
-    case EngineSetupRequested:
-    case InferiorRunRequested:
-    case InferiorRunOk:
-    case InferiorStopRequested:
-    case InferiorStopOk:
-        return true;
-    default:
-        return false;
-    }
-}
-
 bool LldbEngine::acceptsBreakpoint(const BreakpointParameters &bp) const
 {
-    if (runParameters().startMode == AttachCore)
+    if (runParameters().startMode == AttachToCore)
         return false;
     if (bp.isCppBreakpoint())
         return true;
@@ -722,33 +718,12 @@ void LldbEngine::updateAll()
     DebuggerCommand cmd("fetchThreads");
     cmd.callback = [this](const DebuggerResponse &response) {
         threadsHandler()->setThreads(response.data);
-        fetchStack(action(MaximalStackDepth)->value().toInt());
+        fetchStack(debuggerSettings()->maximalStackDepth.value());
         reloadRegisters();
     };
     runCommand(cmd);
 }
 
-void LldbEngine::reloadFullStack()
-{
-    fetchStack(-1);
-}
-
-void LldbEngine::fetchStack(int limit)
-{
-    DebuggerCommand cmd("fetchStack");
-    cmd.arg("nativemixed", isNativeMixedActive());
-    cmd.arg("stacklimit", limit);
-    cmd.arg("context", stackHandler()->currentFrame().context);
-    cmd.callback = [this](const DebuggerResponse &response) {
-        const GdbMi &stack = response.data["stack"];
-        const bool isFull = !stack["hasmore"].toInt();
-        stackHandler()->setFramesAndCurrentIndex(stack["frames"], isFull);
-        activateFrame(stackHandler()->currentIndex());
-    };
-    runCommand(cmd);
-}
-
-
 //////////////////////////////////////////////////////////////////////
 //
 // Watch specific stuff
@@ -776,20 +751,21 @@ void LldbEngine::doUpdateLocals(const UpdateParameters &params)
     watchHandler()->appendWatchersAndTooltipRequests(&cmd);
 
     const static bool alwaysVerbose = qEnvironmentVariableIsSet("QTC_DEBUGGER_PYTHON_VERBOSE");
+    const DebuggerSettings &s = *debuggerSettings();
     cmd.arg("passexceptions", alwaysVerbose);
-    cmd.arg("fancy", boolSetting(UseDebuggingHelpers));
-    cmd.arg("autoderef", boolSetting(AutoDerefPointers));
-    cmd.arg("dyntype", boolSetting(UseDynamicType));
+    cmd.arg("fancy", s.useDebuggingHelpers.value());
+    cmd.arg("autoderef", s.autoDerefPointers.value());
+    cmd.arg("dyntype", s.useDynamicType.value());
     cmd.arg("partialvar", params.partialVariable);
-    cmd.arg("qobjectnames", boolSetting(ShowQObjectNames));
-    cmd.arg("timestamps", boolSetting(LogTimeStamps));
+    cmd.arg("qobjectnames", s.showQObjectNames.value());
+    cmd.arg("timestamps", s.logTimeStamps.value());
 
     StackFrame frame = stackHandler()->currentFrame();
     cmd.arg("context", frame.context);
     cmd.arg("nativemixed", isNativeMixedActive());
 
-    cmd.arg("stringcutoff", action(MaximalStringLength)->value().toString());
-    cmd.arg("displaystringlimit", action(DisplayStringLimit)->value().toString());
+    cmd.arg("stringcutoff", s.maximalStringLength.value());
+    cmd.arg("displaystringlimit", s.displayStringLimit.value());
 
     //cmd.arg("resultvarname", m_resultVarName);
     cmd.arg("partialvar", params.partialVariable);
@@ -833,7 +809,7 @@ QString LldbEngine::errorMessage(QProcess::ProcessError error) const
             return tr("The LLDB process failed to start. Either the "
                 "invoked program \"%1\" is missing, or you may have insufficient "
                 "permissions to invoke the program.")
-                .arg(runParameters().debugger.executable.toUserOutput());
+                .arg(runParameters().debugger.command.executable().toUserOutput());
         case QProcess::Crashed:
             return tr("The LLDB process crashed some time after starting "
                 "successfully.");
@@ -853,9 +829,9 @@ QString LldbEngine::errorMessage(QProcess::ProcessError error) const
     }
 }
 
-void LldbEngine::handleLldbFinished(int exitCode, QProcess::ExitStatus exitStatus)
+void LldbEngine::handleLldbFinished()
 {
-    notifyDebuggerProcessFinished(exitCode, exitStatus, "LLDB");
+    notifyDebuggerProcessFinished(m_lldbProc.exitCode(), m_lldbProc.exitStatus(), "LLDB");
 }
 
 void LldbEngine::readLldbStandardError()
@@ -932,7 +908,7 @@ void LldbEngine::handleStateNotification(const GdbMi &item)
         continueInferior();
     } else if (newState == "enginerunokandinferiorunrunnable") {
         notifyEngineRunOkAndInferiorUnrunnable();
-        if (runParameters().startMode == AttachCore)
+        if (runParameters().startMode == AttachToCore)
             handleAttachedToCore();
     } else if (newState == "inferiorshutdownfinished")
         notifyInferiorShutdownFinished();
@@ -992,6 +968,33 @@ void LldbEngine::reloadDebuggingHelpers()
     updateAll();
 }
 
+void LldbEngine::loadAdditionalQmlStack()
+{
+    fetchStack(-1, true);
+}
+
+void LldbEngine::reloadFullStack()
+{
+    fetchStack(-1, false);
+}
+
+void LldbEngine::fetchStack(int limit, bool extraQml)
+{
+    DebuggerCommand cmd("fetchStack");
+    cmd.arg("nativemixed", isNativeMixedActive());
+    cmd.arg("stacklimit", limit);
+    cmd.arg("context", stackHandler()->currentFrame().context);
+    cmd.arg("extraqml", int(extraQml));
+    cmd.callback = [this](const DebuggerResponse &response) {
+        const GdbMi &stack = response.data["stack"];
+        const bool isFull = !stack["hasmore"].toInt();
+        stackHandler()->setFramesAndCurrentIndex(stack["frames"], isFull);
+        activateFrame(stackHandler()->currentIndex());
+    };
+    runCommand(cmd);
+}
+
+
 void LldbEngine::fetchDisassembler(DisassemblerAgent *agent)
 {
     QPointer<DisassemblerAgent> p(agent);
@@ -1004,7 +1007,7 @@ void LldbEngine::fetchDisassembler(DisassemblerAgent *agent)
     DebuggerCommand cmd("fetchDisassembler");
     cmd.arg("address", loc.address());
     cmd.arg("function", loc.functionName());
-    cmd.arg("flavor", boolSetting(IntelFlavor) ? "intel" : "att");
+    cmd.arg("flavor", debuggerSettings()->intelFlavor.value() ? "intel" : "att");
     cmd.callback = [this, id](const DebuggerResponse &response) {
         DisassemblerLines result;
         QPointer<DisassemblerAgent> agent = m_disassemblerAgents.key(id);
@@ -1102,10 +1105,11 @@ bool LldbEngine::hasCapability(unsigned cap) const
         | OperateByInstructionCapability
         | RunToLineCapability
         | WatchComplexExpressionsCapability
-        | MemoryAddressCapability))
+        | MemoryAddressCapability
+        | AdditionalQmlStackCapability))
         return true;
 
-    if (runParameters().startMode == AttachCore)
+    if (runParameters().startMode == AttachToCore)
         return false;
 
     //return cap == SnapshotCapability;
