@@ -501,34 +501,11 @@ void QmlEngine::closeConnection()
     }
 }
 
-void QmlEngine::runEngine()
-{
-    // we won't get any debug output
-    if (!terminal()) {
-        d->retryOnConnectFail = true;
-        d->automaticConnect = true;
-    }
-
-    QTC_ASSERT(state() == EngineRunRequested, qDebug() << state());
-
-    if (isPrimaryEngine()) {
-        // QML only.
-        if (runParameters().startMode == AttachToRemoteServer)
-            tryToConnect();
-        else if (runParameters().startMode == AttachToRemoteProcess)
-            beginConnection();
-        else
-            startApplicationLauncher();
-    } else {
-        tryToConnect();
-    }
-}
-
 void QmlEngine::startApplicationLauncher()
 {
     if (!d->applicationLauncher.isRunning()) {
         const Runnable runnable = runParameters().inferior;
-        showMessage(tr("Starting %1").arg(runnable.commandLine().toUserOutput()),
+        showMessage(tr("Starting %1").arg(runnable.command.toUserOutput()),
                     NormalMessageFormat);
         d->applicationLauncher.start(runnable);
     }
@@ -576,6 +553,26 @@ void QmlEngine::setupEngine()
 {
     notifyEngineSetupOk();
 
+    // we won't get any debug output
+    if (!terminal()) {
+        d->retryOnConnectFail = true;
+        d->automaticConnect = true;
+    }
+
+    QTC_ASSERT(state() == EngineRunRequested, qDebug() << state());
+
+    if (isPrimaryEngine()) {
+        // QML only.
+        if (runParameters().startMode == AttachToRemoteServer)
+            tryToConnect();
+        else if (runParameters().startMode == AttachToRemoteProcess)
+            beginConnection();
+        else
+            startApplicationLauncher();
+    } else {
+        tryToConnect();
+    }
+
     if (d->automaticConnect)
         beginConnection();
 }
@@ -865,7 +862,7 @@ bool compareConsoleItems(const ConsoleItem *a, const ConsoleItem *b)
 static ConsoleItem *constructLogItemTree(const QVariant &result,
                                          const QString &key = QString())
 {
-    bool sorted = boolSetting(SortStructMembers);
+    bool sorted = debuggerSettings()->sortStructMembers.value();
     if (!result.isValid())
         return nullptr;
 
@@ -1851,7 +1848,7 @@ void QmlEnginePrivate::messageReceived(const QByteArray &data)
 
                         int newColumn = sourceLineText.indexOf('(') + 1;
 
-                        for (const Breakpoint &bp : v8Breakpoints) {
+                        for (const Breakpoint &bp : qAsConst(v8Breakpoints)) {
                             QTC_ASSERT(bp, continue);
                             const BreakpointParameters &params = bp->requestedParameters();
 
@@ -1877,7 +1874,7 @@ void QmlEnginePrivate::messageReceived(const QByteArray &data)
 
                     if (inferiorStop) {
                         //Update breakpoint data
-                        for (const Breakpoint &bp : v8Breakpoints) {
+                        for (const Breakpoint &bp : qAsConst(v8Breakpoints)) {
                             QTC_ASSERT(bp, continue);
                             if (bp->functionName().isEmpty()) {
                                 bp->setFunctionName(invocationText);
@@ -1890,7 +1887,7 @@ void QmlEnginePrivate::messageReceived(const QByteArray &data)
                         }
 
                         if (engine->state() == InferiorRunOk) {
-                            for (const Breakpoint &bp : v8Breakpoints) {
+                            for (const Breakpoint &bp : qAsConst(v8Breakpoints)) {
                                 QTC_ASSERT(bp, continue);
                                 if (breakpointsTemp.contains(bp->responseId()))
                                     clearBreakpoint(bp);
@@ -2039,8 +2036,9 @@ StackFrame QmlEnginePrivate::extractStackFrame(const QVariant &bodyVal)
     stackFrame.function = extractString(body.value("func"));
     if (stackFrame.function.isEmpty())
         stackFrame.function = QCoreApplication::translate("QmlEngine", "Anonymous Function");
-    stackFrame.file = engine->toFileInProject(extractString(body.value("script")));
-    stackFrame.usable = QFileInfo(stackFrame.file).isReadable();
+    stackFrame.file = FilePath::fromString(
+        engine->toFileInProject(extractString(body.value("script"))));
+    stackFrame.usable = stackFrame.file.isReadableFile();
     stackFrame.receiver = extractString(body.value("receiver"));
     stackFrame.line = body.value("line").toInt() + 1;
 
@@ -2136,7 +2134,7 @@ void QmlEnginePrivate::handleFrame(const QVariantMap &response)
 
     // Send watchers list
     if (stackHandler->isContentsValid() && stackHandler->currentFrame().isUsable()) {
-        const QStringList watchers = watchHandler->watchedExpressions();
+        const QStringList watchers = WatchHandler::watchedExpressions();
         for (const QString &exp : watchers) {
             const QString iname = watchHandler->watcherName(exp);
             evaluate(exp, -1, [this, iname, exp](const QVariantMap &response) {
@@ -2229,7 +2227,7 @@ void QmlEnginePrivate::constructChildLogItems(ConsoleItem *item, const QmlV8Obje
     for (const QVariant &property : objectData.properties)
         *(it++) = constructLogItemTree(extractData(property), seenHandles);
 
-    if (boolSetting(SortStructMembers))
+    if (debuggerSettings()->sortStructMembers.value())
         std::sort(children.begin(), children.end(), compareConsoleItems);
 
     foreach (ConsoleItem *child, children)
@@ -2342,7 +2340,7 @@ void QmlEnginePrivate::insertSubItems(WatchItem *parent, const QVariantList &pro
         parent->appendChild(item.release());
     }
 
-    if (boolSetting(SortStructMembers)) {
+    if (debuggerSettings()->sortStructMembers.value()) {
         parent->sortChildren([](const WatchItem *item1, const WatchItem *item2) {
             return item1->name < item2->name;
         });
