@@ -23,10 +23,6 @@
 **
 ****************************************************************************/
 
-#ifndef _WIN32_WINNT
-#define _WIN32_WINNT 0x0400
-#endif
-
 #include "registerpostmortemaction.h"
 
 #include <registryaccess.h>
@@ -35,6 +31,9 @@
 #include <QDir>
 #include <QString>
 
+#ifdef QTCREATOR_PCH_H
+#define CALLBACK WINAPI
+#endif
 #include <windows.h>
 #include <objbase.h>
 #include <shellapi.h>
@@ -44,9 +43,8 @@ using namespace RegistryAccess;
 namespace Debugger {
 namespace Internal {
 
-void RegisterPostMortemAction::registerNow(const QVariant &value)
+void RegisterPostMortemAction::registerNow(bool value)
 {
-    const bool boolValue = value.toBool();
     const QString debuggerExe = QDir::toNativeSeparators(QCoreApplication::applicationDirPath() + '/'
                                 + QLatin1String(debuggerApplicationFileC) + ".exe");
     const ushort *debuggerWString = debuggerExe.utf16();
@@ -58,7 +56,7 @@ void RegisterPostMortemAction::registerNow(const QVariant &value)
     shExecInfo.hwnd   = NULL;
     shExecInfo.lpVerb = L"runas";
     shExecInfo.lpFile = reinterpret_cast<LPCWSTR>(debuggerWString);
-    shExecInfo.lpParameters = boolValue ? L"-register" : L"-unregister";
+    shExecInfo.lpParameters = value ? L"-register" : L"-unregister";
     shExecInfo.lpDirectory  = NULL;
     shExecInfo.nShow        = SW_SHOWNORMAL;
     shExecInfo.hProcess     = NULL;
@@ -68,9 +66,9 @@ void RegisterPostMortemAction::registerNow(const QVariant &value)
     readSettings();
 }
 
-RegisterPostMortemAction::RegisterPostMortemAction(QObject *parent) : Utils::SavedAction(parent)
+RegisterPostMortemAction::RegisterPostMortemAction()
 {
-    connect(this, &SavedAction::valueChanged, this, &RegisterPostMortemAction::registerNow);
+    connect(this, &BoolAspect::valueChanged, this, &RegisterPostMortemAction::registerNow);
 }
 
 void RegisterPostMortemAction::readSettings(const QSettings *)
@@ -84,7 +82,7 @@ void RegisterPostMortemAction::readSettings(const QSettings *)
         registered = isRegistered(handle, debuggerCall(), &errorMessage);
     if (handle)
         RegCloseKey(handle);
-    setValue(registered, false);
+    setValueQuietly(registered);
 }
 
 } // namespace Internal
