@@ -62,23 +62,21 @@ public:
 public:
     QPointer<BaseTextEditor> editor;
     QPointer<DebuggerEngine> engine;
-    TextMark *locationMark;
+    TextMark *locationMark = nullptr;
     QString path;
     QString producer;
 };
 
 SourceAgentPrivate::SourceAgentPrivate()
-  : editor(0)
-  , locationMark(nullptr)
-  , producer(QLatin1String("remote"))
+  : producer("remote")
 {
 }
 
 SourceAgentPrivate::~SourceAgentPrivate()
 {
     if (editor)
-        EditorManager::closeDocument(editor->document());
-    editor = 0;
+        EditorManager::closeDocuments({editor->document()});
+    editor = nullptr;
     delete locationMark;
 }
 
@@ -105,8 +103,8 @@ void SourceAgent::setContent(const QString &filePath, const QString &content)
     d->path = filePath;
 
     if (!d->editor) {
-        QString titlePattern = d->producer + QLatin1String(": ")
-            + Utils::FileName::fromString(filePath).fileName();
+        QString titlePattern = d->producer + ": "
+            + Utils::FilePath::fromString(filePath).fileName();
         d->editor = qobject_cast<BaseTextEditor *>(
             EditorManager::openEditorWithContents(
                 CppEditor::Constants::CPPEDITOR_ID,
@@ -135,11 +133,11 @@ void SourceAgent::updateLocationMarker()
     if (d->locationMark)
         d->editor->textDocument()->removeMark(d->locationMark);
     delete d->locationMark;
-    d->locationMark = 0;
-    if (d->engine->stackHandler()->currentFrame().file == d->path) {
+    d->locationMark = nullptr;
+    if (d->engine->stackHandler()->currentFrame().file == Utils::FilePath::fromString(d->path)) {
         int lineNumber = d->engine->stackHandler()->currentFrame().line;
 
-        d->locationMark = new TextMark(QString(), lineNumber,
+        d->locationMark = new TextMark(Utils::FilePath(), lineNumber,
                                        Constants::TEXT_MARK_CATEGORY_LOCATION);
         d->locationMark->setIcon(Icons::LOCATION.icon());
         d->locationMark->setPriority(TextMark::HighPriority);
