@@ -40,7 +40,6 @@
 #include <utils/basetreeview.h>
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
-#include <utils/savedaction.h>
 
 #include <QApplication>
 #include <QClipboard>
@@ -52,6 +51,7 @@
 #include <QInputDialog>
 #include <QMenu>
 #include <QTextStream>
+#include <QTimer>
 
 using namespace Utils;
 
@@ -70,10 +70,10 @@ StackHandler::StackHandler(DebuggerEngine *engine)
     setObjectName("StackModel");
     setHeader({tr("Level"), tr("Function"), tr("File"), tr("Line"), tr("Address") });
 
-    connect(action(ExpandStack), &QAction::triggered,
-        this, &StackHandler::reloadFullStack);
-    connect(action(MaximalStackDepth), &QAction::triggered,
-        this, &StackHandler::reloadFullStack);
+    connect(debuggerSettings()->expandStack.action(), &QAction::triggered,
+            this, &StackHandler::reloadFullStack);
+    connect(debuggerSettings()->maximalStackDepth.action(), &QAction::triggered,
+            this, &StackHandler::reloadFullStack);
 
     // For now there's always only "the" current thread.
     rootItem()->appendChild(new ThreadDummyItem);
@@ -101,7 +101,7 @@ QVariant StackFrameItem::data(int column, int role) const
         case StackFunctionNameColumn:
             return simplifyType(frame.function);
         case StackFileNameColumn:
-            return frame.file.isEmpty() ? frame.module : FilePath::fromString(frame.file).fileName();
+            return frame.file.isEmpty() ? frame.module : frame.file.fileName();
         case StackLineNumberColumn:
             return frame.line > 0 ? QVariant(frame.line) : QVariant();
         case StackAddressColumn:
@@ -115,7 +115,7 @@ QVariant StackFrameItem::data(int column, int role) const
     if (role == Qt::DecorationRole && column == StackLevelColumn)
         return handler->iconForRow(row);
 
-    if (role == Qt::ToolTipRole && boolSetting(UseToolTipsInStackView))
+    if (role == Qt::ToolTipRole && debuggerSettings()->useToolTipsInStackView.value())
         return frame.toToolTip();
 
     return QVariant();
@@ -257,8 +257,8 @@ void StackHandler::setFramesAndCurrentIndex(const GdbMi &frames, bool isFull)
             targetFrame = i;
     }
 
-    bool canExpand = !isFull && (n >= action(MaximalStackDepth)->value().toInt());
-    action(ExpandStack)->setEnabled(canExpand);
+    bool canExpand = !isFull && (n >= debuggerSettings()->maximalStackDepth.value());
+    debuggerSettings()->expandStack.setEnabled(canExpand);
     setFrames(stackFrames, canExpand);
 
     // We can't jump to any file if we don't have any frames.
@@ -337,7 +337,6 @@ static StackFrame inputFunctionForDisassembly()
     dialog.setInputMode(QInputDialog::TextInput);
     dialog.setLabelText(StackHandler::tr("Function:"));
     dialog.setWindowTitle(StackHandler::tr("Disassemble Function"));
-    dialog.setWindowFlags(dialog.windowFlags() & ~Qt::WindowContextHelpButtonHint);
     if (dialog.exec() != QDialog::Accepted)
         return frame;
     const QString function = dialog.textValue();
@@ -354,6 +353,71 @@ static StackFrame inputFunctionForDisassembly()
     return frame;
 }
 
+template<typename Functor>
+void forEachCell(Functor f, QAbstractItemModel *model, const QModelIndex &idx)
+{
+    f(idx);
+    for (int i = 0, n = model->rowCount(idx); i < n; ++i)
+        forEachCell(f, model, model->index(i, 0, idx));
+}
+
+static QString selectedText(QWidget *widget, bool useAll)
+{
+    QAbstractItemView *view = qobject_cast<QAbstractItemView *>(widget);
+    QTC_ASSERT(view, return {});
+    QAbstractItemModel *model = view->model();
+    QTC_ASSERT(model, return {});
+
+    const int ncols = model->columnCount(QModelIndex());
+    QVector<int> largestColumnWidths(ncols, 0);
+
+    QSet<QModelIndex> selected;
+    if (QItemSelectionModel *selection = view->selectionModel()) {
+        const QModelIndexList list = selection->selectedIndexes();
+        selected = QSet<QModelIndex>(list.begin(), list.end());
+    }
+    if (selected.isEmpty())
+        useAll = true;
+
+    // First, find the widths of the largest columns,
+    // so that we can print them out nicely aligned.
+    forEachCell([ncols, model, &largestColumnWidths, selected, useAll](const QModelIndex &idx) {
+        if (useAll || selected.contains(idx)) {
+            for (int j = 0; j < ncols; ++j) {
+                const QModelIndex sibling = model->sibling(idx.row(), j, idx);
+                const int columnWidth = model->data(sibling, Qt::DisplayRole).toString().size();
+                if (columnWidth > largestColumnWidths.at(j))
+                    largestColumnWidths[j] = columnWidth;
+            }
+        }
+    }, model, QModelIndex());
+
+    QString str;
+    forEachCell([ncols, model, largestColumnWidths, &str, selected, useAll](const QModelIndex &idx) {
+        if (useAll || selected.contains(idx)) {
+            for (int j = 0; j != ncols; ++j) {
+                const QModelIndex sibling = model->sibling(idx.row(), j, idx);
+                const QString columnEntry = model->data(sibling, Qt::DisplayRole).toString();
+                str += columnEntry;
+                const int difference = largestColumnWidths.at(j) - columnEntry.size();
+                // Add one extra space between columns.
+                str += QString(qMax(difference, 0) + 1, QChar(' '));
+            }
+            str += '\n';
+        }
+    }, model, QModelIndex());
+
+    return str;
+}
+
+static void copyTextToClipboard(const QString &str)
+{
+    QClipboard *clipboard = QApplication::clipboard();
+    if (clipboard->supportsSelection())
+        clipboard->setText(str, QClipboard::Selection);
+    clipboard->setText(str, QClipboard::Clipboard);
+}
+
 // Write stack frames as task file for displaying it in the build issues pane.
 void StackHandler::saveTaskFile()
 {
@@ -391,13 +455,20 @@ bool StackHandler::contextMenuEvent(const ItemViewEvent &ev)
         frame = frameAt(row);
     const quint64 address = frame.address;
 
-    menu->addAction(action(ExpandStack));
+    menu->addAction(debuggerSettings()->expandStack.action());
+
+    addAction(menu, tr("Copy Contents to Clipboard"), true, [ev] {
+        copyTextToClipboard(selectedText(ev.view(), true));
+    });
+
+    addAction(menu, tr("Copy Selection to Clipboard"), true, [ev] {
+        copyTextToClipboard(selectedText(ev.view(), false));
+    });
 
-    addAction(menu, tr("Copy Contents to Clipboard"), true, [this] { copyContentsToClipboard(); });
     addAction(menu, tr("Save as Task File..."), true, [this] { saveTaskFile(); });
 
     if (m_engine->hasCapability(CreateFullBacktraceCapability))
-        menu->addAction(action(CreateFullBacktrace));
+        menu->addAction(debuggerSettings()->createFullBacktrace.action());
 
     if (m_engine->hasCapability(AdditionalQmlStackCapability))
         addAction(menu, tr("Load QML Stack"), true, [this] { m_engine->loadAdditionalQmlStack(); });
@@ -444,49 +515,13 @@ bool StackHandler::contextMenuEvent(const ItemViewEvent &ev)
                   [this] { m_engine->loadSymbolsForStack(); });
     }
 
-    if (m_engine->hasCapability(MemoryAddressCapability))
-        menu->addAction(action(UseAddressInStackView));
-
     menu->addSeparator();
-    menu->addAction(action(UseToolTipsInStackView));
-    menu->addSeparator();
-    menu->addAction(action(SettingsDialog));
+    menu->addAction(debuggerSettings()->useToolTipsInStackView.action());
+    menu->addAction(debuggerSettings()->settingsDialog.action());
     menu->popup(ev.globalPos());
     return true;
 }
 
-void StackHandler::copyContentsToClipboard()
-{
-    const int m = columnCount(QModelIndex());
-    QVector<int> largestColumnWidths(m, 0);
-
-    // First, find the widths of the largest columns,
-    // so that we can print them out nicely aligned.
-    forItemsAtLevel<2>([m, &largestColumnWidths](StackFrameItem *item) {
-        for (int j = 0; j < m; ++j) {
-            const int columnWidth = item->data(j, Qt::DisplayRole).toString().size();
-            if (columnWidth > largestColumnWidths.at(j))
-                largestColumnWidths[j] = columnWidth;
-        }
-    });
-
-    QString str;
-    forItemsAtLevel<2>([m, largestColumnWidths, &str](StackFrameItem *item) {
-        for (int j = 0; j != m; ++j) {
-            const QString columnEntry = item->data(j, Qt::DisplayRole).toString();
-            str += columnEntry;
-            const int difference = largestColumnWidths.at(j) - columnEntry.size();
-            // Add one extra space between columns.
-            str += QString(qMax(difference, 0) + 1, QChar(' '));
-        }
-        str += '\n';
-    });
-
-    QClipboard *clipboard = QApplication::clipboard();
-    clipboard->setText(str, QClipboard::Selection);
-    clipboard->setText(str, QClipboard::Clipboard);
-}
-
 void StackHandler::reloadFullStack()
 {
     m_engine->reloadFullStack();
