@@ -128,7 +128,7 @@ int Terminal::write(const QByteArray &msg)
 #ifdef DEBUGGER_USE_TERMINAL
     return ::write(m_masterFd, msg.constData(), msg.size());
 #else
-    Q_UNUSED(msg);
+    Q_UNUSED(msg)
     return -1;
 #endif
 }
@@ -164,17 +164,16 @@ void Terminal::onSlaveReaderActivated(int fd)
     if (got >= 0)
         stdOutReady(QString::fromUtf8(buffer));
 #else
-    Q_UNUSED(fd);
+    Q_UNUSED(fd)
 #endif
 }
 
-TerminalRunner::TerminalRunner(RunControl *runControl, const Runnable &stubRunnable)
-    : RunWorker(runControl)
+TerminalRunner::TerminalRunner(RunControl *runControl,
+                               const std::function<Runnable()> &stubRunnable)
+    : RunWorker(runControl), m_stubRunnable(stubRunnable)
 {
     setId("TerminalRunner");
 
-    m_stubRunnable = stubRunnable;
-
     connect(&m_stubProc, &ConsoleProcess::processError,
             this, &TerminalRunner::stubError);
     connect(&m_stubProc, &ConsoleProcess::processStarted,
@@ -183,10 +182,33 @@ TerminalRunner::TerminalRunner(RunControl *runControl, const Runnable &stubRunna
             this, [this] { reportDone(); });
 }
 
+void TerminalRunner::kickoffProcess()
+{
+    m_stubProc.kickoffProcess();
+}
+
+void TerminalRunner::interruptProcess()
+{
+    m_stubProc.interruptProcess();
+}
+
+void TerminalRunner::setRunAsRoot(bool on)
+{
+    m_runAsRoot = on;
+}
+
 void TerminalRunner::start()
 {
-    m_stubProc.setEnvironment(m_stubRunnable.environment);
-    m_stubProc.setWorkingDirectory(m_stubRunnable.workingDirectory);
+    QTC_ASSERT(m_stubRunnable, reportFailure({}); return);
+    Runnable stub = m_stubRunnable();
+
+    if (m_runAsRoot) {
+        m_stubProc.setRunAsRoot(true);
+        RunControl::provideAskPassEntry(stub.environment);
+    }
+
+    m_stubProc.setEnvironment(stub.environment);
+    m_stubProc.setWorkingDirectory(stub.workingDirectory);
 
     if (HostOsInfo::isWindowsHost()) {
         m_stubProc.setMode(ConsoleProcess::Suspend);
@@ -196,7 +218,8 @@ void TerminalRunner::start()
     }
 
     // Error message for user is delivered via a signal.
-    m_stubProc.start(m_stubRunnable.executable, m_stubRunnable.commandLineArguments);
+    m_stubProc.setCommand(stub.command);
+    m_stubProc.start();
 }
 
 void TerminalRunner::stop()
