@@ -87,7 +87,6 @@ UnstartedAppWatcherDialog::UnstartedAppWatcherDialog(QWidget *parent)
     : QDialog(parent)
 {
     setWindowTitle(tr("Attach to Process Not Yet Started"));
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
     m_kitChooser = new KitChooser(this);
     m_kitChooser->setKitPredicate([](const Kit *k) {
@@ -122,7 +121,7 @@ UnstartedAppWatcherDialog::UnstartedAppWatcherDialog(QWidget *parent)
             if (isLocal(runConfig)) {
                 resetExecutable->setEnabled(true);
                 connect(resetExecutable, &QPushButton::clicked, this, [this, runnable] {
-                    m_pathChooser->setFileName(runnable.executable);
+                    m_pathChooser->setFilePath(runnable.command.executable());
                 });
             }
         }
@@ -192,7 +191,7 @@ bool UnstartedAppWatcherDialog::event(QEvent *e)
 
 void UnstartedAppWatcherDialog::selectExecutable()
 {
-    QString path;
+    Utils::FilePath path;
 
     Project *project = ProjectTree::currentProject();
     Target *activeTarget = project ? project->activeTarget() : nullptr;
@@ -201,16 +200,15 @@ void UnstartedAppWatcherDialog::selectExecutable()
         if (RunConfiguration *runConfig = activeTarget->activeRunConfiguration()) {
             const Runnable runnable = runConfig->runnable();
             if (isLocal(runConfig))
-                path = runnable.executable.toFileInfo().path();
+                path = runnable.command.executable().parentDir();
         }
     }
 
     if (path.isEmpty()) {
-        if (activeTarget && activeTarget->activeBuildConfiguration()) {
-            path = activeTarget->activeBuildConfiguration()->buildDirectory().toString();
-        } else if (project) {
-            path = project->projectDirectory().toString();
-        }
+        if (activeTarget && activeTarget->activeBuildConfiguration())
+            path = activeTarget->activeBuildConfiguration()->buildDirectory();
+        else if (project)
+            path = project->projectDirectory();
     }
     m_pathChooser->setInitialBrowsePathBackup(path);
 }
@@ -257,10 +255,10 @@ void UnstartedAppWatcherDialog::startStopTimer(bool start)
 
 void UnstartedAppWatcherDialog::findProcess()
 {
-    const QString &appName = Utils::FileUtils::normalizePathName(m_pathChooser->path());
+    const QString &appName = m_pathChooser->filePath().normalizedPathName().toString();
     DeviceProcessItem fallback;
     foreach (const DeviceProcessItem &p, DeviceProcessList::localProcesses()) {
-        if (Utils::FileUtils::normalizePathName(p.exe) == appName) {
+        if (Utils::FileUtils::normalizedPathName(p.exe) == appName) {
             pidFound(p);
             return;
         }
@@ -292,8 +290,8 @@ void UnstartedAppWatcherDialog::kitChanged()
 
 bool UnstartedAppWatcherDialog::checkExecutableString() const
 {
-    if (!m_pathChooser->path().isEmpty()) {
-        QFileInfo fileInfo(m_pathChooser->path());
+    if (!m_pathChooser->filePath().toString().isEmpty()) {
+        QFileInfo fileInfo(m_pathChooser->filePath().toString());
         return (fileInfo.exists() && fileInfo.isFile());
     }
     return false;
