@@ -38,6 +38,7 @@
 #include "memoryagent.h"
 #include "registerhandler.h"
 #include "simplifytype.h"
+#include "sourceutils.h"
 #include "watchdelegatewidgets.h"
 #include "watchutils.h"
 
@@ -55,7 +56,6 @@
 #include <utils/checkablemessagebox.h>
 #include <utils/fancylineedit.h>
 #include <utils/qtcassert.h>
-#include <utils/savedaction.h>
 #include <utils/stringutils.h>
 #include <utils/theme/theme.h>
 
@@ -67,17 +67,17 @@
 #include <QJsonArray>
 #include <QJsonObject>
 #include <QLabel>
+#include <QMap>
 #include <QMenu>
 #include <QMimeData>
 #include <QPainter>
-#include <QProcess>
 #include <QSet>
+#include <QTableWidget>
 #include <QTabWidget>
 #include <QTextEdit>
-#include <QTableWidget>
 #include <QTimer>
-#include <QVBoxLayout>
 #include <QToolTip>
+#include <QVBoxLayout>
 
 #include <algorithm>
 #include <cstring>
@@ -132,7 +132,7 @@ template <class T>
 void readNumericVectorHelper(std::vector<double> *v, const QByteArray &ba)
 {
     const auto p = (const T*)ba.data();
-    const int n = ba.size() / sizeof(T);
+    const int n = int(ba.size() / sizeof(T));
     v->resize(n);
     // Losing precision in case of 64 bit ints is ok here, as the result
     // is only used to plot data.
@@ -276,11 +276,15 @@ static void loadSessionData()
 
 class SeparatedView : public QTabWidget
 {
+    Q_OBJECT
 public:
     SeparatedView() : QTabWidget(DebuggerMainWindow::instance())
     {
         setTabsClosable(true);
         connect(this, &QTabWidget::tabCloseRequested, this, &SeparatedView::closeTab);
+        connect(tabBar(), &QTabBar::customContextMenuRequested,
+                this, &SeparatedView::tabBarContextMenuRequested);
+        tabBar()->setContextMenuPolicy(Qt::CustomContextMenu);
         setWindowFlags(windowFlags() | Qt::Window);
         setWindowTitle(WatchHandler::tr("Debugger - %1").arg(Core::Constants::IDE_DISPLAY_NAME));
 
@@ -297,7 +301,7 @@ public:
 
     void saveGeometry()
     {
-        SessionManager::setValue("DebuggerSeparateWidgetGeometry", geometry());
+        SessionManager::setValue("DebuggerSeparateWidgetGeometry", QVariant(geometry()));
     }
 
     ~SeparatedView() override
@@ -326,6 +330,15 @@ public:
         sanitize();
     }
 
+    void tabBarContextMenuRequested(const QPoint &point)
+    {
+        const QWidget *w = widget(tabBar()->tabAt(point));
+        if (!w)
+            return;
+        emit tabBarContextMenuRequestedSignal(tabBar()->mapToGlobal(point),
+                                              w->property(INameProperty).toString());
+    }
+
     void sanitize()
     {
         if (count() == 0)
@@ -355,6 +368,7 @@ public:
         if (!t) {
             t = new T;
             t->setProperty(KeyProperty, key);
+            t->setProperty(INameProperty, item->iname);
             addTab(t, item->name);
         }
         setProperty(INameProperty, item->iname);
@@ -364,6 +378,9 @@ public:
         raise();
         return t;
     }
+
+Q_SIGNALS:
+    void tabBarContextMenuRequestedSignal(const QPoint &position, const QString &watchiName);
 };
 
 class TextEdit : public QTextEdit
@@ -397,6 +414,7 @@ public:
 class WatchModel : public WatchModelBase
 {
     Q_DECLARE_TR_FUNCTIONS(Debugger::Internal::WatchModel)
+    typedef QSet<WatchItem *> WatchItemSet;
 public:
     WatchModel(WatchHandler *handler, DebuggerEngine *engine);
 
@@ -446,9 +464,13 @@ public:
     void grabWidget();
     void ungrabWidget();
     void timerEvent(QTimerEvent *event) override;
-    int m_grabWidgetTimerId = -1;
+private:
+    QMenu *createFormatMenuForManySelected(const WatchItemSet &item, QWidget *parent);
+    void setItemsFormat(const WatchItemSet &items, const DisplayFormat &format);
+    void addCharsPrintableMenu(QMenu *menu);
 
 public:
+    int m_grabWidgetTimerId = -1;
     WatchHandler *m_handler; // Not owned.
     DebuggerEngine *m_engine; // Not owned.
 
@@ -468,6 +490,11 @@ public:
     QHash<QString, TypeInfo> m_reportedTypeInfo;
     QHash<QString, DisplayFormats> m_reportedTypeFormats; // Type name -> Dumper Formats
     QHash<QString, QString> m_valueCache;
+
+    Location m_location;
+
+private:
+    void separatedViewTabBarContextMenuRequested(const QPoint &point, const QString &iname);
 };
 
 WatchModel::WatchModel(WatchHandler *handler, DebuggerEngine *engine)
@@ -505,14 +532,20 @@ WatchModel::WatchModel(WatchHandler *handler, DebuggerEngine *engine)
     connect(&m_requestUpdateTimer, &QTimer::timeout,
         this, &WatchModel::updateStarted);
 
-    connect(action(SortStructMembers), &SavedAction::valueChanged,
+    DebuggerSettings &s = *debuggerSettings();
+    connect(&s.sortStructMembers, &BaseAspect::changed,
         m_engine, &DebuggerEngine::updateLocals);
-    connect(action(ShowStdNamespace), &SavedAction::valueChanged,
+    connect(&s.showStdNamespace, &BaseAspect::changed,
         m_engine, &DebuggerEngine::updateAll);
-    connect(action(ShowQtNamespace), &SavedAction::valueChanged,
+    connect(&s.showQtNamespace, &BaseAspect::changed,
         m_engine, &DebuggerEngine::updateAll);
-    connect(action(ShowQObjectNames), &SavedAction::valueChanged,
+    connect(&s.showQObjectNames, &BaseAspect::changed,
         m_engine, &DebuggerEngine::updateAll);
+    connect(&s.useAnnotationsInMainEditor, &BaseAspect::changed,
+        m_engine, &DebuggerEngine::updateAll);
+
+    connect(m_separatedView, &SeparatedView::tabBarContextMenuRequestedSignal,
+            this, &WatchModel::separatedViewTabBarContextMenuRequested);
 
     connect(SessionManager::instance(), &SessionManager::sessionLoaded,
             this, &loadSessionData);
@@ -555,9 +588,9 @@ static QString niceTypeHelper(const QString &typeIn)
 
 QString WatchModel::removeNamespaces(QString str) const
 {
-    if (!boolSetting(ShowStdNamespace))
+    if (!debuggerSettings()->showStdNamespace.value())
         str.remove("std::");
-    if (!boolSetting(ShowQtNamespace)) {
+    if (!debuggerSettings()->showQtNamespace.value()) {
         const QString qtNamespace = m_engine->qtNamespace();
         if (!qtNamespace.isEmpty())
             str.remove(qtNamespace);
@@ -659,36 +692,7 @@ static QString reformatCharacter(int code, int size, bool isSigned)
 
 static QString quoteUnprintable(const QString &str)
 {
-    if (theUnprintableBase == 0)
-        return str;
-
-    QString encoded;
-    if (theUnprintableBase == -1) {
-        for (const QChar c : str) {
-            int u = c.unicode();
-            if (c.isPrint())
-                encoded += c;
-            else if (u == '\r')
-                encoded += "\\r";
-            else if (u == '\t')
-                encoded += "\\t";
-            else if (u == '\n')
-                encoded += "\\n";
-            else
-                encoded += QString("\\%1").arg(c.unicode(), 3, 8, QLatin1Char('0'));
-        }
-        return encoded;
-    }
-
-    for (const QChar c : str) {
-        if (c.isPrint())
-            encoded += c;
-        else if (theUnprintableBase == 8)
-            encoded += QString("\\%1").arg(c.unicode(), 3, 8, QLatin1Char('0'));
-        else
-            encoded += QString("\\u%1").arg(c.unicode(), 4, 16, QLatin1Char('0'));
-    }
-    return encoded;
+    return escapeUnprintable(str, theUnprintableBase);
 }
 
 static int itemFormat(const WatchItem *item)
@@ -713,22 +717,34 @@ static QString formattedValue(const WatchItem *item)
 
     // Append quoted, printable character also for decimal.
     // FIXME: This is unreliable.
-    if (item->type.endsWith("char")) {
+    const QString type = item->type;
+    if (type == "char8_t" || type.endsWith("char") || type.endsWith("int8_t")) {
         bool ok;
         const int code = item->value.toInt(&ok);
-        bool isUnsigned = item->type == "unsigned char" || item->type == "uchar";
+        bool isUnsigned = type == "char8_t"
+                || type == "unsigned char"
+                || type == "uchar"
+                || type == "uint8_t";
         if (ok)
             return reformatCharacter(code, 1, !isUnsigned);
-    } else if (item->type.endsWith("wchar_t")) {
+    } else if (type == "qint8" || type == "quint8") {
+        bool ok = false;
+        const int code = item->value.toInt(&ok);
+        bool isUnsigned = type == "quint8";
+        if (ok)
+            return reformatCharacter(code, 1, !isUnsigned);
+    } else if (type == "char32_t" || type.endsWith("wchar_t")) {
         bool ok;
         const int code = item->value.toInt(&ok);
+        bool isUnsigned = type == "char32_t";
         if (ok)
-            return reformatCharacter(code, 4, false);
-    } else if (item->type.endsWith("QChar")) {
+            return reformatCharacter(code, 4, !isUnsigned);
+    } else if (type == "char16_t" || type.endsWith("QChar")) {
         bool ok;
         const int code = item->value.toInt(&ok);
+        bool isUnsigned = type == "char16_t";
         if (ok)
-            return reformatCharacter(code, 2, false);
+            return reformatCharacter(code, 2, !isUnsigned);
     }
 
     if (format == HexadecimalIntegerFormat
@@ -858,6 +874,19 @@ static QString displayName(const WatchItem *item)
     else
         result = watchModel(item)->removeNamespaces(item->name);
 
+    // prepend '*'s to indicate where autodereferencing has taken place
+    if (item->autoDerefCount > 0) {
+        // add parentheses for everything except simple variable names (e.g. pointer arithmetics,...)
+        QRegularExpression variableNameRegex("^[a-zA-Z0-9_]+$");
+        bool addParanthesis = !variableNameRegex.match(result).hasMatch();
+        if (addParanthesis)
+            result = "(" + result;
+        for (uint i = 0; i < item->autoDerefCount; i++)
+            result = "*" + result;
+        if (addParanthesis)
+            result += ")";
+    }
+
     // Simplify names that refer to base classes.
     if (result.startsWith('[')) {
         result = simplifyType(result);
@@ -1031,7 +1060,7 @@ QVariant WatchModel::data(const QModelIndex &idx, int role) const
         }
 
         case Qt::ToolTipRole:
-            return boolSetting(UseToolTipsInLocalsView)
+            return debuggerSettings()->useToolTipsInLocalsView.value()
                 ? item->toToolTip() : QVariant();
 
         case Qt::ForegroundRole:
@@ -1103,7 +1132,8 @@ bool WatchModel::setData(const QModelIndex &idx, const QVariant &value, int role
         }
 
         if (auto kev = ev.as<QKeyEvent>(QEvent::KeyPress)) {
-            if (item && kev->key() == Qt::Key_Delete && item->isWatcher()) {
+            if (item && (kev->key() == Qt::Key_Delete || kev->key() == Qt::Key_Backspace)
+                && item->isWatcher()) {
                 foreach (const QModelIndex &idx, ev.selectedRows())
                     removeWatchItem(itemForIndex(idx));
                 return true;
@@ -1173,7 +1203,7 @@ bool WatchModel::setData(const QModelIndex &idx, const QVariant &value, int role
 Qt::ItemFlags WatchModel::flags(const QModelIndex &idx) const
 {
     if (!idx.isValid())
-        return nullptr;
+        return {};
 
     const WatchItem *item = nonRootItemForIndex(idx);
     if (!item)
@@ -1590,7 +1620,8 @@ static QString removeWatchActionText(QString exp)
 static void copyToClipboard(const QString &clipboardText)
 {
     QClipboard *clipboard = QApplication::clipboard();
-    clipboard->setText(clipboardText, QClipboard::Selection);
+    if (clipboard->supportsSelection())
+        clipboard->setText(clipboardText, QClipboard::Selection);
     clipboard->setText(clipboardText, QClipboard::Clipboard);
 }
 
@@ -1665,7 +1696,7 @@ bool WatchModel::contextMenuEvent(const ItemViewEvent &ev)
               [this, item] { removeWatchItem(item); });
 
     addAction(menu, tr("Remove All Expression Evaluators"),
-              canRemoveWatches && !m_handler->watchedExpressions().isEmpty(),
+              canRemoveWatches && !WatchHandler::watchedExpressions().isEmpty(),
               [this] { clearWatches(); });
 
     addAction(menu, tr("Select Widget to Add into Expression Evaluator"),
@@ -1673,29 +1704,37 @@ bool WatchModel::contextMenuEvent(const ItemViewEvent &ev)
               [this] { grabWidget(); });
 
     menu->addSeparator();
-    menu->addMenu(createFormatMenu(item, menu));
+    QModelIndexList mil = ev.currentOrSelectedRows();
+    if (mil.size() > 1) {
+        WatchItemSet wis;
+        for (const QModelIndex &i : mil)
+            wis.insert(itemForIndex(i));
+
+        menu->addMenu(createFormatMenuForManySelected(wis, menu));
+    } else {
+        menu->addMenu(createFormatMenu(item, menu));
+    }
+
     menu->addMenu(createMemoryMenu(item, menu));
     menu->addMenu(createBreakpointMenu(item, menu));
     menu->addSeparator();
 
-    addAction(menu, tr("Expand All Children"),
-              item,
-              [this, item] {
-                m_expandedINames.insert(item->iname);
-                item->forFirstLevelChildren([this](WatchItem *child) {
-                    m_expandedINames.insert(child->iname);
-                });
-                m_engine->updateLocals();
-              });
+    addAction(menu, tr("Expand All Children"), item, [this, name = item ? item->iname : QString()] {
+        m_expandedINames.insert(name);
+        if (auto item = findItem(name)) {
+            item->forFirstLevelChildren(
+                [this](WatchItem *child) { m_expandedINames.insert(child->iname); });
+            m_engine->updateLocals();
+        }
+    });
 
-    addAction(menu, tr("Collapse All Children"),
-              item,
-              [this, item] {
-                item->forFirstLevelChildren([this](WatchItem *child) {
-                    m_expandedINames.remove(child->iname);
-                });
-                m_engine->updateLocals();
-              });
+    addAction(menu, tr("Collapse All Children"), item, [this, name = item ? item->iname : QString()] {
+        if (auto item = findItem(name)) {
+            item->forFirstLevelChildren(
+                [this](WatchItem *child) { m_expandedINames.remove(child->iname); });
+            m_engine->updateLocals();
+        }
+    });
 
     addAction(menu, tr("Close Editor Tooltips"),
               m_engine->toolTipManager()->hasToolTips(),
@@ -1705,13 +1744,17 @@ bool WatchModel::contextMenuEvent(const ItemViewEvent &ev)
               true,
               [this] { copyToClipboard(editorContents()); });
 
-    addAction(menu, tr("Copy Current Value to Clipboard"),
+    addAction(menu,
+              tr("Copy Current Value to Clipboard"),
               item,
-              [item] { copyToClipboard(item->value); });
+              [this, name = item ? item->iname : QString()] {
+                  if (auto item = findItem(name))
+                      copyToClipboard(item->value);
+              });
 
-//    addAction(menu, tr("Copy Selected Rows to Clipboard"),
-//              selectionModel()->hasSelection(),
-//              [this] { copyToClipboard(editorContents(selectionModel()->selectedRows())); });
+    //    addAction(menu, tr("Copy Selected Rows to Clipboard"),
+    //              selectionModel()->hasSelection(),
+    //              [this] { copyToClipboard(editorContents(selectionModel()->selectedRows())); });
 
     addAction(menu, tr("Open View Contents in Editor"),
               m_engine->debuggerActionsEnabled(),
@@ -1719,15 +1762,14 @@ bool WatchModel::contextMenuEvent(const ItemViewEvent &ev)
 
     menu->addSeparator();
 
-    menu->addAction(action(UseDebuggingHelpers));
-    menu->addAction(action(UseToolTipsInLocalsView));
-    menu->addAction(action(AutoDerefPointers));
-    menu->addAction(action(SortStructMembers));
-    menu->addAction(action(UseDynamicType));
-    menu->addAction(action(SettingsDialog));
+    DebuggerSettings &s = *debuggerSettings();
+    menu->addAction(s.useDebuggingHelpers.action());
+    menu->addAction(s.useToolTipsInLocalsView.action());
+    menu->addAction(s.autoDerefPointers.action());
+    menu->addAction(s.sortStructMembers.action());
+    menu->addAction(s.useDynamicType.action());
+    menu->addAction(s.settingsDialog.action());
 
-    menu->addSeparator();
-    menu->addAction(action(SettingsDialog));
     connect(menu, &QMenu::aboutToHide, menu, &QObject::deleteLater);
     menu->popup(ev.globalPos());
     return true;
@@ -1835,6 +1877,26 @@ QMenu *WatchModel::createMemoryMenu(WatchItem *item, QWidget *parent)
     return menu;
 }
 
+void WatchModel::addCharsPrintableMenu(QMenu *menu)
+{
+    auto addBaseChangeAction = [this, menu](const QString &text, int base) {
+        addCheckableAction(menu, text, true, theUnprintableBase == base, [this, base] {
+            theUnprintableBase = base;
+            emit layoutChanged(); // FIXME
+        });
+    };
+    addBaseChangeAction(tr("Treat All Characters as Printable"), 0);
+    addBaseChangeAction(tr("Show Unprintable Characters as Escape Sequences"), -1);
+    addBaseChangeAction(tr("Show Unprintable Characters as Octal"), 8);
+    addBaseChangeAction(tr("Show Unprintable Characters as Hexadecimal"), 16);
+}
+
+void WatchModel::separatedViewTabBarContextMenuRequested(const QPoint &point, const QString &iname)
+{
+    auto menu = createFormatMenu(findItem(iname), m_separatedView);
+    menu->exec(point);
+}
+
 QMenu *WatchModel::createFormatMenu(WatchItem *item, QWidget *parent)
 {
     auto menu = new QMenu(tr("Change Value Display Format"), parent);
@@ -1849,17 +1911,7 @@ QMenu *WatchModel::createFormatMenu(WatchItem *item, QWidget *parent)
     const int typeFormat = theTypeFormats.value(stripForFormat(item->type), AutomaticFormat);
     const int individualFormat = theIndividualFormats.value(iname, AutomaticFormat);
 
-    auto addBaseChangeAction = [this, menu](const QString &text, int base) {
-        addCheckableAction(menu, text, true, theUnprintableBase == base, [this, base] {
-            theUnprintableBase = base;
-            emit layoutChanged(); // FIXME
-        });
-    };
-
-    addBaseChangeAction(tr("Treat All Characters as Printable"), 0);
-    addBaseChangeAction(tr("Show Unprintable Characters as Escape Sequences"), -1);
-    addBaseChangeAction(tr("Show Unprintable Characters as Octal"), 8);
-    addBaseChangeAction(tr("Show Unprintable Characters as Hexadecimal"), 16);
+    addCharsPrintableMenu(menu);
 
     const QString spacer = "     ";
     menu->addSeparator();
@@ -1888,12 +1940,18 @@ QMenu *WatchModel::createFormatMenu(WatchItem *item, QWidget *parent)
                            });
     }
 
+    addAction(menu, tr("Reset All Individual Formats"), true, [this]() {
+        theIndividualFormats.clear();
+        saveFormats();
+        m_engine->updateLocals();
+    });
+
     menu->addSeparator();
     addAction(menu, tr("Change Display for Type \"%1\":").arg(item->type), false);
 
     addCheckableAction(menu, spacer + tr("Automatic"), true, typeFormat == AutomaticFormat,
                        [this, item] {
-                            //const QModelIndexList active = activeRows();
+                           //const QModelIndexList active = activeRows();
                            //for (const QModelIndex &idx : active)
                            //    setModelData(LocalsTypeFormatRole, AutomaticFormat, idx);
                            setTypeFormat(item->type, AutomaticFormat);
@@ -1903,11 +1961,76 @@ QMenu *WatchModel::createFormatMenu(WatchItem *item, QWidget *parent)
     for (int format : alternativeFormats) {
         addCheckableAction(menu, spacer + nameForFormat(format), true, format == typeFormat,
                            [this, format, item] {
-                                setTypeFormat(item->type, format);
-                                m_engine->updateLocals();
+                               setTypeFormat(item->type, format);
+                               m_engine->updateLocals();
                            });
     }
 
+    addAction(menu, tr("Reset All Formats for Types"), true, [this]() {
+        theTypeFormats.clear();
+        saveFormats();
+        m_engine->updateLocals();
+    });
+
+    return menu;
+}
+
+void WatchModel::setItemsFormat(const WatchItemSet &items, const DisplayFormat &format)
+{
+    if (format == AutomaticFormat) {
+        for (WatchItem *item : items)
+            theIndividualFormats.remove(item->iname);
+    } else {
+        for (WatchItem *item : items)
+            theIndividualFormats[item->iname] = format;
+    }
+    saveFormats();
+}
+
+QMenu *WatchModel::createFormatMenuForManySelected(const WatchItemSet &items, QWidget *parent)
+{
+    auto menu = new QMenu(tr("Change Display Format for Selected Values"), parent);
+
+    addCharsPrintableMenu(menu);
+
+    QHash<DisplayFormat, int> allItemsFormats;
+    for (WatchItem *item : items) {
+        const DisplayFormats alternativeFormats = typeFormatList(item);
+        for (const DisplayFormat &format : alternativeFormats) {
+            QHash<DisplayFormat, int>::iterator itr = allItemsFormats.find(format);
+            if (itr != allItemsFormats.end())
+                itr.value() += 1;
+            else
+                allItemsFormats[format] = 1;
+        }
+    }
+
+    const QString spacer = "     ";
+    menu->addSeparator();
+
+    addAction(menu, tr("Change Display for Objects"), false);
+    QString msg = QString(tr("Use Display Format Based on Type"));
+    addCheckableAction(menu, spacer + msg, true, false,
+                       [this, items] {
+                            setItemsFormat(items, AutomaticFormat);
+                            m_engine->updateLocals();
+                       });
+
+    int countOfSelectItems = items.size();
+    for (auto it = allItemsFormats.begin(), end = allItemsFormats.end(); it != end; ++it) {
+        DisplayFormat format = it.key();
+        QString formatName = nameForFormat(format);
+        if (formatName.isEmpty())
+            continue;
+
+        addCheckableAction(menu, spacer + formatName,
+                           it.value() == countOfSelectItems,
+                           false,
+                           [this, format, items] {
+                                setItemsFormat(items, format);
+                                m_engine->updateLocals();
+                           });
+    }
     return menu;
 }
 
@@ -1984,11 +2107,12 @@ void WatchHandler::cleanup()
 {
     m_model->m_expandedINames.clear();
     theWatcherNames.remove(QString());
-    for (const QString &exp : theTemporaryWatchers)
+    for (const QString &exp : qAsConst(theTemporaryWatchers))
         theWatcherNames.remove(exp);
     theTemporaryWatchers.clear();
     saveWatchers();
     m_model->reinitialize();
+    Internal::setValueAnnotations(m_model->m_location, {});
     emit m_model->updateFinished();
     m_model->m_separatedView->hide();
 }
@@ -2002,7 +2126,7 @@ void WatchHandler::insertItems(const GdbMi &data)
 {
     QSet<WatchItem *> itemsToSort;
 
-    const bool sortStructMembers = boolSetting(SortStructMembers);
+    const bool sortStructMembers = debuggerSettings()->sortStructMembers.value();
     for (const GdbMi &child : data) {
         auto item = new WatchItem;
         item->parse(child, sortStructMembers);
@@ -2142,6 +2266,16 @@ void WatchHandler::notifyUpdateFinished()
         }
     });
 
+    QMap<QString, QString> values;
+    if (debuggerSettings()->useAnnotationsInMainEditor.value()) {
+        m_model->forAllItems([&values](WatchItem *item) {
+            const QString expr = item->sourceExpression();
+            if (!expr.isEmpty())
+                values[expr] = item->value;
+        });
+    }
+    Internal::setValueAnnotations(m_model->m_location, values);
+
     m_model->m_contentsValid = true;
     updateLocalsWindow();
     m_model->reexpandItems();
@@ -2369,7 +2503,7 @@ void WatchModel::clearWatches()
         return;
 
     const QDialogButtonBox::StandardButton ret = CheckableMessageBox::doNotAskAgainQuestion(
-                ICore::mainWindow(), tr("Remove All Expression Evaluators"),
+                ICore::dialogParent(), tr("Remove All Expression Evaluators"),
                 tr("Are you sure you want to remove all expression evaluators?"),
                 ICore::settings(), "RemoveAllWatchers");
     if (ret != QDialogButtonBox::Yes)
@@ -2591,7 +2725,7 @@ void WatchHandler::addDumpers(const GdbMi &dumpers)
         DisplayFormats formats;
         formats.append(RawFormat);
         QString reportedFormats = dumper["formats"].data();
-        foreach (const QStringRef &format, reportedFormats.splitRef(',')) {
+        foreach (const QString &format, reportedFormats.split(',')) {
             if (int f = format.toInt())
                 formats.append(DisplayFormat(f));
         }
@@ -2658,6 +2792,11 @@ void WatchHandler::recordTypeInfo(const GdbMi &typeInfo)
     }
 }
 
+void WatchHandler::setLocation(const Location &loc)
+{
+    m_model->m_location = loc;
+}
+
 /////////////////////////////////////////////////////////////////////
 //
 // WatchDelegate
