@@ -26,8 +26,13 @@
 #include "formclasswizarddialog.h"
 #include "formclasswizardpage.h"
 #include "formclasswizardparameters.h"
+
+#include <cppeditor/abstracteditorsupport.h>
 #include <designer/formtemplatewizardpage.h>
 #include <qtsupport/codegenerator.h>
+#include <utils/filepath.h>
+
+using namespace Utils;
 
 enum { FormPageId, ClassPageId };
 
@@ -46,23 +51,19 @@ FormClassWizardDialog::FormClassWizardDialog(const Core::BaseFileWizardFactory *
     setPage(FormPageId, m_formPage);
     setPage(ClassPageId, m_classPage);
 
-    foreach (QWizardPage *p, extensionPages())
+    const auto pages = extensionPages();
+    for (QWizardPage *p : pages)
         addPage(p);
 }
 
-QString FormClassWizardDialog::path() const
+FilePath FormClassWizardDialog::filePath() const
 {
-    return m_classPage->path();
+    return m_classPage->filePath();
 }
 
-void FormClassWizardDialog::setPath(const QString &p)
+void FormClassWizardDialog::setFilePath(const FilePath &p)
 {
-    m_classPage->setPath(p);
-}
-
-bool FormClassWizardDialog::validateCurrentPage()
-{
-    return QWizard::validateCurrentPage();
+    m_classPage->setFilePath(p);
 }
 
 void FormClassWizardDialog::initializePage(int id)
@@ -87,6 +88,7 @@ FormClassWizardParameters FormClassWizardDialog::parameters() const
     m_classPage->getParameters(&rc);
     // Name the ui class in the Ui namespace after the class specified
     rc.uiTemplate = QtSupport::CodeGenerator::changeUiClassName(m_rawFormTemplate, rc.className);
+    rc.usePragmaOnce = CppEditor::AbstractEditorSupport::usePragmaOnce();
     return rc;
 }
 
