@@ -30,11 +30,12 @@
 #include <widgethost.h>
 #include <designer/cpp/formclasswizardpage.h>
 
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpptoolsconstants.h>
-#include <cpptools/cppworkingcopy.h>
-#include <cpptools/insertionpointlocator.h>
-#include <cpptools/symbolfinder.h>
+#include <cppeditor/cppeditorconstants.h>
+#include <cppeditor/cppmodelmanager.h>
+#include <cppeditor/cpptoolsreuse.h>
+#include <cppeditor/cppworkingcopy.h>
+#include <cppeditor/insertionpointlocator.h>
+#include <cppeditor/symbolfinder.h>
 #include <cplusplus/LookupContext.h>
 #include <cplusplus/Overview.h>
 #include <coreplugin/icore.h>
@@ -65,6 +66,7 @@ using namespace Designer::Internal;
 using namespace CPlusPlus;
 using namespace TextEditor;
 using namespace ProjectExplorer;
+using namespace Utils;
 
 static QString msgClassNotFound(const QString &uiClassName, const QList<Document::Ptr> &docList)
 {
@@ -167,7 +169,7 @@ QString fullyQualifiedName(const LookupContext &context, const Name *name, Scope
 // containing a member of the desired class type) or inheriting the desired class
 // in case of forms using the Multiple Inheritance approach
 static const Class *findClass(const Namespace *parentNameSpace, const LookupContext &context,
-                              const QString &className, QString *namespaceName)
+                              const QString &className)
 {
     if (Designer::Constants::Internal::debug)
         qDebug() << Q_FUNC_INFO << className;
@@ -204,14 +206,9 @@ static const Class *findClass(const Namespace *parentNameSpace, const LookupCont
         } else {
             // Check namespaces
             if (const Namespace *ns = sym->asNamespace()) {
-                QString tempNS = *namespaceName;
-                tempNS += o.prettyName(ns->name());
-                tempNS += "::";
-                if (const Class *cl = findClass(ns, context, className, &tempNS)) {
-                    *namespaceName = tempNS;
+                if (const Class *cl = findClass(ns, context, className))
                     return cl;
-                }
-            } // member is namespave
+            } // member is namespace
         } // member is no class
     } // for members
     return nullptr;
@@ -247,22 +244,6 @@ static Function *findDeclaration(const Class *cl, const QString &functionName)
     return nullptr;
 }
 
-// TODO: remove me, this is taken from cppeditor.cpp. Find some common place for this function
-static Document::Ptr findDefinition(Function *functionDeclaration, int *line)
-{
-    CppTools::CppModelManager *cppModelManager = CppTools::CppModelManager::instance();
-    const Snapshot snapshot = cppModelManager->snapshot();
-    CppTools::SymbolFinder symbolFinder;
-    if (Function *fun = symbolFinder.findMatchingDefinition(functionDeclaration, snapshot)) {
-        if (line)
-            *line = fun->line();
-
-        return snapshot.document(QString::fromUtf8(fun->fileName(), fun->fileNameLength()));
-    }
-
-    return Document::Ptr();
-}
-
 static inline BaseTextEditor *editorAt(const QString &fileName, int line, int column)
 {
     return qobject_cast<BaseTextEditor *>(Core::EditorManager::openEditorAt(fileName, line, column,
@@ -277,10 +258,10 @@ static void addDeclaration(const Snapshot &snapshot,
 {
     const QString declaration = "void " + functionName + ";\n";
 
-    CppTools::CppRefactoringChanges refactoring(snapshot);
-    CppTools::InsertionPointLocator find(refactoring);
-    const CppTools::InsertionLocation loc = find.methodDeclarationInClass(
-                fileName, cl, CppTools::InsertionPointLocator::PrivateSlot);
+    CppEditor::CppRefactoringChanges refactoring(snapshot);
+    CppEditor::InsertionPointLocator find(refactoring);
+    const CppEditor::InsertionLocation loc = find.methodDeclarationInClass(
+                fileName, cl, CppEditor::InsertionPointLocator::PrivateSlot);
 
     //
     //! \todo change this to use the Refactoring changes.
@@ -297,50 +278,6 @@ static void addDeclaration(const Snapshot &snapshot,
     }
 }
 
-static Document::Ptr addDefinition(const Snapshot &docTable,
-                                   const QString &headerFileName,
-                                   const QString &className,
-                                   const QString &functionName,
-                                   int *line)
-{
-    const QString definition = "\nvoid " + className + "::" + functionName
-            + "\n{\n" + QString(indentation, ' ') + "\n}\n";
-
-    // we find all documents which include headerFileName
-    const QList<Document::Ptr> docList = findDocumentsIncluding(docTable, headerFileName, false);
-    if (docList.isEmpty())
-        return Document::Ptr();
-
-    QFileInfo headerFI(headerFileName);
-    const QString headerBaseName = headerFI.completeBaseName();
-    for (const Document::Ptr &doc : docList) {
-        const QFileInfo sourceFI(doc->fileName());
-        // we take only those documents which have the same filename
-        if (headerBaseName == sourceFI.baseName()) {
-            //
-            //! \todo change this to use the Refactoring changes.
-            //
-
-            if (BaseTextEditor *editor = editorAt(doc->fileName(), 0, 0)) {
-
-                //
-                //! \todo use the InsertionPointLocator to insert at the correct place.
-                // (we'll have to extend that class first to do definition insertions)
-
-                const QString contents = editor->textDocument()->plainText();
-                int column;
-                editor->convertPosition(contents.length(), line, &column);
-                // gotoLine accepts 0-based column.
-                editor->gotoLine(*line, column - 1);
-                editor->insert(definition);
-                *line += 1;
-            }
-            return doc;
-        }
-    }
-    return Document::Ptr();
-}
-
 static QString addConstRefIfNeeded(const QString &argument)
 {
     if (argument.startsWith("const ") || argument.endsWith('&') || argument.endsWith('*'))
@@ -417,14 +354,14 @@ using ClassDocumentPtrPair = QPair<const Class *, Document::Ptr>;
 
 static ClassDocumentPtrPair
         findClassRecursively(const LookupContext &context, const QString &className,
-                             unsigned maxIncludeDepth, QString *namespaceName)
+                             unsigned maxIncludeDepth)
 {
     const Document::Ptr doc = context.thisDocument();
     const Snapshot docTable = context.snapshot();
     if (Designer::Constants::Internal::debug)
         qDebug() << Q_FUNC_INFO << doc->fileName() << className << maxIncludeDepth;
     // Check document
-    if (const Class *cl = findClass(doc->globalNamespace(), context, className, namespaceName))
+    if (const Class *cl = findClass(doc->globalNamespace(), context, className))
         return ClassDocumentPtrPair(cl, doc);
     if (maxIncludeDepth) {
         // Check the includes
@@ -436,7 +373,7 @@ static ClassDocumentPtrPair
                 const Document::Ptr &includeDoc = it.value();
                 LookupContext context(includeDoc, docTable);
                 const ClassDocumentPtrPair irc = findClassRecursively(context, className,
-                    recursionMaxIncludeDepth, namespaceName);
+                    recursionMaxIncludeDepth);
                 if (irc.first)
                     return irc;
             }
@@ -456,16 +393,18 @@ void QtCreatorIntegration::slotNavigateToSlot(const QString &objectName, const Q
 // Build name of the class as generated by uic, insert Ui namespace
 // "foo::bar::form" -> "foo::bar::Ui::form"
 
-static inline QString uiClassName(QString formObjectName)
+static inline const QStringList uiClassNames(QString formObjectName)
 {
     const int indexOfScope = formObjectName.lastIndexOf("::");
     const int uiNameSpaceInsertionPos = indexOfScope >= 0 ? indexOfScope + 2 : 0;
+    QString alt = formObjectName;
     formObjectName.insert(uiNameSpaceInsertionPos, "Ui::");
-    return formObjectName;
+    alt.insert(uiNameSpaceInsertionPos, "Ui_");
+    return {formObjectName, alt};
 }
 
 static Document::Ptr getParsedDocument(const QString &fileName,
-                                       CppTools::WorkingCopy &workingCopy,
+                                       CppEditor::WorkingCopy &workingCopy,
                                        Snapshot &snapshot)
 {
     QByteArray src;
@@ -473,11 +412,11 @@ static Document::Ptr getParsedDocument(const QString &fileName,
         src = workingCopy.source(fileName);
     } else {
         Utils::FileReader reader;
-        if (reader.fetch(fileName)) // ### FIXME error reporting
+        if (reader.fetch(Utils::FilePath::fromString(fileName))) // ### FIXME error reporting
             src = QString::fromLocal8Bit(reader.data()).toUtf8();
     }
 
-    Document::Ptr doc = snapshot.preprocessedDocument(src, fileName);
+    Document::Ptr doc = snapshot.preprocessedDocument(src, FilePath::fromString(fileName));
     doc->check();
     snapshot.insert(doc);
     return doc;
@@ -508,7 +447,7 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
     const QString uicedName = "ui_" + fi.completeBaseName() + ".h";
 
     // Retrieve code model snapshot restricted to project of ui file or the working copy.
-    Snapshot docTable = CppTools::CppModelManager::instance()->snapshot();
+    Snapshot docTable = CppEditor::CppModelManager::instance()->snapshot();
     Snapshot newDocTable;
     const Project *uiProject = SessionManager::projectForFile(currentUiFile);
     if (uiProject) {
@@ -519,9 +458,9 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
         }
     } else {
         const Utils::FilePath configFileName =
-                Utils::FilePath::fromString(CppTools::CppModelManager::configurationFileName());
-        const CppTools::WorkingCopy::Table elements =
-                CppTools::CppModelManager::instance()->workingCopy().elements();
+                Utils::FilePath::fromString(CppEditor::CppModelManager::configurationFileName());
+        const CppEditor::WorkingCopy::Table elements =
+                CppEditor::CppModelManager::instance()->workingCopy().elements();
         for (auto it = elements.cbegin(), end = elements.cend(); it != end; ++it) {
             const Utils::FilePath &fileName = it.key();
             if (fileName != configFileName)
@@ -549,77 +488,106 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
 
     QDesignerFormWindowInterface *fwi = FormEditorW::activeWidgetHost()->formWindow();
 
-    const QString uiClass = uiClassName(fwi->mainContainer()->objectName());
-
-    if (Designer::Constants::Internal::debug)
-        qDebug() << "Checking docs for " << uiClass;
-
-    // Find the class definition (ui class defined as member or base class)
-    // in the file itself or in the directly included files (order 1).
-    QString namespaceName;
+    QString uiClass;
     const Class *cl = nullptr;
-    Document::Ptr doc;
+    Document::Ptr declDoc;
+    for (const QString &candidate : uiClassNames(fwi->mainContainer()->objectName())) {
+        if (Designer::Constants::Internal::debug)
+            qDebug() << "Checking docs for " << candidate;
 
-    for (const Document::Ptr &d : qAsConst(docMap)) {
-        LookupContext context(d, docTable);
-        const ClassDocumentPtrPair cd = findClassRecursively(context, uiClass, 1u , &namespaceName);
-        if (cd.first) {
-            cl = cd.first;
-            doc = cd.second;
+        // Find the class definition (ui class defined as member or base class)
+        // in the file itself or in the directly included files (order 1).
+        for (const Document::Ptr &d : qAsConst(docMap)) {
+            LookupContext context(d, docTable);
+            const ClassDocumentPtrPair cd = findClassRecursively(context, candidate, 1u);
+            if (cd.first) {
+                cl = cd.first;
+                declDoc = cd.second;
+                break;
+            }
+        }
+        if (cl) {
+            uiClass = candidate;
             break;
         }
-    }
-    if (!cl) {
-        *errorMessage = msgClassNotFound(uiClass, docList);
-        return false;
-    }
 
-    Overview o;
-    const QString className = namespaceName + o.prettyName(cl->name());
-    if (Designer::Constants::Internal::debug)
-        qDebug() << "Found class  " << className << doc->fileName();
+        if (errorMessage->isEmpty())
+            *errorMessage = msgClassNotFound(candidate, docList);
+    }
+    if (!cl)
+        return false;
 
     const QString functionName = "on_" + objectName + '_' + signalSignature;
     const QString functionNameWithParameterNames = addParameterNames(functionName, parameterNames);
 
     if (Designer::Constants::Internal::debug)
-        qDebug() << Q_FUNC_INFO << "Found " << uiClass << doc->fileName() << " checking " << functionName  << functionNameWithParameterNames;
+        qDebug() << Q_FUNC_INFO << "Found " << uiClass << declDoc->fileName() << " checking " << functionName  << functionNameWithParameterNames;
 
-    int line = 0;
-    Document::Ptr sourceDoc;
-
-    if (Function *fun = findDeclaration(cl, functionName)) {
-        sourceDoc = findDefinition(fun, &line);
-        if (!sourceDoc) {
-            // add function definition to cpp file
-            sourceDoc = addDefinition(docTable, doc->fileName(), className, functionNameWithParameterNames, &line);
-        }
-    } else {
+    Function *fun = findDeclaration(cl, functionName);
+    QString declFilePath;
+    if (!fun) {
         // add function declaration to cl
-        CppTools::WorkingCopy workingCopy =
-            CppTools::CppModelManager::instance()->workingCopy();
-        const QString fileName = doc->fileName();
-        getParsedDocument(fileName, workingCopy, docTable);
-        addDeclaration(docTable, fileName, cl, functionNameWithParameterNames);
+        CppEditor::WorkingCopy workingCopy = CppEditor::CppModelManager::instance()->workingCopy();
+        declFilePath = declDoc->fileName();
+        getParsedDocument(declFilePath, workingCopy, docTable);
+        addDeclaration(docTable, declFilePath, cl, functionNameWithParameterNames);
 
-        // add function definition to cpp file
-        sourceDoc = addDefinition(docTable, fileName, className, functionNameWithParameterNames, &line);
+        // Re-load C++ documents.
+        QList<Utils::FilePath> filePaths;
+        for (auto it = docTable.begin(); it != docTable.end(); ++it)
+            filePaths << it.key();
+        workingCopy = CppEditor::CppModelManager::instance()->workingCopy();
+        docTable = CppEditor::CppModelManager::instance()->snapshot();
+        newDocTable = {};
+        for (const auto &file : qAsConst(filePaths)) {
+            const Document::Ptr doc = docTable.document(file);
+            if (doc)
+                newDocTable.insert(doc);
+        }
+        docTable = newDocTable;
+        getParsedDocument(declFilePath, workingCopy, docTable);
+        const Document::Ptr headerDoc = docTable.document(declFilePath);
+        QTC_ASSERT(headerDoc, return false);
+        LookupContext context(headerDoc, docTable);
+        cl = findClass(headerDoc->globalNamespace(), context, uiClass);
+        QTC_ASSERT(cl, return false);
+        fun = findDeclaration(cl, functionName);
+    } else {
+        declFilePath = QLatin1String(fun->fileName());
+    }
+    QTC_ASSERT(fun, return false);
+
+    CppEditor::CppRefactoringChanges refactoring(docTable);
+    CppEditor::SymbolFinder symbolFinder;
+    if (const Function *funImpl = symbolFinder.findMatchingDefinition(fun, docTable, true)) {
+        Core::EditorManager::openEditorAt(QString::fromUtf8(funImpl->fileName()),
+                                          funImpl->line() + 2);
+        return true;
+    }
+    const QString implFilePath = CppEditor::correspondingHeaderOrSource(declFilePath);
+    const CppEditor::InsertionLocation location = CppEditor::insertLocationForMethodDefinition
+            (fun, false, CppEditor::NamespaceHandling::CreateMissing, refactoring, implFilePath);
+
+    if (BaseTextEditor *editor = editorAt(location.fileName(), location.line(), location.column())) {
+        Overview o;
+        const QString className = o.prettyName(cl->name());
+        const QString definition = location.prefix() + "void " + className + "::"
+            + functionNameWithParameterNames + "\n{\n" + QString(indentation, ' ') + "\n}\n"
+            + location.suffix();
+        editor->insert(definition);
+        Core::EditorManager::openEditorAt(location.fileName(),
+                                          location.line() + location.prefix().count('\n') + 2,
+                                          indentation);
+        return true;
     }
 
-    if (!sourceDoc) {
-        *errorMessage = tr("Unable to add the method definition.");
-        return false;
-    }
-
-    // jump to function definition, position within code
-    Core::EditorManager::openEditorAt(sourceDoc->fileName(), line + 2, indentation);
-
-    return true;
+    *errorMessage = tr("Unable to add the method definition.");
+    return false;
 }
 
 void QtCreatorIntegration::slotSyncSettingsToDesigner()
 {
     // Set promotion-relevant parameters on integration.
-    setHeaderSuffix(Utils::mimeTypeForName(CppTools::Constants::CPP_HEADER_MIMETYPE).preferredSuffix());
+    setHeaderSuffix(Utils::mimeTypeForName(CppEditor::Constants::CPP_HEADER_MIMETYPE).preferredSuffix());
     setHeaderLowercase(FormClassWizardPage::lowercaseHeaderFiles());
 }
