@@ -40,7 +40,7 @@ const char SIDE_BY_SIDE_VIEW_ID[] = "DiffEditor.SideBySide";
 const char UNIFIED_VIEW_ID[] = "DiffEditor.Unified";
 const char SELECT_ENCODING[] = "DiffEditor.SelectEncoding";
 
-const char G_TOOLS_DIFF[] = "QtCreator.Group.Tools.Options";
+const char G_TOOLS_DIFF[] = "QtCreator.Group.Tools.Diff";
 
 } // namespace Constants
 } // namespace DiffEditor
