@@ -171,7 +171,7 @@ QString DiffEditorDocument::makePatch(int fileIndex, int chunkIndex,
                                 lastChunk && fileData.lastChunkAtTheEndOfFile);
 }
 
-void DiffEditorDocument::setDiffFiles(const QList<FileData> &data, const QString &directory,
+void DiffEditorDocument::setDiffFiles(const QList<FileData> &data, const FilePath &directory,
                                       const QString &startupFile)
 {
     m_diffFiles = data;
@@ -186,12 +186,12 @@ QList<FileData> DiffEditorDocument::diffFiles() const
     return m_diffFiles;
 }
 
-QString DiffEditorDocument::baseDirectory() const
+FilePath DiffEditorDocument::baseDirectory() const
 {
     return m_baseDirectory;
 }
 
-void DiffEditorDocument::setBaseDirectory(const QString &directory)
+void DiffEditorDocument::setBaseDirectory(const FilePath &directory)
 {
     m_baseDirectory = directory;
 }
@@ -253,11 +253,11 @@ bool DiffEditorDocument::setContents(const QByteArray &contents)
     return true;
 }
 
-QString DiffEditorDocument::fallbackSaveAsPath() const
+FilePath DiffEditorDocument::fallbackSaveAsPath() const
 {
     if (!m_baseDirectory.isEmpty())
         return m_baseDirectory;
-    return QDir::homePath();
+    return FileUtils::homePath();
 }
 
 bool DiffEditorDocument::isSaveAsAllowed() const
@@ -265,7 +265,7 @@ bool DiffEditorDocument::isSaveAsAllowed() const
     return state() == LoadOK;
 }
 
-bool DiffEditorDocument::save(QString *errorString, const QString &fileName, bool autoSave)
+bool DiffEditorDocument::save(QString *errorString, const FilePath &filePath, bool autoSave)
 {
     Q_UNUSED(errorString)
     Q_UNUSED(autoSave)
@@ -273,7 +273,7 @@ bool DiffEditorDocument::save(QString *errorString, const QString &fileName, boo
     if (state() != LoadOK)
         return false;
 
-    const bool ok = write(fileName, format(), plainText(), errorString);
+    const bool ok = write(filePath, format(), plainText(), errorString);
 
     if (!ok)
         return false;
@@ -282,9 +282,8 @@ bool DiffEditorDocument::save(QString *errorString, const QString &fileName, boo
     setDescription(QString());
     Core::EditorManager::clearUniqueId(this);
 
-    const QFileInfo fi(fileName);
     setTemporary(false);
-    setFilePath(FilePath::fromString(fi.absoluteFilePath()));
+    setFilePath(filePath.absoluteFilePath());
     setPreferredDisplayName(QString());
     emit temporaryStateChanged();
 
@@ -306,16 +305,16 @@ bool DiffEditorDocument::reload(QString *errorString, ReloadFlag flag, ChangeTyp
     Q_UNUSED(type)
     if (flag == FlagIgnore)
         return true;
-    return open(errorString, filePath().toString(), filePath().toString()) == OpenResult::Success;
+    return open(errorString, filePath(), filePath()) == OpenResult::Success;
 }
 
-Core::IDocument::OpenResult DiffEditorDocument::open(QString *errorString, const QString &fileName,
-                              const QString &realFileName)
+Core::IDocument::OpenResult DiffEditorDocument::open(QString *errorString, const Utils::FilePath &filePath,
+                              const Utils::FilePath &realFilePath)
 {
-    QTC_CHECK(fileName == realFileName); // does not support autosave
+    QTC_CHECK(filePath == realFilePath); // does not support autosave
     beginReload();
     QString patch;
-    ReadResult readResult = read(fileName, &patch, errorString);
+    ReadResult readResult = read(filePath, &patch, errorString);
     if (readResult == TextFileFormat::ReadIOError
         || readResult == TextFileFormat::ReadMemoryAllocationError) {
         return OpenResult::ReadError;
@@ -326,13 +325,12 @@ Core::IDocument::OpenResult DiffEditorDocument::open(QString *errorString, const
     if (!ok) {
         *errorString = tr("Could not parse patch file \"%1\". "
                           "The content is not of unified diff format.")
-                .arg(fileName);
+                .arg(filePath.toUserOutput());
     } else {
-        const QFileInfo fi(fileName);
         setTemporary(false);
         emit temporaryStateChanged();
-        setFilePath(FilePath::fromString(fi.absoluteFilePath()));
-        setDiffFiles(fileDataList, fi.absolutePath());
+        setFilePath(filePath.absoluteFilePath());
+        setDiffFiles(fileDataList, filePath.absoluteFilePath());
     }
     endReload(ok);
     if (!ok && readResult == TextFileFormat::ReadEncodingError)
@@ -417,7 +415,7 @@ void DiffEditorDocument::beginReload()
     m_state = Reloading;
     emit changed();
     QSignalBlocker blocker(this);
-    setDiffFiles(QList<FileData>(), QString());
+    setDiffFiles(QList<FileData>(), {});
     setDescription(QString());
 }
 
