@@ -88,7 +88,7 @@ public:
 
         FileData fileData;
         if (!reloadInfo.binaryFiles) {
-            const QList<Diff> diffList = differ.cleanupSemantics(
+            const QList<Diff> diffList = Differ::cleanupSemantics(
                         differ.diff(reloadInfo.leftText, reloadInfo.rightText));
 
             QList<Diff> leftDiffList;
@@ -199,7 +199,7 @@ QList<ReloadInput> DiffCurrentFileController::reloadInputList() const
     QList<ReloadInput> result;
 
     auto textDocument = qobject_cast<TextEditor::TextDocument *>(
-        DocumentModel::documentForFilePath(Utils::FilePath::fromString(m_fileName)));
+        DocumentModel::documentForFilePath(FilePath::fromString(m_fileName)));
 
     if (textDocument && textDocument->isModified()) {
         QString errorString;
@@ -207,7 +207,7 @@ QList<ReloadInput> DiffCurrentFileController::reloadInputList() const
 
         QString leftText;
         const Utils::TextFileFormat::ReadResult leftResult
-                = Utils::TextFileFormat::readFile(m_fileName, format.codec,
+            = Utils::TextFileFormat::readFile(FilePath::fromString(m_fileName), format.codec,
                                         &leftText, &format, &errorString);
 
         const QString rightText = textDocument->plainText();
@@ -254,6 +254,7 @@ QList<ReloadInput> DiffOpenFilesController::reloadInputList() const
     const QList<IDocument *> openedDocuments = DocumentModel::openedDocuments();
 
     for (IDocument *doc : openedDocuments) {
+        QTC_ASSERT(doc, continue);
         auto textDocument = qobject_cast<TextEditor::TextDocument *>(doc);
 
         if (textDocument && textDocument->isModified()) {
@@ -262,9 +263,8 @@ QList<ReloadInput> DiffOpenFilesController::reloadInputList() const
 
             QString leftText;
             const QString fileName = textDocument->filePath().toString();
-            const Utils::TextFileFormat::ReadResult leftResult
-                    = Utils::TextFileFormat::readFile(fileName, format.codec,
-                                            &leftText, &format, &errorString);
+            const Utils::TextFileFormat::ReadResult leftResult = Utils::TextFileFormat::readFile(
+                FilePath::fromString(fileName), format.codec, &leftText, &format, &errorString);
 
             const QString rightText = textDocument->plainText();
 
@@ -321,9 +321,8 @@ QList<ReloadInput> DiffModifiedFilesController::reloadInputList() const
 
             QString leftText;
             const QString fileName = textDocument->filePath().toString();
-            const Utils::TextFileFormat::ReadResult leftResult
-                    = Utils::TextFileFormat::readFile(fileName, format.codec,
-                                            &leftText, &format, &errorString);
+            const Utils::TextFileFormat::ReadResult leftResult = Utils::TextFileFormat::readFile(
+                FilePath::fromString(fileName), format.codec, &leftText, &format, &errorString);
 
             const QString rightText = textDocument->plainText();
 
@@ -378,12 +377,10 @@ QList<ReloadInput> DiffExternalFilesController::reloadInputList() const
     QString leftText;
     QString rightText;
 
-    const Utils::TextFileFormat::ReadResult leftResult
-            = Utils::TextFileFormat::readFile(m_leftFileName, format.codec,
-                                    &leftText, &format, &errorString);
-    const Utils::TextFileFormat::ReadResult rightResult
-            = Utils::TextFileFormat::readFile(m_rightFileName, format.codec,
-                                    &rightText, &format, &errorString);
+    const Utils::TextFileFormat::ReadResult leftResult = Utils::TextFileFormat::readFile(
+        FilePath::fromString(m_leftFileName), format.codec, &leftText, &format, &errorString);
+    const Utils::TextFileFormat::ReadResult rightResult = Utils::TextFileFormat::readFile(
+        FilePath::fromString(m_rightFileName), format.codec, &rightText, &format, &errorString);
 
     ReloadInput reloadInput;
     reloadInput.leftText = leftText;
@@ -522,6 +519,7 @@ void DiffEditorPluginPrivate::updateDiffCurrentFileAction()
 void DiffEditorPluginPrivate::updateDiffOpenFilesAction()
 {
     const bool enabled = Utils::anyOf(DocumentModel::openedDocuments(), [](IDocument *doc) {
+            QTC_ASSERT(doc, return false);
             return doc->isModified() && qobject_cast<TextEditor::TextDocument *>(doc);
         });
     m_diffOpenFilesAction->setEnabled(enabled);
@@ -570,32 +568,28 @@ void DiffEditorPluginPrivate::diffOpenFiles()
 
 void DiffEditorPluginPrivate::diffExternalFiles()
 {
-    const QString fileName1 = QFileDialog::getOpenFileName(ICore::dialogParent(),
-                                                     tr("Select First File for Diff"),
-                                                     QString());
-    if (fileName1.isNull())
+    const FilePath filePath1 = FileUtils::getOpenFilePath(nullptr, tr("Select First File for Diff"));
+    if (filePath1.isEmpty())
         return;
-    if (EditorManager::skipOpeningBigTextFile(fileName1))
+    if (EditorManager::skipOpeningBigTextFile(filePath1))
         return;
 
-    const QString fileName2 = QFileDialog::getOpenFileName(ICore::dialogParent(),
-                                                     tr("Select Second File for Diff"),
-                                                     QString());
-    if (fileName2.isNull())
+    const FilePath filePath2 = FileUtils::getOpenFilePath(nullptr, tr("Select Second File for Diff"));
+    if (filePath2.isEmpty())
         return;
-    if (EditorManager::skipOpeningBigTextFile(fileName2))
+    if (EditorManager::skipOpeningBigTextFile(filePath2))
         return;
 
-    const QString documentId = Constants::DIFF_EDITOR_PLUGIN
-            + QLatin1String(".DiffExternalFiles.") + fileName1 + QLatin1Char('.') + fileName2;
-    const QString title = tr("Diff \"%1\", \"%2\"").arg(fileName1, fileName2);
+    const QString documentId = QLatin1String(Constants::DIFF_EDITOR_PLUGIN)
+            + ".DiffExternalFiles." + filePath1.toString() + '.' + filePath2.toString();
+    const QString title = tr("Diff \"%1\", \"%2\"").arg(filePath1.toString(), filePath2.toString());
     auto const document = qobject_cast<DiffEditorDocument *>(
                 DiffEditorController::findOrCreateDocument(documentId, title));
     if (!document)
         return;
 
     if (!DiffEditorController::controller(document))
-        new DiffExternalFilesController(document, fileName1, fileName2);
+        new DiffExternalFilesController(document, filePath1.toString(), filePath2.toString());
     EditorManager::activateEditorForDocument(document);
     document->reload();
 }
