@@ -159,10 +159,10 @@ void DiffEditorWidgetController::patch(bool revert, int fileIndex, int chunkInde
             ? fileData.rightFileInfo.patchBehaviour
             : fileData.leftFileInfo.patchBehaviour;
 
-    const QString workingDirectory = m_document->baseDirectory().isEmpty()
-            ? QFileInfo(fileName).absolutePath()
+    const FilePath workingDirectory = m_document->baseDirectory().isEmpty()
+            ? FilePath::fromString(fileName).absolutePath()
             : m_document->baseDirectory();
-    const QString absFileName = QFileInfo(workingDirectory + '/' + QFileInfo(fileName).fileName()).absoluteFilePath();
+    const FilePath absFilePath = workingDirectory.resolvePath(fileName).absoluteFilePath();
 
     if (patchBehaviour == DiffFileInfo::PatchFile) {
         const int strip = m_document->baseDirectory().isEmpty() ? -1 : 0;
@@ -172,13 +172,13 @@ void DiffEditorWidgetController::patch(bool revert, int fileIndex, int chunkInde
         if (patch.isEmpty())
             return;
 
-        FileChangeBlocker fileChangeBlocker(absFileName);
+        FileChangeBlocker fileChangeBlocker(absFilePath);
         if (PatchTool::runPatch(EditorManager::defaultTextCodec()->fromUnicode(patch),
                                 workingDirectory, strip, revert))
             m_document->reload();
     } else { // PatchEditor
         auto textDocument = qobject_cast<TextEditor::TextDocument *>(
-            DocumentModel::documentForFilePath(Utils::FilePath::fromString(absFileName)));
+            DocumentModel::documentForFilePath(absFilePath));
         if (!textDocument)
             return;
 
@@ -200,9 +200,9 @@ void DiffEditorWidgetController::patch(bool revert, int fileIndex, int chunkInde
             return;
 
         if (PatchTool::runPatch(EditorManager::defaultTextCodec()->fromUnicode(patch),
-                                contentsCopyDir, 0, revert)) {
+                                FilePath::fromString(contentsCopyDir), 0, revert)) {
             QString errorString;
-            if (textDocument->reload(&errorString, contentsCopyFileName))
+            if (textDocument->reload(&errorString, FilePath::fromString(contentsCopyFileName)))
                 m_document->reload();
         }
     }
@@ -215,11 +215,9 @@ void DiffEditorWidgetController::jumpToOriginalFile(const QString &fileName,
     if (!m_document)
         return;
 
-    const QDir dir(m_document->baseDirectory());
-    const QString absoluteFileName = dir.absoluteFilePath(fileName);
-    const QFileInfo fi(absoluteFileName);
-    if (fi.exists() && !fi.isDir())
-        EditorManager::openEditorAt(absoluteFileName, lineNumber, columnNumber);
+    const FilePath filePath = m_document->baseDirectory().resolvePath(fileName);
+    if (filePath.exists() && !filePath.isDir())
+        EditorManager::openEditorAt(filePath.toString(), lineNumber, columnNumber);
 }
 
 void DiffEditorWidgetController::setFontSettings(const FontSettings &fontSettings)
