@@ -25,7 +25,7 @@
 
 #pragma once
 
-#include <coreplugin/id.h>
+#include <utils/id.h>
 
 #include <QIcon>
 #include <QString>
@@ -50,20 +50,20 @@ class IDiffView : public QObject
     Q_OBJECT
 
 public:
-    explicit IDiffView(QObject *parent = 0);
+    explicit IDiffView(QObject *parent = nullptr);
 
     QIcon icon() const;
     QString toolTip() const;
     bool supportsSync() const;
     QString syncToolTip() const;
 
-    Core::Id id() const;
+    Utils::Id id() const;
     virtual QWidget *widget() = 0;
     virtual void setDocument(DiffEditorDocument *document) = 0;
 
     virtual void beginOperation() = 0;
     virtual void setCurrentDiffFileIndex(int index) = 0;
-    virtual void setDiff(const QList<FileData> &diffFileList, const QString &workingDirectory) = 0;
+    virtual void setDiff(const QList<FileData> &diffFileList) = 0;
     virtual void endOperation(bool success) = 0;
 
     virtual void setSync(bool) = 0;
@@ -74,15 +74,15 @@ signals:
 protected:
     void setIcon(const QIcon &icon);
     void setToolTip(const QString &toolTip);
-    void setId(const Core::Id &id);
+    void setId(const Utils::Id &id);
     void setSupportsSync(bool sync);
     void setSyncToolTip(const QString &text);
 
 private:
     QIcon m_icon;
     QString m_toolTip;
-    Core::Id m_id;
-    bool m_supportsSync;
+    Utils::Id m_id;
+    bool m_supportsSync = false;
     QString m_syncToolTip;
 };
 
@@ -93,17 +93,17 @@ class UnifiedView : public IDiffView
 public:
     UnifiedView();
 
-    QWidget *widget();
+    QWidget *widget() override;
     TextEditor::TextEditorWidget *textEditorWidget();
 
-    void setDocument(DiffEditorDocument *document);
+    void setDocument(DiffEditorDocument *document) override;
 
-    void beginOperation();
-    void setCurrentDiffFileIndex(int index);
-    void setDiff(const QList<FileData> &diffFileList, const QString &workingDirectory);
-    void endOperation(bool success);
+    void beginOperation() override;
+    void setCurrentDiffFileIndex(int index) override;
+    void setDiff(const QList<FileData> &diffFileList) override;
+    void endOperation(bool success) override;
 
-    void setSync(bool sync);
+    void setSync(bool sync) override;
 
 private:
     UnifiedDiffEditorWidget *m_widget = nullptr;
@@ -116,18 +116,18 @@ class SideBySideView : public IDiffView
 public:
     SideBySideView();
 
-    QWidget *widget();
+    QWidget *widget() override;
     TextEditor::TextEditorWidget *leftEditorWidget();
     TextEditor::TextEditorWidget *rightEditorWidget();
 
-    void setDocument(DiffEditorDocument *document);
+    void setDocument(DiffEditorDocument *document) override;
 
-    void beginOperation();
-    void setCurrentDiffFileIndex(int index);
-    void setDiff(const QList<FileData> &diffFileList, const QString &workingDirectory);
-    void endOperation(bool success);
+    void beginOperation() override;
+    void setCurrentDiffFileIndex(int index) override;
+    void setDiff(const QList<FileData> &diffFileList) override;
+    void endOperation(bool success) override;
 
-    void setSync(bool sync);
+    void setSync(bool sync) override;
 
 private:
     SideBySideDiffEditorWidget *m_widget;
