@@ -40,7 +40,6 @@ class TextEditorWidget;
 QT_BEGIN_NAMESPACE
 class QMenu;
 class QSplitter;
-class QTextBlock;
 QT_END_NAMESPACE
 
 namespace DiffEditor {
@@ -57,7 +56,6 @@ class SideBySideDiffEditorWidget : public QWidget
     Q_OBJECT
 public:
     explicit SideBySideDiffEditorWidget(QWidget *parent = nullptr);
-    ~SideBySideDiffEditorWidget();
 
     TextEditor::TextEditorWidget *leftEditorWidget() const;
     TextEditor::TextEditorWidget *rightEditorWidget() const;
@@ -65,8 +63,7 @@ public:
     void setDocument(DiffEditorDocument *document);
     DiffEditorDocument *diffDocument() const;
 
-    void setDiff(const QList<FileData> &diffFileList,
-                 const QString &workingDirectory);
+    void setDiff(const QList<FileData> &diffFileList);
     void setCurrentDiffFileIndex(int diffFileIndex);
 
     void setHorizontalSync(bool sync);
@@ -86,9 +83,9 @@ private:
     void slotRightJumpToOriginalFileRequested(int diffFileIndex,
                                               int lineNumber, int columnNumber);
     void slotLeftContextMenuRequested(QMenu *menu, int fileIndex,
-                                      int chunkIndex);
+                                      int chunkIndex, const ChunkSelection &selection);
     void slotRightContextMenuRequested(QMenu *menu, int fileIndex,
-                                       int chunkIndex);
+                                       int chunkIndex, const ChunkSelection &selection);
     void leftVSliderChanged();
     void rightVSliderChanged();
     void leftHSliderChanged();
@@ -110,8 +107,6 @@ private:
     bool m_horizontalSync = false;
 
     QTextCharFormat m_spanLineFormat;
-    Core::IContext *m_leftContext = nullptr;
-    Core::IContext *m_rightContext = nullptr;
 };
 
 } // namespace Internal
