@@ -28,7 +28,6 @@
 #include "fakevimactions.h"
 #include "fakevimhandler.h"
 #include "fakevimtr.h"
-#include "ui_fakevimoptions.h"
 
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
@@ -67,26 +66,31 @@
 #include <texteditor/codeassist/assistinterface.h>
 #include <texteditor/codeassist/genericproposal.h>
 
+#include <utils/aspects.h>
 #include <utils/fancylineedit.h>
 #include <utils/hostosinfo.h>
-#include <utils/qtcassert.h>
+#include <utils/layoutbuilder.h>
 #include <utils/pathchooser.h>
 #include <utils/qtcassert.h>
-#include <utils/savedaction.h>
+#include <utils/qtcassert.h>
 #include <utils/stylehelper.h>
 
-#include <cpptools/cpptoolsconstants.h>
+#include <cppeditor/cppeditorconstants.h>
 
 #include <extensionsystem/pluginmanager.h>
 
+#include <QAction>
 #include <QAbstractTableModel>
 #include <QDebug>
 #include <QFile>
+#include <QGridLayout>
+#include <QGroupBox>
 #include <QGuiApplication>
 #include <QItemDelegate>
 #include <QPainter>
 #include <QPlainTextEdit>
 #include <QPointer>
+#include <QPushButton>
 #include <QRegularExpression>
 #include <QScrollBar>
 #include <QSettings>
@@ -367,139 +371,136 @@ public:
         setCategory(SETTINGS_CATEGORY);
         setDisplayCategory(Tr::tr("FakeVim"));
         setCategoryIconPath(":/fakevim/images/settingscategory_fakevim.png");
+        setLayouter([this](QWidget *widget) { return layoutPage(widget); });
+        setSettings(fakeVimSettings());
     }
 
-    QWidget *widget() override;
-    void apply() override;
-    void finish() override;
-
 private:
+    void layoutPage(QWidget *);
     void copyTextEditorSettings();
     void setQtStyle();
     void setPlainStyle();
-    void updateVimRcWidgets();
-
-    QPointer<QWidget> m_widget;
-    Ui::FakeVimOptionPage m_ui;
-    SavedActionSet m_group;
 };
 
-QWidget *FakeVimOptionPage::widget()
+void FakeVimOptionPage::layoutPage(QWidget *widget)
 {
-    if (!m_widget) {
-        m_widget = new QWidget;
-        m_ui.setupUi(m_widget);
-        const QString vimrcDefault = QLatin1String(HostOsInfo::isAnyUnixHost()
-                ? "$HOME/.vimrc" : "%USERPROFILE%\\_vimrc");
-        m_ui.pathChooserVimRcPath->setExpectedKind(PathChooser::File);
-        m_ui.pathChooserVimRcPath->lineEdit()->setToolTip(Tr::tr("Keep empty to use the default path, i.e. "
-                                                             "%USERPROFILE%\\_vimrc on Windows, ~/.vimrc otherwise."));
-        m_ui.pathChooserVimRcPath->lineEdit()->setPlaceholderText(Tr::tr("Default: %1").arg(vimrcDefault));
+    auto copyTextEditorSettings = new QPushButton(Tr::tr("Copy Text Editor Settings"));
+    auto setQtStyle = new QPushButton(Tr::tr("Set Qt Style"));
+    auto setPlainStyle = new QPushButton(Tr::tr("Set Plain Style"));
 
-        m_group.clear();
-        m_group.insert(theFakeVimSetting(ConfigUseFakeVim), m_ui.checkBoxUseFakeVim);
-        m_group.insert(theFakeVimSetting(ConfigReadVimRc), m_ui.checkBoxReadVimRc);
-        m_group.insert(theFakeVimSetting(ConfigVimRcPath), m_ui.pathChooserVimRcPath);
+    using namespace Layouting;
+    FakeVimSettings &s = *fakeVimSettings();
 
-        m_group.insert(theFakeVimSetting(ConfigExpandTab), m_ui.checkBoxExpandTab);
-        m_group.insert(theFakeVimSetting(ConfigHlSearch), m_ui.checkBoxHlSearch);
-        m_group.insert(theFakeVimSetting(ConfigShiftWidth), m_ui.spinBoxShiftWidth);
-        m_group.insert(theFakeVimSetting(ConfigShowMarks), m_ui.checkBoxShowMarks);
+    Row bools {
+        Column {
+            s.autoIndent,
+            s.smartIndent,
+            s.expandTab,
+            s.smartTab,
+            s.hlSearch,
+            s.showCmd,
+            s.startOfLine,
+            s.passKeys,
+            s.blinkingCursor
+        },
+        Column {
+            s.incSearch,
+            s.useCoreSearch,
+            s.ignoreCase,
+            s.smartCase,
+            s.wrapScan,
+            s.showMarks,
+            s.passControlKey,
+            s.relativeNumber,
+            s.tildeOp
+        }
+    };
 
-        m_group.insert(theFakeVimSetting(ConfigSmartTab), m_ui.checkBoxSmartTab);
-        m_group.insert(theFakeVimSetting(ConfigStartOfLine), m_ui.checkBoxStartOfLine);
-        m_group.insert(theFakeVimSetting(ConfigPassKeys), m_ui.checkBoxPassKeys);
-        m_group.insert(theFakeVimSetting(ConfigTabStop), m_ui.spinBoxTabStop);
-        m_group.insert(theFakeVimSetting(ConfigScrollOff), m_ui.spinBoxScrollOff);
-        m_group.insert(theFakeVimSetting(ConfigBackspace), m_ui.lineEditBackspace);
-        m_group.insert(theFakeVimSetting(ConfigIsKeyword), m_ui.lineEditIsKeyword);
+    Row ints { s.shiftWidth, s.tabStop, s.scrollOff, Stretch() };
 
-        m_group.insert(theFakeVimSetting(ConfigPassControlKey), m_ui.checkBoxPassControlKey);
-        m_group.insert(theFakeVimSetting(ConfigAutoIndent), m_ui.checkBoxAutoIndent);
-        m_group.insert(theFakeVimSetting(ConfigSmartIndent), m_ui.checkBoxSmartIndent);
+    Column strings {
+        s.backspace,
+        s.isKeyword,
+        Row {s.readVimRc, s.vimRcPath}
+    };
 
-        m_group.insert(theFakeVimSetting(ConfigIncSearch), m_ui.checkBoxIncSearch);
-        m_group.insert(theFakeVimSetting(ConfigUseCoreSearch), m_ui.checkBoxUseCoreSearch);
-        m_group.insert(theFakeVimSetting(ConfigSmartCase), m_ui.checkBoxSmartCase);
-        m_group.insert(theFakeVimSetting(ConfigIgnoreCase), m_ui.checkBoxIgnoreCase);
-        m_group.insert(theFakeVimSetting(ConfigWrapScan), m_ui.checkBoxWrapScan);
+    Column {
+        s.useFakeVim,
 
-        m_group.insert(theFakeVimSetting(ConfigShowCmd), m_ui.checkBoxShowCmd);
+        Group {
+            Title(Tr::tr("Vim Behavior")),
+            bools,
+            ints,
+            strings
+        },
 
-        m_group.insert(theFakeVimSetting(ConfigRelativeNumber), m_ui.checkBoxRelativeNumber);
-        m_group.insert(theFakeVimSetting(ConfigBlinkingCursor), m_ui.checkBoxBlinkingCursor);
+        Group {
+            Title(Tr::tr("Plugin Emulation")),
+            s.emulateVimCommentary,
+            s.emulateReplaceWithRegister,
+            s.emulateArgTextObj,
+            s.emulateExchange,
+            s.emulateSurround
+        },
 
-        connect(m_ui.pushButtonCopyTextEditorSettings, &QAbstractButton::clicked,
-                this, &FakeVimOptionPage::copyTextEditorSettings);
-        connect(m_ui.pushButtonSetQtStyle, &QAbstractButton::clicked,
-                this, &FakeVimOptionPage::setQtStyle);
-        connect(m_ui.pushButtonSetPlainStyle, &QAbstractButton::clicked,
-                this, &FakeVimOptionPage::setPlainStyle);
-        connect(m_ui.checkBoxReadVimRc, &QCheckBox::stateChanged,
-                this, &FakeVimOptionPage::updateVimRcWidgets);
-        updateVimRcWidgets();
+        Row { copyTextEditorSettings, setQtStyle, setPlainStyle, Stretch() },
+        Stretch()
 
-    }
-    return m_widget;
-}
+    }.attachTo(widget, true);
 
-void FakeVimOptionPage::apply()
-{
-    m_group.apply(ICore::settings());
-}
+    s.vimRcPath.setEnabler(&s.readVimRc);
 
-void FakeVimOptionPage::finish()
-{
-    m_group.finish();
-    delete m_widget;
+    connect(copyTextEditorSettings, &QAbstractButton::clicked,
+            this, &FakeVimOptionPage::copyTextEditorSettings);
+    connect(setQtStyle, &QAbstractButton::clicked,
+            this, &FakeVimOptionPage::setQtStyle);
+    connect(setPlainStyle, &QAbstractButton::clicked,
+            this, &FakeVimOptionPage::setPlainStyle);
 }
 
 void FakeVimOptionPage::copyTextEditorSettings()
 {
+    FakeVimSettings &s = *fakeVimSettings();
     TabSettings ts = TextEditorSettings::codeStyle()->tabSettings();
     TypingSettings tps = TextEditorSettings::typingSettings();
-    m_ui.checkBoxExpandTab->setChecked(ts.m_tabPolicy != TabSettings::TabsOnlyTabPolicy);
-    m_ui.spinBoxTabStop->setValue(ts.m_tabSize);
-    m_ui.spinBoxShiftWidth->setValue(ts.m_indentSize);
-    m_ui.checkBoxSmartTab->setChecked(
-        tps.m_smartBackspaceBehavior == TypingSettings::BackspaceFollowsPreviousIndents);
-    m_ui.checkBoxAutoIndent->setChecked(true);
-    m_ui.checkBoxSmartIndent->setChecked(tps.m_autoIndent);
-    m_ui.checkBoxIncSearch->setChecked(true);
+    s.expandTab.setValue(ts.m_tabPolicy != TabSettings::TabsOnlyTabPolicy);
+    s.tabStop.setValue(ts.m_tabSize);
+    s.shiftWidth.setValue(ts.m_indentSize);
+    s.smartTab.setValue(tps.m_smartBackspaceBehavior
+                        == TypingSettings::BackspaceFollowsPreviousIndents);
+    s.autoIndent.setValue(true);
+    s.smartIndent.setValue(tps.m_autoIndent);
+    s.incSearch.setValue(true);
 }
 
 void FakeVimOptionPage::setQtStyle()
 {
-    m_ui.checkBoxExpandTab->setChecked(true);
-    m_ui.spinBoxTabStop->setValue(4);
-    m_ui.spinBoxShiftWidth->setValue(4);
-    m_ui.checkBoxSmartTab->setChecked(true);
-    m_ui.checkBoxAutoIndent->setChecked(true);
-    m_ui.checkBoxSmartIndent->setChecked(true);
-    m_ui.checkBoxIncSearch->setChecked(true);
-    m_ui.lineEditBackspace->setText("indent,eol,start");
-    m_ui.checkBoxPassKeys->setChecked(true);
+    FakeVimSettings &s = *fakeVimSettings();
+    s.expandTab.setVolatileValue(true);
+    s.tabStop.setVolatileValue(4);
+    s.shiftWidth.setVolatileValue(4);
+    s.smartTab.setVolatileValue(true);
+    s.autoIndent.setVolatileValue(true);
+    s.smartIndent.setVolatileValue(true);
+    s.incSearch.setVolatileValue(true);
+    s.backspace.setVolatileValue(QString("indent,eol,start"));
+    s.passKeys.setVolatileValue(true);
 }
 
 void FakeVimOptionPage::setPlainStyle()
 {
-    m_ui.checkBoxExpandTab->setChecked(false);
-    m_ui.spinBoxTabStop->setValue(8);
-    m_ui.spinBoxShiftWidth->setValue(8);
-    m_ui.checkBoxSmartTab->setChecked(false);
-    m_ui.checkBoxAutoIndent->setChecked(false);
-    m_ui.checkBoxSmartIndent->setChecked(false);
-    m_ui.checkBoxIncSearch->setChecked(false);
-    m_ui.lineEditBackspace->clear();
-    m_ui.checkBoxPassKeys->setChecked(false);
+    FakeVimSettings &s = *fakeVimSettings();
+    s.expandTab.setVolatileValue(false);
+    s.tabStop.setVolatileValue(8);
+    s.shiftWidth.setVolatileValue(8);
+    s.smartTab.setVolatileValue(false);
+    s.autoIndent.setVolatileValue(false);
+    s.smartIndent.setVolatileValue(false);
+    s.incSearch.setVolatileValue(false);
+    s.backspace.setVolatileValue(QString());
+    s.passKeys.setVolatileValue(false);
 }
 
-void FakeVimOptionPage::updateVimRcWidgets()
-{
-    m_ui.pathChooserVimRcPath->setEnabled(m_ui.checkBoxReadVimRc->isChecked());
-}
-
-
 ///////////////////////////////////////////////////////////////////////
 //
 // FakeVimPluginPrivate
@@ -521,17 +522,17 @@ public:
     void editorAboutToClose(Core::IEditor *);
     void currentEditorAboutToChange(Core::IEditor *);
 
-    void allDocumentsRenamed(const QString &oldName, const QString &newName);
-    void documentRenamed(Core::IDocument *document, const QString &oldName, const QString &newName);
-    void renameFileNameInEditors(const QString &oldName, const QString &newName);
+    void allDocumentsRenamed(const FilePath &oldPath, const FilePath &newPath);
+    void documentRenamed(Core::IDocument *document, const FilePath &oldPath, const FilePath &newPath);
+    void renameFileNameInEditors(const FilePath &oldPath, const FilePath &newPath);
 
-    void setUseFakeVim(const QVariant &value);
+    void setUseFakeVim(bool on);
     void setUseFakeVimInternal(bool on);
     void quitFakeVim();
     void fold(FakeVimHandler *handler, int depth, bool fold);
     void maybeReadVimRc();
-    void setShowRelativeLineNumbers(const QVariant &value);
-    void updateCursorBlinking(const QVariant &value);
+    void setShowRelativeLineNumbers(bool on);
+    void setCursorBlinking(bool on);
 
     void resetCommandBuffer();
     void showCommandBuffer(FakeVimHandler *handler, const QString &contents,
@@ -975,7 +976,7 @@ void FakeVimUserCommandsPage::apply()
 class FakeVimCompletionAssistProvider : public CompletionAssistProvider
 {
 public:
-    IAssistProcessor *createProcessor() const override;
+    IAssistProcessor *createProcessor(const AssistInterface *) const override;
 
     void setActive(const QString &needle, bool forward, FakeVimHandler *handler)
     {
@@ -1112,7 +1113,7 @@ private:
     const FakeVimCompletionAssistProvider *m_provider;
 };
 
-IAssistProcessor *FakeVimCompletionAssistProvider::createProcessor() const
+IAssistProcessor *FakeVimCompletionAssistProvider::createProcessor(const AssistInterface *) const
 {
     return new FakeVimCompletionAssistProcessor(this);
 }
@@ -1162,7 +1163,7 @@ bool FakeVimUserCommandsModel::setData(const QModelIndex &index,
 
 FakeVimPluginPrivate::FakeVimPluginPrivate()
 {
-    m_defaultExCommandMap[CppTools::Constants::SWITCH_HEADER_SOURCE] =
+    m_defaultExCommandMap[CppEditor::Constants::SWITCH_HEADER_SOURCE] =
         QRegularExpression("^A$");
     m_defaultExCommandMap["Coreplugin.OutputPane.previtem"] =
         QRegularExpression("^(cN(ext)?|cp(revious)?)!?( (.*))?$");
@@ -1193,8 +1194,13 @@ bool FakeVimPluginPrivate::initialize()
 */
     readSettings();
 
+    // Vimrc can break test so don't source it if running tests.
+    if (!ExtensionSystem::PluginManager::testRunRequested())
+        maybeReadVimRc();
+
     Command *cmd = nullptr;
-    cmd = ActionManager::registerAction(theFakeVimSetting(ConfigUseFakeVim)->action(),
+
+    cmd = ActionManager::registerAction(fakeVimSettings()->useFakeVim.action(),
         INSTALL_HANDLER, Context(Core::Constants::C_GLOBAL), true);
     cmd->setDefaultKeySequence(QKeySequence(useMacShortcuts ? Tr::tr("Meta+Shift+Y,Meta+Shift+Y")
                                                             : Tr::tr("Alt+Y,Alt+Y")));
@@ -1232,16 +1238,17 @@ bool FakeVimPluginPrivate::initialize()
     connect(DocumentManager::instance(), &DocumentManager::documentRenamed,
             this, &FakeVimPluginPrivate::documentRenamed);
 
-    connect(theFakeVimSetting(ConfigUseFakeVim), &SavedAction::valueChanged,
+    FakeVimSettings &s = *fakeVimSettings();
+    connect(&s.useFakeVim, &FvBoolAspect::valueChanged,
             this, &FakeVimPluginPrivate::setUseFakeVim);
-    connect(theFakeVimSetting(ConfigReadVimRc), &SavedAction::valueChanged,
+    connect(&s.readVimRc, &FvBaseAspect::changed,
             this, &FakeVimPluginPrivate::maybeReadVimRc);
-    connect(theFakeVimSetting(ConfigVimRcPath), &SavedAction::valueChanged,
+    connect(&s.vimRcPath, &FvBaseAspect::changed,
             this, &FakeVimPluginPrivate::maybeReadVimRc);
-    connect(theFakeVimSetting(ConfigRelativeNumber), &SavedAction::valueChanged,
+    connect(&s.relativeNumber, &FvBoolAspect::valueChanged,
             this, &FakeVimPluginPrivate::setShowRelativeLineNumbers);
-    connect(theFakeVimSetting(ConfigBlinkingCursor), &SavedAction::valueChanged,
-            this, &FakeVimPluginPrivate::updateCursorBlinking);
+    connect(&s.blinkingCursor, &FvBoolAspect::valueChanged,
+            this, &FakeVimPluginPrivate::setCursorBlinking);
 
     // Delayed operations.
     connect(this, &FakeVimPluginPrivate::delayedQuitRequested,
@@ -1249,12 +1256,7 @@ bool FakeVimPluginPrivate::initialize()
     connect(this, &FakeVimPluginPrivate::delayedQuitAllRequested,
             this, &FakeVimPluginPrivate::handleDelayedQuitAll, Qt::QueuedConnection);
 
-    // Vimrc can break test so don't source it if running tests.
-    if (!ExtensionSystem::PluginManager::testRunRequested())
-        maybeReadVimRc();
-    //    << "MODE: " << theFakeVimSetting(ConfigUseFakeVim)->value();
-
-    updateCursorBlinking(theFakeVimSetting(ConfigBlinkingCursor)->value());
+    setCursorBlinking(s.blinkingCursor.value());
 
     return true;
 }
@@ -1265,7 +1267,7 @@ void FakeVimPluginPrivate::userActionTriggered(int key)
     FakeVimHandler *handler = m_editorToHandler[editor];
     if (handler) {
         // If disabled, enable FakeVim mode just for single user command.
-        bool enableFakeVim = !theFakeVimSetting(ConfigUseFakeVim)->value().toBool();
+        bool enableFakeVim = !fakeVimSettings()->useFakeVim.value();
         if (enableFakeVim)
             setUseFakeVimInternal(true);
 
@@ -1281,9 +1283,9 @@ void FakeVimPluginPrivate::createRelativeNumberWidget(IEditor *editor)
 {
     if (auto textEditor = TextEditorWidget::fromEditor(editor)) {
         auto relativeNumbers = new RelativeNumbersColumn(textEditor);
-        connect(theFakeVimSetting(ConfigRelativeNumber), &SavedAction::valueChanged,
+        connect(&fakeVimSettings()->relativeNumber, &FvBaseAspect::changed,
                 relativeNumbers, &QObject::deleteLater);
-        connect(theFakeVimSetting(ConfigUseFakeVim), &SavedAction::valueChanged,
+        connect(&fakeVimSettings()->useFakeVim, &FvBaseAspect::changed,
                 relativeNumbers, &QObject::deleteLater);
         relativeNumbers->show();
     }
@@ -1292,14 +1294,14 @@ void FakeVimPluginPrivate::createRelativeNumberWidget(IEditor *editor)
 void FakeVimPluginPrivate::writeSettings()
 {
     QSettings *settings = ICore::settings();
-    theFakeVimSettings()->writeSettings(settings);
+    fakeVimSettings()->writeSettings(settings);
 }
 
 void FakeVimPluginPrivate::readSettings()
 {
     QSettings *settings = ICore::settings();
 
-    theFakeVimSettings()->readSettings(settings);
+    fakeVimSettings()->readSettings(settings);
 
     m_exCommandMap = m_defaultExCommandMap;
     int size = settings->beginReadArray(exCommandMapGroup);
@@ -1327,9 +1329,9 @@ void FakeVimPluginPrivate::maybeReadVimRc()
     //qDebug() << theFakeVimSetting(ConfigReadVimRc)
     //    << theFakeVimSetting(ConfigReadVimRc)->value();
     //qDebug() << theFakeVimSetting(ConfigShiftWidth)->value();
-    if (!theFakeVimSetting(ConfigReadVimRc)->value().toBool())
+    if (!fakeVimSettings()->readVimRc.value())
         return;
-    QString fileName = theFakeVimSetting(ConfigVimRcPath)->value().toString();
+    QString fileName = fakeVimSettings()->vimRcPath.value();
     if (fileName.isEmpty()) {
         fileName = QStandardPaths::writableLocation(QStandardPaths::HomeLocation)
             + QLatin1String(HostOsInfo::isWindowsHost() ? "/_vimrc" : "/.vimrc");
@@ -1539,6 +1541,14 @@ void FakeVimPluginPrivate::editorOpened(IEditor *editor)
     if (!editor)
         return;
 
+    if (m_editorToHandler.contains(editor)) {
+        // We get here via the call from the duplicated handler in case
+        // it was triggered by triggerAction(Core::Constants::SPLIT).
+        // On the other hand, we need the path from there to support the
+        // case of manual calls to IEditor::duplicate().
+        return;
+    }
+
     QWidget *widget = editor->widget();
     if (!widget)
         return;
@@ -1551,6 +1561,14 @@ void FakeVimPluginPrivate::editorOpened(IEditor *editor)
     else
         return;
 
+    // Duplicated editors are not signalled by the EditorManager. Track them nevertheless.
+    connect(editor, &IEditor::editorDuplicated, this, [this](IEditor *duplicate) {
+        editorOpened(duplicate);
+        connect(duplicate, &QObject::destroyed, this, [this, duplicate] {
+            m_editorToHandler.remove(duplicate);
+        });
+    });
+
     auto tew = TextEditorWidget::fromEditor(editor);
 
     //qDebug() << "OPENING: " << editor << editor->widget()
@@ -1628,9 +1646,9 @@ void FakeVimPluginPrivate::editorOpened(IEditor *editor)
             return;
 
         TabSettings tabSettings;
-        tabSettings.m_indentSize = theFakeVimSetting(ConfigShiftWidth)->value().toInt();
-        tabSettings.m_tabSize = theFakeVimSetting(ConfigTabStop)->value().toInt();
-        tabSettings.m_tabPolicy = theFakeVimSetting(ConfigExpandTab)->value().toBool()
+        tabSettings.m_indentSize = fakeVimSettings()->shiftWidth.value();
+        tabSettings.m_tabSize = fakeVimSettings()->tabStop.value();
+        tabSettings.m_tabPolicy = fakeVimSettings()->expandTab.value()
                 ? TabSettings::SpacesOnlyTabPolicy : TabSettings::TabsOnlyTabPolicy;
         tabSettings.m_continuationAlignBehavior =
                 tew->textDocument()->tabSettings().m_continuationAlignBehavior;
@@ -1663,22 +1681,44 @@ void FakeVimPluginPrivate::editorOpened(IEditor *editor)
 
     handler->requestDisableBlockSelection.connect([tew] {
         if (tew)
-            tew->setBlockSelection(false);
+            tew->setTextCursor(tew->textCursor());
     });
 
     handler->requestSetBlockSelection.connect([tew](const QTextCursor &cursor) {
-        if (tew)
-            tew->setBlockSelection(cursor);
+        if (tew) {
+            const TabSettings &tabs = tew->textDocument()->tabSettings();
+            MultiTextCursor mtc;
+            const bool forwardSelection = cursor.anchor() < cursor.position();
+            QTextBlock block = cursor.document()->findBlock(cursor.anchor());
+            const QTextBlock end = forwardSelection ? cursor.block().next() : cursor.block().previous();
+            const int anchor = tabs.columnAt(block.text(), cursor.anchor() - block.position());
+            const int pos = tabs.columnAt(cursor.block().text(), cursor.positionInBlock());
+            while (block.isValid() && block != end) {
+                const int columns = tabs.columnCountForText(block.text());
+                if (columns >= anchor || columns >= pos) {
+                    QTextCursor c(block);
+                    c.setPosition(block.position() + tabs.positionAtColumn(block.text(), anchor));
+                    c.setPosition(block.position() + tabs.positionAtColumn(block.text(), pos),
+                                  QTextCursor::KeepAnchor);
+                    mtc.addCursor(c);
+                }
+                block = forwardSelection ? block.next() : block.previous();
+            }
+            tew->setMultiTextCursor(mtc);
+        }
     });
 
     handler->requestBlockSelection.connect([tew](QTextCursor *cursor) {
-        if (tew && cursor)
-            *cursor = tew->blockSelection();
+        if (tew && cursor) {
+            MultiTextCursor mtc = tew->multiTextCursor();
+            *cursor = mtc.cursors().first();
+            cursor->setPosition(mtc.mainCursor().position(), QTextCursor::KeepAnchor);
+        }
     });
 
     handler->requestHasBlockSelection.connect([tew](bool *on) {
         if (tew && on)
-            *on = tew->hasBlockSelection();
+            *on = tew->multiTextCursor().hasMultipleCursors();
     });
 
     handler->simpleCompletionRequested.connect([this, handler](const QString &needle, bool forward) {
@@ -1833,11 +1873,11 @@ void FakeVimPluginPrivate::editorOpened(IEditor *editor)
     handler->installEventFilter();
 
     // pop up the bar
-    if (theFakeVimSetting(ConfigUseFakeVim)->value().toBool()) {
+    if (fakeVimSettings()->useFakeVim.value()) {
        resetCommandBuffer();
        handler->setupWidget();
 
-       if (theFakeVimSetting(ConfigRelativeNumber)->value().toBool())
+       if (fakeVimSettings()->relativeNumber.value())
            createRelativeNumberWidget(editor);
     }
 }
@@ -1854,34 +1894,33 @@ void FakeVimPluginPrivate::currentEditorAboutToChange(IEditor *editor)
         handler->enterCommandMode();
 }
 
-void FakeVimPluginPrivate::allDocumentsRenamed(const QString &oldName, const QString &newName)
+void FakeVimPluginPrivate::allDocumentsRenamed(const FilePath &oldPath, const FilePath &newPath)
 {
-    renameFileNameInEditors(oldName, newName);
-    FakeVimHandler::updateGlobalMarksFilenames(oldName, newName);
+    renameFileNameInEditors(oldPath, newPath);
+    FakeVimHandler::updateGlobalMarksFilenames(oldPath.toString(), newPath.toString());
 }
 
 void FakeVimPluginPrivate::documentRenamed(
-        IDocument *, const QString &oldName, const QString &newName)
+        IDocument *, const FilePath &oldPath, const FilePath &newPath)
 {
-    renameFileNameInEditors(oldName, newName);
+    renameFileNameInEditors(oldPath, newPath);
 }
 
-void FakeVimPluginPrivate::renameFileNameInEditors(const QString &oldName, const QString &newName)
+void FakeVimPluginPrivate::renameFileNameInEditors(const FilePath &oldPath, const FilePath &newPath)
 {
     foreach (FakeVimHandler *handler, m_editorToHandler.values()) {
-        if (handler->currentFileName() == oldName)
-            handler->setCurrentFileName(newName);
+        if (handler->currentFileName() == oldPath.toString())
+            handler->setCurrentFileName(newPath.toString());
     }
 }
 
-void FakeVimPluginPrivate::setUseFakeVim(const QVariant &value)
+void FakeVimPluginPrivate::setUseFakeVim(bool on)
 {
-    //qDebug() << "SET USE FAKEVIM" << value;
-    bool on = value.toBool();
+    //qDebug() << "SET USE FAKEVIM" << on;
     Find::setUseFakeVim(on);
     setUseFakeVimInternal(on);
-    setShowRelativeLineNumbers(theFakeVimSetting(ConfigRelativeNumber)->value());
-    updateCursorBlinking(theFakeVimSetting(ConfigBlinkingCursor)->value());
+    setShowRelativeLineNumbers(fakeVimSettings()->relativeNumber.value());
+    setCursorBlinking(fakeVimSettings()->blinkingCursor.value());
 }
 
 void FakeVimPluginPrivate::setUseFakeVimInternal(bool on)
@@ -1904,20 +1943,20 @@ void FakeVimPluginPrivate::setUseFakeVimInternal(bool on)
     }
 }
 
-void FakeVimPluginPrivate::setShowRelativeLineNumbers(const QVariant &value)
+void FakeVimPluginPrivate::setShowRelativeLineNumbers(bool on)
 {
-    if (value.toBool() && theFakeVimSetting(ConfigUseFakeVim)->value().toBool()) {
+    if (on && fakeVimSettings()->useFakeVim.value()) {
         foreach (IEditor *editor, m_editorToHandler.keys())
             createRelativeNumberWidget(editor);
     }
 }
 
-void FakeVimPluginPrivate::updateCursorBlinking(const QVariant &value)
+void FakeVimPluginPrivate::setCursorBlinking(bool on)
 {
     if (m_savedCursorFlashTime == 0)
         m_savedCursorFlashTime = QGuiApplication::styleHints()->cursorFlashTime();
 
-    bool blink = value.toBool() || !theFakeVimSetting(ConfigUseFakeVim)->value().toBool();
+    const bool blink = on || !fakeVimSettings()->useFakeVim.value();
     QGuiApplication::styleHints()->setCursorFlashTime(blink ? m_savedCursorFlashTime : 0);
 }
 
@@ -2011,10 +2050,10 @@ void FakeVimPluginPrivate::handleExCommand(FakeVimHandler *handler, bool *handle
         keepOnlyWindow();
     } else if (cmd.cmd == "AS") {
         triggerAction(Core::Constants::SPLIT);
-        triggerAction(CppTools::Constants::SWITCH_HEADER_SOURCE);
+        triggerAction(CppEditor::Constants::SWITCH_HEADER_SOURCE);
     } else if (cmd.cmd == "AV") {
         triggerAction(Core::Constants::SPLIT_SIDE_BY_SIDE);
-        triggerAction(CppTools::Constants::SWITCH_HEADER_SOURCE);
+        triggerAction(CppEditor::Constants::SWITCH_HEADER_SOURCE);
     } else {
         // Check whether one of the configure commands matches.
         const auto end = m_exCommandMap.constEnd();
@@ -2048,7 +2087,7 @@ void FakeVimPluginPrivate::handleDelayedQuitAll(bool forced)
 
 void FakeVimPluginPrivate::quitFakeVim()
 {
-    theFakeVimSetting(ConfigUseFakeVim)->setValue(false);
+    fakeVimSettings()->useFakeVim.setValue(false);
 }
 
 void FakeVimPluginPrivate::resetCommandBuffer()
