@@ -34,11 +34,12 @@
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
 
-#include <cpptools/cppprojectupdaterinterface.h>
+#include <cppeditor/cppprojectupdaterinterface.h>
 
 #include <extensionsystem/pluginmanager.h>
 
 #include <projectexplorer/abi.h>
+#include <projectexplorer/buildinfo.h>
 #include <projectexplorer/buildsteplist.h>
 #include <projectexplorer/buildsystem.h>
 #include <projectexplorer/customexecutablerunconfiguration.h>
@@ -134,9 +135,10 @@ public:
                 || action == Rename;
     }
 
-    RemovedFilesFromProject removeFiles(Node *, const QStringList &filePaths, QStringList *) final;
-    bool renameFile(Node *, const QString &filePath, const QString &newFilePath) final;
-    bool addFiles(Node *, const QStringList &filePaths, QStringList *) final;
+    RemovedFilesFromProject removeFiles(Node *, const FilePaths &filePaths, FilePaths *) final;
+    bool renameFile(Node *, const FilePath &oldFilePath, const FilePath &newFilePath) final;
+    bool addFiles(Node *, const FilePaths &filePaths, FilePaths *) final;
+    QString name() const final { return QLatin1String("generic"); }
 
     FilePath filesFilePath() const { return ::FilePath::fromString(m_filesFileName); }
 
@@ -154,7 +156,7 @@ public:
     void updateDeploymentData();
 
     bool setFiles(const QStringList &filePaths);
-    void removeFiles(const QStringList &filesToRemove);
+    void removeFiles(const FilePaths &filesToRemove);
 
 private:
     QString m_filesFileName;
@@ -170,7 +172,7 @@ private:
     QStringList m_cxxflags;
     QStringList m_cflags;
 
-    CppTools::CppProjectUpdaterInterface *m_cppCodeModelUpdater = nullptr;
+    CppEditor::CppProjectUpdaterInterface *m_cppCodeModelUpdater = nullptr;
 
     Utils::FileSystemWatcher m_deployFileWatcher;
 };
@@ -184,7 +186,8 @@ private:
 
 static bool writeFile(const QString &filePath, const QString &contents)
 {
-    Utils::FileSaver saver(filePath, QIODevice::Text | QIODevice::WriteOnly);
+    Utils::FileSaver saver(Utils::FilePath::fromString(filePath),
+                           QIODevice::Text | QIODevice::WriteOnly);
     return saver.write(contents.toUtf8()) && saver.finalize();
 }
 
@@ -193,7 +196,7 @@ GenericProject::GenericProject(const Utils::FilePath &fileName)
 {
     setId(Constants::GENERICPROJECT_ID);
     setProjectLanguages(Context(ProjectExplorer::Constants::CXX_LANGUAGE_ID));
-    setDisplayName(fileName.toFileInfo().completeBaseName());
+    setDisplayName(fileName.completeBaseName());
     setBuildSystemCreator([](Target *t) { return new GenericBuildSystem(t); });
 }
 
@@ -206,7 +209,7 @@ GenericBuildSystem::GenericBuildSystem(Target *target)
         const bool successFullyCreatedProjectUpdater
             = QMetaObject::invokeMethod(projectUpdaterFactory,
                                         "create",
-                                        Q_RETURN_ARG(CppTools::CppProjectUpdaterInterface *,
+                                        Q_RETURN_ARG(CppEditor::CppProjectUpdaterInterface *,
                                                      m_cppCodeModelUpdater));
         QTC_CHECK(successFullyCreatedProjectUpdater);
     }
@@ -251,7 +254,14 @@ GenericBuildSystem::GenericBuildSystem(Target *target)
     connect(&m_deployFileWatcher, &FileSystemWatcher::fileChanged,
             this, &GenericBuildSystem::updateDeploymentData);
 
-    connect(target, &Target::activeBuildConfigurationChanged, this, [this] { refresh(Everything); });
+    connect(target, &Target::activeBuildConfigurationChanged, this, [this, target] {
+        if (target == project()->activeTarget())
+            refresh(Everything);
+    });
+    connect(project(), &Project::activeTargetChanged, this, [this, target] {
+        if (target == project()->activeTarget())
+            refresh(Everything);
+    });
 }
 
 GenericBuildSystem::~GenericBuildSystem()
@@ -293,9 +303,10 @@ bool GenericBuildSystem::saveRawFileList(const QStringList &rawFileList)
 
 bool GenericBuildSystem::saveRawList(const QStringList &rawList, const QString &fileName)
 {
-    FileChangeBlocker changeGuard(fileName);
+    const FilePath filePath = FilePath::fromString(fileName);
+    FileChangeBlocker changeGuard(filePath);
     // Make sure we can open the file for writing
-    Utils::FileSaver saver(fileName, QIODevice::Text);
+    Utils::FileSaver saver(filePath, QIODevice::Text);
     if (!saver.hasError()) {
         QTextStream stream(saver.file());
         for (const QString &filePath : rawList)
@@ -315,8 +326,9 @@ static void insertSorted(QStringList *list, const QString &value)
         list->insert(it, value);
 }
 
-bool GenericBuildSystem::addFiles(Node *, const QStringList &filePaths, QStringList *)
+bool GenericBuildSystem::addFiles(Node *, const FilePaths &filePaths_, FilePaths *)
 {
+    const QStringList filePaths = Utils::transform(filePaths_, &FilePath::toString);
     const QDir baseDir(projectDirectory().toString());
     QStringList newList = m_rawFileList;
     if (filePaths.size() > m_rawFileList.size()) {
@@ -356,12 +368,12 @@ bool GenericBuildSystem::addFiles(Node *, const QStringList &filePaths, QStringL
     return result;
 }
 
-RemovedFilesFromProject GenericBuildSystem::removeFiles(Node *, const QStringList &filePaths, QStringList *)
+RemovedFilesFromProject GenericBuildSystem::removeFiles(Node *, const FilePaths &filePaths, FilePaths *)
 {
     QStringList newList = m_rawFileList;
 
-    for (const QString &filePath : filePaths) {
-        QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath);
+    for (const FilePath &filePath : filePaths) {
+        QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath.toString());
         if (i != m_rawListEntries.end())
             newList.removeOne(i.value());
     }
@@ -381,17 +393,17 @@ bool GenericBuildSystem::setFiles(const QStringList &filePaths)
     return saveRawFileList(newList);
 }
 
-bool GenericBuildSystem::renameFile(Node *, const QString &filePath, const QString &newFilePath)
+bool GenericBuildSystem::renameFile(Node *, const FilePath &oldFilePath, const FilePath &newFilePath)
 {
     QStringList newList = m_rawFileList;
 
-    QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath);
+    QHash<QString, QString>::iterator i = m_rawListEntries.find(oldFilePath.toString());
     if (i != m_rawListEntries.end()) {
         int index = newList.indexOf(i.value());
         if (index != -1) {
             QDir baseDir(projectDirectory().toString());
             newList.removeAt(index);
-            insertSorted(&newList, baseDir.relativeFilePath(newFilePath));
+            insertSorted(&newList, baseDir.relativeFilePath(newFilePath.toString()));
         }
     }
 
@@ -405,7 +417,7 @@ static QStringList readFlags(const QString &filePath)
         return QStringList();
     QStringList flags;
     for (const auto &line : lines)
-        flags.append(QtcProcess::splitArgs(line));
+        flags.append(ProcessArgs::splitArgs(line));
     return flags;
 }
 
@@ -419,21 +431,17 @@ void GenericBuildSystem::parse(RefreshOptions options)
 
     if (options & Configuration) {
         m_rawProjectIncludePaths = readLines(m_includesFileName);
-        QStringList normalPaths;
-        QStringList frameworkPaths;
+        Utils::FilePaths normalPaths;
+        Utils::FilePaths frameworkPaths;
+        const auto baseDir = Utils::FilePath::fromString(m_includesFileName).parentDir();
         for (const QString &rawPath : qAsConst(m_rawProjectIncludePaths)) {
             if (rawPath.startsWith("-F"))
-                frameworkPaths << rawPath.mid(2);
+                frameworkPaths << baseDir.resolvePath(rawPath.mid(2));
             else
-                normalPaths << rawPath;
+                normalPaths << baseDir.resolvePath(rawPath);
         }
-        const auto stringsToHeaderPaths = [this](const QStringList &paths, HeaderPathType type) {
-            return transform<HeaderPaths>(processEntries(paths),
-                                          [type](const QString &p) { return HeaderPath(p, type);
-            });
-        };
-        m_projectIncludePaths = stringsToHeaderPaths(normalPaths, HeaderPathType::User);
-        m_projectIncludePaths << stringsToHeaderPaths(frameworkPaths, HeaderPathType::Framework);
+        m_projectIncludePaths = toUserHeaderPaths(normalPaths);
+        m_projectIncludePaths << toFrameworkHeaderPaths(frameworkPaths);
         m_cxxflags = readFlags(m_cxxflagsFileName);
         m_cflags = readFlags(m_cflagsFileName);
     }
@@ -466,7 +474,7 @@ void GenericBuildSystem::refresh(RefreshOptions options)
 
     if (options & Files) {
         auto newRoot = std::make_unique<ProjectNode>(projectDirectory());
-        newRoot->setDisplayName(projectFilePath().toFileInfo().completeBaseName());
+        newRoot->setDisplayName(projectFilePath().completeBaseName());
 
         // find the common base directory of all source files
         FilePath baseDir = findCommonSourceRoot();
@@ -553,6 +561,8 @@ void GenericBuildSystem::refreshCppCodeModel()
 {
     if (!m_cppCodeModelUpdater)
         return;
+    if (target() != project()->activeTarget())
+        return;
     QtSupport::CppKitInfo kitInfo(kit());
     QTC_ASSERT(kitInfo.isValid(), return);
 
@@ -562,8 +572,8 @@ void GenericBuildSystem::refreshCppCodeModel()
     rpp.setQtVersion(kitInfo.projectPartQtVersion);
     rpp.setHeaderPaths(m_projectIncludePaths);
     rpp.setConfigFileName(m_configFileName);
-    rpp.setFlagsForCxx({nullptr, m_cxxflags});
-    rpp.setFlagsForC({nullptr, m_cflags});
+    rpp.setFlagsForCxx({nullptr, m_cxxflags, projectDirectory().toString()});
+    rpp.setFlagsForC({nullptr, m_cflags, projectDirectory().toString()});
     rpp.setFiles(m_files);
 
     m_cppCodeModelUpdater->update({project(), kitInfo, activeParseEnvironment(), {rpp}});
@@ -588,14 +598,14 @@ void GenericBuildSystem::updateDeploymentData()
                                                   projectDirectory().toString());
         setDeploymentData(deploymentData);
         if (m_deployFileWatcher.files() != QStringList(deploymentFilePath.toString())) {
-            m_deployFileWatcher.removeFiles(m_deployFileWatcher.files());
+            m_deployFileWatcher.clear();
             m_deployFileWatcher.addFile(deploymentFilePath.toString(),
                                         FileSystemWatcher::WatchModifiedDate);
         }
     }
 }
 
-void GenericBuildSystem::removeFiles(const QStringList &filesToRemove)
+void GenericBuildSystem::removeFiles(const FilePaths &filesToRemove)
 {
     if (removeFiles(nullptr, filesToRemove, nullptr) == RemovedFilesFromProject::Error) {
         TaskHub::addTask(BuildSystemTask(Task::Error,
@@ -638,13 +648,30 @@ ProjectExplorer::DeploymentKnowledge GenericProject::deploymentKnowledge() const
     return DeploymentKnowledge::Approximative;
 }
 
+void GenericProject::configureAsExampleProject(ProjectExplorer::Kit *kit)
+{
+    QList<BuildInfo> infoList;
+    const QList<Kit *> kits(kit != nullptr ? QList<Kit *>({kit}) : KitManager::kits());
+    for (Kit *k : kits) {
+        if (auto factory = BuildConfigurationFactory::find(k, projectFilePath())) {
+            for (int i = 0; i < 5; ++i) {
+                BuildInfo buildInfo;
+                buildInfo.displayName = tr("Build %1").arg(i + 1);
+                buildInfo.factory = factory;
+                buildInfo.kitId = kit->id();
+                buildInfo.buildDirectory = projectFilePath();
+                infoList << buildInfo;
+            }
+        }
+    }
+    setup(infoList);
+}
+
 bool GenericProjectFile::reload(QString *errorString, IDocument::ReloadFlag flag, IDocument::ChangeType type)
 {
     Q_UNUSED(errorString)
     Q_UNUSED(flag)
-    if (type == TypePermissions)
-        return true;
-
+    Q_UNUSED(type)
     if (Target *t = m_project->activeTarget())
         static_cast<GenericBuildSystem *>(t->buildSystem())->refresh(m_options);
 
@@ -664,7 +691,7 @@ void GenericProject::editFilesTriggered()
     }
 }
 
-void GenericProject::removeFilesTriggered(const QStringList &filesToRemove)
+void GenericProject::removeFilesTriggered(const FilePaths &filesToRemove)
 {
     if (Target *t = activeTarget())
         static_cast<GenericBuildSystem *>(t->buildSystem())->removeFiles(filesToRemove);
