@@ -38,11 +38,12 @@ public:
     explicit GenericProject(const Utils::FilePath &filename);
 
     void editFilesTriggered();
-    void removeFilesTriggered(const QStringList &filesToRemove);
+    void removeFilesTriggered(const Utils::FilePaths &filesToRemove);
 
 private:
     RestoreResult fromMap(const QVariantMap &map, QString *errorMessage) final;
     ProjectExplorer::DeploymentKnowledge deploymentKnowledge() const final;
+    void configureAsExampleProject(ProjectExplorer::Kit *kit) override;
 };
 
 } // namespace Internal
