@@ -32,12 +32,14 @@
 
 #include <utils/filesystemwatcher.h>
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 
 #include <QDateTime>
 #include <QFont>
 
 #include <set>
 
+using namespace Utils;
 using namespace VcsBase;
 
 namespace Git {
@@ -49,11 +51,17 @@ enum RootNodes {
     Tags = 2
 };
 
+enum Columns {
+    ColumnBranch = 0,
+    ColumnDateTime = 1,
+    ColumnCount
+};
+
 // --------------------------------------------------------------------------
 // BranchNode:
 // --------------------------------------------------------------------------
 
-class BranchNode
+class BranchNode : public QObject
 {
 public:
     BranchNode() :
@@ -144,6 +152,11 @@ public:
         return fn;
     }
 
+    QString fullRef(bool includePrefix = false) const
+    {
+        return fullName(includePrefix).join('/');
+    }
+
     void insert(const QStringList &path, BranchNode *n)
     {
         BranchNode *current = this;
@@ -173,14 +186,14 @@ public:
 
     QStringList childrenNames() const
     {
-        if (children.count() > 0) {
+        if (!children.isEmpty()) {
             QStringList names;
             for (BranchNode *n : children) {
                 names.append(n->childrenNames());
             }
             return names;
         }
-        return {fullName().join('/')};
+        return {fullRef()};
     }
 
     int rowOf(BranchNode *node)
@@ -188,6 +201,11 @@ public:
         return children.indexOf(node);
     }
 
+    void setUpstreamStatus(UpstreamStatus newStatus)
+    {
+        status = newStatus;
+    }
+
     BranchNode *parent = nullptr;
     QList<BranchNode *> children;
 
@@ -195,13 +213,15 @@ public:
     QString sha;
     QString tracking;
     QDateTime dateTime;
+    UpstreamStatus status;
     mutable QString toolTip;
 };
 
 class BranchModel::Private
 {
 public:
-    explicit Private(GitClient *client) :
+    explicit Private(BranchModel *q, GitClient *client) :
+        q(q),
         client(client),
         rootNode(new BranchNode)
     {
@@ -219,8 +239,9 @@ public:
     void parseOutputLine(const QString &line, bool force = false);
     void flushOldEntries();
 
+    BranchModel *q;
     GitClient *client;
-    QString workingDirectory;
+    FilePath workingDirectory;
     BranchNode *rootNode;
     BranchNode *currentBranch = nullptr;
     BranchNode *headNode = nullptr;
@@ -248,7 +269,7 @@ public:
 
 BranchModel::BranchModel(GitClient *client, QObject *parent) :
     QAbstractItemModel(parent),
-    d(new Private(client))
+    d(new Private(this, client))
 {
     QTC_CHECK(d->client);
 
@@ -285,7 +306,7 @@ QModelIndex BranchModel::parent(const QModelIndex &index) const
     BranchNode *node = indexToNode(index);
     if (node->parent == d->rootNode)
         return QModelIndex();
-    return nodeToIndex(node->parent, 0);
+    return nodeToIndex(node->parent, ColumnBranch);
 }
 
 int BranchModel::rowCount(const QModelIndex &parentIdx) const
@@ -299,11 +320,14 @@ int BranchModel::rowCount(const QModelIndex &parentIdx) const
 int BranchModel::columnCount(const QModelIndex &parent) const
 {
     Q_UNUSED(parent)
-    return 2;
+    return ColumnCount;
 }
 
 QVariant BranchModel::data(const QModelIndex &index, int role) const
 {
+    const QChar arrowUp(0x2191);
+    const QChar arrowDown(0x2193);
+
     BranchNode *node = indexToNode(index);
     if (!node)
         return QVariant();
@@ -312,13 +336,16 @@ QVariant BranchModel::data(const QModelIndex &index, int role) const
     case Qt::DisplayRole: {
         QString res;
         switch (index.column()) {
-        case 0: {
+        case ColumnBranch: {
             res = node->name;
-            if (!node->tracking.isEmpty())
+            if (!node->tracking.isEmpty()) {
+                res += ' ' + arrowUp + QString::number(node->status.ahead);
+                res += ' ' + arrowDown + QString::number(node->status.behind);
                 res += " [" + node->tracking + ']';
+            }
             break;
         }
-        case 1:
+        case ColumnDateTime:
             if (node->isLeaf() && node->dateTime.isValid())
                 res = node->dateTime.toString("yyyy-MM-dd HH:mm");
             break;
@@ -326,7 +353,7 @@ QVariant BranchModel::data(const QModelIndex &index, int role) const
         return res;
     }
     case Qt::EditRole:
-        return index.column() == 0 ? node->fullName().join('/') : QVariant();
+        return index.column() == 0 ? node->fullRef() : QVariant();
     case Qt::ToolTipRole:
         if (!node->isLeaf())
             return QVariant();
@@ -351,7 +378,7 @@ QVariant BranchModel::data(const QModelIndex &index, int role) const
 
 bool BranchModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
-    if (index.column() != 0 || role != Qt::EditRole)
+    if (index.column() != ColumnBranch || role != Qt::EditRole)
         return false;
     BranchNode *node = indexToNode(index);
     if (!node)
@@ -361,7 +388,7 @@ bool BranchModel::setData(const QModelIndex &index, const QVariant &value, int r
     if (newName.isEmpty())
         return false;
 
-    const QString oldName = node->fullName().join('/');
+    const QString oldName = node->fullRef();
     if (oldName == newName)
         return false;
 
@@ -375,7 +402,7 @@ Qt::ItemFlags BranchModel::flags(const QModelIndex &index) const
     if (!node)
         return Qt::NoItemFlags;
     Qt::ItemFlags res = Qt::ItemIsSelectable | Qt::ItemIsEnabled;
-    if (node != d->headNode && node->isLeaf() && node->isLocal() && index.column() == 0)
+    if (node != d->headNode && node->isLeaf() && node->isLocal() && index.column() == ColumnBranch)
         res |= Qt::ItemIsEditable;
     return res;
 }
@@ -396,7 +423,7 @@ void BranchModel::clear()
     d->obsoleteLocalBranches.clear();
 }
 
-bool BranchModel::refresh(const QString &workingDirectory, QString *errorMessage)
+bool BranchModel::refresh(const FilePath &workingDirectory, QString *errorMessage)
 {
     beginResetModel();
     clear();
@@ -416,7 +443,7 @@ bool BranchModel::refresh(const QString &workingDirectory, QString *errorMessage
 
     if (d->workingDirectory != workingDirectory) {
         d->workingDirectory = workingDirectory;
-        d->fsWatcher.removeFiles(d->fsWatcher.files());
+        d->fsWatcher.clear();
         const QString gitDir = d->client->findGitDirForRepository(workingDirectory);
         if (!gitDir.isEmpty())
             d->fsWatcher.addFile(gitDir + "/HEAD", Utils::FileSystemWatcher::WatchModifiedDate);
@@ -484,21 +511,16 @@ void BranchModel::renameTag(const QString &oldName, const QString &newName)
     }
 }
 
-QString BranchModel::workingDirectory() const
+FilePath BranchModel::workingDirectory() const
 {
     return d->workingDirectory;
 }
 
-GitClient *BranchModel::client() const
-{
-    return d->client;
-}
-
 QModelIndex BranchModel::currentBranch() const
 {
     if (!d->currentBranch)
         return QModelIndex();
-    return nodeToIndex(d->currentBranch, 0);
+    return nodeToIndex(d->currentBranch, ColumnBranch);
 }
 
 QString BranchModel::fullName(const QModelIndex &idx, bool includePrefix) const
@@ -510,7 +532,7 @@ QString BranchModel::fullName(const QModelIndex &idx, bool includePrefix) const
         return QString();
     if (node == d->headNode)
         return QString("HEAD");
-    return node->fullName(includePrefix).join('/');
+    return node->fullRef(includePrefix);
 }
 
 QStringList BranchModel::localBranchNames() const
@@ -627,7 +649,7 @@ bool BranchModel::branchIsMerged(const QModelIndex &idx)
         VcsOutputWindow::appendError(errorMessage);
     }
 
-    const QStringList lines = output.split('\n', QString::SkipEmptyParts);
+    const QStringList lines = output.split('\n', Qt::SkipEmptyParts);
     for (const QString &l : lines) {
         QString currentBranch = l.mid(2); // remove first letters (those are either
                                           // "  " or "* " depending on whether it is
@@ -690,7 +712,7 @@ QModelIndex BranchModel::addBranch(const QString &name, bool track, const QModel
         BranchNode *child = (pos == local->count()) ? nullptr : local->children.at(pos);
         if (!child || child->name != nodeName) {
             child = new BranchNode(nodeName);
-            beginInsertRows(nodeToIndex(local, 0), pos, pos);
+            beginInsertRows(nodeToIndex(local, ColumnBranch), pos, pos);
             added = true;
             child->parent = local;
             local->children.insert(pos, child);
@@ -701,11 +723,11 @@ QModelIndex BranchModel::addBranch(const QString &name, bool track, const QModel
     auto newNode = new BranchNode(leafName, startSha, track ? trackedBranch : QString(),
                                   branchDateTime);
     if (!added)
-        beginInsertRows(nodeToIndex(local, 0), pos, pos);
+        beginInsertRows(nodeToIndex(local, ColumnBranch), pos, pos);
     newNode->parent = local;
     local->children.insert(pos, newNode);
     endInsertRows();
-    return nodeToIndex(newNode, 0);
+    return nodeToIndex(newNode, ColumnBranch);
 }
 
 void BranchModel::setRemoteTracking(const QModelIndex &trackingIndex)
@@ -717,6 +739,7 @@ void BranchModel::setRemoteTracking(const QModelIndex &trackingIndex)
     const QString tracking = fullName(trackingIndex, true);
     d->client->synchronousSetTrackingBranch(d->workingDirectory, currentName, tracking);
     d->currentBranch->tracking = shortTracking;
+    updateUpstreamStatus(d->currentBranch);
     emit dataChanged(current, current);
 }
 
@@ -738,6 +761,13 @@ Utils::optional<QString> BranchModel::remoteName(const QModelIndex &idx) const
     return Utils::nullopt;
 }
 
+void BranchModel::refreshCurrentBranch()
+{
+    const QModelIndex currentIndex = currentBranch();
+    BranchNode *node = indexToNode(currentIndex);
+    updateUpstreamStatus(node);
+}
+
 void BranchModel::Private::parseOutputLine(const QString &line, bool force)
 {
     if (line.size() < 3)
@@ -755,7 +785,7 @@ void BranchModel::Private::parseOutputLine(const QString &line, bool force)
     if (strDateTime.isEmpty())
         strDateTime = lineParts.at(4);
     if (!strDateTime.isEmpty()) {
-        const qint64 timeT = strDateTime.leftRef(strDateTime.indexOf(' ')).toLongLong();
+        const qint64 timeT = strDateTime.left(strDateTime.indexOf(' ')).toLongLong();
         dateTime = QDateTime::fromSecsSinceEpoch(timeT);
     }
 
@@ -764,7 +794,7 @@ void BranchModel::Private::parseOutputLine(const QString &line, bool force)
         const qint64 age = dateTime.daysTo(QDateTime::currentDateTime());
         isOld = age > Constants::OBSOLETE_COMMIT_AGE_IN_DAYS;
     }
-    bool showTags = client->settings().boolValue(GitSettings::showTagsKey);
+    const bool showTags = client->settings().showTags.value();
 
     // insert node into tree:
     QStringList nameParts = fullName.split('/');
@@ -826,6 +856,7 @@ void BranchModel::Private::parseOutputLine(const QString &line, bool force)
     root->insert(nameParts, newNode);
     if (current)
         currentBranch = newNode;
+    q->updateUpstreamStatus(newNode);
 }
 
 void BranchModel::Private::flushOldEntries()
@@ -862,7 +893,7 @@ void BranchModel::removeNode(const QModelIndex &idx)
     BranchNode *node = indexToNode(nodeIndex);
     while (node->count() == 0 && node->parent != d->rootNode) {
         BranchNode *parentNode = node->parent;
-        const QModelIndex parentIndex = nodeToIndex(parentNode, 0);
+        const QModelIndex parentIndex = nodeToIndex(parentNode, ColumnBranch);
         const int nodeRow = nodeIndex.row();
         beginRemoveRows(parentIndex, nodeRow, nodeRow);
         parentNode->children.removeAt(nodeRow);
@@ -873,6 +904,22 @@ void BranchModel::removeNode(const QModelIndex &idx)
     }
 }
 
+void BranchModel::updateUpstreamStatus(BranchNode *node)
+{
+    if (node->tracking.isEmpty())
+        return;
+    VcsCommand *command = d->client->asyncUpstreamStatus(
+                d->workingDirectory, node->fullRef(), node->tracking);
+    QObject::connect(command, &VcsCommand::stdOutText, node, [this, node](const QString &text) {
+        const QStringList split = text.trimmed().split('\t');
+        QTC_ASSERT(split.size() == 2, return);
+
+        node->setUpstreamStatus(UpstreamStatus(split.at(0).toInt(), split.at(1).toInt()));
+        const QModelIndex idx = nodeToIndex(node, ColumnBranch);
+        emit dataChanged(idx, idx);
+    });
+}
+
 QString BranchModel::toolTip(const QString &sha) const
 {
     // Show the sha description excluding diff as toolTip
