@@ -25,7 +25,6 @@
 
 #include "changeselectiondialog.h"
 #include "logchangedialog.h"
-#include "gitplugin.h"
 #include "gitclient.h"
 #include "ui_changeselectiondialog.h"
 
@@ -33,9 +32,9 @@
 
 #include <utils/pathchooser.h>
 #include <utils/theme/theme.h>
+
 #include <vcsbase/vcscommand.h>
 
-#include <QProcess>
 #include <QFormLayout>
 #include <QHBoxLayout>
 #include <QPushButton>
@@ -53,17 +52,16 @@ using namespace Utils;
 namespace Git {
 namespace Internal {
 
-ChangeSelectionDialog::ChangeSelectionDialog(const QString &workingDirectory, Core::Id id,
+ChangeSelectionDialog::ChangeSelectionDialog(const FilePath &workingDirectory, Id id,
                                              QWidget *parent) :
     QDialog(parent), m_ui(new Ui::ChangeSelectionDialog)
 {
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
-    m_gitExecutable = GitPlugin::client()->vcsBinary();
+    m_gitExecutable = GitClient::instance()->vcsBinary();
     m_ui->setupUi(this);
     m_ui->workingDirectoryChooser->setExpectedKind(PathChooser::ExistingDirectory);
     m_ui->workingDirectoryChooser->setPromptDialogTitle(tr("Select Git Directory"));
-    m_ui->workingDirectoryChooser->setPath(workingDirectory);
-    m_gitEnvironment = GitPlugin::client()->processEnvironment();
+    m_ui->workingDirectoryChooser->setFilePath(workingDirectory);
+    m_gitEnvironment = GitClient::instance()->processEnvironment();
     m_ui->changeNumberEdit->setFocus();
     m_ui->changeNumberEdit->selectAll();
 
@@ -76,15 +74,15 @@ ChangeSelectionDialog::ChangeSelectionDialog(const QString &workingDirectory, Co
     connect(m_ui->selectFromHistoryButton, &QPushButton::clicked,
             this, &ChangeSelectionDialog::selectCommitFromRecentHistory);
     connect(m_ui->showButton, &QPushButton::clicked,
-            this, std::bind(&ChangeSelectionDialog::accept, this, Show));
+            this, std::bind(&ChangeSelectionDialog::acceptCommand, this, Show));
     connect(m_ui->cherryPickButton, &QPushButton::clicked,
-            this, std::bind(&ChangeSelectionDialog::accept, this, CherryPick));
+            this, std::bind(&ChangeSelectionDialog::acceptCommand, this, CherryPick));
     connect(m_ui->revertButton, &QPushButton::clicked,
-            this, std::bind(&ChangeSelectionDialog::accept, this, Revert));
+            this, std::bind(&ChangeSelectionDialog::acceptCommand, this, Revert));
     connect(m_ui->checkoutButton, &QPushButton::clicked,
-            this, std::bind(&ChangeSelectionDialog::accept, this, Checkout));
+            this, std::bind(&ChangeSelectionDialog::acceptCommand, this, Checkout));
     connect(m_ui->archiveButton, &QPushButton::clicked,
-            this, std::bind(&ChangeSelectionDialog::accept, this, Archive));
+            this, std::bind(&ChangeSelectionDialog::acceptCommand, this, Archive));
 
     if (id == "Git.Revert")
         m_ui->revertButton->setDefault(true);
@@ -118,7 +116,7 @@ QString ChangeSelectionDialog::change() const
 
 void ChangeSelectionDialog::selectCommitFromRecentHistory()
 {
-    QString workingDir = workingDirectory();
+    FilePath workingDir = workingDirectory();
     if (workingDir.isEmpty())
         return;
 
@@ -137,11 +135,11 @@ void ChangeSelectionDialog::selectCommitFromRecentHistory()
     m_ui->changeNumberEdit->setText(dialog.commit());
 }
 
-QString ChangeSelectionDialog::workingDirectory() const
+FilePath ChangeSelectionDialog::workingDirectory() const
 {
-    const QString workingDir = m_ui->workingDirectoryChooser->path();
-    if (workingDir.isEmpty() || !QDir(workingDir).exists())
-        return QString();
+    const FilePath workingDir = m_ui->workingDirectoryChooser->filePath();
+    if (workingDir.isEmpty() || !workingDir.exists())
+        return {};
 
     return Core::VcsManager::findTopLevelForDirectory(workingDir);
 }
@@ -151,20 +149,20 @@ ChangeCommand ChangeSelectionDialog::command() const
     return m_command;
 }
 
-void ChangeSelectionDialog::accept(ChangeCommand command)
+void ChangeSelectionDialog::acceptCommand(ChangeCommand command)
 {
     m_command = command;
     QDialog::accept();
 }
 
 //! Set commit message in details
-void ChangeSelectionDialog::setDetails(int exitCode)
+void ChangeSelectionDialog::setDetails()
 {
     Theme *theme = creatorTheme();
 
     QPalette palette;
-    if (exitCode == 0) {
-        m_ui->detailsText->setPlainText(QString::fromUtf8(m_process->readAllStandardOutput()));
+    if (m_process->result() == QtcProcess::FinishedWithSuccess) {
+        m_ui->detailsText->setPlainText(m_process->stdOut());
         palette.setColor(QPalette::Text, theme->color(Theme::TextColorNormal));
         m_ui->changeNumberEdit->setPalette(palette);
     } else {
@@ -195,7 +193,7 @@ void ChangeSelectionDialog::terminateProcess()
 
 void ChangeSelectionDialog::recalculateCompletion()
 {
-    const QString workingDir = workingDirectory();
+    const FilePath workingDir = workingDirectory();
     if (workingDir == m_oldWorkingDir)
         return;
     m_oldWorkingDir = workingDir;
@@ -204,7 +202,7 @@ void ChangeSelectionDialog::recalculateCompletion()
     if (workingDir.isEmpty())
         return;
 
-    GitClient *client = GitPlugin::client();
+    GitClient *client = GitClient::instance();
     VcsBase::VcsCommand *command = client->asyncForEachRefCmd(
                 workingDir, {"--format=%(refname:short)"});
     connect(this, &QObject::destroyed, command, &VcsBase::VcsCommand::abort);
@@ -218,7 +216,7 @@ void ChangeSelectionDialog::recalculateDetails()
     terminateProcess();
     enableButtons(true);
 
-    const QString workingDir = workingDirectory();
+    const FilePath workingDir = workingDirectory();
     if (workingDir.isEmpty()) {
         m_ui->detailsText->setPlainText(tr("Error: Bad working directory."));
         return;
@@ -230,15 +228,14 @@ void ChangeSelectionDialog::recalculateDetails()
         return;
     }
 
-    m_process = new QProcess(this);
+    m_process = new QtcProcess(this);
     m_process->setWorkingDirectory(workingDir);
-    m_process->setProcessEnvironment(m_gitEnvironment);
+    m_process->setEnvironment(m_gitEnvironment);
+    m_process->setCommand({m_gitExecutable, {"show", "--decorate", "--stat=80", ref}});
 
-    connect(m_process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
-            this, &ChangeSelectionDialog::setDetails);
+    connect(m_process, &QtcProcess::finished, this, &ChangeSelectionDialog::setDetails);
 
-    m_process->start(m_gitExecutable.toString(), {"show", "--decorate", "--stat=80", ref});
-    m_process->closeWriteChannel();
+    m_process->start();
     if (!m_process->waitForStarted())
         m_ui->detailsText->setPlainText(tr("Error: Could not start Git."));
     else
