@@ -25,56 +25,52 @@
 
 #pragma once
 
-#include <utils/pathchooser.h>
+#include <utils/filepath.h>
 
 #include <QDialog>
 #include <QSharedPointer>
 #include <QTimer>
 
+#include <functional>
+
 QT_BEGIN_NAMESPACE
-class QTreeView;
-class QLabel;
 class QModelIndex;
 class QSortFilterProxyModel;
-class QStandardItem;
 class QStringListModel;
 class QPushButton;
-class QDialogButtonBox;
-class QTextBrowser;
 QT_END_NAMESPACE
 
-namespace Utils {
-class FancyLineEdit;
-class ProgressIndicator;
-class TreeView;
-}
+namespace Utils { class ProgressIndicator; }
 
 namespace Gerrit {
 namespace Internal {
-class GerritParameters;
-class GerritModel;
+namespace Ui { class GerritDialog; }
 class GerritChange;
-class QueryValidatingLineEdit;
+class GerritModel;
+class GerritParameters;
+class GerritServer;
 
 class GerritDialog : public QDialog
 {
     Q_OBJECT
 public:
     explicit GerritDialog(const QSharedPointer<GerritParameters> &p,
-                          QWidget *parent = 0);
-    ~GerritDialog();
-    QString repositoryPath() const;
-    void setCurrentPath(const QString &path);
+                          const QSharedPointer<GerritServer> &s,
+                          const Utils::FilePath &repository,
+                          QWidget *parent = nullptr);
+    ~GerritDialog() override;
+    Utils::FilePath repositoryPath() const;
+    void setCurrentPath(const Utils::FilePath &path);
+    void fetchStarted(const QSharedPointer<Gerrit::Internal::GerritChange> &change);
+    void fetchFinished();
+    void refresh();
+    void scheduleUpdateRemotes();
 
 signals:
     void fetchDisplay(const QSharedPointer<Gerrit::Internal::GerritChange> &);
     void fetchCherryPick(const QSharedPointer<Gerrit::Internal::GerritChange> &);
     void fetchCheckout(const QSharedPointer<Gerrit::Internal::GerritChange> &);
 
-public slots:
-    void fetchStarted(const QSharedPointer<Gerrit::Internal::GerritChange> &change);
-    void fetchFinished();
-
 private:
     void slotCurrentChanged();
     void slotActivated(const QModelIndex &);
@@ -82,7 +78,9 @@ private:
     void slotFetchDisplay();
     void slotFetchCherryPick();
     void slotFetchCheckout();
-    void slotRefresh();
+    void remoteChanged();
+    void updateRemotes(bool forceReload = false);
+    void showEvent(QShowEvent *event) override;
 
     void manageProgressIndicator();
 
@@ -93,23 +91,21 @@ private:
     void updateButtons();
 
     const QSharedPointer<GerritParameters> m_parameters;
+    const QSharedPointer<GerritServer> m_server;
     QSortFilterProxyModel *m_filterModel;
+    Ui::GerritDialog *m_ui;
     GerritModel *m_model;
     QStringListModel *m_queryModel;
-    Utils::TreeView *m_treeView;
-    QTextBrowser *m_detailsBrowser;
-    Utils::FancyLineEdit *m_queryLineEdit;
-    Utils::FancyLineEdit *m_filterLineEdit;
-    Utils::PathChooser *m_repositoryChooser;
-    QDialogButtonBox *m_buttonBox;
     QPushButton *m_displayButton;
     QPushButton *m_cherryPickButton;
     QPushButton *m_checkoutButton;
     QPushButton *m_refreshButton;
-    QLabel *m_repositoryChooserLabel;
     Utils::ProgressIndicator *m_progressIndicator;
     QTimer m_progressIndicatorTimer;
-    bool m_fetchRunning;
+    Utils::FilePath m_repository;
+    bool m_fetchRunning = false;
+    bool m_updatingRemotes = false;
+    bool m_shouldUpdateRemotes = false;
 };
 
 } // namespace Internal
