@@ -27,14 +27,16 @@
 #include "gerritparameters.h"
 #include "gerritserver.h"
 #include "../gitclient.h"
-#include "../gitplugin.h"
 
+#include <utils/filepath.h>
 #include <utils/qtcassert.h>
 #include <utils/utilsicons.h>
 
 #include <QFileInfo>
 #include <QHBoxLayout>
 
+using namespace Utils;
+
 namespace Gerrit {
 namespace Internal {
 
@@ -65,7 +67,7 @@ GerritRemoteChooser::GerritRemoteChooser(QWidget *parent) :
             this, [this] { updateRemotes(true); });
 }
 
-void GerritRemoteChooser::setRepository(const QString &repository)
+void GerritRemoteChooser::setRepository(const FilePath &repository)
 {
     m_repository = repository;
 }
@@ -104,7 +106,7 @@ bool GerritRemoteChooser::updateRemotes(bool forceReload)
     m_remotes.clear();
     QString errorMessage; // Mute errors. We'll just fallback to the defaults
     const QMap<QString, QString> remotesList =
-            Git::Internal::GitPlugin::client()->synchronousRemotesList(m_repository, &errorMessage);
+            Git::Internal::GitClient::instance()->synchronousRemotesList(m_repository, &errorMessage);
     for (auto mapIt = remotesList.cbegin(), end = remotesList.cend(); mapIt != end; ++mapIt) {
         GerritServer server;
         if (!server.fillFromRemote(mapIt.value(), *m_parameters, forceReload))
