@@ -31,11 +31,13 @@
 
 #include <coreplugin/icore.h>
 #include <coreplugin/shellcommand.h>
+
 #include <utils/hostosinfo.h>
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 
 #include <QFile>
 #include <QJsonDocument>
+#include <QJsonObject>
 #include <QMessageBox>
 #include <QRegularExpression>
 #include <QSettings>
@@ -159,7 +161,7 @@ bool GerritServer::fillFromRemote(const QString &remote,
         return true;
     }
     curlBinary = parameters.curl;
-    if (curlBinary.isEmpty() || !QFile::exists(curlBinary))
+    if (curlBinary.isEmpty() || !curlBinary.exists())
         return false;
     const StoredHostValidity validity = forceReload ? Invalid : loadSettings();
     switch (validity) {
@@ -240,13 +242,13 @@ QStringList GerritServer::curlArguments() const
 
 int GerritServer::testConnection()
 {
-    static GitClient *const client = GitPlugin::client();
+    static GitClient *const client = GitClient::instance();
     const QStringList arguments = curlArguments() << (url(RestUrl) + accountUrlC);
-    const SynchronousProcessResponse resp = client->vcsFullySynchronousExec(
-                QString(), {curlBinary, arguments},
-                Core::ShellCommand::NoOutput);
-    if (resp.result == SynchronousProcessResponse::Finished) {
-        QString output = resp.stdOut();
+    QtcProcess proc;
+    client->vcsFullySynchronousExec(proc, {}, {curlBinary, arguments},
+                                    Core::ShellCommand::NoOutput);
+    if (proc.result() == QtcProcess::FinishedWithSuccess) {
+        QString output = proc.stdOut();
         // Gerrit returns an empty response for /p/qt-creator/a/accounts/self
         // so consider this as 404.
         if (output.isEmpty())
@@ -262,10 +264,10 @@ int GerritServer::testConnection()
         }
         return Success;
     }
-    if (resp.exitCode == CertificateError)
+    if (proc.exitCode() == CertificateError)
         return CertificateError;
     const QRegularExpression errorRegexp("returned error: (\\d+)");
-    QRegularExpressionMatch match = errorRegexp.match(resp.stdErr());
+    QRegularExpressionMatch match = errorRegexp.match(proc.stdErr());
     if (match.hasMatch())
         return match.captured(1).toInt();
     return UnknownError;
@@ -332,33 +334,31 @@ bool GerritServer::resolveRoot()
 
 void GerritServer::resolveVersion(const GerritParameters &p, bool forceReload)
 {
-    static GitClient *const client = GitPlugin::client();
+    static GitClient *const client = GitClient::instance();
     QSettings *settings = Core::ICore::settings();
     const QString fullVersionKey = "Gerrit/" + host + '/' + versionKey;
     version = settings->value(fullVersionKey).toString();
     if (!version.isEmpty() && !forceReload)
         return;
     if (type == Ssh) {
-        SynchronousProcess process;
+        QtcProcess proc;
         QStringList arguments;
         if (port)
             arguments << p.portFlag << QString::number(port);
         arguments << hostArgument() << "gerrit" << "version";
-        const SynchronousProcessResponse resp = client->vcsFullySynchronousExec(
-                    QString(), {p.ssh, arguments},
-                    Core::ShellCommand::NoOutput);
-        QString stdOut = resp.stdOut().trimmed();
+        client->vcsFullySynchronousExec(proc, {}, {p.ssh, arguments}, Core::ShellCommand::NoOutput);
+        QString stdOut = proc.stdOut().trimmed();
         stdOut.remove("gerrit version ");
         version = stdOut;
     } else {
         const QStringList arguments = curlArguments() << (url(RestUrl) + versionUrlC);
-        const SynchronousProcessResponse resp = client->vcsFullySynchronousExec(
-                    QString(), {curlBinary, arguments},
-                    Core::ShellCommand::NoOutput);
+        QtcProcess proc;
+        client->vcsFullySynchronousExec(proc, {}, {curlBinary, arguments},
+                                        Core::ShellCommand::NoOutput);
         // REST endpoint for version is only available from 2.8 and up. Do not consider invalid
         // if it fails.
-        if (resp.result == SynchronousProcessResponse::Finished) {
-            QString output = resp.stdOut();
+        if (proc.result() == QtcProcess::FinishedWithSuccess) {
+            QString output = proc.stdOut();
             if (output.isEmpty())
                 return;
             output.remove(0, output.indexOf('\n')); // Strip first line
