@@ -39,8 +39,8 @@
 #include <utils/filesearch.h>
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
 #include <utils/runextensions.h>
-#include <utils/synchronousprocess.h>
 #include <utils/textfileformat.h>
 
 #include <QCheckBox>
@@ -71,17 +71,17 @@ namespace {
 
 const char GitGrepRef[] = "GitGrepRef";
 
-class GitGrepRunner : public QObject
+class GitGrepRunner
 {
     using FutureInterfaceType = QFutureInterface<FileSearchResultList>;
 
 public:
-    GitGrepRunner(FutureInterfaceType &fi,
-                  const TextEditor::FileFindParameters &parameters) :
-        m_fi(fi),
-        m_parameters(parameters)
+    GitGrepRunner(const TextEditor::FileFindParameters &parameters)
+        : m_parameters(parameters)
     {
-        m_directory = parameters.additionalParameters.toString();
+        m_directory = FilePath::fromString(parameters.additionalParameters.toString());
+        m_command.reset(GitClient::instance()->createCommand(m_directory));
+        m_vcsBinary = GitClient::instance()->vcsBinary();
     }
 
     struct Match
@@ -106,7 +106,7 @@ public:
         QString filePath = line.left(lineSeparator);
         if (!m_ref.isEmpty() && filePath.startsWith(m_ref))
             filePath.remove(0, m_ref.length());
-        single.fileName = m_directory + '/' + filePath;
+        single.fileName = m_directory.pathAppended(filePath).toString();
         const int textSeparator = line.indexOf(QChar::Null, lineSeparator + 1);
         single.lineNumber = line.mid(lineSeparator + 1, textSeparator - lineSeparator - 1).toInt();
         QString text = line.mid(textSeparator + 1);
@@ -144,19 +144,20 @@ public:
         }
     }
 
-    void read(const QString &text)
+    void read(FutureInterfaceType &fi, const QString &text)
     {
         FileSearchResultList resultList;
         QString t = text;
         QTextStream stream(&t);
-        while (!stream.atEnd() && !m_fi.isCanceled())
+        while (!stream.atEnd() && !fi.isCanceled())
             processLine(stream.readLine(), &resultList);
         if (!resultList.isEmpty())
-            m_fi.reportResult(resultList);
+            fi.reportResult(resultList);
     }
 
-    void exec()
+    void operator()(FutureInterfaceType &fi)
     {
+        Core::ProgressTimer progress(fi, 5);
         QStringList arguments = {
             "-c", "color.grep.match=bold red",
             "-c", "color.grep=always",
@@ -186,47 +187,45 @@ public:
                     return QString(":!" + filter);
                 });
         arguments << "--" << filterArgs << exclusionArgs;
-        QScopedPointer<VcsCommand> command(GitClient::instance()->createCommand(m_directory));
-        command->addFlags(VcsCommand::SilentOutput | VcsCommand::SuppressFailMessage);
-        command->setProgressiveOutput(true);
+        m_command->addFlags(VcsCommand::SilentOutput | VcsCommand::SuppressFailMessage);
+        m_command->setProgressiveOutput(true);
         QFutureWatcher<FileSearchResultList> watcher;
-        watcher.setFuture(m_fi.future());
-        connect(&watcher, &QFutureWatcher<FileSearchResultList>::canceled,
-                command.data(), &VcsCommand::cancel);
-        connect(command.data(), &VcsCommand::stdOutText, this, &GitGrepRunner::read);
-        SynchronousProcessResponse resp = command->runCommand({GitClient::instance()->vcsBinary(), arguments}, 0);
-        switch (resp.result) {
-        case SynchronousProcessResponse::TerminatedAbnormally:
-        case SynchronousProcessResponse::StartFailed:
-        case SynchronousProcessResponse::Hang:
-            m_fi.reportCanceled();
+        QObject::connect(&watcher,
+                         &QFutureWatcher<FileSearchResultList>::canceled,
+                         m_command.get(),
+                         &VcsCommand::cancel);
+        watcher.setFuture(fi.future());
+        QObject::connect(m_command.get(),
+                         &VcsCommand::stdOutText,
+                         [this, &fi](const QString &text) { read(fi, text); });
+        QtcProcess proc;
+        proc.setTimeoutS(0);
+        m_command->runCommand(proc, {m_vcsBinary, arguments});
+        switch (proc.result()) {
+        case QtcProcess::TerminatedAbnormally:
+        case QtcProcess::StartFailed:
+        case QtcProcess::Hang:
+            fi.reportCanceled();
             break;
-        case SynchronousProcessResponse::Finished:
-        case SynchronousProcessResponse::FinishedError:
+        case QtcProcess::FinishedWithSuccess:
+        case QtcProcess::FinishedWithError:
             // When no results are found, git-grep exits with non-zero status.
             // Do not consider this as an error.
             break;
         }
     }
 
-    static void run(QFutureInterface<FileSearchResultList> &fi,
-                    TextEditor::FileFindParameters parameters)
-    {
-        GitGrepRunner runner(fi, parameters);
-        Core::ProgressTimer progress(fi, 5);
-        runner.exec();
-    }
-
 private:
-    FutureInterfaceType m_fi;
-    QString m_directory;
+    FilePath m_vcsBinary;
+    FilePath m_directory;
     QString m_ref;
-    const TextEditor::FileFindParameters &m_parameters;
+    TextEditor::FileFindParameters m_parameters;
+    std::unique_ptr<VcsCommand> m_command;
 };
 
 } // namespace
 
-static bool isGitDirectory(const QString &path)
+static bool isGitDirectory(const FilePath &path)
 {
     static IVersionControl *gitVc = VcsManager::versionControl(VcsBase::Constants::VCS_ID_GIT);
     QTC_ASSERT(gitVc, return false);
@@ -253,7 +252,7 @@ GitGrep::GitGrep(GitClient *client)
     TextEditor::FindInFiles *findInFiles = TextEditor::FindInFiles::instance();
     QTC_ASSERT(findInFiles, return);
     connect(findInFiles, &TextEditor::FindInFiles::pathChanged,
-            m_widget, [this](const QString &path) {
+            m_widget, [this](const FilePath &path) {
         setEnabled(isGitDirectory(path));
     });
     connect(this, &SearchEngine::enabledChanged, m_widget, &QWidget::setEnabled);
@@ -305,20 +304,21 @@ void GitGrep::writeSettings(QSettings *settings) const
 QFuture<FileSearchResultList> GitGrep::executeSearch(const TextEditor::FileFindParameters &parameters,
         TextEditor::BaseFileFind * /*baseFileFind*/)
 {
-    return Utils::runAsync(GitGrepRunner::run, parameters);
+    return Utils::runAsync(GitGrepRunner(parameters));
 }
 
 IEditor *GitGrep::openEditor(const SearchResultItem &item,
                              const TextEditor::FileFindParameters &parameters)
 {
     GitGrepParameters params = parameters.searchEngineParameters.value<GitGrepParameters>();
-    if (params.ref.isEmpty() || item.path.isEmpty())
+    if (params.ref.isEmpty() || item.path().isEmpty())
         return nullptr;
-    const QString path = QDir::fromNativeSeparators(item.path.first());
-    const QString topLevel = parameters.additionalParameters.toString();
+    const QString path = QDir::fromNativeSeparators(item.path().first());
+    const FilePath topLevel = FilePath::fromString(parameters.additionalParameters.toString());
     IEditor *editor = m_client->openShowEditor(
                 topLevel, params.ref, path, GitClient::ShowEditor::OnlyIfDifferent);
-    editor->gotoLine(item.mainRange.begin.line, item.mainRange.begin.column);
+    if (editor)
+        editor->gotoLine(item.mainRange().begin.line, item.mainRange().begin.column);
     return editor;
 }
 
