@@ -34,7 +34,6 @@
 #include "gitsubmiteditor.h"
 #include "remotedialog.h"
 #include "stashdialog.h"
-#include "settingspage.h"
 #include "logchangedialog.h"
 #include "mergetool.h"
 #include "gitutils.h"
@@ -60,6 +59,8 @@
 #include <aggregation/aggregate.h>
 
 #include <texteditor/texteditor.h>
+
+#include <utils/algorithm.h>
 #include <utils/infobar.h>
 #include <utils/parameteraction.h>
 #include <utils/pathchooser.h>
@@ -86,6 +87,7 @@
 #include <QClipboard>
 #include <QFileDialog>
 #include <QMenu>
+#include <QTimer>
 #include <QVBoxLayout>
 
 #ifdef WITH_TESTS
@@ -103,7 +105,7 @@ using namespace std::placeholders;
 namespace Git {
 namespace Internal {
 
-using GitClientMemberFunc = void (GitClient::*)(const QString &) const;
+using GitClientMemberFunc = void (GitClient::*)(const FilePath &) const;
 
 class GitTopicCache : public Core::IVersionControl::TopicCache
 {
@@ -113,13 +115,13 @@ public:
     { }
 
 protected:
-    QString trackFile(const QString &repository) override
+    FilePath trackFile(const FilePath &repository) override
     {
         const QString gitDir = m_client->findGitDirForRepository(repository);
-        return gitDir.isEmpty() ? QString() : (gitDir + "/HEAD");
+        return gitDir.isEmpty() ? FilePath() : FilePath::fromString(gitDir + "/HEAD");
     }
 
-    QString refreshTopic(const QString &repository) override
+    QString refreshTopic(const FilePath &repository) override
     {
         return m_client->synchronousTopic(repository);
     }
@@ -235,23 +237,23 @@ public:
     QString displayName() const final;
     Utils::Id id() const final;
 
-    bool isVcsFileOrDirectory(const Utils::FilePath &fileName) const final;
+    bool isVcsFileOrDirectory(const FilePath &filePath) const final;
 
-    bool managesDirectory(const QString &directory, QString *topLevel) const final;
-    bool managesFile(const QString &workingDirectory, const QString &fileName) const final;
-    QStringList unmanagedFiles(const QStringList &filePaths) const final;
+    bool managesDirectory(const FilePath &directory, FilePath *topLevel) const final;
+    bool managesFile(const FilePath &workingDirectory, const QString &fileName) const final;
+    FilePaths unmanagedFiles(const FilePaths &filePaths) const final;
 
     bool isConfigured() const final;
     bool supportsOperation(Operation operation) const final;
-    bool vcsOpen(const QString &fileName) final;
-    bool vcsAdd(const QString &fileName) final;
-    bool vcsDelete(const QString &filename) final;
-    bool vcsMove(const QString &from, const QString &to) final;
-    bool vcsCreateRepository(const QString &directory) final;
+    bool vcsOpen(const FilePath &filePath) final;
+    bool vcsAdd(const FilePath &filePath) final;
+    bool vcsDelete(const FilePath &filePath) final;
+    bool vcsMove(const FilePath &from, const FilePath &to) final;
+    bool vcsCreateRepository(const FilePath &directory) final;
 
-    void vcsAnnotate(const QString &file, int line) final;
-    void vcsDescribe(const QString &source, const QString &id) final { m_gitClient.show(source, id); };
-    QString vcsTopic(const QString &directory) final;
+    void vcsAnnotate(const FilePath &filePath, int line) final;
+    void vcsDescribe(const FilePath &source, const QString &id) final { m_gitClient.show(source.toString(), id); };
+    QString vcsTopic(const FilePath &directory) final;
 
     Core::ShellCommand *createInitialCheckoutCommand(const QString &url,
                                                      const Utils::FilePath &baseDirectory,
@@ -259,7 +261,7 @@ public:
                                                      const QStringList &extraArgs) final;
 
     void fillLinkContextMenu(QMenu *menu,
-                             const QString &workingDirectory,
+                             const FilePath &workingDirectory,
                              const QString &reference) final
     {
         menu->addAction(tr("&Copy \"%1\"").arg(reference),
@@ -267,15 +269,15 @@ public:
         QAction *action = menu->addAction(tr("&Describe Change %1").arg(reference),
                                           [=] { vcsDescribe(workingDirectory, reference); });
         menu->setDefaultAction(action);
-        GitClient::addChangeActions(menu, workingDirectory, reference);
+        GitClient::addChangeActions(menu, workingDirectory.toString(), reference);
     }
 
-    bool handleLink(const QString &workingDirectory, const QString &reference) final
+    bool handleLink(const FilePath &workingDirectory, const QString &reference) final
     {
         if (reference.contains(".."))
             GitClient::instance()->log(workingDirectory, {}, false, {reference});
         else
-            GitClient::instance()->show(workingDirectory, reference);
+            GitClient::instance()->show(workingDirectory.toString(), reference);
         return true;
     }
 
@@ -285,12 +287,12 @@ public:
 
     bool isCommitEditorOpen() const;
     void startCommit(CommitType commitType = SimpleCommit);
-    void updateBranches(const QString &repository);
+    void updateBranches(const FilePath &repository);
     void updateCurrentBranch();
 
     void manageRemotes();
     void initRepository();
-    void startRebaseFromCommit(const QString &workingDirectory, QString commit);
+    void startRebaseFromCommit(const FilePath &workingDirectory, QString commit);
 
     void updateActions(VcsBase::VcsBasePluginPrivate::ActionState) override;
     bool submitEditorAboutToClose() override;
@@ -370,8 +372,8 @@ public:
     void updateRepositoryBrowserAction();
     Core::IEditor *openSubmitEditor(const QString &fileName, const CommitData &cd);
     void cleanCommitMessageFile();
-    void cleanRepository(const QString &directory);
-    void applyPatch(const QString &workingDirectory, QString file = QString());
+    void cleanRepository(const FilePath &directory);
+    void applyPatch(const FilePath &workingDirectory, QString file = QString());
     void updateVersionWarning();
 
 
@@ -405,11 +407,11 @@ public:
     QPointer<StashDialog> m_stashDialog;
     BranchViewFactory m_branchViewFactory;
     QPointer<RemoteDialog> m_remoteDialog;
-    QString m_submitRepository;
+    FilePath m_submitRepository;
     QString m_commitMessageFileName;
     bool m_submitActionTriggered = false;
 
-    GitSettingsPage settingPage{&m_settings, std::bind(&GitPluginPrivate::onApplySettings, this)};
+    GitSettingsPage settingPage{&m_settings};
 
     GitGrep gitGrep{&m_gitClient};
 
@@ -476,8 +478,11 @@ void GitPluginPrivate::onApplySettings()
     bool gitFoundOk;
     QString errorMessage;
     m_settings.gitExecutable(&gitFoundOk, &errorMessage);
-    if (!gitFoundOk)
-        Core::AsynchronousMessageBox::warning(tr("Git Settings"), errorMessage);
+    if (!gitFoundOk) {
+        QTimer::singleShot(0, this, [this, errorMessage] {
+            Core::AsynchronousMessageBox::warning(tr("Git Settings"), errorMessage);
+        });
+    }
 }
 
 void GitPluginPrivate::cleanCommitMessageFile()
@@ -513,11 +518,11 @@ const VcsBasePluginState &GitPlugin::currentState()
     return dd->currentState();
 }
 
-QString GitPlugin::msgRepositoryLabel(const QString &repository)
+QString GitPlugin::msgRepositoryLabel(const FilePath &repository)
 {
     return repository.isEmpty() ?
             tr("<No repository>")  :
-            tr("Repository: %1").arg(QDir::toNativeSeparators(repository));
+            tr("Repository: %1").arg(repository.toUserOutput());
 }
 
 // Returns a regular expression pattern with characters not allowed
@@ -663,6 +668,7 @@ GitPluginPrivate::GitPluginPrivate()
 
     const QString prefix = "git";
     m_commandLocator = new CommandLocator("Git", prefix, prefix, this);
+    m_commandLocator->setDescription(tr("Triggers a Git version control operation."));
 
     //register actions
     ActionContainer *toolsContainer = ActionManager::actionContainer(Core::Constants::M_TOOLS);
@@ -1022,6 +1028,7 @@ GitPluginPrivate::GitPluginPrivate()
     m_gerritPlugin->updateActions(currentState());
     m_gerritPlugin->addToLocator(m_commandLocator);
 
+    connect(&m_settings, &AspectContainer::applied, this, &GitPluginPrivate::onApplySettings);
 }
 
 void GitPluginPrivate::diffCurrentFile()
@@ -1117,7 +1124,7 @@ void GitPluginPrivate::undoFileChanges(bool revertStaging)
     }
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
-    FileChangeBlocker fcb(state.currentFile());
+    FileChangeBlocker fcb(Utils::FilePath::fromString(state.currentFile()));
     m_gitClient.revert({state.currentFile()}, revertStaging);
 }
 
@@ -1154,11 +1161,11 @@ void GitPluginPrivate::resetRepository()
         return;
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
-    QString topLevel = state.topLevel();
+    FilePath topLevel = state.topLevel();
 
     LogChangeDialog dialog(true, ICore::dialogParent());
     ResetItemDelegate delegate(dialog.widget());
-    dialog.setWindowTitle(tr("Undo Changes to %1").arg(QDir::toNativeSeparators(topLevel)));
+    dialog.setWindowTitle(tr("Undo Changes to %1").arg(topLevel.toUserOutput()));
     if (dialog.runDialog(topLevel, QString(), LogChangeWidget::IncludeRemotes))
         m_gitClient.reset(topLevel, dialog.resetFlag(), dialog.commit());
 }
@@ -1176,12 +1183,12 @@ void GitPluginPrivate::startRebase()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
-    const QString topLevel = state.topLevel();
+    const FilePath topLevel = state.topLevel();
 
     startRebaseFromCommit(topLevel, QString());
 }
 
-void GitPluginPrivate::startRebaseFromCommit(const QString &workingDirectory, QString commit)
+void GitPluginPrivate::startRebaseFromCommit(const FilePath &workingDirectory, QString commit)
 {
     if (!DocumentManager::saveAllModifiedDocuments())
         return;
@@ -1213,7 +1220,7 @@ void GitPluginPrivate::startChangeRelatedAction(const Id &id)
     if (result == QDialog::Rejected)
         return;
 
-    const QString workingDirectory = dialog.workingDirectory();
+    const FilePath workingDirectory = dialog.workingDirectory();
     const QString change = dialog.change();
 
     if (workingDirectory.isEmpty() || change.isEmpty())
@@ -1222,10 +1229,10 @@ void GitPluginPrivate::startChangeRelatedAction(const Id &id)
     if (dialog.command() == Show) {
         const int colon = change.indexOf(':');
         if (colon > 0) {
-            const QString path = QDir(workingDirectory).absoluteFilePath(change.mid(colon + 1));
+            const QString path = QDir(workingDirectory.toString()).absoluteFilePath(change.mid(colon + 1));
             m_gitClient.openShowEditor(workingDirectory, change.left(colon), path);
         } else {
-            m_gitClient.show(workingDirectory, change);
+            m_gitClient.show(workingDirectory.toString(), change);
         }
         return;
     }
@@ -1293,14 +1300,14 @@ void GitPluginPrivate::gitkForCurrentFolder()
      *  one line command mentioned above.
      *
      */
-    QDir dir(state.currentFileDirectory());
+    QDir dir(state.currentFileDirectory().toString());
     if (QFileInfo(dir,".git").exists() || dir.cd(".git")) {
         m_gitClient.launchGitK(state.currentFileDirectory());
     } else {
         QString folderName = dir.absolutePath();
         dir.cdUp();
         folderName = folderName.remove(0, dir.absolutePath().length() + 1);
-        m_gitClient.launchGitK(dir.absolutePath(), folderName);
+        m_gitClient.launchGitK(FilePath::fromString(dir.absolutePath()), folderName);
     }
 }
 
@@ -1353,7 +1360,7 @@ void GitPluginPrivate::startCommit(CommitType commitType)
         VcsOutputWindow::appendError(saver.errorString());
         return;
     }
-    m_commitMessageFileName = saver.fileName();
+    m_commitMessageFileName = saver.filePath().toString();
     openSubmitEditor(m_commitMessageFileName, data);
 }
 
@@ -1396,7 +1403,7 @@ IEditor *GitPluginPrivate::openSubmitEditor(const QString &fileName, const Commi
     }
     IDocument *document = submitEditor->document();
     document->setPreferredDisplayName(title);
-    VcsBase::setSource(document, m_submitRepository);
+    VcsBase::setSource(document, m_submitRepository.toString());
     return editor;
 }
 
@@ -1485,8 +1492,8 @@ void GitPluginPrivate::pull()
         return;
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
-    QString topLevel = state.topLevel();
-    bool rebase = m_settings.boolValue(GitSettings::pullRebaseKey);
+    FilePath topLevel = state.topLevel();
+    bool rebase = m_settings.pullRebase.value();
 
     if (!rebase) {
         QString currentBranch = m_gitClient.synchronousCurrentLocalBranch(topLevel);
@@ -1558,7 +1565,7 @@ void GitPluginPrivate::cleanRepository()
     cleanRepository(state.topLevel());
 }
 
-void GitPluginPrivate::cleanRepository(const QString &directory)
+void GitPluginPrivate::cleanRepository(const FilePath &directory)
 {
     // Find files to be deleted
     QString errorMessage;
@@ -1580,7 +1587,7 @@ void GitPluginPrivate::cleanRepository(const QString &directory)
 
     // Show in dialog
     CleanDialog dialog(ICore::dialogParent());
-    dialog.setFileList(directory, files, ignoredFiles);
+    dialog.setFileList(directory.toString(), files, ignoredFiles);
     dialog.exec();
 }
 
@@ -1615,7 +1622,7 @@ void GitPluginPrivate::promptApplyPatch()
     applyPatch(state.topLevel(), QString());
 }
 
-void GitPluginPrivate::applyPatch(const QString &workingDirectory, QString file)
+void GitPluginPrivate::applyPatch(const FilePath &workingDirectory, QString file)
 {
     // Ensure user has been notified about pending changes
     if (!m_gitClient.beginStashScope(workingDirectory, "Apply-Patch", AllowUnstashed))
@@ -1633,7 +1640,8 @@ void GitPluginPrivate::applyPatch(const QString &workingDirectory, QString file)
     QString errorMessage;
     if (m_gitClient.synchronousApplyPatch(workingDirectory, file, &errorMessage)) {
         if (errorMessage.isEmpty())
-            VcsOutputWindow::appendMessage(tr("Patch %1 successfully applied to %2").arg(file, workingDirectory));
+            VcsOutputWindow::appendMessage(tr("Patch %1 successfully applied to %2")
+                                           .arg(file, workingDirectory.toUserOutput()));
         else
             VcsOutputWindow::appendError(errorMessage);
     } else {
@@ -1650,7 +1658,7 @@ void GitPluginPrivate::stash(bool unstagedOnly)
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
 
-    const QString topLevel = state.topLevel();
+    const FilePath topLevel = state.topLevel();
     m_gitClient.executeSynchronousStash(topLevel, QString(), unstagedOnly);
     if (m_stashDialog)
         m_stashDialog->refresh(topLevel, true);
@@ -1676,7 +1684,7 @@ void GitPluginPrivate::stashPop()
 {
     if (!DocumentManager::saveAllModifiedDocuments())
         return;
-    const QString repository = currentState().topLevel();
+    const FilePath repository = currentState().topLevel();
     m_gitClient.stashPop(repository);
     if (m_stashDialog)
         m_stashDialog->refresh(repository, true);
@@ -1684,7 +1692,7 @@ void GitPluginPrivate::stashPop()
 
 // Create a non-modal dialog with refresh function or raise if it exists
 template <class NonModalDialog>
-    inline void showNonModalDialog(const QString &topLevel,
+    inline void showNonModalDialog(const FilePath &topLevel,
                                    QPointer<NonModalDialog> &dialog)
 {
     if (dialog) {
@@ -1795,7 +1803,7 @@ void GitPluginPrivate::delayedPushToGerrit()
     m_gerritPlugin->push(m_submitRepository);
 }
 
-void GitPluginPrivate::updateBranches(const QString &repository)
+void GitPluginPrivate::updateBranches(const FilePath &repository)
 {
     if (m_branchViewFactory.view())
         m_branchViewFactory.view()->refreshIfSame(repository);
@@ -1821,8 +1829,7 @@ QObject *GitPlugin::remoteCommand(const QStringList &options, const QString &wor
 void GitPluginPrivate::updateRepositoryBrowserAction()
 {
     const bool repositoryEnabled = currentState().hasTopLevel();
-    const bool hasRepositoryBrowserCmd
-            = !m_settings.stringValue(GitSettings::repositoryBrowserCmd).isEmpty();
+    const bool hasRepositoryBrowserCmd = !m_settings.repositoryBrowserCmd.value().isEmpty();
     m_repositoryBrowserAction->setEnabled(repositoryEnabled && hasRepositoryBrowserCmd);
 }
 
@@ -1836,13 +1843,13 @@ Utils::Id GitPluginPrivate::id() const
     return Utils::Id(VcsBase::Constants::VCS_ID_GIT);
 }
 
-bool GitPluginPrivate::isVcsFileOrDirectory(const Utils::FilePath &fileName) const
+bool GitPluginPrivate::isVcsFileOrDirectory(const FilePath &filePath) const
 {
-    if (fileName.fileName().compare(".git", Utils::HostOsInfo::fileNameCaseSensitivity()))
+    if (filePath.fileName().compare(".git", Utils::HostOsInfo::fileNameCaseSensitivity()))
         return false;
-    if (fileName.isDir())
+    if (filePath.isDir())
         return true;
-    QFile file(fileName.toString());
+    QFile file(filePath.toString());
     if (!file.open(QFile::ReadOnly))
         return false;
     return file.read(8) == "gitdir: ";
@@ -1871,36 +1878,34 @@ bool GitPluginPrivate::supportsOperation(Operation operation) const
     return false;
 }
 
-bool GitPluginPrivate::vcsOpen(const QString & /*fileName*/)
+bool GitPluginPrivate::vcsOpen(const FilePath & /*filePath*/)
 {
     return false;
 }
 
-bool GitPluginPrivate::vcsAdd(const QString &fileName)
+bool GitPluginPrivate::vcsAdd(const FilePath &filePath)
 {
-    const QFileInfo fi(fileName);
-    return m_gitClient.synchronousAdd(fi.absolutePath(), {fi.fileName()}, {"--intent-to-add"});
+    return m_gitClient.synchronousAdd(filePath.parentDir(), {filePath.fileName()}, {"--intent-to-add"});
 }
 
-bool GitPluginPrivate::vcsDelete(const QString &fileName)
+bool GitPluginPrivate::vcsDelete(const FilePath &filePath)
 {
-    const QFileInfo fi(fileName);
-    return m_gitClient.synchronousDelete(fi.absolutePath(), true, {fi.fileName()});
+    return m_gitClient.synchronousDelete(filePath.parentDir().absolutePath(), true, {filePath.fileName()});
 }
 
-bool GitPluginPrivate::vcsMove(const QString &from, const QString &to)
+bool GitPluginPrivate::vcsMove(const FilePath &from, const FilePath &to)
 {
-    const QFileInfo fromInfo(from);
-    const QFileInfo toInfo(to);
-    return m_gitClient.synchronousMove(fromInfo.absolutePath(), fromInfo.absoluteFilePath(), toInfo.absoluteFilePath());
+    const QFileInfo fromInfo = from.toFileInfo();
+    const QFileInfo toInfo = to.toFileInfo();
+    return m_gitClient.synchronousMove(from.parentDir().absolutePath(), fromInfo.absoluteFilePath(), toInfo.absoluteFilePath());
 }
 
-bool GitPluginPrivate::vcsCreateRepository(const QString &directory)
+bool GitPluginPrivate::vcsCreateRepository(const FilePath &directory)
 {
     return m_gitClient.synchronousInit(directory);
 }
 
-QString GitPluginPrivate::vcsTopic(const QString &directory)
+QString GitPluginPrivate::vcsTopic(const FilePath &directory)
 {
     QString topic = Core::IVersionControl::vcsTopic(directory);
     const QString commandInProgress = m_gitClient.commandInProgressDescription(directory);
@@ -1917,7 +1922,7 @@ Core::ShellCommand *GitPluginPrivate::createInitialCheckoutCommand(const QString
     QStringList args = {"clone", "--progress"};
     args << extraArgs << url << localName;
 
-    auto command = new VcsBase::VcsCommand(baseDirectory.toString(), m_gitClient.processEnvironment());
+    auto command = new VcsBase::VcsCommand(baseDirectory, m_gitClient.processEnvironment());
     command->addFlags(VcsBase::VcsCommand::SuppressStdErr);
     command->addJob({m_gitClient.vcsBinary(), args}, -1);
     return command;
@@ -1937,28 +1942,27 @@ QStringList GitPluginPrivate::additionalToolsPath() const
     return res;
 }
 
-bool GitPluginPrivate::managesDirectory(const QString &directory, QString *topLevel) const
+bool GitPluginPrivate::managesDirectory(const FilePath &directory, FilePath *topLevel) const
 {
-    const QString topLevelFound = m_gitClient.findRepositoryForDirectory(directory);
+    const FilePath topLevelFound = m_gitClient.findRepositoryForDirectory(directory);
     if (topLevel)
         *topLevel = topLevelFound;
     return !topLevelFound.isEmpty();
 }
 
-bool GitPluginPrivate::managesFile(const QString &workingDirectory, const QString &fileName) const
+bool GitPluginPrivate::managesFile(const FilePath &workingDirectory, const QString &fileName) const
 {
     return m_gitClient.managesFile(workingDirectory, fileName);
 }
 
-QStringList GitPluginPrivate::unmanagedFiles(const QStringList &filePaths) const
+FilePaths GitPluginPrivate::unmanagedFiles(const FilePaths &filePaths) const
 {
     return m_gitClient.unmanagedFiles(filePaths);
 }
 
-void GitPluginPrivate::vcsAnnotate(const QString &file, int line)
+void GitPluginPrivate::vcsAnnotate(const FilePath &filePath, int line)
 {
-    const QFileInfo fi(file);
-    m_gitClient.annotate(fi.absolutePath(), fi.fileName(), QString(), line);
+    m_gitClient.annotate(filePath.absolutePath(), filePath.fileName(), QString(), line);
 }
 
 void GitPlugin::emitFilesChanged(const QStringList &l)
@@ -1968,10 +1972,10 @@ void GitPlugin::emitFilesChanged(const QStringList &l)
 
 void GitPlugin::emitRepositoryChanged(const QString &r)
 {
-    emit dd->repositoryChanged(r);
+    emit dd->repositoryChanged(FilePath::fromString(r));
 }
 
-void GitPlugin::startRebaseFromCommit(const QString &workingDirectory, const QString &commit)
+void GitPlugin::startRebaseFromCommit(const FilePath &workingDirectory, const QString &commit)
 {
     dd->startRebaseFromCommit(workingDirectory, commit);
 }
@@ -1996,12 +2000,12 @@ void GitPlugin::updateCurrentBranch()
     dd->updateCurrentBranch();
 }
 
-void GitPlugin::updateBranches(const QString &repository)
+void GitPlugin::updateBranches(const FilePath &repository)
 {
     dd->updateBranches(repository);
 }
 
-void GitPlugin::gerritPush(const QString &topLevel)
+void GitPlugin::gerritPush(const FilePath &topLevel)
 {
     dd->m_gerritPlugin->push(topLevel);
 }
