@@ -29,7 +29,9 @@
 #include "logchangedialog.h"
 
 #include <coreplugin/coreconstants.h>
+
 #include <utils/completingtextedit.h>
+#include <utils/filepath.h>
 #include <utils/theme/theme.h>
 #include <utils/utilsicons.h>
 
@@ -41,6 +43,8 @@
 #include <QVBoxLayout>
 #include <QMenu>
 
+using namespace Utils;
+
 namespace Git {
 namespace Internal {
 
@@ -61,11 +65,13 @@ GitSubmitEditorWidget::GitSubmitEditorWidget() :
             this, &GitSubmitEditorWidget::authorInformationChanged);
     connect(m_gitSubmitPanelUi.emailLineEdit, &QLineEdit::textChanged,
             this, &GitSubmitEditorWidget::authorInformationChanged);
+    connect(m_gitSubmitPanelUi.showHeadLabel, &QLabel::linkActivated,
+            this, [this] { emit showRequested("HEAD"); });
 }
 
 void GitSubmitEditorWidget::setPanelInfo(const GitSubmitEditorPanelInfo &info)
 {
-    m_gitSubmitPanelUi.repositoryLabel->setText(QDir::toNativeSeparators(info.repository));
+    m_gitSubmitPanelUi.repositoryLabel->setText(info.repository.toUserOutput());
     if (info.branch.contains("(no branch)")) {
         const QString errorColor =
                 Utils::creatorTheme()->color(Utils::Theme::TextColorError).name();
@@ -87,7 +93,7 @@ void GitSubmitEditorWidget::setHasUnmerged(bool e)
 }
 
 void GitSubmitEditorWidget::initialize(CommitType commitType,
-                                       const QString &repository,
+                                       const FilePath &repository,
                                        const GitSubmitEditorPanelData &data,
                                        const GitSubmitEditorPanelInfo &info,
                                        bool enablePush)
@@ -95,6 +101,8 @@ void GitSubmitEditorWidget::initialize(CommitType commitType,
     if (m_isInitialized)
         return;
     m_isInitialized = true;
+    if (commitType != AmendCommit)
+        m_gitSubmitPanelUi.showHeadLabel->hide();
     if (commitType == FixupCommit) {
         auto logChangeGroupBox = new QGroupBox(tr("Select Change"));
         auto logChangeLayout = new QVBoxLayout;
@@ -123,7 +131,7 @@ void GitSubmitEditorWidget::initialize(CommitType commitType,
     }
 }
 
-void GitSubmitEditorWidget::refreshLog(const QString &repository)
+void GitSubmitEditorWidget::refreshLog(const FilePath &repository)
 {
     if (m_logChangeWidget)
         m_logChangeWidget->init(repository);
