@@ -42,6 +42,7 @@
 #include <QTreeView>
 #include <QVBoxLayout>
 
+using namespace Utils;
 using namespace VcsBase;
 
 namespace Git {
@@ -75,9 +76,9 @@ public:
         return QStandardItemModel::data(index, role);
     }
 
-    void setWorkingDirectory(const QString &workingDir) { m_workingDirectory = workingDir; }
+    void setWorkingDirectory(const FilePath &workingDir) { m_workingDirectory = workingDir; }
 private:
-    QString m_workingDirectory;
+    FilePath m_workingDirectory;
     mutable QHash<QString, QString> m_descriptions;
 };
 
@@ -99,17 +100,15 @@ LogChangeWidget::LogChangeWidget(QWidget *parent)
     setFocus();
 }
 
-bool LogChangeWidget::init(const QString &repository, const QString &commit, LogFlags flags)
+bool LogChangeWidget::init(const FilePath &repository, const QString &commit, LogFlags flags)
 {
     m_model->setWorkingDirectory(repository);
     if (!populateLog(repository, commit, flags))
         return false;
     if (m_model->rowCount() > 0)
         return true;
-    if (!(flags & Silent)) {
-        VcsOutputWindow::appendError(
-                    GitClient::instance()->msgNoCommits(flags & IncludeRemotes));
-    }
+    if (!(flags & Silent))
+        VcsOutputWindow::appendError(GitClient::msgNoCommits(flags & IncludeRemotes));
     return false;
 }
 
@@ -173,7 +172,7 @@ void LogChangeWidget::selectionChanged(const QItemSelection &selected,
     }
 }
 
-bool LogChangeWidget::populateLog(const QString &repository, const QString &commit, LogFlags flags)
+bool LogChangeWidget::populateLog(const FilePath &repository, const QString &commit, LogFlags flags)
 {
     const QString currentCommit = this->commit();
     int selected = currentCommit.isEmpty() ? 0 : -1;
@@ -234,6 +233,7 @@ const QStandardItem *LogChangeWidget::currentItem(int column) const
 LogChangeDialog::LogChangeDialog(bool isReset, QWidget *parent) :
     QDialog(parent)
     , m_widget(new LogChangeWidget)
+
     , m_dialogButtonBox(new QDialogButtonBox(this))
 {
     auto layout = new QVBoxLayout(this);
@@ -246,8 +246,7 @@ LogChangeDialog::LogChangeDialog(bool isReset, QWidget *parent) :
         m_resetTypeComboBox->addItem(tr("Hard"), "--hard");
         m_resetTypeComboBox->addItem(tr("Mixed"), "--mixed");
         m_resetTypeComboBox->addItem(tr("Soft"), "--soft");
-        m_resetTypeComboBox->setCurrentIndex(
-                    GitClient::instance()->settings().intValue(GitSettings::lastResetIndexKey));
+        m_resetTypeComboBox->setCurrentIndex(GitClient::settings().lastResetIndex.value());
         popUpLayout->addWidget(m_resetTypeComboBox);
         popUpLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Ignored));
     }
@@ -265,7 +264,7 @@ LogChangeDialog::LogChangeDialog(bool isReset, QWidget *parent) :
     resize(600, 400);
 }
 
-bool LogChangeDialog::runDialog(const QString &repository,
+bool LogChangeDialog::runDialog(const FilePath &repository,
                                 const QString &commit,
                                 LogChangeWidget::LogFlags flags)
 {
@@ -273,10 +272,8 @@ bool LogChangeDialog::runDialog(const QString &repository,
         return false;
 
     if (QDialog::exec() == QDialog::Accepted) {
-        if (m_resetTypeComboBox) {
-            GitClient::instance()->settings().setValue(
-                        GitSettings::lastResetIndexKey, m_resetTypeComboBox->currentIndex());
-        }
+        if (m_resetTypeComboBox)
+            GitClient::settings().lastResetIndex.setValue(m_resetTypeComboBox->currentIndex());
         return true;
     }
     return false;
