@@ -36,6 +36,7 @@
 #include <QProcess>
 #include <QPushButton>
 
+using namespace Utils;
 using namespace VcsBase;
 
 namespace Git {
@@ -49,7 +50,7 @@ MergeTool::~MergeTool()
     delete m_process;
 }
 
-bool MergeTool::start(const QString &workingDirectory, const QStringList &files)
+bool MergeTool::start(const FilePath &workingDirectory, const QStringList &files)
 {
     QStringList arguments;
     arguments << "mergetool" << "-y" << files;
@@ -57,7 +58,7 @@ bool MergeTool::start(const QString &workingDirectory, const QStringList &files)
     env.insert("LANG", "C");
     env.insert("LANGUAGE", "C");
     m_process = new QProcess(this);
-    m_process->setWorkingDirectory(workingDirectory);
+    m_process->setWorkingDirectory(workingDirectory.toString());
     m_process->setProcessEnvironment(env);
     m_process->setProcessChannelMode(QProcess::MergedChannels);
     const Utils::FilePath binary = GitClient::instance()->vcsBinary();
@@ -263,7 +264,7 @@ void MergeTool::done()
         VcsOutputWindow::appendError(tr("Merge tool process terminated with exit code %1")
                                   .arg(exitCode));
     }
-    GitClient::instance()->continueCommandIfNeeded(workingDirectory, exitCode == 0);
+    GitClient::instance()->continueCommandIfNeeded(FilePath::fromString(workingDirectory), exitCode == 0);
     GitPlugin::emitRepositoryChanged(workingDirectory);
     deleteLater();
 }
