@@ -277,7 +277,7 @@ void GlslEditorWidget::updateDocumentNow()
         CreateRanges createRanges(document(), doc);
         createRanges(ast);
 
-        const TextEditor::FontSettings &fontSettings = TextEditor::TextEditorSettings::instance()->fontSettings();
+        const TextEditor::FontSettings &fontSettings = TextEditor::TextEditorSettings::fontSettings();
 
         QTextCharFormat warningFormat = fontSettings.toTextCharFormat(TextEditor::C_WARNING);
         QTextCharFormat errorFormat = fontSettings.toTextCharFormat(TextEditor::C_ERROR);
