@@ -34,9 +34,12 @@
 #include <QBuffer>
 #include <QContextMenuEvent>
 #include <QCoreApplication>
+#include <QDesktopServices>
 #include <QTimer>
 #include <QVBoxLayout>
-#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
+#if QT_VERSION >= QT_VERSION_CHECK(6, 2, 0)
+#include <QWebEngineContextMenuRequest>
+#else
 #include <QWebEngineContextMenuData>
 #endif
 #include <QWebEngineHistory>
@@ -74,10 +77,40 @@ static HelpUrlSchemeHandler *helpUrlSchemeHandler()
     return schemeHandler;
 }
 
+HelpUrlRequestInterceptor::HelpUrlRequestInterceptor(QObject *parent)
+    : QWebEngineUrlRequestInterceptor(parent)
+{}
+
+void HelpUrlRequestInterceptor::interceptRequest(QWebEngineUrlRequestInfo &info)
+{
+    if (!HelpViewer::isLocalUrl(info.requestUrl())
+        && info.navigationType() != QWebEngineUrlRequestInfo::NavigationTypeLink) {
+        info.block(true);
+    }
+}
+
+static HelpUrlRequestInterceptor *helpurlRequestInterceptor()
+{
+    static HelpUrlRequestInterceptor *interceptor = nullptr;
+    if (!interceptor)
+        interceptor = new HelpUrlRequestInterceptor(LocalHelpManager::instance());
+    return interceptor;
+}
+
 WebEngineHelpViewer::WebEngineHelpViewer(QWidget *parent) :
     HelpViewer(parent),
     m_widget(new WebView(this))
 {
+    // some of these should already be that way by default, but better be sure
+    QWebEngineSettings *settings = m_widget->settings();
+    settings->setAttribute(QWebEngineSettings::JavascriptCanOpenWindows, false);
+    settings->setAttribute(QWebEngineSettings::LocalContentCanAccessRemoteUrls, false);
+    settings->setAttribute(QWebEngineSettings::XSSAuditingEnabled, true);
+    settings->setAttribute(QWebEngineSettings::PluginsEnabled, false);
+    settings->setAttribute(QWebEngineSettings::AllowRunningInsecureContent, false);
+    settings->setAttribute(QWebEngineSettings::AllowGeolocationOnInsecureOrigins, false);
+    settings->setAttribute(QWebEngineSettings::AllowWindowActivationFromJavaScript, false);
+
     m_widget->setPage(new WebEngineHelpPage(this));
     auto layout = new QVBoxLayout;
     setLayout(layout);
@@ -98,7 +131,6 @@ WebEngineHelpViewer::WebEngineHelpViewer(QWidget *parent) :
         QTimer::singleShot(/*magic timeout=*/150, this, [this] {
             QUrl urlWithoutFragment = source();
             urlWithoutFragment.setFragment(QString());
-            qDebug() << urlWithoutFragment << m_previousUrlWithoutFragment;
             if (urlWithoutFragment == m_previousUrlWithoutFragment)
                 slotLoadFinished();
             m_previousUrlWithoutFragment = urlWithoutFragment;
@@ -124,13 +156,7 @@ WebEngineHelpViewer::WebEngineHelpViewer(QWidget *parent) :
     QTC_ASSERT(viewProfile, return);
     if (!viewProfile->urlSchemeHandler("qthelp"))
         viewProfile->installUrlSchemeHandler("qthelp", helpUrlSchemeHandler());
-}
-
-QFont WebEngineHelpViewer::viewerFont() const
-{
-    QWebEngineSettings *webSettings = m_widget->settings();
-    return QFont(webSettings->fontFamily(QWebEngineSettings::StandardFont),
-                 webSettings->fontSize(QWebEngineSettings::DefaultFontSize));
+    viewProfile->setUrlRequestInterceptor(helpurlRequestInterceptor());
 }
 
 void WebEngineHelpViewer::setViewerFont(const QFont &font)
@@ -140,11 +166,6 @@ void WebEngineHelpViewer::setViewerFont(const QFont &font)
     webSettings->setFontSize(QWebEngineSettings::DefaultFontSize, font.pointSize());
 }
 
-qreal WebEngineHelpViewer::scale() const
-{
-    return m_widget->zoomFactor();
-}
-
 void WebEngineHelpViewer::setScale(qreal scale)
 {
     m_widget->setZoomFactor(scale);
@@ -227,7 +248,7 @@ bool WebEngineHelpViewer::findText(const QString &text, Core::FindFlags flags, b
     Q_UNUSED(fromSearch)
     if (wrapped)
         *wrapped = false; // missing feature in QWebEngine
-    QWebEnginePage::FindFlags webEngineFlags = 0;
+    QWebEnginePage::FindFlags webEngineFlags;
     if (flags & Core::FindBackward)
         webEngineFlags |= QWebEnginePage::FindBackward;
     if (flags & Core::FindCaseSensitively)
@@ -244,21 +265,6 @@ WebEngineHelpPage *WebEngineHelpViewer::page() const
     return static_cast<WebEngineHelpPage *>(m_widget->page());
 }
 
-void WebEngineHelpViewer::scaleUp()
-{
-    m_widget->setZoomFactor(m_widget->zoomFactor() + 0.1);
-}
-
-void WebEngineHelpViewer::scaleDown()
-{
-    m_widget->setZoomFactor(qMax(qreal(0.1), m_widget->zoomFactor() - qreal(0.1)));
-}
-
-void WebEngineHelpViewer::resetScale()
-{
-    m_widget->setZoomFactor(1.0);
-}
-
 void WebEngineHelpViewer::copy()
 {
     m_widget->triggerPageAction(QWebEnginePage::Copy);
@@ -289,23 +295,55 @@ WebEngineHelpPage::WebEngineHelpPage(QObject *parent)
 {
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 7, 0)
-QWebEnginePage *WebEngineHelpPage::createWindow(QWebEnginePage::WebWindowType)
+bool WebEngineHelpPage::acceptNavigationRequest(const QUrl &url,
+                                                QWebEnginePage::NavigationType type,
+                                                bool isMainFrame)
 {
-    auto viewer = static_cast<WebEngineHelpViewer *>(OpenPagesManager::instance().createPage());
-    return viewer->page();
+    Q_UNUSED(type)
+    Q_UNUSED(isMainFrame)
+    if (HelpViewer::isLocalUrl(url))
+        return true;
+    QDesktopServices::openUrl(url);
+    return false;
 }
-#endif
 
 WebView::WebView(WebEngineHelpViewer *viewer)
-    : QWebEngineView(viewer),
-      m_viewer(viewer)
+    : QWebEngineView(viewer)
+    , m_viewer(viewer)
+{}
+
+bool WebView::event(QEvent *ev)
 {
+    // work around QTBUG-43602
+    if (ev->type() == QEvent::ChildAdded) {
+        auto ce = static_cast<QChildEvent *>(ev);
+        ce->child()->installEventFilter(this);
+    } else if (ev->type() == QEvent::ChildRemoved) {
+        auto ce = static_cast<QChildEvent *>(ev);
+        ce->child()->removeEventFilter(this);
+    }
+    return QWebEngineView::event(ev);
+}
+
+bool WebView::eventFilter(QObject *src, QEvent *e)
+{
+    Q_UNUSED(src)
+    // work around QTBUG-43602
+    if (m_viewer->isScrollWheelZoomingEnabled() && e->type() == QEvent::Wheel) {
+        auto we = static_cast<QWheelEvent *>(e);
+        if (we->modifiers() == Qt::ControlModifier)
+            return true;
+    }
+    return false;
 }
 
 void WebView::contextMenuEvent(QContextMenuEvent *event)
 {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 2, 0)
+    QMenu *menu = createStandardContextMenu();
+#else
     QMenu *menu = page()->createStandardContextMenu();
+#endif
     // insert Open as New Page etc if OpenLinkInThisWindow is also there
     const QList<QAction*> actions = menu->actions();
     auto it = std::find(actions.cbegin(), actions.cend(),
@@ -314,13 +352,11 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         // insert after
         ++it;
         QAction *before = (it == actions.cend() ? 0 : *it);
-#if QT_VERSION < QT_VERSION_CHECK(5, 7, 0)
-        if (m_viewer->isActionVisible(HelpViewer::Action::NewPage)) {
-            QAction *openLinkInNewTab = page()->action(QWebEnginePage::OpenLinkInNewTab);
-            menu->insertAction(before, openLinkInNewTab);
-        }
+#if QT_VERSION >= QT_VERSION_CHECK(6, 2, 0)
+        QUrl url = lastContextMenuRequest()->linkUrl();
 #else
         QUrl url = page()->contextMenuData().linkUrl();
+#endif
         if (m_viewer->isActionVisible(HelpViewer::Action::NewPage)) {
             auto openLink = new QAction(QCoreApplication::translate("HelpViewer",
                                         Constants::TR_OPEN_LINK_AS_NEW_PAGE), menu);
@@ -337,7 +373,6 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
             });
             menu->insertAction(before, openLink);
         }
-#endif
     }
 
     connect(menu, &QMenu::aboutToHide, menu, &QObject::deleteLater);
