@@ -31,10 +31,7 @@
 #include "iosprobe.h"
 
 #include <coreplugin/icore.h>
-#include <utils/algorithm.h>
-#include <utils/synchronousprocess.h>
-#include <utils/qtcassert.h>
-#include <utils/synchronousprocess.h>
+
 #include <projectexplorer/kitmanager.h>
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/devicesupport/devicemanager.h>
@@ -42,14 +39,20 @@
 #include <projectexplorer/toolchain.h>
 #include <projectexplorer/gcctoolchain.h>
 #include <projectexplorer/projectexplorerconstants.h>
+
 #include <debugger/debuggeritemmanager.h>
 #include <debugger/debuggeritem.h>
 #include <debugger/debuggerkitinformation.h>
+
 #include <qtsupport/baseqtversion.h>
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtversionmanager.h>
 #include <qtsupport/qtversionfactory.h>
 
+#include <utils/algorithm.h>
+#include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
+
 #include <QDir>
 #include <QDomDocument>
 #include <QFileInfo>
@@ -57,10 +60,9 @@
 #include <QHash>
 #include <QList>
 #include <QLoggingCategory>
-#include <QProcess>
 #include <QSettings>
-#include <QStringList>
 #include <QStandardPaths>
+#include <QStringList>
 #include <QTimer>
 
 #include <memory>
@@ -229,14 +231,14 @@ static QByteArray decodeProvisioningProfile(const QString &path)
 {
     QTC_ASSERT(!path.isEmpty(), return QByteArray());
 
-    Utils::SynchronousProcess p;
+    Utils::QtcProcess p;
     p.setTimeoutS(3);
     // path is assumed to be valid file path to .mobileprovision
-    const QStringList args = {"smime", "-inform", "der", "-verify", "-in", path};
-    Utils::SynchronousProcessResponse res = p.runBlocking({"openssl", args});
-    if (res.result != Utils::SynchronousProcessResponse::Finished)
+    p.setCommand({"openssl", {"smime", "-inform", "der", "-verify", "-in", path}});
+    p.runBlocking();
+    if (p.result() != Utils::QtcProcess::FinishedWithSuccess)
         qCDebug(iosCommonLog) << "Reading signed provisioning file failed" << path;
-    return res.stdOut().toLatin1();
+    return p.stdOut().toLatin1();
 }
 
 void IosConfigurations::updateAutomaticKitList()
@@ -476,11 +478,17 @@ void IosConfigurations::loadProvisioningData(bool notify)
     QList<QVariantMap> teams;
     for (auto accountiterator = teamMap.cbegin(), end = teamMap.cend();
             accountiterator != end; ++accountiterator) {
-        QVariantMap teamInfo = accountiterator.value().toMap();
-        int provisioningTeamIsFree = teamInfo.value(freeTeamTag).toBool() ? 1 : 0;
-        teamInfo[freeTeamTag] = provisioningTeamIsFree;
-        teamInfo[emailTag] = accountiterator.key();
-        teams.append(teamInfo);
+        // difference between Qt 5 (map) and Qt 6 (list of maps)
+        const bool isList = accountiterator->userType() == QMetaType::QVariantList;
+        const QVariantList teamsList = isList ? accountiterator.value().toList()
+                                              : QVariantList({accountiterator.value()});
+        for (const QVariant &teamInfoIt : teamsList) {
+            QVariantMap teamInfo = teamInfoIt.toMap();
+            int provisioningTeamIsFree = teamInfo.value(freeTeamTag).toBool() ? 1 : 0;
+            teamInfo[freeTeamTag] = provisioningTeamIsFree;
+            teamInfo[emailTag] = accountiterator.key();
+            teams.append(teamInfo);
+        }
     }
 
     // Sort team id's to move the free provisioning teams at last of the list.
@@ -580,8 +588,10 @@ IosToolChainFactory::IosToolChainFactory()
                            ProjectExplorer::Constants::CXX_LANGUAGE_ID});
 }
 
-QList<ToolChain *> IosToolChainFactory::autoDetect(const QList<ToolChain *> &existingToolChains)
+QList<ToolChain *> IosToolChainFactory::autoDetect(const QList<ToolChain *> &existingToolChains,
+                                                  const IDevice::Ptr &device)
 {
+    Q_UNUSED(device);
     QList<ClangToolChain *> existingClangToolChains = clangToolChains(existingToolChains);
     const QList<XcodePlatform> platforms = XcodeProbe::detectPlatforms().values();
     QList<ToolChain *> toolChains;
