@@ -29,7 +29,7 @@
 #include <utils/algorithm.h>
 #include <utils/runextensions.h>
 #include <utils/qtcassert.h>
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 
 #ifdef Q_OS_MAC
 #include <CoreFoundation/CoreFoundation.h>
@@ -42,13 +42,12 @@
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QLoggingCategory>
-#include <QProcess>
 
 using namespace Utils;
 using namespace std;
 
 namespace {
-Q_LOGGING_CATEGORY(simulatorLog, "qtc.ios.simulator", QtWarningMsg)
+static Q_LOGGING_CATEGORY(simulatorLog, "qtc.ios.simulator", QtWarningMsg)
 }
 
 namespace Ios {
@@ -79,33 +78,36 @@ static bool checkForTimeout(const chrono::high_resolution_clock::time_point &sta
     return timedOut;
 }
 
-static bool runCommand(const CommandLine &command, QString *output)
+static bool runCommand(const CommandLine &command, QString *stdOutput, QString *allOutput = nullptr)
 {
-    SynchronousProcess p;
+    QtcProcess p;
     p.setTimeoutS(-1);
-    SynchronousProcessResponse resp = p.runBlocking(command);
-    if (output)
-        *output = resp.stdOut();
-    return resp.result == SynchronousProcessResponse::Finished;
+    p.setCommand(command);
+    p.runBlocking();
+    if (stdOutput)
+        *stdOutput = p.stdOut();
+    if (allOutput)
+        *allOutput = p.allOutput();
+    return p.result() == QtcProcess::FinishedWithSuccess;
 }
 
-static bool runSimCtlCommand(QStringList args, QString *output)
+static bool runSimCtlCommand(QStringList args, QString *output, QString *allOutput = nullptr)
 {
     args.prepend("simctl");
-    return runCommand({"xcrun", args}, output);
+    return runCommand({"xcrun", args}, output, allOutput);
 }
 
 static bool launchSimulator(const QString &simUdid) {
     QTC_ASSERT(!simUdid.isEmpty(), return false);
-    const QString simulatorAppPath = IosConfigurations::developerPath()
-            .pathAppended("Applications/Simulator.app/Contents/MacOS/Simulator").toString();
+    const FilePath simulatorAppPath = IosConfigurations::developerPath()
+            .pathAppended("Applications/Simulator.app/Contents/MacOS/Simulator");
 
     if (IosConfigurations::xcodeVersion() >= QVersionNumber(9)) {
         // For XCode 9 boot the second device instead of launching simulator app twice.
         QString psOutput;
         if (runCommand({"ps", {"-A", "-o", "comm"}}, &psOutput)) {
             for (const QString &comm : psOutput.split('\n')) {
-                if (comm == simulatorAppPath)
+                if (comm == simulatorAppPath.toString())
                     return runSimCtlCommand({"boot", simUdid}, nullptr);
             }
         } else {
@@ -115,7 +117,7 @@ static bool launchSimulator(const QString &simUdid) {
         }
     }
 
-    return QProcess::startDetached(simulatorAppPath, {"--args", "-CurrentDeviceUDID", simUdid});
+    return QtcProcess::startDetached({simulatorAppPath, {"--args", "-CurrentDeviceUDID", simUdid}});
 }
 
 static bool isAvailable(const QJsonObject &object)
@@ -175,54 +177,44 @@ static QList<RuntimeInfo> getAvailableRuntimes()
     return runtimes;
 }
 
-class SimulatorControlPrivate {
-private:
-    SimulatorControlPrivate();
-    ~SimulatorControlPrivate();
+static SimulatorInfo deviceInfo(const QString &simUdid);
+static QString bundleIdentifier(const Utils::FilePath &bundlePath);
+static QString bundleExecutable(const Utils::FilePath &bundlePath);
 
-    static SimulatorInfo deviceInfo(const QString &simUdid);
-    static QString bundleIdentifier(const Utils::FilePath &bundlePath);
-    static QString bundleExecutable(const Utils::FilePath &bundlePath);
+static void startSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                           const QString &simUdid);
+static void installApp(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                       const QString &simUdid,
+                       const Utils::FilePath &bundlePath);
+static void launchApp(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                      const QString &simUdid,
+                      const QString &bundleIdentifier,
+                      bool waitForDebugger,
+                      const QStringList &extraArgs,
+                      const QString &stdoutPath,
+                      const QString &stderrPath);
+static void deleteSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                            const QString &simUdid);
+static void resetSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                           const QString &simUdid);
+static void renameSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                            const QString &simUdid,
+                            const QString &newName);
+static void createSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                            const QString &name,
+                            const DeviceTypeInfo &deviceType,
+                            const RuntimeInfo &runtime);
+static void takeSceenshot(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                          const QString &simUdid,
+                          const QString &filePath);
 
-    void startSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi, const QString &simUdid);
-    void installApp(QFutureInterface<SimulatorControl::ResponseData> &fi, const QString &simUdid,
-                    const Utils::FilePath &bundlePath);
-    void launchApp(QFutureInterface<SimulatorControl::ResponseData> &fi, const QString &simUdid,
-                   const QString &bundleIdentifier, bool waitForDebugger,
-                   const QStringList &extraArgs, const QString &stdoutPath,
-                   const QString &stderrPath);
-    void deleteSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                         const QString &simUdid);
-    void resetSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                        const QString &simUdid);
-    void renameSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                        const QString &simUdid, const QString &newName);
-    void createSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                         const QString &name, const DeviceTypeInfo &deviceType,
-                         const RuntimeInfo &runtime);
-    void takeSceenshot(QFutureInterface<SimulatorControl::ResponseData> &fi, const QString &simUdid,
-                       const QString &filePath);
-
-    static QList<SimulatorInfo> availableDevices;
-    static QList<DeviceTypeInfo> availableDeviceTypes;
-    static QList<RuntimeInfo> availableRuntimes;
-    friend class SimulatorControl;
-};
-
-SimulatorControl::SimulatorControl(QObject *parent) :
-    QObject(parent),
-    d(new SimulatorControlPrivate)
-{
-}
-
-SimulatorControl::~SimulatorControl()
-{
-    delete d;
-}
+static QList<SimulatorInfo> s_availableDevices;
+static QList<DeviceTypeInfo> s_availableDeviceTypes;
+static QList<RuntimeInfo> s_availableRuntimes;
 
 QList<SimulatorInfo> SimulatorControl::availableSimulators()
 {
-    return SimulatorControlPrivate::availableDevices;
+    return s_availableDevices;
 }
 
 static QList<SimulatorInfo> getAllSimulatorDevices()
@@ -264,21 +256,21 @@ QFuture<QList<DeviceTypeInfo> > SimulatorControl::updateDeviceTypes()
 {
     QFuture< QList<DeviceTypeInfo> > future = Utils::runAsync(getAvailableDeviceTypes);
     Utils::onResultReady(future, [](const QList<DeviceTypeInfo> &deviceTypes) {
-        SimulatorControlPrivate::availableDeviceTypes = deviceTypes;
+        s_availableDeviceTypes = deviceTypes;
     });
     return future;
 }
 
 QList<RuntimeInfo> SimulatorControl::availableRuntimes()
 {
-    return SimulatorControlPrivate::availableRuntimes;
+    return s_availableRuntimes;
 }
 
 QFuture<QList<RuntimeInfo> > SimulatorControl::updateRuntimes()
 {
     QFuture< QList<RuntimeInfo> > future = Utils::runAsync(getAvailableRuntimes);
     Utils::onResultReady(future, [](const QList<RuntimeInfo> &runtimes) {
-        SimulatorControlPrivate::availableRuntimes = runtimes;
+        s_availableRuntimes = runtimes;
     });
     return future;
 }
@@ -286,9 +278,8 @@ QFuture<QList<RuntimeInfo> > SimulatorControl::updateRuntimes()
 QFuture< QList<SimulatorInfo> > SimulatorControl::updateAvailableSimulators()
 {
     QFuture< QList<SimulatorInfo> > future = Utils::runAsync(getAvailableSimulators);
-    Utils::onResultReady(future, [](const QList<SimulatorInfo> &devices) {
-        SimulatorControlPrivate::availableDevices = devices;
-    });
+    Utils::onResultReady(future,
+                         [](const QList<SimulatorInfo> &devices) { s_availableDevices = devices; });
     return future;
 }
 
@@ -296,53 +287,60 @@ bool SimulatorControl::isSimulatorRunning(const QString &simUdid)
 {
     if (simUdid.isEmpty())
         return false;
-    return SimulatorControlPrivate::deviceInfo(simUdid).isBooted();
+    return deviceInfo(simUdid).isBooted();
 }
 
 QString SimulatorControl::bundleIdentifier(const Utils::FilePath &bundlePath)
 {
-    return SimulatorControlPrivate::bundleIdentifier(bundlePath);
+    return Internal::bundleIdentifier(bundlePath);
 }
 
 QString SimulatorControl::bundleExecutable(const Utils::FilePath &bundlePath)
 {
-    return SimulatorControlPrivate::bundleExecutable(bundlePath);
+    return Internal::bundleExecutable(bundlePath);
 }
 
-QFuture<SimulatorControl::ResponseData> SimulatorControl::startSimulator(const QString &simUdid) const
+QFuture<SimulatorControl::ResponseData> SimulatorControl::startSimulator(const QString &simUdid)
 {
-    return Utils::runAsync(&SimulatorControlPrivate::startSimulator, d, simUdid);
+    return Utils::runAsync(Internal::startSimulator, simUdid);
 }
 
-QFuture<SimulatorControl::ResponseData>
-SimulatorControl::installApp(const QString &simUdid, const Utils::FilePath &bundlePath) const
+QFuture<SimulatorControl::ResponseData> SimulatorControl::installApp(
+    const QString &simUdid, const Utils::FilePath &bundlePath)
 {
-    return Utils::runAsync(&SimulatorControlPrivate::installApp, d, simUdid, bundlePath);
+    return Utils::runAsync(Internal::installApp, simUdid, bundlePath);
 }
 
-QFuture<SimulatorControl::ResponseData>
-SimulatorControl::launchApp(const QString &simUdid, const QString &bundleIdentifier,
-                            bool waitForDebugger, const QStringList &extraArgs,
-                            const QString &stdoutPath, const QString &stderrPath) const
+QFuture<SimulatorControl::ResponseData> SimulatorControl::launchApp(const QString &simUdid,
+                                                                    const QString &bundleIdentifier,
+                                                                    bool waitForDebugger,
+                                                                    const QStringList &extraArgs,
+                                                                    const QString &stdoutPath,
+                                                                    const QString &stderrPath)
 {
-    return Utils::runAsync(&SimulatorControlPrivate::launchApp, d, simUdid, bundleIdentifier,
-                           waitForDebugger, extraArgs, stdoutPath, stderrPath);
+    return Utils::runAsync(Internal::launchApp,
+                           simUdid,
+                           bundleIdentifier,
+                           waitForDebugger,
+                           extraArgs,
+                           stdoutPath,
+                           stderrPath);
 }
 
-QFuture<SimulatorControl::ResponseData> SimulatorControl::deleteSimulator(const QString &simUdid) const
+QFuture<SimulatorControl::ResponseData> SimulatorControl::deleteSimulator(const QString &simUdid)
 {
-    return Utils::runAsync(&SimulatorControlPrivate::deleteSimulator, d, simUdid);
+    return Utils::runAsync(Internal::deleteSimulator, simUdid);
 }
 
-QFuture<SimulatorControl::ResponseData> SimulatorControl::resetSimulator(const QString &simUdid) const
+QFuture<SimulatorControl::ResponseData> SimulatorControl::resetSimulator(const QString &simUdid)
 {
-    return Utils::runAsync(&SimulatorControlPrivate::resetSimulator, d, simUdid);
+    return Utils::runAsync(Internal::resetSimulator, simUdid);
 }
 
 QFuture<SimulatorControl::ResponseData> SimulatorControl::renameSimulator(const QString &simUdid,
-                                                                          const QString &newName) const
+                                                                          const QString &newName)
 {
-    return Utils::runAsync(&SimulatorControlPrivate::renameSimulator, d, simUdid, newName);
+    return Utils::runAsync(Internal::renameSimulator, simUdid, newName);
 }
 
 QFuture<SimulatorControl::ResponseData>
@@ -350,25 +348,18 @@ SimulatorControl::createSimulator(const QString &name,
                                   const DeviceTypeInfo &deviceType,
                                   const RuntimeInfo &runtime)
 {
-    return Utils::runAsync(&SimulatorControlPrivate::createSimulator, d, name, deviceType, runtime);
+    return Utils::runAsync(Internal::createSimulator, name, deviceType, runtime);
 }
 
 QFuture<SimulatorControl::ResponseData> SimulatorControl::takeSceenshot(const QString &simUdid,
                                                                         const QString &filePath)
 {
-    return Utils::runAsync(&SimulatorControlPrivate::takeSceenshot, d, simUdid, filePath);
+    return Utils::runAsync(Internal::takeSceenshot, simUdid, filePath);
 }
 
 // Static members
-QList<SimulatorInfo> SimulatorControlPrivate::availableDevices;
-QList<DeviceTypeInfo> SimulatorControlPrivate::availableDeviceTypes;
-QList<RuntimeInfo> SimulatorControlPrivate::availableRuntimes;
 
-SimulatorControlPrivate::SimulatorControlPrivate() = default;
-
-SimulatorControlPrivate::~SimulatorControlPrivate() = default;
-
-SimulatorInfo SimulatorControlPrivate::deviceInfo(const QString &simUdid)
+SimulatorInfo deviceInfo(const QString &simUdid)
 {
     auto matchDevice = [simUdid](const SimulatorInfo &device) {
         return device.identifier == simUdid;
@@ -380,7 +371,7 @@ SimulatorInfo SimulatorControlPrivate::deviceInfo(const QString &simUdid)
     return device;
 }
 
-QString SimulatorControlPrivate::bundleIdentifier(const Utils::FilePath &bundlePath)
+QString bundleIdentifier(const Utils::FilePath &bundlePath)
 {
     QString bundleID;
 #ifdef Q_OS_MAC
@@ -400,7 +391,7 @@ QString SimulatorControlPrivate::bundleIdentifier(const Utils::FilePath &bundleP
     return bundleID;
 }
 
-QString SimulatorControlPrivate::bundleExecutable(const Utils::FilePath &bundlePath)
+QString bundleExecutable(const Utils::FilePath &bundlePath)
 {
     QString executable;
 #ifdef Q_OS_MAC
@@ -419,8 +410,7 @@ QString SimulatorControlPrivate::bundleExecutable(const Utils::FilePath &bundleP
     return executable;
 }
 
-void SimulatorControlPrivate::startSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                                             const QString &simUdid)
+void startSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi, const QString &simUdid)
 {
     SimulatorControl::ResponseData response(simUdid);
     SimulatorInfo simInfo = deviceInfo(simUdid);
@@ -436,7 +426,7 @@ void SimulatorControlPrivate::startSimulator(QFutureInterface<SimulatorControl::
     auto start = chrono::high_resolution_clock::now();
     while (simInfo.isShuttingDown() && !checkForTimeout(start, simulatorStartTimeout)) {
         // Wait till the simulator shuts down, if doing so.
-        QThread::currentThread()->msleep(100);
+        QThread::msleep(100);
         simInfo = deviceInfo(simUdid);
     }
 
@@ -477,22 +467,27 @@ void SimulatorControlPrivate::startSimulator(QFutureInterface<SimulatorControl::
     }
 }
 
-void SimulatorControlPrivate::installApp(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                                         const QString &simUdid, const Utils::FilePath &bundlePath)
+void installApp(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                const QString &simUdid,
+                const Utils::FilePath &bundlePath)
 {
     QTC_CHECK(bundlePath.exists());
 
     SimulatorControl::ResponseData response(simUdid);
     response.success = runSimCtlCommand({"install", simUdid, bundlePath.toString()},
+                                        nullptr,
                                         &response.commandOutput);
     if (!fi.isCanceled())
         fi.reportResult(response);
 }
 
-void SimulatorControlPrivate::launchApp(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                                        const QString &simUdid, const QString &bundleIdentifier,
-                                        bool waitForDebugger, const QStringList &extraArgs,
-                                        const QString &stdoutPath, const QString &stderrPath)
+void launchApp(QFutureInterface<SimulatorControl::ResponseData> &fi,
+               const QString &simUdid,
+               const QString &bundleIdentifier,
+               bool waitForDebugger,
+               const QStringList &extraArgs,
+               const QString &stdoutPath,
+               const QString &stderrPath)
 {
     SimulatorControl::ResponseData response(simUdid);
     if (!bundleIdentifier.isEmpty() && !fi.isCanceled()) {
@@ -513,8 +508,9 @@ void SimulatorControlPrivate::launchApp(QFutureInterface<SimulatorControl::Respo
                 args << extraArgument;
         }
 
-        if (runSimCtlCommand(args, &response.commandOutput)) {
-            const QString pIdStr = response.commandOutput.trimmed().split(' ').last().trimmed();
+        QString stdOutput;
+        if (runSimCtlCommand(args, &stdOutput, &response.commandOutput)) {
+            const QString pIdStr = stdOutput.trimmed().split(' ').last().trimmed();
             bool validPid = false;
             response.pID = pIdStr.toLongLong(&validPid);
             response.success = validPid;
@@ -526,61 +522,63 @@ void SimulatorControlPrivate::launchApp(QFutureInterface<SimulatorControl::Respo
     }
 }
 
-void SimulatorControlPrivate::deleteSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                                              const QString &simUdid)
+void deleteSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi, const QString &simUdid)
 {
     SimulatorControl::ResponseData response(simUdid);
-    response.success = runSimCtlCommand({"delete", simUdid}, &response.commandOutput);
+    response.success = runSimCtlCommand({"delete", simUdid}, nullptr, &response.commandOutput);
 
     if (!fi.isCanceled())
         fi.reportResult(response);
 }
 
-void SimulatorControlPrivate::resetSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                                             const QString &simUdid)
+void resetSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi, const QString &simUdid)
 {
     SimulatorControl::ResponseData response(simUdid);
-    response.success = runSimCtlCommand({"erase", simUdid}, &response.commandOutput);
+    response.success = runSimCtlCommand({"erase", simUdid}, nullptr, &response.commandOutput);
 
     if (!fi.isCanceled())
         fi.reportResult(response);
 }
 
-void SimulatorControlPrivate::renameSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                                             const QString &simUdid, const QString &newName)
+void renameSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                     const QString &simUdid,
+                     const QString &newName)
 {
     SimulatorControl::ResponseData response(simUdid);
     response.success = runSimCtlCommand({"rename", simUdid, newName},
+                                        nullptr,
                                         &response.commandOutput);
 
     if (!fi.isCanceled())
         fi.reportResult(response);
 }
 
-void SimulatorControlPrivate::createSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                                              const QString &name,
-                                              const DeviceTypeInfo &deviceType,
-                                              const RuntimeInfo &runtime)
+void createSimulator(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                     const QString &name,
+                     const DeviceTypeInfo &deviceType,
+                     const RuntimeInfo &runtime)
 {
     SimulatorControl::ResponseData response("Invalid");
     if (!name.isEmpty()) {
-        response.success = runSimCtlCommand({"create", name,
-                                             deviceType.identifier,
-                                             runtime.identifier},
-                                            &response.commandOutput);
-        response.simUdid = response.success ? response.commandOutput.trimmed()
-                                            : QString();
+        QString stdOutput;
+        response.success
+            = runSimCtlCommand({"create", name, deviceType.identifier, runtime.identifier},
+                               &stdOutput,
+                               &response.commandOutput);
+        response.simUdid = response.success ? stdOutput.trimmed() : QString();
     }
 
     if (!fi.isCanceled())
         fi.reportResult(response);
 }
 
-void SimulatorControlPrivate::takeSceenshot(QFutureInterface<SimulatorControl::ResponseData> &fi,
-                                            const QString &simUdid, const QString &filePath)
+void takeSceenshot(QFutureInterface<SimulatorControl::ResponseData> &fi,
+                   const QString &simUdid,
+                   const QString &filePath)
 {
     SimulatorControl::ResponseData response(simUdid);
     response.success = runSimCtlCommand({"io", simUdid, "screenshot", filePath},
+                                        nullptr,
                                         &response.commandOutput);
     if (!fi.isCanceled())
         fi.reportResult(response);
