@@ -27,13 +27,14 @@
 
 #include "client.h"
 #include "languageclientutils.h"
+#include "snippet.h"
 
 #include <languageserverprotocol/completion.h>
 #include <texteditor/codeassist/assistinterface.h>
-#include <texteditor/codeassist/assistproposalitem.h>
-#include <texteditor/codeassist/iassistprocessor.h>
 #include <texteditor/codeassist/genericproposal.h>
 #include <texteditor/codeassist/genericproposalmodel.h>
+#include <texteditor/snippets/snippet.h>
+#include <texteditor/snippets/snippetassistcollector.h>
 #include <texteditor/texteditorsettings.h>
 #include <utils/algorithm.h>
 #include <utils/textutils.h>
@@ -53,34 +54,6 @@ using namespace TextEditor;
 
 namespace LanguageClient {
 
-class LanguageClientCompletionItem : public AssistProposalItemInterface
-{
-public:
-    LanguageClientCompletionItem(CompletionItem item);
-
-    // AssistProposalItemInterface interface
-    QString text() const override;
-    bool implicitlyApplies() const override;
-    bool prematurelyApplies(const QChar &typedCharacter) const override;
-    void apply(TextDocumentManipulatorInterface &manipulator, int basePosition) const override;
-    QIcon icon() const override;
-    QString detail() const override;
-    bool isSnippet() const override;
-    bool isValid() const override;
-    quint64 hash() const override;
-
-    const QString &sortText() const;
-
-    bool operator <(const LanguageClientCompletionItem &other) const;
-
-    bool isPerfectMatch(int pos, QTextDocument *doc) const;
-
-private:
-    CompletionItem m_item;
-    mutable QChar m_triggeredCommitCharacter;
-    mutable QString m_sortText;
-};
-
 LanguageClientCompletionItem::LanguageClientCompletionItem(CompletionItem item)
     : m_item(std::move(item))
 { }
@@ -103,10 +76,10 @@ bool LanguageClientCompletionItem::prematurelyApplies(const QChar &typedCharacte
 void LanguageClientCompletionItem::apply(TextDocumentManipulatorInterface &manipulator,
                                          int /*basePosition*/) const
 {
-    const int pos = manipulator.currentPosition();
     if (auto edit = m_item.textEdit()) {
-        applyTextEdit(manipulator, *edit);
+        applyTextEdit(manipulator, *edit, isSnippet());
     } else {
+        const int pos = manipulator.currentPosition();
         const QString textToInsert(m_item.insertText().value_or(text()));
         int length = 0;
         for (auto it = textToInsert.crbegin(), end = textToInsert.crend(); it != end; ++it) {
@@ -123,7 +96,12 @@ void LanguageClientCompletionItem::apply(TextDocumentManipulatorInterface &manip
         QRegularExpressionMatch match = identifier.match(blockTextUntilPosition);
         int matchLength = match.hasMatch() ? match.capturedLength(0) : 0;
         length = qMax(length, matchLength);
-        manipulator.replace(pos - length, length, textToInsert);
+        if (isSnippet()) {
+            manipulator.replace(pos - length, length, {});
+            manipulator.insertCodeSnippet(pos - length, textToInsert, &parseSnippet);
+        } else {
+            manipulator.replace(pos - length, length, textToInsert);
+        }
     }
 
     if (auto additionalEdits = m_item.additionalTextEdits()) {
@@ -131,7 +109,9 @@ void LanguageClientCompletionItem::apply(TextDocumentManipulatorInterface &manip
             applyTextEdit(manipulator, edit);
     }
     if (!m_triggeredCommitCharacter.isNull())
-        manipulator.insertCodeSnippet(manipulator.currentPosition(), m_triggeredCommitCharacter);
+        manipulator.insertCodeSnippet(manipulator.currentPosition(),
+                                      m_triggeredCommitCharacter,
+                                      &Snippet::parse);
 }
 
 QIcon LanguageClientCompletionItem::icon() const
@@ -177,14 +157,12 @@ QString LanguageClientCompletionItem::detail() const
 
 bool LanguageClientCompletionItem::isSnippet() const
 {
-    // FIXME add lsp > creator snippet converter
-    // return m_item.insertTextFormat().value_or(CompletionItem::PlainText);
-    return false;
+    return m_item.insertTextFormat() == CompletionItem::Snippet;
 }
 
 bool LanguageClientCompletionItem::isValid() const
 {
-    return m_item.isValid(nullptr);
+    return m_item.isValid();
 }
 
 quint64 LanguageClientCompletionItem::hash() const
@@ -192,6 +170,16 @@ quint64 LanguageClientCompletionItem::hash() const
     return qHash(m_item.label()); // TODO: naaaa
 }
 
+CompletionItem LanguageClientCompletionItem::item() const
+{
+    return m_item;
+}
+
+QChar LanguageClientCompletionItem::triggeredCommitCharacter() const
+{
+    return m_triggeredCommitCharacter;
+}
+
 const QString &LanguageClientCompletionItem::sortText() const
 {
     if (m_sortText.isEmpty())
@@ -199,6 +187,20 @@ const QString &LanguageClientCompletionItem::sortText() const
     return m_sortText;
 }
 
+bool LanguageClientCompletionItem::hasSortText() const
+{
+    return m_item.sortText().has_value();
+}
+
+QString LanguageClientCompletionItem::filterText() const
+{
+    if (m_filterText.isEmpty()) {
+        const Utils::optional<QString> filterText = m_item.filterText();
+        m_filterText = filterText.has_value() ? filterText.value() : m_item.label();
+    }
+    return m_filterText;
+}
+
 bool LanguageClientCompletionItem::operator <(const LanguageClientCompletionItem &other) const
 {
     return sortText() < other.sortText();
@@ -212,6 +214,8 @@ bool LanguageClientCompletionItem::isPerfectMatch(int pos, QTextDocument *doc) c
         if (!additionalEdits.value().isEmpty())
             return false;
     }
+    if (isSnippet())
+        return false;
     if (auto edit = m_item.textEdit()) {
         auto range = edit->range();
         const int start = positionInText(doc, range.start().line() + 1, range.start().character() + 1);
@@ -228,20 +232,35 @@ class LanguageClientCompletionModel : public GenericProposalModel
 {
 public:
     // GenericProposalModel interface
-    bool isSortable(const QString &/*prefix*/) const override { return true; }
+    bool containsDuplicates() const override { return false; }
+    bool isSortable(const QString &/*prefix*/) const override;
     void sort(const QString &/*prefix*/) override;
     bool supportsPrefixExpansion() const override { return false; }
 
-    QList<LanguageClientCompletionItem *> items() const
-    { return Utils::static_container_cast<LanguageClientCompletionItem *>(m_currentItems); }
+    QList<AssistProposalItemInterface *> items() const { return m_currentItems; }
 };
 
+bool LanguageClientCompletionModel::isSortable(const QString &) const
+{
+    return Utils::anyOf(items(), [](AssistProposalItemInterface *i) {
+        auto item = dynamic_cast<LanguageClientCompletionItem *>(i);
+        return !item || item->hasSortText();
+    });
+}
+
 void LanguageClientCompletionModel::sort(const QString &/*prefix*/)
 {
     std::sort(m_currentItems.begin(), m_currentItems.end(),
               [] (AssistProposalItemInterface *a, AssistProposalItemInterface *b){
-        return *(dynamic_cast<LanguageClientCompletionItem *>(a)) < *(
-                    dynamic_cast<LanguageClientCompletionItem *>(b));
+        const auto lca = dynamic_cast<LanguageClientCompletionItem *>(a);
+        const auto lcb = dynamic_cast<LanguageClientCompletionItem *>(b);
+        if (!lca && !lcb)
+            return a->text() < b->text();
+        if (lca && lcb)
+            return *lca < *lcb;
+        if (lca && !lcb)
+            return true;
+        return false;
     });
 }
 
@@ -260,8 +279,10 @@ public:
             return false;
 
         return m_model->keepPerfectMatch(reason)
-                || !Utils::anyOf(m_model->items(), [this](LanguageClientCompletionItem *item){
-            return item->isPerfectMatch(m_pos, m_document);
+                || !Utils::anyOf(m_model->items(), [this](AssistProposalItemInterface *item){
+            if (const auto lcItem = dynamic_cast<LanguageClientCompletionItem *>(item))
+                return lcItem->isPerfectMatch(m_pos, m_document);
+            return false;
         });
     }
 
@@ -270,36 +291,30 @@ public:
     int m_pos = -1;
 };
 
-
-class LanguageClientCompletionAssistProcessor : public IAssistProcessor
-{
-public:
-    LanguageClientCompletionAssistProcessor(Client *client);
-    ~LanguageClientCompletionAssistProcessor() override;
-    IAssistProposal *perform(const AssistInterface *interface) override;
-    bool running() override;
-    bool needsRestart() const override { return true; }
-    void cancel() override;
-
-private:
-    void handleCompletionResponse(const CompletionRequest::Response &response);
-
-    QPointer<QTextDocument> m_document;
-    QPointer<Client> m_client;
-    Utils::optional<MessageId> m_currentRequest;
-    QMetaObject::Connection m_postponedUpdateConnection;
-    int m_pos = -1;
-};
-
-LanguageClientCompletionAssistProcessor::LanguageClientCompletionAssistProcessor(Client *client)
+LanguageClientCompletionAssistProcessor::LanguageClientCompletionAssistProcessor(
+    Client *client,
+    const QString &snippetsGroup)
     : m_client(client)
-{ }
+    , m_snippetsGroup(snippetsGroup)
+{}
 
 LanguageClientCompletionAssistProcessor::~LanguageClientCompletionAssistProcessor()
 {
     QTC_ASSERT(!running(), cancel());
 }
 
+QTextDocument *LanguageClientCompletionAssistProcessor::document() const
+{
+    return m_document;
+}
+
+QList<AssistProposalItemInterface *> LanguageClientCompletionAssistProcessor::generateCompletionItems(
+    const QList<LanguageServerProtocol::CompletionItem> &items) const
+{
+    return Utils::transform<QList<AssistProposalItemInterface *>>(
+        items, [](const CompletionItem &item) { return new LanguageClientCompletionItem(item); });
+}
+
 static QString assistReasonString(AssistReason reason)
 {
     switch (reason) {
@@ -314,14 +329,13 @@ IAssistProposal *LanguageClientCompletionAssistProcessor::perform(const AssistIn
 {
     QTC_ASSERT(m_client, return nullptr);
     m_pos = interface->position();
+    m_basePos = m_pos;
+    auto isIdentifierChar = [](const QChar &c) { return c.isLetterOrNumber() || c == '_'; };
+    while (m_basePos > 0 && isIdentifierChar(interface->characterAt(m_basePos - 1)))
+        --m_basePos;
     if (interface->reason() == IdleEditor) {
         // Trigger an automatic completion request only when we are on a word with at least n "identifier" characters
-        const QRegularExpression regexp("^[_a-zA-Z0-9]+$");
-        auto hasMatch = [&regexp](const QString &txt) { return regexp.match(txt).hasMatch(); };
-        int delta = 0;
-        while (m_pos - delta > 0 && hasMatch(interface->textAt(m_pos - delta - 1, delta + 1)))
-            ++delta;
-        if (delta < TextEditorSettings::completionSettings().m_characterThreshold)
+        if (m_pos - m_basePos < TextEditorSettings::completionSettings().m_characterThreshold)
             return nullptr;
         if (m_client->documentUpdatePostponed(interface->filePath())) {
             m_postponedUpdateConnection
@@ -363,6 +377,7 @@ IAssistProposal *LanguageClientCompletionAssistProcessor::perform(const AssistIn
     m_client->addAssistProcessor(this);
     m_currentRequest = completionRequest.id();
     m_document = interface->textDocument();
+    m_filePath = interface->filePath();
     qCDebug(LOGLSPCOMPLETION) << QTime::currentTime()
                               << " : request completions at " << m_pos
                               << " by " << assistReasonString(interface->reason());
@@ -377,8 +392,10 @@ bool LanguageClientCompletionAssistProcessor::running()
 void LanguageClientCompletionAssistProcessor::cancel()
 {
     if (m_currentRequest.has_value()) {
-        m_client->cancelRequest(m_currentRequest.value());
-        m_client->removeAssistProcessor(this);
+        if (m_client) {
+            m_client->cancelRequest(m_currentRequest.value());
+            m_client->removeAssistProcessor(this);
+        }
         m_currentRequest.reset();
     } else if (m_postponedUpdateConnection) {
         QObject::disconnect(m_postponedUpdateConnection);
@@ -409,11 +426,15 @@ void LanguageClientCompletionAssistProcessor::handleCompletionResponse(
     } else if (Utils::holds_alternative<QList<CompletionItem>>(*result)) {
         items = Utils::get<QList<CompletionItem>>(*result);
     }
+    auto proposalItems = generateCompletionItems(items);
+    if (!proposalItems.isEmpty() && !m_snippetsGroup.isEmpty()) {
+        proposalItems << TextEditor::SnippetAssistCollector(
+                             m_snippetsGroup, QIcon(":/texteditor/images/snippet.png")).collect();
+    }
     auto model = new LanguageClientCompletionModel();
-    model->loadContent(Utils::transform(items, [](const CompletionItem &item){
-        return static_cast<AssistProposalItemInterface *>(new LanguageClientCompletionItem(item));
-    }));
-    LanguageClientCompletionProposal *proposal = new LanguageClientCompletionProposal(m_pos, model);
+    model->loadContent(proposalItems);
+    LanguageClientCompletionProposal *proposal = new LanguageClientCompletionProposal(m_basePos,
+                                                                                      model);
     proposal->m_document = m_document;
     proposal->m_pos = m_pos;
     proposal->setFragile(true);
@@ -429,9 +450,11 @@ LanguageClientCompletionAssistProvider::LanguageClientCompletionAssistProvider(C
     , m_client(client)
 { }
 
-IAssistProcessor *LanguageClientCompletionAssistProvider::createProcessor() const
+IAssistProcessor *LanguageClientCompletionAssistProvider::createProcessor(
+    const AssistInterface *) const
 {
-    return new LanguageClientCompletionAssistProcessor(m_client);
+    return new LanguageClientCompletionAssistProcessor(m_client,
+                                                       m_snippetsGroup);
 }
 
 IAssistProvider::RunType LanguageClientCompletionAssistProvider::runType() const
