@@ -55,14 +55,15 @@ public:
         , m_type(info.kind())
     { }
 
-    LanguageClientOutlineItem(const DocumentSymbol &info)
+    LanguageClientOutlineItem(const DocumentSymbol &info, const SymbolStringifier &stringifier)
         : m_name(info.name())
         , m_detail(info.detail().value_or(QString()))
         , m_range(info.range())
+        , m_symbolStringifier(stringifier)
         , m_type(info.kind())
     {
         for (const DocumentSymbol &child : info.children().value_or(QList<DocumentSymbol>()))
-            appendChild(new LanguageClientOutlineItem(child));
+            appendChild(new LanguageClientOutlineItem(child, stringifier));
     }
 
     // TreeItem interface
@@ -72,12 +73,15 @@ public:
         case Qt::DecorationRole:
             return symbolIcon(m_type);
         case Qt::DisplayRole:
-            return m_name;
+            return m_symbolStringifier
+                    ? m_symbolStringifier(static_cast<SymbolKind>(m_type), m_name, m_detail)
+                    : m_name;
         default:
             return Utils::TreeItem::data(column, role);
         }
     }
 
+    Range range() const { return m_range; }
     Position pos() const { return m_range.start(); }
     bool contains(const Position &pos) const { return m_range.contains(pos); }
 
@@ -85,6 +89,7 @@ private:
     QString m_name;
     QString m_detail;
     Range m_range;
+    SymbolStringifier m_symbolStringifier;
     int m_type = -1;
 };
 
@@ -102,8 +107,16 @@ public:
     {
         clear();
         for (const DocumentSymbol &symbol : info)
-            rootItem()->appendChild(new LanguageClientOutlineItem(symbol));
+            rootItem()->appendChild(new LanguageClientOutlineItem(symbol, m_symbolStringifier));
     }
+
+    void setSymbolStringifier(const SymbolStringifier &stringifier)
+    {
+        m_symbolStringifier = stringifier;
+    }
+
+private:
+    SymbolStringifier m_symbolStringifier;
 };
 
 class LanguageClientOutlineWidget : public TextEditor::IOutlineWidget
@@ -141,18 +154,19 @@ LanguageClientOutlineWidget::LanguageClientOutlineWidget(Client *client,
             &DocumentSymbolCache::gotSymbols,
             this,
             &LanguageClientOutlineWidget::handleResponse);
-    connect(editor->textDocument(), &TextEditor::TextDocument::contentsChanged, this, [this]() {
-        if (m_client)
-            m_client->documentSymbolCache()->requestSymbols(m_uri);
+    connect(client, &Client::documentUpdated, this, [this](TextEditor::TextDocument *document) {
+        if (m_client && m_uri == DocumentUri::fromFilePath(document->filePath()))
+            m_client->documentSymbolCache()->requestSymbols(m_uri, Schedule::Delayed);
     });
 
-    client->documentSymbolCache()->requestSymbols(m_uri);
+    client->documentSymbolCache()->requestSymbols(m_uri, Schedule::Delayed);
 
     auto *layout = new QVBoxLayout;
     layout->setContentsMargins(0, 0, 0, 0);
     layout->setSpacing(0);
     layout->addWidget(Core::ItemViewFind::createSearchableWrapper(&m_view));
     setLayout(layout);
+    m_model.setSymbolStringifier(m_client->symbolStringifier());
     m_view.setModel(&m_model);
     m_view.setHeaderHidden(true);
     m_view.setExpandsOnDoubleClick(false);
@@ -231,10 +245,16 @@ bool LanguageClientOutlineWidgetFactory::clientSupportsDocumentSymbols(
     DynamicCapabilities dc = client->dynamicCapabilities();
     if (dc.isRegistered(DocumentSymbolsRequest::methodName).value_or(false)) {
         TextDocumentRegistrationOptions options(dc.option(DocumentSymbolsRequest::methodName));
-        return !options.isValid(nullptr)
+        return !options.isValid()
                || options.filterApplies(doc->filePath(), Utils::mimeTypeForName(doc->mimeType()));
     }
-    return client->capabilities().documentSymbolProvider().value_or(false);
+    const Utils::optional<Utils::variant<bool, WorkDoneProgressOptions>> &provider
+        = client->capabilities().documentSymbolProvider();
+    if (!provider.has_value())
+        return false;
+    if (Utils::holds_alternative<bool>(*provider))
+        return Utils::get<bool>(*provider);
+    return true;
 }
 
 bool LanguageClientOutlineWidgetFactory::supportsEditor(Core::IEditor *editor) const
@@ -264,7 +284,7 @@ private:
     void updateModel(const DocumentUri &resultUri, const DocumentSymbolsResult &result);
     void updateEntry();
     void activateEntry();
-    void requestSymbols();
+    void documentUpdated(TextEditor::TextDocument *document);
 
     LanguageClientOutlineModel m_model;
     QPointer<Client> m_client;
@@ -289,6 +309,7 @@ OutlineComboBox::OutlineComboBox(Client *client, TextEditor::BaseTextEditor *edi
     , m_editorWidget(editor->editorWidget())
     , m_uri(DocumentUri::fromFilePath(editor->document()->filePath()))
 {
+    m_model.setSymbolStringifier(client->symbolStringifier());
     setModel(&m_model);
     setMinimumContentsLength(13);
     QSizePolicy policy = sizePolicy();
@@ -298,13 +319,12 @@ OutlineComboBox::OutlineComboBox(Client *client, TextEditor::BaseTextEditor *edi
 
     connect(client->documentSymbolCache(), &DocumentSymbolCache::gotSymbols,
             this, &OutlineComboBox::updateModel);
-    connect(editor->textDocument(), &TextEditor::TextDocument::contentsChanged,
-            this, &OutlineComboBox::requestSymbols);
+    connect(client, &Client::documentUpdated, this, &OutlineComboBox::documentUpdated);
     connect(m_editorWidget, &TextEditor::TextEditorWidget::cursorPositionChanged,
             this, &OutlineComboBox::updateEntry);
     connect(this, QOverload<int>::of(&QComboBox::activated), this, &OutlineComboBox::activateEntry);
 
-    requestSymbols();
+    documentUpdated(editor->textDocument());
 }
 
 void OutlineComboBox::updateModel(const DocumentUri &resultUri, const DocumentSymbolsResult &result)
@@ -326,8 +346,14 @@ void OutlineComboBox::updateModel(const DocumentUri &resultUri, const DocumentSy
 void OutlineComboBox::updateEntry()
 {
     const Position pos(m_editorWidget->textCursor());
-    LanguageClientOutlineItem *itemForCursor = m_model.findNonRootItem(
-        [&](const LanguageClientOutlineItem *item) { return item->contains(pos); });
+    LanguageClientOutlineItem *itemForCursor = nullptr;
+    m_model.forAllItems([&](LanguageClientOutlineItem *candidate){
+        if (!candidate->contains(pos))
+            return;
+        if (itemForCursor && candidate->range().contains(itemForCursor->range()))
+            return; // skip item if the range is equal or bigger than the previous found range
+        itemForCursor = candidate;
+    });
     if (itemForCursor)
         setCurrentIndex(m_model.indexForItem(itemForCursor));
 }
@@ -345,10 +371,10 @@ void OutlineComboBox::activateEntry()
     }
 }
 
-void OutlineComboBox::requestSymbols()
+void OutlineComboBox::documentUpdated(TextEditor::TextDocument *document)
 {
-    if (m_client)
-        m_client->documentSymbolCache()->requestSymbols(m_uri);
+    if (document == m_editorWidget->textDocument())
+        m_client->documentSymbolCache()->requestSymbols(m_uri, Schedule::Delayed);
 }
 
 } // namespace LanguageClient
