@@ -39,28 +39,22 @@ using namespace TextEditor;
 
 namespace LanguageClient {
 
-class CodeActionQuickFixOperation : public QuickFixOperation
+CodeActionQuickFixOperation::CodeActionQuickFixOperation(const CodeAction &action, Client *client)
+    : m_action(action)
+    , m_client(client)
 {
-public:
-    CodeActionQuickFixOperation(const CodeAction &action, Client *client)
-        : m_action(action)
-        , m_client(client)
-    { setDescription(action.title()); }
+    setDescription(action.title());
+}
 
-    void perform() override
-    {
-        if (Utils::optional<WorkspaceEdit> edit = m_action.edit()) {
-            applyWorkspaceEdit(*edit);
-        } else if (Utils::optional<Command> command = m_action.command()) {
-            if (m_client)
-                m_client->executeCommand(*command);
-        }
-    }
-
-private:
-    CodeAction m_action;
-    QPointer<Client> m_client;
-};
+void CodeActionQuickFixOperation::perform()
+{
+    if (!m_client)
+        return;
+    if (Utils::optional<WorkspaceEdit> edit = m_action.edit())
+        applyWorkspaceEdit(m_client, *edit);
+    else if (Utils::optional<Command> command = m_action.command())
+        m_client->executeCommand(*command);
+}
 
 class CommandQuickFixOperation : public QuickFixOperation
 {
@@ -115,7 +109,7 @@ IAssistProposal *LanguageClientQuickFixAssistProcessor::perform(const AssistInte
     auto uri = DocumentUri::fromFilePath(interface->filePath());
     params.setTextDocument(TextDocumentIdentifier(uri));
     CodeActionParams::CodeActionContext context;
-    context.setDiagnostics(m_client->diagnosticsAt(uri, range));
+    context.setDiagnostics(m_client->diagnosticsAt(uri, cursor));
     params.setContext(context);
 
     CodeActionRequest request(params);
@@ -171,7 +165,7 @@ IAssistProvider::RunType LanguageClientQuickFixProvider::runType() const
     return Asynchronous;
 }
 
-IAssistProcessor *LanguageClientQuickFixProvider::createProcessor() const
+IAssistProcessor *LanguageClientQuickFixProvider::createProcessor(const AssistInterface *) const
 {
     return new LanguageClientQuickFixAssistProcessor(m_client);
 }
