@@ -41,7 +41,6 @@
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/icore.h>
-#include <coreplugin/id.h>
 #include <coreplugin/icontext.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
@@ -129,7 +128,7 @@ MacroManagerPrivate::MacroManagerPrivate(MacroManager *qq):
 void MacroManagerPrivate::initialize()
 {
     macros.clear();
-    QDir dir(q->macrosDirectory());
+    const QDir dir(MacroManager::macrosDirectory());
     QStringList filter;
     filter << QLatin1String("*.") + QLatin1String(Constants::M_EXTENSION);
     QStringList files = dir.entryList(filter, QDir::Files);
@@ -144,9 +143,9 @@ void MacroManagerPrivate::initialize()
     }
 }
 
-static Core::Id makeId(const QString &name)
+static Utils::Id makeId(const QString &name)
 {
-    return Core::Id(Macros::Constants::PREFIX_MACRO).withSuffix(name);
+    return Utils::Id(Macros::Constants::PREFIX_MACRO).withSuffix(name);
 }
 
 void MacroManagerPrivate::addMacro(Macro *macro)
@@ -187,7 +186,7 @@ void MacroManagerPrivate::changeMacroDescription(Macro *macro, const QString &de
     if (!macro->load())
         return;
     macro->setDescription(description);
-    macro->save(macro->fileName(), Core::ICore::mainWindow());
+    macro->save(macro->fileName(), Core::ICore::dialogParent());
 
     QAction *action = actions[macro->displayName()];
     QTC_ASSERT(action, return);
@@ -210,9 +209,10 @@ bool MacroManagerPrivate::executeMacro(Macro *macro)
     }
 
     if (error) {
-        QMessageBox::warning(Core::ICore::mainWindow(),
-                             MacroManager::tr("Playing Macro"),
-                             MacroManager::tr("An error occurred while replaying the macro, execution stopped."));
+        QMessageBox::warning(
+            Core::ICore::dialogParent(),
+            MacroManager::tr("Playing Macro"),
+            MacroManager::tr("An error occurred while replaying the macro, execution stopped."));
     }
 
     // Set the focus back to the editor
@@ -225,17 +225,16 @@ bool MacroManagerPrivate::executeMacro(Macro *macro)
 
 void MacroManagerPrivate::showSaveDialog()
 {
-    QWidget *mainWindow = Core::ICore::mainWindow();
-    SaveDialog dialog(mainWindow);
+    SaveDialog dialog(Core::ICore::dialogParent());
     if (dialog.exec()) {
         if (dialog.name().isEmpty())
             return;
 
         // Save in the resource path
-        QString fileName = q->macrosDirectory() + QLatin1Char('/') + dialog.name()
-                           + QLatin1Char('.') + QLatin1String(Constants::M_EXTENSION);
+        const QString fileName = MacroManager::macrosDirectory() + QLatin1Char('/') + dialog.name()
+                + QLatin1Char('.') + QLatin1String(Constants::M_EXTENSION);
         currentMacro->setDescription(dialog.description());
-        currentMacro->save(fileName, mainWindow);
+        currentMacro->save(fileName, Core::ICore::dialogParent());
         addMacro(currentMacro);
     }
 }
@@ -381,14 +380,13 @@ void MacroManager::changeMacro(const QString &name, const QString &description)
 
 void MacroManager::saveLastMacro()
 {
-    if (d->currentMacro->events().count())
+    if (!d->currentMacro->events().isEmpty())
         d->showSaveDialog();
 }
 
 QString MacroManager::macrosDirectory()
 {
-    const QString &path =
-        Core::ICore::userResourcePath() + QLatin1String("/macros");
+    const QString path = Core::ICore::userResourcePath("macros").toString();
     if (QFile::exists(path) || QDir().mkpath(path))
         return path;
     return QString();
