@@ -59,6 +59,7 @@ public:
         {Constants::RUNCONFIGURATION}
     };
     McuSupportOptionsPage optionsPage;
+    McuDependenciesKitAspect environmentPathsKitAspect;
 };
 
 static McuSupportPluginPrivate *dd = nullptr;
@@ -83,8 +84,7 @@ bool McuSupportPlugin::initialize(const QStringList& arguments, QString* errorSt
 
     McuSupportOptions::registerQchFiles();
     McuSupportOptions::registerExamples();
-    ProjectExplorer::JsonWizardFactory::addWizardPath(
-                Utils::FilePath::fromString(":/mcusupport/wizards/"));
+    ProjectExplorer::JsonWizardFactory::addWizardPath(":/mcusupport/wizards/");
 
     return true;
 }
@@ -96,6 +96,7 @@ void McuSupportPlugin::extensionsInitialized()
     connect(KitManager::instance(), &KitManager::kitsLoaded, [](){
         McuSupportOptions::removeOutdatedKits();
         McuSupportOptions::createAutomaticKits();
+        McuSupportOptions::fixExistingKits();
         McuSupportPlugin::askUserAboutMcuSupportKitsSetup();
     });
 }
@@ -120,5 +121,33 @@ void McuSupportPlugin::askUserAboutMcuSupportKitsSetup()
     ICore::infoBar()->addInfo(info);
 }
 
+void McuSupportPlugin::askUserAboutMcuSupportKitsUpgrade()
+{
+    const char upgradeMcuSupportKits[] = "UpgradeMcuSupportKits";
+
+    if (!ICore::infoBar()->canInfoBeAdded(upgradeMcuSupportKits))
+        return;
+
+    Utils::InfoBarEntry info(upgradeMcuSupportKits,
+                             tr("New version of Qt for MCUs detected. Upgrade existing Kits?"),
+                             Utils::InfoBarEntry::GlobalSuppression::Enabled);
+
+    static McuSupportOptions::UpgradeOption selectedOption;
+    const QStringList options = { tr("Create new kits"), tr("Replace existing kits") };
+    selectedOption = McuSupportOptions::UpgradeOption::Keep;
+    info.setComboInfo(options, [options](const QString &selected) {
+        selectedOption = options.indexOf(selected) == 0 ?
+                    McuSupportOptions::UpgradeOption::Keep :
+                    McuSupportOptions::UpgradeOption::Replace;
+    });
+
+    info.setCustomButtonInfo(tr("Proceed"), [upgradeMcuSupportKits] {
+        ICore::infoBar()->removeInfo(upgradeMcuSupportKits);
+        QTimer::singleShot(0, []() { McuSupportOptions::upgradeKits(selectedOption); });
+    });
+
+    ICore::infoBar()->addInfo(info);
+}
+
 } // namespace Internal
 } // namespace McuSupport
