@@ -49,8 +49,11 @@ EditorDiagramView::EditorDiagramView(QWidget *parent)
 {
     auto droputils = new Utils::DropSupport(
                 this,
-                [](QDropEvent *event, Utils::DropSupport *dropSupport)
-            -> bool { return dropSupport->isValueDrop(event); });
+                [](QDropEvent *event, Utils::DropSupport *)
+            -> bool { return Utils::DropSupport::isFileDrop(event)
+                                     || Utils::DropSupport::isValueDrop(event); });
+    connect(droputils, &Utils::DropSupport::filesDropped,
+            this, &EditorDiagramView::dropFiles);
     connect(droputils, &Utils::DropSupport::valuesDropped,
             this, &EditorDiagramView::dropProjectExplorerNodes);
 }
@@ -69,25 +72,41 @@ void EditorDiagramView::wheelEvent(QWheelEvent *wheelEvent)
 {
     if (wheelEvent->modifiers() == Qt::ControlModifier) {
         int degree = wheelEvent->angleDelta().y() / 8;
+        QPoint zoomOrigin = wheelEvent->position().toPoint();
         if (degree > 0)
-            emit zoomIn();
+            emit zoomIn(zoomOrigin);
         else if (degree < 0)
-            emit zoomOut();
+            emit zoomOut(zoomOrigin);
     }
 }
 
 void EditorDiagramView::dropProjectExplorerNodes(const QList<QVariant> &values, const QPoint &pos)
 {
-    foreach (const QVariant &value, values) {
+    for (const auto &value : values) {
         if (value.canConvert<ProjectExplorer::Node *>()) {
             auto node = value.value<ProjectExplorer::Node *>();
             QPointF scenePos = mapToScene(pos);
-            d->pxNodeController->addExplorerNode(
-                        node, diagramSceneModel()->findTopmostElement(scenePos),
-                        scenePos, diagramSceneModel()->diagram());
+            auto folderNode = dynamic_cast<ProjectExplorer::FolderNode *>(node);
+            if (folderNode) {
+                d->pxNodeController->addFileSystemEntry(
+                            folderNode->filePath().toString(), -1, -1,
+                            diagramSceneModel()->findTopmostElement(scenePos),
+                            scenePos, diagramSceneModel()->diagram());
+            }
         }
     }
 }
 
+void EditorDiagramView::dropFiles(const QList<Utils::DropSupport::FileSpec> &files, const QPoint &pos)
+{
+    for (const auto &fileSpec : files) {
+        QPointF scenePos = mapToScene(pos);
+        d->pxNodeController->addFileSystemEntry(
+                    fileSpec.filePath.toString(), fileSpec.line, fileSpec.column,
+                    diagramSceneModel()->findTopmostElement(scenePos),
+                    scenePos, diagramSceneModel()->diagram());
+    }
+}
+
 } // namespace Internal
 } // namespace ModelEditor
