@@ -106,6 +106,8 @@
 
 #include <algorithm>
 
+using namespace Utils;
+
 namespace ModelEditor {
 namespace Internal {
 
@@ -222,7 +224,7 @@ void ModelEditor::init()
 
     d->leftToolBox = new QToolBox(d->leftGroup);
     // Windows style does not truncate the tab label to a very small width (GTK+ does)
-    static QStyle *windowsStyle = QStyleFactory().create("Windows");
+    static QStyle *windowsStyle = QStyleFactory::create("Windows");
     if (windowsStyle)
         d->leftToolBox->setStyle(windowsStyle);
     // TODO improve this (and the diagram colors) for use with dark theme
@@ -595,10 +597,10 @@ void ModelEditor::exportToImage(bool selectedElements)
 #ifndef QT_NO_SVG
         filter += tr(";;SVG (*.svg)");
 #endif // QT_NO_SVG
-        QString fileName = QFileDialog::getSaveFileName(
-                    Core::ICore::dialogParent(),
+        QString fileName = FileUtils::getSaveFilePath(
+                    nullptr,
                     selectedElements ? tr("Export Selected Elements") : tr("Export Diagram"),
-                    d->lastExportDirPath, filter);
+                    FilePath::fromString(d->lastExportDirPath), filter).toString();
         if (!fileName.isEmpty()) {
             qmt::DocumentController *documentController = d->document->documentController();
             qmt::DiagramSceneModel *sceneModel = documentController->diagramsManager()->diagramSceneModel(diagram);
