@@ -25,14 +25,17 @@
 
 #include "modeleditor_plugin.h"
 
+#include "jsextension.h"
+#include "modeleditor_constants.h"
 #include "modeleditorfactory.h"
+#include "modeleditor_global.h"
 #include "modelsmanager.h"
 #include "settingscontroller.h"
-#include "modeleditor_constants.h"
 #include "uicontroller.h"
-#include "jsextension.h"
+#include "actionhandler.h"
 
-#include "qmt/infrastructure/uid.h"
+#include <qmt/infrastructure/uid.h>
+#include <qmt/model/mdiagram.h>
 
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
@@ -45,36 +48,31 @@
 
 #include <QAction>
 #include <QApplication>
-#include <QMessageBox>
-#include <QMainWindow>
-#include <QMenu>
 #include <QItemSelection>
 #include <QClipboard>
-#include <QFontDatabase>
-
-#include <QtPlugin>
 
 namespace ModelEditor {
 namespace Internal {
 
 ModelEditorPlugin *pluginInstance = nullptr;
 
-class ModelEditorPlugin::ModelEditorPluginPrivate
+class ModelEditorPluginPrivate final
 {
 public:
-    ModelsManager *modelsManager = nullptr;
-    UiController *uiController = nullptr;
-    ModelEditorFactory *modelFactory = nullptr;
-    SettingsController *settingsController = nullptr;
+    ModelsManager modelsManager;
+    UiController uiController;
+    ActionHandler actionHandler;
+    ModelEditorFactory modelFactory{&uiController, &actionHandler};
+    SettingsController settingsController;
 };
 
 ModelEditorPlugin::ModelEditorPlugin()
-    : ExtensionSystem::IPlugin(),
-      d(new ModelEditorPluginPrivate)
 {
     pluginInstance = this;
     qRegisterMetaType<QItemSelection>("QItemSelection");
     qRegisterMetaType<qmt::Uid>("qmt::Uid");
+    qRegisterMetaType<qmt::MDiagram *>();
+    qRegisterMetaType<const qmt::MDiagram *>();
 }
 
 ModelEditorPlugin::~ModelEditorPlugin()
@@ -86,41 +84,34 @@ bool ModelEditorPlugin::initialize(const QStringList &arguments, QString *errorS
 {
     Q_UNUSED(arguments)
     Q_UNUSED(errorString)
-
-    d->modelsManager = new ModelsManager(this);
-    d->uiController = new UiController(this);
-    d->modelFactory = new ModelEditorFactory(d->uiController, this);
-    d->settingsController = new SettingsController(this);
+    d = new ModelEditorPluginPrivate;
 
     Core::JsExpander::registerGlobalObject<JsExtension>("Modeling");
 
-    connect(d->settingsController, &SettingsController::saveSettings,
-            d->uiController, &UiController::saveSettings);
-    connect(d->settingsController, &SettingsController::loadSettings,
-            d->uiController, &UiController::loadSettings);
+    connect(&d->settingsController, &SettingsController::saveSettings,
+            &d->uiController, &UiController::saveSettings);
+    connect(&d->settingsController, &SettingsController::loadSettings,
+            &d->uiController, &UiController::loadSettings);
 
     return true;
 }
 
 void ModelEditorPlugin::extensionsInitialized()
 {
-    // Retrieve objects from the plugin manager's object pool
-    // In the extensionsInitialized method, a plugin can be sure that all
-    // plugins that depend on it are completely initialized.
-    d->modelFactory->extensionsInitialized();
-    d->settingsController->load(Core::ICore::settings());
+    d->actionHandler.createActions();
+    d->settingsController.load(Core::ICore::settings());
 }
 
 ExtensionSystem::IPlugin::ShutdownFlag ModelEditorPlugin::aboutToShutdown()
 {
-    d->settingsController->save(Core::ICore::settings());
+    d->settingsController.save(Core::ICore::settings());
     QApplication::clipboard()->clear();
     return SynchronousShutdown;
 }
 
 ModelsManager *ModelEditorPlugin::modelsManager()
 {
-    return pluginInstance->d->modelsManager;
+    return &pluginInstance->d->modelsManager;
 }
 
 } // namespace Internal
