@@ -28,7 +28,6 @@
 #include "nimconstants.h"
 #include "nimproject.h"
 #include "nimbleproject.h"
-#include "nimprojectnode.h"
 
 #include <projectexplorer/target.h>
 #include <projectexplorer/toolchain.h>
@@ -51,7 +50,7 @@ const char EXCLUDED_FILES_KEY[] = "ExcludedFiles";
 NimProjectScanner::NimProjectScanner(Project *project)
     : m_project(project)
 {
-    setFilter([this](const Utils::MimeType &, const FilePath &fp) {
+    m_scanner.setFilter([this](const Utils::MimeType &, const FilePath &fp) {
         const QString path = fp.toString();
         return excludedFiles().contains(path)
                 || path.endsWith(".nimproject")
@@ -70,14 +69,16 @@ NimProjectScanner::NimProjectScanner(Project *project)
     connect(&m_scanner, &TreeScanner::finished, this, [this] {
         // Collect scanned nodes
         std::vector<std::unique_ptr<FileNode>> nodes;
-        for (FileNode *node : m_scanner.release()) {
+        const TreeScanner::Result scanResult = m_scanner.release();
+        for (FileNode *node : scanResult.allFiles) {
             if (!node->path().endsWith(".nim") && !node->path().endsWith(".nimble"))
                 node->setEnabled(false); // Disable files that do not end in .nim
             nodes.emplace_back(node);
         }
 
         // Sync watched dirs
-        const QSet<QString> fsDirs = Utils::transform<QSet>(nodes, &FileNode::directory);
+        const QSet<QString> fsDirs = Utils::transform<QSet>(nodes,
+            [](const std::unique_ptr<FileNode> &fn) { return fn->directory().toString(); });
         const QSet<QString> projectDirs = Utils::toSet(m_directoryWatcher.directories());
         m_directoryWatcher.addDirectories(Utils::toList(fsDirs - projectDirs), FileSystemWatcher::WatchAllChanges);
         m_directoryWatcher.removeDirectories(Utils::toList(projectDirs - fsDirs));
@@ -113,11 +114,6 @@ void NimProjectScanner::saveSettings()
     m_project->setNamedSettings(SETTINGS_KEY, settings);
 }
 
-void NimProjectScanner::setFilter(const TreeScanner::FileFilter &filter)
-{
-    m_scanner.setFilter(filter);
-}
-
 void NimProjectScanner::startScan()
 {
     m_scanner.asyncScanForFiles(m_project->projectDirectory());
@@ -209,7 +205,7 @@ FilePath nimblePathFromKit(Kit *kit)
     // There's no extra setting for "nimble", derive it from the "nim" path.
     const QString nimbleFromPath = QStandardPaths::findExecutable("nimble");
     const FilePath nimPath = nimPathFromKit(kit);
-    const FilePath nimbleFromKit = nimPath.pathAppended(HostOsInfo::withExecutableSuffix("nimble"));
+    const FilePath nimbleFromKit = nimPath.pathAppended("nimble").withExecutableSuffix();
     return nimbleFromKit.exists() ? nimbleFromKit.canonicalPath() : FilePath::fromString(nimbleFromPath);
 }
 
@@ -227,26 +223,26 @@ bool NimBuildSystem::supportsAction(Node *context, ProjectAction action, const N
     return BuildSystem::supportsAction(context, action, node);
 }
 
-bool NimBuildSystem::addFiles(Node *, const QStringList &filePaths, QStringList *)
+bool NimBuildSystem::addFiles(Node *, const FilePaths &filePaths, FilePaths *)
 {
-    return m_projectScanner.addFiles(filePaths);
+    return m_projectScanner.addFiles(Utils::transform(filePaths, &FilePath::toString));
 }
 
 RemovedFilesFromProject NimBuildSystem::removeFiles(Node *,
-                                                    const QStringList &filePaths,
-                                                    QStringList *)
+                                                    const FilePaths &filePaths,
+                                                    FilePaths *)
 {
-    return m_projectScanner.removeFiles(filePaths);
+    return m_projectScanner.removeFiles(Utils::transform(filePaths, &FilePath::toString));
 }
 
-bool NimBuildSystem::deleteFiles(Node *, const QStringList &)
+bool NimBuildSystem::deleteFiles(Node *, const FilePaths &)
 {
     return true;
 }
 
-bool NimBuildSystem::renameFile(Node *, const QString &filePath, const QString &newFilePath)
+bool NimBuildSystem::renameFile(Node *, const FilePath &oldFilePath, const FilePath &newFilePath)
 {
-    return m_projectScanner.renameFile(filePath, newFilePath);
+    return m_projectScanner.renameFile(oldFilePath.toString(), newFilePath.toString());
 }
 
 } // namespace Nim
