@@ -29,7 +29,6 @@
 #include <projectexplorer/projectnodes.h>
 
 #include <QElapsedTimer>
-#include <QFutureWatcher>
 #include <QTimer>
 
 namespace Nim {
@@ -48,9 +47,14 @@ public:
     // Keep for compatibility with Qt Creator 4.10
     QVariantMap toMap() const final;
 
+    QStringList excludedFiles() const;
+    void setExcludedFiles(const QStringList &excludedFiles);
+
 protected:
     // Keep for compatibility with Qt Creator 4.10
     RestoreResult fromMap(const QVariantMap &map, QString *errorMessage) final;
+
+    QStringList m_excludedFiles;
 };
 
 } // namespace Nim
