@@ -29,23 +29,7 @@
 
 namespace Nim {
 
-class NimBuildConfiguration;
-
-class NimRunConfiguration : public ProjectExplorer::RunConfiguration
-{
-    Q_OBJECT
-
-public:
-    NimRunConfiguration(ProjectExplorer::Target *target, Core::Id id);
-
-private:
-    void updateConfiguration();
-    void setActiveBuildConfiguration(NimBuildConfiguration *activeBuildConfiguration);
-
-    NimBuildConfiguration *m_buildConfiguration = nullptr;
-};
-
-class NimRunConfigurationFactory : public ProjectExplorer::FixedRunConfigurationFactory
+class NimRunConfigurationFactory final : public ProjectExplorer::FixedRunConfigurationFactory
 {
 public:
     NimRunConfigurationFactory();
