@@ -89,7 +89,7 @@ public:
         if (url.isValid()) {
             args.append(QStringList{"--host", url.host(), "--port", QString::number(url.port())});
         }
-        appendMessage("PerfParser args: " + args.join(' '), Utils::DebugFormat);
+        appendMessage("PerfParser args: " + args.join(' '), Utils::NormalMessageFormat);
         m_reader.createParser(args);
         m_reader.startParser();
     }
@@ -150,13 +150,11 @@ public:
         QStringList arguments;
         arguments << "record";
         arguments += m_perfRecordArguments;
-        arguments << "-o" << "-" << "--" << perfRunnable.executable.toString()
-                  << Utils::QtcProcess::splitArgs(perfRunnable.commandLineArguments,
-                                                  Utils::OsTypeLinux);
+        arguments << "-o" << "-" << "--" << perfRunnable.command.executable().toString()
+                  << ProcessArgs::splitArgs(perfRunnable.command.arguments(), OsTypeLinux);
 
-        perfRunnable.executable = FilePath::fromString("perf");
-        perfRunnable.commandLineArguments = Utils::QtcProcess::joinArgs(arguments,
-                                                                        Utils::OsTypeLinux);
+        perfRunnable.command.setExecutable("perf");
+        perfRunnable.command.setArguments(ProcessArgs::joinArgs(arguments, OsTypeLinux));
         m_process->start(perfRunnable);
     }
 
