@@ -34,18 +34,12 @@
 #include "target.h"
 #include "task.h"
 
-#include <coreplugin/reaper.h>
-
 #include <utils/fileutils.h>
 #include <utils/outputformatter.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
 
-#include <QDir>
-#include <QHash>
-#include <QPair>
 #include <QTextDecoder>
-#include <QUrl>
 
 #include <algorithm>
 #include <memory>
@@ -103,10 +97,10 @@ class AbstractProcessStep::Private
 public:
     Private(AbstractProcessStep *q) : q(q) {}
 
-    void cleanUp(QProcess *process);
+    void cleanUp(int exitCode, QProcess::ExitStatus status);
 
     AbstractProcessStep *q;
-    std::unique_ptr<Utils::QtcProcess> m_process;
+    std::unique_ptr<QtcProcess> m_process;
     ProcessParameters m_param;
     std::function<CommandLine()> m_commandLineProvider;
     std::function<FilePath()> m_workingDirectoryProvider;
@@ -118,7 +112,7 @@ public:
     OutputFormatter *outputFormatter = nullptr;
 };
 
-AbstractProcessStep::AbstractProcessStep(BuildStepList *bsl, Utils::Id id) :
+AbstractProcessStep::AbstractProcessStep(BuildStepList *bsl, Id id) :
     BuildStep(bsl, id),
     d(new Private(this))
 {
@@ -157,7 +151,7 @@ void AbstractProcessStep::setEnvironmentModifier(const std::function<void (Envir
 
 void AbstractProcessStep::setUseEnglishOutput()
 {
-    d->m_environmentModifier = [](Environment &env) { Environment::setupEnglishOutput(&env); };
+    d->m_environmentModifier = [](Environment &env) { env.setupEnglishOutput(); };
 }
 
 void AbstractProcessStep::setCommandLineProvider(const std::function<CommandLine()> &provider)
@@ -199,11 +193,10 @@ void AbstractProcessStep::setupOutputFormatter(OutputFormatter *formatter)
 
 void AbstractProcessStep::doRun()
 {
-    QDir wd(d->m_param.effectiveWorkingDirectory().toString());
+    const FilePath wd = d->m_param.effectiveWorkingDirectory();
     if (!wd.exists()) {
-        if (!wd.mkpath(wd.absolutePath())) {
-            emit addOutput(tr("Could not create directory \"%1\"")
-                           .arg(QDir::toNativeSeparators(wd.absolutePath())),
+        if (!wd.createDir()) {
+            emit addOutput(tr("Could not create directory \"%1\"").arg(wd.toUserOutput()),
                            BuildStep::OutputFormat::ErrorMessage);
             finish(false);
             return;
@@ -213,7 +206,7 @@ void AbstractProcessStep::doRun()
     const CommandLine effectiveCommand(d->m_param.effectiveCommand(),
                                        d->m_param.effectiveArguments(),
                                        CommandLine::Raw);
-    if (!effectiveCommand.executable().exists()) {
+    if (!effectiveCommand.executable().isExecutableFile()) {
         processStartupFailed();
         finish(false);
         return;
@@ -223,24 +216,24 @@ void AbstractProcessStep::doRun()
             ? QTextCodec::codecForName("UTF-8") : QTextCodec::codecForLocale());
     d->stderrStream = std::make_unique<QTextDecoder>(QTextCodec::codecForLocale());
 
-    d->m_process.reset(new Utils::QtcProcess());
-    d->m_process->setUseCtrlCStub(Utils::HostOsInfo::isWindowsHost());
-    d->m_process->setWorkingDirectory(wd.absolutePath());
+    d->m_process.reset(new QtcProcess());
+    d->m_process->setUseCtrlCStub(HostOsInfo::isWindowsHost());
+    d->m_process->setWorkingDirectory(wd);
     // Enforce PWD in the environment because some build tools use that.
     // PWD can be different from getcwd in case of symbolic links (getcwd resolves symlinks).
     // For example Clang uses PWD for paths in debug info, see QTCREATORBUG-23788
     Environment envWithPwd = d->m_param.environment();
-    envWithPwd.set("PWD", d->m_process->workingDirectory());
+    envWithPwd.set("PWD", d->m_process->workingDirectory().path());
     d->m_process->setEnvironment(envWithPwd);
     d->m_process->setCommand(effectiveCommand);
     if (d->m_lowPriority && ProjectExplorerPlugin::projectExplorerSettings().lowBuildPriority)
         d->m_process->setLowPriority();
 
-    connect(d->m_process.get(), &QProcess::readyReadStandardOutput,
+    connect(d->m_process.get(), &QtcProcess::readyReadStandardOutput,
             this, &AbstractProcessStep::processReadyReadStdOutput);
-    connect(d->m_process.get(), &QProcess::readyReadStandardError,
+    connect(d->m_process.get(), &QtcProcess::readyReadStandardError,
             this, &AbstractProcessStep::processReadyReadStdError);
-    connect(d->m_process.get(), QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
+    connect(d->m_process.get(), &QtcProcess::finished,
             this, &AbstractProcessStep::slotProcessFinished);
 
     d->m_process->start();
@@ -260,7 +253,7 @@ void AbstractProcessStep::setLowPriority()
 
 void AbstractProcessStep::doCancel()
 {
-    Core::Reaper::reap(d->m_process.release());
+    d->cleanUp(-1, QProcess::CrashExit);
 }
 
 ProcessParameters *AbstractProcessStep::processParameters()
@@ -268,11 +261,11 @@ ProcessParameters *AbstractProcessStep::processParameters()
     return &d->m_param;
 }
 
-void AbstractProcessStep::setupProcessParameters(ProcessParameters *params)
+void AbstractProcessStep::setupProcessParameters(ProcessParameters *params) const
 {
     params->setMacroExpander(macroExpander());
 
-    Utils::Environment env = buildEnvironment();
+    Environment env = buildEnvironment();
     if (d->m_environmentModifier)
         d->m_environmentModifier(env);
     params->setEnvironment(env);
@@ -286,11 +279,11 @@ void AbstractProcessStep::setupProcessParameters(ProcessParameters *params)
         params->setCommandLine(d->m_commandLineProvider());
 }
 
-void AbstractProcessStep::Private::cleanUp(QProcess *process)
+void AbstractProcessStep::Private::cleanUp(int exitCode, QProcess::ExitStatus status)
 {
     // The process has finished, leftover data is read in processFinished
-    q->processFinished(process->exitCode(), process->exitStatus());
-    const bool returnValue = q->processSucceeded(process->exitCode(), process->exitStatus())
+    q->processFinished(exitCode, status);
+    const bool returnValue = q->processSucceeded(exitCode, status)
                                 || m_ignoreReturnValue;
 
     m_process.reset();
@@ -309,7 +302,7 @@ void AbstractProcessStep::Private::cleanUp(QProcess *process)
 void AbstractProcessStep::processStarted()
 {
     emit addOutput(tr("Starting: \"%1\" %2")
-                   .arg(QDir::toNativeSeparators(d->m_param.effectiveCommand().toString()),
+                   .arg(d->m_param.effectiveCommand().toUserOutput(),
                         d->m_param.prettyArguments()),
                    BuildStep::OutputFormat::NormalMessage);
 }
@@ -322,7 +315,7 @@ void AbstractProcessStep::processStarted()
 
 void AbstractProcessStep::processFinished(int exitCode, QProcess::ExitStatus status)
 {
-    QString command = QDir::toNativeSeparators(d->m_param.effectiveCommand().toString());
+    QString command = d->m_param.effectiveCommand().toUserOutput();
     if (status == QProcess::NormalExit && exitCode == 0) {
         emit addOutput(tr("The process \"%1\" exited normally.").arg(command),
                        BuildStep::OutputFormat::NormalMessage);
@@ -343,10 +336,14 @@ void AbstractProcessStep::processFinished(int exitCode, QProcess::ExitStatus sta
 
 void AbstractProcessStep::processStartupFailed()
 {
-    emit addOutput(tr("Could not start process \"%1\" %2")
-                   .arg(QDir::toNativeSeparators(d->m_param.effectiveCommand().toString()),
+    emit addOutput(tr("Could not start process \"%1\" %2.")
+                   .arg(d->m_param.effectiveCommand().toUserOutput(),
                         d->m_param.prettyArguments()),
-                   BuildStep::OutputFormat::ErrorMessage);
+                   OutputFormat::ErrorMessage);
+
+    QString err = d->m_process ? d->m_process->errorString() : QString();
+    if (!err.isEmpty())
+        emit addOutput(err, OutputFormat::ErrorMessage);
 }
 
 /*!
@@ -363,8 +360,7 @@ bool AbstractProcessStep::processSucceeded(int exitCode, QProcess::ExitStatus st
 
 void AbstractProcessStep::processReadyReadStdOutput()
 {
-    if (!d->m_process)
-        return;
+    QTC_ASSERT(d->m_process.get(), return);
     stdOutput(d->stdoutStream->toUnicode(d->m_process->readAllStandardOutput()));
 }
 
@@ -381,8 +377,7 @@ void AbstractProcessStep::stdOutput(const QString &output)
 
 void AbstractProcessStep::processReadyReadStdError()
 {
-    if (!d->m_process)
-        return;
+    QTC_ASSERT(d->m_process.get(), return);
     stdError(d->stderrStream->toUnicode(d->m_process->readAllStandardError()));
 }
 
@@ -402,16 +397,12 @@ void AbstractProcessStep::finish(bool success)
     emit finished(success);
 }
 
-void AbstractProcessStep::slotProcessFinished(int, QProcess::ExitStatus)
+void AbstractProcessStep::slotProcessFinished()
 {
-    QProcess *process = d->m_process.get();
-    if (!process) // Happens when the process was canceled and handed over to the Reaper.
-        process = qobject_cast<QProcess *>(sender()); // The process was canceled!
-    if (process) {
-        stdError(d->stderrStream->toUnicode(process->readAllStandardError()));
-        stdOutput(d->stdoutStream->toUnicode(process->readAllStandardOutput()));
-    }
-    d->cleanUp(process);
+    QTC_ASSERT(d->m_process.get(), return);
+    stdError(d->stderrStream->toUnicode(d->m_process->readAllStandardError()));
+    stdOutput(d->stdoutStream->toUnicode(d->m_process->readAllStandardOutput()));
+    d->cleanUp(d->m_process->exitCode(), d->m_process->exitStatus());
 }
 
 } // namespace ProjectExplorer
