@@ -100,6 +100,7 @@ public:
     ApplicationLauncher *q;
 
     bool m_isLocal = true;
+    bool m_runAsRoot = false;
 
     // Local
     QtcProcess m_guiProcess;
@@ -130,18 +131,19 @@ ApplicationLauncherPrivate::ApplicationLauncherPrivate(ApplicationLauncher *pare
         m_guiProcess.setProcessChannelMode(QProcess::MergedChannels);
     } else {
         m_guiProcess.setProcessChannelMode(QProcess::SeparateChannels);
-        connect(&m_guiProcess, &QProcess::readyReadStandardError,
+        connect(&m_guiProcess, &QtcProcess::readyReadStandardError,
                 this, &ApplicationLauncherPrivate::readLocalStandardError);
     }
-    connect(&m_guiProcess, &QProcess::readyReadStandardOutput,
+    connect(&m_guiProcess, &QtcProcess::readyReadStandardOutput,
             this, &ApplicationLauncherPrivate::readLocalStandardOutput);
-    connect(&m_guiProcess, &QProcess::errorOccurred,
+    connect(&m_guiProcess, &QtcProcess::errorOccurred,
             this, &ApplicationLauncherPrivate::localGuiProcessError);
-    connect(&m_guiProcess, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
-            this, &ApplicationLauncherPrivate::localProcessDone);
-    connect(&m_guiProcess, &QProcess::started,
+    connect(&m_guiProcess, &QtcProcess::finished, this, [this] {
+        localProcessDone(m_guiProcess.exitCode(), m_guiProcess.exitStatus());
+    });
+    connect(&m_guiProcess, &QtcProcess::started,
             this, &ApplicationLauncherPrivate::handleProcessStarted);
-    connect(&m_guiProcess, &QProcess::errorOccurred,
+    connect(&m_guiProcess, &QtcProcess::errorOccurred,
             q, &ApplicationLauncher::error);
 
     m_consoleProcess.setSettings(Core::ICore::settings());
@@ -150,8 +152,9 @@ ApplicationLauncherPrivate::ApplicationLauncherPrivate(ApplicationLauncher *pare
             this, &ApplicationLauncherPrivate::handleProcessStarted);
     connect(&m_consoleProcess, &ConsoleProcess::processError,
             this, &ApplicationLauncherPrivate::localConsoleProcessError);
-    connect(&m_consoleProcess, &ConsoleProcess::processStopped,
-            this, &ApplicationLauncherPrivate::localProcessDone);
+    connect(&m_consoleProcess, &ConsoleProcess::processStopped, this, [this] {
+        localProcessDone(m_consoleProcess.exitCode(), m_consoleProcess.exitStatus());
+    });
     connect(&m_consoleProcess, &ConsoleProcess::errorOccurred,
             q, &ApplicationLauncher::error);
 
@@ -179,6 +182,11 @@ void ApplicationLauncher::setUseTerminal(bool on)
     d->m_useTerminal = on;
 }
 
+void ApplicationLauncher::setRunAsRoot(bool on)
+{
+    d->m_runAsRoot = on;
+}
+
 void ApplicationLauncher::stop()
 {
     d->stop();
@@ -194,7 +202,7 @@ void ApplicationLauncherPrivate::stop()
             localProcessDone(0, QProcess::CrashExit);
         } else {
             m_guiProcess.terminate();
-            if (!m_guiProcess.waitForFinished(1000) && m_guiProcess.state() == QProcess::Running) { // This is blocking, so be fast.
+            if (!m_guiProcess.waitForFinished(1000)) { // This is blocking, so be fast.
                 m_guiProcess.kill();
                 m_guiProcess.waitForFinished();
             }
@@ -278,13 +286,13 @@ void ApplicationLauncherPrivate::localGuiProcessError()
         error = ApplicationLauncher::tr("Failed to start program. Path or permissions wrong?");
         break;
     case QProcess::Crashed:
-        error = ApplicationLauncher::tr("The program has unexpectedly finished.");
         status = QProcess::CrashExit;
         break;
     default:
         error = ApplicationLauncher::tr("Some error has occurred while running the program.");
     }
-    emit q->appendMessage(error, ErrorMessageFormat);
+    if (!error.isEmpty())
+        emit q->appendMessage(error, ErrorMessageFormat);
     if (m_processRunning && !isRunning()) {
         m_processRunning = false;
         emit q->processExited(-1, status);
@@ -365,11 +373,16 @@ void ApplicationLauncherPrivate::start(const Runnable &runnable, const IDevice::
 
     if (m_isLocal) {
         // Work around QTBUG-17529 (QtDeclarative fails with 'File name case mismatch' ...)
-        const QString fixedPath = FileUtils::normalizePathName(runnable.workingDirectory);
+        const FilePath fixedPath = runnable.workingDirectory.normalizedPathName();
         m_guiProcess.setWorkingDirectory(fixedPath);
         m_consoleProcess.setWorkingDirectory(fixedPath);
-        m_guiProcess.setEnvironment(runnable.environment);
-        m_consoleProcess.setEnvironment(runnable.environment);
+
+        Environment env = runnable.environment;
+        if (m_runAsRoot)
+            RunControl::provideAskPassEntry(env);
+
+        m_guiProcess.setEnvironment(env);
+        m_consoleProcess.setEnvironment(env);
 
         m_processRunning = true;
     #ifdef Q_OS_WIN
@@ -377,13 +390,19 @@ void ApplicationLauncherPrivate::start(const Runnable &runnable, const IDevice::
             WinDebugInterface::instance()->start(); // Try to start listener again...
     #endif
 
-        if (!m_useTerminal) {
-            m_guiProcess.setCommand(runnable.commandLine());
-            m_guiProcess.closeWriteChannel();
-            m_guiProcess.start();
-        } else {
-            m_consoleProcess.setCommand(runnable.commandLine());
+        CommandLine cmdLine = runnable.command;
+        if (m_runAsRoot) {
+            CommandLine wrapped("sudo", {"-A"});
+            wrapped.addCommandLineAsArgs(cmdLine);
+            cmdLine = wrapped;
+        }
+
+        if (m_useTerminal) {
+            m_consoleProcess.setCommand(cmdLine);
             m_consoleProcess.start();
+        } else {
+            m_guiProcess.setCommand(cmdLine);
+            m_guiProcess.start();
         }
     } else {
         QTC_ASSERT(m_state == Inactive, return);
@@ -401,7 +420,7 @@ void ApplicationLauncherPrivate::start(const Runnable &runnable, const IDevice::
             return;
         }
 
-        if (runnable.executable.isEmpty()) {
+        if (!device->isEmptyCommandAllowed() && runnable.command.isEmpty()) {
             doReportError(ApplicationLauncher::tr("Cannot run: No command given."));
             setFinished();
             return;
