@@ -26,7 +26,6 @@
 #include "baseprojectwizarddialog.h"
 
 #include <coreplugin/documentmanager.h>
-#include <utils/fileutils.h>
 #include <utils/projectintropage.h>
 
 #include <QDir>
@@ -41,45 +40,44 @@
     as default projects' folder should the user wish to do that.
 */
 
+using namespace Utils;
+
 namespace ProjectExplorer {
 
-struct BaseProjectWizardDialogPrivate {
-    explicit BaseProjectWizardDialogPrivate(Utils::ProjectIntroPage *page, int id = -1);
+struct BaseProjectWizardDialogPrivate
+{
+    explicit BaseProjectWizardDialogPrivate(ProjectIntroPage *page, int id = -1)
+        : desiredIntroPageId(id), introPage(page)
+    {}
 
     const int desiredIntroPageId;
-    Utils::ProjectIntroPage *introPage;
-    int introPageId;
-    Utils::Id selectedPlatform;
-    QSet<Utils::Id> requiredFeatureSet;
+    ProjectIntroPage *introPage;
+    int introPageId = -1;
+    Id selectedPlatform;
+    QSet<Id> requiredFeatureSet;
 };
 
-BaseProjectWizardDialogPrivate::BaseProjectWizardDialogPrivate(Utils::ProjectIntroPage *page, int id) :
-    desiredIntroPageId(id),
-    introPage(page),
-    introPageId(-1)
-{
-}
 
 BaseProjectWizardDialog::BaseProjectWizardDialog(const Core::BaseFileWizardFactory *factory,
                                                  QWidget *parent,
                                                  const Core::WizardDialogParameters &parameters) :
     Core::BaseFileWizard(factory, parameters.extraValues(), parent),
-    d(std::make_unique<BaseProjectWizardDialogPrivate>(new Utils::ProjectIntroPage))
+    d(std::make_unique<BaseProjectWizardDialogPrivate>(new ProjectIntroPage))
 {
-    setPath(parameters.defaultPath());
+    setFilePath(parameters.defaultPath());
     setSelectedPlatform(parameters.selectedPlatform());
     setRequiredFeatures(parameters.requiredFeatures());
     init();
 }
 
 BaseProjectWizardDialog::BaseProjectWizardDialog(const Core::BaseFileWizardFactory *factory,
-                                                 Utils::ProjectIntroPage *introPage, int introId,
+                                                 ProjectIntroPage *introPage, int introId,
                                                  QWidget *parent,
                                                  const Core::WizardDialogParameters &parameters) :
     Core::BaseFileWizard(factory, parameters.extraValues(), parent),
     d(std::make_unique<BaseProjectWizardDialogPrivate>(introPage, introId))
 {
-    setPath(parameters.defaultPath());
+    setFilePath(parameters.defaultPath());
     setSelectedPlatform(parameters.selectedPlatform());
     setRequiredFeatures(parameters.requiredFeatures());
     init();
@@ -103,9 +101,9 @@ QString BaseProjectWizardDialog::projectName() const
     return d->introPage->projectName();
 }
 
-QString BaseProjectWizardDialog::path() const
+FilePath BaseProjectWizardDialog::filePath() const
 {
-    return d->introPage->path();
+    return d->introPage->filePath();
 }
 
 void BaseProjectWizardDialog::setIntroDescription(const QString &des)
@@ -113,9 +111,9 @@ void BaseProjectWizardDialog::setIntroDescription(const QString &des)
     d->introPage->setDescription(des);
 }
 
-void BaseProjectWizardDialog::setPath(const QString &path)
+void BaseProjectWizardDialog::setFilePath(const FilePath &path)
 {
-    d->introPage->setPath(path);
+    d->introPage->setFilePath(path);
 }
 
 void BaseProjectWizardDialog::setProjectName(const QString &name)
@@ -128,7 +126,7 @@ void BaseProjectWizardDialog::setProjectList(const QStringList &projectList)
     d->introPage->setProjectList(projectList);
 }
 
-void BaseProjectWizardDialog::setProjectDirectories(const QStringList &directories)
+void BaseProjectWizardDialog::setProjectDirectories(const FilePaths &directories)
 {
     d->introPage->setProjectDirectories(directories);
 }
@@ -142,7 +140,7 @@ void BaseProjectWizardDialog::slotAccepted()
 {
     if (d->introPage->useAsDefaultPath()) {
         // Store the path as default path for new projects if desired.
-        Core::DocumentManager::setProjectsDirectory(Utils::FilePath::fromString(path()));
+        Core::DocumentManager::setProjectsDirectory(filePath());
         Core::DocumentManager::setUseProjectsDirectory(true);
     }
 }
@@ -150,18 +148,18 @@ void BaseProjectWizardDialog::slotAccepted()
 bool BaseProjectWizardDialog::validateCurrentPage()
 {
     if (currentId() == d->introPageId)
-        emit projectParametersChanged(d->introPage->projectName(), d->introPage->path());
+        emit projectParametersChanged(d->introPage->projectName(), d->introPage->filePath().toString());
     return Core::BaseFileWizard::validateCurrentPage();
 }
 
-Utils::ProjectIntroPage *BaseProjectWizardDialog::introPage() const
+ProjectIntroPage *BaseProjectWizardDialog::introPage() const
 {
     return d->introPage;
 }
 
-QString BaseProjectWizardDialog::uniqueProjectName(const QString &path)
+QString BaseProjectWizardDialog::uniqueProjectName(const FilePath &path)
 {
-    const QDir pathDir(path);
+    const QDir pathDir(path.toString());
     //: File path suggestion for a new project. If you choose
     //: to translate it, make sure it is a valid path name without blanks
     //: and using only ascii chars.
@@ -178,26 +176,26 @@ QString BaseProjectWizardDialog::uniqueProjectName(const QString &path)
 
 void BaseProjectWizardDialog::addExtensionPages(const QList<QWizardPage *> &wizardPageList)
 {
-    foreach (QWizardPage *p,wizardPageList)
+    for (QWizardPage *p : wizardPageList)
         addPage(p);
 }
 
-Utils::Id BaseProjectWizardDialog::selectedPlatform() const
+Id BaseProjectWizardDialog::selectedPlatform() const
 {
     return d->selectedPlatform;
 }
 
-void BaseProjectWizardDialog::setSelectedPlatform(Utils::Id platform)
+void BaseProjectWizardDialog::setSelectedPlatform(Id platform)
 {
     d->selectedPlatform = platform;
 }
 
-QSet<Utils::Id> BaseProjectWizardDialog::requiredFeatures() const
+QSet<Id> BaseProjectWizardDialog::requiredFeatures() const
 {
     return d->requiredFeatureSet;
 }
 
-void BaseProjectWizardDialog::setRequiredFeatures(const QSet<Utils::Id> &featureSet)
+void BaseProjectWizardDialog::setRequiredFeatures(const QSet<Id> &featureSet)
 {
     d->requiredFeatureSet = featureSet;
 }
