@@ -40,22 +40,18 @@ CodeStyleSettingsWidget::CodeStyleSettingsWidget(Project *project) : QWidget(),
 
     const EditorConfiguration *config = m_project->editorConfiguration();
 
-    QMap<Core::Id, ICodeStylePreferencesFactory *> factories
-            = TextEditorSettings::codeStyleFactories();
-    QMapIterator<Core::Id, ICodeStylePreferencesFactory *> it(factories);
-    while (it.hasNext()) {
-        it.next();
-        ICodeStylePreferencesFactory *factory = it.value();
-        Core::Id languageId = factory->languageId();
+    for (ICodeStylePreferencesFactory *factory : TextEditorSettings::codeStyleFactories()) {
+        Utils::Id languageId = factory->languageId();
         ICodeStylePreferences *codeStylePreferences = config->codeStyle(languageId);
 
-        auto preview = new CodeStyleEditor(factory, codeStylePreferences, m_ui.stackedWidget);
-        preview->clearMargins();
+        auto preview = factory->createCodeStyleEditor(codeStylePreferences, m_ui.stackedWidget);
+        if (preview && preview->layout())
+            preview->layout()->setContentsMargins(QMargins());
         m_ui.stackedWidget->addWidget(preview);
         m_ui.languageComboBox->addItem(factory->displayName());
     }
 
-    connect(m_ui.languageComboBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+    connect(m_ui.languageComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
             m_ui.stackedWidget, &QStackedWidget::setCurrentIndex);
 }
 
