@@ -37,20 +37,20 @@
 using namespace ProjectExplorer;
 using namespace ProjectExplorer::Internal;
 
-ConfigTaskHandler::ConfigTaskHandler(const Task &pattern, Core::Id page) :
+ConfigTaskHandler::ConfigTaskHandler(const Task &pattern, Utils::Id page) :
     m_pattern(pattern),
     m_targetPage(page)
 { }
 
 bool ConfigTaskHandler::canHandle(const Task &task) const
 {
-    return task.description == m_pattern.description
+    return task.description() == m_pattern.description()
             && task.category == m_pattern.category;
 }
 
 void ConfigTaskHandler::handle(const Task &task)
 {
-    Q_UNUSED(task);
+    Q_UNUSED(task)
     Core::ICore::showOptionsDialog(m_targetPage);
 }
 
