@@ -54,12 +54,12 @@ void CopyTaskHandler::handle(const Task &task)
 
     QApplication::clipboard()->setText(task.file.toUserOutput() + QLatin1Char(':') +
                                        QString::number(task.line) + QLatin1String(": ")
-                                       + type + task.description);
+                                       + type + task.description());
 }
 
-Core::Id CopyTaskHandler::actionManagerId() const
+Utils::Id CopyTaskHandler::actionManagerId() const
 {
-    return Core::Id(Core::Constants::COPY);
+    return Utils::Id(Core::Constants::COPY);
 }
 
 QAction *CopyTaskHandler::createAction(QObject *parent) const
