@@ -30,24 +30,24 @@
 #include "linuxiccparser.h"
 #include "msvcparser.h"
 #include "customparser.h"
-#include "customparserconfigdialog.h"
+#include "projectexplorer.h"
 #include "projectexplorerconstants.h"
 #include "projectmacro.h"
 #include "toolchainmanager.h"
 
 #include <utils/algorithm.h>
-#include <utils/asconst.h>
 #include <utils/detailswidget.h>
 #include <utils/environment.h>
 #include <utils/pathchooser.h>
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 
-#include <QFormLayout>
-#include <QPlainTextEdit>
-#include <QLineEdit>
-#include <QHBoxLayout>
 #include <QComboBox>
-#include <QPushButton>
+#include <QFormLayout>
+#include <QHBoxLayout>
+#include <QLineEdit>
+#include <QPlainTextEdit>
+#include <QUuid>
 
 using namespace Utils;
 
@@ -57,59 +57,32 @@ namespace ProjectExplorer {
 // Helpers:
 // --------------------------------------------------------------------------
 
-static const char compilerCommandKeyC[] = "ProjectExplorer.CustomToolChain.CompilerPath";
 static const char makeCommandKeyC[] = "ProjectExplorer.CustomToolChain.MakePath";
-static const char targetAbiKeyC[] = "ProjectExplorer.CustomToolChain.TargetAbi";
 static const char predefinedMacrosKeyC[] = "ProjectExplorer.CustomToolChain.PredefinedMacros";
 static const char headerPathsKeyC[] = "ProjectExplorer.CustomToolChain.HeaderPaths";
 static const char cxx11FlagsKeyC[] = "ProjectExplorer.CustomToolChain.Cxx11Flags";
 static const char mkspecsKeyC[] = "ProjectExplorer.CustomToolChain.Mkspecs";
 static const char outputParserKeyC[] = "ProjectExplorer.CustomToolChain.OutputParser";
-static const char errorPatternKeyC[] = "ProjectExplorer.CustomToolChain.ErrorPattern";
-static const char errorLineNumberCapKeyC[] = "ProjectExplorer.CustomToolChain.ErrorLineNumberCap";
-static const char errorFileNameCapKeyC[] = "ProjectExplorer.CustomToolChain.ErrorFileNameCap";
-static const char errorMessageCapKeyC[] = "ProjectExplorer.CustomToolChain.ErrorMessageCap";
-static const char errorChannelKeyC[] = "ProjectExplorer.CustomToolChain.ErrorChannel";
-static const char errorExampleKeyC[] = "ProjectExplorer.CustomToolChain.ErrorExample";
-static const char warningPatternKeyC[] = "ProjectExplorer.CustomToolChain.WarningPattern";
-static const char warningLineNumberCapKeyC[] = "ProjectExplorer.CustomToolChain.WarningLineNumberCap";
-static const char warningFileNameCapKeyC[] = "ProjectExplorer.CustomToolChain.WarningFileNameCap";
-static const char warningMessageCapKeyC[] = "ProjectExplorer.CustomToolChain.WarningMessageCap";
-static const char warningChannelKeyC[] = "ProjectExplorer.CustomToolChain.WarningChannel";
-static const char warningExampleKeyC[] = "ProjectExplorer.CustomToolChain.WarningExample";
 
 // --------------------------------------------------------------------------
 // CustomToolChain
 // --------------------------------------------------------------------------
 
-CustomToolChain::CustomToolChain(Detection d) :
-    ToolChain(Constants::CUSTOM_TOOLCHAIN_TYPEID, d),
+CustomToolChain::CustomToolChain() :
+    ToolChain(Constants::CUSTOM_TOOLCHAIN_TYPEID),
     m_outputParserId(GccParser::id())
-{ }
-
-CustomToolChain::CustomToolChain(Core::Id language, Detection d) : CustomToolChain(d)
 {
-    setLanguage(language);
+    setTypeDisplayName(tr("Custom"));
+    setTargetAbiKey("ProjectExplorer.CustomToolChain.TargetAbi");
+    setCompilerCommandKey("ProjectExplorer.CustomToolChain.CompilerPath");
 }
 
-
-QString CustomToolChain::typeDisplayName() const
+CustomParserSettings CustomToolChain::customParserSettings() const
 {
-    return Internal::CustomToolChainFactory::tr("Custom");
-}
-
-Abi CustomToolChain::targetAbi() const
-{
-    return m_targetAbi;
-}
-
-void CustomToolChain::setTargetAbi(const Abi &abi)
-{
-    if (abi == m_targetAbi)
-        return;
-
-    m_targetAbi = abi;
-    toolChainUpdated();
+    return findOrDefault(ProjectExplorerPlugin::customParsers(),
+                         [this](const CustomParserSettings &s) {
+        return s.id == outputParserId();
+    });
 }
 
 bool CustomToolChain::isValid() const
@@ -117,13 +90,13 @@ bool CustomToolChain::isValid() const
     return true;
 }
 
-ToolChain::PredefinedMacrosRunner CustomToolChain::createPredefinedMacrosRunner() const
+ToolChain::MacroInspectionRunner CustomToolChain::createMacroInspectionRunner() const
 {
     const Macros theMacros = m_predefinedMacros;
+    const Utils::Id lang = language();
 
     // This runner must be thread-safe!
-    return [theMacros](const QStringList &cxxflags){
-        QByteArray result;
+    return [theMacros, lang](const QStringList &cxxflags){
         Macros macros = theMacros;
         for (const QString &cxxFlag : cxxflags) {
             if (cxxFlag.startsWith(QLatin1String("-D")))
@@ -132,26 +105,18 @@ ToolChain::PredefinedMacrosRunner CustomToolChain::createPredefinedMacrosRunner(
                 macros.append({cxxFlag.mid(2).trimmed().toUtf8(), MacroType::Undefine});
 
         }
-        return macros;
+        return MacroInspectionReport{macros, ToolChain::languageVersion(lang, macros)};
     };
 }
 
-Macros CustomToolChain::predefinedMacros(const QStringList &cxxflags) const
+Utils::LanguageExtensions CustomToolChain::languageExtensions(const QStringList &) const
 {
-    return createPredefinedMacrosRunner()(cxxflags);
-}
-
-ToolChain::CompilerFlags CustomToolChain::compilerFlags(const QStringList &cxxflags) const
-{
-    for (const QString &cxx11Flag : m_cxx11Flags)
-        if (cxxflags.contains(cxx11Flag))
-            return StandardCxx11;
-    return NoFlags;
+    return LanguageExtension::None;
 }
 
 WarningFlags CustomToolChain::warningFlags(const QStringList &cxxflags) const
 {
-    Q_UNUSED(cxxflags);
+    Q_UNUSED(cxxflags)
     return WarningFlags::Default;
 }
 
@@ -168,90 +133,71 @@ void CustomToolChain::setPredefinedMacros(const Macros &macros)
     toolChainUpdated();
 }
 
-ToolChain::SystemHeaderPathsRunner CustomToolChain::createSystemHeaderPathsRunner() const
+ToolChain::BuiltInHeaderPathsRunner CustomToolChain::createBuiltInHeaderPathsRunner(
+        const Environment &) const
 {
-    const QList<HeaderPath> systemHeaderPaths = m_systemHeaderPaths;
+    const HeaderPaths builtInHeaderPaths = m_builtInHeaderPaths;
 
     // This runner must be thread-safe!
-    return [systemHeaderPaths](const QStringList &cxxFlags, const QString &) {
-        QList<HeaderPath> flagHeaderPaths;
+    return [builtInHeaderPaths](const QStringList &cxxFlags, const QString &, const QString &) {
+        HeaderPaths flagHeaderPaths;
         for (const QString &cxxFlag : cxxFlags) {
             if (cxxFlag.startsWith(QLatin1String("-I")))
-                flagHeaderPaths << HeaderPath(cxxFlag.mid(2).trimmed(), HeaderPath::GlobalHeaderPath);
+                flagHeaderPaths.push_back(HeaderPath::makeBuiltIn(cxxFlag.mid(2).trimmed()));
         }
 
-        return systemHeaderPaths + flagHeaderPaths;
+        return builtInHeaderPaths + flagHeaderPaths;
     };
 }
 
-QList<HeaderPath> CustomToolChain::systemHeaderPaths(const QStringList &cxxFlags,
-                                                     const FileName &fileName) const
-{
-    return createSystemHeaderPathsRunner()(cxxFlags, fileName.toString());
-}
-
 void CustomToolChain::addToEnvironment(Environment &env) const
 {
     if (!m_compilerCommand.isEmpty()) {
-        FileName path = m_compilerCommand.parentDir();
+        FilePath path = m_compilerCommand.parentDir();
         env.prependOrSetPath(path.toString());
-        FileName makePath = m_makeCommand.parentDir();
+        FilePath makePath = m_makeCommand.parentDir();
         if (makePath != path)
             env.prependOrSetPath(makePath.toString());
     }
 }
 
-FileNameList CustomToolChain::suggestedMkspecList() const
+QStringList CustomToolChain::suggestedMkspecList() const
 {
     return m_mkspecs;
 }
 
-IOutputParser *CustomToolChain::outputParser() const
+QList<Utils::OutputLineParser *> CustomToolChain::createOutputParsers() const
 {
     if (m_outputParserId == GccParser::id())
-        return new GccParser;
+        return GccParser::gccParserSuite();
     if (m_outputParserId == ClangParser::id())
-        return new ClangParser;
+        return ClangParser::clangParserSuite();
     if (m_outputParserId == LinuxIccParser::id())
-        return new LinuxIccParser;
+        return LinuxIccParser::iccParserSuite();
     if (m_outputParserId == MsvcParser::id())
-        return new MsvcParser;
-    if (m_outputParserId == CustomParser::id())
-        return new CustomParser(m_customParserSettings);
-    return nullptr;
+        return {new MsvcParser};
+    return {new Internal::CustomParser(customParserSettings())};
+    return {};
 }
 
 QStringList CustomToolChain::headerPathsList() const
 {
-    return Utils::transform(m_systemHeaderPaths, &HeaderPath::path);
+    return Utils::transform<QList>(m_builtInHeaderPaths, &HeaderPath::path);
 }
 
 void CustomToolChain::setHeaderPaths(const QStringList &list)
 {
-    QList<HeaderPath> tmp = Utils::transform(list, [](const QString &headerPath) {
-        return HeaderPath(headerPath.trimmed(), HeaderPath::GlobalHeaderPath);
+    HeaderPaths tmp = Utils::transform<QVector>(list, [](const QString &headerPath) {
+        return HeaderPath::makeBuiltIn(headerPath.trimmed());
     });
 
-    if (m_systemHeaderPaths == tmp)
+    if (m_builtInHeaderPaths == tmp)
         return;
-    m_systemHeaderPaths = tmp;
+    m_builtInHeaderPaths = tmp;
     toolChainUpdated();
 }
 
-void CustomToolChain::setCompilerCommand(const FileName &path)
-{
-    if (path == m_compilerCommand)
-        return;
-    m_compilerCommand = path;
-    toolChainUpdated();
-}
-
-FileName CustomToolChain::compilerCommand() const
-{
-    return m_compilerCommand;
-}
-
-void CustomToolChain::setMakeCommand(const FileName &path)
+void CustomToolChain::setMakeCommand(const FilePath &path)
 {
     if (path == m_makeCommand)
         return;
@@ -259,9 +205,9 @@ void CustomToolChain::setMakeCommand(const FileName &path)
     toolChainUpdated();
 }
 
-QString CustomToolChain::makeCommand(const Environment &) const
+FilePath CustomToolChain::makeCommand(const Environment &) const
 {
-    return m_makeCommand.toString();
+    return m_makeCommand;
 }
 
 void CustomToolChain::setCxx11Flags(const QStringList &flags)
@@ -279,10 +225,7 @@ const QStringList &CustomToolChain::cxx11Flags() const
 
 void CustomToolChain::setMkspecs(const QString &specs)
 {
-    Utils::FileNameList tmp
-            = Utils::transform(specs.split(QLatin1Char(',')),
-                               [](QString fn) { return FileName::fromString(fn); });
-
+    const QStringList tmp = specs.split(',');
     if (tmp == m_mkspecs)
         return;
     m_mkspecs = tmp;
@@ -291,42 +234,19 @@ void CustomToolChain::setMkspecs(const QString &specs)
 
 QString CustomToolChain::mkspecs() const
 {
-    QString list;
-    for (const FileName &spec : m_mkspecs)
-        list.append(spec.toString() + QLatin1Char(','));
-    list.chop(1);
-    return list;
-}
-
-ToolChain *CustomToolChain::clone() const
-{
-    return new CustomToolChain(*this);
+    return m_mkspecs.join(',');
 }
 
 QVariantMap CustomToolChain::toMap() const
 {
     QVariantMap data = ToolChain::toMap();
-    data.insert(QLatin1String(compilerCommandKeyC), m_compilerCommand.toString());
     data.insert(QLatin1String(makeCommandKeyC), m_makeCommand.toString());
-    data.insert(QLatin1String(targetAbiKeyC), m_targetAbi.toString());
     QStringList macros = Utils::transform<QList>(m_predefinedMacros, [](const Macro &m) { return QString::fromUtf8(m.toByteArray()); });
     data.insert(QLatin1String(predefinedMacrosKeyC), macros);
     data.insert(QLatin1String(headerPathsKeyC), headerPathsList());
     data.insert(QLatin1String(cxx11FlagsKeyC), m_cxx11Flags);
     data.insert(QLatin1String(mkspecsKeyC), mkspecs());
     data.insert(QLatin1String(outputParserKeyC), m_outputParserId.toSetting());
-    data.insert(QLatin1String(errorPatternKeyC), m_customParserSettings.error.pattern());
-    data.insert(QLatin1String(errorFileNameCapKeyC), m_customParserSettings.error.fileNameCap());
-    data.insert(QLatin1String(errorLineNumberCapKeyC), m_customParserSettings.error.lineNumberCap());
-    data.insert(QLatin1String(errorMessageCapKeyC), m_customParserSettings.error.messageCap());
-    data.insert(QLatin1String(errorChannelKeyC), m_customParserSettings.error.channel());
-    data.insert(QLatin1String(errorExampleKeyC), m_customParserSettings.error.example());
-    data.insert(QLatin1String(warningPatternKeyC), m_customParserSettings.warning.pattern());
-    data.insert(QLatin1String(warningFileNameCapKeyC), m_customParserSettings.warning.fileNameCap());
-    data.insert(QLatin1String(warningLineNumberCapKeyC), m_customParserSettings.warning.lineNumberCap());
-    data.insert(QLatin1String(warningMessageCapKeyC), m_customParserSettings.warning.messageCap());
-    data.insert(QLatin1String(warningChannelKeyC), m_customParserSettings.warning.channel());
-    data.insert(QLatin1String(warningExampleKeyC), m_customParserSettings.warning.example());
 
     return data;
 }
@@ -336,29 +256,54 @@ bool CustomToolChain::fromMap(const QVariantMap &data)
     if (!ToolChain::fromMap(data))
         return false;
 
-    m_compilerCommand = FileName::fromString(data.value(QLatin1String(compilerCommandKeyC)).toString());
-    m_makeCommand = FileName::fromString(data.value(QLatin1String(makeCommandKeyC)).toString());
-    m_targetAbi = Abi(data.value(QLatin1String(targetAbiKeyC)).toString());
+    m_makeCommand = FilePath::fromString(data.value(QLatin1String(makeCommandKeyC)).toString());
     const QStringList macros = data.value(QLatin1String(predefinedMacrosKeyC)).toStringList();
     m_predefinedMacros = Macro::toMacros(macros.join('\n').toUtf8());
     setHeaderPaths(data.value(QLatin1String(headerPathsKeyC)).toStringList());
     m_cxx11Flags = data.value(QLatin1String(cxx11FlagsKeyC)).toStringList();
     setMkspecs(data.value(QLatin1String(mkspecsKeyC)).toString());
-    setOutputParserId(Core::Id::fromSetting(data.value(QLatin1String(outputParserKeyC))));
-    m_customParserSettings.error.setPattern(data.value(QLatin1String(errorPatternKeyC)).toString());
-    m_customParserSettings.error.setFileNameCap(data.value(QLatin1String(errorFileNameCapKeyC)).toInt());
-    m_customParserSettings.error.setLineNumberCap(data.value(QLatin1String(errorLineNumberCapKeyC)).toInt());
-    m_customParserSettings.error.setMessageCap(data.value(QLatin1String(errorMessageCapKeyC)).toInt());
-    m_customParserSettings.error.setChannel(
-                static_cast<CustomParserExpression::CustomParserChannel>(data.value(QLatin1String(errorChannelKeyC)).toInt()));
-    m_customParserSettings.error.setExample(data.value(QLatin1String(errorExampleKeyC)).toString());
-    m_customParserSettings.warning.setPattern(data.value(QLatin1String(warningPatternKeyC)).toString());
-    m_customParserSettings.warning.setFileNameCap(data.value(QLatin1String(warningFileNameCapKeyC)).toInt());
-    m_customParserSettings.warning.setLineNumberCap(data.value(QLatin1String(warningLineNumberCapKeyC)).toInt());
-    m_customParserSettings.warning.setMessageCap(data.value(QLatin1String(warningMessageCapKeyC)).toInt());
-    m_customParserSettings.warning.setChannel(
-                static_cast<CustomParserExpression::CustomParserChannel>(data.value(QLatin1String(warningChannelKeyC)).toInt()));
-    m_customParserSettings.warning.setExample(data.value(QLatin1String(warningExampleKeyC)).toString());
+    setOutputParserId(Utils::Id::fromSetting(data.value(QLatin1String(outputParserKeyC))));
+
+    // Restore Pre-4.13 settings.
+    if (outputParserId() == Internal::CustomParser::id()) {
+        CustomParserSettings customParserSettings;
+        customParserSettings.error.setPattern(
+                    data.value("ProjectExplorer.CustomToolChain.ErrorPattern").toString());
+        customParserSettings.error.setFileNameCap(
+                    data.value("ProjectExplorer.CustomToolChain.ErrorLineNumberCap").toInt());
+        customParserSettings.error.setLineNumberCap(
+                    data.value("ProjectExplorer.CustomToolChain.ErrorFileNameCap").toInt());
+        customParserSettings.error.setMessageCap(
+                    data.value("ProjectExplorer.CustomToolChain.ErrorMessageCap").toInt());
+        customParserSettings.error.setChannel(
+                    static_cast<CustomParserExpression::CustomParserChannel>(
+                        data.value("ProjectExplorer.CustomToolChain.ErrorChannel").toInt()));
+        customParserSettings.error.setExample(
+                    data.value("ProjectExplorer.CustomToolChain.ErrorExample").toString());
+        customParserSettings.warning.setPattern(
+                    data.value("ProjectExplorer.CustomToolChain.WarningPattern").toString());
+        customParserSettings.warning.setFileNameCap(
+                    data.value("ProjectExplorer.CustomToolChain.WarningLineNumberCap").toInt());
+        customParserSettings.warning.setLineNumberCap(
+                    data.value("ProjectExplorer.CustomToolChain.WarningFileNameCap").toInt());
+        customParserSettings.warning.setMessageCap(
+                    data.value("ProjectExplorer.CustomToolChain.WarningMessageCap").toInt());
+        customParserSettings.warning.setChannel(
+                    static_cast<CustomParserExpression::CustomParserChannel>(
+                        data.value("ProjectExplorer.CustomToolChain.WarningChannel").toInt()));
+        customParserSettings.warning.setExample(
+                    data.value("ProjectExplorer.CustomToolChain.WarningExample").toString());
+        if (!customParserSettings.error.pattern().isEmpty()
+                || !customParserSettings.error.pattern().isEmpty()) {
+            // Found custom parser in old settings, move to new place.
+            customParserSettings.id = Utils::Id::fromString(QUuid::createUuid().toString());
+            setOutputParserId(customParserSettings.id);
+            customParserSettings.displayName = tr("Parser for toolchain %1").arg(displayName());
+            QList<CustomParserSettings> settings = ProjectExplorerPlugin::customParsers();
+            settings << customParserSettings;
+            ProjectExplorerPlugin::setCustomParsers(settings);
+        }
+    }
 
     return true;
 }
@@ -371,72 +316,24 @@ bool CustomToolChain::operator ==(const ToolChain &other) const
     auto customTc = static_cast<const CustomToolChain *>(&other);
     return m_compilerCommand == customTc->m_compilerCommand
             && m_makeCommand == customTc->m_makeCommand
-            && m_targetAbi == customTc->m_targetAbi
+            && targetAbi() == customTc->targetAbi()
             && m_predefinedMacros == customTc->m_predefinedMacros
-            && m_systemHeaderPaths == customTc->m_systemHeaderPaths;
+            && m_builtInHeaderPaths == customTc->m_builtInHeaderPaths;
 }
 
-Core::Id CustomToolChain::outputParserId() const
+Utils::Id CustomToolChain::outputParserId() const
 {
     return m_outputParserId;
 }
 
-static Core::Id convertLegacySettings(Core::Id parserId)
+void CustomToolChain::setOutputParserId(Utils::Id parserId)
 {
-    enum OutputParser
-    {
-        Gcc = 0,
-        Clang = 1,
-        LinuxIcc = 2,
-        Msvc = 3,
-        Custom = 4,
-        OutputParserCount
-    };
-
-    bool ok;
-    const OutputParser index = static_cast<OutputParser>(parserId.toString().toInt(&ok));
-    if (!ok)
-        return parserId;
-
-    switch (index) {
-    case Gcc:
-        return GccParser::id();
-    case Clang:
-        return ClangParser::id();
-    case LinuxIcc:
-        return LinuxIccParser::id();
-    case Msvc:
-        return HostOsInfo::isWindowsHost() ? MsvcParser::id() : CustomParser::id();
-    case Custom:
-        return CustomParser::id();
-    default:
-        return parserId;
-    }
-}
-
-void CustomToolChain::setOutputParserId(Core::Id parserId)
-{
-    parserId = convertLegacySettings(parserId);
-
     if (m_outputParserId == parserId)
         return;
     m_outputParserId = parserId;
     toolChainUpdated();
 }
 
-CustomParserSettings CustomToolChain::customParserSettings() const
-{
-    return m_customParserSettings;
-}
-
-void CustomToolChain::setCustomParserSettings(const CustomParserSettings &settings)
-{
-    if (m_customParserSettings == settings)
-        return;
-    m_customParserSettings = settings;
-    toolChainUpdated();
-}
-
 QList<CustomToolChain::Parser> CustomToolChain::parsers()
 {
     QList<CustomToolChain::Parser> result;
@@ -444,14 +341,12 @@ QList<CustomToolChain::Parser> CustomToolChain::parsers()
     result.append({ClangParser::id(),    tr("Clang")});
     result.append({LinuxIccParser::id(), tr("ICC")});
     result.append({MsvcParser::id(),     tr("MSVC")});
-    result.append({CustomParser::id(),   tr("Custom")});
-
     return result;
 }
 
-ToolChainConfigWidget *CustomToolChain::configurationWidget()
+std::unique_ptr<ToolChainConfigWidget> CustomToolChain::createConfigurationWidget()
 {
-    return new Internal::CustomToolChainConfigWidget(this);
+    return std::make_unique<Internal::CustomToolChainConfigWidget>(this);
 }
 
 namespace Internal {
@@ -462,38 +357,11 @@ namespace Internal {
 
 CustomToolChainFactory::CustomToolChainFactory()
 {
-    setDisplayName(tr("Custom"));
-}
-
-QSet<Core::Id> CustomToolChainFactory::supportedLanguages() const
-{
-    return ToolChainManager::allLanguages();
-}
-
-bool CustomToolChainFactory::canCreate()
-{
-    return true;
-}
-
-ToolChain *CustomToolChainFactory::create(Core::Id language)
-{
-    return new CustomToolChain(language, ToolChain::ManualDetection);
-}
-
-// Used by the ToolChainManager to restore user-generated tool chains
-bool CustomToolChainFactory::canRestore(const QVariantMap &data)
-{
-    return typeIdFromMap(data) == Constants::CUSTOM_TOOLCHAIN_TYPEID;
-}
-
-ToolChain *CustomToolChainFactory::restore(const QVariantMap &data)
-{
-    auto tc = new CustomToolChain(ToolChain::ManualDetection);
-    if (tc->fromMap(data))
-        return tc;
-
-    delete tc;
-    return nullptr;
+    setDisplayName(CustomToolChain::tr("Custom"));
+    setSupportedToolChainType(Constants::CUSTOM_TOOLCHAIN_TYPEID);
+    setSupportsAllLanguages(true);
+    setToolchainConstructor([] { return new CustomToolChain; });
+    setUserCreatable(true);
 }
 
 // --------------------------------------------------------------------------
@@ -516,7 +384,7 @@ public:
 
     QStringList entries() const
     {
-        return textEditWidget()->toPlainText().split(QLatin1Char('\n'), QString::SkipEmptyParts);
+        return textEditWidget()->toPlainText().split(QLatin1Char('\n'), Qt::SkipEmptyParts);
     }
 
     QString text() const
@@ -556,18 +424,20 @@ CustomToolChainConfigWidget::CustomToolChainConfigWidget(CustomToolChain *tc) :
     m_headerDetails(new TextEditDetailsWidget(m_headerPaths)),
     m_cxx11Flags(new QLineEdit),
     m_mkspecs(new QLineEdit),
-    m_errorParserComboBox(new QComboBox),
-    m_customParserSettingsButton(new QPushButton(tr("Custom Parser Settings...")))
+    m_errorParserComboBox(new QComboBox)
 {
     Q_ASSERT(tc);
 
     const QList<CustomToolChain::Parser> parsers = CustomToolChain::parsers();
-    for (auto parser : parsers)
+    for (const auto &parser : parsers)
         m_errorParserComboBox->addItem(parser.displayName, parser.parserId.toString());
+    for (const CustomParserSettings &s : ProjectExplorerPlugin::customParsers())
+        m_errorParserComboBox->addItem(s.displayName, s.id.toString());
 
     auto parserLayoutWidget = new QWidget;
     auto parserLayout = new QHBoxLayout(parserLayoutWidget);
     parserLayout->setContentsMargins(0, 0, 0, 0);
+    m_predefinedMacros->setPlaceholderText(tr("MACRO[=VALUE]"));
     m_predefinedMacros->setTabChangesFocus(true);
     m_predefinedMacros->setToolTip(tr("Each line defines a macro. Format is MACRO[=VALUE]."));
     m_headerPaths->setTabChangesFocus(true);
@@ -586,7 +456,6 @@ CustomToolChainConfigWidget::CustomToolChainConfigWidget(CustomToolChain *tc) :
     m_mainLayout->addRow(tr("C++11 &flags:"), m_cxx11Flags);
     m_mainLayout->addRow(tr("&Qt mkspecs:"), m_mkspecs);
     parserLayout->addWidget(m_errorParserComboBox);
-    parserLayout->addWidget(m_customParserSettingsButton);
     m_mainLayout->addRow(tr("&Error parser:"), parserLayoutWidget);
     addErrorLabel();
 
@@ -603,10 +472,8 @@ CustomToolChainConfigWidget::CustomToolChainConfigWidget(CustomToolChain *tc) :
             this, &CustomToolChainConfigWidget::updateSummaries);
     connect(m_cxx11Flags, &QLineEdit::textChanged, this, &ToolChainConfigWidget::dirty);
     connect(m_mkspecs, &QLineEdit::textChanged, this, &ToolChainConfigWidget::dirty);
-    connect(m_errorParserComboBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+    connect(m_errorParserComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
             this, &CustomToolChainConfigWidget::errorParserChanged);
-    connect(m_customParserSettingsButton, &QAbstractButton::clicked,
-            this, &CustomToolChainConfigWidget::openCustomParserSettingsDialog);
     errorParserChanged();
 }
 
@@ -621,23 +488,9 @@ void CustomToolChainConfigWidget::updateSummaries()
 
 void CustomToolChainConfigWidget::errorParserChanged(int )
 {
-    const auto currentId = Core::Id::fromSetting(m_errorParserComboBox->currentData());
-    m_customParserSettingsButton->setEnabled(currentId == CustomParser::id());
     emit dirty();
 }
 
-void CustomToolChainConfigWidget::openCustomParserSettingsDialog()
-{
-    CustomParserConfigDialog dialog;
-    dialog.setSettings(m_customParserSettings);
-
-    if (dialog.exec() == QDialog::Accepted) {
-        m_customParserSettings = dialog.settings();
-        if (dialog.isDirty())
-            emit dirty();
-    }
-}
-
 void CustomToolChainConfigWidget::applyImpl()
 {
     if (toolChain()->isAutoDetected())
@@ -646,11 +499,11 @@ void CustomToolChainConfigWidget::applyImpl()
     auto tc = static_cast<CustomToolChain *>(toolChain());
     Q_ASSERT(tc);
     QString displayName = tc->displayName();
-    tc->setCompilerCommand(m_compilerCommand->fileName());
-    tc->setMakeCommand(m_makeCommand->fileName());
+    tc->setCompilerCommand(m_compilerCommand->filePath());
+    tc->setMakeCommand(m_makeCommand->filePath());
     tc->setTargetAbi(m_abiWidget->currentAbi());
     Macros macros = Utils::transform<QVector>(
-                m_predefinedDetails->text().split('\n', QString::SkipEmptyParts),
+                m_predefinedDetails->text().split('\n', Qt::SkipEmptyParts),
                 [](const QString &m) {
         return Macro::fromKeyValue(m);
     });
@@ -659,8 +512,7 @@ void CustomToolChainConfigWidget::applyImpl()
     tc->setCxx11Flags(m_cxx11Flags->text().split(QLatin1Char(',')));
     tc->setMkspecs(m_mkspecs->text());
     tc->setDisplayName(displayName); // reset display name
-    tc->setOutputParserId(Core::Id::fromSetting(m_errorParserComboBox->currentData()));
-    tc->setCustomParserSettings(m_customParserSettings);
+    tc->setOutputParserId(Utils::Id::fromSetting(m_errorParserComboBox->currentData()));
 
     setFromToolchain(); // Refresh with actual data from the toolchain. This shows what e.g. the
                         // macro parser did with the input.
@@ -669,11 +521,11 @@ void CustomToolChainConfigWidget::applyImpl()
 void CustomToolChainConfigWidget::setFromToolchain()
 {
     // subwidgets are not yet connected!
-    bool blocked = blockSignals(true);
+    QSignalBlocker blocker(this);
     auto tc = static_cast<CustomToolChain *>(toolChain());
-    m_compilerCommand->setFileName(tc->compilerCommand());
-    m_makeCommand->setFileName(FileName::fromString(tc->makeCommand(Environment())));
-    m_abiWidget->setAbis(QList<Abi>(), tc->targetAbi());
+    m_compilerCommand->setFilePath(tc->compilerCommand());
+    m_makeCommand->setFilePath(tc->makeCommand(Environment()));
+    m_abiWidget->setAbis(Abis(), tc->targetAbi());
     const QStringList macroLines = Utils::transform<QList>(tc->rawPredefinedMacros(), [](const Macro &m) {
         return QString::fromUtf8(m.toKeyValue(QByteArray()));
     });
@@ -683,23 +535,20 @@ void CustomToolChainConfigWidget::setFromToolchain()
     m_mkspecs->setText(tc->mkspecs());
     int index = m_errorParserComboBox->findData(tc->outputParserId().toSetting());
     m_errorParserComboBox->setCurrentIndex(index);
-    m_customParserSettings = tc->customParserSettings();
-    blockSignals(blocked);
 }
 
 bool CustomToolChainConfigWidget::isDirtyImpl() const
 {
     auto tc = static_cast<CustomToolChain *>(toolChain());
     Q_ASSERT(tc);
-    return m_compilerCommand->fileName() != tc->compilerCommand()
-            || m_makeCommand->path() != tc->makeCommand(Environment())
+    return m_compilerCommand->filePath() != tc->compilerCommand()
+            || m_makeCommand->filePath().toString() != tc->makeCommand(Environment()).toString()
             || m_abiWidget->currentAbi() != tc->targetAbi()
             || Macro::toMacros(m_predefinedDetails->text().toUtf8()) != tc->rawPredefinedMacros()
             || m_headerDetails->entries() != tc->headerPathsList()
             || m_cxx11Flags->text().split(QLatin1Char(',')) != tc->cxx11Flags()
             || m_mkspecs->text() != tc->mkspecs()
-            || Core::Id::fromSetting(m_errorParserComboBox->currentData()) == tc->outputParserId()
-            || m_customParserSettings != tc->customParserSettings();
+            || Utils::Id::fromSetting(m_errorParserComboBox->currentData()) == tc->outputParserId();
 }
 
 void CustomToolChainConfigWidget::makeReadOnlyImpl()
