@@ -36,8 +36,10 @@
 #include <utils/persistentsettings.h>
 #include <utils/portlist.h>
 #include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
 #include <utils/stringutils.h>
 
+#include <QDateTime>
 #include <QFileInfo>
 #include <QHash>
 #include <QList>
@@ -56,6 +58,8 @@ const char DeviceManagerKey[] = "DeviceManager";
 const char DeviceListKey[] = "DeviceList";
 const char DefaultDevicesKey[] = "DefaultDevices";
 
+template <class ...Args> using Continuation = std::function<void(Args...)>;
+
 class DeviceManagerPrivate
 {
 public:
@@ -96,7 +100,16 @@ int DeviceManager::deviceCount() const
 
 void DeviceManager::replaceInstance()
 {
+    const QList<Id> newIds =
+        Utils::transform(DeviceManagerPrivate::clonedInstance->d->devices, &IDevice::id);
+
+    for (IDevice::Ptr dev : m_instance->d->devices) {
+        if (!newIds.contains(dev->id()))
+            dev->aboutToBeRemoved();
+    }
+
     copy(DeviceManagerPrivate::clonedInstance, instance(), false);
+
     emit instance()->deviceListReplaced();
     emit instance()->updated();
 }
@@ -141,19 +154,17 @@ void DeviceManager::load()
     QTC_ASSERT(!d->writer, return);
 
     // Only create writer now: We do not want to save before the settings were read!
-    d->writer = new Utils::PersistentSettingsWriter(
-                settingsFilePath(QLatin1String("/devices.xml")),
-                QLatin1String("QtCreatorDevices"));
+    d->writer = new PersistentSettingsWriter(settingsFilePath("devices.xml"), "QtCreatorDevices");
 
     Utils::PersistentSettingsReader reader;
     // read devices file from global settings path
     QHash<Utils::Id, Utils::Id> defaultDevices;
     QList<IDevice::Ptr> sdkDevices;
-    if (reader.load(systemSettingsFilePath(QLatin1String("/devices.xml"))))
+    if (reader.load(systemSettingsFilePath("devices.xml")))
         sdkDevices = fromMap(reader.restoreValues().value(DeviceManagerKey).toMap(), &defaultDevices);
     // read devices file from user settings path
     QList<IDevice::Ptr> userDevices;
-    if (reader.load(settingsFilePath(QLatin1String("/devices.xml"))))
+    if (reader.load(settingsFilePath("devices.xml")))
         userDevices = fromMap(reader.restoreValues().value(DeviceManagerKey).toMap(), &defaultDevices);
     // Insert devices into the model. Prefer the higher device version when there are multiple
     // devices with the same id.
@@ -223,15 +234,14 @@ QVariantMap DeviceManager::toMap() const
     return map;
 }
 
-Utils::FilePath DeviceManager::settingsFilePath(const QString &extension)
+FilePath DeviceManager::settingsFilePath(const QString &extension)
 {
-    return Utils::FilePath::fromString(Core::ICore::userResourcePath() + extension);
+    return Core::ICore::userResourcePath(extension);
 }
 
-Utils::FilePath DeviceManager::systemSettingsFilePath(const QString &deviceFileRelativePath)
+FilePath DeviceManager::systemSettingsFilePath(const QString &deviceFileRelativePath)
 {
-    return Utils::FilePath::fromString(Core::ICore::installerResourcePath()
-                                       + deviceFileRelativePath);
+    return Core::ICore::installerResourcePath(deviceFileRelativePath);
 }
 
 void DeviceManager::addDevice(const IDevice::ConstPtr &_device)
@@ -314,6 +324,20 @@ bool DeviceManager::isLoaded() const
     return d->writer;
 }
 
+IDevice::ConstPtr DeviceManager::deviceForPath(const FilePath &path)
+{
+    for (IDevice::Ptr &dev : instance()->d->devices) {
+        if (dev->handlesFile(path))
+            return dev;
+    }
+    return {};
+}
+
+IDevice::ConstPtr DeviceManager::defaultDesktopDevice()
+{
+    return m_instance->defaultDevice(Constants::DESKTOP_DEVICE_TYPE);
+}
+
 void DeviceManager::setDefaultDevice(Utils::Id id)
 {
     QTC_ASSERT(this != instance(), return);
@@ -347,11 +371,211 @@ const IDeviceFactory *DeviceManager::restoreFactory(const QVariantMap &map)
 
 DeviceManager::DeviceManager(bool isInstance) : d(std::make_unique<DeviceManagerPrivate>())
 {
-    if (isInstance) {
-        QTC_ASSERT(!m_instance, return);
-        m_instance = this;
-        connect(Core::ICore::instance(), &Core::ICore::saveSettingsRequested, this, &DeviceManager::save);
-    }
+    QTC_ASSERT(isInstance == !m_instance, return);
+
+    if (!isInstance)
+        return;
+
+    m_instance = this;
+    connect(Core::ICore::instance(), &Core::ICore::saveSettingsRequested,
+            this, &DeviceManager::save);
+
+    DeviceFileHooks deviceHooks;
+
+    deviceHooks.isExecutableFile = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->isExecutableFile(filePath);
+    };
+
+    deviceHooks.isReadableFile = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->isReadableFile(filePath);
+    };
+
+    deviceHooks.isReadableDir = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->isReadableDirectory(filePath);
+    };
+
+    deviceHooks.isWritableDir = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->isWritableDirectory(filePath);
+    };
+
+    deviceHooks.isWritableFile = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->isWritableFile(filePath);
+    };
+
+    deviceHooks.isFile = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->isFile(filePath);
+    };
+
+    deviceHooks.isDir = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->isDirectory(filePath);
+    };
+
+    deviceHooks.ensureWritableDir = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->ensureWritableDirectory(filePath);
+    };
+
+    deviceHooks.ensureExistingFile = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->ensureExistingFile(filePath);
+    };
+
+    deviceHooks.createDir = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->createDirectory(filePath);
+    };
+
+    deviceHooks.exists = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->exists(filePath);
+    };
+
+    deviceHooks.removeFile = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->removeFile(filePath);
+    };
+
+    deviceHooks.removeRecursively = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->removeRecursively(filePath);
+    };
+
+    deviceHooks.copyFile = [](const FilePath &filePath, const FilePath &target) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->copyFile(filePath, target);
+    };
+
+    deviceHooks.renameFile = [](const FilePath &filePath, const FilePath &target) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->renameFile(filePath, target);
+    };
+
+    deviceHooks.searchInPath = [](const FilePath &filePath, const FilePaths &dirs) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return FilePath{});
+        return device->searchExecutable(filePath.path(), dirs);
+    };
+
+    deviceHooks.symLinkTarget = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return FilePath{});
+        return device->symLinkTarget(filePath);
+    };
+
+    deviceHooks.mapToGlobalPath = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return FilePath{});
+        return device->mapToGlobalPath(filePath);
+    };
+
+    deviceHooks.mapToDevicePath = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return QString{});
+        return device->mapToDevicePath(filePath);
+    };
+
+    deviceHooks.dirEntries = [](const FilePath &filePath, const QStringList &nameFilters,
+                                QDir::Filters filters, QDir::SortFlags sort) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return FilePaths());
+        return device->directoryEntries(filePath, nameFilters, filters, sort);
+    };
+
+    deviceHooks.fileContents = [](const FilePath &filePath, qint64 maxSize, qint64 offset) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return QByteArray());
+        return device->fileContents(filePath, maxSize, offset);
+    };
+
+    deviceHooks.asyncFileContents = [](const Continuation<QByteArray> &cont, const FilePath &filePath,
+            qint64 maxSize, qint64 offset) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return);
+        device->asyncFileContents(cont, filePath, maxSize, offset);
+    };
+
+    deviceHooks.writeFileContents = [](const FilePath &filePath, const QByteArray &data) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->writeFileContents(filePath, data);
+    };
+
+    deviceHooks.lastModified = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return QDateTime());
+        return device->lastModified(filePath);
+    };
+
+    deviceHooks.permissions = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return QFile::Permissions());
+        return device->permissions(filePath);
+    };
+
+    deviceHooks.setPermissions = [](const FilePath &filePath, QFile::Permissions permissions) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return false);
+        return device->setPermissions(filePath, permissions);
+    };
+
+    deviceHooks.osType = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return OsTypeOther);
+        return device->osType();
+    };
+
+    deviceHooks.environment = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return Environment{});
+        return device->systemEnvironment();
+    };
+
+    deviceHooks.fileSize = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return qint64(-1));
+        return device->fileSize(filePath);
+    };
+
+    FileUtils::setDeviceFileHooks(deviceHooks);
+
+    DeviceProcessHooks processHooks;
+
+    processHooks.startProcessHook = [](QtcProcess &process) {
+        FilePath filePath = process.commandLine().executable();
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return);
+        device->runProcess(process);
+    };
+
+    processHooks.systemEnvironmentForBinary = [](const FilePath &filePath) {
+        auto device = DeviceManager::deviceForPath(filePath);
+        QTC_ASSERT(device, return Environment());
+        return device->systemEnvironment();
+    };
+
+    QtcProcess::setRemoteProcessHooks(processHooks);
 }
 
 DeviceManager::~DeviceManager()
