@@ -30,7 +30,6 @@
 #include <projectexplorer/projectexplorerconstants.h>
 
 #include <QCoreApplication>
-#include <QString>
 
 namespace ProjectExplorer {
 namespace Internal {
@@ -38,28 +37,11 @@ namespace Internal {
 DeviceSettingsPage::DeviceSettingsPage()
 {
     setId(Constants::DEVICE_SETTINGS_PAGE_ID);
-    setDisplayName(tr("Devices"));
+    setDisplayName(DeviceSettingsWidget::tr("Devices"));
     setCategory(Constants::DEVICE_SETTINGS_CATEGORY);
     setDisplayCategory(QCoreApplication::translate("ProjectExplorer", "Devices"));
-    setCategoryIcon(Utils::Icon({{":/projectexplorer/images/settingscategory_devices.png",
-                    Utils::Theme::PanelTextColorDark}}, Utils::Icon::Tint));
-}
-
-QWidget *DeviceSettingsPage::widget()
-{
-    if (!m_widget)
-        m_widget = new DeviceSettingsWidget;
-    return m_widget;
-}
-
-void DeviceSettingsPage::apply()
-{
-    m_widget->saveSettings();
-}
-
-void DeviceSettingsPage::finish()
-{
-    delete m_widget;
+    setCategoryIconPath(":/projectexplorer/images/settingscategory_devices.png");
+    setWidgetCreator([] { return new DeviceSettingsWidget; });
 }
 
 } // namespace Internal
