@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,24 +9,20 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
+
 #include "devicesettingswidget.h"
 #include "ui_devicesettingswidget.h"
 
@@ -39,9 +35,10 @@
 #include "idevicefactory.h"
 #include "idevicewidget.h"
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectexplorericons.h>
 
 #include <coreplugin/icore.h>
-#include <extensionsystem/pluginmanager.h>
+
 #include <utils/qtcassert.h>
 #include <utils/algorithm.h>
 
@@ -61,14 +58,14 @@ const char LastDeviceIndexKey[] = "LastDisplayedMaemoDeviceConfig";
 class NameValidator : public QValidator
 {
 public:
-    NameValidator(const DeviceManager *deviceManager, QWidget *parent = 0)
+    NameValidator(const DeviceManager *deviceManager, QWidget *parent = nullptr)
         : QValidator(parent), m_deviceManager(deviceManager)
     {
     }
 
     void setDisplayName(const QString &name) { m_oldName = name; }
 
-    virtual State validate(QString &input, int & /* pos */) const
+    State validate(QString &input, int & /* pos */) const override
     {
         if (input.trimmed().isEmpty()
                 || (input != m_oldName && m_deviceManager->hasDevice(input)))
@@ -76,7 +73,7 @@ public:
         return Acceptable;
     }
 
-    virtual void fixup(QString &input) const
+    void fixup(QString &input) const override
     {
         int dummy = 0;
         if (validate(input, dummy) != Acceptable)
@@ -88,16 +85,16 @@ private:
     const DeviceManager * const m_deviceManager;
 };
 
-DeviceSettingsWidget::DeviceSettingsWidget(QWidget *parent)
-    : QWidget(parent),
-      m_ui(new Ui::DeviceSettingsWidget),
+DeviceSettingsWidget::DeviceSettingsWidget()
+    : m_ui(new Ui::DeviceSettingsWidget),
       m_deviceManager(DeviceManager::cloneInstance()),
       m_deviceManagerModel(new DeviceManagerModel(m_deviceManager, this)),
       m_nameValidator(new NameValidator(m_deviceManager, this)),
-      m_configWidget(0)
+      m_configWidget(nullptr)
 {
     initGui();
-    connect(m_deviceManager, SIGNAL(deviceUpdated(Core::Id)), SLOT(handleDeviceUpdated(Core::Id)));
+    connect(m_deviceManager, &DeviceManager::deviceUpdated,
+            this, &DeviceSettingsWidget::handleDeviceUpdated);
 }
 
 DeviceSettingsWidget::~DeviceSettingsWidget()
@@ -113,10 +110,8 @@ void DeviceSettingsWidget::initGui()
     m_ui->configurationComboBox->setModel(m_deviceManagerModel);
     m_ui->nameLineEdit->setValidator(m_nameValidator);
 
-    const QList<IDeviceFactory *> &factories
-        = ExtensionSystem::PluginManager::getObjects<IDeviceFactory>();
-
-    bool hasDeviceFactories = Utils::anyOf(factories, &IDeviceFactory::canCreate);
+    bool hasDeviceFactories = Utils::anyOf(IDeviceFactory::allDeviceFactories(),
+                                           &IDeviceFactory::canCreate);
 
     m_ui->addConfigButton->setEnabled(hasDeviceFactories);
 
@@ -126,11 +121,17 @@ void DeviceSettingsWidget::initGui()
         lastIndex = 0;
     if (lastIndex < m_ui->configurationComboBox->count())
         m_ui->configurationComboBox->setCurrentIndex(lastIndex);
-    connect(m_ui->configurationComboBox, SIGNAL(currentIndexChanged(int)),
-        SLOT(currentDeviceChanged(int)));
+    connect(m_ui->configurationComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
+            this, &DeviceSettingsWidget::currentDeviceChanged);
     currentDeviceChanged(currentIndex());
-    connect(m_ui->defaultDeviceButton, SIGNAL(clicked()),
-        SLOT(setDefaultDevice()));
+    connect(m_ui->defaultDeviceButton, &QAbstractButton::clicked,
+            this, &DeviceSettingsWidget::setDefaultDevice);
+    connect(m_ui->removeConfigButton, &QAbstractButton::clicked,
+            this, &DeviceSettingsWidget::removeDevice);
+    connect(m_ui->nameLineEdit, &QLineEdit::editingFinished,
+            this, &DeviceSettingsWidget::deviceNameEditingFinished);
+    connect(m_ui->addConfigButton, &QAbstractButton::clicked,
+            this, &DeviceSettingsWidget::addDevice);
 }
 
 void DeviceSettingsWidget::addDevice()
@@ -145,7 +146,7 @@ void DeviceSettingsWidget::addDevice()
     IDeviceFactory *factory = IDeviceFactory::find(toCreate);
     if (!factory)
         return;
-    IDevice::Ptr device = factory->create(toCreate);
+    IDevice::Ptr device = factory->create();
     if (device.isNull())
         return;
 
@@ -154,6 +155,7 @@ void DeviceSettingsWidget::addDevice()
     m_ui->configurationComboBox->setCurrentIndex(m_deviceManagerModel->indexOf(device));
     if (device->hasDeviceTester())
         testDevice();
+    saveSettings();
 }
 
 void DeviceSettingsWidget::removeDevice()
@@ -175,13 +177,13 @@ void DeviceSettingsWidget::displayCurrent()
     m_ui->deviceStateValueIconLabel->show();
     switch (current->deviceState()) {
     case IDevice::DeviceReadyToUse:
-        m_ui->deviceStateValueIconLabel->setPixmap(QPixmap(QLatin1String(":/projectexplorer/images/DeviceReadyToUse.png")));
+        m_ui->deviceStateValueIconLabel->setPixmap(Icons::DEVICE_READY_INDICATOR.pixmap());
         break;
     case IDevice::DeviceConnected:
-        m_ui->deviceStateValueIconLabel->setPixmap(QPixmap(QLatin1String(":/projectexplorer/images/DeviceConnected.png")));
+        m_ui->deviceStateValueIconLabel->setPixmap(Icons::DEVICE_CONNECTED_INDICATOR.pixmap());
         break;
     case IDevice::DeviceDisconnected:
-        m_ui->deviceStateValueIconLabel->setPixmap(QPixmap(QLatin1String(":/projectexplorer/images/DeviceDisconnected.png")));
+        m_ui->deviceStateValueIconLabel->setPixmap(Icons::DEVICE_DISCONNECTED_INDICATOR.pixmap());
         break;
     case IDevice::DeviceStateUnknown:
         m_ui->deviceStateValueIconLabel->hide();
@@ -217,7 +219,7 @@ void DeviceSettingsWidget::updateDeviceFromUi()
 
 void DeviceSettingsWidget::saveSettings()
 {
-    ICore::settings()->setValue(QLatin1String(LastDeviceIndexKey), currentIndex());
+    ICore::settings()->setValueWithDefault(LastDeviceIndexKey, currentIndex(), 0);
     DeviceManager::replaceInstance();
 }
 
@@ -253,8 +255,10 @@ void DeviceSettingsWidget::testDevice()
 {
     const IDevice::ConstPtr &device = currentDevice();
     QTC_ASSERT(device && device->hasDeviceTester(), return);
-    DeviceTestDialog dlg(device, this);
-    dlg.exec();
+    auto dlg = new DeviceTestDialog(m_deviceManager->mutableDevice(device->id()), this);
+    dlg->setAttribute(Qt::WA_DeleteOnClose);
+    dlg->setModal(true);
+    dlg->show();
 }
 
 void DeviceSettingsWidget::handleDeviceUpdated(Id id)
@@ -268,7 +272,7 @@ void DeviceSettingsWidget::currentDeviceChanged(int index)
 {
     qDeleteAll(m_additionalActionButtons);
     delete m_configWidget;
-    m_configWidget = 0;
+    m_configWidget = nullptr;
     m_additionalActionButtons.clear();
     const IDevice::ConstPtr device = m_deviceManagerModel->device(index);
     if (device.isNull()) {
@@ -284,22 +288,31 @@ void DeviceSettingsWidget::currentDeviceChanged(int index)
     if (device->hasDeviceTester()) {
         QPushButton * const button = new QPushButton(tr("Test"));
         m_additionalActionButtons << button;
-        connect(button, SIGNAL(clicked()), SLOT(testDevice()));
+        connect(button, &QAbstractButton::clicked, this, &DeviceSettingsWidget::testDevice);
         m_ui->buttonsLayout->insertWidget(m_ui->buttonsLayout->count() - 1, button);
     }
 
     if (device->canCreateProcessModel()) {
         QPushButton * const button = new QPushButton(tr("Show Running Processes..."));
         m_additionalActionButtons << button;
-        connect(button, SIGNAL(clicked()), SLOT(handleProcessListRequested()));
+        connect(button, &QAbstractButton::clicked,
+                this, &DeviceSettingsWidget::handleProcessListRequested);
         m_ui->buttonsLayout->insertWidget(m_ui->buttonsLayout->count() - 1, button);
     }
 
-    foreach (Id actionId, device->actionIds()) {
-        QPushButton * const button = new QPushButton(device->displayNameForActionId(actionId));
+    for (const IDevice::DeviceAction &deviceAction : device->deviceActions()) {
+        QPushButton * const button = new QPushButton(deviceAction.display);
         m_additionalActionButtons << button;
-        connect(button, &QAbstractButton::clicked, this,
-                [this, actionId] { handleAdditionalActionRequest(actionId); });
+        connect(button, &QAbstractButton::clicked, this, [this, deviceAction] {
+            const IDevice::Ptr device = m_deviceManager->mutableDevice(currentDevice()->id());
+            QTC_ASSERT(device, return);
+            updateDeviceFromUi();
+            deviceAction.execute(device, this);
+            // Widget must be set up from scratch, because the action could have
+            // changed random attributes.
+            currentDeviceChanged(currentIndex());
+        });
+
         m_ui->buttonsLayout->insertWidget(m_ui->buttonsLayout->count() - 1, button);
     }
 
@@ -318,17 +331,6 @@ void DeviceSettingsWidget::clearDetails()
     m_ui->autoDetectionValueLabel->clear();
 }
 
-void DeviceSettingsWidget::handleAdditionalActionRequest(Id actionId)
-{
-    const IDevice::Ptr device = m_deviceManager->mutableDevice(currentDevice()->id());
-    QTC_ASSERT(device, return);
-    updateDeviceFromUi();
-    device->executeAction(actionId, this);
-
-    // Widget must be set up from scratch, because the action could have changed random attributes.
-    currentDeviceChanged(currentIndex());
-}
-
 void DeviceSettingsWidget::handleProcessListRequested()
 {
     QTC_ASSERT(currentDevice()->canCreateProcessModel(), return);
