@@ -33,6 +33,7 @@
 #include "../kitinformation.h"
 #include "../runconfiguration.h"
 
+#include <coreplugin/icore.h>
 #include <ssh/sshconnection.h>
 #include <utils/displayname.h>
 #include <utils/icon.h>
@@ -43,6 +44,7 @@
 #include <QCoreApplication>
 #include <QStandardPaths>
 
+#include <QDateTime>
 #include <QString>
 #include <QUuid>
 
@@ -97,14 +99,15 @@
  * Creates an identical copy of a device object.
  */
 
-
-static Utils::Id newId()
-{
-    return Utils::Id::fromString(QUuid::createUuid().toString());
-}
+using namespace Utils;
 
 namespace ProjectExplorer {
 
+static Id newId()
+{
+    return Id::fromString(QUuid::createUuid().toString());
+}
+
 const char DisplayNameKey[] = "Name";
 const char TypeKey[] = "OsType";
 const char IdKey[] = "InternalId";
@@ -124,7 +127,7 @@ const char TimeoutKey[] = "Timeout";
 const char HostKeyCheckingKey[] = "HostKeyChecking";
 
 const char DebugServerKey[] = "DebugServerKey";
-const char QmlsceneKey[] = "QmlsceneKey";
+const char QmlRuntimeKey[] = "QmlsceneKey";
 
 using AuthType = QSsh::SshConnectionParameters::AuthenticationType;
 const AuthType DefaultAuthType = QSsh::SshConnectionParameters::AuthenticationTypeAll;
@@ -138,22 +141,24 @@ class IDevicePrivate
 public:
     IDevicePrivate() = default;
 
-    Utils::DisplayName displayName;
+    DisplayName displayName;
     QString displayType;
-    Utils::Id type;
+    Id type;
     IDevice::Origin origin = IDevice::AutoDetected;
-    Utils::Id id;
+    Id id;
     IDevice::DeviceState deviceState = IDevice::DeviceStateUnknown;
     IDevice::MachineType machineType = IDevice::Hardware;
-    Utils::OsType osType = Utils::OsTypeOther;
+    OsType osType = OsTypeOther;
     int version = 0; // This is used by devices that have been added by the SDK.
 
     QSsh::SshConnectionParameters sshParameters;
-    Utils::PortList freePorts;
-    QString debugServerPath;
-    QString qmlsceneCommand;
+    PortList freePorts;
+    FilePath debugServerPath;
+    FilePath debugDumperPath = Core::ICore::resourcePath("debugger/");
+    FilePath qmlRunCommand;
+    bool emptyCommandAllowed = false;
 
-    QList<Utils::Icon> deviceIcons;
+    QList<Icon> deviceIcons;
     QList<IDevice::DeviceAction> deviceActions;
     QVariantMap extraData;
     IDevice::OpenTerminal openTerminal;
@@ -171,7 +176,7 @@ void IDevice::setOpenTerminal(const IDevice::OpenTerminal &openTerminal)
     d->openTerminal = openTerminal;
 }
 
-void IDevice::setupId(Origin origin, Utils::Id id)
+void IDevice::setupId(Origin origin, Id id)
 {
     d->origin = origin;
     QTC_CHECK(origin == ManuallyAdded || id.isValid());
@@ -183,12 +188,262 @@ bool IDevice::canOpenTerminal() const
     return bool(d->openTerminal);
 }
 
-void IDevice::openTerminal(const Utils::Environment &env, const QString &workingDir) const
+void IDevice::openTerminal(const Environment &env, const FilePath &workingDir) const
 {
     QTC_ASSERT(canOpenTerminal(), return);
     d->openTerminal(env, workingDir);
 }
 
+bool IDevice::isEmptyCommandAllowed() const
+{
+    return d->emptyCommandAllowed;
+}
+
+void IDevice::setAllowEmptyCommand(bool allow)
+{
+    d->emptyCommandAllowed = allow;
+}
+
+bool IDevice::isAnyUnixDevice() const
+{
+    return d->osType == OsTypeLinux || d->osType == OsTypeMac || d->osType == OsTypeOtherUnix;
+}
+
+FilePath IDevice::mapToGlobalPath(const FilePath &pathOnDevice) const
+{
+    return pathOnDevice;
+}
+
+QString IDevice::mapToDevicePath(const FilePath &globalPath) const
+{
+    return globalPath.path();
+}
+
+bool IDevice::handlesFile(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    return false;
+}
+
+bool IDevice::isExecutableFile(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::isReadableFile(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::isWritableFile(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::isReadableDirectory(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::isWritableDirectory(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::isFile(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::isDirectory(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::ensureWritableDirectory(const FilePath &filePath) const
+{
+    if (isWritableDirectory(filePath))
+        return true;
+    return createDirectory(filePath);
+}
+
+bool IDevice::ensureExistingFile(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::createDirectory(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::exists(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::removeFile(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::removeRecursively(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::copyFile(const FilePath &filePath, const FilePath &target) const
+{
+    Q_UNUSED(filePath);
+    Q_UNUSED(target);
+    QTC_CHECK(false);
+    return false;
+}
+
+bool IDevice::renameFile(const FilePath &filePath, const FilePath &target) const
+{
+    Q_UNUSED(filePath);
+    Q_UNUSED(target);
+    QTC_CHECK(false);
+    return false;
+}
+
+FilePath IDevice::searchExecutableInPath(const QString &fileName) const
+{
+    FilePaths paths;
+    for (const FilePath &path : systemEnvironment().path())
+        paths.append(mapToGlobalPath(path));
+    return searchExecutable(fileName, paths);
+}
+
+FilePath IDevice::searchExecutable(const QString &fileName, const FilePaths &dirs) const
+{
+    for (FilePath dir : dirs) {
+        if (!handlesFile(dir)) // Allow device-local dirs to be used.
+            dir = mapToGlobalPath(dir);
+        QTC_CHECK(handlesFile(dir));
+        const FilePath candidate = dir / fileName;
+        if (isExecutableFile(candidate))
+            return candidate;
+    }
+
+    return {};
+}
+
+FilePath IDevice::symLinkTarget(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return {};
+}
+
+QList<FilePath> IDevice::directoryEntries(const FilePath &filePath,
+                                          const QStringList &nameFilters,
+                                          QDir::Filters filters,
+                                          QDir::SortFlags sort) const
+{
+    Q_UNUSED(filePath);
+    Q_UNUSED(nameFilters);
+    Q_UNUSED(filters);
+    Q_UNUSED(sort);
+    QTC_CHECK(false);
+    return {};
+}
+
+QByteArray IDevice::fileContents(const FilePath &filePath, qint64 limit, qint64 offset) const
+{
+    Q_UNUSED(filePath);
+    Q_UNUSED(limit);
+    Q_UNUSED(offset);
+    QTC_CHECK(false);
+    return {};
+}
+
+void IDevice::asyncFileContents(const Continuation<QByteArray> &cont,
+                                const FilePath &filePath,
+                                qint64 limit, qint64 offset) const
+{
+    cont(fileContents(filePath, limit, offset));
+}
+
+bool IDevice::writeFileContents(const FilePath &filePath, const QByteArray &data) const
+{
+    Q_UNUSED(filePath);
+    Q_UNUSED(data);
+    QTC_CHECK(false);
+    return {};
+}
+
+void IDevice::asyncWriteFileContents(const Continuation<bool> &cont,
+                                     const FilePath &filePath,
+                                     const QByteArray &data) const
+{
+    cont(writeFileContents(filePath, data));
+}
+
+QDateTime IDevice::lastModified(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    return {};
+}
+
+QFileDevice::Permissions IDevice::permissions(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return {};
+}
+
+bool IDevice::setPermissions(const FilePath &filePath, QFile::Permissions) const
+{
+    Q_UNUSED(filePath);
+    QTC_CHECK(false);
+    return false;
+}
+
+void IDevice::runProcess(QtcProcess &process) const
+{
+    Q_UNUSED(process);
+    QTC_CHECK(false);
+}
+
+Environment IDevice::systemEnvironment() const
+{
+    QTC_CHECK(false);
+    return Environment::systemEnvironment();
+}
+
+qint64 IDevice::fileSize(const FilePath &filePath) const
+{
+    Q_UNUSED(filePath)
+    QTC_CHECK(false);
+    return -1;
+}
+
 IDevice::~IDevice() = default;
 
 /*!
@@ -220,7 +475,7 @@ void IDevice::setDisplayType(const QString &type)
     d->displayType = type;
 }
 
-void IDevice::setOsType(Utils::OsType osType)
+void IDevice::setOsType(OsType osType)
 {
     d->osType = osType;
 }
@@ -238,12 +493,12 @@ IDevice::DeviceInfo IDevice::deviceInformation() const
     \sa ProjectExplorer::IDeviceFactory
  */
 
-Utils::Id IDevice::type() const
+Id IDevice::type() const
 {
     return d->type;
 }
 
-void IDevice::setType(Utils::Id type)
+void IDevice::setType(Id type)
 {
     d->type = type;
 }
@@ -269,7 +524,7 @@ bool IDevice::isAutoDetected() const
     \sa ProjectExplorer::DeviceManager::findInactiveAutoDetectedDevice()
 */
 
-Utils::Id IDevice::id() const
+Id IDevice::id() const
 {
     return d->id;
 }
@@ -284,6 +539,11 @@ bool IDevice::isCompatibleWith(const Kit *k) const
     return DeviceTypeKitAspect::deviceTypeId(k) == type();
 }
 
+QVector<Task> IDevice::validate() const
+{
+    return {};
+}
+
 void IDevice::addDeviceAction(const DeviceAction &deviceAction)
 {
     d->deviceActions.append(deviceAction);
@@ -312,7 +572,7 @@ DeviceTester *IDevice::createDeviceTester() const
     return nullptr;
 }
 
-Utils::OsType IDevice::osType() const
+OsType IDevice::osType() const
 {
     return d->osType;
 }
@@ -340,14 +600,14 @@ void IDevice::setDeviceState(const IDevice::DeviceState state)
     d->deviceState = state;
 }
 
-Utils::Id IDevice::typeFromMap(const QVariantMap &map)
+Id IDevice::typeFromMap(const QVariantMap &map)
 {
-    return Utils::Id::fromSetting(map.value(QLatin1String(TypeKey)));
+    return Id::fromSetting(map.value(QLatin1String(TypeKey)));
 }
 
-Utils::Id IDevice::idFromMap(const QVariantMap &map)
+Id IDevice::idFromMap(const QVariantMap &map)
 {
-    return Utils::Id::fromSetting(map.value(QLatin1String(IdKey)));
+    return Id::fromSetting(map.value(QLatin1String(IdKey)));
 }
 
 /*!
@@ -360,7 +620,7 @@ void IDevice::fromMap(const QVariantMap &map)
 {
     d->type = typeFromMap(map);
     d->displayName.fromMap(map, DisplayNameKey);
-    d->id = Utils::Id::fromSetting(map.value(QLatin1String(IdKey)));
+    d->id = Id::fromSetting(map.value(QLatin1String(IdKey)));
     if (!d->id.isValid())
         d->id = newId();
     d->origin = static_cast<Origin>(map.value(QLatin1String(OriginKey), ManuallyAdded).toInt());
@@ -377,7 +637,8 @@ void IDevice::fromMap(const QVariantMap &map)
             ? QSsh::SshConnectionParameters::AuthenticationTypeAll
             : static_cast<AuthType>(storedAuthType);
 
-    d->sshParameters.privateKeyFile = map.value(QLatin1String(KeyFileKey), defaultPrivateKeyFilePath()).toString();
+    d->sshParameters.privateKeyFile =
+        FilePath::fromVariant(map.value(QLatin1String(KeyFileKey), defaultPrivateKeyFilePath()));
     d->sshParameters.timeout = map.value(QLatin1String(TimeoutKey), DefaultTimeout).toInt();
     d->sshParameters.hostKeyCheckingMode = static_cast<QSsh::SshHostKeyCheckingMode>
             (map.value(QLatin1String(HostKeyCheckingKey), QSsh::SshHostKeyCheckingNone).toInt());
@@ -385,12 +646,12 @@ void IDevice::fromMap(const QVariantMap &map)
     QString portsSpec = map.value(PortsSpecKey).toString();
     if (portsSpec.isEmpty())
         portsSpec = "10000-10100";
-    d->freePorts = Utils::PortList::fromString(portsSpec);
+    d->freePorts = PortList::fromString(portsSpec);
     d->machineType = static_cast<MachineType>(map.value(QLatin1String(MachineTypeKey), DefaultMachineType).toInt());
     d->version = map.value(QLatin1String(VersionKey), 0).toInt();
 
-    d->debugServerPath = map.value(QLatin1String(DebugServerKey)).toString();
-    d->qmlsceneCommand = map.value(QLatin1String(QmlsceneKey)).toString();
+    d->debugServerPath = FilePath::fromVariant(map.value(QLatin1String(DebugServerKey)));
+    d->qmlRunCommand = FilePath::fromVariant(map.value(QLatin1String(QmlRuntimeKey)));
     d->extraData = map.value(ExtraDataKey).toMap();
 }
 
@@ -413,15 +674,15 @@ QVariantMap IDevice::toMap() const
     map.insert(QLatin1String(SshPortKey), d->sshParameters.port());
     map.insert(QLatin1String(UserNameKey), d->sshParameters.userName());
     map.insert(QLatin1String(AuthKey), d->sshParameters.authenticationType);
-    map.insert(QLatin1String(KeyFileKey), d->sshParameters.privateKeyFile);
+    map.insert(QLatin1String(KeyFileKey), d->sshParameters.privateKeyFile.toVariant());
     map.insert(QLatin1String(TimeoutKey), d->sshParameters.timeout);
     map.insert(QLatin1String(HostKeyCheckingKey), d->sshParameters.hostKeyCheckingMode);
 
     map.insert(QLatin1String(PortsSpecKey), d->freePorts.toString());
     map.insert(QLatin1String(VersionKey), d->version);
 
-    map.insert(QLatin1String(DebugServerKey), d->debugServerPath);
-    map.insert(QLatin1String(QmlsceneKey), d->qmlsceneCommand);
+    map.insert(QLatin1String(DebugServerKey), d->debugServerPath.toVariant());
+    map.insert(QLatin1String(QmlRuntimeKey), d->qmlRunCommand.toVariant());
     map.insert(ExtraDataKey, d->extraData);
 
     return map;
@@ -474,12 +735,12 @@ QUrl IDevice::toolControlChannel(const ControlChannelHint &) const
     return url;
 }
 
-void IDevice::setFreePorts(const Utils::PortList &freePorts)
+void IDevice::setFreePorts(const PortList &freePorts)
 {
     d->freePorts = freePorts;
 }
 
-Utils::PortList IDevice::freePorts() const
+PortList IDevice::freePorts() const
 {
     return d->freePorts;
 }
@@ -494,32 +755,42 @@ void IDevice::setMachineType(MachineType machineType)
     d->machineType = machineType;
 }
 
-QString IDevice::debugServerPath() const
+FilePath IDevice::debugServerPath() const
 {
     return d->debugServerPath;
 }
 
-void IDevice::setDebugServerPath(const QString &path)
+void IDevice::setDebugServerPath(const FilePath &path)
 {
     d->debugServerPath = path;
 }
 
-QString IDevice::qmlsceneCommand() const
+FilePath IDevice::debugDumperPath() const
 {
-    return d->qmlsceneCommand;
+    return d->debugDumperPath;
 }
 
-void IDevice::setQmlsceneCommand(const QString &path)
+void IDevice::setDebugDumperPath(const Utils::FilePath &path)
 {
-    d->qmlsceneCommand = path;
+    d->debugDumperPath = path;
 }
 
-void IDevice::setExtraData(Utils::Id kind, const QVariant &data)
+FilePath IDevice::qmlRunCommand() const
+{
+    return d->qmlRunCommand;
+}
+
+void IDevice::setQmlRunCommand(const FilePath &path)
+{
+    d->qmlRunCommand = path;
+}
+
+void IDevice::setExtraData(Id kind, const QVariant &data)
 {
     d->extraData.insert(kind.toString(), data);
 }
 
-QVariant IDevice::extraData(Utils::Id kind) const
+QVariant IDevice::extraData(Id kind) const
 {
     return d->extraData.value(kind.toString());
 }
@@ -540,7 +811,7 @@ QString IDevice::defaultPublicKeyFilePath()
     return defaultPrivateKeyFilePath() + QLatin1String(".pub");
 }
 
-void DeviceProcessSignalOperation::setDebuggerCommand(const QString &cmd)
+void DeviceProcessSignalOperation::setDebuggerCommand(const FilePath &cmd)
 {
     m_debuggerCommand = cmd;
 }
