@@ -27,12 +27,6 @@
 
 #include "deviceprocesslist.h"
 
-#include <QString>
-
-QT_BEGIN_NAMESPACE
-class QProcess;
-QT_END_NAMESPACE
-
 namespace ProjectExplorer {
 namespace Internal {
 
@@ -41,8 +35,7 @@ class LocalProcessList : public DeviceProcessList
     Q_OBJECT
 
 public:
-    explicit LocalProcessList(const IDevice::ConstPtr &device, QObject *parent = 0);
-    Qt::ItemFlags flags(const QModelIndex &index) const override;
+    explicit LocalProcessList(const IDevice::ConstPtr &device, QObject *parent = nullptr);
 
     static QList<DeviceProcessItem> getLocalProcesses();
 
@@ -53,8 +46,6 @@ private:
 private:
     void handleUpdate();
     void reportDelayedKillStatus(const QString &errorMessage);
-
-    const qint64 m_myPid;
 };
 
 } // namespace Internal
