@@ -43,7 +43,7 @@ namespace ProjectExplorer {
 
 EnvironmentAspect::EnvironmentAspect()
 {
-    setDisplayName(tr("Run Environment"));
+    setDisplayName(tr("Environment"));
     setId("EnvironmentAspect");
     setConfigWidgetCreator([this] { return new EnvironmentAspectWidget(this); });
 }
@@ -130,12 +130,6 @@ void EnvironmentAspect::toMap(QVariantMap &data) const
     data.insert(QLatin1String(CHANGES_KEY), Utils::EnvironmentItem::toStringList(m_userChanges));
 }
 
-Environment EnvironmentAspect::currentUnmodifiedBaseEnvironment() const
-{
-    QTC_ASSERT(m_base >= 0 && m_base < m_baseEnvironments.size(), return Environment());
-    return m_baseEnvironments.at(m_base).unmodifiedBaseEnvironment();
-}
-
 QString EnvironmentAspect::currentDisplayName() const
 {
     QTC_ASSERT(m_base >= 0 && m_base < m_baseEnvironments.size(), return {});
