@@ -39,26 +39,28 @@
 #include <texteditor/fontsettings.h>
 
 #include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
 #include <utils/runextensions.h>
 
 #include <QDateTime>
 #include <QFutureInterface>
 #include <QFutureWatcher>
-#include <QProcess>
+#include <QTextBlock>
 #include <QThreadPool>
 #include <QTimer>
-#include <QTextBlock>
+
+using namespace Utils;
 
 namespace ProjectExplorer {
 
 Q_GLOBAL_STATIC(QThreadPool, s_extraCompilerThreadPool);
 Q_GLOBAL_STATIC(QList<ExtraCompilerFactory *>, factories);
-Q_GLOBAL_STATIC(QVector<ExtraCompilerFactoryObserver *>, observers);
+
 class ExtraCompilerPrivate
 {
 public:
     const Project *project;
-    Utils::FilePath source;
+    FilePath source;
     FileNameToContentsHash contents;
     Tasks issues;
     QDateTime compileTime;
@@ -71,13 +73,13 @@ public:
     void updateIssues();
 };
 
-ExtraCompiler::ExtraCompiler(const Project *project, const Utils::FilePath &source,
-                             const Utils::FilePaths &targets, QObject *parent) :
+ExtraCompiler::ExtraCompiler(const Project *project, const FilePath &source,
+                             const FilePaths &targets, QObject *parent) :
     QObject(parent), d(std::make_unique<ExtraCompilerPrivate>())
 {
     d->project = project;
     d->source = source;
-    foreach (const Utils::FilePath &target, targets)
+    for (const FilePath &target : targets)
         d->contents.insert(target, QByteArray());
     d->timer.setSingleShot(true);
 
@@ -104,8 +106,8 @@ ExtraCompiler::ExtraCompiler(const Project *project, const Utils::FilePath &sour
             this, &ExtraCompiler::onEditorAboutToClose);
 
     // Use existing target files, where possible. Otherwise run the compiler.
-    QDateTime sourceTime = d->source.toFileInfo().lastModified();
-    foreach (const Utils::FilePath &target, targets) {
+    QDateTime sourceTime = d->source.lastModified();
+    for (const FilePath &target : targets) {
         QFileInfo targetFileInfo(target.toFileInfo());
         if (!targetFileInfo.exists()) {
             d->dirty = true;
@@ -123,11 +125,6 @@ ExtraCompiler::ExtraCompiler(const Project *project, const Utils::FilePath &sour
         if (file.open(QFile::ReadOnly | QFile::Text))
             setContent(target, file.readAll());
     }
-
-    if (d->dirty) {
-        d->dirty = false;
-        QTimer::singleShot(0, this, [this]() { run(d->source); }); // delay till available.
-    }
 }
 
 ExtraCompiler::~ExtraCompiler() = default;
@@ -137,22 +134,22 @@ const Project *ExtraCompiler::project() const
     return d->project;
 }
 
-Utils::FilePath ExtraCompiler::source() const
+FilePath ExtraCompiler::source() const
 {
     return d->source;
 }
 
-QByteArray ExtraCompiler::content(const Utils::FilePath &file) const
+QByteArray ExtraCompiler::content(const FilePath &file) const
 {
     return d->contents.value(file);
 }
 
-Utils::FilePaths ExtraCompiler::targets() const
+FilePaths ExtraCompiler::targets() const
 {
     return d->contents.keys();
 }
 
-void ExtraCompiler::forEachTarget(std::function<void (const Utils::FilePath &)> func)
+void ExtraCompiler::forEachTarget(std::function<void (const FilePath &)> func)
 {
     for (auto it = d->contents.constBegin(), end = d->contents.constEnd(); it != end; ++it)
         func(it.key());
@@ -173,6 +170,11 @@ QThreadPool *ExtraCompiler::extraCompilerThreadPool()
     return s_extraCompilerThreadPool();
 }
 
+bool ExtraCompiler::isDirty() const
+{
+    return d->dirty;
+}
+
 void ExtraCompiler::onTargetsBuilt(Project *project)
 {
     if (project != d->project || BuildManager::isBuilding(project))
@@ -180,11 +182,11 @@ void ExtraCompiler::onTargetsBuilt(Project *project)
 
     // This is mostly a fall back for the cases when the generator couldn't be run.
     // It pays special attention to the case where a source file was newly created
-    const QDateTime sourceTime = d->source.toFileInfo().lastModified();
+    const QDateTime sourceTime = d->source.lastModified();
     if (d->compileTime.isValid() && d->compileTime >= sourceTime)
         return;
 
-    forEachTarget([&](const Utils::FilePath &target) {
+    forEachTarget([&](const FilePath &target) {
         QFileInfo fi(target.toFileInfo());
         QDateTime generateTime = fi.exists() ? fi.lastModified() : QDateTime();
         if (generateTime.isValid() && (generateTime > sourceTime)) {
@@ -249,21 +251,21 @@ void ExtraCompiler::onEditorAboutToClose(Core::IEditor *editor)
     d->lastEditor = nullptr;
 }
 
-Utils::Environment ExtraCompiler::buildEnvironment() const
+Environment ExtraCompiler::buildEnvironment() const
 {
     if (Target *target = project()->activeTarget()) {
         if (BuildConfiguration *bc = target->activeBuildConfiguration()) {
             return bc->environment();
         } else {
-            Utils::EnvironmentItems changes =
+            EnvironmentItems changes =
                     EnvironmentKitAspect::environmentChanges(target->kit());
-            Utils::Environment env = Utils::Environment::systemEnvironment();
+            Environment env = Environment::systemEnvironment();
             env.modify(changes);
             return env;
         }
     }
 
-    return Utils::Environment::systemEnvironment();
+    return Environment::systemEnvironment();
 }
 
 void ExtraCompiler::setCompileIssues(const Tasks &issues)
@@ -283,14 +285,14 @@ void ExtraCompilerPrivate::updateIssues()
 
     QList<QTextEdit::ExtraSelection> selections;
     const QTextDocument *document = widget->document();
-    foreach (const Task &issue, issues) {
+    for (const Task &issue : qAsConst(issues)) {
         QTextEdit::ExtraSelection selection;
         QTextCursor cursor(document->findBlockByNumber(issue.line - 1));
         cursor.movePosition(QTextCursor::StartOfLine);
         cursor.movePosition(QTextCursor::EndOfLine, QTextCursor::KeepAnchor);
         selection.cursor = cursor;
 
-        const auto fontSettings = TextEditor::TextEditorSettings::instance()->fontSettings();
+        const auto fontSettings = TextEditor::TextEditorSettings::fontSettings();
         selection.format = fontSettings.toTextCharFormat(issue.type == Task::Warning ?
                 TextEditor::C_WARNING : TextEditor::C_ERROR);
         selection.format.setToolTip(issue.description());
@@ -300,7 +302,7 @@ void ExtraCompilerPrivate::updateIssues()
     widget->setExtraSelections(TextEditor::TextEditorWidget::CodeWarningsSelection, selections);
 }
 
-void ExtraCompiler::setContent(const Utils::FilePath &file, const QByteArray &contents)
+void ExtraCompiler::setContent(const FilePath &file, const QByteArray &contents)
 {
     auto it = d->contents.find(file);
     if (it != d->contents.end()) {
@@ -322,21 +324,13 @@ ExtraCompilerFactory::~ExtraCompilerFactory()
     factories->removeAll(this);
 }
 
-void ExtraCompilerFactory::annouceCreation(const Project *project,
-                                           const Utils::FilePath &source,
-                                           const Utils::FilePaths &targets)
-{
-    for (ExtraCompilerFactoryObserver *observer : *observers)
-        observer->newExtraCompiler(project, source, targets);
-}
-
 QList<ExtraCompilerFactory *> ExtraCompilerFactory::extraCompilerFactories()
 {
     return *factories();
 }
 
-ProcessExtraCompiler::ProcessExtraCompiler(const Project *project, const Utils::FilePath &source,
-                                           const Utils::FilePaths &targets, QObject *parent) :
+ProcessExtraCompiler::ProcessExtraCompiler(const Project *project, const FilePath &source,
+                                           const FilePaths &targets, QObject *parent) :
     ExtraCompiler(project, source, targets, parent)
 { }
 
@@ -354,20 +348,21 @@ void ProcessExtraCompiler::run(const QByteArray &sourceContents)
     runImpl(contents);
 }
 
-void ProcessExtraCompiler::run(const Utils::FilePath &fileName)
+QFuture<FileNameToContentsHash> ProcessExtraCompiler::run()
 {
+    const FilePath fileName = source();
     ContentProvider contents = [fileName]() {
         QFile file(fileName.toString());
         if (!file.open(QFile::ReadOnly | QFile::Text))
             return QByteArray();
         return file.readAll();
     };
-    runImpl(contents);
+    return runImpl(contents);
 }
 
-Utils::FilePath ProcessExtraCompiler::workingDirectory() const
+FilePath ProcessExtraCompiler::workingDirectory() const
 {
-    return Utils::FilePath();
+    return FilePath();
 }
 
 QStringList ProcessExtraCompiler::arguments() const
@@ -387,7 +382,7 @@ Tasks ProcessExtraCompiler::parseIssues(const QByteArray &stdErr)
     return {};
 }
 
-void ProcessExtraCompiler::runImpl(const ContentProvider &provider)
+QFuture<FileNameToContentsHash> ProcessExtraCompiler::runImpl(const ContentProvider &provider)
 {
     if (m_watcher)
         delete m_watcher;
@@ -396,17 +391,18 @@ void ProcessExtraCompiler::runImpl(const ContentProvider &provider)
     connect(m_watcher, &QFutureWatcher<FileNameToContentsHash>::finished,
             this, &ProcessExtraCompiler::cleanUp);
 
-    m_watcher->setFuture(Utils::runAsync(extraCompilerThreadPool(),
+    m_watcher->setFuture(runAsync(extraCompilerThreadPool(),
                                          &ProcessExtraCompiler::runInThread, this,
                                          command(), workingDirectory(), arguments(), provider,
                                          buildEnvironment()));
+    return m_watcher->future();
 }
 
 void ProcessExtraCompiler::runInThread(
         QFutureInterface<FileNameToContentsHash> &futureInterface,
-        const Utils::FilePath &cmd, const Utils::FilePath &workDir,
+        const FilePath &cmd, const FilePath &workDir,
         const QStringList &args, const ContentProvider &provider,
-        const Utils::Environment &env)
+        const Environment &env)
 {
     if (cmd.isEmpty() || !cmd.toFileInfo().isExecutable())
         return;
@@ -415,29 +411,22 @@ void ProcessExtraCompiler::runInThread(
     if (sourceContents.isNull() || !prepareToRun(sourceContents))
         return;
 
-    QProcess process;
+    QtcProcess process;
 
-    process.setProcessEnvironment(env.toProcessEnvironment());
+    process.setEnvironment(env);
     if (!workDir.isEmpty())
         process.setWorkingDirectory(workDir.toString());
-    process.start(cmd.toString(), args, QIODevice::ReadWrite);
-    if (!process.waitForStarted()) {
-        handleProcessError(&process);
+    process.setCommand({ cmd, args });
+    process.setWriteData(sourceContents);
+    process.start();
+    if (!process.waitForStarted())
         return;
-    }
-    bool isCanceled = futureInterface.isCanceled();
-    if (!isCanceled) {
-        handleProcessStarted(&process, sourceContents);
-        forever {
-            bool done = process.waitForFinished(200) || process.state() == QProcess::NotRunning;
-            isCanceled = futureInterface.isCanceled();
-            if (done || isCanceled)
-                break;
-        }
-    }
 
-    isCanceled |= process.state() == QProcess::Running;
-    if (isCanceled) {
+    while (!futureInterface.isCanceled())
+        if (process.waitForFinished(200))
+            break;
+
+    if (futureInterface.isCanceled()) {
         process.kill();
         process.waitForFinished();
         return;
@@ -465,14 +454,4 @@ void ProcessExtraCompiler::cleanUp()
     setCompileTime(QDateTime::currentDateTime());
 }
 
-ExtraCompilerFactoryObserver::ExtraCompilerFactoryObserver()
-{
-    observers->push_back(this);
-}
-
-ExtraCompilerFactoryObserver::~ExtraCompilerFactoryObserver()
-{
-    observers->removeOne(this);
-}
-
 } // namespace ProjectExplorer
