@@ -75,6 +75,8 @@
 #include <QToolButton>
 #include <QVBoxLayout>
 
+using namespace Utils;
+
 const int PATH_ROLE = Qt::UserRole;
 const int ID_ROLE = Qt::UserRole + 1;
 const int SORT_ROLE = Qt::UserRole + 2;
@@ -96,6 +98,7 @@ static FolderNavigationWidgetFactory *m_instance = nullptr;
 QVector<FolderNavigationWidgetFactory::RootDirectory>
     FolderNavigationWidgetFactory::m_rootDirectories;
 
+Utils::FilePath FolderNavigationWidgetFactory::m_fallbackSyncFilePath;
 
 static QWidget *createHLine()
 {
@@ -204,7 +207,7 @@ static QVector<FolderNode *> renamableFolderNodes(const Utils::FilePath &before,
         if (node->asFileNode()
                 && node->filePath() == before
                 && node->parentFolderNode()
-                && node->parentFolderNode()->canRenameFile(before.toString(), after.toString())) {
+                && node->parentFolderNode()->canRenameFile(before, after)) {
             folderNodes.append(node->parentFolderNode());
         }
     });
@@ -225,9 +228,9 @@ bool FolderNavigationModel::setData(const QModelIndex &index, const QVariant &va
                    && role == Qt::EditRole && value.canConvert<QString>(),
                return false);
     const QString afterFileName = value.toString();
-    const QString beforeFilePath = filePath(index);
-    const QString parentPath = filePath(parent(index));
-    const QString afterFilePath = parentPath + '/' + afterFileName;
+    const Utils::FilePath beforeFilePath = Utils::FilePath::fromString(filePath(index));
+    const Utils::FilePath parentPath = Utils::FilePath::fromString(filePath(parent(index)));
+    const Utils::FilePath afterFilePath = parentPath.pathAppended(afterFileName);
     if (beforeFilePath == afterFilePath)
         return false;
     // need to rename through file system model, which takes care of not changing our selection
@@ -235,9 +238,8 @@ bool FolderNavigationModel::setData(const QModelIndex &index, const QVariant &va
     // for files we can do more than just rename on disk, for directories the user is on his/her own
     if (success && fileInfo(index).isFile()) {
         Core::DocumentManager::renamedFile(beforeFilePath, afterFilePath);
-        const QVector<FolderNode *> folderNodes
-            = renamableFolderNodes(Utils::FilePath::fromString(beforeFilePath),
-                                   Utils::FilePath::fromString(afterFilePath));
+        const QVector<FolderNode *> folderNodes = renamableFolderNodes(beforeFilePath,
+                                                                       afterFilePath);
         QVector<FolderNode *> failedNodes;
         for (FolderNode *folder : folderNodes) {
             if (!folder->renameFile(beforeFilePath, afterFilePath))
@@ -248,7 +250,7 @@ bool FolderNavigationModel::setData(const QModelIndex &index, const QVariant &va
             const QString errorMessage
                 = FolderNavigationWidget::tr("The file \"%1\" was renamed to \"%2\", "
                      "but the following projects could not be automatically changed: %3")
-                      .arg(beforeFilePath, afterFilePath, projects);
+                      .arg(beforeFilePath.toUserOutput(), afterFilePath.toUserOutput(), projects);
             QTimer::singleShot(0, Core::ICore::instance(), [errorMessage] {
                 QMessageBox::warning(Core::ICore::dialogParent(),
                                      ProjectExplorerPlugin::tr("Project Editing Failed"),
@@ -421,7 +423,7 @@ FolderNavigationWidget::FolderNavigationWidget(QWidget *parent) : QWidget(parent
             this,
             [this](int index) {
                 const auto directory = m_rootSelector->itemData(index).value<Utils::FilePath>();
-                m_rootSelector->setToolTip(directory.toString());
+                m_rootSelector->setToolTip(directory.toUserOutput());
                 setRootDirectory(directory);
                 const QModelIndex rootIndex = m_sortProxyModel->mapToSource(m_listView->rootIndex());
                 const QModelIndex fileIndex = m_sortProxyModel->mapToSource(m_listView->currentIndex());
@@ -484,8 +486,13 @@ void FolderNavigationWidget::insertRootDirectory(
         m_rootSelector->setCurrentIndex(index);
     if (previousIndex < m_rootSelector->count())
         m_rootSelector->removeItem(previousIndex);
-    if (m_autoSync) // we might find a better root for current selection now
-        handleCurrentEditorChanged(Core::EditorManager::currentEditor());
+    if (Core::EditorManager::currentEditor()) {
+        if (m_autoSync) // we might find a better root for current selection now
+            handleCurrentEditorChanged(Core::EditorManager::currentEditor());
+    } else if (m_rootAutoSync) {
+        // assume the new root is better (e.g. because a project was opened)
+        m_rootSelector->setCurrentIndex(index);
+    }
 }
 
 void FolderNavigationWidget::removeRootDirectory(const QString &id)
@@ -511,7 +518,7 @@ void FolderNavigationWidget::addNewItem()
                                    Utils::filtered(Core::IWizardFactory::allWizardFactories(),
                                                    Utils::equal(&Core::IWizardFactory::kind,
                                                                 Core::IWizardFactory::FileWizard)),
-                                   path.toString());
+                                   path);
 }
 
 void FolderNavigationWidget::editCurrentItem()
@@ -540,18 +547,17 @@ void FolderNavigationWidget::removeCurrentItem()
     const QModelIndex current = m_sortProxyModel->mapToSource(m_listView->currentIndex());
     if (!current.isValid() || m_fileSystemModel->isDir(current))
         return;
-    const QString filePath = m_fileSystemModel->filePath(current);
-    Utils::RemoveFileDialog dialog(filePath, Core::ICore::dialogParent());
+    const FilePath filePath = FilePath::fromString(m_fileSystemModel->filePath(current));
+    RemoveFileDialog dialog(filePath, Core::ICore::dialogParent());
     dialog.setDeleteFileVisible(false);
     if (dialog.exec() == QDialog::Accepted) {
-        const QVector<FolderNode *> folderNodes = removableFolderNodes(
-            Utils::FilePath::fromString(filePath));
+        const QVector<FolderNode *> folderNodes = removableFolderNodes(filePath);
         const QVector<FolderNode *> failedNodes = Utils::filtered(folderNodes,
                 [filePath](FolderNode *folder) {
                     return folder->removeFiles({filePath}) != RemovedFilesFromProject::Ok;
         });
         Core::FileChangeBlocker changeGuard(filePath);
-        Core::FileUtils::removeFile(filePath, true /*delete from disk*/);
+        Core::FileUtils::removeFiles({filePath}, true /*delete from disk*/);
         if (!failedNodes.isEmpty()) {
             const QString projects = projectNames(failedNodes).join(", ");
             const QString errorMessage
@@ -566,6 +572,15 @@ void FolderNavigationWidget::removeCurrentItem()
     }
 }
 
+void FolderNavigationWidget::syncWithFilePath(const Utils::FilePath &filePath)
+{
+    if (filePath.isEmpty())
+        return;
+    if (m_rootAutoSync)
+        selectBestRootForFile(filePath);
+    selectFile(filePath);
+}
+
 bool FolderNavigationWidget::autoSynchronization() const
 {
     return m_autoSync;
@@ -598,10 +613,7 @@ void FolderNavigationWidget::handleCurrentEditorChanged(Core::IEditor *editor)
     if (!m_autoSync || !editor || editor->document()->filePath().isEmpty()
             || editor->document()->isTemporary())
         return;
-    const Utils::FilePath filePath = editor->document()->filePath();
-    if (m_rootAutoSync)
-        selectBestRootForFile(filePath);
-    selectFile(filePath);
+    syncWithFilePath(editor->document()->filePath());
 }
 
 void FolderNavigationWidget::selectBestRootForFile(const Utils::FilePath &filePath)
@@ -647,7 +659,8 @@ int FolderNavigationWidget::bestRootForFile(const Utils::FilePath &filePath)
     int commonLength = 0;
     for (int i = 1; i < m_rootSelector->count(); ++i) {
         const auto root = m_rootSelector->itemData(i).value<Utils::FilePath>();
-        if (filePath.isChildOf(root) && root.toString().size() > commonLength) {
+        if ((filePath == root || filePath.isChildOf(root))
+                && root.toString().size() > commonLength) {
             index = i;
             commonLength = root.toString().size();
         }
@@ -681,7 +694,7 @@ void FolderNavigationWidget::openProjectsInDirectory(const QModelIndex &index)
 {
     const QStringList projectFiles = projectsInDirectory(index);
     if (!projectFiles.isEmpty())
-        Core::ICore::instance()->openFiles(projectFiles);
+        Core::ICore::openFiles(Utils::transform(projectFiles, &FilePath::fromString));
 }
 
 void FolderNavigationWidget::createNewFolder(const QModelIndex &parent)
@@ -794,7 +807,7 @@ void FolderNavigationWidget::contextMenuEvent(QContextMenuEvent *ev)
     if (action == actionOpenFile) {
         openItem(current);
     } else if (action == actionOpenAsProject) {
-        ProjectExplorerPlugin::openProject(filePath.toString());
+        ProjectExplorerPlugin::openProject(filePath);
     } else if (action == actionOpenProjects)
         openProjectsInDirectory(current);
     else if (action == newFolder) {
@@ -854,7 +867,8 @@ FolderNavigationWidgetFactory::FolderNavigationWidgetFactory()
     setDisplayName(tr("File System"));
     setPriority(400);
     setId("File System");
-    setActivationSequence(QKeySequence(Core::useMacShortcuts ? tr("Meta+Y") : tr("Alt+Y")));
+    setActivationSequence(QKeySequence(Core::useMacShortcuts ? tr("Meta+Y,Meta+F")
+                                                             : tr("Alt+Y,Alt+F")));
     insertRootDirectory({QLatin1String("A.Computer"),
                          0 /*sortValue*/,
                          FolderNavigationWidget::tr("Computer"),
@@ -876,7 +890,7 @@ FolderNavigationWidgetFactory::FolderNavigationWidgetFactory()
 Core::NavigationView FolderNavigationWidgetFactory::createWidget()
 {
     auto fnw = new FolderNavigationWidget;
-    for (const RootDirectory &root : m_rootDirectories)
+    for (const RootDirectory &root : qAsConst(m_rootDirectories))
         fnw->insertRootDirectory(root);
     connect(this,
             &FolderNavigationWidgetFactory::rootDirectoryAdded,
@@ -886,6 +900,8 @@ Core::NavigationView FolderNavigationWidgetFactory::createWidget()
             &FolderNavigationWidgetFactory::rootDirectoryRemoved,
             fnw,
             &FolderNavigationWidget::removeRootDirectory);
+    if (!Core::EditorManager::currentDocument() && !m_fallbackSyncFilePath.isEmpty())
+        fnw->syncWithFilePath(m_fallbackSyncFilePath);
 
     Core::NavigationView n;
     n.widget = fnw;
@@ -903,15 +919,28 @@ Core::NavigationView FolderNavigationWidgetFactory::createWidget()
     return n;
 }
 
-void FolderNavigationWidgetFactory::saveSettings(QSettings *settings, int position, QWidget *widget)
+const bool kHiddenFilesDefault = false;
+const bool kAutoSyncDefault = true;
+const bool kShowBreadCrumbsDefault = true;
+const bool kRootAutoSyncDefault = true;
+
+void FolderNavigationWidgetFactory::saveSettings(Utils::QtcSettings *settings,
+                                                 int position,
+                                                 QWidget *widget)
 {
     auto fnw = qobject_cast<FolderNavigationWidget *>(widget);
     QTC_ASSERT(fnw, return);
     const QString base = kSettingsBase + QString::number(position);
-    settings->setValue(base + kHiddenFilesKey, fnw->hiddenFilesFilter());
-    settings->setValue(base + kSyncKey, fnw->autoSynchronization());
-    settings->setValue(base + kShowBreadCrumbs, fnw->isShowingBreadCrumbs());
-    settings->setValue(base + kSyncRootWithEditor, fnw->rootAutoSynchronization());
+    settings->setValueWithDefault(base + kHiddenFilesKey,
+                                  fnw->hiddenFilesFilter(),
+                                  kHiddenFilesDefault);
+    settings->setValueWithDefault(base + kSyncKey, fnw->autoSynchronization(), kAutoSyncDefault);
+    settings->setValueWithDefault(base + kShowBreadCrumbs,
+                                  fnw->isShowingBreadCrumbs(),
+                                  kShowBreadCrumbsDefault);
+    settings->setValueWithDefault(base + kSyncRootWithEditor,
+                                  fnw->rootAutoSynchronization(),
+                                  kRootAutoSyncDefault);
 }
 
 void FolderNavigationWidgetFactory::restoreSettings(QSettings *settings, int position, QWidget *widget)
@@ -919,10 +948,12 @@ void FolderNavigationWidgetFactory::restoreSettings(QSettings *settings, int pos
     auto fnw = qobject_cast<FolderNavigationWidget *>(widget);
     QTC_ASSERT(fnw, return);
     const QString base = kSettingsBase + QString::number(position);
-    fnw->setHiddenFilesFilter(settings->value(base + kHiddenFilesKey, false).toBool());
-    fnw->setAutoSynchronization(settings->value(base + kSyncKey, true).toBool());
-    fnw->setShowBreadCrumbs(settings->value(base + kShowBreadCrumbs, true).toBool());
-    fnw->setRootAutoSynchronization(settings->value(base + kSyncRootWithEditor, true).toBool());
+    fnw->setHiddenFilesFilter(settings->value(base + kHiddenFilesKey, kHiddenFilesDefault).toBool());
+    fnw->setAutoSynchronization(settings->value(base + kSyncKey, kAutoSyncDefault).toBool());
+    fnw->setShowBreadCrumbs(
+        settings->value(base + kShowBreadCrumbs, kShowBreadCrumbsDefault).toBool());
+    fnw->setRootAutoSynchronization(
+        settings->value(base + kSyncRootWithEditor, kRootAutoSyncDefault).toBool());
 }
 
 void FolderNavigationWidgetFactory::insertRootDirectory(const RootDirectory &directory)
@@ -943,6 +974,11 @@ void FolderNavigationWidgetFactory::removeRootDirectory(const QString &id)
     emit m_instance->rootDirectoryRemoved(id);
 }
 
+void FolderNavigationWidgetFactory::setFallbackSyncFilePath(const FilePath &filePath)
+{
+    m_fallbackSyncFilePath = filePath;
+}
+
 int FolderNavigationWidgetFactory::rootIndex(const QString &id)
 {
     return Utils::indexOf(m_rootDirectories,
