@@ -32,35 +32,24 @@
 
 namespace ProjectExplorer {
 
-class PROJECTEXPLORER_EXPORT GnuMakeParser : public ProjectExplorer::IOutputParser
+class PROJECTEXPLORER_EXPORT GnuMakeParser : public ProjectExplorer::OutputTaskParser
 {
     Q_OBJECT
 
 public:
     explicit GnuMakeParser();
 
-    void stdOutput(const QString &line) override;
-    void stdError(const QString &line) override;
-
-    void setWorkingDirectory(const QString &workingDirectory) override;
-
-    QStringList searchDirectories() const;
-
+private:
+    Result handleLine(const QString &line, Utils::OutputFormat type) override;
     bool hasFatalErrors() const override;
 
-    void taskAdded(const ProjectExplorer::Task &task, int linkedLines, int skippedLines) override;
-
-private:
-    void addDirectory(const QString &dir);
-    void removeDirectory(const QString &dir);
+    void emitTask(const ProjectExplorer::Task &task);
 
     QRegularExpression m_makeDir;
     QRegularExpression m_makeLine;
     QRegularExpression m_threeStarError;
     QRegularExpression m_errorInMakefile;
 
-    QStringList m_directories;
-
     bool m_suppressIssues = false;
 
     int m_fatalErrorCount = 0;
@@ -79,7 +68,7 @@ public:
     explicit GnuMakeParserTester(GnuMakeParser *parser, QObject *parent = nullptr);
     void parserIsAboutToBeDeleted();
 
-    QStringList directories;
+    Utils::FilePaths directories;
     GnuMakeParser *parser;
 };
 #endif
