@@ -54,7 +54,6 @@
 #include <QLabel>
 #include <QListView>
 #include <QRegularExpression>
-#include <QRegularExpressionValidator>
 #include <QStandardItem>
 #include <QTextEdit>
 #include <QVariant>
@@ -63,6 +62,8 @@
 
 using namespace Utils;
 
+namespace ProjectExplorer {
+
 const char NAME_KEY[] = "name";
 const char DISPLAY_NAME_KEY[] = "trDisplayName";
 const char TOOLTIP_KEY[] = "trToolTip";
@@ -76,8 +77,7 @@ const char DATA_KEY[] = "data";
 const char IS_COMPLETE_KEY[] = "isComplete";
 const char IS_COMPLETE_MESSAGE_KEY[] = "trIncompleteMessage";
 
-namespace {
-QVariant consumeValue(QVariantMap &map, const QString &key, const QVariant &defaultValue = QVariant())
+static QVariant consumeValue(QVariantMap &map, const QString &key, const QVariant &defaultValue = {})
 {
     QVariantMap::iterator i = map.find(key);
     if (i != map.end()) {
@@ -88,7 +88,7 @@ QVariant consumeValue(QVariantMap &map, const QString &key, const QVariant &defa
     return defaultValue;
 }
 
-void warnAboutUnsupportedKeys(const QVariantMap &map, const QString &name, const QString &type = QString())
+static void warnAboutUnsupportedKeys(const QVariantMap &map, const QString &name, const QString &type = {})
 {
     if (!map.isEmpty()) {
 
@@ -99,45 +99,38 @@ void warnAboutUnsupportedKeys(const QVariantMap &map, const QString &name, const
         qWarning().noquote() << QString("Field %1 has unsupported keys: %2").arg(typeAndName, map.keys().join(", "));
     }
 }
-} // namespace
 
-namespace ProjectExplorer {
 
 // --------------------------------------------------------------------
 // Helper:
 // --------------------------------------------------------------------
 
-class LineEditValidator : public QRegularExpressionValidator
+class LineEdit : public FancyLineEdit
 {
 public:
-    LineEditValidator(MacroExpander *expander, const QRegularExpression &pattern, QObject *parent) :
-        QRegularExpressionValidator(pattern, parent)
+    LineEdit(MacroExpander *expander, const QRegularExpression &pattern)
     {
+        if (pattern.pattern().isEmpty() || !pattern.isValid())
+            return;
         m_expander.setDisplayName(JsonFieldPage::tr("Line Edit Validator Expander"));
         m_expander.setAccumulating(true);
         m_expander.registerVariable("INPUT", JsonFieldPage::tr("The text edit input to fix up."),
                                     [this]() { return m_currentInput; });
         m_expander.registerSubProvider([expander]() -> MacroExpander * { return expander; });
+        setValidationFunction([this, pattern](FancyLineEdit *, QString *) {
+            return pattern.match(text()).hasMatch();
+        });
     }
 
-    void setFixupExpando(const QString &expando)
-    {
-        m_fixupExpando = expando;
-    }
+    void setFixupExpando(const QString &expando) { m_fixupExpando = expando; }
 
-    QValidator::State validate(QString &input, int &pos) const override
-    {
-        fixup(input);
-        return QRegularExpressionValidator::validate(input, pos);
-    }
-
-    void fixup(QString &fixup) const override
+private:
+    QString fixInputString(const QString &string) override
     {
         if (m_fixupExpando.isEmpty())
-            return;
-
-        m_currentInput = fixup;
-        fixup = m_expander.expand(m_fixupExpando);
+            return string;
+        m_currentInput = string;
+        return m_expander.expand(m_fixupExpando);
     }
 
 private:
@@ -159,7 +152,7 @@ JsonFieldPage::Field::~Field()
     delete d->m_label;
 }
 
-QString JsonFieldPage::Field::type()
+QString JsonFieldPage::Field::type() const
 {
     return d->m_type;
 }
@@ -307,17 +300,17 @@ QWidget *JsonFieldPage::Field::widget(const QString &displayName, JsonFieldPage
     return d->m_widget;
 }
 
-QString JsonFieldPage::Field::name()
+QString JsonFieldPage::Field::name() const
 {
     return d->m_name;
 }
 
-QString JsonFieldPage::Field::displayName()
+QString JsonFieldPage::Field::displayName() const
 {
     return d->m_displayName;
 }
 
-QString JsonFieldPage::Field::toolTip()
+QString JsonFieldPage::Field::toolTip() const
 {
     return d->m_toolTip;
 }
@@ -327,12 +320,12 @@ QString JsonFieldPage::Field::persistenceKey() const
     return d->m_persistenceKey;
 }
 
-bool JsonFieldPage::Field::isMandatory()
+bool JsonFieldPage::Field::isMandatory() const
 {
     return d->m_isMandatory;
 }
 
-bool JsonFieldPage::Field::hasSpan()
+bool JsonFieldPage::Field::hasSpan() const
 {
     return d->m_hasSpan;
 }
@@ -357,11 +350,11 @@ QWidget *JsonFieldPage::Field::widget() const
     return d->m_widget;
 }
 
-void JsonFieldPage::Field::setTexts(const QString &n, const QString &dn, const QString &tt)
+void JsonFieldPage::Field::setTexts(const QString &name, const QString &displayName, const QString &toolTip)
 {
-    d->m_name = n;
-    d->m_displayName = dn;
-    d->m_toolTip = tt;
+    d->m_name = name;
+    d->m_displayName = displayName;
+    d->m_toolTip = toolTip;
 }
 
 void JsonFieldPage::Field::setIsMandatory(bool b)
@@ -395,6 +388,29 @@ void JsonFieldPage::Field::setPersistenceKey(const QString &key)
     d->m_persistenceKey = key;
 }
 
+inline QDebug &operator<<(QDebug &debug, const JsonFieldPage::Field::FieldPrivate &field)
+{
+    debug << "name:" << field.m_name
+          << "; displayName:" << field.m_displayName
+          << "; type:" << field.m_type
+          << "; mandatory:" << field.m_isMandatory
+          << "; hasUserChanges:" << field.m_hasUserChanges
+          << "; visibleExpression:" << field.m_visibleExpression
+          << "; enabledExpression:" << field.m_enabledExpression
+          << "; isComplete:" << field.m_isCompleteExpando
+          << "; isCompleteMessage:" << field.m_isCompleteExpandoMessage
+          << "; persistenceKey:" << field.m_persistenceKey;
+
+    return debug;
+}
+
+QDebug &operator<<(QDebug &debug, const JsonFieldPage::Field &field)
+{
+    debug << "Field{_: " << *field.d << "; subclass: " << field.toString() << "}";
+
+    return debug;
+}
+
 // --------------------------------------------------------------------
 // LabelFieldData:
 // --------------------------------------------------------------------
@@ -508,7 +524,7 @@ bool LineEditField::parseData(const QVariant &data, QString *errorMessage)
     m_restoreLastHistoryItem = consumeValue(tmp, "restoreLastHistoryItem", false).toBool();
     QString pattern = consumeValue(tmp, "validator").toString();
     if (!pattern.isEmpty()) {
-        m_validatorRegExp = QRegularExpression(pattern);
+        m_validatorRegExp = QRegularExpression('^' + pattern + '$');
         if (!m_validatorRegExp.isValid()) {
             *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonFieldPage",
                                                         "LineEdit (\"%1\") has an invalid regular expression \"%2\" in \"validator\".")
@@ -539,13 +555,8 @@ bool LineEditField::parseData(const QVariant &data, QString *errorMessage)
 QWidget *LineEditField::createWidget(const QString &displayName, JsonFieldPage *page)
 {
     Q_UNUSED(displayName)
-    auto w = new FancyLineEdit;
-
-    if (m_validatorRegExp.isValid()) {
-        auto lv = new LineEditValidator(page->expander(), m_validatorRegExp, w);
-        lv->setFixupExpando(m_fixupExpando);
-        w->setValidator(lv);
-    }
+    const auto w = new LineEdit(page->expander(), m_validatorRegExp);
+    w->setFixupExpando(m_fixupExpando);
 
     if (!m_historyId.isEmpty())
         w->setHistoryCompleter(m_historyId, m_restoreLastHistoryItem);
@@ -592,7 +603,7 @@ bool LineEditField::validate(MacroExpander *expander, QString *message)
 
     const bool baseValid = JsonFieldPage::Field::validate(expander, message);
     m_isValidating = false;
-    return baseValid && !w->text().isEmpty();
+    return baseValid && !w->text().isEmpty() && w->isValid();
 }
 
 void LineEditField::initializeData(MacroExpander *expander)
@@ -656,7 +667,7 @@ void LineEditField::setupCompletion(FancyLineEdit *lineEdit)
                     classes << (entry.extraInfo + "::" + entry.displayName);
                 if (m_completion == Completion::Namespaces) {
                     if (!project
-                            || entry.fileName.startsWith(project->projectDirectory().toString())) {
+                            || entry.filePath.startsWith(project->projectDirectory().toString())) {
                         namespaces << entry.extraInfo;
                     }
                 }
@@ -689,6 +700,14 @@ void LineEditField::setupCompletion(FancyLineEdit *lineEdit)
     }));
 }
 
+void LineEditField::setText(const QString &text)
+{
+    m_currentText = text;
+
+    auto w = qobject_cast<FancyLineEdit *>(widget());
+    w->setText(m_currentText);
+}
+
 // --------------------------------------------------------------------
 // TextEditFieldData:
 // --------------------------------------------------------------------
@@ -791,8 +810,8 @@ bool PathChooserField::parseData(const QVariant &data, QString *errorMessage)
 
     QVariantMap tmp = data.toMap();
 
-    m_path = consumeValue(tmp, "path").toString();
-    m_basePath = consumeValue(tmp, "basePath").toString();
+    m_path = FilePath::fromVariant(consumeValue(tmp, "path"));
+    m_basePath = FilePath::fromVariant(consumeValue(tmp, "basePath"));
     m_historyId = consumeValue(tmp, "historyId").toString();
 
     QString kindStr = consumeValue(tmp, "kind", "existingDirectory").toString();
@@ -831,7 +850,7 @@ QWidget *PathChooserField::createWidget(const QString &displayName, JsonFieldPag
     if (!m_historyId.isEmpty())
         w->setHistoryCompleter(m_historyId);
     QObject::connect(w, &PathChooser::pathChanged, [this, w] {
-        if (w->filePath().toString() != m_path)
+        if (w->filePath() != m_path)
             setHasUserChanges();
     });
     return w;
@@ -867,23 +886,19 @@ void PathChooserField::initializeData(MacroExpander *expander)
 {
     auto w = qobject_cast<PathChooser *>(widget());
     QTC_ASSERT(w, return);
-    w->setBaseDirectory(expander->expand(FilePath::fromString(m_basePath)));
+    w->setBaseDirectory(expander->expand(m_basePath));
     w->setExpectedKind(m_kind);
-
-    if (m_currentPath.isNull())
-        w->setPath(expander->expand(m_path));
-    else
-        w->setPath(m_currentPath);
+    w->setFilePath(expander->expand(m_path));
 }
 
 void PathChooserField::fromSettings(const QVariant &value)
 {
-    m_path = value.toString();
+    m_path = FilePath::fromVariant(value);
 }
 
 QVariant PathChooserField::toSettings() const
 {
-    return qobject_cast<PathChooser *>(widget())->filePath().toString();
+    return qobject_cast<PathChooser *>(widget())->filePath().toVariant();
 }
 
 // --------------------------------------------------------------------
@@ -907,7 +922,7 @@ bool CheckBoxField::parseData(const QVariant &data, QString *errorMessage)
     m_checkedValue = consumeValue(tmp, "checkedValue", true).toString();
     m_uncheckedValue = consumeValue(tmp, "uncheckedValue", false).toString();
     if (m_checkedValue == m_uncheckedValue) {
-        *errorMessage= QCoreApplication::translate("ProjectExplorer::JsonFieldPage",
+        *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonFieldPage",
                                                    "CheckBox (\"%1\") values for checked and unchecked state are identical.")
                 .arg(name());
        return false;
@@ -941,6 +956,15 @@ void CheckBoxField::setup(JsonFieldPage *page, const QString &name)
     });
 }
 
+void CheckBoxField::setChecked(bool value)
+{
+    auto w = qobject_cast<QCheckBox *>(widget());
+    QTC_ASSERT(w, return);
+
+    w->setChecked(value);
+    w->clicked(value);
+}
+
 bool CheckBoxField::validate(MacroExpander *expander, QString *message)
 {
     if (!JsonFieldPage::Field::validate(expander, message))
@@ -979,8 +1003,8 @@ QVariant CheckBoxField::toSettings() const
 std::unique_ptr<QStandardItem> createStandardItemFromListItem(const QVariant &item, QString *errorMessage)
 {
     if (item.type() == QVariant::List) {
-        *errorMessage  = QCoreApplication::translate("ProjectExplorer::JsonFieldPage",
-                                                     "No JSON lists allowed inside List items.");
+        *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonFieldPage",
+                                                    "No JSON lists allowed inside List items.");
         return {};
     }
     auto standardItem = std::make_unique<QStandardItem>();
@@ -1112,7 +1136,7 @@ void ListField::initializeData(MacroExpander *expander)
                     qWarning().noquote() << QString("Icon file \"%1\" not found.").arg(QDir::toNativeSeparators(iconPath));
                 }
             } else {
-                qWarning().noquote() <<  QString("%1 (\"%2\") has no parentWidget JsonFieldPage to get the icon path.").arg(type(), name());
+                qWarning().noquote() << QString("%1 (\"%2\") has no parentWidget JsonFieldPage to get the icon path.").arg(type(), name());
             }
         }
         expandedValuesItems.append(expandedValuesItem);
@@ -1133,6 +1157,14 @@ QStandardItemModel *ListField::itemModel()
     return m_itemModel;
 }
 
+void ListField::selectRow(int row)
+{
+    auto index = itemModel()->index(row, 0);
+    selectionModel()->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect);
+
+    this->updateIndex();
+}
+
 QItemSelectionModel *ListField::selectionModel() const
 {
     return m_selectionModel;
@@ -1143,7 +1175,7 @@ void ListField::setSelectionModel(QItemSelectionModel *selectionModel)
     m_selectionModel = selectionModel;
 }
 
-QSize ListField::maxIconSize()
+QSize ListField::maxIconSize() const
 {
     return m_maxIconSize;
 }
@@ -1184,7 +1216,7 @@ QVariant ListField::toSettings() const
 
 void ComboBoxField::setup(JsonFieldPage *page, const QString &name)
 {
-    auto w = qobject_cast<QComboBox*>(widget());
+    auto w = qobject_cast<QComboBox *>(widget());
     QTC_ASSERT(w, return);
     w->setModel(itemModel());
     w->setInsertPolicy(QComboBox::NoInsert);
@@ -1228,17 +1260,31 @@ void ComboBoxField::initializeData(MacroExpander *expander)
 {
     ListField::initializeData(expander);
     // refresh also the current text of the combobox
-    auto w = qobject_cast<QComboBox*>(widget());
+    auto w = qobject_cast<QComboBox *>(widget());
     w->setCurrentIndex(selectionModel()->currentIndex().row());
 }
 
 QVariant ComboBoxField::toSettings() const
 {
-    if (auto w = qobject_cast<QComboBox*>(widget()))
+    if (auto w = qobject_cast<QComboBox *>(widget()))
         return w->currentData(ValueRole);
     return {};
 }
 
+void ComboBoxField::selectRow(int row)
+{
+    ListField::selectRow(row);
+
+    auto w = qobject_cast<QComboBox *>(widget());
+    w->setCurrentIndex(row);
+}
+
+int ComboBoxField::selectedRow() const
+{
+    auto w = qobject_cast<QComboBox *>(widget());
+    return w->currentIndex();
+}
+
 void IconListField::setup(JsonFieldPage *page, const QString &name)
 {
     auto w = qobject_cast<QListView*>(widget());
@@ -1280,7 +1326,7 @@ void IconListField::initializeData(MacroExpander *expander)
     w->setSpacing(spacing);
     w->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
-    // adding a third hight of the icon to see following items if there are some
+    // adding 1/3 height of the icon to see following items if there are some
     w->setMinimumHeight(maxIconSize().height() + maxIconSize().height() / 3);
     w->setIconSize(maxIconSize());
 }
