@@ -423,7 +423,7 @@ void JsonWizard::handleNewPages(int pageId)
 
 void JsonWizard::handleError(const QString &message)
 {
-    Core::MessageManager::write(message, Core::MessageManager::ModeSwitch);
+    Core::MessageManager::writeDisrupting(message);
 }
 
 QString JsonWizard::stringify(const QVariant &v) const
@@ -447,18 +447,18 @@ void JsonWizard::openFiles(const JsonWizard::GeneratorFiles &files)
         if (!QFileInfo::exists(file.path())) {
             errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
                                                        "\"%1\" does not exist in the file system.")
-                    .arg(QDir::toNativeSeparators(file.path()));
+                    .arg(file.filePath().toUserOutput());
             break;
         }
         if (file.attributes() & Core::GeneratedFile::OpenProjectAttribute) {
             ProjectExplorerPlugin::OpenProjectResult result
-                    = ProjectExplorerPlugin::instance()->openProject(file.path());
+                    = ProjectExplorerPlugin::openProject(file.filePath());
             if (!result) {
                 errorMessage = result.errorMessage();
                 if (errorMessage.isEmpty()) {
                     errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
                                                                "Failed to open \"%1\" as a project.")
-                            .arg(QDir::toNativeSeparators(file.path()));
+                            .arg(file.filePath().toUserOutput());
                 }
                 break;
             }
@@ -470,7 +470,7 @@ void JsonWizard::openFiles(const JsonWizard::GeneratorFiles &files)
             if (!editor) {
                 errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
                                                            "Failed to open an editor for \"%1\".")
-                        .arg(QDir::toNativeSeparators(file.path()));
+                        .arg(file.filePath().toUserOutput());
                 break;
             } else if (file.attributes() & Core::GeneratedFile::TemporaryFile) {
                 editor->document()->setTemporary(true);
@@ -514,7 +514,7 @@ void JsonWizard::openProjectForNode(Node *node)
 
     Utils::optional<FilePath> projFilePath = projNode->visibleAfterAddFileAction();
 
-    if (projFilePath && !Core::EditorManager::openEditor(projFilePath.value().toString())) {
+    if (projFilePath && !Core::EditorManager::openEditor(projFilePath.value())) {
             auto errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
                                                        "Failed to open an editor for \"%1\".")
                     .arg(QDir::toNativeSeparators(projFilePath.value().toString()));
