@@ -34,7 +34,7 @@
 
 #include <utils/algorithm.h>
 #include <utils/displayname.h>
-#include <utils/fileutils.h>
+#include <utils/filepath.h>
 #include <utils/icon.h>
 #include <utils/macroexpander.h>
 #include <utils/optional.h>
@@ -42,10 +42,7 @@
 #include <utils/stringutils.h>
 #include <utils/utilsicons.h>
 
-#include <QApplication>
-#include <QFileInfo>
 #include <QIcon>
-#include <QStyle>
 #include <QTextStream>
 #include <QUuid>
 
@@ -98,20 +95,30 @@ public:
         for (KitAspect *aspect : KitManager::kitAspects())
             aspect->addToMacroExpander(kit, &m_macroExpander);
 
-        // This provides the same global fall back as the global expander
-        // without relying on the currentKit() discovery process there.
-        m_macroExpander.registerVariable(Constants::VAR_CURRENTKIT_NAME,
+        // TODO: Remove the "Current" variants in ~4.16
+        m_macroExpander.registerVariable("CurrentKit:Name",
             tr("The name of the currently active kit."),
             [kit] { return kit->displayName(); },
             false);
-        m_macroExpander.registerVariable(Constants::VAR_CURRENTKIT_FILESYSTEMNAME,
+        m_macroExpander.registerVariable("Kit:Name",
+            tr("The name of the kit."),
+            [kit] { return kit->displayName(); });
+
+        m_macroExpander.registerVariable("CurrentKit:FileSystemName",
             tr("The name of the currently active kit in a filesystem-friendly version."),
             [kit] { return kit->fileSystemFriendlyName(); },
             false);
-        m_macroExpander.registerVariable(Constants::VAR_CURRENTKIT_ID,
+        m_macroExpander.registerVariable("Kit:FileSystemName",
+            tr("The name of the kit in a filesystem-friendly version."),
+            [kit] { return kit->fileSystemFriendlyName(); });
+
+        m_macroExpander.registerVariable("CurrentKit:Id",
             tr("The id of the currently active kit."),
             [kit] { return kit->id().toString(); },
             false);
+        m_macroExpander.registerVariable("Kit:Id",
+            tr("The id of the kit."),
+            [kit] { return kit->id().toString(); });
     }
 
     DisplayName m_unexpandedDisplayName;
@@ -142,8 +149,13 @@ public:
 // Kit:
 // -------------------------------------------------------------------------
 
-Kit::Kit(Id id) :
-    d(std::make_unique<Internal::KitPrivate>(id, this))
+Kit::Predicate Kit::defaultPredicate()
+{
+    return [](const Kit *k) { return k->isValid(); };
+}
+
+Kit::Kit(Id id)
+    : d(std::make_unique<Internal::KitPrivate>(id, this))
 {
 }
 
@@ -367,7 +379,7 @@ int Kit::weight() const
     });
 }
 
-static QIcon iconForDeviceType(Core::Id deviceType)
+static QIcon iconForDeviceType(Utils::Id deviceType)
 {
     const IDeviceFactory *factory = Utils::findOrDefault(IDeviceFactory::allDeviceFactories(),
         [&deviceType](const IDeviceFactory *factory) {
@@ -386,7 +398,7 @@ QIcon Kit::icon() const
         return d->m_cachedIcon;
     }
 
-    const Core::Id deviceType = d->m_deviceTypeForIcon.isValid()
+    const Utils::Id deviceType = d->m_deviceTypeForIcon.isValid()
             ? d->m_deviceTypeForIcon : DeviceTypeKitAspect::deviceTypeId(this);
     const QIcon deviceTypeIcon = iconForDeviceType(deviceType);
     if (!deviceTypeIcon.isNull()) {
@@ -548,27 +560,50 @@ QVariantMap Kit::toMap() const
     return data;
 }
 
-void Kit::addToEnvironment(Environment &env) const
+void Kit::addToBuildEnvironment(Environment &env) const
 {
     for (KitAspect *aspect : KitManager::kitAspects())
-        aspect->addToEnvironment(this, env);
+        aspect->addToBuildEnvironment(this, env);
 }
 
-IOutputParser *Kit::createOutputParser() const
+void Kit::addToRunEnvironment(Environment &env) const
 {
-    auto first = new OsParser;
     for (KitAspect *aspect : KitManager::kitAspects())
-        first->appendOutputParser(aspect->createOutputParser(this));
-    return first;
+        aspect->addToRunEnvironment(this, env);
 }
 
-QString Kit::toHtml(const Tasks &additional) const
+Environment Kit::buildEnvironment() const
+{
+    Environment env = Environment::systemEnvironment(); // FIXME: Use build device
+    addToBuildEnvironment(env);
+    return env;
+}
+
+Environment Kit::runEnvironment() const
+{
+    Environment env = Environment::systemEnvironment(); // FIXME: Use run device
+    addToRunEnvironment(env);
+    return env;
+}
+
+QList<OutputLineParser *> Kit::createOutputParsers() const
+{
+    QList<OutputLineParser *> parsers{new OsParser};
+    for (KitAspect *aspect : KitManager::kitAspects())
+        parsers << aspect->createOutputParsers(this);
+    return parsers;
+}
+
+QString Kit::toHtml(const Tasks &additional, const QString &extraText) const
 {
     QString result;
     QTextStream str(&result);
     str << "<html><body>";
     str << "<h3>" << displayName() << "</h3>";
 
+    if (!extraText.isEmpty())
+        str << "<p>" << extraText << "</p>";
+
     if (!isValid() || hasWarning() || !additional.isEmpty())
         str << "<p>" << ProjectExplorer::toHtml(additional + validate()) << "</p>";
 
@@ -728,4 +763,17 @@ void Kit::kitUpdated()
     d->m_mustNotify = false;
 }
 
+
+static Id replacementKey() { return "IsReplacementKit"; }
+
+void ProjectExplorer::Kit::makeReplacementKit()
+{
+    setValueSilently(replacementKey(), true);
+}
+
+bool Kit::isReplacementKit() const
+{
+    return value(replacementKey()).toBool();
+}
+
 } // namespace ProjectExplorer
