@@ -39,6 +39,7 @@
 
 #include <utils/algorithm.h>
 #include <utils/itemviews.h>
+#include <utils/layoutbuilder.h>
 #include <utils/stringutils.h>
 #include <utils/styledbar.h>
 #include <utils/stylehelper.h>
@@ -106,7 +107,7 @@ public:
             return p->displayName();
         if (const auto t = qobject_cast<Target *>(object()))
             return t->displayName();
-        return static_cast<ProjectConfiguration *>(object())->displayName();
+        return static_cast<ProjectConfiguration *>(object())->expandedDisplayName();
 
     }
     QString displayName() const
@@ -237,6 +238,18 @@ public:
 protected:
     void resetOptimalWidth()
     {
+        if (m_resetScheduled)
+            return;
+        m_resetScheduled = true;
+        QMetaObject::invokeMethod(this, &SelectorView::doResetOptimalWidth, Qt::QueuedConnection);
+    }
+
+private:
+    void keyPressEvent(QKeyEvent *event) override;
+    void keyReleaseEvent(QKeyEvent *event) override;
+    void doResetOptimalWidth()
+    {
+        m_resetScheduled = false;
         int width = 0;
         QFontMetrics fn(font());
         theModel()->forItemsAtLevel<1>([this, &width, &fn](const GenericItem *item) {
@@ -245,12 +258,9 @@ protected:
         setOptimalWidth(width);
     }
 
-private:
-    void keyPressEvent(QKeyEvent *event) override;
-    void keyReleaseEvent(QKeyEvent *event) override;
-
     int m_maxCount = 0;
     int m_optimalWidth = 0;
+    bool m_resetScheduled = false;
 };
 
 class ProjectListView : public SelectorView
@@ -564,15 +574,14 @@ class KitAreaWidget : public QWidget
     Q_OBJECT
 public:
     explicit KitAreaWidget(QWidget *parent = nullptr)
-        : QWidget(parent), m_layout(new QGridLayout(this))
+        : QWidget(parent)
     {
-        m_layout->setContentsMargins(3, 3, 3, 3);
         connect(KitManager::instance(), &KitManager::kitUpdated, this, &KitAreaWidget::updateKit);
     }
 
     ~KitAreaWidget() override { setKit(nullptr); }
 
-    void setKit(ProjectExplorer::Kit *k)
+    void setKit(Kit *k)
     {
         qDeleteAll(m_widgets);
         m_widgets.clear();
@@ -580,25 +589,22 @@ public:
         if (!k)
             return;
 
-        foreach (QLabel *l, m_labels)
-            l->deleteLater();
-        m_labels.clear();
+        delete layout();
 
-        int row = 0;
+        LayoutBuilder builder(LayoutBuilder::GridLayout);
         for (KitAspect *aspect : KitManager::kitAspects()) {
             if (k && k->isMutable(aspect->id())) {
                 KitAspectWidget *widget = aspect->createConfigWidget(k);
                 m_widgets << widget;
                 QLabel *label = new QLabel(aspect->displayName());
-                m_labels << label;
-
-                m_layout->addWidget(label, row, 0);
-                m_layout->addWidget(widget->mainWidget(), row, 1);
-                m_layout->addWidget(widget->buttonWidget(), row, 2);
-
-                ++row;
+                builder.addItem(label);
+                widget->addToLayout(builder);
+                builder.finishRow();
             }
         }
+        builder.attachTo(this);
+        layout()->setContentsMargins(3, 3, 3, 3);
+
         m_kit = k;
 
         setHidden(m_widgets.isEmpty());
@@ -611,16 +617,17 @@ private:
             return;
 
         bool addedMutables = false;
-        QList<Utils::Id> knownIdList = Utils::transform(m_widgets, &KitAspectWidget::kitInformationId);
+        QList<const KitAspect *> knownList
+            = Utils::transform(m_widgets, &KitAspectWidget::kitInformation);
 
         for (KitAspect *aspect : KitManager::kitAspects()) {
             const Utils::Id currentId = aspect->id();
-            if (m_kit->isMutable(currentId) && !knownIdList.removeOne(currentId)) {
+            if (m_kit->isMutable(currentId) && !knownList.removeOne(aspect)) {
                 addedMutables = true;
                 break;
             }
         }
-        const bool removedMutables = !knownIdList.isEmpty();
+        const bool removedMutables = !knownList.isEmpty();
 
         if (addedMutables || removedMutables) {
             // Redo whole setup if the number of mutable settings did change
@@ -632,10 +639,8 @@ private:
         }
     }
 
-    QGridLayout *m_layout;
     Kit *m_kit = nullptr;
     QList<KitAspectWidget *> m_widgets;
-    QList<QLabel *> m_labels;
 };
 
 /////////
@@ -681,7 +686,7 @@ MiniProjectTargetSelector::MiniProjectTargetSelector(QAction *targetSelectorActi
     m_summaryLabel->setContentsMargins(3, 3, 3, 3);
     m_summaryLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
     QPalette pal = m_summaryLabel->palette();
-    pal.setColor(QPalette::Window, Utils::StyleHelper().baseColor());
+    pal.setColor(QPalette::Window, StyleHelper::baseColor());
     m_summaryLabel->setPalette(pal);
     m_summaryLabel->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
     m_summaryLabel->setTextInteractionFlags(m_summaryLabel->textInteractionFlags() | Qt::LinksAccessibleByMouse);
@@ -1481,7 +1486,7 @@ void MiniProjectTargetSelector::updateActionAndSummary()
                 deployConfig = dc->displayName();
 
             if (RunConfiguration *rc = target->activeRunConfiguration())
-                runConfig = rc->displayName();
+                runConfig = rc->expandedDisplayName();
 
             targetToolTipText = target->overlayIconToolTip();
             targetIcon = createCenteredIcon(target->icon(), target->overlayIcon());
@@ -1530,7 +1535,7 @@ void MiniProjectTargetSelector::updateSummary()
                                    activeTarget->activeDeployConfiguration()->displayName()));
             if (!m_listWidgets[RUN]->isVisibleTo(this) && activeTarget->activeRunConfiguration())
                 summary.append(tr("Run: <b>%1</b><br/>").arg(
-                                   activeTarget->activeRunConfiguration()->displayName()));
+                                   activeTarget->activeRunConfiguration()->expandedDisplayName()));
         } else if (startupProject->needsConfiguration()) {
             summary = tr("<style type=text/css>"
                          "a:link {color: rgb(128, 128, 255, 240);}</style>"
@@ -1554,7 +1559,7 @@ void MiniProjectTargetSelector::updateSummary()
 void MiniProjectTargetSelector::paintEvent(QPaintEvent *)
 {
     QPainter painter(this);
-    painter.fillRect(rect(), Utils::StyleHelper().baseColor());
+    painter.fillRect(rect(), StyleHelper::baseColor());
     painter.setPen(creatorTheme()->color(Theme::MiniProjectTargetSelectorBorderColor));
     // draw border on top and right
     QRectF borderRect = QRectF(rect()).adjusted(0.5, 0.5, -0.5, -0.5);
