@@ -40,7 +40,6 @@
 #include <utils/optional.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
-#include <utils/synchronousprocess.h>
 #include <utils/runextensions.h>
 #include <utils/temporarydirectory.h>
 #include <utils/pathchooser.h>
@@ -51,7 +50,7 @@
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
-#include <QProcess>
+#include <QLoggingCategory>
 #include <QRegularExpression>
 #include <QSettings>
 #include <QVector>
@@ -66,10 +65,9 @@ using namespace Utils;
 #define KEY_ROOT "ProjectExplorer.MsvcToolChain."
 static const char varsBatKeyC[] = KEY_ROOT "VarsBat";
 static const char varsBatArgKeyC[] = KEY_ROOT "VarsBatArg";
-static const char supportedAbiKeyC[] = KEY_ROOT "SupportedAbi";
 static const char environModsKeyC[] = KEY_ROOT "environmentModifications";
 
-enum { debug = 0 };
+static Q_LOGGING_CATEGORY(Log, "qtc.projectexplorer.toolchain.msvc", QtWarningMsg);
 
 namespace ProjectExplorer {
 namespace Internal {
@@ -105,10 +103,21 @@ const MsvcPlatform platforms[]
        {MsvcToolChain::arm, "arm", "/bin/arm", "vcvarsarm.bat"},
        {MsvcToolChain::x86_arm, "x86_arm", "/bin/x86_arm", "vcvarsx86_arm.bat"},
        {MsvcToolChain::amd64_arm, "amd64_arm", "/bin/amd64_arm", "vcvarsamd64_arm.bat"},
-       {MsvcToolChain::amd64_x86, "amd64_x86", "/bin/amd64_x86", "vcvarsamd64_x86.bat"}};
+       {MsvcToolChain::amd64_x86, "amd64_x86", "/bin/amd64_x86", "vcvarsamd64_x86.bat"},
+       {MsvcToolChain::x86_arm64, "x86_arm64", "/bin/x86_arm64", "vcvarsx86_arm64.bat"},
+       {MsvcToolChain::amd64_arm64, "amd64_arm64", "/bin/amd64_arm64", "vcvarsamd64_arm64.bat"}};
 
 static QList<const MsvcToolChain *> g_availableMsvcToolchains;
 
+static const MsvcPlatform *platformEntryFromName(const QString &name)
+{
+    for (const MsvcPlatform &p : platforms) {
+        if (name == QLatin1String(p.name))
+            return &p;
+    }
+    return nullptr;
+}
+
 static const MsvcPlatform *platformEntry(MsvcToolChain::Platform t)
 {
     for (const MsvcPlatform &p : platforms) {
@@ -125,26 +134,37 @@ static QString platformName(MsvcToolChain::Platform t)
     return QString();
 }
 
-static bool hostSupportsPlatform(MsvcToolChain::Platform platform)
+static bool hostPrefersPlatform(MsvcToolChain::Platform platform)
 {
-    switch (Utils::HostOsInfo::hostArchitecture()) {
-    case Utils::HostOsInfo::HostArchitectureAMD64:
-        if (platform == MsvcToolChain::amd64 || platform == MsvcToolChain::amd64_arm
-            || platform == MsvcToolChain::amd64_x86)
-            return true;
-        Q_FALLTHROUGH(); // all x86 toolchains are also working on an amd64 host
-    case Utils::HostOsInfo::HostArchitectureX86:
+    switch (HostOsInfo::hostArchitecture()) {
+    case HostOsInfo::HostArchitectureAMD64:
+        return platform == MsvcToolChain::amd64 || platform == MsvcToolChain::amd64_arm
+               || platform == MsvcToolChain::amd64_x86 || platform == MsvcToolChain::amd64_arm64;
+    case HostOsInfo::HostArchitectureX86:
         return platform == MsvcToolChain::x86 || platform == MsvcToolChain::x86_amd64
-               || platform == MsvcToolChain::x86_ia64 || platform == MsvcToolChain::x86_arm;
-    case Utils::HostOsInfo::HostArchitectureArm:
+               || platform == MsvcToolChain::x86_ia64 || platform == MsvcToolChain::x86_arm
+               || platform == MsvcToolChain::x86_arm64;
+    case HostOsInfo::HostArchitectureArm:
         return platform == MsvcToolChain::arm;
-    case Utils::HostOsInfo::HostArchitectureItanium:
+    case HostOsInfo::HostArchitectureItanium:
         return platform == MsvcToolChain::ia64;
     default:
         return false;
     }
 }
 
+static bool hostSupportsPlatform(MsvcToolChain::Platform platform)
+{
+    if (hostPrefersPlatform(platform))
+        return true;
+    // The x86 host toolchains are not the preferred toolchains on amd64 but they are still
+    // supported by that host
+    return HostOsInfo::hostArchitecture() == HostOsInfo::HostArchitectureAMD64
+           && (platform == MsvcToolChain::x86 || platform == MsvcToolChain::x86_amd64
+               || platform == MsvcToolChain::x86_ia64 || platform == MsvcToolChain::x86_arm
+               || platform == MsvcToolChain::x86_arm64);
+}
+
 static QString fixRegistryPath(const QString &path)
 {
     QString result = QDir::fromNativeSeparators(path);
@@ -236,35 +256,35 @@ static Utils::optional<VisualStudioInstallation> detectCppBuildTools2017()
 static QVector<VisualStudioInstallation> detectVisualStudioFromVsWhere(const QString &vswhere)
 {
     QVector<VisualStudioInstallation> installations;
-    Utils::SynchronousProcess vsWhereProcess;
+    QtcProcess vsWhereProcess;
     vsWhereProcess.setCodec(QTextCodec::codecForName("UTF-8"));
     const int timeoutS = 5;
     vsWhereProcess.setTimeoutS(timeoutS);
-    const CommandLine cmd(vswhere,
-            {"-products", "*", "-prerelease", "-legacy", "-format", "json", "-utf8"});
-    Utils::SynchronousProcessResponse response = vsWhereProcess.runBlocking(cmd);
-    switch (response.result) {
-    case Utils::SynchronousProcessResponse::Finished:
+    vsWhereProcess.setCommand({FilePath::fromString(vswhere),
+                        {"-products", "*", "-prerelease", "-legacy", "-format", "json", "-utf8"}});
+    vsWhereProcess.runBlocking();
+    switch (vsWhereProcess.result()) {
+    case QtcProcess::FinishedWithSuccess:
         break;
-    case Utils::SynchronousProcessResponse::StartFailed:
+    case QtcProcess::StartFailed:
         qWarning().noquote() << QDir::toNativeSeparators(vswhere) << "could not be started.";
         return installations;
-    case Utils::SynchronousProcessResponse::FinishedError:
+    case QtcProcess::FinishedWithError:
         qWarning().noquote().nospace() << QDir::toNativeSeparators(vswhere)
                                        << " finished with exit code "
-                                       << response.exitCode << ".";
+                                       << vsWhereProcess.exitCode() << ".";
         return installations;
-    case Utils::SynchronousProcessResponse::TerminatedAbnormally:
+    case QtcProcess::TerminatedAbnormally:
         qWarning().noquote().nospace()
-            << QDir::toNativeSeparators(vswhere) << " crashed. Exit code: " << response.exitCode;
+            << QDir::toNativeSeparators(vswhere) << " crashed. Exit code: " << vsWhereProcess.exitCode();
         return installations;
-    case Utils::SynchronousProcessResponse::Hang:
+    case QtcProcess::Hang:
         qWarning().noquote() << QDir::toNativeSeparators(vswhere) << "did not finish in" << timeoutS
                              << "seconds.";
         return installations;
     }
 
-    QByteArray output = response.stdOut().toUtf8();
+    QByteArray output = vsWhereProcess.stdOut().toUtf8();
     QJsonParseError error;
     const QJsonDocument doc = QJsonDocument::fromJson(output, &error);
     if (error.error != QJsonParseError::NoError || doc.isNull()) {
@@ -366,6 +386,8 @@ static unsigned char wordWidthForPlatform(MsvcToolChain::Platform platform)
     case ProjectExplorer::Internal::MsvcToolChain::x86_amd64:
     case ProjectExplorer::Internal::MsvcToolChain::ia64:
     case ProjectExplorer::Internal::MsvcToolChain::x86_ia64:
+    case ProjectExplorer::Internal::MsvcToolChain::amd64_arm64:
+    case ProjectExplorer::Internal::MsvcToolChain::x86_arm64:
         return 64;
     }
 
@@ -383,6 +405,8 @@ static Abi::Architecture archForPlatform(MsvcToolChain::Platform platform)
     case ProjectExplorer::Internal::MsvcToolChain::arm:
     case ProjectExplorer::Internal::MsvcToolChain::x86_arm:
     case ProjectExplorer::Internal::MsvcToolChain::amd64_arm:
+    case ProjectExplorer::Internal::MsvcToolChain::x86_arm64:
+    case ProjectExplorer::Internal::MsvcToolChain::amd64_arm64:
         return Abi::ArmArchitecture;
     case ProjectExplorer::Internal::MsvcToolChain::ia64:
     case ProjectExplorer::Internal::MsvcToolChain::x86_ia64:
@@ -405,7 +429,9 @@ static Abi findAbiOfMsvc(MsvcToolChain::Type type,
         else if (version == QLatin1String("v7.0A") || version == QLatin1String("v7.1"))
             msvcVersionString = QLatin1String("10.0");
     }
-    if (msvcVersionString.startsWith(QLatin1String("16.")))
+    if (msvcVersionString.startsWith(QLatin1String("17.")))
+        flavor = Abi::WindowsMsvc2022Flavor;
+    else if (msvcVersionString.startsWith(QLatin1String("16.")))
         flavor = Abi::WindowsMsvc2019Flavor;
     else if (msvcVersionString.startsWith(QLatin1String("15.")))
         flavor = Abi::WindowsMsvc2017Flavor;
@@ -610,8 +636,8 @@ Macros MsvcToolChain::msvcPredefinedMacros(const QStringList &cxxflags,
         qWarning("%s: %s", Q_FUNC_INFO, qPrintable(saver.errorString()));
         return predefinedMacros;
     }
-    Utils::SynchronousProcess cpp;
-    cpp.setEnvironment(env.toStringList());
+    Utils::QtcProcess cpp;
+    cpp.setEnvironment(env);
     cpp.setWorkingDirectory(Utils::TemporaryDirectory::masterDirectoryPath());
     QStringList arguments;
     const Utils::FilePath binary = env.searchInPath(QLatin1String("cl.exe"));
@@ -622,12 +648,13 @@ Macros MsvcToolChain::msvcPredefinedMacros(const QStringList &cxxflags,
 
     if (language() == ProjectExplorer::Constants::C_LANGUAGE_ID)
         arguments << QLatin1String("/TC");
-    arguments << toProcess << QLatin1String("/EP") << QDir::toNativeSeparators(saver.fileName());
-    SynchronousProcessResponse response = cpp.runBlocking({binary, arguments});
-    if (response.result != Utils::SynchronousProcessResponse::Finished || response.exitCode != 0)
+    arguments << toProcess << QLatin1String("/EP") << saver.filePath().toUserOutput();
+    cpp.setCommand({binary, arguments});
+    cpp.runBlocking();
+    if (cpp.result() != QtcProcess::FinishedWithSuccess)
         return predefinedMacros;
 
-    const QStringList output = Utils::filtered(response.stdOut().split('\n'),
+    const QStringList output = Utils::filtered(cpp.stdOut().split('\n'),
                                                [](const QString &s) { return s.startsWith('V'); });
     for (const QString &line : output)
         predefinedMacros.append(Macro::fromKeyValue(line.mid(1)));
@@ -729,16 +756,6 @@ void MsvcToolChain::environmentModifications(
                 outEnv.set(envIter.key(), expandedValue);
         }
 
-        if (debug) {
-            const QStringList newVars = outEnv.toStringList();
-            const QStringList oldVars = inEnv.toStringList();
-            QDebug nsp = qDebug().nospace();
-            foreach (const QString &n, newVars) {
-                if (!oldVars.contains(n))
-                    nsp << n << '\n';
-            }
-        }
-
         diff = inEnv.diff(outEnv, true);
         for (int i = diff.size() - 1; i >= 0; --i) {
             if (diff.at(i).name.startsWith(QLatin1Char('='))) { // Exclude "=C:", "=EXITCODE"
@@ -769,9 +786,16 @@ void MsvcToolChain::updateEnvironmentModifications(Utils::EnvironmentItems modif
 {
     Utils::EnvironmentItem::sort(&modifications);
     if (modifications != m_environmentModifications) {
+        if (Log().isDebugEnabled()) {
+            qCDebug(Log) << "Update environment for " << displayName();
+            for (const EnvironmentItem &item : qAsConst(modifications))
+                qCDebug(Log) << '\t' << item;
+        }
         m_environmentModifications = modifications;
         rescanForCompiler();
         toolChainUpdated();
+    } else {
+        qCDebug(Log) << "No updates for " << displayName();
     }
 }
 
@@ -815,6 +839,7 @@ MsvcToolChain::MsvcToolChain(Utils::Id typeId)
     setDisplayName("Microsoft Visual C++ Compiler");
     setTypeDisplayName(tr("MSVC"));
     addToAvailableMsvcToolchains(this);
+    setTargetAbiKey(KEY_ROOT "SupportedAbi");
 }
 
 void MsvcToolChain::inferWarningsForLevel(int warningLevel, WarningFlags &flags)
@@ -841,16 +866,6 @@ MsvcToolChain::~MsvcToolChain()
     g_availableMsvcToolchains.removeOne(this);
 }
 
-Abi MsvcToolChain::targetAbi() const
-{
-    return m_abi;
-}
-
-void MsvcToolChain::setTargetAbi(const Abi &abi)
-{
-    m_abi = abi;
-}
-
 bool MsvcToolChain::isValid() const
 {
     if (m_vcvarsBat.isEmpty())
@@ -861,14 +876,14 @@ bool MsvcToolChain::isValid() const
 
 QString MsvcToolChain::originalTargetTriple() const
 {
-    return m_abi.wordWidth() == 64 ? QLatin1String("x86_64-pc-windows-msvc")
-                                   : QLatin1String("i686-pc-windows-msvc");
+    return targetAbi().wordWidth() == 64 ? QLatin1String("x86_64-pc-windows-msvc")
+                                         : QLatin1String("i686-pc-windows-msvc");
 }
 
 QStringList MsvcToolChain::suggestedMkspecList() const
 {
     // "win32-msvc" is the common MSVC mkspec introduced in Qt 5.8.1
-    switch (m_abi.osFlavor()) {
+    switch (targetAbi().osFlavor()) {
     case Abi::WindowsMsvc2005Flavor:
         return {"win32-msvc",
                 "win32-msvc2005"};
@@ -909,22 +924,61 @@ QStringList MsvcToolChain::suggestedMkspecList() const
     case Abi::WindowsMsvc2019Flavor:
         return {"win32-msvc",
                 "win32-msvc2019",
+                "win32-arm64-msvc",
                 "winrt-arm-msvc2019",
                 "winrt-x86-msvc2019",
                 "winrt-x64-msvc2019"};
+    case Abi::WindowsMsvc2022Flavor:
+        return {"win32-msvc",
+                "win32-msvc2022",
+                "win32-arm64-msvc"};
     default:
         break;
     }
     return {};
 }
 
+Abis MsvcToolChain::supportedAbis() const
+{
+    Abi abi = targetAbi();
+    Abis abis = {abi};
+    switch (abi.osFlavor()) {
+    case Abi::WindowsMsvc2022Flavor:
+        abis << Abi(abi.architecture(),
+                    abi.os(),
+                    Abi::WindowsMsvc2019Flavor,
+                    abi.binaryFormat(),
+                    abi.wordWidth(),
+                    abi.param());
+        Q_FALLTHROUGH();
+    case Abi::WindowsMsvc2019Flavor:
+        abis << Abi(abi.architecture(),
+                    abi.os(),
+                    Abi::WindowsMsvc2017Flavor,
+                    abi.binaryFormat(),
+                    abi.wordWidth(),
+                    abi.param());
+        Q_FALLTHROUGH();
+    case Abi::WindowsMsvc2017Flavor:
+        abis << Abi(abi.architecture(),
+                    abi.os(),
+                    Abi::WindowsMsvc2015Flavor,
+                    abi.binaryFormat(),
+                    abi.wordWidth(),
+                    abi.param());
+        break;
+    default:
+        break;
+    }
+    return abis;
+}
+
 QVariantMap MsvcToolChain::toMap() const
 {
     QVariantMap data = ToolChain::toMap();
     data.insert(QLatin1String(varsBatKeyC), m_vcvarsBat);
     if (!m_varsBatArg.isEmpty())
         data.insert(QLatin1String(varsBatArgKeyC), m_varsBatArg);
-    data.insert(QLatin1String(supportedAbiKeyC), m_abi.toString());
     Utils::EnvironmentItem::sort(&m_environmentModifications);
     data.insert(QLatin1String(environModsKeyC),
                 Utils::EnvironmentItem::toVariantList(m_environmentModifications));
@@ -940,8 +994,6 @@ bool MsvcToolChain::fromMap(const QVariantMap &data)
     m_vcvarsBat = QDir::fromNativeSeparators(data.value(QLatin1String(varsBatKeyC)).toString());
     m_varsBatArg = data.value(QLatin1String(varsBatArgKeyC)).toString();
 
-    const QString abiString = data.value(QLatin1String(supportedAbiKeyC)).toString();
-    m_abi = Abi::fromString(abiString);
     m_environmentModifications = Utils::EnvironmentItem::itemsFromVariantList(
         data.value(QLatin1String(environModsKeyC)).toList());
     rescanForCompiler();
@@ -951,7 +1003,7 @@ bool MsvcToolChain::fromMap(const QVariantMap &data)
                                       m_vcvarsBat,
                                       m_varsBatArg));
 
-    const bool valid = !m_vcvarsBat.isEmpty() && m_abi.isValid();
+    const bool valid = !m_vcvarsBat.isEmpty() && targetAbi().isValid();
     if (!valid)
         g_availableMsvcToolchains.removeOne(this);
 
@@ -963,6 +1015,11 @@ std::unique_ptr<ToolChainConfigWidget> MsvcToolChain::createConfigurationWidget(
     return std::make_unique<MsvcToolChainConfigWidget>(this);
 }
 
+bool MsvcToolChain::hostPrefersToolchain() const
+{
+    return hostPrefersPlatform(platform());
+}
+
 bool static hasFlagEffectOnMacros(const QString &flag)
 {
     if (flag.startsWith("-") || flag.startsWith("/")) {
@@ -1065,6 +1122,12 @@ WarningFlags MsvcToolChain::warningFlags(const QStringList &cflags) const
     return flags;
 }
 
+QStringList MsvcToolChain::includedFiles(const QStringList &flags,
+                                         const QString &directoryPath) const
+{
+    return ToolChain::includedFiles("/FI", flags, directoryPath);
+}
+
 ToolChain::BuiltInHeaderPathsRunner MsvcToolChain::createBuiltInHeaderPathsRunner(
         const Environment &env) const
 {
@@ -1073,13 +1136,12 @@ ToolChain::BuiltInHeaderPathsRunner MsvcToolChain::createBuiltInHeaderPathsRunne
 
     return [this, fullEnv](const QStringList &, const QString &, const QString &) {
         QMutexLocker locker(&m_headerPathsMutex);
-        if (m_headerPaths.isEmpty()) {
-            m_headerPaths = transform<QVector>(fullEnv.pathListValue("INCLUDE"),
-                                               [](const FilePath &p) {
-                return HeaderPath(p.toString(), HeaderPathType::BuiltIn);
-            });
-        }
-        return m_headerPaths;
+        const auto envList = fullEnv.toStringList();
+        const auto it = m_headerPathsPerEnv.constFind(envList);
+        if (it != m_headerPathsPerEnv.cend())
+            return *it;
+        return *m_headerPathsPerEnv.insert(envList,
+                                           toBuiltInHeaderPaths(fullEnv.pathListValue("INCLUDE")));
     };
 }
 
@@ -1087,8 +1149,7 @@ void MsvcToolChain::addToEnvironment(Utils::Environment &env) const
 {
     // We cache the full environment (incoming + modifications by setup script).
     if (!m_resultEnvironment.size() || env != m_lastEnvironment) {
-        if (debug)
-            qDebug() << "addToEnvironment: " << displayName();
+        qCDebug(Log) << "addToEnvironment: " << displayName();
         m_lastEnvironment = env;
         m_resultEnvironment = readEnvironmentSetting(env);
     }
@@ -1118,8 +1179,8 @@ FilePath MsvcToolChain::makeCommand(const Environment &environment) const
     FilePath command;
     if (useJom) {
         tmp = environment.searchInPath(jom,
-                                       {Utils::FilePath::fromString(
-                                           QCoreApplication::applicationDirPath())});
+                                       {Core::ICore::libexecPath(),
+                                        Core::ICore::libexecPath("jom")});
         if (!tmp.isEmpty())
             command = tmp;
     }
@@ -1139,18 +1200,13 @@ FilePath MsvcToolChain::makeCommand(const Environment &environment) const
     return command;
 }
 
-Utils::FilePath MsvcToolChain::compilerCommand() const
-{
-    return m_compilerCommand;
-}
-
 void MsvcToolChain::rescanForCompiler()
 {
     Utils::Environment env = Utils::Environment::systemEnvironment();
     addToEnvironment(env);
 
-    m_compilerCommand
-        = env.searchInPath(QLatin1String("cl.exe"), {}, [](const Utils::FilePath &name) {
+    setCompilerCommand(
+          env.searchInPath(QLatin1String("cl.exe"), {}, [](const Utils::FilePath &name) {
               QDir dir(QDir::cleanPath(name.toFileInfo().absolutePath() + QStringLiteral("/..")));
               do {
                   if (QFile::exists(dir.absoluteFilePath(QStringLiteral("vcvarsall.bat")))
@@ -1158,7 +1214,7 @@ void MsvcToolChain::rescanForCompiler()
                       return true;
               } while (dir.cdUp() && !dir.isRoot());
               return false;
-          });
+          }));
 }
 
 QList<OutputLineParser *> MsvcToolChain::createOutputParsers() const
@@ -1169,7 +1225,7 @@ QList<OutputLineParser *> MsvcToolChain::createOutputParsers() const
 void MsvcToolChain::setupVarsBat(const Abi &abi, const QString &varsBat, const QString &varsBatArg)
 {
     m_lastEnvironment = Utils::Environment::systemEnvironment();
-    m_abi = abi;
+    setTargetAbiNoSignal(abi);
     m_vcvarsBat = varsBat;
     m_varsBatArg = varsBatArg;
 
@@ -1184,11 +1240,20 @@ void MsvcToolChain::setupVarsBat(const Abi &abi, const QString &varsBat, const Q
 void MsvcToolChain::resetVarsBat()
 {
     m_lastEnvironment = Utils::Environment::systemEnvironment();
-    m_abi = Abi();
+    setTargetAbiNoSignal(Abi());
     m_vcvarsBat.clear();
     m_varsBatArg.clear();
 }
 
+MsvcToolChain::Platform MsvcToolChain::platform() const
+{
+    QStringList args = m_varsBatArg.split(' ');
+    if (const MsvcPlatform *entry = platformEntryFromName(args.value(0)))
+        return entry->platform;
+    return Utils::HostOsInfo::hostArchitecture() == Utils::HostOsInfo::HostArchitectureAMD64 ? amd64
+                                                                                             : x86;
+}
+
 // --------------------------------------------------------------------------
 // MsvcBasedToolChainConfigWidget: Creates a simple GUI without error label
 // to display name and varsBat. Derived classes should add the error label and
@@ -1241,7 +1306,7 @@ MsvcToolChainConfigWidget::MsvcToolChainConfigWidget(ToolChain *tc)
     m_varsBatPathCombo->setObjectName("varsBatCombo");
     m_varsBatPathCombo->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
     m_varsBatPathCombo->setEditable(true);
-    for (const MsvcToolChain *tmpTc : g_availableMsvcToolchains) {
+    for (const MsvcToolChain *tmpTc : qAsConst(g_availableMsvcToolchains)) {
         const QString nativeVcVars = QDir::toNativeSeparators(tmpTc->varsBat());
         if (!tmpTc->varsBat().isEmpty()
                 && m_varsBatPathCombo->findText(nativeVcVars) == -1) {
@@ -1257,10 +1322,10 @@ MsvcToolChainConfigWidget::MsvcToolChainConfigWidget(ToolChain *tc)
     m_varsBatArchCombo->addItem("arm", MsvcToolChain::arm);
     m_varsBatArchCombo->addItem("x86_amd64", MsvcToolChain::x86_amd64);
     m_varsBatArchCombo->addItem("x86_arm", MsvcToolChain::x86_arm);
-//    m_varsBatArchCombo->addItem("x86_arm64", MsvcToolChain::x86_arm64);
+    m_varsBatArchCombo->addItem("x86_arm64", MsvcToolChain::x86_arm64);
     m_varsBatArchCombo->addItem("amd64_x86", MsvcToolChain::amd64_x86);
     m_varsBatArchCombo->addItem("amd64_arm", MsvcToolChain::amd64_arm);
-//    m_varsBatArchCombo->addItem("amd64_arm64", MsvcToolChain::amd64_arm64);
+    m_varsBatArchCombo->addItem("amd64_arm64", MsvcToolChain::amd64_arm64);
     m_varsBatArchCombo->addItem("ia64", MsvcToolChain::ia64);
     m_varsBatArchCombo->addItem("x86_ia64", MsvcToolChain::x86_ia64);
     m_varsBatArgumentsEdit->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
@@ -1347,7 +1412,7 @@ void MsvcToolChainConfigWidget::updateAbis()
     // choose one appropriately.
     Abis supportedAbis;
     Abi targetAbi;
-    for (const MsvcToolChain *tc : g_availableMsvcToolchains) {
+    for (const MsvcToolChain *tc : qAsConst(g_availableMsvcToolchains)) {
         if (tc->varsBat() == normalizedVcVars && tc->targetAbi().wordWidth() == wordWidth
             && tc->targetAbi().architecture() == arch && tc->language() == currentTc->language()) {
             // We need to filter out duplicates as there might be multiple toolchains with
@@ -1448,7 +1513,7 @@ void ClangClToolChainConfigWidget::setFromClangClToolChain()
     m_nameDisplayLabel->setText(currentTC->displayName());
     m_varsBatDisplayCombo->clear();
     m_varsBatDisplayCombo->addItem(msvcVarsToDisplay(*currentTC));
-    for (const MsvcToolChain *tc : g_availableMsvcToolchains) {
+    for (const MsvcToolChain *tc : qAsConst(g_availableMsvcToolchains)) {
         const QString varsToDisplay = msvcVarsToDisplay(*tc);
         if (m_varsBatDisplayCombo->findText(varsToDisplay) == -1)
             m_varsBatDisplayCombo->addItem(varsToDisplay);
@@ -1480,14 +1545,14 @@ static const MsvcToolChain *findMsvcToolChain(const QString &displayedVarsBat)
 
 static QVersionNumber clangClVersion(const QString &clangClPath)
 {
-    SynchronousProcess clangClProcess;
-    const SynchronousProcessResponse response
-        = clangClProcess.runBlocking({clangClPath, {"--version"}});
-    if (response.result != SynchronousProcessResponse::Finished || response.exitCode != 0)
+    QtcProcess clangClProcess;
+    clangClProcess.setCommand({FilePath::fromString(clangClPath), {"--version"}});
+    clangClProcess.runBlocking();
+    if (clangClProcess.result() != QtcProcess::FinishedWithSuccess)
         return {};
     const QRegularExpressionMatch match = QRegularExpression(
                                               QStringLiteral("clang version (\\d+(\\.\\d+)+)"))
-                                              .match(response.stdOut());
+                                              .match(clangClProcess.stdOut());
     if (!match.hasMatch())
         return {};
     return QVersionNumber::fromString(match.captured(1));
@@ -1507,7 +1572,9 @@ static const MsvcToolChain *selectMsvcToolChain(const QString &displayedVarsBat,
     QTC_CHECK(displayedVarsBat.isEmpty());
     const QVersionNumber version = clangClVersion(clangClPath);
     if (version.majorVersion() >= 6) {
-        toolChain = findMsvcToolChain(wordWidth, Abi::WindowsMsvc2019Flavor);
+        toolChain = findMsvcToolChain(wordWidth, Abi::WindowsMsvc2022Flavor);
+        if (!toolChain)
+            toolChain = findMsvcToolChain(wordWidth, Abi::WindowsMsvc2019Flavor);
         if (!toolChain)
             toolChain = findMsvcToolChain(wordWidth, Abi::WindowsMsvc2017Flavor);
     }
@@ -1629,8 +1696,8 @@ ClangClToolChain::ClangClToolChain()
 
 bool ClangClToolChain::isValid() const
 {
-    return MsvcToolChain::isValid() && compilerCommand().exists()
-           && compilerCommand().fileName() == "clang-cl.exe";
+    const QFileInfo fi(clangPath());
+    return MsvcToolChain::isValid() && fi.exists() && fi.fileName() == "clang-cl.exe";
 }
 
 void ClangClToolChain::addToEnvironment(Utils::Environment &env) const
@@ -1694,26 +1761,32 @@ bool ClangClToolChain::operator==(const ToolChain &other) const
     return m_clangPath == clangClTc->m_clangPath;
 }
 
+int ClangClToolChain::priority() const
+{
+    return MsvcToolChain::priority() - 1;
+}
+
 Macros ClangClToolChain::msvcPredefinedMacros(const QStringList &cxxflags,
                                               const Utils::Environment &env) const
 {
     if (!cxxflags.contains("--driver-mode=g++"))
         return MsvcToolChain::msvcPredefinedMacros(cxxflags, env);
 
-    Utils::SynchronousProcess cpp;
-    cpp.setEnvironment(env.toStringList());
+    QtcProcess cpp;
+    cpp.setEnvironment(env);
     cpp.setWorkingDirectory(Utils::TemporaryDirectory::masterDirectoryPath());
 
     QStringList arguments = cxxflags;
     arguments.append(gccPredefinedMacrosOptions(language()));
     arguments.append("-");
-    Utils::SynchronousProcessResponse response = cpp.runBlocking({compilerCommand(), arguments});
-    if (response.result != Utils::SynchronousProcessResponse::Finished || response.exitCode != 0) {
+    cpp.setCommand({compilerCommand(), arguments});
+    cpp.runBlocking();
+    if (cpp.result() != Utils::QtcProcess::FinishedWithSuccess) {
         // Show the warning but still parse the output.
         QTC_CHECK(false && "clang-cl exited with non-zero code.");
     }
 
-    return Macro::toMacros(response.allRawOutput());
+    return Macro::toMacros(cpp.allRawOutput());
 }
 
 Utils::LanguageVersion ClangClToolChain::msvcLanguageVersion(const QStringList &cxxflags,
@@ -1730,7 +1803,7 @@ ClangClToolChain::BuiltInHeaderPathsRunner ClangClToolChain::createBuiltInHeader
 {
     {
         QMutexLocker locker(&m_headerPathsMutex);
-        m_headerPaths.clear();
+        m_headerPathsPerEnv.clear();
     }
 
     return MsvcToolChain::createBuiltInHeaderPathsRunner(env);
@@ -1764,8 +1837,7 @@ QString MsvcToolChainFactory::vcVarsBatFor(const QString &basePath,
     return result;
 }
 
-static QList<ToolChain *> findOrCreateToolChain(const QList<ToolChain *> &alreadyKnown,
-                                                const QString &name,
+static QList<ToolChain *> findOrCreateToolChain(const QList<ToolChain *> &alreadyKnown, const QString &name,
                                                 const Abi &abi,
                                                 const QString &varsBat,
                                                 const QString &varsBatArg)
@@ -1810,7 +1882,9 @@ static void detectCppBuildTools2015(QList<ToolChain *> *list)
     const Entry entries[] = {{" (x86)", "x86", Abi::X86Architecture, Abi::PEFormat, 32},
                              {" (x64)", "amd64", Abi::X86Architecture, Abi::PEFormat, 64},
                              {" (x86_arm)", "x86_arm", Abi::ArmArchitecture, Abi::PEFormat, 32},
-                             {" (x64_arm)", "amd64_arm", Abi::ArmArchitecture, Abi::PEFormat, 64}};
+                             {" (x64_arm)", "amd64_arm", Abi::ArmArchitecture, Abi::PEFormat, 32},
+                             {" (x86_arm64)", "x86_arm64", Abi::ArmArchitecture, Abi::PEFormat, 64},
+                             {" (x64_arm64)", "amd64_arm64", Abi::ArmArchitecture, Abi::PEFormat, 64}};
 
     const QString name = "Microsoft Visual C++ Build Tools";
     const QString vcVarsBat = windowsProgramFilesDir() + '/' + name + "/vcbuildtools.bat";
@@ -1833,8 +1907,14 @@ static void detectCppBuildTools2015(QList<ToolChain *> *list)
     }
 }
 
-QList<ToolChain *> MsvcToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown)
+QList<ToolChain *> MsvcToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown,
+                                                    const IDevice::Ptr &device)
 {
+    if (!device.isNull()) {
+        // FIXME currently no support for msvc toolchains on a device
+        return {};
+    }
+
     QList<ToolChain *> results;
 
     // 1) Installed SDKs preferred over standalone Visual studio
@@ -1896,6 +1976,8 @@ QList<ToolChain *> MsvcToolChainFactory::autoDetect(const QList<ToolChain *> &al
                                                  MsvcToolChain::arm,
                                                  MsvcToolChain::x86_arm,
                                                  MsvcToolChain::amd64_arm,
+                                                 MsvcToolChain::x86_arm64,
+                                                 MsvcToolChain::amd64_arm64,
                                                  MsvcToolChain::ia64,
                                                  MsvcToolChain::x86_ia64};
 
@@ -1916,7 +1998,7 @@ QList<ToolChain *> MsvcToolChainFactory::autoDetect(const QList<ToolChain *> &al
 
     detectCppBuildTools2015(&results);
 
-    for (ToolChain *tc : results)
+    for (ToolChain *tc : qAsConst(results))
         tc->setDetection(ToolChain::AutoDetection);
 
     return results;
@@ -1935,10 +2017,14 @@ bool ClangClToolChainFactory::canCreate() const
     return !g_availableMsvcToolchains.isEmpty();
 }
 
-QList<ToolChain *> ClangClToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown)
+QList<ToolChain *> ClangClToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown,
+                                                       const IDevice::Ptr &device)
 {
     Q_UNUSED(alreadyKnown)
-
+    if (!device.isNull()) {
+        // FIXME currently no support for msvc toolchains on a device
+        return {};
+    }
 #ifdef Q_OS_WIN64
     const char registryNode[] = "HKEY_LOCAL_MACHINE\\SOFTWARE\\WOW6432Node\\LLVM\\LLVM";
 #else
@@ -1948,13 +2034,11 @@ QList<ToolChain *> ClangClToolChainFactory::autoDetect(const QList<ToolChain *>
     QList<ToolChain *> results;
     QList<ToolChain *> known = alreadyKnown;
 
-    QString qtCreatorsClang = Core::ICore::clangExecutable(CLANG_BINDIR);
+    FilePath qtCreatorsClang = Core::ICore::clangExecutable(CLANG_BINDIR);
     if (!qtCreatorsClang.isEmpty()) {
-        qtCreatorsClang = Utils::FilePath::fromString(qtCreatorsClang)
-                              .parentDir()
-                              .pathAppended("clang-cl.exe")
-                              .toString();
-        results.append(detectClangClToolChainInPath(qtCreatorsClang, alreadyKnown, "", true));
+        qtCreatorsClang = qtCreatorsClang.parentDir().pathAppended("clang-cl.exe");
+        results.append(detectClangClToolChainInPath(qtCreatorsClang.toString(),
+                                                    alreadyKnown, "", true));
         known.append(results);
     }
 
@@ -1986,6 +2070,11 @@ bool MsvcToolChain::operator==(const ToolChain &other) const
            && m_varsBatArg == msvcTc->m_varsBatArg;
 }
 
+int MsvcToolChain::priority() const
+{
+    return hostPrefersToolchain() ? PriorityHigh : PriorityNormal;
+}
+
 void MsvcToolChain::cancelMsvcToolChainDetection()
 {
     envModThreadPool()->clear();
@@ -2004,7 +2093,7 @@ Utils::optional<QString> MsvcToolChain::generateEnvironmentSettings(const Utils:
     Utils::TempFileSaver saver(Utils::TemporaryDirectory::masterDirectoryPath() + "/XXXXXX.bat");
 
     QByteArray call = "call ";
-    call += Utils::QtcProcess::quoteArg(batchFile).toLocal8Bit();
+    call += ProcessArgs::quoteArg(batchFile).toLocal8Bit();
     if (!batchArgs.isEmpty()) {
         call += ' ';
         call += batchArgs.toLocal8Bit();
@@ -2021,31 +2110,29 @@ Utils::optional<QString> MsvcToolChain::generateEnvironmentSettings(const Utils:
         return QString();
     }
 
-    Utils::SynchronousProcess run;
+    Utils::QtcProcess run;
 
     // As of WinSDK 7.1, there is logic preventing the path from being set
     // correctly if "ORIGINALPATH" is already set. That can cause problems
     // if Creator is launched within a session set up by setenv.cmd.
     Utils::Environment runEnv = env;
     runEnv.unset(QLatin1String("ORIGINALPATH"));
-    run.setEnvironment(runEnv.toStringList());
-    run.setTimeoutS(30);
+    run.setEnvironment(runEnv);
+    run.setTimeoutS(60);
     Utils::FilePath cmdPath = Utils::FilePath::fromUserInput(
         QString::fromLocal8Bit(qgetenv("COMSPEC")));
     if (cmdPath.isEmpty())
         cmdPath = env.searchInPath(QLatin1String("cmd.exe"));
     // Windows SDK setup scripts require command line switches for environment expansion.
-    CommandLine cmd(cmdPath, {"/E:ON", "/V:ON", "/c", QDir::toNativeSeparators(saver.fileName())});
-    if (debug)
-        qDebug() << "readEnvironmentSetting: " << call << cmd.toUserOutput()
+    CommandLine cmd(cmdPath, {"/E:ON", "/V:ON", "/c", saver.filePath().toUserOutput()});
+    qCDebug(Log) << "readEnvironmentSetting: " << call << cmd.toUserOutput()
                  << " Env: " << runEnv.size();
     run.setCodec(QTextCodec::codecForName("UTF-8"));
-    Utils::SynchronousProcessResponse response = run.runBlocking(cmd);
+    run.setCommand(cmd);
+    run.runBlocking();
 
-    if (response.result != Utils::SynchronousProcessResponse::Finished) {
-        const QString message = !response.stdErr().isEmpty()
-                                    ? response.stdErr()
-                                    : response.exitMessage(cmdPath.toString(), 10);
+    if (run.result() != QtcProcess::FinishedWithSuccess) {
+        const QString message = !run.stdErr().isEmpty() ? run.stdErr() : run.exitMessage();
         qWarning().noquote() << message;
         QString command = QDir::toNativeSeparators(batchFile);
         if (!batchArgs.isEmpty())
@@ -2057,7 +2144,7 @@ Utils::optional<QString> MsvcToolChain::generateEnvironmentSettings(const Utils:
     }
 
     // The SDK/MSVC scripts do not return exit codes != 0. Check on stdout.
-    const QString stdOut = response.stdOut();
+    const QString stdOut = run.stdOut();
 
     //
     // Now parse the file to get the environment settings
