@@ -28,6 +28,7 @@
 #include <utils/fileutils.h>
 #include <utils/macroexpander.h>
 #include <utils/qtcprocess.h>
+#include <utils/theme/theme.h>
 
 #include <QDir>
 
@@ -44,32 +45,23 @@
     \sa ProjectExplorer::AbstractProcessStep
 */
 
-using namespace ProjectExplorer;
+using namespace Utils;
 
-ProcessParameters::ProcessParameters() :
-    m_macroExpander(nullptr),
-    m_commandMissing(false)
-{
-}
+namespace ProjectExplorer {
+
+ProcessParameters::ProcessParameters() = default;
 
 /*!
-    Sets the executable to run.
+    Sets the command to run.
 */
-
-void ProcessParameters::setCommand(const QString &cmd)
+void ProcessParameters::setCommandLine(const CommandLine &cmdLine)
 {
-    m_command = cmd;
+    m_command = cmdLine;
     m_effectiveCommand.clear();
-}
-
-/*!
-    Sets the command line arguments used by the process.
-*/
-
-void ProcessParameters::setArguments(const QString &arguments)
-{
-    m_arguments = arguments;
     m_effectiveArguments.clear();
+
+    effectiveCommand();
+    effectiveArguments();
 }
 
 /*!
@@ -78,10 +70,12 @@ void ProcessParameters::setArguments(const QString &arguments)
     Should be called from init().
 */
 
-void ProcessParameters::setWorkingDirectory(const QString &workingDirectory)
+void ProcessParameters::setWorkingDirectory(const FilePath &workingDirectory)
 {
     m_workingDirectory = workingDirectory;
     m_effectiveWorkingDirectory.clear();
+
+    effectiveWorkingDirectory();
 }
 
 /*!
@@ -103,13 +97,14 @@ void ProcessParameters::setWorkingDirectory(const QString &workingDirectory)
     Gets the fully expanded working directory.
 */
 
-QString ProcessParameters::effectiveWorkingDirectory() const
+FilePath ProcessParameters::effectiveWorkingDirectory() const
 {
     if (m_effectiveWorkingDirectory.isEmpty()) {
-        QString wds = m_workingDirectory;
+        m_effectiveWorkingDirectory = m_workingDirectory;
+        QString path = m_workingDirectory.path();
         if (m_macroExpander)
-            wds = m_macroExpander->expand(wds);
-        m_effectiveWorkingDirectory = QDir::cleanPath(m_environment.expandVariables(wds));
+            path = m_macroExpander->expand(path);
+        m_effectiveWorkingDirectory.setPath(QDir::cleanPath(m_environment.expandVariables(path)));
     }
     return m_effectiveWorkingDirectory;
 }
@@ -118,14 +113,19 @@ QString ProcessParameters::effectiveWorkingDirectory() const
     Gets the fully expanded command name to run.
 */
 
-QString ProcessParameters::effectiveCommand() const
+FilePath ProcessParameters::effectiveCommand() const
 {
     if (m_effectiveCommand.isEmpty()) {
-        QString cmd = m_command;
+        FilePath cmd = m_command.executable();
         if (m_macroExpander)
             cmd = m_macroExpander->expand(cmd);
-        m_effectiveCommand =
-                m_environment.searchInPath(cmd, QStringList(effectiveWorkingDirectory())).toString();
+        if (cmd.needsDevice()) {
+            // Assume this is already good. FIXME: It is possibly not, so better fix  searchInPath.
+            m_effectiveCommand = cmd;
+        } else {
+            m_effectiveCommand = m_environment.searchInPath(cmd.toString(),
+                                                            {effectiveWorkingDirectory()});
+        }
         m_commandMissing = m_effectiveCommand.isEmpty();
         if (m_commandMissing)
             m_effectiveCommand = cmd;
@@ -146,7 +146,7 @@ bool ProcessParameters::commandMissing() const
 QString ProcessParameters::effectiveArguments() const
 {
     if (m_effectiveArguments.isEmpty()) {
-        m_effectiveArguments = m_arguments;
+        m_effectiveArguments = m_command.arguments();
         if (m_macroExpander)
             m_effectiveArguments = m_macroExpander->expand(m_effectiveArguments);
     }
@@ -155,44 +155,53 @@ QString ProcessParameters::effectiveArguments() const
 
 QString ProcessParameters::prettyCommand() const
 {
-    QString cmd = m_command;
+    QString cmd = m_command.executable().toString();
     if (m_macroExpander)
         cmd = m_macroExpander->expand(cmd);
-    return Utils::FileName::fromString(cmd).fileName();
+    return FilePath::fromString(cmd).fileName();
 }
 
 QString ProcessParameters::prettyArguments() const
 {
     QString margs = effectiveArguments();
-    QString workDir = effectiveWorkingDirectory();
-    Utils::QtcProcess::SplitError err;
-    Utils::QtcProcess::Arguments args =
-            Utils::QtcProcess::prepareArgs(margs, &err, Utils::HostOsInfo::hostOs(), &m_environment, &workDir);
-    if (err != Utils::QtcProcess::SplitOk)
+    FilePath workDir = effectiveWorkingDirectory();
+    ProcessArgs::SplitError err;
+    ProcessArgs args =
+            ProcessArgs::prepareArgs(margs, &err, HostOsInfo::hostOs(), &m_environment, &workDir);
+    if (err != ProcessArgs::SplitOk)
         return margs; // Sorry, too complex - just fall back.
     return args.toString();
 }
 
+static QString invalidCommandMessage(const QString &displayName)
+{
+    return QString("<b>%1:</b> <font color='%3'>%2</font>")
+                    .arg(displayName,
+                         QtcProcess::tr("Invalid command"),
+                         creatorTheme()->color(Theme::TextColorError).name());
+}
+
 QString ProcessParameters::summary(const QString &displayName) const
 {
+    if (m_commandMissing)
+        return invalidCommandMessage(displayName);
+
     return QString::fromLatin1("<b>%1:</b> %2 %3")
             .arg(displayName,
-                 Utils::QtcProcess::quoteArg(prettyCommand()),
+                 ProcessArgs::quoteArg(prettyCommand()),
                  prettyArguments());
 }
 
 QString ProcessParameters::summaryInWorkdir(const QString &displayName) const
 {
+    if (m_commandMissing)
+        return invalidCommandMessage(displayName);
+
     return QString::fromLatin1("<b>%1:</b> %2 %3 in %4")
             .arg(displayName,
-                 Utils::QtcProcess::quoteArg(prettyCommand()),
+                 ProcessArgs::quoteArg(prettyCommand()),
                  prettyArguments(),
-                 QDir::toNativeSeparators(effectiveWorkingDirectory()));
+                 QDir::toNativeSeparators(effectiveWorkingDirectory().toString()));
 }
 
-void ProcessParameters::resolveAll()
-{
-    effectiveCommand();
-    effectiveArguments();
-    effectiveWorkingDirectory();
-}
+} // ProcessExplorer
