@@ -24,6 +24,8 @@
 ****************************************************************************/
 
 #include "projectconfiguration.h"
+
+#include "kitinformation.h"
 #include "target.h"
 
 #include <utils/algorithm.h>
@@ -33,6 +35,7 @@
 #include <QWidget>
 
 using namespace ProjectExplorer;
+using namespace Utils;
 
 const char CONFIGURATION_ID_KEY[] = "ProjectExplorer.ProjectConfiguration.Id";
 const char DISPLAY_NAME_KEY[] = "ProjectExplorer.ProjectConfiguration.DisplayName";
@@ -43,6 +46,8 @@ ProjectConfiguration::ProjectConfiguration(QObject *parent, Utils::Id id)
     : QObject(parent)
     , m_id(id)
 {
+    m_aspects.setOwnsSubAspects(true);
+
     QTC_CHECK(parent);
     QTC_CHECK(id.isValid());
     setObjectName(id.toString());
@@ -140,7 +145,25 @@ void ProjectConfiguration::acquaintAspects()
         aspect->acquaintSiblings(m_aspects);
 }
 
+void ProjectConfiguration::doPostInit()
+{
+    for (const std::function<void()> &postInit : qAsConst(m_postInit))
+        postInit();
+}
+
+FilePath ProjectConfiguration::mapFromBuildDeviceToGlobalPath(const FilePath &path) const
+{
+    IDevice::ConstPtr dev = BuildDeviceKitAspect::device(kit());
+    QTC_ASSERT(dev, return path);
+    return dev->mapToGlobalPath(path);
+}
+
 Utils::Id ProjectExplorer::idFromMap(const QVariantMap &map)
 {
     return Utils::Id::fromSetting(map.value(QLatin1String(CONFIGURATION_ID_KEY)));
 }
+
+QString ProjectConfiguration::expandedDisplayName() const
+{
+    return m_target->macroExpander()->expand(m_displayName.value());
+}
