@@ -64,6 +64,8 @@ const Icon DEVICE_DISCONNECTED_INDICATOR({
         {":/utils/images/filledcircle.png", Theme::IconsStopColor}}, Icon::Tint);
 const Icon DEVICE_DISCONNECTED_INDICATOR_OVERLAY({
         {":/projectexplorer/images/devicestatusindicator.png", Theme::IconsStopToolBarColor}});
+const Icon WIZARD_IMPORT_AS_PROJECT({
+        {":/projectexplorer/images/importasproject.png", Theme::PanelTextColorDark}}, Icon::Tint);
 
 const Icon DEBUG_START_FLAT({
         {":/projectexplorer/images/run_mask.png", Theme::IconsRunToolBarColor},
