@@ -35,17 +35,17 @@
 #include <utils/stringutils.h>
 
 #include <coreplugin/icore.h>
-#include <texteditor/texteditorsettings.h>
-#include <texteditor/icodestylepreferences.h>
-#include <texteditor/icodestylepreferencesfactory.h>
-#include <texteditor/normalindenter.h>
-#include <texteditor/tabsettings.h>
-#include <texteditor/storagesettings.h>
+#include <projectexplorer/editorconfiguration.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projecttree.h>
-#include <projectexplorer/editorconfiguration.h>
+#include <texteditor/icodestylepreferences.h>
+#include <texteditor/icodestylepreferencesfactory.h>
+#include <texteditor/storagesettings.h>
+#include <texteditor/tabsettings.h>
+#include <texteditor/texteditorsettings.h>
+#include <texteditor/textindenter.h>
 #include <utils/mimetypes/mimedatabase.h>
-#
+
 #include <QPointer>
 #include <QDebug>
 #include <QFileInfo>
@@ -55,6 +55,7 @@
 
 using namespace TextEditor;
 using namespace Core;
+using namespace Utils;
 
 /*!
     \class ProjectExplorer::Internal::ProjectFileWizardExtension
@@ -102,12 +103,12 @@ ProjectFileWizardExtension::~ProjectFileWizardExtension()
     delete m_context;
 }
 
-static QString generatedProjectFilePath(const QList<GeneratedFile> &files)
+static FilePath generatedProjectFilePath(const QList<GeneratedFile> &files)
 {
-    foreach (const GeneratedFile &file, files)
+    for (const GeneratedFile &file : files)
         if (file.attributes() & GeneratedFile::OpenProjectAttribute)
-            return file.path();
-    return QString();
+            return file.filePath();
+    return {};
 }
 
 void ProjectFileWizardExtension::firstExtensionPageShown(
@@ -120,7 +121,7 @@ void ProjectFileWizardExtension::firstExtensionPageShown(
     QStringList fileNames = Utils::transform(files, &GeneratedFile::path);
     m_context->page->setFiles(fileNames);
 
-    QStringList filePaths;
+    FilePaths filePaths;
     ProjectAction projectAction;
     const IWizardFactory::WizardKind kind = m_context->wizard->kind();
     if (kind == IWizardFactory::ProjectWizard) {
@@ -128,13 +129,13 @@ void ProjectFileWizardExtension::firstExtensionPageShown(
         filePaths << generatedProjectFilePath(files);
     } else {
         projectAction = AddNewFile;
-        filePaths = Utils::transform(files, &GeneratedFile::path);
+        filePaths = Utils::transform(files, &GeneratedFile::filePath);
     }
 
     // Static cast from void * to avoid qobject_cast (which needs a valid object) in value().
     auto contextNode = static_cast<Node *>(extraValues.value(QLatin1String(Constants::PREFERRED_PROJECT_NODE)).value<void *>());
     auto project = static_cast<Project *>(extraValues.value(Constants::PROJECT_POINTER).value<void *>());
-    const QString path = extraValues.value(Constants::PREFERRED_PROJECT_NODE_PATH).toString();
+    const FilePath path = FilePath::fromVariant(extraValues.value(Constants::PREFERRED_PROJECT_NODE_PATH));
 
     m_context->page->initializeProjectTree(findWizardContextNode(contextNode, project, path),
                                            filePaths, m_context->wizard->kind(),
@@ -151,12 +152,12 @@ void ProjectFileWizardExtension::firstExtensionPageShown(
 }
 
 Node *ProjectFileWizardExtension::findWizardContextNode(Node *contextNode, Project *project,
-                                                        const QString &path)
+                                                        const FilePath &path)
 {
     if (contextNode && !ProjectTree::hasNode(contextNode)) {
         if (SessionManager::projects().contains(project) && project->rootProjectNode()) {
             contextNode = project->rootProjectNode()->findNode([path](const Node *n) {
-                return path == n->filePath().toString();
+                return path == n->filePath();
             });
         }
     }
@@ -189,7 +190,7 @@ bool ProjectFileWizardExtension::processFiles(
             errorMessage->clear();
         }
         message.append(tr("Open project anyway?"));
-        if (QMessageBox::question(ICore::mainWindow(), tr("Version Control Failure"), message,
+        if (QMessageBox::question(ICore::dialogParent(), tr("Version Control Failure"), message,
                                   QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
             return false;
     }
@@ -203,7 +204,7 @@ bool ProjectFileWizardExtension::processProject(
 {
     *removeOpenProjectAttribute = false;
 
-    QString generatedProject = generatedProjectFilePath(files);
+    const FilePath generatedProject = generatedProjectFilePath(files);
 
     FolderNode *folder = m_context->page->currentNode();
     if (!folder)
@@ -211,15 +212,16 @@ bool ProjectFileWizardExtension::processProject(
     if (m_context->wizard->kind() == IWizardFactory::ProjectWizard) {
         if (!static_cast<ProjectNode *>(folder)->addSubProject(generatedProject)) {
             *errorMessage = tr("Failed to add subproject \"%1\"\nto project \"%2\".")
-                            .arg(generatedProject).arg(folder->filePath().toUserOutput());
+                            .arg(generatedProject.toUserOutput()).arg(folder->filePath().toUserOutput());
             return false;
         }
         *removeOpenProjectAttribute = true;
     } else {
-        QStringList filePaths = Utils::transform(files, &GeneratedFile::path);
+        FilePaths filePaths = Utils::transform(files, &GeneratedFile::filePath);
         if (!folder->addFiles(filePaths)) {
-            *errorMessage = tr("Failed to add one or more files to project\n\"%1\" (%2).").
-                    arg(folder->filePath().toUserOutput(), filePaths.join(QLatin1Char(',')));
+            *errorMessage = tr("Failed to add one or more files to project\n\"%1\" (%2).")
+                    .arg(folder->filePath().toUserOutput())
+                    .arg(FilePath::formatFilePaths(filePaths, ","));
             return false;
         }
     }
@@ -256,10 +258,10 @@ void ProjectFileWizardExtension::applyCodeStyle(GeneratedFile *file) const
     Indenter *indenter = nullptr;
     if (factory) {
         indenter = factory->createIndenter(&doc);
-        indenter->setFileName(Utils::FilePath::fromString(file->path()));
+        indenter->setFileName(file->filePath());
     }
     if (!indenter)
-        indenter = new NormalIndenter(&doc);
+        indenter = new TextIndenter(&doc);
 
     ICodeStylePreferences *codeStylePrefs = codeStylePreferences(baseProject, languageId);
     indenter->setCodeStylePreferences(codeStylePrefs);
@@ -273,7 +275,7 @@ void ProjectFileWizardExtension::applyCodeStyle(GeneratedFile *file) const
     if (TextEditorSettings::storageSettings().m_cleanWhitespace) {
         QTextBlock block = doc.firstBlock();
         while (block.isValid()) {
-            codeStylePrefs->currentTabSettings().removeTrailingWhitespace(cursor, block);
+            TabSettings::removeTrailingWhitespace(cursor, block);
             block = block.next();
         }
     }
