@@ -43,6 +43,7 @@
 
 #include <QLoggingCategory>
 #include <QMessageBox>
+#include <QPushButton>
 #include <QString>
 
 namespace ProjectExplorer {
@@ -109,12 +110,28 @@ const QList<BuildInfo> ProjectImporter::import(const Utils::FilePath &importPath
                                   .arg(importPath.toUserOutput(), projectFilePath().toUserOutput()));
     };
     qCDebug(log) << "Examining directory" << absoluteImportPath.toString();
-    QList<void *> dataList = examineDirectory(absoluteImportPath);
+    QString warningMessage;
+    QList<void *> dataList = examineDirectory(absoluteImportPath, &warningMessage);
     if (dataList.isEmpty()) {
         qCDebug(log) << "Nothing to import found in" << absoluteImportPath.toString();
         handleFailure();
         return result;
     }
+    if (!warningMessage.isEmpty()) {
+        qCDebug(log) << "Warning when examining" << absoluteImportPath.toString();
+        // we should ask user before importing
+        if (silent)
+            return result;
+        QMessageBox dialog(Core::ICore::dialogParent());
+        dialog.setWindowTitle(tr("Import Warning"));
+        dialog.setText(warningMessage);
+        dialog.setIcon(QMessageBox::Warning);
+        QPushButton *acceptButton = dialog.addButton(tr("Import Build"), QMessageBox::AcceptRole);
+        dialog.addButton(QMessageBox::Cancel);
+        dialog.exec();
+        if (dialog.clickedButton() != acceptButton)
+            return result;
+    }
 
     qCDebug(log) << "Looking for kits";
     foreach (void *data, dataList) {
@@ -380,7 +397,7 @@ static ProjectImporter::ToolChainData createToolChains(const ToolChainDescriptio
         if (data.tcs.isEmpty())
             continue;
 
-        for (ToolChain *tc : data.tcs)
+        for (ToolChain *tc : qAsConst(data.tcs))
             ToolChainManager::registerToolChain(tc);
 
         data.areTemporary = true;
@@ -399,7 +416,7 @@ ProjectImporter::findOrCreateToolChains(const ToolChainDescription &tcd) const
                Utils::Environment::systemEnvironment().isSameExecutable(
                     tc->compilerCommand().toString(), tcd.compilerPath.toString());
     });
-    for (const ToolChain *tc : result.tcs) {
+    for (const ToolChain *tc : qAsConst(result.tcs)) {
         const QByteArray tcId = tc->id();
         result.areTemporary = result.areTemporary ? true : hasKitWithTemporaryData(ToolChainKitAspect::id(), tcId);
     }
