@@ -61,14 +61,44 @@
 #include <QVBoxLayout>
 
 #include <functional>
+#include <iterator>
 #include <tuple>
 #include <vector>
 
+using namespace Core;
 using namespace Utils;
 
 namespace ProjectExplorer {
 namespace Internal {
 
+/// An output iterator whose assignment operator appends a clone of the operand to the list of
+/// children of the WrapperNode passed to the constructor.
+class Appender
+{
+public:
+    using iterator_category = std::output_iterator_tag;
+    using value_type = void;
+    using difference_type = void;
+    using pointer = void;
+    using reference = void;
+
+    explicit Appender(WrapperNode *parent) : m_parent(parent) {}
+
+    Appender &operator=(const WrapperNode *node)
+    {
+        if (node)
+            m_parent->appendClone(*node);
+        return *this;
+    }
+
+    Appender &operator*() { return *this; }
+    Appender &operator++() { return *this; }
+    Appender &operator++(int) { return *this; }
+
+private:
+    WrapperNode *m_parent;
+};
+
 bool compareNodes(const Node *n1, const Node *n2)
 {
     if (n1->priority() > n2->priority())
@@ -82,9 +112,7 @@ bool compareNodes(const Node *n1, const Node *n2)
 
     const int filePathResult = caseFriendlyCompare(n1->filePath().toString(),
                                  n2->filePath().toString());
-    if (filePathResult != 0)
-        return filePathResult < 0;
-    return n1 < n2; // sort by pointer value
+    return filePathResult < 0;
 }
 
 static bool sortWrapperNodes(const WrapperNode *w1, const WrapperNode *w2)
@@ -92,6 +120,71 @@ static bool sortWrapperNodes(const WrapperNode *w1, const WrapperNode *w2)
     return compareNodes(w1->m_node, w2->m_node);
 }
 
+/// Appends to `dest` clones of children of `first` and `second`, removing duplicates (recursively).
+///
+/// \param first, second
+///   Nodes with children sorted by sortWrapperNodes.
+/// \param dest
+///   Node to which to append clones of children of `first` and `second`, with duplicates removed.
+static void appendMergedChildren(const WrapperNode *first, const WrapperNode *second, WrapperNode *dest)
+{
+    setUnionMerge(first->begin(), first->end(),
+                  second->begin(), second->end(),
+                  Appender(dest),
+                  [dest](const WrapperNode *childOfFirst, const WrapperNode *childOfSecond)
+                  -> const WrapperNode * {
+                      if (childOfSecond->hasChildren()) {
+                          if (childOfFirst->hasChildren()) {
+                              WrapperNode *mergeResult = new WrapperNode(childOfFirst->m_node);
+                              dest->appendChild(mergeResult);
+                              appendMergedChildren(childOfFirst, childOfSecond, mergeResult);
+                              // mergeResult has already been appended to the parent's list of
+                              // children -- there's no need for the Appender to do it again.
+                              // That's why we return a null pointer.
+                              return nullptr;
+                          } else {
+                              return childOfSecond;
+                          }
+                      } else {
+                          return childOfFirst;
+                      }
+                  },
+                  sortWrapperNodes);
+}
+
+/// Given a node `parent` with children sorted by the criteria defined in sortWrapperNodes(), merge
+/// any children that are equal according to those criteria.
+static void mergeDuplicates(WrapperNode *parent)
+{
+    // We assume all descendants of 'parent' are sorted
+    int childIndex = 0;
+    while (childIndex + 1 < parent->childCount()) {
+        const WrapperNode *child = parent->childAt(childIndex);
+        const WrapperNode *nextChild = parent->childAt(childIndex + 1);
+        Q_ASSERT_X(!sortWrapperNodes(nextChild, child), __func__, "Children are not sorted");
+        if (!sortWrapperNodes(child, nextChild)) {
+            // child and nextChild must have the same priorities, display names and folder paths.
+            // Replace them by a single node 'mergeResult` containing the union of their children.
+            auto mergeResult = new WrapperNode(child->m_node);
+            parent->insertChild(childIndex, mergeResult);
+            appendMergedChildren(child, nextChild, mergeResult);
+            // Now we can remove the original children
+            parent->removeChildAt(childIndex + 2);
+            parent->removeChildAt(childIndex + 1);
+        } else {
+            ++childIndex;
+        }
+    }
+}
+
+void WrapperNode::appendClone(const WrapperNode &node)
+{
+    WrapperNode *clone = new WrapperNode(node.m_node);
+    appendChild(clone);
+    for (const WrapperNode *child : node)
+        clone->appendClone(*child);
+}
+
 FlatModel::FlatModel(QObject *parent)
     : TreeModel<WrapperNode, WrapperNode>(new WrapperNode(nullptr), parent)
 {
@@ -107,9 +200,6 @@ FlatModel::FlatModel(QObject *parent)
 
     for (Project *project : SessionManager::projects())
         handleProjectAdded(project);
-
-    m_disabledTextColor = Utils::creatorTheme()->color(Utils::Theme::TextColorDisabled);
-    m_enabledTextColor = Utils::creatorTheme()->color(Utils::Theme::TextColorNormal);
 }
 
 QVariant FlatModel::data(const QModelIndex &index, int role) const
@@ -145,7 +235,7 @@ QVariant FlatModel::data(const QModelIndex &index, int role) const
     }
     case Qt::DecorationRole: {
         if (!folderNode)
-            return Core::FileIconProvider::icon(node->filePath().toFileInfo());
+            return node->asFileNode()->icon();
         if (!project)
             return folderNode->icon();
         static QIcon warnIcon = Utils::Icons::WARNING.icon();
@@ -166,7 +256,8 @@ QVariant FlatModel::data(const QModelIndex &index, int role) const
         return font;
     }
     case Qt::ForegroundRole:
-        return node->isEnabled() ? m_enabledTextColor : m_disabledTextColor;
+        return node->isEnabled() ? QVariant()
+                                 : Utils::creatorTheme()->color(Utils::Theme::TextColorDisabled);
     case Project::FilePathRole:
         return node->filePath().toString();
     case Project::isParsingRole:
@@ -214,25 +305,35 @@ bool FlatModel::setData(const QModelIndex &index, const QVariant &value, int rol
 
     // The base name of the file was changed. Go look for other files with the same base name
     // and offer to rename them as well.
-    if (orgFilePath != newFilePath && orgFileInfo.suffix() == newFilePath.toFileInfo().suffix()) {
+    if (orgFilePath != newFilePath && orgFilePath.suffix() == newFilePath.suffix()) {
         const QList<Node *> candidateNodes = ProjectTree::siblingsWithSameBaseName(node);
         if (!candidateNodes.isEmpty()) {
+            QStringList fileNames = transform<QStringList>(candidateNodes, [](const Node *n) {
+                return n->filePath().fileName();
+            });
+            fileNames.removeDuplicates();
             const QMessageBox::StandardButton reply = QMessageBox::question(
                         Core::ICore::dialogParent(), tr("Rename More Files?"),
                         tr("Would you like to rename these files as well?\n    %1")
-                        .arg(transform<QStringList>(candidateNodes, [](const Node *n) {
-                                 return n->filePath().toFileInfo().fileName();
-                             }).join("\n    ")));
-            if (reply == QMessageBox::Yes) {
+                        .arg(fileNames.join("\n    ")),
+                        QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel,
+                        QMessageBox::Yes);
+            switch (reply) {
+            case QMessageBox::Yes:
                 for (Node * const n : candidateNodes) {
                     QString targetFilePath = orgFileInfo.absolutePath() + '/'
-                            + newFilePath.toFileInfo().completeBaseName();
-                    const QString suffix = n->filePath().toFileInfo().suffix();
+                                             + newFilePath.completeBaseName();
+                    const QString suffix = n->filePath().suffix();
                     if (!suffix.isEmpty())
                         targetFilePath.append('.').append(suffix);
                     toRename.emplace_back(std::make_tuple(n, n->filePath(),
                                                           FilePath::fromString(targetFilePath)));
                 }
+                break;
+            case QMessageBox::Cancel:
+                return false;
+            default:
+                break;
             }
         }
     }
@@ -395,13 +496,24 @@ void FlatModel::saveExpandData()
 
 void FlatModel::addFolderNode(WrapperNode *parent, FolderNode *folderNode, QSet<Node *> *seen)
 {
+    bool hasHiddenSourcesOrHeaders = false;
+
     for (Node *node : folderNode->nodes()) {
         if (m_filterGeneratedFiles && node->isGenerated())
             continue;
         if (m_filterDisabledFiles && !node->isEnabled())
             continue;
         if (FolderNode *subFolderNode = node->asFolderNode()) {
-            const bool isHidden = m_filterProjects && !subFolderNode->showInSimpleTree();
+            bool isHidden = m_filterProjects && !subFolderNode->showInSimpleTree();
+            if (m_hideSourceGroups) {
+                if (subFolderNode->isVirtualFolderType()) {
+                    auto vnode = static_cast<VirtualFolderNode *>(subFolderNode);
+                    if (vnode->isSourcesOrHeaders()) {
+                        isHidden = true;
+                        hasHiddenSourcesOrHeaders = true;
+                    }
+                }
+            }
             if (!isHidden && !seen->contains(subFolderNode)) {
                 seen->insert(subFolderNode);
                 auto node = new WrapperNode(subFolderNode);
@@ -418,6 +530,11 @@ void FlatModel::addFolderNode(WrapperNode *parent, FolderNode *folderNode, QSet<
             }
         }
     }
+
+    if (hasHiddenSourcesOrHeaders) {
+        parent->sortChildren(&sortWrapperNodes);
+        mergeDuplicates(parent);
+    }
 }
 
 bool FlatModel::trimEmptyDirectories(WrapperNode *parent)
@@ -446,10 +563,10 @@ QStringList FlatModel::mimeTypes() const
 QMimeData *FlatModel::mimeData(const QModelIndexList &indexes) const
 {
     auto data = new DropMimeData;
-    foreach (const QModelIndex &index, indexes) {
+    for (const QModelIndex &index : indexes) {
         if (Node *node = nodeForIndex(index)) {
             if (node->asFileNode())
-                data->addFile(node->filePath().toString());
+                data->addFile(node->filePath());
             data->addValue(QVariant::fromValue(node));
         }
     }
@@ -617,8 +734,8 @@ bool FlatModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int r
         return true;
 
     // Some helper functions for the file operations.
-    const auto targetFilePath = [&targetDir](const QString &sourceFilePath) {
-        return targetDir.pathAppended(QFileInfo(sourceFilePath).fileName()).toString();
+    const auto targetFilePath = [&targetDir](const FilePath &sourceFilePath) {
+        return targetDir.pathAppended(sourceFilePath.fileName());
     };
 
     struct VcsInfo {
@@ -635,29 +752,26 @@ bool FlatModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int r
         if (it != vcsHash.constEnd())
             return it.value();
         VcsInfo vcsInfo;
-        vcsInfo.vcs = Core::VcsManager::findVersionControlForDirectory(dir, &vcsInfo.repoDir);
+        vcsInfo.vcs = VcsManager::findVersionControlForDirectory(FilePath::fromString(dir), &vcsInfo.repoDir);
         vcsHash.insert(dir, vcsInfo);
         return vcsInfo;
     };
 
     // Now do the actual work.
-    const QStringList sourceFiles = transform(fileNodes, [](const Node *n) {
-        return n->filePath().toString();
-    });
-    QStringList failedRemoveFromProject;
-    QStringList failedAddToProject;
-    QStringList failedCopyOrMove;
-    QStringList failedDelete;
-    QStringList failedVcsOp;
+    const FilePaths sourceFiles = transform(fileNodes, [](const Node *n) { return n->filePath(); });
+    FilePaths failedRemoveFromProject;
+    FilePaths failedAddToProject;
+    FilePaths failedCopyOrMove;
+    FilePaths failedDelete;
+    FilePaths failedVcsOp;
     switch (dlg.dropAction()) {
     case DropAction::CopyWithFiles: {
-        QStringList filesToAdd;
-        Core::IVersionControl * const vcs = Core::VcsManager::findVersionControlForDirectory(
-                    targetDir.toString());
+        FilePaths filesToAdd;
+        IVersionControl * const vcs = VcsManager::findVersionControlForDirectory(targetDir);
         const bool addToVcs = vcs && vcs->supportsOperation(Core::IVersionControl::AddOperation);
-        for (const QString &sourceFile : sourceFiles) {
-            const QString targetFile = targetFilePath(sourceFile);
-            if (QFile::copy(sourceFile, targetFile)) {
+        for (const FilePath &sourceFile : sourceFiles) {
+            const FilePath targetFile = targetFilePath(sourceFile);
+            if (sourceFile.copyFile(targetFile)) {
                 filesToAdd << targetFile;
                 if (addToVcs && !vcs->vcsAdd(targetFile))
                     failedVcsOp << targetFile;
@@ -672,14 +786,14 @@ bool FlatModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int r
         targetProjectNode->addFiles(sourceFiles, &failedAddToProject);
         break;
     case DropAction::MoveWithFiles: {
-        QStringList filesToAdd;
-        QStringList filesToRemove;
+        FilePaths filesToAdd;
+        FilePaths filesToRemove;
         const VcsInfo targetVcs = vcsInfoForFile(targetDir.toString());
         const bool vcsAddPossible = targetVcs.vcs
                 && targetVcs.vcs->supportsOperation(Core::IVersionControl::AddOperation);
-        for (const QString &sourceFile : sourceFiles) {
-            const QString targetFile = targetFilePath(sourceFile);
-            const VcsInfo sourceVcs = vcsInfoForFile(sourceFile);
+        for (const FilePath &sourceFile : sourceFiles) {
+            const FilePath targetFile = targetFilePath(sourceFile);
+            const VcsInfo sourceVcs = vcsInfoForFile(sourceFile.toString());
             if (sourceVcs.vcs && targetVcs.vcs && sourceVcs == targetVcs
                     && sourceVcs.vcs->supportsOperation(Core::IVersionControl::MoveOperation)) {
                 if (sourceVcs.vcs->vcsMove(sourceFile, targetFile)) {
@@ -690,7 +804,7 @@ bool FlatModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int r
                 }
                 continue;
             }
-            if (!QFile::copy(sourceFile, targetFile)) {
+            if (!sourceFile.copyFile(targetFile)) {
                 failedCopyOrMove << sourceFile;
                 continue;
             }
@@ -702,7 +816,7 @@ bool FlatModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int r
                     && !sourceVcs.vcs->vcsDelete(sourceFile)) {
                 failedVcsOp << sourceFile;
             }
-            if (QFile::exists(sourceFile) && !QFile::remove(sourceFile))
+            if (sourceFile.exists() && !sourceFile.removeFile())
                 failedDelete << sourceFile;
             if (vcsAddPossible && !targetVcs.vcs->vcsAdd(targetFile))
                 failedVcsOp << targetFile;
@@ -721,9 +835,8 @@ bool FlatModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int r
     }
 
     // Summary for the user in case anything went wrong.
-    const auto makeUserFileList = [](const QStringList &files) {
-        return transform(files, [](const QString &f) { return QDir::toNativeSeparators(f); })
-                .join("\n  ");
+    const auto makeUserFileList = [](const FilePaths &files) {
+        return FilePath::formatFilePaths(files, "\n  ");
     };
     if (!failedAddToProject.empty() || !failedRemoveFromProject.empty()
             || !failedCopyOrMove.empty() || !failedDelete.empty() || !failedVcsOp.empty()) {
@@ -802,6 +915,14 @@ void FlatModel::setTrimEmptyDirectories(bool filter)
     rebuildModel();
 }
 
+void FlatModel::setHideSourceGroups(bool filter)
+{
+    if (filter == m_hideSourceGroups)
+        return;
+    m_hideSourceGroups = filter;
+    rebuildModel();
+}
+
 bool FlatModel::projectFilterEnabled()
 {
     return m_filterProjects;
