@@ -37,7 +37,6 @@
 #include <coreplugin/iversioncontrol.h>
 #include <coreplugin/vcsmanager.h>
 
-#include <utils/algorithm.h>
 #include <utils/fileutils.h>
 #include <utils/hostosinfo.h>
 #include <utils/mimetypes/mimedatabase.h>
@@ -45,17 +44,23 @@
 #include <utils/pointeralgorithm.h>
 #include <utils/qtcassert.h>
 #include <utils/stringutils.h>
+#include <utils/utilsicons.h>
 
-#include <QFileInfo>
 #include <QDir>
+#include <QFileInfo>
 #include <QIcon>
 #include <QStyle>
+#include <QThread>
 #include <QTimer>
 
 #include <memory>
 
+using namespace Utils;
+
 namespace ProjectExplorer {
 
+QHash<QString, QIcon> DirectoryIcon::m_cache;
+
 static FolderNode *recursiveFindOrCreateFolderNode(FolderNode *folder,
                                                    const Utils::FilePath &directory,
                                                    const Utils::FilePath &overrideBaseDir,
@@ -66,7 +71,7 @@ static FolderNode *recursiveFindOrCreateFolderNode(FolderNode *folder,
     Utils::FilePath directoryWithoutPrefix;
     bool isRelative = false;
 
-    if (path.isEmpty() || path.toFileInfo().isRoot()) {
+    if (path.isEmpty() || path.toDir().isRoot()) {
         directoryWithoutPrefix = directory;
         isRelative = false;
     } else {
@@ -248,6 +253,35 @@ bool Node::isEnabled() const
     return parent ? parent->isEnabled() : true;
 }
 
+QIcon FileNode::icon() const
+{
+    if (hasError())
+        return Utils::Icons::WARNING.icon();
+    if (m_icon.isNull())
+        m_icon = Core::FileIconProvider::icon(filePath());
+    return m_icon;
+}
+
+void FileNode::setIcon(const QIcon icon)
+{
+    m_icon = icon;
+}
+
+bool FileNode::hasError() const
+{
+    return m_hasError;
+}
+
+void FileNode::setHasError(bool error)
+{
+    m_hasError = error;
+}
+
+void FileNode::setHasError(bool error) const
+{
+    m_hasError = error;
+}
+
 /*!
   Returns \c true if the file is automatically generated by a compile step.
   */
@@ -304,32 +338,33 @@ FileType Node::fileTypeForMimeType(const Utils::MimeType &mt)
 
 FileType Node::fileTypeForFileName(const Utils::FilePath &file)
 {
-    return fileTypeForMimeType(Utils::mimeTypeForFile(file.toString(),
-                                                      Utils::MimeMatchMode::MatchExtension));
+    return fileTypeForMimeType(Utils::mimeTypeForFile(file, Utils::MimeMatchMode::MatchExtension));
 }
 
-QString Node::pathOrDirectory(bool dir) const
+FilePath Node::pathOrDirectory(bool dir) const
 {
-    QString location;
+    FilePath location;
     const FolderNode *folder = asFolderNode();
     if (isVirtualFolderType() && folder) {
         // Virtual Folder case
         // If there are files directly below or no subfolders, take the folder path
         if (!folder->fileNodes().isEmpty() || folder->folderNodes().isEmpty()) {
-            location = m_filePath.toString();
+            location = m_filePath;
         } else {
             // Otherwise we figure out a commonPath from the subfolders
             QStringList list;
             foreach (FolderNode *f, folder->folderNodes())
                 list << f->filePath().toString() + QLatin1Char('/');
-            location = Utils::commonPath(list);
+            location = FilePath::fromString(Utils::commonPath(list));
         }
 
-        QFileInfo fi(location);
+        QTC_CHECK(!location.needsDevice());
+        QFileInfo fi = location.toFileInfo();
         while ((!fi.exists() || !fi.isDir()) && !fi.isRoot())
             fi.setFile(fi.absolutePath());
-        location = fi.absoluteFilePath();
+        location = FilePath::fromString(fi.absoluteFilePath());
     } else if (!m_filePath.isEmpty()) {
+        QTC_CHECK(!m_filePath.needsDevice());
         QFileInfo fi = m_filePath.toFileInfo();
         // remove any /suffixes, which e.g. ResourceNode uses
         // Note this could be removed again by making path() a true path again
@@ -338,9 +373,9 @@ QString Node::pathOrDirectory(bool dir) const
             fi.setFile(fi.absolutePath());
 
         if (dir)
-            location = fi.isDir() ? fi.absoluteFilePath() : fi.absolutePath();
+            location = FilePath::fromString(fi.isDir() ? fi.absoluteFilePath() : fi.absolutePath());
         else
-            location = fi.absoluteFilePath();
+            location = FilePath::fromString(fi.absoluteFilePath());
     }
     return location;
 }
@@ -382,62 +417,6 @@ FileType FileNode::fileType() const
     return m_fileType;
 }
 
-static QList<FileNode *> scanForFilesRecursively(QFutureInterface<QList<FileNode*>> &future,
-                                                 double progressStart, double progressRange,
-                                                 const Utils::FilePath &directory,
-                                                 const std::function<FileNode *(const Utils::FilePath &)> factory,
-                                                 QSet<QString> &visited,
-                                                 const QList<Core::IVersionControl*> &versionControls)
-{
-    QList<FileNode *> result;
-
-    const QDir baseDir = QDir(directory.toString());
-
-    // Do not follow directory loops:
-    const int visitedCount = visited.count();
-    visited.insert(baseDir.canonicalPath());
-    if (visitedCount == visited.count())
-        return result;
-
-    const QFileInfoList entries = baseDir.entryInfoList(QStringList(), QDir::AllEntries|QDir::NoDotAndDotDot);
-    double progress = 0;
-    const double progressIncrement = progressRange / static_cast<double>(entries.count());
-    int lastIntProgress = 0;
-    for (const QFileInfo &entry : entries) {
-        if (future.isCanceled())
-            return result;
-
-        const Utils::FilePath entryName = Utils::FilePath::fromString(entry.absoluteFilePath());
-        if (!Utils::contains(versionControls, [&entryName](const Core::IVersionControl *vc) {
-                             return vc->isVcsFileOrDirectory(entryName);
-            })) {
-            if (entry.isDir())
-                result.append(scanForFilesRecursively(future, progress, progressIncrement, entryName, factory, visited, versionControls));
-            else if (FileNode *node = factory(entryName))
-                result.append(node);
-        }
-        progress += progressIncrement;
-        const int intProgress = std::min(static_cast<int>(progressStart + progress), future.progressMaximum());
-        if (lastIntProgress < intProgress) {
-            future.setProgressValue(intProgress);
-            lastIntProgress = intProgress;
-        }
-    }
-    future.setProgressValue(std::min(static_cast<int>(progressStart + progressRange), future.progressMaximum()));
-    return result;
-}
-
-QList<FileNode *>
-FileNode::scanForFiles(QFutureInterface<QList<FileNode *>> &future,
-                       const Utils::FilePath &directory,
-                       const std::function<FileNode *(const Utils::FilePath &)> factory)
-{
-    QSet<QString> visited;
-    future.setProgressRange(0, 1000000);
-    return scanForFilesRecursively(future, 0.0, 1000000.0, directory, factory, visited,
-                                   Core::VcsManager::versionControls());
-}
-
 bool FileNode::supportsAction(ProjectAction action, const Node *node) const
 {
     if (action == InheritedFromParent)
@@ -481,16 +460,28 @@ QString FolderNode::displayName() const
 }
 
 /*!
-  Contains the icon that should be used in a view. Default is the directory icon
- (QStyle::S_PDirIcon).
-  s\a setIcon()
+    Contains the icon that should be used in a view. Default is the directory icon
+    (QStyle::S_PDirIcon). Calling this method is only safe in the UI thread.
+
+    \sa setIcon()
  */
 QIcon FolderNode::icon() const
 {
+    QTC_CHECK(QThread::currentThread() == QCoreApplication::instance()->thread());
+
     // Instantiating the Icon provider is expensive.
-    if (m_icon.isNull())
-        m_icon = Core::FileIconProvider::icon(QFileIconProvider::Folder);
-    return m_icon;
+    if (auto strPtr = Utils::get_if<QString>(&m_icon)) {
+        m_icon = QIcon(*strPtr);
+    } else if (auto directoryIconPtr = Utils::get_if<DirectoryIcon>(&m_icon)) {
+        m_icon = directoryIconPtr->icon();
+    } else if (auto creatorPtr = Utils::get_if<IconCreator>(&m_icon)) {
+        m_icon = (*creatorPtr)();
+    } else {
+        auto iconPtr = Utils::get_if<QIcon>(&m_icon);
+        if (!iconPtr || iconPtr->isNull())
+            m_icon = Core::FileIconProvider::icon(QFileIconProvider::Folder);
+    }
+    return Utils::get<QIcon>(m_icon);
 }
 
 Node *FolderNode::findNode(const std::function<bool(Node *)> &filter)
@@ -724,11 +715,38 @@ void FolderNode::setDisplayName(const QString &name)
     m_displayName = name;
 }
 
+/*!
+    Sets the \a icon for this node. Note that creating QIcon instances is only safe in the UI thread.
+*/
 void FolderNode::setIcon(const QIcon &icon)
 {
     m_icon = icon;
 }
 
+/*!
+    Sets the \a directoryIcon that is used to create the icon for this node on demand.
+*/
+void FolderNode::setIcon(const DirectoryIcon &directoryIcon)
+{
+    m_icon = directoryIcon;
+}
+
+/*!
+    Sets the \a path that is used to create the icon for this node on demand.
+*/
+void FolderNode::setIcon(const QString &path)
+{
+    m_icon = path;
+}
+
+/*!
+    Sets the \a iconCreator function that is used to create the icon for this node on demand.
+*/
+void FolderNode::setIcon(const IconCreator &iconCreator)
+{
+    m_icon = iconCreator;
+}
+
 void FolderNode::setLocationInfo(const QVector<FolderNode::LocationInfo> &info)
 {
     m_locations = info;
@@ -757,7 +775,7 @@ bool FolderNode::supportsAction(ProjectAction action, const Node *node) const
     return parentFolder && parentFolder->supportsAction(action, node);
 }
 
-bool FolderNode::addFiles(const QStringList &filePaths, QStringList *notAdded)
+bool FolderNode::addFiles(const FilePaths &filePaths, FilePaths *notAdded)
 {
     ProjectNode *pn = managingProject();
     if (pn)
@@ -765,15 +783,14 @@ bool FolderNode::addFiles(const QStringList &filePaths, QStringList *notAdded)
     return false;
 }
 
-RemovedFilesFromProject FolderNode::removeFiles(const QStringList &filePaths,
-                                                QStringList *notRemoved)
+RemovedFilesFromProject FolderNode::removeFiles(const FilePaths &filePaths, FilePaths *notRemoved)
 {
     if (ProjectNode * const pn = managingProject())
         return pn->removeFiles(filePaths, notRemoved);
     return RemovedFilesFromProject::Error;
 }
 
-bool FolderNode::deleteFiles(const QStringList &filePaths)
+bool FolderNode::deleteFiles(const FilePaths &filePaths)
 {
     ProjectNode *pn = managingProject();
     if (pn)
@@ -781,19 +798,19 @@ bool FolderNode::deleteFiles(const QStringList &filePaths)
     return false;
 }
 
-bool FolderNode::canRenameFile(const QString &filePath, const QString &newFilePath)
+bool FolderNode::canRenameFile(const FilePath &oldFilePath, const FilePath &newFilePath)
 {
     ProjectNode *pn = managingProject();
     if (pn)
-        return pn->canRenameFile(filePath, newFilePath);
+        return pn->canRenameFile(oldFilePath, newFilePath);
     return false;
 }
 
-bool FolderNode::renameFile(const QString &filePath, const QString &newFilePath)
+bool FolderNode::renameFile(const FilePath &oldFilePath, const FilePath &newFilePath)
 {
     ProjectNode *pn = managingProject();
     if (pn)
-        return pn->renameFile(filePath, newFilePath);
+        return pn->renameFile(oldFilePath, newFilePath);
     return false;
 }
 
@@ -804,7 +821,7 @@ bool FolderNode::addDependencies(const QStringList &dependencies)
     return false;
 }
 
-FolderNode::AddNewInformation FolderNode::addNewInformation(const QStringList &files, Node *context) const
+FolderNode::AddNewInformation FolderNode::addNewInformation(const FilePaths &files, Node *context) const
 {
     Q_UNUSED(files)
     return AddNewInformation(displayName(), context == this ? 120 : 100);
@@ -852,7 +869,7 @@ bool FolderNode::showWhenEmpty() const
 
   \sa ProjectExplorer::FileNode, ProjectExplorer::ProjectNode
 */
-VirtualFolderNode::VirtualFolderNode(const Utils::FilePath &folderPath) :
+VirtualFolderNode::VirtualFolderNode(const FilePath &folderPath) :
     FolderNode(folderPath)
 {
 }
@@ -870,7 +887,7 @@ VirtualFolderNode::VirtualFolderNode(const Utils::FilePath &folderPath) :
 /*!
   Creates an uninitialized project node object.
   */
-ProjectNode::ProjectNode(const Utils::FilePath &projectFilePath) :
+ProjectNode::ProjectNode(const FilePath &projectFilePath) :
     FolderNode(projectFilePath)
 {
     setPriority(DefaultProjectPriority);
@@ -878,13 +895,13 @@ ProjectNode::ProjectNode(const Utils::FilePath &projectFilePath) :
     setDisplayName(projectFilePath.fileName());
 }
 
-bool ProjectNode::canAddSubProject(const QString &proFilePath) const
+bool ProjectNode::canAddSubProject(const FilePath &proFilePath) const
 {
     Q_UNUSED(proFilePath)
     return false;
 }
 
-bool ProjectNode::addSubProject(const QString &proFilePath)
+bool ProjectNode::addSubProject(const FilePath &proFilePath)
 {
     Q_UNUSED(proFilePath)
     return false;
@@ -895,45 +912,44 @@ QStringList ProjectNode::subProjectFileNamePatterns() const
     return QStringList();
 }
 
-bool ProjectNode::removeSubProject(const QString &proFilePath)
+bool ProjectNode::removeSubProject(const FilePath &proFilePath)
 {
     Q_UNUSED(proFilePath)
     return false;
 }
 
-bool ProjectNode::addFiles(const QStringList &filePaths, QStringList *notAdded)
+bool ProjectNode::addFiles(const FilePaths &filePaths, FilePaths *notAdded)
 {
     if (BuildSystem *bs = buildSystem())
         return bs->addFiles(this, filePaths, notAdded);
     return false;
 }
 
-RemovedFilesFromProject ProjectNode::removeFiles(const QStringList &filePaths,
-                                                 QStringList *notRemoved)
+RemovedFilesFromProject ProjectNode::removeFiles(const FilePaths &filePaths, FilePaths *notRemoved)
 {
     if (BuildSystem *bs = buildSystem())
         return bs->removeFiles(this, filePaths, notRemoved);
     return RemovedFilesFromProject::Error;
 }
 
-bool ProjectNode::deleteFiles(const QStringList &filePaths)
+bool ProjectNode::deleteFiles(const FilePaths &filePaths)
 {
     if (BuildSystem *bs = buildSystem())
         return bs->deleteFiles(this, filePaths);
     return false;
 }
 
-bool ProjectNode::canRenameFile(const QString &filePath, const QString &newFilePath)
+bool ProjectNode::canRenameFile(const Utils::FilePath &oldFilePath, const Utils::FilePath &newFilePath)
 {
     if (BuildSystem *bs = buildSystem())
-        return bs->canRenameFile(this, filePath, newFilePath);
+        return bs->canRenameFile(this, oldFilePath, newFilePath);
     return true;
 }
 
-bool ProjectNode::renameFile(const QString &filePath, const QString &newFilePath)
+bool ProjectNode::renameFile(const Utils::FilePath &oldFilePath, const Utils::FilePath &newFilePath)
 {
     if (BuildSystem *bs = buildSystem())
-        return bs->renameFile(this, filePath, newFilePath);
+        return bs->renameFile(this, oldFilePath, newFilePath);
     return false;
 }
 
@@ -1016,8 +1032,8 @@ QString ContainerNode::displayName() const
 {
     QString name = m_project->displayName();
 
-    const QFileInfo fi = m_project->projectFilePath().toFileInfo();
-    const QString dir = fi.isDir() ? fi.absoluteFilePath() : fi.absolutePath();
+    const FilePath fp = m_project->projectFilePath();
+    const FilePath dir = fp.isDir() ? fp.absoluteFilePath() : fp.absolutePath();
     if (Core::IVersionControl *vc = Core::VcsManager::findVersionControlForDirectory(dir)) {
         QString vcsTopic = vc->vcsTopic(dir);
         if (!vcsTopic.isEmpty())
@@ -1048,4 +1064,34 @@ void ContainerNode::handleSubTreeChanged(FolderNode *node)
     m_project->handleSubTreeChanged(node);
 }
 
+/*!
+    \class ProjectExplorer::DirectoryIcon
+
+    The DirectoryIcon class represents a directory icon with an overlay.
+
+    The QIcon is created on demand and globally cached, so other DirectoryIcon
+    instances with the same overlay share the same QIcon instance.
+*/
+
+/*!
+    Creates a DirectoryIcon for the specified \a overlay.
+*/
+DirectoryIcon::DirectoryIcon(const QString &overlay)
+    : m_overlay(overlay)
+{}
+
+/*!
+    Returns the icon for this DirectoryIcon. Calling this method is only safe in the UI thread.
+*/
+QIcon DirectoryIcon::icon() const
+{
+    QTC_CHECK(QThread::currentThread() == QCoreApplication::instance()->thread());
+    const auto it = m_cache.find(m_overlay);
+    if (it != m_cache.end())
+        return it.value();
+    const QIcon icon = Core::FileIconProvider::directoryIcon(m_overlay);
+    m_cache.insert(m_overlay, icon);
+    return icon;
+}
+
 } // namespace ProjectExplorer
