@@ -34,6 +34,7 @@
 #include <utils/fileutils.h>
 #include <utils/id.h>
 #include <utils/optional.h>
+#include <utils/variant.h>
 
 #include <functional>
 
@@ -93,6 +94,20 @@ class FolderNode;
 class ProjectNode;
 class ContainerNode;
 
+class PROJECTEXPLORER_EXPORT DirectoryIcon
+{
+public:
+    explicit DirectoryIcon(const QString &overlay);
+
+    QIcon icon() const; // only safe in UI thread
+
+private:
+    QString m_overlay;
+    static QHash<QString, QIcon> m_cache;
+};
+
+using IconCreator = std::function<QIcon()>;
+
 // Documentation inside.
 class PROJECTEXPLORER_EXPORT Node
 {
@@ -161,8 +176,8 @@ public:
     static FileType fileTypeForMimeType(const Utils::MimeType &mt);
     static FileType fileTypeForFileName(const Utils::FilePath &file);
 
-    QString path() const { return pathOrDirectory(false); }
-    QString directory() const { return pathOrDirectory(true); }
+    Utils::FilePath path() const { return pathOrDirectory(false); }
+    Utils::FilePath directory() const { return pathOrDirectory(true); }
 
 protected:
     Node();
@@ -172,7 +187,7 @@ protected:
     void setFilePath(const Utils::FilePath &filePath);
 
 private:
-    QString pathOrDirectory(bool dir) const;
+    Utils::FilePath pathOrDirectory(bool dir) const;
 
     FolderNode *m_parentFolderNode = nullptr;
     Utils::FilePath m_filePath;
@@ -200,15 +215,20 @@ public:
     FileNode *asFileNode() final { return this; }
     const FileNode *asFileNode() const final { return this; }
 
-    static QList<FileNode *>
-    scanForFiles(QFutureInterface<QList<FileNode *>> &future,
-                 const Utils::FilePath &directory,
-                 const std::function<FileNode *(const Utils::FilePath &fileName)> factory);
     bool supportsAction(ProjectAction action, const Node *node) const override;
     QString displayName() const override;
 
+    bool hasError() const;
+    void setHasError(const bool error);
+    void setHasError(const bool error) const;
+
+    QIcon icon() const;
+    void setIcon(const QIcon icon);
+
 private:
     FileType m_fileType;
+    mutable QIcon m_icon;
+    mutable bool m_hasError = false;
 };
 
 // Documentation inside.
@@ -218,6 +238,7 @@ public:
     explicit FolderNode(const Utils::FilePath &folderPath);
 
     QString displayName() const override;
+    // only safe from UI thread
     QIcon icon() const;
 
     bool isFolderNodeType() const override { return true; }
@@ -253,7 +274,11 @@ public:
     bool replaceSubtree(Node *oldNode, std::unique_ptr<Node> &&newNode);
 
     void setDisplayName(const QString &name);
+    // you have to make sure the QIcon is created in the UI thread if you are calling setIcon(QIcon)
     void setIcon(const QIcon &icon);
+    void setIcon(const DirectoryIcon &directoryIcon);
+    void setIcon(const QString &path);
+    void setIcon(const IconCreator &iconCreator);
 
     class LocationInfo
     {
@@ -282,12 +307,13 @@ public:
 
     bool supportsAction(ProjectAction action, const Node *node) const override;
 
-    virtual bool addFiles(const QStringList &filePaths, QStringList *notAdded = nullptr);
-    virtual RemovedFilesFromProject removeFiles(const QStringList &filePaths,
-                                                QStringList *notRemoved = nullptr);
-    virtual bool deleteFiles(const QStringList &filePaths);
-    virtual bool canRenameFile(const QString &filePath, const QString &newFilePath);
-    virtual bool renameFile(const QString &filePath, const QString &newFilePath);
+    virtual bool addFiles(const Utils::FilePaths &filePaths, Utils::FilePaths *notAdded = nullptr);
+    virtual RemovedFilesFromProject removeFiles(const Utils::FilePaths &filePaths,
+                                                Utils::FilePaths *notRemoved = nullptr);
+    virtual bool deleteFiles(const Utils::FilePaths &filePaths);
+    virtual bool canRenameFile(const Utils::FilePath &oldFilePath,
+                               const Utils::FilePath &newFilePath);
+    virtual bool renameFile(const Utils::FilePath &oldFilePath, const Utils::FilePath &newFilePath);
     virtual bool addDependencies(const QStringList &dependencies);
 
     class AddNewInformation
@@ -300,7 +326,7 @@ public:
         int priority;
     };
 
-    virtual AddNewInformation addNewInformation(const QStringList &files, Node *context) const;
+    virtual AddNewInformation addNewInformation(const Utils::FilePaths &files, Node *context) const;
 
 
     // determines if node will be shown in the flat view, by default folder and projects aren't shown
@@ -328,7 +354,7 @@ private:
 
     QString m_displayName;
     QString m_addFileFilter;
-    mutable QIcon m_icon;
+    mutable Utils::variant<QIcon, DirectoryIcon, QString, IconCreator> m_icon;
     bool m_showWhenEmpty = false;
 };
 
@@ -339,6 +365,12 @@ public:
 
     bool isFolderNodeType() const override { return false; }
     bool isVirtualFolderType() const override { return true; }
+
+    bool isSourcesOrHeaders() const { return m_isSourcesOrHeaders; }
+    void setIsSourcesOrHeaders(bool on) { m_isSourcesOrHeaders = on; }
+
+private:
+    bool m_isSourcesOrHeaders = false; // "Sources" or "Headers"
 };
 
 // Documentation inside.
@@ -347,10 +379,10 @@ class PROJECTEXPLORER_EXPORT ProjectNode : public FolderNode
 public:
     explicit ProjectNode(const Utils::FilePath &projectFilePath);
 
-    virtual bool canAddSubProject(const QString &proFilePath) const;
-    virtual bool addSubProject(const QString &proFile);
+    virtual bool canAddSubProject(const Utils::FilePath &proFilePath) const;
+    virtual bool addSubProject(const Utils::FilePath &proFile);
     virtual QStringList subProjectFileNamePatterns() const;
-    virtual bool removeSubProject(const QString &proFilePath);
+    virtual bool removeSubProject(const Utils::FilePath &proFilePath);
     virtual Utils::optional<Utils::FilePath> visibleAfterAddFileAction() const {
         return Utils::nullopt;
     }
@@ -359,12 +391,12 @@ public:
     bool isProjectNodeType() const override { return true; }
     bool showInSimpleTree() const override { return true; }
 
-    bool addFiles(const QStringList &filePaths, QStringList *notAdded = nullptr) final;
-    RemovedFilesFromProject removeFiles(const QStringList &filePaths,
-                                        QStringList *notRemoved = nullptr) final;
-    bool deleteFiles(const QStringList &filePaths) final;
-    bool canRenameFile(const QString &filePath, const QString &newFilePath) final;
-    bool renameFile(const QString &filePath, const QString &newFilePath) final;
+    bool addFiles(const Utils::FilePaths &filePaths, Utils::FilePaths *notAdded = nullptr) final;
+    RemovedFilesFromProject removeFiles(const Utils::FilePaths &filePaths,
+                                        Utils::FilePaths *notRemoved = nullptr) final;
+    bool deleteFiles(const Utils::FilePaths &filePaths) final;
+    bool canRenameFile(const Utils::FilePath &oldFilePath, const Utils::FilePath &newFilePath) final;
+    bool renameFile(const Utils::FilePath &oldFilePath, const Utils::FilePath &newFilePath) final;
     bool addDependencies(const QStringList &dependencies) final;
     bool supportsAction(ProjectAction action, const Node *node) const final;
 
