@@ -30,6 +30,7 @@
 #include "projectnodes.h"
 #include "projecttreewidget.h"
 #include "session.h"
+#include "target.h"
 
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
@@ -38,15 +39,16 @@
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
-#include <coreplugin/infobar.h>
 #include <coreplugin/modemanager.h>
 #include <coreplugin/navigationwidget.h>
 #include <coreplugin/vcsmanager.h>
 
 #include <utils/algorithm.h>
+#include <utils/infobar.h>
 #include <utils/qtcassert.h>
 
 #include <QApplication>
+#include <QFileInfo>
 #include <QMenu>
 #include <QTimer>
 
@@ -104,6 +106,18 @@ Project *ProjectTree::currentProject()
     return s_instance->m_currentProject;
 }
 
+Target *ProjectTree::currentTarget()
+{
+    Project *p = currentProject();
+    return p ? p->activeTarget() : nullptr;
+}
+
+BuildSystem *ProjectTree::currentBuildSystem()
+{
+    Target *t = currentTarget();
+    return t ? t->buildSystem() : nullptr;
+}
+
 Node *ProjectTree::currentNode()
 {
     s_instance->update();
@@ -139,13 +153,8 @@ void ProjectTree::nodeChanged(ProjectTreeWidget *widget)
 void ProjectTree::update()
 {
     ProjectTreeWidget *focus = m_focusForContextMenu;
-    static QPointer<ProjectTreeWidget> lastFocusedProjectTreeWidget;
-    if (!focus) {
-        focus = Utils::findOrDefault(m_projectTreeWidgets, &ProjectTree::hasFocus);
-        lastFocusedProjectTreeWidget = focus;
-    }
     if (!focus)
-        focus = lastFocusedProjectTreeWidget;
+        focus = currentWidget();
 
     if (focus)
         updateFromProjectTreeWidget(focus);
@@ -218,7 +227,7 @@ void ProjectTree::setCurrent(Node *node, Project *project)
 
     if (node != m_currentNode) {
         m_currentNode = node;
-        emit currentNodeChanged();
+        emit currentNodeChanged(node);
     }
 
     if (changedProject) {
@@ -231,12 +240,12 @@ void ProjectTree::setCurrent(Node *node, Project *project)
 void ProjectTree::sessionChanged()
 {
     if (m_currentProject) {
-        Core::DocumentManager::setDefaultLocationForNewFiles(m_currentProject->projectDirectory().toString());
+        Core::DocumentManager::setDefaultLocationForNewFiles(m_currentProject->projectDirectory());
     } else if (Project *project = SessionManager::startupProject()) {
-        Core::DocumentManager::setDefaultLocationForNewFiles(project->projectDirectory().toString());
+        Core::DocumentManager::setDefaultLocationForNewFiles(project->projectDirectory());
         updateFromNode(nullptr); // Make startup project current if there is no other current
     } else {
-        Core::DocumentManager::setDefaultLocationForNewFiles(QString());
+        Core::DocumentManager::setDefaultLocationForNewFiles({});
     }
     update();
 }
@@ -271,15 +280,21 @@ void ProjectTree::sessionAndTreeChanged()
     emit treeChanged();
 }
 
+void ProjectTree::expandCurrentNodeRecursively()
+{
+    if (const auto w = currentWidget())
+        w->expandCurrentNodeRecursively();
+}
+
 void ProjectTree::collapseAll()
 {
-    if (auto w = Utils::findOrDefault(s_instance->m_projectTreeWidgets, &ProjectTree::hasFocus))
+    if (const auto w = currentWidget())
         w->collapseAll();
 }
 
 void ProjectTree::expandAll()
 {
-    if (auto w = Utils::findOrDefault(s_instance->m_projectTreeWidgets, &ProjectTree::hasFocus))
+    if (const auto w = currentWidget())
         w->expandAll();
 }
 
@@ -294,8 +309,8 @@ void ProjectTree::updateExternalFileWarning()
     auto document = qobject_cast<Core::IDocument *>(sender());
     if (!document || document->filePath().isEmpty())
         return;
-    Core::InfoBar *infoBar = document->infoBar();
-    Core::Id externalFileId(EXTERNAL_FILE_WARNING);
+    Utils::InfoBar *infoBar = document->infoBar();
+    Utils::Id externalFileId(EXTERNAL_FILE_WARNING);
     if (!document->isModified()) {
         infoBar->removeInfo(externalFileId);
         return;
@@ -314,14 +329,15 @@ void ProjectTree::updateExternalFileWarning()
             return;
         // External file. Test if it under the same VCS
         QString topLevel;
-        if (Core::VcsManager::findVersionControlForDirectory(projectDir.toString(), &topLevel)
+        if (Core::VcsManager::findVersionControlForDirectory(projectDir, &topLevel)
                 && fileName.isChildOf(FilePath::fromString(topLevel))) {
             return;
         }
     }
-    infoBar->addInfo(Core::InfoBarEntry(externalFileId,
-                                        tr("<b>Warning:</b> This file is outside the project directory."),
-                                        Core::InfoBarEntry::GlobalSuppression::Enabled));
+    infoBar->addInfo(
+        Utils::InfoBarEntry(externalFileId,
+                            tr("<b>Warning:</b> This file is outside the project directory."),
+                            Utils::InfoBarEntry::GlobalSuppression::Enabled));
 }
 
 bool ProjectTree::hasFocus(ProjectTreeWidget *widget)
@@ -331,11 +347,15 @@ bool ProjectTree::hasFocus(ProjectTreeWidget *widget)
                 || s_instance->m_focusForContextMenu == widget);
 }
 
+ProjectTreeWidget *ProjectTree::currentWidget() const
+{
+    return findOrDefault(m_projectTreeWidgets, &ProjectTree::hasFocus);
+}
+
 void ProjectTree::showContextMenu(ProjectTreeWidget *focus, const QPoint &globalPos, Node *node)
 {
     QMenu *contextMenu = nullptr;
-    Project *project = projectForNode(node);
-    emit s_instance->aboutToShowContextMenu(project, node);
+    emit s_instance->aboutToShowContextMenu(node);
 
     if (!node) {
         contextMenu = Core::ActionManager::actionContainer(Constants::M_SESSIONCONTEXT)->menu();
@@ -351,7 +371,7 @@ void ProjectTree::showContextMenu(ProjectTreeWidget *focus, const QPoint &global
         contextMenu = Core::ActionManager::actionContainer(Constants::M_FILECONTEXT)->menu();
     }
 
-    if (contextMenu && contextMenu->actions().count() > 0) {
+    if (contextMenu && !contextMenu->actions().isEmpty()) {
         s_instance->m_focusForContextMenu = focus;
         contextMenu->popup(globalPos);
         connect(contextMenu, &QMenu::aboutToHide,
@@ -371,19 +391,24 @@ void ProjectTree::highlightProject(Project *project, const QString &message)
         projectTreeWidget->showMessage(project->rootProjectNode(), message);
 }
 
+/*!
+    Registers the function \a treeChange to be run on a (sub tree of the)
+    project tree when it is created. The function must be thread-safe, and
+    applying the function on the same tree a second time must be a no-op.
+*/
 void ProjectTree::registerTreeManager(const TreeManagerFunction &treeChange)
 {
     if (treeChange)
         s_instance->m_treeManagers.append(treeChange);
 }
 
-void ProjectTree::applyTreeManager(FolderNode *folder)
+void ProjectTree::applyTreeManager(FolderNode *folder, ConstructionPhase phase)
 {
     if (!folder)
         return;
 
     for (TreeManagerFunction &f : s_instance->m_treeManagers)
-        f(folder);
+        f(folder, phase);
 }
 
 bool ProjectTree::hasNode(const Node *node)
@@ -433,22 +458,54 @@ Node *ProjectTree::nodeForFile(const FilePath &fileName)
 {
     Node *node = nullptr;
     for (const Project *project : SessionManager::projects()) {
-        if (ProjectNode *projectNode = project->rootProjectNode()) {
-            projectNode->forEachGenericNode([&](Node *n) {
-                if (n->filePath() == fileName) {
-                    // prefer file nodes
-                    if (!node || (!node->asFileNode() && n->asFileNode()))
-                        node = n;
-                }
-            });
-        }
+        project->nodeForFilePath(fileName, [&](const Node *n) {
+            if (!node || (!node->asFileNode() && n->asFileNode()))
+                node = const_cast<Node *>(n);
+            return false;
+        });
+        // early return:
+        if (node && node->asFileNode())
+            return node;
     }
     return node;
 }
 
+const QList<Node *> ProjectTree::siblingsWithSameBaseName(const Node *fileNode)
+{
+    ProjectNode *productNode = fileNode->parentProjectNode();
+    while (productNode && !productNode->isProduct())
+        productNode = productNode->parentProjectNode();
+    if (!productNode)
+        return {};
+    const QFileInfo fi = fileNode->filePath().toFileInfo();
+    const auto filter = [&fi](const Node *n) {
+        return n->asFileNode()
+                && n->filePath().toFileInfo().dir() == fi.dir()
+                && n->filePath().completeBaseName() == fi.completeBaseName()
+                && n->filePath().toString() != fi.filePath();
+    };
+    return productNode->findNodes(filter);
+}
+
 void ProjectTree::hideContextMenu()
 {
-    m_focusForContextMenu = nullptr;
+    if (m_keepCurrentNodeRequests == 0)
+        m_focusForContextMenu = nullptr;
+}
+
+ProjectTree::CurrentNodeKeeper::CurrentNodeKeeper()
+    : m_active(ProjectTree::instance()->m_focusForContextMenu)
+{
+    if (m_active)
+        ++ProjectTree::instance()->m_keepCurrentNodeRequests;
+}
+
+ProjectTree::CurrentNodeKeeper::~CurrentNodeKeeper()
+{
+    if (m_active && --ProjectTree::instance()->m_keepCurrentNodeRequests == 0) {
+        ProjectTree::instance()->m_focusForContextMenu = nullptr;
+        ProjectTree::instance()->update();
+    }
 }
 
 } // namespace ProjectExplorer
