@@ -152,11 +152,11 @@ public:
         setDragDropMode(QAbstractItemView::DragDrop);
         viewport()->setAcceptDrops(true);
         setDropIndicatorShown(true);
-        m_context = new IContext(this);
-        m_context->setContext(Context(ProjectExplorer::Constants::C_PROJECT_TREE));
-        m_context->setWidget(this);
+        auto context = new IContext(this);
+        context->setContext(Context(ProjectExplorer::Constants::C_PROJECT_TREE));
+        context->setWidget(this);
 
-        ICore::addContextObject(m_context);
+        ICore::addContextObject(context);
 
         connect(this, &ProjectTreeView::expanded,
                 this, &ProjectTreeView::invalidateSize);
@@ -204,12 +204,6 @@ public:
         NavigationTreeView::setModel(newModel);
     }
 
-    ~ProjectTreeView() override
-    {
-        ICore::removeContextObject(m_context);
-        delete m_context;
-    }
-
     int sizeHintForColumn(int column) const override
     {
         if (m_cachedSize < 0)
@@ -220,7 +214,6 @@ public:
 
 private:
     mutable int m_cachedSize = -1;
-    IContext *m_context;
 };
 
 /*!
@@ -257,6 +250,12 @@ ProjectTreeWidget::ProjectTreeWidget(QWidget *parent) : QWidget(parent)
     connect(m_filterGeneratedFilesAction, &QAction::toggled,
             this, &ProjectTreeWidget::setGeneratedFilesFilter);
 
+    m_filterDisabledFilesAction = new QAction(tr("Hide Disabled Files"), this);
+    m_filterDisabledFilesAction->setCheckable(true);
+    m_filterDisabledFilesAction->setChecked(false);
+    connect(m_filterDisabledFilesAction, &QAction::toggled,
+            this, &ProjectTreeWidget::setDisabledFilesFilter);
+
     const char focusActionId[] = "ProjectExplorer.FocusDocumentInProjectTree";
     if (!ActionManager::command(focusActionId)) {
         auto focusDocumentInProjectTree = new QAction(tr("Focus Document in Project Tree"), this);
@@ -274,6 +273,12 @@ ProjectTreeWidget::ProjectTreeWidget(QWidget *parent) : QWidget(parent)
     connect(m_trimEmptyDirectoriesAction, &QAction::toggled,
             this, &ProjectTreeWidget::setTrimEmptyDirectories);
 
+    m_hideSourceGroupsAction = new QAction(tr("Hide Source and Header Groups"), this);
+    m_hideSourceGroupsAction->setCheckable(true);
+    m_hideSourceGroupsAction->setChecked(false);
+    connect(m_hideSourceGroupsAction, &QAction::toggled,
+            this, &ProjectTreeWidget::setHideSourceGroups);
+
     // connections
     connect(m_model, &FlatModel::renamed,
             this, &ProjectTreeWidget::renamed);
@@ -290,13 +295,12 @@ ProjectTreeWidget::ProjectTreeWidget(QWidget *parent) : QWidget(parent)
     connect(m_view, &QTreeView::collapsed,
             m_model, &FlatModel::onCollapsed);
 
-    m_toggleSync = new QToolButton;
+    m_toggleSync = new QAction(this);
     m_toggleSync->setIcon(Icons::LINK_TOOLBAR.icon());
     m_toggleSync->setCheckable(true);
     m_toggleSync->setChecked(autoSynchronization());
     m_toggleSync->setToolTip(tr("Synchronize with Editor"));
-    connect(m_toggleSync, &QAbstractButton::clicked,
-            this, &ProjectTreeWidget::toggleAutoSynchronization);
+    connect(m_toggleSync, &QAction::triggered, this, &ProjectTreeWidget::toggleAutoSynchronization);
 
     setCurrentItem(ProjectTree::currentNode());
     setAutoSynchronization(true);
@@ -354,32 +358,20 @@ void ProjectTreeWidget::rowsInserted(const QModelIndex &parent, int start, int e
 
 Node *ProjectTreeWidget::nodeForFile(const FilePath &fileName)
 {
+    if (fileName.isEmpty())
+        return nullptr;
     Node *bestNode = nullptr;
     int bestNodeExpandCount = INT_MAX;
 
-    // FIXME: Check that the values used make sense in the context.
-    auto priority = [](Node *node) {
-        if (node->asFileNode())
-            return 1;
-        if (node->isFolderNodeType())
-            return 2;
-        if (node->isVirtualFolderType())
-            return 3;
-        if (node->isProjectNodeType())
-            return 4;
-        QTC_CHECK(false);
-        return 1;
-    };
-
     // FIXME: Looks like this could be done with less cycles.
     for (Project *project : SessionManager::projects()) {
         if (ProjectNode *projectNode = project->rootProjectNode()) {
             projectNode->forEachGenericNode([&](Node *node) {
                 if (node->filePath() == fileName) {
-                    if (!bestNode || priority(node) < priority(bestNode)) {
+                    if (!bestNode || node->priority() < bestNode->priority()) {
                         bestNode = node;
                         bestNodeExpandCount = ProjectTreeWidget::expandedCount(node);
-                    } else if (priority(node) == priority(bestNode)) {
+                    } else if (node->priority() == bestNode->priority()) {
                         int nodeExpandCount = ProjectTreeWidget::expandedCount(node);
                         if (nodeExpandCount < bestNodeExpandCount) {
                             bestNode = node;
@@ -394,11 +386,6 @@ Node *ProjectTreeWidget::nodeForFile(const FilePath &fileName)
     return bestNode;
 }
 
-QToolButton *ProjectTreeWidget::toggleSync()
-{
-    return m_toggleSync;
-}
-
 void ProjectTreeWidget::toggleAutoSynchronization()
 {
     setAutoSynchronization(!m_autoSync);
@@ -424,6 +411,20 @@ void ProjectTreeWidget::setAutoSynchronization(bool sync)
         syncFromDocumentManager();
 }
 
+void ProjectTreeWidget::expandNodeRecursively(const QModelIndex &index)
+{
+    const int rc = index.model()->rowCount(index);
+    for (int i = 0; i < rc; ++i)
+        expandNodeRecursively(index.model()->index(i, index.column(), index));
+    if (rc > 0)
+        m_view->expand(index);
+}
+
+void ProjectTreeWidget::expandCurrentNodeRecursively()
+{
+    expandNodeRecursively(m_view->currentIndex());
+}
+
 void ProjectTreeWidget::collapseAll()
 {
     m_view->collapseAll();
@@ -434,6 +435,28 @@ void ProjectTreeWidget::expandAll()
     m_view->expandAll();
 }
 
+QList<QToolButton *> ProjectTreeWidget::createToolButtons()
+{
+    auto filter = new QToolButton(this);
+    filter->setIcon(Icons::FILTER.icon());
+    filter->setToolTip(tr("Filter Tree"));
+    filter->setPopupMode(QToolButton::InstantPopup);
+    filter->setProperty("noArrow", true);
+
+    auto filterMenu = new QMenu(filter);
+    filterMenu->addAction(m_filterProjectsAction);
+    filterMenu->addAction(m_filterGeneratedFilesAction);
+    filterMenu->addAction(m_filterDisabledFilesAction);
+    filterMenu->addAction(m_trimEmptyDirectoriesAction);
+    filterMenu->addAction(m_hideSourceGroupsAction);
+    filter->setMenu(filterMenu);
+
+    auto toggleSync = new QToolButton;
+    toggleSync->setDefaultAction(m_toggleSync);
+
+    return {filter, toggleSync};
+}
+
 void ProjectTreeWidget::editCurrentItem()
 {
     m_delayedRename.clear();
@@ -450,8 +473,7 @@ void ProjectTreeWidget::editCurrentItem()
     if (!editor)
         return;
 
-    const QString text = editor->text();
-    const int dotIndex = text.lastIndexOf(QLatin1Char('.'));
+    const int dotIndex = Utils::FilePath::fromString(editor->text()).completeBaseName().length();
     if (dotIndex > 0)
         editor->setSelection(0, dotIndex);
 }
@@ -497,6 +519,7 @@ void ProjectTreeWidget::setCurrentItem(Node *node)
         }
     } else {
         m_view->clearSelection();
+        m_view->setCurrentIndex({});
     }
 }
 
@@ -540,7 +563,7 @@ void ProjectTreeWidget::openItem(const QModelIndex &mainIndex)
     Node *node = m_model->nodeForIndex(mainIndex);
     if (!node || !node->asFileNode())
         return;
-    IEditor *editor = EditorManager::openEditor(node->filePath().toString());
+    IEditor *editor = EditorManager::openEditor(node->filePath());
     if (editor && node->line() >= 0)
         editor->gotoLine(node->line());
 }
@@ -557,22 +580,44 @@ void ProjectTreeWidget::setGeneratedFilesFilter(bool filter)
     m_filterGeneratedFilesAction->setChecked(filter);
 }
 
+void ProjectTreeWidget::setDisabledFilesFilter(bool filter)
+{
+    m_model->setDisabledFilesFilterEnabled(filter);
+    m_filterDisabledFilesAction->setChecked(filter);
+}
+
 void ProjectTreeWidget::setTrimEmptyDirectories(bool filter)
 {
     m_model->setTrimEmptyDirectories(filter);
     m_trimEmptyDirectoriesAction->setChecked(filter);
 }
 
+void ProjectTreeWidget::setHideSourceGroups(bool filter)
+{
+    m_model->setHideSourceGroups(filter);
+    m_hideSourceGroupsAction->setChecked(filter);
+}
+
 bool ProjectTreeWidget::generatedFilesFilter()
 {
     return m_model->generatedFilesFilterEnabled();
 }
 
+bool ProjectTreeWidget::disabledFilesFilter()
+{
+    return m_model->disabledFilesFilterEnabled();
+}
+
 bool ProjectTreeWidget::trimEmptyDirectoriesFilter()
 {
     return m_model->trimEmptyDirectoriesEnabled();
 }
 
+bool ProjectTreeWidget::hideSourceGroups()
+{
+    return m_model->hideSourceGroups();
+}
+
 bool ProjectTreeWidget::projectFilter()
 {
     return m_model->projectFilterEnabled();
@@ -589,43 +634,61 @@ ProjectTreeWidgetFactory::ProjectTreeWidgetFactory()
 
 NavigationView ProjectTreeWidgetFactory::createWidget()
 {
-    NavigationView n;
     auto ptw = new ProjectTreeWidget;
-    n.widget = ptw;
-
-    auto filter = new QToolButton;
-    filter->setIcon(Icons::FILTER.icon());
-    filter->setToolTip(tr("Filter Tree"));
-    filter->setPopupMode(QToolButton::InstantPopup);
-    filter->setProperty("noArrow", true);
-    auto filterMenu = new QMenu(filter);
-    filterMenu->addAction(ptw->m_filterProjectsAction);
-    filterMenu->addAction(ptw->m_filterGeneratedFilesAction);
-    filterMenu->addAction(ptw->m_trimEmptyDirectoriesAction);
-    filter->setMenu(filterMenu);
-
-    n.dockToolBarWidgets << filter << ptw->toggleSync();
-    return n;
+    return {ptw, ptw->createToolButtons()};
 }
 
-void ProjectTreeWidgetFactory::saveSettings(QSettings *settings, int position, QWidget *widget)
+const bool kProjectFilterDefault = false;
+const bool kHideGeneratedFilesDefault = true;
+const bool kHideDisabledFilesDefault = false;
+const bool kTrimEmptyDirsDefault = true;
+const bool kHideSourceGroupsDefault = false;
+const bool kSyncDefault = true;
+const char kBaseKey[] = "ProjectTreeWidget.";
+const char kProjectFilterKey[] = ".ProjectFilter";
+const char kHideGeneratedFilesKey[] = ".GeneratedFilter";
+const char kHideDisabledFilesKey[] = ".DisabledFilesFilter";
+const char kTrimEmptyDirsKey[] = ".TrimEmptyDirsFilter";
+const char kSyncKey[] = ".SyncWithEditor";
+const char kHideSourceGroupsKey[] = ".HideSourceGroups";
+
+void ProjectTreeWidgetFactory::saveSettings(QtcSettings *settings, int position, QWidget *widget)
 {
     auto ptw = qobject_cast<ProjectTreeWidget *>(widget);
     Q_ASSERT(ptw);
-    const QString baseKey = QLatin1String("ProjectTreeWidget.") + QString::number(position);
-    settings->setValue(baseKey + QLatin1String(".ProjectFilter"), ptw->projectFilter());
-    settings->setValue(baseKey + QLatin1String(".GeneratedFilter"), ptw->generatedFilesFilter());
-    settings->setValue(baseKey + QLatin1String(".TrimEmptyDirsFilter"), ptw->trimEmptyDirectoriesFilter());
-    settings->setValue(baseKey + QLatin1String(".SyncWithEditor"), ptw->autoSynchronization());
+    const QString baseKey = kBaseKey + QString::number(position);
+    settings->setValueWithDefault(baseKey + kProjectFilterKey,
+                                  ptw->projectFilter(),
+                                  kProjectFilterDefault);
+    settings->setValueWithDefault(baseKey + kHideGeneratedFilesKey,
+                                  ptw->generatedFilesFilter(),
+                                  kHideGeneratedFilesDefault);
+    settings->setValueWithDefault(baseKey + kHideDisabledFilesKey,
+                                  ptw->disabledFilesFilter(),
+                                  kHideDisabledFilesDefault);
+    settings->setValueWithDefault(baseKey + kTrimEmptyDirsKey,
+                                  ptw->trimEmptyDirectoriesFilter(),
+                                  kTrimEmptyDirsDefault);
+    settings->setValueWithDefault(baseKey + kHideSourceGroupsKey,
+                                  ptw->hideSourceGroups(),
+                                  kHideSourceGroupsDefault);
+    settings->setValueWithDefault(baseKey + kSyncKey, ptw->autoSynchronization(), kSyncDefault);
 }
 
 void ProjectTreeWidgetFactory::restoreSettings(QSettings *settings, int position, QWidget *widget)
 {
     auto ptw = qobject_cast<ProjectTreeWidget *>(widget);
     Q_ASSERT(ptw);
-    const QString baseKey = QLatin1String("ProjectTreeWidget.") + QString::number(position);
-    ptw->setProjectFilter(settings->value(baseKey + QLatin1String(".ProjectFilter"), false).toBool());
-    ptw->setGeneratedFilesFilter(settings->value(baseKey + QLatin1String(".GeneratedFilter"), true).toBool());
-    ptw->setTrimEmptyDirectories(settings->value(baseKey + QLatin1String(".TrimEmptyDirsFilter"), true).toBool());
-    ptw->setAutoSynchronization(settings->value(baseKey +  QLatin1String(".SyncWithEditor"), true).toBool());
+    const QString baseKey = kBaseKey + QString::number(position);
+    ptw->setProjectFilter(
+        settings->value(baseKey + kProjectFilterKey, kProjectFilterDefault).toBool());
+    ptw->setGeneratedFilesFilter(
+        settings->value(baseKey + kHideGeneratedFilesKey, kHideGeneratedFilesDefault).toBool());
+    ptw->setDisabledFilesFilter(
+        settings->value(baseKey + kHideDisabledFilesKey, kHideDisabledFilesDefault).toBool());
+    ptw->setTrimEmptyDirectories(
+        settings->value(baseKey + kTrimEmptyDirsKey, kTrimEmptyDirsDefault).toBool());
+    ptw->setHideSourceGroups(
+        settings->value(baseKey + kHideSourceGroupsKey, kHideSourceGroupsDefault).toBool());
+    ptw->setAutoSynchronization(settings->value(baseKey + kSyncKey, kSyncDefault).toBool());
 }
