@@ -58,7 +58,7 @@ public:
     bool generatedFilesFilter();
     bool disabledFilesFilter();
     bool trimEmptyDirectoriesFilter();
-    QToolButton *toggleSync();
+    bool hideSourceGroups();
     Node *currentNode();
     void sync(ProjectExplorer::Node *node);
     void showMessage(ProjectExplorer::Node *node, const QString &message);
@@ -71,11 +71,14 @@ public:
     void collapseAll();
     void expandAll();
 
+    QList<QToolButton *> createToolButtons();
+
 private:
     void setProjectFilter(bool filter);
     void setGeneratedFilesFilter(bool filter);
     void setDisabledFilesFilter(bool filter);
     void setTrimEmptyDirectories(bool filter);
+    void setHideSourceGroups(bool filter);
 
     void handleCurrentItemChange(const QModelIndex &current);
     void showContextMenu(const QPoint &pos);
@@ -96,7 +99,8 @@ private:
     QAction *m_filterGeneratedFilesAction = nullptr;
     QAction *m_filterDisabledFilesAction = nullptr;
     QAction *m_trimEmptyDirectoriesAction = nullptr;
-    QToolButton *m_toggleSync = nullptr;
+    QAction *m_toggleSync = nullptr;
+    QAction *m_hideSourceGroupsAction = nullptr;
 
     QString m_modelId;
     bool m_autoSync = true;
@@ -114,7 +118,7 @@ public:
 
     Core::NavigationView createWidget() override;
     void restoreSettings(QSettings *settings, int position, QWidget *widget) override;
-    void saveSettings(QSettings *settings, int position, QWidget *widget) override;
+    void saveSettings(Utils::QtcSettings *settings, int position, QWidget *widget) override;
 };
 
 } // namespace Internal
