@@ -199,7 +199,7 @@ static QFont sizedFont(int size, const QWidget *widget, bool underline = false)
 static QPixmap pixmap(const QString &id, const Theme::Color &color)
 {
     const QString fileName = QString(":/welcome/images/%1.png").arg(id);
-    return Icon({{fileName, color}}, Icon::Tint).pixmap();
+    return Icon({{FilePath::fromString(fileName), color}}, Icon::Tint).pixmap();
 }
 
 class BaseDelegate : public QAbstractItemDelegate
@@ -320,14 +320,14 @@ public:
             QFontMetrics fm(option.widget->font());
             for (const QString &project : projects) {
                 // Project name.
-                QFileInfo fi(project);
-                QString completeBase = fi.completeBaseName();
+                FilePath projectPath = FilePath::fromString(project);
+                QString completeBase = projectPath.completeBaseName();
                 painter->setPen(textColor);
                 painter->drawText(x1, yy, completeBase);
                 yy += 18;
 
                 // Project path.
-                QString pathWithTilde = Utils::withTildeHomePath(QDir::toNativeSeparators(project));
+                QString pathWithTilde = Utils::withTildeHomePath(projectPath.toUserOutput());
                 painter->setPen(foregroundColor1);
                 painter->drawText(x1, yy, fm.elidedText(pathWithTilde, Qt::ElideMiddle, rc.width() - 40));
                 yy += 22;
@@ -454,7 +454,7 @@ public:
         painter->drawPixmap(x + 11, y + 6, projectIcon);
 
         QString projectName = idx.data(Qt::DisplayRole).toString();
-        QString projectPath = idx.data(ProjectModel::FilePathRole).toString();
+        FilePath projectPath = FilePath::fromVariant(idx.data(ProjectModel::FilePathRole));
 
         painter->setPen(themeColor(Theme::Welcome_ForegroundSecondaryColor));
         painter->setFont(sizedFont(10, option.widget));
@@ -468,7 +468,7 @@ public:
 
         painter->setPen(themeColor(Theme::Welcome_ForegroundPrimaryColor));
         painter->setFont(sizedFont(13, option.widget));
-        QString pathWithTilde = Utils::withTildeHomePath(QDir::toNativeSeparators(projectPath));
+        QString pathWithTilde = Utils::withTildeHomePath(projectPath.toUserOutput());
         painter->drawText(x + 36, secondBase, pathWithTilde);
     }
 
@@ -562,8 +562,7 @@ public:
 
         auto manageSessionsButton = new WelcomePageButton(this);
         manageSessionsButton->setText(ProjectWelcomePage::tr("Manage"));
-        const Icon icon({{":/utils/images/settings.png", Theme::Welcome_ForegroundSecondaryColor}}, Icon::Tint);
-        manageSessionsButton->setIcon(icon.pixmap());
+        manageSessionsButton->setIcon(pixmap("settings", Theme::Welcome_ForegroundSecondaryColor));
         manageSessionsButton->setOnClicked([] { ProjectExplorerPlugin::showSessionManager(); });
 
         auto newButton = new WelcomePageButton(this);
@@ -614,14 +613,12 @@ public:
 
         auto vbox1 = new QVBoxLayout;
         vbox1->setContentsMargins(0, 0, 0, 0);
-        vbox1->addStrut(200);
         vbox1->addItem(hbox11);
         vbox1->addSpacing(16);
         vbox1->addWidget(sessionsList);
 
         auto vbox2 = new QVBoxLayout;
         vbox2->setContentsMargins(0, 0, 0, 0);
-        vbox1->addStrut(200);
         vbox2->addItem(hbox21);
         vbox2->addSpacing(16);
         vbox2->addWidget(projectsList);
@@ -629,9 +626,10 @@ public:
         auto hbox = new QHBoxLayout(this);
         hbox->setContentsMargins(30, 27, 0, 27);
         hbox->addItem(vbox1);
+        hbox->setStretchFactor(vbox1, 1);
         hbox->addSpacing(16);
         hbox->addItem(vbox2);
-        hbox->setStretchFactor(vbox2, 2);
+        hbox->setStretchFactor(vbox2, 3);
     }
 
     SessionDelegate m_sessionDelegate;
