@@ -76,12 +76,13 @@ public:
     WorkingDirectoryAspect();
 
     void addToLayout(Utils::LayoutBuilder &builder) override;
-    void acquaintSiblings(const Utils::BaseAspects &) override;
+    void acquaintSiblings(const Utils::AspectContainer &) override;
 
-    Utils::FilePath workingDirectory(const Utils::MacroExpander *expander) const;
+    Utils::FilePath workingDirectory() const;
     Utils::FilePath defaultWorkingDirectory() const;
     Utils::FilePath unexpandedWorkingDirectory() const;
     void setDefaultWorkingDirectory(const Utils::FilePath &defaultWorkingDirectory);
+    void setMacroExpander(Utils::MacroExpander *macroExpander);
     Utils::PathChooser *pathChooser() const;
 
 private:
@@ -95,6 +96,7 @@ private:
     Utils::FilePath m_defaultWorkingDirectory;
     QPointer<Utils::PathChooser> m_chooser;
     QPointer<QToolButton> m_resetButton;
+    Utils::MacroExpander *m_macroExpander = nullptr;
 };
 
 class PROJECTEXPLORER_EXPORT ArgumentsAspect : public Utils::BaseAspect
@@ -147,6 +149,14 @@ public:
     UseDyldSuffixAspect();
 };
 
+class PROJECTEXPLORER_EXPORT RunAsRootAspect : public Utils::BoolAspect
+{
+    Q_OBJECT
+
+public:
+    RunAsRootAspect();
+};
+
 class PROJECTEXPLORER_EXPORT ExecutableAspect : public Utils::BaseAspect
 {
     Q_OBJECT
@@ -166,7 +176,7 @@ public:
     void setExecutablePathStyle(Utils::OsType osType);
     void setHistoryCompleter(const QString &historyCompleterKey);
     void setExpectedKind(const Utils::PathChooser::Kind expectedKind);
-    void setEnvironment(const Utils::Environment &env);
+    void setEnvironmentChange(const Utils::EnvironmentChange &change);
     void setDisplayStyle(Utils::StringAspect::DisplayStyle style);
 
 protected:
