@@ -29,7 +29,6 @@
 #include "sessiondialog.h"
 
 #include <coreplugin/actionmanager/actionmanager.h>
-#include <coreplugin/id.h>
 
 #include <utils/algorithm.h>
 #include <utils/fileutils.h>
@@ -39,6 +38,7 @@
 #include <QDir>
 
 using namespace Core;
+using namespace Utils;
 
 namespace ProjectExplorer {
 namespace Internal {
@@ -165,7 +165,7 @@ QVariant SessionModel::data(const QModelIndex &index, int role) const
 
 QHash<int, QByteArray> SessionModel::roleNames() const
 {
-    static QHash<int, QByteArray> extraRoles{
+    static const QHash<int, QByteArray> extraRoles{
         {Qt::DisplayRole, "sessionName"},
         {DefaultSessionRole, "defaultSession"},
         {ActiveSessionRole, "activeSession"},
@@ -173,7 +173,9 @@ QHash<int, QByteArray> SessionModel::roleNames() const
         {ProjectsPathRole, "projectsPath"},
         {ProjectsDisplayRole, "projectsName"}
     };
-    return QAbstractTableModel::roleNames().unite(extraRoles);
+    QHash<int, QByteArray> roles = QAbstractTableModel::roleNames();
+    Utils::addToHash(&roles, extraRoles);
+    return roles;
 }
 
 void SessionModel::sort(int column, Qt::SortOrder order)
@@ -181,15 +183,25 @@ void SessionModel::sort(int column, Qt::SortOrder order)
     beginResetModel();
     const auto cmp = [column, order](const QString &s1, const QString &s2) {
         bool isLess;
-        if (column == 0)
+        if (column == 0) {
+            if (s1 == s2)
+                return false;
             isLess = s1 < s2;
-        else
-            isLess = SessionManager::sessionDateTime(s1) < SessionManager::sessionDateTime(s2);
+        }
+        else {
+            const auto s1time = SessionManager::sessionDateTime(s1);
+            const auto s2time = SessionManager::sessionDateTime(s2);
+            if (s1time == s2time)
+                return false;
+            isLess = s1time < s2time;
+        }
         if (order == Qt::DescendingOrder)
             isLess = !isLess;
         return isLess;
     };
     Utils::sort(m_sortedSessions, cmp);
+    m_currentSortColumn = column;
+    m_currentSortOrder = order;
     endResetModel();
 }
 
@@ -234,6 +246,8 @@ void SessionModel::deleteSessions(const QStringList &sessions)
         return;
     beginResetModel();
     SessionManager::deleteSessions(sessions);
+    m_sortedSessions = SessionManager::sessions();
+    sort(m_currentSortColumn, m_currentSortOrder);
     endResetModel();
 }
 
@@ -265,6 +279,7 @@ void SessionModel::runSessionNameInputDialog(SessionNameInputDialog *sessionInpu
         createSession(newSession);
         m_sortedSessions = SessionManager::sessions();
         endResetModel();
+        sort(m_currentSortColumn, m_currentSortOrder);
 
         if (sessionInputDialog->isSwitchToRequested())
             switchToSession(newSession);
