@@ -50,6 +50,7 @@
 #include <coreplugin/coreconstants.h>
 
 #include <utils/algorithm.h>
+#include <utils/commandline.h>
 #include <utils/macroexpander.h>
 #include <utils/qtcassert.h>
 #include <utils/stringutils.h>
@@ -122,6 +123,8 @@ public:
     ProjectConfigurationModel m_buildConfigurationModel;
     ProjectConfigurationModel m_deployConfigurationModel;
     ProjectConfigurationModel m_runConfigurationModel;
+
+    bool m_shuttingDown = false;
 };
 
 
@@ -158,6 +161,11 @@ Target::Target(Project *project, Kit *k, _constructor_tag) :
 
     d->m_macroExpander.registerVariable("sourceDir", tr("Source directory"),
             [project] { return project->projectDirectory().toUserOutput(); });
+    d->m_macroExpander.registerVariable("BuildSystem:Name", tr("Build system"), [this] {
+        if (const BuildSystem * const bs = buildSystem())
+            return bs->name();
+        return QString();
+    });
 
     // TODO: Remove in ~4.16.
     d->m_macroExpander.registerVariable(Constants::VAR_CURRENTPROJECT_NAME,
@@ -174,14 +182,14 @@ Target::Target(Project *project, Kit *k, _constructor_tag) :
             if (RunConfiguration * const rc = activeRunConfiguration())
                 return rc->displayName();
             return QString();
-        });
+        }, false);
     d->m_macroExpander.registerFileVariables("CurrentRun:Executable",
         tr("The currently active run configuration's executable (if applicable)."),
-        [this]() -> QString {
+        [this]() -> FilePath {
             if (RunConfiguration * const rc = activeRunConfiguration())
-                return rc->commandLine().executable().toString();
-            return QString();
-        });
+                return rc->commandLine().executable();
+            return FilePath();
+        }, false);
     d->m_macroExpander.registerPrefix("CurrentRun:Env", tr("Variables in the current run environment."),
                              [this](const QString &var) {
         if (RunConfiguration * const rc = activeRunConfiguration()) {
@@ -189,16 +197,16 @@ Target::Target(Project *project, Kit *k, _constructor_tag) :
                 return envAspect->environment().expandedValueForKey(var);
         }
         return QString();
-    });
+    }, false);
     d->m_macroExpander.registerVariable("CurrentRun:WorkingDir",
                                tr("The currently active run configuration's working directory."),
                                [this] {
         if (RunConfiguration * const rc = activeRunConfiguration()) {
             if (const auto wdAspect = rc->aspect<WorkingDirectoryAspect>())
-                return wdAspect->workingDirectory(&d->m_macroExpander).toString();
+                return wdAspect->workingDirectory().toString();
         }
         return QString();
-    });
+    }, false);
 }
 
 Target::~Target()
@@ -232,6 +240,16 @@ bool Target::isActive() const
     return project()->activeTarget() == this;
 }
 
+void Target::markAsShuttingDown()
+{
+    d->m_shuttingDown = true;
+}
+
+bool Target::isShuttingDown() const
+{
+    return d->m_shuttingDown;
+}
+
 Project *Target::project() const
 {
     return static_cast<Project *>(parent());
@@ -511,6 +529,9 @@ RunConfiguration *Target::activeRunConfiguration() const
 
 void Target::setActiveRunConfiguration(RunConfiguration *rc)
 {
+    if (isShuttingDown())
+        return;
+
     if ((!rc && d->m_runConfigurations.isEmpty()) ||
         (rc && d->m_runConfigurations.contains(rc) &&
          rc != d->m_activeRunConfiguration)) {
@@ -668,7 +689,7 @@ void Target::updateDefaultRunConfigurations()
                 present = true;
             }
         }
-        if (!present && project()->knowsAllBuildExecutables())
+        if (!present)
             toRemove.append(rc);
     }
     configuredCount -= toRemove.count();
