@@ -171,7 +171,7 @@ bool operator<(const Task &a, const Task &b)
 }
 
 
-uint qHash(const Task &task)
+Utils::QHashValueType qHash(const Task &task)
 {
     return task.taskId;
 }
@@ -206,9 +206,12 @@ bool containsType(const Tasks &issues, Task::TaskType type)
 
 // CompilerTask
 
-CompileTask::CompileTask(TaskType type, const QString &desc, const FilePath &file, int line)
+CompileTask::CompileTask(TaskType type, const QString &desc,
+                         const FilePath &file, int line, int column_)
     : Task(type, desc, file, line, ProjectExplorer::Constants::TASK_CATEGORY_COMPILE)
-{}
+{
+    column = column_;
+}
 
 // BuildSystemTask
 
