@@ -35,11 +35,16 @@
 #include <utils/qtcassert.h>
 
 #include <QCoreApplication>
+#include <QDir>
+#include <QFileInfo>
 #include <QUuid>
 
+using namespace Utils;
+
 static const char ID_KEY[] = "ProjectExplorer.ToolChain.Id";
 static const char DISPLAY_NAME_KEY[] = "ProjectExplorer.ToolChain.DisplayName";
 static const char AUTODETECT_KEY[] = "ProjectExplorer.ToolChain.Autodetect";
+static const char DETECTION_SOURCE_KEY[] = "ProjectExplorer.ToolChain.DetectionSource";
 static const char LANGUAGE_KEY_V1[] = "ProjectExplorer.ToolChain.Language"; // For QtCreator <= 4.2
 static const char LANGUAGE_KEY_V2[] = "ProjectExplorer.ToolChain.LanguageV2"; // For QtCreator > 4.2
 
@@ -57,7 +62,7 @@ class ToolChainPrivate
 public:
     using Detection = ToolChain::Detection;
 
-    explicit ToolChainPrivate(Core::Id typeId) :
+    explicit ToolChainPrivate(Utils::Id typeId) :
         m_id(QUuid::createUuid().toByteArray()),
         m_typeId(typeId),
         m_predefinedMacrosCache(new ToolChain::MacrosCache::element_type()),
@@ -68,12 +73,17 @@ public:
     }
 
     QByteArray m_id;
-    QSet<Core::Id> m_supportedLanguages;
+    FilePath m_compilerCommand;
+    QString m_compilerCommandKey;
+    Abi m_targetAbi;
+    QString m_targetAbiKey;
+    QSet<Utils::Id> m_supportedLanguages;
     mutable QString m_displayName;
     QString m_typeDisplayName;
-    Core::Id m_typeId;
-    Core::Id m_language;
+    Utils::Id m_typeId;
+    Utils::Id m_language;
     Detection m_detection = ToolChain::UninitializedDetection;
+    QString m_detectionSource;
 
     ToolChain::MacrosCache m_predefinedMacrosCache;
     ToolChain::HeaderPathsCache m_headerPathsCache;
@@ -82,17 +92,17 @@ public:
 
 // Deprecated used from QtCreator <= 4.2
 
-Core::Id fromLanguageV1(int language)
+Utils::Id fromLanguageV1(int language)
 {
     switch (language)
     {
     case Deprecated::Toolchain::C :
-        return Core::Id(Constants::C_LANGUAGE_ID);
+        return Utils::Id(Constants::C_LANGUAGE_ID);
     case Deprecated::Toolchain::Cxx:
-        return Core::Id(Constants::CXX_LANGUAGE_ID);
+        return Utils::Id(Constants::CXX_LANGUAGE_ID);
     case Deprecated::Toolchain::None:
     default:
-        return Core::Id();
+        return Utils::Id();
     }
 }
 
@@ -123,12 +133,12 @@ QString languageId(Language l)
 
 // --------------------------------------------------------------------------
 
-ToolChain::ToolChain(Core::Id typeId) :
+ToolChain::ToolChain(Utils::Id typeId) :
     d(std::make_unique<Internal::ToolChainPrivate>(typeId))
 {
 }
 
-void ToolChain::setLanguage(Core::Id language)
+void ToolChain::setLanguage(Utils::Id language)
 {
     QTC_ASSERT(!d->m_language.isValid() || isAutoDetected(), return);
     QTC_ASSERT(language.isValid(), return);
@@ -155,11 +165,21 @@ void ToolChain::setDisplayName(const QString &name)
     toolChainUpdated();
 }
 
+bool ToolChain::isAutoDetected() const
+{
+    return detection() == AutoDetection || detection() == AutoDetectionFromSdk;
+}
+
 ToolChain::Detection ToolChain::detection() const
 {
     return d->m_detection;
 }
 
+QString ToolChain::detectionSource() const
+{
+    return d->m_detectionSource;
+}
+
 QByteArray ToolChain::id() const
 {
     return d->m_id;
@@ -170,7 +190,7 @@ QStringList ToolChain::suggestedMkspecList() const
     return {};
 }
 
-Core::Id ToolChain::typeId() const
+Utils::Id ToolChain::typeId() const
 {
     return d->m_typeId;
 }
@@ -180,7 +200,21 @@ Abis ToolChain::supportedAbis() const
     return {targetAbi()};
 }
 
-Core::Id ToolChain::language() const
+bool ToolChain::isValid() const
+{
+    if (compilerCommand().isEmpty())
+        return false;
+    return compilerCommand().isExecutableFile();
+}
+
+QStringList ToolChain::includedFiles(const QStringList &flags, const QString &directory) const
+{
+    Q_UNUSED(flags)
+    Q_UNUSED(directory)
+    return {};
+}
+
+Utils::Id ToolChain::language() const
 {
     return d->m_language;
 }
@@ -198,7 +232,7 @@ bool ToolChain::operator == (const ToolChain &tc) const
 
 ToolChain *ToolChain::clone() const
 {
-    for (ToolChainFactory *f : Internal::g_toolChainFactories) {
+    for (ToolChainFactory *f : qAsConst(Internal::g_toolChainFactories)) {
         if (f->supportedToolChainType() == d->m_typeId) {
             ToolChain *tc = f->create();
             QTC_ASSERT(tc, return nullptr);
@@ -225,6 +259,7 @@ QVariantMap ToolChain::toMap() const
     result.insert(QLatin1String(ID_KEY), idToSave);
     result.insert(QLatin1String(DISPLAY_NAME_KEY), displayName());
     result.insert(QLatin1String(AUTODETECT_KEY), isAutoDetected());
+    result.insert(QLatin1String(DETECTION_SOURCE_KEY), d->m_detectionSource);
     // <Compatibility with QtC 4.2>
     int oldLanguageId = -1;
     if (language() == ProjectExplorer::Constants::C_LANGUAGE_ID)
@@ -235,6 +270,10 @@ QVariantMap ToolChain::toMap() const
         result.insert(LANGUAGE_KEY_V1, oldLanguageId);
     // </Compatibility>
     result.insert(QLatin1String(LANGUAGE_KEY_V2), language().toSetting());
+    if (!d->m_targetAbiKey.isEmpty())
+        result.insert(d->m_targetAbiKey, d->m_targetAbi.toString());
+    if (!d->m_compilerCommandKey.isEmpty())
+        result.insert(d->m_compilerCommandKey, d->m_compilerCommand.toVariant());
     return result;
 }
 
@@ -248,14 +287,12 @@ void ToolChain::toolChainUpdated()
 
 void ToolChain::setDetection(ToolChain::Detection de)
 {
-    if (d->m_detection == de)
-        return;
-    if (d->m_detection == ToolChain::UninitializedDetection) {
-        d->m_detection = de;
-    } else {
-        d->m_detection = de;
-        toolChainUpdated();
-    }
+    d->m_detection = de;
+}
+
+void ToolChain::setDetectionSource(const QString &source)
+{
+    d->m_detectionSource = source;
 }
 
 QString ToolChain::typeDisplayName() const
@@ -263,6 +300,48 @@ QString ToolChain::typeDisplayName() const
     return d->m_typeDisplayName;
 }
 
+Abi ToolChain::targetAbi() const
+{
+    return d->m_targetAbi;
+}
+
+void ToolChain::setTargetAbi(const Abi &abi)
+{
+    if (abi == d->m_targetAbi)
+        return;
+
+    d->m_targetAbi = abi;
+    toolChainUpdated();
+}
+
+void ToolChain::setTargetAbiNoSignal(const Abi &abi)
+{
+    d->m_targetAbi = abi;
+}
+
+void ToolChain::setTargetAbiKey(const QString &abiKey)
+{
+    d->m_targetAbiKey = abiKey;
+}
+
+FilePath ToolChain::compilerCommand() const
+{
+    return d->m_compilerCommand;
+}
+
+void ToolChain::setCompilerCommand(const FilePath &command)
+{
+    if (command == d->m_compilerCommand)
+        return;
+    d->m_compilerCommand = command;
+    toolChainUpdated();
+}
+
+void ToolChain::setCompilerCommandKey(const QString &commandKey)
+{
+    d->m_compilerCommandKey = commandKey;
+}
+
 void ToolChain::setTypeDisplayName(const QString &typeName)
 {
     d->m_typeDisplayName = typeName;
@@ -282,11 +361,12 @@ bool ToolChain::fromMap(const QVariantMap &data)
     const QString id = data.value(QLatin1String(ID_KEY)).toString();
     int pos = id.indexOf(QLatin1Char(':'));
     QTC_ASSERT(pos > 0, return false);
-    d->m_typeId = Core::Id::fromString(id.left(pos));
+    d->m_typeId = Utils::Id::fromString(id.left(pos));
     d->m_id = id.mid(pos + 1).toUtf8();
 
     const bool autoDetect = data.value(QLatin1String(AUTODETECT_KEY), false).toBool();
     d->m_detection = autoDetect ? AutoDetection : ManualDetection;
+    d->m_detectionSource = data.value(DETECTION_SOURCE_KEY).toString();
 
     if (data.contains(LANGUAGE_KEY_V2)) {
         // remove hack to trim language id in 4.4: This is to fix up broken language
@@ -294,15 +374,20 @@ bool ToolChain::fromMap(const QVariantMap &data)
         const QString langId = data.value(QLatin1String(LANGUAGE_KEY_V2)).toString();
         const int pos = langId.lastIndexOf('.');
         if (pos >= 0)
-            d->m_language = Core::Id::fromString(langId.mid(pos + 1));
+            d->m_language = Utils::Id::fromString(langId.mid(pos + 1));
         else
-            d->m_language = Core::Id::fromString(langId);
+            d->m_language = Utils::Id::fromString(langId);
     } else if (data.contains(LANGUAGE_KEY_V1)) { // Import from old settings
         d->m_language = Internal::fromLanguageV1(data.value(QLatin1String(LANGUAGE_KEY_V1)).toInt());
     }
 
     if (!d->m_language.isValid())
-        d->m_language = Core::Id(Constants::CXX_LANGUAGE_ID);
+        d->m_language = Utils::Id(Constants::CXX_LANGUAGE_ID);
+
+    if (!d->m_targetAbiKey.isEmpty())
+        d->m_targetAbi = Abi::fromString(data.value(d->m_targetAbiKey).toString());
+
+    d->m_compilerCommand = FilePath::fromVariant(data.value(d->m_compilerCommandKey));
 
     return true;
 }
@@ -345,7 +430,7 @@ Utils::LanguageVersion ToolChain::cxxLanguageVersion(const QByteArray &cplusplus
     return LanguageVersion::CXX03;
 }
 
-Utils::LanguageVersion ToolChain::languageVersion(const Core::Id &language, const Macros &macros)
+Utils::LanguageVersion ToolChain::languageVersion(const Utils::Id &language, const Macros &macros)
 {
     using Utils::LanguageVersion;
 
@@ -384,6 +469,24 @@ Utils::LanguageVersion ToolChain::languageVersion(const Core::Id &language, cons
     }
 }
 
+QStringList ToolChain::includedFiles(const QString &option,
+                                     const QStringList &flags,
+                                     const QString &directoryPath)
+{
+    QStringList result;
+
+    for (int i = 0; i < flags.size(); ++i) {
+        if (flags[i] == option && i + 1 < flags.size()) {
+            QString includeFile = flags[++i];
+            if (!QFileInfo(includeFile).isAbsolute())
+                includeFile = directoryPath + "/" + includeFile;
+            result.append(QDir::cleanPath(includeFile));
+        }
+    }
+
+    return result;
+}
+
 /*!
     Used by the tool chain kit information to validate the kit.
 */
@@ -436,9 +539,11 @@ const QList<ToolChainFactory *> ToolChainFactory::allToolChainFactories()
     return Internal::g_toolChainFactories;
 }
 
-QList<ToolChain *> ToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown)
+QList<ToolChain *> ToolChainFactory::autoDetect(const QList<ToolChain *> &alreadyKnown,
+                                                const IDevice::Ptr &device)
 {
     Q_UNUSED(alreadyKnown)
+    Q_UNUSED(device)
     return {};
 }
 
@@ -486,9 +591,9 @@ QByteArray ToolChainFactory::idFromMap(const QVariantMap &data)
     return rawIdData(data).second.toUtf8();
 }
 
-Core::Id ToolChainFactory::typeIdFromMap(const QVariantMap &data)
+Utils::Id ToolChainFactory::typeIdFromMap(const QVariantMap &data)
 {
-    return Core::Id::fromString(rawIdData(data).first);
+    return Utils::Id::fromString(rawIdData(data).first);
 }
 
 void ToolChainFactory::autoDetectionToMap(QVariantMap &data, bool detected)
@@ -496,7 +601,7 @@ void ToolChainFactory::autoDetectionToMap(QVariantMap &data, bool detected)
     data.insert(QLatin1String(AUTODETECT_KEY), detected);
 }
 
-ToolChain *ToolChainFactory::createToolChain(Core::Id toolChainType)
+ToolChain *ToolChainFactory::createToolChain(Utils::Id toolChainType)
 {
     for (ToolChainFactory *factory : qAsConst(Internal::g_toolChainFactories)) {
         if (factory->m_supportedToolChainType == toolChainType) {
@@ -509,22 +614,22 @@ ToolChain *ToolChainFactory::createToolChain(Core::Id toolChainType)
     return nullptr;
 }
 
-QSet<Core::Id> ToolChainFactory::supportedLanguages() const
+QList<Utils::Id> ToolChainFactory::supportedLanguages() const
 {
     return m_supportsAllLanguages ? ToolChainManager::allLanguages() : m_supportedLanguages;
 }
 
-Core::Id ToolChainFactory::supportedToolChainType() const
+Utils::Id ToolChainFactory::supportedToolChainType() const
 {
     return m_supportedToolChainType;
 }
 
-void ToolChainFactory::setSupportedToolChainType(const Core::Id &supportedToolChain)
+void ToolChainFactory::setSupportedToolChainType(const Utils::Id &supportedToolChain)
 {
     m_supportedToolChainType = supportedToolChain;
 }
 
-void ToolChainFactory::setSupportedLanguages(const QSet<Core::Id> &supportedLanguages)
+void ToolChainFactory::setSupportedLanguages(const QList<Utils::Id> &supportedLanguages)
 {
     m_supportedLanguages = supportedLanguages;
 }
