@@ -42,29 +42,32 @@ static const char buildRe[] = "=== BUILD (AGGREGATE )?TARGET (.*) OF PROJECT (.*
 static const char signatureChangeEndsWithPattern[] = ": replacing existing signature";
 
 XcodebuildParser::XcodebuildParser()
+    : m_failureRe(QLatin1String(failureRe))
+    , m_successRe(QLatin1String(successRe))
+    , m_buildRe(QLatin1String(buildRe))
 {
     setObjectName(QLatin1String("XcodeParser"));
-    m_failureRe.setPattern(QLatin1String(failureRe));
     QTC_CHECK(m_failureRe.isValid());
-    m_successRe.setPattern(QLatin1String(successRe));
     QTC_CHECK(m_successRe.isValid());
-    m_buildRe.setPattern(QLatin1String(buildRe));
     QTC_CHECK(m_buildRe.isValid());
 }
 
 OutputLineParser::Result XcodebuildParser::handleLine(const QString &line, OutputFormat type)
 {
+    static const QStringList notesPatterns({"note: Build preparation complete",
+                                            "note: Building targets in parallel",
+                                            "note: Planning build"});
     const QString lne = rightTrimmed(line);
     if (type == StdOutFormat) {
-        if (m_buildRe.indexIn(lne) > -1) {
+        QRegularExpressionMatch match = m_buildRe.match(line);
+        if (match.hasMatch() || notesPatterns.contains(lne)) {
             m_xcodeBuildParserState = InXcodebuild;
-            m_lastTarget = m_buildRe.cap(2);
-            m_lastProject = m_buildRe.cap(3);
             return Status::Done;
         }
         if (m_xcodeBuildParserState == InXcodebuild
                 || m_xcodeBuildParserState == UnknownXcodebuildState) {
-            if (m_successRe.indexIn(lne) > -1) {
+            match = m_successRe.match(lne);
+            if (match.hasMatch()) {
                 m_xcodeBuildParserState = OutsideXcodebuild;
                 return Status::Done;
             }
@@ -83,10 +86,10 @@ OutputLineParser::Result XcodebuildParser::handleLine(const QString &line, Outpu
         }
         return Status::NotHandled;
     }
-    if (m_failureRe.indexIn(lne) > -1) {
+    const QRegularExpressionMatch match = m_failureRe.match(lne);
+    if (match.hasMatch()) {
         ++m_fatalErrorCount;
         m_xcodeBuildParserState = UnknownXcodebuildState;
-        // unfortunately the m_lastTarget, m_lastProject might not be in sync
         scheduleTask(CompileTask(Task::Error, tr("Xcodebuild failed.")), 1);
     }
     if (m_xcodeBuildParserState == OutsideXcodebuild)
@@ -190,6 +193,19 @@ void ProjectExplorerPlugin::testXcodebuildParserParsing_data()
             << Tasks()
             << QString()
             << XcodebuildParser::OutsideXcodebuild;
+    QTest::newRow("switch outside->in->outside (new)")
+            << XcodebuildParser::OutsideXcodebuild
+            << QString::fromLatin1("outside\n"
+                                   "note: Build preparation complete\n"
+                                   "in xcodebuild\n"
+                                   "in xcodebuild2\n"
+                                   "** BUILD SUCCEEDED **\n"
+                                   "outside2")
+            << OutputParserTester::STDOUT
+            << QString::fromLatin1("outside\noutside2\n") << QString::fromLatin1("in xcodebuild\nin xcodebuild2\n")
+            << Tasks()
+            << QString()
+            << XcodebuildParser::OutsideXcodebuild;
     QTest::newRow("switch Unknown->in->outside")
             << XcodebuildParser::UnknownXcodebuildState
             << QString::fromLatin1("unknown\n"
@@ -236,7 +252,7 @@ void ProjectExplorerPlugin::testXcodebuildParserParsing_data()
             << (Tasks()
                 << CompileTask(Task::Warning,
                                XcodebuildParser::tr("Replacing signature"),
-                               FilePath::fromString("/somepath/somefile.app")))
+                               "/somepath/somefile.app"))
             << QString()
             << XcodebuildParser::InXcodebuild;
 
