@@ -63,10 +63,13 @@ public:
     explicit PythonBuildSystem(Target *target);
 
     bool supportsAction(Node *context, ProjectAction action, const Node *node) const override;
-    bool addFiles(Node *, const QStringList &filePaths, QStringList *) override;
-    RemovedFilesFromProject removeFiles(Node *, const QStringList &filePaths, QStringList *) override;
-    bool deleteFiles(Node *, const QStringList &) override;
-    bool renameFile(Node *, const QString &filePath, const QString &newFilePath) override;
+    bool addFiles(Node *, const Utils::FilePaths &filePaths, Utils::FilePaths *) override;
+    RemovedFilesFromProject removeFiles(Node *, const Utils::FilePaths &filePaths, Utils::FilePaths *) override;
+    bool deleteFiles(Node *, const Utils::FilePaths &) override;
+    bool renameFile(Node *,
+                    const Utils::FilePath &oldFilePath,
+                    const Utils::FilePath &newFilePath) override;
+    QString name() const override { return QLatin1String("python"); }
 
     bool saveRawFileList(const QStringList &rawFileList);
     bool saveRawList(const QStringList &rawList, const QString &fileName);
@@ -205,7 +208,7 @@ public:
     PythonProjectNode(const Utils::FilePath &path)
         : ProjectNode(path)
     {
-        setDisplayName(path.toFileInfo().completeBaseName());
+        setDisplayName(path.completeBaseName());
         setAddFileFilter("*.py");
     }
 };
@@ -215,7 +218,7 @@ PythonProject::PythonProject(const FilePath &fileName)
 {
     setId(PythonProjectId);
     setProjectLanguages(Context(ProjectExplorer::Constants::PYTHON_LANGUAGE_ID));
-    setDisplayName(fileName.toFileInfo().completeBaseName());
+    setDisplayName(fileName.completeBaseName());
 
     setNeedsBuildConfigurations(false);
     setBuildSystemCreator([](Target *t) { return new PythonBuildSystem(t); });
@@ -268,7 +271,7 @@ void PythonBuildSystem::triggerParsing()
     if (modelManager) {
         auto projectInfo = modelManager->defaultProjectInfoForProject(project());
 
-        for (const QString &importPath : m_qmlImportPaths) {
+        for (const QString &importPath : qAsConst(m_qmlImportPaths)) {
             const Utils::FilePath filePath = Utils::FilePath::fromString(importPath);
             projectInfo.importPaths.maybeInsert(filePath, QmlJS::Dialect::Qml);
         }
@@ -290,23 +293,24 @@ bool PythonBuildSystem::saveRawFileList(const QStringList &rawFileList)
 
 bool PythonBuildSystem::saveRawList(const QStringList &rawList, const QString &fileName)
 {
-    FileChangeBlocker changeGuarg(fileName);
+    const FilePath filePath = FilePath::fromString(fileName);
+    FileChangeBlocker changeGuarg(filePath);
     bool result = false;
 
     // New project file
     if (fileName.endsWith(".pyproject")) {
-        FileSaver saver(fileName, QIODevice::ReadOnly | QIODevice::Text);
+        FileSaver saver(filePath, QIODevice::ReadOnly | QIODevice::Text);
         if (!saver.hasError()) {
             QString content = QTextStream(saver.file()).readAll();
             if (saver.finalize(ICore::dialogParent())) {
                 QString errorMessage;
                 result = writePyProjectFile(fileName, content, rawList, &errorMessage);
                 if (!errorMessage.isEmpty())
-                    MessageManager::write(errorMessage);
+                    MessageManager::writeDisrupting(errorMessage);
             }
         }
     } else { // Old project file
-        FileSaver saver(fileName, QIODevice::WriteOnly | QIODevice::Text);
+        FileSaver saver(filePath, QIODevice::WriteOnly | QIODevice::Text);
         if (!saver.hasError()) {
             QTextStream stream(saver.file());
             for (const QString &filePath : rawList)
@@ -346,23 +350,23 @@ bool PythonBuildSystem::writePyProjectFile(const QString &fileName, QString &con
     return true;
 }
 
-bool PythonBuildSystem::addFiles(Node *, const QStringList &filePaths, QStringList *)
+bool PythonBuildSystem::addFiles(Node *, const FilePaths &filePaths, FilePaths *)
 {
     QStringList newList = m_rawFileList;
 
     const QDir baseDir(projectDirectory().toString());
-    for (const QString &filePath : filePaths)
-        newList.append(baseDir.relativeFilePath(filePath));
+    for (const FilePath &filePath : filePaths)
+        newList.append(baseDir.relativeFilePath(filePath.toString()));
 
     return saveRawFileList(newList);
 }
 
-RemovedFilesFromProject PythonBuildSystem::removeFiles(Node *, const QStringList &filePaths, QStringList *)
+RemovedFilesFromProject PythonBuildSystem::removeFiles(Node *, const FilePaths &filePaths, FilePaths *)
 {
     QStringList newList = m_rawFileList;
 
-    for (const QString &filePath : filePaths) {
-        const QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath);
+    for (const FilePath &filePath : filePaths) {
+        const QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath.toString());
         if (i != m_rawListEntries.end())
             newList.removeOne(i.value());
     }
@@ -372,21 +376,21 @@ RemovedFilesFromProject PythonBuildSystem::removeFiles(Node *, const QStringList
     return res ? RemovedFilesFromProject::Ok : RemovedFilesFromProject::Error;
 }
 
-bool PythonBuildSystem::deleteFiles(Node *, const QStringList &)
+bool PythonBuildSystem::deleteFiles(Node *, const FilePaths &)
 {
     return true;
 }
 
-bool PythonBuildSystem::renameFile(Node *, const QString &filePath, const QString &newFilePath)
+bool PythonBuildSystem::renameFile(Node *, const FilePath &oldFilePath, const FilePath &newFilePath)
 {
     QStringList newList = m_rawFileList;
 
-    const QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath);
+    const QHash<QString, QString>::iterator i = m_rawListEntries.find(oldFilePath.toString());
     if (i != m_rawListEntries.end()) {
         const int index = newList.indexOf(i.value());
         if (index != -1) {
             const QDir baseDir(projectDirectory().toString());
-            newList.replace(index, baseDir.relativeFilePath(newFilePath));
+            newList.replace(index, baseDir.relativeFilePath(newFilePath.toString()));
         }
     }
 
@@ -404,12 +408,12 @@ void PythonBuildSystem::parse()
         QString errorMessage;
         m_rawFileList = readLinesJson(filePath, &errorMessage);
         if (!errorMessage.isEmpty())
-            MessageManager::write(errorMessage);
+            MessageManager::writeFlashing(errorMessage);
 
         errorMessage.clear();
         m_rawQmlImportPathList = readImportPathsJson(filePath, &errorMessage);
         if (!errorMessage.isEmpty())
-            MessageManager::write(errorMessage);
+            MessageManager::writeFlashing(errorMessage);
     } else if (filePath.endsWith(".pyqtc")) {
         // To keep compatibility with PyQt we keep the compatibility with plain
         // text files as project files.
