@@ -52,6 +52,7 @@
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectexplorericons.h>
 #include <projectexplorer/projectmanager.h>
 #include <projectexplorer/projecttree.h>
 #include <projectexplorer/runcontrol.h>
@@ -63,6 +64,7 @@
 
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
+#include <utils/utilsicons.h>
 
 #include <QAction>
 
@@ -159,7 +161,7 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     command->setAttribute(Core::Command::CA_UpdateText);
     command->setDescription(m_buildFile->text());
     command->setDefaultKeySequence(QKeySequence(tr("Ctrl+Alt+B")));
-    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_BUILD);
+    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_FILE);
     connect(m_buildFile, &QAction::triggered, this, &QbsProjectManagerPlugin::buildFile);
 
     m_buildProductCtx = new QAction(tr("Build"), this);
@@ -176,7 +178,7 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     command->setAttribute(Core::Command::CA_UpdateText);
     command->setDescription(m_buildFile->text());
     command->setDefaultKeySequence(QKeySequence(tr("Ctrl+Alt+Shift+B")));
-    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_BUILD);
+    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_PRODUCT);
     connect(m_buildProduct, &QAction::triggered, this, &QbsProjectManagerPlugin::buildProduct);
 
     m_cleanProductCtx = new QAction(tr("Clean"), this);
@@ -187,12 +189,13 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     connect(m_cleanProductCtx, &QAction::triggered,
             this, &QbsProjectManagerPlugin::cleanProductContextMenu);
 
-    m_cleanProduct = new Utils::ParameterAction(tr("Clean Product"), tr("Clean Product \"%1\""),
-                                                Utils::ParameterAction::AlwaysEnabled, this);
+    m_cleanProduct = new QAction(Utils::Icons::CLEAN.icon(), tr("Clean"), this);
+    m_cleanProduct->setWhatsThis(tr("Clean Product"));
     command = Core::ActionManager::registerAction(m_cleanProduct, Constants::ACTION_CLEAN_PRODUCT);
     command->setAttribute(Core::Command::CA_Hide);
     command->setAttribute(Core::Command::CA_UpdateText);
-    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_CLEAN);
+    command->setDescription(m_cleanProduct->whatsThis());
+    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_PRODUCT);
     connect(m_cleanProduct, &QAction::triggered, this, &QbsProjectManagerPlugin::cleanProduct);
 
     m_rebuildProductCtx = new QAction(tr("Rebuild"), this);
@@ -203,14 +206,14 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     connect(m_rebuildProductCtx, &QAction::triggered,
             this, &QbsProjectManagerPlugin::rebuildProductContextMenu);
 
-    m_rebuildProduct = new Utils::ParameterAction(
-                tr("Rebuild Product"), tr("Rebuild Product \"%1\""),
-                Utils::ParameterAction::AlwaysEnabled, this);
+    m_rebuildProduct = new QAction(ProjectExplorer::Icons::REBUILD.icon(), tr("Rebuild"), this);
+    m_rebuildProduct->setWhatsThis(tr("Rebuild Product"));
     command = Core::ActionManager::registerAction(m_rebuildProduct,
                                                   Constants::ACTION_REBUILD_PRODUCT);
     command->setAttribute(Core::Command::CA_Hide);
     command->setAttribute(Core::Command::CA_UpdateText);
-    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_REBUILD);
+    command->setDescription(m_rebuildProduct->whatsThis());
+    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_PRODUCT);
     connect(m_rebuildProduct, &QAction::triggered, this, &QbsProjectManagerPlugin::rebuildProduct);
 
     m_buildSubprojectCtx = new QAction(tr("Build"), this);
@@ -263,7 +266,7 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     });
 
     // Run initial setup routines
-    updateContextActions();
+    updateContextActions(ProjectTree::currentNode());
     updateReparseQbsAction();
     updateBuildActions();
 
@@ -281,10 +284,9 @@ void QbsProjectManagerPlugin::targetWasAdded(Target *target)
             this, &QbsProjectManagerPlugin::projectChanged);
 }
 
-void QbsProjectManagerPlugin::updateContextActions()
+void QbsProjectManagerPlugin::updateContextActions(Node *node)
 {
     auto project = qobject_cast<Internal::QbsProject *>(ProjectTree::currentProject());
-    const Node *node = ProjectTree::currentNode();
     bool isEnabled = !BuildManager::isBuilding(project)
             && project && project->activeTarget()
             && !project->activeTarget()->buildSystem()->isParsing()
@@ -358,10 +360,8 @@ void QbsProjectManagerPlugin::updateBuildActions()
     m_buildProduct->setParameter(productName);
     m_cleanProduct->setEnabled(enabled);
     m_cleanProduct->setVisible(productVisible);
-    m_cleanProduct->setParameter(productName);
     m_rebuildProduct->setEnabled(enabled);
     m_rebuildProduct->setVisible(productVisible);
-    m_rebuildProduct->setParameter(productName);
 }
 
 void QbsProjectManagerPlugin::projectChanged()
@@ -372,7 +372,7 @@ void QbsProjectManagerPlugin::projectChanged()
         updateReparseQbsAction();
 
     if (!project || project == ProjectTree::currentProject())
-        updateContextActions();
+        updateContextActions(ProjectTree::currentNode());
 
     if (!project || project == currentEditorProject())
         updateBuildActions();
