@@ -40,34 +40,39 @@
 #include <QTextStream>
 
 using namespace ProjectExplorer;
-using namespace QmakeProjectManager;
-using namespace QmakeProjectManager::Internal;
+using namespace Utils;
+
+namespace QmakeProjectManager {
+namespace Internal {
 
 static void fillLibraryPlatformTypes(QComboBox *comboBox)
 {
     comboBox->clear();
-    comboBox->addItem("Windows (*.lib lib*.a)", int(Utils::OsTypeWindows));
-    comboBox->addItem("Linux (lib*.so lib*.a)", int(Utils::OsTypeLinux));
-    comboBox->addItem("macOS (*.dylib *.a *.framework)", int(Utils::OsTypeMac));
-    const int currentIndex = comboBox->findData(int(Utils::HostOsInfo::hostOs()));
+    comboBox->addItem("Windows (*.lib lib*.a)", int(OsTypeWindows));
+    comboBox->addItem("Linux (lib*.so lib*.a)", int(OsTypeLinux));
+    comboBox->addItem("macOS (*.dylib *.a *.framework)", int(OsTypeMac));
+    const int currentIndex = comboBox->findData(int(HostOsInfo::hostOs()));
     comboBox->setCurrentIndex(std::max(0, currentIndex));
 }
 
 LibraryDetailsController::LibraryDetailsController(
         Ui::LibraryDetailsWidget *libraryDetails,
-        const QString &proFile, QObject *parent) :
+        const FilePath &proFile, QObject *parent) :
     QObject(parent),
     m_proFile(proFile),
     m_libraryDetailsWidget(libraryDetails)
 {
+    fillLibraryPlatformTypes(m_libraryDetailsWidget->libraryTypeComboBox);
     setPlatformsVisible(true);
     setLinkageGroupVisible(true);
     setMacLibraryGroupVisible(true);
     setPackageLineEditVisible(false);
-    setMacLibraryRadiosVisible(!Utils::HostOsInfo::isMacHost());
-    setLinkageRadiosVisible(Utils::HostOsInfo::isWindowsHost());
+    const bool isMacOs = libraryPlatformType() == OsTypeMac;
+    const bool isWindows = libraryPlatformType() == OsTypeWindows;
+    setMacLibraryRadiosVisible(!isMacOs);
+    setLinkageRadiosVisible(isWindows);
 
-    connect(m_libraryDetailsWidget->includePathChooser, &Utils::PathChooser::rawPathChanged,
+    connect(m_libraryDetailsWidget->includePathChooser, &PathChooser::rawPathChanged,
             this, &LibraryDetailsController::slotIncludePathChanged);
     connect(m_libraryDetailsWidget->frameworkRadio, &QAbstractButton::clicked,
             this, &LibraryDetailsController::slotMacLibraryTypeChanged);
@@ -83,8 +88,6 @@ LibraryDetailsController::LibraryDetailsController(
             this, &LibraryDetailsController::slotPlatformChanged);
     connect(m_libraryDetailsWidget->winCheckBox, &QAbstractButton::clicked,
             this, &LibraryDetailsController::slotPlatformChanged);
-
-    fillLibraryPlatformTypes(m_libraryDetailsWidget->libraryTypeComboBox);
 }
 
 Ui::LibraryDetailsWidget *LibraryDetailsController::libraryDetailsWidget() const
@@ -107,9 +110,9 @@ AddLibraryWizard::MacLibraryType LibraryDetailsController::macLibraryType() cons
     return m_macLibraryType;
 }
 
-Utils::OsType LibraryDetailsController::libraryPlatformType() const
+OsType LibraryDetailsController::libraryPlatformType() const
 {
-    return Utils::OsType(m_libraryDetailsWidget->libraryTypeComboBox->currentData().value<int>());
+    return OsType(m_libraryDetailsWidget->libraryTypeComboBox->currentData().value<int>());
 }
 
 QString LibraryDetailsController::libraryPlatformFilter() const
@@ -120,7 +123,7 @@ QString LibraryDetailsController::libraryPlatformFilter() const
 void LibraryDetailsController::updateGui()
 {
     // read values from gui
-    m_platforms = nullptr;
+    m_platforms = {};
     if (libraryDetailsWidget()->linCheckBox->isChecked())
         m_platforms |= AddLibraryWizard::LinuxPlatform;
     if (libraryDetailsWidget()->macCheckBox->isChecked())
@@ -170,9 +173,34 @@ void LibraryDetailsController::updateGui()
         libraryDetailsWidget()->includePathChooser->setPath(suggestedIncludePath());
 
     setIgnoreGuiSignals(false);
+
+    // UGLY HACK BEGIN
+    //
+    // We need to invoke QWizardPrivate::updateLayout() method to properly
+    // recalculate the new minimum size for the whole wizard.
+    // This is done internally by QWizard e.g. when a new wizard page is being shown.
+    // Unfortunately, QWizard doesn't expose this method currently.
+    // Since the current implementation of QWizard::setTitleFormat() sets the
+    // format and calls QWizardPrivate::updateLayout() unconditionally
+    // we use it as a hacky solution to the above issue.
+    // For reference please see: QTBUG-88666
+    if (!m_wizard) {
+        QWidget *widget = libraryDetailsWidget()->detailsLayout->parentWidget();
+        while (widget) {
+            QWizard *wizard = qobject_cast<QWizard *>(widget);
+            if (wizard) {
+                m_wizard = wizard;
+                break;
+            }
+            widget = widget->parentWidget();
+        }
+    }
+    QTC_ASSERT(m_wizard, return);
+    m_wizard->setTitleFormat(m_wizard->titleFormat());
+    // UGLY HACK END
 }
 
-QString LibraryDetailsController::proFile() const
+FilePath LibraryDetailsController::proFile() const
 {
     return m_proFile;
 }
@@ -344,7 +372,7 @@ void LibraryDetailsController::slotMacLibraryTypeChanged()
             && libraryDetailsWidget()->frameworkRadio->isChecked()) {
         setIgnoreGuiSignals(true);
         libraryDetailsWidget()->dynamicRadio->setChecked(true);
-        setIgnoreGuiSignals(true);
+        setIgnoreGuiSignals(false);
     }
 
     updateGui();
@@ -371,7 +399,7 @@ static QString smartQuote(const QString &aString)
 {
     // The OS type is not important in that case, but use always the same
     // in order not to generate different quoting depending on host platform
-    return Utils::QtcProcess::quoteArg(aString, Utils::OsTypeLinux);
+    return ProcessArgs::quoteArg(aString, OsTypeLinux);
 }
 
 static QString appendSeparator(const QString &aString)
@@ -448,7 +476,7 @@ static QString generateLibsSnippet(AddLibraryWizard::Platforms platforms,
                                                                | AddLibraryWizard::WindowsMSVCPlatform);
 
     AddLibraryWizard::Platforms diffPlatforms = platforms ^ commonPlatforms;
-    AddLibraryWizard::Platforms generatedPlatforms = nullptr;
+    AddLibraryWizard::Platforms generatedPlatforms;
 
     QString snippetMessage;
     QTextStream str(&snippetMessage);
@@ -534,7 +562,7 @@ static QString generatePreTargetDepsSnippet(AddLibraryWizard::Platforms platform
     QString snippetMessage;
     QTextStream str(&snippetMessage);
     str << "\n";
-    AddLibraryWizard::Platforms generatedPlatforms = nullptr;
+    AddLibraryWizard::Platforms generatedPlatforms;
     AddLibraryWizard::Platforms windowsPlatforms = platforms
             & (AddLibraryWizard::WindowsMinGWPlatform | AddLibraryWizard::WindowsMSVCPlatform);
     AddLibraryWizard::Platforms commonPlatforms = platforms;
@@ -596,15 +624,15 @@ static QString generatePreTargetDepsSnippet(AddLibraryWizard::Platforms platform
 
 NonInternalLibraryDetailsController::NonInternalLibraryDetailsController(
         Ui::LibraryDetailsWidget *libraryDetails,
-        const QString &proFile, QObject *parent) :
+        const FilePath &proFile, QObject *parent) :
     LibraryDetailsController(libraryDetails, proFile, parent)
 {
     setLibraryComboBoxVisible(false);
     setLibraryPathChooserVisible(true);
 
-    connect(libraryDetailsWidget()->libraryPathChooser, &Utils::PathChooser::validChanged,
+    connect(libraryDetailsWidget()->libraryPathChooser, &PathChooser::validChanged,
             this, &LibraryDetailsController::completeChanged);
-    connect(libraryDetailsWidget()->libraryPathChooser, &Utils::PathChooser::rawPathChanged,
+    connect(libraryDetailsWidget()->libraryPathChooser, &PathChooser::rawPathChanged,
             this, &NonInternalLibraryDetailsController::slotLibraryPathChanged);
     connect(libraryDetailsWidget()->removeSuffixCheckBox, &QAbstractButton::toggled,
             this, &NonInternalLibraryDetailsController::slotRemoveSuffixChanged);
@@ -614,15 +642,15 @@ NonInternalLibraryDetailsController::NonInternalLibraryDetailsController(
             this, &NonInternalLibraryDetailsController::slotLinkageTypeChanged);
     connect(libraryDetailsWidget()->libraryTypeComboBox, &QComboBox::currentTextChanged,
             this, &NonInternalLibraryDetailsController::slotLibraryTypeChanged);
-    slotLibraryTypeChanged();
+    handleLibraryTypeChange();
 }
 
 AddLibraryWizard::LinkageType NonInternalLibraryDetailsController::suggestedLinkageType() const
 {
     AddLibraryWizard::LinkageType type = AddLibraryWizard::NoLinkage;
-    if (libraryPlatformType() != Utils::OsTypeWindows) {
+    if (libraryPlatformType() != OsTypeWindows) {
         if (libraryDetailsWidget()->libraryPathChooser->isValid()) {
-            QFileInfo fi(libraryDetailsWidget()->libraryPathChooser->path());
+            QFileInfo fi(libraryDetailsWidget()->libraryPathChooser->filePath().toString());
             if (fi.suffix() == QLatin1String("a"))
                 type = AddLibraryWizard::StaticLinkage;
             else
@@ -635,9 +663,9 @@ AddLibraryWizard::LinkageType NonInternalLibraryDetailsController::suggestedLink
 AddLibraryWizard::MacLibraryType NonInternalLibraryDetailsController::suggestedMacLibraryType() const
 {
     AddLibraryWizard::MacLibraryType type = AddLibraryWizard::NoLibraryType;
-    if (libraryPlatformType() == Utils::OsTypeMac) {
+    if (libraryPlatformType() == OsTypeMac) {
         if (libraryDetailsWidget()->libraryPathChooser->isValid()) {
-            QFileInfo fi(libraryDetailsWidget()->libraryPathChooser->path());
+            QFileInfo fi(libraryDetailsWidget()->libraryPathChooser->filePath().toString());
             if (fi.suffix() == QLatin1String("framework"))
                 type = AddLibraryWizard::FrameworkType;
             else
@@ -651,7 +679,7 @@ QString NonInternalLibraryDetailsController::suggestedIncludePath() const
 {
     QString includePath;
     if (libraryDetailsWidget()->libraryPathChooser->isValid()) {
-        QFileInfo fi(libraryDetailsWidget()->libraryPathChooser->path());
+        QFileInfo fi(libraryDetailsWidget()->libraryPathChooser->filePath().toString());
         includePath = fi.absolutePath();
         QFileInfo dfi(includePath);
         // TODO: Win: remove debug or release folder first if appropriate
@@ -669,25 +697,29 @@ QString NonInternalLibraryDetailsController::suggestedIncludePath() const
 void NonInternalLibraryDetailsController::updateWindowsOptionsEnablement()
 {
     bool ena = platforms() & (AddLibraryWizard::WindowsMinGWPlatform | AddLibraryWizard::WindowsMSVCPlatform);
-    if (libraryPlatformType() == Utils::OsTypeWindows) {
+    if (libraryPlatformType() == OsTypeWindows) {
         libraryDetailsWidget()->addSuffixCheckBox->setEnabled(ena);
         ena = true;
     }
     libraryDetailsWidget()->winGroupBox->setEnabled(ena);
 }
 
+void NonInternalLibraryDetailsController::handleLinkageTypeChange()
+{
+    if (isMacLibraryRadiosVisible()
+            && libraryDetailsWidget()->staticRadio->isChecked()) {
+        setIgnoreGuiSignals(true);
+        libraryDetailsWidget()->libraryRadio->setChecked(true);
+        setIgnoreGuiSignals(false);
+    }
+}
+
 void NonInternalLibraryDetailsController::slotLinkageTypeChanged()
 {
     if (guiSignalsIgnored())
         return;
 
-    if (isMacLibraryRadiosVisible()
-            && libraryDetailsWidget()->staticRadio->isChecked()) {
-        setIgnoreGuiSignals(true);
-        libraryDetailsWidget()->libraryRadio->setChecked(true);
-        setIgnoreGuiSignals(true);
-    }
-
+    handleLinkageTypeChange();
     updateGui();
 }
 
@@ -699,30 +731,34 @@ void NonInternalLibraryDetailsController::slotRemoveSuffixChanged(bool ena)
     }
 }
 
-void NonInternalLibraryDetailsController::slotLibraryTypeChanged()
+void NonInternalLibraryDetailsController::handleLibraryTypeChange()
 {
     libraryDetailsWidget()->libraryPathChooser->setPromptDialogFilter(libraryPlatformFilter());
-    const bool isMacOs = libraryPlatformType() == Utils::OsTypeMac;
-    const bool isWindows = libraryPlatformType() == Utils::OsTypeWindows;
-    libraryDetailsWidget()->libraryPathChooser->setExpectedKind(isMacOs ? Utils::PathChooser::Any
-                                                                        : Utils::PathChooser::File);
+    const bool isMacOs = libraryPlatformType() == OsTypeMac;
+    const bool isWindows = libraryPlatformType() == OsTypeWindows;
+    libraryDetailsWidget()->libraryPathChooser->setExpectedKind(isMacOs ? PathChooser::Any
+                                                                        : PathChooser::File);
     setMacLibraryRadiosVisible(!isMacOs);
     setLinkageRadiosVisible(isWindows);
     setRemoveSuffixVisible(isWindows);
-
-    updateWindowsOptionsEnablement();
-    slotLibraryPathChanged();
-    slotLinkageTypeChanged();
-    libraryDetailsWidget()->detailsLayout->parentWidget()->window()->adjustSize();
+    handleLibraryPathChange();
+    handleLinkageTypeChange();
 }
 
-void NonInternalLibraryDetailsController::slotLibraryPathChanged()
+void NonInternalLibraryDetailsController::slotLibraryTypeChanged()
 {
-    if (libraryPlatformType() == Utils::OsTypeWindows) {
+    handleLibraryTypeChange();
+    updateGui();
+    emit completeChanged();
+}
+
+void NonInternalLibraryDetailsController::handleLibraryPathChange()
+{
+    if (libraryPlatformType() == OsTypeWindows) {
         bool subfoldersEnabled = true;
         bool removeSuffixEnabled = true;
         if (libraryDetailsWidget()->libraryPathChooser->isValid()) {
-            QFileInfo fi(libraryDetailsWidget()->libraryPathChooser->path());
+            QFileInfo fi(libraryDetailsWidget()->libraryPathChooser->filePath().toString());
             QFileInfo dfi(fi.absolutePath());
             const QString parentFolderName = dfi.fileName().toLower();
             if (parentFolderName != QLatin1String("debug") &&
@@ -741,9 +777,12 @@ void NonInternalLibraryDetailsController::slotLibraryPathChanged()
                 libraryDetailsWidget()->addSuffixCheckBox->setChecked(true);
         }
     }
+}
 
+void NonInternalLibraryDetailsController::slotLibraryPathChanged()
+{
+    handleLibraryPathChange();
     updateGui();
-
     emit completeChanged();
 }
 
@@ -755,18 +794,18 @@ bool NonInternalLibraryDetailsController::isComplete() const
 
 QString NonInternalLibraryDetailsController::snippet() const
 {
-    QString libPath = libraryDetailsWidget()->libraryPathChooser->path();
+    QString libPath = libraryDetailsWidget()->libraryPathChooser->filePath().toString();
     QFileInfo fi(libPath);
     QString libName;
     const bool removeSuffix = isWindowsGroupVisible()
             && libraryDetailsWidget()->removeSuffixCheckBox->isChecked();
-    if (libraryPlatformType() == Utils::OsTypeWindows) {
+    if (libraryPlatformType() == OsTypeWindows) {
         libName = fi.completeBaseName();
         if (removeSuffix && !libName.isEmpty()) // remove last letter which needs to be "d"
             libName = libName.left(libName.size() - 1);
         if (fi.completeSuffix() == QLatin1String("a")) // the mingw lib case
             libName = libName.mid(3); // cut the "lib" prefix
-    } else if (libraryPlatformType() == Utils::OsTypeMac) {
+    } else if (libraryPlatformType() == OsTypeMac) {
         if (macLibraryType() == AddLibraryWizard::FrameworkType)
             libName = fi.completeBaseName();
         else
@@ -780,7 +819,7 @@ QString NonInternalLibraryDetailsController::snippet() const
     if (isWindowsGroupVisible()) {
         // when we are on Win but we don't generate the code for Win
         // we still need to remove "debug" or "release" subfolder
-        const bool useSubfoldersCondition = (libraryPlatformType() == Utils::OsTypeWindows)
+        const bool useSubfoldersCondition = (libraryPlatformType() == OsTypeWindows)
                                             ? true : platforms() & (AddLibraryWizard::WindowsMinGWPlatform
                                                                     | AddLibraryWizard::WindowsMSVCPlatform);
         if (useSubfoldersCondition)
@@ -792,16 +831,16 @@ QString NonInternalLibraryDetailsController::snippet() const
     QString targetRelativePath;
     QString includeRelativePath;
     if (isIncludePathVisible()) { // generate also the path to lib
-        QFileInfo pfi(proFile());
+        QFileInfo pfi = proFile().toFileInfo();
         QDir pdir = pfi.absoluteDir();
         QString absoluteLibraryPath = fi.absolutePath();
-        if (libraryPlatformType() == Utils::OsTypeWindows && useSubfolders) { // drop last subfolder which needs to be "debug" or "release"
+        if (libraryPlatformType() == OsTypeWindows && useSubfolders) { // drop last subfolder which needs to be "debug" or "release"
             QFileInfo libfi(absoluteLibraryPath);
             absoluteLibraryPath = libfi.absolutePath();
         }
         targetRelativePath = appendSeparator(pdir.relativeFilePath(absoluteLibraryPath));
 
-        const QString includePath = libraryDetailsWidget()->includePathChooser->path();
+        const QString includePath = libraryDetailsWidget()->includePathChooser->filePath().toString();
         if (!includePath.isEmpty())
             includeRelativePath = pdir.relativeFilePath(includePath);
     }
@@ -825,7 +864,7 @@ QString NonInternalLibraryDetailsController::snippet() const
 
 PackageLibraryDetailsController::PackageLibraryDetailsController(
     Ui::LibraryDetailsWidget *libraryDetails,
-    const QString &proFile, QObject *parent)
+    const FilePath &proFile, QObject *parent)
     : NonInternalLibraryDetailsController(libraryDetails, proFile, parent)
 {
     setPlatformsVisible(false);
@@ -860,11 +899,11 @@ QString PackageLibraryDetailsController::snippet() const
 
 bool PackageLibraryDetailsController::isLinkPackageGenerated() const
 {
-    const Project *project = SessionManager::projectForFile(Utils::FilePath::fromString(proFile()));
+    const Project *project = SessionManager::projectForFile(proFile());
     if (!project)
         return false;
 
-    const ProjectNode *projectNode = project->findNodeForBuildKey(proFile());
+    const ProjectNode *projectNode = project->findNodeForBuildKey(proFile().toString());
     if (!projectNode)
         return false;
 
@@ -884,7 +923,7 @@ bool PackageLibraryDetailsController::isLinkPackageGenerated() const
 
 SystemLibraryDetailsController::SystemLibraryDetailsController(
     Ui::LibraryDetailsWidget *libraryDetails,
-    const QString &proFile, QObject *parent)
+    const FilePath &proFile, QObject *parent)
     : NonInternalLibraryDetailsController(libraryDetails, proFile, parent)
 {
     setIncludePathVisible(false);
@@ -897,7 +936,7 @@ SystemLibraryDetailsController::SystemLibraryDetailsController(
 
 ExternalLibraryDetailsController::ExternalLibraryDetailsController(
     Ui::LibraryDetailsWidget *libraryDetails,
-    const QString &proFile, QObject *parent)
+    const FilePath &proFile, QObject *parent)
     : NonInternalLibraryDetailsController(libraryDetails, proFile, parent)
 {
     setIncludePathVisible(true);
@@ -912,9 +951,9 @@ void ExternalLibraryDetailsController::updateWindowsOptionsEnablement()
 
     bool subfoldersEnabled = true;
     bool removeSuffixEnabled = true;
-    if (libraryPlatformType() == Utils::OsTypeWindows
+    if (libraryPlatformType() == OsTypeWindows
             && libraryDetailsWidget()->libraryPathChooser->isValid()) {
-        QFileInfo fi(libraryDetailsWidget()->libraryPathChooser->path());
+        QFileInfo fi(libraryDetailsWidget()->libraryPathChooser->filePath().toString());
         QFileInfo dfi(fi.absolutePath());
         const QString parentFolderName = dfi.fileName().toLower();
         if (parentFolderName != QLatin1String("debug") &&
@@ -931,9 +970,8 @@ void ExternalLibraryDetailsController::updateWindowsOptionsEnablement()
 
 /////////////
 
-InternalLibraryDetailsController::InternalLibraryDetailsController(
-        Ui::LibraryDetailsWidget *libraryDetails,
-        const QString &proFile, QObject *parent)
+InternalLibraryDetailsController::InternalLibraryDetailsController(Ui::LibraryDetailsWidget *libraryDetails,
+        const FilePath &proFile, QObject *parent)
     : LibraryDetailsController(libraryDetails, proFile, parent)
 {
     setLinkageRadiosVisible(false);
@@ -943,7 +981,7 @@ InternalLibraryDetailsController::InternalLibraryDetailsController(
     setWindowsGroupVisible(true);
     setRemoveSuffixVisible(false);
 
-    if (Utils::HostOsInfo::isWindowsHost())
+    if (HostOsInfo::isWindowsHost())
         libraryDetailsWidget()->useSubfoldersCheckBox->setEnabled(true);
 
     connect(libraryDetailsWidget()->libraryComboBox,
@@ -997,7 +1035,7 @@ QString InternalLibraryDetailsController::suggestedIncludePath() const
 
 void InternalLibraryDetailsController::updateWindowsOptionsEnablement()
 {
-    if (Utils::HostOsInfo::isWindowsHost())
+    if (HostOsInfo::isWindowsHost())
         libraryDetailsWidget()->addSuffixCheckBox->setEnabled(true);
     libraryDetailsWidget()->winGroupBox->setEnabled(platforms()
                                 & (AddLibraryWizard::WindowsMinGWPlatform | AddLibraryWizard::WindowsMSVCPlatform));
@@ -1010,7 +1048,7 @@ void InternalLibraryDetailsController::updateProFile()
     libraryDetailsWidget()->libraryComboBox->clear();
 
     const QmakeProject *project
-            = dynamic_cast<QmakeProject *>(SessionManager::projectForFile(Utils::FilePath::fromString(proFile())));
+            = dynamic_cast<QmakeProject *>(SessionManager::projectForFile(proFile()));
     if (!project)
         return;
 
@@ -1018,8 +1056,12 @@ void InternalLibraryDetailsController::updateProFile()
 
     m_rootProjectPath = project->projectDirectory().toString();
 
+    auto t = project->activeTarget();
+    auto bs = dynamic_cast<QmakeBuildSystem *>(t ? t->buildSystem() : nullptr);
+    QTC_ASSERT(bs, return);
+
     QDir rootDir(m_rootProjectPath);
-    foreach (QmakeProFile *proFile, project->rootProFile()->allProFiles()) {
+    foreach (QmakeProFile *proFile, bs->rootProFile()->allProFiles()) {
         QmakeProjectManager::ProjectType type = proFile->projectType();
         if (type != ProjectType::SharedLibraryTemplate && type != ProjectType::StaticLibraryTemplate)
             continue;
@@ -1050,7 +1092,7 @@ void InternalLibraryDetailsController::slotCurrentLibraryChanged()
                         currentIndex, Qt::ToolTipRole).toString());
         QmakeProFile *proFile = m_proFiles.at(currentIndex);
         const QStringList configVar = proFile->variableValue(Variable::Config);
-        if (Utils::HostOsInfo::isWindowsHost()) {
+        if (HostOsInfo::isWindowsHost()) {
             bool useSubfolders = false;
             if (configVar.contains(QLatin1String("debug_and_release"))
                 && configVar.contains(QLatin1String("debug_and_release_target")))
@@ -1089,10 +1131,10 @@ QString InternalLibraryDetailsController::snippet() const
 
     // relative path for the project for which we insert the snippet,
     // it's relative to the root project
-    const QString proRelavitePath = rootDir.relativeFilePath(proFile());
+    const QString proRelavitePath = rootDir.relativeFilePath(proFile().toString());
 
     // project for which we insert the snippet
-    const Project *project = SessionManager::projectForFile(Utils::FilePath::fromString(proFile()));
+    const Project *project = SessionManager::projectForFile(proFile());
 
     // the build directory of the active build configuration
     QDir rootBuildDir = rootDir; // If the project is unconfigured use the project dir
@@ -1106,14 +1148,14 @@ QString InternalLibraryDetailsController::snippet() const
     QDir projectBuildDir(pfi.absolutePath());
 
     // current project node from combobox
-    QFileInfo fi(proFile());
+    QFileInfo fi = proFile().toFileInfo();
     QDir projectSrcDir(fi.absolutePath());
 
     // project node which we want to link against
     TargetInformation targetInfo = m_proFiles.at(currentIndex)->targetInformation();
 
     const QString targetRelativePath = appendSeparator(projectBuildDir.relativeFilePath(targetInfo.buildDir.toString()));
-    const QString includeRelativePath = projectSrcDir.relativeFilePath(libraryDetailsWidget()->includePathChooser->path());
+    const QString includeRelativePath = projectSrcDir.relativeFilePath(libraryDetailsWidget()->includePathChooser->filePath().toString());
 
     const bool useSubfolders = libraryDetailsWidget()->useSubfoldersCheckBox->isChecked();
     const bool addSuffix = libraryDetailsWidget()->addSuffixCheckBox->isChecked();
@@ -1134,3 +1176,6 @@ QString InternalLibraryDetailsController::snippet() const
                                useSubfolders, addSuffix);
     return snippetMessage;
 }
+
+} // Internal
+} // QmakeProjectManager
