@@ -26,27 +26,26 @@
 #include "makefileparse.h"
 
 #include <qtsupport/qtversionmanager.h>
-#include <qtsupport/baseqtversion.h>
+
 #include <utils/qtcprocess.h>
 
 #include <QDebug>
 #include <QDir>
 #include <QFileInfo>
-#include <QRegExp>
-#include <QTextStream>
 #include <QLoggingCategory>
+#include <QRegularExpression>
+#include <QTextStream>
 
-using namespace QmakeProjectManager;
-using namespace Internal;
+using namespace ProjectExplorer;
+using namespace QtSupport;
+using namespace Utils;
 
-using Utils::FilePath;
-using Utils::QtcProcess;
-using QtSupport::QtVersionManager;
-using QtSupport::BaseQtVersion;
+namespace QmakeProjectManager {
+namespace Internal {
 
-static QString findQMakeLine(const QString &makefile, const QString &key)
+static QString findQMakeLine(const FilePath &makefile, const QString &key)
 {
-    QFile fi(makefile);
+    QFile fi(makefile.toString());
     if (fi.exists() && fi.open(QFile::ReadOnly)) {
         QTextStream ts(&fi);
         while (!ts.atEnd()) {
@@ -71,11 +70,11 @@ void MakeFileParse::parseArgs(const QString &args, const QString &project,
                               QList<QMakeAssignment> *assignments,
                               QList<QMakeAssignment> *afterAssignments)
 {
-    QRegExp regExp(QLatin1String("([^\\s\\+-]*)\\s*(\\+=|=|-=|~=)(.*)"));
+    const QRegularExpression regExp(QLatin1String("^([^\\s\\+-]*)\\s*(\\+=|=|-=|~=)(.*)$"));
     bool after = false;
     bool ignoreNext = false;
     m_unparsedArguments = args;
-    QtcProcess::ArgIterator ait(&m_unparsedArguments);
+    ProcessArgs::ArgIterator ait(&m_unparsedArguments);
     while (ait.next()) {
         if (ignoreNext) {
             // Ignoring
@@ -87,11 +86,12 @@ void MakeFileParse::parseArgs(const QString &args, const QString &project,
             after = true;
             ait.deleteArg();
         } else if (ait.value().contains(QLatin1Char('='))) {
-            if (regExp.exactMatch(ait.value())) {
+            const QRegularExpressionMatch match = regExp.match(ait.value());
+            if (match.hasMatch()) {
                 QMakeAssignment qa;
-                qa.variable = regExp.cap(1);
-                qa.op = regExp.cap(2);
-                qa.value = regExp.cap(3).trimmed();
+                qa.variable = match.captured(1);
+                qa.op = match.captured(2);
+                qa.value = match.captured(3).trimmed();
                 if (after)
                     afterAssignments->append(qa);
                 else
@@ -124,13 +124,12 @@ void dumpQMakeAssignments(const QList<QMakeAssignment> &list)
     }
 }
 
-void MakeFileParse::parseAssignments(QList<QMakeAssignment> *assignments)
+QList<QMakeAssignment> MakeFileParse::parseAssignments(const QList<QMakeAssignment> &assignments)
 {
     bool foundSeparateDebugInfo = false;
     bool foundForceDebugInfo = false;
-    QList<QMakeAssignment> oldAssignments = *assignments;
-    assignments->clear();
-    foreach (const QMakeAssignment &qa, oldAssignments) {
+    QList<QMakeAssignment> filteredAssignments;
+    foreach (const QMakeAssignment &qa, assignments) {
         if (qa.variable == QLatin1String("CONFIG")) {
             QStringList values = qa.value.split(QLatin1Char(' '));
             QStringList newValues;
@@ -159,26 +158,6 @@ void MakeFileParse::parseAssignments(QList<QMakeAssignment> *assignments)
                         m_qmakeBuildConfig.explicitBuildAll = false;
                         m_qmakeBuildConfig.explicitNoBuildAll = true;
                     }
-                } else if (value == QLatin1String("x86")) {
-                    if (qa.op == QLatin1String("+="))
-                        m_config.archConfig = QMakeStepConfig::X86;
-                    else
-                        m_config.archConfig = QMakeStepConfig::NoArch;
-                } else if (value == QLatin1String("x86_64")) {
-                    if (qa.op == QLatin1String("+="))
-                        m_config.archConfig = QMakeStepConfig::X86_64;
-                    else
-                        m_config.archConfig = QMakeStepConfig::NoArch;
-                } else if (value == QLatin1String("ppc")) {
-                    if (qa.op == QLatin1String("+="))
-                        m_config.archConfig = QMakeStepConfig::PowerPC;
-                    else
-                        m_config.archConfig = QMakeStepConfig::NoArch;
-                } else if (value == QLatin1String("ppc64")) {
-                    if (qa.op == QLatin1String("+="))
-                        m_config.archConfig = QMakeStepConfig::PowerPC64;
-                    else
-                        m_config.archConfig = QMakeStepConfig::NoArch;
                 } else if (value == QLatin1String("iphonesimulator")) {
                     if (qa.op == QLatin1String("+="))
                         m_config.osType = QMakeStepConfig::IphoneSimulator;
@@ -191,24 +170,27 @@ void MakeFileParse::parseAssignments(QList<QMakeAssignment> *assignments)
                         m_config.osType = QMakeStepConfig::NoOsType;
                 } else if (value == QLatin1String("qml_debug")) {
                     if (qa.op == QLatin1String("+="))
-                        m_config.linkQmlDebuggingQQ2 = true;
+                        m_config.linkQmlDebuggingQQ2 = TriState::Enabled;
                     else
-                        m_config.linkQmlDebuggingQQ2 = false;
+                        m_config.linkQmlDebuggingQQ2 = TriState::Disabled;
                 } else if (value == QLatin1String("qtquickcompiler")) {
                     if (qa.op == QLatin1String("+="))
-                        m_config.useQtQuickCompiler = true;
+                        m_config.useQtQuickCompiler = TriState::Enabled;
                     else
-                        m_config.useQtQuickCompiler = false;
+                        m_config.useQtQuickCompiler = TriState::Disabled;
                 } else if (value == QLatin1String("force_debug_info")) {
                     if (qa.op == QLatin1String("+="))
                         foundForceDebugInfo = true;
                     else
                         foundForceDebugInfo = false;
                 } else if (value == QLatin1String("separate_debug_info")) {
-                    if (qa.op == QLatin1String("+="))
+                    if (qa.op == QLatin1String("+=")) {
                         foundSeparateDebugInfo = true;
-                    else
+                        m_config.separateDebugInfo = TriState::Enabled;
+                    } else {
                         foundSeparateDebugInfo = false;
+                        m_config.separateDebugInfo = TriState::Disabled;
+                    }
                 } else {
                     newValues.append(value);
                 }
@@ -216,42 +198,44 @@ void MakeFileParse::parseAssignments(QList<QMakeAssignment> *assignments)
             if (!newValues.isEmpty()) {
                 QMakeAssignment newQA = qa;
                 newQA.value = newValues.join(QLatin1Char(' '));
-                assignments->append(newQA);
+                filteredAssignments.append(newQA);
             }
         } else {
-            assignments->append(qa);
+            filteredAssignments.append(qa);
         }
     }
 
     if (foundForceDebugInfo && foundSeparateDebugInfo) {
-        m_config.separateDebugInfo = true;
+        m_config.separateDebugInfo = TriState::Enabled;
     } else if (foundForceDebugInfo) {
         // Found only force_debug_info, so readd it
         QMakeAssignment newQA;
         newQA.variable = QLatin1String("CONFIG");
         newQA.op = QLatin1String("+=");
         newQA.value = QLatin1String("force_debug_info");
-        assignments->append(newQA);
+        filteredAssignments.append(newQA);
     } else if (foundSeparateDebugInfo) {
         // Found only separate_debug_info, so readd it
         QMakeAssignment newQA;
         newQA.variable = QLatin1String("CONFIG");
         newQA.op = QLatin1String("+=");
         newQA.value = QLatin1String("separate_debug_info");
-        assignments->append(newQA);
+        filteredAssignments.append(newQA);
     }
+    return filteredAssignments;
 }
 
-static FilePath findQMakeBinaryFromMakefile(const QString &makefile)
+static FilePath findQMakeBinaryFromMakefile(const FilePath &makefile)
 {
-    QFile fi(makefile);
+    QFile fi(makefile.toString());
     if (fi.exists() && fi.open(QFile::ReadOnly)) {
         QTextStream ts(&fi);
-        QRegExp r1(QLatin1String("QMAKE\\s*=(.*)"));
+        const QRegularExpression r1(QLatin1String("^QMAKE\\s*=(.*)$"));
         while (!ts.atEnd()) {
             QString line = ts.readLine();
-            if (r1.exactMatch(line)) {
-                QFileInfo qmake(r1.cap(1).trimmed());
+            const QRegularExpressionMatch match = r1.match(line);
+            if (match.hasMatch()) {
+                QFileInfo qmake(match.captured(1).trimmed());
                 QString qmakePath = qmake.filePath();
                 if (!QString::fromLatin1(QTC_HOST_EXE_SUFFIX).isEmpty()
                         && !qmakePath.endsWith(QLatin1String(QTC_HOST_EXE_SUFFIX))) {
@@ -267,10 +251,10 @@ static FilePath findQMakeBinaryFromMakefile(const QString &makefile)
     return FilePath();
 }
 
-MakeFileParse::MakeFileParse(const QString &makefile)
+MakeFileParse::MakeFileParse(const FilePath &makefile, Mode mode) : m_mode(mode)
 {
     qCDebug(logging()) << "Parsing makefile" << makefile;
-    if (!QFileInfo::exists(makefile)) {
+    if (!makefile.exists()) {
         qCDebug(logging()) << "**doesn't exist";
         m_state = MakefileMissing;
         return;
@@ -291,7 +275,7 @@ MakeFileParse::MakeFileParse(const QString &makefile)
     project = project.trimmed();
 
     // Src Pro file
-    m_srcProFile = QDir::cleanPath(QFileInfo(makefile).absoluteDir().filePath(project));
+    m_srcProFile = makefile.parentDir().resolvePath(project);
     qCDebug(logging()) << "  source .pro file:" << m_srcProFile;
 
     QString command = findQMakeLine(makefile, QLatin1String("# Command:")).trimmed();
@@ -312,12 +296,12 @@ MakeFileParse::MakefileState MakeFileParse::makeFileState() const
     return m_state;
 }
 
-Utils::FilePath MakeFileParse::qmakePath() const
+FilePath MakeFileParse::qmakePath() const
 {
     return m_qmakePath;
 }
 
-QString MakeFileParse::srcProFile() const
+FilePath MakeFileParse::srcProFile() const
 {
     return m_srcProFile;
 }
@@ -364,30 +348,36 @@ void MakeFileParse::parseCommandLine(const QString &command, const QString &proj
     dumpQMakeAssignments(assignments);
 
     // Filter out CONFIG arguments we know into m_qmakeBuildConfig and m_config
-    parseAssignments(&assignments);
+    const QList<QMakeAssignment> filteredAssignments = parseAssignments(assignments);
     qCDebug(logging()) << "  After parsing";
-    dumpQMakeAssignments(assignments);
+    dumpQMakeAssignments(filteredAssignments);
 
     qCDebug(logging()) << "  Explicit Debug" << m_qmakeBuildConfig.explicitDebug;
     qCDebug(logging()) << "  Explicit Release" << m_qmakeBuildConfig.explicitRelease;
     qCDebug(logging()) << "  Explicit BuildAll" << m_qmakeBuildConfig.explicitBuildAll;
     qCDebug(logging()) << "  Explicit NoBuildAll" << m_qmakeBuildConfig.explicitNoBuildAll;
-    qCDebug(logging()) << "  TargetArch" << m_config.archConfig;
     qCDebug(logging()) << "  OsType" << m_config.osType;
-    qCDebug(logging()) << "  LinkQmlDebuggingQQ2" << m_config.linkQmlDebuggingQQ2;
-    qCDebug(logging()) << "  Qt Quick Compiler" << m_config.useQtQuickCompiler;
-    qCDebug(logging()) << "  Separate Debug Info" << m_config.separateDebugInfo;
+    qCDebug(logging()) << "  LinkQmlDebuggingQQ2"
+                       << (m_config.linkQmlDebuggingQQ2 == TriState::Enabled);
+    qCDebug(logging()) << "  Qt Quick Compiler"
+                       <<  (m_config.useQtQuickCompiler == TriState::Enabled);
+    qCDebug(logging()) << "  Separate Debug Info"
+                       << (m_config.separateDebugInfo == TriState::Enabled);
 
     // Create command line of all unfiltered arguments
-    foreach (const QMakeAssignment &qa, assignments)
-        QtcProcess::addArg(&m_unparsedArguments, qa.variable + qa.op + qa.value);
+    const QList<QMakeAssignment> &assignmentsToUse = m_mode == Mode::FilterKnownConfigValues
+            ? filteredAssignments : assignments;
+    foreach (const QMakeAssignment &qa, assignmentsToUse)
+        ProcessArgs::addArg(&m_unparsedArguments, qa.variable + qa.op + qa.value);
     if (!afterAssignments.isEmpty()) {
-        QtcProcess::addArg(&m_unparsedArguments, QLatin1String("-after"));
+        ProcessArgs::addArg(&m_unparsedArguments, QLatin1String("-after"));
         foreach (const QMakeAssignment &qa, afterAssignments)
-            QtcProcess::addArg(&m_unparsedArguments, qa.variable + qa.op + qa.value);
+            ProcessArgs::addArg(&m_unparsedArguments, qa.variable + qa.op + qa.value);
     }
 }
 
+} // Internal
+} // QmakeProjectManager
 
 // Unit tests:
 
@@ -406,7 +396,6 @@ void QmakeProjectManagerPlugin::testMakefileParser_data()
     QTest::addColumn<QString>("command");
     QTest::addColumn<QString>("project");
     QTest::addColumn<QString>("unparsedArguments");
-    QTest::addColumn<int>("archConfig");
     QTest::addColumn<int>("osType");
     QTest::addColumn<bool>("linkQmlDebuggingQQ2");
     QTest::addColumn<bool>("useQtQuickCompiler");
@@ -417,85 +406,85 @@ void QmakeProjectManagerPlugin::testMakefileParser_data()
             << QString::fromLatin1("-spec linux-g++ CONFIG+=debug CONFIG+=qml_debug -o Makefile ../untitled7/untitled7.pro")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.7 extra1")
             << QString::fromLatin1("SOMETHING=ELSE -spec linux-g++ CONFIG+=debug CONFIG+=qml_debug -o Makefile ../untitled7/untitled7.pro")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.7 extra2")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE CONFIG+=debug CONFIG+=qml_debug -o Makefile ../untitled7/untitled7.pro")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.7 extra3")
             << QString::fromLatin1("-spec linux-g++ CONFIG+=debug SOMETHING=ELSE CONFIG+=qml_debug -o Makefile ../untitled7/untitled7.pro")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.7 extra4")
             << QString::fromLatin1("-spec linux-g++ CONFIG+=debug CONFIG+=qml_debug SOMETHING=ELSE -o Makefile ../untitled7/untitled7.pro")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.7 extra5")
             << QString::fromLatin1("-spec linux-g++ CONFIG+=debug CONFIG+=qml_debug -o Makefile SOMETHING=ELSE ../untitled7/untitled7.pro")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.7 extra6")
             << QString::fromLatin1("-spec linux-g++ CONFIG+=debug CONFIG+=qml_debug -o Makefile ../untitled7/untitled7.pro SOMETHING=ELSE")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.8")
             << QString::fromLatin1("-o Makefile ../untitled7/untitled7.pro -spec linux-g++ CONFIG+=debug CONFIG+=qml_debug")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.8 extra1")
             << QString::fromLatin1("SOMETHING=ELSE -o Makefile ../untitled7/untitled7.pro -spec linux-g++ CONFIG+=debug CONFIG+=qml_debug")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.8 extra2")
             << QString::fromLatin1("-o Makefile SOMETHING=ELSE ../untitled7/untitled7.pro -spec linux-g++ CONFIG+=debug CONFIG+=qml_debug")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.8 extra3")
             << QString::fromLatin1("-o Makefile ../untitled7/untitled7.pro SOMETHING=ELSE -spec linux-g++ CONFIG+=debug CONFIG+=qml_debug")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.8 extra4")
             << QString::fromLatin1("-o Makefile ../untitled7/untitled7.pro -spec linux-g++ SOMETHING=ELSE CONFIG+=debug CONFIG+=qml_debug")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.8 extra5")
             << QString::fromLatin1("-o Makefile ../untitled7/untitled7.pro -spec linux-g++ CONFIG+=debug SOMETHING=ELSE CONFIG+=qml_debug")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
     QTest::newRow("Qt 5.8 extra6")
             << QString::fromLatin1("-o Makefile ../untitled7/untitled7.pro -spec linux-g++ CONFIG+=debug CONFIG+=qml_debug SOMETHING=ELSE")
             << QString::fromLatin1("../untitled7/untitled7.pro")
             << QString::fromLatin1("-spec linux-g++ SOMETHING=ELSE")
-            << static_cast<int>(QMakeStepConfig::NoArch) << static_cast<int>(QMakeStepConfig::NoOsType)
+            << static_cast<int>(QMakeStepConfig::NoOsType)
             << true << false << false << 2;
 }
 
@@ -504,25 +493,23 @@ void QmakeProjectManagerPlugin::testMakefileParser()
     QFETCH(QString, command);
     QFETCH(QString, project);
     QFETCH(QString, unparsedArguments);
-    QFETCH(int, archConfig);
     QFETCH(int, osType);
     QFETCH(bool, linkQmlDebuggingQQ2);
     QFETCH(bool, useQtQuickCompiler);
     QFETCH(bool, separateDebugInfo);
     QFETCH(int, effectiveBuildConfig);
 
-    MakeFileParse parser("/tmp/something");
+    MakeFileParse parser("/tmp/something", MakeFileParse::Mode::FilterKnownConfigValues);
     parser.parseCommandLine(command, project);
 
-    QCOMPARE(Utils::QtcProcess::splitArgs(parser.unparsedArguments()),
-             Utils::QtcProcess::splitArgs(unparsedArguments));
-    QCOMPARE(parser.effectiveBuildConfig(nullptr), effectiveBuildConfig);
+    QCOMPARE(Utils::ProcessArgs::splitArgs(parser.unparsedArguments()),
+             Utils::ProcessArgs::splitArgs(unparsedArguments));
+    QCOMPARE(parser.effectiveBuildConfig({}), effectiveBuildConfig);
 
     const QMakeStepConfig qmsc = parser.config();
-    QCOMPARE(qmsc.archConfig, static_cast<QMakeStepConfig::TargetArchConfig>(archConfig));
     QCOMPARE(qmsc.osType, static_cast<QMakeStepConfig::OsType>(osType));
-    QCOMPARE(qmsc.linkQmlDebuggingQQ2, linkQmlDebuggingQQ2);
-    QCOMPARE(qmsc.useQtQuickCompiler, useQtQuickCompiler);
-    QCOMPARE(qmsc.separateDebugInfo, separateDebugInfo);
+    QCOMPARE(qmsc.linkQmlDebuggingQQ2 == TriState::Enabled, linkQmlDebuggingQQ2);
+    QCOMPARE(qmsc.useQtQuickCompiler == TriState::Enabled, useQtQuickCompiler);
+    QCOMPARE(qmsc.separateDebugInfo == TriState::Enabled, separateDebugInfo);
 }
 #endif
