@@ -50,10 +50,32 @@
 #include <QFileInfo>
 
 using namespace ProjectExplorer;
-using namespace QmakeProjectManager;
-using namespace QmakeProjectManager::Internal;
+using namespace Utils;
 
-QmakeMakeStep::QmakeMakeStep(BuildStepList *bsl, Utils::Id id)
+namespace QmakeProjectManager {
+namespace Internal {
+
+class QmakeMakeStep : public MakeStep
+{
+    Q_DECLARE_TR_FUNCTIONS(QmakeProjectManager::QmakeMakeStep)
+
+public:
+    QmakeMakeStep(BuildStepList *bsl, Id id);
+
+private:
+    void finish(bool success) override;
+    bool init() override;
+    void setupOutputFormatter(OutputFormatter *formatter) override;
+    void doRun() override;
+    QStringList displayArguments() const override;
+
+    bool m_scriptTarget = false;
+    FilePath m_makeFileToCheck;
+    bool m_unalignedBuildDir;
+    bool m_ignoredNonTopLevelBuild = false;
+};
+
+QmakeMakeStep::QmakeMakeStep(BuildStepList *bsl, Id id)
     : MakeStep(bsl, id)
 {
     if (bsl->id() == ProjectExplorer::Constants::BUILDSTEPS_CLEAN) {
@@ -71,8 +93,8 @@ bool QmakeMakeStep::init()
 
     const auto bc = static_cast<QmakeBuildConfiguration *>(buildConfiguration());
 
-    const Utils::CommandLine unmodifiedMake = effectiveMakeCommand(Execution);
-    const Utils::FilePath makeExecutable = unmodifiedMake.executable();
+    const CommandLine unmodifiedMake = effectiveMakeCommand(Execution);
+    const FilePath makeExecutable = unmodifiedMake.executable();
     if (makeExecutable.isEmpty())
         emit addTask(makeCommandMissingTask());
 
@@ -87,14 +109,14 @@ bool QmakeMakeStep::init()
     ProcessParameters *pp = processParameters();
     pp->setMacroExpander(bc->macroExpander());
 
-    Utils::FilePath workingDirectory;
+    FilePath workingDirectory;
     if (bc->subNodeBuild())
         workingDirectory = bc->qmakeBuildSystem()->buildDir(bc->subNodeBuild()->filePath());
     else
         workingDirectory = bc->buildDirectory();
     pp->setWorkingDirectory(workingDirectory);
 
-    Utils::CommandLine makeCmd(makeExecutable);
+    CommandLine makeCmd(makeExecutable);
 
     QmakeProjectManager::QmakeProFileNode *subProFile = bc->subNodeBuild();
     if (subProFile) {
@@ -114,18 +136,18 @@ bool QmakeMakeStep::init()
         if (makefile != "Makefile")
             makeCmd.addArgs({"-f", makefile});
 
-        m_makeFileToCheck = QDir(workingDirectory.toString()).filePath(makefile);
+        m_makeFileToCheck = workingDirectory / makefile;
     } else {
-        QString makefile = bc->makefile();
+        FilePath makefile = bc->makefile();
         if (!makefile.isEmpty()) {
-            makeCmd.addArgs({"-f", makefile});
-            m_makeFileToCheck = QDir(workingDirectory.toString()).filePath(makefile);
+            makeCmd.addArgs({"-f", makefile.path()});
+            m_makeFileToCheck = workingDirectory / makefile.path();
         } else {
-            m_makeFileToCheck = QDir(workingDirectory.toString()).filePath("Makefile");
+            m_makeFileToCheck = workingDirectory / "Makefile";
         }
     }
 
-    makeCmd.addArgs(unmodifiedMake.arguments(), Utils::CommandLine::Raw);
+    makeCmd.addArgs(unmodifiedMake.arguments(), CommandLine::Raw);
 
     if (bc->fileNodeBuild() && subProFile) {
         QString objectsDir = subProFile->objectsDirectory();
@@ -140,8 +162,8 @@ bool QmakeMakeStep::init()
         }
 
         if (subProFile->isObjectParallelToSource()) {
-            const Utils::FilePath sourceFileDir = bc->fileNodeBuild()->filePath().parentDir();
-            const Utils::FilePath proFileDir = subProFile->proFile()->sourceDir().canonicalPath();
+            const FilePath sourceFileDir = bc->fileNodeBuild()->filePath().parentDir();
+            const FilePath proFileDir = subProFile->proFile()->sourceDir().canonicalPath();
             if (!objectsDir.endsWith('/'))
                 objectsDir += QLatin1Char('/');
             objectsDir += sourceFileDir.relativeChildPath(proFileDir).toString();
@@ -154,9 +176,8 @@ bool QmakeMakeStep::init()
             relObjectsDir.clear();
         if (!relObjectsDir.isEmpty())
             relObjectsDir += '/';
-        QString objectFile = relObjectsDir +
-                bc->fileNodeBuild()->filePath().toFileInfo().baseName() +
-                subProFile->objectExtension();
+        QString objectFile = relObjectsDir + bc->fileNodeBuild()->filePath().baseName()
+                             + subProFile->objectExtension();
         makeCmd.addArg(objectFile);
     }
 
@@ -179,22 +200,22 @@ bool QmakeMakeStep::init()
     return true;
 }
 
-void QmakeMakeStep::setupOutputFormatter(Utils::OutputFormatter *formatter)
+void QmakeMakeStep::setupOutputFormatter(OutputFormatter *formatter)
 {
-    formatter->addLineParser(new ProjectExplorer::GnuMakeParser());
+    formatter->addLineParser(new GnuMakeParser());
     ToolChain *tc = ToolChainKitAspect::cxxToolChain(kit());
     OutputTaskParser *xcodeBuildParser = nullptr;
     if (tc && tc->targetAbi().os() == Abi::DarwinOS) {
         xcodeBuildParser = new XcodebuildParser;
         formatter->addLineParser(xcodeBuildParser);
     }
-    QList<Utils::OutputLineParser *> additionalParsers = kit()->createOutputParsers();
+    QList<OutputLineParser *> additionalParsers = kit()->createOutputParsers();
 
     // make may cause qmake to be run, add last to make sure it has a low priority.
     additionalParsers << new QMakeParser;
 
     if (xcodeBuildParser) {
-        for (Utils::OutputLineParser * const p : qAsConst(additionalParsers))
+        for (OutputLineParser * const p : qAsConst(additionalParsers))
             p->setRedirectionDetector(xcodeBuildParser);
     }
     formatter->addLineParsers(additionalParsers);
@@ -210,7 +231,7 @@ void QmakeMakeStep::doRun()
         return;
     }
 
-    if (!QFileInfo::exists(m_makeFileToCheck)) {
+    if (!m_makeFileToCheck.exists()) {
         if (!ignoreReturnValue())
             emit addOutput(tr("Cannot find Makefile. Check your build settings."), BuildStep::OutputFormat::NormalMessage);
         const bool success = ignoreReturnValue();
@@ -236,7 +257,7 @@ QStringList QmakeMakeStep::displayArguments() const
 {
     const auto bc = static_cast<QmakeBuildConfiguration *>(buildConfiguration());
     if (bc && !bc->makefile().isEmpty())
-        return {"-f", bc->makefile()};
+        return {"-f", bc->makefile().path()};
     return {};
 }
 
@@ -250,3 +271,6 @@ QmakeMakeStepFactory::QmakeMakeStepFactory()
     setSupportedProjectType(Constants::QMAKEPROJECT_ID);
     setDisplayName(MakeStep::defaultDisplayName());
 }
+
+} // Internal
+} // QmakeProjectManager
