@@ -29,12 +29,13 @@
 #include "qmakeprojectmanagerconstants.h"
 #include "qmakebuildconfiguration.h"
 
+#include <android/androidconstants.h>
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/iversioncontrol.h>
 #include <coreplugin/vcsmanager.h>
-#include <cpptools/cpptoolsconstants.h>
+#include <cppeditor/cppeditorconstants.h>
 #include <projectexplorer/editorconfiguration.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorerconstants.h>
@@ -68,8 +69,8 @@ namespace QmakeProjectManager {
 
 static Q_LOGGING_CATEGORY(qmakeParse, "qtc.qmake.parsing", QtWarningMsg);
 
-uint qHash(Variable key, uint seed) { return ::qHash(static_cast<int>(key), seed); }
-uint qHash(FileOrigin fo) { return ::qHash(int(fo)); }
+Utils::QHashValueType qHash(Variable key, uint seed) { return ::qHash(static_cast<int>(key), seed); }
+Utils::QHashValueType qHash(FileOrigin fo) { return ::qHash(int(fo)); }
 
 namespace Internal {
 
@@ -164,7 +165,7 @@ FilePath QmakePriFile::directoryPath() const
 
 QString QmakePriFile::displayName() const
 {
-    return filePath().toFileInfo().completeBaseName();
+    return filePath().completeBaseName();
 }
 
 QmakePriFile *QmakePriFile::parent() const
@@ -495,92 +496,88 @@ void QmakePriFile::setIncludedInExactParse(bool b)
     m_includedInExactParse = b;
 }
 
-bool QmakePriFile::canAddSubProject(const QString &proFilePath) const
+bool QmakePriFile::canAddSubProject(const FilePath &proFilePath) const
 {
-    QFileInfo fi(proFilePath);
-    if (fi.suffix() == QLatin1String("pro")
-        || fi.suffix() == QLatin1String("pri"))
-        return true;
-    return false;
+    return proFilePath.suffix() == "pro" || proFilePath.suffix() == "pri";
 }
 
-static QString simplifyProFilePath(const QString &proFilePath)
+static FilePath simplifyProFilePath(const FilePath &proFilePath)
 {
     // if proFilePath is like: _path_/projectName/projectName.pro
     // we simplify it to: _path_/projectName
-    QFileInfo fi(proFilePath);
+    QFileInfo fi = proFilePath.toFileInfo(); // FIXME
     const QString parentPath = fi.absolutePath();
     QFileInfo parentFi(parentPath);
     if (parentFi.fileName() == fi.completeBaseName())
-        return parentPath;
+        return FilePath::fromString(parentPath);
     return proFilePath;
 }
 
-bool QmakePriFile::addSubProject(const QString &proFile)
+bool QmakePriFile::addSubProject(const FilePath &proFile)
 {
-    QStringList uniqueProFilePaths;
-    if (!m_recursiveEnumerateFiles.contains(FilePath::fromString(proFile)))
+    FilePaths uniqueProFilePaths;
+    if (!m_recursiveEnumerateFiles.contains(proFile))
         uniqueProFilePaths.append(simplifyProFilePath(proFile));
 
-    QStringList failedFiles;
+    FilePaths failedFiles;
     changeFiles(QLatin1String(Constants::PROFILE_MIMETYPE), uniqueProFilePaths, &failedFiles, AddToProFile);
 
     return failedFiles.isEmpty();
 }
 
-bool QmakePriFile::removeSubProjects(const QString &proFilePath)
+bool QmakePriFile::removeSubProjects(const FilePath &proFilePath)
 {
-    QStringList failedOriginalFiles;
-    changeFiles(QLatin1String(Constants::PROFILE_MIMETYPE), QStringList(proFilePath), &failedOriginalFiles, RemoveFromProFile);
+    FilePaths failedOriginalFiles;
+    changeFiles(QLatin1String(Constants::PROFILE_MIMETYPE), {proFilePath}, &failedOriginalFiles, RemoveFromProFile);
 
-    QStringList simplifiedProFiles = Utils::transform(failedOriginalFiles, &simplifyProFilePath);
+    FilePaths simplifiedProFiles = Utils::transform(failedOriginalFiles, &simplifyProFilePath);
 
-    QStringList failedSimplifiedFiles;
+    FilePaths failedSimplifiedFiles;
     changeFiles(QLatin1String(Constants::PROFILE_MIMETYPE), simplifiedProFiles, &failedSimplifiedFiles, RemoveFromProFile);
 
     return failedSimplifiedFiles.isEmpty();
 }
 
-bool QmakePriFile::addFiles(const QStringList &filePaths, QStringList *notAdded)
+bool QmakePriFile::addFiles(const FilePaths &filePaths, FilePaths *notAdded)
 {
     // If a file is already referenced in the .pro file then we don't add them.
     // That ignores scopes and which variable was used to reference the file
     // So it's obviously a bit limited, but in those cases you need to edit the
     // project files manually anyway.
 
-    using TypeFileMap = QMap<QString, QStringList>;
+    using TypeFileMap = QMap<QString, FilePaths>;
     // Split into lists by file type and bulk-add them.
     TypeFileMap typeFileMap;
-    foreach (const QString &file, filePaths) {
+    for (const FilePath &file : filePaths) {
         const Utils::MimeType mt = Utils::mimeTypeForFile(file);
         typeFileMap[mt.name()] << file;
     }
 
-    QStringList failedFiles;
+    FilePaths failedFiles;
     foreach (const QString &type, typeFileMap.keys()) {
-        const QStringList typeFiles = typeFileMap.value(type);
-        QStringList qrcFiles; // the list of qrc files referenced from ui files
+        const FilePaths typeFiles = typeFileMap.value(type);
+        FilePaths qrcFiles; // the list of qrc files referenced from ui files
         if (type == QLatin1String(ProjectExplorer::Constants::RESOURCE_MIMETYPE)) {
-            foreach (const QString &formFile, typeFiles) {
-                QStringList resourceFiles = formResources(formFile);
-                foreach (const QString &resourceFile, resourceFiles)
+            for (const FilePath &formFile : typeFiles) {
+                const FilePaths resourceFiles = formResources(formFile);
+                for (const FilePath &resourceFile : resourceFiles)
                     if (!qrcFiles.contains(resourceFile))
                         qrcFiles.append(resourceFile);
             }
         }
 
-        QStringList uniqueQrcFiles;
-        foreach (const QString &file, qrcFiles) {
-            if (!m_recursiveEnumerateFiles.contains(FilePath::fromString(file)))
+        FilePaths uniqueQrcFiles;
+        for (const FilePath &file : qAsConst(qrcFiles)) {
+            if (!m_recursiveEnumerateFiles.contains(file))
                 uniqueQrcFiles.append(file);
         }
 
-        QStringList uniqueFilePaths;
-        foreach (const QString &file, typeFiles) {
-            if (!m_recursiveEnumerateFiles.contains(FilePath::fromString(file)))
+        FilePaths uniqueFilePaths;
+        for (const FilePath &file : typeFiles) {
+            if (!m_recursiveEnumerateFiles.contains(file))
                 uniqueFilePaths.append(file);
         }
-        uniqueFilePaths.sort();
+        FilePath::sort(uniqueFilePaths);
 
         changeFiles(type, uniqueFilePaths, &failedFiles, AddToProFile);
         if (notAdded)
@@ -592,19 +589,18 @@ bool QmakePriFile::addFiles(const QStringList &filePaths, QStringList *notAdded)
     return failedFiles.isEmpty();
 }
 
-bool QmakePriFile::removeFiles(const QStringList &filePaths,
-                              QStringList *notRemoved)
+bool QmakePriFile::removeFiles(const FilePaths &filePaths, FilePaths *notRemoved)
 {
-    QStringList failedFiles;
-    using TypeFileMap = QMap<QString, QStringList>;
+    FilePaths failedFiles;
+    using TypeFileMap = QMap<QString, FilePaths>;
     // Split into lists by file type and bulk-add them.
     TypeFileMap typeFileMap;
-    foreach (const QString &file, filePaths) {
+    for (const FilePath &file : filePaths) {
         const Utils::MimeType mt = Utils::mimeTypeForFile(file);
         typeFileMap[mt.name()] << file;
     }
     foreach (const QString &type, typeFileMap.keys()) {
-        const QStringList typeFiles = typeFileMap.value(type);
+        const FilePaths typeFiles = typeFileMap.value(type);
         changeFiles(type, typeFiles, &failedFiles, RemoveFromProFile);
         if (notRemoved)
             *notRemoved = failedFiles;
@@ -612,31 +608,31 @@ bool QmakePriFile::removeFiles(const QStringList &filePaths,
     return failedFiles.isEmpty();
 }
 
-bool QmakePriFile::deleteFiles(const QStringList &filePaths)
+bool QmakePriFile::deleteFiles(const FilePaths &filePaths)
 {
     removeFiles(filePaths);
     return true;
 }
 
-bool QmakePriFile::canRenameFile(const QString &filePath, const QString &newFilePath)
+bool QmakePriFile::canRenameFile(const FilePath &oldFilePath, const FilePath &newFilePath)
 {
     if (newFilePath.isEmpty())
         return false;
 
-    bool changeProFileOptional = deploysFolder(QFileInfo(filePath).absolutePath());
+    bool changeProFileOptional = deploysFolder(oldFilePath.absolutePath().toString());
     if (changeProFileOptional)
         return true;
 
-    return renameFile(filePath, newFilePath, Change::TestOnly);
+    return renameFile(oldFilePath, newFilePath, Change::TestOnly);
 }
 
-bool QmakePriFile::renameFile(const QString &filePath, const QString &newFilePath)
+bool QmakePriFile::renameFile(const FilePath &oldFilePath, const FilePath &newFilePath)
 {
     if (newFilePath.isEmpty())
         return false;
 
-    bool changeProFileOptional = deploysFolder(QFileInfo(filePath).absolutePath());
-    if (renameFile(filePath, newFilePath, Change::Save))
+    bool changeProFileOptional = deploysFolder(oldFilePath.absolutePath().toString());
+    if (renameFile(oldFilePath, newFilePath, Change::Save))
         return true;
     return changeProFileOptional;
 }
@@ -704,16 +700,16 @@ bool QmakePriFile::saveModifiedEditors()
     return true;
 }
 
-QStringList QmakePriFile::formResources(const QString &formFile) const
+FilePaths QmakePriFile::formResources(const FilePath &formFile) const
 {
     QStringList resourceFiles;
-    QFile file(formFile);
+    QFile file(formFile.toString());
     if (!file.open(QIODevice::ReadOnly))
-        return resourceFiles;
+        return {};
 
     QXmlStreamReader reader(&file);
 
-    QFileInfo fi(formFile);
+    QFileInfo fi(formFile.toString());
     QDir formDir = fi.absoluteDir();
     while (!reader.atEnd()) {
         reader.readNext();
@@ -736,7 +732,7 @@ QStringList QmakePriFile::formResources(const QString &formFile) const
     if (reader.hasError())
         qWarning() << "Could not read form file:" << formFile;
 
-    return resourceFiles;
+    return Utils::transform(resourceFiles, &FilePath::fromString);
 }
 
 bool QmakePriFile::ensureWriteableProFile(const QString &file)
@@ -745,8 +741,9 @@ bool QmakePriFile::ensureWriteableProFile(const QString &file)
     QFileInfo fi(file);
     if (!fi.isWritable()) {
         // Try via vcs manager
-        Core::IVersionControl *versionControl = Core::VcsManager::findVersionControlForDirectory(fi.absolutePath());
-        if (!versionControl || !versionControl->vcsOpen(file)) {
+        Core::IVersionControl *versionControl =
+            Core::VcsManager::findVersionControlForDirectory(FilePath::fromString(fi.absolutePath()));
+        if (!versionControl || !versionControl->vcsOpen(FilePath::fromString(file))) {
             bool makeWritable = QFile::setPermissions(file, fi.permissions() | QFile::WriteUser);
             if (!makeWritable) {
                 QMessageBox::warning(Core::ICore::dialogParent(),
@@ -769,12 +766,12 @@ QPair<ProFile *, QStringList> QmakePriFile::readProFile()
         QString contents;
         {
             QString errorMsg;
-            if (TextFileFormat::readFile(
-                        filePath().toString(),
-                        Core::EditorManager::defaultTextCodec(),
-                        &contents,
-                        &m_textFormat,
-                        &errorMsg) != TextFileFormat::ReadSuccess) {
+            if (TextFileFormat::readFile(filePath(),
+                                         Core::EditorManager::defaultTextCodec(),
+                                         &contents,
+                                         &m_textFormat,
+                                         &errorMsg)
+                != TextFileFormat::ReadSuccess) {
                 QmakeBuildSystem::proFileParseError(errorMsg, filePath());
                 return qMakePair(includeFile, lines);
             }
@@ -797,7 +794,7 @@ bool QmakePriFile::prepareForChange()
     return saveModifiedEditors() && ensureWriteableProFile(filePath().toString());
 }
 
-bool QmakePriFile::renameFile(const QString &oldName, const QString &newName, Change mode)
+bool QmakePriFile::renameFile(const FilePath &oldFilePath, const FilePath &newFilePath, Change mode)
 {
     if (!prepareForChange())
         return false;
@@ -811,14 +808,12 @@ bool QmakePriFile::renameFile(const QString &oldName, const QString &newName, Ch
 
     QDir priFileDir = QDir(m_qmakeProFile->directoryPath().toString());
     ProWriter::VarLocations removedLocations;
-    const QStringList notChanged = ProWriter::removeFiles(
-                includeFile,
-                &lines,
-                priFileDir,
-                QStringList(oldName),
-                varNamesForRemoving(),
-                &removedLocations
-                );
+    const QStringList notChanged = ProWriter::removeFiles(includeFile,
+                                                          &lines,
+                                                          priFileDir,
+                                                          {oldFilePath.toString()},
+                                                          varNamesForRemoving(),
+                                                          &removedLocations);
 
     includeFile->deref();
     if (!notChanged.isEmpty())
@@ -827,7 +822,7 @@ bool QmakePriFile::renameFile(const QString &oldName, const QString &newName, Ch
 
     int endLine = lines.count();
     reverseForeach(removedLocations,
-                   [this, &newName, &lines, &endLine](const ProWriter::VarLocation &loc) {
+                   [this, &newFilePath, &lines, &endLine](const ProWriter::VarLocation &loc) {
         QStringList currentLines = lines.mid(loc.second, endLine - loc.second);
         const QString currentContents = currentLines.join('\n');
 
@@ -840,7 +835,11 @@ bool QmakePriFile::renameFile(const QString &oldName, const QString &newName, Ch
                                                        QMakeParser::FullGrammar);
         QTC_ASSERT(proFile, return); // The file should still be valid after what we did.
 
-        ProWriter::addFiles(proFile, &currentLines, {newName}, loc.first, continuationIndent());
+        ProWriter::addFiles(proFile,
+                            &currentLines,
+                            {newFilePath.toString()},
+                            loc.first,
+                            continuationIndent());
         lines = lines.mid(0, loc.second) + currentLines + lines.mid(endLine);
         endLine = loc.second;
         proFile->deref();
@@ -852,9 +851,9 @@ bool QmakePriFile::renameFile(const QString &oldName, const QString &newName, Ch
 }
 
 void QmakePriFile::changeFiles(const QString &mimeType,
-                                 const QStringList &filePaths,
-                                 QStringList *notChanged,
-                                 ChangeType change, Change mode)
+                               const FilePaths &filePaths,
+                               FilePaths *notChanged,
+                               ChangeType change, Change mode)
 {
     if (filePaths.isEmpty())
         return;
@@ -876,12 +875,18 @@ void QmakePriFile::changeFiles(const QString &mimeType,
                            << filePaths << "change type:" << int(change) << "mode:" << int(mode);
     if (change == AddToProFile) {
         // Use the first variable for adding.
-        ProWriter::addFiles(includeFile, &lines, filePaths, varNameForAdding(mimeType),
+        ProWriter::addFiles(includeFile, &lines,
+                            Utils::transform(filePaths, &FilePath::toString),
+                            varNameForAdding(mimeType),
                             continuationIndent());
         notChanged->clear();
     } else { // RemoveFromProFile
         QDir priFileDir = QDir(m_qmakeProFile->directoryPath().toString());
-        *notChanged = ProWriter::removeFiles(includeFile, &lines, priFileDir, filePaths, varNamesForRemoving());
+        *notChanged = Utils::transform(
+                    ProWriter::removeFiles(includeFile, &lines, priFileDir,
+                                           Utils::transform(filePaths, &FilePath::toString),
+                                           varNamesForRemoving()),
+                    &FilePath::fromString);
     }
 
     // save file
@@ -929,9 +934,9 @@ void QmakePriFile::save(const QStringList &lines)
 {
     {
         QTC_ASSERT(m_textFormat.codec, return);
-        FileChangeBlocker changeGuard(filePath().toString());
+        FileChangeBlocker changeGuard(filePath());
         QString errorMsg;
-        if (!m_textFormat.writeFile(filePath().toString(), lines.join('\n'), &errorMsg)) {
+        if (!m_textFormat.writeFile(filePath(), lines.join('\n'), &errorMsg)) {
             QMessageBox::critical(Core::ICore::dialogParent(), QCoreApplication::translate(
                                       "QmakePriFile", "File Error"), errorMsg);
         }
@@ -1018,7 +1023,7 @@ QString QmakePriFile::varNameForAdding(const QString &mimeType)
     }
 
     if (mimeType == QLatin1String(ProjectExplorer::Constants::CPP_SOURCE_MIMETYPE)
-               || mimeType == QLatin1String(CppTools::Constants::OBJECTIVE_CPP_SOURCE_MIMETYPE)
+               || mimeType == QLatin1String(CppEditor::Constants::OBJECTIVE_CPP_SOURCE_MIMETYPE)
                || mimeType == QLatin1String(ProjectExplorer::Constants::C_SOURCE_MIMETYPE)) {
         return QLatin1String("SOURCES");
     }
@@ -1139,9 +1144,9 @@ QByteArray QmakeProFile::cxxDefines() const
     QByteArray result;
     foreach (const QString &def, variableValue(Variable::Defines)) {
         // 'def' is shell input, so interpret it.
-        QtcProcess::SplitError error = QtcProcess::SplitOk;
-        const QStringList args = QtcProcess::splitArgs(def, HostOsInfo::hostOs(), false, &error);
-        if (error != QtcProcess::SplitOk || args.size() == 0)
+        ProcessArgs::SplitError error = ProcessArgs::SplitOk;
+        const QStringList args = ProcessArgs::splitArgs(def, HostOsInfo::hostOs(), false, &error);
+        if (error != ProcessArgs::SplitOk || args.size() == 0)
             continue;
 
         result += "#define ";
@@ -1181,7 +1186,7 @@ QmakeProFile::~QmakeProFile()
         m_parseFutureWatcher->cancel();
         m_parseFutureWatcher->waitForFinished();
         if (m_readerExact)
-            applyAsyncEvaluate();
+            applyAsyncEvaluate(false);
         delete m_parseFutureWatcher;
     }
     cleanupProFileReaders();
@@ -1189,9 +1194,10 @@ QmakeProFile::~QmakeProFile()
 
 void QmakeProFile::setupFutureWatcher()
 {
-    m_parseFutureWatcher = new QFutureWatcher<Internal::QmakeEvalResult *>;
-    QObject::connect(m_parseFutureWatcher, &QFutureWatcherBase::finished,
-                     [this](){ applyAsyncEvaluate(); });
+    m_parseFutureWatcher = new QFutureWatcher<Internal::QmakeEvalResultPtr>;
+    QObject::connect(m_parseFutureWatcher, &QFutureWatcherBase::finished, [this]() {
+        applyAsyncEvaluate(true);
+    });
 }
 
 bool QmakeProFile::isParent(QmakeProFile *node)
@@ -1213,7 +1219,7 @@ QString QmakeProFile::displayName() const
 QList<QmakeProFile *> QmakeProFile::allProFiles()
 {
     QList<QmakeProFile *> result = { this };
-    for (QmakePriFile *c : m_children) {
+    for (QmakePriFile *c : qAsConst(m_children)) {
         auto proC = dynamic_cast<QmakeProFile *>(c);
         if (proC)
             result.append(proC->allProFiles());
@@ -1286,10 +1292,10 @@ void QmakeProFile::asyncUpdate()
         m_readerExact->setExact(false);
     m_parseFutureWatcher->waitForFinished();
     QmakeEvalInput input = evalInput();
-    QFuture<QmakeEvalResult *> future = Utils::runAsync(ProjectExplorerPlugin::sharedThreadPool(),
-                                                        QThread::LowestPriority,
-                                                        &QmakeProFile::asyncEvaluate,
-                                                        this, input);
+    QFuture<QmakeEvalResultPtr> future = Utils::runAsync(ProjectExplorerPlugin::sharedThreadPool(),
+                                                         QThread::LowestPriority,
+                                                         &QmakeProFile::asyncEvaluate,
+                                                         this, input);
     m_parseFutureWatcher->setFuture(future);
 }
 
@@ -1352,7 +1358,10 @@ static bool evaluateOne(const QmakeEvalInput &input, ProFile *pro,
 
         // We don't increase/decrease m_qmakeGlobalsRefCnt here, because the outer profilereaders keep m_qmakeGlobals alive anyway
         auto bpReader = new QtSupport::ProFileReader(input.qmakeGlobals, input.qmakeVfs); // needs to access m_qmakeGlobals, m_qmakeVfs
-        bpReader->setOutputDir(input.buildDirectory.toString());
+
+        // FIXME: Currently intentional.
+        // Core parts of the ProParser hard-assert on non-local items.
+        bpReader->setOutputDir(input.buildDirectory.path());
         bpReader->setCumulative(cumulative);
         bpReader->setExtraVars(basevars);
         bpReader->setExtraConfigs(basecfgs);
@@ -1366,9 +1375,9 @@ static bool evaluateOne(const QmakeEvalInput &input, ProFile *pro,
     return true;
 }
 
-QmakeEvalResult *QmakeProFile::evaluate(const QmakeEvalInput &input)
+QmakeEvalResultPtr QmakeProFile::evaluate(const QmakeEvalInput &input)
 {
-    auto *result = new QmakeEvalResult;
+    QmakeEvalResultPtr result(new QmakeEvalResult);
     QtSupport::ProFileReader *exactBuildPassReader = nullptr;
     QtSupport::ProFileReader *cumulativeBuildPassReader = nullptr;
     ProFile *pro;
@@ -1553,12 +1562,12 @@ QmakeEvalResult *QmakeProFile::evaluate(const QmakeEvalInput &input)
                 = exactReader->values(QLatin1String("TARGET_VERSION_EXT"));
         result->newVarValues[Variable::StaticLibExtension] = exactReader->values(QLatin1String("QMAKE_EXTENSION_STATICLIB"));
         result->newVarValues[Variable::ShLibExtension] = exactReader->values(QLatin1String("QMAKE_EXTENSION_SHLIB"));
-        result->newVarValues[Variable::AndroidArch] = exactReader->values(QLatin1String("ANDROID_TARGET_ARCH"));
-        result->newVarValues[Variable::AndroidDeploySettingsFile] = exactReader->values(QLatin1String("ANDROID_DEPLOYMENT_SETTINGS_FILE"));
-        result->newVarValues[Variable::AndroidPackageSourceDir] = exactReader->values(QLatin1String("ANDROID_PACKAGE_SOURCE_DIR"));
-        result->newVarValues[Variable::AndroidAbis] = exactReader->values(QLatin1String("ANDROID_ABIS"));
-        result->newVarValues[Variable::AndroidApplicationArguments] = exactReader->values(QLatin1String("ANDROID_APPLICATION_ARGUMENTS"));
-        result->newVarValues[Variable::AndroidExtraLibs] = exactReader->values(QLatin1String("ANDROID_EXTRA_LIBS"));
+        result->newVarValues[Variable::AndroidAbi] = exactReader->values(QLatin1String(Android::Constants::ANDROID_TARGET_ARCH));
+        result->newVarValues[Variable::AndroidDeploySettingsFile] = exactReader->values(QLatin1String(Android::Constants::ANDROID_DEPLOYMENT_SETTINGS_FILE));
+        result->newVarValues[Variable::AndroidPackageSourceDir] = exactReader->values(QLatin1String(Android::Constants::ANDROID_PACKAGE_SOURCE_DIR));
+        result->newVarValues[Variable::AndroidAbis] = exactReader->values(QLatin1String(Android::Constants::ANDROID_ABIS));
+        result->newVarValues[Variable::AndroidApplicationArgs] = exactReader->values(QLatin1String(Android::Constants::ANDROID_APPLICATION_ARGUMENTS));
+        result->newVarValues[Variable::AndroidExtraLibs] = exactReader->values(QLatin1String(Android::Constants::ANDROID_EXTRA_LIBS));
         result->newVarValues[Variable::AppmanPackageDir] = exactReader->values(QLatin1String("AM_PACKAGE_DIR"));
         result->newVarValues[Variable::AppmanManifest] = exactReader->values(QLatin1String("AM_MANIFEST"));
         result->newVarValues[Variable::IsoIcons] = exactReader->values(QLatin1String("ISO_ICONS"));
@@ -1590,7 +1599,7 @@ QmakeEvalResult *QmakeProFile::evaluate(const QmakeEvalInput &input)
         toCompare.pop_front();
 
         // Loop prevention: Make sure that exact same node is not in our parent chain
-        for (QmakeIncludedPriFile *priFile : tree->children) {
+        for (QmakeIncludedPriFile *priFile : qAsConst(tree->children)) {
             bool loop = input.parentFilePaths.contains(priFile->name);
             for (const QmakePriFile *n = pn; n && !loop; n = n->parent()) {
                 if (n->filePath() == priFile->name)
@@ -1626,15 +1635,14 @@ QmakeEvalResult *QmakeProFile::evaluate(const QmakeEvalInput &input)
     return result;
 }
 
-void QmakeProFile::asyncEvaluate(QFutureInterface<QmakeEvalResult *> &fi, QmakeEvalInput input)
+void QmakeProFile::asyncEvaluate(QFutureInterface<QmakeEvalResultPtr> &fi, QmakeEvalInput input)
 {
-    QmakeEvalResult *evalResult = evaluate(input);
-    fi.reportResult(evalResult);
+    fi.reportResult(evaluate(input));
 }
 
-void QmakeProFile::applyAsyncEvaluate()
+void QmakeProFile::applyAsyncEvaluate(bool apply)
 {
-    if (m_parseFutureWatcher->isFinished())
+    if (apply)
         applyEvaluate(m_parseFutureWatcher->result());
     m_buildSystem->decrementPendingEvaluateFutures();
 }
@@ -1644,9 +1652,8 @@ bool sortByParserNodes(Node *a, Node *b)
     return a->filePath() < b->filePath();
 }
 
-void QmakeProFile::applyEvaluate(QmakeEvalResult *evalResult)
+void QmakeProFile::applyEvaluate(const QmakeEvalResultPtr &result)
 {
-    QScopedPointer<QmakeEvalResult> result(evalResult);
     if (!m_readerExact)
         return;
 
@@ -1655,7 +1662,7 @@ void QmakeProFile::applyEvaluate(QmakeEvalResult *evalResult)
         return;
     }
 
-    foreach (const QString &error, evalResult->errors)
+    foreach (const QString &error, result->errors)
         QmakeBuildSystem::proFileParseError(error, filePath());
 
     // we are changing what is executed in that case
@@ -1967,7 +1974,7 @@ TargetInformation QmakeProFile::targetInformation(QtSupport::ProFileReader *read
     // Target
     result.target = readerBuildPass->value(QLatin1String("TARGET"));
     if (result.target.isEmpty())
-        result.target = projectFilePath.toFileInfo().baseName();
+        result.target = projectFilePath.baseName();
 
     result.valid = true;
 
@@ -2067,13 +2074,13 @@ FilePaths QmakeProFile::generatedFiles(const FilePath &buildDir,
         if (location.isEmpty())
             return { };
         location = location.pathAppended("ui_"
-                                         + sourceFile.toFileInfo().completeBaseName()
+                                         + sourceFile.completeBaseName()
                                          + singleVariableValue(Variable::HeaderExtension));
         return { Utils::FilePath::fromString(QDir::cleanPath(location.toString())) };
     } else if (sourceFileType == FileType::StateChart) {
         if (buildDir.isEmpty())
             return { };
-        const FilePath location = buildDir.pathAppended(sourceFile.toFileInfo().completeBaseName());
+        const FilePath location = buildDir.pathAppended(sourceFile.completeBaseName());
         return {
             location.stringAppended(singleVariableValue(Variable::HeaderExtension)),
             location.stringAppended(singleVariableValue(Variable::CppExtension))
