@@ -30,7 +30,8 @@
 #include "proparser/profileevaluator.h"
 
 #include <coreplugin/idocument.h>
-#include <cpptools/generatedcodemodelsupport.h>
+#include <cppeditor/generatedcodemodelsupport.h>
+#include <utils/porting.h>
 #include <utils/textfileformat.h>
 
 #include <QFutureWatcher>
@@ -96,12 +97,12 @@ enum class Variable {
     TargetVersionExt,
     StaticLibExtension,
     ShLibExtension,
-    AndroidArch,
-    AndroidDeploySettingsFile,
+    AndroidAbi,
     AndroidAbis,
+    AndroidDeploySettingsFile,
     AndroidPackageSourceDir,
     AndroidExtraLibs,
-    AndroidApplicationArguments,
+    AndroidApplicationArgs,
     AppmanPackageDir,
     AppmanManifest,
     IsoIcons,
@@ -109,19 +110,20 @@ enum class Variable {
     QmakeCc,
     QmakeCxx
 };
-uint qHash(Variable key, uint seed = 0);
+Utils::QHashValueType qHash(Variable key, uint seed = 0);
 
 namespace Internal {
 Q_DECLARE_LOGGING_CATEGORY(qmakeNodesLog)
 class QmakeEvalInput;
 class QmakeEvalResult;
+using QmakeEvalResultPtr = std::shared_ptr<QmakeEvalResult>; // FIXME: Use unique_ptr once we require Qt 6
 class QmakePriFileEvalResult;
 } // namespace Internal;
 
 class InstallsList;
 
 enum class FileOrigin { ExactParse, CumulativeParse };
-uint qHash(FileOrigin fo);
+Utils::QHashValueType qHash(FileOrigin fo);
 using SourceFile = QPair<Utils::FilePath, FileOrigin>;
 using SourceFiles = QSet<SourceFile>;
 
@@ -157,16 +159,16 @@ public:
 
     void update(const Internal::QmakePriFileEvalResult &result);
 
-    bool canAddSubProject(const QString &proFilePath) const;
+    bool canAddSubProject(const Utils::FilePath &proFilePath) const;
 
-    bool addSubProject(const QString &proFile);
-    bool removeSubProjects(const QString &proFilePath);
+    bool addSubProject(const Utils::FilePath &proFile);
+    bool removeSubProjects(const Utils::FilePath &proFilePath);
 
-    bool addFiles(const QStringList &filePaths, QStringList *notAdded = nullptr);
-    bool removeFiles(const QStringList &filePaths, QStringList *notRemoved = nullptr);
-    bool deleteFiles(const QStringList &filePaths);
-    bool canRenameFile(const QString &filePath, const QString &newFilePath);
-    bool renameFile(const QString &filePath, const QString &newFilePath);
+    bool addFiles(const Utils::FilePaths &filePaths, Utils::FilePaths *notAdded = nullptr);
+    bool removeFiles(const Utils::FilePaths &filePaths, Utils::FilePaths *notRemoved = nullptr);
+    bool deleteFiles(const Utils::FilePaths &filePaths);
+    bool canRenameFile(const Utils::FilePath &oldFilePath, const Utils::FilePath &newFilePath);
+    bool renameFile(const Utils::FilePath &oldFilePath, const Utils::FilePath &newFilePath);
     bool addDependencies(const QStringList &dependencies);
 
     bool setProVariable(const QString &var, const QStringList &values,
@@ -203,10 +205,10 @@ protected:
     };
 
     enum class Change { Save, TestOnly };
-    bool renameFile(const QString &oldName, const QString &newName, Change mode);
+    bool renameFile(const Utils::FilePath &oldFilePath, const Utils::FilePath &newFilePath, Change mode);
     void changeFiles(const QString &mimeType,
-                     const QStringList &filePaths,
-                     QStringList *notChanged,
+                     const Utils::FilePaths &filePaths,
+                     Utils::FilePaths *notChanged,
                      ChangeType change,
                      Change mode = Change::Save);
 
@@ -221,7 +223,7 @@ private:
     static QPair<ProFile *, QStringList> readProFileFromContents(const QString &contents);
     void save(const QStringList &lines);
     bool saveModifiedEditors();
-    QStringList formResources(const QString &formFile) const;
+    Utils::FilePaths formResources(const Utils::FilePath &formFile) const;
     static QStringList baseVPaths(QtSupport::ProFileReader *reader, const QString &projectDir, const QString &buildDir);
     static QStringList fullVPaths(const QStringList &baseVPaths, QtSupport::ProFileReader *reader, const QString &qmakeVariable, const QString &projectDir);
     static void extractSources(QHash<int, Internal::QmakePriFileEvalResult *> proToResult,
@@ -352,15 +354,16 @@ private:
     void setParseInProgress(bool b);
     void setValidParseRecursive(bool b);
 
-    void applyAsyncEvaluate();
+    void applyAsyncEvaluate(bool apply);
 
     void setupReader();
     Internal::QmakeEvalInput evalInput() const;
 
-    static Internal::QmakeEvalResult *evaluate(const Internal::QmakeEvalInput &input);
-    void applyEvaluate(Internal::QmakeEvalResult *parseResult);
+    static Internal::QmakeEvalResultPtr evaluate(const Internal::QmakeEvalInput &input);
+    void applyEvaluate(const Internal::QmakeEvalResultPtr &parseResult);
 
-    void asyncEvaluate(QFutureInterface<Internal::QmakeEvalResult *> &fi, Internal::QmakeEvalInput input);
+    void asyncEvaluate(QFutureInterface<Internal::QmakeEvalResultPtr> &fi,
+                       Internal::QmakeEvalInput input);
     void cleanupProFileReaders();
 
     void updateGeneratedFiles(const Utils::FilePath &buildDir);
@@ -399,7 +402,7 @@ private:
     QMap<QString, QStringList> m_wildcardDirectoryContents;
 
     // Async stuff
-    QFutureWatcher<Internal::QmakeEvalResult *> *m_parseFutureWatcher = nullptr;
+    QFutureWatcher<Internal::QmakeEvalResultPtr> *m_parseFutureWatcher = nullptr;
     QtSupport::ProFileReader *m_readerExact = nullptr;
     QtSupport::ProFileReader *m_readerCumulative = nullptr;
 };
