@@ -38,10 +38,10 @@
 #include <coreplugin/icore.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cppprojectupdater.h>
-#include <cpptools/generatedcodemodelsupport.h>
-#include <cpptools/projectinfo.h>
+#include <cppeditor/cppmodelmanager.h>
+#include <cppeditor/cppprojectupdater.h>
+#include <cppeditor/generatedcodemodelsupport.h>
+#include <cppeditor/projectinfo.h>
 
 #include <projectexplorer/buildinfo.h>
 #include <projectexplorer/buildmanager.h>
@@ -121,9 +121,9 @@ public:
     {
         Q_UNUSED(errorString)
         Q_UNUSED(flag)
-        if (type == TypePermissions)
-            return true;
-        m_priFile->scheduleUpdate();
+        Q_UNUSED(type)
+        if (m_priFile)
+            m_priFile->scheduleUpdate();
         return true;
     }
 
@@ -174,7 +174,7 @@ QmakeProject::QmakeProject(const FilePath &fileName) :
 {
     setId(Constants::QMAKEPROJECT_ID);
     setProjectLanguages(Core::Context(ProjectExplorer::Constants::CXX_LANGUAGE_ID));
-    setDisplayName(fileName.toFileInfo().completeBaseName());
+    setDisplayName(fileName.completeBaseName());
     setCanBuildProducts();
     setHasMakeInstallEquivalent(true);
 }
@@ -219,7 +219,7 @@ DeploymentKnowledge QmakeProject::deploymentKnowledge() const
 QmakeBuildSystem::QmakeBuildSystem(QmakeBuildConfiguration *bc)
     : BuildSystem(bc)
     , m_qmakeVfs(new QMakeVfs)
-    , m_cppCodeModelUpdater(new CppTools::CppProjectUpdater)
+    , m_cppCodeModelUpdater(new CppEditor::CppProjectUpdater)
 {
     setParseDelay(0);
 
@@ -284,8 +284,11 @@ QmakeBuildSystem::~QmakeBuildSystem()
     delete m_qmakeVfs;
     m_qmakeVfs = nullptr;
 
-    m_asyncUpdateFutureInterface.reportCanceled();
-    m_asyncUpdateFutureInterface.reportFinished();
+    if (m_asyncUpdateFutureInterface) {
+        m_asyncUpdateFutureInterface->reportCanceled();
+        m_asyncUpdateFutureInterface->reportFinished();
+        m_asyncUpdateFutureInterface.reset();
+    }
 }
 
 void QmakeBuildSystem::updateCodeModels()
@@ -352,8 +355,11 @@ void QmakeBuildSystem::updateCppCodeModel()
             rpp.setBuildTargetType(BuildTargetType::Unknown);
             break;
         }
-        rpp.setFlagsForCxx({kitInfo.cxxToolChain, pro->variableValue(Variable::CppFlags)});
-        rpp.setFlagsForC({kitInfo.cToolChain, pro->variableValue(Variable::CFlags)});
+        const QString includeFileBaseDir = pro->sourceDir().toString();
+        rpp.setFlagsForCxx({kitInfo.cxxToolChain, pro->variableValue(Variable::CppFlags),
+                            includeFileBaseDir});
+        rpp.setFlagsForC({kitInfo.cToolChain, pro->variableValue(Variable::CFlags),
+                          includeFileBaseDir});
         rpp.setMacros(ProjectExplorer::Macro::toMacros(pro->cxxDefines()));
         rpp.setPreCompiledHeaders(pro->variableValue(Variable::PrecompiledHeader));
         rpp.setSelectedForBuilding(pro->includedInExactParse());
@@ -367,14 +373,13 @@ void QmakeBuildSystem::updateCppCodeModel()
         // Header paths
         ProjectExplorer::HeaderPaths headerPaths;
         foreach (const QString &inc, pro->variableValue(Variable::IncludePath)) {
-            const ProjectExplorer::HeaderPath headerPath{inc, HeaderPathType::User};
+            const auto headerPath = HeaderPath::makeUser(inc);
             if (!headerPaths.contains(headerPath))
                 headerPaths += headerPath;
         }
 
         if (kitInfo.qtVersion && !kitInfo.qtVersion->frameworkPath().isEmpty())
-            headerPaths += {kitInfo.qtVersion->frameworkPath().toString(),
-                            HeaderPathType::Framework};
+            headerPaths += HeaderPath::makeFramework(kitInfo.qtVersion->frameworkPath());
         rpp.setHeaderPaths(headerPaths);
 
         // Files and generators
@@ -387,7 +392,7 @@ void QmakeBuildSystem::updateCppCodeModel()
             });
         }
         generators.append(proGenerators);
-        fileList.prepend(CppTools::CppModelManager::configurationFileName());
+        fileList.prepend(CppEditor::CppModelManager::configurationFileName());
         rpp.setFiles(fileList, [cumulativeSourceFiles](const QString &filePath) {
             // Keep this lambda thread-safe!
             return !cumulativeSourceFiles.contains(filePath);
@@ -396,8 +401,7 @@ void QmakeBuildSystem::updateCppCodeModel()
         rpps.append(rpp);
     }
 
-    CppTools::GeneratedCodeModelSupport::update(generators);
-    m_cppCodeModelUpdater->update({project(), kitInfo, activeParseEnvironment(), rpps});
+    m_cppCodeModelUpdater->update({project(), kitInfo, activeParseEnvironment(), rpps}, generators);
 }
 
 void QmakeBuildSystem::updateQmlJSCodeModel()
@@ -449,8 +453,7 @@ void QmakeBuildSystem::updateQmlJSCodeModel()
     // library then chances of the project being a QML project is quite high.
     // This assumption fails when there are no QDeclarativeEngine/QDeclarativeView (QtQuick 1)
     // or QQmlEngine/QQuickView (QtQuick 2) instances.
-    if (hasQmlLib)
-        project()->addProjectLanguage(ProjectExplorer::Constants::QMLJS_LANGUAGE_ID);
+    project()->setProjectLanguage(ProjectExplorer::Constants::QMLJS_LANGUAGE_ID, hasQmlLib);
 
     projectInfo.activeResourceFiles.removeDuplicates();
     projectInfo.allResourceFiles.removeDuplicates();
@@ -589,8 +592,9 @@ void QmakeBuildSystem::incrementPendingEvaluateFutures()
     }
     ++m_pendingEvaluateFuturesCount;
     TRACE("pending inc to: " << m_pendingEvaluateFuturesCount);
-    m_asyncUpdateFutureInterface.setProgressRange(m_asyncUpdateFutureInterface.progressMinimum(),
-                                                  m_asyncUpdateFutureInterface.progressMaximum() + 1);
+    m_asyncUpdateFutureInterface->setProgressRange(m_asyncUpdateFutureInterface->progressMinimum(),
+                                                   m_asyncUpdateFutureInterface->progressMaximum()
+                                                       + 1);
 }
 
 void QmakeBuildSystem::decrementPendingEvaluateFutures()
@@ -603,15 +607,17 @@ void QmakeBuildSystem::decrementPendingEvaluateFutures()
         return; // We are closing the project!
     }
 
-    m_asyncUpdateFutureInterface.setProgressValue(m_asyncUpdateFutureInterface.progressValue() + 1);
+    m_asyncUpdateFutureInterface->setProgressValue(m_asyncUpdateFutureInterface->progressValue()
+                                                   + 1);
     if (m_pendingEvaluateFuturesCount == 0) {
         // We are done!
         setRootProjectNode(QmakeNodeTreeBuilder::buildTree(this));
 
         if (!m_rootProFile->validParse())
-            m_asyncUpdateFutureInterface.reportCanceled();
+            m_asyncUpdateFutureInterface->reportCanceled();
 
-        m_asyncUpdateFutureInterface.reportFinished();
+        m_asyncUpdateFutureInterface->reportFinished();
+        m_asyncUpdateFutureInterface.reset();
         m_cancelEvaluate = false;
 
         // TODO clear the profile cache ?
@@ -657,12 +663,26 @@ void QmakeBuildSystem::asyncUpdate()
         m_qmakeVfs->invalidateCache();
     }
 
-    m_asyncUpdateFutureInterface.setProgressRange(0, 0);
-    Core::ProgressManager::addTask(m_asyncUpdateFutureInterface.future(),
+    m_asyncUpdateFutureInterface.reset(new QFutureInterface<void>);
+    m_asyncUpdateFutureInterface->setProgressRange(0, 0);
+    Core::ProgressManager::addTask(m_asyncUpdateFutureInterface->future(),
                                    tr("Reading Project \"%1\"").arg(project()->displayName()),
                                    Constants::PROFILE_EVALUATE);
 
-    m_asyncUpdateFutureInterface.reportStarted();
+    m_asyncUpdateFutureInterface->reportStarted();
+    const auto watcher = new QFutureWatcher<void>(this);
+    connect(watcher, &QFutureWatcher<void>::canceled, this, [this, watcher] {
+        if (!m_qmakeGlobals)
+            return;
+        m_qmakeGlobals->killProcesses();
+        watcher->disconnect();
+        watcher->deleteLater();
+    });
+    connect(watcher, &QFutureWatcher<void>::finished, this, [watcher] {
+        watcher->disconnect();
+        watcher->deleteLater();
+    });
+    watcher->setFuture(m_asyncUpdateFutureInterface->future());
 
     const Kit *const k = kit();
     QtSupport::BaseQtVersion *const qtVersion = QtSupport::QtKitAspect::qtVersion(k);
@@ -673,12 +693,32 @@ void QmakeBuildSystem::asyncUpdate()
                       .arg(project()->displayName(), k->displayName())
                 : tr("Cannot parse project \"%1\": No kit selected.").arg(project()->displayName());
         proFileParseError(errorMessage, project()->projectFilePath());
-        m_asyncUpdateFutureInterface.reportCanceled();
-        m_asyncUpdateFutureInterface.reportFinished();
+        m_asyncUpdateFutureInterface->reportCanceled();
+        m_asyncUpdateFutureInterface->reportFinished();
+        m_asyncUpdateFutureInterface.reset();
         return;
     }
 
+    // Make sure we ignore requests for re-evaluation for files whose QmakePriFile objects
+    // will get deleted during the parse.
+    const auto docUpdater = [](Core::IDocument *doc) {
+        static_cast<QmakePriFileDocument *>(doc)->setPriFile(nullptr);
+    };
+    if (m_asyncUpdateState != AsyncFullUpdatePending) {
+        QSet<FilePath> projectFilePaths;
+        for (QmakeProFile * const file : qAsConst(m_partialEvaluate)) {
+            QVector<QmakePriFile *> priFiles = file->children();
+            for (int i = 0; i < priFiles.count(); ++i) {
+                const QmakePriFile * const priFile = priFiles.at(i);
+                projectFilePaths << priFile->filePath();
+                priFiles << priFile->children();
+            }
+        }
+        project()->updateExtraProjectFiles(projectFilePaths, docUpdater);
+    }
+
     if (m_asyncUpdateState == AsyncFullUpdatePending) {
+        project()->updateExtraProjectFiles(docUpdater);
         rootProFile()->asyncUpdate();
     } else {
         foreach (QmakeProFile *file, m_partialEvaluate)
@@ -712,7 +752,7 @@ Tasks QmakeProject::projectIssues(const Kit *k) const
     // that is not the one from the current kit.
     const QList<BaseQtVersion *> qtsContainingThisProject
             = QtVersionManager::versions([filePath = projectFilePath()](const BaseQtVersion *qt) {
-        return qt->isValid() && qt->isSubProject(filePath);
+        return qt->isValid() && qt->isQtSubProject(filePath);
     });
     if (!qtsContainingThisProject.isEmpty()
             && !qtsContainingThisProject.contains(const_cast<BaseQtVersion *>(qtFromKit))) {
@@ -755,11 +795,13 @@ FilePath QmakeBuildSystem::buildDir(const FilePath &proFilePath) const
 {
     const QDir srcDirRoot = QDir(projectDirectory().toString());
     const QString relativeDir = srcDirRoot.relativeFilePath(proFilePath.parentDir().toString());
-    const QString buildConfigBuildDir = buildConfiguration()->buildDirectory().toString();
-    const QString buildDir = buildConfigBuildDir.isEmpty()
-                                 ? projectDirectory().toString()
+    const FilePath buildConfigBuildDir = buildConfiguration()->buildDirectory();
+    FilePath buildDir = buildConfigBuildDir.isEmpty()
+                                 ? projectDirectory()
                                  : buildConfigBuildDir;
-    return FilePath::fromString(QDir::cleanPath(QDir(buildDir).absoluteFilePath(relativeDir)));
+    // FIXME: Convoluted.
+    buildDir.setPath(QDir::cleanPath(QDir(buildDir.path()).absoluteFilePath(relativeDir)));
+    return buildDir;
 }
 
 void QmakeBuildSystem::proFileParseError(const QString &errorMessage, const FilePath &filePath)
@@ -788,7 +830,7 @@ QtSupport::ProFileReader *QmakeBuildSystem::createProFileReader(const QmakeProFi
         m_qmakeSysroot = SysRootKitAspect::sysRoot(k).toString();
 
         if (qtVersion && qtVersion->isValid()) {
-            m_qmakeGlobals->qmake_abslocation = QDir::cleanPath(qtVersion->qmakeCommand().toString());
+            m_qmakeGlobals->qmake_abslocation = QDir::cleanPath(qtVersion->qmakeFilePath().toString());
             qtVersion->applyProperties(m_qmakeGlobals.get());
         }
         m_qmakeGlobals->setDirectories(rootProFile()->sourceDir().toString(),
@@ -821,7 +863,9 @@ QtSupport::ProFileReader *QmakeBuildSystem::createProFileReader(const QmakeProFi
 
     auto reader = new QtSupport::ProFileReader(m_qmakeGlobals.get(), m_qmakeVfs);
 
-    reader->setOutputDir(buildDir(qmakeProFile->filePath()).toString());
+    // FIXME: Currently intentional.
+    // Core parts of the ProParser hard-assert on non-local items
+    reader->setOutputDir(buildDir(qmakeProFile->filePath()).path());
 
     return reader;
 }
@@ -1067,11 +1111,7 @@ void CentralizedFolderWatcher::delayedFolderChanged(const QString &folder)
 void QmakeProject::configureAsExampleProject(Kit *kit)
 {
     QList<BuildInfo> infoList;
-    QList<Kit *> kits;
-    if (kit)
-        kits.append(kit);
-    else
-        kits = KitManager::kits();
+    const QList<Kit *> kits(kit != nullptr ? QList<Kit *>({kit}) : KitManager::kits());
     for (Kit *k : kits) {
         if (QtSupport::QtKitAspect::qtVersion(k) != nullptr) {
             if (auto factory = BuildConfigurationFactory::find(k, projectFilePath()))
@@ -1108,31 +1148,31 @@ void QmakeBuildSystem::updateBuildSystemData()
 
         const QStringList &config = node->variableValue(Variable::Config);
 
-        QString destDir = ti.destDir.toString();
-        QString workingDir;
+        FilePath destDir = ti.destDir;
+        FilePath workingDir;
         if (!destDir.isEmpty()) {
             bool workingDirIsBaseDir = false;
-            if (destDir == ti.buildTarget)
+            if (destDir.path() == ti.buildTarget)
                 workingDirIsBaseDir = true;
-            if (QDir::isRelativePath(destDir))
-                destDir = QDir::cleanPath(ti.buildDir.toString() + '/' + destDir);
+            if (QDir::isRelativePath(destDir.path()))
+                destDir = ti.buildDir / destDir.path();
 
             if (workingDirIsBaseDir)
-                workingDir = ti.buildDir.toString();
+                workingDir = ti.buildDir;
             else
                 workingDir = destDir;
         } else {
-            workingDir = ti.buildDir.toString();
+            workingDir = ti.buildDir;
         }
 
         if (HostOsInfo::isMacHost() && config.contains("app_bundle"))
-            workingDir += '/' + ti.target + ".app/Contents/MacOS";
+            workingDir = workingDir / (ti.target + ".app/Contents/MacOS");
 
         BuildTargetInfo bti;
-        bti.targetFilePath = FilePath::fromString(executableFor(node->proFile()));
+        bti.targetFilePath = executableFor(node->proFile());
         bti.projectFilePath = node->filePath();
-        bti.workingDirectory = FilePath::fromString(workingDir);
-        bti.displayName = bti.projectFilePath.toFileInfo().completeBaseName();
+        bti.workingDirectory = workingDir;
+        bti.displayName = bti.projectFilePath.completeBaseName();
         const FilePath relativePathInProject
                 = bti.projectFilePath.relativeChildPath(projectDirectory());
         if (!relativePathInProject.isEmpty()) {
@@ -1157,7 +1197,7 @@ void QmakeBuildSystem::updateBuildSystemData()
             QmakeProFile *proFile = node->proFile();
             QTC_ASSERT(proFile, return);
             const QString proDirectory = buildDir(proFile->filePath()).toString();
-            foreach (QString dir, libDirectories) {
+            for (QString dir : libDirectories) {
                 // Fix up relative entries like "LIBS+=-L.."
                 const QFileInfo fi(dir);
                 if (!fi.isAbsolute())
@@ -1191,8 +1231,8 @@ void QmakeBuildSystem::collectData(const QmakeProFile *file, DeploymentData &dep
         if (!item.active)
             continue;
         for (const auto &localFile : item.files) {
-            deploymentData.addFile(localFile.fileName, item.path, item.executable
-                                   ? DeployableFile::TypeExecutable : DeployableFile::TypeNormal);
+            deploymentData.addFile(FilePath::fromString(localFile.fileName), item.path,
+               item.executable ? DeployableFile::TypeExecutable : DeployableFile::TypeNormal);
         }
     }
 
@@ -1219,7 +1259,7 @@ void QmakeBuildSystem::collectData(const QmakeProFile *file, DeploymentData &dep
 
 void QmakeBuildSystem::collectApplicationData(const QmakeProFile *file, DeploymentData &deploymentData)
 {
-    QString executable = executableFor(file);
+    const FilePath executable = executableFor(file);
     if (!executable.isEmpty())
         deploymentData.addFile(executable, file->installsList().targetPath,
                                DeployableFile::TypeExecutable);
@@ -1229,8 +1269,8 @@ static FilePath destDirFor(const TargetInformation &ti)
 {
     if (ti.destDir.isEmpty())
         return ti.buildDir;
-    if (QDir::isRelativePath(ti.destDir.toString()))
-        return FilePath::fromString(QDir::cleanPath(ti.buildDir.toString() + '/' + ti.destDir.toString()));
+    if (QDir::isRelativePath(ti.destDir.path()))
+        return ti.buildDir / ti.destDir.path();
     return ti.destDir;
 }
 
@@ -1262,7 +1302,7 @@ void QmakeBuildSystem::collectLibraryData(const QmakeProFile *file, DeploymentDa
         }
         targetFileName += targetVersionExt + QLatin1Char('.');
         targetFileName += QLatin1String(isStatic ? "lib" : "dll");
-        deploymentData.addFile(destDirFor(ti).toString() + '/' + targetFileName, targetPath);
+        deploymentData.addFile(destDirFor(ti) / targetFileName, targetPath);
         break;
     }
     case Abi::DarwinOS: {
@@ -1285,7 +1325,7 @@ void QmakeBuildSystem::collectLibraryData(const QmakeProFile *file, DeploymentDa
             targetFileName += file->singleVariableValue(isStatic
                     ? Variable::StaticLibExtension : Variable::ShLibExtension);
         }
-        deploymentData.addFile(destDir.toString() + '/' + targetFileName, targetPath);
+        deploymentData.addFile(destDir / targetFileName, targetPath);
         break;
     }
     case Abi::LinuxOS:
@@ -1300,7 +1340,7 @@ void QmakeBuildSystem::collectLibraryData(const QmakeProFile *file, DeploymentDa
             targetFileName += QLatin1Char('a');
         } else {
             targetFileName += QLatin1String("so");
-            deploymentData.addFile(destDirFor(ti).toString() + '/' + targetFileName, targetPath);
+            deploymentData.addFile(destDirFor(ti) / targetFileName, targetPath);
             if (nameIsVersioned) {
                 QString version = file->singleVariableValue(Variable::Version);
                 if (version.isEmpty())
@@ -1311,8 +1351,8 @@ void QmakeBuildSystem::collectLibraryData(const QmakeProFile *file, DeploymentDa
                 targetFileName += QLatin1Char('.');
                 while (!versionComponents.isEmpty()) {
                     const QString versionString = versionComponents.join(QLatin1Char('.'));
-                    deploymentData.addFile(destDirFor(ti).toString() + '/'
-                            + targetFileName + versionString, targetPath);
+                    deploymentData.addFile(destDirFor(ti) / targetFileName + versionString,
+                                           targetPath);
                     versionComponents.removeLast();
                 }
             }
@@ -1358,8 +1398,8 @@ void QmakeBuildSystem::testToolChain(ToolChain *tc, const FilePath &path) const
     // Suppress warnings on Apple machines where compilers in /usr/bin point into Xcode.
     // This will suppress some valid warnings, but avoids annoying Apple users with
     // spurious warnings all the time!
-    if (pair.first.toString().startsWith("/usr/bin/")
-            && pair.second.toString().contains("/Contents/Developer/Toolchains/")) {
+    if (pair.first.path().startsWith("/usr/bin/")
+            && pair.second.path().contains("/Contents/Developer/Toolchains/")) {
         return;
     }
     TaskHub::addTask(
@@ -1383,16 +1423,16 @@ void QmakeBuildSystem::warnOnToolChainMismatch(const QmakeProFile *pro) const
                   getFullPathOf(pro, Variable::QmakeCxx, bc));
 }
 
-QString QmakeBuildSystem::executableFor(const QmakeProFile *file)
+FilePath QmakeBuildSystem::executableFor(const QmakeProFile *file)
 {
     const ToolChain *const tc = ToolChainKitAspect::cxxToolChain(kit());
     if (!tc)
-        return QString();
+        return {};
 
     TargetInformation ti = file->targetInformation();
     QString target;
 
-    QTC_ASSERT(file, return QString());
+    QTC_ASSERT(file, return {});
 
     if (tc->targetAbi().os() == Abi::DarwinOS
             && file->variableValue(Variable::Config).contains("app_bundle")) {
@@ -1404,7 +1444,7 @@ QString QmakeBuildSystem::executableFor(const QmakeProFile *file)
         else
             target = ti.target + extension;
     }
-    return QDir(destDirFor(ti).toString()).absoluteFilePath(target);
+    return (destDirFor(ti) / target).absoluteFilePath();
 }
 
 ProjectImporter *QmakeProject::projectImporter() const
@@ -1429,18 +1469,17 @@ void QmakeBuildSystem::triggerParsing()
     asyncUpdate();
 }
 
-QStringList QmakeBuildSystem::filesGeneratedFrom(const QString &input) const
+FilePaths QmakeBuildSystem::filesGeneratedFrom(const FilePath &input) const
 {
     if (!project()->rootProjectNode())
         return {};
 
-    if (const FileNode *file = fileNodeOf(project()->rootProjectNode(), FilePath::fromString(input))) {
+    if (const FileNode *file = fileNodeOf(project()->rootProjectNode(), input)) {
         const QmakeProFileNode *pro = dynamic_cast<QmakeProFileNode *>(file->parentFolderNode());
         QTC_ASSERT(pro, return {});
         if (const QmakeProFile *proFile = pro->proFile())
-            return Utils::transform(proFile->generatedFiles(buildDir(pro->filePath()),
-                                                            file->filePath(), file->fileType()),
-                                    &FilePath::toString);
+            return proFile->generatedFiles(buildDir(pro->filePath()),
+                                           file->filePath(), file->fileType());
     }
     return {};
 }
