@@ -26,17 +26,17 @@
 #include "qtwizard.h"
 
 #include <qmakeprojectmanager/qmakeproject.h>
-#include <qmakeprojectmanager/qmakeprojectmanager.h>
 #include <qmakeprojectmanager/qmakeprojectmanagerconstants.h>
 
 #include <coreplugin/icore.h>
 
-#include <cpptools/cpptoolsconstants.h>
+#include <cppeditor/cppeditorconstants.h>
 
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/targetsetuppage.h>
+#include <projectexplorer/task.h>
 
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtsupportconstants.h>
@@ -102,17 +102,15 @@ bool QtWizard::qt4ProjectPostGenerateFiles(const QWizard *w,
 
 QString QtWizard::templateDir()
 {
-    QString rc = Core::ICore::resourcePath();
-    rc += QLatin1String("/templates/qt4project");
-    return rc;
+    return Core::ICore::resourcePath("templates/qt4project").toString();
 }
 
 bool QtWizard::lowerCaseFiles()
 {
-    QString lowerCaseSettingsKey = QLatin1String(CppTools::Constants::CPPTOOLS_SETTINGSGROUP);
+    QString lowerCaseSettingsKey = QLatin1String(CppEditor::Constants::CPPEDITOR_SETTINGSGROUP);
     lowerCaseSettingsKey += QLatin1Char('/');
-    lowerCaseSettingsKey += QLatin1String(CppTools::Constants::LOWERCASE_CPPFILES_KEY);
-    const bool lowerCaseDefault = CppTools::Constants::lowerCaseFilesDefault;
+    lowerCaseSettingsKey += QLatin1String(CppEditor::Constants::LOWERCASE_CPPFILES_KEY);
+    const bool lowerCaseDefault = CppEditor::Constants::LOWERCASE_CPPFILES_DEFAULT;
     return Core::ICore::settings()->value(lowerCaseSettingsKey, QVariant(lowerCaseDefault)).toBool();
 }
 
@@ -147,7 +145,7 @@ BaseQmakeProjectWizardDialog::BaseQmakeProjectWizardDialog(
     m_profileIds = Utils::transform(parameters.extraValues()
                                         .value(ProjectExplorer::Constants::PROJECT_KIT_IDS)
                                         .toStringList(),
-                                    &Core::Id::fromString);
+                                    &Utils::Id::fromString);
 
     connect(this, &BaseProjectWizardDialog::projectParametersChanged,
             this, &BaseQmakeProjectWizardDialog::generateProfileName);
@@ -164,7 +162,7 @@ BaseQmakeProjectWizardDialog::BaseQmakeProjectWizardDialog(
     m_profileIds = Utils::transform(parameters.extraValues()
                                         .value(ProjectExplorer::Constants::PROJECT_KIT_IDS)
                                         .toStringList(),
-                                    &Core::Id::fromString);
+                                    &Utils::Id::fromString);
     connect(this, &BaseProjectWizardDialog::projectParametersChanged,
             this, &BaseQmakeProjectWizardDialog::generateProfileName);
 }
@@ -178,14 +176,23 @@ BaseQmakeProjectWizardDialog::~BaseQmakeProjectWizardDialog()
 int BaseQmakeProjectWizardDialog::addTargetSetupPage(int id)
 {
     m_targetSetupPage = new ProjectExplorer::TargetSetupPage;
-    const Core::Id platform = selectedPlatform();
-    QSet<Core::Id> features = {QtSupport::Constants::FEATURE_DESKTOP};
-    if (!platform.isValid())
-        m_targetSetupPage->setPreferredKitPredicate(QtKitAspect::qtVersionPredicate(features));
-    else
-        m_targetSetupPage->setPreferredKitPredicate(QtKitAspect::platformPredicate(platform));
 
-    m_targetSetupPage->setRequiredKitPredicate(QtKitAspect::qtVersionPredicate(requiredFeatures()));
+    m_targetSetupPage->setTasksGenerator([this](const Kit *k) -> Tasks {
+        if (!QtKitAspect::qtVersionPredicate(requiredFeatures())(k))
+            return {
+                ProjectExplorer::CompileTask(Task::Error, tr("Required Qt features not present."))};
+
+        const Utils::Id platform = selectedPlatform();
+        if (platform.isValid() && !QtKitAspect::platformPredicate(platform)(k))
+            return {ProjectExplorer::CompileTask(
+                ProjectExplorer::Task::Warning,
+                tr("Qt version does not target the expected platform."))};
+        QSet<Utils::Id> features = {QtSupport::Constants::FEATURE_DESKTOP};
+        if (!QtKitAspect::qtVersionPredicate(features)(k))
+            return {ProjectExplorer::CompileTask(ProjectExplorer::Task::Unknown,
+                                                 tr("Qt version does not provide all features."))};
+        return {};
+    });
 
     resize(900, 450);
     if (id >= 0)
@@ -209,7 +216,7 @@ bool BaseQmakeProjectWizardDialog::writeUserFile(const QString &proFileName) con
     return success;
 }
 
-QList<Core::Id> BaseQmakeProjectWizardDialog::selectedKits() const
+QList<Utils::Id> BaseQmakeProjectWizardDialog::selectedKits() const
 {
     if (!m_targetSetupPage)
         return m_profileIds;
