@@ -63,7 +63,7 @@ public:
     static QString formSuffix();
     static QString profileSuffix();
 
-    // Query CppTools settings for the class wizard settings
+    // Query CppEditor settings for the class wizard settings
     static bool lowerCaseFiles();
 
     static bool qt4ProjectPostGenerateFiles(const QWizard *w, const Core::GeneratedFiles &l, QString *errorMessage);
@@ -114,13 +114,13 @@ public:
     int addTargetSetupPage(int id = -1);
 
     bool writeUserFile(const QString &proFileName) const;
-    QList<Core::Id> selectedKits() const;
+    QList<Utils::Id> selectedKits() const;
 
 private:
     void generateProfileName(const QString &name, const QString &path);
 
     ProjectExplorer::TargetSetupPage *m_targetSetupPage = nullptr;
-    QList<Core::Id> m_profileIds;
+    QList<Utils::Id> m_profileIds;
 };
 
 } // namespace Internal
