@@ -456,7 +456,7 @@ void ActionEditorDialog::fillAndSetTargetItem(const QString &value, bool useDefa
 {
     if (m_comboBoxType->currentIndex() == ConnectionType::Action) {
         m_actionTargetItem->clear();
-        for (const auto &connection : m_connections) {
+        for (const auto &connection : qAsConst(m_connections)) {
             if (!connection.methods.isEmpty())
                 m_actionTargetItem->addItem(connection.item);
         }
@@ -471,8 +471,8 @@ void ActionEditorDialog::fillAndSetTargetItem(const QString &value, bool useDefa
         }
     } else { // ConnectionType::Assignment
         m_assignmentTargetItem->clear();
-        for (const auto &connection : m_connections) {
-            if (!connection.properties.isEmpty())
+        for (const auto &connection : qAsConst(m_connections)) {
+            if (!connection.properties.isEmpty() && connection.hasWriteableProperties())
                 m_assignmentTargetItem->addItem(connection.item);
         }
 
@@ -516,8 +516,10 @@ void ActionEditorDialog::fillAndSetTargetProperty(const QString &value, bool use
         if (idx == -1) {
             insertAndSetUndefined(m_assignmentTargetProperty);
         } else {
-            for (const auto &property : m_connections[idx].properties)
-                m_assignmentTargetProperty->addItem(property.name, property.type);
+            for (const auto &property : qAsConst(m_connections[idx].properties)) {
+                if (property.isWriteable)
+                    m_assignmentTargetProperty->addItem(property.name, property.type);
+            }
 
             if (m_assignmentTargetProperty->findText(value) != -1) {
                 m_assignmentTargetProperty->setCurrentText(value);
@@ -537,7 +539,7 @@ void ActionEditorDialog::fillAndSetSourceItem(const QString &value, bool useDefa
     const TypeName targetPropertyType = m_assignmentTargetProperty->currentData().value<TypeName>();
 
     if (!targetPropertyType.isEmpty()) {
-        for (const ConnectionOption &connection : m_connections) {
+        for (const ConnectionOption &connection : qAsConst(m_connections)) {
             if (!connection.containsType(targetPropertyType))
                 continue;
 
@@ -545,7 +547,7 @@ void ActionEditorDialog::fillAndSetSourceItem(const QString &value, bool useDefa
         }
 
         // Add Constants
-        for (const SingletonOption &singleton : m_singletons) {
+        for (const SingletonOption &singleton : qAsConst(m_singletons)) {
             if (!singleton.containsType(targetPropertyType))
                 continue;
 
@@ -581,7 +583,7 @@ void ActionEditorDialog::fillAndSetSourceProperty(const QString &value,
             else
                 insertAndSetUndefined(m_assignmentSourceProperty);
         } else if (targetProperty == "state") {
-            for (const auto &state : m_states)
+            for (const auto &state : qAsConst(m_states))
                 m_assignmentSourceProperty->addItem(state, specificItem);
 
             if (m_assignmentSourceProperty->findText(value) != -1)
@@ -610,7 +612,7 @@ void ActionEditorDialog::fillAndSetSourceProperty(const QString &value,
                 m_assignmentSourceProperty->addItem("false", specificItem);
                 specificsEnd = 2;
             } else if (targetProperty == "state") {
-                for (const auto &state : m_states)
+                for (const auto &state : qAsConst(m_states))
                     m_assignmentSourceProperty->addItem(state, specificItem);
 
                 specificsEnd = m_states.count();
@@ -620,14 +622,14 @@ void ActionEditorDialog::fillAndSetSourceProperty(const QString &value,
                 m_assignmentSourceProperty->insertSeparator(specificsEnd);
 
             if (sourceItemType == singletonItem) {
-                for (const auto &property : m_singletons[idx].properties) {
+                for (const auto &property : qAsConst(m_singletons[idx].properties)) {
                     if (targetPropertyType.isEmpty() // TODO isEmpty correct?!
                         || property.type == targetPropertyType
                         || (isNumeric(property.type) && isNumeric(targetPropertyType)))
                         m_assignmentSourceProperty->addItem(property.name, property.type);
                 }
             } else {
-                for (const auto &property : m_connections[idx].properties) {
+                for (const auto &property : qAsConst(m_connections[idx].properties)) {
                     if (targetPropertyType.isEmpty() // TODO isEmpty correct?!
                         || property.type == targetPropertyType
                         || (isNumeric(property.type) && isNumeric(targetPropertyType)))
