@@ -24,6 +24,7 @@
 ****************************************************************************/
 
 #include "changestyleaction.h"
+#include "designermcumanager.h"
 
 #include <projectexplorer/project.h>
 #include <projectexplorer/session.h>
@@ -48,6 +49,24 @@ static QString styleConfigFileName(const QString &qmlFileName)
 
 ChangeStyleWidgetAction::ChangeStyleWidgetAction(QObject *parent) : QWidgetAction(parent)
 {
+    // The Default style was renamed to Basic in Qt 6. In Qt 6, "Default"
+    // will result in a platform-specific style being chosen.
+    items = {
+        {"Basic", "Basic", {}},
+        {"Default", "Default", {}},
+        {"Fusion", "Fusion", {}},
+        {"Imagine", "Imagine", {}},
+        {"Material Light", "Material", "Light"},
+        {"Material Dark", "Material", "Dark"},
+        {"Universal Light", "Universal", "Light"},
+        {"Universal Dark", "Universal", "Dark"},
+        {"Universal System", "Universal", "System"}
+    };
+
+    if (Utils::HostOsInfo::isMacHost())
+        items.append({"macOS", "macOS", {}});
+    if (Utils::HostOsInfo::isWindowsHost())
+        items.append({"Windows", "Windows", {}});
 }
 
 void ChangeStyleWidgetAction::handleModelUpdate(const QString &style)
@@ -55,6 +74,48 @@ void ChangeStyleWidgetAction::handleModelUpdate(const QString &style)
     emit modelUpdated(style);
 }
 
+const QList<StyleWidgetEntry> ChangeStyleWidgetAction::styleItems() const
+{
+    return items;
+}
+
+void ChangeStyleWidgetAction::changeStyle(const QString &style)
+{
+    if (style.isEmpty())
+        return;
+
+    const Utils::FilePath configFileName = Utils::FilePath::fromString(styleConfigFileName(qmlFileName));
+
+    if (configFileName.exists()) {
+        QSettings infiFile(configFileName.toString(), QSettings::IniFormat);
+
+        int contains = -1;
+
+        for (const auto &item : qAsConst(items)) {
+            if (item.displayName == style) {
+                contains = items.indexOf(item);
+                break;
+            }
+        }
+
+        if (contains >= 0) {
+            const QString styleName = items.at(contains).styleName;
+            const QString styleTheme = items.at(contains).styleTheme;
+
+            infiFile.setValue("Controls/Style", styleName);
+
+            if (!styleTheme.isEmpty())
+                infiFile.setValue((styleName + "/Theme"), styleTheme);
+        }
+        else {
+            infiFile.setValue("Controls/Style", style);
+        }
+
+        if (view)
+            view->resetPuppet();
+    }
+}
+
 const char enabledTooltip[] = QT_TRANSLATE_NOOP("ChangeStyleWidgetAction",
                                                 "Change style for Qt Quick Controls 2.");
 const char disbledTooltip[] = QT_TRANSLATE_NOOP("ChangeStyleWidgetAction",
@@ -64,11 +125,10 @@ QWidget *ChangeStyleWidgetAction::createWidget(QWidget *parent)
 {
     auto comboBox = new QComboBox(parent);
     comboBox->setToolTip(tr(enabledTooltip));
-    comboBox->addItem("Default");
-    comboBox->addItem("Fusion");
-    comboBox->addItem("Imagine");
-    comboBox->addItem("Material");
-    comboBox->addItem("Universal");
+
+    for (const auto &item : qAsConst(items))
+        comboBox->addItem(item.displayName);
+
     comboBox->setEditable(true);
     comboBox->setCurrentIndex(0);
 
@@ -83,6 +143,10 @@ QWidget *ChangeStyleWidgetAction::createWidget(QWidget *parent)
             comboBox->setDisabled(true);
             comboBox->setToolTip(tr(disbledTooltip));
             comboBox->setCurrentIndex(0);
+        } else if (DesignerMcuManager::instance().isMCUProject()) {
+            comboBox->setDisabled(true);
+            //TODO: add tooltip regarding MCU limitations, however we are behind string freeze
+            comboBox->setEditText(style);
         } else {
             comboBox->setDisabled(false);
             comboBox->setToolTip(tr(enabledTooltip));
@@ -90,27 +154,8 @@ QWidget *ChangeStyleWidgetAction::createWidget(QWidget *parent)
         }
     });
 
-    connect(comboBox,
-        #if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-            QOverload<const QString &>::of(&QComboBox::activated),
-        #else
-            &QComboBox::textActivated,
-        #endif
-            this,
-            [this](const QString &style) {
-
-        if (style.isEmpty())
-            return;
-
-        const Utils::FilePath configFileName = Utils::FilePath::fromString(styleConfigFileName(qmlFileName));
-
-        if (configFileName.exists()) {
-             QSettings infiFile(configFileName.toString(), QSettings::IniFormat);
-             infiFile.setValue("Controls/Style", style);
-             if (view)
-                 view->resetPuppet();
-        }
-    });
+    connect(comboBox, &QComboBox::textActivated,
+            this, &ChangeStyleWidgetAction::changeStyle);
 
     return comboBox;
 }
@@ -132,11 +177,27 @@ void ChangeStyleAction::currentContextChanged(const SelectionContext &selectionC
 
         if (Utils::FilePath::fromString(confFileName).exists()) {
             QSettings infiFile(confFileName, QSettings::IniFormat);
-            m_action->handleModelUpdate(infiFile.value("Controls/Style", "Default").toString());
+            const QString styleName = infiFile.value("Controls/Style", "Basic").toString();
+            const QString styleTheme = infiFile.value(styleName + "/Theme", "").toString();
+            const auto items = m_action->styleItems();
+
+            QString comboBoxEntry = styleName;
+
+            for (const auto &item : items) {
+                if (item.styleName == styleName) {
+                    if (!styleTheme.isEmpty() && (item.styleTheme == styleTheme)) {
+                        comboBoxEntry.append(" ");
+                        comboBoxEntry.append(styleTheme);
+
+                        break;
+                    }
+                }
+            }
+
+            m_action->handleModelUpdate(comboBoxEntry);
         } else {
             m_action->handleModelUpdate("");
         }
-
     }
 }
 
