@@ -51,7 +51,7 @@
 #include <componentcore_constants.h>
 #include <stylesheetmerger.h>
 
-#include <limits>
+#include <designermcumanager.h>
 #include <qmldesignerplugin.h>
 
 #include <coreplugin/messagebox.h>
@@ -82,10 +82,13 @@
 #include <QPushButton>
 #include <QGridLayout>
 #include <QPointer>
+#include <QMessageBox>
+#include <QPair>
 
 #include <algorithm>
 #include <functional>
 #include <cmath>
+#include <limits>
 
 #include <bindingeditor/signallist.h>
 
@@ -185,7 +188,7 @@ void toFront(const SelectionContext &selectionState)
             if (index != lastIndex)
                 parentProperty.slide(index, lastIndex);
         }
-    } catch (const RewritingException &e) { //better save then sorry
+    } catch (const RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -206,7 +209,7 @@ void toBack(const SelectionContext &selectionState)
                 parentProperty.slide(index, 0);
         }
 
-    } catch (const RewritingException &e) { //better save then sorry
+    } catch (const RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -270,7 +273,7 @@ void setVisible(const SelectionContext &selectionState)
 
     try {
         selectionState.selectedModelNodes().constFirst().variantProperty("visible").setValue(selectionState.toggled());
-    } catch (const RewritingException &e) { //better save then sorry
+    } catch (const RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -283,7 +286,7 @@ void setFillWidth(const SelectionContext &selectionState)
 
     try {
         selectionState.firstSelectedModelNode().variantProperty("Layout.fillWidth").setValue(selectionState.toggled());
-    } catch (const RewritingException &e) { //better save then sorry
+    } catch (const RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -296,7 +299,7 @@ void setFillHeight(const SelectionContext &selectionState)
 
     try {
         selectionState.firstSelectedModelNode().variantProperty("Layout.fillHeight").setValue(selectionState.toggled());
-    } catch (const RewritingException &e) { //better save then sorry
+    } catch (const RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -520,9 +523,14 @@ void layoutFlowPositioner(const SelectionContext &selectionContext)
 void layoutRowLayout(const SelectionContext &selectionContext)
 {
     try {
-        LayoutInGridLayout::ensureLayoutImport(selectionContext);
-        layoutHelperFunction(selectionContext, "QtQuick.Layouts.RowLayout", compareByX);
-    } catch (RewritingException &e) { //better save then sorry
+        if (DesignerMcuManager::instance().isMCUProject()) {
+            layoutHelperFunction(selectionContext, "QtQuick.Row", compareByX);
+        }
+        else {
+            LayoutInGridLayout::ensureLayoutImport(selectionContext);
+            layoutHelperFunction(selectionContext, "QtQuick.Layouts.RowLayout", compareByX);
+        }
+    } catch (RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -530,9 +538,14 @@ void layoutRowLayout(const SelectionContext &selectionContext)
 void layoutColumnLayout(const SelectionContext &selectionContext)
 {
     try {
-        LayoutInGridLayout::ensureLayoutImport(selectionContext);
-        layoutHelperFunction(selectionContext, "QtQuick.Layouts.ColumnLayout", compareByY);
-    } catch (RewritingException &e) { //better save then sorry
+        if (DesignerMcuManager::instance().isMCUProject()) {
+            layoutHelperFunction(selectionContext, "QtQuick.Column", compareByX);
+        }
+        else {
+            LayoutInGridLayout::ensureLayoutImport(selectionContext);
+            layoutHelperFunction(selectionContext, "QtQuick.Layouts.ColumnLayout", compareByY);
+        }
+    } catch (RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -540,9 +553,16 @@ void layoutColumnLayout(const SelectionContext &selectionContext)
 void layoutGridLayout(const SelectionContext &selectionContext)
 {
     try {
-        LayoutInGridLayout::ensureLayoutImport(selectionContext);
-        LayoutInGridLayout::layout(selectionContext);
-    } catch (RewritingException &e) { //better save then sorry
+        Q_ASSERT(!DesignerMcuManager::instance().isMCUProject()); //remove this line when grids are finally supported
+
+        if (DesignerMcuManager::instance().isMCUProject()) {
+            //qt for mcu doesn't support any grids yet
+        }
+        else {
+            LayoutInGridLayout::ensureLayoutImport(selectionContext);
+            LayoutInGridLayout::layout(selectionContext);
+        }
+    } catch (RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -631,7 +651,7 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
 
     if (!qmlObjectNode.isValid()) {
         QString title = QCoreApplication::translate("ModelNodeOperations", "Go to Implementation");
-        QString description = QCoreApplication::translate("ModelNodeOperations", "Invalid item.");
+        QString description = QCoreApplication::translate("ModelNodeOperations", "Invalid component.");
         Core::AsynchronousMessageBox::warning(title, description);
         return;
     }
@@ -647,7 +667,7 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
 
     const Utils::FilePath currentDesignDocument = QmlDesignerPlugin::instance()->documentManager().currentDesignDocument()->fileName();
     const QString fileName = currentDesignDocument.toString();
-    const QString typeName = currentDesignDocument.toFileInfo().baseName();
+    const QString typeName = currentDesignDocument.baseName();
 
     QStringList signalNames = cleanSignalNames(getSortedSignalNameList(selectionState.selectedModelNodes().constFirst()));
 
@@ -972,74 +992,100 @@ Utils::FilePath projectFilePath()
     return Utils::FilePath();
 }
 
-bool addFontToProject(const QStringList &fileNames, const QString &defaultDirectory)
-{
-
-    QString adjustedDefaultDirectory = defaultDirectory;
-    Utils::FilePath fonts = projectFilePath().pathAppended("fonts");
-
-    if (fonts.exists())
-        adjustedDefaultDirectory = fonts.toString();
-
-    QString directory = AddImagesDialog::getDirectory(fileNames, adjustedDefaultDirectory);
-
-    if (directory.isEmpty())
-        return true;
-
-    bool allSuccessful = true;
-    for (const QString &fileName : fileNames) {
-        const QString targetFile = directory + "/" + QFileInfo(fileName).fileName();
-        const bool success = QFile::copy(fileName, targetFile);
-
-        auto document = QmlDesignerPlugin::instance()->currentDesignDocument();
-
-        QTC_ASSERT(document, return false);
-
-        if (success) {
-            ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(document->fileName());
-            if (node) {
-                ProjectExplorer::FolderNode *containingFolder = node->parentFolderNode();
-                if (containingFolder)
-                    containingFolder->addFiles(QStringList(targetFile));
-            }
-        } else {
-            allSuccessful = false;
-        }
-    }
-
-    return allSuccessful;
-}
-
-
-bool addImageToProject(const QStringList &fileNames, const QString &defaultDirectory)
+static AddFilesResult addFilesToProject(const QStringList &fileNames, const QString &defaultDirectory)
 {
     QString directory = AddImagesDialog::getDirectory(fileNames, defaultDirectory);
-
     if (directory.isEmpty())
-        return true;
+        return AddFilesResult::Cancelled;
 
-    bool allSuccessful = true;
+    DesignDocument *document = QmlDesignerPlugin::instance()->currentDesignDocument();
+    QTC_ASSERT(document, return AddFilesResult::Failed);
+
+    QList<QPair<QString, QString>> copyList;
+    QStringList removeList;
     for (const QString &fileName : fileNames) {
         const QString targetFile = directory + "/" + QFileInfo(fileName).fileName();
-        const bool success = QFile::copy(fileName, targetFile);
-
-        auto document = QmlDesignerPlugin::instance()->currentDesignDocument();
-
-        QTC_ASSERT(document, return false);
-
-        if (success) {
-            ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(document->fileName());
-            if (node) {
-                ProjectExplorer::FolderNode *containingFolder = node->parentFolderNode();
-                if (containingFolder)
-                    containingFolder->addFiles(QStringList(targetFile));
+        if (QFileInfo::exists(targetFile)) {
+            const QString title = QCoreApplication::translate(
+                        "ModelNodeOperations", "Overwrite Existing File?");
+            const QString question = QCoreApplication::translate(
+                        "ModelNodeOperations", "File already exists. Overwrite?\n\"%1\"").arg(targetFile);
+            if (QMessageBox::question(qobject_cast<QWidget *>(Core::ICore::dialogParent()),
+                                      title, question, QMessageBox::Yes | QMessageBox::No)
+                    != QMessageBox::Yes) {
+                continue;
             }
-        } else {
-            allSuccessful = false;
+            removeList.append(targetFile);
+        }
+        copyList.append({fileName, targetFile});
+    }
+    // Defer actual file operations after we have dealt with possible popup dialogs to avoid
+    // unnecessarily refreshing file models multiple times during the operation
+    for (const auto &file : qAsConst(removeList))
+        QFile::remove(file);
+
+    for (const auto &filePair : qAsConst(copyList)) {
+        const bool success = QFile::copy(filePair.first, filePair.second);
+        if (!success)
+            return AddFilesResult::Failed;
+
+        ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(document->fileName());
+        if (node) {
+            ProjectExplorer::FolderNode *containingFolder = node->parentFolderNode();
+            if (containingFolder)
+                containingFolder->addFiles({Utils::FilePath::fromString(filePair.second)});
         }
     }
 
-    return allSuccessful;
+    return AddFilesResult::Succeeded;
+}
+
+static QString getAssetDefaultDirectory(const QString &assetDir, const QString &defaultDirectory)
+{
+    QString adjustedDefaultDirectory = defaultDirectory;
+
+    Utils::FilePath contentPath = projectFilePath();
+
+    if (contentPath.pathAppended("content").exists())
+            contentPath = contentPath.pathAppended("content");
+
+    Utils::FilePath assetPath = contentPath.pathAppended(assetDir);
+
+    if (!assetPath.exists()) {
+        // Create the default asset type directory if it doesn't exist
+        QDir dir(contentPath.toString());
+        dir.mkpath(assetDir);
+    }
+
+    if (assetPath.exists() && assetPath.isDir())
+        adjustedDefaultDirectory = assetPath.toString();
+
+    return adjustedDefaultDirectory;
+}
+
+AddFilesResult addFontToProject(const QStringList &fileNames, const QString &defaultDirectory)
+{
+    return addFilesToProject(fileNames, getAssetDefaultDirectory("fonts", defaultDirectory));
+}
+
+AddFilesResult addSoundToProject(const QStringList &fileNames, const QString &defaultDirectory)
+{
+    return addFilesToProject(fileNames, getAssetDefaultDirectory("sounds", defaultDirectory));
+}
+
+AddFilesResult addShaderToProject(const QStringList &fileNames, const QString &defaultDirectory)
+{
+    return addFilesToProject(fileNames, getAssetDefaultDirectory("shaders", defaultDirectory));
+}
+
+AddFilesResult addImageToProject(const QStringList &fileNames, const QString &defaultDirectory)
+{
+    return addFilesToProject(fileNames, getAssetDefaultDirectory("images", defaultDirectory));
+}
+
+AddFilesResult addVideoToProject(const QStringList &fileNames, const QString &defaultDirectory)
+{
+    return addFilesToProject(fileNames, getAssetDefaultDirectory("videos", defaultDirectory));
 }
 
 void createFlowActionArea(const SelectionContext &selectionContext)
@@ -1437,7 +1483,7 @@ void styleMerge(const SelectionContext &selectionContext, const QString &templat
     QPlainTextEdit textEditTemplate;
     Utils::FileReader reader;
 
-    QTC_ASSERT(reader.fetch(templateFile), return);
+    QTC_ASSERT(reader.fetch(Utils::FilePath::fromString(templateFile)), return);
     QString qmlTemplateString = QString::fromUtf8(reader.data());
     QString imports;
 
@@ -1535,7 +1581,7 @@ void editAnnotation(const SelectionContext &selectionContext)
 {
     ModelNode selectedNode = selectionContext.currentSingleSelectedNode();
 
-    AnnotationEditor::showWidget(selectedNode);
+    ModelNodeEditorProxy::fromModelNode<AnnotationEditor>(selectedNode);
 }
 
 QVariant previewImageDataForGenericNode(const ModelNode &modelNode)
@@ -1560,6 +1606,15 @@ void openSignalDialog(const SelectionContext &selectionContext)
     SignalList::showWidget(selectionContext.currentSingleSelectedNode());
 }
 
+void updateImported3DAsset(const SelectionContext &selectionContext)
+{
+    if (selectionContext.view()) {
+        selectionContext.view()->emitCustomNotification(
+                    "UpdateImported3DAsset", {selectionContext.currentSingleSelectedNode()});
+
+    }
+}
+
 } // namespace ModelNodeOperations
 
 } //QmlDesigner
