@@ -46,6 +46,11 @@ ModelNode SelectionContext::targetNode() const
     return m_targetNode;
 }
 
+ModelNode SelectionContext::rootNode() const
+{
+    return view()->rootModelNode();
+}
+
 bool SelectionContext::singleNodeIsSelected() const
 {
     return view()->hasSingleSelectedModelNode();
@@ -73,7 +78,8 @@ QList<ModelNode> SelectionContext::selectedModelNodes() const
 
 bool SelectionContext::hasSingleSelectedModelNode() const
 {
-    return view()->hasSelectedModelNodes();
+    return view()->hasSingleSelectedModelNode()
+            && firstSelectedModelNode().isValid();
 }
 
 AbstractView *SelectionContext::view() const
@@ -118,12 +124,17 @@ bool SelectionContext::isValid() const
 
 bool SelectionContext::fastUpdate() const
 {
-    return m_updateMode == UpdateMode::Fast;
+    return m_updateReason != UpdateMode::Normal;
 }
 
 void SelectionContext::setUpdateMode(UpdateMode mode)
 {
-    m_updateMode = mode;
+    m_updateReason = mode;
+}
+
+SelectionContext::UpdateMode SelectionContext::updateReason() const
+{
+    return m_updateReason;
 }
 
 } //QmlDesigner
