@@ -29,7 +29,10 @@
 #include "curveitem.h"
 #include "navigation2d.h"
 #include "treeitem.h"
-#include "utils.h"
+#include "curveeditorutils.h"
+
+#include <theme.h>
+#include <utils/fileutils.h>
 
 #include <QAction>
 #include <QMenu>
@@ -42,6 +45,18 @@
 
 namespace QmlDesigner {
 
+template< typename T >
+T* nextParentOfType(QWidget* widget)
+{
+    auto* p = widget->parent();
+    while (p) {
+        if (T* w = qobject_cast<T*>(p))
+            return w;
+        p = p->parent();
+    }
+    return nullptr;
+}
+
 GraphicsView::GraphicsView(CurveEditorModel *model, QWidget *parent)
     : QGraphicsView(parent)
     , m_dragging(false)
@@ -62,28 +77,40 @@ GraphicsView::GraphicsView(CurveEditorModel *model, QWidget *parent)
     setResizeAnchor(QGraphicsView::NoAnchor);
     setRenderHint(QPainter::Antialiasing, true);
     setTransformationAnchor(QGraphicsView::NoAnchor);
-    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
     setViewportUpdateMode(QGraphicsView::FullViewportUpdate);
 
     connect(&m_dialog, &CurveEditorStyleDialog::styleChanged, this, &GraphicsView::setStyle);
 
     auto itemSlot = [this](unsigned int id, const AnimationCurve &curve) {
-        applyZoom(m_zoomX, m_zoomY);
         m_model->setCurve(id, curve);
+        applyZoom(m_zoomX, m_zoomY);
     };
 
     connect(m_scene, &GraphicsScene::curveChanged, itemSlot);
 
-    QmlDesigner::Navigation2dFilter *filter = new QmlDesigner::Navigation2dFilter(this);
+    QmlDesigner::Navigation2dFilter *filter = new QmlDesigner::Navigation2dFilter(viewport());
+    connect(filter, &Navigation2dFilter::panChanged, [this](const QPointF &direction) {
+        QScrollBar* verticalBar = nullptr;
+        if (QScrollArea* area = nextParentOfType< QScrollArea >(this))
+            verticalBar = area->verticalScrollBar();
+        Navigation2dFilter::scroll(direction, horizontalScrollBar(), verticalBar);
+    });
+
     auto zoomChanged = &QmlDesigner::Navigation2dFilter::zoomChanged;
     connect(filter, zoomChanged, [this](double scale, const QPointF &pos) {
         applyZoom(m_zoomX + scale, m_zoomY, mapToGlobal(pos.toPoint()));
     });
-    installEventFilter(filter);
+    viewport()->installEventFilter(filter);
 
     applyZoom(m_zoomX, m_zoomY);
     update();
+
+    const QString css = Theme::replaceCssColors(
+        QString::fromUtf8(Utils::FileReader::fetchQrc(":/qmldesigner/scrollbar.css")));
+    horizontalScrollBar()->setStyleSheet(css);
+    verticalScrollBar()->setStyleSheet(css);
 }
 
 GraphicsView::~GraphicsView()
@@ -663,11 +690,15 @@ void GraphicsView::drawValueScale(QPainter *painter, const QRectF &rect)
         painter->drawText(textRect, Qt::AlignCenter, valueText);
     };
 
-    double density = 1. / (static_cast<double>(fm.height()) * m_style.labelDensityY);
-    Axis axis = Axis::compute(minimumValue(), maximumValue(), rect.height(), density);
-    const double eps = 1.0e-10;
-    for (double i = axis.lmin; i <= axis.lmax + eps; i += axis.lstep)
-        paintLabeledTick(i);
+    double min = minimumValue();
+    double max = maximumValue();
+    if (std::isfinite(min) && std::isfinite(max) && rect.isValid()) {
+        double density = 1. / (static_cast<double>(fm.height()) * m_style.labelDensityY);
+        Axis axis = Axis::compute(min, max, rect.height(), density);
+        const double eps = 1.0e-10;
+        for (double i = axis.lmin; i <= axis.lmax + eps; i += axis.lstep)
+            paintLabeledTick(i);
+    }
 
     painter->restore();
 }
