@@ -24,13 +24,14 @@
 ****************************************************************************/
 
 #include "handleitem.h"
+#include "curvesegment.h"
 #include "graphicsscene.h"
 #include "keyframeitem.h"
-#include "utils.h"
+#include "curveeditorutils.h"
 
 #include <QPainter>
 
-namespace DesignTools {
+namespace QmlDesigner {
 
 struct HandleGeometry
 {
@@ -40,8 +41,11 @@ struct HandleGeometry
         handle = QRectF(topLeft, -topLeft);
         toKeyframe = QLineF(QPointF(0.0, 0.0), -pos);
         angle = -toKeyframe.angle() + 45.0;
+        bbox = handle.united(QRectF(-pos, QSizeF(1.0,1.0)));
     }
 
+    QRectF bbox;
+
     QRectF handle;
 
     QLineF toKeyframe;
@@ -72,6 +76,14 @@ bool HandleItem::keyframeSelected() const
     return false;
 }
 
+CurveSegment HandleItem::segment() const
+{
+    if (KeyframeItem *parent = qgraphicsitem_cast<KeyframeItem *>(parentItem()))
+        return parent->segment(m_slot);
+
+    return CurveSegment();
+}
+
 KeyframeItem *HandleItem::keyframe() const
 {
     if (KeyframeItem *parent = qgraphicsitem_cast<KeyframeItem *>(parentItem()))
@@ -88,7 +100,7 @@ HandleItem::Slot HandleItem::slot() const
 QRectF HandleItem::boundingRect() const
 {
     HandleGeometry geom(pos(), m_style);
-    return geom.handle;
+    return geom.bbox;
 }
 
 bool HandleItem::contains(const QPointF &point) const
@@ -146,20 +158,42 @@ void HandleItem::setStyle(const CurveEditorStyle &style)
 QVariant HandleItem::itemChange(QGraphicsItem::GraphicsItemChange change, const QVariant &value)
 {
     if (change == ItemPositionChange) {
-        if (keyframe()) {
+
+        if (!scene())
+            return QGraphicsItem::itemChange(change, value);
+
+        if (KeyframeItem *keyItem = keyframe()) {
+            CurveSegment seg = segment();
+            if (!seg.isLegal())
+                return value;
+
             QPointF pos = value.toPointF();
+            QPointF relativePosition = keyItem->transform().inverted().map(pos);
+
             if (m_slot == HandleItem::Slot::Left) {
                 if (pos.x() > 0.0)
                     pos.rx() = 0.0;
 
+                Keyframe key = seg.right();
+                key.setLeftHandle(key.position() + relativePosition);
+                seg.setRight(key);
+
             } else if (m_slot == HandleItem::Slot::Right) {
                 if (pos.x() < 0.0)
                     pos.rx() = 0.0;
+
+                Keyframe key = seg.left();
+                key.setRightHandle(key.position() + relativePosition);
+                seg.setLeft(key);
             }
-            return QVariant(pos);
+
+            if (seg.isLegal())
+                m_validPos = pos;
+
+            return QVariant(m_validPos);
         }
     }
     return QGraphicsItem::itemChange(change, value);
 }
 
-} // End namespace DesignTools.
+} // End namespace QmlDesigner.
