@@ -91,6 +91,9 @@ void FormEditorView::modelAttached(Model *model)
 //This function does the setup of the initial FormEditorItem tree in the scene
 void FormEditorView::setupFormEditorItemTree(const QmlItemNode &qmlItemNode)
 {
+    if (!qmlItemNode.hasFormEditorItem())
+        return;
+
     if (qmlItemNode.isFlowTransition()) {
         m_scene->addFormEditorItem(qmlItemNode, FormEditorScene::FlowTransition);
         if (qmlItemNode.hasNodeParent())
@@ -129,19 +132,24 @@ void FormEditorView::setupFormEditorItemTree(const QmlItemNode &qmlItemNode)
         m_scene->synchronizeTransformation(rootItem);
         formEditorWidget()->setRootItemRect(qmlItemNode.instanceBoundingRect());
 
-        for (const QmlObjectNode &nextNode : qmlItemNode.allDirectSubNodes()) {
-            if (QmlItemNode::isValidQmlItemNode(nextNode) && nextNode.toQmlItemNode().isFlowItem()) {
-                setupFormEditorItemTree(nextNode.toQmlItemNode());
+        const QList<QmlObjectNode> allDirectSubNodes = qmlItemNode.allDirectSubNodes();
+        for (const QmlObjectNode &childNode : allDirectSubNodes) {
+            if (QmlItemNode::isValidQmlItemNode(childNode)
+                && childNode.toQmlItemNode().isFlowItem()) {
+                setupFormEditorItemTree(childNode.toQmlItemNode());
             }
         }
 
-        for (const QmlObjectNode &nextNode : qmlItemNode.allDirectSubNodes()) {
-            if (QmlVisualNode::isValidQmlVisualNode(nextNode) && nextNode.toQmlVisualNode().isFlowTransition()) {
-                setupFormEditorItemTree(nextNode.toQmlItemNode());
-            } else if (QmlVisualNode::isValidQmlVisualNode(nextNode) && nextNode.toQmlVisualNode().isFlowDecision()) {
-                setupFormEditorItemTree(nextNode.toQmlItemNode());
-            } else if (QmlVisualNode::isValidQmlVisualNode(nextNode) && nextNode.toQmlVisualNode().isFlowWildcard()) {
-                setupFormEditorItemTree(nextNode.toQmlItemNode());
+        for (const QmlObjectNode &childNode : allDirectSubNodes) {
+            if (QmlVisualNode::isValidQmlVisualNode(childNode)
+                && childNode.toQmlVisualNode().isFlowTransition()) {
+                setupFormEditorItemTree(childNode.toQmlItemNode());
+            } else if (QmlVisualNode::isValidQmlVisualNode(childNode)
+                       && childNode.toQmlVisualNode().isFlowDecision()) {
+                setupFormEditorItemTree(childNode.toQmlItemNode());
+            } else if (QmlVisualNode::isValidQmlVisualNode(childNode)
+                       && childNode.toQmlVisualNode().isFlowWildcard()) {
+                setupFormEditorItemTree(childNode.toQmlItemNode());
             }
         }
     } else {
@@ -194,16 +202,6 @@ void FormEditorView::removeNodeFromScene(const QmlItemNode &qmlItemNode)
         m_currentTool->itemsAboutToRemoved(removedItemList);
 }
 
-void FormEditorView::hideNodeFromScene(const QmlItemNode &qmlItemNode)
-{
-    if (FormEditorItem *item = m_scene->itemForQmlItemNode(qmlItemNode)) {
-        QList<FormEditorItem*> removedItems = scene()->itemsForQmlItemNodes(qmlItemNode.allSubModelNodes());
-        removedItems.append(item);
-        m_currentTool->itemsAboutToRemoved(removedItems);
-        item->setFormEditorVisible(false);
-    }
-}
-
 void FormEditorView::createFormEditorWidget()
 {
     m_formEditorWidget = QPointer<FormEditorWidget>(new FormEditorWidget(this));
@@ -243,10 +241,7 @@ void FormEditorView::temporaryBlockView(int duration)
 
 void FormEditorView::nodeCreated(const ModelNode &node)
 {
-    //If the node has source for components/custom parsers we ignore it.
-    if (QmlItemNode::isValidQmlItemNode(node) && node.nodeSourceType() == ModelNode::NodeWithoutSource) //only setup QmlItems
-        setupFormEditorItemTree(QmlItemNode(node));
-    else if (QmlVisualNode::isFlowTransition(node))
+    if (QmlVisualNode::isFlowTransition(node))
         setupFormEditorItemTree(QmlItemNode(node));
 }
 
@@ -331,21 +326,28 @@ void FormEditorView::propertiesAboutToBeRemoved(const QList<AbstractProperty>& p
     m_currentTool->itemsAboutToRemoved(removedItems);
 }
 
-static inline bool hasNodeSourceParent(const ModelNode &node)
+static inline bool hasNodeSourceOrNonItemParent(const ModelNode &node)
 {
     if (node.hasParentProperty() && node.parentProperty().parentModelNode().isValid()) {
         ModelNode parent = node.parentProperty().parentModelNode();
-        if (parent.nodeSourceType() != ModelNode::NodeWithoutSource)
+        if (parent.nodeSourceType() != ModelNode::NodeWithoutSource
+                || !QmlItemNode::isItemOrWindow(parent)) {
             return true;
-        return hasNodeSourceParent(parent);
+        }
+        return hasNodeSourceOrNonItemParent(parent);
     }
     return false;
 }
 
 void FormEditorView::nodeReparented(const ModelNode &node, const NodeAbstractProperty &/*newPropertyParent*/, const NodeAbstractProperty &/*oldPropertyParent*/, AbstractView::PropertyChangeFlags /*propertyChange*/)
 {
-    if (hasNodeSourceParent(node))
-        hideNodeFromScene(node);
+    addOrRemoveFormEditorItem(node);
+}
+
+void FormEditorView::nodeSourceChanged(const ModelNode &node, const QString &newNodeSource)
+{
+    Q_UNUSED(newNodeSource)
+    addOrRemoveFormEditorItem(node);
 }
 
 WidgetInfo FormEditorView::widgetInfo()
@@ -562,7 +564,7 @@ void FormEditorView::changeToCustomTool()
 
         const ModelNode selectedModelNode = selectedModelNodes().constFirst();
 
-        for (AbstractCustomTool *customTool : m_customToolList) {
+        for (AbstractCustomTool *customTool : qAsConst(m_customToolList)) {
             if (customTool->wantHandleItem(selectedModelNode) > handlingRank) {
                 handlingRank = customTool->wantHandleItem(selectedModelNode);
                 selectedCustomTool = customTool;
@@ -598,8 +600,7 @@ void FormEditorView::auxiliaryDataChanged(const ModelNode &node, const PropertyN
     if (name == "invisible") {
         if (FormEditorItem *item = scene()->itemForQmlItemNode(QmlItemNode(node))) {
             bool isInvisible = data.toBool();
-            if (item->isFormEditorVisible())
-                item->setVisible(!isInvisible);
+            item->setVisible(!isInvisible);
             ModelNode newNode(node);
             if (isInvisible)
                 newNode.deselectNode();
@@ -632,8 +633,21 @@ void FormEditorView::auxiliaryDataChanged(const ModelNode &node, const PropertyN
     }
 }
 
+static void updateTransitions(FormEditorScene *scene, const QmlItemNode &qmlItemNode)
+{
+    QmlFlowTargetNode flowItem(qmlItemNode);
+    if (flowItem.isValid() && flowItem.flowView().isValid()) {
+        const auto nodes = flowItem.flowView().transitions();
+        for (const ModelNode &node : nodes) {
+            if (FormEditorItem *item = scene->itemForQmlItemNode(node))
+                item->updateGeometry();
+        }
+    };
+}
+
 void FormEditorView::instancesCompleted(const QVector<ModelNode> &completedNodeList)
 {
+    const bool isFlow = rootModelNode().isValid() && QmlItemNode(rootModelNode()).isFlowView();
     QList<FormEditorItem*> itemNodeList;
     for (const ModelNode &node : completedNodeList) {
         const QmlItemNode qmlItemNode(node);
@@ -641,6 +655,8 @@ void FormEditorView::instancesCompleted(const QVector<ModelNode> &completedNodeL
             if (FormEditorItem *item = scene()->itemForQmlItemNode(qmlItemNode)) {
                 scene()->synchronizeParent(qmlItemNode);
                 itemNodeList.append(item);
+                if (isFlow && qmlItemNode.isFlowItem())
+                    updateTransitions(scene(), qmlItemNode);
             }
         }
     }
@@ -832,8 +848,42 @@ bool FormEditorView::isMoveToolAvailable() const
 
 void FormEditorView::resetNodeInstanceView()
 {
+    ModelNode currentStateNode = currentState().modelNode();
     setCurrentStateNode(rootModelNode());
     resetPuppet();
+    setCurrentStateNode(currentStateNode);
+}
+
+void FormEditorView::addOrRemoveFormEditorItem(const ModelNode &node)
+{
+    // If node is not connected to scene root, don't do anything yet to avoid duplicated effort,
+    // as any removal or addition will remove or add descendants as well.
+    if (!node.isInHierarchy())
+        return;
+
+    QmlItemNode itemNode(node);
+
+    auto removeItemFromScene = [this, &itemNode]() {
+        if (FormEditorItem *item = m_scene->itemForQmlItemNode(itemNode)) {
+            QList<FormEditorItem *> removed = scene()->itemsForQmlItemNodes(itemNode.allSubModelNodes());
+            removed.append(item);
+            m_currentTool->itemsAboutToRemoved(removed);
+            removeNodeFromScene(itemNode);
+        }
+    };
+    if (hasNodeSourceOrNonItemParent(node)) {
+        removeItemFromScene();
+    } else if (itemNode.isValid()) {
+        if (node.nodeSourceType() == ModelNode::NodeWithoutSource) {
+            if (!m_scene->itemForQmlItemNode(itemNode)) {
+                setupFormEditorItemTree(itemNode);
+                // Simulate selection change to refresh tools
+                selectedNodesChanged(selectedModelNodes(), {});
+            }
+        } else {
+            removeItemFromScene();
+        }
+    }
 }
 
 void FormEditorView::reset()
