@@ -23,25 +23,29 @@
 **
 ****************************************************************************/
 #include "itemlibraryassetimporter.h"
+#include "assetimportupdatedialog.h"
 #include "qmldesignerplugin.h"
 #include "qmldesignerconstants.h"
 
 #include "rewriterview.h"
 #include "model.h"
 #include "puppetcreator.h"
+#include "rewritertransaction.h"
+#include "rewritingexception.h"
 
-#include <QtCore/qdir.h>
-#include <QtCore/qdiriterator.h>
-#include <QtCore/qsavefile.h>
-#include <QtCore/qfile.h>
-#include <QtCore/qloggingcategory.h>
-#include <QtCore/qtemporarydir.h>
-#include <QtWidgets/qapplication.h>
-#include <QtWidgets/qmessagebox.h>
+#include <utils/algorithm.h>
 
-#ifdef IMPORT_QUICK3D_ASSETS
-#include <QtQuick3DAssetImport/private/qssgassetimportmanager_p.h>
-#endif
+#include <QApplication>
+#include <QDir>
+#include <QDirIterator>
+#include <QFile>
+#include <QJsonDocument>
+#include <QJsonObject>
+#include <QLoggingCategory>
+#include <QMessageBox>
+#include <QPushButton>
+#include <QSaveFile>
+#include <QTemporaryDir>
 
 namespace
 {
@@ -53,9 +57,6 @@ namespace QmlDesigner {
 ItemLibraryAssetImporter::ItemLibraryAssetImporter(QObject *parent) :
     QObject (parent)
 {
-#ifdef IMPORT_QUICK3D_ASSETS
-    m_quick3DAssetImporter.reset(new QSSGAssetImportManager);
-#endif
 }
 
 ItemLibraryAssetImporter::~ItemLibraryAssetImporter() {
@@ -66,14 +67,14 @@ ItemLibraryAssetImporter::~ItemLibraryAssetImporter() {
 void ItemLibraryAssetImporter::importQuick3D(const QStringList &inputFiles,
                                              const QString &importPath,
                                              const QVector<QJsonObject> &options,
-                                             const QHash<QString, int> &extToImportOptionsMap)
+                                             const QHash<QString, int> &extToImportOptionsMap,
+                                             const QSet<QString> &preselectedFilesForOverwrite)
 {
     if (m_isImporting)
         cancelImport();
     reset();
     m_isImporting = true;
 
-#ifdef IMPORT_QUICK3D_ASSETS
     if (!m_tempDir->isValid()) {
         addError(tr("Could not create a temporary directory for import."));
         notifyFinished();
@@ -82,27 +83,30 @@ void ItemLibraryAssetImporter::importQuick3D(const QStringList &inputFiles,
 
     m_importPath = importPath;
 
-    parseFiles(inputFiles, options, extToImportOptionsMap);
+    parseFiles(inputFiles, options, extToImportOptionsMap, preselectedFilesForOverwrite);
 
     if (!isCancelled()) {
-        // Wait for icon generation processes to finish
+        const auto parseData = m_parseData;
+        for (const auto &pd : parseData) {
+            if (!startImportProcess(pd)) {
+                addError(tr("Failed to start import 3D asset process."),
+                         pd.sourceInfo.absoluteFilePath());
+                m_parseData.remove(pd.importId);
+            }
+        }
+    }
+
+    if (!isCancelled()) {
+        // Wait for puppet processes to finish
         if (m_qmlPuppetProcesses.empty()) {
-            finalizeQuick3DImport();
+            postImport();
         } else {
             m_qmlPuppetCount = static_cast<int>(m_qmlPuppetProcesses.size());
-            const QString progressTitle = tr("Generating icons.");
+            const QString progressTitle = tr("Importing 3D assets.");
             addInfo(progressTitle);
             notifyProgress(0, progressTitle);
         }
     }
-#else
-    Q_UNUSED(inputFiles)
-    Q_UNUSED(importPath)
-    Q_UNUSED(options)
-    Q_UNUSED(extToImportOptionsMap)
-    addError(tr("Importing 3D assets requires building against Qt Quick 3D module."));
-    notifyFinished();
-#endif
 }
 
 bool ItemLibraryAssetImporter::isImporting() const
@@ -135,67 +139,47 @@ void ItemLibraryAssetImporter::addInfo(const QString &infoMsg, const QString &sr
     emit infoReported(infoMsg, srcPath);
 }
 
-bool ItemLibraryAssetImporter::isQuick3DAsset(const QString &fileName) const
+void ItemLibraryAssetImporter::importProcessFinished(int exitCode, QProcess::ExitStatus exitStatus)
 {
-#ifdef IMPORT_QUICK3D_ASSETS
-    static QStringList quick3DExt;
-    if (quick3DExt.isEmpty()) {
-        const auto exts = m_quick3DAssetImporter->getSupportedExtensions();
-        for (const auto &ext : exts)
-            quick3DExt << ext;
+    Q_UNUSED(exitStatus)
+
+    ++m_qmlImportFinishedCount;
+
+    m_qmlPuppetProcesses.erase(
+        std::remove_if(m_qmlPuppetProcesses.begin(), m_qmlPuppetProcesses.end(),
+                       [&](const auto &entry) {
+        return !entry || entry->state() == QProcess::NotRunning;
+    }));
+
+    if (m_parseData.contains(-exitCode)) {
+        const ParseData pd = m_parseData.take(-exitCode);
+        addError(tr("Asset import process failed for: \"%1\".").arg(pd.sourceInfo.absoluteFilePath()));
+    }
+
+    if (m_qmlImportFinishedCount == m_qmlPuppetCount) {
+        notifyProgress(100);
+        QTimer::singleShot(0, this, &ItemLibraryAssetImporter::postImport);
+    } else {
+        notifyProgress(int(100. * (double(m_qmlImportFinishedCount) / double(m_qmlPuppetCount))));
     }
-    return quick3DExt.contains(QFileInfo(fileName).suffix().toLower());
-#else
-    Q_UNUSED(fileName)
-    return false;
-#endif
 }
 
-QVariantMap ItemLibraryAssetImporter::supportedOptions(const QString &modelFile) const
-{
-#ifdef IMPORT_QUICK3D_ASSETS
-    return m_quick3DAssetImporter->getOptionsForFile(modelFile);
-#else
-    Q_UNUSED(modelFile)
-    return {};
-#endif
-}
-
-QHash<QString, QVariantMap> ItemLibraryAssetImporter::allOptions() const
-{
-#ifdef IMPORT_QUICK3D_ASSETS
-    return m_quick3DAssetImporter->getAllOptions();
-#else
-    return {};
-#endif
-}
-
-QHash<QString, QStringList> ItemLibraryAssetImporter::supportedExtensions() const
-{
-#ifdef IMPORT_QUICK3D_ASSETS
-    return m_quick3DAssetImporter->getSupportedExtensions();
-#else
-    return {};
-#endif
-}
-
-void ItemLibraryAssetImporter::processFinished(int exitCode, QProcess::ExitStatus exitStatus)
+void ItemLibraryAssetImporter::iconProcessFinished(int exitCode, QProcess::ExitStatus exitStatus)
 {
     Q_UNUSED(exitCode)
     Q_UNUSED(exitStatus)
 
     m_qmlPuppetProcesses.erase(
-        std::remove_if(m_qmlPuppetProcesses.begin(), m_qmlPuppetProcesses.end(), [&](const auto &entry) {
+        std::remove_if(m_qmlPuppetProcesses.begin(), m_qmlPuppetProcesses.end(),
+                       [&](const auto &entry) {
         return !entry || entry->state() == QProcess::NotRunning;
     }));
 
-    const QString progressTitle = tr("Generating icons.");
     if (m_qmlPuppetProcesses.empty()) {
-        notifyProgress(100, progressTitle);
-        finalizeQuick3DImport();
+        notifyProgress(100);
+        QTimer::singleShot(0, this, &ItemLibraryAssetImporter::finalizeQuick3DImport);
     } else {
-        notifyProgress(int(100. * (1. - double(m_qmlPuppetCount) / double(m_qmlPuppetProcesses.size()))),
-                       progressTitle);
+        notifyProgress(int(100. * (1. - (double(m_qmlPuppetProcesses.size()) / double(m_qmlPuppetCount)))));
     }
 }
 
@@ -210,19 +194,21 @@ void ItemLibraryAssetImporter::reset()
     m_isImporting = false;
     m_cancelled = false;
 
-#ifdef IMPORT_QUICK3D_ASSETS
     delete m_tempDir;
     m_tempDir = new QTemporaryDir;
     m_importFiles.clear();
     m_overwrittenImports.clear();
     m_qmlPuppetProcesses.clear();
     m_qmlPuppetCount = 0;
-#endif
+    m_qmlImportFinishedCount = 0;
+    m_parseData.clear();
+    m_requiredImports.clear();
 }
 
 void ItemLibraryAssetImporter::parseFiles(const QStringList &filePaths,
                                           const QVector<QJsonObject> &options,
-                                          const QHash<QString, int> &extToImportOptionsMap)
+                                          const QHash<QString, int> &extToImportOptionsMap,
+                                          const QSet<QString> &preselectedFilesForOverwrite)
 {
     if (isCancelled())
         return;
@@ -235,99 +221,124 @@ void ItemLibraryAssetImporter::parseFiles(const QStringList &filePaths,
         notifyProgress(qRound(quota * (count + value)), progressTitle);
     };
     for (const QString &file : filePaths) {
-        if (isCancelled())
-            return;
-        if (isQuick3DAsset(file)) {
-            int index = extToImportOptionsMap.value(QFileInfo(file).suffix());
-            parseQuick3DAsset(file, options[index].toVariantMap());
+        int index = extToImportOptionsMap.value(QFileInfo(file).suffix());
+        ParseData pd;
+        pd.options = options[index];
+        if (preParseQuick3DAsset(file, pd, preselectedFilesForOverwrite)) {
+            pd.importId = ++m_importIdCounter;
+            m_parseData.insert(pd.importId, pd);
         }
         notifyProgress(qRound(++count * quota), progressTitle);
     }
-    notifyProgress(100, progressTitle);
 }
 
-void ItemLibraryAssetImporter::parseQuick3DAsset(const QString &file, const QVariantMap &options)
+bool ItemLibraryAssetImporter::preParseQuick3DAsset(const QString &file, ParseData &pd,
+                                                    const QSet<QString> &preselectedFilesForOverwrite)
 {
-#ifdef IMPORT_QUICK3D_ASSETS
-    addInfo(tr("Parsing 3D Model"), file);
+    pd.targetDir = QDir(m_importPath);
+    pd.outDir = QDir(m_tempDir->path());
+    pd.sourceInfo = QFileInfo(file);
+    pd.assetName = pd.sourceInfo.completeBaseName();
 
-    QString errorString;
-
-    QDir targetDir(m_importPath);
-    QDir outDir(m_tempDir->path());
-    QFileInfo sourceInfo(file);
-    QString assetName = sourceInfo.completeBaseName();
-
-    if (!assetName.isEmpty()) {
+    if (!pd.assetName.isEmpty()) {
         // Fix name so it plays nice with imports
-        for (QChar &currentChar : assetName) {
+        for (QChar &currentChar : pd.assetName) {
             if (!currentChar.isLetter() && !currentChar.isDigit())
                 currentChar = QLatin1Char('_');
         }
-        const QChar firstChar = assetName[0];
+        const QChar firstChar = pd.assetName[0];
         if (firstChar.isDigit())
-            assetName[0] = QLatin1Char('_');
+            pd.assetName[0] = QLatin1Char('_');
         if (firstChar.isLower())
-            assetName[0] = firstChar.toUpper();
+            pd.assetName[0] = firstChar.toUpper();
     }
 
-    QString targetDirPath = targetDir.filePath(assetName);
+    pd.targetDirPath = pd.targetDir.filePath(pd.assetName);
 
-    if (outDir.exists(assetName)) {
-        addWarning(tr("Skipped import of duplicate asset: \"%1\"").arg(assetName));
-        return;
+    if (pd.outDir.exists(pd.assetName)) {
+        addWarning(tr("Skipped import of duplicate asset: \"%1\".").arg(pd.assetName));
+        return false;
     }
 
-    QString originalAssetName = assetName;
-    if (targetDir.exists(assetName)) {
+    pd.originalAssetName = pd.assetName;
+    if (pd.targetDir.exists(pd.assetName)) {
         // If we have a file system with case insensitive filenames, assetName may be
         // different from the existing name. Modify assetName to ensure exact match to
         // the overwritten old asset capitalization
-        const QStringList assetDirs = targetDir.entryList({assetName}, QDir::Dirs);
+        const QStringList assetDirs = pd.targetDir.entryList({pd.assetName}, QDir::Dirs);
         if (assetDirs.size() == 1) {
-            assetName = assetDirs[0];
-            targetDirPath = targetDir.filePath(assetName);
+            pd.assetName = assetDirs[0];
+            pd.targetDirPath = pd.targetDir.filePath(pd.assetName);
         }
-        if (!confirmAssetOverwrite(assetName)) {
-            addWarning(tr("Skipped import of existing asset: \"%1\"").arg(assetName));
-            return;
+        OverwriteResult result = preselectedFilesForOverwrite.isEmpty()
+                ? confirmAssetOverwrite(pd.assetName)
+                : OverwriteResult::Update;
+        if (result == OverwriteResult::Skip) {
+            addWarning(tr("Skipped import of existing asset: \"%1\".").arg(pd.assetName));
+            return false;
+        } else if (result == OverwriteResult::Update) {
+            // Add generated icons and existing source asset file, as those will always need
+            // to be overwritten
+            QSet<QString> alwaysOverwrite;
+            QString iconPath = pd.targetDirPath + '/' + Constants::QUICK_3D_ASSET_ICON_DIR;
+            // Note: Despite the name, QUICK_3D_ASSET_LIBRARY_ICON_SUFFIX is not a traditional file
+            // suffix. It's guaranteed to be in the generated icon filename, though.
+            QStringList filters {QStringLiteral("*%1*").arg(Constants::QUICK_3D_ASSET_LIBRARY_ICON_SUFFIX)};
+            QDirIterator iconIt(iconPath, filters, QDir::Files);
+            while (iconIt.hasNext()) {
+                iconIt.next();
+                alwaysOverwrite.insert(iconIt.fileInfo().absoluteFilePath());
+            }
+            alwaysOverwrite.insert(sourceSceneTargetFilePath(pd));
+            alwaysOverwrite.insert(pd.targetDirPath + '/' + Constants::QUICK_3D_ASSET_IMPORT_DATA_NAME);
+
+            Internal::AssetImportUpdateDialog dlg {pd.targetDirPath,
+                                                   preselectedFilesForOverwrite,
+                                                   alwaysOverwrite,
+                                                   qobject_cast<QWidget *>(parent())};
+            int exitVal = dlg.exec();
+
+            QStringList overwriteFiles;
+            if (exitVal == QDialog::Accepted)
+                overwriteFiles = dlg.selectedFiles();
+            if (!overwriteFiles.isEmpty()) {
+                overwriteFiles.append(Utils::toList(alwaysOverwrite));
+                m_overwrittenImports.insert(pd.targetDirPath, overwriteFiles);
+            } else {
+                addWarning(tr("No files selected for overwrite, skipping import: \"%1\".").arg(pd.assetName));
+                return false;
+            }
+
+        } else {
+            m_overwrittenImports.insert(pd.targetDirPath, {});
         }
-        m_overwrittenImports << targetDirPath;
     }
 
-    outDir.mkpath(assetName);
+    pd.outDir.mkpath(pd.assetName);
 
-    if (!outDir.cd(assetName)) {
-        addError(tr("Could not access temporary asset directory: \"%1\"")
-                 .arg(outDir.filePath(assetName)));
-        return;
+    if (!pd.outDir.cd(pd.assetName)) {
+        addError(tr("Could not access temporary asset directory: \"%1\".")
+                 .arg(pd.outDir.filePath(pd.assetName)));
+        return false;
     }
+    return true;
+}
 
-    addInfo(tr("Generating 3D assets for: \"%1\"").arg(targetDir.absoluteFilePath(assetName)));
-
-    if (m_quick3DAssetImporter->importFile(
-                sourceInfo.absoluteFilePath(), outDir, options, &errorString)
-            != QSSGAssetImportManager::ImportState::Success) {
-        addError(tr("Failed to import 3D asset with error: %1").arg(errorString),
-                 sourceInfo.absoluteFilePath());
-        return;
-    }
-
-    // The importer is reset after every import to avoid issues with it caching various things
-    m_quick3DAssetImporter.reset(new QSSGAssetImportManager);
-
-    if (originalAssetName != assetName) {
+void ItemLibraryAssetImporter::postParseQuick3DAsset(const ParseData &pd)
+{
+    QDir outDir = pd.outDir;
+    if (pd.originalAssetName != pd.assetName) {
         // Fix the generated qml file name
-        const QString assetQml = originalAssetName + ".qml";
+        const QString assetQml = pd.originalAssetName + ".qml";
         if (outDir.exists(assetQml))
-            outDir.rename(assetQml, assetName + ".qml");
+            outDir.rename(assetQml, pd.assetName + ".qml");
     }
 
     QHash<QString, QString> assetFiles;
     const int outDirPathSize = outDir.path().size();
     auto insertAsset = [&](const QString &filePath) {
         QString targetPath = filePath.mid(outDirPathSize);
-        targetPath.prepend(targetDirPath);
+        targetPath.prepend(pd.targetDirPath);
         assetFiles.insert(filePath, targetPath);
     };
 
@@ -348,8 +359,11 @@ void ItemLibraryAssetImporter::parseQuick3DAsset(const QString &file, const QVar
                 qmlInfo.append("module ");
                 qmlInfo.append(m_importPath.split('/').last());
                 qmlInfo.append(".");
-                qmlInfo.append(assetName);
+                qmlInfo.append(pd.assetName);
                 qmlInfo.append('\n');
+                m_requiredImports.append(Import::createLibraryImport(
+                                             QStringLiteral("%1.%2").arg(pd.targetDir.dirName(),
+                                                                         pd.assetName), version));
                 while (qmlIt.hasNext()) {
                     qmlIt.next();
                     QFileInfo fi = QFileInfo(qmlIt.filePath());
@@ -381,8 +395,26 @@ void ItemLibraryAssetImporter::parseQuick3DAsset(const QString &file, const QVar
                                 out << "canBeDroppedInFormEditor: false" << Qt::endl;
                                 out << "canBeDroppedInView3D: true" << Qt::endl;
                                 file.close();
+
+                                // Add quick3D import unless it is already added
+                                if (m_requiredImports.first().url() != "QtQuick3D") {
+                                    QByteArray import3dStr{"import QtQuick3D"};
+                                    int importIdx = content.indexOf(import3dStr);
+                                    if (importIdx != -1 && importIdx < braceIdx) {
+                                        importIdx += import3dStr.size();
+                                        int nlIdx = content.indexOf('\n', importIdx);
+                                        QByteArray versionStr = content.mid(importIdx, nlIdx - importIdx).trimmed();
+                                        // There could be 'as abc' after version, so just take first part
+                                        QList<QByteArray> parts = versionStr.split(' ');
+                                        QString impVersion;
+                                        if (parts.size() >= 1)
+                                            impVersion = QString::fromUtf8(parts[0]);
+                                        m_requiredImports.prepend(Import::createLibraryImport(
+                                                                     "QtQuick3D", impVersion));
+                                    }
+                                }
                             }
-                            if (generateComponentIcon(24, iconFileName, qmlIt.filePath())) {
+                            if (startIconProcess(24, iconFileName, qmlIt.filePath())) {
                                 // Since icon is generated by external process, the file won't be
                                 // ready for asset gathering below, so assume its generation succeeds
                                 // and add it now.
@@ -395,11 +427,23 @@ void ItemLibraryAssetImporter::parseQuick3DAsset(const QString &file, const QVar
                 qmldirFile.write(qmlInfo.toUtf8());
                 qmldirFile.commit();
             } else {
-                addError(tr("Failed to create qmldir file for asset: \"%1\"").arg(assetName));
+                addError(tr("Failed to create qmldir file for asset: \"%1\".").arg(pd.assetName));
             }
         }
     }
 
+    // Generate import metadata file
+    const QString sourcePath = pd.sourceInfo.absoluteFilePath();
+    QString importDataFileName = outDir.absoluteFilePath(Constants::QUICK_3D_ASSET_IMPORT_DATA_NAME);
+    QSaveFile importDataFile(importDataFileName);
+    if (importDataFile.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
+        QJsonObject optObj;
+        optObj.insert(Constants::QUICK_3D_ASSET_IMPORT_DATA_OPTIONS_KEY, pd.options);
+        optObj.insert(Constants::QUICK_3D_ASSET_IMPORT_DATA_SOURCE_KEY, sourcePath);
+        importDataFile.write(QJsonDocument{optObj}.toJson());
+        importDataFile.commit();
+    }
+
     // Gather all generated files
     QDirIterator dirIt(outDir.path(), QDir::Files, QDirIterator::Subdirectories);
     while (dirIt.hasNext()) {
@@ -408,30 +452,34 @@ void ItemLibraryAssetImporter::parseQuick3DAsset(const QString &file, const QVar
     }
 
     // Copy the original asset into a subdirectory
-    assetFiles.insert(sourceInfo.absoluteFilePath(),
-                      targetDirPath + QStringLiteral("/source scene/") + sourceInfo.fileName());
+    assetFiles.insert(sourcePath, sourceSceneTargetFilePath(pd));
     m_importFiles.insert(assetFiles);
-
-#else
-    Q_UNUSED(file)
-    Q_UNUSED(options)
-#endif
 }
 
 void ItemLibraryAssetImporter::copyImportedFiles()
 {
-#ifdef IMPORT_QUICK3D_ASSETS
     if (!m_overwrittenImports.isEmpty()) {
         const QString progressTitle = tr("Removing old overwritten assets.");
         addInfo(progressTitle);
         notifyProgress(0, progressTitle);
 
         int counter = 0;
-        for (const QString &dirPath : m_overwrittenImports) {
-            QDir dir(dirPath);
-            if (dir.exists())
-                dir.removeRecursively();
+        auto it = m_overwrittenImports.constBegin();
+        while (it != m_overwrittenImports.constEnd()) {
+            QDir dir(it.key());
+            if (dir.exists()) {
+                const auto &overwrittenFiles = it.value();
+                if (overwrittenFiles.isEmpty()) {
+                    // Overwrite entire import
+                    dir.removeRecursively();
+                } else {
+                    // Overwrite just selected files
+                    for (const auto &fileName : overwrittenFiles)
+                        QFile::remove(fileName);
+                }
+            }
             notifyProgress((100 * ++counter) / m_overwrittenImports.size(), progressTitle);
+            ++it;
         }
     }
 
@@ -449,7 +497,7 @@ void ItemLibraryAssetImporter::copyImportedFiles()
             // by filesystem watchers.
             QHash<QString, QString>::const_iterator it = assetFiles.begin();
             while (it != assetFiles.end()) {
-                if (QFileInfo::exists(it.key())) {
+                if (QFileInfo::exists(it.key()) && !QFileInfo::exists(it.value())) {
                     QDir targetDir = QFileInfo(it.value()).dir();
                     if (!targetDir.exists())
                         targetDir.mkpath(".");
@@ -461,31 +509,83 @@ void ItemLibraryAssetImporter::copyImportedFiles()
         }
         notifyProgress(100, progressTitle);
     }
-#endif
 }
 
-void ItemLibraryAssetImporter::notifyProgress(int value, const QString &text) const
+void ItemLibraryAssetImporter::notifyProgress(int value, const QString &text)
 {
-    emit progressChanged(value, text);
+    m_progressTitle = text;
+    emit progressChanged(value, m_progressTitle);
     keepUiAlive();
 }
 
+void ItemLibraryAssetImporter::notifyProgress(int value)
+{
+    notifyProgress(value, m_progressTitle);
+}
+
 void ItemLibraryAssetImporter::keepUiAlive() const
 {
     QApplication::processEvents();
 }
 
-bool ItemLibraryAssetImporter::confirmAssetOverwrite(const QString &assetName)
+ItemLibraryAssetImporter::OverwriteResult ItemLibraryAssetImporter::confirmAssetOverwrite(const QString &assetName)
 {
     const QString title = tr("Overwrite Existing Asset?");
-    const QString question = tr("Asset already exists. Overwrite?\n\"%1\"").arg(assetName);
-    return QMessageBox::question(qobject_cast<QWidget *>(parent()),
-                                 title, question,
-                                 QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes;
+    const QString question = tr("Asset already exists. Overwrite existing or skip?\n\"%1\"").arg(assetName);
+
+    QMessageBox msgBox {QMessageBox::Question, title, question, QMessageBox::NoButton,
+                        qobject_cast<QWidget *>(parent())};
+    QPushButton *updateButton = msgBox.addButton(tr("Overwrite Selected Files"), QMessageBox::NoRole);
+    QPushButton *overwriteButton = msgBox.addButton(tr("Overwrite All Files"), QMessageBox::NoRole);
+    QPushButton *skipButton = msgBox.addButton(tr("Skip"), QMessageBox::NoRole);
+    msgBox.setDefaultButton(overwriteButton);
+    msgBox.setEscapeButton(skipButton);
+
+    msgBox.exec();
+
+    if (msgBox.clickedButton() == updateButton)
+        return OverwriteResult::Update;
+    else if (msgBox.clickedButton() == overwriteButton)
+        return OverwriteResult::Overwrite;
+    return OverwriteResult::Skip;
 }
 
-bool ItemLibraryAssetImporter::generateComponentIcon(int size, const QString &iconFile,
-                                                     const QString &iconSource)
+bool ItemLibraryAssetImporter::startImportProcess(const ParseData &pd)
+{
+    auto doc = QmlDesignerPlugin::instance()->currentDesignDocument();
+    Model *model = doc ? doc->currentModel() : nullptr;
+
+    if (model) {
+        PuppetCreator puppetCreator(doc->currentTarget(), model);
+        puppetCreator.createQml2PuppetExecutableIfMissing();
+        QStringList puppetArgs;
+        QJsonDocument optDoc(pd.options);
+
+        puppetArgs << "--import3dAsset" << pd.sourceInfo.absoluteFilePath()
+                   << pd.outDir.absolutePath() << QString::number(pd.importId)
+                   << QString::fromUtf8(optDoc.toJson());
+
+        QProcessUniquePointer process = puppetCreator.createPuppetProcess(
+            "custom",
+            {},
+            std::function<void()>(),
+            [&](int exitCode, QProcess::ExitStatus exitStatus) {
+                importProcessFinished(exitCode, exitStatus);
+            },
+            puppetArgs);
+
+        if (process->waitForStarted(5000)) {
+            m_qmlPuppetProcesses.push_back(std::move(process));
+            return true;
+        } else {
+            process.reset();
+        }
+    }
+    return false;
+}
+
+bool ItemLibraryAssetImporter::startIconProcess(int size, const QString &iconFile,
+                                                const QString &iconSource)
 {
     auto doc = QmlDesignerPlugin::instance()->currentDesignDocument();
     Model *model = doc ? doc->currentModel() : nullptr;
@@ -500,7 +600,7 @@ bool ItemLibraryAssetImporter::generateComponentIcon(int size, const QString &ic
             {},
             std::function<void()>(),
             [&](int exitCode, QProcess::ExitStatus exitStatus) {
-                processFinished(exitCode, exitStatus);
+                iconProcessFinished(exitCode, exitStatus);
             },
             puppetArgs);
 
@@ -514,9 +614,30 @@ bool ItemLibraryAssetImporter::generateComponentIcon(int size, const QString &ic
     return false;
 }
 
+void ItemLibraryAssetImporter::postImport()
+{
+    Q_ASSERT(m_qmlPuppetProcesses.empty());
+
+    if (!isCancelled()) {
+        for (const auto &pd : qAsConst(m_parseData))
+            postParseQuick3DAsset(pd);
+    }
+
+    if (!isCancelled()) {
+        // Wait for icon generation processes to finish
+        if (m_qmlPuppetProcesses.empty()) {
+            finalizeQuick3DImport();
+        } else {
+            m_qmlPuppetCount = static_cast<int>(m_qmlPuppetProcesses.size());
+            const QString progressTitle = tr("Generating icons.");
+            addInfo(progressTitle);
+            notifyProgress(0, progressTitle);
+        }
+    }
+}
+
 void ItemLibraryAssetImporter::finalizeQuick3DImport()
 {
-#ifdef IMPORT_QUICK3D_ASSETS
     if (!isCancelled()) {
         // Don't allow cancel anymore as existing asset overwrites are not trivially recoverable.
         // Also, on Windows at least you can't delete a subdirectory of a watched directory,
@@ -543,10 +664,32 @@ void ItemLibraryAssetImporter::finalizeQuick3DImport()
             timer->callOnTimeout([this, timer, progressTitle, model, doc]() {
                 if (!isCancelled()) {
                     notifyProgress(++counter * 5, progressTitle);
-                    if (counter == 10) {
+                    if (counter < 10) {
+                        // Do not proceed while application isn't active as the filesystem
+                        // watcher qmljs uses won't trigger unless application is active
+                        if (QApplication::applicationState() != Qt::ApplicationActive)
+                            --counter;
+                    } else if (counter == 10) {
                         model->rewriterView()->textModifier()->replace(0, 0, {});
                     } else if (counter == 19) {
-                        doc->updateSubcomponentManager();
+                        try {
+                            const QList<Import> currentImports = model->imports();
+                            QList<Import> newImportsToAdd;
+                            for (auto &imp : qAsConst(m_requiredImports)) {
+                                if (!currentImports.contains(imp))
+                                    newImportsToAdd.append(imp);
+                            }
+                            if (!newImportsToAdd.isEmpty()) {
+                                RewriterTransaction transaction
+                                        = model->rewriterView()->beginRewriterTransaction(
+                                            QByteArrayLiteral("ItemLibraryAssetImporter::finalizeQuick3DImport"));
+
+                                model->changeImports(newImportsToAdd, {});
+                                transaction.commit();
+                            }
+                        } catch (const RewritingException &e) {
+                            addError(tr("Failed to update imports: %1").arg(e.description()));
+                        }
                     } else if (counter >= 20) {
                         if (!m_overwrittenImports.isEmpty())
                             model->rewriterView()->emitCustomNotification("asset_import_update");
@@ -562,7 +705,11 @@ void ItemLibraryAssetImporter::finalizeQuick3DImport()
             notifyFinished();
         }
     }
-#endif
+}
+
+QString ItemLibraryAssetImporter::sourceSceneTargetFilePath(const ParseData &pd)
+{
+    return pd.targetDirPath + QStringLiteral("/source scene/") + pd.sourceInfo.fileName();
 }
 
 bool ItemLibraryAssetImporter::isCancelled() const
