@@ -32,12 +32,16 @@
 
 #include <QAbstractItemModel>
 #include <QPointer>
+#include <QDateTime>
+
+QT_FORWARD_DECLARE_CLASS(QPixmap)
 
 namespace QmlDesigner {
 
 class Model;
 class NavigatorView;
 class ModelNode;
+class DesignerActionManager;
 
 class NavigatorTreeModel : public QAbstractItemModel, public NavigatorModelInterface
 {
@@ -45,6 +49,14 @@ class NavigatorTreeModel : public QAbstractItemModel, public NavigatorModelInter
 
 public:
 
+    enum ColumnType {
+        Name = 0,
+        Alias,
+        Visibility,
+        Lock,
+        Count
+    };
+
     explicit NavigatorTreeModel(QObject *parent = nullptr);
     ~NavigatorTreeModel() override;
 
@@ -87,19 +99,42 @@ public:
     void notifyModelNodesRemoved(const QList<ModelNode> &modelNodes) override;
     void notifyModelNodesInserted(const QList<ModelNode> &modelNodes) override;
     void notifyModelNodesMoved(const QList<ModelNode> &modelNodes) override;
+    void notifyIconsChanged() override;
     void setFilter(bool showOnlyVisibleItems) override;
+    void setOrder(bool reverseItemOrder) override;
     void resetModel() override;
 
+    void updateToolTipPixmap(const ModelNode &node, const QPixmap &pixmap);
+
+signals:
+    void toolTipPixmapUpdated(const QString &id, const QPixmap &pixmap) const;
+
 private:
-    void moveNodesInteractive(NodeAbstractProperty &parentProperty, const QList<ModelNode> &modelNodes, int targetIndex);
+    void moveNodesInteractive(NodeAbstractProperty &parentProperty, const QList<ModelNode> &modelNodes,
+                              int targetIndex, bool executeInTransaction = true);
     void handleInternalDrop(const QMimeData *mimeData, int rowNumber, const QModelIndex &dropModelIndex);
     void handleItemLibraryItemDrop(const QMimeData *mimeData, int rowNumber, const QModelIndex &dropModelIndex);
-    void handleItemLibraryImageDrop(const QMimeData *mimeData, int rowNumber, const QModelIndex &dropModelIndex);
+    ModelNode handleItemLibraryImageDrop(const QString &imagePath, NodeAbstractProperty targetProperty,
+                                         const QModelIndex &rowModelIndex);
+    ModelNode handleItemLibraryFontDrop(const QString &fontFamily, NodeAbstractProperty targetProperty,
+                                        const QModelIndex &rowModelIndex);
+    ModelNode handleItemLibraryShaderDrop(const QString &shaderPath, bool isFragShader,
+                                          NodeAbstractProperty targetProperty, const QModelIndex &rowModelIndex);
+    ModelNode handleItemLibrarySoundDrop(const QString &soundPath, NodeAbstractProperty targetProperty,
+                                         const QModelIndex &rowModelIndex);
+    ModelNode handleItemLibraryTexture3dDrop(const QString &tex3DPath, NodeAbstractProperty targetProperty,
+                                             const QModelIndex &rowModelIndex);
+    bool dropAsImage3dTexture(const ModelNode &targetNode, const NodeAbstractProperty &targetProp,
+                              const QString &imagePath, ModelNode &newNode);
+    ModelNode createTextureNode(const NodeAbstractProperty &targetProp, const QString &imagePath);
     QList<QPersistentModelIndex> nodesToPersistentIndex(const QList<ModelNode> &modelNodes);
+    void addImport(const QString &importName);
 
     QPointer<NavigatorView> m_view;
     mutable QHash<ModelNode, QModelIndex> m_nodeIndexHash;
     bool m_showOnlyVisibleItems = true;
+    bool m_reverseItemOrder = false;
+    DesignerActionManager *m_actionManager = nullptr;
 };
 
 } // namespace QmlDesigner
