@@ -28,6 +28,7 @@
 #include "navigatorwidget.h"
 #include "qmldesignerconstants.h"
 #include "qmldesignericons.h"
+#include "qmldesignerplugin.h"
 
 #include "nameitemdelegate.h"
 #include "iconcheckboxitemdelegate.h"
@@ -40,17 +41,24 @@
 #include <variantproperty.h>
 #include <qmlitemnode.h>
 #include <rewritingexception.h>
+#include <nodeinstanceview.h>
+#include <theme.h>
 
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/icore.h>
 
+#include <projectexplorer/project.h>
+#include <projectexplorer/session.h>
+#include <projectexplorer/projectnodes.h>
+
 #include <utils/algorithm.h>
 #include <utils/icon.h>
 #include <utils/utilsicons.h>
+#include <utils/stylehelper.h>
 
 #include <QHeaderView>
 #include <QTimer>
-
+#include <QPixmap>
 
 static inline void setScenePos(const QmlDesigner::ModelNode &modelNode,const QPointF &pos)
 {
@@ -68,61 +76,43 @@ static inline void setScenePos(const QmlDesigner::ModelNode &modelNode,const QPo
     }
 }
 
+static inline void moveNodesUp(const QList<QmlDesigner::ModelNode> &nodes)
+{
+    for (const auto &node : nodes) {
+        if (!node.isRootNode() && node.parentProperty().isNodeListProperty()) {
+            int oldIndex = node.parentProperty().indexOf(node);
+            int index = oldIndex;
+            index--;
+            if (index < 0)
+                index = node.parentProperty().count() - 1; //wrap around
+            if (oldIndex != index)
+                node.parentProperty().toNodeListProperty().slide(oldIndex, index);
+        }
+    }
+}
+
+static inline void moveNodesDown(const QList<QmlDesigner::ModelNode> &nodes)
+{
+    for (const auto &node : nodes) {
+        if (!node.isRootNode() && node.parentProperty().isNodeListProperty()) {
+            int oldIndex = node.parentProperty().indexOf(node);
+            int index = oldIndex;
+            index++;
+            if (index >= node.parentProperty().count())
+                index = 0; //wrap around
+            if (oldIndex != index)
+                node.parentProperty().toNodeListProperty().slide(oldIndex, index);
+        }
+    }
+}
+
 namespace QmlDesigner {
 
 NavigatorView::NavigatorView(QObject* parent) :
     AbstractView(parent),
-    m_blockSelectionChangedSignal(false),
-    m_widget(new NavigatorWidget(this)),
-    m_treeModel(new NavigatorTreeModel(this))
+    m_blockSelectionChangedSignal(false)
 {
-#ifndef QMLDESIGNER_TEST
-    auto navigatorContext = new Internal::NavigatorContext(m_widget.data());
-    Core::ICore::addContextObject(navigatorContext);
-#endif
 
-    m_treeModel->setView(this);
-    m_widget->setTreeModel(m_treeModel.data());
-    m_currentModelInterface = m_treeModel;
-
-    connect(treeWidget()->selectionModel(), &QItemSelectionModel::selectionChanged, this, &NavigatorView::changeSelection);
-
-    connect(m_widget.data(), &NavigatorWidget::leftButtonClicked, this, &NavigatorView::leftButtonClicked);
-    connect(m_widget.data(), &NavigatorWidget::rightButtonClicked, this, &NavigatorView::rightButtonClicked);
-    connect(m_widget.data(), &NavigatorWidget::downButtonClicked, this, &NavigatorView::downButtonClicked);
-    connect(m_widget.data(), &NavigatorWidget::upButtonClicked, this, &NavigatorView::upButtonClicked);
-    connect(m_widget.data(), &NavigatorWidget::filterToggled, this, &NavigatorView::filterToggled);
-
-#ifndef QMLDESIGNER_TEST
-    auto idDelegate = new NameItemDelegate(this);
-    IconCheckboxItemDelegate *showDelegate =
-            new IconCheckboxItemDelegate(this,
-                                         Utils::Icons::EYE_OPEN_TOOLBAR.icon(),
-                                         Utils::Icons::EYE_CLOSED_TOOLBAR.icon());
-
-    IconCheckboxItemDelegate *exportDelegate =
-            new IconCheckboxItemDelegate(this,
-                                         Icons::EXPORT_CHECKED.icon(),
-                                         Icons::EXPORT_UNCHECKED.icon());
-
-#ifdef _LOCK_ITEMS_
-    IconCheckboxItemDelegate *lockDelegate =
-            new IconCheckboxItemDelegate(this,
-                                         Utils::Icons::LOCKED_TOOLBAR.icon(),
-                                         Utils::Icons::UNLOCKED_TOOLBAR.icon());
-#endif
-
-
-    treeWidget()->setItemDelegateForColumn(0, idDelegate);
-#ifdef _LOCK_ITEMS_
-    treeWidget()->setItemDelegateForColumn(1,lockDelegate);
-    treeWidget()->setItemDelegateForColumn(2,showDelegate);
-#else
-    treeWidget()->setItemDelegateForColumn(1, exportDelegate);
-    treeWidget()->setItemDelegateForColumn(2, showDelegate);
-#endif
-
-#endif //QMLDESIGNER_TEST
 }
 
 NavigatorView::~NavigatorView()
@@ -138,11 +128,15 @@ bool NavigatorView::hasWidget() const
 
 WidgetInfo NavigatorView::widgetInfo()
 {
+    if (!m_widget)
+        setupWidget();
+
     return createWidgetInfo(m_widget.data(),
                             new WidgetInfo::ToolBarWidgetDefaultFactory<NavigatorWidget>(m_widget.data()),
                             QStringLiteral("Navigator"),
                             WidgetInfo::LeftPane,
-                            0);
+                            0,
+                            tr("Navigator"));
 }
 
 void NavigatorView::modelAttached(Model *model)
@@ -151,26 +145,103 @@ void NavigatorView::modelAttached(Model *model)
 
     QTreeView *treeView = treeWidget();
 
-    treeView->header()->setSectionResizeMode(0, QHeaderView::Stretch);
-    treeView->header()->resizeSection(1,26);
+    treeView->header()->setSectionResizeMode(NavigatorTreeModel::ColumnType::Name, QHeaderView::Stretch);
+    treeView->header()->resizeSection(NavigatorTreeModel::ColumnType::Alias, 26);
+    treeView->header()->resizeSection(NavigatorTreeModel::ColumnType::Visibility, 26);
+    treeView->header()->resizeSection(NavigatorTreeModel::ColumnType::Lock, 26);
     treeView->setIndentation(20);
 
     m_currentModelInterface->setFilter(false);
 
-
     QTimer::singleShot(0, this, [this, treeView]() {
         m_currentModelInterface->setFilter(
                     DesignerSettings::getValue(DesignerSettingsKey::NAVIGATOR_SHOW_ONLY_VISIBLE_ITEMS).toBool());
+
+        m_currentModelInterface->setOrder(
+                    DesignerSettings::getValue(DesignerSettingsKey::NAVIGATOR_REVERSE_ITEM_ORDER).toBool());
+
+        // Expand everything to begin with to ensure model node to index cache is populated
         treeView->expandAll();
+
+        if (AbstractView::model() && m_expandMap.contains(AbstractView::model()->fileUrl())) {
+            const QHash<QString, bool> localExpandMap = m_expandMap[AbstractView::model()->fileUrl()];
+            auto it = localExpandMap.constBegin();
+            while (it != localExpandMap.constEnd()) {
+                const ModelNode node = modelNodeForId(it.key());
+                // When editing subcomponent, the current root node may be marked collapsed in the
+                // full file view, but we never want to actually collapse it, so skip it.
+                if (!node.isRootNode()) {
+                    const QModelIndex index = indexForModelNode(node);
+                    if (index.isValid())
+                        treeWidget()->setExpanded(index, it.value());
+                }
+                ++it;
+            }
+        }
     });
 
-#ifdef _LOCK_ITEMS_
-    treeView->header()->resizeSection(2,20);
-#endif
+    clearExplorerWarnings();
+}
+
+void NavigatorView::clearExplorerWarnings()
+{
+    QList<ModelNode> allNodes;
+    addNodeAndSubModelNodesToList(rootModelNode(), allNodes);
+    for (ModelNode node : allNodes) {
+        if (node.metaInfo().isFileComponent()) {
+            const ProjectExplorer::FileNode *fnode = fileNodeForModelNode(node);
+            if (fnode)
+                fnode->setHasError(false);
+        }
+    }
+}
+
+void NavigatorView::addNodeAndSubModelNodesToList(const ModelNode &node, QList<ModelNode> &nodes)
+{
+    nodes.append(node);
+    for (ModelNode subNode : node.allSubModelNodes()) {
+        addNodeAndSubModelNodesToList(subNode, nodes);
+    }
 }
 
 void NavigatorView::modelAboutToBeDetached(Model *model)
 {
+    QHash<QString, bool> &localExpandMap = m_expandMap[model->fileUrl()];
+
+    // If detaching full document model, recreate expand map from scratch to remove stale entries.
+    // Otherwise just update it (subcomponent edit case).
+    bool fullUpdate = true;
+    if (DesignDocument *document = QmlDesignerPlugin::instance()->currentDesignDocument())
+        fullUpdate = !document->inFileComponentModelActive();
+    if (fullUpdate)
+        localExpandMap.clear();
+
+    if (currentModel()) {
+        // Store expand state of the navigator tree
+        const ModelNode rootNode = rootModelNode();
+        const QModelIndex rootIndex = indexForModelNode(rootNode);
+
+        std::function<void(const QModelIndex &)> gatherExpandedState;
+        gatherExpandedState = [&](const QModelIndex &index) {
+            if (index.isValid()) {
+                const int rowCount = currentModel()->rowCount(index);
+                for (int i = 0; i < rowCount; ++i) {
+                    const QModelIndex childIndex = currentModel()->index(i, 0, index);
+                    const ModelNode node = modelNodeForIndex(childIndex);
+                    if (node.isValid()) {
+                        // Just store collapsed states as everything is expanded by default
+                        if (!treeWidget()->isExpanded(childIndex))
+                            localExpandMap.insert(node.id(), false);
+                        else if (!fullUpdate)
+                            localExpandMap.remove(node.id());
+                    }
+                    gatherExpandedState(childIndex);
+                }
+            }
+        };
+        gatherExpandedState(rootIndex);
+    }
+
     AbstractView::modelAboutToBeDetached(model);
 }
 
@@ -191,6 +262,17 @@ void NavigatorView::bindingPropertiesChanged(const QList<BindingProperty> & prop
     }
 }
 
+void NavigatorView::customNotification(const AbstractView *view, const QString &identifier,
+                                       const QList<ModelNode> &nodeList, const QList<QVariant> &data)
+{
+    Q_UNUSED(view)
+    Q_UNUSED(nodeList)
+    Q_UNUSED(data)
+
+    if (identifier == "asset_import_update")
+        m_currentModelInterface->notifyIconsChanged();
+}
+
 void NavigatorView::handleChangedExport(const ModelNode &modelNode, bool exported)
 {
     const ModelNode rootNode = rootModelNode();
@@ -199,7 +281,7 @@ void NavigatorView::handleChangedExport(const ModelNode &modelNode, bool exporte
     if (rootNode.hasProperty(modelNodeId))
         rootNode.removeProperty(modelNodeId);
     if (exported) {
-        executeInTransaction("NavigatorTreeModel:exportItem", [this, modelNode](){
+        executeInTransaction("NavigatorTreeModel:exportItem", [modelNode](){
             QmlObjectNode qmlObjectNode(modelNode);
             qmlObjectNode.ensureAliasExport();
         });
@@ -223,6 +305,11 @@ void NavigatorView::enableWidget()
         m_widget->enableNavigator();
 }
 
+void NavigatorView::modelNodePreviewPixmapChanged(const ModelNode &node, const QPixmap &pixmap)
+{
+    m_treeModel->updateToolTipPixmap(node, pixmap);
+}
+
 ModelNode NavigatorView::modelNodeForIndex(const QModelIndex &modelIndex) const
 {
     return modelIndex.model()->data(modelIndex, ModelNodeRole).value<ModelNode>();
@@ -249,6 +336,9 @@ void NavigatorView::nodeReparented(const ModelNode &modelNode,
     else
         m_currentModelInterface->notifyModelNodesMoved({modelNode});
     treeWidget()->expand(indexForModelNode(modelNode));
+
+    // make sure selection is in sync again
+    QTimer::singleShot(0, this, &NavigatorView::updateItemSelection);
 }
 
 void NavigatorView::nodeIdChanged(const ModelNode& modelNode, const QString & /*newId*/, const QString & /*oldId*/)
@@ -256,7 +346,7 @@ void NavigatorView::nodeIdChanged(const ModelNode& modelNode, const QString & /*
     m_currentModelInterface->notifyDataChanged(modelNode);
 }
 
-void NavigatorView::propertiesAboutToBeRemoved(const QList<AbstractProperty>& /*propertyList*/)
+void NavigatorView::propertiesAboutToBeRemoved(const QList<AbstractProperty> &/*propertyList*/)
 {
 }
 
@@ -273,7 +363,7 @@ void NavigatorView::propertiesRemoved(const QList<AbstractProperty> &propertyLis
     m_currentModelInterface->notifyModelNodesRemoved(modelNodes);
 }
 
-void NavigatorView::rootNodeTypeChanged(const QString & /*type*/, int /*majorVersion*/, int /*minorVersion*/)
+void NavigatorView::rootNodeTypeChanged(const QString &/*type*/, int /*majorVersion*/, int /*minorVersion*/)
 {
     m_currentModelInterface->notifyDataChanged(rootModelNode());
 }
@@ -284,30 +374,29 @@ void NavigatorView::nodeTypeChanged(const ModelNode &modelNode, const TypeName &
 }
 
 void NavigatorView::auxiliaryDataChanged(const ModelNode &modelNode,
-                                         const PropertyName & /*name*/,
-                                         const QVariant & /*data*/)
+                                         const PropertyName &name,
+                                         const QVariant &data)
 {
+    Q_UNUSED(name)
+    Q_UNUSED(data)
+
     m_currentModelInterface->notifyDataChanged(modelNode);
 }
 
 void NavigatorView::instanceErrorChanged(const QVector<ModelNode> &errorNodeList)
 {
-    foreach (const ModelNode &modelNode, errorNodeList)
+    for (const ModelNode &modelNode : errorNodeList) {
         m_currentModelInterface->notifyDataChanged(modelNode);
+        propagateInstanceErrorToExplorer(modelNode);
+    }
 }
 
-void NavigatorView::nodeOrderChanged(const NodeListProperty & listProperty,
-                                     const ModelNode & /*node*/,
-                                     int /*oldIndex*/)
+void NavigatorView::nodeOrderChanged(const NodeListProperty &listProperty)
 {
-    bool blocked = blockSelectionChangedSignal(true);
-
     m_currentModelInterface->notifyModelNodesMoved(listProperty.directSubNodes());
 
     // make sure selection is in sync again
-    updateItemSelection();
-
-    blockSelectionChangedSignal(blocked);
+    QTimer::singleShot(0, this, &NavigatorView::updateItemSelection);
 }
 
 void NavigatorView::changeToComponent(const QModelIndex &index)
@@ -316,7 +405,7 @@ void NavigatorView::changeToComponent(const QModelIndex &index)
         const ModelNode doubleClickNode = modelNodeForIndex(index);
         if (doubleClickNode.metaInfo().isFileComponent())
             Core::EditorManager::openEditor(doubleClickNode.metaInfo().componentFileName(),
-                                            Core::Id(), Core::EditorManager::DoNotMakeVisible);
+                                            Utils::Id(), Core::EditorManager::DoNotMakeVisible);
     }
 }
 
@@ -330,6 +419,52 @@ QAbstractItemModel *NavigatorView::currentModel() const
     return treeWidget()->model();
 }
 
+const ProjectExplorer::FileNode *NavigatorView::fileNodeForModelNode(const ModelNode &node) const
+{
+    QString filename = node.metaInfo().componentFileName();
+    Utils::FilePath filePath = Utils::FilePath::fromString(filename);
+    ProjectExplorer::Project *currentProject = ProjectExplorer::SessionManager::projectForFile(
+        filePath);
+
+    if (!currentProject) {
+        filePath = Utils::FilePath::fromString(node.model()->fileUrl().toLocalFile());
+
+        /* If the component does not belong to the project then we can fallback to the current file */
+        currentProject = ProjectExplorer::SessionManager::projectForFile(filePath);
+    }
+    if (!currentProject)
+        return nullptr;
+
+    return currentProject->nodeForFilePath(filePath)->asFileNode();
+}
+
+const ProjectExplorer::FileNode *NavigatorView::fileNodeForIndex(const QModelIndex &index) const
+{
+    if (index.isValid() && currentModel()->data(index, Qt::UserRole).isValid()) {
+        ModelNode node = modelNodeForIndex(index);
+        if (node.metaInfo().isFileComponent()) {
+            return fileNodeForModelNode(node);
+        }
+    }
+
+    return nullptr;
+}
+
+void NavigatorView::propagateInstanceErrorToExplorer(const ModelNode &modelNode) {
+    QModelIndex index = indexForModelNode(modelNode);;
+
+    do {
+        const ProjectExplorer::FileNode *fnode = fileNodeForIndex(index);
+        if (fnode) {
+            fnode->setHasError(true);
+            return;
+        }
+        else {
+            index = index.parent();
+        }
+    } while (index.isValid());
+}
+
 void NavigatorView::leftButtonClicked()
 {
     if (selectedModelNodes().count() > 1)
@@ -337,7 +472,7 @@ void NavigatorView::leftButtonClicked()
 
     bool blocked = blockSelectionChangedSignal(true);
 
-    foreach (const ModelNode &node, selectedModelNodes()) {
+    for (const ModelNode &node : selectedModelNodes()) {
         if (!node.isRootNode() && !node.parentProperty().parentModelNode().isRootNode()) {
             if (QmlItemNode::isValidQmlItemNode(node)) {
                 QPointF scenePos = QmlItemNode(node).instanceScenePosition();
@@ -360,11 +495,23 @@ void NavigatorView::rightButtonClicked()
         return; //Semantics are unclear for multi selection.
 
     bool blocked = blockSelectionChangedSignal(true);
-    foreach (const ModelNode &node, selectedModelNodes()) {
+    bool reverse = DesignerSettings::getValue(DesignerSettingsKey::NAVIGATOR_REVERSE_ITEM_ORDER).toBool();
+
+    for (const ModelNode &node : selectedModelNodes()) {
         if (!node.isRootNode() && node.parentProperty().isNodeListProperty() && node.parentProperty().count() > 1) {
             int index = node.parentProperty().indexOf(node);
-            index--;
-            if (index >= 0) { //for the first node the semantics are not clear enough. Wrapping would be irritating.
+
+            bool indexOk = false;
+
+            if (reverse) {
+                index++;
+                indexOk = (index < node.parentProperty().count());
+            } else {
+                index--;
+                indexOk = (index >= 0);
+            }
+
+            if (indexOk) { //for the first node the semantics are not clear enough. Wrapping would be irritating.
                 ModelNode newParent = node.parentProperty().toNodeListProperty().at(index);
 
                 if (QmlItemNode::isValidQmlItemNode(node)
@@ -388,17 +535,13 @@ void NavigatorView::rightButtonClicked()
 void NavigatorView::upButtonClicked()
 {
     bool blocked = blockSelectionChangedSignal(true);
-    foreach (const ModelNode &node, selectedModelNodes()) {
-        if (!node.isRootNode() && node.parentProperty().isNodeListProperty()) {
-            int oldIndex = node.parentProperty().indexOf(node);
-            int index = oldIndex;
-            index--;
-            if (index < 0)
-                index = node.parentProperty().count() - 1; //wrap around
-            if (oldIndex != index)
-                node.parentProperty().toNodeListProperty().slide(oldIndex, index);
-        }
-    }
+    bool reverse = DesignerSettings::getValue(DesignerSettingsKey::NAVIGATOR_REVERSE_ITEM_ORDER).toBool();
+
+    if (reverse)
+        moveNodesDown(selectedModelNodes());
+    else
+        moveNodesUp(selectedModelNodes());
+
     updateItemSelection();
     blockSelectionChangedSignal(blocked);
 }
@@ -406,17 +549,13 @@ void NavigatorView::upButtonClicked()
 void NavigatorView::downButtonClicked()
 {
     bool blocked = blockSelectionChangedSignal(true);
-    foreach (const ModelNode &node, selectedModelNodes()) {
-        if (!node.isRootNode() && node.parentProperty().isNodeListProperty()) {
-            int oldIndex = node.parentProperty().indexOf(node);
-            int index = oldIndex;
-            index++;
-            if (index >= node.parentProperty().count())
-                index = 0; //wrap around
-            if (oldIndex != index)
-                node.parentProperty().toNodeListProperty().slide(oldIndex, index);
-        }
-    }
+    bool reverse = DesignerSettings::getValue(DesignerSettingsKey::NAVIGATOR_REVERSE_ITEM_ORDER).toBool();
+
+    if (reverse)
+        moveNodesUp(selectedModelNodes());
+    else
+        moveNodesDown(selectedModelNodes());
+
     updateItemSelection();
     blockSelectionChangedSignal(blocked);
 }
@@ -428,6 +567,13 @@ void NavigatorView::filterToggled(bool flag)
     DesignerSettings::setValue(DesignerSettingsKey::NAVIGATOR_SHOW_ONLY_VISIBLE_ITEMS, flag);
 }
 
+void NavigatorView::reverseOrderToggled(bool flag)
+{
+    m_currentModelInterface->setOrder(flag);
+    treeWidget()->expandAll();
+    DesignerSettings::setValue(DesignerSettingsKey::NAVIGATOR_REVERSE_ITEM_ORDER, flag);
+}
+
 void NavigatorView::changeSelection(const QItemSelection & /*newSelection*/, const QItemSelection &/*deselected*/)
 {
     if (m_blockSelectionChangedSignal)
@@ -436,7 +582,6 @@ void NavigatorView::changeSelection(const QItemSelection & /*newSelection*/, con
     QSet<ModelNode> nodeSet;
 
     for (const QModelIndex &index : treeWidget()->selectionModel()->selectedIndexes()) {
-
         const ModelNode modelNode = modelNodeForIndex(index);
         if (modelNode.isValid())
             nodeSet.insert(modelNode);
@@ -449,33 +594,49 @@ void NavigatorView::changeSelection(const QItemSelection & /*newSelection*/, con
 
 void NavigatorView::selectedNodesChanged(const QList<ModelNode> &/*selectedNodeList*/, const QList<ModelNode> &/*lastSelectedNodeList*/)
 {
-    updateItemSelection();
+    // Update selection asynchronously to ensure NavigatorTreeModel's index cache is up to date
+    QTimer::singleShot(0, this, &NavigatorView::updateItemSelection);
 }
 
 void NavigatorView::updateItemSelection()
 {
+    if (!isAttached())
+        return;
+
     QItemSelection itemSelection;
-    foreach (const ModelNode &node, selectedModelNodes()) {
+    for (const ModelNode &node : selectedModelNodes()) {
         const QModelIndex index = indexForModelNode(node);
+
         if (index.isValid()) {
             const QModelIndex beginIndex(currentModel()->index(index.row(), 0, index.parent()));
             const QModelIndex endIndex(currentModel()->index(index.row(), currentModel()->columnCount(index.parent()) - 1, index.parent()));
             if (beginIndex.isValid() && endIndex.isValid())
                 itemSelection.select(beginIndex, endIndex);
-        }
+        } else {
+            // if the node index is invalid expand ancestors manually if they are valid.
+            ModelNode parentNode = node;
+            while (parentNode.hasParentProperty()) {
+                parentNode = parentNode.parentProperty().parentQmlObjectNode();
+                QModelIndex parentIndex = indexForModelNode(parentNode);
+                if (parentIndex.isValid())
+                    treeWidget()->expand(parentIndex);
+                else
+                    break;
+            }
+         }
     }
 
     bool blocked = blockSelectionChangedSignal(true);
-    treeWidget()->selectionModel()->select(itemSelection, QItemSelectionModel::ClearAndSelect);
+    treeWidget()->selectionModel()->select(itemSelection, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
     blockSelectionChangedSignal(blocked);
 
     if (!selectedModelNodes().isEmpty())
         treeWidget()->scrollTo(indexForModelNode(selectedModelNodes().constFirst()));
 
-    // make sure selected nodes a visible
-    foreach (const QModelIndex &selectedIndex, itemSelection.indexes()) {
+    // make sure selected nodes are visible
+    for (const QModelIndex &selectedIndex : itemSelection.indexes()) {
         if (selectedIndex.column() == 0)
-            expandRecursively(selectedIndex);
+            expandAncestors(selectedIndex);
     }
 }
 
@@ -499,9 +660,9 @@ bool NavigatorView::blockSelectionChangedSignal(bool block)
     return oldValue;
 }
 
-void NavigatorView::expandRecursively(const QModelIndex &index)
+void NavigatorView::expandAncestors(const QModelIndex &index)
 {
-    QModelIndex currentIndex = index;
+    QModelIndex currentIndex = index.parent();
     while (currentIndex.isValid()) {
         if (!treeWidget()->isExpanded(currentIndex))
             treeWidget()->expand(currentIndex);
@@ -513,9 +674,122 @@ void NavigatorView::reparentAndCatch(NodeAbstractProperty property, const ModelN
 {
     try {
         property.reparentHere(modelNode);
-    }  catch (Exception &exception) {
+    } catch (Exception &exception) {
         exception.showException();
     }
 }
 
+void NavigatorView::setupWidget()
+{
+    m_widget = new NavigatorWidget(this);
+    m_treeModel = new NavigatorTreeModel(this);
+
+#ifndef QMLDESIGNER_TEST
+    auto navigatorContext = new Internal::NavigatorContext(m_widget.data());
+    Core::ICore::addContextObject(navigatorContext);
+#endif
+
+    m_treeModel->setView(this);
+    m_widget->setTreeModel(m_treeModel.data());
+    m_currentModelInterface = m_treeModel;
+
+    connect(treeWidget()->selectionModel(), &QItemSelectionModel::selectionChanged, this, &NavigatorView::changeSelection);
+
+    connect(m_widget.data(), &NavigatorWidget::leftButtonClicked, this, &NavigatorView::leftButtonClicked);
+    connect(m_widget.data(), &NavigatorWidget::rightButtonClicked, this, &NavigatorView::rightButtonClicked);
+    connect(m_widget.data(), &NavigatorWidget::downButtonClicked, this, &NavigatorView::downButtonClicked);
+    connect(m_widget.data(), &NavigatorWidget::upButtonClicked, this, &NavigatorView::upButtonClicked);
+    connect(m_widget.data(), &NavigatorWidget::filterToggled, this, &NavigatorView::filterToggled);
+    connect(m_widget.data(), &NavigatorWidget::reverseOrderToggled, this, &NavigatorView::reverseOrderToggled);
+
+#ifndef QMLDESIGNER_TEST
+    const QString fontName = "qtds_propertyIconFont.ttf";
+    const QSize size = QSize(28, 28);
+
+    const QString visibilityOnUnicode = Theme::getIconUnicode(Theme::Icon::visibilityOn);
+    const QString visibilityOffUnicode = Theme::getIconUnicode(Theme::Icon::visibilityOff);
+
+    const QString aliasOnUnicode = Theme::getIconUnicode(Theme::Icon::alias);
+    const QString aliasOffUnicode = aliasOnUnicode;
+
+    const QString lockOnUnicode = Theme::getIconUnicode(Theme::Icon::lockOn);
+    const QString lockOffUnicode = Theme::getIconUnicode(Theme::Icon::lockOff);
+
+    auto visibilityIconOffNormal = Utils::StyleHelper::IconFontHelper(
+        visibilityOffUnicode, Theme::getColor(Theme::DSnavigatorIcon), size, QIcon::Normal, QIcon::Off);
+    auto visibilityIconOffHover = Utils::StyleHelper::IconFontHelper(
+        visibilityOffUnicode, Theme::getColor(Theme::DSnavigatorIconHover), size, QIcon::Active, QIcon::Off);
+    auto visibilityIconOffSelected = Utils::StyleHelper::IconFontHelper(
+        visibilityOffUnicode, Theme::getColor(Theme::DSnavigatorIconSelected), size, QIcon::Selected, QIcon::Off);
+    auto visibilityIconOnNormal = Utils::StyleHelper::IconFontHelper(
+        visibilityOnUnicode, Theme::getColor(Theme::DSnavigatorIcon), size, QIcon::Normal, QIcon::On);
+    auto visibilityIconOnHover = Utils::StyleHelper::IconFontHelper(
+        visibilityOnUnicode, Theme::getColor(Theme::DSnavigatorIconHover), size, QIcon::Active, QIcon::On);
+    auto visibilityIconOnSelected = Utils::StyleHelper::IconFontHelper(
+        visibilityOnUnicode, Theme::getColor(Theme::DSnavigatorIconSelected), size, QIcon::Selected, QIcon::On);
+
+    const QIcon visibilityIcon = Utils::StyleHelper::getIconFromIconFont(
+                fontName, {visibilityIconOffNormal,
+                           visibilityIconOffHover,
+                           visibilityIconOffSelected,
+                           visibilityIconOnNormal,
+                           visibilityIconOnHover,
+                           visibilityIconOnSelected});
+
+    auto aliasIconOffNormal = Utils::StyleHelper::IconFontHelper(
+        aliasOffUnicode, Theme::getColor(Theme::DSnavigatorIcon), size, QIcon::Normal, QIcon::Off);
+    auto aliasIconOffHover = Utils::StyleHelper::IconFontHelper(
+        aliasOffUnicode, Theme::getColor(Theme::DSnavigatorIconHover), size, QIcon::Active, QIcon::Off);
+    auto aliasIconOffSelected = Utils::StyleHelper::IconFontHelper(
+        aliasOffUnicode, Theme::getColor(Theme::DSnavigatorIconSelected), size, QIcon::Selected, QIcon::Off);
+    auto aliasIconOnNormal = Utils::StyleHelper::IconFontHelper(
+        aliasOnUnicode, Theme::getColor(Theme::DSnavigatorAliasIconChecked), size, QIcon::Normal, QIcon::On);
+    auto aliasIconOnHover = Utils::StyleHelper::IconFontHelper(
+        aliasOnUnicode, Theme::getColor(Theme::DSnavigatorAliasIconChecked), size, QIcon::Active, QIcon::On);
+    auto aliasIconOnSelected = Utils::StyleHelper::IconFontHelper(
+        aliasOnUnicode, Theme::getColor(Theme::DSnavigatorAliasIconChecked), size, QIcon::Selected, QIcon::On);
+
+    const QIcon aliasIcon = Utils::StyleHelper::getIconFromIconFont(
+                fontName, {aliasIconOffNormal,
+                           aliasIconOffHover,
+                           aliasIconOffSelected,
+                           aliasIconOnNormal,
+                           aliasIconOnHover,
+                           aliasIconOnSelected});
+
+    auto lockIconOffNormal = Utils::StyleHelper::IconFontHelper(
+        lockOffUnicode, Theme::getColor(Theme::DSnavigatorIcon), size, QIcon::Normal, QIcon::Off);
+    auto lockIconOffHover = Utils::StyleHelper::IconFontHelper(
+        lockOffUnicode, Theme::getColor(Theme::DSnavigatorIconHover), size, QIcon::Active, QIcon::Off);
+    auto lockIconOffSelected = Utils::StyleHelper::IconFontHelper(
+        lockOffUnicode, Theme::getColor(Theme::DSnavigatorIconSelected), size, QIcon::Selected, QIcon::Off);
+    auto lockIconOnNormal = Utils::StyleHelper::IconFontHelper(
+        lockOnUnicode, Theme::getColor(Theme::DSnavigatorIcon), size, QIcon::Normal, QIcon::On);
+    auto lockIconOnHover = Utils::StyleHelper::IconFontHelper(
+        lockOnUnicode, Theme::getColor(Theme::DSnavigatorIconHover), size, QIcon::Active, QIcon::On);
+    auto lockIconOnSelected = Utils::StyleHelper::IconFontHelper(
+        lockOnUnicode, Theme::getColor(Theme::DSnavigatorIconSelected), size, QIcon::Selected, QIcon::On);
+
+    const QIcon lockIcon = Utils::StyleHelper::getIconFromIconFont(
+                fontName, {lockIconOffNormal,
+                           lockIconOffHover,
+                           lockIconOffSelected,
+                           lockIconOnNormal,
+                           lockIconOnHover,
+                           lockIconOnSelected});
+
+    auto idDelegate = new NameItemDelegate(this);
+
+    auto visibilityDelegate = new IconCheckboxItemDelegate(this, visibilityIcon);
+    auto aliasDelegate = new IconCheckboxItemDelegate(this, aliasIcon);
+    auto lockDelegate = new IconCheckboxItemDelegate(this, lockIcon);
+
+    treeWidget()->setItemDelegateForColumn(NavigatorTreeModel::ColumnType::Name, idDelegate);
+    treeWidget()->setItemDelegateForColumn(NavigatorTreeModel::ColumnType::Alias, aliasDelegate);
+    treeWidget()->setItemDelegateForColumn(NavigatorTreeModel::ColumnType::Visibility, visibilityDelegate);
+    treeWidget()->setItemDelegateForColumn(NavigatorTreeModel::ColumnType::Lock, lockDelegate);
+
+#endif //QMLDESIGNER_TEST
+}
+
 } // namespace QmlDesigner
