@@ -49,6 +49,8 @@ NavigatorWidget::NavigatorWidget(NavigatorView *view)
     : m_treeView(new NavigatorTreeView)
     , m_navigatorView(view)
 {
+    setAcceptDrops(true);
+
     m_treeView->setDragEnabled(true);
     m_treeView->setAcceptDrops(true);
     m_treeView->setSelectionMode(QAbstractItemView::ExtendedSelection);
@@ -126,7 +128,7 @@ QList<QToolButton *> NavigatorWidget::createToolBarWidgets()
     filter->setPopupMode(QToolButton::InstantPopup);
     filter->setProperty("noArrow", true);
     auto filterMenu = new QMenu(filter);
-    auto filterAction = new QAction(tr("Show only visible items."), nullptr);
+    auto filterAction = new QAction(tr("Show Only Visible Components"), nullptr);
     filterAction->setCheckable(true);
 
     bool filterFlag = DesignerSettings::getValue(DesignerSettingsKey::NAVIGATOR_SHOW_ONLY_VISIBLE_ITEMS).toBool();
@@ -135,7 +137,7 @@ QList<QToolButton *> NavigatorWidget::createToolBarWidgets()
     connect(filterAction, &QAction::toggled, this, &NavigatorWidget::filterToggled);
     filterMenu->addAction(filterAction);
 
-    auto reverseAction = new QAction(tr("Reverse item order."), nullptr);
+    auto reverseAction = new QAction(tr("Reverse Component Order"), nullptr);
     reverseAction->setCheckable(true);
 
     bool reverseFlag = DesignerSettings::getValue(DesignerSettingsKey::NAVIGATOR_REVERSE_ITEM_ORDER).toBool();
@@ -184,4 +186,19 @@ NavigatorView *NavigatorWidget::navigatorView() const
     return m_navigatorView.data();
 }
 
+void NavigatorWidget::dragEnterEvent(QDragEnterEvent *dragEnterEvent)
+{
+    const DesignerActionManager &actionManager = QmlDesignerPlugin::instance()
+                                                     ->viewManager().designerActionManager();
+    if (actionManager.externalDragHasSupportedAssets(dragEnterEvent->mimeData()))
+        dragEnterEvent->acceptProposedAction();
+}
+
+void NavigatorWidget::dropEvent(QDropEvent *dropEvent)
+{
+    const DesignerActionManager &actionManager = QmlDesignerPlugin::instance()
+                                                     ->viewManager().designerActionManager();
+    actionManager.handleExternalAssetsDrop(dropEvent->mimeData());
+}
+
 }
