@@ -330,7 +330,7 @@ void AlignDistribute::alignObjects(Target target, AlignTo alignTo, const QString
         };
 
     view->executeInTransaction("DesignerActionManager|" + operationName, [&]() {
-        for (const ModelNode &modelNode : selectedNodes) {
+        for (const ModelNode &modelNode : qAsConst(selectedNodes)) {
             QTC_ASSERT(!modelNode.isRootNode(), continue);
             if (QmlItemNode::isValidQmlItemNode(modelNode)) {
                 QmlItemNode qmlItemNode(modelNode);
@@ -455,7 +455,7 @@ void AlignDistribute::distributeObjects(Target target, AlignTo alignTo, const QS
             return;
     }
 
-    for (const ModelNode &modelNode : selectedNodes) {
+    for (const ModelNode &modelNode : qAsConst(selectedNodes)) {
         if (QmlItemNode::isValidQmlItemNode(modelNode)) {
             QmlItemNode qmlItemNode(modelNode);
             qreal currentPosition;
@@ -494,7 +494,7 @@ void AlignDistribute::distributeObjects(Target target, AlignTo alignTo, const QS
     const QByteArray operationName = "distribute" + QVariant::fromValue(target).toByteArray();
 
     view->executeInTransaction("DesignerActionManager|" + operationName, [&]() {
-        for (const ModelNode &modelNode : selectedNodes) {
+        for (const ModelNode &modelNode : qAsConst(selectedNodes)) {
             QTC_ASSERT(!modelNode.isRootNode(), continue);
             if (QmlItemNode::isValidQmlItemNode(modelNode)) {
                 QmlItemNode qmlItemNode(modelNode);
@@ -590,7 +590,7 @@ void AlignDistribute::distributeSpacing(Dimension dimension,
         }
     }
 
-    for (const ModelNode &modelNode : selectedNodes) {
+    for (const ModelNode &modelNode : qAsConst(selectedNodes)) {
         if (QmlItemNode::isValidQmlItemNode(modelNode)) {
             const QmlItemNode qmlItemNode(modelNode);
             qreal currentPosition;
@@ -627,11 +627,11 @@ void AlignDistribute::distributeSpacing(Dimension dimension,
                                                                  : "distributeSpacingVertical";
 
     view->executeInTransaction("DesignerActionManager|" + operationName, [&]() {
-        for (const ModelNode &modelNode : selectedNodes) {
+        for (const ModelNode &modelNode : qAsConst(selectedNodes)) {
             QTC_ASSERT(!modelNode.isRootNode(), continue);
             if (QmlItemNode::isValidQmlItemNode(modelNode)) {
                 QmlItemNode qmlItemNode(modelNode);
-                qreal parentPos;
+                qreal parentPos = 0;
                 QByteArray propertyName;
                 switch (dimension) {
                 case Dimension::X: {
