@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2019 The Qt Company Ltd.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
@@ -27,6 +27,7 @@
 
 #include <qmlitemnode.h>
 
+#include <QDir>
 #include <QObject>
 #include <QStringList>
 #include <QUrl>
@@ -40,10 +41,12 @@ class FileResourcesModel : public QObject
     Q_PROPERTY(QString filter READ filter WRITE setFilter)
     Q_PROPERTY(QVariant modelNodeBackendProperty READ modelNodeBackend WRITE setModelNodeBackend NOTIFY modelNodeBackendChanged)
     Q_PROPERTY(QUrl path READ path WRITE setPath)
-    Q_PROPERTY(QStringList fileModel READ fileModel NOTIFY fileModelChanged)
+    Q_PROPERTY(QUrl docPath READ docPath)
+    Q_PROPERTY(QStringList fullPathModel READ fullPathModel NOTIFY fullPathModelChanged)
+    Q_PROPERTY(QStringList fileNameModel READ fileNameModel NOTIFY fileNameModelChanged)
 
 public:
-    explicit FileResourcesModel(QObject *parent = 0);
+    explicit FileResourcesModel(QObject *parent = nullptr);
 
     void setModelNodeBackend(const QVariant &modelNodeBackend);
     QString fileName() const;
@@ -51,10 +54,13 @@ public:
     void setFileNameStr(const QString &fileName);
     void setPath(const QUrl &url);
     QUrl path() const;
+    QUrl docPath() const;
     void setFilter(const QString &filter);
     QString filter() const;
-    QStringList fileModel() const;
+    QStringList fullPathModel() const;
+    QStringList fileNameModel() const;
     void setupModel();
+    void refreshModel();
 
     Q_INVOKABLE void openFileDialog();
 
@@ -63,9 +69,8 @@ public:
 signals:
     void fileNameChanged(const QUrl &fileName);
     void modelNodeBackendChanged();
-    void fileModelChanged();
-
-public slots:
+    void fullPathModelChanged();
+    void fileNameModelChanged();
 
 private:
     QVariant modelNodeBackend() const;
@@ -73,12 +78,13 @@ private:
 private:
     QUrl m_fileName;
     QUrl m_path;
+    QDir m_dirPath;
+    QDir m_docPath;
     QString m_filter;
-    bool m_lock;
     QString m_currentPath;
-    QString m_lastModelPath;
-    QStringList m_model;
-
+    QString m_lastResourcePath;
+    QStringList m_fullPathModel;
+    QStringList m_fileNameModel;
 };
 
 QML_DECLARE_TYPE(FileResourcesModel)
