@@ -47,8 +47,9 @@
 #include <QApplication>
 #include <QDir>
 #include <QFileInfo>
-#include <QVector3D>
 #include <QVector2D>
+#include <QVector3D>
+#include <QVector4D>
 
 #include <QLoggingCategory>
 
@@ -343,6 +344,18 @@ void PropertyEditorQmlBackend::setValue(const QmlObjectNode & , const PropertyNa
             if (propertyValue)
                 propertyValue->setValue(QVariant(vecValue[i]));
         }
+    } else if (value.type() == QVariant::Vector4D) {
+        const char *suffix[4] = {"_x", "_y", "_z", "_w"};
+        auto vecValue = value.value<QVector4D>();
+        for (int i = 0; i < 4; ++i) {
+            PropertyName subPropName(name.size() + 2, '\0');
+            subPropName.replace(0, name.size(), name);
+            subPropName.replace(name.size(), 2, suffix[i]);
+            auto propertyValue = qobject_cast<PropertyEditorValue *>(
+                variantToQObject(m_backendValuesPropertyMap.value(QString::fromUtf8(subPropName))));
+            if (propertyValue)
+                propertyValue->setValue(QVariant(vecValue[i]));
+        }
     } else {
         PropertyName propertyName = name;
         propertyName.replace('.', '_');
@@ -530,8 +543,13 @@ void PropertyEditorQmlBackend::initialSetup(const TypeName &typeName, const QUrl
     contextObject()->setGlobalBaseUrl(QUrl());
 }
 
-QString PropertyEditorQmlBackend::propertyEditorResourcesPath() {
-    return Core::ICore::resourcePath() + QStringLiteral("/qmldesigner/propertyEditorQmlSources");
+QString PropertyEditorQmlBackend::propertyEditorResourcesPath()
+{
+#ifdef SHARE_QML_PATH
+    if (qEnvironmentVariableIsSet("LOAD_QML_FROM_SOURCE"))
+        return QLatin1String(SHARE_QML_PATH) + "/propertyEditorQmlSources";
+#endif
+    return Core::ICore::resourcePath("qmldesigner/propertyEditorQmlSources").toString();
 }
 
 inline bool dotPropertyHeuristic(const QmlObjectNode &node, const NodeMetaInfo &type, const PropertyName &name)
@@ -577,9 +595,9 @@ QString PropertyEditorQmlBackend::templateGeneration(const NodeMetaInfo &type,
 
     for (const QmlJS::SimpleReaderNode::Ptr &node : nodes) {
         if (node->propertyNames().contains("separateSection"))
-            separateSectionTypes.append(variantToStringList(node->property("typeNames")));
+            separateSectionTypes.append(variantToStringList(node->property("typeNames").value));
 
-        allTypes.append(variantToStringList(node->property("typeNames")));
+        allTypes.append(variantToStringList(node->property("typeNames").value));
     }
 
     const QList<PropertyName> allProperties = type.propertyNames();
@@ -593,9 +611,12 @@ QString PropertyEditorQmlBackend::templateGeneration(const NodeMetaInfo &type,
             continue; // private API
 
         if (!superType.hasProperty(propertyName)
-                && type.propertyIsWritable(propertyName)
-                && dotPropertyHeuristic(node, type, propertyName)) {
-            const QString typeName = QString::fromLatin1(type.propertyTypeName(propertyName));
+            && type.propertyIsWritable(propertyName)
+            && dotPropertyHeuristic(node, type, propertyName)) {
+            QString typeName = QString::fromLatin1(type.propertyTypeName(propertyName));
+
+            if (typeName == "alias" && node.isValid())
+                typeName = QString::fromLatin1(node.instanceType(propertyName));
 
             // Check if a template for the type exists
             if (allTypes.contains(typeName)) {
@@ -620,10 +641,13 @@ QString PropertyEditorQmlBackend::templateGeneration(const NodeMetaInfo &type,
 
     // Filter out the properties which have a basic type e.g. int, string, bool
     QList<PropertyName> basicProperties;
-    for (auto k : propertyMap.keys()) {
-        if (propertyMap.value(k).empty()) {
-            basicProperties.append(k);
-            propertyMap.remove(k);
+    auto it = propertyMap.begin();
+    while (it != propertyMap.end()) {
+        if (it.value().empty()) {
+            basicProperties.append(it.key());
+            it = propertyMap.erase(it);
+        } else {
+            ++it;
         }
     }
 
@@ -642,8 +666,8 @@ QString PropertyEditorQmlBackend::templateGeneration(const NodeMetaInfo &type,
         QString filledTemplate;
         for (const QmlJS::SimpleReaderNode::Ptr &n : nodes) {
             // Check if we have a template for the type
-            if (variantToStringList(n->property(QStringLiteral("typeNames"))).contains(QString::fromLatin1(typeName))) {
-                const QString fileName = propertyTemplatesPath() + n->property(QStringLiteral("sourceFile")).toString();
+            if (variantToStringList(n->property(QStringLiteral("typeNames")).value).contains(QString::fromLatin1(typeName))) {
+                const QString fileName = propertyTemplatesPath() + n->property(QStringLiteral("sourceFile")).value.toString();
                 QFile file(fileName);
                 if (file.open(QIODevice::ReadOnly)) {
                     QString source = QString::fromUtf8(file.readAll());
@@ -658,71 +682,76 @@ QString PropertyEditorQmlBackend::templateGeneration(const NodeMetaInfo &type,
     };
 
     // QML specfics preparation
-    QStringList imports = variantToStringList(templateConfiguration()->property(QStringLiteral("imports")));
+    QStringList imports = variantToStringList(templateConfiguration()->property(QStringLiteral("imports")).value);
     QString qmlTemplate = imports.join(QLatin1Char('\n')) + QLatin1Char('\n');
     bool emptyTemplate = true;
 
     const QString anchorLeftRight = "anchors.left: parent.left\nanchors.right: parent.right\n";
-    const QString paddingLeftTopBottom = "leftPadding: 0\ntopPadding: 0\nbottomPadding: 0\n";
 
     qmlTemplate += "Column {\n";
-    qmlTemplate += anchorLeftRight;
+    qmlTemplate += "width: parent.width\n";
 
     if (node.modelNode().isComponent())
         qmlTemplate += "ComponentButton {}\n";
 
-    qmlTemplate += "Section {\n";
-    qmlTemplate += "caption: \"User added properties\"\n";
-    qmlTemplate += anchorLeftRight;
-    qmlTemplate += paddingLeftTopBottom;
-    qmlTemplate += "Column {\n";
-    qmlTemplate += "width: parent.width\n";
+    QString qmlInnerTemplate = "";
+
+    qmlInnerTemplate += "Section {\n";
+    qmlInnerTemplate += "caption: \""+ QObject::tr("User Added Properties") + "\"\n";
+    qmlInnerTemplate += anchorLeftRight;
+    qmlInnerTemplate += "leftPadding: 0\n";
+    qmlInnerTemplate += "rightPadding: 0\n";
+    qmlInnerTemplate += "bottomPadding: 0\n";
+    qmlInnerTemplate += "Column {\n";
+    qmlInnerTemplate += "width: parent.width\n";
 
     // First the section containing properties of basic type e.g. int, string, bool
     if (!basicProperties.empty()) {
         emptyTemplate = false;
 
-        qmlTemplate += "Column {\n";
-        qmlTemplate += "width: parent.width\n";
-        qmlTemplate += "leftPadding: 8\n";
-        qmlTemplate += "rightPadding: 0\n";
-        qmlTemplate += "topPadding: 4\n";
-        qmlTemplate += "bottomPadding: 4\n";
-        qmlTemplate += "SectionLayout {\n";
+        qmlInnerTemplate += "Column {\n";
+        qmlInnerTemplate += "width: parent.width\n";
+        qmlInnerTemplate += "leftPadding: 8\n";
+        qmlInnerTemplate += "bottomPadding: 10\n";
+        qmlInnerTemplate += "SectionLayout {\n";
 
         for (const auto &p : qAsConst(basicProperties))
-            qmlTemplate += findAndFillTemplate(p, p);
+            qmlInnerTemplate += findAndFillTemplate(p, p);
 
-        qmlTemplate += "}\n"; // SectionLayout
-        qmlTemplate += "}\n"; // Column
+        qmlInnerTemplate += "}\n"; // SectionLayout
+        qmlInnerTemplate += "}\n"; // Column
     }
 
     // Second the section containing properties of complex type for which no specific template exists e.g. Button
     if (!propertyMap.empty()) {
         emptyTemplate = false;
-        for (const auto &k : propertyMap.keys()) {
-            TypeName parentTypeName = type.propertyTypeName(k);
+        for (auto it = propertyMap.cbegin(); it != propertyMap.cend(); ++it) {
+            const auto &key = it.key();
+            TypeName parentTypeName = type.propertyTypeName(key);
             // alias resolution only possible with instance
             if (parentTypeName == "alias" && node.isValid())
-                parentTypeName = node.instanceType(k);
+                parentTypeName = node.instanceType(key);
 
-            qmlTemplate += "Section {\n";
-            qmlTemplate += QStringLiteral("caption: \"%1 - %2\"\n").arg(QString::fromUtf8(k)).arg(QString::fromUtf8(parentTypeName));
-            qmlTemplate += anchorLeftRight;
-            qmlTemplate += "expanded: false\n";
-            qmlTemplate += "level: 1\n";
-            qmlTemplate += "SectionLayout {\n";
+            qmlInnerTemplate += "Section {\n";
+            qmlInnerTemplate += QStringLiteral("caption: \"%1 - %2\"\n")
+                    .arg(QString::fromUtf8(key), QString::fromUtf8(parentTypeName));
+            qmlInnerTemplate += anchorLeftRight;
+            qmlInnerTemplate += "leftPadding: 8\n";
+            qmlInnerTemplate += "rightPadding: 0\n";
+            qmlInnerTemplate += "expanded: false\n";
+            qmlInnerTemplate += "level: 1\n";
+            qmlInnerTemplate += "SectionLayout {\n";
 
-            auto properties = propertyMap.value(k);
+            auto properties = it.value();
             Utils::sort(properties);
 
             for (const auto &p : qAsConst(properties)) {
                 const PropertyName shortName = p.contains('.') ? p.split('.').last() : p;
-                qmlTemplate += findAndFillTemplate(shortName, p);
+                qmlInnerTemplate += findAndFillTemplate(shortName, p);
             }
 
-            qmlTemplate += "}\n"; // SectionLayout
-            qmlTemplate += "}\n"; // Section
+            qmlInnerTemplate += "}\n"; // SectionLayout
+            qmlInnerTemplate += "}\n"; // Section
         }
     }
 
@@ -730,33 +759,17 @@ QString PropertyEditorQmlBackend::templateGeneration(const NodeMetaInfo &type,
     if (!separateSectionProperties.empty()) {
         emptyTemplate = false;
         Utils::sort(separateSectionProperties);
-        for (const auto &p : qAsConst(separateSectionProperties)) {
-            TypeName parentTypeName = type.propertyTypeName(p);
-            // alias resolution only possible with instance
-            if (parentTypeName == "alias" && node.isValid())
-                parentTypeName = node.instanceType(p);
-
-            qmlTemplate += "Section {\n";
-            qmlTemplate += QStringLiteral("caption: \"%1 - %2\"\n").arg(QString::fromUtf8(p)).arg(QString::fromUtf8(parentTypeName));
-            qmlTemplate += anchorLeftRight;
-            qmlTemplate += paddingLeftTopBottom;
-            qmlTemplate += "level: 1\n";
-            qmlTemplate += "Column {\n";
-            qmlTemplate += "width: parent.width\n";
-
-            qmlTemplate += findAndFillTemplate(p, p);
-
-            qmlTemplate += "}\n"; // Column
-            qmlTemplate += "}\n"; // Section
-        }
+        for (const auto &p : qAsConst(separateSectionProperties))
+            qmlInnerTemplate += findAndFillTemplate(p, p);
     }
 
-    qmlTemplate += "}\n"; // Column
-    qmlTemplate += "}\n"; // Section
-    qmlTemplate += "}\n"; // Column
+    qmlInnerTemplate += "}\n"; // Column
+    qmlInnerTemplate += "}\n"; // Section
 
-    if (emptyTemplate)
-        return QString();
+    if (!emptyTemplate)
+        qmlTemplate += qmlInnerTemplate;
+
+    qmlTemplate += "}\n"; // Column
 
     return qmlTemplate;
 }
@@ -904,7 +917,10 @@ QString PropertyEditorQmlBackend::locateQmlFile(const NodeMetaInfo &info, const
 
     const QString withoutDir = relativePath.split(QStringLiteral("/")).constLast();
 
-    if (importDirVersion.exists(withoutDir))
+    int lastSlash = importDirVersion.absoluteFilePath(withoutDir).lastIndexOf("/");
+    QString dirPath = importDirVersion.absoluteFilePath(withoutDir).left(lastSlash);
+
+    if (importDirVersion.exists(withoutDir) && !dirPath.endsWith("QtQuick/Controls.2/designer") && !dirPath.endsWith("QtQuick/Controls/designer"))
         return importDirVersion.absoluteFilePath(withoutDir);
 
     const QString withoutDirWithVersion = relativePathWithVersion.split(QStringLiteral("/")).constLast();
