@@ -30,6 +30,7 @@
 #include "propertyeditortransaction.h"
 
 #include <qmldesignerconstants.h>
+#include <qmltimeline.h>
 #include <nodemetainfo.h>
 
 #include <invalididexception.h>
@@ -43,8 +44,9 @@
 #include <theme.h>
 
 #include <coreplugin/icore.h>
-#include <utils/fileutils.h>
 #include <coreplugin/messagebox.h>
+#include <utils/fileutils.h>
+#include <utils/qtcassert.h>
 
 #include <QCoreApplication>
 #include <QDir>
@@ -54,6 +56,7 @@
 #include <QTimer>
 #include <QShortcut>
 #include <QApplication>
+#include <QScopedPointer>
 
 enum {
     debug = false
@@ -69,22 +72,25 @@ static bool propertyIsAttachedLayoutProperty(const PropertyName &propertyName)
 PropertyEditorView::PropertyEditorView(QWidget *parent) :
         AbstractView(parent),
         m_parent(parent),
-        m_updateShortcut(0),
+        m_updateShortcut(nullptr),
         m_timerId(0),
         m_stackedWidget(new PropertyEditorWidget(parent)),
-        m_qmlBackEndForCurrentType(0),
+        m_qmlBackEndForCurrentType(nullptr),
         m_locked(false),
         m_setupCompleted(false),
         m_singleShotTimer(new QTimer(this))
 {
     m_qmlDir = PropertyEditorQmlBackend::propertyEditorResourcesPath();
 
-    m_updateShortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_F3), m_stackedWidget);
+    if (Utils::HostOsInfo::isMacHost())
+        m_updateShortcut = new QShortcut(QKeySequence(Qt::ALT + Qt::Key_F3), m_stackedWidget);
+    else
+        m_updateShortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_F3), m_stackedWidget);
     connect(m_updateShortcut, &QShortcut::activated, this, &PropertyEditorView::reloadQml);
 
     m_stackedWidget->setStyleSheet(Theme::replaceCssColors(
-            QString::fromUtf8(Utils::FileReader::fetchQrc(QStringLiteral(":/qmldesigner/stylesheet.css")))));
-    m_stackedWidget->setMinimumWidth(320);
+        QString::fromUtf8(Utils::FileReader::fetchQrc(":/qmldesigner/stylesheet.css"))));
+    m_stackedWidget->setMinimumWidth(340);
     m_stackedWidget->move(0, 0);
     connect(m_stackedWidget, &PropertyEditorWidget::resized, this, &PropertyEditorView::updateSize);
 
@@ -141,7 +147,7 @@ void PropertyEditorView::changeValue(const QString &name)
     if (propertyName == "className")
         return;
 
-    if (!m_selectedNode.isValid())
+    if (noValidSelection())
         return;
 
     if (propertyName == "id") {
@@ -151,16 +157,17 @@ void PropertyEditorView::changeValue(const QString &name)
         if (newId == m_selectedNode.id())
             return;
 
-        if (m_selectedNode.isValidId(newId)  && !hasId(newId)) {
+        if (QmlDesigner::ModelNode::isValidId(newId)  && !hasId(newId)) {
             m_selectedNode.setIdWithRefactoring(newId);
         } else {
             m_locked = true;
             value->setValue(m_selectedNode.id());
             m_locked = false;
-            if (!m_selectedNode.isValidId(newId))
-                Core::AsynchronousMessageBox::warning(tr("Invalid Id"),  tr("%1 is an invalid id.").arg(newId));
+            QString errMsg = QmlDesigner::ModelNode::getIdValidityErrorMessage(newId);
+            if (!errMsg.isEmpty())
+                Core::AsynchronousMessageBox::warning(tr("Invalid ID"), errMsg);
             else
-                Core::AsynchronousMessageBox::warning(tr("Invalid Id"),  tr("%1 already exists.").arg(newId));
+                Core::AsynchronousMessageBox::warning(tr("Invalid ID"), tr("%1 already exists.").arg(newId));
         }
         return;
     }
@@ -169,19 +176,24 @@ void PropertyEditorView::changeValue(const QString &name)
     underscoreName.replace('.', '_');
     PropertyEditorValue *value = m_qmlBackEndForCurrentType->propertyValueForName(QString::fromLatin1(underscoreName));
 
-    if (value ==0)
+    if (value == nullptr)
         return;
 
-    QmlObjectNode qmlObjectNode(m_selectedNode);
+    if (propertyName.endsWith("__AUX")) {
+        commitAuxValueToModel(propertyName, value->value());
+        return;
+    }
+
+    const NodeMetaInfo metaInfo = QmlObjectNode(m_selectedNode).modelNode().metaInfo();
 
     QVariant castedValue;
 
-    if (qmlObjectNode.modelNode().metaInfo().isValid() && qmlObjectNode.modelNode().metaInfo().hasProperty(propertyName)) {
-        castedValue = qmlObjectNode.modelNode().metaInfo().propertyCastedValue(propertyName, value->value());
+    if (metaInfo.isValid() && metaInfo.hasProperty(propertyName)) {
+        castedValue = metaInfo.propertyCastedValue(propertyName, value->value());
     } else if (propertyIsAttachedLayoutProperty(propertyName)) {
         castedValue = value->value();
     } else {
-        qWarning() << "PropertyEditor:" <<propertyName << "cannot be casted (metainfo)";
+        qWarning() << "PropertyEditor:" << propertyName << "cannot be casted (metainfo)";
         return ;
     }
 
@@ -190,10 +202,14 @@ void PropertyEditorView::changeValue(const QString &name)
         return ;
     }
 
-    if (qmlObjectNode.modelNode().metaInfo().isValid() && qmlObjectNode.modelNode().metaInfo().hasProperty(propertyName)) {
-        if (qmlObjectNode.modelNode().metaInfo().propertyTypeName(propertyName) == "QUrl"
-                || qmlObjectNode.modelNode().metaInfo().propertyTypeName(propertyName) == "url") { //turn absolute local file paths into relative paths
-                QString filePath = castedValue.toUrl().toString();
+    bool propertyTypeUrl = false;
+
+    if (metaInfo.isValid() && metaInfo.hasProperty(propertyName)) {
+        if (metaInfo.propertyTypeName(propertyName) == "QUrl"
+                || metaInfo.propertyTypeName(propertyName) == "url") {
+            // turn absolute local file paths into relative paths
+            propertyTypeUrl = true;
+            QString filePath = castedValue.toUrl().toString();
             QFileInfo fi(filePath);
             if (fi.exists() && fi.isAbsolute()) {
                 QDir fileDir(QFileInfo(model()->fileUrl().toLocalFile()).absolutePath());
@@ -202,6 +218,9 @@ void PropertyEditorView::changeValue(const QString &name)
         }
     }
 
+    if (name == "state" && castedValue.toString() == "base state")
+        castedValue = "";
+
     if (castedValue.type() == QVariant::Color) {
         QColor color = castedValue.value<QColor>();
         QColor newColor = QColor(color.name());
@@ -209,20 +228,24 @@ void PropertyEditorView::changeValue(const QString &name)
         castedValue = QVariant(newColor);
     }
 
-    try {
-        if (!value->value().isValid()) { //reset
-            qmlObjectNode.removeProperty(propertyName);
-        } else {
-            if (castedValue.isValid() && !castedValue.isNull()) {
-                m_locked = true;
-                qmlObjectNode.setVariantProperty(propertyName, castedValue);
-                m_locked = false;
-            }
+    if (!value->value().isValid()
+            || (propertyTypeUrl && value->value().toString().isEmpty())) { // reset
+        removePropertyFromModel(propertyName);
+    } else {
+        // QVector*D(0, 0, 0) detects as null variant though it is valid value
+        if (castedValue.isValid()
+            && (!castedValue.isNull() || castedValue.type() == QVariant::Vector2D
+                || castedValue.type() == QVariant::Vector3D
+                || castedValue.type() == QVariant::Vector4D)) {
+            commitVariantValueToModel(propertyName, castedValue);
         }
     }
-    catch (const RewritingException &e) {
-        e.showException();
-    }
+}
+
+static bool isTrueFalseLiteral(const QString &expression)
+{
+    return (expression.compare("false", Qt::CaseInsensitive) == 0)
+           || (expression.compare("true", Qt::CaseInsensitive) == 0);
 }
 
 void PropertyEditorView::changeExpression(const QString &propertyName)
@@ -235,16 +258,14 @@ void PropertyEditorView::changeExpression(const QString &propertyName)
     if (locked())
         return;
 
-    if (!m_selectedNode.isValid())
+    if (noValidSelection())
         return;
 
-    RewriterTransaction transaction = beginRewriterTransaction(QByteArrayLiteral("PropertyEditorView::changeExpression"));
-
-    try {
+    executeInTransaction("PropertyEditorView::changeExpression", [this, name](){
         PropertyName underscoreName(name);
         underscoreName.replace('.', '_');
 
-        QmlObjectNode qmlObjectNode(m_selectedNode);
+        QScopedPointer<QmlObjectNode> qmlObjectNode {QmlObjectNode::getQmlObjectNodeOfCorrectType(m_selectedNode)};
         PropertyEditorValue *value = m_qmlBackEndForCurrentType->propertyValueForName(QString::fromLatin1(underscoreName));
 
         if (!value) {
@@ -252,54 +273,59 @@ void PropertyEditorView::changeExpression(const QString &propertyName)
             return;
         }
 
-        if (qmlObjectNode.modelNode().metaInfo().isValid() && qmlObjectNode.modelNode().metaInfo().hasProperty(name)) {
-            if (qmlObjectNode.modelNode().metaInfo().propertyTypeName(name) == "QColor") {
+        if (qmlObjectNode->modelNode().metaInfo().isValid() && qmlObjectNode->modelNode().metaInfo().hasProperty(name)) {
+            if (qmlObjectNode->modelNode().metaInfo().propertyTypeName(name) == "QColor") {
                 if (QColor(value->expression().remove('"')).isValid()) {
-                    qmlObjectNode.setVariantProperty(name, QColor(value->expression().remove('"')));
-                    transaction.commit(); //committing in the try block
+                    qmlObjectNode->setVariantProperty(name, QColor(value->expression().remove('"')));
                     return;
                 }
-            } else if (qmlObjectNode.modelNode().metaInfo().propertyTypeName(name) == "bool") {
-                if (value->expression().compare(QLatin1String("false"), Qt::CaseInsensitive) == 0
-                        || value->expression().compare(QLatin1String("true"), Qt::CaseInsensitive) == 0) {
-                    if (value->expression().compare(QLatin1String("true"), Qt::CaseInsensitive) == 0)
-                        qmlObjectNode.setVariantProperty(name, true);
+            } else if (qmlObjectNode->modelNode().metaInfo().propertyTypeName(name) == "bool") {
+                if (isTrueFalseLiteral(value->expression())) {
+                    if (value->expression().compare("true", Qt::CaseInsensitive) == 0)
+                        qmlObjectNode->setVariantProperty(name, true);
                     else
-                        qmlObjectNode.setVariantProperty(name, false);
-                    transaction.commit(); //committing in the try block
+                        qmlObjectNode->setVariantProperty(name, false);
                     return;
                 }
-            } else if (qmlObjectNode.modelNode().metaInfo().propertyTypeName(name) == "int") {
+            } else if (qmlObjectNode->modelNode().metaInfo().propertyTypeName(name) == "int") {
                 bool ok;
                 int intValue = value->expression().toInt(&ok);
                 if (ok) {
-                    qmlObjectNode.setVariantProperty(name, intValue);
-                    transaction.commit(); //committing in the try block
+                    qmlObjectNode->setVariantProperty(name, intValue);
                     return;
                 }
-            } else if (qmlObjectNode.modelNode().metaInfo().propertyTypeName(name) == "qreal") {
+            } else if (qmlObjectNode->modelNode().metaInfo().propertyTypeName(name) == "qreal") {
                 bool ok;
                 qreal realValue = value->expression().toDouble(&ok);
                 if (ok) {
-                    qmlObjectNode.setVariantProperty(name, realValue);
-                    transaction.commit(); //committing in the try block
+                    qmlObjectNode->setVariantProperty(name, realValue);
+                    return;
+                }
+            } else if (qmlObjectNode->modelNode().metaInfo().propertyTypeName(name) == "QVariant") {
+                bool ok;
+                qreal realValue = value->expression().toDouble(&ok);
+                if (ok) {
+                    qmlObjectNode->setVariantProperty(name, realValue);
+                    return;
+                } else if (isTrueFalseLiteral(value->expression())) {
+                    if (value->expression().compare("true", Qt::CaseInsensitive) == 0)
+                        qmlObjectNode->setVariantProperty(name, true);
+                    else
+                        qmlObjectNode->setVariantProperty(name, false);
                     return;
                 }
             }
         }
 
-        if (value->expression().isEmpty())
+        if (value->expression().isEmpty()) {
+            value->resetValue();
             return;
+        }
 
-        if (qmlObjectNode.expression(name) != value->expression() || !qmlObjectNode.propertyAffectedByCurrentState(name))
-            qmlObjectNode.setBindingProperty(name, value->expression());
+        if (qmlObjectNode->expression(name) != value->expression() || !qmlObjectNode->propertyAffectedByCurrentState(name))
+            qmlObjectNode->setBindingProperty(name, value->expression());
 
-        transaction.commit(); //committing in the try block
-    }
-
-    catch (const RewritingException &e) {
-        e.showException();
-    }
+    }); /* end of transaction */
 }
 
 void PropertyEditorView::exportPopertyAsAlias(const QString &name)
@@ -310,12 +336,10 @@ void PropertyEditorView::exportPopertyAsAlias(const QString &name)
     if (locked())
         return;
 
-    if (!m_selectedNode.isValid())
+    if (noValidSelection())
         return;
 
-    RewriterTransaction transaction = beginRewriterTransaction(QByteArrayLiteral("PropertyEditorView::exportPopertyAsAlias"));
-
-    try {
+    executeInTransaction("PropertyEditorView::exportPopertyAsAlias", [this, name](){
         const QString id = m_selectedNode.validId();
         QString upperCasePropertyName = name;
         upperCasePropertyName.replace(0, 1, upperCasePropertyName.at(0).toUpper());
@@ -325,15 +349,11 @@ void PropertyEditorView::exportPopertyAsAlias(const QString &name)
         PropertyName propertyName = aliasName.toUtf8();
         if (rootModelNode().hasProperty(propertyName)) {
             Core::AsynchronousMessageBox::warning(tr("Cannot Export Property as Alias"),
-                                                  tr("Property %1 does already exist for root item.").arg(aliasName));
+                                                  tr("Property %1 does already exist for root component.").arg(aliasName));
             return;
         }
         rootModelNode().bindingProperty(propertyName).setDynamicTypeNameAndExpression("alias", id + "." + name);
-
-        transaction.commit(); //committing in the try block
-    } catch (const RewritingException &e) {
-        e.showException();
-    }
+    });
 }
 
 void PropertyEditorView::removeAliasExport(const QString &name)
@@ -344,12 +364,10 @@ void PropertyEditorView::removeAliasExport(const QString &name)
     if (locked())
         return;
 
-    if (!m_selectedNode.isValid())
+    if (noValidSelection())
         return;
 
-    RewriterTransaction transaction = beginRewriterTransaction(QByteArrayLiteral("PropertyEditorView::exportPopertyAsAlias"));
-
-    try {
+    executeInTransaction("PropertyEditorView::exportPopertyAsAlias", [this, name](){
         const QString id = m_selectedNode.validId();
 
         for (const BindingProperty &property : rootModelNode().bindingProperties())
@@ -357,10 +375,7 @@ void PropertyEditorView::removeAliasExport(const QString &name)
                 rootModelNode().removeProperty(property.name());
                 break;
             }
-        transaction.commit(); //committing in the try block
-    } catch (const RewritingException &e) {
-        e.showException();
-    }
+    });
 }
 
 bool PropertyEditorView::locked() const
@@ -368,11 +383,16 @@ bool PropertyEditorView::locked() const
     return m_locked;
 }
 
+void PropertyEditorView::currentTimelineChanged(const ModelNode &)
+{
+    m_qmlBackEndForCurrentType->contextObject()->setHasActiveTimeline(QmlTimeline::hasActiveTimeline(this));
+}
+
 void PropertyEditorView::updateSize()
 {
     if (!m_qmlBackEndForCurrentType)
         return;
-    QWidget* frame = m_qmlBackEndForCurrentType->widget()->findChild<QWidget*>("propertyEditorFrame");
+    auto frame = m_qmlBackEndForCurrentType->widget()->findChild<QWidget*>("propertyEditorFrame");
     if (frame)
         frame->resize(m_stackedWidget->size());
 }
@@ -403,9 +423,11 @@ void PropertyEditorView::timerEvent(QTimerEvent *timerEvent)
 
 void PropertyEditorView::resetView()
 {
-    if (model() == 0)
+    if (model() == nullptr)
         return;
 
+    setSelelectedModelNode();
+
     m_locked = true;
 
     if (debug)
@@ -434,13 +456,16 @@ void PropertyEditorView::resetView()
 void PropertyEditorView::setupQmlBackend()
 {
     TypeName specificsClassName;
-    QUrl qmlFile(PropertyEditorQmlBackend::getQmlUrlForModelNode(m_selectedNode, specificsClassName));
+
+    const NodeMetaInfo commonAncestor = PropertyEditorQmlBackend::findCommonAncestor(m_selectedNode);
+
+    const QUrl qmlFile(PropertyEditorQmlBackend::getQmlUrlForMetaInfo(commonAncestor, specificsClassName));
     QUrl qmlSpecificsFile;
 
     TypeName diffClassName;
-    if (m_selectedNode.isValid()) {
-        diffClassName = m_selectedNode.metaInfo().typeName();
-        foreach (const NodeMetaInfo &metaInfo, m_selectedNode.metaInfo().classHierarchy()) {
+    if (commonAncestor.isValid()) {
+        diffClassName = commonAncestor.typeName();
+        foreach (const NodeMetaInfo &metaInfo, commonAncestor.classHierarchy()) {
             if (PropertyEditorQmlBackend::checkIfUrlExists(qmlSpecificsFile))
                 break;
             qmlSpecificsFile = PropertyEditorQmlBackend::getQmlFileUrl(metaInfo.typeName() + "Specifics", metaInfo);
@@ -453,8 +478,8 @@ void PropertyEditorView::setupQmlBackend()
 
     QString specificQmlData;
 
-    if (m_selectedNode.isValid() && m_selectedNode.metaInfo().isValid() && diffClassName != m_selectedNode.type())
-        specificQmlData = PropertyEditorQmlBackend::templateGeneration(m_selectedNode.metaInfo(), model()->metaInfo(diffClassName), m_selectedNode);
+    if (commonAncestor.isValid() && m_selectedNode.metaInfo().isValid() && diffClassName != m_selectedNode.type())
+        specificQmlData = PropertyEditorQmlBackend::templateGeneration(commonAncestor, model()->metaInfo(diffClassName), m_selectedNode);
 
     PropertyEditorQmlBackend *currentQmlBackend = m_qmlBackendHash.value(qmlFile.toString());
 
@@ -466,12 +491,14 @@ void PropertyEditorView::setupQmlBackend()
         m_stackedWidget->addWidget(currentQmlBackend->widget());
         m_qmlBackendHash.insert(qmlFile.toString(), currentQmlBackend);
 
-        QmlObjectNode qmlObjectNode;
+        QScopedPointer<QmlObjectNode> qmlObjectNode;
         if (m_selectedNode.isValid()) {
-            qmlObjectNode = QmlObjectNode(m_selectedNode);
-            Q_ASSERT(qmlObjectNode.isValid());
+            qmlObjectNode.reset(QmlObjectNode::getQmlObjectNodeOfCorrectType(m_selectedNode));
+            Q_ASSERT(qmlObjectNode->isValid());
+            currentQmlBackend->setup(*qmlObjectNode, currentStateName, qmlSpecificsFile, this);
+        } else {
+            qmlObjectNode.reset(new QmlObjectNode);
         }
-        currentQmlBackend->setup(qmlObjectNode, currentStateName, qmlSpecificsFile, this);
         currentQmlBackend->context()->setContextProperty("finishedNotify", QVariant(false));
         if (specificQmlData.isEmpty())
             currentQmlBackend->contextObject()->setSpecificQmlData(specificQmlData);
@@ -481,14 +508,16 @@ void PropertyEditorView::setupQmlBackend()
         currentQmlBackend->setSource(qmlFile);
         currentQmlBackend->context()->setContextProperty("finishedNotify", QVariant(true));
     } else {
-        QmlObjectNode qmlObjectNode;
+        QScopedPointer<QmlObjectNode> qmlObjectNode;
         if (m_selectedNode.isValid())
-            qmlObjectNode = QmlObjectNode(m_selectedNode);
+            qmlObjectNode.reset(QmlObjectNode::getQmlObjectNodeOfCorrectType(m_selectedNode));
+        else
+            qmlObjectNode.reset(new QmlObjectNode);
 
         currentQmlBackend->context()->setContextProperty("finishedNotify", QVariant(false));
         if (specificQmlData.isEmpty())
             currentQmlBackend->contextObject()->setSpecificQmlData(specificQmlData);
-        currentQmlBackend->setup(qmlObjectNode, currentStateName, qmlSpecificsFile, this);
+        currentQmlBackend->setup(*qmlObjectNode, currentStateName, qmlSpecificsFile, this);
         currentQmlBackend->contextObject()->setGlobalBaseUrl(qmlFile);
         currentQmlBackend->contextObject()->setSpecificQmlData(specificQmlData);
     }
@@ -503,21 +532,85 @@ void PropertyEditorView::setupQmlBackend()
 
 }
 
-void PropertyEditorView::selectedNodesChanged(const QList<ModelNode> &selectedNodeList,
-                                          const QList<ModelNode> &lastSelectedNodeList)
+void PropertyEditorView::commitVariantValueToModel(const PropertyName &propertyName, const QVariant &value)
 {
-    Q_UNUSED(lastSelectedNodeList);
+    m_locked = true;
+    try {
+        RewriterTransaction transaction = beginRewriterTransaction("PropertyEditorView::commitVariantValueToMode");
 
-    if (selectedNodeList.isEmpty() || selectedNodeList.count() > 1)
-        select(ModelNode());
-    else if (m_selectedNode != selectedNodeList.first())
-        select(selectedNodeList.first());
+        for (const ModelNode &node : m_selectedNode.view()->selectedModelNodes()) {
+            if (QmlObjectNode::isValidQmlObjectNode(node)) {
+                QScopedPointer<QmlObjectNode>{QmlObjectNode::getQmlObjectNodeOfCorrectType(node)}
+                    ->setVariantProperty(propertyName, value);
+            }
+        }
+        transaction.commit();
+    }
+    catch (const RewritingException &e) {
+        e.showException();
+    }
+    m_locked = false;
+}
+
+void PropertyEditorView::commitAuxValueToModel(const PropertyName &propertyName, const QVariant &value)
+{
+    m_locked = true;
+
+    PropertyName name = propertyName;
+    name.chop(5);
+
+    try {
+        if (value.isValid()) {
+            for (const ModelNode &node : m_selectedNode.view()->selectedModelNodes()) {
+                node.setAuxiliaryData(name, value);
+            }
+        } else {
+            for (const ModelNode &node : m_selectedNode.view()->selectedModelNodes()) {
+                node.removeAuxiliaryData(name);
+            }
+        }
+    }
+    catch (const Exception &e) {
+        e.showException();
+    }
+    m_locked = false;
+}
+
+void PropertyEditorView::removePropertyFromModel(const PropertyName &propertyName)
+{
+    m_locked = true;
+    try {
+        RewriterTransaction transaction = beginRewriterTransaction("PropertyEditorView::removePropertyFromModel");
+
+        for (const ModelNode &node : m_selectedNode.view()->selectedModelNodes()) {
+            if (QmlObjectNode::isValidQmlObjectNode(node))
+                QmlObjectNode(node).removeProperty(propertyName);
+        }
+
+        transaction.commit();
+    }
+    catch (const RewritingException &e) {
+        e.showException();
+    }
+    m_locked = false;
+}
+
+bool PropertyEditorView::noValidSelection() const
+{
+    QTC_ASSERT(m_qmlBackEndForCurrentType, return true);
+    return !QmlObjectNode::isValidQmlObjectNode(m_selectedNode);
+}
+
+void PropertyEditorView::selectedNodesChanged(const QList<ModelNode> &,
+                                          const QList<ModelNode> &)
+{
+    select();
 }
 
 void PropertyEditorView::nodeAboutToBeRemoved(const ModelNode &removedNode)
 {
     if (m_selectedNode.isValid() && removedNode.isValid() && m_selectedNode == removedNode)
-        select(m_selectedNode.parentProperty().parentModelNode());
+        select();
 }
 
 void PropertyEditorView::modelAttached(Model *model)
@@ -530,10 +623,11 @@ void PropertyEditorView::modelAttached(Model *model)
     m_locked = true;
 
     if (!m_setupCompleted) {
-        m_singleShotTimer->setSingleShot(true);
-        m_singleShotTimer->setInterval(100);
-        connect(m_singleShotTimer, &QTimer::timeout, this, &PropertyEditorView::setupPanes);
-        m_singleShotTimer->start();
+        QTimer::singleShot(50, this, [this]{
+            PropertyEditorView::setupPanes();
+            /* workaround for QTBUG-75847 */
+            reloadQml();
+        });
     }
 
     m_locked = false;
@@ -551,10 +645,7 @@ void PropertyEditorView::modelAboutToBeDetached(Model *model)
 
 void PropertyEditorView::propertiesRemoved(const QList<AbstractProperty>& propertyList)
 {
-    if (!m_selectedNode.isValid())
-        return;
-
-    if (!QmlObjectNode(m_selectedNode).isValid())
+    if (noValidSelection())
         return;
 
     foreach (const AbstractProperty &property, propertyList) {
@@ -566,9 +657,18 @@ void PropertyEditorView::propertiesRemoved(const QList<AbstractProperty>& proper
         if (node == m_selectedNode || QmlObjectNode(m_selectedNode).propertyChangeForCurrentState() == node) {
             setValue(m_selectedNode, property.name(), QmlObjectNode(m_selectedNode).instanceValue(property.name()));
 
-            if (propertyIsAttachedLayoutProperty(property.name()))
+            if (propertyIsAttachedLayoutProperty(property.name())) {
                 m_qmlBackEndForCurrentType->setValueforLayoutAttachedProperties(m_selectedNode, property.name());
 
+                if (property.name() == "Layout.margins") {
+                    m_qmlBackEndForCurrentType->setValueforLayoutAttachedProperties(m_selectedNode, "Layout.topMargin");
+                    m_qmlBackEndForCurrentType->setValueforLayoutAttachedProperties(m_selectedNode, "Layout.bottomMargin");
+                    m_qmlBackEndForCurrentType->setValueforLayoutAttachedProperties(m_selectedNode, "Layout.leftMargin");
+                    m_qmlBackEndForCurrentType->setValueforLayoutAttachedProperties(m_selectedNode, "Layout.rightMargin");
+
+                }
+            }
+
             if ("width" == property.name() || "height" == property.name()) {
                 const QmlItemNode qmlItemNode = m_selectedNode;
                 if (qmlItemNode.isValid() && qmlItemNode.isInLayout())
@@ -583,11 +683,7 @@ void PropertyEditorView::propertiesRemoved(const QList<AbstractProperty>& proper
 
 void PropertyEditorView::variantPropertiesChanged(const QList<VariantProperty>& propertyList, PropertyChangeFlags /*propertyChange*/)
 {
-
-    if (!m_selectedNode.isValid())
-        return;
-
-    if (!QmlObjectNode(m_selectedNode).isValid())
+    if (noValidSelection())
         return;
 
     foreach (const VariantProperty &property, propertyList) {
@@ -607,10 +703,7 @@ void PropertyEditorView::variantPropertiesChanged(const QList<VariantProperty>&
 
 void PropertyEditorView::bindingPropertiesChanged(const QList<BindingProperty>& propertyList, PropertyChangeFlags /*propertyChange*/)
 {
-    if (!m_selectedNode.isValid())
-        return;
-
-       if (!QmlObjectNode(m_selectedNode).isValid())
+    if (noValidSelection())
         return;
 
     foreach (const BindingProperty &property, propertyList) {
@@ -631,9 +724,22 @@ void PropertyEditorView::bindingPropertiesChanged(const QList<BindingProperty>&
     }
 }
 
+void PropertyEditorView::auxiliaryDataChanged(const ModelNode &node, const PropertyName &name, const QVariant &)
+{
+
+    if (noValidSelection())
+        return;
+
+    if (!node.isSelected())
+        return;
+
+    m_qmlBackEndForCurrentType->setValueforAuxiliaryProperties(m_selectedNode, name);
+
+}
+
 void PropertyEditorView::instanceInformationsChanged(const QMultiHash<ModelNode, InformationName> &informationChangedHash)
 {
-    if (!m_selectedNode.isValid())
+    if (noValidSelection())
         return;
 
     m_locked = true;
@@ -646,7 +752,7 @@ void PropertyEditorView::instanceInformationsChanged(const QMultiHash<ModelNode,
 
 void PropertyEditorView::nodeIdChanged(const ModelNode& node, const QString& newId, const QString& /*oldId*/)
 {
-    if (!m_selectedNode.isValid())
+    if (noValidSelection())
         return;
 
     if (!QmlObjectNode(m_selectedNode).isValid())
@@ -659,19 +765,29 @@ void PropertyEditorView::nodeIdChanged(const ModelNode& node, const QString& new
     }
 }
 
-void PropertyEditorView::select(const ModelNode &node)
+void PropertyEditorView::select()
 {
     if (m_qmlBackEndForCurrentType)
         m_qmlBackEndForCurrentType->emitSelectionToBeChanged();
 
-    if (QmlObjectNode(node).isValid())
-        m_selectedNode = node;
-    else
-        m_selectedNode = ModelNode();
-
     delayedResetView();
 }
 
+void PropertyEditorView::setSelelectedModelNode()
+{
+    const auto selectedNodeList = selectedModelNodes();
+
+    m_selectedNode = ModelNode();
+
+    if (selectedNodeList.isEmpty())
+        return;
+
+    const ModelNode node = selectedNodeList.constFirst();
+
+    if (QmlObjectNode(node).isValid())
+            m_selectedNode = node;
+}
+
 bool PropertyEditorView::hasWidget() const
 {
     return true;
@@ -679,7 +795,7 @@ bool PropertyEditorView::hasWidget() const
 
 WidgetInfo PropertyEditorView::widgetInfo()
 {
-    return createWidgetInfo(m_stackedWidget, 0, QStringLiteral("Properties"), WidgetInfo::RightPane, 0);
+    return createWidgetInfo(m_stackedWidget, nullptr, QStringLiteral("Properties"), WidgetInfo::RightPane, 0, tr("Properties"));
 }
 
 void PropertyEditorView::currentStateChanged(const ModelNode &node)
@@ -697,7 +813,7 @@ void PropertyEditorView::instancePropertyChanged(const QList<QPair<ModelNode, Pr
         return;
     m_locked = true;
 
-    typedef QPair<ModelNode, PropertyName> ModelNodePropertyPair;
+    using ModelNodePropertyPair = QPair<ModelNode, PropertyName>;
     foreach (const ModelNodePropertyPair &propertyPair, propertyList) {
         const ModelNode modelNode = propertyPair.first;
         const QmlObjectNode qmlObjectNode(modelNode);
@@ -753,9 +869,9 @@ void PropertyEditorView::reloadQml()
         m_stackedWidget->removeWidget(widget);
         delete widget;
     }
-    m_qmlBackEndForCurrentType = 0;
+    m_qmlBackEndForCurrentType = nullptr;
 
-    delayedResetView();
+    resetView();
 }
 
 
