@@ -25,22 +25,23 @@
 
 #include "quick2propertyeditorview.h"
 
-#include "propertyeditorvalue.h"
+#include "aligndistribute.h"
+#include "annotationeditor/annotationeditor.h"
+#include "bindingeditor/actioneditor.h"
+#include "bindingeditor/bindingeditor.h"
+#include "colorpalettebackend.h"
 #include "fileresourcesmodel.h"
 #include "gradientmodel.h"
-#include "gradientpresetdefaultlistmodel.h"
 #include "gradientpresetcustomlistmodel.h"
+#include "gradientpresetdefaultlistmodel.h"
 #include "itemfiltermodel.h"
-#include "simplecolorpalettemodel.h"
-#include "bindingeditor/bindingeditor.h"
-#include "bindingeditor/actioneditor.h"
-#include "annotationeditor/annotationeditor.h"
-#include "qmlanchorbindingproxy.h"
-#include "theme.h"
-#include "aligndistribute.h"
 #include "propertyeditorcontextobject.h"
-#include "tooltip.h"
+#include "propertyeditorqmlbackend.h"
+#include "propertyeditorvalue.h"
+#include "qmlanchorbindingproxy.h"
 #include "richtexteditor/richtexteditorproxy.h"
+#include "theme.h"
+#include "tooltip.h"
 
 namespace QmlDesigner {
 
@@ -62,7 +63,7 @@ void Quick2PropertyEditorView::registerQmlTypes()
         GradientPresetDefaultListModel::registerDeclarativeType();
         GradientPresetCustomListModel::registerDeclarativeType();
         ItemFilterModel::registerDeclarativeType();
-        SimpleColorPaletteModel::registerDeclarativeType();
+        ColorPaletteBackend::registerDeclarativeType();
         Internal::QmlAnchorBindingProxy::registerDeclarativeType();
         BindingEditor::registerDeclarativeType();
         ActionEditor::registerDeclarativeType();
@@ -71,6 +72,22 @@ void Quick2PropertyEditorView::registerQmlTypes()
         Tooltip::registerDeclarativeType();
         EasingCurveEditor::registerDeclarativeType();
         RichTextEditorProxy::registerDeclarativeType();
+
+        const QString resourcePath = PropertyEditorQmlBackend::propertyEditorResourcesPath();
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        QUrl regExpUrl = QUrl::fromLocalFile(resourcePath + "/RegExpValidator.qml");
+        qmlRegisterType(regExpUrl, "HelperWidgets", 2, 0, "RegExpValidator");
+
+        const QString qtPrefix = "/Qt6";
+#else
+        const QString qtPrefix = "/Qt5";
+#endif
+        qmlRegisterType(QUrl::fromLocalFile(resourcePath + qtPrefix + "HelperWindow.qml"),
+                        "HelperWidgets",
+                        2,
+                        0,
+                        "HelperWindow");
     }
 }
 
