@@ -70,14 +70,6 @@
 
 namespace QmlDesigner {
 
-static int deleteKey()
-{
-    if (Utils::HostOsInfo::isMacHost())
-        return Qt::Key_Backspace;
-
-    return Qt::Key_Delete;
-}
-
 QList<QmlTimelineKeyframeGroup> allTimelineFrames(const QmlTimeline &timeline)
 {
     QList<QmlTimelineKeyframeGroup> returnList;
@@ -144,8 +136,7 @@ void TimelineGraphicsScene::onShow()
             int cf = std::round(timeline.currentKeyframe());
             setCurrentFrame(cf);
         }
-
-        emit m_layout->zoomChanged(0);
+        emit m_layout->zoomChanged(zoom());
     }
 }
 
@@ -511,7 +502,11 @@ QGraphicsView *AbstractScrollGraphicsScene::rulerView() const
 
 QmlTimeline TimelineGraphicsScene::currentTimeline() const
 {
-    return QmlTimeline(timelineModelNode());
+    QmlTimeline timeline(timelineModelNode());
+    if (timeline.isValid()) {
+        QTC_ASSERT(timeline == timelineView()->currentTimeline(), ;);
+    }
+    return timelineView()->currentTimeline();
 }
 
 QRectF AbstractScrollGraphicsScene::selectionBounds() const
@@ -561,7 +556,7 @@ void AbstractScrollGraphicsScene::selectKeyframes(const SelectionMode &mode,
 
 void AbstractScrollGraphicsScene::clearSelection()
 {
-    for (auto *keyframe : m_selectedKeyframes)
+    for (auto *keyframe : qAsConst(m_selectedKeyframes))
         if (keyframe)
             keyframe->setHighlighted(false);
 
@@ -594,6 +589,9 @@ void TimelineGraphicsScene::mousePressEvent(QGraphicsSceneMouseEvent *event)
 void TimelineGraphicsScene::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 {
     auto topItem = TimelineMovableAbstractItem::topMoveableItem(itemsAt(event->scenePos()));
+
+    m_layout->ruler()->updatePlaybackLoop(event);
+
     m_tools.mouseMoveEvent(topItem, event);
     QGraphicsScene::mouseMoveEvent(event);
 }
@@ -601,6 +599,10 @@ void TimelineGraphicsScene::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 void TimelineGraphicsScene::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
     auto topItem = TimelineMovableAbstractItem::topMoveableItem(itemsAt(event->scenePos()));
+
+
+    m_layout->ruler()->updatePlaybackLoop(event);
+
     /* The tool has handle the event last. */
     QGraphicsScene::mouseReleaseEvent(event);
     m_tools.mouseReleaseEvent(topItem, event);
@@ -663,7 +665,7 @@ void TimelineGraphicsScene::keyReleaseEvent(QKeyEvent *keyEvent)
         return;
     }
 
-    if (deleteKey() == keyEvent->key())
+    if (TimelineUtils::isDeleteKey(keyEvent->key()))
         handleKeyframeDeletion();
 
     QGraphicsScene::keyReleaseEvent(keyEvent);
@@ -691,6 +693,11 @@ void TimelineGraphicsScene::invalidateSections()
     invalidateLayout();
 }
 
+TimelineRulerSectionItem *TimelineGraphicsScene::layoutRuler() const
+{
+    return m_layout->ruler();
+}
+
 TimelineView *TimelineGraphicsScene::timelineView() const
 {
     return m_parent->timelineView();
@@ -822,7 +829,7 @@ bool TimelineGraphicsScene::event(QEvent *event)
 {
     switch (event->type()) {
     case QEvent::ShortcutOverride:
-        if (static_cast<QKeyEvent *>(event)->key() == deleteKey()) {
+        if (TimelineUtils::isDeleteKey(static_cast<QKeyEvent *>(event)->key())) {
             QGraphicsScene::keyPressEvent(static_cast<QKeyEvent *>(event));
             event->accept();
             return true;
