@@ -71,6 +71,7 @@ TimelineView::TimelineView(QObject *parent)
     , m_timelineWidget(nullptr)
 {
     EasingCurve::registerStreamOperators();
+    setEnabled(false);
 }
 
 TimelineView::~TimelineView() = default;
@@ -299,6 +300,44 @@ TimelineWidget *TimelineView::widget() const
     return m_timelineWidget;
 }
 
+QList<QmlModelState> getAllStates(TimelineView* view)
+{
+    QmlVisualNode visNode(view->rootModelNode());
+    if (visNode.isValid()) {
+        return visNode.states().allStates();
+    }
+    return {};
+}
+
+QString getStateName(TimelineView* view, bool& enableInBaseState)
+{
+    QString currentStateName;
+    if (QmlModelState state = view->currentState(); state.isValid()) {
+        if (!state.isBaseState()) {
+            enableInBaseState = false;
+            return state.name();
+        }
+    }
+    return QString();
+}
+
+void enableInCurrentState(
+    TimelineView* view, const QString& stateName,
+    const ModelNode& node, const PropertyName& propertyName)
+{
+    if (!stateName.isEmpty()) {
+        for (auto& state : getAllStates(view)) {
+            if (state.isValid()) {
+                QmlPropertyChanges propertyChanges(state.propertyChanges(node));
+                if (state.name() == stateName)
+                    propertyChanges.modelNode().variantProperty(propertyName).setValue(true);
+                else
+                    propertyChanges.modelNode().variantProperty(propertyName).setValue(false);
+            }
+        }
+    }
+}
+
 const QmlTimeline TimelineView::addNewTimeline()
 {
     const TypeName timelineType = "QtQuick.Timeline.Timeline";
@@ -321,6 +360,7 @@ const QmlTimeline TimelineView::addNewTimeline()
 
     executeInTransaction("TimelineView::addNewTimeline", [=, &timelineNode]() {
         bool hasTimelines = getTimelines().isEmpty();
+        QString currentStateName = getStateName(this, hasTimelines);
 
         timelineNode = createModelNode(timelineType,
                                        metaInfo.majorVersion(),
@@ -332,6 +372,8 @@ const QmlTimeline TimelineView::addNewTimeline()
         timelineNode.variantProperty("enabled").setValue(hasTimelines);
 
         rootModelNode().defaultNodeListProperty().reparentHere(timelineNode);
+
+        enableInCurrentState(this, currentStateName, timelineNode, "enabled");
     });
 
     return QmlTimeline(timelineNode);
@@ -352,6 +394,9 @@ ModelNode TimelineView::addAnimation(QmlTimeline timeline)
     ModelNode animationNode;
 
     executeInTransaction("TimelineView::addAnimation", [=, &animationNode]() {
+        bool hasAnimations = getAnimations(timeline).isEmpty();
+        QString currentStateName = getStateName(this, hasAnimations);
+
         animationNode = createModelNode(animationType,
                                         metaInfo.majorVersion(),
                                         metaInfo.minorVersion());
@@ -363,12 +408,14 @@ ModelNode TimelineView::addAnimation(QmlTimeline timeline)
 
         animationNode.variantProperty("loops").setValue(1);
 
-        animationNode.variantProperty("running").setValue(getAnimations(timeline).isEmpty());
+        animationNode.variantProperty("running").setValue(hasAnimations);
 
         timeline.modelNode().nodeListProperty("animations").reparentHere(animationNode);
 
         if (timeline.modelNode().hasProperty("currentFrame"))
             timeline.modelNode().removeProperty("currentFrame");
+
+        enableInCurrentState(this, currentStateName, animationNode, "running");
     });
 
     return animationNode;
@@ -378,6 +425,7 @@ void TimelineView::addNewTimelineDialog()
 {
     auto timeline = addNewTimeline();
     addAnimation(timeline);
+    setCurrentTimeline(timeline);
     openSettingsDialog();
 }
 
@@ -404,13 +452,13 @@ void TimelineView::openSettingsDialog()
 
 void TimelineView::setTimelineRecording(bool value)
 {
-    ModelNode node = widget()->graphicsScene()->currentTimeline();
+    const ModelNode node = timelineForState(currentState()).modelNode();
 
     if (value && node.isValid()) {
         activateTimelineRecording(node);
     } else {
         deactivateTimelineRecording();
-        activateTimeline(node);
+        setCurrentTimeline(node);
     }
 }
 
@@ -419,34 +467,22 @@ void TimelineView::customNotification(const AbstractView * /*view*/,
                                       const QList<ModelNode> &nodeList,
                                       const QList<QVariant> &data)
 {
+    Q_UNUSED(nodeList)
+    Q_UNUSED(data)
     if (identifier == QStringLiteral("reset QmlPuppet")) {
         QmlTimeline timeline = widget()->graphicsScene()->currentTimeline();
         if (timeline.isValid())
             timeline.modelNode().removeAuxiliaryData("currentFrame@NodeInstance");
-    } else if (identifier == "INSERT_KEYFRAME" && !nodeList.isEmpty() && !data.isEmpty()) {
-        insertKeyframe(nodeList.constFirst(), data.constFirst().toString().toUtf8());
     }
 }
 
 void TimelineView::insertKeyframe(const ModelNode &target, const PropertyName &propertyName)
 {
-    QmlTimeline timeline = widget()->graphicsScene()->currentTimeline();
-    ModelNode targetNode = target;
-    if (timeline.isValid() && targetNode.isValid()
-        && QmlObjectNode::isValidQmlObjectNode(targetNode)) {
-        executeInTransaction("TimelineView::insertKeyframe", [=, &timeline, &targetNode]() {
-            targetNode.validId();
+    QmlTimeline timeline = currentTimeline();
 
-            QmlTimelineKeyframeGroup timelineFrames(
-                timeline.keyframeGroup(targetNode, propertyName));
-
-            QTC_ASSERT(timelineFrames.isValid(), return );
-
-            const qreal frame
-                = timeline.modelNode().auxiliaryData("currentFrame@NodeInstance").toReal();
-            const QVariant value = QmlObjectNode(targetNode).instanceValue(propertyName);
-
-            timelineFrames.setValue(value, frame);
+    if (timeline.isValid() && target.isValid() && QmlObjectNode::isValidQmlObjectNode(target)) {
+        executeInTransaction("TimelineView::insertKeyframe", [=, &timeline, &target]() {
+            timeline.insertKeyframe(target, propertyName);
         });
     }
 }
