@@ -60,6 +60,7 @@
 #include <QVBoxLayout>
 #include <QtGlobal>
 #include <QSpacerItem>
+#include <QVariantAnimation>
 
 #include <cmath>
 
@@ -117,18 +118,21 @@ TimelineWidget::TimelineWidget(TimelineView *view)
     , m_toolbar(new TimelineToolBar(this))
     , m_rulerView(new QGraphicsView(this))
     , m_graphicsView(new QGraphicsView(this))
-    , m_scrollbar(new Navigation2dScrollBar(this))
+    , m_scrollbar(new QScrollBar(this))
     , m_statusBar(new QLabel(this))
     , m_timelineView(view)
     , m_graphicsScene(new TimelineGraphicsScene(this))
     , m_addButton(new QPushButton(this))
     , m_onboardingContainer(new QWidget(this))
+    , m_loopPlayback(false)
+    , m_playbackSpeed(1)
+    , m_playbackAnimation(new QVariantAnimation(this))
 {
     setWindowTitle(tr("Timeline", "Title of timeline view"));
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
     const QString css = Theme::replaceCssColors(QString::fromUtf8(
-        Utils::FileReader::fetchQrc(QLatin1String(":/qmldesigner/scrollbar.css"))));
+        Utils::FileReader::fetchQrc(":/qmldesigner/scrollbar.css")));
 
     m_scrollbar->setStyleSheet(css);
     m_scrollbar->setOrientation(Qt::Horizontal);
@@ -156,7 +160,6 @@ TimelineWidget::TimelineWidget(TimelineView *view)
     m_graphicsView->setFrameShape(QFrame::NoFrame);
     m_graphicsView->setFrameShadow(QFrame::Plain);
     m_graphicsView->setLineWidth(0);
-    m_graphicsView->setVerticalScrollBar(new Navigation2dScrollBar);
     m_graphicsView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
     m_graphicsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
@@ -205,9 +208,10 @@ TimelineWidget::TimelineWidget(TimelineView *view)
     auto *topSpacer = new QSpacerItem(40, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);
     auto *bottomSpacer = new QSpacerItem(40, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);
 
-    QString labelText =
-            tr("This file does not contain a timeline. <br><br> \
-            To create an animation, add a timeline by clicking the + button.");
+    const QString labelText =
+            tr("This file does not contain a timeline. <br><br>"
+               "To create an animation, add a timeline by clicking the + button.");
+
     onboardingTopLabel->setText(labelText);
     onboardingTopLabel->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
 
@@ -238,6 +242,23 @@ TimelineWidget::TimelineWidget(TimelineView *view)
     widgetLayout->addLayout(contentLayout);
     this->setLayout(widgetLayout);
 
+    {
+        QPalette timelinePalette;
+        timelinePalette.setColor(QPalette::Text, Utils::creatorTheme()->color(
+                                     Utils::Theme::DStextColor));
+        timelinePalette.setColor(QPalette::WindowText, timelinePalette.color(QPalette::Text));
+        timelinePalette.setColor(QPalette::Window, Utils::creatorTheme()->color(
+                                     Utils::Theme::QmlDesigner_BackgroundColorDarkAlternate));
+
+        onboardingTopLabel->setPalette(timelinePalette);
+        onboardingBottomLabel->setPalette(timelinePalette);
+        setPalette(timelinePalette);
+        setAutoFillBackground(true);
+        m_graphicsView->setPalette(timelinePalette);
+        m_graphicsView->setBackgroundRole(QPalette::Window);
+        m_statusBar->setPalette(timelinePalette);
+    }
+
     connectToolbar();
 
     auto setScrollOffset = [this]() { graphicsScene()->setScrollOffset(m_scrollbar->value()); };
@@ -252,13 +273,37 @@ TimelineWidget::TimelineWidget(TimelineView *view)
         m_timelineView->addNewTimelineDialog();
     });
 
-    Navigation2dFilter *filter = new Navigation2dFilter(this, m_scrollbar);
-    connect(filter, &Navigation2dFilter::zoomChanged, [this](double scale, const QPointF& pos) {
-        int s = static_cast<int>(std::round(scale*100.));
-        double ps = m_graphicsScene->mapFromScene(pos.x());
-        m_graphicsScene->setZoom(std::clamp(m_graphicsScene->zoom() + s, 0, 100), ps);
+    Navigation2dFilter *filter = new Navigation2dFilter(m_graphicsView->viewport());
+    connect(filter, &Navigation2dFilter::panChanged, [this](const QPointF &direction) {
+        Navigation2dFilter::scroll(direction, m_scrollbar, m_graphicsView->verticalScrollBar());
     });
-    installEventFilter(filter);
+
+    connect(filter, &Navigation2dFilter::zoomChanged, [this](double scale, const QPointF &pos) {
+        int s = static_cast<int>(std::round(scale*100.));
+        int scaleFactor = std::clamp(m_graphicsScene->zoom() + s, 0, 100);
+        double ps = m_graphicsScene->mapFromScene(pos.x());
+        m_graphicsScene->setZoom(scaleFactor, ps);
+        m_toolbar->setScaleFactor(scaleFactor);
+    });
+    m_graphicsView->viewport()->installEventFilter(filter);
+
+    m_playbackAnimation->stop();
+    auto playAnimation = [this](QVariant frame) { graphicsScene()->setCurrentFrame(qRound(frame.toDouble())); };
+    connect(m_playbackAnimation, &QVariantAnimation::valueChanged, playAnimation);
+
+    auto updatePlaybackLoopValues = [this]() {
+        updatePlaybackValues();
+    };
+    connect(graphicsScene()->layoutRuler(), &TimelineRulerSectionItem::playbackLoopValuesChanged, updatePlaybackLoopValues);
+
+    auto setPlaybackState = [this](QAbstractAnimation::State newState, QAbstractAnimation::State oldState) {
+        Q_UNUSED(oldState)
+        m_toolbar->setPlayState(newState == QAbstractAnimation::State::Running);
+    };
+    connect(m_playbackAnimation, &QVariantAnimation::stateChanged, setPlaybackState);
+
+    auto onFinish = [this]() { graphicsScene()->setCurrentFrame(m_playbackAnimation->startValue().toInt()); };
+    connect(m_playbackAnimation, &QVariantAnimation::finished, onFinish);
 }
 
 void TimelineWidget::connectToolbar()
@@ -302,6 +347,21 @@ void TimelineWidget::connectToolbar()
 
     connect(m_toolbar, &TimelineToolBar::recordToggled, this, &TimelineWidget::setTimelineRecording);
 
+    connect(m_toolbar, &TimelineToolBar::playTriggered, this, &TimelineWidget::toggleAnimationPlayback);
+
+    auto setLoopAnimation = [this](bool loop) {
+        graphicsScene()->layoutRuler()->setPlaybackLoopEnabled(loop);
+        if (m_playbackAnimation->state() == QAbstractAnimation::Running)
+            m_playbackAnimation->pause();
+        m_loopPlayback = loop;
+    };
+    connect(m_toolbar, &TimelineToolBar::loopPlaybackToggled, setLoopAnimation);
+    auto setPlaybackSpeed = [this](float val) {
+        m_playbackSpeed = val;
+        updatePlaybackValues();
+    };
+    connect(m_toolbar, &TimelineToolBar::playbackSpeedChanged, setPlaybackSpeed);
+
     connect(m_toolbar,
             &TimelineToolBar::openEasingCurveEditor,
             this,
@@ -311,19 +371,6 @@ void TimelineWidget::connectToolbar()
             &TimelineToolBar::settingDialogClicked,
             m_timelineView,
             &TimelineView::openSettingsDialog);
-
-    for (auto action : QmlDesignerPlugin::instance()->designerActionManager().designerActions()) {
-        if (action->menuId() == "LivePreview") {
-            QObject::connect(m_toolbar,
-                             &TimelineToolBar::playTriggered,
-                             action->action(),
-                             [action]() {
-                                 action->action()->setChecked(false);
-                                 action->action()->triggered(true);
-                             });
-        }
-    }
-
     setTimelineActive(false);
 }
 
@@ -365,6 +412,54 @@ void TimelineWidget::openEasingCurveEditor()
     }
 }
 
+void TimelineWidget::updatePlaybackValues()
+{
+    QmlTimeline currentTimeline = graphicsScene()->currentTimeline();
+    qreal endFrame = currentTimeline.endKeyframe();
+    qreal startFrame = currentTimeline.startKeyframe();
+    qreal duration = currentTimeline.duration();
+
+    if (m_loopPlayback) {
+        m_playbackAnimation->setLoopCount(-1);
+        qreal loopStart = graphicsScene()->layoutRuler()->playbackLoopStart();
+        qreal loopEnd = graphicsScene()->layoutRuler()->playbackLoopEnd();
+        startFrame = qRound(startFrame + loopStart);
+        endFrame = qRound(startFrame + (loopEnd - loopStart));
+        duration = endFrame - startFrame;
+    } else {
+        m_playbackAnimation->setLoopCount(1);
+    }
+
+    if (duration > 0.0f) {
+        qreal animationDuration = duration * (1.0 / m_playbackSpeed);
+        m_playbackAnimation->setDuration(animationDuration);
+    } else {
+        if (m_playbackAnimation->state() == QAbstractAnimation::Running)
+            m_playbackAnimation->stop();
+    }
+    qreal newCurrentTime = (currentTimeline.currentKeyframe() - startFrame) * (1.0 / m_playbackSpeed);
+    if (qRound(m_playbackAnimation->startValue().toDouble()) != qRound(startFrame)
+        || qRound(m_playbackAnimation->endValue().toDouble()) != qRound(endFrame)) {
+        newCurrentTime = 0;
+    }
+    m_playbackAnimation->setStartValue(qRound(startFrame));
+    m_playbackAnimation->setEndValue(qRound(endFrame));
+    m_playbackAnimation->setCurrentTime(newCurrentTime);
+}
+
+void TimelineWidget::toggleAnimationPlayback()
+{
+    QmlTimeline currentTimeline = graphicsScene()->currentTimeline();
+    if (currentTimeline.isValid() && m_playbackSpeed > 0.0) {
+        if (m_playbackAnimation->state() == QAbstractAnimation::Running) {
+            m_playbackAnimation->pause();
+        } else {
+            updatePlaybackValues();
+            m_playbackAnimation->start();
+        }
+    }
+}
+
 void TimelineWidget::setTimelineRecording(bool value)
 {
     ModelNode node = timelineView()->modelNodeForId(m_toolbar->currentTimelineId());
@@ -373,7 +468,7 @@ void TimelineWidget::setTimelineRecording(bool value)
         timelineView()->activateTimelineRecording(node);
     } else {
         timelineView()->deactivateTimelineRecording();
-        timelineView()->activateTimeline(node);
+        timelineView()->setCurrentTimeline(node);
     }
 
     graphicsScene()->invalidateRecordButtonsStatus();
@@ -387,7 +482,7 @@ void TimelineWidget::contextHelp(const Core::IContext::HelpCallback &callback) c
         callback({});
 }
 
-void TimelineWidget::init()
+void TimelineWidget::init(int zoom)
 {
     QmlTimeline currentTimeline = m_timelineView->timelineForState(m_timelineView->currentState());
     if (currentTimeline.isValid()) {
@@ -399,13 +494,13 @@ void TimelineWidget::init()
         m_statusBar->clear();
     }
 
-    invalidateTimelineDuration(m_graphicsScene->currentTimeline());
+    invalidateTimelineDuration(currentTimeline);
 
     m_graphicsScene->setWidth(m_graphicsView->viewport()->width());
 
     // setScaleFactor uses QSignalBlocker.
-    m_toolbar->setScaleFactor(0);
-    m_graphicsScene->setZoom(0);
+    m_toolbar->setScaleFactor(zoom);
+    m_graphicsScene->setZoom(zoom);
 }
 
 void TimelineWidget::reset()
@@ -428,7 +523,7 @@ TimelineToolBar *TimelineWidget::toolBar() const
 void TimelineWidget::invalidateTimelineDuration(const QmlTimeline &timeline)
 {
     if (timelineView() && timelineView()->model()) {
-        QmlTimeline currentTimeline = graphicsScene()->currentTimeline();
+        QmlTimeline currentTimeline = timelineView()->currentTimeline();
         if (currentTimeline.isValid() && currentTimeline == timeline) {
             m_toolbar->setStartFrame(timeline.startKeyframe());
             m_toolbar->setEndFrame(timeline.endKeyframe());
@@ -452,7 +547,7 @@ void TimelineWidget::invalidateTimelineDuration(const QmlTimeline &timeline)
 void TimelineWidget::invalidateTimelinePosition(const QmlTimeline &timeline)
 {
     if (timelineView() && timelineView()->model()) {
-        QmlTimeline currentTimeline = graphicsScene()->currentTimeline();
+        QmlTimeline currentTimeline = timelineView()->currentTimeline();
         if (currentTimeline.isValid() && currentTimeline == timeline) {
             qreal frame = getcurrentFrame(timeline);
             m_toolbar->setCurrentFrame(frame);
@@ -473,11 +568,20 @@ void TimelineWidget::setupScrollbar(int min, int max, int current)
 
 void TimelineWidget::setTimelineId(const QString &id)
 {
+    auto currentState = m_timelineView->currentState();
+    auto timelineOfState = m_timelineView->timelineForState(currentState.modelNode());
+
+    bool active = false;
+    if (auto node = timelineOfState.modelNode(); node.isValid())
+        active = node.id() == id;
+
     const bool empty = m_timelineView->getTimelines().isEmpty();
-    setTimelineActive(!empty);
-    if (m_timelineView->isAttached() && !empty) {
-        m_toolbar->setCurrentTimeline(m_timelineView->modelNodeForId(id));
-        m_toolbar->setCurrentState(m_timelineView->currentState().name());
+    const bool valid = active && !empty;
+
+    setTimelineActive(valid);
+    if (m_timelineView->isAttached() && valid) {
+        m_toolbar->setCurrentTimeline(timelineOfState);
+        m_toolbar->setCurrentState(currentState.name());
     } else {
         m_toolbar->setCurrentTimeline({});
         m_toolbar->setCurrentState({});
@@ -501,6 +605,7 @@ void TimelineWidget::setTimelineActive(bool b)
         m_graphicsView->setVisible(false);
         m_rulerView->setVisible(false);
         m_scrollbar->setVisible(false);
+        m_statusBar->clear();
         m_addButton->setVisible(true);
         m_onboardingContainer->setVisible(true);
     }
@@ -515,10 +620,9 @@ void TimelineWidget::showEvent(QShowEvent *event)
 {
     Q_UNUSED(event)
 
-    m_timelineView->setEnabled(true);
+    int zoom = m_toolbar->scaleFactor();
 
-    if (m_timelineView->model())
-        init();
+    m_timelineView->setEnabled(true);
 
     graphicsScene()->setWidth(m_graphicsView->viewport()->width());
     graphicsScene()->invalidateLayout();
@@ -526,6 +630,10 @@ void TimelineWidget::showEvent(QShowEvent *event)
     graphicsScene()->onShow();
 
     QWidget::showEvent(event);
+
+    //All the events have to be fully processed before we call init()
+    if (m_timelineView->model())
+        QTimer::singleShot(0, [this, zoom]() { init(zoom); });
 }
 
 void TimelineWidget::resizeEvent(QResizeEvent *event)
