@@ -90,7 +90,7 @@ TransitionEditorWidget::TransitionEditorWidget(TransitionEditorView *view)
     , m_toolbar(new TransitionEditorToolBar(this))
     , m_rulerView(new QGraphicsView(this))
     , m_graphicsView(new QGraphicsView(this))
-    , m_scrollbar(new Navigation2dScrollBar(this))
+    , m_scrollbar(new QScrollBar(this))
     , m_statusBar(new QLabel(this))
     , m_transitionEditorView(view)
     , m_graphicsScene(new TransitionEditorGraphicsScene(this))
@@ -100,8 +100,8 @@ TransitionEditorWidget::TransitionEditorWidget(TransitionEditorView *view)
     setWindowTitle(tr("Transition", "Title of transition view"));
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
-    const QString css = Theme::replaceCssColors(QString::fromUtf8(
-        Utils::FileReader::fetchQrc(QLatin1String(":/qmldesigner/scrollbar.css"))));
+    const QString css = Theme::replaceCssColors(
+        QString::fromUtf8(Utils::FileReader::fetchQrc(":/qmldesigner/scrollbar.css")));
 
     m_scrollbar->setStyleSheet(css);
     m_scrollbar->setOrientation(Qt::Horizontal);
@@ -129,7 +129,6 @@ TransitionEditorWidget::TransitionEditorWidget(TransitionEditorView *view)
     m_graphicsView->setFrameShape(QFrame::NoFrame);
     m_graphicsView->setFrameShadow(QFrame::Plain);
     m_graphicsView->setLineWidth(0);
-    m_graphicsView->setVerticalScrollBar(new Navigation2dScrollBar);
     m_graphicsView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
     m_graphicsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
@@ -223,13 +222,19 @@ TransitionEditorWidget::TransitionEditorWidget(TransitionEditorView *view)
         m_transitionEditorView->addNewTransition();
     });
 
-    Navigation2dFilter *filter = new Navigation2dFilter(this, m_scrollbar);
-    connect(filter, &Navigation2dFilter::zoomChanged, [this](double scale, const QPointF& pos) {
-        int s = static_cast<int>(std::round(scale*100.));
-        double ps = m_graphicsScene->mapFromScene(pos.x());
-        m_graphicsScene->setZoom(std::clamp(m_graphicsScene->zoom() + s, 0, 100), ps);
+    Navigation2dFilter *filter = new Navigation2dFilter(m_graphicsView->viewport());
+    connect(filter, &Navigation2dFilter::panChanged, [this](const QPointF &direction) {
+        Navigation2dFilter::scroll(direction, m_scrollbar, m_graphicsView->verticalScrollBar());
     });
-    installEventFilter(filter);
+
+    connect(filter, &Navigation2dFilter::zoomChanged, [this](double scale, const QPointF &pos) {
+        int s = static_cast<int>(std::round(scale*100.));
+        int scaleFactor = std::clamp(m_graphicsScene->zoom() + s, 0, 100);
+        double ps = m_graphicsScene->mapFromScene(pos.x());
+        m_graphicsScene->setZoom(scaleFactor, ps);
+        m_toolbar->setScaleFactor(scaleFactor);
+    });
+    m_graphicsView->viewport()->installEventFilter(filter);
 }
 
 void TransitionEditorWidget::setTransitionActive(bool b)
@@ -321,15 +326,18 @@ void TransitionEditorWidget::contextHelp(const Core::IContext::HelpCallback &cal
         callback({});
 }
 
-void TransitionEditorWidget::init()
+void TransitionEditorWidget::init(int zoom)
 {
     ModelNode root = transitionEditorView()->rootModelNode();
     ModelNode transition;
 
     if (root.isValid() && root.hasProperty("transitions")) {
         NodeAbstractProperty transitions = root.nodeAbstractProperty("transitions");
-        if (transitions.isValid())
-            transition = transitions.directSubNodes().first();
+        if (transitions.isValid()) {
+            const QList<ModelNode> directSubNodes = transitions.directSubNodes();
+            if (!directSubNodes.isEmpty())
+                transition = directSubNodes.constFirst();
+        }
     }
 
     m_graphicsScene->setTransition(transition);
@@ -337,7 +345,7 @@ void TransitionEditorWidget::init()
 
     m_graphicsScene->setWidth(m_graphicsView->viewport()->width());
 
-    m_toolbar->setScaleFactor(40);
+    m_toolbar->setScaleFactor(zoom);
 
     m_toolbar->setCurrentTransition(transition);
 
@@ -347,13 +355,13 @@ void TransitionEditorWidget::init()
 
     m_toolbar->setDuration(duration);
 
-    m_graphicsScene->setZoom(40);
+    m_graphicsScene->setZoom(zoom);
 }
 
 void TransitionEditorWidget::updateData(const ModelNode &transition)
 {
     if (!transition.isValid()) {
-        init();
+        init(m_toolbar->scaleFactor());
         return;
     }
 
@@ -401,7 +409,7 @@ void TransitionEditorWidget::showEvent(QShowEvent *event)
     m_transitionEditorView->setEnabled(true);
 
     if (m_transitionEditorView->model())
-        init();
+        init(m_toolbar->scaleFactor());
 
     graphicsScene()->setWidth(m_graphicsView->viewport()->width());
     graphicsScene()->invalidateLayout();
