@@ -91,17 +91,13 @@ bool ChangeImportsVisitor::equals(QmlJS::AST::UiImport *ast, const Import &impor
     else if (import.isFileImport())
         equal = ast->fileName == import.file();
 
-    if (equal) {
-        equal = (!ast->version || (ast->version->minorVersion == 0 && ast->version->majorVersion == 0))
-                && import.version().isEmpty();
-        if (!equal && ast->version) {
-            const QStringList versions = import.version().split('.');
-            if (versions.size() >= 1 && versions[0].toInt() == ast->version->majorVersion) {
-                if (versions.size() >= 2)
-                    equal = versions[1].toInt() == ast->version->minorVersion;
-                else
-                    equal = ast->version->minorVersion == 0;
-            }
+    if (equal && ast->version) {
+        const QStringList versions = import.version().split('.');
+        if (versions.size() >= 1 && versions[0].toInt() == ast->version->majorVersion) {
+            if (versions.size() >= 2)
+                equal = versions[1].toInt() == ast->version->minorVersion;
+            else
+                equal = ast->version->minorVersion == 0;
         }
     }
 
