@@ -45,6 +45,7 @@ QT_BEGIN_NAMESPACE
 class QStyle;
 class QToolButton;
 class QImage;
+class QPixmap;
 QT_END_NAMESPACE
 
 namespace QmlDesigner {
@@ -141,11 +142,12 @@ public:
                          const QString &nodeSource = QString(),
                          ModelNode::NodeSourceType nodeSourceType = ModelNode::NodeWithoutSource);
 
-    const ModelNode rootModelNode() const;
+    ModelNode rootModelNode() const;
     ModelNode rootModelNode();
 
     void setSelectedModelNodes(const QList<ModelNode> &selectedNodeList);
     void setSelectedModelNode(const ModelNode &modelNode);
+
     void selectModelNode(const ModelNode &node);
     void deselectModelNode(const ModelNode &node);
     void clearSelectedModelNodes();
@@ -186,6 +188,8 @@ public:
     void emitInstanceToken(const QString &token, int number, const QVector<ModelNode> &nodeVector);
     void emitRenderImage3DChanged(const QImage &image);
     void emitUpdateActiveScene3D(const QVariantMap &sceneState);
+    void emitModelNodelPreviewPixmapChanged(const ModelNode &node, const QPixmap &pixmap);
+    void emitImport3DSupportChanged(const QVariantMap &supportMap);
 
     void sendTokenToInstances(const QString &token, int number, const QVector<ModelNode> &nodeVector);
 
@@ -201,6 +205,7 @@ public:
     virtual void propertiesAboutToBeRemoved(const QList<AbstractProperty>& propertyList);
     virtual void propertiesRemoved(const QList<AbstractProperty>& propertyList);
     virtual void variantPropertiesChanged(const QList<VariantProperty>& propertyList, PropertyChangeFlags propertyChange);
+    virtual void bindingPropertiesAboutToBeChanged(const QList<BindingProperty> &propertyList);
     virtual void bindingPropertiesChanged(const QList<BindingProperty>& propertyList, PropertyChangeFlags propertyChange);
     virtual void signalHandlerPropertiesChanged(const QVector<SignalHandlerProperty>& propertyList, PropertyChangeFlags propertyChange);
     virtual void rootNodeTypeChanged(const QString &type, int majorVersion, int minorVersion);
@@ -227,7 +232,10 @@ public:
 
     virtual void fileUrlChanged(const QUrl &oldUrl, const QUrl &newUrl);
 
-    virtual void nodeOrderChanged(const NodeListProperty &listProperty, const ModelNode &movedNode, int oldIndex);
+    virtual void nodeOrderChanged(const NodeListProperty &listProperty);
+    virtual void nodeOrderChanged(const NodeListProperty &listProperty,
+                                  const ModelNode &movedNode,
+                                  int oldIndex);
 
     virtual void importsChanged(const QList<Import> &addedImports, const QList<Import> &removedImports);
     virtual void possibleImportsChanged(const QList<Import> &possibleImports);
@@ -245,6 +253,8 @@ public:
 
     virtual void renderImage3DChanged(const QImage &image);
     virtual void updateActiveScene3D(const QVariantMap &sceneState);
+    virtual void updateImport3DSupport(const QVariantMap &supportMap);
+    virtual void modelNodePreviewPixmapChanged(const ModelNode &node, const QPixmap &pixmap);
 
     void changeRootNodeType(const TypeName &type, int majorVersion, int minorVersion);
 
@@ -270,13 +280,33 @@ public:
 
     virtual void contextHelp(const Core::IContext::HelpCallback &callback) const;
 
-    void activateTimeline(const ModelNode &timeline);
+    void setCurrentTimeline(const ModelNode &timeline);
     void activateTimelineRecording(const ModelNode &timeline);
     void deactivateTimelineRecording();
 
     using OperationBlock = std::function<void()>;
     bool executeInTransaction(const QByteArray &identifier, const OperationBlock &lambda);
 
+    bool isEnabled() const;
+    void setEnabled(bool b);
+
+    bool isBlockingNotifications() const { return m_isBlockingNotifications; }
+
+    class NotificationBlocker
+    {
+    public:
+        NotificationBlocker(AbstractView *view)
+            : m_view{view}
+        {
+            m_view->m_isBlockingNotifications = true;
+        }
+
+        ~NotificationBlocker() { m_view->m_isBlockingNotifications = false; }
+
+    private:
+        AbstractView *m_view;
+    };
+
 protected:
     void setModel(Model * model);
     void removeModel();
@@ -290,9 +320,10 @@ protected:
 private: //functions
     QList<ModelNode> toModelNodeList(const QList<Internal::InternalNodePointer> &nodeList) const;
 
-
 private:
     QPointer<Model> m_model;
+    bool m_enabled = true;
+    bool m_isBlockingNotifications = false;
 };
 
 QMLDESIGNERCORE_EXPORT QList<Internal::InternalNodePointer> toInternalNodeList(const QList<ModelNode> &nodeList);
