@@ -29,6 +29,8 @@
 
 #include "propertycontainer.h"
 #include <QPointer>
+#include <QSet>
+#include <memory>
 
 namespace QmlDesigner {
 
@@ -46,15 +48,13 @@ QMLDESIGNERCORE_EXPORT QDebug operator<<(QDebug debug, const ItemLibraryEntry &i
 
 class QMLDESIGNERCORE_EXPORT ItemLibraryEntry
 {
-    //friend class QmlDesigner::MetaInfo;
-    //friend class QmlDesigner::Internal::MetaInfoParser;
     friend QMLDESIGNERCORE_EXPORT QDataStream& operator<<(QDataStream& stream, const ItemLibraryEntry &itemLibraryEntry);
     friend QMLDESIGNERCORE_EXPORT QDataStream& operator>>(QDataStream& stream, ItemLibraryEntry &itemLibraryEntry);
     friend QMLDESIGNERCORE_EXPORT QDebug operator<<(QDebug debug, const ItemLibraryEntry &itemLibraryEntry);
 
 public:
     ItemLibraryEntry();
-    ~ItemLibraryEntry();
+    ~ItemLibraryEntry() = default;
 
     QString name() const;
     TypeName typeName() const;
@@ -67,9 +67,8 @@ public:
     QString qmlPath() const;
     QString qmlSource() const;
     QString requiredImport() const;
-
-    ItemLibraryEntry(const ItemLibraryEntry &other);
-    ItemLibraryEntry& operator=(const ItemLibraryEntry &other);
+    QString customComponentSource() const;
+    QStringList extraFilePaths() const;
 
     using Property = QmlDesigner::PropertyContainer;
 
@@ -86,9 +85,11 @@ public:
     void setQmlPath(const QString &qml);
     void setRequiredImport(const QString &requiredImport);
     void addHints(const QHash<QString, QString> &hints);
+    void setCustomComponentSource(const QString &source);
+    void addExtraFilePath(const QString &extraFile);
 
 private:
-    QExplicitlySharedDataPointer<Internal::ItemLibraryEntryData> m_data;
+    std::shared_ptr<Internal::ItemLibraryEntryData> m_data;
 };
 
 class QMLDESIGNERCORE_EXPORT ItemLibraryInfo : public QObject
@@ -99,21 +100,22 @@ class QMLDESIGNERCORE_EXPORT ItemLibraryInfo : public QObject
 public:
 
     QList<ItemLibraryEntry> entries() const;
-    QList<ItemLibraryEntry> entriesForType(const QByteArray &typeName, int majorVersion, int minorVersion) const;
-    ItemLibraryEntry entry(const QString &name) const;
+    QList<ItemLibraryEntry> entriesForType(const QByteArray &typeName, int majorVersion = 1,
+                                           int minorVersion = 0) const;
 
     void addEntries(const QList<ItemLibraryEntry> &entries, bool overwriteDuplicate = false);
     bool containsEntry(const ItemLibraryEntry &entry);
     void clearEntries();
 
     QStringList blacklistImports() const;
-    QStringList showTagsForImports() const;
+    QSet<QString> priorityImports() const;
 
     void addBlacklistImports(const QStringList &list);
-    void addShowTagsForImports(const QStringList &list);
+    void addPriorityImports(const QSet<QString> &set);
 
 signals:
     void entriesChanged();
+    void priorityImportsChanged();
 
 private: // functions
     ItemLibraryInfo(QObject *parent = nullptr);
@@ -124,7 +126,7 @@ private: // variables
     QPointer<ItemLibraryInfo> m_baseInfo;
 
     QStringList m_blacklistImports;
-    QStringList m_showTagsForImports;
+    QSet<QString> m_priorityImports;
 };
 
 } // namespace QmlDesigner
