@@ -39,7 +39,6 @@ class ItemLibraryEntry;
 
 namespace Internal {
 
-
 class MetaInfoReader : protected QmlJS::SimpleAbstractStreamReader
 {
     Q_DECLARE_TR_FUNCTIONS(QmlDesigner::Internal::MetaInfoReader)
@@ -55,9 +54,12 @@ public:
     void setQualifcation(const TypeName &qualification);
 
 protected:
-    void elementStart(const QString &name) override;
+    void elementStart(const QString &name, const QmlJS::SourceLocation &nameLocation) override;
     void elementEnd() override;
-    void propertyDefinition(const QString &name, const QVariant &value) override;
+    void propertyDefinition(const QString &name,
+                            const QmlJS::SourceLocation &nameLocation,
+                            const QVariant &value,
+                            const QmlJS::SourceLocation &valueLocation) override;
 
 private:
     enum ParserSate { Error,
@@ -70,7 +72,8 @@ private:
                       ParsingItemLibrary,
                       ParsingHints,
                       ParsingProperty,
-                      ParsingQmlSource
+                      ParsingQmlSource,
+                      ParsingExtraFile
                     };
 
     ParserSate readDocument(const QString &name);
@@ -80,12 +83,14 @@ private:
     ParserSate readItemLibraryEntryElement(const QString &name);
     ParserSate readPropertyElement(const QString &name);
     ParserSate readQmlSourceElement(const QString &name);
+    ParserSate readExtraFileElement(const QString &name);
 
     void readTypeProperty(const QString &name, const QVariant &value);
     void readImportsProperty(const QString &name, const QVariant &value);
     void readItemLibraryEntryProperty(const QString &name, const QVariant &value);
     void readPropertyProperty(const QString &name, const QVariant &value);
     void readQmlSourceProperty(const QString &name, const QVariant &value);
+    void readExtraFileProperty(const QString &name, const QVariant &value);
     void readHint(const QString &name, const QVariant &value);
 
     void setVersion(const QString &versionNumber);
