@@ -64,12 +64,12 @@ using PropertyListType = QList<QPair<PropertyName, QVariant> >;
 
 class QMLDESIGNERCORE_EXPORT Model : public QObject
 {
-    friend class QmlDesigner::ModelNode;
-    friend class QmlDesigner::AbstractProperty;
-    friend class QmlDesigner::AbstractView;
-    friend class Internal::ModelPrivate;
-    friend class Internal::WriteLocker;
-    friend class QmlDesigner::Internal::NodeMetaInfoPrivate;
+    friend ModelNode;
+    friend AbstractProperty;
+    friend AbstractView;
+    friend Internal::ModelPrivate;
+    friend Internal::WriteLocker;
+    friend Internal::NodeMetaInfoPrivate;
 
     Q_OBJECT
 
@@ -81,12 +81,14 @@ public:
     static Model *create(TypeName type, int major = 1, int minor = 1, Model *metaInfoPropxyModel = nullptr);
 
     QUrl fileUrl() const;
+    QUrl projectUrl() const;
     void setFileUrl(const QUrl &url);
 
     const MetaInfo metaInfo() const;
     MetaInfo metaInfo();
     NodeMetaInfo metaInfo(const TypeName &typeName, int majorVersion = -1, int minorVersion = -1);
     bool hasNodeMetaInfo(const TypeName &typeName, int majorVersion = -1, int minorVersion = -1);
+    void setMetaInfo(const MetaInfo &metaInfo);
 
     void attachView(AbstractView *view);
     void detachView(AbstractView *view, ViewNotification emitDetachNotify = NotifyView);
@@ -94,9 +96,9 @@ public:
     // Editing sub-components:
 
     // Imports:
-    QList<Import> imports() const;
-    QList<Import> possibleImports() const;
-    QList<Import> usedImports() const;
+    const QList<Import> &imports() const;
+    const QList<Import> &possibleImports() const;
+    const QList<Import> &usedImports() const;
     void changeImports(const QList<Import> &importsToBeAdded, const QList<Import> &importsToBeRemoved);
     void setPossibleImports(const QList<Import> &possibleImports);
     void setUsedImports(const QList<Import> &usedImports);
