@@ -36,10 +36,11 @@ class QmlObjectNode;
 class QmlModelStateGroup;
 class Annotation;
 class AnnotationEditor;
+class StatesEditorView;
 
 class QMLDESIGNERCORE_EXPORT QmlModelState : public QmlModelNodeFacade
 {
-    friend class StatesEditorView;
+    friend StatesEditorView;
 
 public:
     QmlModelState();
@@ -49,6 +50,7 @@ public:
     QList<QmlModelStateOperation> stateOperations(const ModelNode &node) const;
     QList<QmlPropertyChanges> propertyChanges() const;
     QList<QmlModelStateOperation> stateOperations() const;
+    QList<QmlModelStateOperation> allInvalidStateOperations() const;
 
     bool hasPropertyChanges(const ModelNode &node) const;
 
