@@ -36,6 +36,7 @@
 #include <QMultiHash>
 #include <QPointer>
 #include <QFileInfo>
+#include <QDir>
 
 namespace QmlDesigner {
 
@@ -45,9 +46,10 @@ class QMLDESIGNERCORE_EXPORT SubComponentManager : public QObject
 {
     Q_OBJECT
 public:
-    explicit SubComponentManager(Model *model, QObject *parent = 0);
+    explicit SubComponentManager(Model *model, QObject *parent = nullptr);
 
     void update(const QUrl &fileUrl, const QList<Import> &imports);
+    void addAndParseImport(const Import &import);
 
     QStringList qmlFiles() const;
     QStringList directories() const;
@@ -57,14 +59,18 @@ private: // functions
     void parseFile(const QString &canonicalFilePath,  bool addToLibrary, const QString&);
     void parseFile(const QString &canonicalFilePath);
 
-    void addImport(int pos, const Import &import);
-    void removeImport(int pos);
+    bool addImport(const Import &import, int index = -1);
+    void removeImport(int index);
     void parseDirectories();
-    QList<QFileInfo> watchedFiles(const QString &canonicalDirPath);
+    QFileInfoList watchedFiles(const QString &canonicalDirPath);
     void unregisterQmlFile(const QFileInfo &fileInfo, const QString &qualifier);
     void registerQmlFile(const QFileInfo &fileInfo, const QString &qualifier, bool addToLibrary);
     Model *model() const;
     QStringList importPaths() const;
+    void parseQuick3DAssetsDir(const QString &quick3DAssetsPath);
+    void parseQuick3DAssetsItem(const QString &importUrl, const QString &quick3DAssetsPath = {});
+    QStringList quick3DAssetPaths() const;
+    TypeName resolveDirQualifier(const QString &dirPath) const;
 
 private: // variables
     QFileSystemWatcher m_watcher;
@@ -72,6 +78,7 @@ private: // variables
     // key: canonical directory path
     QMultiHash<QString,QString> m_dirToQualifier;
     QUrl m_filePath;
+    QDir m_filePathDir;
     QPointer<Model> m_model;
 };
 
