@@ -27,16 +27,14 @@
 
 #include "nodeinstanceserverinterface.h"
 
-#include <QPointer>
-#include <QProcess>
 #include <QElapsedTimer>
 #include <QFile>
+#include <QPointer>
 #include <QTimer>
 
 QT_BEGIN_NAMESPACE
 class QLocalServer;
 class QLocalSocket;
-class QProcess;
 QT_END_NAMESPACE
 
 namespace ProjectExplorer {
@@ -48,13 +46,15 @@ namespace QmlDesigner {
 class NodeInstanceClientInterface;
 class NodeInstanceView;
 class NodeInstanceClientProxy;
+class BaseConnectionManager;
 class ConnectionManagerInterface;
 
 class NodeInstanceServerProxy : public NodeInstanceServerInterface
 {
-    friend class BaseConnectionManager;
     Q_OBJECT
 
+    friend BaseConnectionManager;
+
 public:
     explicit NodeInstanceServerProxy(NodeInstanceView *nodeInstanceView,
                                      ProjectExplorer::Target *target,
