@@ -26,7 +26,6 @@
 #include "puppetbuildprogressdialog.h"
 #include "ui_puppetbuildprogressdialog.h"
 
-#include <QProcess>
 #include <QtDebug>
 #include <QApplication>
 #include <coreplugin/icore.h>
@@ -43,7 +42,7 @@ PuppetBuildProgressDialog::PuppetBuildProgressDialog() :
     setWindowModality(Qt::ApplicationModal);
     ui->setupUi(this);
     ui->buildProgressBar->setMaximum(85);
-    connect(ui->useFallbackPuppetPushButton, SIGNAL(clicked()), this, SLOT(setUseFallbackPuppet()));
+    connect(ui->useFallbackPuppetPushButton, &QAbstractButton::clicked, this, &PuppetBuildProgressDialog::setUseFallbackPuppet);
 }
 
 PuppetBuildProgressDialog::~PuppetBuildProgressDialog()
@@ -71,7 +70,7 @@ void PuppetBuildProgressDialog::setErrorMessage(const QString &message)
 {
     ui->label->setText(QString::fromLatin1("<font color='red'>%1</font>").arg(message));
     ui->useFallbackPuppetPushButton->setText(tr("OK"));
-    connect(ui->useFallbackPuppetPushButton, SIGNAL(clicked()), this, SLOT(accept()));
+    connect(ui->useFallbackPuppetPushButton, &QAbstractButton::clicked, this, &QDialog::accept);
 }
 
 void PuppetBuildProgressDialog::setUseFallbackPuppet()
