@@ -47,6 +47,7 @@
 
 #include <qmlprojectmanager/qmlmultilanguageaspect.h>
 
+#include <qmlprojectmanager/qmlproject.h>
 #include <qtsupport/baseqtversion.h>
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtsupportconstants.h>
@@ -109,7 +110,7 @@ QDateTime PuppetCreator::qtLastModified() const
 {
     QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitAspect::qtVersion(m_target->kit());
     if (currentQtVersion)
-        return currentQtVersion->libraryPath().toFileInfo().lastModified();
+        return currentQtVersion->libraryPath().lastModified();
 
     return QDateTime();
 }
@@ -167,6 +168,20 @@ QString PuppetCreator::getStyleConfigFileName() const
     return QString();
 }
 
+bool PuppetCreator::usesVirtualKeyboard() const
+{
+#ifndef QMLDESIGNER_TEST
+    if (m_target) {
+        auto *qmlbuild = qobject_cast<QmlProjectManager::QmlBuildSystem *>(m_target->buildSystem());
+
+        const Utils::EnvironmentItem virtualKeyboard("QT_IM_MODULE", "qtvirtualkeyboard");
+        return qmlbuild && qmlbuild->environment().indexOf(virtualKeyboard);
+    }
+
+#endif
+    return false;
+}
+
 PuppetCreator::PuppetCreator(ProjectExplorer::Target *target, const Model *model)
 
     : m_target(target)
@@ -306,7 +321,7 @@ bool PuppetCreator::build(const QString &qmlPuppetProjectFilePath) const
             qmakeArguments.append("CONFIG+=release");
 #endif
             qmakeArguments.append(qmlPuppetProjectFilePath);
-            buildSucceeded = startBuildProcess(buildDirectory.path(), qmakeCommand(), qmakeArguments, &progressDialog);
+            buildSucceeded = startBuildProcess(buildDirectory.path().path(), qmakeCommand(), qmakeArguments, &progressDialog);
             if (buildSucceeded) {
                 progressDialog.show();
                 QString buildingCommand = buildCommand();
@@ -315,7 +330,7 @@ bool PuppetCreator::build(const QString &qmlPuppetProjectFilePath) const
                     buildArguments.append("-j");
                     buildArguments.append(idealProcessCount());
                 }
-                buildSucceeded = startBuildProcess(buildDirectory.path(), buildingCommand, buildArguments, &progressDialog);
+                buildSucceeded = startBuildProcess(buildDirectory.path().path(), buildingCommand, buildArguments, &progressDialog);
             }
 
             if (!buildSucceeded) {
@@ -351,6 +366,19 @@ static void warnAboutInvalidKit()
                                                                        ).arg(Core::Constants::IDE_DISPLAY_NAME));
 }
 
+static Utils::FilePath pathForBinPuppet(ProjectExplorer::Target *target)
+{
+    if (!target || !target->kit())
+        return {};
+
+    QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitAspect::qtVersion(target->kit());
+
+    if (currentQtVersion)
+        return currentQtVersion->binPath().pathAppended("qml2puppet").withExecutableSuffix();
+
+    return {};
+}
+
 void PuppetCreator::createQml2PuppetExecutableIfMissing()
 {
     m_availablePuppetType = FallbackPuppet;
@@ -372,12 +400,15 @@ void PuppetCreator::createQml2PuppetExecutableIfMissing()
                 m_qml2PuppetForKitPuppetHash.insert(m_target->id(), m_availablePuppetType);
             }
         }
+    } else if (m_target && m_target->kit() && m_target->kit()->isValid()) {
+        if (pathForBinPuppet(m_target).isExecutableFile())
+            m_availablePuppetType = BinPathPuppet;
     }
 }
 
 QString PuppetCreator::defaultPuppetToplevelBuildDirectory()
 {
-    return Core::ICore::userResourcePath() + "/qmlpuppet/";
+    return Core::ICore::userResourcePath("qmlpuppet/").toString();
 }
 
 QString PuppetCreator::qmlPuppetToplevelBuildDirectory() const
@@ -400,6 +431,9 @@ QString PuppetCreator::qmlPuppetDirectory(PuppetType puppetType) const
                 + '/' + QString::fromLatin1(qtHash());
 
 #ifndef QMLDESIGNER_TEST
+    if (puppetType == BinPathPuppet)
+        return pathForBinPuppet(m_target).toFileInfo().absoluteDir().canonicalPath();
+
     return qmlPuppetFallbackDirectory(m_designerSettings);
 #else
     return QString();
@@ -409,9 +443,9 @@ QString PuppetCreator::qmlPuppetDirectory(PuppetType puppetType) const
 QString PuppetCreator::defaultPuppetFallbackDirectory()
 {
     if (Utils::HostOsInfo::isMacHost())
-        return Core::ICore::libexecPath() + "/qmldesigner";
+        return Core::ICore::libexecPath("qmldesigner").toString();
     else
-        return Core::ICore::libexecPath();
+        return Core::ICore::libexecPath().toString();
 }
 
 QString PuppetCreator::qmlPuppetFallbackDirectory(const DesignerSettings &settings)
@@ -437,7 +471,7 @@ static void filterOutQtBaseImportPath(QStringList *stringList)
 {
     Utils::erase(*stringList, [](const QString &string) {
         QDir dir(string);
-        return dir.dirName() == "qml" && !dir.entryInfoList(QStringList("QtQuick.2"), QDir::Dirs).isEmpty();
+        return dir.dirName() == "qml" && !dir.entryInfoList(QStringList("QtTest"), QDir::Dirs).isEmpty();
     });
 }
 
@@ -446,18 +480,20 @@ QProcessEnvironment PuppetCreator::processEnvironment() const
     static const QString pathSep = Utils::HostOsInfo::pathListSeparator();
     Utils::Environment environment = Utils::Environment::systemEnvironment();
     if (QTC_GUARD(m_target)) {
-        if (!useOnlyFallbackPuppet())
-            m_target->kit()->addToEnvironment(environment);
-        const QtSupport::BaseQtVersion *qt = QtSupport::QtKitAspect::qtVersion(m_target->kit());
-        if (QTC_GUARD(qt)) { // Kits without a Qt version should not have a puppet!
-            // Update PATH to include QT_HOST_BINS
-            const Utils::FilePath qtBinPath = qt->hostBinPath();
-            environment.prependOrSetPath(qtBinPath.toString());
+        if (!useOnlyFallbackPuppet() || m_availablePuppetType == BinPathPuppet) {
+            m_target->kit()->addToBuildEnvironment(environment);
+            const QtSupport::BaseQtVersion *qt = QtSupport::QtKitAspect::qtVersion(m_target->kit());
+            if (QTC_GUARD(qt)) { // Kits without a Qt version should not have a puppet!
+                // Update PATH to include QT_HOST_BINS
+                const Utils::FilePath qtBinPath = qt->hostBinPath();
+                environment.prependOrSetPath(qtBinPath.toString());
+            }
         }
     }
     environment.set("QML_BAD_GUI_RENDER_LOOP", "true");
     environment.set("QML_PUPPET_MODE", "true");
     environment.set("QML_DISABLE_DISK_CACHE", "true");
+    environment.set("QMLPUPPET_RENDER_EFFECTS", "true");
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     if (!environment.hasKey("QT_SCREEN_SCALE_FACTORS") && !environment.hasKey("QT_SCALE_FACTOR")
             && QApplication::testAttribute(Qt::AA_EnableHighDpiScaling))
@@ -472,7 +508,7 @@ QProcessEnvironment PuppetCreator::processEnvironment() const
 #else
     const QString controlsStyle;
 #endif
-    if (!controlsStyle.isEmpty() && controlsStyle != "Default") {
+    if (!controlsStyle.isEmpty()) {
         environment.set("QT_QUICK_CONTROLS_STYLE", controlsStyle);
         environment.set("QT_LABS_CONTROLS_STYLE", controlsStyle);
     }
@@ -483,25 +519,39 @@ QProcessEnvironment PuppetCreator::processEnvironment() const
 
     const QString styleConfigFileName = getStyleConfigFileName();
 
-    /* QT_QUICK_CONTROLS_CONF is not supported for Qt Version < 5.8.1,
-     * but we can manually at least set the correct style. */
-    if (!styleConfigFileName.isEmpty()) {
-        QSettings infiFile(styleConfigFileName, QSettings::IniFormat);
-        environment.set("QT_QUICK_CONTROLS_STYLE", infiFile.value("Controls/Style", "Default").toString());
-    }
-
     if (!m_qrcMapping.isEmpty()) {
         environment.set("QMLDESIGNER_RC_PATHS", m_qrcMapping);
     }
 
+    if (usesVirtualKeyboard()) {
+        environment.set("QT_IM_MODULE", "qtvirtualkeyboard");
+        environment.set("QT_VIRTUALKEYBOARD_DESKTOP_DISABLE", "1");
+    }
+
 #ifndef QMLDESIGNER_TEST
     // set env var if QtQuick3D import exists
     QmlDesigner::Import import = QmlDesigner::Import::createLibraryImport("QtQuick3D", "1.0");
     if (m_model->hasImport(import, true, true))
         environment.set("QMLDESIGNER_QUICK3D_MODE", "true");
-    import = QmlDesigner::Import::createLibraryImport("QtCharts", "2.0");
+
+    import = QmlDesigner::Import::createLibraryImport("QtQuick3D.Particles3D", "1.0");
     if (m_model->hasImport(import, true, true))
+        environment.set("QMLDESIGNER_QUICK3D_PARTICLES3D_MODE", "true");
+
+    import = QmlDesigner::Import::createLibraryImport("QtCharts", "2.0");
+    if (m_model->hasImport(import, true, true)) {
         environment.set("QMLDESIGNER_FORCE_QAPPLICATION", "true");
+    } else if (m_target) {
+        auto bs = qobject_cast<QmlProjectManager::QmlBuildSystem *>(m_target->buildSystem());
+        if (bs && bs->widgetApp())
+            environment.set("QMLDESIGNER_FORCE_QAPPLICATION", "true");
+    }
+
+    bool particlemode = QmlDesigner::DesignerSettings::getValue("particleMode").toBool();
+    if (!particlemode)
+        environment.set("QT_QUICK3D_DISABLE_PARTICLE_SYSTEMS", "1");
+    else
+        environment.set("QT_QUICK3D_EDITOR_PARTICLE_SYSTEMS", "1");
 #endif
 
     QStringList importPaths = m_model->importPaths();
@@ -548,8 +598,7 @@ QProcessEnvironment PuppetCreator::processEnvironment() const
 
 QString PuppetCreator::buildCommand() const
 {
-    Utils::Environment environment = Utils::Environment::systemEnvironment();
-    m_target->kit()->addToEnvironment(environment);
+    const Utils::Environment environment = m_target->kit()->buildEnvironment();
 
     if (ToolChain *toolChain = ToolChainKitAspect::cxxToolChain(m_target->kit()))
         return toolChain->makeCommand(environment).toString();
@@ -561,7 +610,7 @@ QString PuppetCreator::qmakeCommand() const
 {
     QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitAspect::qtVersion(m_target->kit());
     if (currentQtVersion)
-        return currentQtVersion->qmakeCommand().toString();
+        return currentQtVersion->qmakeFilePath().toString();
 
     return QString();
 }
@@ -619,7 +668,7 @@ bool PuppetCreator::startBuildProcess(const QString &buildDirectoryPath,
 
 QString PuppetCreator::puppetSourceDirectoryPath()
 {
-    return Core::ICore::resourcePath() + "/qml/qmlpuppet";
+    return Core::ICore::resourcePath("qml/qmlpuppet").toString();
 }
 
 QString PuppetCreator::qml2PuppetProjectFile()
