@@ -36,6 +36,11 @@
 
 namespace QmlDesigner {
 
+Internal::NodeListPropertyIterator::value_type Internal::NodeListPropertyIterator::operator*() const
+{
+    return {m_nodeListProperty->at(m_currentIndex), m_model, m_view};
+}
+
 NodeListProperty::NodeListProperty() = default;
 
 NodeListProperty::NodeListProperty(const NodeListProperty &property, AbstractView *view)
@@ -54,6 +59,20 @@ NodeListProperty::NodeListProperty(const Internal::InternalNodeListProperty::Poi
 {
 }
 
+Internal::InternalNodeListPropertyPointer &NodeListProperty::internalNodeListProperty() const
+{
+    if (m_internalNodeListProperty)
+        return m_internalNodeListProperty;
+
+    if (internalNode()->hasProperty(name())) {
+        Internal::InternalProperty::Pointer internalProperty = internalNode()->property(name());
+        if (internalProperty->isNodeListProperty())
+            m_internalNodeListProperty = internalProperty->toNodeListProperty();
+    }
+
+    return m_internalNodeListProperty;
+}
+
 static QList<ModelNode> internalNodesToModelNodes(const QList<Internal::InternalNode::Pointer> &inputList, Model* model, AbstractView *view)
 {
     QList<ModelNode> modelNodeList;
@@ -63,21 +82,20 @@ static QList<ModelNode> internalNodesToModelNodes(const QList<Internal::Internal
     return modelNodeList;
 }
 
-const QList<ModelNode> NodeListProperty::toModelNodeList() const
+QList<ModelNode> NodeListProperty::toModelNodeList() const
 {
     if (!isValid())
         throw InvalidPropertyException(__LINE__, __FUNCTION__, __FILE__, "<invalid node list property>");
 
-    if (internalNode()->hasProperty(name())) {
-        Internal::InternalProperty::Pointer internalProperty = internalNode()->property(name());
-        if (internalProperty->isNodeListProperty())
-            return internalNodesToModelNodes(internalProperty->toNodeListProperty()->nodeList(), model(), view());
-    }
+    if (internalNodeListProperty())
+        return internalNodesToModelNodes(m_internalNodeListProperty->toNodeListProperty()->nodeList(),
+                                         model(),
+                                         view());
 
     return QList<ModelNode>();
 }
 
-const QList<QmlObjectNode> NodeListProperty::toQmlObjectNodeList() const
+QList<QmlObjectNode> NodeListProperty::toQmlObjectNodeList() const
 {
     if (model()->nodeInstanceView())
         return QList<QmlObjectNode>();
@@ -129,14 +147,51 @@ ModelNode NodeListProperty::at(int index) const
     if (!isValid())
         throw InvalidPropertyException(__LINE__, __FUNCTION__, __FILE__, "<invalid node list property>");
 
-    Internal::InternalNodeListProperty::Pointer internalProperty = internalNode()->nodeListProperty(name());
-    if (internalProperty)
-        return ModelNode(internalProperty->at(index), model(), view());
-
+    if (internalNodeListProperty())
+        return ModelNode(m_internalNodeListProperty->at(index), model(), view());
 
     return ModelNode();
 }
 
+void NodeListProperty::iterSwap(NodeListProperty::iterator &first, NodeListProperty::iterator &second)
+{
+    if (!internalNodeListProperty())
+        return;
+
+    std::swap(m_internalNodeListProperty->at(first.m_currentIndex),
+              m_internalNodeListProperty->at(second.m_currentIndex));
+}
+
+NodeListProperty::iterator NodeListProperty::rotate(NodeListProperty::iterator first,
+                                                    NodeListProperty::iterator newFirst,
+                                                    NodeListProperty::iterator last)
+{
+    if (!internalNodeListProperty())
+        return {};
+
+    auto begin = m_internalNodeListProperty->begin();
+
+    auto iter = std::rotate(std::next(begin, first.m_currentIndex),
+                            std::next(begin, newFirst.m_currentIndex),
+                            std::next(begin, last.m_currentIndex));
+
+    privateModel()->notifyNodeOrderChanged(m_internalNodeListProperty);
+
+    return {iter - begin, internalNodeListProperty().data(), model(), view()};
+}
+
+void NodeListProperty::reverse(NodeListProperty::iterator first, NodeListProperty::iterator last)
+{
+    if (!internalNodeListProperty())
+        return;
+
+    auto begin = m_internalNodeListProperty->begin();
+
+    std::reverse(std::next(begin, first.m_currentIndex), std::next(begin, last.m_currentIndex));
+
+    privateModel()->notifyNodeOrderChanged(m_internalNodeListProperty);
+}
+
 void NodeListProperty::reverseModelNodes(const QList<ModelNode> &nodes)
 {
     ModelNode firstNode = nodes.first();
@@ -157,4 +212,27 @@ void NodeListProperty::reverseModelNodes(const QList<ModelNode> &nodes)
         parentProperty.swap(selectedNodeIndices[i], selectedNodeIndices[selectedNodeIndices.size() - 1 - i]);
 }
 
+Internal::NodeListPropertyIterator NodeListProperty::begin()
+{
+    return const_cast<const NodeListProperty *>(this)->begin();
+}
+
+Internal::NodeListPropertyIterator NodeListProperty::end()
+{
+    return const_cast<const NodeListProperty *>(this)->end();
+}
+
+Internal::NodeListPropertyIterator NodeListProperty::begin() const
+{
+    return {0, internalNodeListProperty().data(), model(), view()};
+}
+
+Internal::NodeListPropertyIterator NodeListProperty::end() const
+{
+    auto nodeListProperty = internalNodeListProperty();
+    auto size = nodeListProperty ? nodeListProperty->size() : 0;
+
+    return {size, nodeListProperty.data(), model(), view()};
+}
+
 } // namespace QmlDesigner
