@@ -27,6 +27,8 @@
 #include "qmlitemnode.h"
 #include "qmlstate.h"
 #include "qmltimelinekeyframegroup.h"
+#include "qmlvisualnode.h"
+#include "qml3dnode.h"
 #include "variantproperty.h"
 #include "nodeproperty.h"
 #include <invalidmodelnodeexception.h>
@@ -43,7 +45,9 @@
 #include <qmldesignerplugin.h>
 #endif
 
-#include <QRegExp>
+#include <utils/qtcassert.h>
+
+#include <QRegularExpression>
 
 namespace QmlDesigner {
 
@@ -245,8 +249,9 @@ bool QmlObjectNode::isTranslatableText(const PropertyName &name) const
     if (modelNode().metaInfo().isValid() && modelNode().metaInfo().hasProperty(name))
         if (modelNode().metaInfo().propertyTypeName(name) == "QString" || modelNode().metaInfo().propertyTypeName(name) == "string") {
             if (modelNode().hasBindingProperty(name)) {
-                static QRegExp regularExpressionPatter(QLatin1String("qsTr(|Id|anslate)\\(\".*\"\\)"));
-                return regularExpressionPatter.exactMatch(modelNode().bindingProperty(name).expression());
+                static QRegularExpression regularExpressionPattern(
+                            QLatin1String("^qsTr(|Id|anslate)\\(\".*\"\\)$"));
+                return modelNode().bindingProperty(name).expression().contains(regularExpressionPattern);
             }
 
             return false;
@@ -258,9 +263,12 @@ bool QmlObjectNode::isTranslatableText(const PropertyName &name) const
 QString QmlObjectNode::stripedTranslatableText(const PropertyName &name) const
 {
     if (modelNode().hasBindingProperty(name)) {
-        static QRegExp regularExpressionPatter(QLatin1String("qsTr(|Id|anslate)\\(\"(.*)\"\\)"));
-        if (regularExpressionPatter.exactMatch(modelNode().bindingProperty(name).expression()))
-            return regularExpressionPatter.cap(2);
+        static QRegularExpression regularExpressionPattern(
+                    QLatin1String("^qsTr(|Id|anslate)\\(\"(.*)\"\\)$"));
+        const QRegularExpressionMatch match = regularExpressionPattern.match(
+                    modelNode().bindingProperty(name).expression());
+        if (match.hasMatch())
+            return match.captured(2);
         return instanceValue(name).toString();
     }
     return instanceValue(name).toString();
@@ -320,16 +328,58 @@ QmlPropertyChanges QmlObjectNode::propertyChangeForCurrentState() const
 static void removeStateOperationsForChildren(const QmlObjectNode &node)
 {
     if (node.isValid()) {
-        foreach (QmlModelStateOperation stateOperation, node.allAffectingStatesOperations()) {
+        for (QmlModelStateOperation stateOperation : node.allAffectingStatesOperations()) {
             stateOperation.modelNode().destroy(); //remove of belonging StatesOperations
         }
 
-        foreach (const QmlObjectNode &childNode, node.modelNode().directSubModelNodes()) {
+        for (const QmlObjectNode childNode : node.modelNode().directSubModelNodes()) {
             removeStateOperationsForChildren(childNode);
         }
     }
 }
 
+static void removeAnimationsFromAnimation(const ModelNode &animation)
+{
+    QTC_ASSERT(animation.isValid(), return);
+
+    const QList<ModelNode> propertyAnimations = animation.subModelNodesOfType(
+        "QtQuick.PropertyAnimation");
+
+    for (const ModelNode &child : propertyAnimations) {
+        if (!child.hasBindingProperty("target")) {
+            ModelNode nonConst = animation;
+            nonConst.destroy();
+            return;
+        }
+    }
+}
+
+static void removeAnimationsFromTransition(const ModelNode &transition, const QmlObjectNode &node)
+{
+    QTC_ASSERT(node.isValid(), return);
+    QTC_ASSERT(transition.isValid(), return);
+
+    const auto children = transition.directSubModelNodes();
+    for (const ModelNode &parallel : children)
+        removeAnimationsFromAnimation(parallel);
+}
+
+static void removeDanglingAnimationsFromTransitions(const QmlObjectNode &node)
+{
+    QTC_ASSERT(node.isValid(), return);
+
+    auto root = node.view()->rootModelNode();
+
+    if (root.isValid() && root.hasProperty("transitions")) {
+        NodeAbstractProperty transitions = root.nodeAbstractProperty("transitions");
+        if (transitions.isValid()) {
+            const auto transitionNodes = transitions.directSubNodes();
+            for (const auto &transition : transitionNodes)
+                removeAnimationsFromTransition(transition, node);
+        }
+    }
+}
+
 static void removeAliasExports(const QmlObjectNode &node)
 {
 
@@ -350,6 +400,24 @@ static void removeAliasExports(const QmlObjectNode &node)
 
 }
 
+static void removeLayerEnabled(const ModelNode &node)
+{
+    QTC_ASSERT(node.isValid(), return );
+    if (node.parentProperty().isValid() && node.parentProperty().name() == "layer.effect") {
+        ModelNode parent = node.parentProperty().parentModelNode();
+        if (parent.isValid() && parent.hasProperty("layer.enabled"))
+            parent.removeProperty("layer.enabled");
+    }
+}
+
+static void deleteAllReferencesToNodeAndChildren(const ModelNode &node)
+{
+    BindingProperty::deleteAllReferencesTo(node);
+    const auto subNodes = node.allSubModelNodes();
+    for (const ModelNode &child : subNodes)
+        BindingProperty::deleteAllReferencesTo(child);
+}
+
 /*!
     Deletes this object's node and its dependencies from the model.
     Everything that belongs to this Object, the ModelNode, and ChangeOperations
@@ -360,21 +428,44 @@ void QmlObjectNode::destroy()
     if (!isValid())
         throw new InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
 
+    removeLayerEnabled(modelNode());
     removeAliasExports(modelNode());
 
     for (QmlModelStateOperation stateOperation : allAffectingStatesOperations()) {
         stateOperation.modelNode().destroy(); //remove of belonging StatesOperations
     }
 
-    for (const ModelNode &timelineNode : view()->allModelNodes()) {
-        if (QmlTimeline::isValidQmlTimeline(timelineNode)) {
-            QmlTimeline timeline(timelineNode);
-            timeline.destroyKeyframesForTarget(modelNode());
-        }
+    QVector<ModelNode> timelineNodes;
+    const auto allNodes = view()->allModelNodes();
+    for (const auto &timelineNode : allNodes) {
+        if (QmlTimeline::isValidQmlTimeline(timelineNode))
+            timelineNodes.append(timelineNode);
+    }
+
+    const auto subNodes = modelNode().allSubModelNodesAndThisNode();
+    for (auto &timelineNode : qAsConst(timelineNodes)) {
+        QmlTimeline timeline(timelineNode);
+        for (const auto &subNode : subNodes)
+            timeline.destroyKeyframesForTarget(subNode);
+    }
+
+    bool wasFlowEditorTarget = false;
+    if (QmlFlowTargetNode::isFlowEditorTarget(modelNode())) {
+        QmlFlowTargetNode(modelNode()).destroyTargets();
+        wasFlowEditorTarget = true;
     }
 
     removeStateOperationsForChildren(modelNode());
+    deleteAllReferencesToNodeAndChildren(modelNode());
+
+    removeDanglingAnimationsFromTransitions(modelNode());
+
+    QmlFlowViewNode root(view()->rootModelNode());
+
     modelNode().destroy();
+
+    if (wasFlowEditorTarget && root.isValid())
+        root.removeDanglingTransitions();
 }
 
 void QmlObjectNode::ensureAliasExport()
@@ -476,6 +567,16 @@ QList<QmlModelState> QmlObjectNode::allDefinedStates() const
     return returnList;
 }
 
+QList<QmlModelStateOperation> QmlObjectNode::allInvalidStateOperations() const
+{
+    QList<QmlModelStateOperation> result;
+
+    const auto allStates =  allDefinedStates();
+    for (const auto &state : allStates)
+        result.append(state.allInvalidStateOperations());
+    return result;
+}
+
 
 /*!
     Removes a variant property of the object specified by \a name from the
@@ -652,6 +753,11 @@ QmlItemNode QmlObjectNode::instanceParentItem() const
     return QmlItemNode();
 }
 
+QmlItemNode QmlObjectNode::modelParentItem() const
+{
+    return modelNode().parentProperty().parentModelNode();
+}
+
 void QmlObjectNode::setId(const QString &id)
 {
     modelNode().setIdWithRefactoring(id);
@@ -688,9 +794,9 @@ QmlItemNode QmlObjectNode::toQmlItemNode() const
     return QmlItemNode(modelNode());
 }
 
-uint qHash(const QmlObjectNode &node)
+QmlVisualNode QmlObjectNode::toQmlVisualNode() const
 {
-    return qHash(node.modelNode());
+     return QmlVisualNode(modelNode());
 }
 
 QString QmlObjectNode::simplifiedTypeName() const
@@ -698,4 +804,25 @@ QString QmlObjectNode::simplifiedTypeName() const
     return modelNode().simplifiedTypeName();
 }
 
+QStringList QmlObjectNode::allStateNames() const
+{
+    return nodeInstance().allStateNames();
+}
+
+QmlObjectNode *QmlObjectNode::getQmlObjectNodeOfCorrectType(const ModelNode &modelNode)
+{
+    // Create QmlObjectNode of correct type for the modelNode
+    // Note: Currently we are only interested in differentiating 3D nodes, so no check for
+    // visual nodes is done for efficiency reasons
+    if (modelNode.isValid() && modelNode.isSubclassOf("QtQuick3D.Node"))
+        return new Qml3DNode(modelNode);
+    return new QmlObjectNode(modelNode);
+}
+
+bool QmlObjectNode::isBlocked(const PropertyName &propName) const
+{
+    Q_UNUSED(propName)
+    return false;
+}
+
 } //QmlDesigner
