@@ -32,7 +32,9 @@
 #include "bindingproperty.h"
 #include "qmlchangeset.h"
 #include "qmlitemnode.h"
+#include "annotation.h"
 
+#include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 
 namespace QmlDesigner {
@@ -135,6 +137,12 @@ QList<QmlModelStateOperation> QmlModelState::stateOperations() const
     return returnList;
 }
 
+QList<QmlModelStateOperation> QmlModelState::allInvalidStateOperations() const
+{
+    return Utils::filtered(stateOperations(), [](const QmlModelStateOperation &operation) {
+        return !operation.target().isValid();
+    });
+}
 
 /*!
     Adds a change set for \a node to this state, but only if it does not
@@ -148,10 +156,8 @@ void QmlModelState::addChangeSetIfNotExists(const ModelNode &node)
 
     if (!hasPropertyChanges(node)) {
         ModelNode newChangeSet;
-        if (view()->majorQtQuickVersion() > 1)
-            newChangeSet = modelNode().view()->createModelNode("QtQuick.PropertyChanges", 2, 0);
-        else
-            newChangeSet = modelNode().view()->createModelNode("QtQuick.PropertyChanges", 1, 0);
+
+        newChangeSet = modelNode().view()->createModelNode("QtQuick.PropertyChanges", view()->majorQtQuickVersion(), 0);
 
         modelNode().nodeListProperty("changes").reparentHere(newChangeSet);
 
@@ -283,10 +289,62 @@ ModelNode QmlModelState::createQmlState(AbstractView *view, const PropertyListTy
 {
     QTC_CHECK(view->majorQtQuickVersion() < 3);
 
-    if (view->majorQtQuickVersion() > 1)
-        return view->createModelNode("QtQuick.State", 2, 0, propertyList);
-    else
-        return view->createModelNode("QtQuick.State", 1, 0, propertyList);
+    return view->createModelNode("QtQuick.State", 2, 0, propertyList);
+}
+
+void QmlModelState::setAsDefault()
+{
+    if ((!isBaseState()) && (modelNode().isValid())) {
+        view()->rootModelNode().variantProperty("state").setValue(name());
+    }
+}
+
+bool QmlModelState::isDefault() const
+{
+    if ((!isBaseState()) && (modelNode().isValid())) {
+        if (view()->rootModelNode().hasProperty("state")) {
+            return (view()->rootModelNode().variantProperty("state").value() == name());
+        }
+    }
+
+    return false;
+}
+
+void QmlModelState::setAnnotation(const Annotation &annotation, const QString &id)
+{
+    if (modelNode().isValid()) {
+        modelNode().setCustomId(id);
+        modelNode().setAnnotation(annotation);
+    }
+}
+
+Annotation QmlModelState::annotation() const
+{
+    if (modelNode().isValid())
+        return modelNode().annotation();
+    return {};
+}
+
+QString QmlModelState::annotationName() const
+{
+    if (modelNode().isValid())
+        return modelNode().customId();
+    return {};
+}
+
+bool QmlModelState::hasAnnotation() const
+{
+    if (modelNode().isValid())
+        return modelNode().hasAnnotation() || modelNode().hasCustomId();
+    return false;
+}
+
+void QmlModelState::removeAnnotation()
+{
+    if (modelNode().isValid()) {
+        modelNode().removeCustomId();
+        modelNode().removeAnnotation();
+    }
 }
 
 QmlModelState QmlModelState::createBaseState(const AbstractView *view)
