@@ -74,7 +74,9 @@ void QmlTimelineKeyframeGroup::setTarget(const ModelNode &target)
 {
     QTC_ASSERT(isValid(), return );
 
-    modelNode().bindingProperty("target").setExpression(target.id());
+    ModelNode nonConstTarget = target;
+
+    modelNode().bindingProperty("target").setExpression(nonConstTarget.validId());
 }
 
 PropertyName QmlTimelineKeyframeGroup::propertyName() const
@@ -153,6 +155,13 @@ QmlTimeline QmlTimelineKeyframeGroup::timeline() const
     return {};
 }
 
+bool QmlTimelineKeyframeGroup::isDangling() const
+{
+    QTC_ASSERT(isValid(), return false);
+
+    return !target().isValid() || keyframes().isEmpty();
+}
+
 void QmlTimelineKeyframeGroup::setValue(const QVariant &value, qreal currentFrame)
 {
     QTC_ASSERT(isValid(), return );
@@ -255,12 +264,12 @@ qreal QmlTimelineKeyframeGroup::maxActualKeyframe() const
     return max;
 }
 
-const QList<ModelNode> QmlTimelineKeyframeGroup::keyframes() const
+QList<ModelNode> QmlTimelineKeyframeGroup::keyframes() const
 {
     return modelNode().defaultNodeListProperty().toModelNodeList();
 }
 
-const QList<ModelNode> QmlTimelineKeyframeGroup::keyframePositions() const
+QList<ModelNode> QmlTimelineKeyframeGroup::keyframePositions() const
 {
     QList<ModelNode> returnValues;
     for (const ModelNode &childNode : modelNode().defaultNodeListProperty().toModelNodeList()) {
@@ -294,6 +303,22 @@ QmlTimelineKeyframeGroup QmlTimelineKeyframeGroup::keyframeGroupForKeyframe(cons
     return QmlTimelineKeyframeGroup();
 }
 
+QList<QmlTimelineKeyframeGroup> QmlTimelineKeyframeGroup::allInvalidTimelineKeyframeGroups(AbstractView *view)
+{
+    QList<QmlTimelineKeyframeGroup> ret;
+
+    QTC_ASSERT(view, return ret);
+    QTC_ASSERT(view->model(), return ret);
+    QTC_ASSERT(view->rootModelNode().isValid(), return ret);
+
+    const auto groups = view->rootModelNode().subModelNodesOfType("QtQuick.Timeline.KeyframeGroup");
+    for (const QmlTimelineKeyframeGroup group : groups) {
+        if (group.isDangling())
+            ret.append(group);
+    }
+    return ret;
+}
+
 void QmlTimelineKeyframeGroup::moveAllKeyframes(qreal offset)
 {
     for (const ModelNode &childNode : modelNode().defaultNodeListProperty().toModelNodeList()) {
