@@ -150,7 +150,7 @@ DesignModeWidget::DesignModeWidget()
 
 DesignModeWidget::~DesignModeWidget()
 {
-    for (QPointer<QWidget> widget : m_viewWidgets) {
+    for (QPointer<QWidget> widget : qAsConst(m_viewWidgets)) {
         if (widget)
             widget.clear();
     }
@@ -215,7 +215,7 @@ void DesignModeWidget::disableWidgets()
 static void addSpacerToToolBar(QToolBar *toolBar)
 {
     QWidget* empty = new QWidget();
-    empty->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Preferred);
+    empty->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
     toolBar->addWidget(empty);
 }
 
@@ -231,45 +231,42 @@ void DesignModeWidget::setup()
 
     ADS::DockManager::setConfigFlags(ADS::DockManager::DefaultNonOpaqueConfig);
     ADS::DockManager::setConfigFlag(ADS::DockManager::FocusHighlighting, true);
+    ADS::DockManager::setConfigFlag(ADS::DockManager::DockAreaHasCloseButton, false);
+    ADS::DockManager::setConfigFlag(ADS::DockManager::DockAreaHasUndockButton, false);
+    ADS::DockManager::setConfigFlag(ADS::DockManager::DockAreaHasTabsMenuButton, false);
+    ADS::DockManager::setConfigFlag(ADS::DockManager::OpaqueSplitterResize, true);
+    ADS::DockManager::setConfigFlag(ADS::DockManager::AllTabsHaveCloseButton, true);
     m_dockManager = new ADS::DockManager(this);
     m_dockManager->setSettings(settings);
-    m_dockManager->setWorkspacePresetsPath(Core::ICore::resourcePath() + QLatin1String("/qmldesigner/workspacePresets/"));
+    m_dockManager->setWorkspacePresetsPath(
+        Core::ICore::resourcePath("qmldesigner/workspacePresets/").toString());
 
     QString sheet = QString::fromUtf8(Utils::FileReader::fetchQrc(":/qmldesigner/dockwidgets.css"));
     m_dockManager->setStyleSheet(Theme::replaceCssColors(sheet));
 
     // Setup icons
-    const QColor buttonColor(Theme::getColor(Theme::QmlDesigner_TabLight)); // TODO Use correct color roles
-    const QColor tabColor(Theme::getColor(Theme::QmlDesigner_TabDark));
+    const QColor iconColor(Theme::getColor(Theme::DStitleBarIcon));
 
     const QString closeUnicode = Theme::getIconUnicode(Theme::Icon::adsClose);
     const QString menuUnicode = Theme::getIconUnicode(Theme::Icon::adsDropDown);
     const QString undockUnicode = Theme::getIconUnicode(Theme::Icon::adsDetach);
 
     const QString fontName = "qtds_propertyIconFont.ttf";
-    const QIcon closeIcon = Utils::StyleHelper::getIconFromIconFont(fontName, closeUnicode, 28, 28, buttonColor);
-    const QIcon menuIcon = Utils::StyleHelper::getIconFromIconFont(fontName, menuUnicode, 28, 28, buttonColor);
-    const QIcon undockIcon = Utils::StyleHelper::getIconFromIconFont(fontName, undockUnicode, 28, 28, buttonColor);
+    const QSize size = QSize(28, 28);
 
-    auto closeIconNormal = Utils::StyleHelper::IconFontHelper(closeUnicode,
-                                                              tabColor,
-                                                              QSize(28, 28),
-                                                              QIcon::Normal,
-                                                              QIcon::Off);
+    auto tabCloseIconNormal = Utils::StyleHelper::IconFontHelper(
+        closeUnicode, Theme::getColor(Theme::DStabInactiveIcon), size, QIcon::Normal, QIcon::Off);
+    auto tabCloseIconActive = Utils::StyleHelper::IconFontHelper(
+        closeUnicode, Theme::getColor(Theme::DStabActiveIcon), size, QIcon::Active, QIcon::Off);
+    auto tabCloseIconFocus = Utils::StyleHelper::IconFontHelper(
+        closeUnicode, Theme::getColor(Theme::DStabFocusIcon), size, QIcon::Selected, QIcon::Off);
 
-    auto closeIconFocused = Utils::StyleHelper::IconFontHelper(closeUnicode,
-                                                               Theme::getColor(Theme::DStextColor),
-                                                               QSize(28, 28),
-                                                               QIcon::Normal,
-                                                               QIcon::On);
+    const QIcon tabsCloseIcon = Utils::StyleHelper::getIconFromIconFont(
+                fontName, {tabCloseIconNormal,
+                           tabCloseIconActive,
+                           tabCloseIconFocus});
 
-    const QIcon tabsCloseIcon = Utils::StyleHelper::getIconFromIconFont(fontName, {closeIconNormal, closeIconFocused});
-
-    m_dockManager->iconProvider().registerCustomIcon(ADS::TabCloseIcon, tabsCloseIcon);
-    m_dockManager->iconProvider().registerCustomIcon(ADS::DockAreaMenuIcon, menuIcon);
-    m_dockManager->iconProvider().registerCustomIcon(ADS::DockAreaUndockIcon, undockIcon);
-    m_dockManager->iconProvider().registerCustomIcon(ADS::DockAreaCloseIcon, closeIcon);
-    m_dockManager->iconProvider().registerCustomIcon(ADS::FloatingWidgetCloseIcon, closeIcon);
+    ADS::DockManager::iconProvider().registerCustomIcon(ADS::TabCloseIcon, tabsCloseIcon);
 
     // Setup Actions and Menus
     Core::ActionContainer *mview = Core::ActionManager::actionContainer(Core::Constants::M_VIEW);
@@ -468,8 +465,9 @@ void DesignModeWidget::setup()
             m_dockManager->openWorkspace(workspaceComboBox->currentText());
     });
 
-    const QIcon gaIcon = Utils::StyleHelper::getIconFromIconFont(fontName,
-                                                                 Theme::getIconUnicode(Theme::Icon::annotationBubble), 36, 36);
+    const QIcon gaIcon = Utils::StyleHelper::getIconFromIconFont(
+                fontName, Theme::getIconUnicode(Theme::Icon::annotationBubble),
+                36, 36, Theme::getColor(Theme::IconsBaseColor));
     toolBar->addAction(gaIcon, tr("Edit global annotation for current file."), [&](){
         ModelNode node = currentDesignDocument()->rewriterView()->rootModelNode();
 
@@ -509,7 +507,7 @@ void DesignModeWidget::aboutToShowWorkspaces()
     auto sortedWorkspaces = m_dockManager->workspaces();
     Utils::sort(sortedWorkspaces);
 
-    for (const auto &workspace : sortedWorkspaces)
+    for (const auto &workspace : qAsConst(sortedWorkspaces))
     {
         QAction *action = ag->addAction(workspace);
         action->setData(workspace);
