@@ -24,10 +24,17 @@
 ****************************************************************************/
 
 #pragma once
-
+#include <QTableWidget>
 
 namespace QmlDesigner {
 namespace GenerateResource {
+struct ResourceFile
+{
+    QString fileName;
+    bool inProject;
+};
     void generateMenuEntry();
+    QStringList getFileList(const QList<ResourceFile> &);
+    QTableWidget* createFilesTable(const QList<ResourceFile> &);
 }
 } // namespace QmlDesigner
