@@ -64,13 +64,19 @@ const char EDIT3D_ORIENTATION[]    = "QmlDesigner.Editor3D.OrientationToggle";
 const char EDIT3D_EDIT_LIGHT[]     = "QmlDesigner.Editor3D.EditLightToggle";
 const char EDIT3D_EDIT_SHOW_GRID[] = "QmlDesigner.Editor3D.ToggleGrid";
 const char EDIT3D_RESET_VIEW[]     = "QmlDesigner.Editor3D.ResetView";
+const char EDIT3D_PARTICLE_MODE[]     = "QmlDesigner.Editor3D.ParticleViewModeToggle";
+const char EDIT3D_PARTICLES_PLAY[]    = "QmlDesigner.Editor3D.ParticlesPlay";
+const char EDIT3D_PARTICLES_RESTART[] = "QmlDesigner.Editor3D.ParticlesRestart";
+
 
 const char QML_DESIGNER_SUBFOLDER[] = "/designer/";
 const char QUICK_3D_ASSETS_FOLDER[] = "/Quick3DAssets";
 const char QUICK_3D_ASSET_LIBRARY_ICON_SUFFIX[] = "_libicon";
 const char QUICK_3D_ASSET_ICON_DIR[] = "_icons";
+const char QUICK_3D_ASSET_IMPORT_DATA_NAME[] = "_importdata.json";
+const char QUICK_3D_ASSET_IMPORT_DATA_OPTIONS_KEY[] = "import_options";
+const char QUICK_3D_ASSET_IMPORT_DATA_SOURCE_KEY[] = "source_scene";
 const char DEFAULT_ASSET_IMPORT_FOLDER[] = "/asset_imports";
-const char QT_QUICK_3D_MODULE_NAME[] = "QtQuick3D";
 
 // Menus
 const char M_VIEW_WORKSPACES[] = "QmlDesigner.Menu.View.Workspaces";
@@ -86,7 +92,6 @@ const char EVENT_ANNOTATION_ADDED[] = "Annotation Added";
 const char EVENT_RESOURCE_IMPORTED[] = "Resource Imported ";
 const char EVENT_ACTION_EXECUTED[] = "Action Executed ";
 const char EVENT_IMPORT_ADDED[] = "Import Added ";
-const char EVENT_IMPORT_ADDED_FLOWTAG[] = "Import Added FlowTag ";
 const char EVENT_BINDINGEDITOR_OPENED[] = "Binding Editor Opened";
 const char EVENT_RICHTEXT_OPENED[] = "Richtext Editor Opened";
 const char EVENT_FORMEDITOR_TIME[] = "Form Editor";
