@@ -24,15 +24,16 @@
 ****************************************************************************/
 
 #include "qmldesignerplugin.h"
-#include "exception.h"
-#include "qmldesignerconstants.h"
-#include "designmodewidget.h"
-#include "settingspage.h"
 #include "designmodecontext.h"
-#include "openuiqmlfiledialog.h"
+#include "designmodewidget.h"
+#include "exception.h"
+#include "generatecmakelists.h"
 #include "generateresource.h"
 #include "nodeinstanceview.h"
-#include "gestures.h"
+#include "openuiqmlfiledialog.h"
+#include "qmldesignerconstants.h"
+#include "qmldesignerprojectmanager.h"
+#include "settingspage.h"
 
 #include <metainfo.h>
 #include <connectionview.h>
@@ -43,6 +44,7 @@
 #include <texttool/texttool.h>
 #include <timelineeditor/timelineview.h>
 #include <transitioneditor/transitioneditorview.h>
+#include <eventlist/eventlistpluginview.h>
 #include <pathtool/pathtool.h>
 
 #include <qmljseditor/qmljseditor.h>
@@ -55,20 +57,23 @@
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/designmode.h>
+#include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
 #include <coreplugin/messagebox.h>
 #include <coreplugin/modemanager.h>
-#include <coreplugin/editormanager/editormanager.h>
+#include <extensionsystem/pluginmanager.h>
 #include <extensionsystem/pluginspec.h>
-#include <qmljs/qmljsmodelmanagerinterface.h>
-#include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/project.h>
-#include <projectexplorer/target.h>
+#include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/session.h>
+#include <projectexplorer/target.h>
+#include <sqlitelibraryinitializer.h>
+#include <qmljs/qmljsmodelmanagerinterface.h>
 
 #include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
+#include <utils/algorithm.h>
 
 #include <QAction>
 #include <QTimer>
@@ -114,6 +119,7 @@ QtQuickDesignerFactory::QtQuickDesignerFactory()
 class QmlDesignerPluginPrivate
 {
 public:
+    QmlDesignerProjectManager projectManager;
     ViewManager viewManager;
     DocumentManager documentManager;
     ShortCutManager shortCutManager;
@@ -209,7 +215,8 @@ QmlDesignerPlugin::~QmlDesignerPlugin()
 ////////////////////////////////////////////////////
 bool QmlDesignerPlugin::initialize(const QStringList & /*arguments*/, QString *errorMessage/* = 0*/)
 {
-    QDir{}.mkpath(Core::ICore::cacheResourcePath());
+    Sqlite::LibraryInitializer::initialize();
+    QDir{}.mkpath(Core::ICore::cacheResourcePath().toString());
 
     if (!Utils::HostOsInfo::canCreateOpenGLContext(errorMessage))
         return false;
@@ -217,24 +224,28 @@ bool QmlDesignerPlugin::initialize(const QStringList & /*arguments*/, QString *e
     if (DesignerSettings::getValue(DesignerSettingsKey::STANDALONE_MODE).toBool())
         GenerateResource::generateMenuEntry();
 
-    QString fontPath = Core::ICore::resourcePath() +
-            QStringLiteral("/qmldesigner/propertyEditorQmlSources/imports/StudioTheme/icons.ttf");
+    GenerateCmake::generateMenuEntry();
+
+    const QString fontPath
+        = Core::ICore::resourcePath(
+                "qmldesigner/propertyEditorQmlSources/imports/StudioTheme/icons.ttf")
+              .toString();
     if (QFontDatabase::addApplicationFont(fontPath) < 0)
         qCWarning(qmldesignerLog) << "Could not add font " << fontPath << "to font database";
 
-    TwoFingerSwipe::registerRecognizer();
-
     return true;
 }
 
 bool QmlDesignerPlugin::delayedInitialize()
 {
     // adding default path to item library plugins
-    const QString pluginPath = Utils::HostOsInfo::isMacHost()
-            ? QString(QCoreApplication::applicationDirPath() + "/../PlugIns/QmlDesigner")
-            : QString(QCoreApplication::applicationDirPath() + "/../"
-                      + QLatin1String(IDE_LIBRARY_BASENAME) + "/qtcreator/plugins/qmldesigner");
-    MetaInfo::setPluginPaths(QStringList(pluginPath));
+    const QString postfix = Utils::HostOsInfo::isMacHost() ? QString("/QmlDesigner")
+                                                           : QString("/qmldesigner");
+    const QStringList pluginPaths =
+        Utils::transform(ExtensionSystem::PluginManager::pluginPaths(), [postfix](const QString &p) {
+            return QString(p + postfix);
+        });
+    MetaInfo::setPluginPaths(pluginPaths);
 
     d->settings.fromSettings(Core::ICore::settings());
 
@@ -246,6 +257,10 @@ bool QmlDesignerPlugin::delayedInitialize()
 
         auto curveEditorView = new QmlDesigner::CurveEditorView;
         d->viewManager.registerViewTakingOwnership(curveEditorView);
+
+        auto eventlistView = new QmlDesigner::EventListPluginView;
+        d->viewManager.registerViewTakingOwnership(eventlistView);
+        eventlistView->registerActions();
     }
 
     auto transitionEditorView = new QmlDesigner::TransitionEditorView;
@@ -564,11 +579,16 @@ void QmlDesignerPlugin::emitUsageStatisticsContextAction(const QString &identifi
     emitUsageStatistics(Constants::EVENT_ACTION_EXECUTED + identifier);
 }
 
-ImageCache &QmlDesignerPlugin::imageCache()
+AsynchronousImageCache &QmlDesignerPlugin::imageCache()
 {
     return m_instance->d->viewManager.imageCache();
 }
 
+void QmlDesignerPlugin::registerPreviewImageProvider(QQmlEngine *engine)
+{
+    m_instance->d->projectManager.registerPreviewImageProvider(engine);
+}
+
 void QmlDesignerPlugin::emitUsageStatisticsTime(const QString &identifier, int elapsed)
 {
     emit instance()->usageStatisticsUsageTimer(identifier, elapsed);
