@@ -30,9 +30,13 @@
 #include <designersettings.h>
 
 #include <utils/utilsicons.h>
+#include <projectexplorer/kitinformation.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/project.h>
+#include <projectexplorer/target.h>
+
+#include <android/androidconstants.h>
 
 #include <QLabel>
 #include <QComboBox>
@@ -44,18 +48,37 @@ using namespace ProjectExplorer;
 
 const Utils::Icon previewIcon({
         {":/qmlpreviewplugin/images/live_preview.png", Utils::Theme::IconsBaseColor}});
+const QByteArray livePreviewId = "LivePreview";
 
 static void handleAction(const SelectionContext &context)
 {
     if (context.view()->isAttached()) {
-        if (context.toggled())
-            ProjectExplorerPlugin::runStartupProject(Constants::QML_PREVIEW_RUN_MODE);
-         else
+        if (context.toggled()) {
+            bool skipDeploy = false;
+            if (const Target *startupTarget = SessionManager::startupTarget()) {
+                const Kit *kit = startupTarget->kit();
+                if (kit
+                    && (kit->supportedPlatforms().contains(Android::Constants::ANDROID_DEVICE_TYPE)
+                        || DeviceTypeKitAspect::deviceTypeId(kit)
+                               == Android::Constants::ANDROID_DEVICE_TYPE)) {
+                    skipDeploy = true;
+                    // In case of an android kit we don't want the live preview button to be toggled
+                    // when the emulator is started as we don't have control over its run status.
+                    DesignerActionManager &designerActionManager = QmlDesignerPlugin::instance()
+                                                                       ->designerActionManager();
+                    if (const ActionInterface *interface = designerActionManager.actionByMenuId(
+                            livePreviewId))
+                        interface->action()->setChecked(false);
+                }
+            }
+            ProjectExplorerPlugin::runStartupProject(Constants::QML_PREVIEW_RUN_MODE, skipDeploy);
+        } else {
             QmlPreviewPlugin::stopAllRunControls();
+        }
     }
 }
 
-QmlPreviewAction::QmlPreviewAction() : ModelNodeAction("LivePreview",
+QmlPreviewAction::QmlPreviewAction() : ModelNodeAction(livePreviewId,
                                                        "Live Preview",
                                                        previewIcon.icon(),
                                                        QmlPreviewPlugin::tr("Show Live Preview"),
@@ -172,7 +195,7 @@ QWidget *FpsLabelAction::createWidget(QWidget *parent)
 
 void FpsLabelAction::refreshFpsLabel(quint16 frames)
 {
-    for (const auto &labelPointer : fpsHandlerLabelList) {
+    for (const auto &labelPointer : qAsConst(fpsHandlerLabelList)) {
         if (labelPointer)
             labelPointer->setText(QString("%1 FPS").arg(frames));
     }
@@ -217,14 +240,13 @@ SwitchLanguageComboboxAction::SwitchLanguageComboboxAction(QObject *parent)
 QWidget *SwitchLanguageComboboxAction::createWidget(QWidget *parent)
 {
     QPointer<QComboBox> comboBox = new QComboBox(parent);
-    // FIXME: this combobox does not work at the moment
-    comboBox->setDisabled(true);
     const QString toolTip(tr("Switch the language used by preview."));
     comboBox->setToolTip(toolTip);
     comboBox->addItem(tr("Default"));
 
     auto refreshComboBoxFunction = [this, comboBox, toolTip] (ProjectExplorer::Project *project) {
         if (comboBox && project) {
+            comboBox->setDisabled(true);
             QString errorMessage;
             auto locales = project->availableQmlPreviewTranslations(&errorMessage);
             if (!errorMessage.isEmpty())
@@ -234,6 +256,7 @@ QWidget *SwitchLanguageComboboxAction::createWidget(QWidget *parent)
                 comboBox->addItem(tr("Default"));
                 comboBox->addItems(locales);
                 m_previousLocales = locales;
+                comboBox->setEnabled(true);
             }
         }
     };
