@@ -328,7 +328,7 @@ bool isLiteral(AST::Node *ast)
 
 QStringList qmlJSAutoComplete(QTextDocument *textDocument,
                               int position,
-                              const QString &fileName,
+                              const Utils::FilePath &fileName,
                               TextEditor::AssistReason reason,
                               const SemanticInfo &info)
 {
@@ -515,7 +515,7 @@ bool QmlJSCompletionAssistProvider::isContinuationChar(const QChar &c) const
     return isIdentifierChar(c, false);
 }
 
-IAssistProcessor *QmlJSCompletionAssistProvider::createProcessor() const
+IAssistProcessor *QmlJSCompletionAssistProvider::createProcessor(const AssistInterface *) const
 {
     return new QmlJSCompletionAssistProcessor;
 }
@@ -552,8 +552,6 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
     if (assistInterface->reason() == IdleEditor && !acceptsIdleEditor())
         return nullptr;
 
-    const QString &fileName = m_interface->fileName();
-
     m_startPosition = assistInterface->position();
     while (isIdentifierChar(m_interface->textDocument()->characterAt(m_startPosition - 1), false, false))
         --m_startPosition;
@@ -567,10 +565,9 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
         return nullptr;
 
     const Document::Ptr document = semanticInfo.document;
-    const QFileInfo currentFileInfo(fileName);
 
     bool isQmlFile = false;
-    if (currentFileInfo.suffix() == QLatin1String("qml"))
+    if (m_interface->filePath().endsWith(".qml"))
         isQmlFile = true;
 
     const QList<AST::Node *> path = semanticInfo.rangePath(m_interface->position());
@@ -891,7 +888,8 @@ bool QmlJSCompletionAssistProcessor::acceptsIdleEditor() const
             ++startPos;
 
             const QString &word = m_interface->textAt(startPos, cursorPos - startPos);
-            if (word.length() > 2 && isIdentifierChar(word.at(0), true)) {
+            if (word.length() >= TextEditorSettings::completionSettings().m_characterThreshold
+                    && isIdentifierChar(word.at(0), true)) {
                 for (int i = 1; i < word.length(); ++i) {
                     if (!isIdentifierChar(word.at(i)))
                         return false;
@@ -982,7 +980,7 @@ bool QmlJSCompletionAssistProcessor::completeUrl(const QString &relativeBasePath
 // ------------------------------
 QmlJSCompletionAssistInterface::QmlJSCompletionAssistInterface(QTextDocument *textDocument,
                                                                int position,
-                                                               const QString &fileName,
+                                                               const Utils::FilePath &fileName,
                                                                AssistReason reason,
                                                                const SemanticInfo &info)
     : AssistInterface(textDocument, position, fileName, reason)
