@@ -45,6 +45,7 @@
 #include <utils/infobar.h>
 
 #include <QDebug>
+#include <QTextCodec>
 
 const char QML_UI_FILE_WARNING[] = "QmlJSEditor.QmlUiFileWarning";
 
@@ -89,7 +90,7 @@ protected:
         QString text;
         for (; id; id = id->next) {
             if (!id->name.isEmpty())
-                text += id->name;
+                text += id->name.toString();
             else
                 text += QLatin1Char('?');
 
@@ -174,7 +175,7 @@ protected:
         QString text;
         for (; id; id = id->next) {
             if (!id->name.isEmpty())
-                text += id->name;
+                text += id->name.toString();
             else
                 text += QLatin1Char('?');
 
@@ -294,12 +295,12 @@ protected:
         init(&decl, ast);
 
         decl.text.fill(QLatin1Char(' '), _depth);
-        decl.text += ast->name;
+        decl.text += ast->name.toString();
 
         decl.text += QLatin1Char('(');
         for (FormalParameterList *it = ast->formals; it; it = it->next) {
             if (!it->element->bindingIdentifier.isEmpty())
-                decl.text += it->element->bindingIdentifier;
+                decl.text += it->element->bindingIdentifier.toString();
 
             if (it->next)
                 decl.text += QLatin1String(", ");
@@ -319,7 +320,7 @@ protected:
 
         Declaration decl;
         decl.text.fill(QLatin1Char(' '), _depth);
-        decl.text += ast->bindingIdentifier;
+        decl.text += ast->bindingIdentifier.toString();
 
         const SourceLocation first = ast->identifierToken;
         decl.startLine = first.startLine;
@@ -342,12 +343,12 @@ protected:
             init(&decl, ast);
 
             decl.text.fill(QLatin1Char(' '), _depth);
-            decl.text += field->name;
+            decl.text += field->name.toString();
 
             decl.text += QLatin1Char('(');
             for (FormalParameterList *it = funcExpr->formals; it; it = it->next) {
                 if (!it->element->bindingIdentifier.isEmpty())
-                    decl.text += it->element->bindingIdentifier;
+                    decl.text += it->element->bindingIdentifier.toString();
 
                 if (it->next)
                     decl.text += QLatin1String(", ");
@@ -655,9 +656,15 @@ QmlJSEditorDocument::QmlJSEditorDocument(Utils::Id id)
     connect(this, &TextEditor::TextDocument::tabSettingsChanged,
             d, &Internal::QmlJSEditorDocumentPrivate::invalidateFormatterCache);
     setSyntaxHighlighter(new QmlJSHighlighter(document()));
+    setCodec(QTextCodec::codecForName("UTF-8")); // qml files are defined to be utf-8
     setIndenter(new Internal::Indenter(document()));
 }
 
+bool QmlJSEditorDocument::supportsCodec(const QTextCodec *codec) const
+{
+    return codec == QTextCodec::codecForName("UTF-8");
+}
+
 QmlJSEditorDocument::~QmlJSEditorDocument()
 {
     delete d;
