@@ -174,7 +174,7 @@ protected:
 class CollectionTask : protected Visitor
 {
 public:
-    CollectionTask(QFutureInterface<SemanticHighlighter::Use> futureInterface,
+    CollectionTask(QFutureInterface<SemanticHighlighter::Use> &futureInterface,
                    const QmlJSTools::SemanticInfo &semanticInfo)
         : m_futureInterface(futureInterface)
         , m_semanticInfo(semanticInfo)
@@ -224,18 +224,22 @@ public:
 protected:
     void accept(Node *ast)
     {
+        if (m_futureInterface.isCanceled())
+            return;
         if (ast)
             ast->accept(this);
     }
 
     void scopedAccept(Node *ast, Node *child)
     {
+        if (m_futureInterface.isCanceled())
+            return;
         m_scopeBuilder.push(ast);
         accept(child);
         m_scopeBuilder.pop();
     }
 
-    void processName(const QStringView &name, SourceLocation location)
+    void processName(QStringView name, SourceLocation location)
     {
         if (name.isEmpty())
             return;
@@ -404,7 +408,7 @@ protected:
                 length = end-begin;
             }
 
-            const TextEditor::FontSettings &fontSettings = TextEditor::TextEditorSettings::instance()->fontSettings();
+            const TextEditor::FontSettings &fontSettings = TextEditor::TextEditorSettings::fontSettings();
 
             QTextCharFormat format;
             if (d.isWarning())
@@ -441,7 +445,7 @@ protected:
                 length = end-begin;
             }
 
-            const TextEditor::FontSettings &fontSettings = TextEditor::TextEditorSettings::instance()->fontSettings();
+            const TextEditor::FontSettings &fontSettings = TextEditor::TextEditorSettings::fontSettings();
 
             QTextCharFormat format;
             if (d.severity == Severity::Warning
@@ -528,7 +532,7 @@ private:
         m_uses.reserve(chunkSize);
     }
 
-    QFutureInterface<SemanticHighlighter::Use> m_futureInterface;
+    QFutureInterface<SemanticHighlighter::Use> &m_futureInterface;
     const QmlJSTools::SemanticInfo &m_semanticInfo;
     ScopeChain m_scopeChain;
     ScopeBuilder m_scopeBuilder;
@@ -553,6 +557,7 @@ SemanticHighlighter::SemanticHighlighter(QmlJSEditorDocument *document)
             this, &SemanticHighlighter::applyResults);
     connect(&m_watcher, &QFutureWatcherBase::finished,
             this, &SemanticHighlighter::finished);
+    m_futureSynchronizer.setCancelOnWait(true);
 }
 
 void SemanticHighlighter::rerun(const QmlJSTools::SemanticInfo &semanticInfo)
@@ -560,8 +565,10 @@ void SemanticHighlighter::rerun(const QmlJSTools::SemanticInfo &semanticInfo)
     m_watcher.cancel();
 
     m_startRevision = m_document->document()->revision();
-    m_watcher.setFuture(Utils::runAsync(QThread::LowestPriority,
-                                        &SemanticHighlighter::run, this, semanticInfo));
+    auto future = Utils::runAsync(QThread::LowestPriority, &SemanticHighlighter::run,
+                                  this, semanticInfo);
+    m_watcher.setFuture(future);
+    m_futureSynchronizer.addFuture(future);
 }
 
 void SemanticHighlighter::cancel()
