@@ -30,6 +30,7 @@
 #include "qmljsindenter.h"
 #include "qmljsqtstylecodeformatter.h"
 
+#include <texteditor/fontsettings.h>
 #include <texteditor/snippets/snippetprovider.h>
 #include <texteditor/tabsettings.h>
 #include <texteditor/simplecodestylepreferences.h>
@@ -127,8 +128,7 @@ QmlJSCodeStyleSettingsPage::QmlJSCodeStyleSettingsPage()
     setDisplayName(QCoreApplication::translate("QmlJSTools", Constants::QML_JS_CODE_STYLE_SETTINGS_NAME));
     setCategory(QmlJSEditor::Constants::SETTINGS_CATEGORY_QML);
     setDisplayCategory(QCoreApplication::translate("QmlJSEditor", "Qt Quick"));
-    setCategoryIcon(Utils::Icon({{":/qmljstools/images/settingscategory_qml.png",
-                    Utils::Theme::PanelTextColorDark}}, Utils::Icon::Tint));
+    setCategoryIconPath(":/qmljstools/images/settingscategory_qml.png");
 }
 
 QWidget *QmlJSCodeStyleSettingsPage::widget()
