@@ -68,8 +68,6 @@ private:
 
 class QmlJSCodeStyleSettingsPage : public Core::IOptionsPage
 {
-    Q_OBJECT
-
 public:
     QmlJSCodeStyleSettingsPage();
 
@@ -83,4 +81,4 @@ private:
 };
 
 } // namespace Internal
-} // namespace CppTools
+} // namespace QmlJSTools
