@@ -66,6 +66,11 @@ void QmlPreviewConnectionManager::setFpsHandler(QmlPreviewFpsHandler fpsHandler)
     m_fpsHandler = fpsHandler;
 }
 
+void QmlPreviewConnectionManager::setQmlDebugTranslationClientCreator(QmlDebugTranslationClientCreator creator)
+{
+    m_createDebugTranslationClientMethod = creator;
+}
+
 void QmlPreviewConnectionManager::createClients()
 {
     createPreviewClient();
@@ -113,9 +118,9 @@ QUrl QmlPreviewConnectionManager::findValidI18nDirectoryAsUrl(const QString &loc
 
 void QmlPreviewConnectionManager::createDebugTranslationClient()
 {
-    m_qmlDebugTranslationClient = new QmlDebugTranslationClient(connection());
+    m_qmlDebugTranslationClient = m_createDebugTranslationClientMethod(connection());
     connect(this, &QmlPreviewConnectionManager::language,
-                     m_qmlDebugTranslationClient, [this](const QString &locale) {
+                     m_qmlDebugTranslationClient.get(), [this](const QString &locale) {
         m_lastUsedLanguage = locale;
         // findValidI18nDirectoryAsUrl does not work if we didn't load any file
         // service expects a context URL.
@@ -124,10 +129,7 @@ void QmlPreviewConnectionManager::createDebugTranslationClient()
             m_qmlDebugTranslationClient->changeLanguage(findValidI18nDirectoryAsUrl(locale), locale);
         }
     });
-    connect(this, &QmlPreviewConnectionManager::changeElideWarning,
-            m_qmlDebugTranslationClient, &QmlDebugTranslationClient::changeElideWarning);
-
-    connect(m_qmlDebugTranslationClient.data(), &QmlDebugTranslationClient::debugServiceUnavailable,
+    connect(m_qmlDebugTranslationClient.get(), &QmlDebugTranslationClient::debugServiceUnavailable,
                      this, []() {
         QMessageBox::warning(Core::ICore::dialogParent(), "Error connect to QML DebugTranslation service",
                              "QML DebugTranslation feature is not available for this version of Qt.");
@@ -196,11 +198,13 @@ void QmlPreviewConnectionManager::createPreviewClient()
             m_qmlPreviewClient->announceError(path);
     });
 
-    connect(m_qmlPreviewClient.data(), &QmlPreviewClient::errorReported,
-                     this, [](const QString &error) {
-        Core::MessageManager::write("Error loading QML Live Preview:");
-        Core::MessageManager::write(error);
-    });
+    connect(m_qmlPreviewClient.data(),
+            &QmlPreviewClient::errorReported,
+            this,
+            [](const QString &error) {
+                Core::MessageManager::writeDisrupting("Error loading QML Live Preview:");
+                Core::MessageManager::writeSilently(error);
+            });
 
     connect(m_qmlPreviewClient.data(), &QmlPreviewClient::fpsReported,
                      this, [this](const QmlPreviewClient::FpsInfo &frames) {
@@ -258,10 +262,8 @@ void QmlPreviewConnectionManager::clearClient(QObject *client)
 void QmlPreviewConnectionManager::destroyClients()
 {
     clearClient(m_qmlPreviewClient);
-    clearClient(m_qmlDebugTranslationClient);
-    m_fileSystemWatcher.removeFiles(m_fileSystemWatcher.files());
-    QTC_ASSERT(m_fileSystemWatcher.directories().isEmpty(),
-               m_fileSystemWatcher.removeDirectories(m_fileSystemWatcher.directories()));
+    clearClient(m_qmlDebugTranslationClient.release());
+    m_fileSystemWatcher.clear();
 }
 
 } // namespace Internal
