@@ -52,6 +52,8 @@ QmlPreviewRunner::QmlPreviewRunner(const QmlPreviewRunnerSetting &settings)
     m_connectionManager.setFileLoader(settings.fileLoader);
     m_connectionManager.setFileClassifier(settings.fileClassifier);
     m_connectionManager.setFpsHandler(settings.fpsHandler);
+    m_connectionManager.setQmlDebugTranslationClientCreator(
+        settings.createDebugTranslationClientMethod);
 
     connect(this, &QmlPreviewRunner::loadFile,
             &m_connectionManager, &Internal::QmlPreviewConnectionManager::loadFile);
@@ -62,8 +64,6 @@ QmlPreviewRunner::QmlPreviewRunner(const QmlPreviewRunnerSetting &settings)
             &m_connectionManager, &Internal::QmlPreviewConnectionManager::zoom);
     connect(this, &QmlPreviewRunner::language,
             &m_connectionManager, &Internal::QmlPreviewConnectionManager::language);
-    connect(this, &QmlPreviewRunner::changeElideWarning,
-            &m_connectionManager, &Internal::QmlPreviewConnectionManager::changeElideWarning);
 
     connect(&m_connectionManager, &Internal::QmlPreviewConnectionManager::connectionOpened,
             this, [this, settings]() {
@@ -71,8 +71,6 @@ QmlPreviewRunner::QmlPreviewRunner(const QmlPreviewRunnerSetting &settings)
             emit zoom(settings.zoom);
         if (!settings.language.isEmpty())
             emit language(settings.language);
-        if (settings.translationElideWarning)
-            emit changeElideWarning(true);
 
         emit ready();
     });
@@ -130,7 +128,7 @@ LocalQmlPreviewSupport::LocalQmlPreviewSupport(ProjectExplorer::RunControl *runC
 
     setStarter([this, runControl, serverUrl] {
         ProjectExplorer::Runnable runnable = runControl->runnable();
-        QStringList qmlProjectRunConfigurationArguments = runnable.commandLine().splitArguments();
+        QStringList qmlProjectRunConfigurationArguments = runnable.command.splitArguments();
 
         const auto currentTarget = runControl->target();
         const auto *qmlBuildSystem = qobject_cast<QmlProjectManager::QmlBuildSystem *>(currentTarget->buildSystem());
@@ -144,15 +142,13 @@ LocalQmlPreviewSupport::LocalQmlPreviewSupport(ProjectExplorer::RunControl *runC
 
             if (!currentFile.isEmpty() && qmlProjectRunConfigurationArguments.last().contains(mainScriptFromProject)) {
                 qmlProjectRunConfigurationArguments.removeLast();
-                auto commandLine = Utils::CommandLine(runnable.commandLine().executable(), qmlProjectRunConfigurationArguments);
-                commandLine.addArg(currentFile);
-                runnable.setCommandLine(commandLine);
+                runnable.command = Utils::CommandLine(runnable.command.executable(), qmlProjectRunConfigurationArguments);
+                runnable.command.addArg(currentFile);
             }
         }
 
-        Utils::QtcProcess::addArg(&runnable.commandLineArguments,
-                                  QmlDebug::qmlDebugLocalArguments(QmlDebug::QmlPreviewServices,
-                                                                   serverUrl.path()));
+        runnable.command.addArg(QmlDebug::qmlDebugLocalArguments(QmlDebug::QmlPreviewServices,
+                                                                 serverUrl.path()));
         doStart(runnable, {});
     });
 }
