@@ -31,11 +31,10 @@
 #include <QtTest>
 #include <QVariant>
 
-Q_DECLARE_METATYPE(QmlPreview::Internal::TestFileLoader)
-Q_DECLARE_METATYPE(QmlPreview::Internal::TestFpsHandler)
+Q_DECLARE_METATYPE(QmlPreview::TestFileLoader)
+Q_DECLARE_METATYPE(QmlPreview::TestFpsHandler)
 
 namespace QmlPreview {
-namespace Internal {
 
 QmlPreviewPluginTest::QmlPreviewPluginTest(QObject *parent) : QObject(parent)
 {
@@ -78,21 +77,6 @@ void QmlPreviewPluginTest::testZoomFactorProperty()
     QCOMPARE(spy.count(), 2);
 }
 
-void QmlPreviewPluginTest::testLocaleProperty()
-{
-    ExtensionSystem::IPlugin *plugin = getPlugin();
-    QVERIFY(plugin);
-
-    QSignalSpy spy(plugin, SIGNAL(localeChanged(QString)));
-
-    QCOMPARE(plugin->property("locale").toString(), QString());
-    plugin->setProperty("locale", "de_DE");
-    QCOMPARE(plugin->property("locale").toString(), QString("de_DE"));
-    plugin->setProperty("locale", "qt_QT");
-    QCOMPARE(plugin->property("locale").toString(), QString("qt_QT"));
-    QCOMPARE(spy.count(), 2);
-}
-
 void QmlPreviewPluginTest::testFpsHandlerProperty()
 {
     ExtensionSystem::IPlugin *plugin = getPlugin();
@@ -105,5 +89,4 @@ void QmlPreviewPluginTest::testFpsHandlerProperty()
     handler(stats);
 }
 
-} // namespace Internal
 } // namespace QmlPreview
