@@ -46,17 +46,18 @@ FlameGraphView::FlameGraphView(QmlProfilerModelManager *manager, QWidget *parent
     setObjectName("QmlProfiler.FlameGraph.Dock");
     setWindowTitle(tr("Flame Graph"));
 
-    qmlRegisterType<FlameGraph::FlameGraph>("FlameGraph", 1, 0, "FlameGraph");
-    qmlRegisterUncreatableType<FlameGraphModel>("QmlProfilerFlameGraphModel", 1, 0,
+#if QT_VERSION < QT_VERSION_CHECK(6, 2, 0)
+    qmlRegisterType<FlameGraph::FlameGraph>("QtCreator.Tracing", 1, 0, "FlameGraph");
+    qmlRegisterUncreatableType<FlameGraphModel>("QtCreator.QmlProfiler", 1, 0,
                                                 "QmlProfilerFlameGraphModel",
                                                 QLatin1String("use the context property"));
-    qmlRegisterUncreatableType<QAbstractItemModel>("AbstractItemModel", 1, 0, "AbstractItemModel",
-                                                   QLatin1String("only for Qt 5.4"));
+#endif // Qt < 6.2
 
     Timeline::TimelineTheme::setupTheme(m_content->engine());
 
     m_content->rootContext()->setContextProperty(QStringLiteral("flameGraphModel"), m_model);
-    m_content->setSource(QUrl(QStringLiteral("qrc:/qmlprofiler/QmlProfilerFlameGraphView.qml")));
+    m_content->setSource(
+                QUrl(QStringLiteral("qrc:/QtCreator/QmlProfiler/QmlProfilerFlameGraphView.qml")));
     m_content->setClearColor(Utils::creatorTheme()->color(Utils::Theme::Timeline_BackgroundColor1));
 
     m_content->setResizeMode(QQuickWidget::SizeRootObjectToView);
