@@ -43,15 +43,16 @@ class QmlProfilerAnimationsModel : public QmlProfilerTimelineModel
     Q_OBJECT
 public:
 
-    struct QmlPaintEventData {
+    struct Item {
         int framerate;
         int animationcount;
         int typeId;
     };
 
-    QmlProfilerAnimationsModel(QmlProfilerModelManager *manager, QObject *parent = 0);
+    QmlProfilerAnimationsModel(QmlProfilerModelManager *manager,
+                               Timeline::TimelineModelAggregator *parent);
 
-    int rowMaxValue(int rowNumber) const override;
+    qint64 rowMaxValue(int rowNumber) const override;
 
     int typeId(int index) const override;
     Q_INVOKABLE int expandedRow(int index) const override;
@@ -63,13 +64,12 @@ public:
     QVariantList labels() const override;
     QVariantMap details(int index) const override;
 
-    bool accepted(const QmlEventType &type) const override;
     void loadEvent(const QmlEvent &event, const QmlEventType &type) override;
     void finalize() override;
     void clear() override;
 
 private:
-    QVector<QmlProfilerAnimationsModel::QmlPaintEventData> m_data;
+    QVector<Item> m_data;
     int m_maxGuiThreadAnimations = 0;
     int m_maxRenderThreadAnimations = 0;
     qint64 m_minNextStartTimes[2];
