@@ -34,7 +34,11 @@ namespace Internal {
 class BindingLoopMaterial : public QSGMaterial {
 public:
     QSGMaterialType *type() const override;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QSGMaterialShader *createShader() const override;
+#else
+    QSGMaterialShader *createShader(QSGRendererInterface::RenderMode) const override;
+#endif // < Qt 6
     BindingLoopMaterial();
 };
 
@@ -62,7 +66,8 @@ private:
 };
 
 struct Point2DWithOffset {
-    float x, y, x2, y2;
+    float x, y; // vec4 vertexCoord
+    float x2, y2; // vec2 postScaleOffset
     void set(float nx, float ny, float nx2, float ny2);
 };
 
@@ -195,12 +200,16 @@ Timeline::TimelineRenderPass::State *QmlProfilerBindingLoopsRenderPass::update(
 
 const QSGGeometry::AttributeSet &BindlingLoopsGeometry::point2DWithOffset()
 {
-    static QSGGeometry::Attribute data[] = {
-        QSGGeometry::Attribute::create(0, 2, GL_FLOAT, true),
-        QSGGeometry::Attribute::create(1, 2, GL_FLOAT),
+    static const QSGGeometry::Attribute data[] = {
+        // vec4 vertexCoord
+        QSGGeometry::Attribute::createWithAttributeType(0, 2, QSGGeometry::FloatType,
+                                                        QSGGeometry::PositionAttribute),
+        // vec2 postScaleOffset
+        QSGGeometry::Attribute::createWithAttributeType(1, 2, QSGGeometry::FloatType,
+                                                        QSGGeometry::UnknownAttribute),
     };
-    static QSGGeometry::AttributeSet attrs = {
-        2,
+    static const QSGGeometry::AttributeSet attrs = {
+        sizeof(data) / sizeof(data[0]),
         sizeof(Point2DWithOffset),
         data
     };
@@ -215,10 +224,10 @@ Point2DWithOffset *BindlingLoopsGeometry::vertexData()
     const QSGGeometry::Attribute *attributes = geometry->attributes();
     Q_ASSERT(attributes[0].position == 0);
     Q_ASSERT(attributes[0].tupleSize == 2);
-    Q_ASSERT(attributes[0].type == GL_FLOAT);
+    Q_ASSERT(attributes[0].type == QSGGeometry::FloatType);
     Q_ASSERT(attributes[1].position == 1);
     Q_ASSERT(attributes[1].tupleSize == 2);
-    Q_ASSERT(attributes[1].type == GL_FLOAT);
+    Q_ASSERT(attributes[1].type == QSGGeometry::FloatType);
     Q_UNUSED(attributes)
     return static_cast<Point2DWithOffset *>(geometry->vertexData());
 }
@@ -290,36 +299,73 @@ class BindingLoopMaterialShader : public QSGMaterialShader
 public:
     BindingLoopMaterialShader();
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     void updateState(const RenderState &state, QSGMaterial *newEffect,
                      QSGMaterial *oldEffect) override;
     char const *const *attributeNames() const override;
+#else // < Qt 6
+    bool updateUniformData(RenderState &state, QSGMaterial *, QSGMaterial *) override;
+#endif // < Qt 6
 
 private:
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     void initialize() override;
 
     int m_matrix_id = 0;
     int m_z_range_id = 0;
     int m_color_id = 0;
+#endif // < Qt 6
 };
 
 BindingLoopMaterialShader::BindingLoopMaterialShader()
     : QSGMaterialShader()
 {
-    setShaderSourceFile(QOpenGLShader::Vertex, QStringLiteral(":/qmlprofiler/bindingloops.vert"));
-    setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/qmlprofiler/bindingloops.frag"));
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    setShaderSourceFile(QOpenGLShader::Vertex,
+                        QStringLiteral(":/QtCreator/QmlProfiler/bindingloops.vert"));
+    setShaderSourceFile(QOpenGLShader::Fragment,
+                        QStringLiteral(":/QtCreator/QmlProfiler/bindingloops.frag"));
+#else // < Qt 6
+    setShaderFileName(VertexStage, ":/QtCreator/QmlProfiler/bindingloops_qt6.vert.qsb");
+    setShaderFileName(FragmentStage, ":/QtCreator/QmlProfiler/bindingloops_qt6.frag.qsb");
+#endif // < Qt 6
 }
 
+static QColor bindingLoopsColor()
+{
+    return Utils::creatorTheme()->color(Utils::Theme::Timeline_HighlightColor);
+}
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 void BindingLoopMaterialShader::updateState(const RenderState &state, QSGMaterial *, QSGMaterial *)
 {
     if (state.isMatrixDirty()) {
         program()->setUniformValue(m_matrix_id, state.combinedMatrix());
         program()->setUniformValue(m_z_range_id, GLfloat(1.0));
-        program()->setUniformValue(
-                    m_color_id,
-                    Utils::creatorTheme()->color(Utils::Theme::Timeline_HighlightColor));
+        program()->setUniformValue(m_color_id, bindingLoopsColor());
     }
 }
+#else // < Qt 6
+bool BindingLoopMaterialShader::updateUniformData(RenderState &state, QSGMaterial *, QSGMaterial *)
+{
+    QByteArray *buf = state.uniformData();
 
+    // mat4 matrix
+    if (state.isMatrixDirty()) {
+        const QMatrix4x4 m = state.combinedMatrix();
+        memcpy(buf->data(), m.constData(), 64);
+    }
+
+    // vec4 bindingLoopsColor
+    const QColor color = bindingLoopsColor();
+    const float colorArray[] = { color.redF(), color.greenF(), color.blueF(), color.alphaF() };
+    memcpy(buf->data() + 64, colorArray, 16);
+
+    return true;
+}
+#endif // < Qt 6
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 char const *const *BindingLoopMaterialShader::attributeNames() const
 {
     static const char *const attr[] = {"vertexCoord", "postScaleOffset", nullptr};
@@ -332,11 +378,14 @@ void BindingLoopMaterialShader::initialize()
     m_z_range_id = program()->uniformLocation("_qt_zRange");
     m_color_id = program()->uniformLocation("bindingLoopsColor");
 }
-
+#endif // < Qt 6
 
 BindingLoopMaterial::BindingLoopMaterial()
 {
     setFlag(QSGMaterial::Blending, false);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    setFlag(QSGMaterial::CustomCompileStep, true);
+#endif // >= Qt 6
 }
 
 QSGMaterialType *BindingLoopMaterial::type() const
@@ -345,7 +394,11 @@ QSGMaterialType *BindingLoopMaterial::type() const
     return &type;
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 QSGMaterialShader *BindingLoopMaterial::createShader() const
+#else // < Qt 6
+QSGMaterialShader *BindingLoopMaterial::createShader(QSGRendererInterface::RenderMode) const
+#endif // < Qt 6
 {
     return new BindingLoopMaterialShader;
 }
