@@ -314,9 +314,9 @@ void QmlProfilerTool::finalizeRunControl(QmlProfilerRunner *runWorker)
     if (auto aspect = static_cast<QmlProfilerRunConfigurationAspect *>(
                 runControl->aspect(Constants::SETTINGS))) {
         if (auto settings = static_cast<const QmlProfilerSettings *>(aspect->currentSettings())) {
-            d->m_profilerConnections->setFlushInterval(settings->flushEnabled() ?
-                                                           settings->flushInterval() : 0);
-            d->m_profilerModelManager->setAggregateTraces(settings->aggregateTraces());
+            d->m_profilerConnections->setFlushInterval(settings->flushEnabled.value() ?
+                                                           settings->flushInterval.value() : 0);
+            d->m_profilerModelManager->setAggregateTraces(settings->aggregateTraces.value());
         }
     }
 
@@ -571,12 +571,7 @@ ProjectExplorer::RunControl *QmlProfilerTool::attachToWaitingApplication()
 
 void QmlProfilerTool::logState(const QString &msg)
 {
-    MessageManager::write(msg, MessageManager::Flash);
-}
-
-void QmlProfilerTool::logError(const QString &msg)
-{
-    MessageManager::write(msg);
+    MessageManager::writeFlashing(msg);
 }
 
 void QmlProfilerTool::showErrorDialog(const QString &error)
@@ -591,11 +586,11 @@ void QmlProfilerTool::showErrorDialog(const QString &error)
     errorDialog->show();
 }
 
-void saveLastTraceFile(const QString &filename)
+static void saveLastTraceFile(const FilePath &filePath)
 {
     QmlProfilerSettings *settings = QmlProfilerPlugin::globalSettings();
-    if (filename != settings->lastTraceFile()) {
-        settings->setLastTraceFile(filename);
+    if (filePath != settings->lastTraceFile.filePath()) {
+        settings->lastTraceFile.setFilePath(filePath);
         settings->writeGlobalSettings();
     }
 }
@@ -604,16 +599,16 @@ void QmlProfilerTool::showSaveDialog()
 {
     QLatin1String tFile(QtdFileExtension);
     QLatin1String zFile(QztFileExtension);
-    QString filename = QFileDialog::getSaveFileName(
-                ICore::dialogParent(), tr("Save QML Trace"),
-                QmlProfilerPlugin::globalSettings()->lastTraceFile(),
+    FilePath filePath = FileUtils::getSaveFilePath(
+                nullptr, tr("Save QML Trace"),
+                QmlProfilerPlugin::globalSettings()->lastTraceFile.filePath(),
                 tr("QML traces (*%1 *%2)").arg(zFile).arg(tFile));
-    if (!filename.isEmpty()) {
-        if (!filename.endsWith(zFile) && !filename.endsWith(tFile))
-            filename += zFile;
-        saveLastTraceFile(filename);
+    if (!filePath.isEmpty()) {
+        if (!filePath.endsWith(zFile) && !filePath.endsWith(tFile))
+            filePath = filePath + zFile;
+        saveLastTraceFile(filePath);
         Debugger::enableMainWindow(false);
-        Core::ProgressManager::addTask(d->m_profilerModelManager->save(filename),
+        Core::ProgressManager::addTask(d->m_profilerModelManager->save(filePath.toString()),
                                        tr("Saving Trace Data"), TASK_SAVE,
                                        Core::ProgressManager::ShowInApplicationIcon);
     }
@@ -628,18 +623,18 @@ void QmlProfilerTool::showLoadDialog()
 
     QLatin1String tFile(QtdFileExtension);
     QLatin1String zFile(QztFileExtension);
-    QString filename = QFileDialog::getOpenFileName(
-                ICore::dialogParent(), tr("Load QML Trace"),
-                QmlProfilerPlugin::globalSettings()->lastTraceFile(),
+    FilePath filePath = FileUtils::getOpenFilePath(
+                nullptr, tr("Load QML Trace"),
+                QmlProfilerPlugin::globalSettings()->lastTraceFile.filePath(),
                 tr("QML traces (*%1 *%2)").arg(zFile).arg(tFile));
 
-    if (!filename.isEmpty()) {
-        saveLastTraceFile(filename);
+    if (!filePath.isEmpty()) {
+        saveLastTraceFile(filePath);
         Debugger::enableMainWindow(false);
         connect(d->m_profilerModelManager, &QmlProfilerModelManager::recordedFeaturesChanged,
                 this, &QmlProfilerTool::setRecordedFeatures);
         d->m_profilerModelManager->populateFileFinder();
-        Core::ProgressManager::addTask(d->m_profilerModelManager->load(filename),
+        Core::ProgressManager::addTask(d->m_profilerModelManager->load(filePath.toString()),
                                        tr("Loading Trace Data"), TASK_LOAD);
     }
 }
