@@ -60,6 +60,11 @@ void QmlProjectItem::setQtForMCUs(bool b)
     m_qtForMCUs = b;
 }
 
+void QmlProjectItem::setQt6Project(bool qt6Project)
+{
+    m_qt6Project = qt6Project;
+}
+
 void QmlProjectItem::setImportPaths(const QStringList &importPaths)
 {
     if (m_importPaths != importPaths)
@@ -72,6 +77,18 @@ void QmlProjectItem::setFileSelectors(const QStringList &selectors)
         m_fileSelectors = selectors;
 }
 
+void QmlProjectItem::setSupportedLanguages(const QStringList &languages)
+{
+    if (m_supportedLanguages != languages)
+        m_supportedLanguages = languages;
+}
+
+void QmlProjectItem::setPrimaryLanguage(const QString &language)
+{
+    if (m_primaryLanguage != language)
+        m_primaryLanguage = language;
+}
+
 /* Returns list of absolute paths */
 QStringList QmlProjectItem::files() const
 {
