@@ -27,13 +27,15 @@
 
 #include "fileformat/qmlprojectfileformat.h"
 #include "fileformat/qmlprojectitem.h"
-#include "qmlprojectrunconfiguration.h"
 #include "qmlprojectconstants.h"
 #include "qmlprojectmanagerconstants.h"
 #include "qmlprojectnodes.h"
+#include "qmlprojectplugin.h"
+#include "qmlprojectrunconfiguration.h"
 
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/editormanager/documentmodel.h>
+#include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
@@ -53,11 +55,15 @@
 #include <texteditor/textdocument.h>
 
 #include <utils/algorithm.h>
+#include <utils/infobar.h>
+#include <utils/qtcprocess.h>
 
 #include <QDebug>
+#include <QLoggingCategory>
+#include <QMessageBox>
 #include <QRegularExpression>
 #include <QTextCodec>
-#include <QLoggingCategory>
+#include <QTimer>
 
 using namespace Core;
 using namespace ProjectExplorer;
@@ -70,24 +76,83 @@ Q_LOGGING_CATEGORY(infoLogger, "QmlProjectManager.QmlBuildSystem", QtInfoMsg)
 
 namespace QmlProjectManager {
 
+static bool isQtDesignStudio()
+{
+    QSettings *settings = Core::ICore::settings();
+    const QString qdsStandaloneEntry = "QML/Designer/StandAloneMode"; //entry from qml settings
+
+    return settings->value(qdsStandaloneEntry, false).toBool();
+}
+static int preferedQtTarget(Target *target)
+{
+    if (target) {
+        const QmlBuildSystem *buildSystem = qobject_cast<QmlBuildSystem *>(target->buildSystem());
+        if (buildSystem && buildSystem->qt6Project())
+            return 6;
+    }
+    return 5;
+}
+
+const char openInQDSAppSetting[] = "OpenInQDSApp";
+
 QmlProject::QmlProject(const Utils::FilePath &fileName)
     : Project(QString::fromLatin1(Constants::QMLPROJECT_MIMETYPE), fileName)
 {
     setId(QmlProjectManager::Constants::QML_PROJECT_ID);
     setProjectLanguages(Context(ProjectExplorer::Constants::QMLJS_LANGUAGE_ID));
-    setDisplayName(fileName.toFileInfo().completeBaseName());
+    setDisplayName(fileName.completeBaseName());
 
     setNeedsBuildConfigurations(false);
     setBuildSystemCreator([](Target *t) { return new QmlBuildSystem(t); });
+
+    if (!isQtDesignStudio()) {
+        if (QmlProjectPlugin::qdsInstallationExists()) {
+            auto lambda = [fileName]() {
+                if (Core::ICore::infoBar()->canInfoBeAdded(openInQDSAppSetting)) {
+                    Utils::InfoBarEntry
+                        info(openInQDSAppSetting,
+                             tr("Would you like to open the project in Qt Design Studio?"),
+                             Utils::InfoBarEntry::GlobalSuppression::Disabled);
+                    info.setCustomButtonInfo(tr("Open in Qt Design Studio"), [&, fileName] {
+                        Core::ICore::infoBar()->removeInfo(openInQDSAppSetting);
+                        QmlProjectPlugin::openQDS(fileName);
+                    });
+                    Core::ICore::infoBar()->addInfo(info);
+                }
+            };
+
+            QTimer::singleShot(0, this, lambda);
+        }
+    } else {
+        m_openFileConnection = connect(
+            this, &QmlProject::anyParsingFinished, this, [this](Target *target, bool success) {
+                if (m_openFileConnection)
+                    disconnect(m_openFileConnection);
+
+                if (target && success) {
+                    const Utils::FilePath &folder = projectDirectory();
+                    const Utils::FilePaths &uiFiles = files([&](const ProjectExplorer::Node *node) {
+                        return node->filePath().completeSuffix() == "ui.qml"
+                               && node->filePath().parentDir() == folder;
+                    });
+                    if (!uiFiles.isEmpty()) {
+                        Utils::FilePath currentFile;
+                        if (auto cd = Core::EditorManager::currentDocument())
+                            currentFile = cd->filePath();
+
+                        if (currentFile.isEmpty() || !isKnownFile(currentFile))
+                            Core::EditorManager::openEditor(uiFiles.first(), Utils::Id());
+                    }
+                }
+            });
+    }
 }
 
 QmlBuildSystem::QmlBuildSystem(Target *target)
     : BuildSystem(target)
 {
-    const QString normalized
-            = Utils::FileUtils::normalizePathName(target->project()
-                      ->projectFilePath().toFileInfo().canonicalFilePath());
-    m_canonicalProjectDir = Utils::FilePath::fromString(normalized).parentDir();
+    m_canonicalProjectDir =
+            target->project()->projectFilePath().canonicalPath().normalizedPathName().parentDir();
 
     connect(target->project(), &Project::projectFileIsDirty,
             this, &QmlBuildSystem::refreshProjectFile);
@@ -106,10 +171,7 @@ QmlBuildSystem::QmlBuildSystem(Target *target)
     updateDeploymentData();
 }
 
-QmlBuildSystem::~QmlBuildSystem()
-{
-    delete m_projectItem.data();
-}
+QmlBuildSystem::~QmlBuildSystem() = default;
 
 void QmlBuildSystem::triggerParsing()
 {
@@ -137,39 +199,41 @@ void QmlBuildSystem::parseProject(RefreshOptions options)
 {
     if (options & Files) {
         if (options & ProjectFile)
-            delete m_projectItem.data();
+            m_projectItem.reset();
         if (!m_projectItem) {
-              QString errorMessage;
-              m_projectItem = QmlProjectFileFormat::parseProjectFile(projectFilePath(), &errorMessage);
-              if (m_projectItem) {
-                  connect(m_projectItem.data(), &QmlProjectItem::qmlFilesChanged,
-                          this, &QmlBuildSystem::refreshFiles);
+            QString errorMessage;
+            m_projectItem.reset(
+                QmlProjectFileFormat::parseProjectFile(projectFilePath(), &errorMessage));
+            if (m_projectItem) {
+                connect(m_projectItem.get(),
+                        &QmlProjectItem::qmlFilesChanged,
+                        this,
+                        &QmlBuildSystem::refreshFiles);
 
-              } else {
-                  MessageManager::write(tr("Error while loading project file %1.")
-                                        .arg(projectFilePath().toUserOutput()),
-                                        MessageManager::NoModeSwitch);
-                  MessageManager::write(errorMessage);
-              }
+            } else {
+                MessageManager::writeFlashing(
+                    tr("Error while loading project file %1.").arg(projectFilePath().toUserOutput()));
+                MessageManager::writeSilently(errorMessage);
+            }
         }
         if (m_projectItem) {
-            m_projectItem.data()->setSourceDirectory(canonicalProjectDir().toString());
+            m_projectItem->setSourceDirectory(canonicalProjectDir().toString());
             if (m_projectItem->targetDirectory().isEmpty())
                 m_projectItem->setTargetDirectory(canonicalProjectDir().toString());
 
             if (auto modelManager = QmlJS::ModelManagerInterface::instance())
-                modelManager->updateSourceFiles(m_projectItem.data()->files(), true);
+                modelManager->updateSourceFiles(m_projectItem->files(), true);
 
-            QString mainFilePath = m_projectItem.data()->mainFile();
+            QString mainFilePath = m_projectItem->mainFile();
             if (!mainFilePath.isEmpty()) {
                 mainFilePath
                         = QDir(canonicalProjectDir().toString()).absoluteFilePath(mainFilePath);
                 Utils::FileReader reader;
                 QString errorMessage;
-                if (!reader.fetch(mainFilePath, &errorMessage)) {
-                    MessageManager::write(tr("Warning while loading project file %1.")
-                                          .arg(projectFilePath().toUserOutput()));
-                    MessageManager::write(errorMessage);
+                if (!reader.fetch(Utils::FilePath::fromString(mainFilePath), &errorMessage)) {
+                    MessageManager::writeFlashing(tr("Warning while loading project file %1.")
+                                                      .arg(projectFilePath().toUserOutput()));
+                    MessageManager::writeSilently(errorMessage);
                 }
             }
         }
@@ -207,21 +271,33 @@ void QmlBuildSystem::refresh(RefreshOptions options)
 QString QmlBuildSystem::mainFile() const
 {
     if (m_projectItem)
-        return m_projectItem.data()->mainFile();
+        return m_projectItem->mainFile();
     return QString();
 }
 
+Utils::FilePath QmlBuildSystem::mainFilePath() const
+{
+    return projectDirectory().pathAppended(mainFile());
+}
+
 bool QmlBuildSystem::qtForMCUs() const
 {
     if (m_projectItem)
-        return m_projectItem.data()->qtForMCUs();
+        return m_projectItem->qtForMCUs();
+    return false;
+}
+
+bool QmlBuildSystem::qt6Project() const
+{
+    if (m_projectItem)
+        return m_projectItem->qt6Project();
     return false;
 }
 
 void QmlBuildSystem::setMainFile(const QString &mainFilePath)
 {
     if (m_projectItem)
-        m_projectItem.data()->setMainFile(mainFilePath);
+        m_projectItem->setMainFile(mainFilePath);
 }
 
 Utils::FilePath QmlBuildSystem::targetDirectory() const
@@ -246,24 +322,50 @@ Utils::FilePath QmlBuildSystem::targetFile(const Utils::FilePath &sourceFile) co
 Utils::EnvironmentItems QmlBuildSystem::environment() const
 {
     if (m_projectItem)
-        return m_projectItem.data()->environment();
+        return m_projectItem->environment();
     return {};
 }
 
 QStringList QmlBuildSystem::customImportPaths() const
 {
     if (m_projectItem)
-        return m_projectItem.data()->importPaths();
+        return m_projectItem->importPaths();
     return {};
 }
 
 QStringList QmlBuildSystem::customFileSelectors() const
 {
     if (m_projectItem)
-        return m_projectItem.data()->fileSelectors();
+        return m_projectItem->fileSelectors();
     return {};
 }
 
+QStringList QmlBuildSystem::supportedLanguages() const
+{
+    if (m_projectItem)
+        return m_projectItem->supportedLanguages();
+    return {};
+}
+
+void QmlBuildSystem::setSupportedLanguages(QStringList languages)
+{
+    if (m_projectItem)
+        m_projectItem->setSupportedLanguages(languages);
+}
+
+QString QmlBuildSystem::primaryLanguage() const
+{
+    if (m_projectItem)
+        return m_projectItem->primaryLanguage();
+    return {};
+}
+
+void QmlBuildSystem::setPrimaryLanguage(QString language)
+{
+    if (m_projectItem)
+        m_projectItem->setPrimaryLanguage(language);
+}
+
 void QmlBuildSystem::refreshProjectFile()
 {
     refresh(QmlBuildSystem::ProjectFile | Files);
@@ -319,9 +421,9 @@ Tasks QmlProject::projectIssues(const Kit *k) const
 
     if (dev->type() == ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE) {
         if (version->type() == QtSupport::Constants::DESKTOPQT) {
-            if (version->qmlsceneCommand().isEmpty()) {
+            if (version->qmlRuntimeFilePath().isEmpty()) {
                 result.append(createProjectTask(Task::TaskType::Error,
-                                                tr("Qt version has no qmlscene command.")));
+                                                tr("Qt version has no QML utility.")));
             }
         } else {
             // Non-desktop Qt on a desktop device? We don't support that.
@@ -329,8 +431,8 @@ Tasks QmlProject::projectIssues(const Kit *k) const
                                             tr("Non-desktop Qt is used with a desktop device.")));
         }
     } else {
-        // If not a desktop device, don't check the Qt version for qmlscene.
-        // The device is responsible for providing it and we assume qmlscene can be found
+        // If not a desktop device, don't check the Qt version for qml runtime binary.
+        // The device is responsible for providing it and we assume qml runtime can be found
         // in $PATH if it's not explicitly given.
     }
 
@@ -345,8 +447,10 @@ Project::RestoreResult QmlProject::fromMap(const QVariantMap &map, QString *erro
 
     if (!activeTarget()) {
         // find a kit that matches prerequisites (prefer default one)
-        const QList<Kit*> kits = Utils::filtered(KitManager::kits(), [this](const Kit *k) {
-            return !containsType(projectIssues(k), Task::TaskType::Error);
+        const QList<Kit *> kits = Utils::filtered(KitManager::kits(), [this](const Kit *k) {
+            return !containsType(projectIssues(k), Task::TaskType::Error)
+                   && DeviceTypeKitAspect::deviceTypeId(k)
+                          == ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE;
         });
 
         if (!kits.isEmpty()) {
@@ -355,6 +459,29 @@ Project::RestoreResult QmlProject::fromMap(const QVariantMap &map, QString *erro
             else
                 addTargetForKit(kits.first());
         }
+
+        if (isQtDesignStudio()) {
+            auto setKitWithVersion = [&](int qtMajorVersion) {
+                const QList<Kit *> qtVersionkits
+                    = Utils::filtered(kits, [qtMajorVersion](const Kit *k) {
+                          QtSupport::BaseQtVersion *version = QtSupport::QtKitAspect::qtVersion(k);
+                          return (version && version->qtVersion().majorVersion == qtMajorVersion);
+                      });
+                if (!qtVersionkits.isEmpty()) {
+                    if (qtVersionkits.contains(KitManager::defaultKit()))
+                        addTargetForDefaultKit();
+                    else
+                        addTargetForKit(qtVersionkits.first());
+                }
+            };
+
+            int preferedVersion = preferedQtTarget(activeTarget());
+
+            if (activeTarget())
+                removeTarget(activeTarget());
+
+            setKitWithVersion(preferedVersion);
+        }
     }
 
     return RestoreResult::Ok;
@@ -397,7 +524,7 @@ void QmlBuildSystem::updateDeploymentData()
     ProjectExplorer::DeploymentData deploymentData;
     for (const QString &file : m_projectItem->files()) {
         deploymentData.addFile(
-                    file,
+                    FilePath::fromString(file),
                     targetFile(Utils::FilePath::fromString(file)).parentDir().toString());
     }
 
@@ -408,10 +535,22 @@ QVariant QmlBuildSystem::additionalData(Id id) const
 {
     if (id == Constants::customFileSelectorsData)
         return customFileSelectors();
+    if (id == Constants::supportedLanguagesData)
+        return supportedLanguages();
+    if (id == Constants::primaryLanguageData)
+        return primaryLanguage();
     if (id == Constants::customForceFreeTypeData)
         return forceFreeType();
     if (id == Constants::customQtForMCUs)
         return qtForMCUs();
+    if (id == Constants::customQt6Project)
+        return qt6Project();
+    if (id == Constants::mainFilePath)
+        return mainFilePath().toString();
+    if (id == Constants::customImportPaths)
+        return customImportPaths();
+    if (id == Constants::canonicalProjectDir)
+        return canonicalProjectDir().toString();
     return {};
 }
 
@@ -442,24 +581,31 @@ QmlProject *QmlBuildSystem::qmlProject() const
 bool QmlBuildSystem::forceFreeType() const
 {
     if (m_projectItem)
-        return m_projectItem.data()->forceFreeType();
+        return m_projectItem->forceFreeType();
     return false;
 }
 
-bool QmlBuildSystem::addFiles(Node *context, const QStringList &filePaths, QStringList *)
+bool QmlBuildSystem::widgetApp() const
+{
+    if (m_projectItem)
+        return m_projectItem->widgetApp();
+    return false;
+}
+
+bool QmlBuildSystem::addFiles(Node *context, const FilePaths &filePaths, FilePaths *)
 {
     if (!dynamic_cast<QmlProjectNode *>(context))
         return false;
 
-    QStringList toAdd;
-    foreach (const QString &filePath, filePaths) {
-        if (!m_projectItem.data()->matchesFile(filePath))
+    FilePaths toAdd;
+    for (const FilePath &filePath : filePaths) {
+        if (!m_projectItem->matchesFile(filePath.toString()))
             toAdd << filePaths;
     }
     return toAdd.isEmpty();
 }
 
-bool QmlBuildSystem::deleteFiles(Node *context, const QStringList &filePaths)
+bool QmlBuildSystem::deleteFiles(Node *context, const FilePaths &filePaths)
 {
     if (dynamic_cast<QmlProjectNode *>(context))
         return true;
@@ -467,15 +613,15 @@ bool QmlBuildSystem::deleteFiles(Node *context, const QStringList &filePaths)
     return BuildSystem::deleteFiles(context, filePaths);
 }
 
-bool QmlBuildSystem::renameFile(Node * context, const QString &filePath, const QString &newFilePath)
+bool QmlBuildSystem::renameFile(Node * context, const FilePath &oldFilePath, const FilePath &newFilePath)
 {
     if (dynamic_cast<QmlProjectNode *>(context)) {
-        if (filePath.endsWith(mainFile())) {
-            setMainFile(newFilePath);
+        if (oldFilePath.endsWith(mainFile())) {
+            setMainFile(newFilePath.toString());
 
             // make sure to change it also in the qmlproject file
             const Utils::FilePath qmlProjectFilePath = project()->projectFilePath();
-            Core::FileChangeBlocker fileChangeBlocker(qmlProjectFilePath.toString());
+            Core::FileChangeBlocker fileChangeBlocker(qmlProjectFilePath);
             const QList<Core::IEditor *> editors = Core::DocumentModel::editorsForFilePath(qmlProjectFilePath);
             TextEditor::TextDocument *document = nullptr;
             if (!editors.isEmpty()) {
@@ -489,20 +635,20 @@ bool QmlBuildSystem::renameFile(Node * context, const QString &filePath, const Q
             QString error;
             Utils::TextFileFormat textFileFormat;
             const QTextCodec *codec = QTextCodec::codecForName("UTF-8"); // qml files are defined to be utf-8
-            if (Utils::TextFileFormat::readFile(qmlProjectFilePath.toString(), codec, &fileContent, &textFileFormat, &error)
+            if (Utils::TextFileFormat::readFile(qmlProjectFilePath, codec, &fileContent, &textFileFormat, &error)
                     != Utils::TextFileFormat::ReadSuccess) {
                 qWarning() << "Failed to read file" << qmlProjectFilePath << ":" << error;
             }
 
             // find the mainFile and do the file name with brackets in a capture group and mask the . with \.
-            QString originalFileName = QFileInfo(filePath).fileName();
+            QString originalFileName = oldFilePath.fileName();
             originalFileName.replace(".", "\\.");
             const QRegularExpression expression(QString("mainFile:\\s*\"(%1)\"").arg(originalFileName));
             const QRegularExpressionMatch match = expression.match(fileContent);
 
-            fileContent.replace(match.capturedStart(1), match.capturedLength(1), QFileInfo(newFilePath).fileName());
+            fileContent.replace(match.capturedStart(1), match.capturedLength(1), newFilePath.fileName());
 
-            if (!textFileFormat.writeFile(qmlProjectFilePath.toString(), fileContent, &error))
+            if (!textFileFormat.writeFile(qmlProjectFilePath, fileContent, &error))
                 qWarning() << "Failed to write file" << qmlProjectFilePath << ":" << error;
 
             refresh(Everything);
@@ -511,7 +657,7 @@ bool QmlBuildSystem::renameFile(Node * context, const QString &filePath, const Q
         return true;
     }
 
-    return BuildSystem::renameFile(context, filePath, newFilePath);
+    return BuildSystem::renameFile(context, oldFilePath, newFilePath);
 }
 
 } // namespace QmlProjectManager
