@@ -34,12 +34,14 @@
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
 
-#include <projectexplorer/kitinformation.h>
-#include <projectexplorer/projectexplorer.h>
-#include <projectexplorer/runcontrol.h>
-#include <projectexplorer/runconfigurationaspects.h>
-#include <projectexplorer/target.h>
 #include <projectexplorer/environmentaspect.h>
+#include <projectexplorer/kitinformation.h>
+#include <projectexplorer/kitmanager.h>
+#include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/runconfigurationaspects.h>
+#include <projectexplorer/runcontrol.h>
+#include <projectexplorer/session.h>
+#include <projectexplorer/target.h>
 
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtsupportconstants.h>
@@ -64,25 +66,34 @@ namespace Internal {
 
 // QmlProjectRunConfiguration
 
+static bool isQtDesignStudio()
+{
+    QSettings *settings = Core::ICore::settings();
+    const QString qdsStandaloneEntry = "QML/Designer/StandAloneMode"; //entry from qml settings
+
+    return settings->value(qdsStandaloneEntry, false).toBool();
+}
+
 class QmlProjectRunConfiguration final : public RunConfiguration
 {
     Q_DECLARE_TR_FUNCTIONS(QmlProjectManager::QmlProjectRunConfiguration)
 
 public:
-    QmlProjectRunConfiguration(Target *target, Utils::Id id);
+    QmlProjectRunConfiguration(Target *target, Id id);
 
 private:
-    Runnable runnable() const final;
     QString disabledReason() const final;
     bool isEnabled() const final;
 
     QString mainScript() const;
-    Utils::FilePath qmlScenePath() const;
+    FilePath qmlRuntimeFilePath() const;
     QString commandLineArguments() const;
+    void createQtVersionAspect();
 
     StringAspect *m_qmlViewerAspect = nullptr;
     QmlMainFileAspect *m_qmlMainFileAspect = nullptr;
     QmlMultiLanguageAspect *m_multiLanguageAspect = nullptr;
+    SelectionAspect *m_qtversionAspect = nullptr;
 };
 
 QmlProjectRunConfiguration::QmlProjectRunConfiguration(Target *target, Id id)
@@ -98,12 +109,14 @@ QmlProjectRunConfiguration::QmlProjectRunConfiguration(Target *target, Id id)
     argumentAspect->setSettingsKey(Constants::QML_VIEWER_ARGUMENTS_KEY);
 
     setCommandLineGetter([this] {
-        return CommandLine(qmlScenePath(), commandLineArguments(), CommandLine::Raw);
+        return CommandLine(qmlRuntimeFilePath(), commandLineArguments(), CommandLine::Raw);
     });
 
     m_qmlMainFileAspect = addAspect<QmlMainFileAspect>(target);
     connect(m_qmlMainFileAspect, &QmlMainFileAspect::changed, this, &RunConfiguration::update);
 
+    createQtVersionAspect();
+
     connect(target, &Target::kitChanged, this, &RunConfiguration::update);
 
     m_multiLanguageAspect = addAspect<QmlMultiLanguageAspect>(target);
@@ -137,18 +150,13 @@ QmlProjectRunConfiguration::QmlProjectRunConfiguration(Target *target, Id id)
         return envModifier(environment);
     });
 
-    setDisplayName(tr("QML Scene", "QMLRunConfiguration display name."));
-    update();
-}
+    setRunnableModifier([this](Runnable &r) {
+        const QmlBuildSystem *bs = static_cast<QmlBuildSystem *>(activeBuildSystem());
+        r.workingDirectory = bs->targetDirectory();
+    });
 
-Runnable QmlProjectRunConfiguration::runnable() const
-{
-    Runnable r;
-    r.setCommandLine(commandLine());
-    r.environment = aspect<EnvironmentAspect>()->environment();
-    const QmlBuildSystem *bs = static_cast<QmlBuildSystem *>(activeBuildSystem());
-    r.workingDirectory = bs->targetDirectory().toString();
-    return r;
+    setDisplayName(tr("QML Utility", "QMLRunConfiguration display name."));
+    update();
 }
 
 QString QmlProjectRunConfiguration::disabledReason() const
@@ -156,18 +164,18 @@ QString QmlProjectRunConfiguration::disabledReason() const
     if (mainScript().isEmpty())
         return tr("No script file to execute.");
 
-    const FilePath viewer = qmlScenePath();
+    const FilePath viewer = qmlRuntimeFilePath();
     if (DeviceTypeKitAspect::deviceTypeId(kit())
             == ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE
             && !viewer.exists()) {
-        return tr("No qmlscene found.");
+        return tr("No QML utility found.");
     }
     if (viewer.isEmpty())
-        return tr("No qmlscene binary specified for target device.");
+        return tr("No QML utility specified for target device.");
     return RunConfiguration::disabledReason();
 }
 
-FilePath QmlProjectRunConfiguration::qmlScenePath() const
+FilePath QmlProjectRunConfiguration::qmlRuntimeFilePath() const
 {
     const QString qmlViewer = m_qmlViewerAspect->value();
     if (!qmlViewer.isEmpty())
@@ -175,23 +183,23 @@ FilePath QmlProjectRunConfiguration::qmlScenePath() const
 
     Kit *kit = target()->kit();
     BaseQtVersion *version = QtKitAspect::qtVersion(kit);
-    if (!version) // No Qt version in Kit. Don't try to run qmlscene.
+    if (!version) // No Qt version in Kit. Don't try to run QML runtime.
         return {};
 
     const Id deviceType = DeviceTypeKitAspect::deviceTypeId(kit);
     if (deviceType == ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE) {
         // If not given explicitly by Qt Version, try to pick it from $PATH.
         const bool isDesktop = version->type() == QtSupport::Constants::DESKTOPQT;
-        return FilePath::fromString(isDesktop ? version->qmlsceneCommand() : QString("qmlscene"));
+        return isDesktop ? version->qmlRuntimeFilePath() : "qmlscene";
     }
 
     IDevice::ConstPtr dev = DeviceKitAspect::device(kit);
-    if (dev.isNull()) // No device set. We don't know where to run qmlscene.
+    if (dev.isNull()) // No device set. We don't know where a QML utility is.
         return {};
 
-    const QString qmlscene = dev->qmlsceneCommand();
+    const FilePath qmlRuntime = dev->qmlRunCommand();
     // If not given explicitly by device, try to pick it from $PATH.
-    return FilePath::fromString(qmlscene.isEmpty() ? QString("qmlscene") : qmlscene);
+    return qmlRuntime.isEmpty() ? "qmlscene" : qmlRuntime;
 }
 
 QString QmlProjectRunConfiguration::commandLineArguments() const
@@ -205,37 +213,103 @@ QString QmlProjectRunConfiguration::commandLineArguments() const
     const QmlBuildSystem *bs = qobject_cast<QmlBuildSystem *>(target()->buildSystem());
     foreach (const QString &importPath,
              QmlBuildSystem::makeAbsolute(bs->targetDirectory(), bs->customImportPaths())) {
-        QtcProcess::addArg(&args, "-I", osType);
-        QtcProcess::addArg(&args, importPath, osType);
+        ProcessArgs::addArg(&args, "-I", osType);
+        ProcessArgs::addArg(&args, importPath, osType);
     }
 
     for (const QString &fileSelector : bs->customFileSelectors()) {
-        QtcProcess::addArg(&args, "-S", osType);
-        QtcProcess::addArg(&args, fileSelector, osType);
+        ProcessArgs::addArg(&args, "-S", osType);
+        ProcessArgs::addArg(&args, fileSelector, osType);
     }
 
-    if (Utils::HostOsInfo::isWindowsHost() && bs->forceFreeType()) {
-        Utils::QtcProcess::addArg(&args, "-platform", osType);
-        Utils::QtcProcess::addArg(&args, "windows:fontengine=freetype", osType);
+    if (HostOsInfo::isWindowsHost() && bs->forceFreeType()) {
+        ProcessArgs::addArg(&args, "-platform", osType);
+        ProcessArgs::addArg(&args, "windows:fontengine=freetype", osType);
+    }
+
+    if (bs->qt6Project() && bs->widgetApp()) {
+        ProcessArgs::addArg(&args, "--apptype", osType);
+        ProcessArgs::addArg(&args, "widget", osType);
     }
 
     const QString main = bs->targetFile(FilePath::fromString(mainScript())).toString();
     if (!main.isEmpty())
-        QtcProcess::addArg(&args, main, osType);
+        ProcessArgs::addArg(&args, main, osType);
 
     if (m_multiLanguageAspect && m_multiLanguageAspect->value())
-        QtcProcess::addArg(&args, "-qmljsdebugger=file:unused_if_debugger_arguments_added,services:DebugTranslation", osType);
+        ProcessArgs::addArg(&args, "-qmljsdebugger=file:unused_if_debugger_arguments_added,services:DebugTranslation", osType);
 
     return args;
 }
 
+void QmlProjectRunConfiguration::createQtVersionAspect()
+{
+    if (!isQtDesignStudio())
+        return;
+
+    m_qtversionAspect = addAspect<SelectionAspect>();
+    m_qtversionAspect->setDisplayStyle(SelectionAspect::DisplayStyle::ComboBox);
+    m_qtversionAspect->setLabelText(tr("Qt Version:"));
+    m_qtversionAspect->setSettingsKey("QmlProjectManager.kit");
+
+    Kit *kit = target()->kit();
+    BaseQtVersion *version = QtKitAspect::qtVersion(kit);
+
+    if (version) {
+        const QmlBuildSystem *buildSystem = qobject_cast<QmlBuildSystem *>(target()->buildSystem());
+        const bool isQt6Project = buildSystem && buildSystem->qt6Project();
+
+        if (isQt6Project) {
+            m_qtversionAspect->addOption(tr("Qt 6"));
+            m_qtversionAspect->setReadOnly(true);
+        } else { /* Only if this is not a Qt 6 project changing kits makes sense */
+            m_qtversionAspect->addOption(tr("Qt 5"));
+            m_qtversionAspect->addOption(tr("Qt 6"));
+
+            const int valueForVersion = version->qtVersion().majorVersion == 6 ? 1 : 0;
+
+            m_qtversionAspect->setValue(valueForVersion);
+
+            connect(m_qtversionAspect, &SelectionAspect::changed, this, [&]() {
+                QTC_ASSERT(target(), return );
+                auto project = target()->project();
+                QTC_ASSERT(project, return );
+
+                int oldValue = !m_qtversionAspect->value();
+                const int preferedQtVersion = m_qtversionAspect->value() > 0 ? 6 : 5;
+                Kit *currentKit = target()->kit();
+
+                const QList<Kit *> kits = Utils::filtered(KitManager::kits(), [&](const Kit *k) {
+                    QtSupport::BaseQtVersion *version = QtSupport::QtKitAspect::qtVersion(k);
+                    return (version && version->qtVersion().majorVersion == preferedQtVersion)
+                           && DeviceTypeKitAspect::deviceTypeId(k)
+                                  == ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE;
+                });
+
+                if (kits.contains(currentKit))
+                    return;
+
+                if (!kits.isEmpty()) {
+                    auto newTarget = target()->project()->target(kits.first());
+                    if (!newTarget)
+                        newTarget = project->addTargetForKit(kits.first());
+
+                    SessionManager::setActiveTarget(project, newTarget, SetActive::Cascade);
+
+                    /* Reset the aspect. We changed the target and this aspect should not change. */
+                    m_qtversionAspect->blockSignals(true);
+                    m_qtversionAspect->setValue(oldValue);
+                    m_qtversionAspect->blockSignals(false);
+                }
+            });
+        }
+    }
+}
+
 bool QmlProjectRunConfiguration::isEnabled() const
 {
-    if (m_qmlMainFileAspect->isQmlFilePresent() && !commandLine().executable().isEmpty()) {
-        BuildSystem *bs = activeBuildSystem();
-        return !bs->isParsing() && bs->hasParsingData();
-    }
-    return false;
+    return m_qmlMainFileAspect->isQmlFilePresent() && !commandLine().executable().isEmpty()
+            && activeBuildSystem()->hasParsingData();
 }
 
 QString QmlProjectRunConfiguration::mainScript() const
@@ -246,10 +320,10 @@ QString QmlProjectRunConfiguration::mainScript() const
 // QmlProjectRunConfigurationFactory
 
 QmlProjectRunConfigurationFactory::QmlProjectRunConfigurationFactory()
-    : FixedRunConfigurationFactory(QmlProjectRunConfiguration::tr("QML Scene"), false)
+    : FixedRunConfigurationFactory(QmlProjectRunConfiguration::tr("QML Runtime"), false)
 {
     registerRunConfiguration<QmlProjectRunConfiguration>
-            ("QmlProjectManager.QmlRunConfiguration.QmlScene");
+            ("QmlProjectManager.QmlRunConfiguration.Qml");
     addSupportedProjectType(QmlProjectManager::Constants::QML_PROJECT_ID);
 }
 
