@@ -77,7 +77,10 @@ QnxDeployQtLibrariesDialog::QnxDeployQtLibrariesDialog(const IDevice::ConstPtr &
     connect(m_uploadService, &AbstractRemoteLinuxDeployService::errorMessage,
             m_ui->deployLogWindow, &QPlainTextEdit::appendPlainText);
     connect(m_uploadService, &AbstractRemoteLinuxDeployService::warningMessage,
-            m_ui->deployLogWindow, &QPlainTextEdit::appendPlainText);
+            this, [this](const QString &message) {
+        if (!message.contains("stat:"))
+            m_ui->deployLogWindow->appendPlainText(message);
+    });
     connect(m_uploadService, &AbstractRemoteLinuxDeployService::stdOutData,
             m_ui->deployLogWindow, &QPlainTextEdit::appendPlainText);
     connect(m_uploadService, &AbstractRemoteLinuxDeployService::stdErrData,
@@ -166,12 +169,11 @@ void QnxDeployQtLibrariesDialog::updateProgress(const QString &progressMessage)
 {
     QTC_CHECK(m_state == Uploading);
 
-    if (!progressMessage.startsWith(QLatin1String("Uploading file")))
-        return;
-
-    ++m_progressCount;
-
-    m_ui->deployProgress->setValue(m_progressCount);
+    const int progress = progressMessage.count("sftp> put");
+    if (progress != 0) {
+        m_progressCount += progress;
+        m_ui->deployProgress->setValue(m_progressCount);
+    }
 }
 
 void QnxDeployQtLibrariesDialog::handleUploadFinished()
@@ -229,22 +231,22 @@ QList<DeployableFile> QnxDeployQtLibrariesDialog::gatherFiles()
             m_ui->qtLibraryCombo->itemData(m_ui->qtLibraryCombo->currentIndex()).toInt();
 
 
-    QnxQtVersion *qtVersion = dynamic_cast<QnxQtVersion *>(QtVersionManager::version(qtVersionId));
+    auto qtVersion = dynamic_cast<const QnxQtVersion *>(QtVersionManager::version(qtVersionId));
 
     QTC_ASSERT(qtVersion, return result);
 
     if (Utils::HostOsInfo::isWindowsHost()) {
-        result.append(gatherFiles(qtVersion->qmakeProperty("QT_INSTALL_LIBS"),
-                                  QString(), QStringList() << QLatin1String("*.so.?")));
-        result.append(gatherFiles(qtVersion->qmakeProperty("QT_INSTALL_LIBS")
-                                  + QLatin1String("/fonts")));
+        result.append(gatherFiles(qtVersion->libraryPath().toString(),
+                                  QString(),
+                                  QStringList() << QLatin1String("*.so.?")));
+        result.append(gatherFiles(qtVersion->libraryPath().toString() + QLatin1String("/fonts")));
     } else {
-        result.append(gatherFiles(qtVersion->qmakeProperty("QT_INSTALL_LIBS")));
+        result.append(gatherFiles(qtVersion->libraryPath().toString()));
     }
 
-    result.append(gatherFiles(qtVersion->qmakeProperty("QT_INSTALL_PLUGINS")));
-    result.append(gatherFiles(qtVersion->qmakeProperty("QT_INSTALL_IMPORTS")));
-    result.append(gatherFiles(qtVersion->qmakeProperty("QT_INSTALL_QML")));
+    result.append(gatherFiles(qtVersion->pluginPath().toString()));
+    result.append(gatherFiles(qtVersion->importsPath().toString()));
+    result.append(gatherFiles(qtVersion->qmlPath().toString()));
 
     return result;
 }
@@ -256,16 +258,24 @@ QList<DeployableFile> QnxDeployQtLibrariesDialog::gatherFiles(
     if (dirPath.isEmpty())
         return result;
 
+    static const QStringList unusedDirs = {"include", "mkspecs", "cmake", "pkgconfig"};
+    const QString dp = dirPath.endsWith('/') ? dirPath.left(dirPath.size() - 1) : dirPath;
+    if (unusedDirs.contains(dp))
+        return result;
+
     QDir dir(dirPath);
     QFileInfoList list = dir.entryInfoList(nameFilters,
             QDir::Dirs | QDir::Files | QDir::NoDotAndDotDot);
 
-    for (int i = 0; i < list.size(); ++i) {
-        QFileInfo fileInfo = list.at(i);
+    for (auto &fileInfo : list) {
         if (fileInfo.isDir()) {
             result.append(gatherFiles(fileInfo.absoluteFilePath(), baseDirPath.isEmpty() ?
                                           dirPath : baseDirPath));
         } else {
+            static const QStringList unusedSuffixes = {"cmake", "la", "prl", "a", "pc"};
+            if (unusedSuffixes.contains(fileInfo.suffix()))
+                continue;
+
             QString remoteDir;
             if (baseDirPath.isEmpty()) {
                 remoteDir = fullRemoteDirectory() + QLatin1Char('/') +
@@ -276,7 +286,8 @@ QList<DeployableFile> QnxDeployQtLibrariesDialog::gatherFiles(
                 remoteDir = fullRemoteDirectory() + QLatin1Char('/') +
                         baseDir.relativeFilePath(dirPath);
             }
-            result.append(DeployableFile(fileInfo.absoluteFilePath(), remoteDir));
+            result.append(DeployableFile(Utils::FilePath::fromString(fileInfo.absoluteFilePath()),
+                                         remoteDir));
         }
     }
 
@@ -293,12 +304,9 @@ void QnxDeployQtLibrariesDialog::checkRemoteDirectoryExistance()
     QTC_CHECK(m_state == Inactive);
 
     m_state = CheckingRemoteDirectory;
-
     m_ui->deployLogWindow->appendPlainText(tr("Checking existence of \"%1\"")
                                            .arg(fullRemoteDirectory()));
-
-    const QByteArray cmd = "test -d " + fullRemoteDirectory().toLatin1();
-    m_processRunner->run(cmd, m_device->sshParameters());
+    m_processRunner->run("test -d " + fullRemoteDirectory(), m_device->sshParameters());
 }
 
 void QnxDeployQtLibrariesDialog::removeRemoteDirectory()
@@ -306,11 +314,8 @@ void QnxDeployQtLibrariesDialog::removeRemoteDirectory()
     QTC_CHECK(m_state == CheckingRemoteDirectory);
 
     m_state = RemovingRemoteDirectory;
-
     m_ui->deployLogWindow->appendPlainText(tr("Removing \"%1\"").arg(fullRemoteDirectory()));
-
-    const QByteArray cmd = "rm -rf " + fullRemoteDirectory().toLatin1();
-    m_processRunner->run(cmd, m_device->sshParameters());
+    m_processRunner->run("rm -rf " + fullRemoteDirectory(), m_device->sshParameters());
 }
 
 } // namespace Internal
