@@ -28,6 +28,7 @@
 
 #include <ssh/sshremoteprocessrunner.h>
 #include <utils/qtcassert.h>
+#include <utils/fileutils.h>
 
 namespace Qnx {
 namespace Internal {
@@ -54,19 +55,24 @@ QnxDeviceTester::QnxDeviceTester(QObject *parent)
 
     m_commandsToTest << QLatin1String("awk")
                      << QLatin1String("cat")
+                     << QLatin1String("cut")
+                     << QLatin1String("df")
                      << QLatin1String("grep")
                      << QLatin1String("kill")
                      << QLatin1String("netstat")
+                     << QLatin1String("mkdir")
                      << QLatin1String("print")
                      << QLatin1String("printf")
-                     << QLatin1String("ps")
+                     << QLatin1String("pidin")
                      << QLatin1String("read")
+                     << QLatin1String("rm")
                      << QLatin1String("sed")
                      << QLatin1String("sleep")
+                     << QLatin1String("tail")
                      << QLatin1String("uname");
 }
 
-void QnxDeviceTester::testDevice(const ProjectExplorer::IDevice::ConstPtr &deviceConfiguration)
+void QnxDeviceTester::testDevice(const ProjectExplorer::IDevice::Ptr &deviceConfiguration)
 {
     QTC_ASSERT(m_state == Inactive, return);
 
@@ -86,6 +92,7 @@ void QnxDeviceTester::stopTest()
     case GenericTest:
         m_genericTester->stopTest();
         break;
+    case VarRunTest:
     case CommandsTest:
         m_processRunner->cancel();
         break;
@@ -105,6 +112,31 @@ void QnxDeviceTester::handleGenericTestFinished(TestResult result)
         return;
     }
 
+    m_state = VarRunTest;
+    emit progressMessage(tr("Checking that files can be created in /var/run..."));
+    m_processRunner->run(QStringLiteral("rm %1 > /dev/null 2>&1; echo ABC > %1 && rm %1")
+                             .arg("/var/run/qtc_xxxx.pid"),
+                         m_deviceConfiguration->sshParameters());
+}
+
+void QnxDeviceTester::handleVarRunProcessFinished(const QString &error)
+{
+    QTC_ASSERT(m_state == VarRunTest, return);
+
+    if (error.isEmpty()) {
+        if (m_processRunner->processExitCode() == 0) {
+            emit progressMessage(tr("Files can be created in /var/run.") + QLatin1Char('\n'));
+        } else {
+            emit errorMessage(tr("Files cannot be created in /var/run.") + QLatin1Char('\n'));
+            m_result = TestFailure;
+        }
+    } else {
+        emit errorMessage(tr("An error occurred while checking that"
+                             " files can be created in /var/run.")
+                          + QLatin1Char('\n'));
+        m_result = TestFailure;
+    }
+
     m_state = CommandsTest;
 
     QnxDevice::ConstPtr qnxDevice = m_deviceConfiguration.dynamicCast<const QnxDevice>();
@@ -113,12 +145,17 @@ void QnxDeviceTester::handleGenericTestFinished(TestResult result)
     testNextCommand();
 }
 
-void QnxDeviceTester::handleProcessFinished(int exitStatus)
+void QnxDeviceTester::handleProcessFinished(const QString &error)
 {
+    if (m_state == VarRunTest) {
+        handleVarRunProcessFinished(error);
+        return;
+    }
+
     QTC_ASSERT(m_state == CommandsTest, return);
 
     const QString command = m_commandsToTest[m_currentCommandIndex];
-    if (exitStatus == QSsh::SshRemoteProcess::NormalExit) {
+    if (error.isEmpty()) {
         if (m_processRunner->processExitCode() == 0) {
             emit progressMessage(tr("%1 found.").arg(command) + QLatin1Char('\n'));
         } else {
@@ -126,7 +163,7 @@ void QnxDeviceTester::handleProcessFinished(int exitStatus)
             m_result = TestFailure;
         }
     } else {
-        emit errorMessage(tr("An error occurred checking for %1.").arg(command)  + QLatin1Char('\n'));
+        emit errorMessage(tr("An error occurred while checking for %1.").arg(command)  + QLatin1Char('\n'));
         m_result = TestFailure;
     }
     testNextCommand();
@@ -153,15 +190,15 @@ void QnxDeviceTester::testNextCommand()
     QString command = m_commandsToTest[m_currentCommandIndex];
     emit progressMessage(tr("Checking for %1...").arg(command));
 
-    m_processRunner->run("command -v " + command.toLatin1(), m_deviceConfiguration->sshParameters());
+    m_processRunner->run("command -v " + command, m_deviceConfiguration->sshParameters());
 }
 
 void QnxDeviceTester::setFinished()
 {
     m_state = Inactive;
-    disconnect(m_genericTester, 0, this, 0);
+    disconnect(m_genericTester, nullptr, this, nullptr);
     if (m_processRunner)
-        disconnect(m_processRunner, 0, this, 0);
+        disconnect(m_processRunner, nullptr, this, nullptr);
     emit finished(m_result);
 }
 
