@@ -29,27 +29,26 @@
 #include <utils/algorithm.h>
 #include <utils/fileutils.h>
 #include <utils/hostosinfo.h>
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 #include <utils/temporaryfile.h>
 
 #include <QDebug>
 #include <QDir>
 #include <QDirIterator>
 #include <QDomDocument>
-#include <QProcess>
 #include <QStandardPaths>
 #include <QApplication>
 
 using namespace ProjectExplorer;
-using namespace Qnx;
-using namespace Qnx::Internal;
+using namespace Utils;
+
+namespace Qnx {
+namespace Internal {
 
-namespace {
 const char *EVAL_ENV_VARS[] = {
     "QNX_TARGET", "QNX_HOST", "QNX_CONFIGURATION", "QNX_CONFIGURATION_EXCLUSIVE",
     "MAKEFLAGS", "LD_LIBRARY_PATH", "PATH", "QDE", "CPUVARDIR", "PYTHONPATH"
 };
-}
 
 QString QnxUtils::cpuDirFromAbi(const Abi &abi)
 {
@@ -79,18 +78,17 @@ QString QnxUtils::cpuDirShortDescription(const QString &cpuDir)
     return cpuDir;
 }
 
-Utils::EnvironmentItems QnxUtils::qnxEnvironmentFromEnvFile(const QString &fileName)
+EnvironmentItems QnxUtils::qnxEnvironmentFromEnvFile(const FilePath &filePath)
 {
-    Utils::EnvironmentItems items;
+    EnvironmentItems items;
 
-    if (!QFileInfo::exists(fileName))
+    if (!filePath.exists())
         return items;
 
-    const bool isWindows = Utils::HostOsInfo::isWindowsHost();
+    const bool isWindows = HostOsInfo::isWindowsHost();
 
     // locking creating sdp-env file wrapper script
-    Utils::TemporaryFile tmpFile(QString::fromLatin1("sdp-env-eval-XXXXXX")
-                                 + QString::fromLatin1(isWindows ? ".bat" : ".sh"));
+    TemporaryFile tmpFile("sdp-env-eval-XXXXXX" + QString::fromLatin1(isWindows ? ".bat" : ".sh"));
     if (!tmpFile.open())
         return items;
     tmpFile.setTextModeEnabled(true);
@@ -98,71 +96,68 @@ Utils::EnvironmentItems QnxUtils::qnxEnvironmentFromEnvFile(const QString &fileN
     // writing content to wrapper script
     QTextStream fileContent(&tmpFile);
     if (isWindows)
-        fileContent << QLatin1String("@echo off\n")
-                    << QLatin1String("call ") << fileName << QLatin1Char('\n');
+        fileContent << "@echo off\n"
+                    << "call " << filePath.path() << '\n';
     else
-        fileContent << QLatin1String("#!/bin/bash\n")
-                    << QLatin1String(". ") << fileName << QLatin1Char('\n');
+        fileContent << "#!/bin/bash\n"
+                    << ". " << filePath.path() << '\n';
     QString linePattern = QString::fromLatin1(isWindows ? "echo %1=%%1%" : "echo %1=$%1");
     for (int i = 0, len = sizeof(EVAL_ENV_VARS) / sizeof(const char *); i < len; ++i)
         fileContent << linePattern.arg(QLatin1String(EVAL_ENV_VARS[i])) << QLatin1Char('\n');
     tmpFile.close();
 
     // running wrapper script
-    QProcess process;
+    QtcProcess process;
     if (isWindows)
-        process.start(QLatin1String("cmd.exe"),
-                QStringList() << QLatin1String("/C") << tmpFile.fileName());
+        process.setCommand({"cmd.exe", {"/C", tmpFile.fileName()}});
     else
-        process.start(QLatin1String("/bin/bash"),
-                QStringList() << tmpFile.fileName());
+        process.setCommand({"/bin/bash", {tmpFile.fileName()}});
+    process.start();
 
     // waiting for finish
     QApplication::setOverrideCursor(Qt::BusyCursor);
-    bool waitResult = process.waitForFinished(10000) || process.state() == QProcess::NotRunning;
+    bool waitResult = process.waitForFinished(10000);
     QApplication::restoreOverrideCursor();
     if (!waitResult) {
-        Utils::SynchronousProcess::stopProcess(process);
+        process.stopProcess();
         return items;
     }
 
-    if (process.exitStatus() != QProcess::NormalExit || process.exitCode() != 0)
+    if (process.result() != QtcProcess::FinishedWithSuccess)
         return items;
 
     // parsing process output
-    QTextStream str(&process);
-    while (!str.atEnd()) {
-        QString line = str.readLine();
-        int equalIndex = line.indexOf(QLatin1Char('='));
+    const QString output = process.stdOut();
+    for (const QString &line : output.split('\n')) {
+        int equalIndex = line.indexOf('=');
         if (equalIndex < 0)
             continue;
         QString var = line.left(equalIndex);
         QString value = line.mid(equalIndex + 1);
-        items.append(Utils::EnvironmentItem(var, value));
+        items.append(EnvironmentItem(var, value));
     }
 
     return items;
 }
 
-QString QnxUtils::envFilePath(const QString &sdpPath)
+FilePath QnxUtils::envFilePath(const FilePath &sdpPath)
 {
-    QDir sdp(sdpPath);
-    QStringList entries;
-    if (Utils::HostOsInfo::isWindowsHost())
-        entries = sdp.entryList(QStringList(QLatin1String("*-env.bat")));
+    FilePaths entries;
+    if (sdpPath.osType() == OsTypeWindows)
+        entries = sdpPath.dirEntries({"*-env.bat"});
     else
-        entries = sdp.entryList(QStringList(QLatin1String("*-env.sh")));
+        entries = sdpPath.dirEntries({"*-env.sh"});
 
     if (!entries.isEmpty())
-        return sdp.absoluteFilePath(entries.first());
+        return entries.first();
 
-    return QString();
+    return {};
 }
 
 QString QnxUtils::defaultTargetVersion(const QString &sdpPath)
 {
     foreach (const ConfigInstallInformation &sdpInfo, installedConfigs()) {
-        if (!sdpInfo.path.compare(sdpPath, Utils::HostOsInfo::fileNameCaseSensitivity()))
+        if (!sdpInfo.path.compare(sdpPath, HostOsInfo::fileNameCaseSensitivity()))
             return sdpInfo.version;
     }
 
@@ -212,14 +207,13 @@ QList<ConfigInstallInformation> QnxUtils::installedConfigs(const QString &config
     return sdpList;
 }
 
-Utils::EnvironmentItems QnxUtils::qnxEnvironment(const QString &sdpPath)
+EnvironmentItems QnxUtils::qnxEnvironment(const FilePath &sdpPath)
 {
     return qnxEnvironmentFromEnvFile(envFilePath(sdpPath));
 }
 
-QList<QnxTarget> QnxUtils::findTargets(const Utils::FilePath &basePath)
+QList<QnxTarget> QnxUtils::findTargets(const FilePath &basePath)
 {
-    using namespace Utils;
     QList<QnxTarget> result;
 
     QDirIterator iterator(basePath.toString());
@@ -261,3 +255,6 @@ Abis QnxUtils::convertAbis(const Abis &abis)
 {
     return Utils::transform(abis, &QnxUtils::convertAbi);
 }
+
+} // Internal
+} // Qnx
