@@ -98,7 +98,7 @@ public:
     virtual bool equals(BaseQtVersion *other);
 
     bool isAutodetected() const;
-    QString autodetectionSource() const;
+    QString detectionSource() const;
 
     QString displayName() const;
     QString unexpandedDisplayName() const;
@@ -119,24 +119,27 @@ public:
     virtual QString toHtml(bool verbose) const;
 
     ProjectExplorer::Abis qtAbis() const;
+    bool hasAbi(ProjectExplorer::Abi::OS, ProjectExplorer::Abi::OSFlavor flavor = ProjectExplorer::Abi::UnknownFlavor) const;
 
     void applyProperties(QMakeGlobals *qmakeGlobals) const;
     virtual void addToEnvironment(const ProjectExplorer::Kit *k, Utils::Environment &env) const;
-    virtual Utils::Environment qmakeRunEnvironment() const;
+    Utils::Environment qmakeRunEnvironment() const;
 
     // source path defined by qmake property QT_INSTALL_PREFIX/src or by qmake.stash QT_SOURCE_TREE
     Utils::FilePath sourcePath() const;
     // returns source path for installed qt packages and empty string for self build qt
     Utils::FilePath qtPackageSourcePath() const;
-    bool isInSourceDirectory(const Utils::FilePath &filePath);
-    bool isSubProject(const Utils::FilePath &filePath) const;
+    bool isInQtSourceDirectory(const Utils::FilePath &filePath) const;
+    bool isQtSubProject(const Utils::FilePath &filePath) const;
 
+    Utils::FilePath rccFilePath() const;
     // used by UiCodeModelSupport
-    QString uicCommand() const;
-    QString designerCommand() const;
-    QString linguistCommand() const;
-    QString qscxmlcCommand() const;
-    QString qmlsceneCommand() const;
+    Utils::FilePath uicFilePath() const;
+    Utils::FilePath designerFilePath() const;
+    Utils::FilePath linguistFilePath() const;
+    Utils::FilePath qscxmlcFilePath() const;
+    Utils::FilePath qmlRuntimeFilePath() const;
+    Utils::FilePath qmlplugindumpFilePath() const;
 
     QString qtVersionString() const;
     QtVersionNumber qtVersion() const;
@@ -148,7 +151,7 @@ public:
     bool hasDemos() const;
 
     // former local functions
-    Utils::FilePath qmakeCommand() const;
+    Utils::FilePath qmakeFilePath() const;
 
     /// @returns the name of the mkspec
     QString mkspec() const;
@@ -180,11 +183,7 @@ public:
     static bool isQtQuickCompilerSupported(const ProjectExplorer::Kit *k, QString *reason = nullptr);
     bool isQtQuickCompilerSupported(QString *reason = nullptr) const;
 
-    QString qmlDumpTool(bool debugVersion) const;
-
-    bool hasQmlDump() const;
     bool hasQmlDumpWithRelocatableFlag() const;
-    bool needsQmlDump() const;
 
     virtual QtConfigWidget *createConfigurationWidget() const;
 
@@ -197,6 +196,7 @@ public:
     Utils::FilePath prefix() const;
 
     Utils::FilePath binPath() const;
+    Utils::FilePath libExecPath() const;
     Utils::FilePath configurationPath() const;
     Utils::FilePath dataPath() const;
     Utils::FilePath demosPath() const;
@@ -211,11 +211,11 @@ public:
     Utils::FilePath translationsPath() const;
 
     Utils::FilePath hostBinPath() const;
+    Utils::FilePath hostLibexecPath() const;
     Utils::FilePath hostDataPath() const;
     Utils::FilePath hostPrefixPath() const;
 
     Utils::FilePath mkspecsPath() const;
-    Utils::FilePath qmlBinPath() const;
     Utils::FilePath librarySearchPath() const;
 
     Utils::FilePaths directoriesToIgnoreInProjectTree() const;
@@ -254,6 +254,7 @@ protected:
 
     void ensureMkSpecParsed() const;
     virtual void parseMkSpec(ProFileEvaluator *) const;
+    virtual void setupQmakeRunEnvironment(Utils::Environment &env) const;
 
 private:
     void updateDefaultDisplayName();
