@@ -26,17 +26,20 @@
 #include "profilereader.h"
 
 #include <coreplugin/icore.h>
+#include <projectexplorer/taskhub.h>
 
+#include <QCoreApplication>
 #include <QDebug>
 
+using namespace ProjectExplorer;
 using namespace QtSupport;
 
 static QString format(const QString &fileName, int lineNo, const QString &msg)
 {
     if (lineNo > 0)
         return QString::fromLatin1("%1(%2): %3").arg(fileName, QString::number(lineNo), msg);
-    else if (lineNo)
-        return QString::fromLatin1("%1: %3").arg(fileName, msg);
+    else if (!fileName.isEmpty())
+        return QString::fromLatin1("%1: %2").arg(fileName, msg);
     else
         return msg;
 }
@@ -45,34 +48,55 @@ ProMessageHandler::ProMessageHandler(bool verbose, bool exact)
     : m_verbose(verbose)
     , m_exact(exact)
     //: Prefix used for output from the cumulative evaluation of project files.
-    , m_prefix(tr("[Inexact] "))
+    , m_prefix(QCoreApplication::translate("ProMessageHandler", "[Inexact] "))
 {
-    connect(this, &ProMessageHandler::writeMessage,
-            Core::MessageManager::instance(), &Core::MessageManager::write, Qt::QueuedConnection);
+}
+
+ProMessageHandler::~ProMessageHandler()
+{
+    if (!m_messages.isEmpty())
+        Core::MessageManager::writeFlashing(m_messages);
+}
+
+static void addTask(Task::TaskType type,
+                    const QString &description,
+                    const Utils::FilePath &file = {},
+                    int line = -1)
+{
+    QMetaObject::invokeMethod(TaskHub::instance(), [=]() {
+        TaskHub::addTask(BuildSystemTask(type, description, file, line));
+    });
 }
 
 void ProMessageHandler::message(int type, const QString &msg, const QString &fileName, int lineNo)
 {
     if ((type & CategoryMask) == ErrorMessage && ((type & SourceMask) == SourceParser || m_verbose)) {
-        QString fmsg = format(fileName, lineNo, msg);
-        if ((type & SourceMask) == SourceParser || m_exact)
-            emit writeMessage(fmsg, Core::MessageManager::NoModeSwitch);
-        else
-            emit writeMessage(m_prefix + fmsg, Core::MessageManager::NoModeSwitch);
+        // parse error in qmake files
+        if (m_exact) {
+            addTask(Task::Error, msg, Utils::FilePath::fromString(fileName), lineNo);
+        } else {
+            appendMessage(format(fileName, lineNo, msg));
+        }
     }
 }
 
 void ProMessageHandler::fileMessage(int type, const QString &msg)
 {
-    Q_UNUSED(type)
-    if (m_verbose) {
-        if (m_exact)
-            emit writeMessage(msg, Core::MessageManager::NoModeSwitch);
-        else
-            emit writeMessage(m_prefix + msg, Core::MessageManager::NoModeSwitch);
-    }
+    // message(), warning() or error() calls in qmake files
+    if (!m_verbose)
+        return;
+    if (m_exact && type == QMakeHandler::ErrorMessage)
+        addTask(Task::Error, msg);
+    else if (m_exact && type == QMakeHandler::WarningMessage)
+        addTask(Task::Warning, msg);
+    else
+        appendMessage(msg);
 }
 
+void ProMessageHandler::appendMessage(const QString &msg)
+{
+    m_messages << (m_exact ? msg : m_prefix + msg);
+}
 
 ProFileReader::ProFileReader(QMakeGlobals *option, QMakeVfs *vfs)
     : QMakeParser(ProFileCacheManager::instance()->cache(), vfs, this)
@@ -91,6 +115,7 @@ ProFileReader::~ProFileReader()
 void ProFileReader::setCumulative(bool on)
 {
     ProMessageHandler::setVerbose(!on);
+    ProMessageHandler::setExact(!on);
     ProFileEvaluator::setCumulative(on);
 }
 
