@@ -58,7 +58,6 @@
 
 #include <QDesktopServices>
 #include <QDir>
-#include <QFileDialog>
 #include <QMessageBox>
 #include <QSortFilterProxyModel>
 #include <QTextBrowser>
@@ -103,7 +102,7 @@ public:
         return m_version;
     }
 
-    QVariant data(int column, int role) const
+    QVariant data(int column, int role) const final
     {
         if (!m_version)
             return TreeItem::data(column, role);
@@ -112,7 +111,7 @@ public:
             if (column == 0)
                 return m_version->displayName();
             if (column == 1)
-                return m_version->qmakeCommand().toUserOutput();
+                return m_version->qmakeFilePath().toUserOutput();
         }
 
         if (role == Qt::FontRole && m_changed) {
@@ -128,7 +127,7 @@ public:
             const QString row = "<tr><td>%1:</td><td>%2</td></tr>";
             return QString("<table>"
                          + row.arg(tr("Qt Version"), m_version->qtVersionString())
-                         + row.arg(tr("Location of qmake"), m_version->qmakeCommand().toUserOutput())
+                         + row.arg(tr("Location of qmake"), m_version->qmakeFilePath().toUserOutput())
                          + "</table>");
         }
 
@@ -261,11 +260,12 @@ QtOptionsPageWidget::QtOptionsPageWidget()
     m_ui.versionInfoWidget->setWidget(versionInfoWidget);
     m_ui.versionInfoWidget->setState(DetailsWidget::NoSummary);
 
-    m_autoItem = new StaticTreeItem(tr("Auto-detected"));
-    m_manualItem = new StaticTreeItem(tr("Manual"));
+    m_autoItem = new StaticTreeItem({ProjectExplorer::Constants::msgAutoDetected()},
+                                    {ProjectExplorer::Constants::msgAutoDetectedToolTip()});
+    m_manualItem = new StaticTreeItem(ProjectExplorer::Constants::msgManual());
 
     m_model = new TreeModel<TreeItem, TreeItem, QtVersionItem>();
-    m_model->setHeader({tr("Name"), tr("qmake Location")});
+    m_model->setHeader({tr("Name"), tr("qmake Path")});
     m_model->rootItem()->appendChild(m_autoItem);
     m_model->rootItem()->appendChild(m_manualItem);
 
@@ -601,25 +601,23 @@ QtOptionsPageWidget::~QtOptionsPageWidget()
 
 void QtOptionsPageWidget::addQtDir()
 {
-    FilePath qtVersion = FilePath::fromString(
-                QFileDialog::getOpenFileName(this,
-                                             tr("Select a qmake Executable"),
-                                             QString(),
-                                             BuildableHelperLibrary::filterForQmakeFileDialog(),
-                                             0,
-                                             QFileDialog::DontResolveSymlinks));
+    FilePath qtVersion = FileUtils::getOpenFilePath(this,
+                                                    tr("Select a qmake Executable"),
+                                                    {},
+                                                    BuildableHelperLibrary::filterForQmakeFileDialog(),
+                                                    0,
+                                                    QFileDialog::DontResolveSymlinks);
     if (qtVersion.isEmpty())
         return;
 
-    QFileInfo fi = qtVersion.toFileInfo();
     // should add all qt versions here ?
-    if (BuildableHelperLibrary::isQtChooser(fi))
-        qtVersion = FilePath::fromString(BuildableHelperLibrary::qtChooserToQmakePath(fi.symLinkTarget()));
+    if (BuildableHelperLibrary::isQtChooser(qtVersion))
+        qtVersion = BuildableHelperLibrary::qtChooserToQmakePath(qtVersion.symLinkTarget());
 
     auto checkAlreadyExists = [qtVersion](TreeItem *parent) {
         for (int i = 0; i < parent->childCount(); ++i) {
             auto item = static_cast<QtVersionItem *>(parent->childAt(i));
-            if (item->version()->qmakeCommand() == qtVersion) {
+            if (item->version()->qmakeFilePath() == qtVersion) {
                 return std::make_pair(true, item->version()->displayName());
             }
         }
@@ -672,14 +670,13 @@ void QtOptionsPageWidget::removeQtDir()
 void QtOptionsPageWidget::editPath()
 {
     BaseQtVersion *current = currentVersion();
-    QString dir = currentVersion()->qmakeCommand().toFileInfo().absolutePath();
-    FilePath qtVersion = FilePath::fromString(
-        QFileDialog::getOpenFileName(this,
-                                     tr("Select a qmake Executable"),
-                                     dir,
-                                     BuildableHelperLibrary::filterForQmakeFileDialog(),
-                                     nullptr,
-                                     QFileDialog::DontResolveSymlinks));
+    FilePath qtVersion =
+            FileUtils::getOpenFilePath(this,
+                                       tr("Select a qmake Executable"),
+                                       current->qmakeFilePath().absolutePath(),
+                                       BuildableHelperLibrary::filterForQmakeFileDialog(),
+                                       nullptr,
+                                       QFileDialog::DontResolveSymlinks);
     if (qtVersion.isEmpty())
         return;
     BaseQtVersion *version = QtVersionFactory::createQtVersionFromQMakePath(qtVersion);
@@ -770,7 +767,7 @@ void QtOptionsPageWidget::updateWidgets()
     BaseQtVersion *version = currentVersion();
     if (version) {
         m_versionUi.nameEdit->setText(version->unexpandedDisplayName());
-        m_versionUi.qmakePath->setText(version->qmakeCommand().toUserOutput());
+        m_versionUi.qmakePath->setText(version->qmakeFilePath().toUserOutput());
         m_configurationWidget = version->createConfigurationWidget();
         if (m_configurationWidget) {
             m_versionUi.formLayout->addRow(m_configurationWidget);
@@ -802,9 +799,9 @@ static QString qtVersionsFile(const QString &baseDir)
            + Core::Constants::IDE_ID + '/' + "qtversion.xml";
 }
 
-static Utils::optional<QString> currentlyLinkedQtDir(bool *hasInstallSettings)
+static Utils::optional<FilePath> currentlyLinkedQtDir(bool *hasInstallSettings)
 {
-    const QString installSettingsFilePath = settingsFile(Core::ICore::resourcePath());
+    const QString installSettingsFilePath = settingsFile(Core::ICore::resourcePath().toString());
     const bool installSettingsExist = QFile::exists(installSettingsFilePath);
     if (hasInstallSettings)
         *hasInstallSettings = installSettingsExist;
@@ -812,7 +809,7 @@ static Utils::optional<QString> currentlyLinkedQtDir(bool *hasInstallSettings)
         const QVariant value = QSettings(installSettingsFilePath, QSettings::IniFormat)
                                    .value(kInstallSettingsKey);
         if (value.isValid())
-            return value.toString();
+            return FilePath::fromVariant(value);
     }
     return {};
 }
@@ -820,18 +817,20 @@ static Utils::optional<QString> currentlyLinkedQtDir(bool *hasInstallSettings)
 static QString linkingPurposeText()
 {
     return QtOptionsPageWidget::tr(
-        "Linking with a Qt installation automatically registers Qt versions and kits.");
+        "Linking with a Qt installation automatically registers Qt versions and kits, and other "
+        "tools that were installed with that Qt installer, in this Qt Creator installation. Other "
+        "Qt Creator installations are not affected.");
 }
 
 static bool canLinkWithQt(QString *toolTip)
 {
     bool canLink = true;
     bool installSettingsExist;
-    const Utils::optional<QString> installSettingsValue = currentlyLinkedQtDir(
+    const Utils::optional<FilePath> installSettingsValue = currentlyLinkedQtDir(
         &installSettingsExist);
     QStringList tip;
     tip << linkingPurposeText();
-    if (!FilePath::fromString(Core::ICore::resourcePath()).isWritablePath()) {
+    if (!Core::ICore::resourcePath().isWritableDir()) {
         canLink = false;
         tip << QtOptionsPageWidget::tr("%1's resource directory is not writable.")
                    .arg(Core::Constants::IDE_DISPLAY_NAME);
@@ -844,10 +843,10 @@ static bool canLinkWithQt(QString *toolTip)
         tip << QtOptionsPageWidget::tr("%1 is part of a Qt installation.")
                    .arg(Core::Constants::IDE_DISPLAY_NAME);
     }
-    const QString link = installSettingsValue ? *installSettingsValue : QString();
+    const FilePath link = installSettingsValue ? *installSettingsValue : FilePath();
     if (!link.isEmpty())
         tip << QtOptionsPageWidget::tr("%1 is currently linked to \"%2\".")
-                   .arg(QString(Core::Constants::IDE_DISPLAY_NAME), link);
+                   .arg(QString(Core::Constants::IDE_DISPLAY_NAME), link.toUserOutput());
     if (toolTip)
         *toolTip = tip.join("\n\n");
     return canLink;
@@ -944,11 +943,11 @@ static bool validateQtInstallDir(FancyLineEdit *input, QString *errorString)
     return true;
 }
 
-static QString defaultQtInstallationPath()
+static FilePath defaultQtInstallationPath()
 {
     if (HostOsInfo::isWindowsHost())
-        return {"C:/Qt"};
-    return QDir::homePath() + "/Qt";
+        return FilePath::fromString({"C:/Qt"});
+    return FileUtils::homePath() / "Qt";
 }
 
 void QtOptionsPageWidget::linkWithQt()
@@ -960,7 +959,9 @@ void QtOptionsPageWidget::linkWithQt()
     dialog.setWindowTitle(title);
     auto layout = new QVBoxLayout;
     dialog.setLayout(layout);
-    layout->addWidget(new QLabel(linkingPurposeText()));
+    auto tipLabel = new QLabel(linkingPurposeText());
+    tipLabel->setWordWrap(true);
+    layout->addWidget(tipLabel);
     auto pathLayout = new QHBoxLayout;
     layout->addLayout(pathLayout);
     auto pathLabel = new QLabel(tr("Qt installation path:"));
@@ -979,9 +980,10 @@ void QtOptionsPageWidget::linkWithQt()
             return false;
         return validateQtInstallDir(input, errorString);
     });
-    const Utils::optional<QString> currentLink = currentlyLinkedQtDir(nullptr);
-    pathInput->setPath(currentLink ? *currentLink : defaultQtInstallationPath());
+    const Utils::optional<FilePath> currentLink = currentlyLinkedQtDir(nullptr);
+    pathInput->setFilePath(currentLink ? *currentLink : defaultQtInstallationPath());
     auto buttons = new QDialogButtonBox;
+    layout->addStretch(10);
     layout->addWidget(buttons);
     auto linkButton = buttons->addButton(tr("Link with Qt"), QDialogButtonBox::AcceptRole);
     connect(linkButton, &QPushButton::clicked, &dialog, &QDialog::accept);
@@ -991,7 +993,7 @@ void QtOptionsPageWidget::linkWithQt()
     unlinkButton->setEnabled(currentLink.has_value());
     connect(unlinkButton, &QPushButton::clicked, &dialog, [&dialog, &askForRestart] {
         bool removeSettingsFile = false;
-        const QString filePath = settingsFile(Core::ICore::resourcePath());
+        const QString filePath = settingsFile(Core::ICore::resourcePath().toString());
         {
             QSettings installSettings(filePath, QSettings::IniFormat);
             installSettings.remove(kInstallSettingsKey);
@@ -1010,7 +1012,7 @@ void QtOptionsPageWidget::linkWithQt()
     if (dialog.result() == QDialog::Accepted) {
         const Utils::optional<QString> settingsDir = settingsDirForQtDir(pathInput->rawPath());
         if (QTC_GUARD(settingsDir)) {
-            QSettings(settingsFile(Core::ICore::resourcePath()), QSettings::IniFormat)
+            QSettings(settingsFile(Core::ICore::resourcePath().toString()), QSettings::IniFormat)
                 .setValue(kInstallSettingsKey, *settingsDir);
             askForRestart = true;
         }
