@@ -35,8 +35,9 @@ class QTSUPPORT_EXPORT QtVersionManager : public QObject
     Q_OBJECT
     // for getUniqueId();
     friend class BaseQtVersion;
-    friend class Internal::BaseQtVersionPrivate;
+    friend class QtVersionFactory;
     friend class Internal::QtOptionsPageWidget;
+
 public:
     static QtVersionManager *instance();
     QtVersionManager();
@@ -62,17 +63,29 @@ public:
     static void addVersion(BaseQtVersion *version);
     static void removeVersion(BaseQtVersion *version);
 
+    // Call latest in extensionsInitialized of plugin depending on QtSupport
+    static void registerExampleSet(const QString &displayName,
+                                   const QString &manifestPath,
+                                   const QString &examplesPath);
+
 signals:
     // content of BaseQtVersion objects with qmake path might have changed
     void qtVersionsChanged(const QList<int> &addedIds, const QList<int> &removedIds, const QList<int> &changedIds);
     void qtVersionsLoaded();
 
 private:
+    enum class DocumentationSetting { HighestOnly, All, None };
+
+    static void updateDocumentation(const QList<BaseQtVersion *> &added,
+                                    const QList<BaseQtVersion *> &removed,
+                                    const QList<BaseQtVersion *> &allNew);
     void updateFromInstaller(bool emitSignal = true);
     void triggerQtVersionRestore();
 
     // Used by QtOptionsPage
     static void setNewQtVersions(const QList<BaseQtVersion *> &newVersions);
+    static void setDocumentationSetting(const DocumentationSetting &setting);
+    static DocumentationSetting documentationSetting();
     // Used by QtVersion
     static int getUniqueId();
 };
