@@ -52,7 +52,7 @@ Q_GLOBAL_STATIC(AreasOfInterest, welcomeScreenAreas)
 
 static inline QString fileNameForPath(const QString &path)
 {
-    return Utils::FileName::fromString(path).fileName();
+    return Utils::FilePath::fromString(path).fileName();
 }
 
 static QRect cropRectForAreaOfInterest(const QSize &imageSize, const QSize &cropSize, const QRect &areaOfInterest)
@@ -82,17 +82,20 @@ QImage ScreenshotCropper::croppedImage(const QImage &sourceImage, const QString
 {
     const QRect areaOfInterest = welcomeScreenAreas()->areas.value(fileNameForPath(filePath));
 
+    QImage result;
     if (areaOfInterest.isValid()) {
         const QRect cropRect = cropRectForAreaOfInterest(sourceImage.size(), cropSize, areaOfInterest);
         const QSize cropRectSize = cropRect.size();
-        const QImage result = sourceImage.copy(cropRect);
-        if (cropRectSize.width() > cropSize.width() || cropRectSize.height() > cropSize.height())
-            return result.scaled(cropSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
-        else
+        result = sourceImage.copy(cropRect);
+        if (cropRectSize.width() <= cropSize.width() && cropRectSize.height() <= cropSize.height())
             return result;
+    } else {
+        result = sourceImage;
     }
 
-    return sourceImage.scaled(cropSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+    if (result.format() != QImage::Format_ARGB32)
+        result = result.convertToFormat(QImage::Format_ARGB32);
+    return result.scaled(cropSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
 }
 
 static int areaAttribute(const QXmlStreamAttributes &attributes, const QString &name)
@@ -152,9 +155,7 @@ bool ScreenshotCropper::saveAreasOfInterest(const QString &areasXmlFile, QMap<QS
     writer.setAutoFormatting(true);
     writer.writeStartDocument();
     writer.writeStartElement(xmlTagAreas);
-    QMapIterator<QString, QRect> i(areas);
-    while (i.hasNext()) {
-        i.next();
+    for (auto i = areas.cbegin(), end = areas.cend(); i != end; ++i) {
         writer.writeStartElement(xmlTagArea);
         writer.writeAttribute(xmlAttributeImage, i.key());
         writer.writeAttribute(xmlAttributeX, QString::number(i.value().x()));
