@@ -33,9 +33,9 @@
 #include <ssh/sshconnectionmanager.h>
 
 #include <QDateTime>
-#include <QFileInfo>
 
 using namespace ProjectExplorer;
+using namespace Utils;
 using namespace QSsh;
 
 namespace RemoteLinux {
@@ -61,7 +61,7 @@ public:
     QString sysroot;
 };
 
-uint qHash(const DeployParameters &p) {
+auto qHash(const DeployParameters &p) {
     return qHash(qMakePair(qMakePair(p.file, p.host), p.sysroot));
 }
 
@@ -109,14 +109,14 @@ void DeploymentTimeInfo::saveDeploymentTimeStamp(const DeployableFile &deployabl
 {
     d->lastDeployed.insert(
                 d->parameters(deployableFile, kit),
-                { deployableFile.localFilePath().toFileInfo().lastModified(), remoteTimestamp });
+                { deployableFile.localFilePath().lastModified(), remoteTimestamp });
 }
 
 bool DeploymentTimeInfo::hasLocalFileChanged(const DeployableFile &deployableFile,
                                              const Kit *kit) const
 {
     const auto &lastDeployed = d->lastDeployed.value(d->parameters(deployableFile, kit));
-    const QDateTime lastModified = deployableFile.localFilePath().toFileInfo().lastModified();
+    const QDateTime lastModified = deployableFile.localFilePath().lastModified();
     return !lastDeployed.local.isValid() || lastModified != lastDeployed.local;
 }
 
@@ -180,7 +180,8 @@ void DeploymentTimeInfo::importDeployTimes(const QVariantMap &map)
                                sysrootList.size());
 
     for (int i = 0; i < elemCount; ++i) {
-        const DeployableFile df(fileList.at(i).toString(), remotePathList.at(i).toString());
+        const DeployableFile df(FilePath::fromVariant(fileList.at(i)),
+                                remotePathList.at(i).toString());
         const DeployParameters dp{df, hostList.at(i).toString(), sysrootList.at(i).toString()};
         d->lastDeployed.insert(dp, { localTimesList.at(i).toDateTime(),
                                      remoteTimesList.length() > i
