@@ -43,6 +43,7 @@
 
 using namespace ProjectExplorer;
 using namespace QSsh;
+using namespace Utils;
 
 namespace RemoteLinux {
 namespace Internal {
@@ -63,7 +64,7 @@ public:
         return file;
     }
 
-    bool incremental = false;
+    IncrementalDeployment incremental = IncrementalDeployment::NotSupported;
     bool ignoreMissingFiles = false;
     QHash<SshRemoteProcess *, DeployableFile> remoteProcs;
     QQueue<DeployableFile> filesToStat;
@@ -92,7 +93,7 @@ void GenericDirectUploadService::setDeployableFiles(const QList<DeployableFile>
     d->deployableFiles = deployableFiles;
 }
 
-void GenericDirectUploadService::setIncrementalDeployment(bool incremental)
+void GenericDirectUploadService::setIncrementalDeployment(IncrementalDeployment incremental)
 {
     d->incremental = incremental;
 }
@@ -198,7 +199,7 @@ void GenericDirectUploadService::stopDeployment()
 void GenericDirectUploadService::runStat(const DeployableFile &file)
 {
     // We'd like to use --format=%Y, but it's not supported by busybox.
-    const QString statCmd = "stat -t " + Utils::QtcProcess::quoteArgUnix(file.remoteFilePath());
+    const QString statCmd = "stat -t " + Utils::ProcessArgs::quoteArgUnix(file.remoteFilePath());
     SshRemoteProcess * const statProc = connection()->createRemoteProcess(statCmd).release();
     statProc->setParent(this);
     connect(statProc, &SshRemoteProcess::done, this,
@@ -232,17 +233,12 @@ QList<DeployableFile> GenericDirectUploadService::collectFilesToUpload(
         const DeployableFile &deployable) const
 {
     QList<DeployableFile> collected;
-    QFileInfo fileInfo = deployable.localFilePath().toFileInfo();
-    if (fileInfo.isDir()) {
-        const QStringList files = QDir(deployable.localFilePath().toString())
-            .entryList(QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot);
-        for (const QString &fileName : files) {
-            const QString localFilePath = deployable.localFilePath().toString()
-                + QLatin1Char('/') + fileName;
-            const QString remoteDir = deployable.remoteDirectory() + QLatin1Char('/')
-                + fileInfo.fileName();
+    FilePath localFile = deployable.localFilePath();
+    if (localFile.isDir()) {
+        const FilePaths files = localFile.dirEntries(QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot);
+        const QString remoteDir = deployable.remoteDirectory() + '/' + localFile.fileName();
+        for (const FilePath &localFilePath : files)
             collected.append(collectFilesToUpload(DeployableFile(localFilePath, remoteDir)));
-        }
     } else {
         collected << deployable;
     }
@@ -274,10 +270,13 @@ void GenericDirectUploadService::queryFiles()
     const QList<DeployableFile> &filesToCheck = d->state == PreChecking
             ? d->deployableFiles : d->filesToUpload;
     for (const DeployableFile &file : filesToCheck) {
-        if (d->state == PreChecking && (!d->incremental || hasLocalFileChanged(file))) {
+        if (d->state == PreChecking && (d->incremental != IncrementalDeployment::Enabled
+                                        || hasLocalFileChanged(file))) {
             d->filesToUpload.append(file);
             continue;
         }
+        if (d->incremental == IncrementalDeployment::NotSupported)
+            continue;
         if (d->remoteProcs.size() >= MaxConcurrentStatCalls)
             d->filesToStat << file;
         else
@@ -298,7 +297,7 @@ void GenericDirectUploadService::uploadFiles()
     }
     emit progressMessage(tr("%n file(s) need to be uploaded.", "", d->filesToUpload.size()));
     FilesToTransfer filesToTransfer;
-    for (const DeployableFile &f : d->filesToUpload) {
+    for (const DeployableFile &f : qAsConst(d->filesToUpload)) {
         if (!f.localFilePath().exists()) {
             const QString message = tr("Local file \"%1\" does not exist.")
                     .arg(f.localFilePath().toUserOutput());
@@ -337,11 +336,11 @@ void GenericDirectUploadService::chmod()
     QTC_ASSERT(d->state == PostProcessing, return);
     if (!Utils::HostOsInfo::isWindowsHost())
         return;
-    for (const DeployableFile &f : d->filesToUpload) {
+    for (const DeployableFile &f : qAsConst(d->filesToUpload)) {
         if (!f.isExecutable())
             continue;
         const QString command = QLatin1String("chmod a+x ")
-                + Utils::QtcProcess::quoteArgUnix(f.remoteFilePath());
+                + Utils::ProcessArgs::quoteArgUnix(f.remoteFilePath());
         SshRemoteProcess * const chmodProc
                 = connection()->createRemoteProcess(command).release();
         chmodProc->setParent(this);
