@@ -49,11 +49,12 @@ private:
     QString fullCommandLine(const ProjectExplorer::Runnable &) const override;
     qint64 processId() const override;
 
-    QStringList rcFilesToSource() const;
+    const QStringList rcFilesToSource() const;
 
     QStringList m_rcFilesToSource;
-    QByteArray m_processIdString;
-    qint64 m_processId;
+    QByteArray m_output;
+    qint64 m_processId = 0;
+    bool m_pidParsed = false;
 };
 
 } // namespace RemoteLinux
