@@ -31,9 +31,8 @@
 #include <ssh/sshconnection.h>
 #include <utils/theme/theme.h>
 
-#include <QFileDialog>
-
 using namespace ProjectExplorer;
+using namespace Utils;
 
 namespace RemoteLinux {
 namespace Internal {
@@ -50,9 +49,8 @@ using namespace Internal;
 PublicKeyDeploymentDialog *PublicKeyDeploymentDialog::createDialog(
         const IDevice::ConstPtr &deviceConfig, QWidget *parent)
 {
-    const QString &dir = QFileInfo(deviceConfig->sshParameters().privateKeyFile).path();
-    const QString publicKeyFileName = QFileDialog::getOpenFileName(parent
-            ? parent : Core::ICore::dialogParent(),
+    const FilePath dir = deviceConfig->sshParameters().privateKeyFile.parentDir();
+    const FilePath publicKeyFileName = FileUtils::getOpenFilePath(nullptr,
         tr("Choose Public Key File"), dir,
         tr("Public Key Files (*.pub);;All Files (*)"));
     if (publicKeyFileName.isEmpty())
@@ -61,7 +59,7 @@ PublicKeyDeploymentDialog *PublicKeyDeploymentDialog::createDialog(
 }
 
 PublicKeyDeploymentDialog::PublicKeyDeploymentDialog(const IDevice::ConstPtr &deviceConfig,
-        const QString &publicKeyFileName, QWidget *parent)
+        const FilePath &publicKeyFileName, QWidget *parent)
     : QProgressDialog(parent), d(new PublicKeyDeploymentDialogPrivate)
 {
     setAutoReset(false);
@@ -109,7 +107,9 @@ void PublicKeyDeploymentDialog::handleDeploymentFinished(const QString &errorMsg
         buttonText = errorMsg;
         textColor = Utils::creatorTheme()->color(Utils::Theme::TextColorError).name();
     }
-    setLabelText(QString::fromLatin1("<font color=\"%1\">%2</font>").arg(textColor, buttonText));
+    setLabelText(QString::fromLatin1("<font color=\"%1\">%2</font>")
+            .arg(textColor)
+            .arg(buttonText.replace("\n", "<br/>")));
     setCancelButtonText(tr("Close"));
 }
 
