@@ -28,6 +28,7 @@
 #include <ssh/sshremoteprocessrunner.h>
 #include <ssh/sshconnection.h>
 #include <utils/qtcassert.h>
+#include <utils/fileutils.h>
 
 using namespace RemoteLinux;
 using namespace ProjectExplorer;
@@ -118,7 +119,7 @@ void RemoteLinuxSignalOperation::interruptProcess(const QString &filePath)
 void RemoteLinuxSignalOperation::runnerProcessFinished()
 {
     m_errorMessage.clear();
-    if (m_runner->processExitStatus() != QSsh::SshRemoteProcess::NormalExit) {
+    if (m_runner->processExitStatus() != QProcess::NormalExit) {
         m_errorMessage = m_runner->processErrorString();
     } else if (m_runner->processExitCode() != 0) {
         m_errorMessage = tr("Exit code is %1. stderr:").arg(m_runner->processExitCode())
