@@ -46,6 +46,7 @@
 #include <utils/algorithm.h>
 #include <utils/parameteraction.h>
 #include <utils/qtcassert.h>
+#include <utils/threadutils.h>
 
 #include <QCoreApplication>
 #include <QAction>
@@ -59,6 +60,7 @@
 #include <QApplication>
 
 using namespace ProjectExplorer;
+using namespace Utils;
 
 namespace ResourceEditor {
 namespace Internal {
@@ -132,7 +134,7 @@ public:
     void copyPathContextMenu();
     void copyUrlContextMenu();
 
-    void updateContextActions();
+    void updateContextActions(Node *node);
 
     ResourceEditorW * currentEditor() const;
 
@@ -246,20 +248,35 @@ ResourceEditorPluginPrivate::ResourceEditorPluginPrivate(ResourceEditorPlugin *q
 
 void ResourceEditorPlugin::extensionsInitialized()
 {
-    ProjectTree::registerTreeManager([](FolderNode *folder) {
-        QList<FileNode *> toReplace;
-        folder->forEachNode([&toReplace](FileNode *fn) {
-            if (fn->fileType() == FileType::Resource)
-                toReplace.append(fn);
-        });
-
-        for (FileNode *file : toReplace) {
-            FolderNode *const pn = file->parentFolderNode();
-            QTC_ASSERT(pn, continue);
-            const Utils::FilePath path = file->filePath();
-            auto topLevel = std::make_unique<ResourceTopLevelNode>(path, pn->filePath());
-            topLevel->setIsGenerated(file->isGenerated());
-            pn->replaceSubtree(file, std::move(topLevel));
+    ProjectTree::registerTreeManager([](FolderNode *folder, ProjectTree::ConstructionPhase phase) {
+        switch (phase) {
+        case ProjectTree::AsyncPhase: {
+            QList<FileNode *> toReplace;
+            folder->forEachNode([&toReplace](FileNode *fn) {
+                if (fn->fileType() == FileType::Resource)
+                    toReplace.append(fn);
+            }, {}, [](const FolderNode *fn) {
+                return dynamic_cast<const ResourceTopLevelNode *>(fn) == nullptr;
+            });
+            for (FileNode *file : qAsConst(toReplace)) {
+                FolderNode *const pn = file->parentFolderNode();
+                QTC_ASSERT(pn, continue);
+                const Utils::FilePath path = file->filePath();
+                auto topLevel = std::make_unique<ResourceTopLevelNode>(path, pn->filePath());
+                topLevel->setEnabled(file->isEnabled());
+                topLevel->setIsGenerated(file->isGenerated());
+                pn->replaceSubtree(file, std::move(topLevel));
+            }
+            break;
+        }
+        case ProjectTree::FinalPhase: {
+            folder->forEachNode({}, [](FolderNode *fn) {
+                auto *topLevel = dynamic_cast<ResourceTopLevelNode *>(fn);
+                if (topLevel)
+                    topLevel->setupWatcherIfNeeded();
+            });
+            break;
+        }
         }
     });
 }
@@ -321,18 +338,18 @@ void ResourceEditorPluginPrivate::removeFileContextMenu()
 {
     auto rfn = dynamic_cast<ResourceTopLevelNode *>(ProjectTree::currentNode());
     QTC_ASSERT(rfn, return);
-    QString path = rfn->filePath().toString();
+    FilePath path = rfn->filePath();
     FolderNode *parent = rfn->parentFolderNode();
     QTC_ASSERT(parent, return);
-    if (parent->removeFiles(QStringList() << path) != RemovedFilesFromProject::Ok)
+    if (parent->removeFiles({path}) != RemovedFilesFromProject::Ok)
         QMessageBox::warning(Core::ICore::dialogParent(),
                              tr("File Removal Failed"),
-                             tr("Removing file %1 from the project failed.").arg(path));
+                             tr("Removing file %1 from the project failed.").arg(path.toUserOutput()));
 }
 
 void ResourceEditorPluginPrivate::openEditorContextMenu()
 {
-    Core::EditorManager::openEditor(ProjectTree::currentNode()->filePath().toString());
+    Core::EditorManager::openEditor(ProjectTree::currentNode()->filePath());
 }
 
 void ResourceEditorPluginPrivate::copyPathContextMenu()
@@ -367,9 +384,8 @@ void ResourceEditorPluginPrivate::renamePrefixContextMenu()
     node->renamePrefix(prefix, dialog.lang());
 }
 
-void ResourceEditorPluginPrivate::updateContextActions()
+void ResourceEditorPluginPrivate::updateContextActions(Node *node)
 {
-    const Node *node = ProjectTree::currentNode();
     const bool isResourceNode = dynamic_cast<const ResourceTopLevelNode *>(node);
     m_addPrefix->setEnabled(isResourceNode);
     m_addPrefix->setVisible(isResourceNode);
@@ -402,7 +418,7 @@ void ResourceEditorPluginPrivate::updateContextActions()
     m_removeNonExisting->setVisible(isResourceNode);
 
     if (isResourceNode)
-        Core::EditorManager::populateOpenWithMenu(m_openWithMenu, node->filePath().toString());
+        Core::EditorManager::populateOpenWithMenu(m_openWithMenu, node->filePath());
     else
         m_openWithMenu->clear();
     m_openWithMenu->menuAction()->setVisible(!m_openWithMenu->actions().isEmpty());
