@@ -41,13 +41,13 @@
 #include "warningitem.h"
 
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 
 #include <QAction>
 #include <QGuiApplication>
 #include <QClipboard>
 #include <QMenu>
 #include <QMimeData>
-#include <QThread>
 #include <QUndoStack>
 
 using namespace ScxmlEditor::PluginInterface;
@@ -201,7 +201,7 @@ void GraphicsScene::cut()
 void GraphicsScene::removeSelectedItems()
 {
     QVector<ScxmlTag*> tags = SceneUtils::findRemovedTags(m_baseItems);
-    if (tags.count() > 0) {
+    if (!tags.isEmpty()) {
         m_document->undoStack()->beginMacro(tr("Remove items"));
 
         // Then remove found tags
@@ -235,7 +235,7 @@ void GraphicsScene::copy()
     if (tags.isEmpty() && m_document->currentTag())
         tags << m_document->currentTag();
 
-    if (tags.count() > 0) {
+    if (!tags.isEmpty()) {
         auto mime = new QMimeData;
         QByteArray result = m_document->content(tags);
         mime->setText(QLatin1String(result));
@@ -286,7 +286,7 @@ void GraphicsScene::paste(const QPointF &targetPos)
     QString strMinPos = QLatin1String(mimeData->data("StateChartEditor/CopiedMinPos"));
     QPointF minPos(0, 0);
     if (!strMinPos.isEmpty()) {
-        QStringList coords = strMinPos.split(":", QString::SkipEmptyParts);
+        QStringList coords = strMinPos.split(":", Qt::SkipEmptyParts);
         if (coords.count() == 2)
             minPos = QPointF(coords[0].toDouble(), coords[1].toDouble());
     }
@@ -679,7 +679,7 @@ void GraphicsScene::keyPressEvent(QKeyEvent *event)
 {
     QGraphicsItem *focusItem = this->focusItem();
     if (!focusItem || focusItem->type() != TextType) {
-        if (event->key() == Qt::Key_Delete)
+        if (event->key() == Qt::Key_Delete || event->key() == Qt::Key_Backspace)
             removeSelectedItems();
     }
     QGraphicsScene::keyPressEvent(event);
@@ -831,7 +831,8 @@ void GraphicsScene::addWarningItem(WarningItem *item)
     if (!m_allWarnings.contains(item)) {
         m_allWarnings << item;
         if (!m_autoLayoutRunning && !m_initializing)
-            QMetaObject::invokeMethod(this, "warningVisibilityChanged", Qt::QueuedConnection, Q_ARG(int, 0));
+            QMetaObject::invokeMethod(this, [this] { warningVisibilityChanged(0); },
+                                      Qt::QueuedConnection);
     }
 }
 
@@ -840,7 +841,8 @@ void GraphicsScene::removeWarningItem(WarningItem *item)
     m_allWarnings.removeAll(item);
 
     if (!m_autoLayoutRunning && !m_initializing)
-        QMetaObject::invokeMethod(this, "warningVisibilityChanged", Qt::QueuedConnection, Q_ARG(int, 0));
+        QMetaObject::invokeMethod(this, [this] { warningVisibilityChanged(0); },
+                                  Qt::QueuedConnection);
 }
 
 void GraphicsScene::warningVisibilityChanged(int type, WarningItem *item)
