@@ -49,12 +49,15 @@ public:
     explicit ScxmlEditorDocument(Common::MainWidget *designWidget, QObject *parent = nullptr);
 
     // IDocument
-    OpenResult open(QString *errorString, const QString &fileName, const QString &realFileName) override;
-    bool save(QString *errorString, const QString &fileName, bool autoSave) override;
+    OpenResult open(QString *errorString,
+                    const Utils::FilePath &filePath,
+                    const Utils::FilePath &realFilePath) override;
+    bool save(QString *errorString, const Utils::FilePath &filePath, bool autoSave) override;
     bool shouldAutoSave() const override;
     bool isSaveAsAllowed() const override;
     bool isModified() const override;
     bool reload(QString *errorString, ReloadFlag flag, ChangeType type) override;
+    bool supportsCodec(const QTextCodec *codec) const override;
 
     // Internal
     Common::MainWidget *designWidget() const;
