@@ -28,20 +28,22 @@
 #include "subversionplugin.h"
 #include "subversionsettings.h"
 
+#include <coreplugin/editormanager/editormanager.h>
+
 #include <vcsbase/vcscommand.h>
 #include <vcsbase/vcsbaseconstants.h>
 #include <vcsbase/vcsbasediffeditorcontroller.h>
 #include <vcsbase/vcsbaseeditor.h>
 #include <vcsbase/vcsbaseeditorconfig.h>
 #include <vcsbase/vcsbaseplugin.h>
-#include <utils/qtcassert.h>
-#include <utils/synchronousprocess.h>
+
 #include <diffeditor/diffeditorcontroller.h>
 #include <diffeditor/diffutils.h>
-#include <coreplugin/editormanager/editormanager.h>
 
 #include <utils/algorithm.h>
 #include <utils/hostosinfo.h>
+#include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
 
 #include <QDir>
 #include <QFileInfo>
@@ -60,42 +62,42 @@ class SubversionLogConfig : public VcsBaseEditorConfig
 {
     Q_OBJECT
 public:
-    SubversionLogConfig(VcsBaseClientSettings &settings, QToolBar *toolBar) :
+    SubversionLogConfig(SubversionSettings &settings, QToolBar *toolBar) :
         VcsBaseEditorConfig(toolBar)
     {
-        mapSetting(addToggleButton(QLatin1String("--verbose"), tr("Verbose"),
+        mapSetting(addToggleButton("--verbose", tr("Verbose"),
                                    tr("Show files changed in each revision")),
-                   settings.boolPointer(SubversionSettings::logVerboseKey));
+                   &settings.logVerbose);
     }
 };
 
-SubversionClient::SubversionClient() : VcsBaseClient(new SubversionSettings)
+SubversionClient::SubversionClient(SubversionSettings *settings) : VcsBaseClient(settings)
 {
-    setLogConfigCreator([this](QToolBar *toolBar) {
-        return new SubversionLogConfig(settings(), toolBar);
+    setLogConfigCreator([settings](QToolBar *toolBar) {
+        return new SubversionLogConfig(*settings, toolBar);
     });
 }
 
-bool SubversionClient::doCommit(const QString &repositoryRoot,
+bool SubversionClient::doCommit(const FilePath &repositoryRoot,
                                 const QStringList &files,
                                 const QString &commitMessageFile,
                                 const QStringList &extraOptions) const
 {
     const QStringList svnExtraOptions =
             QStringList(extraOptions)
-            << SubversionClient::addAuthenticationOptions(settings())
+            << SubversionClient::addAuthenticationOptions(static_cast<SubversionSettings &>(settings()))
             << QLatin1String(Constants::NON_INTERACTIVE_OPTION)
             << QLatin1String("--encoding") << QLatin1String("UTF-8")
             << QLatin1String("--file") << commitMessageFile;
 
     QStringList args(vcsCommandString(CommitCommand));
-    SynchronousProcessResponse resp =
-            vcsSynchronousExec(repositoryRoot, args << svnExtraOptions << escapeFiles(files),
-                               VcsCommand::ShowStdOut | VcsCommand::NoFullySync);
-    return resp.result == SynchronousProcessResponse::Finished;
+    QtcProcess proc;
+    vcsSynchronousExec(proc, repositoryRoot, args << svnExtraOptions << escapeFiles(files),
+                       VcsCommand::ShowStdOut | VcsCommand::NoFullySync);
+    return proc.result() == QtcProcess::FinishedWithSuccess;
 }
 
-void SubversionClient::commit(const QString &repositoryRoot,
+void SubversionClient::commit(const FilePath &repositoryRoot,
                               const QStringList &files,
                               const QString &commitMessageFile,
                               const QStringList &extraOptions)
@@ -117,13 +119,13 @@ Id SubversionClient::vcsEditorKind(VcsCommandTag cmd) const
 }
 
 // Add authorization options to the command line arguments.
-QStringList SubversionClient::addAuthenticationOptions(const VcsBaseClientSettings &settings)
+QStringList SubversionClient::addAuthenticationOptions(const SubversionSettings &settings)
 {
-    if (!static_cast<const SubversionSettings &>(settings).hasAuthentication())
+    if (!settings.hasAuthentication())
         return QStringList();
 
-    const QString userName = settings.stringValue(SubversionSettings::userKey);
-    const QString password = settings.stringValue(SubversionSettings::passwordKey);
+    const QString userName = settings.userName.value();
+    const QString password = settings.password.value();
 
     if (userName.isEmpty())
         return QStringList();
@@ -138,7 +140,7 @@ QStringList SubversionClient::addAuthenticationOptions(const VcsBaseClientSettin
     return rc;
 }
 
-QString SubversionClient::synchronousTopic(const QString &repository)
+QString SubversionClient::synchronousTopic(const FilePath &repository) const
 {
     QStringList args;
 
@@ -149,12 +151,12 @@ QString SubversionClient::synchronousTopic(const QString &repository)
     else
         svnVersionBinary = svnVersionBinary.left(pos + 1);
     svnVersionBinary.append(HostOsInfo::withExecutableSuffix("svnversion"));
-    const SynchronousProcessResponse result
-            = vcsFullySynchronousExec(repository, {svnVersionBinary, args});
-    if (result.result != SynchronousProcessResponse::Finished)
+    QtcProcess proc;
+    vcsFullySynchronousExec(proc, repository, {FilePath::fromString(svnVersionBinary), args});
+    if (proc.result() != QtcProcess::FinishedWithSuccess)
         return QString();
 
-    return result.stdOut().trimmed();
+    return proc.stdOut().trimmed();
 }
 
 QString SubversionClient::escapeFile(const QString &file)
@@ -171,14 +173,17 @@ class SubversionDiffEditorController : public VcsBaseDiffEditorController
 {
     Q_OBJECT
 public:
-    SubversionDiffEditorController(IDocument *document,
-                                   const QString &workingDirectory);
+    SubversionDiffEditorController(IDocument *document, const QStringList &authOptions)
+        : VcsBaseDiffEditorController(document), m_authenticationOptions(authOptions)
+    {
+        forceContextLineCount(3); // SVN cannot change that when using internal diff
+        setReloader([this] { m_changeNumber ? requestDescription() : requestDiff(); });
+    }
 
     void setFilesList(const QStringList &filesList);
     void setChangeNumber(int changeNumber);
 
 protected:
-    void reload() override;
     void processCommandOutput(const QString &output) override;
 
 private:
@@ -186,19 +191,12 @@ private:
     void requestDiff();
 
     enum State { Idle, GettingDescription, GettingDiff };
-    State m_state;
+    State m_state = Idle;
     QStringList m_filesList;
     int m_changeNumber = 0;
+    QStringList m_authenticationOptions;
 };
 
-SubversionDiffEditorController::SubversionDiffEditorController(
-        IDocument *document,
-        const QString &workingDirectory)
-    : VcsBaseDiffEditorController(document, SubversionPlugin::instance()->client(), workingDirectory)
-    , m_state(Idle)
-{
-    forceContextLineCount(3); // SVN cannot change that when using internal diff
-}
 
 void SubversionDiffEditorController::setFilesList(const QStringList &filesList)
 {
@@ -221,7 +219,7 @@ void SubversionDiffEditorController::requestDescription()
     m_state = GettingDescription;
 
     QStringList args(QLatin1String("log"));
-    args << SubversionClient::addAuthenticationOptions(client()->settings());
+    args << m_authenticationOptions;
     args << QLatin1String("-r");
     args << QString::number(m_changeNumber);
     runCommand(QList<QStringList>() << args, VcsCommand::SshPasswordPrompt);
@@ -233,7 +231,7 @@ void SubversionDiffEditorController::requestDiff()
 
     QStringList args;
     args << QLatin1String("diff");
-    args << SubversionClient::addAuthenticationOptions(client()->settings());
+    args << m_authenticationOptions;
     args << QLatin1String("--internal-diff");
     if (ignoreWhitespace())
         args << QLatin1String("-x") << QLatin1String("-uw");
@@ -246,15 +244,6 @@ void SubversionDiffEditorController::requestDiff()
     runCommand(QList<QStringList>() << args, 0);
 }
 
-void SubversionDiffEditorController::reload()
-{
-    if (m_changeNumber) {
-        requestDescription();
-    } else {
-        requestDiff();
-    }
-}
-
 void SubversionDiffEditorController::processCommandOutput(const QString &output)
 {
     QTC_ASSERT(m_state != Idle, return);
@@ -271,19 +260,25 @@ void SubversionDiffEditorController::processCommandOutput(const QString &output)
 SubversionDiffEditorController *SubversionClient::findOrCreateDiffEditor(const QString &documentId,
                                                          const QString &source,
                                                          const QString &title,
-                                                         const QString &workingDirectory) const
+                                                         const FilePath &workingDirectory)
 {
+    auto &settings = static_cast<SubversionSettings &>(this->settings());
     IDocument *document = DiffEditorController::findOrCreateDocument(documentId, title);
     auto controller = qobject_cast<SubversionDiffEditorController *>(
                 DiffEditorController::controller(document));
-    if (!controller)
-        controller = new SubversionDiffEditorController(document, workingDirectory);
-    VcsBasePlugin::setSource(document, source);
+    if (!controller) {
+        controller = new SubversionDiffEditorController(document, addAuthenticationOptions(settings));
+        controller->setVcsBinary(settings.binaryPath.filePath());
+        controller->setVcsTimeoutS(settings.timeout.value());
+        controller->setProcessEnvironment(processEnvironment());
+        controller->setWorkingDirectory(workingDirectory);
+    }
+    VcsBase::setSource(document, source);
     EditorManager::activateEditorForDocument(document);
     return controller;
 }
 
-void SubversionClient::diff(const QString &workingDirectory, const QStringList &files, const QStringList &extraOptions)
+void SubversionClient::diff(const FilePath &workingDirectory, const QStringList &files, const QStringList &extraOptions)
 {
     Q_UNUSED(extraOptions)
 
@@ -292,21 +287,21 @@ void SubversionClient::diff(const QString &workingDirectory, const QStringList &
             + QLatin1String(".Diff.") + VcsBaseEditor::getTitleId(workingDirectory, files);
     const QString title = vcsEditorTitle(vcsCmdString, documentId);
 
-    SubversionDiffEditorController *controller = findOrCreateDiffEditor(documentId, workingDirectory, title,
-                                                        workingDirectory);
+    SubversionDiffEditorController *controller =
+            findOrCreateDiffEditor(documentId, workingDirectory.toString(), title, workingDirectory);
     controller->setFilesList(files);
     controller->requestReload();
 }
 
-void SubversionClient::log(const QString &workingDir,
+void SubversionClient::log(const FilePath &workingDir,
                            const QStringList &files,
                            const QStringList &extraOptions,
                            bool enableAnnotationContextMenu)
 {
-    const auto logCount = settings().intValue(SubversionSettings::logCountKey);
-    QStringList svnExtraOptions =
-            QStringList(extraOptions)
-            << SubversionClient::addAuthenticationOptions(settings());
+    auto &settings = static_cast<SubversionSettings &>(this->settings());
+    const int logCount = settings.logCount.value();
+    QStringList svnExtraOptions = extraOptions;
+    svnExtraOptions.append(SubversionClient::addAuthenticationOptions(settings));
     if (logCount > 0)
         svnExtraOptions << QLatin1String("-l") << QString::number(logCount);
 
@@ -315,37 +310,20 @@ void SubversionClient::log(const QString &workingDir,
     VcsBaseClient::log(workingDir, escapeFiles(files), svnExtraOptions, enableAnnotationContextMenu);
 }
 
-void SubversionClient::describe(const QString &workingDirectory, int changeNumber, const QString &title)
+void SubversionClient::describe(const FilePath &workingDirectory, int changeNumber, const QString &title)
 {
     const QString documentId = QLatin1String(Constants::SUBVERSION_PLUGIN)
             + QLatin1String(".Describe.") + VcsBaseEditor::editorTag(DiffOutput,
-                                                        workingDirectory,
+                                                        workingDirectory.toString(),
                                                         QStringList(),
                                                         QString::number(changeNumber));
 
-    SubversionDiffEditorController *controller = findOrCreateDiffEditor(documentId, workingDirectory, title, workingDirectory);
+    SubversionDiffEditorController *controller =
+            findOrCreateDiffEditor(documentId, workingDirectory.toString(), title, workingDirectory);
     controller->setChangeNumber(changeNumber);
     controller->requestReload();
 }
 
-QString SubversionClient::findTopLevelForFile(const QFileInfo &file) const
-{
-    Q_UNUSED(file)
-    return QString();
-}
-
-QStringList SubversionClient::revisionSpec(const QString &revision) const
-{
-    Q_UNUSED(revision)
-    return QStringList();
-}
-
-VcsBaseClient::StatusItem SubversionClient::parseStatusLine(const QString &line) const
-{
-    Q_UNUSED(line)
-    return VcsBaseClient::StatusItem();
-}
-
 } // namespace Internal
 } // namespace Subversion
 
