@@ -27,18 +27,19 @@
 #include "textdocument.h"
 
 #include <aggregation/aggregate.h>
-#include <coreplugin/icore.h>
-#include <coreplugin/progressmanager/progressmanager.h>
-#include <coreplugin/progressmanager/futureprogress.h>
 #include <coreplugin/dialogs/readonlyfilesdialog.h>
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/find/ifindsupport.h>
-#include <texteditor/texteditor.h>
+#include <coreplugin/icore.h>
+#include <coreplugin/progressmanager/futureprogress.h>
+#include <coreplugin/progressmanager/progressmanager.h>
 #include <texteditor/refactoringchanges.h>
+#include <texteditor/texteditor.h>
 #include <utils/algorithm.h>
 #include <utils/fadingindicator.h>
 #include <utils/filesearch.h>
+#include <utils/futuresynchronizer.h>
 #include <utils/qtcassert.h>
 #include <utils/stylehelper.h>
 
@@ -111,8 +112,11 @@ public:
 class BaseFileFindPrivate
 {
 public:
+    BaseFileFindPrivate() { m_futureSynchronizer.setCancelOnWait(true); }
+
     QPointer<IFindSupport> m_currentFindSupport;
 
+    Utils::FutureSynchronizer m_futureSynchronizer;
     QLabel *m_resultLabel = nullptr;
     // models in native path format
     QStringListModel m_filterStrings;
@@ -227,6 +231,17 @@ void BaseFileFind::setCurrentSearchEngine(int index)
     emit currentSearchEngineChanged();
 }
 
+static QString displayText(const QString &line)
+{
+    QString result = line;
+    auto end = result.end();
+    for (auto it = result.begin(); it != end; ++it) {
+        if (!it->isSpace() && !it->isPrint())
+            *it = QChar('?');
+    }
+    return result;
+}
+
 static void displayResult(QFutureWatcher<FileSearchResultList> *watcher,
                           SearchResult *search, int index)
 {
@@ -234,14 +249,11 @@ static void displayResult(QFutureWatcher<FileSearchResultList> *watcher,
     QList<SearchResultItem> items;
     for (const FileSearchResult &result : results) {
         SearchResultItem item;
-        item.path = QStringList(QDir::toNativeSeparators(result.fileName));
-        item.mainRange.begin.line = result.lineNumber;
-        item.mainRange.begin.column = result.matchStart;
-        item.mainRange.end = item.mainRange.begin;
-        item.mainRange.end.column += result.matchLength;
-        item.text = result.matchingLine;
-        item.useTextEditorFont = true;
-        item.userData = result.regexpCapturedTexts;
+        item.setFilePath(Utils::FilePath::fromString(result.fileName));
+        item.setMainRange(result.lineNumber, result.matchStart, result.matchLength);
+        item.setLineText(displayText(result.matchingLine));
+        item.setUseTextEditorFont(true);
+        item.setUserData(result.regexpCapturedTexts);
         items << item;
     }
     search->addResults(items, SearchResult::AddOrdered);
@@ -311,8 +323,10 @@ void BaseFileFind::runSearch(SearchResult *search)
     connect(watcher, &QFutureWatcherBase::finished, search, [watcher, search]() {
         search->finishSearch(watcher->isCanceled());
     });
-    watcher->setFuture(executeSearch(parameters));
-    FutureProgress *progress = ProgressManager::addTask(QFuture<void>(watcher->future()),
+    QFuture<FileSearchResultList> future = executeSearch(parameters);
+    watcher->setFuture(future);
+    d->m_futureSynchronizer.addFuture(future);
+    FutureProgress *progress = ProgressManager::addTask(future,
                                                         tr("Searching"),
                                                         Constants::TASK_SEARCH);
     connect(search, &SearchResult::countChanged, progress, [progress](int c) {
@@ -343,7 +357,7 @@ void BaseFileFind::doReplace(const QString &text,
                              const QList<SearchResultItem> &items,
                              bool preserveCase)
 {
-    const QStringList files = replaceAll(text, items, preserveCase);
+    const FilePaths files = replaceAll(text, items, preserveCase);
     if (!files.isEmpty()) {
         Utils::FadingIndicator::showText(ICore::dialogParent(),
             tr("%n occurrences replaced.", nullptr, items.size()),
@@ -482,25 +496,24 @@ void BaseFileFind::recheckEnabled(SearchResult *search)
     search->setSearchAgainEnabled(isEnabled());
 }
 
-QStringList BaseFileFind::replaceAll(const QString &text,
-                                     const QList<SearchResultItem> &items,
-                                     bool preserveCase)
+FilePaths BaseFileFind::replaceAll(const QString &text,
+                                   const QList<SearchResultItem> &items,
+                                   bool preserveCase)
 {
     if (items.isEmpty())
-        return QStringList();
+        return {};
 
     RefactoringChanges refactoring;
 
-    QHash<QString, QList<SearchResultItem> > changes;
+    QHash<FilePath, QList<SearchResultItem> > changes;
     for (const SearchResultItem &item : items)
-        changes[QDir::fromNativeSeparators(item.path.first())].append(item);
+        changes[FilePath::fromUserInput(item.path().first())].append(item);
 
     // Checking for files without write permissions
     QSet<FilePath> roFiles;
     for (auto it = changes.cbegin(), end = changes.cend(); it != end; ++it) {
-        const QFileInfo fileInfo(it.key());
-        if (!fileInfo.isWritable())
-            roFiles.insert(FilePath::fromString(it.key()));
+        if (!it.key().isWritableFile())
+            roFiles.insert(it.key());
     }
 
     // Query the user for permissions
@@ -508,39 +521,39 @@ QStringList BaseFileFind::replaceAll(const QString &text,
         ReadOnlyFilesDialog roDialog(Utils::toList(roFiles), ICore::dialogParent());
         roDialog.setShowFailWarning(true, tr("Aborting replace."));
         if (roDialog.exec() == ReadOnlyFilesDialog::RO_Cancel)
-            return QStringList();
+            return {};
     }
 
     for (auto it = changes.cbegin(), end = changes.cend(); it != end; ++it) {
-        const QString fileName = it.key();
+        const FilePath filePath = it.key();
         const QList<SearchResultItem> changeItems = it.value();
 
         ChangeSet changeSet;
-        RefactoringFilePtr file = refactoring.file(fileName);
+        RefactoringFilePtr file = refactoring.file(filePath);
         QSet<QPair<int, int> > processed;
         for (const SearchResultItem &item : changeItems) {
-            const QPair<int, int> &p = qMakePair(item.mainRange.begin.line,
-                                                 item.mainRange.begin.column);
+            const QPair<int, int> &p = qMakePair(item.mainRange().begin.line,
+                                                 item.mainRange().begin.column);
             if (processed.contains(p))
                 continue;
             processed.insert(p);
 
             QString replacement;
-            if (item.userData.canConvert<QStringList>() && !item.userData.toStringList().isEmpty()) {
-                replacement = Utils::expandRegExpReplacement(text, item.userData.toStringList());
+            if (item.userData().canConvert<QStringList>() && !item.userData().toStringList().isEmpty()) {
+                replacement = Utils::expandRegExpReplacement(text, item.userData().toStringList());
             } else if (preserveCase) {
-                const QString originalText = (item.mainRange.length(item.text) == 0)
-                                                 ? item.text
-                                                 : item.mainRange.mid(item.text);
+                const QString originalText = (item.mainRange().length(item.lineText()) == 0)
+                                                 ? item.lineText()
+                                                 : item.mainRange().mid(item.lineText());
                 replacement = Utils::matchCaseReplacement(originalText, text);
             } else {
                 replacement = text;
             }
 
-            const int start = file->position(item.mainRange.begin.line,
-                                             item.mainRange.begin.column + 1);
-            const int end = file->position(item.mainRange.end.line,
-                                           item.mainRange.end.column + 1);
+            const int start = file->position(item.mainRange().begin.line,
+                                             item.mainRange().begin.column + 1);
+            const int end = file->position(item.mainRange().end.line,
+                                           item.mainRange().end.column + 1);
             changeSet.replace(start, end, replacement);
         }
         file->setChangeSet(changeSet);
