@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -33,6 +28,9 @@
 
 #include "tabsettingswidget.h"
 
+#include <coreplugin/coreconstants.h>
+#include <coreplugin/icore.h>
+
 #include <texteditor/typingsettings.h>
 #include <texteditor/storagesettings.h>
 #include <texteditor/behaviorsettings.h>
@@ -64,18 +62,19 @@ BehaviorSettingsWidget::BehaviorSettingsWidget(QWidget *parent)
     QList<int> mibs = QTextCodec::availableMibs();
     Utils::sort(mibs);
     QList<int>::iterator firstNonNegative =
-        std::find_if(mibs.begin(), mibs.end(), std::bind2nd(std::greater_equal<int>(), 0));
+        std::find_if(mibs.begin(), mibs.end(), [](int n) { return n >=0; });
     if (firstNonNegative != mibs.end())
         std::rotate(mibs.begin(), firstNonNegative, mibs.end());
     foreach (int mib, mibs) {
-        QTextCodec *codec = QTextCodec::codecForMib(mib);
-        QString compoundName = QLatin1String(codec->name());
-        foreach (const QByteArray &alias, codec->aliases()) {
-            compoundName += QLatin1String(" / ");
-            compoundName += QString::fromLatin1(alias);
+        if (QTextCodec *codec = QTextCodec::codecForMib(mib)) {
+            QString compoundName = QLatin1String(codec->name());
+            foreach (const QByteArray &alias, codec->aliases()) {
+                compoundName += QLatin1String(" / ");
+                compoundName += QString::fromLatin1(alias);
+            }
+            d->m_ui.encodingBox->addItem(compoundName);
+            d->m_codecs.append(codec);
         }
-        d->m_ui.encodingBox->addItem(compoundName);
-        d->m_codecs.append(codec);
     }
 
     // Qt5 doesn't list the system locale (QTBUG-34283), so add it manually
@@ -85,36 +84,43 @@ BehaviorSettingsWidget::BehaviorSettingsWidget(QWidget *parent)
         d->m_codecs.prepend(QTextCodec::codecForLocale());
     }
 
-    connect(d->m_ui.autoIndent, SIGNAL(toggled(bool)),
-            this, SLOT(slotTypingSettingsChanged()));
-    connect(d->m_ui.smartBackspaceBehavior, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(slotTypingSettingsChanged()));
-    connect(d->m_ui.tabKeyBehavior, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(slotTypingSettingsChanged()));
-    connect(d->m_ui.cleanWhitespace, SIGNAL(clicked(bool)),
-            this, SLOT(slotStorageSettingsChanged()));
-    connect(d->m_ui.inEntireDocument, SIGNAL(clicked(bool)),
-            this, SLOT(slotStorageSettingsChanged()));
-    connect(d->m_ui.addFinalNewLine, SIGNAL(clicked(bool)),
-            this, SLOT(slotStorageSettingsChanged()));
-    connect(d->m_ui.cleanIndentation, SIGNAL(clicked(bool)),
-            this, SLOT(slotStorageSettingsChanged()));
-    connect(d->m_ui.mouseHiding, SIGNAL(clicked()),
-            this, SLOT(slotBehaviorSettingsChanged()));
-    connect(d->m_ui.mouseNavigation, SIGNAL(clicked()),
-            this, SLOT(slotBehaviorSettingsChanged()));
-    connect(d->m_ui.scrollWheelZooming, SIGNAL(clicked(bool)),
-            this, SLOT(slotBehaviorSettingsChanged()));
-    connect(d->m_ui.camelCaseNavigation, SIGNAL(clicked()),
-            this, SLOT(slotBehaviorSettingsChanged()));
-    connect(d->m_ui.utf8BomBox, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(slotExtraEncodingChanged()));
-    connect(d->m_ui.encodingBox, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(slotEncodingBoxChanged(int)));
-    connect(d->m_ui.constrainTooltipsBox, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(slotBehaviorSettingsChanged()));
-    connect(d->m_ui.keyboardTooltips, SIGNAL(clicked()),
-            this, SLOT(slotBehaviorSettingsChanged()));
+    d->m_ui.defaultLineEndings->addItems(ExtraEncodingSettings::lineTerminationModeNames());
+
+    auto currentIndexChanged = QOverload<int>::of(&QComboBox::currentIndexChanged);
+    connect(d->m_ui.autoIndent, &QAbstractButton::toggled,
+            this, &BehaviorSettingsWidget::slotTypingSettingsChanged);
+    connect(d->m_ui.smartBackspaceBehavior, currentIndexChanged,
+            this, &BehaviorSettingsWidget::slotTypingSettingsChanged);
+    connect(d->m_ui.tabKeyBehavior, currentIndexChanged,
+            this, &BehaviorSettingsWidget::slotTypingSettingsChanged);
+    connect(d->m_ui.cleanWhitespace, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotStorageSettingsChanged);
+    connect(d->m_ui.inEntireDocument, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotStorageSettingsChanged);
+    connect(d->m_ui.addFinalNewLine, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotStorageSettingsChanged);
+    connect(d->m_ui.cleanIndentation, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotStorageSettingsChanged);
+    connect(d->m_ui.skipTrailingWhitespace, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotStorageSettingsChanged);
+    connect(d->m_ui.mouseHiding, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotBehaviorSettingsChanged);
+    connect(d->m_ui.mouseNavigation, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotBehaviorSettingsChanged);
+    connect(d->m_ui.scrollWheelZooming, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotBehaviorSettingsChanged);
+    connect(d->m_ui.camelCaseNavigation, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotBehaviorSettingsChanged);
+    connect(d->m_ui.utf8BomBox, currentIndexChanged,
+            this, &BehaviorSettingsWidget::slotExtraEncodingChanged);
+    connect(d->m_ui.encodingBox, currentIndexChanged,
+            this, &BehaviorSettingsWidget::slotEncodingBoxChanged);
+    connect(d->m_ui.constrainTooltipsBox, currentIndexChanged,
+            this, &BehaviorSettingsWidget::slotBehaviorSettingsChanged);
+    connect(d->m_ui.keyboardTooltips, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotBehaviorSettingsChanged);
+    connect(d->m_ui.smartSelectionChanging, &QAbstractButton::clicked,
+            this, &BehaviorSettingsWidget::slotBehaviorSettingsChanged);
 }
 
 BehaviorSettingsWidget::~BehaviorSettingsWidget()
@@ -133,17 +139,30 @@ void BehaviorSettingsWidget::setActive(bool active)
 
 void BehaviorSettingsWidget::setAssignedCodec(QTextCodec *codec)
 {
+    const QString codecName = Core::ICore::settings()->value(
+                Core::Constants::SETTINGS_DEFAULTTEXTENCODING).toString();
+
+    int rememberedSystemPosition = -1;
     for (int i = 0; i < d->m_codecs.size(); ++i) {
         if (codec == d->m_codecs.at(i)) {
-            d->m_ui.encodingBox->setCurrentIndex(i);
-            break;
+            if (d->m_ui.encodingBox->itemText(i) == codecName) {
+                d->m_ui.encodingBox->setCurrentIndex(i);
+                return;
+            } else { // we've got System matching encoding - but have explicitly set the codec
+                rememberedSystemPosition = i;
+            }
         }
     }
+    if (rememberedSystemPosition != -1)
+        d->m_ui.encodingBox->setCurrentIndex(rememberedSystemPosition);
 }
 
-QTextCodec *BehaviorSettingsWidget::assignedCodec() const
+QByteArray BehaviorSettingsWidget::assignedCodecName() const
 {
-    return d->m_codecs.at(d->m_ui.encodingBox->currentIndex());
+    return d->m_ui.encodingBox->currentIndex() == 0
+            ? QByteArray("System")   // we prepend System to the available codecs
+            : d->m_codecs.at(d->m_ui.encodingBox->currentIndex())->name();
+
 }
 
 void BehaviorSettingsWidget::setCodeStyle(ICodeStylePreferences *preferences)
@@ -156,6 +175,8 @@ void BehaviorSettingsWidget::setAssignedTypingSettings(const TypingSettings &typ
     d->m_ui.autoIndent->setChecked(typingSettings.m_autoIndent);
     d->m_ui.smartBackspaceBehavior->setCurrentIndex(typingSettings.m_smartBackspaceBehavior);
     d->m_ui.tabKeyBehavior->setCurrentIndex(typingSettings.m_tabKeyBehavior);
+
+    d->m_ui.preferSingleLineComments->setChecked(typingSettings.m_preferSingleLineComments);
 }
 
 void BehaviorSettingsWidget::assignedTypingSettings(TypingSettings *typingSettings) const
@@ -165,6 +186,8 @@ void BehaviorSettingsWidget::assignedTypingSettings(TypingSettings *typingSettin
         (TypingSettings::SmartBackspaceBehavior)(d->m_ui.smartBackspaceBehavior->currentIndex());
     typingSettings->m_tabKeyBehavior =
         (TypingSettings::TabKeyBehavior)(d->m_ui.tabKeyBehavior->currentIndex());
+
+    typingSettings->m_preferSingleLineComments = d->m_ui.preferSingleLineComments->isChecked();
 }
 
 void BehaviorSettingsWidget::setAssignedStorageSettings(const StorageSettings &storageSettings)
@@ -173,6 +196,9 @@ void BehaviorSettingsWidget::setAssignedStorageSettings(const StorageSettings &s
     d->m_ui.inEntireDocument->setChecked(storageSettings.m_inEntireDocument);
     d->m_ui.cleanIndentation->setChecked(storageSettings.m_cleanIndentation);
     d->m_ui.addFinalNewLine->setChecked(storageSettings.m_addFinalNewLine);
+    d->m_ui.skipTrailingWhitespace->setChecked(storageSettings.m_skipTrailingWhitespace);
+    d->m_ui.ignoreFileTypes->setText(storageSettings.m_ignoreFileTypes);
+    d->m_ui.ignoreFileTypes->setEnabled(d->m_ui.skipTrailingWhitespace->isChecked());
 }
 
 void BehaviorSettingsWidget::assignedStorageSettings(StorageSettings *storageSettings) const
@@ -181,6 +207,8 @@ void BehaviorSettingsWidget::assignedStorageSettings(StorageSettings *storageSet
     storageSettings->m_inEntireDocument = d->m_ui.inEntireDocument->isChecked();
     storageSettings->m_cleanIndentation = d->m_ui.cleanIndentation->isChecked();
     storageSettings->m_addFinalNewLine = d->m_ui.addFinalNewLine->isChecked();
+    storageSettings->m_skipTrailingWhitespace = d->m_ui.skipTrailingWhitespace->isChecked();
+    storageSettings->m_ignoreFileTypes = d->m_ui.ignoreFileTypes->text();
 }
 
 void BehaviorSettingsWidget::updateConstrainTooltipsBoxTooltip() const
@@ -202,6 +230,7 @@ void BehaviorSettingsWidget::setAssignedBehaviorSettings(const BehaviorSettings
     d->m_ui.constrainTooltipsBox->setCurrentIndex(behaviorSettings.m_constrainHoverTooltips ? 1 : 0);
     d->m_ui.camelCaseNavigation->setChecked(behaviorSettings.m_camelCaseNavigation);
     d->m_ui.keyboardTooltips->setChecked(behaviorSettings.m_keyboardTooltips);
+    d->m_ui.smartSelectionChanging->setChecked(behaviorSettings.m_smartSelectionChanging);
     updateConstrainTooltipsBoxTooltip();
 }
 
@@ -213,6 +242,7 @@ void BehaviorSettingsWidget::assignedBehaviorSettings(BehaviorSettings *behavior
     behaviorSettings->m_constrainHoverTooltips = (d->m_ui.constrainTooltipsBox->currentIndex() == 1);
     behaviorSettings->m_camelCaseNavigation = d->m_ui.camelCaseNavigation->isChecked();
     behaviorSettings->m_keyboardTooltips = d->m_ui.keyboardTooltips->isChecked();
+    behaviorSettings->m_smartSelectionChanging = d->m_ui.smartSelectionChanging->isChecked();
 }
 
 void BehaviorSettingsWidget::setAssignedExtraEncodingSettings(
@@ -228,6 +258,16 @@ void BehaviorSettingsWidget::assignedExtraEncodingSettings(
         (ExtraEncodingSettings::Utf8BomSetting)d->m_ui.utf8BomBox->currentIndex();
 }
 
+void BehaviorSettingsWidget::setAssignedLineEnding(int lineEnding)
+{
+    d->m_ui.defaultLineEndings->setCurrentIndex(lineEnding);
+}
+
+int BehaviorSettingsWidget::assignedLineEnding() const
+{
+    return d->m_ui.defaultLineEndings->currentIndex();
+}
+
 TabSettingsWidget *BehaviorSettingsWidget::tabSettingsWidget() const
 {
     return d->m_ui.tabPreferencesWidget->tabSettingsWidget();
@@ -244,6 +284,10 @@ void BehaviorSettingsWidget::slotStorageSettingsChanged()
 {
     StorageSettings settings;
     assignedStorageSettings(&settings);
+
+    bool ignoreFileTypesEnabled = d->m_ui.cleanWhitespace->isChecked() && d->m_ui.skipTrailingWhitespace->isChecked();
+    d->m_ui.ignoreFileTypes->setEnabled(ignoreFileTypesEnabled);
+
     emit storageSettingsChanged(settings);
 }
 
