@@ -41,6 +41,7 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <extensionsystem/pluginmanager.h>
 #include <utils/algorithm.h>
+#include <utils/executeondestruction.h>
 #include <utils/qtcassert.h>
 
 #include <QKeyEvent>
@@ -125,7 +126,7 @@ CodeAssistantPrivate::CodeAssistantPrivate(CodeAssistant *assistant)
     connect(&m_automaticProposalTimer, &QTimer::timeout,
             this, &CodeAssistantPrivate::automaticProposalTimeout);
 
-    m_settings = TextEditorSettings::completionSettings();
+    updateFromCompletionSettings(TextEditorSettings::completionSettings());
     connect(TextEditorSettings::instance(), &TextEditorSettings::completionSettingsChanged,
             this, &CodeAssistantPrivate::updateFromCompletionSettings);
 
@@ -157,17 +158,16 @@ void CodeAssistantPrivate::invoke(AssistKind kind, IAssistProvider *provider)
                         m_proposal->basePosition(),
                         m_editorWidget->position() - m_proposal->basePosition()));
     } else {
-        destroyContext();
         requestProposal(ExplicitlyInvoked, kind, provider);
     }
 }
 
 bool CodeAssistantPrivate::requestActivationCharProposal()
 {
+    if (m_editorWidget->multiTextCursor().hasMultipleCursors())
+        return false;
     if (m_assistKind == Completion && m_settings.m_completionTrigger != ManualCompletion) {
         if (CompletionAssistProvider *provider = identifyActivationSequence()) {
-            if (isWaitingForProposal())
-                cancelCurrentRequest();
             requestProposal(ActivationCharacter, Completion, provider);
             return true;
         }
@@ -194,10 +194,10 @@ void CodeAssistantPrivate::requestProposal(AssistReason reason,
                                            AssistKind kind,
                                            IAssistProvider *provider)
 {
-    QTC_ASSERT(!isWaitingForProposal(), return);
-
-    if (m_editorWidget->hasBlockSelection())
-        return; // TODO
+    // make sure to cleanup old proposals if we cannot find a new assistant
+    Utils::ExecuteOnDestruction earlyReturnContextClear([this]() { destroyContext(); });
+    if (isWaitingForProposal())
+        cancelCurrentRequest();
 
     if (!provider) {
         if (kind == Completion)
@@ -215,9 +215,12 @@ void CodeAssistantPrivate::requestProposal(AssistReason reason,
     if (!assistInterface)
         return;
 
+    // We got an assist provider and interface so no need to reset the current context anymore
+    earlyReturnContextClear.reset({});
+
     m_assistKind = kind;
     m_requestProvider = provider;
-    IAssistProcessor *processor = provider->createProcessor();
+    IAssistProcessor *processor = provider->createProcessor(assistInterface);
 
     switch (provider->runType()) {
     case IAssistProvider::Synchronous: {
@@ -252,9 +255,14 @@ void CodeAssistantPrivate::requestProposal(AssistReason reason,
         break;
     }
     case IAssistProvider::Asynchronous: {
-        processor->setAsyncCompletionAvailableHandler([this, reason, processor](IAssistProposal *newProposal) {
-            // do not delete this processor directly since this function is called from within the processor
-            QTimer::singleShot(0, [processor]() { delete processor; });
+        processor->setAsyncCompletionAvailableHandler([this, reason, processor](
+                IAssistProposal *newProposal) {
+            if (!processor->running()) {
+                // do not delete this processor directly since this function is called from within the processor
+                QMetaObject::invokeMethod(QCoreApplication::instance(), [processor]() {
+                    delete processor;
+                }, Qt::QueuedConnection);
+            }
             if (processor != m_asyncProcessor)
                 return;
             invalidateCurrentRequestData();
@@ -264,7 +272,10 @@ void CodeAssistantPrivate::requestProposal(AssistReason reason,
                 requestProposal(reason, m_assistKind, m_requestProvider);
             } else {
                 displayProposal(newProposal, reason);
-                emit q->finished();
+                if (processor && processor->running())
+                    m_asyncProcessor = processor;
+                else
+                    emit q->finished();
             }
         });
 
@@ -275,6 +286,8 @@ void CodeAssistantPrivate::requestProposal(AssistReason reason,
         } else if (!processor->running()) {
             delete processor;
         } else { // ...async request was triggered
+            if (IAssistProposal *newProposal = processor->immediateProposal(assistInterface))
+                displayProposal(newProposal, reason);
             QTC_CHECK(!m_asyncProcessor);
             m_asyncProcessor = processor;
         }
@@ -324,6 +337,16 @@ void CodeAssistantPrivate::displayProposal(IAssistProposal *newProposal, AssistR
     if (!newProposal->hasItemsToPropose(prefix, reason)) {
         if (newProposal->isCorrective(m_editorWidget))
             newProposal->makeCorrection(m_editorWidget);
+        destroyContext();
+        return;
+    }
+
+    if (m_proposalWidget
+        && basePosition == proposalCandidate->basePosition()
+        && m_proposalWidget->supportsModelUpdate(proposalCandidate->id())) {
+        m_proposal.reset(proposalCandidate.take());
+        m_proposalWidget->updateModel(m_proposal->model());
+        m_proposalWidget->updateProposal(prefix);
         return;
     }
 
@@ -405,7 +428,7 @@ void CodeAssistantPrivate::finalizeProposal()
 
 bool CodeAssistantPrivate::isDisplayingProposal() const
 {
-    return m_proposalWidget != nullptr && m_proposalWidget->isVisible();
+    return m_proposalWidget != nullptr && m_proposalWidget->proposalIsVisible();
 }
 
 bool CodeAssistantPrivate::isWaitingForProposal() const
@@ -461,7 +484,6 @@ void CodeAssistantPrivate::notifyChange()
             if (!isDisplayingProposal())
                 requestActivationCharProposal();
         } else {
-            destroyContext();
             requestProposal(ExplicitlyInvoked, m_assistKind, m_requestProvider);
         }
     }
@@ -480,7 +502,7 @@ void CodeAssistantPrivate::destroyContext()
         cancelCurrentRequest();
     } else if (m_proposalWidget) {
         m_editorWidget->keepAutoCompletionHighlight(false);
-        if (m_proposalWidget->isVisible())
+        if (m_proposalWidget->proposalIsVisible())
             m_proposalWidget->closeProposal();
         disconnect(m_proposalWidget, &QObject::destroyed,
                    this, &CodeAssistantPrivate::finalizeProposal);
@@ -506,8 +528,11 @@ void CodeAssistantPrivate::startAutomaticProposalTimer()
 
 void CodeAssistantPrivate::automaticProposalTimeout()
 {
-    if (isWaitingForProposal() || (isDisplayingProposal() && !m_proposal->isFragile()))
+    if (isWaitingForProposal()
+        || m_editorWidget->multiTextCursor().hasMultipleCursors()
+        || (isDisplayingProposal() && !m_proposal->isFragile())) {
         return;
+    }
 
     requestProposal(IdleEditor, Completion);
 }
@@ -561,6 +586,9 @@ bool CodeAssistantPrivate::eventFilter(QObject *o, QEvent *e)
                 destroyContext();
             else if (!keyText.isEmpty() && !m_receivedContentWhileWaiting)
                 m_receivedContentWhileWaiting = true;
+        } else if (type == QEvent::KeyRelease
+                   && static_cast<QKeyEvent *>(e)->key() == Qt::Key_Escape) {
+            destroyContext();
         }
     }
 
