@@ -154,7 +154,10 @@ FunctionHintProposalWidget::FunctionHintProposalWidget()
 
     connect(upArrow, &QAbstractButton::clicked, this, &FunctionHintProposalWidget::previousPage);
     connect(downArrow, &QAbstractButton::clicked, this, &FunctionHintProposalWidget::nextPage);
-    connect(d->m_popupFrame.data(), &QObject::destroyed, this, &FunctionHintProposalWidget::abort);
+    connect(d->m_popupFrame.data(), &QObject::destroyed, this, [this](){
+        qApp->removeEventFilter(this);
+        deleteLater();
+    });
 
     setFocusPolicy(Qt::NoFocus);
 }
@@ -219,10 +222,15 @@ void FunctionHintProposalWidget::closeProposal()
     abort();
 }
 
+bool FunctionHintProposalWidget::proposalIsVisible() const
+{
+    return d->m_popupFrame && d->m_popupFrame->isVisible();
+}
+
 void FunctionHintProposalWidget::abort()
 {
     qApp->removeEventFilter(this);
-    if (d->m_popupFrame->isVisible())
+    if (proposalIsVisible())
         d->m_popupFrame->close();
     deleteLater();
 }
