@@ -30,16 +30,18 @@
 #include <texteditor/completionsettings.h>
 
 #include <QDebug>
+#include <QElapsedTimer>
 #include <QRegularExpression>
 #include <QtAlgorithms>
 #include <QHash>
 
 #include <algorithm>
+#include <utils/algorithm.h>
 
 using namespace TextEditor;
 
 QT_BEGIN_NAMESPACE
-uint qHash(const AssistProposalItem &item)
+auto qHash(const AssistProposalItem &item)
 {
     return qHash(item.text());
 }
@@ -297,28 +299,46 @@ void GenericProposalModel::filter(const QString &prefix)
         convertCaseSensitivity(TextEditorSettings::completionSettings().m_caseSensitivity);
     const QRegularExpression regExp = FuzzyMatcher::createRegExp(prefix, caseSensitivity);
 
+    QElapsedTimer timer;
+    timer.start();
+
     m_currentItems.clear();
     const QString lowerPrefix = prefix.toLower();
+    const bool checkInfix = prefix.size() >= 3;
     for (const auto &item : qAsConst(m_originalItems)) {
-        const QString &text = item->text();
+        const QString &text = item->filterText();
+
+        // Direct match?
+        if (text.startsWith(prefix)) {
+            m_currentItems.append(item);
+            item->setProposalMatch(text.length() == prefix.length()
+                                   ? AssistProposalItemInterface::ProposalMatch::Full
+                                   : AssistProposalItemInterface::ProposalMatch::Exact);
+            continue;
+        }
+
+        if (text.startsWith(lowerPrefix, Qt::CaseInsensitive)) {
+            m_currentItems.append(item);
+            item->setProposalMatch(AssistProposalItemInterface::ProposalMatch::Prefix);
+            continue;
+        }
+
+        if (checkInfix && text.contains(lowerPrefix, Qt::CaseInsensitive)) {
+            m_currentItems.append(item);
+            item->setProposalMatch(AssistProposalItemInterface::ProposalMatch::Infix);
+            continue;
+        }
+
+        // Our fuzzy matcher can become unusably slow with certain inputs, so skip it
+        // if we'd become unresponsive. See QTCREATORBUG-25419.
+        if (timer.elapsed() > 100)
+            continue;
+
         const QRegularExpressionMatch match = regExp.match(text);
         const bool hasPrefixMatch = match.capturedStart() == 0;
-        const bool hasInfixMatch = prefix.size() >= 3 && match.hasMatch();
-        if (hasPrefixMatch || hasInfixMatch) {
+        const bool hasInfixMatch = checkInfix && match.hasMatch();
+        if (hasPrefixMatch || hasInfixMatch)
             m_currentItems.append(item);
-            if (text.startsWith(prefix)) {
-                // Direct match
-                item->setProposalMatch(text.length() == prefix.length()
-                                       ? AssistProposalItemInterface::ProposalMatch::Full
-                                       : AssistProposalItemInterface::ProposalMatch::Exact);
-                continue;
-            }
-
-            if (text.startsWith(lowerPrefix, Qt::CaseInsensitive))
-                item->setProposalMatch(AssistProposalItemInterface::ProposalMatch::Prefix);
-            else if (text.contains(lowerPrefix, Qt::CaseInsensitive))
-                item->setProposalMatch(AssistProposalItemInterface::ProposalMatch::Infix);
-        }
     }
 }
 
@@ -398,3 +418,13 @@ AssistProposalItemInterface *GenericProposalModel::proposalItem(int index) const
 {
     return m_currentItems.at(index);
 }
+
+int GenericProposalModel::indexOf(
+    const std::function<bool(AssistProposalItemInterface *)> &predicate) const
+{
+    for (int index = 0, end = m_currentItems.size(); index < end; ++index) {
+        if (predicate(m_currentItems.at(index)))
+            return index;
+    }
+    return -1;
+}
