@@ -92,11 +92,9 @@ QByteArray CodeStylePoolPrivate::generateUniqueId(const QByteArray &id) const
 }
 }
 
-static QString customCodeStylesPath()
+static Utils::FilePath customCodeStylesPath()
 {
-    QString path = Core::ICore::userResourcePath();
-    path.append(QLatin1String("/codestyles/"));
-    return path;
+    return Core::ICore::userResourcePath("codestyles");
 }
 
 CodeStylePool::CodeStylePool(ICodeStylePreferencesFactory *factory, QObject *parent)
@@ -114,7 +112,7 @@ CodeStylePool::~CodeStylePool()
 QString CodeStylePool::settingsDir() const
 {
     const QString suffix = d->m_factory ? d->m_factory->languageId().toString() : QLatin1String("default");
-    return customCodeStylesPath().append(suffix);
+    return customCodeStylesPath().pathAppended(suffix).toString();
 }
 
 Utils::FilePath CodeStylePool::settingsPath(const QByteArray &id) const
@@ -237,7 +235,7 @@ ICodeStylePreferences *CodeStylePool::loadCodeStyle(const Utils::FilePath &fileN
     reader.load(fileName);
     QVariantMap m = reader.restoreValues();
     if (m.contains(QLatin1String(codeStyleDataKey))) {
-        const QByteArray id = fileName.toFileInfo().completeBaseName().toUtf8();
+        const QByteArray id = fileName.completeBaseName().toUtf8();
         const QString displayName = reader.restoreValue(QLatin1String(displayNameKey)).toString();
         const QVariantMap map = reader.restoreValue(QLatin1String(codeStyleDataKey)).toMap();
         if (d->m_factory) {
@@ -263,7 +261,7 @@ void CodeStylePool::slotSaveCodeStyle()
 
 void CodeStylePool::saveCodeStyle(ICodeStylePreferences *codeStyle) const
 {
-    const QString codeStylesPath = customCodeStylesPath();
+    const QString codeStylesPath = customCodeStylesPath().toString();
 
     // Create the base directory when it doesn't exist
     if (!QFile::exists(codeStylesPath) && !QDir().mkpath(codeStylesPath)) {
