@@ -43,6 +43,7 @@
 #include <QDebug>
 
 using namespace TextEditor;
+using namespace Utils;
 
 namespace TextEditor {
 namespace Internal {
@@ -52,8 +53,8 @@ class CodeStyleDialog : public QDialog
     Q_OBJECT
 public:
     CodeStyleDialog(ICodeStylePreferencesFactory *factory,
-                    ICodeStylePreferences *codeStyle, QWidget *parent = 0);
-    ~CodeStyleDialog();
+                    ICodeStylePreferences *codeStyle, QWidget *parent = nullptr);
+    ~CodeStyleDialog() override;
     ICodeStylePreferences *codeStyle() const;
 private:
     void slotCopyClicked();
@@ -62,28 +63,26 @@ private:
     ICodeStylePreferences *m_codeStyle;
     QLineEdit *m_lineEdit;
     QDialogButtonBox *m_buttons;
-    QLabel *m_warningLabel;
-    QPushButton *m_copyButton;
+    QLabel *m_warningLabel = nullptr;
+    QPushButton *m_copyButton = nullptr;
     QString m_originalDisplayName;
 };
 
 CodeStyleDialog::CodeStyleDialog(ICodeStylePreferencesFactory *factory,
                 ICodeStylePreferences *codeStyle, QWidget *parent)
-    : QDialog(parent),
-      m_warningLabel(0),
-      m_copyButton(0)
+    : QDialog(parent)
 {
     setWindowTitle(tr("Edit Code Style"));
-    QVBoxLayout *layout = new QVBoxLayout(this);
+    auto layout = new QVBoxLayout(this);
     QLabel *label = new QLabel(tr("Code style name:"));
     m_lineEdit = new QLineEdit(codeStyle->displayName(), this);
-    QHBoxLayout *nameLayout = new QHBoxLayout();
+    auto nameLayout = new QHBoxLayout;
     nameLayout->addWidget(label);
     nameLayout->addWidget(m_lineEdit);
     layout->addLayout(nameLayout);
 
     if (codeStyle->isReadOnly()) {
-        QHBoxLayout *warningLayout = new QHBoxLayout();
+        auto warningLayout = new QHBoxLayout;
         m_warningLabel = new QLabel(
                     tr("You cannot save changes to a built-in code style. "
                        "Copy it first to create your own version."), this);
@@ -152,21 +151,18 @@ CodeStyleDialog::~CodeStyleDialog()
     delete m_codeStyle;
 }
 
-}
-}
+} // Internal
 
 CodeStyleSelectorWidget::CodeStyleSelectorWidget(ICodeStylePreferencesFactory *factory, QWidget *parent) :
     QWidget(parent),
     m_factory(factory),
-    m_codeStyle(0),
-    m_ui(new Internal::Ui::CodeStyleSelectorWidget),
-    m_ignoreGuiSignals(false)
+    m_ui(new Internal::Ui::CodeStyleSelectorWidget)
 {
     m_ui->setupUi(this);
     m_ui->importButton->setEnabled(false);
     m_ui->exportButton->setEnabled(false);
 
-    connect(m_ui->delegateComboBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::activated),
+    connect(m_ui->delegateComboBox, QOverload<int>::of(&QComboBox::activated),
             this, &CodeStyleSelectorWidget::slotComboBoxActivated);
     connect(m_ui->copyButton, &QAbstractButton::clicked,
             this, &CodeStyleSelectorWidget::slotCopyClicked);
@@ -239,12 +235,10 @@ void CodeStyleSelectorWidget::slotComboBoxActivated(int index)
 
     if (index < 0 || index >= m_ui->delegateComboBox->count())
         return;
-    ICodeStylePreferences *delegate =
-            m_ui->delegateComboBox->itemData(index).value<ICodeStylePreferences *>();
+    auto delegate = m_ui->delegateComboBox->itemData(index).value<ICodeStylePreferences *>();
 
-    const bool wasBlocked = blockSignals(true);
+    QSignalBlocker blocker(this);
     m_codeStyle->setCurrentDelegate(delegate);
-    blockSignals(wasBlocked);
 }
 
 void CodeStyleSelectorWidget::slotCurrentDelegateChanged(ICodeStylePreferences *delegate)
@@ -320,7 +314,7 @@ void CodeStyleSelectorWidget::slotRemoveClicked()
                            this);
 
     // Change the text and role of the discard button
-    QPushButton *deleteButton = static_cast<QPushButton*>(messageBox.button(QMessageBox::Discard));
+    auto deleteButton = static_cast<QPushButton*>(messageBox.button(QMessageBox::Discard));
     deleteButton->setText(tr("Delete"));
     messageBox.addButton(deleteButton, QMessageBox::AcceptRole);
     messageBox.setDefaultButton(deleteButton);
@@ -332,9 +326,9 @@ void CodeStyleSelectorWidget::slotRemoveClicked()
 
 void CodeStyleSelectorWidget::slotImportClicked()
 {
-    const Utils::FileName fileName =
-            Utils::FileName::fromString(QFileDialog::getOpenFileName(this, tr("Import Code Style"), QString(),
-                                                                     tr("Code styles (*.xml);;All files (*)")));
+    const FilePath fileName =
+            FileUtils::getOpenFilePath(this, tr("Import Code Style"), {},
+                                       tr("Code styles (*.xml);;All files (*)"));
     if (!fileName.isEmpty()) {
         CodeStylePool *codeStylePool = m_codeStyle->delegatingPool();
         ICodeStylePreferences *importedStyle = codeStylePool->importCodeStyle(fileName);
@@ -349,12 +343,12 @@ void CodeStyleSelectorWidget::slotImportClicked()
 void CodeStyleSelectorWidget::slotExportClicked()
 {
     ICodeStylePreferences *currentPreferences = m_codeStyle->currentPreferences();
-    const QString fileName = QFileDialog::getSaveFileName(this, tr("Export Code Style"),
-                             QString::fromUtf8(currentPreferences->id() + ".xml"),
+    const FilePath filePath = FileUtils::getSaveFilePath(this, tr("Export Code Style"),
+                             FilePath::fromString(QString::fromUtf8(currentPreferences->id() + ".xml")),
                              tr("Code styles (*.xml);;All files (*)"));
-    if (!fileName.isEmpty()) {
+    if (!filePath.isEmpty()) {
         CodeStylePool *codeStylePool = m_codeStyle->delegatingPool();
-        codeStylePool->exportCodeStyle(Utils::FileName::fromString(fileName), currentPreferences);
+        codeStylePool->exportCodeStyle(filePath, currentPreferences);
     }
 }
 
@@ -391,7 +385,7 @@ void CodeStyleSelectorWidget::slotCodeStyleRemoved(ICodeStylePreferences *codeSt
 
 void CodeStyleSelectorWidget::slotUpdateName()
 {
-    ICodeStylePreferences *changedCodeStyle = qobject_cast<ICodeStylePreferences *>(sender());
+    auto changedCodeStyle = qobject_cast<ICodeStylePreferences *>(sender());
     if (!changedCodeStyle)
         return;
 
@@ -428,4 +422,6 @@ QString CodeStyleSelectorWidget::displayName(ICodeStylePreferences *codeStyle) c
     return name;
 }
 
+} // TextEditor
+
 #include "codestyleselectorwidget.moc"
