@@ -141,7 +141,7 @@ bool FontSettings::equals(const FontSettings &f) const
             && m_scheme == f.m_scheme;
 }
 
-uint qHash(const TextStyle &textStyle)
+auto qHash(const TextStyle &textStyle)
 {
     return ::qHash(quint8(textStyle));
 }
@@ -202,7 +202,7 @@ QTextCharFormat FontSettings::toTextCharFormat(TextStyle category) const
     return tf;
 }
 
-uint qHash(TextStyles textStyles)
+auto qHash(TextStyles textStyles)
 {
     return ::qHash(reinterpret_cast<quint64&>(textStyles));
 }
@@ -417,6 +417,10 @@ bool FontSettings::loadColorScheme(const QString &fileName,
     foreach (const FormatDescription &desc, descriptions) {
         const TextStyle id = desc.id();
         if (!m_scheme.contains(id)) {
+            if (id == C_NAMESPACE && m_scheme.contains(C_TYPE)) {
+                m_scheme.setFormatFor(C_NAMESPACE, m_scheme.formatFor(C_TYPE));
+                continue;
+            }
             Format format;
             const Format &descFormat = desc.format();
             // Default fallback for background and foreground is C_TEXT, which is set through
@@ -504,20 +508,19 @@ int FontSettings::defaultFontSize()
  */
 QString FontSettings::defaultSchemeFileName(const QString &fileName)
 {
-    QString defaultScheme = Core::ICore::resourcePath();
-    defaultScheme += QLatin1String("/styles/");
+    Utils::FilePath defaultScheme = Core::ICore::resourcePath("styles");
 
-    if (!fileName.isEmpty() && QFile::exists(defaultScheme + fileName)) {
-        defaultScheme += fileName;
+    if (!fileName.isEmpty() && (defaultScheme / fileName).exists()) {
+        defaultScheme = defaultScheme / fileName;
     } else {
         const QString themeScheme = Utils::creatorTheme()->defaultTextEditorColorScheme();
-        if (!themeScheme.isEmpty() && QFile::exists(defaultScheme + themeScheme))
-            defaultScheme += themeScheme;
+        if (!themeScheme.isEmpty() && (defaultScheme / themeScheme).exists())
+            defaultScheme = defaultScheme / themeScheme;
         else
-            defaultScheme += QLatin1String("default.xml");
+            defaultScheme = defaultScheme / "default.xml";
     }
 
-    return defaultScheme;
+    return defaultScheme.toString();
 }
 
 } // namespace TextEditor
