@@ -26,42 +26,46 @@
 #include "highlighter.h"
 
 #include "highlightersettings.h"
-#include "textdocumentlayout.h"
 #include "tabsettings.h"
-#include "texteditorsettings.h"
+#include "textdocumentlayout.h"
 #include "texteditor.h"
+#include "texteditorsettings.h"
 
 #include <coreplugin/editormanager/documentmodel.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/messagemanager.h>
 #include <utils/mimetypes/mimedatabase.h>
-#include <utils/stylehelper.h>
 #include <utils/qtcassert.h>
+#include <utils/stylehelper.h>
 
 #include <DefinitionDownloader>
-#include <Format>
 #include <FoldingRegion>
+#include <Format>
 #include <Repository>
 #include <SyntaxHighlighter>
 
-#include <QDir>
+#include <QLoggingCategory>
 #include <QMetaEnum>
 
-using namespace TextEditor;
+using namespace Utils;
 
-static const char kDefinitionForMimeType[] = "definitionForMimeType";
-static const char kDefinitionForExtension[] = "definitionForExtension";
-static const char kDefinitionForFilePath[] = "definitionForFilePath";
+namespace TextEditor {
 
-KSyntaxHighlighting::Repository *highlightRepository()
+static Q_LOGGING_CATEGORY(highlighterLog, "qtc.editor.highlighter", QtWarningMsg)
+
+const char kDefinitionForMimeType[] = "definitionForMimeType";
+const char kDefinitionForExtension[] = "definitionForExtension";
+const char kDefinitionForFilePath[] = "definitionForFilePath";
+
+static KSyntaxHighlighting::Repository *highlightRepository()
 {
     static KSyntaxHighlighting::Repository *repository = nullptr;
     if (!repository) {
         repository = new KSyntaxHighlighting::Repository();
-        repository->addCustomSearchPath(TextEditorSettings::highlighterSettings().definitionFilesPath());
-        QDir dir(Core::ICore::resourcePath() + QLatin1String("/generic-highlighter/syntax"));
-        if (dir.exists() && dir.cdUp())
-            repository->addCustomSearchPath(dir.path());
+        repository->addCustomSearchPath(TextEditorSettings::highlighterSettings().definitionFilesPath().toString());
+        const FilePath dir = Core::ICore::resourcePath("generic-highlighter/syntax");
+        if (dir.exists())
+            repository->addCustomSearchPath(dir.parentDir().path());
     }
     return repository;
 }
@@ -125,18 +129,18 @@ Highlighter::Definitions Highlighter::definitionsForDocument(const TextDocument
     // If we check the MIME type first and then skip the pattern, the definition for "*.rb.xml" is
     // never considered.
     // The KSyntaxHighlighting CLI also completely ignores MIME types.
-    const Utils::FilePath &filePath = document->filePath();
+    const FilePath &filePath = document->filePath();
     Definitions definitions = definitionsForFileName(filePath);
     if (definitions.isEmpty()) {
         // check for *.in filename since those are usually used for
         // cmake configure_file input filenames without the .in extension
-        if (filePath.endsWith(".in")) {
-            definitions = definitionsForFileName(
-                Utils::FilePath::fromString(filePath.toFileInfo().completeBaseName()));
-        }
+        if (filePath.endsWith(".in"))
+            definitions = definitionsForFileName(FilePath::fromString(filePath.completeBaseName()));
+        if (filePath.fileName() == "qtquickcontrols2.conf")
+            definitions = definitionsForFileName(filePath.stringAppended(".ini"));
     }
     if (definitions.isEmpty()) {
-        const Utils::MimeType &mimeType = Utils::mimeTypeForName(document->mimeType());
+        const MimeType &mimeType = Utils::mimeTypeForName(document->mimeType());
         if (mimeType.isValid())
             definitions = definitionsForMimeType(mimeType.name());
     }
@@ -166,17 +170,17 @@ Highlighter::Definitions Highlighter::definitionsForMimeType(const QString &mime
     return definitions;
 }
 
-Highlighter::Definitions Highlighter::definitionsForFileName(const Utils::FilePath &fileName)
+Highlighter::Definitions Highlighter::definitionsForFileName(const FilePath &fileName)
 {
     Definitions definitions
         = highlightRepository()->definitionsForFileName(fileName.fileName()).toList();
 
     if (definitions.size() > 1) {
-        const QString &fileExtension = fileName.toFileInfo().completeSuffix();
+        const QString &fileExtension = fileName.completeSuffix();
         const Definition &rememberedDefinition
             = fileExtension.isEmpty()
                   ? definitionForSetting(kDefinitionForFilePath,
-                                         fileName.toFileInfo().canonicalFilePath())
+                                         fileName.absoluteFilePath().toString())
                   : definitionForSetting(kDefinitionForExtension, fileExtension);
         if (rememberedDefinition.isValid() && definitions.contains(rememberedDefinition))
             definitions = {rememberedDefinition};
@@ -192,11 +196,11 @@ void Highlighter::rememberDefinitionForDocument(const Highlighter::Definition &d
     if (!definition.isValid())
         return;
     const QString &mimeType = document->mimeType();
-    const QString &fileExtension = document->filePath().toFileInfo().completeSuffix();
-    const QString &path = document->filePath().toFileInfo().canonicalFilePath();
+    const FilePath &path = document->filePath();
+    const QString &fileExtension = path.completeSuffix();
     QSettings *settings = Core::ICore::settings();
     settings->beginGroup(Constants::HIGHLIGHTER_SETTINGS_CATEGORY);
-    const Definitions &fileNameDefinitions = definitionsForFileName(document->filePath());
+    const Definitions &fileNameDefinitions = definitionsForFileName(path);
     if (fileNameDefinitions.contains(definition)) {
         if (!fileExtension.isEmpty()) {
             const QString id(kDefinitionForExtension);
@@ -206,7 +210,7 @@ void Highlighter::rememberDefinitionForDocument(const Highlighter::Definition &d
         } else if (!path.isEmpty()) {
             const QString id(kDefinitionForFilePath);
             QMap<QString, QVariant> map = settings->value(id).toMap();
-            map.insert(document->filePath().toFileInfo().absoluteFilePath(), definition.name());
+            map.insert(path.absoluteFilePath().toString(), definition.name());
             settings->setValue(id, map);
         }
     } else if (!mimeType.isEmpty()) {
@@ -228,7 +232,7 @@ void Highlighter::clearDefinitionForDocumentCache()
     settings->endGroup();
 }
 
-void Highlighter::addCustomHighlighterPath(const Utils::FilePath &path)
+void Highlighter::addCustomHighlighterPath(const FilePath &path)
 {
     highlightRepository()->addCustomSearchPath(path.toString());
 }
@@ -236,22 +240,19 @@ void Highlighter::addCustomHighlighterPath(const Utils::FilePath &path)
 void Highlighter::downloadDefinitions(std::function<void()> callback) {
     auto downloader =
         new KSyntaxHighlighting::DefinitionDownloader(highlightRepository());
-    connect(downloader, &KSyntaxHighlighting::DefinitionDownloader::done,
-            [downloader, callback]() {
-                Core::MessageManager::write(tr("Highlighter updates: done"),
-                                            Core::MessageManager::ModeSwitch);
-                downloader->deleteLater();
-                reload();
-                if (callback)
-                    callback();
-            });
+    connect(downloader, &KSyntaxHighlighting::DefinitionDownloader::done, [downloader, callback]() {
+        Core::MessageManager::writeFlashing(tr("Highlighter updates: done"));
+        downloader->deleteLater();
+        reload();
+        if (callback)
+            callback();
+    });
     connect(downloader,
             &KSyntaxHighlighting::DefinitionDownloader::informationMessage,
             [](const QString &message) {
-                Core::MessageManager::write(tr("Highlighter updates:") + ' ' +
-                                                message,
-                                            Core::MessageManager::ModeSwitch);
+                Core::MessageManager::writeSilently(tr("Highlighter updates:") + ' ' + message);
             });
+    Core::MessageManager::writeDisrupting(tr("Highlighter updates: starting"));
     downloader->start();
 }
 
@@ -330,18 +331,18 @@ void Highlighter::applyFormat(int offset, int length, const KSyntaxHighlighting:
         const QColor textColor = format.textColor(defaultTheme);
         if (format.hasBackgroundColor(defaultTheme)) {
             const QColor backgroundColor = format.hasBackgroundColor(defaultTheme);
-            if (Utils::StyleHelper::isReadableOn(backgroundColor, textColor)) {
+            if (StyleHelper::isReadableOn(backgroundColor, textColor)) {
                 qformat.setForeground(textColor);
                 qformat.setBackground(backgroundColor);
-            } else if (Utils::StyleHelper::isReadableOn(qformat.background().color(), textColor)) {
+            } else if (StyleHelper::isReadableOn(qformat.background().color(), textColor)) {
                 qformat.setForeground(textColor);
             }
-        } else if (Utils::StyleHelper::isReadableOn(qformat.background().color(), textColor)) {
+        } else if (StyleHelper::isReadableOn(qformat.background().color(), textColor)) {
             qformat.setForeground(textColor);
         }
     } else if (format.hasBackgroundColor(defaultTheme)) {
         const QColor backgroundColor = format.hasBackgroundColor(defaultTheme);
-        if (Utils::StyleHelper::isReadableOn(backgroundColor, qformat.foreground().color()))
+        if (StyleHelper::isReadableOn(backgroundColor, qformat.foreground().color()))
             qformat.setBackground(backgroundColor);
     }
 
@@ -371,14 +372,22 @@ void Highlighter::applyFolding(int offset,
     const bool fromStart = TabSettings::firstNonSpace(text) == offset;
     const bool toEnd = (offset + length) == (text.length() - TabSettings::trailingWhitespaces(text));
     if (region.type() == KSyntaxHighlighting::FoldingRegion::Begin) {
-        TextDocumentLayout::changeBraceDepth(block, 1);
-        // if there is only a folding begin in the line move the current block into the fold
-        if (fromStart && toEnd) {
+        const int newBraceDepth = TextDocumentLayout::braceDepth(block) + 1;
+        TextDocumentLayout::setBraceDepth(block, newBraceDepth);
+        qCDebug(highlighterLog) << "Found folding start from '" << offset << "' to '" << length
+                                << "' resulting in the bracedepth '" << newBraceDepth << "' in :";
+        qCDebug(highlighterLog) << text;
+        // if there is only a folding begin marker in the line move the current block into the fold
+        if (fromStart && toEnd && length <= 1) {
             data->setFoldingIndent(TextDocumentLayout::braceDepth(block));
             data->setFoldingStartIncluded(true);
         }
     } else if (region.type() == KSyntaxHighlighting::FoldingRegion::End) {
-        TextDocumentLayout::changeBraceDepth(block, -1);
+        const int newBraceDepth = qMax(0, TextDocumentLayout::braceDepth(block) - 1);
+        qCDebug(highlighterLog) << "Found folding end from '" << offset << "' to '" << length
+                                << "' resulting in the bracedepth '" << newBraceDepth << "' in :";
+        qCDebug(highlighterLog) << text;
+        TextDocumentLayout::setBraceDepth(block, newBraceDepth);
         // if the folding end is at the end of the line move the current block into the fold
         if (toEnd)
             data->setFoldingEndIncluded(true);
@@ -386,3 +395,5 @@ void Highlighter::applyFolding(int offset,
             data->setFoldingIndent(TextDocumentLayout::braceDepth(block));
     }
 }
+
+} // TextEditor
