@@ -48,9 +48,10 @@ LineNumberFilter::LineNumberFilter(QObject *parent)
 {
     setId("Line in current document");
     setDisplayName(tr("Line in Current Document"));
+    setDescription(tr("Jumps to the given line in the current document."));
     setPriority(High);
-    setShortcutString("l");
-    setIncludedByDefault(true);
+    setDefaultShortcutString("l");
+    setDefaultIncludedByDefault(true);
 }
 
 void LineNumberFilter::prepareSearch(const QString &entry)
