@@ -30,34 +30,76 @@
 
 #include <utils/qtcassert.h>
 
-#include <QTextDocument>
 #include <QTextBlock>
+#include <QTextDocument>
+
+#include <algorithm>
 
 using namespace TextEditor;
 using namespace TextEditor::SemanticHighlighter;
 
 namespace {
 
-QTextLayout::FormatRange rangeForResult(const HighlightingResult &result,
-                                        const QHash<int, QTextCharFormat> &kindToFormat)
-{
+class Range {
+public:
     QTextLayout::FormatRange formatRange;
+    QTextBlock block;
+};
+using Ranges = QVector<Range>;
 
-    formatRange.start = int(result.column) - 1;
-    formatRange.length = int(result.length);
-    formatRange.format = result.useTextSyles
+const Ranges rangesForResult(const HighlightingResult &result, const QTextBlock &startBlock,
+                             const QHash<int, QTextCharFormat> &kindToFormat)
+{
+    const QTextCharFormat format = result.useTextSyles
         ? TextEditorSettings::fontSettings().toTextCharFormat(result.textStyles)
         : kindToFormat.value(result.kind);
-    return formatRange;
+    if (!format.isValid())
+        return {};
+
+    HighlightingResult curResult = result;
+    QTextBlock curBlock = startBlock;
+    Ranges ranges;
+    while (curBlock.isValid()) {
+        Range range;
+        range.block = curBlock;
+        range.formatRange.format = format;
+        range.formatRange.start = curResult.column - 1;
+        range.formatRange.length = std::min(curResult.length,
+                                            curBlock.length() - range.formatRange.start);
+        ranges << range;
+        if (range.formatRange.length == curResult.length)
+            break;
+        curBlock = curBlock.next();
+        curResult.column = 1;
+        curResult.length -= range.formatRange.length;
+    }
+
+    return ranges;
+}
+
+const Ranges rangesForResult(
+        const HighlightingResult &result,
+        QTextDocument *doc,
+        const QHash<int, QTextCharFormat> &kindToFormat,
+        const Splitter &splitter = {})
+{
+    const QTextBlock startBlock = doc->findBlockByNumber(result.line - 1);
+    if (splitter) {
+        Ranges ranges;
+        for (const auto &[newResult, newBlock] : splitter(result, startBlock))
+            ranges << rangesForResult(newResult, newBlock, kindToFormat);
+        return ranges;
+    }
+    return rangesForResult(result, startBlock, kindToFormat);
 }
 
 }
 
-void SemanticHighlighter::incrementalApplyExtraAdditionalFormats(
-        SyntaxHighlighter *highlighter,
+void SemanticHighlighter::incrementalApplyExtraAdditionalFormats(SyntaxHighlighter *highlighter,
         const QFuture<HighlightingResult> &future,
         int from, int to,
-        const QHash<int, QTextCharFormat> &kindToFormat)
+        const QHash<int, QTextCharFormat> &kindToFormat,
+        const Splitter &splitter)
 {
     if (to <= from)
         return;
@@ -81,39 +123,21 @@ void SemanticHighlighter::incrementalApplyExtraAdditionalFormats(
 
     QTextDocument *doc = highlighter->document();
     QTC_ASSERT(currentBlockNumber < doc->blockCount(), return);
-    QTextBlock b = doc->findBlockByNumber(currentBlockNumber);
+    QTextBlock currentBlock = doc->findBlockByNumber(currentBlockNumber);
 
-    HighlightingResult result = future.resultAt(from);
-    for (int i = from; i < to && b.isValid(); ) {
-        const int blockNumber = int(result.line) - 1;
-        QTC_ASSERT(blockNumber < doc->blockCount(), return);
+    std::map<QTextBlock, QVector<QTextLayout::FormatRange>> formatRanges;
+    for (int i = from; i < to; ++i) {
+        for (const Range &range : rangesForResult(future.resultAt(i), doc, kindToFormat, splitter))
+            formatRanges[range.block].append(range.formatRange);
+    }
 
-        // clear formats of blocks until blockNumber
-        while (currentBlockNumber < blockNumber) {
-            highlighter->clearExtraFormats(b);
-            b = b.next();
-            ++currentBlockNumber;
+    for (auto &[block, ranges] : formatRanges) {
+        while (currentBlock < block) {
+            highlighter->clearExtraFormats(currentBlock);
+            currentBlock = currentBlock.next();
         }
-
-        // collect all the formats for the current line
-        QVector<QTextLayout::FormatRange> formats;
-        formats.reserve(to - from);
-        forever {
-            const QTextLayout::FormatRange formatRange = rangeForResult(result, kindToFormat);
-            if (formatRange.format.isValid())
-                formats.append(formatRange);
-
-            ++i;
-            if (i >= to)
-                break;
-            result = future.resultAt(i);
-            const int nextBlockNumber = int(result.line) - 1;
-            if (nextBlockNumber != blockNumber)
-                break;
-        }
-        highlighter->setExtraFormats(b, std::move(formats));
-        b = b.next();
-        ++currentBlockNumber;
+        highlighter->setExtraFormats(block, std::move(ranges));
+        currentBlock = block.next();
     }
 }
 
@@ -128,47 +152,34 @@ void SemanticHighlighter::setExtraAdditionalFormats(SyntaxHighlighter *highlight
     QTextDocument *doc = highlighter->document();
     QTC_ASSERT(doc, return );
 
-    QVector<QVector<QTextLayout::FormatRange>> ranges(doc->blockCount());
+    std::map<QTextBlock, QVector<QTextLayout::FormatRange>> formatRanges;
 
     for (auto result : results) {
-        const QTextLayout::FormatRange formatRange = rangeForResult(result, kindToFormat);
-        if (formatRange.format.isValid())
-            ranges[int(result.line) - 1].append(formatRange);
+        for (const Range &range : rangesForResult(result, doc, kindToFormat))
+            formatRanges[range.block].append(range.formatRange);
     }
 
-    for (int blockNumber = 0; blockNumber < ranges.count(); ++blockNumber) {
-        if (!ranges[blockNumber].isEmpty()) {
-            QTextBlock b = doc->findBlockByNumber(blockNumber);
-            QTC_ASSERT(b.isValid(), return );
-            highlighter->setExtraFormats(b, std::move(ranges[blockNumber]));
-        }
-    }
+    for (auto &[block, ranges] : formatRanges)
+        highlighter->setExtraFormats(block, std::move(ranges));
 }
 
 void SemanticHighlighter::clearExtraAdditionalFormatsUntilEnd(
         SyntaxHighlighter *highlighter,
         const QFuture<HighlightingResult> &future)
 {
-    // find block number of last result
-    int lastBlockNumber = 0;
+    const QTextDocument * const doc = highlighter->document();
+    QTextBlock firstBlockToClear = doc->begin();
     for (int i = future.resultCount() - 1; i >= 0; --i) {
         const HighlightingResult &result = future.resultAt(i);
         if (result.line) {
-            lastBlockNumber = int(result.line) - 1;
+            const QTextBlock blockForLine = doc->findBlockByNumber(result.line - 1);
+            const QTextBlock lastBlockWithResults = doc->findBlock(
+                        blockForLine.position() + result.column - 1 + result.length);
+            firstBlockToClear = lastBlockWithResults.next();
             break;
         }
     }
 
-    QTextDocument *doc = highlighter->document();
-
-    const int firstBlockToClear = lastBlockNumber + 1;
-    if (firstBlockToClear >= doc->blockCount())
-        return;
-
-    QTextBlock b = doc->findBlockByNumber(firstBlockToClear);
-
-    while (b.isValid()) {
+    for (QTextBlock b = firstBlockToClear; b.isValid(); b = b.next())
         highlighter->clearExtraFormats(b);
-        b = b.next();
-    }
 }
