@@ -51,7 +51,6 @@
 #include <QScrollBar>
 #include <QStringList>
 #include <QTextCodec>
-#include <QTimer>
 
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/icore.h>
@@ -82,15 +81,12 @@ public:
     {
     }
 
-    QTextCursor indentOrUnindent(const QTextCursor &textCursor, bool doIndent,
-                                 const TabSettings &tabSettings,
-                                 bool blockSelection = false, int column = 0,
-                                 int *offset = nullptr);
+    MultiTextCursor indentOrUnindent(const MultiTextCursor &cursor, bool doIndent, const TabSettings &tabSettings);
     void resetRevisions();
     void updateRevisions();
 
 public:
-    QString m_defaultPath;
+    FilePath m_defaultPath;
     QString m_suggestedFileName;
     TypingSettings m_typingSettings;
     StorageSettings m_storageSettings;
@@ -106,153 +102,97 @@ public:
     QScopedPointer<Indenter> m_indenter;
     QScopedPointer<Formatter> m_formatter;
 
-    bool m_fileIsReadOnly = false;
     int m_autoSaveRevision = -1;
+    bool m_silentReload = false;
 
     TextMarks m_marksCache; // Marks not owned
     Utils::Guard m_modificationChangedGuard;
 };
 
-QTextCursor TextDocumentPrivate::indentOrUnindent(const QTextCursor &textCursor, bool doIndent,
-                                                  const TabSettings &tabSettings,
-                                                  bool blockSelection, int columnIn, int *offset)
+MultiTextCursor TextDocumentPrivate::indentOrUnindent(const MultiTextCursor &cursors,
+                                                      bool doIndent,
+                                                      const TabSettings &tabSettings)
 {
-    QTextCursor cursor = textCursor;
-    cursor.beginEditBlock();
-
-    // Indent or unindent the selected lines
-    int pos = cursor.position();
-    int column = blockSelection ? columnIn
-               : tabSettings.columnAt(cursor.block().text(), cursor.positionInBlock());
-    int anchor = cursor.anchor();
-    int start = qMin(anchor, pos);
-    int end = qMax(anchor, pos);
-    bool modified = true;
-
-    QTextBlock startBlock = m_document.findBlock(start);
-    QTextBlock endBlock = m_document.findBlock(blockSelection ? end : qMax(end - 1, 0)).next();
-    const bool cursorAtBlockStart = (textCursor.position() == startBlock.position());
-    const bool anchorAtBlockStart = (textCursor.anchor() == startBlock.position());
-    const bool oneLinePartial = (startBlock.next() == endBlock)
-                              && (start > startBlock.position() || end < endBlock.position() - 1);
-
-    // Make sure one line selection will get processed in "for" loop
-    if (startBlock == endBlock)
-        endBlock = endBlock.next();
-
-    if (cursor.hasSelection() && !blockSelection && !oneLinePartial) {
-        for (QTextBlock block = startBlock; block != endBlock; block = block.next()) {
-            const QString text = block.text();
-            int indentPosition = tabSettings.lineIndentPosition(text);
-            if (!doIndent && !indentPosition)
-                indentPosition = tabSettings.firstNonSpace(text);
-            int targetColumn = tabSettings.indentedColumn(
-                        tabSettings.columnAt(text, indentPosition), doIndent);
-            cursor.setPosition(block.position() + indentPosition);
-            cursor.insertText(tabSettings.indentationString(0, targetColumn, 0, block));
-            cursor.setPosition(block.position());
-            cursor.setPosition(block.position() + indentPosition, QTextCursor::KeepAnchor);
-            cursor.removeSelectedText();
-        }
-        // make sure that selection that begins in first column stays at first column
-        // even if we insert text at first column
-        if (cursorAtBlockStart) {
-            cursor = textCursor;
-            cursor.setPosition(startBlock.position(), QTextCursor::KeepAnchor);
-        } else if (anchorAtBlockStart) {
-            cursor = textCursor;
-            cursor.setPosition(startBlock.position(), QTextCursor::MoveAnchor);
-            cursor.setPosition(textCursor.position(), QTextCursor::KeepAnchor);
+    MultiTextCursor result;
+    bool first = true;
+    for (const QTextCursor &textCursor : cursors) {
+        QTextCursor cursor = textCursor;
+        if (first) {
+            cursor.beginEditBlock();
+            first = false;
         } else {
-            modified = false;
+            cursor.joinPreviousEditBlock();
         }
-    } else if (cursor.hasSelection() && !blockSelection && oneLinePartial) {
-        // Only one line partially selected.
-        cursor.removeSelectedText();
-    } else {
-        // Indent or unindent at cursor position
-        int maxTargetColumn = -1;
 
-        class BlockIndenter
-        {
-        public:
-            BlockIndenter(const QTextBlock &_block,
-                          const int column,
-                          const TabSettings &_tabSettings)
-                : block(_block)
-                , text(block.text())
-                , tabSettings(_tabSettings)
-            {
-                indentPosition = tabSettings.positionAtColumn(text, column, nullptr, true);
-                spaces = tabSettings.spacesLeftFromPosition(text, indentPosition);
-            }
+        // Indent or unindent the selected lines
+        int pos = cursor.position();
+        int column = tabSettings.columnAt(cursor.block().text(), cursor.positionInBlock());
+        int anchor = cursor.anchor();
+        int start = qMin(anchor, pos);
+        int end = qMax(anchor, pos);
 
-            void indent(const int targetColumn) const
-            {
-                const int startColumn = tabSettings.columnAt(text, indentPosition - spaces);
-                QTextCursor cursor(block);
-                cursor.setPosition(block.position() + indentPosition);
-                cursor.setPosition(block.position() + indentPosition - spaces, QTextCursor::KeepAnchor);
+        QTextBlock startBlock = m_document.findBlock(start);
+        QTextBlock endBlock = m_document.findBlock(qMax(end - 1, 0)).next();
+        const bool cursorAtBlockStart = (cursor.position() == startBlock.position());
+        const bool anchorAtBlockStart = (cursor.anchor() == startBlock.position());
+        const bool oneLinePartial = (startBlock.next() == endBlock)
+                                    && (start > startBlock.position()
+                                        || end < endBlock.position() - 1)
+                                    && !cursors.hasMultipleCursors();
+
+        // Make sure one line selection will get processed in "for" loop
+        if (startBlock == endBlock)
+            endBlock = endBlock.next();
+
+        if (cursor.hasSelection()) {
+            if (oneLinePartial) {
                 cursor.removeSelectedText();
-                cursor.insertText(tabSettings.indentationString(startColumn, targetColumn, 0, block));
+            } else {
+                for (QTextBlock block = startBlock; block != endBlock; block = block.next()) {
+                    const QString text = block.text();
+                    int indentPosition = tabSettings.lineIndentPosition(text);
+                    if (!doIndent && !indentPosition)
+                        indentPosition = TabSettings::firstNonSpace(text);
+                    int targetColumn
+                        = tabSettings.indentedColumn(tabSettings.columnAt(text, indentPosition),
+                                                     doIndent);
+                    cursor.setPosition(block.position() + indentPosition);
+                    cursor.insertText(tabSettings.indentationString(0, targetColumn, 0, block));
+                    cursor.setPosition(block.position());
+                    cursor.setPosition(block.position() + indentPosition, QTextCursor::KeepAnchor);
+                    cursor.removeSelectedText();
+                }
+                // make sure that selection that begins in first column stays at first column
+                // even if we insert text at first column
+                if (cursorAtBlockStart) {
+                    cursor = textCursor;
+                    cursor.setPosition(startBlock.position(), QTextCursor::KeepAnchor);
+                } else if (anchorAtBlockStart) {
+                    cursor = textCursor;
+                    cursor.setPosition(startBlock.position(), QTextCursor::MoveAnchor);
+                    cursor.setPosition(textCursor.position(), QTextCursor::KeepAnchor);
+                }
             }
-
-            int targetColumn(bool doIndent) const
-            {
-                const int optimumTargetColumn
-                    = tabSettings.indentedColumn(tabSettings.columnAt(block.text(), indentPosition),
-                                                 doIndent);
-                const int minimumTargetColumn = tabSettings.columnAt(text, indentPosition - spaces);
-                return std::max(optimumTargetColumn, minimumTargetColumn);
-            }
-
-            const QTextBlock &textBlock() { return block; }
-
-        private:
-            QTextBlock block;
-            const QString text;
-            int indentPosition;
-            int spaces;
-            const TabSettings &tabSettings;
-        };
-
-        std::vector<BlockIndenter> blocks;
-
-        for (QTextBlock block = startBlock; block != endBlock; block = block.next()) {
-            QString text = block.text();
-
-            const int blockColumn = tabSettings.columnAt(text, text.size());
-            if (blockColumn < column) {
-                cursor.setPosition(block.position() + text.size());
-                cursor.insertText(tabSettings.indentationString(blockColumn, column, 0, block));
-                text = block.text();
-            }
-
-            blocks.emplace_back(BlockIndenter(block, column, tabSettings));
-            maxTargetColumn = std::max(maxTargetColumn, blocks.back().targetColumn(doIndent));
+        } else {
+            QString text = startBlock.text();
+            int indentPosition = tabSettings.positionAtColumn(text, column, nullptr, true);
+            int spaces = tabSettings.spacesLeftFromPosition(text, indentPosition);
+            int startColumn = tabSettings.columnAt(text, indentPosition - spaces);
+            int targetColumn = tabSettings.indentedColumn(tabSettings.columnAt(text, indentPosition),
+                                                          doIndent);
+            cursor.setPosition(startBlock.position() + indentPosition);
+            cursor.setPosition(startBlock.position() + indentPosition - spaces,
+                               QTextCursor::KeepAnchor);
+            cursor.removeSelectedText();
+            cursor.insertText(
+                tabSettings.indentationString(startColumn, targetColumn, 0, startBlock));
         }
-        for (const BlockIndenter &blockIndenter : blocks)
-            blockIndenter.indent(maxTargetColumn);
 
-        // Preserve initial anchor of block selection
-        if (blockSelection) {
-            if (offset)
-                *offset = maxTargetColumn - column;
-            startBlock = pos < anchor ? blocks.front().textBlock() : blocks.back().textBlock();
-            start = startBlock.position()
-                    + tabSettings.positionAtColumn(startBlock.text(), maxTargetColumn);
-            endBlock = pos > anchor ? blocks.front().textBlock() : blocks.back().textBlock();
-            end = endBlock.position()
-                  + tabSettings.positionAtColumn(endBlock.text(), maxTargetColumn);
-
-            cursor.setPosition(end);
-            cursor.setPosition(start, QTextCursor::KeepAnchor);
-        }
+        cursor.endEditBlock();
+        result.addCursor(cursor);
     }
 
-    cursor.endEditBlock();
-
-    return modified ? cursor : textCursor;
+    return result;
 }
 
 void TextDocumentPrivate::resetRevisions()
@@ -428,6 +368,13 @@ QAction *TextDocument::createDiffAgainstCurrentFileAction(
     return diffAction;
 }
 
+#ifdef WITH_TESTS
+void TextDocument::setSilentReload()
+{
+    d->m_silentReload = true;
+}
+#endif
+
 void TextDocument::triggerPendingUpdates()
 {
     if (d->m_fontSettingsNeedsApply)
@@ -498,16 +445,14 @@ void TextDocument::autoFormatOrIndent(const QTextCursor &cursor)
     d->m_indenter->autoIndent(cursor, tabSettings());
 }
 
-QTextCursor TextDocument::indent(const QTextCursor &cursor, bool blockSelection, int column,
-                                 int *offset)
+Utils::MultiTextCursor TextDocument::indent(const Utils::MultiTextCursor &cursor)
 {
-    return d->indentOrUnindent(cursor, true, tabSettings(), blockSelection, column, offset);
+    return d->indentOrUnindent(cursor, true, tabSettings());
 }
 
-QTextCursor TextDocument::unindent(const QTextCursor &cursor, bool blockSelection, int column,
-                                   int *offset)
+Utils::MultiTextCursor TextDocument::unindent(const Utils::MultiTextCursor &cursor)
 {
-    return d->indentOrUnindent(cursor, false, tabSettings(), blockSelection, column, offset);
+    return d->indentOrUnindent(cursor, false, tabSettings());
 }
 
 void TextDocument::setFormatter(Formatter *formatter)
@@ -520,15 +465,25 @@ void TextDocument::autoFormat(const QTextCursor &cursor)
     using namespace Utils::Text;
     if (!d->m_formatter)
         return;
-    if (QFutureWatcher<Replacements> *watcher = d->m_formatter->format(cursor, tabSettings())) {
-        connect(watcher, &QFutureWatcher<Replacements>::finished, this, [this, watcher]() {
+    if (QFutureWatcher<ChangeSet> *watcher = d->m_formatter->format(cursor, tabSettings())) {
+        connect(watcher, &QFutureWatcher<ChangeSet>::finished, this, [this, watcher]() {
             if (!watcher->isCanceled())
-                Utils::Text::applyReplacements(document(), watcher->result());
+                applyChangeSet(watcher->result());
             delete watcher;
         });
     }
 }
 
+bool TextDocument::applyChangeSet(const ChangeSet &changeSet)
+{
+    if (changeSet.isEmpty())
+        return true;
+    RefactoringChanges changes;
+    const RefactoringFilePtr file = changes.file(filePath());
+    file->setChangeSet(changeSet);
+    return file->apply();
+}
+
 const ExtraEncodingSettings &TextDocument::extraEncodingSettings() const
 {
     return d->m_extraEncodingSettings;
@@ -555,7 +510,7 @@ bool TextDocument::isSaveAsAllowed() const
     return true;
 }
 
-QString TextDocument::fallbackSaveAsPath() const
+FilePath TextDocument::fallbackSaveAsPath() const
 {
     return d->m_defaultPath;
 }
@@ -565,7 +520,7 @@ QString TextDocument::fallbackSaveAsFileName() const
     return d->m_suggestedFileName;
 }
 
-void TextDocument::setFallbackSaveAsPath(const QString &defaultPath)
+void TextDocument::setFallbackSaveAsPath(const FilePath &defaultPath)
 {
     d->m_defaultPath = defaultPath;
 }
@@ -592,7 +547,7 @@ SyntaxHighlighter *TextDocument::syntaxHighlighter() const
  * If \a autoSave is true, the cursor will be restored and some signals suppressed
  * and we do not clean up the text file (cleanWhitespace(), ensureFinalNewLine()).
  */
-bool TextDocument::save(QString *errorString, const QString &saveFileName, bool autoSave)
+bool TextDocument::save(QString *errorString, const FilePath &filePath, bool autoSave)
 {
     QTextCursor cursor(&d->m_document);
 
@@ -630,11 +585,9 @@ bool TextDocument::save(QString *errorString, const QString &saveFileName, bool
         if (d->m_storageSettings.m_addFinalNewLine)
           ensureFinalNewLine(cursor);
         cursor.endEditBlock();
-      }
+    }
 
-    QString fName = filePath().toString();
-    if (!saveFileName.isEmpty())
-        fName = saveFileName;
+    const Utils::FilePath &savePath = filePath.isEmpty() ? this->filePath() : filePath;
 
     // check if UTF8-BOM has to be added or removed
     Utils::TextFileFormat saveFormat = format();
@@ -651,7 +604,7 @@ bool TextDocument::save(QString *errorString, const QString &saveFileName, bool
         }
     }
 
-    const bool ok = write(fName, saveFormat, d->m_document.toPlainText(), errorString);
+    const bool ok = write(savePath, saveFormat, d->m_document.toPlainText(), errorString);
 
     // restore text cursor and scroll bar positions
     if (autoSave && undos < d->m_document.availableUndoSteps()) {
@@ -673,9 +626,8 @@ bool TextDocument::save(QString *errorString, const QString &saveFileName, bool
         return true;
 
     // inform about the new filename
-    const QFileInfo fi(fName);
     d->m_document.setModified(false); // also triggers update of the block revisions
-    setFilePath(Utils::FilePath::fromUserInput(fi.absoluteFilePath()));
+    setFilePath(savePath.absoluteFilePath());
     emit changed();
     return true;
 }
@@ -699,14 +651,14 @@ void TextDocument::setFilePath(const Utils::FilePath &newName)
 {
     if (newName == filePath())
         return;
-    IDocument::setFilePath(Utils::FilePath::fromUserInput(newName.toFileInfo().absoluteFilePath()));
+    IDocument::setFilePath(newName.absoluteFilePath().cleanPath());
 }
 
-bool TextDocument::isFileReadOnly() const
+IDocument::ReloadBehavior TextDocument::reloadBehavior(ChangeTrigger state, ChangeType type) const
 {
-    if (filePath().isEmpty()) //have no corresponding file, so editing is ok
-        return false;
-    return d->m_fileIsReadOnly;
+    if (d->m_silentReload)
+        return IDocument::BehaviorSilent;
+    return BaseTextDocument::reloadBehavior(state, type);
 }
 
 bool TextDocument::isModified() const
@@ -714,46 +666,35 @@ bool TextDocument::isModified() const
     return d->m_document.isModified();
 }
 
-void TextDocument::checkPermissions()
+Core::IDocument::OpenResult TextDocument::open(QString *errorString,
+                                               const Utils::FilePath &filePath,
+                                               const Utils::FilePath &realFilePath)
 {
-    bool previousReadOnly = d->m_fileIsReadOnly;
-    if (!filePath().isEmpty()) {
-        d->m_fileIsReadOnly = !filePath().toFileInfo().isWritable();
-    } else {
-        d->m_fileIsReadOnly = false;
-    }
-    if (previousReadOnly != d->m_fileIsReadOnly)
-        emit changed();
-}
-
-Core::IDocument::OpenResult TextDocument::open(QString *errorString, const QString &fileName,
-                                               const QString &realFileName)
-{
-    emit aboutToOpen(fileName, realFileName);
-    OpenResult success = openImpl(errorString, fileName, realFileName, /*reload =*/ false);
+    emit aboutToOpen(filePath, realFilePath);
+    OpenResult success = openImpl(errorString, filePath, realFilePath, /*reload =*/ false);
     if (success == OpenResult::Success) {
-        setMimeType(Utils::mimeTypeForFile(fileName).name());
+        setMimeType(Utils::mimeTypeForFile(filePath).name());
         emit openFinishedSuccessfully();
     }
     return success;
 }
 
-Core::IDocument::OpenResult TextDocument::openImpl(QString *errorString, const QString &fileName,
-                                                   const QString &realFileName, bool reload)
+Core::IDocument::OpenResult TextDocument::openImpl(QString *errorString,
+                                                   const Utils::FilePath &filePath,
+                                                   const Utils::FilePath &realFilePath,
+                                                   bool reload)
 {
     QStringList content;
 
     ReadResult readResult = Utils::TextFileFormat::ReadIOError;
 
-    if (!fileName.isEmpty()) {
-        const QFileInfo fi(fileName);
-        d->m_fileIsReadOnly = !fi.isWritable();
-        readResult = read(realFileName, &content, errorString);
+    if (!filePath.isEmpty()) {
+        readResult = read(realFilePath, &content, errorString);
         const int chunks = content.size();
 
         // Don't call setUndoRedoEnabled(true) when reload is true and filenames are different,
         // since it will reset the undo's clear index
-        if (!reload || fileName == realFileName)
+        if (!reload || filePath == realFilePath)
             d->m_document.setUndoRedoEnabled(reload);
 
         QTextCursor c(&d->m_document);
@@ -787,7 +728,7 @@ Core::IDocument::OpenResult TextDocument::openImpl(QString *errorString, const Q
 
         // Don't call setUndoRedoEnabled(true) when reload is true and filenames are different,
         // since it will reset the undo's clear index
-        if (!reload || fileName == realFileName)
+        if (!reload || filePath == realFilePath)
             d->m_document.setUndoRedoEnabled(true);
 
         auto documentLayout =
@@ -795,8 +736,8 @@ Core::IDocument::OpenResult TextDocument::openImpl(QString *errorString, const Q
         QTC_ASSERT(documentLayout, return OpenResult::CannotHandle);
         documentLayout->lastSaveRevision = d->m_autoSaveRevision = d->m_document.revision();
         d->updateRevisions();
-        d->m_document.setModified(fileName != realFileName);
-        setFilePath(Utils::FilePath::fromUserInput(fi.absoluteFilePath()));
+        d->m_document.setModified(filePath != realFilePath);
+        setFilePath(filePath);
     }
     if (readResult == Utils::TextFileFormat::ReadIOError)
         return OpenResult::ReadError;
@@ -812,10 +753,10 @@ bool TextDocument::reload(QString *errorString, QTextCodec *codec)
 
 bool TextDocument::reload(QString *errorString)
 {
-    return reload(errorString, filePath().toString());
+    return reload(errorString, filePath());
 }
 
-bool TextDocument::reload(QString *errorString, const QString &realFileName)
+bool TextDocument::reload(QString *errorString, const FilePath &realFilePath)
 {
     emit aboutToReload();
     auto documentLayout =
@@ -824,8 +765,8 @@ bool TextDocument::reload(QString *errorString, const QString &realFileName)
     if (documentLayout)
         marks = documentLayout->documentClosing(); // removes text marks non-permanently
 
-    const QString &file = filePath().toString();
-    bool success = openImpl(errorString, file, realFileName, /*reload =*/ true) == OpenResult::Success;
+    bool success = openImpl(errorString, filePath(), realFilePath, /*reload =*/true)
+                   == OpenResult::Success;
 
     if (documentLayout)
         documentLayout->documentReloaded(marks, this); // re-adds text marks
@@ -864,12 +805,7 @@ bool TextDocument::reload(QString *errorString, ReloadFlag flag, ChangeType type
             modificationChanged(true);
         return true;
     }
-    if (type == TypePermissions) {
-        checkPermissions();
-        return true;
-    } else {
-        return reload(errorString);
-    }
+    return reload(errorString);
 }
 
 void TextDocument::setSyntaxHighlighter(SyntaxHighlighter *highlighter)
@@ -899,7 +835,7 @@ void TextDocument::cleanWhitespace(const QTextCursor &cursor)
 void TextDocument::cleanWhitespace(QTextCursor &cursor, bool inEntireDocument,
                                    bool cleanIndentation)
 {
-    const QString fileName(filePath().fileName());
+    const bool removeTrailingWhitespace = d->m_storageSettings.removeTrailingWhitespace(filePath().fileName());
 
     auto documentLayout = qobject_cast<TextDocumentLayout*>(d->m_document.documentLayout());
     Q_ASSERT(cursor.visualNavigation() == false);
@@ -926,13 +862,13 @@ void TextDocument::cleanWhitespace(QTextCursor &cursor, bool inEntireDocument,
     foreach (block, blocks) {
         QString blockText = block.text();
 
-        if (d->m_storageSettings.removeTrailingWhitespace(fileName))
-            currentTabSettings.removeTrailingWhitespace(cursor, block);
+        if (removeTrailingWhitespace)
+            TabSettings::removeTrailingWhitespace(cursor, block);
 
         const int indent = indentations[block.blockNumber()];
         if (cleanIndentation && !currentTabSettings.isIndentationClean(block, indent)) {
             cursor.setPosition(block.position());
-            int firstNonSpace = currentTabSettings.firstNonSpace(blockText);
+            const int firstNonSpace = TabSettings::firstNonSpace(blockText);
             if (firstNonSpace == blockText.length()) {
                 cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
                 cursor.removeSelectedText();
@@ -1040,7 +976,8 @@ void TextDocument::removeMarkFromMarksCache(TextMark *mark)
     auto scheduleLayoutUpdate = [documentLayout](){
         // make sure all destructors that may directly or indirectly call this function are
         // completed before updating.
-        QTimer::singleShot(0, documentLayout, &QPlainTextDocumentLayout::requestUpdate);
+        QMetaObject::invokeMethod(documentLayout, &QPlainTextDocumentLayout::requestUpdate,
+                                  Qt::QueuedConnection);
     };
 
     if (d->m_marksCache.isEmpty()) {
