@@ -387,24 +387,24 @@ TextDocumentLayout::~TextDocumentLayout()
 
 void TextDocumentLayout::setParentheses(const QTextBlock &block, const Parentheses &parentheses)
 {
-    if (parentheses.isEmpty()) {
-        if (TextBlockUserData *userData = testUserData(block))
-            userData->clearParentheses();
-    } else {
-        userData(block)->setParentheses(parentheses);
-    }
+    if (TextDocumentLayout::parentheses(block) == parentheses)
+        return;
+
+    userData(block)->setParentheses(parentheses);
+    if (auto layout = qobject_cast<TextDocumentLayout *>(block.document()->documentLayout()))
+        emit layout->parenthesesChanged(block);
 }
 
 Parentheses TextDocumentLayout::parentheses(const QTextBlock &block)
 {
-    if (TextBlockUserData *userData = testUserData(block))
+    if (TextBlockUserData *userData = textUserData(block))
         return userData->parentheses();
     return Parentheses();
 }
 
 bool TextDocumentLayout::hasParentheses(const QTextBlock &block)
 {
-    if (TextBlockUserData *userData = testUserData(block))
+    if (TextBlockUserData *userData = textUserData(block))
         return userData->hasParentheses();
     return false;
 }
@@ -416,21 +416,21 @@ bool TextDocumentLayout::setIfdefedOut(const QTextBlock &block)
 
 bool TextDocumentLayout::clearIfdefedOut(const QTextBlock &block)
 {
-    if (TextBlockUserData *userData = testUserData(block))
+    if (TextBlockUserData *userData = textUserData(block))
         return userData->clearIfdefedOut();
     return false;
 }
 
 bool TextDocumentLayout::ifdefedOut(const QTextBlock &block)
 {
-    if (TextBlockUserData *userData = testUserData(block))
+    if (TextBlockUserData *userData = textUserData(block))
         return userData->ifdefedOut();
     return false;
 }
 
 int TextDocumentLayout::braceDepthDelta(const QTextBlock &block)
 {
-    if (TextBlockUserData *userData = testUserData(block))
+    if (TextBlockUserData *userData = textUserData(block))
         return userData->braceDepthDelta();
     return 0;
 }
@@ -461,7 +461,7 @@ void TextDocumentLayout::changeBraceDepth(QTextBlock &block, int delta)
 void TextDocumentLayout::setLexerState(const QTextBlock &block, int state)
 {
     if (state == 0) {
-        if (TextBlockUserData *userData = testUserData(block))
+        if (TextBlockUserData *userData = textUserData(block))
             userData->setLexerState(0);
     } else {
         userData(block)->setLexerState(qMax(0,state));
@@ -470,7 +470,7 @@ void TextDocumentLayout::setLexerState(const QTextBlock &block, int state)
 
 int TextDocumentLayout::lexerState(const QTextBlock &block)
 {
-    if (TextBlockUserData *userData = testUserData(block))
+    if (TextBlockUserData *userData = textUserData(block))
         return userData->lexerState();
     return 0;
 }
@@ -478,7 +478,7 @@ int TextDocumentLayout::lexerState(const QTextBlock &block)
 void TextDocumentLayout::setFoldingIndent(const QTextBlock &block, int indent)
 {
     if (indent == 0) {
-        if (TextBlockUserData *userData = testUserData(block))
+        if (TextBlockUserData *userData = textUserData(block))
             userData->setFoldingIndent(0);
     } else {
         userData(block)->setFoldingIndent(indent);
@@ -487,7 +487,7 @@ void TextDocumentLayout::setFoldingIndent(const QTextBlock &block, int indent)
 
 int TextDocumentLayout::foldingIndent(const QTextBlock &block)
 {
-    if (TextBlockUserData *userData = testUserData(block))
+    if (TextBlockUserData *userData = textUserData(block))
         return userData->foldingIndent();
     return 0;
 }
@@ -505,7 +505,7 @@ bool TextDocumentLayout::canFold(const QTextBlock &block)
 
 bool TextDocumentLayout::isFolded(const QTextBlock &block)
 {
-    if (TextBlockUserData *userData = testUserData(block))
+    if (TextBlockUserData *userData = textUserData(block))
         return userData->folded();
     return false;
 }
@@ -514,7 +514,7 @@ void TextDocumentLayout::setFolded(const QTextBlock &block, bool folded)
 {
     if (folded)
         userData(block)->setFolded(true);
-    else if (TextBlockUserData *userData = testUserData(block))
+    else if (TextBlockUserData *userData = textUserData(block))
         userData->setFolded(false);
     else
         return;
@@ -605,7 +605,7 @@ void TextDocumentLayout::updateMarksLineNumber()
     QTextBlock block = document()->begin();
     int blockNumber = 0;
     while (block.isValid()) {
-        if (const TextBlockUserData *userData = testUserData(block)) {
+        if (const TextBlockUserData *userData = textUserData(block)) {
             for (TextMark *mrk : userData->marks())
                 mrk->updateLineNumber(blockNumber + 1);
         }
@@ -616,7 +616,7 @@ void TextDocumentLayout::updateMarksLineNumber()
 
 void TextDocumentLayout::updateMarksBlock(const QTextBlock &block)
 {
-    if (const TextBlockUserData *userData = testUserData(block)) {
+    if (const TextBlockUserData *userData = textUserData(block)) {
         for (TextMark *mrk : userData->marks())
             mrk->updateBlock(block);
     }
@@ -625,7 +625,7 @@ void TextDocumentLayout::updateMarksBlock(const QTextBlock &block)
 QRectF TextDocumentLayout::blockBoundingRect(const QTextBlock &block) const
 {
     QRectF boundingRect = QPlainTextDocumentLayout::blockBoundingRect(block);
-    if (TextBlockUserData *userData = testUserData(block))
+    if (TextBlockUserData *userData = textUserData(block))
         boundingRect.adjust(0, 0, 0, userData->additionalAnnotationHeight());
     return boundingRect;
 }
@@ -684,4 +684,25 @@ void TextDocumentLayout::FoldValidator::finalize()
     }
 }
 
+QDebug operator<<(QDebug debug, const Parenthesis &parenthesis)
+{
+    QDebugStateSaver saver(debug);
+    debug << (parenthesis.type == Parenthesis::Opened ? "Opening " : "Closing ") << parenthesis.chr
+          << " at " << parenthesis.pos;
+
+    return debug;
+}
+
+bool Parenthesis::operator==(const Parenthesis &other) const
+{
+    return pos == other.pos && chr == other.chr && source == other.source && type == other.type;
+}
+
+void insertSorted(Parentheses &list, const Parenthesis &elem)
+{
+    const auto it = std::lower_bound(list.begin(), list.end(), elem,
+            [](const auto &p1, const auto &p2) { return p1.pos < p2.pos; });
+    list.insert(it, elem);
+}
+
 } // namespace TextEditor
