@@ -30,14 +30,14 @@
 #include "textmark.h"
 #include "textdocument.h"
 
+#include <utils/id.h>
+
 #include <State>
 
 #include <QTextBlockUserData>
 #include <QPlainTextDocumentLayout>
 
 namespace TextEditor {
-struct Parenthesis;
-using Parentheses = QVector<Parenthesis>;
 
 struct TEXTEDITOR_EXPORT Parenthesis
 {
@@ -48,8 +48,15 @@ struct TEXTEDITOR_EXPORT Parenthesis
         : pos(position), chr(c), type(t) {}
     int pos = -1;
     QChar chr;
+    Utils::Id source;
     Type type = Opened;
+
+    bool operator==(const Parenthesis &other) const;
 };
+using Parentheses = QVector<Parenthesis>;
+TEXTEDITOR_EXPORT void insertSorted(Parentheses &list, const Parenthesis &elem);
+
+TEXTEDITOR_EXPORT QDebug operator<<(QDebug debug, const Parenthesis &parenthesis);
 
 class TEXTEDITOR_EXPORT CodeFormatterData
 {
@@ -228,6 +235,7 @@ public:
 signals:
     void updateExtraArea();
     void foldChanged(const int blockNumber, bool folded);
+    void parenthesesChanged(const QTextBlock block);
 };
 
 } // namespace TextEditor
