@@ -54,7 +54,9 @@ const char *nameForStyle(TextStyle style)
     case C_NUMBER:              return "Number";
     case C_STRING:              return "String";
     case C_TYPE:                return "Type";
+    case C_NAMESPACE:           return "Namespace";
     case C_LOCAL:               return "Local";
+    case C_PARAMETER:           return "Parameter";
     case C_GLOBAL:              return "Global";
     case C_FIELD:               return "Field";
     // TODO: Rename "Static" to "Enumeration" in next major update,
@@ -113,6 +115,7 @@ const char *nameForStyle(TextStyle style)
     case C_DECLARATION:         return "Declaration";
     case C_FUNCTION_DEFINITION: return "FunctionDefinition";
     case C_OUTPUT_ARGUMENT:     return "OutputArgument";
+    case C_STATIC_MEMBER:       return "StaticMember";
 
     case C_LAST_STYLE_SENTINEL: return "LastStyleSentinel";
     }
