@@ -30,22 +30,24 @@
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/icore.h>
-#include <coreplugin/infobar.h>
 #include <coreplugin/settingsdatabase.h>
 #include <coreplugin/shellcommand.h>
+#include <utils/algorithm.h>
 #include <utils/fileutils.h>
-#include <utils/synchronousprocess.h>
+#include <utils/infobar.h>
+#include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
 
 #include <QDate>
 #include <QDomDocument>
 #include <QFile>
 #include <QFileInfo>
+#include <QLabel>
 #include <QMenu>
 #include <QMetaEnum>
 #include <QPointer>
 #include <QProcessEnvironment>
 #include <QTimer>
-#include <QtPlugin>
 
 namespace {
     static const char UpdaterGroup[] = "Updater";
@@ -72,8 +74,12 @@ public:
     QString m_collectedOutput;
     QTimer *m_checkUpdatesTimer = nullptr;
 
-    bool m_automaticCheck = true;
-    UpdateInfoPlugin::CheckUpdateInterval m_checkInterval = UpdateInfoPlugin::WeeklyCheck;
+    struct Settings
+    {
+        bool automaticCheck = true;
+        UpdateInfoPlugin::CheckUpdateInterval checkInterval = UpdateInfoPlugin::WeeklyCheck;
+    };
+    Settings m_settings;
     QDate m_lastCheckDate;
 };
 
@@ -123,16 +129,17 @@ void UpdateInfoPlugin::startCheckForUpdates()
 {
     stopCheckForUpdates();
 
-    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-    env.insert(QLatin1String("QT_LOGGING_RULES"), QLatin1String("*=false"));
-    d->m_checkUpdatesCommand = new ShellCommand(QString(), env);
+    d->m_checkUpdatesCommand = new ShellCommand({}, Utils::Environment::systemEnvironment());
     d->m_checkUpdatesCommand->setDisplayName(tr("Checking for Updates"));
     connect(d->m_checkUpdatesCommand, &ShellCommand::stdOutText, this, &UpdateInfoPlugin::collectCheckForUpdatesOutput);
     connect(d->m_checkUpdatesCommand, &ShellCommand::finished, this, &UpdateInfoPlugin::checkForUpdatesFinished);
-    d->m_checkUpdatesCommand->addJob({Utils::FilePath::fromFileInfo(d->m_maintenanceTool), {"--checkupdates"}},
+    d->m_checkUpdatesCommand->addJob({Utils::FilePath::fromString(d->m_maintenanceTool),
+                                      {"ch", "-g", "*=false,ifw.package.*=true"}},
                                      60 * 3, // 3 minutes timeout
-                                     /*workingDirectory=*/QString(),
-                                     [](int /*exitCode*/) { return Utils::SynchronousProcessResponse::Finished; });
+                                     /*workingDirectory=*/{},
+                                     [](int /*exitCode*/) {
+                                         return Utils::QtcProcess::FinishedWithSuccess;
+                                     });
     d->m_checkUpdatesCommand->execute();
     d->m_progress = d->m_checkUpdatesCommand->futureProgress();
     if (d->m_progress) {
@@ -159,18 +166,24 @@ void UpdateInfoPlugin::collectCheckForUpdatesOutput(const QString &contents)
     d->m_collectedOutput += contents;
 }
 
-static QStringList availableUpdates(const QDomDocument &document)
+struct Update
+{
+    QString name;
+    QString version;
+};
+
+static QList<Update> availableUpdates(const QDomDocument &document)
 {
     if (document.isNull() || !document.firstChildElement().hasChildNodes())
         return {};
-    QStringList result;
+    QList<Update> result;
     const QDomNodeList updates = document.firstChildElement().elementsByTagName("update");
     for (int i = 0; i < updates.size(); ++i) {
         const QDomNode node = updates.item(i);
         if (node.isElement()) {
             const QDomElement element = node.toElement();
             if (element.hasAttribute("name"))
-                result.append(element.attribute("name"));
+                result.append({element.attribute("name"), element.attribute("version")});
         }
     }
     return result;
@@ -192,21 +205,26 @@ void UpdateInfoPlugin::checkForUpdatesFinished()
         if (d->m_progress)
             d->m_progress->setKeepOnFinish(FutureProgress::HideOnFinish);
         emit newUpdatesAvailable(true);
-        Core::InfoBarEntry info(InstallUpdates,
-                                tr("New updates are available. Start the update?"));
+        Utils::InfoBarEntry info(InstallUpdates, tr("New updates are available. Start the update?"));
         info.setCustomButtonInfo(tr("Start Update"), [this] {
             Core::ICore::infoBar()->removeInfo(InstallUpdates);
             startUpdater();
         });
-        const QStringList updates = availableUpdates(document);
+        const QList<Update> updates = availableUpdates(document);
         info.setDetailsWidgetCreator([updates]() -> QWidget * {
-            const QString updateText = updates.join("</li><li>");
+            const QString updateText = Utils::transform(updates, [](const Update &u) {
+                                           return u.version.isEmpty()
+                                                      ? u.name
+                                                      : tr("%1 (%2)", "Package name and version")
+                                                            .arg(u.name, u.version);
+                                       }).join("</li><li>");
             auto label = new QLabel;
             label->setText("<qt><p>" + tr("Available updates:") + "<ul><li>" + updateText
                            + "</li></ul></p></qt>");
             label->setContentsMargins(0, 0, 0, 8);
             return label;
         });
+        Core::ICore::infoBar()->removeInfo(InstallUpdates); // remove any existing notifications
         Core::ICore::infoBar()->unsuppressInfo(InstallUpdates);
         Core::ICore::infoBar()->addInfo(info);
     } else {
@@ -261,49 +279,59 @@ bool UpdateInfoPlugin::initialize(const QStringList & /* arguments */, QString *
 
 void UpdateInfoPlugin::loadSettings() const
 {
+    UpdateInfoPluginPrivate::Settings def;
     QSettings *settings = ICore::settings();
-    const QString updaterKey = QLatin1String(UpdaterGroup) + QLatin1Char('/');
-    d->m_maintenanceTool = settings->value(updaterKey + QLatin1String(MaintenanceToolKey)).toString();
-    d->m_lastCheckDate = settings->value(updaterKey + QLatin1String(LastCheckDateKey), QDate()).toDate();
-    d->m_automaticCheck = settings->value(updaterKey + QLatin1String(AutomaticCheckKey), true).toBool();
-    const QString checkInterval = settings->value(updaterKey + QLatin1String(CheckIntervalKey)).toString();
+    const QString updaterKey = QLatin1String(UpdaterGroup) + '/';
+    d->m_maintenanceTool = settings->value(updaterKey + MaintenanceToolKey).toString();
+    d->m_lastCheckDate = settings->value(updaterKey + LastCheckDateKey, QDate()).toDate();
+    d->m_settings.automaticCheck
+        = settings->value(updaterKey + AutomaticCheckKey, def.automaticCheck).toBool();
     const QMetaObject *mo = metaObject();
     const QMetaEnum me = mo->enumerator(mo->indexOfEnumerator(CheckIntervalKey));
-    if (me.isValid()) {
+    if (QTC_GUARD(me.isValid())) {
+        const QString checkInterval = settings
+                                          ->value(updaterKey + CheckIntervalKey,
+                                                  me.valueToKey(def.checkInterval))
+                                          .toString();
         bool ok = false;
         const int newValue = me.keyToValue(checkInterval.toUtf8(), &ok);
         if (ok)
-            d->m_checkInterval = static_cast<CheckUpdateInterval>(newValue);
+            d->m_settings.checkInterval = static_cast<CheckUpdateInterval>(newValue);
     }
 }
 
 void UpdateInfoPlugin::saveSettings()
 {
-    QSettings *settings = ICore::settings();
-    settings->beginGroup(QLatin1String(UpdaterGroup));
-    settings->setValue(QLatin1String(LastCheckDateKey), d->m_lastCheckDate);
-    settings->setValue(QLatin1String(AutomaticCheckKey), d->m_automaticCheck);
+    UpdateInfoPluginPrivate::Settings def;
+    Utils::QtcSettings *settings = ICore::settings();
+    settings->beginGroup(UpdaterGroup);
+    settings->setValueWithDefault(LastCheckDateKey, d->m_lastCheckDate, QDate());
+    settings->setValueWithDefault(AutomaticCheckKey,
+                                  d->m_settings.automaticCheck,
+                                  def.automaticCheck);
     // Note: don't save MaintenanceToolKey on purpose! This setting may be set only by installer.
     // If creator is run not from installed SDK, the setting can be manually created here:
     // [CREATOR_INSTALLATION_LOCATION]/share/qtcreator/QtProject/QtCreator.ini or
     // [CREATOR_INSTALLATION_LOCATION]/Qt Creator.app/Contents/Resources/QtProject/QtCreator.ini on OS X
     const QMetaObject *mo = metaObject();
     const QMetaEnum me = mo->enumerator(mo->indexOfEnumerator(CheckIntervalKey));
-    settings->setValue(QLatin1String(CheckIntervalKey), QLatin1String(me.valueToKey(d->m_checkInterval)));
+    settings->setValueWithDefault(CheckIntervalKey,
+                                  QString::fromUtf8(me.valueToKey(d->m_settings.checkInterval)),
+                                  QString::fromUtf8(me.valueToKey(def.checkInterval)));
     settings->endGroup();
 }
 
 bool UpdateInfoPlugin::isAutomaticCheck() const
 {
-    return d->m_automaticCheck;
+    return d->m_settings.automaticCheck;
 }
 
 void UpdateInfoPlugin::setAutomaticCheck(bool on)
 {
-    if (d->m_automaticCheck == on)
+    if (d->m_settings.automaticCheck == on)
         return;
 
-    d->m_automaticCheck = on;
+    d->m_settings.automaticCheck = on;
     if (on)
         startAutoCheckForUpdates();
     else
@@ -312,15 +340,15 @@ void UpdateInfoPlugin::setAutomaticCheck(bool on)
 
 UpdateInfoPlugin::CheckUpdateInterval UpdateInfoPlugin::checkUpdateInterval() const
 {
-    return d->m_checkInterval;
+    return d->m_settings.checkInterval;
 }
 
 void UpdateInfoPlugin::setCheckUpdateInterval(UpdateInfoPlugin::CheckUpdateInterval interval)
 {
-    if (d->m_checkInterval == interval)
+    if (d->m_settings.checkInterval == interval)
         return;
 
-    d->m_checkInterval = interval;
+    d->m_settings.checkInterval = interval;
 }
 
 QDate UpdateInfoPlugin::lastCheckDate() const
@@ -339,7 +367,7 @@ void UpdateInfoPlugin::setLastCheckDate(const QDate &date)
 
 QDate UpdateInfoPlugin::nextCheckDate() const
 {
-    return nextCheckDate(d->m_checkInterval);
+    return nextCheckDate(d->m_settings.checkInterval);
 }
 
 QDate UpdateInfoPlugin::nextCheckDate(CheckUpdateInterval interval) const
@@ -356,7 +384,7 @@ QDate UpdateInfoPlugin::nextCheckDate(CheckUpdateInterval interval) const
 
 void UpdateInfoPlugin::startUpdater()
 {
-    QProcess::startDetached(d->m_maintenanceTool, QStringList(QLatin1String("--updater")));
+    Utils::QtcProcess::startDetached({Utils::FilePath::fromString(d->m_maintenanceTool), {"--updater"}});
 }
 
 } //namespace Internal
