@@ -27,9 +27,7 @@
 
 #include <QObject>
 
-QT_BEGIN_NAMESPACE
-class QIODevice;
-QT_END_NAMESPACE
+namespace Utils { class FilePath; }
 
 namespace Valgrind {
 namespace Callgrind {
@@ -50,19 +48,17 @@ class Parser : public QObject
     Q_OBJECT
 
 public:
-    explicit Parser(QObject *parent = 0);
-    ~Parser();
+    Parser();
+    ~Parser() override;
 
     // get and take ownership of the parsing results. If this function is not called the repository
     // will be destroyed when the parser is destroyed. Subsequent calls return null.
     ParseData *takeData();
+    void parse(const Utils::FilePath &filePath);
 
 signals:
     void parserDataReady();
 
-public Q_SLOTS:
-    void parse(QIODevice *stream);
-
 private:
     class Private;
     Private *const d;
