@@ -133,8 +133,6 @@ public:
 
     void selectFunction(const Function *);
     void setCostFormat(CostDelegate::CostFormat format);
-    void enableCycleDetection(bool enabled);
-    void shortenTemplates(bool enabled);
     void setCostEvent(int index);
 
     /// This function will add custom text marks to the editor
@@ -191,16 +189,14 @@ public:
     // Navigation
     QAction *m_goBack = nullptr;
     QAction *m_goNext = nullptr;
-    QLineEdit *m_searchFilter = nullptr;
+    QPointer<QLineEdit> m_searchFilter = nullptr;
 
     // Cost formatting
     QAction *m_filterProjectCosts = nullptr;
     QAction *m_costAbsolute = nullptr;
     QAction *m_costRelative = nullptr;
     QAction *m_costRelativeToParent = nullptr;
-    QAction *m_cycleDetection = nullptr;
-    QAction *m_shortenTemplates = nullptr;
-    QComboBox *m_eventCombo = nullptr;
+    QPointer<QComboBox> m_eventCombo;
 
     QTimer m_updateTimer;
 
@@ -290,7 +286,7 @@ CallgrindToolPrivate::CallgrindToolPrivate()
         runControl->createMainWorker();
         const auto runnable = dlg.runnable();
         runControl->setRunnable(runnable);
-        runControl->setDisplayName(runnable.executable.toUserOutput());
+        runControl->setDisplayName(runnable.command.executable().toUserOutput());
         ProjectExplorerPlugin::startRunControl(runControl);
     });
 
@@ -382,7 +378,7 @@ CallgrindToolPrivate::CallgrindToolPrivate()
     action->setIcon(kCachegrindIcon.icon());
     action->setToolTip(CallgrindTool::tr("Open results in KCachegrind."));
     connect(action, &QAction::triggered, this, [this, settings] {
-        QProcess::startDetached(settings->kcachegrindExecutable(), { m_lastFileName });
+        QtcProcess::startDetached({FilePath::fromString(settings->kcachegrindExecutable.value()), { m_lastFileName }});
     });
 
     // dump action
@@ -488,40 +484,20 @@ CallgrindToolPrivate::CallgrindToolPrivate()
     m_perspective.addToolBarWidget(button);
     }
 
-    // Cycle detection
-    //action = new QAction("Cycle Detection", this); ///FIXME: icon
-    action = m_cycleDetection = new QAction("O", this); ///FIXME: icon
-    action->setToolTip(CallgrindTool::tr("Enable cycle detection to properly handle recursive or circular function calls."));
-    action->setCheckable(true);
-    connect(action, &QAction::toggled, &m_dataModel, &DataModel::enableCycleDetection);
-    connect(action, &QAction::toggled, settings, &ValgrindGlobalSettings::setDetectCycles);
-
-    // Shorter template signature
-    action = m_shortenTemplates = new QAction("<>", this);
-    action->setToolTip(CallgrindTool::tr("Remove template parameter lists when displaying function names."));
-    action->setCheckable(true);
-    connect(action, &QAction::toggled, &m_dataModel, &DataModel::setShortenTemplates);
-    connect(action, &QAction::toggled, settings, &ValgrindGlobalSettings::setShortenTemplates);
-
     // Filtering
-    action = m_filterProjectCosts = new QAction(CallgrindTool::tr("Show Project Costs Only"), this);
-    action->setIcon(Utils::Icons::FILTER.icon());
-    action->setToolTip(CallgrindTool::tr("Show only profiling info that originated from this project source."));
-    action->setCheckable(true);
+    action = m_filterProjectCosts = settings->filterExternalIssues.action();
     connect(action, &QAction::toggled, this, &CallgrindToolPrivate::handleFilterProjectCosts);
 
     // Filter
-    ///FIXME: find workaround for https://bugreports.qt.io/browse/QTCREATORBUG-3247
     m_searchFilter = new QLineEdit;
     m_searchFilter->setPlaceholderText(CallgrindTool::tr("Filter..."));
     connect(m_searchFilter, &QLineEdit::textChanged,
             &m_updateTimer, QOverload<>::of(&QTimer::start));
 
-    setCostFormat(settings->costFormat());
-    enableCycleDetection(settings->detectCycles());
+    setCostFormat(CostDelegate::CostFormat(settings->costFormat.value()));
 
-    m_perspective.addToolBarAction(m_cycleDetection);
-    m_perspective.addToolBarAction(m_shortenTemplates);
+    m_perspective.addToolBarAction(settings->detectCycles.action());
+    m_perspective.addToolBarAction(settings->shortenTemplates.action());
     m_perspective.addToolBarAction(m_filterProjectCosts);
     m_perspective.addToolBarWidget(m_searchFilter);
 
@@ -570,8 +546,10 @@ void CallgrindToolPrivate::setBusyCursor(bool busy)
 void CallgrindToolPrivate::selectFunction(const Function *func)
 {
     if (!func) {
-        m_flatView->clearSelection();
-        m_visualization->setFunction(nullptr);
+        if (m_flatView)
+            m_flatView->clearSelection();
+        if (m_visualization)
+            m_visualization->setFunction(nullptr);
         m_callersModel.clear();
         m_calleesModel.clear();
         return;
@@ -579,15 +557,18 @@ void CallgrindToolPrivate::selectFunction(const Function *func)
 
     const QModelIndex index = m_dataModel.indexForObject(func);
     const QModelIndex proxyIndex = m_proxyModel.mapFromSource(index);
-    m_flatView->selectionModel()->clearSelection();
-    m_flatView->selectionModel()->setCurrentIndex(proxyIndex,
-                                                  QItemSelectionModel::ClearAndSelect |
-                                                  QItemSelectionModel::Rows);
-    m_flatView->scrollTo(proxyIndex);
+    if (m_flatView) {
+        m_flatView->selectionModel()->clearSelection();
+        m_flatView->selectionModel()->setCurrentIndex(proxyIndex,
+                                                      QItemSelectionModel::ClearAndSelect |
+                                                      QItemSelectionModel::Rows);
+        m_flatView->scrollTo(proxyIndex);
+    }
 
     m_callersModel.setCalls(func->incomingCalls(), func);
     m_calleesModel.setCalls(func->outgoingCalls(), func);
-    m_visualization->setFunction(func);
+    if (m_visualization)
+        m_visualization->setFunction(func);
 
     const Function *item = m_stackBrowser.current();
     if (!item || item != func)
@@ -640,16 +621,6 @@ void CallgrindToolPrivate::setCostEvent(int index)
     m_callersModel.setCostEvent(index);
 }
 
-void CallgrindToolPrivate::enableCycleDetection(bool enabled)
-{
-    m_cycleDetection->setChecked(enabled);
-}
-
-void CallgrindToolPrivate::shortenTemplates(bool enabled)
-{
-    m_shortenTemplates->setChecked(enabled);
-}
-
 // Following functions can be called with actions=0 or widgets=0
 // depending on initialization sequence (whether callgrind was current).
 CostDelegate::CostFormat CallgrindToolPrivate::costFormat() const
@@ -671,7 +642,7 @@ void CallgrindToolPrivate::updateCostFormat()
         m_callersView->setCostFormat(format);
     }
     if (ValgrindGlobalSettings *settings = ValgrindGlobalSettings::instance())
-        settings->setCostFormat(format);
+        settings->costFormat.setValue(format);
 }
 
 void CallgrindToolPrivate::handleFilterProjectCosts()
@@ -722,7 +693,8 @@ void CallgrindToolPrivate::visualisationFunctionSelected(const Function *functio
 void CallgrindToolPrivate::setParseData(ParseData *data)
 {
     // we have new parse data, invalidate filters in the proxy model
-    m_visualization->setFunction(nullptr);
+    if (m_visualization)
+        m_visualization->setFunction(nullptr);
 
     // invalidate parse data in the data model
     delete m_dataModel.parseData();
@@ -738,7 +710,8 @@ void CallgrindToolPrivate::setParseData(ParseData *data)
     m_calleesModel.setParseData(data);
     m_callersModel.setParseData(data);
 
-    updateEventCombo();
+    if (m_eventCombo)
+        updateEventCombo();
 
     // clear history for new data
     m_stackBrowser.clear();
@@ -789,9 +762,9 @@ void CallgrindToolPrivate::setupRunner(CallgrindToolRunner *toolRunner)
     // apply project settings
     ValgrindProjectSettings settings;
     settings.fromMap(runControl->settingsData(ANALYZER_VALGRIND_SETTINGS));
-    m_visualization->setMinimumInclusiveCostRatio(settings.visualisationMinimumInclusiveCostRatio() / 100.0);
-    m_proxyModel.setMinimumInclusiveCostRatio(settings.minimumInclusiveCostRatio() / 100.0);
-    m_dataModel.setVerboseToolTipsEnabled(settings.enableEventToolTips());
+    m_visualization->setMinimumInclusiveCostRatio(settings.visualizationMinimumInclusiveCostRatio.value() / 100.0);
+    m_proxyModel.setMinimumInclusiveCostRatio(settings.minimumInclusiveCostRatio.value() / 100.0);
+    m_dataModel.setVerboseToolTipsEnabled(settings.enableEventToolTips.value());
 
     m_toolBusy = true;
     updateRunActions();
@@ -908,17 +881,18 @@ void CallgrindToolPrivate::slotRequestDump()
 
 void CallgrindToolPrivate::loadExternalLogFile()
 {
-    const QString filePath = QFileDialog::getOpenFileName(
-                ICore::dialogParent(),
+    const FilePath filePath = FileUtils::getOpenFilePath(
+                nullptr,
                 CallgrindTool::tr("Open Callgrind Log File"),
-                QString(),
+                {},
                 CallgrindTool::tr("Callgrind Output (callgrind.out*);;All Files (*)"));
     if (filePath.isEmpty())
         return;
 
-    QFile logFile(filePath);
+    QFile logFile(filePath.toString());
     if (!logFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
-        QString msg = CallgrindTool::tr("Callgrind: Failed to open file for reading: %1").arg(filePath);
+        QString msg = CallgrindTool::tr("Callgrind: Failed to open file for reading: %1")
+                .arg(filePath.toUserOutput());
         TaskHub::addTask(Task::Error, msg, Debugger::Constants::ANALYZERTASK_ID);
         TaskHub::requestPopup();
         return;
@@ -928,7 +902,7 @@ void CallgrindToolPrivate::loadExternalLogFile()
     QCoreApplication::processEvents();
 
     Parser parser;
-    parser.parse(&logFile);
+    parser.parse(filePath);
     takeParserData(parser.takeData());
 }
 
@@ -949,7 +923,8 @@ void CallgrindToolPrivate::takeParserData(ParseData *data)
     doClear(true);
 
     setParseData(data);
-    const QString kcachegrindExecutable = ValgrindGlobalSettings::instance()->kcachegrindExecutable();
+    const QString kcachegrindExecutable =
+            ValgrindGlobalSettings::instance()->kcachegrindExecutable.value();
     const bool kcachegrindExists = !Utils::Environment::systemEnvironment().searchInPath(
                 kcachegrindExecutable).isEmpty();
     m_startKCachegrind->setEnabled(kcachegrindExists && !m_lastFileName.isEmpty());
