@@ -128,7 +128,7 @@ public:
     void start() override;
     void stop() override;
 
-    QStringList suppressionFiles() const;
+    const Utils::FilePaths suppressionFiles() const;
 
 signals:
     void internalParserError(const QString &errorString);
@@ -191,14 +191,14 @@ QStringList MemcheckToolRunner::toolArguments() const
 {
     QStringList arguments = {"--tool=memcheck", "--gen-suppressions=all"};
 
-    if (m_settings.trackOrigins())
+    if (m_settings.trackOrigins.value())
         arguments << "--track-origins=yes";
 
-    if (m_settings.showReachable())
+    if (m_settings.showReachable.value())
         arguments << "--show-reachable=yes";
 
     QString leakCheckValue;
-    switch (m_settings.leakCheckOnFinish()) {
+    switch (m_settings.leakCheckOnFinish.value()) {
     case ValgrindBaseSettings::LeakCheckOnFinishNo:
         leakCheckValue = "no";
         break;
@@ -212,20 +212,22 @@ QStringList MemcheckToolRunner::toolArguments() const
     }
     arguments << "--leak-check=" + leakCheckValue;
 
-    foreach (const QString &file, m_settings.suppressionFiles())
-        arguments << QString("--suppressions=%1").arg(file);
+    for (const FilePath &file : m_settings.suppressions.value())
+        arguments << QString("--suppressions=%1").arg(file.path());
 
-    arguments << QString("--num-callers=%1").arg(m_settings.numCallers());
+    arguments << QString("--num-callers=%1").arg(m_settings.numCallers.value());
 
     if (m_withGdb)
         arguments << "--vgdb=yes" << "--vgdb-error=0";
 
+    arguments << Utils::ProcessArgs::splitArgs(m_settings.memcheckArguments.value());
+
     return arguments;
 }
 
-QStringList MemcheckToolRunner::suppressionFiles() const
+const FilePaths MemcheckToolRunner::suppressionFiles() const
 {
-    return m_settings.suppressionFiles();
+    return m_settings.suppressions.value();
 }
 
 void MemcheckToolRunner::startDebugger(qint64 valgrindPid)
@@ -690,7 +692,7 @@ MemcheckToolPrivate::MemcheckToolPrivate()
         rc->createMainWorker();
         const auto runnable = dlg.runnable();
         rc->setRunnable(runnable);
-        rc->setDisplayName(runnable.executable.toUserOutput());
+        rc->setDisplayName(runnable.command.executable().toUserOutput());
         ProjectExplorerPlugin::startRunControl(rc);
     });
 
@@ -749,9 +751,9 @@ void MemcheckToolPrivate::heobAction()
         return;
     }
 
-    QString executable = sr.executable.toString();
-    const QString workingDirectory = Utils::FileUtils::normalizePathName(sr.workingDirectory);
-    const QString commandLineArguments = sr.commandLineArguments;
+    FilePath executable = sr.command.executable();
+    const QString workingDirectory = sr.workingDirectory.normalizedPathName().toString();
+    const QString commandLineArguments = sr.command.arguments();
     const QStringList envStrings = sr.environment.toStringList();
 
     // target executable
@@ -761,10 +763,10 @@ void MemcheckToolPrivate::heobAction()
         TaskHub::requestPopup();
         return;
     }
-    if (!QFile::exists(executable))
-        executable = Utils::HostOsInfo::withExecutableSuffix(executable);
-    if (!QFile::exists(executable)) {
-        const QString msg = MemcheckTool::tr("Heob: Cannot find %1.").arg(executable);
+    if (!executable.exists())
+        executable = executable.withExecutableSuffix();
+    if (!executable.exists()) {
+        const QString msg = MemcheckTool::tr("Heob: Cannot find %1.").arg(executable.toUserOutput());
         TaskHub::addTask(Task::Error, msg, Debugger::Constants::ANALYZERTASK_ID);
         TaskHub::requestPopup();
         return;
@@ -772,8 +774,11 @@ void MemcheckToolPrivate::heobAction()
 
     // make executable a relative path if possible
     const QString wdSlashed = workingDirectory + '/';
-    if (executable.startsWith(wdSlashed, Qt::CaseInsensitive))
-        executable.remove(0, wdSlashed.size());
+    QString executablePath = executable.path();
+    if (executablePath.startsWith(wdSlashed, Qt::CaseInsensitive)) {
+        executablePath.remove(0, wdSlashed.size());
+        executable.setPath(executablePath);
+    }
 
     // heob arguments
     HeobDialog dialog(Core::ICore::dialogParent());
@@ -820,7 +825,7 @@ void MemcheckToolPrivate::heobAction()
     QFile::remove(xmlPath);
 
     // full command line
-    QString arguments = heob + heobArguments + " \"" + executable + '\"';
+    QString arguments = heob + heobArguments + " \"" + executable.path() + '\"';
     if (!commandLineArguments.isEmpty())
         arguments += ' ' + commandLineArguments;
     QByteArray argumentsCopy(reinterpret_cast<const char *>(arguments.utf16()), arguments.size() * 2 + 2);
@@ -914,22 +919,22 @@ void MemcheckToolPrivate::updateFromSettings()
         foreach (const QVariant &v, action->data().toList()) {
             bool ok;
             int kind = v.toInt(&ok);
-            if (ok && !m_settings->visibleErrorKinds().contains(kind))
+            if (ok && !m_settings->visibleErrorKinds.value().contains(kind))
                 contained = false;
         }
         action->setChecked(contained);
     }
 
-    m_filterProjectAction->setChecked(!m_settings->filterExternalIssues());
+    m_filterProjectAction->setChecked(!m_settings->filterExternalIssues.value());
     m_errorView->settingsChanged(m_settings);
 
-    connect(m_settings, &ValgrindBaseSettings::visibleErrorKindsChanged,
+    connect(&m_settings->visibleErrorKinds, &IntegersAspect::valueChanged,
             &m_errorProxyModel, &MemcheckErrorFilterProxyModel::setAcceptedKinds);
-    m_errorProxyModel.setAcceptedKinds(m_settings->visibleErrorKinds());
+    m_errorProxyModel.setAcceptedKinds(m_settings->visibleErrorKinds.value());
 
-    connect(m_settings, &ValgrindBaseSettings::filterExternalIssuesChanged,
+    connect(&m_settings->filterExternalIssues, &BoolAspect::valueChanged,
             &m_errorProxyModel, &MemcheckErrorFilterProxyModel::setFilterExternalIssues);
-    m_errorProxyModel.setFilterExternalIssues(m_settings->filterExternalIssues());
+    m_errorProxyModel.setFilterExternalIssues(m_settings->filterExternalIssues.value());
 }
 
 void MemcheckToolPrivate::maybeActiveRunConfigurationChanged()
@@ -986,14 +991,15 @@ void MemcheckToolPrivate::setupRunner(MemcheckToolRunner *runTool)
     clearErrorView();
     m_loadExternalLogFile->setDisabled(true);
 
-    QString dir = runControl->project()->projectDirectory().toString() + '/';
+    const FilePath dir = runControl->project()->projectDirectory();
     const QString name = runTool->executable().fileName();
 
-    m_errorView->setDefaultSuppressionFile(dir + name + ".supp");
+    m_errorView->setDefaultSuppressionFile(dir.pathAppended(name + ".supp"));
 
-    foreach (const QString &file, runTool->suppressionFiles()) {
-        QAction *action = m_filterMenu->addAction(FilePath::fromString(file).fileName());
-        action->setToolTip(file);
+    const FilePaths suppressionFiles = runTool->suppressionFiles();
+    for (const FilePath &file : suppressionFiles) {
+        QAction *action = m_filterMenu->addAction(file.fileName());
+        action->setToolTip(file.toUserOutput());
         connect(action, &QAction::triggered, this, [file] {
             EditorManager::openEditorAt(file, 0);
         });
@@ -1004,7 +1010,7 @@ void MemcheckToolPrivate::setupRunner(MemcheckToolRunner *runTool)
 void MemcheckToolPrivate::loadShowXmlLogFile(const QString &filePath, const QString &exitMsg)
 {
     clearErrorView();
-    m_settings->setFilterExternalIssues(false);
+    m_settings->filterExternalIssues.setValue(false);
     m_filterProjectAction->setChecked(true);
     m_perspective.select();
     Core::ModeManager::activateMode(Debugger::Constants::MODE_DEBUG);
@@ -1015,16 +1021,16 @@ void MemcheckToolPrivate::loadShowXmlLogFile(const QString &filePath, const QStr
 
 void MemcheckToolPrivate::loadExternalXmlLogFile()
 {
-    const QString filePath = QFileDialog::getOpenFileName(
-                ICore::dialogParent(),
+    const FilePath filePath = FileUtils::getOpenFilePath(
+                nullptr,
                 MemcheckTool::tr("Open Memcheck XML Log File"),
-                QString(),
+                {},
                 MemcheckTool::tr("XML Files (*.xml);;All Files (*)"));
     if (filePath.isEmpty())
         return;
 
     m_exitMsg.clear();
-    loadXmlLogFile(filePath);
+    loadXmlLogFile(filePath.toString());
 }
 
 void MemcheckToolPrivate::loadXmlLogFile(const QString &filePath)
@@ -1090,7 +1096,7 @@ void MemcheckToolPrivate::updateErrorFilter()
     QTC_ASSERT(m_errorView, return);
     QTC_ASSERT(m_settings, return);
 
-    m_settings->setFilterExternalIssues(!m_filterProjectAction->isChecked());
+    m_settings->filterExternalIssues.setValue(!m_filterProjectAction->isChecked());
 
     QList<int> errorKinds;
     foreach (QAction *a, m_errorFilterActions) {
@@ -1103,7 +1109,7 @@ void MemcheckToolPrivate::updateErrorFilter()
                 errorKinds << kind;
         }
     }
-    m_settings->setVisibleErrorKinds(errorKinds);
+    m_settings->visibleErrorKinds.setValue(errorKinds);
 }
 
 int MemcheckToolPrivate::updateUiAfterFinishedHelper()
@@ -1199,7 +1205,7 @@ HeobDialog::HeobDialog(QWidget *parent) :
 
     auto profilesLayout = new QHBoxLayout;
     m_profilesCombo = new QComboBox;
-    for (const auto &profile : m_profiles)
+    for (const auto &profile : qAsConst(m_profiles))
         m_profilesCombo->addItem(settings->value(profile + "/" + heobProfileNameC).toString());
     if (hasSelProfile) {
         int selIdx = m_profiles.indexOf(selProfile);
@@ -1419,13 +1425,13 @@ void HeobDialog::updateProfile()
     int leakRecording = settings->value(heobLeakRecordingC, 2).toInt();
     bool attach = settings->value(heobAttachC, false).toBool();
     const QString extraArgs = settings->value(heobExtraArgsC).toString();
-    QString path = settings->value(heobPathC).toString();
+    FilePath path = FilePath::fromVariant(settings->value(heobPathC));
     settings->endGroup();
 
     if (path.isEmpty()) {
         const QString heobPath = QStandardPaths::findExecutable("heob32.exe");
         if (!heobPath.isEmpty())
-            path = QFileInfo(heobPath).path();
+            path = FilePath::fromUserInput(heobPath);
     }
 
     m_xmlEdit->setText(xml);
@@ -1438,7 +1444,7 @@ void HeobDialog::updateProfile()
     m_leakRecordingCombo->setCurrentIndex(leakRecording);
     m_attachCheck->setChecked(attach);
     m_extraArgsEdit->setText(extraArgs);
-    m_pathChooser->setPath(path);
+    m_pathChooser->setFilePath(path);
 }
 
 void HeobDialog::updateEnabled()
@@ -1641,7 +1647,7 @@ void HeobData::processFinished()
             debugger->setAttachPid(ProcessHandle(m_data[1]));
             debugger->setRunControlName(tr("Process %1").arg(m_data[1]));
             debugger->setInferiorDevice(DeviceKitAspect::device(m_kit));
-            debugger->setStartMode(AttachExternal);
+            debugger->setStartMode(AttachToLocalProcess);
             debugger->setCloseMode(DetachAtClose);
             debugger->setContinueAfterAttach(true);
             debugger->setInferiorExecutable(FilePath::fromString(Utils::imageName(m_data[1])));
