@@ -58,11 +58,7 @@ ValgrindToolRunner::ValgrindToolRunner(RunControl *runControl)
     runControl->setIcon(ProjectExplorer::Icons::ANALYZER_START_SMALL_TOOLBAR);
     setSupportsReRunning(false);
 
-    m_settings = runControl->runConfiguration()
-            ->currentSettings<ValgrindBaseSettings>(ANALYZER_VALGRIND_SETTINGS);
-
-    if (!m_settings)
-        m_settings = ValgrindPlugin::globalSettings();
+    m_settings.fromMap(runControl->settingsData(ANALYZER_VALGRIND_SETTINGS));
 }
 
 void ValgrindToolRunner::start()
@@ -76,17 +72,21 @@ void ValgrindToolRunner::start()
     m_progress.reportStarted();
 
 #if VALGRIND_DEBUG_OUTPUT
-    emit outputReceived(tr("Valgrind options: %1").arg(toolArguments().join(QLatin1Char(' '))), DebugFormat);
-    emit outputReceived(tr("Working directory: %1").arg(runnable().workingDirectory), DebugFormat);
-    emit outputReceived(tr("Command line arguments: %1").arg(runnable().debuggeeArgs), DebugFormat);
+    emit outputReceived(tr("Valgrind options: %1").arg(toolArguments().join(' ')), LogMessageFormat);
+    emit outputReceived(tr("Working directory: %1").arg(runnable().workingDirectory), LogMessageFormat);
+    emit outputReceived(tr("Command line arguments: %1").arg(runnable().debuggeeArgs), LogMessageFormat);
 #endif
 
-    m_runner.setValgrindExecutable(m_settings->valgrindExecutable());
-    m_runner.setValgrindArguments(genericToolArguments() + toolArguments());
+    CommandLine valgrind{m_settings.valgrindExecutable.filePath()};
+    valgrind.addArgs(m_settings.valgrindArguments.value(), CommandLine::Raw);
+    valgrind.addArgs(genericToolArguments());
+    valgrind.addArgs(toolArguments());
+
+    m_runner.setValgrindCommand(valgrind);
     m_runner.setDevice(device());
     m_runner.setDebuggee(runnable());
 
-    if (auto aspect = runControl()->runConfiguration()->aspect<TerminalAspect>())
+    if (auto aspect = runControl()->aspect<TerminalAspect>())
         m_runner.setUseTerminal(aspect->useTerminal());
 
     connect(&m_runner, &ValgrindRunner::processOutputReceived,
@@ -115,28 +115,28 @@ void ValgrindToolRunner::stop()
     m_runner.stop();
 }
 
-QString ValgrindToolRunner::executable() const
+FilePath ValgrindToolRunner::executable() const
 {
-    return runnable().executable;
+    return runnable().command.executable();
 }
 
 QStringList ValgrindToolRunner::genericToolArguments() const
 {
-    QTC_ASSERT(m_settings, return QStringList());
     QString smcCheckValue;
-    switch (m_settings->selfModifyingCodeDetection()) {
+
+    switch (m_settings.selfModifyingCodeDetection.value()) {
     case ValgrindBaseSettings::DetectSmcNo:
-        smcCheckValue = QLatin1String("none");
+        smcCheckValue = "none";
         break;
     case ValgrindBaseSettings::DetectSmcEverywhere:
-        smcCheckValue = QLatin1String("all");
+        smcCheckValue = "all";
         break;
     case ValgrindBaseSettings::DetectSmcEverywhereButFile:
-        smcCheckValue = QLatin1String("all-non-file");
+        smcCheckValue = "all-non-file";
         break;
     case ValgrindBaseSettings::DetectSmcStackOnly:
     default:
-        smcCheckValue = QLatin1String("stack");
+        smcCheckValue = "stack";
         break;
     }
     return {"--smc-check=" + smcCheckValue};
@@ -150,7 +150,7 @@ void ValgrindToolRunner::handleProgressCanceled()
 
 void ValgrindToolRunner::handleProgressFinished()
 {
-    QApplication::alert(ICore::mainWindow(), 3000);
+    QApplication::alert(ICore::dialogParent(), 3000);
 }
 
 void ValgrindToolRunner::runnerFinished()
@@ -175,7 +175,7 @@ void ValgrindToolRunner::receiveProcessOutput(const QString &output, OutputForma
 void ValgrindToolRunner::receiveProcessError(const QString &message, QProcess::ProcessError error)
 {
     if (error == QProcess::FailedToStart) {
-        const QString valgrind = m_settings->valgrindExecutable();
+        const QString valgrind = m_settings.valgrindExecutable.value();
         if (!valgrind.isEmpty())
             appendMessage(tr("Error: \"%1\" could not be started: %2").arg(valgrind, message), ErrorMessageFormat);
         else
@@ -189,8 +189,8 @@ void ValgrindToolRunner::receiveProcessError(const QString &message, QProcess::P
     if (m_isStopping)
         return;
 
-    QObject *obj = ExtensionSystem::PluginManager::getObjectByName(QLatin1String("AppOutputPane"));
-    if (IOutputPane *pane = qobject_cast<IOutputPane *>(obj))
+    QObject *obj = ExtensionSystem::PluginManager::getObjectByName("AppOutputPane");
+    if (auto pane = qobject_cast<IOutputPane *>(obj))
         pane->popup(IOutputPane::NoModeSwitch);
 }
 
