@@ -478,7 +478,7 @@ void ValgrindMemcheckParserTest::testValgrindGarbage()
 void ValgrindMemcheckParserTest::testParserStop()
 {
     ValgrindRunner runner;
-    runner.setValgrindCommand({fakeValgrindExecutable(),
+    runner.setValgrindCommand({FilePath::fromString(fakeValgrindExecutable()),
                                {QString("--xml-socket=127.0.0.1:%1").arg(m_server->serverPort()),
                                 "-i", dataFile("memcheck-output-sample1.xml"), "--wait", "5" }});
     runner.setProcessChannelMode(QProcess::ForwardedChannels);
@@ -500,7 +500,7 @@ void ValgrindMemcheckParserTest::testRealValgrind()
     qDebug() << "running exe:" << executable << " HINT: set VALGRIND_TEST_BIN to change this";
 
     ProjectExplorer::Runnable debuggee;
-    debuggee.executable = FilePath::fromString(executable);
+    debuggee.command.setExecutable(FilePath::fromString(executable));
     debuggee.environment = sysEnv;
     ValgrindRunner runner;
     runner.setValgrindCommand({"valgrind", {}});
@@ -537,12 +537,12 @@ void ValgrindMemcheckParserTest::testValgrindStartError()
     QFETCH(QString, debuggeeArgs);
 
     ProjectExplorer::Runnable debuggeeExecutable;
-    debuggeeExecutable.executable = FilePath::fromString(debuggee);
+    debuggeeExecutable.command.setExecutable(FilePath::fromString(debuggee));
+    debuggeeExecutable.command.setArguments(debuggeeArgs);
     debuggeeExecutable.environment = Utils::Environment::systemEnvironment();
-    debuggeeExecutable.commandLineArguments = debuggeeArgs;
 
     ValgrindRunner runner;
-    runner.setValgrindCommand({valgrindExe, valgrindArgs});
+    runner.setValgrindCommand({FilePath::fromString(valgrindExe), valgrindArgs});
     runner.setDebuggee(debuggeeExecutable);
     runner.setDevice(ProjectExplorer::DeviceManager::instance()->defaultDevice(
                          ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE));
