@@ -40,7 +40,7 @@ class VCSBASE_EXPORT VcsEditorFactory : public TextEditor::TextEditorFactory
 public:
     VcsEditorFactory(const VcsBaseEditorParameters *parameters,
                      const EditorWidgetCreator editorWidgetCreator,
-                     std::function<void(const QString &, const QString &)> describeFunc);
+                     std::function<void(const Utils::FilePath &, const QString &)> describeFunc);
 };
 
 } // namespace VcsBase
