@@ -29,6 +29,7 @@
 #include "ui_submiteditorwidget.h"
 
 #include <utils/algorithm.h>
+#include <utils/theme/theme.h>
 
 #include <QDebug>
 #include <QPointer>
@@ -434,6 +435,12 @@ void SubmitEditorWidget::updateSubmitAction()
     }
 }
 
+void SubmitEditorWidget::changeEvent(QEvent *event)
+{
+    if (event->type() == QEvent::EnabledChange)
+        verifyDescription();
+}
+
 // Enable diff depending on selected files
 void SubmitEditorWidget::updateDiffAction()
 {
@@ -503,9 +510,66 @@ void SubmitEditorWidget::hideDescription()
     setDescriptionMandatory(false);
 }
 
+void SubmitEditorWidget::verifyDescription()
+{
+    if (!isEnabled()) {
+        d->m_ui.descriptionHint->setText(QString());
+        d->m_ui.descriptionHint->setToolTip(QString());
+        return;
+    }
+
+    auto fontColor = [](Utils::Theme::Color color) {
+        return QString("<font color=\"%1\">")
+                .arg(Utils::creatorTheme()->color(color).name());
+    };
+    const QString hint = fontColor(Utils::Theme::OutputPanes_TestWarnTextColor);
+    const QString warning = fontColor(Utils::Theme::TextColorError);
+
+    const QChar newLine = '\n';
+    const int descriptionLength = d->m_description.length();
+    int subjectLength = d->m_description.indexOf(newLine);
+    int secondLineLength = 0;
+    if (subjectLength >= 0) {
+        const int secondLineStart = subjectLength + 1;
+        int secondLineEnd = d->m_description.indexOf(newLine, secondLineStart);
+        if (secondLineEnd == -1)
+            secondLineEnd = descriptionLength;
+        secondLineLength = secondLineEnd - secondLineStart;
+    } else {
+        subjectLength = descriptionLength;
+    }
+
+    enum { MinSubjectLength = 20, MaxSubjectLength = 72, WarningSubjectLength = 55 };
+    QStringList hints;
+    if (subjectLength < MinSubjectLength)
+        hints.append(warning + tr("Warning: The commit subject is very short."));
+
+    if (subjectLength > MaxSubjectLength)
+        hints.append(warning + tr("Warning: The commit subject is too long."));
+    else if (subjectLength > WarningSubjectLength)
+        hints.append(hint + tr("Hint: Aim for a shorter commit subject."));
+
+    if (secondLineLength > 0)
+        hints.append(hint + tr("Hint: The second line of a commit message should be empty."));
+
+    d->m_ui.descriptionHint->setText(hints.join("<br>"));
+    if (!d->m_ui.descriptionHint->text().isEmpty()) {
+        d->m_ui.descriptionHint->setToolTip(
+                    tr("<p>Writing good commit messages</p>"
+                       "<ul>"
+                       "<li>Avoid very short commit messages.</li>"
+                       "<li>Consider the first line as subject (like in email) "
+                       "and keep it shorter than %n characters.</li>"
+                       "<li>After an empty second line, a longer description can be added.</li>"
+                       "<li>Describe why the change was done, not how it was done.</li>"
+                       "</ul>", nullptr, MaxSubjectLength));
+        }
+}
+
 void SubmitEditorWidget::descriptionTextChanged()
 {
     d->m_description = cleanupDescription(d->m_ui.description->toPlainText());
+    verifyDescription();
     wrapDescription();
     trimDescription();
     // append field entries
