@@ -24,12 +24,14 @@
 ****************************************************************************/
 
 #include "vcsbasediffeditorcontroller.h"
-#include "vcsbaseclient.h"
 #include "vcscommand.h"
+#include "vcsbaseclientsettings.h"
 
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 #include <diffeditor/diffutils.h>
+
+#include <utils/environment.h>
 #include <utils/qtcassert.h>
 #include <utils/runextensions.h>
 
@@ -37,6 +39,7 @@
 
 using namespace DiffEditor;
 using namespace Core;
+using namespace Utils;
 
 namespace VcsBase {
 
@@ -82,9 +85,7 @@ private:
 class VcsBaseDiffEditorControllerPrivate
 {
 public:
-    VcsBaseDiffEditorControllerPrivate(VcsBaseDiffEditorController *controller,
-                                       VcsBaseClientImpl *client,
-                                       const QString &workingDirectory);
+    VcsBaseDiffEditorControllerPrivate(VcsBaseDiffEditorController *q) : q(q) {}
     ~VcsBaseDiffEditorControllerPrivate();
 
     void processingFinished();
@@ -94,8 +95,10 @@ public:
     void commandFinished(bool success);
 
     VcsBaseDiffEditorController *q;
-    VcsBaseClientImpl *m_client;
-    const QString m_directory;
+    FilePath m_directory;
+    Environment m_processEnvironment;
+    FilePath m_vcsBinary;
+    int m_vscTimeoutS;
     QString m_startupFile;
     QString m_output;
     QString m_displayName;
@@ -129,16 +132,6 @@ void VcsCommandResultProxy::commandFinished(bool success)
     m_target->commandFinished(success);
 }
 
-VcsBaseDiffEditorControllerPrivate::VcsBaseDiffEditorControllerPrivate(
-        VcsBaseDiffEditorController *controller,
-        VcsBaseClientImpl *client,
-        const QString &workingDirectory)
-    : q(controller)
-    , m_client(client)
-    , m_directory(workingDirectory)
-{
-}
-
 VcsBaseDiffEditorControllerPrivate::~VcsBaseDiffEditorControllerPrivate()
 {
     cancelReload();
@@ -230,14 +223,10 @@ void VcsBaseDiffEditorControllerPrivate::commandFinished(bool success)
 
 /////////////////////
 
-VcsBaseDiffEditorController::VcsBaseDiffEditorController(IDocument *document,
-                                                         VcsBaseClientImpl *client,
-                                                         const QString &workingDirectory)
+VcsBaseDiffEditorController::VcsBaseDiffEditorController(Core::IDocument *document)
     : DiffEditorController(document)
-    , d(new VcsBaseDiffEditorControllerPrivate(this, client, workingDirectory))
-{
-    setBaseDirectory(workingDirectory);
-}
+    , d(new VcsBaseDiffEditorControllerPrivate(this))
+{}
 
 VcsBaseDiffEditorController::~VcsBaseDiffEditorController()
 {
@@ -252,7 +241,7 @@ void VcsBaseDiffEditorController::runCommand(const QList<QStringList> &args, uns
     // and "Waiting for data..." will be shown.
     d->cancelReload();
 
-    d->m_command = new VcsCommand(workingDirectory(), d->m_client->processEnvironment());
+    d->m_command = new VcsCommand(workingDirectory(), d->m_processEnvironment);
     d->m_command->setDisplayName(d->m_displayName);
     d->m_command->setCodec(codec ? codec : EditorManager::defaultTextCodec());
     d->m_commandResultProxy = new VcsCommandResultProxy(d->m_command.data(), d);
@@ -261,7 +250,7 @@ void VcsBaseDiffEditorController::runCommand(const QList<QStringList> &args, uns
     for (const QStringList &arg : args) {
         QTC_ASSERT(!arg.isEmpty(), continue);
 
-        d->m_command->addJob({d->m_client->vcsBinary(), arg}, d->m_client->vcsTimeoutS());
+        d->m_command->addJob({d->m_vcsBinary, arg}, d->m_vscTimeoutS);
     }
 
     d->m_command->execute();
@@ -272,12 +261,7 @@ void VcsBaseDiffEditorController::processCommandOutput(const QString &output)
     d->processDiff(output);
 }
 
-VcsBaseClientImpl *VcsBaseDiffEditorController::client() const
-{
-    return d->m_client;
-}
-
-QString VcsBaseDiffEditorController::workingDirectory() const
+FilePath VcsBaseDiffEditorController::workingDirectory() const
 {
     return d->m_directory;
 }
@@ -297,6 +281,27 @@ void VcsBaseDiffEditorController::setDisplayName(const QString &displayName)
     d->m_displayName = displayName;
 }
 
+void VcsBase::VcsBaseDiffEditorController::setWorkingDirectory(const FilePath &workingDir)
+{
+    d->m_directory = workingDir;
+    setBaseDirectory(workingDir);
+}
+
+void VcsBaseDiffEditorController::setVcsTimeoutS(int value)
+{
+    d->m_vscTimeoutS = value;
+}
+
+void VcsBaseDiffEditorController::setVcsBinary(const Utils::FilePath &path)
+{
+    d->m_vcsBinary = path;
+}
+
+void VcsBaseDiffEditorController::setProcessEnvironment(const Environment &value)
+{
+    d->m_processEnvironment = value;
+}
+
 } // namespace VcsBase
 
 #include "vcsbasediffeditorcontroller.moc"
