@@ -91,6 +91,7 @@
     \sa VcsBase::VcsBaseEditorWidget
 */
 
+using namespace Core;
 using namespace TextEditor;
 using namespace Utils;
 
@@ -329,7 +330,8 @@ QString ChangeTextCursorHandler::currentContents() const
 
 void ChangeTextCursorHandler::slotDescribe()
 {
-    emit editorWidget()->describeRequested(editorWidget()->source(), m_currentChange);
+    emit editorWidget()->describeRequested(FilePath::fromString(editorWidget()->source()),
+                                           m_currentChange);
 }
 
 void ChangeTextCursorHandler::slotCopyRevision()
@@ -835,14 +837,14 @@ void VcsBaseEditorWidget::setHighlightingEnabled(bool e)
     dh->setEnabled(e);
 }
 
-QString VcsBaseEditorWidget::workingDirectory() const
+FilePath VcsBaseEditorWidget::workingDirectory() const
 {
-    return d->m_workingDirectory;
+    return FilePath::fromString(d->m_workingDirectory);
 }
 
-void VcsBaseEditorWidget::setWorkingDirectory(const QString &wd)
+void VcsBaseEditorWidget::setWorkingDirectory(const FilePath &wd)
 {
-    d->m_workingDirectory = wd;
+    d->m_workingDirectory = wd.toString();
 }
 
 QTextCodec *VcsBaseEditorWidget::codec() const
@@ -956,18 +958,19 @@ void VcsBaseEditorWidget::slotCursorPositionChanged()
     // Adapt entries combo to new position
     // if the cursor goes across a file line.
     const int newCursorLine = textCursor().blockNumber();
-    if (newCursorLine == d->m_cursorLine)
-        return;
-    // Which section does it belong to?
-    d->m_cursorLine = newCursorLine;
-    const int section = sectionOfLine(d->m_cursorLine, d->m_entrySections);
-    if (section != -1) {
-        QComboBox *entriesComboBox = d->entriesComboBox();
-        if (entriesComboBox->currentIndex() != section) {
-            QSignalBlocker blocker(entriesComboBox);
-            entriesComboBox->setCurrentIndex(section);
+    if (newCursorLine != d->m_cursorLine) {
+        // Which section does it belong to?
+        d->m_cursorLine = newCursorLine;
+        const int section = sectionOfLine(d->m_cursorLine, d->m_entrySections);
+        if (section != -1) {
+            QComboBox *entriesComboBox = d->entriesComboBox();
+            if (entriesComboBox->currentIndex() != section) {
+                QSignalBlocker blocker(entriesComboBox);
+                entriesComboBox->setCurrentIndex(section);
+            }
         }
     }
+    TextEditorWidget::slotCursorPositionChanged();
 }
 
 void VcsBaseEditorWidget::contextMenuEvent(QContextMenuEvent *e)
@@ -981,8 +984,10 @@ void VcsBaseEditorWidget::contextMenuEvent(QContextMenuEvent *e)
             handler->fillContextMenu(menu, d->m_parameters->type);
         }
     }
-    if (!menu)
-        menu = createStandardContextMenu();
+    if (!menu) {
+        menu = new QMenu;
+        appendStandardContextMenuActions(menu);
+    }
     switch (d->m_parameters->type) {
     case LogOutput: // log might have diff
     case DiffOutput: {
@@ -1296,6 +1301,11 @@ QTextCodec *VcsBaseEditor::getCodec(const QString &source)
     return sys;
 }
 
+QTextCodec *VcsBaseEditor::getCodec(const FilePath &workingDirectory, const QStringList &files)
+{
+    return getCodec(workingDirectory.toString(), files);
+}
+
 QTextCodec *VcsBaseEditor::getCodec(const QString &workingDirectory, const QStringList &files)
 {
     if (files.empty())
@@ -1348,13 +1358,12 @@ bool VcsBaseEditor::gotoLineOfEditor(Core::IEditor *e, int lineNumber)
 
 // Return source file or directory string depending on parameters
 // ('git diff XX' -> 'XX' , 'git diff XX file' -> 'XX/file').
-QString VcsBaseEditor::getSource(const QString &workingDirectory,
-                                 const QString &fileName)
+QString VcsBaseEditor::getSource(const FilePath &workingDirectory, const QString &fileName)
 {
     if (fileName.isEmpty())
-        return workingDirectory;
+        return workingDirectory.toString();
 
-    QString rc = workingDirectory;
+    QString rc = workingDirectory.toString();
     const QChar slash = QLatin1Char('/');
     if (!rc.isEmpty() && !(rc.endsWith(slash) || rc.endsWith(QLatin1Char('\\'))))
         rc += slash;
@@ -1362,15 +1371,14 @@ QString VcsBaseEditor::getSource(const QString &workingDirectory,
     return rc;
 }
 
-QString VcsBaseEditor::getSource(const QString &workingDirectory,
-                                 const QStringList &fileNames)
+QString VcsBaseEditor::getSource(const FilePath &workingDirectory, const QStringList &fileNames)
 {
-    return fileNames.size() == 1 ?
-            getSource(workingDirectory, fileNames.front()) :
-            workingDirectory;
+    return fileNames.size() == 1
+            ? getSource(workingDirectory, fileNames.front())
+            : workingDirectory.toString();
 }
 
-QString VcsBaseEditor::getTitleId(const QString &workingDirectory,
+QString VcsBaseEditor::getTitleId(const FilePath &workingDirectory,
                                   const QStringList &fileNames,
                                   const QString &revision)
 {
@@ -1383,7 +1391,7 @@ QString VcsBaseEditor::getTitleId(const QString &workingDirectory,
     QString rc;
     switch (nonEmptyFileNames.size()) {
     case 0:
-        rc = workingDirectory;
+        rc = workingDirectory.toString();
         break;
     case 1:
         rc = nonEmptyFileNames.front();
@@ -1453,13 +1461,14 @@ QString VcsBaseEditorWidget::findDiffFile(const QString &f) const
         if (sourceFileInfo.isFile())
             return sourceFileInfo.absoluteFilePath();
 
-        const QString topLevel = Core::VcsManager::findTopLevelForDirectory(sourceDir);
+        const FilePath topLevel =
+            VcsManager::findTopLevelForDirectory(FilePath::fromString(sourceDir));
         if (topLevel.isEmpty())
             return QString();
 
-        const QFileInfo topLevelFileInfo(topLevel + slash + f);
-        if (topLevelFileInfo.isFile())
-            return topLevelFileInfo.absoluteFilePath();
+        const FilePath topLevelFile = topLevel.pathAppended(f);
+        if (topLevelFile.isFile())
+            return topLevelFile.absoluteFilePath().toString();
     }
 
     // 3) Try working directory
@@ -1487,7 +1496,7 @@ void VcsBaseEditorWidget::slotAnnotateRevision()
         QString workingDirectory = d->m_workingDirectory;
         if (workingDirectory.isEmpty())
             workingDirectory = QFileInfo(fileName).absolutePath();
-        emit annotateRevisionRequested(workingDirectory,
+        emit annotateRevisionRequested(FilePath::fromString(workingDirectory),
                                        QDir(workingDirectory).relativeFilePath(fileName),
                                        a->data().toString(), currentLine);
     }
@@ -1533,7 +1542,7 @@ bool VcsBaseEditorWidget::canApplyDiffChunk(const DiffChunk &dc) const
 bool VcsBaseEditorWidget::applyDiffChunk(const DiffChunk &dc, bool revert) const
 {
     return Core::PatchTool::runPatch(dc.asPatch(d->m_workingDirectory),
-                                   d->m_workingDirectory, 0, revert);
+                                     FilePath::fromString(d->m_workingDirectory), 0, revert);
 }
 
 QString VcsBaseEditorWidget::fileNameFromDiffSpecification(const QTextBlock &inBlock, QString *header) const
