@@ -30,10 +30,9 @@
 #include <texteditor/texteditor.h>
 
 #include <QSet>
+#include <QTextCodec>
 
 QT_BEGIN_NAMESPACE
-class QRegExp;
-class QTextCodec;
 class QTextCursor;
 QT_END_NAMESPACE
 
@@ -46,8 +45,9 @@ class VcsBaseEditorWidgetPrivate;
 
 class BaseAnnotationHighlighter;
 class VcsBaseEditorWidget;
-class VcsBaseEditorParameterWidget;
+class VcsBaseEditorConfig;
 class VcsCommand;
+class VcsEditorFactory;
 
 // Documentation inside
 enum EditorContentType
@@ -93,7 +93,8 @@ public:
     // The codec should be set on editors displaying diff or annotation
     // output.
     static QTextCodec *getCodec(const QString &source);
-    static QTextCodec *getCodec(const QString &workingDirectory, const QStringList &files);
+    static QTextCodec *getCodec(const Utils::FilePath &workingDirectory, const QStringList &files);
+    static QTextCodec *getCodec(const QString &workingDirectory, const QStringList &files); // FIXME: Remove
 
     // Utility to return the widget from the IEditor returned by the editor
     // manager which is a BaseTextEditor.
@@ -111,13 +112,13 @@ public:
     // Convenience functions to determine the source to pass on to a diff
     // editor if one has a call consisting of working directory and file arguments.
     // ('git diff XX' -> 'XX' , 'git diff XX file' -> 'XX/file').
-    static QString getSource(const QString &workingDirectory, const QString &fileName);
-    static QString getSource(const QString &workingDirectory, const QStringList &fileNames);
+    static QString getSource(const Utils::FilePath &workingDirectory, const QString &fileName);
+    static QString getSource(const Utils::FilePath &workingDirectory, const QStringList &fileNames);
     // Convenience functions to determine an title/id to identify the editor
     // from the arguments (','-joined arguments or directory) + revision.
-    static QString getTitleId(const QString &workingDirectory,
+    static QString getTitleId(const Utils::FilePath &workingDirectory,
                               const QStringList &fileNames,
-                              const QString &revision = QString());
+                              const QString &revision = {});
 
     /* Tagging editors: Sometimes, an editor should be re-used, for example, when showing
      * a diff of the same file with different diff-options. In order to be able to find
@@ -127,19 +128,14 @@ public:
     static QString editorTag(EditorContentType t, const QString &workingDirectory, const QStringList &files,
                              const QString &revision = QString());
     void finalizeInitialization() override;
-signals:
-    void describeRequested(const QString &source, const QString &change);
-    void annotateRevisionRequested(const QString &workingDirectory, const QString &file,
-                                   const QString &change, int line);
 };
 
 class VCSBASE_EXPORT VcsBaseEditorWidget : public TextEditor::TextEditorWidget
 {
     Q_PROPERTY(QString source READ source WRITE setSource)
-    Q_PROPERTY(QString workingDirectory READ workingDirectory WRITE setWorkingDirectory)
+    Q_PROPERTY(Utils::FilePath workingDirectory READ workingDirectory WRITE setWorkingDirectory)
     Q_PROPERTY(QTextCodec *codec READ codec WRITE setCodec)
     Q_PROPERTY(QString annotateRevisionTextFormat READ annotateRevisionTextFormat WRITE setAnnotateRevisionTextFormat)
-    Q_PROPERTY(QString copyRevisionTextFormat READ copyRevisionTextFormat WRITE setCopyRevisionTextFormat)
     Q_PROPERTY(bool isFileLogAnnotateEnabled READ isFileLogAnnotateEnabled WRITE setFileLogAnnotateEnabled)
     Q_OBJECT
 
@@ -148,9 +144,13 @@ protected:
     // virtual functions).
     VcsBaseEditorWidget();
     // Pattern for diff header. File name must be in the first capture group
-    void setDiffFilePattern(const QRegExp &pattern);
+    void setDiffFilePattern(const QString &pattern);
     // Pattern for log entry. hash/revision number must be in the first capture group
-    void setLogEntryPattern(const QRegExp &pattern);
+    void setLogEntryPattern(const QString &pattern);
+    // Pattern for annotation entry. hash/revision number must be in the first capture group
+    void setAnnotationEntryPattern(const QString &pattern);
+    // Pattern for annotation separator. Lookup will stop on match.
+    void setAnnotationSeparatorPattern(const QString &pattern);
     virtual bool supportChangeLinks() const;
     virtual QString fileNameForLine(int line) const;
 
@@ -158,9 +158,11 @@ protected:
     int lineNumberDigits() const override;
 
 public:
+    typedef std::function<void(const Utils::FilePath &, const QString &)> DescribeFunc;
+
     void finalizeInitialization() override;
     // FIXME: Consolidate these into finalizeInitialization
-    void setDescribeSlot(QObject *describeReceiver, const char *describeSlot);
+    void setDescribeFunc(DescribeFunc describeFunc);
     // void
     virtual void init();
     //
@@ -186,21 +188,19 @@ public:
     QString annotatePreviousRevisionTextFormat() const;
     void setAnnotatePreviousRevisionTextFormat(const QString &);
 
-    // Format for "Copy" revision menu entries. Should contain '%1" placeholder
-    QString copyRevisionTextFormat() const;
-    void setCopyRevisionTextFormat(const QString &);
-
     // Enable "Annotate" context menu in file log view
     // (set to true if the source is a single file and the VCS implements it)
     bool isFileLogAnnotateEnabled() const;
     void setFileLogAnnotateEnabled(bool e);
 
+    void setHighlightingEnabled(bool e);
+
     QTextCodec *codec() const;
     void setCodec(QTextCodec *);
 
     // Base directory for diff views
-    QString workingDirectory() const;
-    void setWorkingDirectory(const QString &wd);
+    Utils::FilePath workingDirectory() const;
+    void setWorkingDirectory(const Utils::FilePath &wd);
 
     int firstLineNumber() const;
     void setFirstLineNumber(int firstLineNumber);
@@ -209,8 +209,8 @@ public:
 
     EditorContentType contentType() const;
 
-    bool setConfigurationWidget(VcsBaseEditorParameterWidget *w);
-    VcsBaseEditorParameterWidget *configurationWidget() const;
+    void setEditorConfig(VcsBaseEditorConfig *config);
+    VcsBaseEditorConfig *editorConfig() const;
 
     void setCommand(VcsCommand *command);
 
@@ -220,8 +220,8 @@ signals:
     // These signals also exist in the opaque editable (IEditor) that is
     // handled by the editor manager for convenience. They are emitted
     // for LogOutput/AnnotateOutput content types.
-    void describeRequested(const QString &source, const QString &change);
-    void annotateRevisionRequested(const QString &workingDirectory, const QString &file,
+    void describeRequested(const Utils::FilePath &source, const QString &change);
+    void annotateRevisionRequested(const Utils::FilePath &workingDirectory, const QString &file,
                                    const QString &change, int lineNumber);
     void diffChunkApplied(const VcsBase::DiffChunk &dc);
     void diffChunkReverted(const VcsBase::DiffChunk &dc);
@@ -247,14 +247,14 @@ protected:
 
     // Implement to return a set of change identifiers in
     // annotation mode
-    virtual QSet<QString> annotationChanges() const = 0;
+    QSet<QString> annotationChanges() const;
     // Implement to identify a change number at the cursor position
     virtual QString changeUnderCursor(const QTextCursor &) const = 0;
     // Factory functions for highlighters
     virtual BaseAnnotationHighlighter *createAnnotationHighlighter(const QSet<QString> &changes) const = 0;
     // Returns a local file name from the diff file specification
     // (text cursor at position above change hunk)
-    QString fileNameFromDiffSpecification(const QTextBlock &inBlock, QString *header = 0) const;
+    QString fileNameFromDiffSpecification(const QTextBlock &inBlock, QString *header = nullptr) const;
 
     // Implement to return decorated annotation change for "Annotate version"
     virtual QString decorateVersion(const QString &revision) const;
@@ -297,8 +297,11 @@ private:
 
 #ifdef WITH_TESTS
 public:
-    static void testDiffFileResolving(const char *id);
-    static void testLogResolving(const char *id, QByteArray &data, const QByteArray &entry1, const QByteArray &entry2);
+    static void testDiffFileResolving(const VcsEditorFactory &factory);
+    static void testLogResolving(const VcsEditorFactory &factory,
+                                 const QByteArray &data,
+                                 const QByteArray &entry1,
+                                 const QByteArray &entry2);
 #endif
 };
 
