@@ -38,7 +38,7 @@
 #include <projectexplorer/project.h>
 #include <projectexplorer/session.h>
 #include <utils/qtcassert.h>
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 
 #include <QDebug>
 #include <QDir>
@@ -227,21 +227,21 @@ StateListener::StateListener(QObject *parent) : QObject(parent)
 
 QString StateListener::windowTitleVcsTopic(const QString &filePath)
 {
-    QString searchPath;
+    FilePath searchPath;
     if (!filePath.isEmpty()) {
-        searchPath = QFileInfo(filePath).absolutePath();
+        searchPath = FilePath::fromString(filePath).absolutePath();
     } else {
         // use single project's information if there is only one loaded.
         const QList<Project *> projects = SessionManager::projects();
         if (projects.size() == 1)
-            searchPath = projects.first()->projectDirectory().toString();
+            searchPath = projects.first()->projectDirectory();
     }
     if (searchPath.isEmpty())
         return QString();
     QString topLevelPath;
     IVersionControl *vc = VcsManager::findVersionControlForDirectory(
                 searchPath, &topLevelPath);
-    return (vc && !topLevelPath.isEmpty()) ? vc->vcsTopic(topLevelPath) : QString();
+    return (vc && !topLevelPath.isEmpty()) ? vc->vcsTopic(FilePath::fromString(topLevelPath)) : QString();
 }
 
 static inline QString displayNameOfEditor(const QString &fileName)
@@ -291,8 +291,8 @@ void StateListener::slotStateChanged()
                 state.currentFileDirectory = currentFi.absolutePath();
                 state.currentFileName = currentFi.fileName();
             }
-            fileControl = VcsManager::findVersionControlForDirectory(state.currentFileDirectory,
-                                                                     &state.currentFileTopLevel);
+            fileControl = VcsManager::findVersionControlForDirectory(
+                FilePath::fromString(state.currentFileDirectory), &state.currentFileTopLevel);
         }
 
         if (!fileControl)
@@ -308,8 +308,8 @@ void StateListener::slotStateChanged()
     if (currentProject) {
         state.currentProjectPath = currentProject->projectDirectory().toString();
         state.currentProjectName = currentProject->displayName();
-        projectControl = VcsManager::findVersionControlForDirectory(state.currentProjectPath,
-                                                                    &state.currentProjectTopLevel);
+        projectControl = VcsManager::findVersionControlForDirectory(
+            FilePath::fromString(state.currentProjectPath), &state.currentProjectTopLevel);
         if (projectControl) {
             // If we have both, let the file's one take preference
             if (fileControl && projectControl != fileControl)
@@ -381,14 +381,14 @@ QString VcsBasePluginState::currentFileName() const
     return data->m_state.currentFileName;
 }
 
-QString VcsBasePluginState::currentFileTopLevel() const
+FilePath VcsBasePluginState::currentFileTopLevel() const
 {
-    return data->m_state.currentFileTopLevel;
+    return FilePath::fromString(data->m_state.currentFileTopLevel);
 }
 
-QString VcsBasePluginState::currentFileDirectory() const
+FilePath VcsBasePluginState::currentFileDirectory() const
 {
-    return data->m_state.currentFileDirectory;
+    return FilePath::fromString(data->m_state.currentFileDirectory);
 }
 
 QString VcsBasePluginState::relativeCurrentFile() const
@@ -407,9 +407,9 @@ QString VcsBasePluginState::currentPatchFileDisplayName() const
     return data->m_state.currentPatchFileDisplayName;
 }
 
-QString VcsBasePluginState::currentProjectPath() const
+FilePath VcsBasePluginState::currentProjectPath() const
 {
-    return data->m_state.currentProjectPath;
+    return FilePath::fromString(data->m_state.currentProjectPath);
 }
 
 QString VcsBasePluginState::currentProjectName() const
@@ -417,9 +417,9 @@ QString VcsBasePluginState::currentProjectName() const
     return data->m_state.currentProjectName;
 }
 
-QString VcsBasePluginState::currentProjectTopLevel() const
+FilePath VcsBasePluginState::currentProjectTopLevel() const
 {
-    return data->m_state.currentProjectTopLevel;
+    return FilePath::fromString(data->m_state.currentProjectTopLevel);
 }
 
 QString VcsBasePluginState::relativeCurrentProject() const
@@ -435,9 +435,9 @@ bool VcsBasePluginState::hasTopLevel() const
     return data->m_state.hasFile() || data->m_state.hasProject();
 }
 
-QString VcsBasePluginState::topLevel() const
+FilePath VcsBasePluginState::topLevel() const
 {
-    return hasFile() ? data->m_state.currentFileTopLevel : data->m_state.currentProjectTopLevel;
+    return FilePath::fromString(hasFile() ? data->m_state.currentFileTopLevel : data->m_state.currentProjectTopLevel);
 }
 
 bool VcsBasePluginState::equals(const Internal::State &rhs) const
@@ -638,27 +638,28 @@ void VcsBasePluginPrivate::createRepository()
 {
     QTC_ASSERT(supportsOperation(IVersionControl::CreateRepositoryOperation), return);
     // Find current starting directory
-    QString directory;
+    FilePath directory;
     if (const Project *currentProject = ProjectTree::currentProject())
-        directory = currentProject->projectFilePath().absolutePath().toString();
+        directory = currentProject->projectFilePath().absolutePath();
     // Prompt for a directory that is not under version control yet
     QWidget *mw = ICore::dialogParent();
     do {
-        directory = QFileDialog::getExistingDirectory(mw, tr("Choose Repository Directory"), directory);
+        directory = FileUtils::getExistingDirectory(nullptr, tr("Choose Repository Directory"), directory);
         if (directory.isEmpty())
             return;
         const IVersionControl *managingControl = VcsManager::findVersionControlForDirectory(directory);
         if (managingControl == nullptr)
             break;
         const QString question = tr("The directory \"%1\" is already managed by a version control system (%2)."
-                                    " Would you like to specify another directory?").arg(directory, managingControl->displayName());
+                                    " Would you like to specify another directory?")
+                                    .arg(directory.toUserOutput(), managingControl->displayName());
 
         if (!ask(mw, tr("Repository already under version control"), question))
             return;
     } while (true);
     // Create
     const bool rc = vcsCreateRepository(directory);
-    const QString nativeDir = QDir::toNativeSeparators(directory);
+    const QString nativeDir = directory.toUserOutput();
     if (rc) {
         QMessageBox::information(mw, tr("Repository Created"),
                                  tr("A version control repository has been created in %1.").
@@ -695,15 +696,16 @@ bool VcsBasePluginPrivate::raiseSubmitEditor() const
 // AutoFS is used (due its automatically creating mountpoints when querying
 // a directory). In addition, bail out when reaching the home directory
 // of the user or root (generally avoid '/', where mountpoints are created).
-QString findRepositoryForDirectory(const QString &dirS, const QString &checkFile)
+FilePath findRepositoryForFile(const FilePath &fileOrDir, const QString &checkFile)
 {
+    const FilePath dirS = fileOrDir.isDir() ? fileOrDir : fileOrDir.parentDir();
     qCDebug(findRepoLog) << ">" << dirS << checkFile;
-    QTC_ASSERT(!dirS.isEmpty() && !checkFile.isEmpty(), return QString());
+    QTC_ASSERT(!dirS.isEmpty() && !checkFile.isEmpty(), return {});
 
     const QString root = QDir::rootPath();
     const QString home = QDir::homePath();
 
-    QDir directory(dirS);
+    QDir directory(dirS.toString());
     do {
         const QString absDirPath = directory.absolutePath();
         if (absDirPath == root || absDirPath == home)
@@ -711,17 +713,17 @@ QString findRepositoryForDirectory(const QString &dirS, const QString &checkFile
 
         if (QFileInfo(directory, checkFile).isFile()) {
             qCDebug(findRepoLog) << "<" << absDirPath;
-            return absDirPath;
+            return FilePath::fromString(absDirPath);
         }
     } while (!directory.isRoot() && directory.cdUp());
     qCDebug(findRepoLog) << "< bailing out at" << directory.absolutePath();
-    return QString();
+    return {};
 }
 
 // Is SSH prompt configured?
 QString sshPrompt()
 {
-    return Internal::VcsPlugin::instance()->settings().sshPasswordPrompt;
+    return Internal::VcsPlugin::instance()->settings().sshPasswordPrompt.value();
 }
 
 bool isSshPromptConfigured()
@@ -742,31 +744,14 @@ QString source(IDocument *document)
     return document->property(SOURCE_PROPERTY).toString();
 }
 
-void setProcessEnvironment(QProcessEnvironment *e,
-                           bool forceCLocale,
-                           const QString &sshPromptBinary)
+void setProcessEnvironment(Environment *e, bool forceCLocale, const QString &sshPromptBinary)
 {
     if (forceCLocale) {
-        e->insert("LANG", "C");
-        e->insert("LANGUAGE", "C");
+        e->set("LANG", "C");
+        e->set("LANGUAGE", "C");
     }
     if (!sshPromptBinary.isEmpty())
-        e->insert("SSH_ASKPASS", sshPromptBinary);
-}
-
-// Run a process synchronously, returning Utils::SynchronousProcessResponse
-// response struct and using the VcsBasePlugin flags as applicable
-SynchronousProcessResponse runVcs(const QString &workingDir,
-                                  const CommandLine &cmd,
-                                  int timeOutS,
-                                  unsigned flags,
-                                  QTextCodec *outputCodec,
-                                  const QProcessEnvironment &env)
-{
-    VcsCommand command(workingDir, env.isEmpty() ? QProcessEnvironment::systemEnvironment() : env);
-    command.addFlags(flags);
-    command.setCodec(outputCodec);
-    return command.runCommand(cmd, timeOutS);
+        e->set("SSH_ASKPASS", sshPromptBinary);
 }
 
 } // namespace VcsBase
