@@ -27,12 +27,12 @@
 
 #include "vcsbase_global.h"
 
+#include <coreplugin/icontext.h>
 #include <coreplugin/iversioncontrol.h>
 #include <coreplugin/vcsmanager.h>
+
 #include <extensionsystem/iplugin.h>
 
-#include <QList>
-#include <QProcessEnvironment>
 #include <QSharedDataPointer>
 
 QT_BEGIN_NAMESPACE
@@ -41,14 +41,12 @@ class QTextCodec;
 QT_END_NAMESPACE
 
 namespace Utils {
-class CommandLine;
-class SynchronousProcessResponse;
+class Environment;
 } // namespace Utils
 
 namespace Core {
 class Context;
 class IVersionControl;
-class Id;
 class IDocument;
 } // namespace Core
 
@@ -59,7 +57,7 @@ namespace Internal { class State; }
 class VcsBaseSubmitEditor;
 class VcsBasePluginPrivate;
 class VcsBasePluginStateData;
-class VcsBasePlugin;
+class VcsBasePluginPrivate;
 
 // Documentation inside.
 class VCSBASE_EXPORT VcsBasePluginState
@@ -81,8 +79,8 @@ public:
     // Current file.
     QString currentFile() const;
     QString currentFileName() const;
-    QString currentFileDirectory() const;
-    QString currentFileTopLevel() const;
+    Utils::FilePath currentFileDirectory() const;
+    Utils::FilePath currentFileTopLevel() const;
     // Convenience: Returns file relative to top level.
     QString relativeCurrentFile() const;
 
@@ -92,9 +90,9 @@ public:
     QString currentPatchFileDisplayName() const;
 
     // Current project.
-    QString currentProjectPath() const;
+    Utils::FilePath currentProjectPath() const;
     QString currentProjectName() const;
-    QString currentProjectTopLevel() const;
+    Utils::FilePath currentProjectTopLevel() const;
     /* Convenience: Returns project path relative to top level if it
      * differs from top level (else empty string) as an argument list to do
      * eg a 'vcs diff <args>' */
@@ -102,14 +100,14 @@ public:
 
     // Top level directory for actions on the top level. Preferably
     // the file one.
-    QString topLevel() const;
+    Utils::FilePath topLevel() const;
 
     bool equals(const VcsBasePluginState &rhs) const;
 
     friend VCSBASE_EXPORT QDebug operator<<(QDebug in, const VcsBasePluginState &state);
 
 private:
-    friend class VcsBasePlugin;
+    friend class VcsBasePluginPrivate;
     bool equals(const Internal::State &s) const;
     void setState(const Internal::State &s);
 
@@ -123,58 +121,41 @@ inline bool operator==(const VcsBasePluginState &s1, const VcsBasePluginState &s
 inline bool operator!=(const VcsBasePluginState &s1, const VcsBasePluginState &s2)
 { return !s1.equals(s2); }
 
-class VCSBASE_EXPORT VcsBasePlugin : public ExtensionSystem::IPlugin
+// Convenience that searches for the repository specifically for version control
+// systems that do not have directories like "CVS" in each managed subdirectory
+// but have a directory at the top of the repository like ".git" containing
+// a well known file. See implementation for gory details.
+VCSBASE_EXPORT Utils::FilePath findRepositoryForFile(const Utils::FilePath &fileOrDir,
+                                                     const QString &checkFile);
+
+// Returns SSH prompt configured in settings.
+VCSBASE_EXPORT QString sshPrompt();
+// Returns whether an SSH prompt is configured.
+VCSBASE_EXPORT bool isSshPromptConfigured();
+
+// Set up the environment for a version control command line call.
+// Sets up SSH graphical password prompting (note that the latter
+// requires a terminal-less process) and sets LANG to 'C' to force English
+// (suppress LOCALE warnings/parse commands output) if desired.
+VCSBASE_EXPORT void setProcessEnvironment(Utils::Environment *e,
+                                          bool forceCLocale,
+                                          const QString &sshPasswordPrompt = sshPrompt());
+// Sets the source of editor contents, can be directory or file.
+VCSBASE_EXPORT void setSource(Core::IDocument *document, const QString &source);
+// Returns the source of editor contents.
+VCSBASE_EXPORT QString source(Core::IDocument *document);
+
+class VCSBASE_EXPORT VcsBasePluginPrivate : public Core::IVersionControl
 {
     Q_OBJECT
 
 protected:
-    explicit VcsBasePlugin();
-
-    template<class T, typename... Args>
-    T *initializeVcs(const Core::Context &context, Args&&... args)
-    {
-        T *vc = Core::VcsManager::registerVersionControl<T>(std::forward<Args>(args)...);
-        initializeVcs(vc, context);
-        return vc;
-    }
-
-    void extensionsInitialized() override;
+    explicit VcsBasePluginPrivate(const Core::Context &context);
 
 public:
-    ~VcsBasePlugin() override;
+    void extensionsInitialized();
 
     const VcsBasePluginState &currentState() const;
-    Core::IVersionControl *versionControl() const;
-
-    // Convenience that searches for the repository specifically for version control
-    // systems that do not have directories like "CVS" in each managed subdirectory
-    // but have a directory at the top of the repository like ".git" containing
-    // a well known file. See implementation for gory details.
-    static QString findRepositoryForDirectory(const QString &dir, const QString &checkFile);
-
-    // Set up the environment for a version control command line call.
-    // Sets up SSH graphical password prompting (note that the latter
-    // requires a terminal-less process) and sets LANG to 'C' to force English
-    // (suppress LOCALE warnings/parse commands output) if desired.
-    static void setProcessEnvironment(QProcessEnvironment *e,
-                                      bool forceCLocale,
-                                      const QString &sshPasswordPrompt = sshPrompt());
-    // Returns SSH prompt configured in settings.
-    static QString sshPrompt();
-    // Returns whether an SSH prompt is configured.
-    static bool isSshPromptConfigured();
-
-    // Sets the source of editor contents, can be directory or file.
-    static void setSource(Core::IDocument *document, const QString &source);
-    // Returns the source of editor contents.
-    static QString source(Core::IDocument *document);
-
-    static Utils::SynchronousProcessResponse runVcs(const QString &workingDir,
-                                                    const Utils::CommandLine &cmd,
-                                                    int timeOutS,
-                                                    unsigned flags = 0,
-                                                    QTextCodec *outputCodec = nullptr,
-                                                    const QProcessEnvironment &env = QProcessEnvironment());
 
     // Display name of the commit action
     virtual QString commitDisplayName() const;
@@ -218,11 +199,14 @@ protected:
 
 private:
     void slotSubmitEditorAboutToClose(VcsBaseSubmitEditor *submitEditor, bool *result);
-    void slotStateChanged(const VcsBase::Internal::State &s, Core::IVersionControl *vc);
+    void slotStateChanged(const Internal::State &s, Core::IVersionControl *vc);
 
-    void initializeVcs(Core::IVersionControl *vc, const Core::Context &context);
+    bool supportsRepositoryCreation() const;
 
-    VcsBasePluginPrivate *d;
+    QPointer<VcsBaseSubmitEditor> m_submitEditor;
+    Core::Context m_context;
+    VcsBasePluginState m_state;
+    int m_actionState = -1;
 };
 
 } // namespace VcsBase
