@@ -50,7 +50,7 @@
 #include <utils/fileutils.h>
 #include <utils/icon.h>
 #include <utils/qtcassert.h>
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 #include <utils/temporarydirectory.h>
 #include <utils/theme/theme.h>
 
@@ -128,7 +128,7 @@ using namespace Utils;
 
 static inline QString submitMessageCheckScript()
 {
-    return VcsPlugin::instance()->settings().submitMessageCheckScript;
+    return VcsPlugin::instance()->settings().submitMessageCheckScript.value();
 }
 
 class VcsBaseSubmitEditorPrivate
@@ -141,7 +141,7 @@ public:
     QToolBar *m_toolWidget = nullptr;
     VcsBaseSubmitEditorParameters m_parameters;
     QString m_displayName;
-    QString m_checkScriptWorkingDirectory;
+    FilePath m_checkScriptWorkingDirectory;
     SubmitEditorFile m_file;
 
     QPointer<QAction> m_diffAction;
@@ -199,32 +199,33 @@ void VcsBaseSubmitEditor::setParameters(const VcsBaseSubmitEditorParameters &par
     connect(descriptionEdit, &QTextEdit::textChanged,
             this, &VcsBaseSubmitEditor::fileContentsChanged);
 
-    const CommonVcsSettings settings = VcsPlugin::instance()->settings();
+    const CommonVcsSettings &settings = VcsPlugin::instance()->settings();
     // Add additional context menu settings
-    if (!settings.submitMessageCheckScript.isEmpty() || !settings.nickNameMailMap.isEmpty()) {
+    if (!settings.submitMessageCheckScript.value().isEmpty()
+            || !settings.nickNameMailMap.value().isEmpty()) {
         auto sep = new QAction(this);
         sep->setSeparator(true);
         d->m_widget->addDescriptionEditContextMenuAction(sep);
         // Run check action
-        if (!settings.submitMessageCheckScript.isEmpty()) {
+        if (!settings.submitMessageCheckScript.value().isEmpty()) {
             auto checkAction = new QAction(tr("Check Message"), this);
             connect(checkAction, &QAction::triggered,
                     this, &VcsBaseSubmitEditor::slotCheckSubmitMessage);
             d->m_widget->addDescriptionEditContextMenuAction(checkAction);
         }
         // Insert nick
-        if (!settings.nickNameMailMap.isEmpty()) {
+        if (!settings.nickNameMailMap.value().isEmpty()) {
             auto insertAction = new QAction(tr("Insert Name..."), this);
             connect(insertAction, &QAction::triggered, this, &VcsBaseSubmitEditor::slotInsertNickName);
             d->m_widget->addDescriptionEditContextMenuAction(insertAction);
         }
     }
     // Do we have user fields?
-    if (!settings.nickNameFieldListFile.isEmpty())
-        createUserFields(settings.nickNameFieldListFile);
+    if (!settings.nickNameFieldListFile.value().isEmpty())
+        createUserFields(settings.nickNameFieldListFile.value());
 
     // wrapping. etc
-    slotUpdateEditorSettings(settings);
+    slotUpdateEditorSettings();
     connect(VcsPlugin::instance(), &VcsPlugin::settingsChanged,
             this, &VcsBaseSubmitEditor::slotUpdateEditorSettings);
     connect(Core::EditorManager::instance(), &Core::EditorManager::currentEditorChanged,
@@ -250,10 +251,11 @@ VcsBaseSubmitEditor::~VcsBaseSubmitEditor()
     delete d;
 }
 
-void VcsBaseSubmitEditor::slotUpdateEditorSettings(const CommonVcsSettings &s)
+void VcsBaseSubmitEditor::slotUpdateEditorSettings()
 {
-    setLineWrapWidth(s.lineWrapWidth);
-    setLineWrap(s.lineWrap);
+    const CommonVcsSettings &s = VcsPlugin::instance()->settings();
+    setLineWrapWidth(s.lineWrapWidth.value());
+    setLineWrap(s.lineWrap.value());
 }
 
 // Return a trimmed list of non-empty field texts
@@ -272,8 +274,11 @@ static inline QStringList fieldTexts(const QString &fileContents)
 void VcsBaseSubmitEditor::createUserFields(const QString &fieldConfigFile)
 {
     FileReader reader;
-    if (!reader.fetch(fieldConfigFile, QIODevice::Text, Core::ICore::dialogParent()))
+    if (!reader.fetch(FilePath::fromString(fieldConfigFile),
+                      QIODevice::Text,
+                      Core::ICore::dialogParent())) {
         return;
+    }
     // Parse into fields
     const QStringList fields = fieldTexts(QString::fromUtf8(reader.data()));
     if (fields.empty())
@@ -347,10 +352,10 @@ Core::IDocument *VcsBaseSubmitEditor::document() const
 
 QString VcsBaseSubmitEditor::checkScriptWorkingDirectory() const
 {
-    return d->m_checkScriptWorkingDirectory;
+    return d->m_checkScriptWorkingDirectory.toString();
 }
 
-void VcsBaseSubmitEditor::setCheckScriptWorkingDirectory(const QString &s)
+void VcsBaseSubmitEditor::setCheckScriptWorkingDirectory(const FilePath &s)
 {
     d->m_checkScriptWorkingDirectory = s;
 }
@@ -514,12 +519,13 @@ static QString withUnusedMnemonic(QString string, const QList<QPushButton *> &ot
 
 VcsBaseSubmitEditor::PromptSubmitResult
         VcsBaseSubmitEditor::promptSubmit(VcsBasePluginPrivate *plugin,
-                                          bool *promptSetting,
+                                          bool *promptSettingOld,
                                           bool forcePrompt,
-                                          bool canCommitOnFailure)
+                                          bool canCommitOnFailure,
+                                          BoolAspect *promptSetting)
 {
-    bool dummySetting = false;
-    if (!promptSetting)
+    BoolAspect dummySetting;
+    if (!promptSetting && !promptSettingOld)
         promptSetting = &dummySetting;
     auto submitWidget = static_cast<SubmitEditorWidget *>(this->widget());
 
@@ -530,7 +536,8 @@ VcsBaseSubmitEditor::PromptSubmitResult
 
     QString errorMessage;
 
-    const bool prompt = forcePrompt || *promptSetting;
+    const bool value = promptSettingOld ? *promptSettingOld : promptSetting->value();
+    const bool prompt = forcePrompt || value;
 
     // Pop up a message depending on whether the check succeeded and the
     // user wants to be prompted
@@ -552,9 +559,9 @@ VcsBaseSubmitEditor::PromptSubmitResult
     }
     mb.setText(message);
     mb.setCheckBoxText(tr("Prompt to %1").arg(commitName.toLower()));
-    mb.setChecked(*promptSetting);
+    mb.setChecked(value);
     // Provide check box to turn off prompt ONLY if it was not forced
-    mb.setCheckBoxVisible(*promptSetting && !forcePrompt);
+    mb.setCheckBoxVisible(value && !forcePrompt);
     QDialogButtonBox::StandardButtons buttons = QDialogButtonBox::Close | QDialogButtonBox::Cancel;
     if (canCommit || canCommitOnFailure)
         buttons |= QDialogButtonBox::Ok;
@@ -570,8 +577,12 @@ VcsBaseSubmitEditor::PromptSubmitResult
         commitButton->setText(withUnusedMnemonic(commitName,
                               {cancelButton, mb.button(QDialogButtonBox::Close)}));
     }
-    if (mb.exec() == QDialog::Accepted)
-        *promptSetting = mb.isChecked();
+    if (mb.exec() == QDialog::Accepted) {
+        if (promptSettingOld)
+            *promptSettingOld = mb.isChecked();
+        else
+            promptSetting->setValue(mb.isChecked());
+    }
     QAbstractButton *chosen = mb.clickedButton();
     if (!chosen || chosen == cancelButton)
         return SubmitCanceled;
@@ -627,13 +638,13 @@ bool VcsBaseSubmitEditor::checkSubmitMessage(QString *errorMessage) const
     return rc;
 }
 
-static inline QString msgCheckScript(const QString &workingDir, const QString &cmd)
+static QString msgCheckScript(const FilePath &workingDir, const QString &cmd)
 {
     const QString nativeCmd = QDir::toNativeSeparators(cmd);
     return workingDir.isEmpty() ?
            VcsBaseSubmitEditor::tr("Executing %1").arg(nativeCmd) :
            VcsBaseSubmitEditor::tr("Executing [%1] %2").
-           arg(QDir::toNativeSeparators(workingDir), nativeCmd);
+           arg(workingDir.toUserOutput(), nativeCmd);
 }
 
 bool VcsBaseSubmitEditor::runSubmitMessageCheckScript(const QString &checkScript, QString *errorMessage) const
@@ -646,19 +657,19 @@ bool VcsBaseSubmitEditor::runSubmitMessageCheckScript(const QString &checkScript
     // Run check process
     VcsOutputWindow::appendShellCommandLine(msgCheckScript(d->m_checkScriptWorkingDirectory,
                                                            checkScript));
-    QProcess checkProcess;
+    QtcProcess checkProcess;
     if (!d->m_checkScriptWorkingDirectory.isEmpty())
         checkProcess.setWorkingDirectory(d->m_checkScriptWorkingDirectory);
-    checkProcess.start(checkScript, QStringList(saver.fileName()));
-    checkProcess.closeWriteChannel();
+    checkProcess.setCommand({FilePath::fromString(checkScript), {saver.filePath().toString()}});
+    checkProcess.start();
     if (!checkProcess.waitForStarted()) {
         *errorMessage = tr("The check script \"%1\" could not be started: %2").arg(checkScript, checkProcess.errorString());
         return false;
     }
     QByteArray stdOutData;
     QByteArray stdErrData;
-    if (!SynchronousProcess::readDataFromProcess(checkProcess, 30, &stdOutData, &stdErrData, false)) {
-        SynchronousProcess::stopProcess(checkProcess);
+    if (!checkProcess.readDataFromProcess(30, &stdOutData, &stdErrData, false)) {
+        checkProcess.stopProcess();
         *errorMessage = tr("The check script \"%1\" timed out.").
                         arg(QDir::toNativeSeparators(checkScript));
         return false;
