@@ -31,6 +31,8 @@
 
 namespace VcsBase {
 
+class VcsOutputWindow;
+
 class VCSBASE_EXPORT VcsCommand : public Core::ShellCommand
 {
     Q_OBJECT
@@ -41,21 +43,23 @@ public:
         ExpectRepoChanges = 0x2000, // Expect changes in repository by the command
     };
 
-    VcsCommand(const QString &defaultWorkingDirectory, const QProcessEnvironment &environment);
+    VcsCommand(const Utils::FilePath &defaultWorkingDirectory, const Utils::Environment &environment);
 
-    const QProcessEnvironment processEnvironment() const override;
+    const Utils::Environment processEnvironment() const override;
 
-    Utils::SynchronousProcessResponse runCommand(const Utils::CommandLine &command,
-                                                 int timeoutS,
-                                                 const QString &workDirectory = QString(),
-                                                 const Utils::ExitCodeInterpreter &interpreter = Utils::defaultExitCodeInterpreter) override;
+    void runCommand(Utils::QtcProcess &process,
+                    const Utils::CommandLine &command,
+                    const Utils::FilePath &workDirectory = {}) override;
+
+protected:
+    void addTask(QFuture<void> &future) override;
 
 private:
-    unsigned processFlags() const override;
-    void emitRepositoryChanged(const QString &workingDirectory);
+    void emitRepositoryChanged(const Utils::FilePath &workingDirectory);
 
     void coreAboutToClose() override;
 
+    QString m_sshPrompt;
     bool m_preventRepositoryChanged;
 };
 
