@@ -27,16 +27,14 @@
 
 #include <coreplugin/editormanager/editormanager.h>
 
-#include <aggregation/aggregate.h>
-#include <coreplugin/find/basetextfind.h>
 #include <coreplugin/outputwindow.h>
 #include <utils/fileutils.h>
-#include <utils/outputformatter.h>
 #include <utils/qtcprocess.h>
 #include <texteditor/behaviorsettings.h>
 #include <texteditor/fontsettings.h>
 #include <texteditor/texteditorsettings.h>
 #include <utils/theme/theme.h>
+#include <vcsbase/vcsoutputformatter.h>
 
 #include <QAction>
 #include <QContextMenuEvent>
@@ -46,7 +44,7 @@
 #include <QPlainTextEdit>
 #include <QPoint>
 #include <QPointer>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QTextBlock>
 #include <QTextBlockUserData>
 #include <QTextCharFormat>
@@ -96,21 +94,22 @@ class OutputWindowPlainTextEdit : public Core::OutputWindow
 {
 public:
     explicit OutputWindowPlainTextEdit(QWidget *parent = nullptr);
-    ~OutputWindowPlainTextEdit() override;
 
     void appendLines(const QString &s, const QString &repository = QString());
     void appendLinesWithStyle(const QString &s, VcsOutputWindow::MessageStyle style,
                               const QString &repository = QString());
+    VcsOutputLineParser *parser();
 
 protected:
     void contextMenuEvent(QContextMenuEvent *event) override;
+    void handleLink(const QPoint &pos) override;
 
 private:
     void setFormat(VcsOutputWindow::MessageStyle style);
     QString identifierUnderCursor(const QPoint &pos, QString *repository = nullptr) const;
 
     Utils::OutputFormat m_format;
-    OutputFormatter *m_formatter = nullptr;
+    VcsOutputLineParser *m_parser = nullptr;
 };
 
 OutputWindowPlainTextEdit::OutputWindowPlainTextEdit(QWidget *parent) :
@@ -119,17 +118,9 @@ OutputWindowPlainTextEdit::OutputWindowPlainTextEdit(QWidget *parent) :
     setReadOnly(true);
     setUndoRedoEnabled(false);
     setFrameStyle(QFrame::NoFrame);
-    m_formatter = new OutputFormatter;
-    m_formatter->setBoldFontEnabled(false);
-    m_formatter->setPlainTextEdit(this);
-    auto agg = new Aggregation::Aggregate;
-    agg->add(this);
-    agg->add(new Core::BaseTextFind(this));
-}
-
-OutputWindowPlainTextEdit::~OutputWindowPlainTextEdit()
-{
-    delete m_formatter;
+    outputFormatter()->setBoldFontEnabled(false);
+    m_parser = new VcsOutputLineParser;
+    setLineParsers({m_parser});
 }
 
 // Search back for beginning of word
@@ -173,10 +164,17 @@ QString OutputWindowPlainTextEdit::identifierUnderCursor(const QPoint &widgetPos
 
 void OutputWindowPlainTextEdit::contextMenuEvent(QContextMenuEvent *event)
 {
-    QMenu *menu = createStandardContextMenu();
+    const QString href = anchorAt(event->pos());
+    QMenu *menu = href.isEmpty() ? createStandardContextMenu(event->pos()) : new QMenu;
     // Add 'open file'
     QString repository;
     const QString token = identifierUnderCursor(event->pos(), &repository);
+    if (!repository.isEmpty()) {
+        if (VcsOutputLineParser * const p = parser()) {
+            if (!href.isEmpty())
+                p->fillLinkContextMenu(menu, FilePath::fromString(repository), href);
+        }
+    }
     QAction *openAction = nullptr;
     if (!token.isEmpty()) {
         // Check for a file, expand via repository if relative
@@ -190,9 +188,12 @@ void OutputWindowPlainTextEdit::contextMenuEvent(QContextMenuEvent *event)
             openAction->setData(fi.absoluteFilePath());
         }
     }
-    // Add 'clear'
-    menu->addSeparator();
-    QAction *clearAction = menu->addAction(VcsOutputWindow::tr("Clear"));
+    QAction *clearAction = nullptr;
+    if (href.isEmpty()) {
+        // Add 'clear'
+        menu->addSeparator();
+        clearAction = menu->addAction(VcsOutputWindow::tr("Clear"));
+    }
 
     // Run
     QAction *action = menu->exec(event->globalPos());
@@ -209,6 +210,23 @@ void OutputWindowPlainTextEdit::contextMenuEvent(QContextMenuEvent *event)
     delete menu;
 }
 
+void OutputWindowPlainTextEdit::handleLink(const QPoint &pos)
+{
+    const QString href = anchorAt(pos);
+    if (href.isEmpty())
+        return;
+    QString repository;
+    identifierUnderCursor(pos, &repository);
+    if (repository.isEmpty()) {
+        OutputWindow::handleLink(pos);
+        return;
+    }
+    if (outputFormatter()->handleFileLink(href))
+        return;
+    if (VcsOutputLineParser * const p = parser())
+        p->handleVcsLink(FilePath::fromString(repository), href);
+}
+
 void OutputWindowPlainTextEdit::appendLines(const QString &s, const QString &repository)
 {
     if (s.isEmpty())
@@ -216,10 +234,7 @@ void OutputWindowPlainTextEdit::appendLines(const QString &s, const QString &rep
 
     const int previousLineCount = document()->lineCount();
 
-    const QChar newLine('\n');
-    const QChar lastChar = s.at(s.size() - 1);
-    const bool appendNewline = (lastChar != '\r' && lastChar != newLine);
-    m_formatter->appendMessage(appendNewline ? s + newLine : s, m_format);
+    outputFormatter()->appendMessage(s, m_format);
 
     // Scroll down
     moveCursor(QTextCursor::End);
@@ -246,19 +261,24 @@ void OutputWindowPlainTextEdit::appendLinesWithStyle(const QString &s,
     }
 }
 
+VcsOutputLineParser *OutputWindowPlainTextEdit::parser()
+{
+    return m_parser;
+}
+
 void OutputWindowPlainTextEdit::setFormat(VcsOutputWindow::MessageStyle style)
 {
-    m_formatter->setBoldFontEnabled(style == VcsOutputWindow::Command);
+    outputFormatter()->setBoldFontEnabled(style == VcsOutputWindow::Command);
 
     switch (style) {
     case VcsOutputWindow::Warning:
         m_format = LogMessageFormat;
         break;
     case VcsOutputWindow::Error:
-        m_format = ErrorMessageFormat;
+        m_format = StdErrFormat;
         break;
     case VcsOutputWindow::Message:
-        m_format = NormalMessageFormat;
+        m_format = StdOutFormat;
         break;
     case VcsOutputWindow::Command:
         m_format = NormalMessageFormat;
@@ -278,7 +298,7 @@ class VcsOutputWindowPrivate
 public:
     Internal::OutputWindowPlainTextEdit widget;
     QString repository;
-    QRegExp passwordRegExp;
+    const QRegularExpression passwordRegExp = QRegularExpression("://([^@:]+):([^@]+)@");
 };
 
 static VcsOutputWindow *m_instance = nullptr;
@@ -287,7 +307,6 @@ static VcsOutputWindowPrivate *d = nullptr;
 VcsOutputWindow::VcsOutputWindow()
 {
     d = new VcsOutputWindowPrivate;
-    d->passwordRegExp = QRegExp("://([^@:]+):([^@]+)@");
     Q_ASSERT(d->passwordRegExp.isValid());
     m_instance = this;
 
@@ -299,25 +318,16 @@ VcsOutputWindow::VcsOutputWindow()
     updateBehaviorSettings();
     setupContext(Internal::C_VCS_OUTPUT_PANE, &d->widget);
 
-    connect(this, &IOutputPane::zoomIn, &d->widget, &Core::OutputWindow::zoomIn);
-    connect(this, &IOutputPane::zoomOut, &d->widget, &Core::OutputWindow::zoomOut);
-    connect(this, &IOutputPane::resetZoom, &d->widget, &Core::OutputWindow::resetZoom);
+    connect(this, &IOutputPane::zoomInRequested, &d->widget, &Core::OutputWindow::zoomIn);
+    connect(this, &IOutputPane::zoomOutRequested, &d->widget, &Core::OutputWindow::zoomOut);
+    connect(this, &IOutputPane::resetZoomRequested, &d->widget, &Core::OutputWindow::resetZoom);
     connect(TextEditor::TextEditorSettings::instance(), &TextEditor::TextEditorSettings::behaviorSettingsChanged,
             this, updateBehaviorSettings);
 }
 
-static QString filterPasswordFromUrls(const QString &input)
+static QString filterPasswordFromUrls(QString input)
 {
-    int pos = 0;
-    QString result = input;
-    while ((pos = d->passwordRegExp.indexIn(result, pos)) >= 0) {
-        QString tmp = result.left(pos + 3) + d->passwordRegExp.cap(1) + ":***@";
-        int newStart = tmp.count();
-        tmp += result.midRef(pos + d->passwordRegExp.matchedLength());
-        result = tmp;
-        pos = newStart;
-    }
-    return result;
+    return input.replace(d->passwordRegExp, "://\\1:***@");
 }
 
 VcsOutputWindow::~VcsOutputWindow()
@@ -348,12 +358,6 @@ void VcsOutputWindow::clearContents()
     d->widget.clear();
 }
 
-void VcsOutputWindow::visibilityChanged(bool visible)
-{
-    if (visible)
-        d->widget.setFocus();
-}
-
 void VcsOutputWindow::setFocus()
 {
     d->widget.setFocus();
@@ -425,40 +429,11 @@ void VcsOutputWindow::appendWarning(const QString &text)
     append(text, Warning, false);
 }
 
-// Helper to format arguments for log windows hiding common password options.
-static inline QString formatArguments(const QStringList &args)
+QString VcsOutputWindow::msgExecutionLogEntry(const FilePath &workingDir, const CommandLine &command)
 {
-    const char passwordOptionC[] = "--password";
-
-    QString rc;
-    QTextStream str(&rc);
-    const int size = args.size();
-    // Skip authentication options
-    for (int i = 0; i < size; i++) {
-        const QString arg = filterPasswordFromUrls(args.at(i));
-        if (i)
-            str << ' ';
-        if (arg.startsWith(QString::fromLatin1(passwordOptionC) + '=')) {
-            str << QtcProcess::quoteArg("--password=********");
-            continue;
-        }
-        str << QtcProcess::quoteArg(arg);
-        if (arg == passwordOptionC) {
-            str << ' ' << QtcProcess::quoteArg("********");
-            i++;
-        }
-    }
-    return rc;
-}
-
-QString VcsOutputWindow::msgExecutionLogEntry(const QString &workingDir, const CommandLine &command)
-{
-    const QString args = formatArguments(command.splitArguments());
-    const QString nativeExecutable = QtcProcess::quoteArg(command.executable().toUserOutput());
     if (workingDir.isEmpty())
-        return tr("Running: %1 %2").arg(nativeExecutable, args) + '\n';
-    return tr("Running in %1: %2 %3").
-            arg(QDir::toNativeSeparators(workingDir), nativeExecutable, args) + '\n';
+        return tr("Running: %1").arg(command.toUserOutput()) + '\n';
+    return tr("Running in %1: %2").arg(workingDir.toUserOutput(), command.toUserOutput()) + '\n';
 }
 
 void VcsOutputWindow::appendShellCommandLine(const QString &text)
@@ -466,7 +441,7 @@ void VcsOutputWindow::appendShellCommandLine(const QString &text)
     append(filterPasswordFromUrls(text), Command, true);
 }
 
-void VcsOutputWindow::appendCommand(const QString &workingDirectory, const CommandLine &command)
+void VcsOutputWindow::appendCommand(const FilePath &workingDirectory, const CommandLine &command)
 {
     appendShellCommandLine(msgExecutionLogEntry(workingDirectory, command));
 }
