@@ -27,6 +27,8 @@
 
 #include <extensionsystem/iplugin.h>
 
+#include <QFuture>
+
 QT_BEGIN_NAMESPACE
 class QStandardItemModel;
 QT_END_NAMESPACE
@@ -51,8 +53,9 @@ public:
     bool initialize(const QStringList &arguments, QString *errorMessage) override;
 
     static VcsPlugin *instance();
+    static void addFuture(const QFuture<void> &future);
 
-    CommonVcsSettings settings() const;
+    CommonVcsSettings &settings() const;
 
     // Model of user nick names used for the submit
     // editor. Stored centrally here to achieve delayed
@@ -60,7 +63,7 @@ public:
     QStandardItemModel *nickNameModel();
 
 signals:
-    void settingsChanged(const VcsBase::Internal::CommonVcsSettings &s);
+    void settingsChanged();
     void submitEditorAboutToClose(VcsBase::VcsBaseSubmitEditor *e, bool *result);
 
 private:
