@@ -121,6 +121,7 @@ private:
     QList<IWelcomePage *> m_pluginList;
     QList<WelcomePageButton *> m_pageButtons;
     Id m_activePage;
+    Id m_defaultPage;
 };
 
 class WelcomePlugin final : public ExtensionSystem::IPlugin
@@ -174,11 +175,12 @@ public:
         : QWidget(parent), m_iconSource(iconSource), m_title(title), m_openUrl(openUrl)
     {
         setAutoFillBackground(true);
-        setMinimumHeight(30);
+        setMinimumHeight(35);
         setToolTip(m_openUrl);
 
         const QString fileName = QString(":/welcome/images/%1.png").arg(iconSource);
-        const Icon icon({{fileName, Theme::Welcome_ForegroundPrimaryColor}}, Icon::Tint);
+        const Icon icon({{FilePath::fromString(fileName), Theme::Welcome_ForegroundPrimaryColor}},
+                        Icon::Tint);
 
         m_icon = new QLabel;
         m_icon->setPixmap(icon.pixmap());
@@ -274,7 +276,8 @@ public:
             auto getStartedButton = new WelcomePageButton(this);
             getStartedButton->setText(tr("Get Started Now"));
             getStartedButton->setOnClicked([] {
-                QDesktopServices::openUrl(QString("qthelp://org.qt-project.qtcreator/doc/index.html"));
+                QDesktopServices::openUrl(
+                    QString("qthelp://org.qt-project.qtcreator/doc/creator-getting-started.html"));
             });
             l->addWidget(getStartedButton);
 
@@ -286,7 +289,6 @@ public:
         {
             auto l = new QVBoxLayout;
             l->setContentsMargins(0, 0, 0, 0);
-            l->setSpacing(5);
             l->addWidget(new IconAndLink("download", tr("Get Qt"), "https://www.qt.io/download", this));
             l->addWidget(new IconAndLink("qtaccount", tr("Qt Account"), "https://account.qt.io", this));
             l->addWidget(new IconAndLink("community", tr("Online Community"), "https://forum.qt.io", this));
@@ -358,8 +360,10 @@ WelcomeMode::WelcomeMode()
 
 WelcomeMode::~WelcomeMode()
 {
-    QSettings *settings = ICore::settings();
-    settings->setValue(currentPageSettingsKeyC, m_activePage.toSetting());
+    QtcSettings *settings = ICore::settings();
+    settings->setValueWithDefault(currentPageSettingsKeyC,
+                                  m_activePage.toSetting(),
+                                  m_defaultPage.toSetting());
     delete m_modeWidget;
 }
 
@@ -371,13 +375,14 @@ void WelcomeMode::initPlugins()
     for (IWelcomePage *page : IWelcomePage::allWelcomePages())
         addPage(page);
 
-    if (!m_activePage.isValid() && !m_pageButtons.isEmpty()) {
+    if (!m_pageButtons.isEmpty()) {
         const int welcomeIndex = Utils::indexOf(m_pluginList,
                                                 Utils::equal(&IWelcomePage::id,
                                                              Utils::Id("Examples")));
         const int defaultIndex = welcomeIndex >= 0 ? welcomeIndex : 0;
-        m_activePage = m_pluginList.at(defaultIndex)->id();
-        m_pageButtons.at(defaultIndex)->click();
+        m_defaultPage = m_pluginList.at(defaultIndex)->id();
+        if (!m_activePage.isValid())
+            m_pageButtons.at(defaultIndex)->click();
     }
 }
 
@@ -395,7 +400,7 @@ bool WelcomeMode::openDroppedFiles(const QList<QUrl> &urls)
     const QList<QUrl> localUrls = Utils::filtered(urls, &QUrl::isLocalFile);
     if (!localUrls.isEmpty()) {
         QTimer::singleShot(0, [localUrls]() {
-            ICore::openFiles(Utils::transform(localUrls, &QUrl::toLocalFile), ICore::SwitchMode);
+            ICore::openFiles(Utils::transform(localUrls, &FilePath::fromUrl), ICore::SwitchMode);
         });
         return true;
     }
@@ -434,7 +439,7 @@ void WelcomeMode::addPage(IWelcomePage *page)
     auto onClicked = [this, pageId, stackPage] {
         m_activePage = pageId;
         m_pageStack->setCurrentWidget(stackPage);
-        for (WelcomePageButton *pageButton : m_pageButtons)
+        for (WelcomePageButton *pageButton : qAsConst(m_pageButtons))
             pageButton->recheckActive();
     };
 
