@@ -40,7 +40,6 @@
 #include <utils/portlist.h>
 
 #include <QFileInfo>
-#include <QCoreApplication>
 #include <QIcon>
 #include <QLoggingCategory>
 #include <QWizard>
@@ -56,7 +55,7 @@ using namespace Utils;
 namespace WinRt {
 namespace Internal {
 
-Q_LOGGING_CATEGORY(winrtDeviceLog, "qtc.winrt.deviceParser", QtWarningMsg)
+static Q_LOGGING_CATEGORY(winrtDeviceLog, "qtc.winrt.deviceParser", QtWarningMsg)
 
 WinRtDevice::WinRtDevice()
 {
@@ -99,7 +98,7 @@ QVariantMap WinRtDevice::toMap() const
     return map;
 }
 
-QString WinRtDevice::displayNameForType(Core::Id type)
+QString WinRtDevice::displayNameForType(Utils::Id type)
 {
     if (type == Constants::WINRT_DEVICE_TYPE_LOCAL)
         return QCoreApplication::translate("WinRt::Internal::WinRtDevice",
@@ -116,7 +115,7 @@ QString WinRtDevice::displayNameForType(Core::Id type)
 
 // Factory
 
-WinRtDeviceFactory::WinRtDeviceFactory(Core::Id deviceType)
+WinRtDeviceFactory::WinRtDeviceFactory(Utils::Id deviceType)
     : ProjectExplorer::IDeviceFactory(deviceType)
 {
     if (allPrerequisitesLoaded()) {
@@ -137,25 +136,22 @@ WinRtDeviceFactory::WinRtDeviceFactory(Core::Id deviceType)
 void WinRtDeviceFactory::autoDetect()
 {
     qCDebug(winrtDeviceLog) << __FUNCTION__;
-    MessageManager::write(tr("Running Windows Runtime device detection."));
     const QString runnerFilePath = findRunnerFilePath();
     if (runnerFilePath.isEmpty()) {
-        MessageManager::write(tr("No winrtrunner.exe found."));
+        qCDebug(winrtDeviceLog) << "No winrtrunner.exe found.";
         return;
     }
 
     if (!m_process) {
         qCDebug(winrtDeviceLog) << __FUNCTION__ << "Creating process";
         m_process = new Utils::QtcProcess(this);
-        connect(m_process, &QProcess::errorOccurred, this, &WinRtDeviceFactory::onProcessError);
-        connect(m_process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
-                this, &WinRtDeviceFactory::onProcessFinished);
+        connect(m_process, &QtcProcess::errorOccurred, this, &WinRtDeviceFactory::onProcessError);
+        connect(m_process, &QtcProcess::finished, this, &WinRtDeviceFactory::onProcessFinished);
     }
 
-    const CommandLine cmd{runnerFilePath, {"--list-devices"}};
+    const CommandLine cmd{FilePath::fromString(runnerFilePath), {"--list-devices"}};
     m_process->setCommand(cmd);
     qCDebug(winrtDeviceLog) << __FUNCTION__ << "Starting process" << cmd.toUserOutput();
-    MessageManager::write(cmd.toUserOutput());
     m_process->start();
     qCDebug(winrtDeviceLog) << __FUNCTION__ << "Process started";
 }
@@ -179,12 +175,14 @@ void WinRtDeviceFactory::onPrerequisitesLoaded()
 
 void WinRtDeviceFactory::onProcessError()
 {
-    MessageManager::write(tr("Error while executing winrtrunner: %1")
-                                .arg(m_process->errorString()), MessageManager::Flash);
+    MessageManager::writeDisrupting(
+        tr("Error while executing winrtrunner: %1").arg(m_process->errorString()));
 }
 
-void WinRtDeviceFactory::onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus)
+void WinRtDeviceFactory::onProcessFinished()
 {
+    int exitCode = m_process->exitCode();
+    QProcess::ExitStatus exitStatus = m_process->exitStatus();
     qCDebug(winrtDeviceLog) << __FUNCTION__ << "Exit code:" << exitCode <<"\tExit status:"
                             << exitStatus;
     if (exitStatus == QProcess::CrashExit) {
@@ -193,8 +191,7 @@ void WinRtDeviceFactory::onProcessFinished(int exitCode, QProcess::ExitStatus ex
     }
 
     if (exitCode != 0) {
-        MessageManager::write(tr("winrtrunner returned with exit code %1.")
-                                    .arg(exitCode), MessageManager::Flash);
+        MessageManager::writeFlashing(tr("winrtrunner returned with exit code %1.").arg(exitCode));
         return;
     }
 
@@ -314,7 +311,7 @@ void WinRtDeviceFactory::parseRunnerOutput(const QByteArray &output) const
             }
 
             const IDevice::MachineType machineType = machineTypeFromLine(line);
-            Core::Id deviceType;
+            Utils::Id deviceType;
             QString name;
             QString internalName = QStringLiteral("WinRT.");
             if (state == AppxState) {
@@ -337,7 +334,7 @@ void WinRtDeviceFactory::parseRunnerOutput(const QByteArray &output) const
                     deviceType = Constants::WINRT_DEVICE_TYPE_PHONE;
             }
             internalName += QString::number(deviceId);
-            const Core::Id internalId = Core::Id::fromString(internalName);
+            const Utils::Id internalId = Utils::Id::fromString(internalName);
             ++numFound;
             if (DeviceManager::instance()->find(internalId)) {
                 qCDebug(winrtDeviceLog) << __FUNCTION__ << "Skipping device with ID" << deviceId;
@@ -361,7 +358,7 @@ void WinRtDeviceFactory::parseRunnerOutput(const QByteArray &output) const
         message += QLatin1Char(' ');
         message += tr("%n of them are new.", nullptr, numNew);
     }
-    MessageManager::write(message);
+    qCDebug(winrtDeviceLog) << message;
 }
 
 } // Internal
