@@ -38,32 +38,34 @@ namespace Internal {
 // UninstallAfterStopAspect
 
 UninstallAfterStopAspect::UninstallAfterStopAspect()
-    : BaseBoolAspect("WinRtRunConfigurationUninstallAfterStopId")
+    : BoolAspect("WinRtRunConfigurationUninstallAfterStopId")
 {
-    setLabel(WinRtRunConfiguration::tr("Uninstall package after application stops"));
+    setLabel(WinRtRunConfiguration::tr("Uninstall package after application stops"),
+             LabelPlacement::AtCheckBox);
 }
 
 // LoopbackExemptClientAspect
 
 LoopbackExemptClientAspect::LoopbackExemptClientAspect()
-    : BaseBoolAspect("WinRtRunConfigurationLoopbackExemptClient")
+    : BoolAspect("WinRtRunConfigurationLoopbackExemptClient")
 {
-    setLabel(WinRtRunConfiguration::tr("Enable localhost communication for "
-                                       "clients"));
+    setLabel(WinRtRunConfiguration::tr("Enable localhost communication for clients"),
+             LabelPlacement::AtCheckBox);
 }
 
 // LoopbackExemptServerAspect
 
 LoopbackExemptServerAspect::LoopbackExemptServerAspect()
-    : BaseBoolAspect("WinRtRunConfigurationLoopbackExemptServer")
+    : BoolAspect("WinRtRunConfigurationLoopbackExemptServer")
 {
     setLabel(WinRtRunConfiguration::tr("Enable localhost communication for "
-                                       "servers (requires elevated rights)"));
+                                       "servers (requires elevated rights)"),
+             LabelPlacement::AtCheckBox);
 }
 
 // WinRtRunConfiguration
 
-WinRtRunConfiguration::WinRtRunConfiguration(Target *target, Core::Id id)
+WinRtRunConfiguration::WinRtRunConfiguration(Target *target, Utils::Id id)
     : RunConfiguration(target, id)
 {
     setDisplayName(tr("Run App Package"));
