@@ -55,7 +55,7 @@ public:
 
 signals:
     void started();
-    void finished(int exitCode, QProcess::ExitStatus exitStatus);
+    void finished();
     void error(QProcess::ProcessError error);
 
 private:
@@ -63,7 +63,7 @@ private:
 
     void onProcessReadyReadStdOut();
     void onProcessReadyReadStdErr();
-    void onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus);
+    void onProcessFinished();
     void onProcessError(QProcess::ProcessError processError);
 
     void startWinRtRunner(const RunConf &conf);
@@ -71,12 +71,12 @@ private:
 
     ProjectExplorer::RunWorker *m_worker = nullptr;
     WinRtDevice::ConstPtr m_device;
-    Utils::Environment m_environment;
     QString m_runnerFilePath;
     QString m_executableFilePath;
     QString m_debuggerExecutable;
     QString m_debuggerArguments;
     QString m_arguments;
+    QStringList m_loopbackArguments;
     bool m_uninstallAfterStop = false;
     Utils::QtcProcess *m_process = nullptr;
 };
