@@ -31,11 +31,11 @@ namespace Help {
 namespace Icons {
 
 const Utils::Icon MODE_HELP_CLASSIC(
-        QLatin1String(":/help/images/mode_help.png"));
+        ":/help/images/mode_help.png");
 const Utils::Icon MODE_HELP_FLAT({
-        {QLatin1String(":/help/images/mode_help_mask.png"), Utils::Theme::IconsBaseColor}});
+        {":/help/images/mode_help_mask.png", Utils::Theme::IconsBaseColor}});
 const Utils::Icon MODE_HELP_FLAT_ACTIVE({
-        {QLatin1String(":/help/images/mode_help_mask.png"), Utils::Theme::IconsModeHelpActiveColor}});
+        {":/help/images/mode_help_mask.png", Utils::Theme::IconsModeHelpActiveColor}});
 const Utils::Icon MACOS_TOUCHBAR_HELP(
         ":/help/images/macos_touchbar_help.png");
 
