@@ -34,6 +34,7 @@
 #include <utils/fancylineedit.h>
 #include <utils/hostosinfo.h>
 #include <utils/navigationtreeview.h>
+#include <utils/qtcassert.h>
 #include <utils/styledbar.h>
 
 #include <QAbstractItemModel>
@@ -51,7 +52,6 @@
 #include <QHelpLink>
 #endif
 
-
 using namespace Help::Internal;
 
 IndexWindow::IndexWindow()
@@ -118,15 +118,12 @@ void IndexWindow::setOpenInNewPageActionVisible(bool visible)
 
 void IndexWindow::filterIndices(const QString &filter)
 {
-    QModelIndex bestMatch;
-    if (filter.contains(QLatin1Char('*')))
-        bestMatch = m_filteredIndexModel->filter(filter, filter);
-    else
-        bestMatch = m_filteredIndexModel->filter(filter, QString());
-    if (bestMatch.isValid()) {
-        m_indexWidget->setCurrentIndex(bestMatch);
-        m_indexWidget->scrollTo(bestMatch);
-    }
+    const QString wildcard = filter.contains(QLatin1Char('*')) ? filter : QString();
+    const QModelIndex bestMatch = m_filteredIndexModel->filter(filter, wildcard);
+    if (!bestMatch.isValid())
+        return;
+    m_indexWidget->setCurrentIndex(bestMatch);
+    m_indexWidget->scrollTo(bestMatch);
 }
 
 bool IndexWindow::eventFilter(QObject *obj, QEvent *e)
@@ -215,12 +212,15 @@ void IndexWindow::open(const QModelIndex &index, bool newPage)
 
 Qt::DropActions IndexFilterModel::supportedDragActions() const
 {
+    if (!sourceModel())
+        return Qt::IgnoreAction;
     return sourceModel()->supportedDragActions();
 }
 
 QModelIndex IndexFilterModel::index(int row, int column, const QModelIndex &parent) const
 {
     Q_UNUSED(parent)
+    QTC_ASSERT(row < m_toSource.size(), return {});
     return createIndex(row, column);
 }
 
@@ -232,16 +232,34 @@ QModelIndex IndexFilterModel::parent(const QModelIndex &child) const
 
 int IndexFilterModel::rowCount(const QModelIndex &parent) const
 {
-    if (parent.isValid())
+    if (parent.isValid()) // our items don't have children
         return 0;
     return m_toSource.size();
 }
 
 int IndexFilterModel::columnCount(const QModelIndex &parent) const
 {
+    if (!sourceModel())
+        return 0;
     return sourceModel()->columnCount(mapToSource(parent));
 }
 
+QVariant IndexFilterModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+    // we don't show header
+    Q_UNUSED(section)
+    Q_UNUSED(orientation)
+    Q_UNUSED(role)
+    return {};
+}
+
+bool IndexFilterModel::hasChildren(const QModelIndex &parent) const
+{
+    if (parent.isValid()) // our items don't have children
+        return false;
+    return m_toSource.count();
+}
+
 void IndexFilterModel::setSourceModel(QAbstractItemModel *sm)
 {
     QAbstractItemModel *previousModel = sourceModel();
@@ -293,28 +311,32 @@ QModelIndex IndexFilterModel::filter(const QString &filter, const QString &wildc
     m_wildcard = wildcard;
     m_toSource.clear();
 
+    if (!sourceModel()) {
+        endResetModel();
+        return {};
+    }
+
     // adapted copy from QHelpIndexModel
 
     if (filter.isEmpty() && wildcard.isEmpty()) {
-        int count = sourceModel()->rowCount();
+        const int count = sourceModel()->rowCount();
         m_toSource.reserve(count);
         for (int i = 0; i < count; ++i)
             m_toSource.append(i);
         endResetModel();
+        if (m_toSource.isEmpty())
+            return {};
         return index(0, 0);
     }
 
-    QHelpIndexModel *indexModel = qobject_cast<QHelpIndexModel *>(sourceModel());
-    const QStringList indices = indexModel->stringList();
-    int goodMatch = -1;
-    int perfectMatch = -1;
+    const QStringList indices = static_cast<QHelpIndexModel *>(sourceModel())->stringList();
 
-    if (!wildcard.isEmpty()) {
-        const QRegularExpression regExp(QRegularExpression::wildcardToRegularExpression(wildcard),
-                                        QRegularExpression::CaseInsensitiveOption);
+    const auto match = [this, &indices, &filter] (std::function<bool(const QString &)> matcher) {
+        int goodMatch = -1;
+        int perfectMatch = -1;
         int i = 0;
         for (const QString &index : indices) {
-            if (index.contains(regExp)) {
+            if (matcher(index)) {
                 m_toSource.append(i);
                 if (perfectMatch == -1 && index.startsWith(filter, Qt::CaseInsensitive)) {
                     if (goodMatch == -1)
@@ -328,46 +350,45 @@ QModelIndex IndexFilterModel::filter(const QString &filter, const QString &wildc
             }
             ++i;
         }
-    } else {
-        int i = 0;
-        foreach (const QString &index, indices) {
-            if (index.contains(filter, Qt::CaseInsensitive)) {
-                m_toSource.append(i);
-                if (perfectMatch == -1 && index.startsWith(filter, Qt::CaseInsensitive)) {
-                    if (goodMatch == -1)
-                        goodMatch = m_toSource.size() - 1;
-                    if (filter.length() == index.length()){
-                        perfectMatch = m_toSource.size() - 1;
-                    }
-                } else if (perfectMatch > -1 && index == filter) {
-                    perfectMatch = m_toSource.size() - 1;
-                }
-            }
-            ++i;
-        }
-    }
+        return perfectMatch >= 0 ? perfectMatch : goodMatch;
+    };
 
-    if (perfectMatch == -1)
-        perfectMatch = qMax(0, goodMatch);
+    const auto matchSimpleOrRegExp = [&] () {
+        if (wildcard.isEmpty()) {
+            return match([&filter] (const QString &index) {
+                return index.contains(filter, Qt::CaseInsensitive);
+            });
+        }
+        const QRegularExpression regExp(QRegularExpression::wildcardToRegularExpression(wildcard),
+                                        QRegularExpression::CaseInsensitiveOption);
+        return match([&regExp] (const QString &index) {
+            return index.contains(regExp);
+        });
+    };
+    const int matchedIndex = matchSimpleOrRegExp();
 
     endResetModel();
-    return index(perfectMatch, 0, QModelIndex());
+    if (matchedIndex == -1)
+        return {};
+    return index(matchedIndex, 0);
 }
 
 QModelIndex IndexFilterModel::mapToSource(const QModelIndex &proxyIndex) const
 {
-    if (!proxyIndex.isValid() || proxyIndex.parent().isValid() || proxyIndex.row() >= m_toSource.size())
-        return QModelIndex();
-    return index(m_toSource.at(proxyIndex.row()), proxyIndex.column());
+    if (!sourceModel() || !proxyIndex.isValid()
+            || proxyIndex.parent().isValid() || proxyIndex.row() >= m_toSource.size()) {
+        return {};
+    }
+    return sourceModel()->index(m_toSource.at(proxyIndex.row()), proxyIndex.column());
 }
 
 QModelIndex IndexFilterModel::mapFromSource(const QModelIndex &sourceIndex) const
 {
     if (!sourceIndex.isValid() || sourceIndex.parent().isValid())
-        return QModelIndex();
-    int i = m_toSource.indexOf(sourceIndex.row());
+        return {};
+    const int i = m_toSource.indexOf(sourceIndex.row());
     if (i < 0)
-        return QModelIndex();
+        return {};
     return index(i, sourceIndex.column());
 }
 
