@@ -458,6 +458,7 @@ QMakeEvaluator::writeFile(const QString &ctx, const QString &fn, QIODevice::Open
 void QMakeEvaluator::runProcess(QProcess *proc, const QString &command) const
 {
     proc->setWorkingDirectory(currentDirectory());
+    proc->setStandardInputFile(QProcess::nullDevice());
 # ifdef PROEVALUATOR_SETENV
     if (!m_option->environment.isEmpty()) {
         QProcessEnvironment env = m_option->environment;
@@ -472,13 +473,33 @@ void QMakeEvaluator::runProcess(QProcess *proc, const QString &command) const
         proc->setProcessEnvironment(env);
     }
 # endif
+# ifdef PROEVALUATOR_THREAD_SAFE
+    if (m_option->canceled)
+        return;
+# endif
 # ifdef Q_OS_WIN
     proc->setNativeArguments(QLatin1String("/v:off /s /c \"") + command + QLatin1Char('"'));
     proc->start(m_option->getEnv(QLatin1String("COMSPEC")), QStringList());
 # else
     proc->start(QLatin1String("/bin/sh"), QStringList() << QLatin1String("-c") << command);
 # endif
-    proc->waitForFinished(-1);
+# ifdef PROEVALUATOR_THREAD_SAFE
+    while (true) {
+        if (proc->waitForFinished(100))
+            break;
+        if (m_option->canceled) {
+            proc->terminate();
+            if (proc->waitForFinished(1000))
+                break;
+            proc->kill();
+            proc->waitForFinished(1000);
+            break;
+        }
+    }
+# else
+    proc->waitForFinished(-1); // If have have single thread we can't cancel it using
+                               // synchronous API of QProcess
+# endif
 }
 #endif
 
@@ -489,7 +510,7 @@ QByteArray QMakeEvaluator::getCommandOutput(const QString &args, int *exitCode)
     QProcess proc;
     runProcess(&proc, args);
     *exitCode = (proc.exitStatus() == QProcess::NormalExit) ? proc.exitCode() : -1;
-    QByteArray errout = proc.readAllStandardError();
+    QByteArray errout = proc.isReadable() ? proc.readAllStandardError() : QByteArray();
 # ifdef PROEVALUATOR_FULL
     // FIXME: Qt really should have the option to set forwarding per channel
     fputs(errout.constData(), stderr);
@@ -502,7 +523,7 @@ QByteArray QMakeEvaluator::getCommandOutput(const QString &args, int *exitCode)
             QString::fromLocal8Bit(errout));
     }
 # endif
-    out = proc.readAllStandardOutput();
+    out = proc.isReadable() ? proc.readAllStandardOutput() : QByteArray();
 # ifdef Q_OS_WIN
     // FIXME: Qt's line end conversion on sequential files should really be fixed
     out.replace("\r\n", "\n");
