@@ -42,66 +42,127 @@ QT_BEGIN_NAMESPACE
 //
 ///////////////////////////////////////////////////////////////////////
 
-ProFileCache::~ProFileCache()
+ProFileCache::ProFileCache()
 {
-    foreach (const Entry &ent, parsed_files)
-        if (ent.pro)
-            ent.pro->deref();
+    QMakeVfs::ref();
 }
 
-void ProFileCache::discardFile(const QString &fileName)
+ProFileCache::~ProFileCache()
+{
+    for (const auto &keyValuePair : parsed_files)
+        if (keyValuePair.second.pro)
+            keyValuePair.second.pro->deref();
+    QMakeVfs::deref();
+}
+
+void ProFileCache::discardFile(const QString &fileName, QMakeVfs *vfs)
+{
+    int eid = vfs->idForFileName(fileName, QMakeVfs::VfsExact | QMakeVfs::VfsAccessedOnly);
+    if (eid)
+        discardFile(eid);
+    int cid = vfs->idForFileName(fileName, QMakeVfs::VfsCumulative | QMakeVfs::VfsAccessedOnly);
+    if (cid && cid != eid)
+        discardFile(cid);
+}
+
+void ProFileCache::discardFile(int id)
 {
 #ifdef PROPARSER_THREAD_SAFE
     QMutexLocker lck(&mutex);
 #endif
-    QHash<QString, Entry>::Iterator it = parsed_files.find(fileName);
+    auto it = parsed_files.find(id);
     if (it != parsed_files.end()) {
+        Entry &entry = it->second;
 #ifdef PROPARSER_THREAD_SAFE
-        if (it->locker) {
-            if (!it->locker->done) {
-                ++it->locker->waiters;
-                it->locker->cond.wait(&mutex);
-                if (!--it->locker->waiters) {
-                    delete it->locker;
-                    it->locker = 0;
+        Entry::Locker *locker = entry.locker;
+        if (locker) { // Either it's still being prepared or it's already done but someone
+                      // else started waiting for it when it wasn't ready yet
+                      // and is still waiting.
+            if (!locker->done) {
+                ++locker->waiters;
+                locker->cond.wait(&mutex);          // Mutex is unlocked and relocked,
+                                                    // everything may happen in this time window
+                it = parsed_files.find(id);         // In meantime the iterator could have
+                                                    // been invalidated, search for it again
+                Q_ASSERT(it != parsed_files.end()); // This would mean that some other
+                                                    // waiter removed our entry, but only
+                                                    // the last waiter should do it
+                if (!--locker->waiters) {
+                    delete locker;
+                } else {
+                    // There are still other waiters. Do nothing now.
+                    locker->removeOnLastWaiter = true;
+                    return;
                 }
+            } else if (locker->waiters) {
+                // It's done but some waiters are still awaiting to be woken.
+                locker->removeOnLastWaiter = true; // Mark it for other waiters
+                return;
             }
         }
 #endif
-        if (it->pro)
-            it->pro->deref();
+        if (entry.pro)
+            entry.pro->deref();
         parsed_files.erase(it);
     }
 }
 
-void ProFileCache::discardFiles(const QString &prefix)
+void ProFileCache::discardFiles(const QString &prefix, QMakeVfs *vfs)
 {
 #ifdef PROPARSER_THREAD_SAFE
     QMutexLocker lck(&mutex);
 #endif
-    QHash<QString, Entry>::Iterator
-            it = parsed_files.begin(),
-            end = parsed_files.end();
-    while (it != end)
-        if (it.key().startsWith(prefix)) {
+    auto it = parsed_files.begin();
+    while (it != parsed_files.end()) {
+        const int id = it->first;
+        // Note: this is empty for virtual files from other VFSes.
+        const QString fn = vfs->fileNameForId(id);
+        if (fn.startsWith(prefix)) {
+            bool continueFromScratch = false;
+            Entry &entry = it->second;
 #ifdef PROPARSER_THREAD_SAFE
-            if (it->locker) {
-                if (!it->locker->done) {
-                    ++it->locker->waiters;
-                    it->locker->cond.wait(&mutex);
-                    if (!--it->locker->waiters) {
-                        delete it->locker;
-                        it->locker = 0;
+            Entry::Locker *locker = entry.locker;
+            if (locker) { // Either it's still being prepared or it's already done but someone
+                          // else started waiting for it when it wasn't ready yet
+                          // and is still waiting.
+                if (!locker->done) { // It's still being prepared.
+                    ++locker->waiters;
+                    locker->cond.wait(&mutex);          // Mutex is unlocked and relocked,
+                                                        // everything may happen in this time window
+                    it = parsed_files.find(id);         // In meantime the iterator could have
+                                                        // been invalidated, search for it again
+                    Q_ASSERT(it != parsed_files.end()); // This would mean that some other
+                                                        // waiter removed our entry, but only
+                                                        // the last waiter should do it
+                    if (!--locker->waiters) {
+                        delete locker;                  // No more waiters, remove locker and entry
+                        continueFromScratch = true;
+                    } else {
+                        // There are still other waiters. Do nothing now
+                        locker->removeOnLastWaiter = true;
+                        it = parsed_files.begin();      // Start from scratch, as new matching
+                                                        // entries could have appeared.
+                                                        // If we hit it again, we will skip it
+                                                        // as the locker is already marked as done
+                        continue;
                     }
+                } else if (locker->waiters) {
+                    // It's done but some waiters are still awaiting to be woken.
+                    locker->removeOnLastWaiter = true; // Mark it for other waiters
+                    ++it; // We skip it for now, other waiter will do the job
+                    continue;
                 }
             }
 #endif
-            if (it->pro)
-                it->pro->deref();
+            if (entry.pro)
+                entry.pro->deref();
             it = parsed_files.erase(it);
+            if (continueFromScratch)
+                it = parsed_files.begin();
         } else {
             ++it;
         }
+    }
 }
 
 ////////// Parser ///////////
@@ -115,6 +176,7 @@ static struct {
     QString strfor;
     QString strdefineTest;
     QString strdefineReplace;
+    QString strbypassNesting;
     QString stroption;
     QString strreturn;
     QString strnext;
@@ -138,6 +200,7 @@ void QMakeParser::initialize()
     statics.strfor = QLatin1String("for");
     statics.strdefineTest = QLatin1String("defineTest");
     statics.strdefineReplace = QLatin1String("defineReplace");
+    statics.strbypassNesting = QLatin1String("bypassNesting");
     statics.stroption = QLatin1String("option");
     statics.strreturn = QLatin1String("return");
     statics.strnext = QLatin1String("next");
@@ -162,52 +225,64 @@ QMakeParser::QMakeParser(ProFileCache *cache, QMakeVfs *vfs, QMakeParserHandler
 ProFile *QMakeParser::parsedProFile(const QString &fileName, ParseFlags flags)
 {
     ProFile *pro;
-    if ((flags & (ParseUseCache|ParseOnlyCached)) && m_cache) {
+    QMakeVfs::VfsFlags vfsFlags = ((flags & ParseCumulative) ? QMakeVfs::VfsCumulative
+                                                             : QMakeVfs::VfsExact);
+    int id = m_vfs->idForFileName(fileName, vfsFlags);
+    if ((flags & ParseUseCache) && m_cache) {
         ProFileCache::Entry *ent;
 #ifdef PROPARSER_THREAD_SAFE
-        QMutexLocker locker(&m_cache->mutex);
+        QMutexLocker lck(&m_cache->mutex);
 #endif
-        QHash<QString, ProFileCache::Entry>::Iterator it;
-#ifdef PROEVALUATOR_DUAL_VFS
-        QString virtFileName = ((flags & ParseCumulative) ? '-' : '+') + fileName;
-        it = m_cache->parsed_files.find(virtFileName);
-        if (it == m_cache->parsed_files.end())
-#endif
-            it = m_cache->parsed_files.find(fileName);
+        auto it = m_cache->parsed_files.find(id);
         if (it != m_cache->parsed_files.end()) {
-            ent = &*it;
+            ent = &it->second;
 #ifdef PROPARSER_THREAD_SAFE
-            if (ent->locker && !ent->locker->done) {
-                ++ent->locker->waiters;
-                QThreadPool::globalInstance()->releaseThread();
-                ent->locker->cond.wait(locker.mutex());
-                QThreadPool::globalInstance()->reserveThread();
-                if (!--ent->locker->waiters) {
-                    delete ent->locker;
-                    ent->locker = 0;
+            ProFileCache::Entry::Locker *locker = ent->locker;
+            if (locker) { // Either it's still being prepared or it's already done but someone
+                          // else started waiting for it when it wasn't ready yet
+                          // and is still waiting.
+                if (!locker->done) { // It's still being prepared.
+                    ++locker->waiters;
+                    QThreadPool::globalInstance()->releaseThread();
+                    locker->cond.wait(lck.mutex());  // Mutex is unlocked and relocked,
+                                                     // everything may happen in this time window
+                    QThreadPool::globalInstance()->reserveThread();
+
+                    it = m_cache->parsed_files.find(id);         // In meantime the iterator could have
+                                                                 // been invalidated, search for it again
+                    Q_ASSERT(it != m_cache->parsed_files.end()); // This would mean that some other
+                                                                 // waiter removed our entry, but only
+                                                                 // the last waiter should do it
+                    const bool remove = locker->removeOnLastWaiter; // Some waiter marked this
+                                                                    // entry for removal.
+                    if (!--locker->waiters) {                    // If we were the last waiter
+                        delete locker;                           // we do it now
+                        ent->locker = 0;
+                        if (remove) {
+                            if (ent->pro)
+                                ent->pro->deref();
+                            m_cache->parsed_files.erase(it);
+                            return nullptr;
+                        }
+                    }
+                    if (remove)
+                        return nullptr;
+                } else if (locker->removeOnLastWaiter) {
+                    return nullptr;
                 }
             }
 #endif
             if ((pro = ent->pro))
                 pro->ref();
-        } else if (!(flags & ParseOnlyCached)) {
-            QString contents;
-            QMakeVfs::VfsFlags vfsFlags =
-                    ((flags & ParseCumulative) ? QMakeVfs::VfsCumulative : QMakeVfs::VfsExact);
-            bool virt = false;
-#ifdef PROEVALUATOR_DUAL_VFS
-            virt = m_vfs->readVirtualFile(fileName, vfsFlags, &contents);
-            if (virt)
-                ent = &m_cache->parsed_files[virtFileName];
-            else
-#endif
-                ent = &m_cache->parsed_files[fileName];
+        } else {
+            ent = &m_cache->parsed_files[id];
 #ifdef PROPARSER_THREAD_SAFE
             ent->locker = new ProFileCache::Entry::Locker;
-            locker.unlock();
+            lck.unlock(); // We unlock the mutex and open a time window
 #endif
-            if (virt || readFile(fileName, vfsFlags | QMakeVfs::VfsNoVirtual, flags, &contents)) {
-                pro = parsedProBlock(QStringRef(&contents), fileName, 1, FullGrammar);
+            QString contents;
+            if (readFile(id, flags, &contents)) {
+                pro = parsedProBlock(Utils::make_stringview(contents), id, fileName, 1, FullGrammar);
                 pro->itemsRef()->squeeze();
                 pro->ref();
             } else {
@@ -215,55 +290,54 @@ ProFile *QMakeParser::parsedProFile(const QString &fileName, ParseFlags flags)
             }
             ent->pro = pro;
 #ifdef PROPARSER_THREAD_SAFE
-            locker.relock();
-            if (ent->locker->waiters) {
-                ent->locker->done = true;
+            lck.relock(); // We relock it back and close the window. Someone could have added a new
+                          // entry to the parsed_files list (or removed one). However, no one
+                          // was awoken for our id in this time window.
+            it = m_cache->parsed_files.find(id);         // In meantime the iterator could have
+                                                         // been invalidated, search for it again
+            Q_ASSERT(it != m_cache->parsed_files.end()); // This would mean that our reference
+                                                         // is also invalid
+            if (ent->locker->waiters) {             // Someone runs getter or wants to remove it
+                ent->locker->done = true;           // The structure is ready
                 ent->locker->cond.wakeAll();
-            } else {
+            } else {          // No one was interested, remove locker as everything is ready now
                 delete ent->locker;
                 ent->locker = 0;
             }
 #endif
-        } else {
-            pro = 0;
         }
-    } else if (!(flags & ParseOnlyCached)) {
+    } else {
         QString contents;
-        QMakeVfs::VfsFlags vfsFlags =
-                ((flags & ParseCumulative) ? QMakeVfs::VfsCumulative : QMakeVfs::VfsExact);
-        if (readFile(fileName, vfsFlags, flags, &contents))
-            pro = parsedProBlock(QStringRef(&contents), fileName, 1, FullGrammar);
+        if (readFile(id, flags, &contents))
+            pro = parsedProBlock(Utils::make_stringview(contents), id, fileName, 1, FullGrammar);
         else
             pro = 0;
-    } else {
-        pro = 0;
     }
     return pro;
 }
 
 ProFile *QMakeParser::parsedProBlock(
-        const QStringRef &contents, const QString &name, int line, SubGrammar grammar)
+    Utils::StringView contents, int id, const QString &name, int line, SubGrammar grammar)
 {
-    ProFile *pro = new ProFile(name);
+    ProFile *pro = new ProFile(id, name);
     read(pro, contents, line, grammar);
     return pro;
 }
 
-void QMakeParser::discardFileFromCache(const QString &fileName)
+void QMakeParser::discardFileFromCache(int id)
 {
     if (m_cache)
-        m_cache->discardFile(fileName);
+        m_cache->discardFile(id);
 }
 
-bool QMakeParser::readFile(
-        const QString &fn, QMakeVfs::VfsFlags vfsFlags, ParseFlags flags, QString *contents)
+bool QMakeParser::readFile(int id, ParseFlags flags, QString *contents)
 {
     QString errStr;
-    QMakeVfs::ReadResult result = m_vfs->readFile(fn, vfsFlags, contents, &errStr);
+    QMakeVfs::ReadResult result = m_vfs->readFile(id, contents, &errStr);
     if (result != QMakeVfs::ReadOk) {
         if (m_handler && ((flags & ParseReportMissing) || result != QMakeVfs::ReadNotFound))
             m_handler->message(QMakeParserHandler::ParserIoError,
-                               fL1S("Cannot read %1: %2").arg(fn, errStr));
+                               fL1S("Cannot read %1: %2").arg(m_vfs->fileNameForId(id), errStr));
         return false;
     }
     return true;
@@ -307,7 +381,7 @@ void QMakeParser::finalizeHashStr(ushort *buf, uint len)
     buf[-2] = (ushort)(hash >> 16);
 }
 
-void QMakeParser::read(ProFile *pro, const QStringRef &in, int line, SubGrammar grammar)
+void QMakeParser::read(ProFile *pro, Utils::StringView in, int line, SubGrammar grammar)
 {
     m_proFile = pro;
     m_lineNo = line;
@@ -355,7 +429,7 @@ void QMakeParser::read(ProFile *pro, const QStringRef &in, int line, SubGrammar
     QStack<ParseCtx> xprStack;
     xprStack.reserve(10);
 
-    const ushort *cur = (const ushort *)in.unicode();
+    const ushort *cur = (const ushort *)in.data();
     const ushort *inend = cur + in.length();
     m_canElse = false;
   freshLine:
@@ -885,7 +959,7 @@ void QMakeParser::read(ProFile *pro, const QStringRef &in, int line, SubGrammar
     flushScopes(tokPtr);
     if (m_blockstack.size() > 1 || m_blockstack.top().braceLevel)
         parseError(fL1S("Missing closing brace(s)."));
-    while (m_blockstack.size())
+    while (!m_blockstack.isEmpty())
         leaveScope(tokPtr);
     tokBuff.resize(tokPtr - (ushort *)tokBuff.constData()); // Reserved capacity stays
     *pro->itemsRef() = tokBuff;
@@ -1171,6 +1245,25 @@ void QMakeParser::finalizeCall(ushort *&tokPtr, ushort *uc, ushort *ptr, int arg
                 }
                 parseError(fL1S("%1(function) requires one literal argument.").arg(*defName));
                 return;
+            } else if (m_tmp == statics.strbypassNesting) {
+                if (*uce != TokFuncTerminator) {
+                    bogusTest(tokPtr, fL1S("%1() requires zero arguments.").arg(m_tmp));
+                    return;
+                }
+                if (!(m_blockstack.top().nest & NestFunction)) {
+                    bogusTest(tokPtr, fL1S("Unexpected %1().").arg(m_tmp));
+                    return;
+                }
+                if (m_invert) {
+                    bogusTest(tokPtr, fL1S("Unexpected NOT operator in front of %1().").arg(m_tmp));
+                    return;
+                }
+                flushScopes(tokPtr);
+                putLineMarker(tokPtr);
+                putOperator(tokPtr);
+                putTok(tokPtr, TokBypassNesting);
+                enterScope(tokPtr, true, StCtrl);
+                return;
             } else if (m_tmp == statics.strreturn) {
                 if (m_blockstack.top().nest & NestFunction) {
                     if (argc > 1) {
@@ -1238,7 +1331,7 @@ void QMakeParser::finalizeCall(ushort *&tokPtr, ushort *uc, ushort *ptr, int arg
 bool QMakeParser::resolveVariable(ushort *xprPtr, int tlen, int needSep, ushort **ptr,
                                   ushort **buf, QString *xprBuff,
                                   ushort **tokPtr, QString *tokBuff,
-                                  const ushort *cur, const QStringRef &in)
+                                  const ushort *cur, Utils::StringView in)
 {
     QString out;
     m_tmp.setRawData((const QChar *)xprPtr, tlen);
@@ -1439,7 +1532,7 @@ static bool getBlock(const ushort *tokens, int limit, int &offset, QString *outS
         "TokReturn", "TokBreak", "TokNext",
         "TokNot", "TokAnd", "TokOr",
         "TokBranch", "TokForLoop",
-        "TokTestDef", "TokReplaceDef"
+        "TokTestDef", "TokReplaceDef", "TokBypassNesting"
     };
 
     while (offset != limit) {
@@ -1523,6 +1616,9 @@ static bool getBlock(const ushort *tokens, int limit, int &offset, QString *outS
                 if (ok)
                     ok = getSubBlock(tokens, limit, offset, outStr, indent, "body");
                 break;
+            case TokBypassNesting:
+                ok = getSubBlock(tokens, limit, offset, outStr, indent, "block");
+                break;
             default:
                 Q_ASSERT(!"unhandled token");
             }
