@@ -51,6 +51,7 @@
 
 #include <QtCore/qdir.h>
 #include <QtCore/qprocess.h>
+#include <QtCore/qregularexpression.h>
 
 #include <algorithm>
 
@@ -58,6 +59,9 @@ using namespace qbs;
 using Internal::HostOsInfo;
 using Internal::Tr;
 
+constexpr char kUninstallRegistryKey[] = "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\" \
+                                         "Windows\\CurrentVersion\\Uninstall\\";
+
 static QString qsystem(const QString &exe, const QStringList &args = QStringList())
 {
     QProcess p;
@@ -107,13 +111,22 @@ class ToolchainDetails
 public:
     explicit ToolchainDetails(const QFileInfo &compiler)
     {
-        auto baseName = compiler.completeBaseName();
-        if (baseName.contains(QRegExp(QLatin1String("\\d$")))) {
-            const auto dashIndex = baseName.lastIndexOf(QLatin1Char('-'));
-            version = baseName.mid(dashIndex + 1);
-            baseName = baseName.left(dashIndex);
+        auto baseName = HostOsInfo::stripExecutableSuffix(compiler.fileName());
+        // Extract the version sub-string if it exists. We assume that a version
+        // sub-string located after the compiler prefix && suffix. E.g. this code
+        // parses a version from the compiler names, like this:
+        // - avr-gcc-4.9.2.exe
+        // - arm-none-eabi-gcc-8.2.1
+        // - rl78-elf-gcc-4.9.2.201902-GNURL78
+        const QRegularExpression re(QLatin1String("-(\\d+|\\d+\\.\\d+|" \
+                                                  "\\d+\\.\\d+\\.\\d+|" \
+                                                  "\\d+\\.\\d+\\.\\d+\\.\\d+)" \
+                                                  "[-[0-9a-zA-Z]*]?$"));
+        const QRegularExpressionMatch match = re.match(baseName);
+        if (match.hasMatch()) {
+            version = match.captured(1);
+            baseName = baseName.left(match.capturedStart());
         }
-
         const auto dashIndex = baseName.lastIndexOf(QLatin1Char('-'));
         suffix = baseName.mid(dashIndex + 1);
         prefix = baseName.left(dashIndex + 1);
@@ -125,9 +138,9 @@ public:
 };
 
 static void setCommonProperties(Profile &profile, const QFileInfo &compiler,
-        const QStringList &toolchainTypes, const ToolchainDetails &details)
+        const QString &toolchainType, const ToolchainDetails &details)
 {
-    if (toolchainTypes.contains(QStringLiteral("mingw")))
+    if (toolchainType == QStringLiteral("mingw"))
         profile.setValue(QStringLiteral("qbs.targetPlatform"),
                          QStringLiteral("windows"));
 
@@ -136,7 +149,7 @@ static void setCommonProperties(Profile &profile, const QFileInfo &compiler,
 
     profile.setValue(QStringLiteral("cpp.toolchainInstallPath"),
                      compiler.absolutePath());
-    profile.setValue(QStringLiteral("qbs.toolchain"), toolchainTypes);
+    profile.setValue(QStringLiteral("qbs.toolchainType"), toolchainType);
 
     if (!standardCompilerFileNames().contains(
                 HostOsInfo::appendExecutableSuffix(details.suffix))) {
@@ -269,33 +282,28 @@ static QStringList gnuRegistrySearchPaths()
     if (!HostOsInfo::isWindowsHost())
         return {};
 
-    // Registry token for the "GNU Tools for ARM Embedded Processors".
-    static const char kRegistryToken[] = "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\" \
-                                         "Windows\\CurrentVersion\\Uninstall\\";
+#ifdef Q_OS_WIN64
+    static const char kRegistryNode[] = "HKEY_LOCAL_MACHINE\\SOFTWARE\\WOW6432Node\\ARM";
+#else
+    static const char kRegistryNode[] = "HKEY_LOCAL_MACHINE\\SOFTWARE\\ARM";
+#endif
 
     QStringList searchPaths;
-
-    QSettings registry(QLatin1String(kRegistryToken), QSettings::NativeFormat);
-    const auto productGroups = registry.childGroups();
-    for (const QString &productKey : productGroups) {
-        if (!productKey.startsWith(
-                    QLatin1String("GNU Tools for ARM Embedded Processors"))) {
-            continue;
+    QSettings registry(QLatin1String(kRegistryNode), QSettings::NativeFormat);
+    const auto groupKeys = registry.childGroups();
+    for (const QString &groupKey : groupKeys) {
+        registry.beginGroup(groupKey);
+        const QString rootPath = registry.value(QStringLiteral("InstallFolder")).toString();
+        if (!rootPath.isEmpty()) {
+            const QFileInfo toolchainPath(rootPath + QLatin1String("/bin"));
+            if (toolchainPath.exists()) {
+                const auto filePath = toolchainPath.absoluteFilePath();
+                if (!searchPaths.contains(filePath))
+                    searchPaths.push_back(filePath);
+            }
         }
-        registry.beginGroup(productKey);
-        QString uninstallFilePath = registry.value(
-                    QLatin1String("UninstallString")).toString();
-        if (uninstallFilePath.startsWith(QLatin1Char('"')))
-            uninstallFilePath.remove(0, 1);
-        if (uninstallFilePath.endsWith(QLatin1Char('"')))
-            uninstallFilePath.remove(uninstallFilePath.size() - 1, 1);
         registry.endGroup();
-
-        const QString toolkitRootPath = QFileInfo(uninstallFilePath).path();
-        const QString toolchainPath = toolkitRootPath + QLatin1String("/bin");
-        searchPaths.push_back(toolchainPath);
     }
-
     return searchPaths;
 }
 
@@ -378,13 +386,77 @@ static QStringList atmelRegistrySearchPaths()
     return searchPaths;
 }
 
+static QStringList renesasRl78RegistrySearchPaths()
+{
+    if (!HostOsInfo::isWindowsHost())
+        return {};
+
+    QStringList searchPaths;
+
+    QSettings registry(QLatin1String(kUninstallRegistryKey), QSettings::NativeFormat);
+    const auto productGroups = registry.childGroups();
+    for (const QString &productKey : productGroups) {
+        // Registry token for the "Renesas RL78" toolchain.
+        if (!productKey.startsWith(
+                    QLatin1String("GCC for Renesas RL78"))) {
+            continue;
+        }
+        registry.beginGroup(productKey);
+        const QString installLocation = registry.value(
+                    QLatin1String("InstallLocation")).toString();
+        registry.endGroup();
+        if (installLocation.isEmpty())
+            continue;
+
+        const QFileInfo toolchainPath(QDir(installLocation).absolutePath()
+                + QLatin1String("/rl78-elf/rl78-elf/bin"));
+        if (!toolchainPath.exists())
+            continue;
+        searchPaths.push_back(toolchainPath.absoluteFilePath());
+    }
+
+    return searchPaths;
+}
+
+static QStringList mplabX32RegistrySearchPaths()
+{
+    if (!HostOsInfo::isWindowsHost())
+        return {};
+
+    QStringList searchPaths;
+
+    QSettings registry(QLatin1String(kUninstallRegistryKey), QSettings::NativeFormat);
+    const auto productGroups = registry.childGroups();
+    for (const QString &productKey : productGroups) {
+        // Registry token for the "MPLAB X32" toolchain.
+        if (!productKey.startsWith(
+                    QLatin1String("MPLAB XC32 Compiler"))) {
+            continue;
+        }
+        registry.beginGroup(productKey);
+        const QString installLocation = registry.value(
+                    QLatin1String("InstallLocation")).toString();
+        registry.endGroup();
+        if (installLocation.isEmpty())
+            continue;
+
+        const QFileInfo toolchainPath(QDir(installLocation).absolutePath()
+                + QLatin1String("/bin"));
+        if (!toolchainPath.exists())
+            continue;
+        searchPaths.push_back(toolchainPath.absoluteFilePath());
+    }
+
+    return searchPaths;
+}
+
 Profile createGccProfile(const QFileInfo &compiler, Settings *settings,
-                         const QStringList &toolchainTypes,
+                         const QString &toolchainType,
                          const QString &profileName)
 {
     const QString machineName = gccMachineName(compiler);
 
-    if (toolchainTypes.contains(QLatin1String("mingw"))) {
+    if (toolchainType == QLatin1String("mingw")) {
         if (!validMinGWMachines().contains(machineName)) {
             throw ErrorInfo(Tr::tr("Detected gcc platform '%1' is not supported.")
                             .arg(machineName));
@@ -396,9 +468,9 @@ Profile createGccProfile(const QFileInfo &compiler, Settings *settings,
 
     const ToolchainDetails details(compiler);
 
-    setCommonProperties(profile, compiler, toolchainTypes, details);
+    setCommonProperties(profile, compiler, toolchainType, details);
 
-    if (HostOsInfo::isWindowsHost() && toolchainTypes.contains(QLatin1String("clang"))) {
+    if (HostOsInfo::isWindowsHost() && toolchainType == QLatin1String("clang")) {
         const QStringList profileNames = settings->profiles();
         bool foundMingw = false;
         for (const QString &profileName : profileNames) {
@@ -423,7 +495,7 @@ Profile createGccProfile(const QFileInfo &compiler, Settings *settings,
         }
     }
 
-    if (!toolchainTypes.contains(QLatin1String("clang"))) {
+    if (toolchainType != QLatin1String("clang")) {
         // Check whether auxiliary tools reside within the toolchain's install path.
         // This might not be the case when using icecc or another compiler wrapper.
         const QString compilerDirPath = compiler.absolutePath();
@@ -446,17 +518,20 @@ Profile createGccProfile(const QFileInfo &compiler, Settings *settings,
     return profile;
 }
 
-void gccProbe(Settings *settings, QList<Profile> &profiles, const QString &compilerName)
+void gccProbe(Settings *settings, std::vector<Profile> &profiles, const QString &compilerName)
 {
     qbsInfo() << Tr::tr("Trying to detect %1...").arg(compilerName);
 
     QStringList searchPaths;
-    searchPaths << systemSearchPaths() << gnuRegistrySearchPaths()
-                << atmelRegistrySearchPaths();
+    searchPaths << systemSearchPaths()
+                << gnuRegistrySearchPaths()
+                << atmelRegistrySearchPaths()
+                << renesasRl78RegistrySearchPaths()
+                << mplabX32RegistrySearchPaths();
 
     std::vector<QFileInfo> candidates;
     const auto filters = buildCompilerNameFilters(compilerName);
-    for (const auto &searchPath : searchPaths) {
+    for (const auto &searchPath : qAsConst(searchPaths)) {
         const QDir dir(searchPath);
         const QStringList fileNames = dir.entryList(
                     filters, QDir::Files | QDir::Executable);
@@ -466,7 +541,7 @@ void gccProbe(Settings *settings, QList<Profile> &profiles, const QString &compi
                     || fileName.startsWith(QLatin1String("c99-gcc"))) {
                 continue;
             }
-            const QFileInfo candidate = dir.filePath(fileName);
+            const QFileInfo candidate(dir.filePath(fileName));
             // Filter duplicates.
             const auto existingEnd = candidates.end();
             const auto existingIt = std::find_if(
@@ -505,11 +580,15 @@ void gccProbe(Settings *settings, QList<Profile> &profiles, const QString &compi
     }
 
     for (const auto &candidate : qAsConst(candidates)) {
-        const QStringList toolchainTypes = toolchainTypeFromCompilerName(
+        const QString toolchainType = toolchainTypeFromCompilerName(
                     candidate.baseName());
         const QString profileName = buildProfileName(candidate);
-        auto profile = createGccProfile(candidate, settings,
-                                        toolchainTypes, profileName);
-        profiles.push_back(std::move(profile));
+        try {
+            auto profile = createGccProfile(candidate, settings,
+                                            toolchainType, profileName);
+            profiles.push_back(std::move(profile));
+        } catch (const qbs::ErrorInfo &info) {
+            qbsWarning() << Tr::tr("Skipping %1: %2").arg(profileName, info.toString());
+        }
     }
 }
