@@ -58,6 +58,7 @@
 #include <logging/translator.h>
 #include <tools/buildgraphlocker.h>
 #include <tools/fileinfo.h>
+#include <tools/jsliterals.h>
 #include <tools/persistence.h>
 #include <tools/profile.h>
 #include <tools/profiling.h>
@@ -71,6 +72,7 @@
 
 #include <algorithm>
 #include <functional>
+#include <memory>
 #include <unordered_map>
 
 namespace qbs {
@@ -273,7 +275,7 @@ static void updateProductAndRulePointers(const ResolvedProductPtr &newProduct)
             const auto it = ruleMap.find(oldRule);
             if (it != ruleMap.cend())
                 return it->second;
-            for (const RuleConstPtr &r : qAsConst(newProduct->rules)) {
+            for (const auto &r : qAsConst(newProduct->rules)) {
                 if (*r == *oldRule) {
                     ruleMap.insert(std::make_pair(oldRule, r));
                     return r;
@@ -362,7 +364,7 @@ void BuildGraphLoader::trackProjectChanges()
     ChildListHash childLists;
     if (!changedProducts.empty()) {
         oldBuildData = std::make_shared<ProjectBuildData>(restoredProject->buildData.get());
-        for (const ResolvedProductConstPtr &product : qAsConst(allRestoredProducts)) {
+        for (const auto &product : qAsConst(allRestoredProducts)) {
             if (!product->buildData)
                 continue;
 
@@ -451,7 +453,7 @@ void BuildGraphLoader::trackProjectChanges()
         updateGeneratedArtifacts(product.get());
     }
 
-    for (const ResolvedProductConstPtr &changedProduct : qAsConst(changedProducts)) {
+    for (const auto &changedProduct : qAsConst(changedProducts)) {
         rescueOldBuildData(changedProduct,
                            m_freshProductsByName.value(changedProduct->uniqueName()),
                            childLists, rescuableArtifactData.value(changedProduct->uniqueName()));
@@ -614,7 +616,7 @@ bool BuildGraphLoader::hasProductFileChanged(const std::vector<ResolvedProductPt
                         FileInfo::path(group->location.filePath()),
                         product->topLevelProject()->buildDirectory);
                 Set<QString> wcFiles;
-                for (const SourceArtifactConstPtr &sourceArtifact : group->wildcards->files)
+                for (const auto &sourceArtifact : group->wildcards->files)
                     wcFiles += sourceArtifact->absoluteFilePath;
                 if (files == wcFiles)
                     continue;
@@ -641,8 +643,8 @@ bool BuildGraphLoader::hasBuildSystemFileChanged(const Set<QString> &buildSystem
         const auto generatedChecker = [&file, restoredProject](const ModuleProviderInfo &mpi) {
             return file.startsWith(mpi.outputDirPath(restoredProject->buildDirectory));
         };
-        const bool fileWasCreatedByModuleProvider = any_of(restoredProject->moduleProviderInfo,
-                                                           generatedChecker);
+        const bool fileWasCreatedByModuleProvider =
+                any_of(restoredProject->moduleProviderInfo.providers, generatedChecker);
         const FileTime referenceTime = fileWasCreatedByModuleProvider
                 ? restoredProject->lastEndResolveTime : restoredProject->lastStartResolveTime;
         if (referenceTime < fi.lastModified()) {
@@ -741,9 +743,9 @@ static bool dependenciesAreEqual(const ResolvedProductConstPtr &p1,
         return false;
     Set<QString> names1;
     Set<QString> names2;
-    for (const ResolvedProductConstPtr &dep : qAsConst(p1->dependencies))
+    for (const auto &dep : qAsConst(p1->dependencies))
         names1 << dep->uniqueName();
-    for (const ResolvedProductConstPtr &dep : qAsConst(p2->dependencies))
+    for (const auto &dep : qAsConst(p2->dependencies))
         names2 << dep->uniqueName();
     return names1 == names2;
 }
@@ -867,9 +869,23 @@ bool BuildGraphLoader::checkConfigCompatibility()
     if (!m_parameters.overrideBuildGraphData()) {
         if (!m_parameters.overriddenValues().empty()
                 && m_parameters.overriddenValues() != restoredProject->overriddenValues) {
+            const auto toUserOutput = [](const QVariantMap &propMap) {
+                QString o;
+                for (auto it = propMap.begin(); it != propMap.end(); ++it) {
+                    if (!o.isEmpty())
+                        o += QLatin1Char(' ');
+                    o.append(it.key()).append(QLatin1Char(':')).append(toJSLiteral(it.value()));
+                }
+                return o;
+            };
             throw ErrorInfo(Tr::tr("Property values set on the command line differ from the "
-                                   "ones used for the previous build. Use the 'resolve' command if "
-                                   "you really want to rebuild with the new properties."));
+                                   "ones used for the previous build.\n"
+                                   "Old property values: %1\n"
+                                   "New property values: %2\n"
+                                   "Use the 'resolve' command if "
+                                   "you really want to rebuild with the new properties.")
+                            .arg(toUserOutput(restoredProject->overriddenValues))
+                            .arg(toUserOutput(m_parameters.overriddenValues())));
             }
         m_parameters.setOverriddenValues(restoredProject->overriddenValues);
         if (m_parameters.topLevelProfile() != restoredProject->profile()) {
@@ -909,7 +925,7 @@ void BuildGraphLoader::rescueOldBuildData(const ResolvedProductConstPtr &restore
     if (!restoredProduct->buildData)
         return;
     if (!newlyResolvedProduct->buildData)
-        newlyResolvedProduct->buildData.reset(new ProductBuildData);
+        newlyResolvedProduct->buildData = std::make_unique<ProductBuildData>();
 
     qCDebug(lcBuildGraph) << "rescue data of product" << restoredProduct->uniqueName();
     QBS_CHECK(newlyResolvedProduct->buildData);
