@@ -46,10 +46,13 @@
 
 #include <QtCore/qobject.h>
 #include <QtCore/qprocess.h>
-#include <QtCore/qtextcodec.h>
 #include <QtCore/qtextstream.h>
 #include <QtCore/qvariant.h>
-
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+#include <QtCore5Compat/qtextcodec.h>
+#else
+#include <QtCore/qtextcodec.h>
+#endif
 #include <QtScript/qscriptable.h>
 #include <QtScript/qscriptengine.h>
 #include <QtScript/qscriptvalue.h>
@@ -63,7 +66,6 @@ class Process : public QObject, public QScriptable, public ResourceAcquiringScri
 public:
     static QScriptValue ctor(QScriptContext *context, QScriptEngine *engine);
     Process(QScriptContext *context);
-    ~Process();
 
     Q_INVOKABLE QString getEnv(const QString &name);
     Q_INVOKABLE void setEnv(const QString &name, const QString &value);
@@ -81,6 +83,7 @@ public:
     Q_INVOKABLE void kill();
 
     Q_INVOKABLE QString readLine();
+    Q_INVOKABLE bool atEnd() const;
     Q_INVOKABLE QString readStdOut();
     Q_INVOKABLE QString readStdErr();
 
@@ -99,10 +102,10 @@ private:
     // ResourceAcquiringScriptObject implementation
     void releaseResources() override;
 
-    QProcess *m_qProcess;
+    std::unique_ptr<QProcess> m_qProcess;
     QProcessEnvironment m_environment;
     QString m_workingDirectory;
-    QTextStream *m_textStream;
+    QTextCodec *m_codec = nullptr;
 };
 
 QScriptValue Process::ctor(QScriptContext *context, QScriptEngine *engine)
@@ -113,7 +116,7 @@ QScriptValue Process::ctor(QScriptContext *context, QScriptEngine *engine)
         t = new Process(context);
         break;
     default:
-        return context->throwError(QLatin1String("Process()"));
+        return context->throwError(QStringLiteral("Process()"));
     }
 
     const auto se = static_cast<ScriptEngine *>(engine);
@@ -121,7 +124,7 @@ QScriptValue Process::ctor(QScriptContext *context, QScriptEngine *engine)
     const DubiousContextList dubiousContexts ({
             DubiousContext(EvalContext::PropertyEvaluation, DubiousContext::SuggestMoving)
     });
-    se->checkContext(QLatin1String("qbs.Process"), dubiousContexts);
+    se->checkContext(QStringLiteral("qbs.Process"), dubiousContexts);
 
     QScriptValue obj = engine->newQObject(t, QScriptEngine::QtOwnership);
 
@@ -135,23 +138,18 @@ QScriptValue Process::ctor(QScriptContext *context, QScriptEngine *engine)
         t->m_environment
             = QProcessEnvironment(*reinterpret_cast<QProcessEnvironment*>(v.value<void*>()));
     }
+    se->setUsesIo();
 
     return obj;
 }
 
-Process::~Process()
-{
-    delete m_textStream;
-    delete m_qProcess;
-}
-
 Process::Process(QScriptContext *context)
 {
     Q_UNUSED(context);
     Q_ASSERT(thisObject().engine() == engine());
 
-    m_qProcess = new QProcess;
-    m_textStream = new QTextStream(m_qProcess);
+    m_qProcess = std::make_unique<QProcess>();
+    m_codec = QTextCodec::codecForName("UTF-8");
 }
 
 QString Process::getEnv(const QString &name)
@@ -186,7 +184,7 @@ bool Process::start(const QString &program, const QStringList &arguments)
         m_qProcess->setWorkingDirectory(m_workingDirectory);
 
     m_qProcess->setProcessEnvironment(m_environment);
-    m_qProcess->start(findExecutable(program), arguments);
+    m_qProcess->start(findExecutable(program), arguments, QIODevice::ReadWrite | QIODevice::Text);
     return m_qProcess->waitForStarted();
 }
 
@@ -211,8 +209,8 @@ int Process::exec(const QString &program, const QStringList &arguments, bool thr
         } else if (m_qProcess->exitStatus() == QProcess::CrashExit || m_qProcess->exitCode() != 0) {
             QString errorMessage = m_qProcess->error() == QProcess::Crashed
                     ? Tr::tr("Error running '%1': %2").arg(program, m_qProcess->errorString())
-                    : Tr::tr("Process '%1' finished with exit code %2.").arg(program).arg(
-                          m_qProcess->exitCode());
+                    : Tr::tr("Process '%1 %2' finished with exit code %3.").arg(program).
+                      arg(arguments.join(QStringLiteral(" "))).arg(m_qProcess->exitCode());
             const QString stdOut = readStdOut();
             if (!stdOut.isEmpty())
                 errorMessage.append(Tr::tr(" The standard output was:\n")).append(stdOut);
@@ -232,10 +230,7 @@ void Process::close()
     if (!m_qProcess)
         return;
     Q_ASSERT(thisObject().engine() == engine());
-    delete m_textStream;
-    m_textStream = nullptr;
-    delete m_qProcess;
-    m_qProcess = nullptr;
+    m_qProcess.reset();
 }
 
 bool Process::waitForFinished(int msecs)
@@ -260,27 +255,36 @@ void Process::kill()
 void Process::setCodec(const QString &codec)
 {
     Q_ASSERT(thisObject().engine() == engine());
-    m_textStream->setCodec(qPrintable(codec));
+    const auto newCodec = QTextCodec::codecForName(qPrintable(codec));
+    if (newCodec)
+        m_codec = newCodec;
 }
 
 QString Process::readLine()
 {
-    return m_textStream->readLine();
+    auto result = m_codec->toUnicode(m_qProcess->readLine());
+    if (!result.isEmpty() && result.back() == QLatin1Char('\n'))
+        result.chop(1);
+    return result;
+}
+
+bool Process::atEnd() const
+{
+    return m_qProcess->atEnd();
 }
 
 QString Process::readStdOut()
 {
-    return m_textStream->readAll();
+    return m_codec->toUnicode(m_qProcess->readAllStandardOutput());
 }
 
 QString Process::readStdErr()
 {
-    return m_textStream->codec()->toUnicode(m_qProcess->readAllStandardError());
+    return m_codec->toUnicode(m_qProcess->readAllStandardError());
 }
 
 void Process::closeWriteChannel()
 {
-    m_textStream->flush();
     m_qProcess->closeWriteChannel();
 }
 
@@ -303,22 +307,20 @@ void Process::releaseResources()
 
 void Process::write(const QString &str)
 {
-    (*m_textStream) << str;
+    m_qProcess->write(m_codec->fromUnicode(str));
 }
 
 void Process::writeLine(const QString &str)
 {
-    (*m_textStream) << str;
-    if (HostOsInfo::isWindowsHost())
-        (*m_textStream) << '\r';
-    (*m_textStream) << '\n';
+    m_qProcess->write(m_codec->fromUnicode(str));
+    m_qProcess->putChar('\n');
 }
 
 QScriptValue Process::js_shellQuote(QScriptContext *context, QScriptEngine *engine)
 {
     if (Q_UNLIKELY(context->argumentCount() < 2)) {
         return context->throwError(QScriptContext::SyntaxError,
-                                   QLatin1String("shellQuote expects at least 2 arguments"));
+                                   QStringLiteral("shellQuote expects at least 2 arguments"));
     }
     const QString program = context->argument(0).toString();
     const QStringList args = context->argument(1).toVariant().toStringList();
@@ -338,7 +340,7 @@ void initializeJsExtensionProcess(QScriptValue extensionObject)
     using namespace qbs::Internal;
     QScriptEngine *engine = extensionObject.engine();
     QScriptValue obj = engine->newQMetaObject(&Process::staticMetaObject, engine->newFunction(&Process::ctor));
-    extensionObject.setProperty(QLatin1String("Process"), obj);
+    extensionObject.setProperty(QStringLiteral("Process"), obj);
     obj.setProperty(QStringLiteral("shellQuote"), engine->newFunction(Process::js_shellQuote, 3));
 }
 
