@@ -125,7 +125,7 @@ ItemDeclaration BuiltinDeclarations::declarationsForType(ItemType type) const
 }
 
 ItemType BuiltinDeclarations::typeForName(const QString &typeName,
-                                          const CodeLocation location) const
+                                          const CodeLocation &location) const
 {
     const auto it = m_typeMap.constFind(typeName);
     if (it == m_typeMap.constEnd())
@@ -433,6 +433,8 @@ void BuiltinDeclarations::addProductItem()
     item << PropertyDeclaration(StringConstants::multiplexConfigurationIdProperty(),
                                 PropertyDeclaration::String, QString(),
                                 PropertyDeclaration::ReadOnlyFlag);
+    item << PropertyDeclaration(StringConstants::qbsModuleProviders(),
+                                PropertyDeclaration::StringList);
     insert(item);
 }
 
@@ -473,6 +475,8 @@ void BuiltinDeclarations::addProjectItem()
     item << PropertyDeclaration(StringConstants::qbsSearchPathsProperty(),
                                 PropertyDeclaration::StringList,
                                 QString(), PropertyDeclaration::PropertyNotAvailableInConfig);
+    item << PropertyDeclaration(StringConstants::qbsModuleProviders(),
+                                PropertyDeclaration::StringList);
     insert(item);
 }
 
