@@ -39,6 +39,7 @@
 
 #include "loader.h"
 
+#include "evaluator.h"
 #include "language.h"
 #include "moduleloader.h"
 #include "projectresolver.h"
@@ -46,20 +47,23 @@
 
 #include <logging/translator.h>
 #include <tools/fileinfo.h>
+#include <tools/profile.h>
 #include <tools/progressobserver.h>
 #include <tools/qbsassert.h>
+#include <tools/settings.h>
 #include <tools/setupprojectparameters.h>
+#include <tools/stringconstants.h>
 
-#include <QDir>
-#include <QObject>
-#include <QTimer>
+#include <QtCore/qdir.h>
+#include <QtCore/qobject.h>
+#include <QtCore/qtimer.h>
 
 namespace qbs {
 namespace Internal {
 
-Loader::Loader(ScriptEngine *engine, const Logger &logger)
-    : m_logger(logger)
-    , m_progressObserver(0)
+Loader::Loader(ScriptEngine *engine, Logger logger)
+    : m_logger(std::move(logger))
+    , m_progressObserver(nullptr)
     , m_engine(engine)
 {
     m_logger.storeWarnings();
@@ -73,7 +77,7 @@ void Loader::setProgressObserver(ProgressObserver *observer)
 void Loader::setSearchPaths(const QStringList &_searchPaths)
 {
     QStringList searchPaths;
-    foreach (const QString &searchPath, _searchPaths) {
+    for (const QString &searchPath : _searchPaths) {
         if (!FileInfo::exists(searchPath)) {
             m_logger.qbsWarning() << Tr::tr("Search path '%1' does not exist.")
                     .arg(QDir::toNativeSeparators(searchPath));
@@ -85,14 +89,46 @@ void Loader::setSearchPaths(const QStringList &_searchPaths)
     m_searchPaths = searchPaths;
 }
 
-void Loader::setOldProbes(const QHash<QString, QList<ProbeConstPtr> > &oldProbes)
+void Loader::setOldProjectProbes(const std::vector<ProbeConstPtr> &oldProbes)
 {
-    m_oldProbes = oldProbes;
+    m_oldProjectProbes = oldProbes;
 }
 
-TopLevelProjectPtr Loader::loadProject(const SetupProjectParameters &parameters)
+void Loader::setOldProductProbes(const QHash<QString, std::vector<ProbeConstPtr>> &oldProbes)
 {
+    m_oldProductProbes = oldProbes;
+}
+
+void Loader::setStoredProfiles(const QVariantMap &profiles)
+{
+    m_storedProfiles = profiles;
+}
+
+void Loader::setStoredModuleProviderInfo(const StoredModuleProviderInfo &providerInfo)
+{
+    m_storedModuleProviderInfo = providerInfo;
+}
+
+TopLevelProjectPtr Loader::loadProject(const SetupProjectParameters &_parameters)
+{
+    SetupProjectParameters parameters = _parameters;
+
+    if (parameters.topLevelProfile().isEmpty()) {
+        Settings settings(parameters.settingsDirectory());
+        QString profileName = settings.defaultProfile();
+        if (profileName.isEmpty()) {
+            m_logger.qbsDebug() << Tr::tr("No profile specified and no default profile exists. "
+                                          "Using default property values.");
+            profileName = Profile::fallbackName();
+        }
+        parameters.setTopLevelProfile(profileName);
+        parameters.expandBuildConfiguration();
+    }
+
+    setupProjectFilePath(parameters);
     QBS_CHECK(QFileInfo(parameters.projectFilePath()).isAbsolute());
+    m_logger.qbsDebug() << "Using project file '"
+                        << QDir::toNativeSeparators(parameters.projectFilePath()) << "'.";
 
     m_engine->setEnvironment(parameters.adjustedEnvironment());
     m_engine->clearExceptions();
@@ -121,15 +157,21 @@ TopLevelProjectPtr Loader::loadProject(const SetupProjectParameters &parameters)
     }
 
     const FileTime resolveTime = FileTime::currentTime();
-    ModuleLoader moduleLoader(m_engine, m_logger);
+    Evaluator evaluator(m_engine);
+    ModuleLoader moduleLoader(&evaluator, m_logger);
     moduleLoader.setProgressObserver(m_progressObserver);
     moduleLoader.setSearchPaths(m_searchPaths);
-    moduleLoader.setOldProbes(m_oldProbes);
+    moduleLoader.setOldProjectProbes(m_oldProjectProbes);
+    moduleLoader.setOldProductProbes(m_oldProductProbes);
+    moduleLoader.setLastResolveTime(m_lastResolveTime);
+    moduleLoader.setStoredProfiles(m_storedProfiles);
+    moduleLoader.setStoredModuleProviderInfo(m_storedModuleProviderInfo);
     const ModuleLoaderResult loadResult = moduleLoader.load(parameters);
-    ProjectResolver resolver(moduleLoader.evaluator(), loadResult, parameters, m_logger);
+    ProjectResolver resolver(&evaluator, loadResult, std::move(parameters), m_logger);
     resolver.setProgressObserver(m_progressObserver);
     const TopLevelProjectPtr project = resolver.resolve();
-    project->lastResolveTime = resolveTime;
+    project->lastStartResolveTime = resolveTime;
+    project->lastEndResolveTime = FileTime::currentTime();
 
     // E.g. if the top-level project is disabled.
     if (m_progressObserver)
@@ -138,5 +180,43 @@ TopLevelProjectPtr Loader::loadProject(const SetupProjectParameters &parameters)
     return project;
 }
 
+void Loader::setupProjectFilePath(SetupProjectParameters &parameters)
+{
+    QString projectFilePath = parameters.projectFilePath();
+    if (projectFilePath.isEmpty())
+        projectFilePath = QDir::currentPath();
+    const QFileInfo projectFileInfo(projectFilePath);
+    if (!projectFileInfo.exists())
+        throw ErrorInfo(Tr::tr("Project file '%1' cannot be found.").arg(projectFilePath));
+    if (projectFileInfo.isRelative())
+        projectFilePath = projectFileInfo.absoluteFilePath();
+    if (projectFileInfo.isFile()) {
+        parameters.setProjectFilePath(projectFilePath);
+        return;
+    }
+    if (!projectFileInfo.isDir())
+        throw ErrorInfo(Tr::tr("Project file '%1' has invalid type.").arg(projectFilePath));
+
+    const QStringList &actualFileNames
+            = QDir(projectFilePath).entryList(StringConstants::qbsFileWildcards(), QDir::Files);
+    if (actualFileNames.empty()) {
+        QString error;
+        if (parameters.projectFilePath().isEmpty())
+            error = Tr::tr("No project file given and none found in current directory.\n");
+        else
+            error = Tr::tr("No project file found in directory '%1'.").arg(projectFilePath);
+        throw ErrorInfo(error);
+    }
+    if (actualFileNames.size() > 1) {
+        throw ErrorInfo(Tr::tr("More than one project file found in directory '%1'.")
+                .arg(projectFilePath));
+    }
+    projectFilePath.append(QLatin1Char('/')).append(actualFileNames.front());
+
+    projectFilePath = QDir::current().filePath(projectFilePath);
+    projectFilePath = QDir::cleanPath(projectFilePath);
+    parameters.setProjectFilePath(projectFilePath);
+}
+
 } // namespace Internal
 } // namespace qbs
