@@ -73,8 +73,10 @@ namespace Internal {
 class Evaluator;
 class Item;
 class ItemReader;
+class ModuleProviderLoader;
 class ProgressObserver;
 class QualifiedId;
+class SearchPathsManager;
 
 using ModulePropertiesPerGroup = std::unordered_map<const Item *, QualifiedIdSet>;
 
@@ -106,9 +108,9 @@ struct ModuleLoaderResult
 
     std::shared_ptr<ItemPool> itemPool;
     Item *root;
-    QHash<Item *, ProductInfo> productInfos;
+    std::unordered_map<Item *, ProductInfo> productInfos;
     std::vector<ProbeConstPtr> projectProbes;
-    ModuleProviderInfoList moduleProviderInfo;
+    StoredModuleProviderInfo storedModuleProviderInfo;
     Set<QString> qbsFiles;
     QVariantMap profileConfigs;
 };
@@ -131,12 +133,13 @@ public:
     void setOldProductProbes(const QHash<QString, std::vector<ProbeConstPtr>> &oldProbes);
     void setLastResolveTime(const FileTime &time) { m_lastResolveTime = time; }
     void setStoredProfiles(const QVariantMap &profiles);
-    void setStoredModuleProviderInfo(const ModuleProviderInfoList &moduleProviderInfo);
+    void setStoredModuleProviderInfo(const StoredModuleProviderInfo &moduleProviderInfo);
     Evaluator *evaluator() const { return m_evaluator; }
 
     ModuleLoaderResult load(const SetupProjectParameters &parameters);
 
 private:
+    friend class ModuleProviderLoader;
     class ProductSortByDependencies;
 
     class ContextBase
@@ -185,10 +188,7 @@ private:
         std::unordered_map<const Item *, std::vector<ErrorInfo>> unknownProfilePropertyErrors;
         QStringList searchPaths;
 
-        std::vector<QStringList> newlyAddedModuleProviderSearchPaths;
-        Set<QualifiedId> knownModuleProviders;
-        QVariantMap theModuleProviderConfig;
-        bool moduleProviderConfigRetrieved = false;
+        std::optional<QVariantMap> theModuleProviderConfig;
 
         // The key corresponds to DeferredDependsContext.exportingProductItem, which is the
         // only value from that data structure that we still need here.
@@ -345,19 +345,6 @@ private:
     QStringList findExistingModulePaths(
             const QStringList &searchPaths, const QualifiedId &moduleName);
 
-    enum class ModuleProviderLookup { Regular, Fallback };
-    struct ModuleProviderResult
-    {
-        ModuleProviderResult() = default;
-        ModuleProviderResult(bool ran, bool added)
-            : providerFound(ran), providerAddedSearchPaths(added) {}
-        bool providerFound = false;
-        bool providerAddedSearchPaths = false;
-    };
-    ModuleProviderResult findModuleProvider(const QualifiedId &name, ProductContext &product,
-            ModuleProviderLookup lookupType, const CodeLocation &dependsItemLocation);
-    QVariantMap moduleProviderConfig(ProductContext &product);
-
     static void setScopeForDescendants(Item *item, Item *scope);
     void overrideItemProperties(Item *item, const QString &buildConfigKey,
                                 const QVariantMap &buildConfig);
@@ -415,6 +402,7 @@ private:
     ProgressObserver *m_progressObserver;
     const std::unique_ptr<ItemReader> m_reader;
     Evaluator *m_evaluator;
+    const std::unique_ptr<ModuleProviderLoader> m_moduleProviderLoader;
     QMap<QString, QStringList> m_moduleDirListCache;
     QHash<std::pair<QString, QualifiedId>, std::optional<QString>> m_existingModulePathCache;
 
@@ -455,9 +443,6 @@ private:
     std::unordered_map<ProductContext *, Set<DeferredDependsContext>> m_productsWithDeferredDependsItems;
     Set<Item *> m_exportsWithDeferredDependsItems;
 
-    ModuleProviderInfoList m_moduleProviderInfo;
-    Set<QString> m_tempQbsFiles;
-
     SetupProjectParameters m_parameters;
     std::unique_ptr<Settings> m_settings;
     Version m_qbsVersion;
@@ -466,6 +451,7 @@ private:
     qint64 m_elapsedTimeProbes = 0;
     qint64 m_elapsedTimePrepareProducts = 0;
     qint64 m_elapsedTimeProductDependencies = 0;
+    qint64 m_elapsedTimeModuleProviders = 0;
     qint64 m_elapsedTimeTransitiveDependencies = 0;
     qint64 m_elapsedTimeHandleProducts = 0;
     qint64 m_elapsedTimePropertyChecking = 0;
