@@ -232,14 +232,14 @@ static void makeSubProjectNamesUniqe(const ResolvedProjectPtr &parentProject)
 TopLevelProjectPtr ProjectResolver::resolveTopLevelProject()
 {
     if (m_progressObserver)
-        m_progressObserver->setMaximum(m_loadResult.productInfos.size());
+        m_progressObserver->setMaximum(int(m_loadResult.productInfos.size()));
     const TopLevelProjectPtr project = TopLevelProject::create();
     project->buildDirectory = TopLevelProject::deriveBuildDirectory(m_setupParams.buildRoot(),
             TopLevelProject::deriveId(m_setupParams.finalBuildConfigurationTree()));
     project->buildSystemFiles = m_loadResult.qbsFiles;
     project->profileConfigs = m_loadResult.profileConfigs;
     project->probes = m_loadResult.projectProbes;
-    project->moduleProviderInfo = m_loadResult.moduleProviderInfo;
+    project->moduleProviderInfo = m_loadResult.storedModuleProviderInfo;
     ProjectContext projectContext;
     projectContext.project = project;
 
@@ -635,13 +635,14 @@ SourceArtifactPtr ProjectResolver::createSourceArtifact(const ResolvedProductPtr
     return artifact;
 }
 
-static QualifiedIdSet propertiesToEvaluate(const QList<QualifiedId> &initialProps,
-                                              const PropertyDependencies &deps)
+static QualifiedIdSet propertiesToEvaluate(std::deque<QualifiedId> initialProps,
+                                           const PropertyDependencies &deps)
 {
-    QList<QualifiedId> remainingProps = initialProps;
+    std::deque<QualifiedId> remainingProps = std::move(initialProps);
     QualifiedIdSet allProperties;
     while (!remainingProps.empty()) {
-        const QualifiedId prop = remainingProps.takeFirst();
+        const QualifiedId prop = remainingProps.front();
+        remainingProps.pop_front();
         const auto insertResult = allProperties.insert(prop);
         if (!insertResult.second)
             continue;
@@ -655,16 +656,17 @@ QVariantMap ProjectResolver::resolveAdditionalModuleProperties(const Item *group
                                                                const QVariantMap &currentValues)
 {
     // Step 1: Retrieve the properties directly set in the group
-    const ModulePropertiesPerGroup &mp = m_loadResult.productInfos.value(m_productContext->item)
-            .modulePropertiesSetInGroups;
+    const ModulePropertiesPerGroup &mp = mapValue(
+            m_loadResult.productInfos, m_productContext->item).modulePropertiesSetInGroups;
     const auto it = mp.find(group);
     if (it == mp.end())
         return {};
     const QualifiedIdSet &propsSetInGroup = it->second;
 
     // Step 2: Gather all properties that depend on these properties.
-    const QualifiedIdSet &propsToEval
-            = propertiesToEvaluate(propsSetInGroup.toList(), m_evaluator->propertyDependencies());
+    const QualifiedIdSet &propsToEval = propertiesToEvaluate(
+            rangeTo<std::deque<QualifiedId>>(propsSetInGroup),
+            m_evaluator->propertyDependencies());
 
     // Step 3: Evaluate all these properties and replace their values in the map
     QVariantMap modulesMap = currentValues;
@@ -900,7 +902,7 @@ void ProjectResolver::adaptExportedPropertyValues(const Item *shadowProductItem)
     const std::function<QVariant(const QVariantMap &, const QVariant &)> mapper
             = [&stringListMapper, &mapper](
             const QVariantMap &mappings, const QVariant &value) -> QVariant {
-        switch (static_cast<QMetaType::Type>(value.type())) {
+        switch (static_cast<QMetaType::Type>(value.userType())) {
         case QMetaType::QString:
             return stringMapper(mappings, value.toString());
         case QMetaType::QStringList:
@@ -945,7 +947,7 @@ void ProjectResolver::collectExportedProductDependencies()
             }
         }
         const ModuleLoaderResult::ProductInfo &importingProductInfo
-                = m_loadResult.productInfos.value(importingProductItem);
+                = mapValue(m_loadResult.productInfos, importingProductItem);
         const ProductDependencyInfos &depInfos
                 = getProductDependencies(dummyProduct, importingProductInfo);
         for (const auto &dep : depInfos.dependencies) {
@@ -1610,7 +1612,7 @@ void ProjectResolver::resolveProductDependencies(const ProjectContext &projectCo
             continue;
         Item *productItem = m_productItemMap.value(rproduct);
         const ModuleLoaderResult::ProductInfo &productInfo
-                = m_loadResult.productInfos.value(productItem);
+                = mapValue(m_loadResult.productInfos, productItem);
         const ProductDependencyInfos &depInfos = getProductDependencies(rproduct, productInfo);
         if (depInfos.hasDisabledDependency)
             disabledDependency = true;
