@@ -85,13 +85,15 @@ bool operator==(const ScriptEngine::PropertyCacheKey &lhs,
             && lhs.m_propertyName == rhs.m_propertyName;
 }
 
-static inline uint combineHash(uint h1, uint h2, uint seed)
+
+
+static inline QHashValueType combineHash(QHashValueType h1, QHashValueType h2, QHashValueType seed)
 {
     // stolen from qHash(QPair)
     return ((h1 << 16) | (h1 >> 16)) ^ h2 ^ seed;
 }
 
-uint qHash(const ScriptEngine::PropertyCacheKey &k, uint seed = 0)
+QHashValueType qHash(const ScriptEngine::PropertyCacheKey &k, QHashValueType seed = 0)
 {
     return combineHash(qHash(k.m_moduleName),
                        combineHash(qHash(k.m_propertyName), qHash(k.m_propertyMap), seed), seed);
@@ -381,7 +383,7 @@ void ScriptEngine::importFile(const QString &filePath, QScriptValue &targetObjec
     if (Q_UNLIKELY(!file.open(QFile::ReadOnly)))
         throw ErrorInfo(tr("Cannot open '%1'.").arg(filePath));
     QTextStream stream(&file);
-    stream.setCodec("UTF-8");
+    setupDefaultCodec(stream);
     const QString sourceCode = stream.readAll();
     file.close();
     m_currentDirPathStack.push(FileInfo::path(filePath));
@@ -478,19 +480,18 @@ QScriptValue ScriptEngine::js_require(QScriptContext *context, QScriptEngine *qt
             return context->throwError(
                         ScriptEngine::tr("require: internal error. No search paths."));
 
-        const QString uri = moduleName;
         if (engine->m_logger.debugEnabled()) {
             engine->m_logger.qbsDebug()
-                    << "[require] loading extension " << uri;
+                    << "[require] loading extension " << moduleName;
         }
 
-        QString uriAsPath = uri;
-        uriAsPath.replace(QLatin1Char('.'), QLatin1Char('/'));
+        QString moduleNameAsPath = moduleName;
+        moduleNameAsPath.replace(QLatin1Char('.'), QLatin1Char('/'));
         const QStringList searchPaths = engine->m_extensionSearchPathsStack.top();
-        const QString dirPath = findExtensionDir(searchPaths, uriAsPath);
+        const QString dirPath = findExtensionDir(searchPaths, moduleNameAsPath);
         if (dirPath.isEmpty()) {
-            if (uri.startsWith(QStringLiteral("qbs.")))
-                return loadInternalExtension(context, engine, uri);
+            if (moduleName.startsWith(QStringLiteral("qbs.")))
+                return loadInternalExtension(context, engine, moduleName);
         } else {
             QDirIterator dit(dirPath, StringConstants::jsFileWildcards(),
                              QDir::Files | QDir::Readable);
