@@ -47,10 +47,10 @@
 #include <QtCore/qdatetime.h>
 #include <QtCore/qdir.h>
 #include <QtCore/qfileinfo.h>
-#include <QtCore/qregexp.h>
+#include <QtCore/qregularexpression.h>
 
 #if defined(Q_OS_UNIX)
-#include <errno.h>
+#include <cerrno>
 #include <sys/stat.h>
 #include <unistd.h>
 #elif defined(Q_OS_WIN)
@@ -133,16 +133,16 @@ void FileInfo::splitIntoDirectoryAndFileName(const QString &filePath, QString *d
     *fileName = filePath.mid(idx + 1);
 }
 
-void FileInfo::splitIntoDirectoryAndFileName(const QString &filePath, QStringRef *dirPath, QStringRef *fileName)
+void FileInfo::splitIntoDirectoryAndFileName(const QString &filePath, QStringView *dirPath, QStringView *fileName)
 {
     int idx = filePath.lastIndexOf(QLatin1Char('/'));
     if (idx < 0) {
-        dirPath->clear();
-        *fileName = QStringRef(&filePath);
+        *dirPath = QStringView();
+        *fileName = QStringView(filePath);
         return;
     }
-    *dirPath = filePath.leftRef(idx);
-    *fileName = filePath.midRef(idx + 1);
+    *dirPath = QStringView(filePath).left(idx);
+    *fileName = QStringView(filePath).mid(idx + 1);
 }
 
 bool FileInfo::exists(const QString &fp)
@@ -180,12 +180,7 @@ bool FileInfo::isAbsolute(const QString &path, HostOsInfo::HostOs hostOs)
     return false;
 }
 
-bool FileInfo::isPattern(const QString &str)
-{
-    return isPattern(QStringRef(&str));
-}
-
-bool FileInfo::isPattern(const QStringRef &str)
+bool FileInfo::isPattern(QStringView str)
 {
     for (const QChar &ch : str) {
         if (ch == QLatin1Char('*') || ch == QLatin1Char('?')
@@ -242,17 +237,6 @@ QString FileInfo::resolvePath(const QString &base, const QString &rel, HostOsInf
     return r;
 }
 
-bool FileInfo::globMatches(const QRegExp &regexp, const QString &fileName)
-{
-    const QString pattern = regexp.pattern();
-    // May be it's simple wildcard, i.e. "*.cpp"?
-    if (pattern.startsWith(QLatin1Char('*')) && !isPattern(pattern.midRef(1))) {
-        // Yes, it's rather simple to just check the extension
-        return fileName.endsWith(pattern.midRef(1));
-    }
-    return regexp.exactMatch(fileName);
-}
-
 #ifdef Q_OS_WIN
 static QString prependLongPathPrefix(const QString &absolutePath)
 {
@@ -353,9 +337,70 @@ QString applicationDirPath()
 
 #elif defined(Q_OS_UNIX)
 
+namespace GetFileTimes {
+
+enum TimeType { LastModified, LastStatusChanged };
+
+// SFINAE magic inspired by
+// https://github.com/qt/qtbase/blob/5.13/src/corelib/io/qfilesystemengine_unix.cpp
+template<
+        TimeType type,
+        typename T,
+        std::enable_if_t<(&T::st_mtim, &T::st_ctim, true), int> = 0>
+static inline FileTime get(const T &stat)
+{
+    if constexpr (type == LastModified)
+        return stat.st_mtim;
+    else if constexpr (type == LastStatusChanged)
+        return stat.st_ctim;
+}
+
+// we are interested in real members, not compatibility macros
+#  if defined(st_mtimespec)
+#    undef st_mtimespec
+#  endif
+
+#  if defined(st_ctimespec)
+#    undef st_ctimespec
+#  endif
+
+template<
+        TimeType type,
+        typename T,
+        std::enable_if_t<(&T::st_mtimespec, &T::st_ctimespec, true), int> = 0>
+static inline FileTime get(const T &stat)
+{
+    if constexpr (type == LastModified)
+        return stat.st_mtimespec;
+    else if constexpr (type == LastStatusChanged)
+        return stat.st_ctimespec;
+}
+
+#  if defined(st_mtimensec)
+#    undef st_mtimensec
+#  endif
+
+#  if defined(st_ctimensec)
+#    undef st_ctimensec
+#  endif
+
+template<
+        TimeType type,
+        typename T,
+        std::enable_if_t<(&T::st_mtimensec, &T::st_ctimensec, true), int> = 0>
+static inline FileTime get(const T &stat)
+{
+    if constexpr (type == LastModified)
+        return FileTime::InternalType{stat.st_mtime, stat.st_mtimensec};
+    else if constexpr (type == LastStatusChanged)
+        return FileTime::InternalType{stat.st_ctime, stat.st_ctimensec};
+}
+
+} // namespace GetFileTimes
+
 FileInfo::FileInfo(const QString &fileName)
 {
-    if (stat(fileName.toLocal8Bit(), &m_stat) == -1) {
+    if (stat(fileName.toLocal8Bit().constData(), &m_stat) == -1) {
         m_stat.st_mtime = 0;
         m_stat.st_mode = 0;
     }
@@ -368,24 +413,12 @@ bool FileInfo::exists() const
 
 FileTime FileInfo::lastModified() const
 {
-#if APPLE_STAT_TIMESPEC
-    return m_stat.st_mtimespec;
-#elif HAS_CLOCK_GETTIME
-    return m_stat.st_mtim;
-#else
-    return m_stat.st_mtime;
-#endif
+    return GetFileTimes::get<GetFileTimes::LastModified>(m_stat);
 }
 
 FileTime FileInfo::lastStatusChange() const
 {
-#if APPLE_STAT_TIMESPEC
-    return m_stat.st_ctimespec;
-#elif HAS_CLOCK_GETTIME
-    return m_stat.st_ctim;
-#else
-    return m_stat.st_ctime;
-#endif
+    return GetFileTimes::get<GetFileTimes::LastStatusChanged>(m_stat);
 }
 
 bool FileInfo::isDir() const
@@ -450,7 +483,7 @@ static QByteArray storedLinkTarget(const QString &filePath)
     const QByteArray nativeFilePath = QFile::encodeName(filePath);
     ssize_t len;
     while (true) {
-        struct stat sb;
+        struct stat sb{};
         if (lstat(nativeFilePath.constData(), &sb)) {
             qWarning("storedLinkTarget: lstat for %s failed with error code %d",
                      nativeFilePath.constData(), errno);
