@@ -40,6 +40,8 @@ public:
     TestBlackbox();
 
 private slots:
+    void allowedValues();
+    void allowedValues_data();
     void addFileTagToGeneratedArtifact();
     void alwaysRun();
     void alwaysRun_data();
@@ -60,6 +62,10 @@ private slots:
     void buildDirectories();
     void buildEnvChange();
     void buildGraphVersions();
+    void buildVariantDefaults_data();
+    void buildVariantDefaults();
+    void capnproto();
+    void capnproto_data();
     void changedFiles_data();
     void changedFiles();
     void changedInputsFromDependencies();
@@ -83,9 +89,11 @@ private slots:
     void conflictingArtifacts();
     void cxxLanguageVersion();
     void cxxLanguageVersion_data();
+    void conanfileProbe_data();
+    void conanfileProbe();
     void cpuFeatures();
     void dependenciesProperty();
-    void dependencyProfileMismatch();
+    void dependencyScanningLoop();
     void deprecatedProperty();
     void disappearedProfile();
     void discardUnusedData();
@@ -96,6 +104,7 @@ private slots:
     void dynamicMultiplexRule();
     void dynamicProject();
     void dynamicRuleOutputs();
+    void emptyProfile();
     void enableExceptions();
     void enableExceptions_data();
     void enableRtti();
@@ -118,6 +127,7 @@ private slots:
     void externalLibs();
     void fileDependencies();
     void fileTagsFilterMerging();
+    void freedesktop();
     void generatedArtifactAsInputToDynamicRule();
     void generateLinkerMapFile();
     void generator();
@@ -125,6 +135,7 @@ private slots:
     void groupsInModules();
     void grpc_data();
     void grpc();
+    void hostOsProperties();
     void ico();
     void importAssignment();
     void importChangeTracking();
@@ -133,8 +144,6 @@ private slots:
     void importingProduct();
     void importsConflict();
     void includeLookup();
-    void innoSetup();
-    void innoSetupDependencies();
     void inputTagsChangeTracking_data();
     void inputTagsChangeTracking();
     void inputsFromDependencies();
@@ -164,6 +173,7 @@ private slots:
     void jsExtensionsTemporaryDir();
     void jsExtensionsTextFile();
     void jsExtensionsBinaryFile();
+    void lastModuleCandidateBroken();
     void ld();
     void linkerMode();
     void linkerVariant_data();
@@ -171,7 +181,10 @@ private slots:
     void lexyacc();
     void lexyaccOutputs();
     void lexyaccOutputs_data();
+    void linkerLibraryDuplicates();
+    void linkerLibraryDuplicates_data();
     void linkerScripts();
+    void linkerModuleDefinition();
     void listProducts();
     void listPropertiesWithOuter();
     void listPropertyOrder();
@@ -230,17 +243,30 @@ private slots:
     void probesAndArrayProperties();
     void probesInNestedModules();
     void productDependenciesByType();
+    void productInExportedModule();
     void productProperties();
     void propertyAssignmentOnNonPresentModule();
     void propertyAssignmentInFailedModule();
     void propertyChanges();
+    void propertyEvaluationContext();
     void propertyPrecedence();
     void properQuoting();
     void propertiesInExportItems();
     void protobuf_data();
     void protobuf();
+    void protobufLibraryInstall();
+    void providersProperties();
     void pseudoMultiplexing();
     void qbsConfig();
+    void qbsConfigAddProfile();
+    void qbsConfigAddProfile_data();
+    void qbsModuleProviders();
+    void qbsModuleProviders_data();
+    void qbsModuleProvidersCliOverride();
+    void qbsModuleProvidersCliOverride_data();
+    void qbsModuleProvidersCompatibility();
+    void qbsModuleProvidersCompatibility_data();
+    void qbspkgconfigModuleProvider();
     void qbsSession();
     void qbsVersion();
     void qtBug51237();
@@ -249,6 +275,8 @@ private slots:
     void recursiveRenaming();
     void recursiveWildcards();
     void referenceErrorInExport();
+    void removeDuplicateLibraries_data();
+    void removeDuplicateLibraries();
     void reproducibleBuild();
     void reproducibleBuild_data();
     void require();
@@ -261,7 +289,11 @@ private slots:
     void ruleCycle();
     void ruleWithNoInputs();
     void ruleWithNonRequiredInputs();
+    void sanitizer_data();
+    void sanitizer();
     void scannerItem();
+    void scanResultInOtherProduct();
+    void scanResultInNonDependency();
     void setupBuildEnvironment();
     void setupRunEnvironment();
     void smartRelinking();
@@ -281,6 +313,7 @@ private slots:
     void suspiciousCalls();
     void suspiciousCalls_data();
     void systemIncludePaths();
+    void distributionIncludePaths();
     void systemRunPaths();
     void systemRunPaths_data();
     void tar();
@@ -295,8 +328,10 @@ private slots:
     void trackRemoveFile();
     void trackRemoveFileTag();
     void trackRemoveProduct();
+    void transitiveInvalidDependencies();
     void transitiveOptionalDependencies();
     void typescript();
+    void undefinedTargetPlatform();
     void usingsAsSoleInputsNonMultiplexed();
     void variantSuffix();
     void variantSuffix_data();
@@ -309,8 +344,6 @@ private slots:
     void wholeArchive_data();
     void wildCardsAndRules();
     void wildcardRenaming();
-    void wix();
-    void wixDependencies();
     void zip();
     void zip_data();
     void zipInvalid();
