@@ -35,11 +35,11 @@ namespace RegistryAccess {
 static QString winErrorMessage(unsigned long error)
 {
     QString rc = QString::fromLatin1("#%1: ").arg(error);
-    ushort *lpMsgBuf;
+    char16_t *lpMsgBuf;
 
     const int len = FormatMessage(
             FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
-            NULL, error, 0, (LPTSTR)&lpMsgBuf, 0, NULL);
+            nullptr, error, 0, (LPTSTR)&lpMsgBuf, 0, nullptr);
     if (len) {
        rc = QString::fromUtf16(lpMsgBuf, len);
         LocalFree(lpMsgBuf);
@@ -63,14 +63,14 @@ static bool registryReadBinaryKey(HKEY handle, // HKEY_LOCAL_MACHINE, etc.
     DWORD type;
     DWORD size;
     // get size and retrieve
-    LONG rc = RegQueryValueEx(handle, valueName, 0, &type, 0, &size);
+    LONG rc = RegQueryValueEx(handle, valueName, nullptr, &type, nullptr, &size);
     if (rc != ERROR_SUCCESS) {
         *errorMessage = msgRegistryOperationFailed("read", valueName, msgFunctionFailed("RegQueryValueEx1", rc));
         return false;
     }
-    BYTE *dataC = new BYTE[size + 1];
+    auto dataC = new BYTE[size + 1];
     // Will be Utf16 in case of a string
-    rc = RegQueryValueEx(handle, valueName, 0, &type, dataC, &size);
+    rc = RegQueryValueEx(handle, valueName, nullptr, &type, dataC, &size);
     if (rc != ERROR_SUCCESS) {
         *errorMessage = msgRegistryOperationFailed("read", valueName, msgFunctionFailed("RegQueryValueEx2", rc));
         return false;
@@ -90,7 +90,7 @@ bool registryReadStringKey(HKEY handle, // HKEY_LOCAL_MACHINE, etc.
         return false;
     data += '\0';
     data += '\0';
-    *s = QString::fromUtf16(reinterpret_cast<const unsigned short*>(data.data()));
+    *s = QString::fromUtf16(reinterpret_cast<const char16_t *>(data.data()));
     return true;
 }
 
@@ -101,7 +101,7 @@ bool openRegistryKey(HKEY category, // HKEY_LOCAL_MACHINE, etc.
                      AccessMode mode,
                      QString *errorMessage)
 {
-    Q_UNUSED(debuggerRegistryKeyC);  // avoid warning from MinGW
+    Q_UNUSED(debuggerRegistryKeyC)  // avoid warning from MinGW
 
     REGSAM accessRights = KEY_READ;
     if (readWrite)
