@@ -30,6 +30,7 @@
 #include <QTextBlock>
 #include <QTextCursor>
 #include <QTextDocument>
+#include <QBuffer>
 
 #include <cplusplus/Control.h>
 #include <cplusplus/Parser.h>
@@ -88,15 +89,51 @@ static const char generatedHeader[] =
 
 static QIODevice::OpenMode openFlags = QIODevice::WriteOnly | QIODevice::Text;
 
-static void closeAndPrintFilePath(QFile &file)
+static void closeBufferAndWriteIfChanged(QBuffer& textBuffer, const QString& filePath)
 {
-    if (file.isOpen()) {
-        const QString filePath = QFileInfo(file).canonicalFilePath();
-        std::cout << QDir::toNativeSeparators(filePath).toLatin1().constData() << std::endl;
+    textBuffer.close();
+
+    std::string filePathForLog = QDir::toNativeSeparators(filePath).toLatin1().toStdString();
+
+    QFile file(filePath);
+    if (! file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+        std::cerr << "Cannot open " << filePathForLog << std::endl;
+        return;
+    }
+    QTextStream fileReader(&file);
+    QBuffer currentTextBuffer;
+    currentTextBuffer.open(QIODevice::WriteOnly | QIODevice::Text);
+    QTextStream currentTextBufferWriter(&currentTextBuffer);
+    while (!fileReader.atEnd()) {
+        currentTextBufferWriter << fileReader.readLine() << Qt::endl;
+    }
+    file.close();
+    currentTextBuffer.close();
+
+    if (currentTextBuffer.data() != textBuffer.data()) {
+        if (!file.open(QIODevice::WriteOnly)) {
+            std::cerr << "Cannot open " << filePathForLog << std::endl;
+            return;
+        }
+        file.write(textBuffer.data());
         file.close();
+
+        std::cout << "changed: " << filePathForLog << std::endl;
+    } else {
+        std::cout << "not changed: " << filePathForLog << std::endl;
     }
 }
 
+static void writeIfChanged(const QString& string, const QString& filePath)
+{
+    QBuffer buffer;
+    buffer.open(openFlags);
+    QTextStream output(&buffer);
+    output << string;
+    closeBufferAndWriteIfChanged(buffer, filePath);
+}
+
+
 class ASTNodes
 {
 public:
@@ -209,23 +246,22 @@ public:
     {
         QFileInfo fileInfo(_cplusplusDir, QLatin1String("ASTVisit.cpp"));
 
-        QFile file(fileInfo.absoluteFilePath());
-        if (! file.open(openFlags))
-            return;
-
-        QTextStream output(&file);
+        QBuffer buffer;
+        buffer.open(openFlags);
+        QTextStream output(&buffer);
         out = &output;
 
-        *out << copyrightHeader << generatedHeader <<
-            "\n"
-            "#include \"AST.h\"\n"
-            "#include \"ASTVisitor.h\"\n"
-            "\n"
-            "using namespace CPlusPlus;\n" << endl;
+        *out << copyrightHeader << generatedHeader
+             << "\n"
+                "#include \"AST.h\"\n"
+                "#include \"ASTVisitor.h\"\n"
+                "\n"
+                "using namespace CPlusPlus;\n"
+             << Qt::endl;
 
         accept(ast);
 
-        closeAndPrintFilePath(file);
+        closeBufferAndWriteIfChanged(buffer, fileInfo.absoluteFilePath());
     }
 
 protected:
@@ -245,7 +281,7 @@ protected:
 
     void visitMembers(Class *klass)
     {
-        // *out << "        // visit " << className.constData() << endl;
+        // *out << "        // visit " << className.constData() << Qt::endl;
         for (int i = 0; i < klass->memberCount(); ++i) {
             Symbol *member = klass->memberAt(i);
             if (! member->name())
@@ -266,7 +302,8 @@ protected:
                     QByteArray typeName = namedTy->name()->identifier()->chars();
 
                     if (typeName.endsWith("AST") && memberName != "next")
-                        *out << "        accept(" << memberName.constData() << ", visitor);" << endl;
+                        *out << "        accept(" << memberName.constData() << ", visitor);"
+                             << Qt::endl;
                 }
             }
         }
@@ -315,18 +352,16 @@ protected:
 
         classMap.insert(className, ast);
 
-        *out
-                << "void " << className.constData() << "::accept0(ASTVisitor *visitor)" << endl
-                << "{" << endl
-                << "    if (visitor->visit(this)) {" << endl;
+        *out << "void " << className.constData() << "::accept0(ASTVisitor *visitor)" << Qt::endl
+             << "{" << Qt::endl
+             << "    if (visitor->visit(this)) {" << Qt::endl;
 
         visitMembers(klass);
 
-        *out
-                << "    }" << endl
-                << "    visitor->endVisit(this);" << endl
-                << "}" << endl
-                << endl;
+        *out << "    }" << Qt::endl
+             << "    visitor->endVisit(this);" << Qt::endl
+             << "}" << Qt::endl
+             << Qt::endl;
 
         return true;
     }
@@ -346,22 +381,21 @@ public:
     {
         QFileInfo fileInfo(_cplusplusDir, QLatin1String("ASTMatch0.cpp"));
 
-        QFile file(fileInfo.absoluteFilePath());
-        if (! file.open(openFlags))
-            return;
-
-        QTextStream output(&file);
+        QBuffer buffer;
+        buffer.open(openFlags);
+        QTextStream output(&buffer);
         out = &output;
-        *out << copyrightHeader << generatedHeader <<
-            "\n"
-            "#include \"AST.h\"\n"
-            "#include \"ASTMatcher.h\"\n"
-            "\n"
-            "using namespace CPlusPlus;\n" << endl;
+        *out << copyrightHeader << generatedHeader
+             << "\n"
+                "#include \"AST.h\"\n"
+                "#include \"ASTMatcher.h\"\n"
+                "\n"
+                "using namespace CPlusPlus;\n"
+             << Qt::endl;
 
         accept(ast);
 
-        closeAndPrintFilePath(file);
+        closeBufferAndWriteIfChanged(buffer, fileInfo.absoluteFilePath());
     }
 
 protected:
@@ -384,11 +418,12 @@ protected:
         Overview oo;
         const QString className = oo(klass->name());
 
-        *out << "    if (" << className << " *_other = pattern->as" << className.left(className.length() - 3) << "())" << endl;
+        *out << "    if (" << className << " *_other = pattern->as"
+             << className.left(className.length() - 3) << "())" << Qt::endl;
 
-        *out << "        return matcher->match(this, _other);" << endl;
+        *out << "        return matcher->match(this, _other);" << Qt::endl;
 
-        *out << endl;
+        *out << Qt::endl;
     }
 
     bool checkMethod(Symbol *accept0Method) const
@@ -427,16 +462,13 @@ protected:
 
         classMap.insert(className, ast);
 
-        *out
-                << "bool " << className.constData() << "::match0(AST *pattern, ASTMatcher *matcher)" << endl
-                << "{" << endl;
+        *out << "bool " << className.constData() << "::match0(AST *pattern, ASTMatcher *matcher)"
+             << Qt::endl
+             << "{" << Qt::endl;
 
         visitMembers(klass);
 
-        *out
-                << "    return false;" << endl
-                << "}" << endl
-                << endl;
+        *out << "    return false;" << Qt::endl << "}" << Qt::endl << Qt::endl;
 
         return true;
     }
@@ -457,30 +489,27 @@ public:
     {
         QFileInfo fileInfo(_cplusplusDir, QLatin1String("ASTMatcher.cpp"));
 
-        QFile file(fileInfo.absoluteFilePath());
-        if (! file.open(openFlags))
-            return;
-
-        QTextStream output(&file);
+        QBuffer buffer;
+        buffer.open(openFlags);
+        QTextStream output(&buffer);
         out = &output;
 
-        *out << copyrightHeader << endl
-                << generatedHeader
-                << "#include \"AST.h\"" << endl
-                << "#include \"ASTMatcher.h\"" << endl
-                << endl
-                << "using namespace CPlusPlus;" << endl
-                << endl
-                << "ASTMatcher::ASTMatcher()" << endl
-                << "{ }" << endl
-                << endl
-                << "ASTMatcher::~ASTMatcher()" << endl
-                << "{ }" << endl
-                << endl;
+        *out << copyrightHeader << Qt::endl
+             << generatedHeader << "#include \"AST.h\"" << Qt::endl
+             << "#include \"ASTMatcher.h\"" << Qt::endl
+             << Qt::endl
+             << "using namespace CPlusPlus;" << Qt::endl
+             << Qt::endl
+             << "ASTMatcher::ASTMatcher()" << Qt::endl
+             << "{ }" << Qt::endl
+             << Qt::endl
+             << "ASTMatcher::~ASTMatcher()" << Qt::endl
+             << "{ }" << Qt::endl
+             << Qt::endl;
 
         accept(ast);
 
-        closeAndPrintFilePath(file);
+        closeBufferAndWriteIfChanged(buffer, fileInfo.absoluteFilePath());
     }
 
 protected:
@@ -512,10 +541,8 @@ protected:
 
             const QByteArray memberName = QByteArray::fromRawData(id->chars(), id->size());
             if (member->type()->isIntegerType() && memberName.endsWith("_token")) {
-
-                *out
-                        << "    pattern->" << memberName << " = node->" << memberName << ";" << endl
-                        << endl;
+                *out << "    pattern->" << memberName << " = node->" << memberName << ";" << Qt::endl
+                     << Qt::endl;
 
             } else if (PointerType *ptrTy = member->type()->asPointerType()) {
 
@@ -523,12 +550,13 @@ protected:
                     QByteArray typeName = namedTy->name()->identifier()->chars();
 
                     if (typeName.endsWith("AST")) {
-                        *out
-                                << "    if (! pattern->" << memberName << ")" << endl
-                                << "        pattern->" << memberName << " = node->" << memberName << ";" << endl
-                                << "    else if (! AST::match(node->" << memberName << ", pattern->" << memberName << ", this))" << endl
-                                << "        return false;" << endl
-                                << endl;
+                        *out << "    if (! pattern->" << memberName << ")" << Qt::endl
+                             << "        pattern->" << memberName << " = node->" << memberName
+                             << ";" << Qt::endl
+                             << "    else if (! AST::match(node->" << memberName << ", pattern->"
+                             << memberName << ", this))" << Qt::endl
+                             << "        return false;" << Qt::endl
+                             << Qt::endl;
                     }
                 }
             }
@@ -578,19 +606,16 @@ protected:
 
         classMap.insert(className, ast);
 
-        *out
-                << "bool ASTMatcher::match(" << className.constData() << " *node, " << className.constData() << " *pattern)" << endl
-                << "{" << endl
-                << "    (void) node;" << endl
-                << "    (void) pattern;" << endl
-                << endl;
+        *out << "bool ASTMatcher::match(" << className.constData() << " *node, "
+             << className.constData() << " *pattern)" << Qt::endl
+             << "{" << Qt::endl
+             << "    (void) node;" << Qt::endl
+             << "    (void) pattern;" << Qt::endl
+             << Qt::endl;
 
         visitMembers(klass);
 
-        *out
-                << "    return true;" << endl
-                << "}" << endl
-                << endl;
+        *out << "    return true;" << Qt::endl << "}" << Qt::endl << Qt::endl;
 
         return true;
     }
@@ -610,24 +635,20 @@ public:
     {
         QFileInfo fileInfo(_cplusplusDir, QLatin1String("ASTClone.cpp"));
 
-        QFile file(fileInfo.absoluteFilePath());
-        if (! file.open(openFlags))
-            return;
-
-        QTextStream output(&file);
+        QBuffer buffer;
+        buffer.open(openFlags);
+        QTextStream output(&buffer);
         out = &output;
 
-        *out << copyrightHeader
-                << generatedHeader
-                << "#include \"AST.h\"" << endl
-                << "#include \"MemoryPool.h\"" << endl
-                << endl
-                << "using namespace CPlusPlus;" << endl
-                << endl;
+        *out << copyrightHeader << generatedHeader << "#include \"AST.h\"" << Qt::endl
+             << "#include \"MemoryPool.h\"" << Qt::endl
+             << Qt::endl
+             << "using namespace CPlusPlus;" << Qt::endl
+             << Qt::endl;
 
         accept(ast);
 
-        closeAndPrintFilePath(file);
+        closeBufferAndWriteIfChanged(buffer, fileInfo.absoluteFilePath());
     }
 
 protected:
@@ -659,18 +680,22 @@ protected:
 
             const QByteArray memberName = QByteArray::fromRawData(id->chars(), id->size());
             if (member->type()->isIntegerType() && memberName.endsWith("_token")) {
-                *out << "    ast->" << memberName << " = " << memberName << ";" << endl;
+                *out << "    ast->" << memberName << " = " << memberName << ";" << Qt::endl;
             } else if (PointerType *ptrTy = member->type()->asPointerType()) {
                 if (NamedType *namedTy = ptrTy->elementType()->asNamedType()) {
                     QByteArray typeName = namedTy->name()->identifier()->chars();
 
                     if (typeName.endsWith("ListAST")) {
-                        *out << "    for (" << typeName << " *iter = " << memberName << ", **ast_iter = &ast->" << memberName << ";" << endl
-                             << "         iter; iter = iter->next, ast_iter = &(*ast_iter)->next)" << endl
-                             << "        *ast_iter = new (pool) " << typeName << "((iter->value) ? iter->value->clone(pool) : 0);" << endl;
+                        *out << "    for (" << typeName << " *iter = " << memberName
+                             << ", **ast_iter = &ast->" << memberName << ";" << Qt::endl
+                             << "         iter; iter = iter->next, ast_iter = &(*ast_iter)->next)"
+                             << Qt::endl
+                             << "        *ast_iter = new (pool) " << typeName
+                             << "((iter->value) ? iter->value->clone(pool) : nullptr);" << Qt::endl;
                     } else if (typeName.endsWith("AST")) {
-                        *out << "    if (" << memberName << ")" << endl
-                             << "        ast->" << memberName << " = " << memberName << "->clone(pool);" << endl;
+                        *out << "    if (" << memberName << ")" << Qt::endl
+                             << "        ast->" << memberName << " = " << memberName
+                             << "->clone(pool);" << Qt::endl;
                     }
                 }
             }
@@ -722,14 +747,15 @@ protected:
 
         classMap.insert(className, ast);
 
-        *out << className.constData() << " *" << className.constData() << "::" << "clone(MemoryPool *pool) const" << endl
-             << "{" << endl
-             << "    " << className.constData() << " *ast = new (pool) " << className.constData() << ";" << endl;
+        *out << className.constData() << " *" << className.constData() << "::"
+             << "clone(MemoryPool *pool) const" << Qt::endl
+             << "{" << Qt::endl
+             << "    " << className.constData() << " *ast = new (pool) " << className.constData()
+             << ";" << Qt::endl;
 
         visitMembers(klass);
 
-        *out << "    return ast;" << endl
-             << "}" << endl << endl;
+        *out << "    return ast;" << Qt::endl << "}" << Qt::endl << Qt::endl;
 
         return false;
     }
@@ -740,27 +766,21 @@ class GenerateDumpers: protected ASTVisitor
     QTextStream out;
 
 public:
-    GenerateDumpers(QFile *file, TranslationUnit *unit)
-        : ASTVisitor(unit), out(file)
+    GenerateDumpers(QBuffer *buffer, TranslationUnit *unit)
+        : ASTVisitor(unit), out(buffer)
     { }
 
     static void go(const QString &fileName, TranslationUnit *unit)
     {
-        QFile file(fileName);
-        if (! file.open(openFlags)) {
-            std::cerr << "Cannot open dumpers file." << std::endl;
-            return;
-        }
-
-        GenerateDumpers d(&file, unit);
-        d.out << copyrightHeader
-                << generatedHeader
-                << endl;
+        QBuffer buffer;
+        buffer.open(openFlags);
 
+        GenerateDumpers d(&buffer, unit);
+        d.out << copyrightHeader << generatedHeader << Qt::endl;
 
         d.accept(unit->ast());
 
-        closeAndPrintFilePath(file);
+        closeBufferAndWriteIfChanged(buffer, fileName);
     }
 
 protected:
@@ -792,17 +812,18 @@ protected:
 
             const QByteArray memberName = QByteArray::fromRawData(id->chars(), id->size());
             if (member->type()->isIntegerType() && memberName.endsWith("_token")) {
-                out << "    if (ast->" << memberName << ")" << endl;
-                out << "        terminal(ast->" << memberName << ", ast);" << endl;
+                out << "    if (ast->" << memberName << ")" << Qt::endl;
+                out << "        terminal(ast->" << memberName << ", ast);" << Qt::endl;
             } else if (PointerType *ptrTy = member->type()->asPointerType()) {
                 if (NamedType *namedTy = ptrTy->elementType()->asNamedType()) {
                     QByteArray typeName = namedTy->name()->identifier()->chars();
 
                     if (typeName.endsWith("ListAST")) {
-                        out << "    for (" << typeName << " *iter = ast->" << memberName << "; iter; iter = iter->next)" << endl
-                            << "        nonterminal(iter->value);" << endl;
+                        out << "    for (" << typeName << " *iter = ast->" << memberName
+                            << "; iter; iter = iter->next)" << Qt::endl
+                            << "        nonterminal(iter->value);" << Qt::endl;
                     } else if (typeName.endsWith("AST")) {
-                        out << "    nonterminal(ast->" << memberName << ");" << endl;
+                        out << "    nonterminal(ast->" << memberName << ");" << Qt::endl;
                     }
                 }
             }
@@ -854,13 +875,12 @@ protected:
 
         classMap.insert(className, ast);
 
-        out << "virtual bool visit(" << className.constData() << " *ast)" << endl
-            << "{" << endl;
+        out << "virtual bool visit(" << className.constData() << " *ast)" << Qt::endl
+            << "{" << Qt::endl;
 
         visitMembers(klass);
 
-        out << "    return false;" << endl
-            << "}" << endl << endl;
+        out << "    return false;" << Qt::endl << "}" << Qt::endl << Qt::endl;
 
         return false;
     }
@@ -954,43 +974,42 @@ struct GenInfo
 
 void generateFirstToken(QTextStream &os, const QString &className, const QStringList &fields)
 {
-    os << "int " << className << "::firstToken() const" << endl << "{" << endl;
+    os << "int " << className << "::firstToken() const" << Qt::endl << "{" << Qt::endl;
 
     for (const QString &field : fields) {
-        os << "    if (" << field << ")" << endl;
+        os << "    if (" << field << ")" << Qt::endl;
 
         if (field.endsWith(QLatin1String("_token"))) {
-            os << "        return " << field << ";" << endl;
+            os << "        return " << field << ";" << Qt::endl;
         } else {
-            os << "        if (int candidate = " << field << "->firstToken())" << endl;
-            os << "            return candidate;" << endl;
+            os << "        if (int candidate = " << field << "->firstToken())" << Qt::endl;
+            os << "            return candidate;" << Qt::endl;
         }
     }
 
-    os << "    return 0;" << endl;
-    os << "}" << endl << endl;
+    os << "    return 0;" << Qt::endl;
+    os << "}" << Qt::endl << Qt::endl;
 }
 
 void generateLastToken(QTextStream &os, const QString &className, const QStringList &fields)
 {
-    os << "int "<< className << "::lastToken() const" << endl
-            << "{" << endl;
+    os << "int " << className << "::lastToken() const" << Qt::endl << "{" << Qt::endl;
 
     for (int i = fields.size() - 1; i >= 0; --i) {
         const QString field = fields.at(i);
 
-        os << "    if (" << field << ")" << endl;
+        os << "    if (" << field << ")" << Qt::endl;
 
         if (field.endsWith(QLatin1String("_token"))) {
-            os << "        return " << field << " + 1;" << endl;
+            os << "        return " << field << " + 1;" << Qt::endl;
         } else {
-            os << "        if (int candidate = " << field << "->lastToken())" << endl;
-            os << "            return candidate;" << endl;
+            os << "        if (int candidate = " << field << "->lastToken())" << Qt::endl;
+            os << "            return candidate;" << Qt::endl;
         }
     }
 
-    os << "    return 1;" << endl;
-    os << "}" << endl << endl;
+    os << "    return 1;" << Qt::endl;
+    os << "}" << Qt::endl << Qt::endl;
 }
 
 void generateAST_cpp(const Snapshot &snapshot, const QDir &cplusplusDir)
@@ -1015,7 +1034,7 @@ void generateAST_cpp(const Snapshot &snapshot, const QDir &cplusplusDir)
     QTextDocument cpp_document;
     cpp_document.setPlainText(QString::fromUtf8(source));
 
-    Document::Ptr AST_cpp_document = snapshot.preprocessedDocument(source, fileName);
+    Document::Ptr AST_cpp_document = snapshot.preprocessedDocument(source, Utils::FilePath::fromString(fileName));
     AST_cpp_document->check();
 
     Overview oo;
@@ -1147,28 +1166,24 @@ void generateAST_cpp(const Snapshot &snapshot, const QDir &cplusplusDir)
     for (StringClassSpecifierASTMapConstIt it = classesNeedingFirstToken.constBegin(); it != cfend; ++it) {
         const QString &className = it.key();
         const QStringList fields = collectFieldNames(it.value(), true);
-        os << "/** \\generated */" << endl;
+        os << "/** \\generated */" << Qt::endl;
         generateFirstToken(os, className, fields);
         if (ClassSpecifierAST *classAST = classesNeedingLastToken.value(className, nullptr)) {
             const QStringList fields = collectFieldNames(classAST, true);
-            os << "/** \\generated */" << endl;
+            os << "/** \\generated */" << Qt::endl;
             generateLastToken(os, className, fields);
             classesNeedingLastToken.remove(className);
         }
     }
     const StringClassSpecifierASTMapConstIt clend = classesNeedingLastToken.constEnd();
     for (StringClassSpecifierASTMapConstIt it = classesNeedingLastToken.constBegin(); it != clend; ++it) {
-        os << "/** \\generated */" << endl;
+        os << "/** \\generated */" << Qt::endl;
         generateLastToken(os, it.key(), collectFieldNames(it.value(), true));
     }
     tc.setPosition(documentEnd);
     tc.insertText(newMethods);
 
-    if (file.open(openFlags)) {
-        QTextStream out(&file);
-        out << cpp_document.toPlainText();
-        closeAndPrintFilePath(file);
-    }
+    writeIfChanged(cpp_document.toPlainText(), fileName);
 }
 
 void generateASTVisitor_H(const Snapshot &, const QDir &cplusplusDir,
@@ -1177,13 +1192,9 @@ void generateASTVisitor_H(const Snapshot &, const QDir &cplusplusDir,
   QFileInfo fileASTVisitor_h(cplusplusDir, QLatin1String("ASTVisitor.h"));
   Q_ASSERT(fileASTVisitor_h.exists());
 
-  const QString fileName = fileASTVisitor_h.absoluteFilePath();
-
-  QFile file(fileName);
-  if (! file.open(openFlags))
-    return;
-
-  QTextStream out(&file);
+  QBuffer buffer;
+  buffer.open(openFlags);
+  QTextStream out(&buffer);
   out << copyrightHeader <<
 "\n"
 "#pragma once\n"
@@ -1256,7 +1267,7 @@ void generateASTVisitor_H(const Snapshot &, const QDir &cplusplusDir,
 "\n"
 "} // namespace CPlusPlus\n";
 
-  closeAndPrintFilePath(file);
+  closeBufferAndWriteIfChanged(buffer, fileASTVisitor_h.absoluteFilePath());
 }
 
 void generateASTMatcher_H(const Snapshot &, const QDir &cplusplusDir,
@@ -1265,13 +1276,9 @@ void generateASTMatcher_H(const Snapshot &, const QDir &cplusplusDir,
   QFileInfo fileASTMatcher_h(cplusplusDir, QLatin1String("ASTMatcher.h"));
   Q_ASSERT(fileASTMatcher_h.exists());
 
-  const QString fileName = fileASTMatcher_h.absoluteFilePath();
-
-  QFile file(fileName);
-  if (! file.open(openFlags))
-    return;
-
-  QTextStream out(&file);
+  QBuffer buffer;
+  buffer.open(openFlags);
+  QTextStream out(&buffer);
   out << copyrightHeader <<
 "\n"
 "#pragma once\n"
@@ -1296,7 +1303,7 @@ void generateASTMatcher_H(const Snapshot &, const QDir &cplusplusDir,
 "\n"
 "} // namespace CPlusPlus\n";
 
-  closeAndPrintFilePath(file);
+  closeBufferAndWriteIfChanged(buffer, fileASTMatcher_h.absoluteFilePath());
 }
 
 QStringList generateAST_H(const Snapshot &snapshot, const QDir &cplusplusDir, const QString &dumpersFile)
@@ -1318,7 +1325,7 @@ QStringList generateAST_H(const Snapshot &snapshot, const QDir &cplusplusDir, co
     QTextDocument document;
     document.setPlainText(QString::fromUtf8(source));
 
-    AST_h_document  = snapshot.preprocessedDocument(source, fileName);
+    AST_h_document  = snapshot.preprocessedDocument(source, Utils::FilePath::fromString(fileName));
     AST_h_document->check();
 
     FindASTNodes process(AST_h_document, &document);
@@ -1369,11 +1376,7 @@ QStringList generateAST_H(const Snapshot &snapshot, const QDir &cplusplusDir, co
                 replacementCastMethods.value(classAST));
     }
 
-    if (file.open(openFlags)) {
-        QTextStream out(&file);
-        out << document.toPlainText();
-        closeAndPrintFilePath(file);
-    }
+    writeIfChanged(document.toPlainText(), fileName);
 
     Accept0CG cg(cplusplusDir, AST_h_document->translationUnit());
     cg(AST_h_document->translationUnit()->ast());
@@ -1454,7 +1457,7 @@ void generateASTFwd_h(const Snapshot &snapshot, const QDir &cplusplusDir, const
 
     QTextDocument document;
     document.setPlainText(QString::fromUtf8(source));
-    Document::Ptr doc = snapshot.preprocessedDocument(source, fileName);
+    Document::Ptr doc = snapshot.preprocessedDocument(source, Utils::FilePath::fromString(fileName));
     doc->check();
 
     FindASTForwards process(doc, &document);
@@ -1469,11 +1472,7 @@ void generateASTFwd_h(const Snapshot &snapshot, const QDir &cplusplusDir, const
 
     cursors.first().insertText(replacement);
 
-    if (file.open(openFlags)) {
-        QTextStream out(&file);
-        out << document.toPlainText();
-        closeAndPrintFilePath(file);
-    }
+    writeIfChanged(document.toPlainText(), fileName);
 }
 
 void generateASTPatternBuilder_h(const QDir &cplusplusDir)
@@ -1481,33 +1480,29 @@ void generateASTPatternBuilder_h(const QDir &cplusplusDir)
     typedef QPair<QString, QString> StringPair;
 
     QFileInfo fileInfo(cplusplusDir, QLatin1String("ASTPatternBuilder.h"));
-    QFile file(fileInfo.absoluteFilePath());
-    if (! file.open(openFlags))
-        return;
 
     Overview oo;
-    QTextStream out(&file);
+    QBuffer buffer;
+    buffer.open(openFlags);
+    QTextStream out(&buffer);
 
-    out
-            << copyrightHeader
-            << generatedHeader
-            << "#pragma once" << endl
-            << endl
-            << "#include \"CPlusPlusForwardDeclarations.h\"" << endl
-            << "#include \"AST.h\"" << endl
-            << "#include \"MemoryPool.h\"" << endl
-            << endl
-            << "namespace CPlusPlus {" << endl
-            << endl
-            << "class CPLUSPLUS_EXPORT ASTPatternBuilder" << endl
-            << "{" << endl
-            << "    MemoryPool pool;" << endl
-            << endl
-            << "public:" << endl
-            << "    ASTPatternBuilder() {}" << endl
-            << endl
-            << "    void reset() { pool.reset(); }" << endl
-            << endl;
+    out << copyrightHeader << generatedHeader << "#pragma once" << Qt::endl
+        << Qt::endl
+        << "#include \"CPlusPlusForwardDeclarations.h\"" << Qt::endl
+        << "#include \"AST.h\"" << Qt::endl
+        << "#include \"MemoryPool.h\"" << Qt::endl
+        << Qt::endl
+        << "namespace CPlusPlus {" << Qt::endl
+        << Qt::endl
+        << "class CPLUSPLUS_EXPORT ASTPatternBuilder" << Qt::endl
+        << "{" << Qt::endl
+        << "    MemoryPool pool;" << Qt::endl
+        << Qt::endl
+        << "public:" << Qt::endl
+        << "    ASTPatternBuilder() {}" << Qt::endl
+        << Qt::endl
+        << "    void reset() { pool.reset(); }" << Qt::endl
+        << Qt::endl;
 
     Control *control = AST_h_document->control();
     QSet<QString> classesSet;
@@ -1563,44 +1558,35 @@ void generateASTPatternBuilder_h(const QDir &cplusplusDir)
             }
         }
 
-        out
-                << ")" << endl
-                << "    {" << endl
-                << "        " << className << " *ast = new (&pool) " << className << ';' << endl;
-
+        out << ")" << Qt::endl
+            << "    {" << Qt::endl
+            << "        " << className << " *ast = new (&pool) " << className << ';' << Qt::endl;
 
         for (const StringPair &p : qAsConst(args))
-            out << "        ast->" << p.second << " = " << p.second << ';' << endl;
+            out << "        ast->" << p.second << " = " << p.second << ';' << Qt::endl;
 
-        out
-                << "        return ast;" << endl
-                << "    }" << endl
-                << endl;
+        out << "        return ast;" << Qt::endl << "    }" << Qt::endl << Qt::endl;
     }
 
-    QStringList classesList = classesSet.toList();
+    QStringList classesList = Utils::toList(classesSet);
     Utils::sort(classesList);
     for (const QString &className : qAsConst(classesList)) {
         const QString methodName = className.left(className.length() - 3);
         const QString elementName = className.left(className.length() - 7) + QLatin1String("AST");
-        out
-                << "    " << className << " *" << methodName << "("
-                << elementName << " *value, " << className << " *next = 0)" << endl
-                << "    {" << endl
-                << "        " << className << " *list = new (&pool) " << className << ";" << endl
-                << "        list->next = next;" << endl
-                << "        list->value = value;" << endl
-                << "        return list;" << endl
-                << "    }" << endl
-                << endl;
+        out << "    " << className << " *" << methodName << "(" << elementName << " *value, "
+            << className << " *next = nullptr)" << Qt::endl
+            << "    {" << Qt::endl
+            << "        " << className << " *list = new (&pool) " << className << ";" << Qt::endl
+            << "        list->next = next;" << Qt::endl
+            << "        list->value = value;" << Qt::endl
+            << "        return list;" << Qt::endl
+            << "    }" << Qt::endl
+            << Qt::endl;
     }
 
-    out
-            << "};" << endl
-            << endl
-            << "} // end of namespace CPlusPlus" << endl;
+    out << "};" << Qt::endl << Qt::endl << "} // end of namespace CPlusPlus" << Qt::endl;
 
-    closeAndPrintFilePath(file);
+    closeBufferAndWriteIfChanged(buffer, fileInfo.absoluteFilePath());
 }
 
 void printUsage()
