@@ -334,10 +334,6 @@ void IconLister::addProjectExplorerIcons()
 
         {QIcon(":/projectexplorer/images/category_buildrun.png"), "category_buildrun.png", prefix,
          ""},
-        {QIcon(":/projectexplorer/images/projectexplorer.png"), "projectexplorer.png", prefix,
-         ""},
-        {QIcon(":/projectexplorer/images/session.png"), "session.png", prefix,
-         ""},
         {QIcon(":/projectexplorer/images/BuildSettings.png"), "BuildSettings.png", prefix,
          ""},
         {QIcon(":/projectexplorer/images/CodeStyleSettings.png"), "CodeStyleSettings.png", prefix,
@@ -348,10 +344,6 @@ void IconLister::addProjectExplorerIcons()
          ""},
         {QIcon(":/projectexplorer/images/ProjectDependencies.png"), "ProjectDependencies.png", prefix,
          ""},
-        {QIcon(":/projectexplorer/images/MaemoDevice.png"), "MaemoDevice.png", prefix,
-         ""},
-        {QIcon(":/projectexplorer/images/Simulator.png"), "Simulator.png", prefix,
-         ""},
         {QIcon(":/projectexplorer/images/targetpanel_bottom.png"), "targetpanel_bottom.png", prefix,
          ""},
         {QIcon(":/projectexplorer/images/unconfigured.png"), "unconfigured.png", prefix,
@@ -1034,16 +1026,16 @@ void IconLister::addProfilerTimelineIcons()
     using namespace Utils;
     const QString prefix = "Profiler";
     const QList<IconInfo> icons = {
-        {Icon({{":/tracing/ico_rangeselection.png", Theme::IconsBaseColor}}).icon(), "rangeselection", prefix,
-         ""},
-        {Icon({{":/tracing/ico_rangeselected.png", Theme::IconsBaseColor}}).icon(), "rangeselected", prefix,
-         ""},
-        {Icon({{":/tracing/ico_selectionmode.png", Theme::IconsBaseColor}}).icon(), "selectionmode", prefix,
-         ""},
-        {Icon({{":/tracing/ico_edit.png", Theme::IconsBaseColor}}).icon(), "edit", prefix,
-         ""},
-        {Icon({{":/tracing/range_handle.png", Theme::IconsBaseColor}}).icon(), "range_handle", prefix,
-         ""},
+        {Icon({{":/QtCreator/Tracing/ico_rangeselection.png", Theme::IconsBaseColor}}).icon(),
+         "rangeselection", prefix, ""},
+        {Icon({{":/QtCreator/Tracing/ico_rangeselected.png", Theme::IconsBaseColor}}).icon(),
+         "rangeselected", prefix, ""},
+        {Icon({{":/QtCreator/Tracing/ico_selectionmode.png", Theme::IconsBaseColor}}).icon(),
+         "selectionmode", prefix, ""},
+        {Icon({{":/QtCreator/Tracing/ico_edit.png", Theme::IconsBaseColor}}).icon(),
+         "edit", prefix, ""},
+        {Icon({{":/QtCreator/Tracing/range_handle.png", Theme::IconsBaseColor}}).icon(),
+         "range_handle", prefix, ""},
     };
     m_icons.append(icons);
 }
@@ -1089,16 +1081,6 @@ void IconLister::addWizardIcons()
          ""},
         {QIcon(wizardsPath + "projects/vcs/subversion/icon.png"), "subversion_vcs", prefix,
          ""},
-        {QIcon(wizardsPath + "qtquickstyleicons/default.png"), "default_qtquickstyle", prefix,
-         ""},
-        {QIcon(wizardsPath + "qtquickstyleicons/material-dark.png"), "material-dark_qtquickstyle", prefix,
-         ""},
-        {QIcon(wizardsPath + "qtquickstyleicons/material-light.png"), "material-light_qtquickstyle", prefix,
-         ""},
-        {QIcon(wizardsPath + "qtquickstyleicons/universal-dark.png"), "universal-dark_qtquickstyle", prefix,
-         ""},
-        {QIcon(wizardsPath + "qtquickstyleicons/universal-light.png"), "universal-light_qtquickstyle", prefix,
-         ""},
     };
     m_icons.append(icons);
 }
