@@ -80,7 +80,7 @@ int main(int argc, char *argv[])
 {
     QCoreApplication app(argc, argv);
     app.setApplicationName(QLatin1String("perfparser"));
-    app.setApplicationVersion(QLatin1String("4.13"));
+    app.setApplicationVersion(QLatin1String("4.14"));
 
     if (qEnvironmentVariableIsSet("PERFPARSER_DEBUG_WAIT")) {
 #ifdef Q_OS_LINUX
@@ -304,6 +304,10 @@ int main(int argc, char *argv[])
         infile->seek(filePos);
 
         // first send features, as it may contain better event descriptions
+        if (header.hasFeature(PerfHeader::COMPRESSED) && !data.setCompressed(features.compressed())) {
+            qApp->exit(DataError);
+            return;
+        }
         unwind.features(features);
 
         const auto& attrs = attributes.attributes();
@@ -444,8 +448,12 @@ void PerfTcpSocket::processError(QAbstractSocket::SocketError error)
 PerfTcpSocket::PerfTcpSocket(QCoreApplication *app, const QString &host, quint16 port) :
     QTcpSocket(app), host(host), port(port)
 {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+    connect(this, &QAbstractSocket::errorOccurred, this, &PerfTcpSocket::processError);
+#else
     connect(this, QOverload<QAbstractSocket::SocketError>::of(&QTcpSocket::error),
             this, &PerfTcpSocket::processError);
+#endif
     connect(this, &QAbstractSocket::disconnected, this, &QIODevice::close);
 }
 
