@@ -130,7 +130,7 @@ static bool memoryRead(Dwfl *, Dwarf_Addr addr, Dwarf_Word *result, void *arg)
 
     /* Check overflow. */
     if (addr + sizeof(Dwarf_Word) < addr) {
-        qDebug() << "Invalid memory read requested by dwfl" << hex << addr;
+        qDebug() << "Invalid memory read requested by dwfl" << Qt::hex << addr;
         ui->firstGuessedFrame = ui->frames.length();
         return false;
     }
@@ -146,7 +146,7 @@ static bool memoryRead(Dwfl *, Dwarf_Addr addr, Dwarf_Word *result, void *arg)
         // not stack, try reading from ELF
         if (ui->unwind->ipIsInKernelSpace(addr)) {
             // DWARF unwinding is not done for the kernel
-            qWarning() << "DWARF unwind tried to access kernel space" << hex << addr;
+            qWarning() << "DWARF unwind tried to access kernel space" << Qt::hex << addr;
             return false;
         }
         if (!accessDsoMem(ui, addr, result, wordWidth)) {
@@ -317,7 +317,8 @@ void PerfSymbolTable::registerElf(const PerfRecordMmap &mmap, const QByteArray &
 }
 
 int PerfSymbolTable::insertSubprogram(CuDieRangeMapping *cudie, Dwarf_Die *top, Dwarf_Addr entry,
-                                      qint32 binaryId, qint32 binaryPathId,
+                                      quint64 offset, quint64 size, quint64 relAddr,
+                                      qint32 binaryId, qint32 binaryPathId, qint32 actualPathId,
                                       qint32 inlineCallLocationId, bool isKernel)
 {
     int line = 0;
@@ -327,15 +328,15 @@ int PerfSymbolTable::insertSubprogram(CuDieRangeMapping *cudie, Dwarf_Die *top,
     const QByteArray file = dwarf_decl_file(top);
 
     qint32 fileId = m_unwind->resolveString(file);
-    int locationId = m_unwind->resolveLocation(PerfUnwind::Location(entry, fileId, m_pid, line,
+    int locationId = m_unwind->resolveLocation(PerfUnwind::Location(entry, relAddr, fileId, m_pid, line,
                                                                     column, inlineCallLocationId));
     qint32 symId = m_unwind->resolveString(cudie->dieName(top));
-    m_unwind->resolveSymbol(locationId, PerfUnwind::Symbol(symId, binaryId, binaryPathId, isKernel));
+    m_unwind->resolveSymbol(locationId, PerfUnwind::Symbol{symId, offset, size, binaryId, binaryPathId, actualPathId, isKernel});
 
     return locationId;
 }
 
-int PerfSymbolTable::parseDie(CuDieRangeMapping *cudie, Dwarf_Die *top, qint32 binaryId, qint32 binaryPathId,
+int PerfSymbolTable::parseDie(CuDieRangeMapping *cudie, Dwarf_Die *top, quint64 offset, quint64 size, quint64 relAddr, qint32 binaryId, qint32 binaryPathId, qint32 actualPathId,
                               bool isKernel, Dwarf_Files *files, Dwarf_Addr entry, qint32 parentLocationId)
 {
     int tag = dwarf_tag(top);
@@ -359,17 +360,17 @@ int PerfSymbolTable::parseDie(CuDieRangeMapping *cudie, Dwarf_Die *top, qint32 b
         location.parentLocationId = parentLocationId;
 
         int callLocationId = m_unwind->resolveLocation(location);
-        return insertSubprogram(cudie, top, entry, binaryId, binaryPathId, callLocationId, isKernel);
+        return insertSubprogram(cudie, top, entry, offset, size, relAddr, binaryId, binaryPathId, actualPathId, callLocationId, isKernel);
     }
     case DW_TAG_subprogram:
-        return insertSubprogram(cudie, top, entry, binaryId, binaryPathId, -1, isKernel);
+        return insertSubprogram(cudie, top, entry, offset, size, relAddr, binaryId, binaryPathId, actualPathId, -1, isKernel);
     default:
         return -1;
     }
 }
 
 qint32 PerfSymbolTable::parseDwarf(CuDieRangeMapping *cudie, SubProgramDie *subprogram, const QVector<Dwarf_Die> &inlined,
-                                   Dwarf_Addr bias, qint32 binaryId, qint32 binaryPathId, bool isKernel)
+                                   Dwarf_Addr bias, quint64 offset, quint64 size, quint64 relAddr, qint32 binaryId, qint32 binaryPathId, qint32 actualPathId, bool isKernel)
 {
     Dwarf_Files *files = nullptr;
     dwarf_getsrcfiles(cudie->cudie(), &files, nullptr);
@@ -381,7 +382,7 @@ qint32 PerfSymbolTable::parseDwarf(CuDieRangeMapping *cudie, SubProgramDie *subp
         if (dwarf_entrypc(&scope, &entry) == 0 && entry != 0)
             scopeAddr += entry;
 
-        auto locationId = parseDie(cudie, &scope, binaryId, binaryPathId, isKernel, files, scopeAddr, parentLocationId);
+        auto locationId = parseDie(cudie, &scope, offset, size, relAddr, binaryId, binaryPathId, actualPathId, isKernel, files, scopeAddr, parentLocationId);
         if (locationId != -1)
             parentLocationId = locationId;
     };
@@ -722,7 +723,19 @@ static QByteArray fakeSymbolFromSection(Dwfl_Module *mod, Dwarf_Addr addr)
     return sym;
 }
 
-static PerfAddressCache::SymbolCache cacheSymbols(Dwfl_Module *module, quint64 elfStart)
+static quint64 symbolAddress(quint64 addr, bool isArmArch)
+{
+    // For dwfl API call we need the raw pointer into symtab, so we need to adjust ip.
+    return (!isArmArch || (addr & 1)) ? addr : addr + 1;
+}
+
+static quint64 alignedAddress(quint64 addr, bool isArmArch)
+{
+    // Adjust addr back. The symtab entries are 1 off for all practical purposes.
+    return (isArmArch && (addr & 1)) ? addr - 1 : addr;
+}
+
+static PerfAddressCache::SymbolCache cacheSymbols(Dwfl_Module *module, quint64 elfStart, bool isArmArch)
 {
     PerfAddressCache::SymbolCache cache;
 
@@ -731,8 +744,10 @@ static PerfAddressCache::SymbolCache cacheSymbols(Dwfl_Module *module, quint64 e
         GElf_Sym sym;
         GElf_Addr symAddr;
         const auto symbol = dwfl_module_getsym_info(module, i, &sym, &symAddr, nullptr, nullptr, nullptr);
-        if (symbol)
-            cache.append({symAddr - elfStart, sym.st_size, symbol});
+        if (symbol) {
+            const quint64 start = alignedAddress(sym.st_value, isArmArch);
+            cache.append({symAddr - elfStart, start, sym.st_size, symbol});
+        }
     }
     return cache;
 }
@@ -751,35 +766,42 @@ int PerfSymbolTable::lookupFrame(Dwarf_Addr ip, bool isKernel,
 
     qint32 binaryId = -1;
     qint32 binaryPathId = -1;
+    qint32 actualPathId = -1;
     quint64 elfStart = 0;
     if (elf.isValid()) {
         binaryId = m_unwind->resolveString(elf.originalFileName);
         binaryPathId = m_unwind->resolveString(elf.originalPath);
+        actualPathId = m_unwind->resolveString(elf.localFile.absoluteFilePath().toUtf8());
         elfStart = elf.hasBaseAddr() ? elf.baseAddr : elf.addr;
     }
 
     Dwfl_Module *mod = module(ip, elf);
 
-    PerfUnwind::Location addressLocation(
-                (m_unwind->architecture() != PerfRegisterInfo::ARCH_ARM || (ip & 1))
-                ? ip : ip + 1, -1, m_pid);
+    const bool isArmArch = (m_unwind->architecture() == PerfRegisterInfo::ARCH_ARM);
+    PerfUnwind::Location addressLocation(symbolAddress(ip, isArmArch), 0, -1, m_pid);
     PerfUnwind::Location functionLocation(addressLocation);
 
     QByteArray symname;
     GElf_Off off = 0;
 
+    quint64 start = 0;
+    quint64 size = 0;
+    quint64 relAddr = 0;
     if (mod) {
         if (!addressCache->hasSymbolCache(elf.originalPath)) {
             // cache all symbols in a sorted lookup table and demangle them on-demand
             // note that the symbols within the symtab aren't necessarily sorted,
             // which makes searching repeatedly via dwfl_module_addrinfo potentially very slow
-            addressCache->setSymbolCache(elf.originalPath, cacheSymbols(mod, elfStart));
+            addressCache->setSymbolCache(elf.originalPath, cacheSymbols(mod, elfStart, isArmArch));
         }
 
         auto cachedAddrInfo = addressCache->findSymbol(elf.originalPath, addressLocation.address - elfStart);
         if (cachedAddrInfo.isValid()) {
             off = addressLocation.address - elfStart - cachedAddrInfo.offset;
             symname = cachedAddrInfo.symname;
+            start = cachedAddrInfo.value;
+            size = cachedAddrInfo.size;
+            relAddr = alignedAddress(start + off, isArmArch);
 
             Dwarf_Addr bias = 0;
             functionLocation.address -= off; // in case we don't find anything better
@@ -821,8 +843,8 @@ int PerfSymbolTable::lookupFrame(Dwarf_Addr ip, bool isKernel,
                     addressLocation.parentLocationId = m_unwind->lookupLocation(functionLocation);
                     // otherwise resolve the inline chain if possible
                     if (!scopes.isEmpty() && !m_unwind->hasSymbol(addressLocation.parentLocationId)) {
-                        functionLocation.parentLocationId = parseDwarf(cudie, subprogram, scopes, bias,
-                                                                       binaryId, binaryPathId, isKernel);
+                        functionLocation.parentLocationId = parseDwarf(cudie, subprogram, scopes, bias, start, size, relAddr,
+                                                                       binaryId, binaryPathId, actualPathId, isKernel);
                     }
                 }
             }
@@ -840,7 +862,7 @@ int PerfSymbolTable::lookupFrame(Dwarf_Addr ip, bool isKernel,
             // no sufficient debug information. Use what we already know
             qint32 symId = m_unwind->resolveString(symname);
             m_unwind->resolveSymbol(addressLocation.parentLocationId,
-                                    PerfUnwind::Symbol(symId, binaryId, binaryPathId, isKernel));
+                                    PerfUnwind::Symbol(symId, start, size, binaryId, binaryPathId, actualPathId, isKernel));
         }
     } else {
         if (isKernel) {
@@ -861,10 +883,11 @@ int PerfSymbolTable::lookupFrame(Dwarf_Addr ip, bool isKernel,
         if (!m_unwind->hasSymbol(addressLocation.parentLocationId)) {
             qint32 symId = m_unwind->resolveString(symname);
             m_unwind->resolveSymbol(addressLocation.parentLocationId,
-                                    PerfUnwind::Symbol(symId, binaryId, binaryPathId, isKernel));
+                                    PerfUnwind::Symbol(symId, start, size, binaryId, binaryPathId, actualPathId, isKernel));
         }
     }
-
+    // relAddr - relative address of the function start added with offset from the function start
+    addressLocation.relAddr = relAddr;
     Q_ASSERT(addressLocation.parentLocationId != -1);
     Q_ASSERT(m_unwind->hasSymbol(addressLocation.parentLocationId));
 
