@@ -35,8 +35,6 @@
 
 #include <iostream>
 
-#include <QDir>
-
 // CMakeTools file stuff:
 const char COUNT[] = "CMakeTools.Count";
 const char PREFIX[] = "CMakeTools.";
@@ -92,7 +90,7 @@ bool AddCMakeOperation::setArguments(const QStringList &args)
 
         if (current == "--path") {
             ++i; // skip next;
-            m_path = QDir::fromNativeSeparators(next);
+            m_path = next;
             continue;
         }
         if (next.isNull()) {
@@ -157,7 +155,7 @@ bool AddCMakeOperation::test() const
     if (!unchanged.isEmpty())
         return false;
 
-    // Make sure name stays unique:
+    // add 2nd cmake
     map = addCMake(map, "{some-cm-id}", "name", "/tmp/test",
                    KeyValuePairList() << KeyValuePair("ExtraKey", QVariant("ExtraValue")));
     if (map.value(COUNT).toInt() != 2
@@ -175,7 +173,7 @@ bool AddCMakeOperation::test() const
     cmData = map.value(QString::fromLatin1(PREFIX) + '1').toMap();
         if (cmData.count() != 5
                 || cmData.value(ID_KEY).toString() != "{some-cm-id}"
-                || cmData.value(DISPLAYNAME_KEY).toString() != "name2"
+                || cmData.value(DISPLAYNAME_KEY).toString() != "name"
                 || cmData.value(AUTODETECTED_KEY).toBool() != true
                 || cmData.value(PATH_KEY).toString() != "/tmp/test"
                 || cmData.value("ExtraKey").toString() != "ExtraValue")
@@ -203,22 +201,15 @@ QVariantMap AddCMakeOperation::addCMake(const QVariantMap &map, const QString &i
         return QVariantMap();
     }
 
-    // Sanity check: Make sure displayName is unique.
-    QStringList nameKeys = FindKeyOperation::findKey(map, DISPLAYNAME_KEY);
-    QStringList nameList;
-    foreach (const QString &nameKey, nameKeys)
-        nameList << GetOperation::get(map, nameKey).toString();
-    const QString uniqueName = makeUnique(displayName, nameList);
-
     QVariantMap result = RmKeysOperation::rmKeys(map, {COUNT});
 
     const QString cm = QString::fromLatin1(PREFIX) + QString::number(count);
 
     KeyValuePairList data;
     data << KeyValuePair({cm, ID_KEY}, QVariant(id));
-    data << KeyValuePair({cm, DISPLAYNAME_KEY}, QVariant(uniqueName));
+    data << KeyValuePair({cm, DISPLAYNAME_KEY}, QVariant(displayName));
     data << KeyValuePair({cm, AUTODETECTED_KEY}, QVariant(true));
-    data << KeyValuePair({cm, PATH_KEY}, QVariant(path));
+    data << KeyValuePair({cm, PATH_KEY}, Utils::FilePath::fromUserInput(path).toVariant());
     KeyValuePairList extraList;
     foreach (const KeyValuePair &pair, extra)
         extraList << KeyValuePair(QStringList({cm}) << pair.key, pair.value);
