@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -52,10 +47,6 @@ const char ABIS[] = "Abis";
 const char BINARY[] = "Binary";
 const char ENGINE_TYPE[] = "EngineType";
 
-AddDebuggerOperation::AddDebuggerOperation()
-    : m_engine(0)
-{ }
-
 QString AddDebuggerOperation::name() const
 {
     return QLatin1String("addDebugger");
@@ -63,7 +54,7 @@ QString AddDebuggerOperation::name() const
 
 QString AddDebuggerOperation::helpText() const
 {
-    return QLatin1String("add a debugger to Qt Creator");
+    return QLatin1String("add a debugger");
 }
 
 QString AddDebuggerOperation::argumentsHelpText() const
@@ -78,8 +69,6 @@ QString AddDebuggerOperation::argumentsHelpText() const
 
 bool AddDebuggerOperation::setArguments(const QStringList &args)
 {
-    m_engine = 0;
-
     for (int i = 0; i < args.count(); ++i) {
         const QString current = args.at(i);
         const QString next = ((i + 1) < args.count()) ? args.at(i + 1) : QString();
@@ -182,7 +171,7 @@ bool AddDebuggerOperation::test() const
 
 QVariantMap AddDebuggerOperation::addDebugger(const QVariantMap &map,
                                               const QString &id, const QString &displayName,
-                                              const quint32 &engine, const QString &binary,
+                                              int engine, const QString &binary,
                                               const QStringList &abis, const KeyValuePairList &extra)
 {
     // Sanity check: Make sure autodetection source is not in use already:
@@ -213,22 +202,17 @@ QVariantMap AddDebuggerOperation::addDebugger(const QVariantMap &map,
     toRemove << QLatin1String(COUNT);
     QVariantMap cleaned = RmKeysOperation::rmKeys(map, toRemove);
 
-    // Sanity check: Make sure displayName is unique.
-    QStringList nameKeys = FindKeyOperation::findKey(map, QLatin1String(DISPLAYNAME));
-    QStringList nameList;
-    foreach (const QString &nameKey, nameKeys)
-        nameList << GetOperation::get(map, nameKey).toString();
-    const QString uniqueName = makeUnique(displayName, nameList);
-
     // insert data:
     KeyValuePairList data;
     data << KeyValuePair(QStringList() << debugger << QLatin1String(ID), QVariant(id));
-    data << KeyValuePair(QStringList() << debugger << QLatin1String(DISPLAYNAME), QVariant(uniqueName));
+    data << KeyValuePair(QStringList() << debugger << QLatin1String(DISPLAYNAME),
+                         QVariant(displayName));
     data << KeyValuePair(QStringList() << debugger << QLatin1String(AUTODETECTED), QVariant(true));
 
     data << KeyValuePair(QStringList() << debugger << QLatin1String(ABIS), QVariant(abis));
     data << KeyValuePair(QStringList() << debugger << QLatin1String(ENGINE_TYPE), QVariant(engine));
-    data << KeyValuePair(QStringList() << debugger << QLatin1String(BINARY), QVariant(binary));
+    data << KeyValuePair(QStringList() << debugger << QLatin1String(BINARY),
+                         Utils::FilePath::fromUserInput(binary).toVariant());
 
     data << KeyValuePair(QStringList() << QLatin1String(COUNT), QVariant(count + 1));
 
