@@ -35,8 +35,6 @@
 
 #include <iostream>
 
-#include <QDir>
-
 // ToolChain file stuff:
 const char COUNT[] = "ToolChain.Count";
 const char PREFIX[] = "ToolChain.";
@@ -46,7 +44,6 @@ const char VERSION[] = "Version";
 const char ID[] = "ProjectExplorer.ToolChain.Id";
 const char DISPLAYNAME[] = "ProjectExplorer.ToolChain.DisplayName";
 const char AUTODETECTED[] = "ProjectExplorer.ToolChain.Autodetect";
-const char LANGUAGE_KEY[] = "ProjectExplorer.ToolChain.Language";
 const char LANGUAGE_KEY_V2[] = "ProjectExplorer.ToolChain.LanguageV2";
 
 // GCC ToolChain:
@@ -107,7 +104,7 @@ bool AddToolChainOperation::setArguments(const QStringList &args)
 
         if (current == "--path") {
             ++i; // skip next;
-            m_path = QDir::fromNativeSeparators(next);
+            m_path = next;
             continue;
         }
 
@@ -260,32 +257,26 @@ QVariantMap AddToolChainOperation::addToolChain(const QVariantMap &map, const QS
     KeyValuePairList data;
     data << KeyValuePair({tc, ID}, QVariant(id));
 
-    // Language compatibility hack for 4.2:
+    // Language compatibility hack for old Qt components that use the language spec from 4.2.
+    // Some Qt 5.15 components were actually still using this.
     QString newLang; // QtC 4.3 and later
-    QString oldLang; // QtC 4.2
-    int langInt = lang.toInt(&ok);
-    Q_UNUSED(langInt)
+    lang.toInt(&ok);
     if (lang == "2" || lang == "Cxx") {
         newLang = "Cxx";
-        oldLang = "2";
     } else if (lang == "1" || lang == "C") {
         newLang = "C";
-        oldLang = "1";
     } else if (ok) {
         std::cerr << "Error: Language ID must be 1 for C, 2 for Cxx "
-                  << "or a string like (\"C\", \"Cxx\", \"Nim\", etc.)" << std::endl;
+                  << "or a string like \"C\", \"Cxx\", \"Nim\" (was \""
+                  << qPrintable(lang) << "\")" << std::endl;
         return {};
     } else if (!ok) {
         newLang = lang;
-        oldLang = "";
     }
-    if (!oldLang.isEmpty())
-        data << KeyValuePair({tc, LANGUAGE_KEY}, QVariant(oldLang));
-    if (!newLang.isEmpty())
-        data << KeyValuePair({tc, LANGUAGE_KEY_V2}, QVariant(newLang));
+    data << KeyValuePair({tc, LANGUAGE_KEY_V2}, QVariant(newLang));
     data << KeyValuePair({tc, DISPLAYNAME}, QVariant(displayName));
     data << KeyValuePair({tc, AUTODETECTED}, QVariant(true));
-    data << KeyValuePair({tc, PATH}, QVariant(path));
+    data << KeyValuePair({tc, PATH}, Utils::FilePath::fromUserInput(path).toVariant());
     data << KeyValuePair({tc, TARGET_ABI}, QVariant(abi));
     QVariantList abis;
     QStringList abiStrings = supportedAbis.split(',');
