@@ -112,6 +112,7 @@ private Q_SLOTS:
     void using_insideNamespace();
     void using_insideFunction();
     void templatedFunction_QTCREATORBUG9749();
+    void templateInNamespaceTypeOutside();
 
     void usingInDifferentNamespace_QTCREATORBUG7978();
 
@@ -175,7 +176,7 @@ void tst_FindUsages::inlineMethod()
     QVERIFY(arg);
     QCOMPARE(arg->identifier()->chars(), "arg");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(arg);
     QCOMPARE(findUsages.usages().size(), 2);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -210,7 +211,7 @@ void tst_FindUsages::lambdaCaptureByValue()
     QVERIFY(d);
     QCOMPARE(d->name()->identifier()->chars(), "test");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(d);
     QCOMPARE(findUsages.usages().size(), 3);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -245,7 +246,7 @@ void tst_FindUsages::lambdaCaptureByReference()
     QVERIFY(d);
     QCOMPARE(d->name()->identifier()->chars(), "test");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(d);
     QCOMPARE(findUsages.usages().size(), 3);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -279,7 +280,7 @@ void tst_FindUsages::shadowedNames_1()
     QVERIFY(d);
     QCOMPARE(d->name()->identifier()->chars(), "a");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(d);
     QCOMPARE(findUsages.usages().size(), 2);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -314,7 +315,7 @@ void tst_FindUsages::shadowedNames_2()
     QVERIFY(d);
     QCOMPARE(d->name()->identifier()->chars(), "a");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(d);
     QCOMPARE(findUsages.usages().size(), 3);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -363,7 +364,7 @@ void tst_FindUsages::staticVariables()
     QVERIFY(d);
     QCOMPARE(d->name()->identifier()->chars(), "Foo");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(d);
     QCOMPARE(findUsages.usages().size(), 5);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -398,7 +399,7 @@ void foo2(int b=bar()){}    // 3rd result
     Snapshot snapshot;
     snapshot.insert(doc);
 
-    FindUsages find(src, doc, snapshot);
+    FindUsages find(src, doc, snapshot, true);
     find(s);
 
     QCOMPARE(find.usages().size(), 3);
@@ -465,7 +466,7 @@ struct Struct{
     Snapshot snapshot;
     snapshot.insert(doc);
 
-    FindUsages find(src, doc, snapshot);
+    FindUsages find(src, doc, snapshot, true);
 
     find(memberFunctionFoo);
     QCOMPARE(find.usages().size(), 2);
@@ -539,7 +540,7 @@ int main() {
     Snapshot snapshot;
     snapshot.insert(doc);
 
-    FindUsages find(src, doc, snapshot);
+    FindUsages find(src, doc, snapshot, true);
     find(sv);
     QCOMPARE(find.usages().size(), 7);
     QCOMPARE(find.usages().at(0).type, Usage::Type::Declaration);
@@ -598,7 +599,7 @@ int main()
     Snapshot snapshot;
     snapshot.insert(doc);
 
-    FindUsages find(src, doc, snapshot);
+    FindUsages find(src, doc, snapshot, true);
     find(sv);
     QCOMPARE(find.usages().size(), 7);
     QCOMPARE(find.usages().at(0).type, Usage::Type::Declaration);
@@ -663,7 +664,7 @@ void tst_FindUsages::objc_args()
     Argument *arg = methodImpl->argumentAt(0)->asArgument();
     QCOMPARE(arg->identifier()->chars(), "arg");
 
-    FindUsages findUsages(objcSource, doc, snapshot);
+    FindUsages findUsages(objcSource, doc, snapshot, true);
     findUsages(arg);
     QCOMPARE(findUsages.usages().size(), 2);
     QCOMPARE(findUsages.references().size(), 2);
@@ -701,7 +702,7 @@ void tst_FindUsages::qproperty_1()
     QCOMPARE(setX_method->identifier()->chars(), "setX");
     QCOMPARE(setX_method->argumentCount(), 1);
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(setX_method);
     QCOMPARE(findUsages.usages().size(), 2);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Other);
@@ -747,7 +748,7 @@ void tst_FindUsages::instantiateTemplateWithNestedClass()
     QVERIFY(barDeclaration);
     QCOMPARE(barDeclaration->name()->identifier()->chars(), "bar");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(barDeclaration);
     QCOMPARE(findUsages.usages().size(), 2);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -793,7 +794,7 @@ void tst_FindUsages::operatorAsteriskOfNestedClassOfTemplateClass_QTCREATORBUG90
     QVERIFY(fooDeclaration);
     QCOMPARE(fooDeclaration->name()->identifier()->chars(), "foo");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(fooDeclaration);
 
     QCOMPARE(findUsages.usages().size(), 2);
@@ -838,7 +839,7 @@ void tst_FindUsages::anonymousClass_QTCREATORBUG8963()
     QVERIFY(isNotIntDeclaration);
     QCOMPARE(isNotIntDeclaration->name()->identifier()->chars(), "isNotInt");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(isNotIntDeclaration);
 
     QCOMPARE(findUsages.usages().size(), 2);
@@ -881,7 +882,7 @@ void tst_FindUsages::anonymousClass_QTCREATORBUG11859()
     QVERIFY(fooAsMemberOfAnonymousStruct);
     QCOMPARE(fooAsMemberOfAnonymousStruct->name()->identifier()->chars(), "Foo");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(fooAsStruct);
     QCOMPARE(findUsages.references().size(), 1);
     QCOMPARE(findUsages.usages().size(), 1);
@@ -927,7 +928,7 @@ void tst_FindUsages::using_insideGlobalNamespace()
     Class *structSymbol = nsSymbol->memberAt(0)->asClass();
     QVERIFY(structSymbol);
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(structSymbol);
 
     QCOMPARE(findUsages.usages().size(), 3);
@@ -973,7 +974,7 @@ void tst_FindUsages::using_insideNamespace()
     Class *structSymbol = nsSymbol->memberAt(0)->asClass();
     QVERIFY(structSymbol);
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(structSymbol);
 
     QCOMPARE(findUsages.usages().size(), 3);
@@ -1016,7 +1017,7 @@ void tst_FindUsages::using_insideFunction()
     Class *structSymbol = nsSymbol->memberAt(0)->asClass();
     QVERIFY(structSymbol);
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(structSymbol);
 
     QCOMPARE(findUsages.usages().size(), 3);
@@ -1064,7 +1065,7 @@ void tst_FindUsages::operatorArrowOfNestedClassOfTemplateClass_QTCREATORBUG9005(
     QVERIFY(fooDeclaration);
     QCOMPARE(fooDeclaration->name()->identifier()->chars(), "foo");
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(fooDeclaration);
     QCOMPARE(findUsages.usages().size(), 2);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -1101,7 +1102,7 @@ void tst_FindUsages::templateClassParameters()
     TypenameArgument *templArgument = templateClassTS->templateParameterAt(0)->asTypenameArgument();
     QVERIFY(templArgument);
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(templArgument);
     QCOMPARE(findUsages.usages().size(), 5);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -1147,7 +1148,7 @@ void tst_FindUsages::templateClass_className()
     QVERIFY(classTS);
     QCOMPARE(classTS->memberCount(), 2);
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(classTS);
     QCOMPARE(findUsages.usages().size(), 7);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -1187,7 +1188,7 @@ void tst_FindUsages::templateFunctionParameters()
     TypenameArgument *templArgument = templateFunctionTS->templateParameterAt(0)->asTypenameArgument();
     QVERIFY(templArgument);
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(templArgument);
     QCOMPARE(findUsages.usages().size(), 4);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -1221,13 +1222,52 @@ void tst_FindUsages::templatedFunction_QTCREATORBUG9749()
     QCOMPARE(funcTempl->memberCount(), 2);
     Function *func = funcTempl->memberAt(1)->asFunction();
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(func);
     QCOMPARE(findUsages.usages().size(), 2);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
     QCOMPARE(findUsages.usages().at(1).type, Usage::Type::Other);
 }
 
+void tst_FindUsages::templateInNamespaceTypeOutside()
+{
+    const QByteArray src = R"(
+struct S { int v; };
+namespace N {
+template <typename T> struct Ptr {
+    T* operator->() { return {}; }
+};
+template<typename T> Ptr<T> makePtr() { return {}; }
+}
+int func()
+{
+    auto s = N::makePtr<S>();
+    return s->v;
+})";
+
+    Document::Ptr doc = Document::create("templateInNamespaceTypeOutside");
+    doc->setUtf8Source(src);
+    doc->parse();
+    doc->check();
+
+    QVERIFY(doc->diagnosticMessages().isEmpty());
+    QCOMPARE(doc->globalSymbolCount(), 3);
+
+    Snapshot snapshot;
+    snapshot.insert(doc);
+
+    Class * const structDecl = doc->globalSymbolAt(0)->asClass();
+    QVERIFY(structDecl);
+    QCOMPARE(structDecl->memberCount(), 1);
+    Symbol * const v = structDecl->memberAt(0);
+
+    FindUsages findUsages(src, doc, snapshot, true);
+    findUsages(v);
+    QCOMPARE(findUsages.usages().size(), 2);
+    QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
+    QCOMPARE(findUsages.usages().at(1).type, Usage::Type::Read);
+}
+
 void tst_FindUsages::usingInDifferentNamespace_QTCREATORBUG7978()
 {
     const QByteArray src = "\n"
@@ -1262,7 +1302,7 @@ void tst_FindUsages::usingInDifferentNamespace_QTCREATORBUG7978()
     QCOMPARE(ns->memberCount(), 1);
     Template *templateClass = ns->memberAt(0)->asTemplate();
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(templateClass);
     QCOMPARE(findUsages.usages().size(), 3);
     QCOMPARE(findUsages.usages().at(0).type, Usage::Type::Declaration);
@@ -1291,7 +1331,7 @@ void tst_FindUsages::unicodeIdentifier()
     Declaration *declaration = doc->globalSymbolAt(0)->asDeclaration();
     QVERIFY(declaration);
 
-    FindUsages findUsages(src, doc, snapshot);
+    FindUsages findUsages(src, doc, snapshot, true);
     findUsages(declaration);
     const QList<Usage> usages = findUsages.usages();
     QCOMPARE(usages.size(), 2);
@@ -1323,7 +1363,7 @@ void tst_FindUsages::inAlignas()
     QVERIFY(c);
     QCOMPARE(c->name()->identifier()->chars(), "One");
 
-    FindUsages find(src, doc, snapshot);
+    FindUsages find(src, doc, snapshot, true);
     find(c);
     QCOMPARE(find.usages().size(), 2);
     QCOMPARE(find.usages()[0].type, Usage::Type::Declaration);
@@ -1366,7 +1406,7 @@ void tst_FindUsages::memberAccessAsTemplate()
         QVERIFY(c);
         QCOMPARE(c->name()->identifier()->chars(), "Foo");
 
-        FindUsages find(src, doc, snapshot);
+        FindUsages find(src, doc, snapshot, true);
         find(c);
         QCOMPARE(find.usages().size(), 2);
         QCOMPARE(find.usages()[0].type, Usage::Type::Declaration);
@@ -1387,7 +1427,7 @@ void tst_FindUsages::memberAccessAsTemplate()
         QVERIFY(f);
         QCOMPARE(f->name()->identifier()->chars(), "templateFunc");
 
-        FindUsages find(src, doc, snapshot);
+        FindUsages find(src, doc, snapshot, true);
         find(f);
         QCOMPARE(find.usages().size(), 2);
         QCOMPARE(find.usages()[0].type, Usage::Type::Declaration);
@@ -1430,7 +1470,7 @@ void tst_FindUsages::variadicFunctionTemplate()
         QVERIFY(v);
         QCOMPARE(v->name()->identifier()->chars(), "value");
 
-        FindUsages find(src, doc, snapshot);
+        FindUsages find(src, doc, snapshot, true);
         find(v);
         QCOMPARE(find.usages().size(), 4);
         QCOMPARE(find.usages().at(0).type, Usage::Type::Declaration);
@@ -1481,7 +1521,7 @@ void tst_FindUsages::typeTemplateParameterWithDefault()
         QVERIFY(sv);
         QCOMPARE(sv->name()->identifier()->chars(), "value");
 
-        FindUsages find(src, doc, snapshot);
+        FindUsages find(src, doc, snapshot, true);
         find(xv);
         QCOMPARE(find.usages().size(), 2);
         QCOMPARE(find.usages().at(0).type, Usage::Type::Declaration);
@@ -1525,7 +1565,7 @@ void tst_FindUsages::resolveOrder_for_templateFunction_vs_function()
         QVERIFY(xv);
         QCOMPARE(xv->name()->identifier()->chars(), "value");
 
-        FindUsages find(src, doc, snapshot);
+        FindUsages find(src, doc, snapshot, true);
         find(xv);
         QCOMPARE(find.usages().size(), 2);
         QCOMPARE(find.usages().at(0).type, Usage::Type::Declaration);
@@ -1567,7 +1607,7 @@ void tst_FindUsages::templateArrowOperator_with_defaultType()
         QVERIFY(sv);
         QCOMPARE(sv->name()->identifier()->chars(), "value");
 
-        FindUsages find(src, doc, snapshot);
+        FindUsages find(src, doc, snapshot, true);
         find(sv);
         QCOMPARE(find.usages().size(), 3);
         QCOMPARE(find.usages().at(0).type, Usage::Type::Declaration);
@@ -1639,7 +1679,7 @@ void tst_FindUsages::templateSpecialization_with_IntArgument()
         QCOMPARE(sv[1]->name()->identifier()->chars(), "value");
         QCOMPARE(sv[2]->name()->identifier()->chars(), "value");
 
-        FindUsages find(src, doc, snapshot);
+        FindUsages find(src, doc, snapshot, true);
 
         find(sv[0]);
         QCOMPARE(find.usages().size(), 2);
@@ -1726,7 +1766,7 @@ void tst_FindUsages::templateSpecialization_with_BoolArgument()
         QCOMPARE(sv[0]->name()->identifier()->chars(), "value");
         QCOMPARE(sv[1]->name()->identifier()->chars(), "value");
 
-        FindUsages find(src, doc, snapshot);
+        FindUsages find(src, doc, snapshot, true);
 
         find(sv[0]);
         QCOMPARE(find.usages().size(), 2);
@@ -1802,7 +1842,7 @@ void tst_FindUsages::templatePartialSpecialization()
         QCOMPARE(sv[0]->name()->identifier()->chars(), "value");
         QCOMPARE(sv[1]->name()->identifier()->chars(), "value");
 
-        FindUsages find(src, doc, snapshot);
+        FindUsages find(src, doc, snapshot, true);
 
         find(sv[0]);
         QCOMPARE(find.usages().size(), 2);
@@ -1858,7 +1898,7 @@ int main()
     Snapshot snapshot;
     snapshot.insert(doc);
 
-    FindUsages find(src, doc, snapshot);
+    FindUsages find(src, doc, snapshot, true);
 
     Class *s[3];
     Declaration *sv[3];
@@ -1923,7 +1963,7 @@ int main(){
     QVERIFY(sv);
     QCOMPARE(sv->name()->identifier()->chars(), "value");
 
-    FindUsages find(src, doc, snapshot);
+    FindUsages find(src, doc, snapshot, true);
     find(sv);
     QCOMPARE(find.usages().size(), 2);
     QCOMPARE(find.usages().at(0).type, Usage::Type::Initialization);
@@ -1968,7 +2008,7 @@ int main(){
     QVERIFY(sv);
     QCOMPARE(sv->name()->identifier()->chars(), "value");
 
-    FindUsages find(src, doc, snapshot);
+    FindUsages find(src, doc, snapshot, true);
     find(sv);
     QCOMPARE(find.usages().size(), 2);
     QCOMPARE(find.usages().at(0).type, Usage::Type::Declaration);
@@ -2008,7 +2048,7 @@ int main(){}
     QVERIFY(sv);
     QCOMPARE(sv->name()->identifier()->chars(), "value");
 
-    FindUsages find(src, doc, snapshot);
+    FindUsages find(src, doc, snapshot, true);
     find(sv);
     QCOMPARE(find.usages().size(), 2);
     QCOMPARE(find.usages().at(0).type, Usage::Type::Declaration);
@@ -2021,7 +2061,7 @@ void tst_FindUsages::writableRefs()
 struct S {
     S() : value2(value) {}
     static int value;
-    int value2;
+    int value2 : 2;
     static void *p;
     static const void *p2;
     struct Nested {
@@ -2121,7 +2161,7 @@ int main()
     QCOMPARE(sv->name()->identifier()->chars(), "value");
 
     // Access to struct member
-    FindUsages find(src, doc, snapshot);
+    FindUsages find(src, doc, snapshot, true);
     find(sv);
     QCOMPARE(find.usages().size(), 31);
     QCOMPARE(find.usages().at(0).type, Usage::Type::Read);
@@ -2248,7 +2288,6 @@ int main()
     QCOMPARE(find.usages().at(28).type, Usage::Type::Read);
     QCOMPARE(find.usages().at(29).type, Usage::Type::Read);
     QCOMPARE(find.usages().at(31).type, Usage::Type::Other);
-    QEXPECT_FAIL(nullptr, "parser does not expose static specifier", Continue);
     QCOMPARE(find.usages().at(32).type, Usage::Type::Other);
 
     // Usages of struct type
