@@ -37,6 +37,8 @@
 #endif
 #endif
 
+#include <QtGlobal>
+
 #include <private/qdatetime_p.h>
 #include <private/qfile_p.h>
 #include <private/qfileinfo_p.h>
@@ -101,7 +103,11 @@ typedef boost::unordered::unordered_set<int>::key_equal P_;
 typedef boost::unordered::unordered_set<int>::allocator_type A_;
 
 typedef boost::unordered::detail::set<A_, T_, H_, P_> Uset_types;
+#if BOOST_VERSION <= (1 * 100000 + 64 * 100)
 typedef boost::unordered::detail::table_impl<Uset_types>::table UsetTable;
+#else
+typedef boost::unordered::detail::table<Uset_types> UsetTable;
+#endif
 typedef boost::unordered_set<int> Uset;
 
 OFFSET_ACCESS(Uset::table, Uset, table_);
@@ -138,6 +144,10 @@ void tst_offsets::offsets()
     QCOMPARE(actual, expect);
 }
 
+QT_BEGIN_NAMESPACE
+extern quintptr Q_CORE_EXPORT qtHookData[];
+QT_END_NAMESPACE
+
 void tst_offsets::offsets_data()
 {
     QTest::addColumn<int>("actual");
@@ -145,11 +155,32 @@ void tst_offsets::offsets_data()
     QTest::addColumn<int>("expected64");
 
     const int qtVersion = QT_VERSION;
+    const quintptr qtTypeVersion = qtHookData[6];
 
-    if (qtVersion >= 0x50700)
+    if (qtTypeVersion >= 20)
 #ifdef Q_OS_WIN
 #   ifdef Q_CC_MSVC
-        OFFSET_TEST(QFilePrivate, fileName) << 184 << 248;
+        OFFSET_TEST(QFilePrivate, fileName) << 0 << 304;
+#   else // MinGW
+        OFFSET_TEST(QFilePrivate, fileName) << 0 << 304;
+#   endif
+#else
+        OFFSET_TEST(QFilePrivate, fileName) << 0 << 304;
+#endif
+    else if (qtVersion > 0x50600 && qtTypeVersion >= 17)
+#ifdef Q_OS_WIN
+#   ifdef Q_CC_MSVC
+        OFFSET_TEST(QFilePrivate, fileName) << 164 << 224;
+#   else // MinGW
+        OFFSET_TEST(QFilePrivate, fileName) << 160 << 224;
+#   endif
+#else
+        OFFSET_TEST(QFilePrivate, fileName) << 156 << 224;
+#endif
+    else if (qtVersion >= 0x50700)
+#ifdef Q_OS_WIN
+#   ifdef Q_CC_MSVC
+        OFFSET_TEST(QFilePrivate, fileName) << 176 << 248;
 #   else // MinGW
         OFFSET_TEST(QFilePrivate, fileName) << 172 << 248;
 #   endif
@@ -205,16 +236,23 @@ void tst_offsets::offsets_data()
         OFFSET_TEST(QFilePrivate, fileName) << 140 << 232;
 #endif
 
-        OFFSET_TEST(QFileSystemEntry, m_filePath) << 0 << 0;
-        OFFSET_TEST(QFileInfoPrivate, fileEntry) << 4 << 8;
+    OFFSET_TEST(QFileSystemEntry, m_filePath) << 0 << 0;
+    OFFSET_TEST(QFileInfoPrivate, fileEntry) << 4 << 8;
 
-        QTest::newRow("sizeof(QObjectData)") << int(sizeof(QObjectData))
-            << 28 << 48; // vptr + 3 ptr + 2 int + ptr
+    // Qt5: vptr + 3 ptr + 2 int + ptr
+    // Qt6: vptr + objectlist + 8 unit:1 + uint:24 + int + ptr + bindingstorage (+ ptr)
+    int size32 = qtVersion >= 0x60000 ? 56 : 28;
+    int size64 = qtVersion >= 0x60000 ? 72 : 48;
+    if (qtTypeVersion >= 21) { // the additional ptr was introduced with qtTypeVersion 21
+        size32 += 4;
+        size64 += 8;
+    }
+    QTest::newRow("sizeof(QObjectData)") << int(sizeof(QObjectData)) << size32 << size64;
 
     if (qtVersion >= 0x50000)
-        OFFSET_TEST(QObjectPrivate, extraData) << 28 << 48; // sizeof(QObjectData)
+        OFFSET_TEST(QObjectPrivate, extraData) << size32 << size64; // sizeof(QObjectData)
     else
-        OFFSET_TEST(QObjectPrivate, extraData) << 32 << 56; // sizeof(QObjectData) + 1 ptr
+        OFFSET_TEST(QObjectPrivate, extraData) << size32 + 4 << size64 + 8; // sizeof(QObjectData) + 1 ptr
 
 #if QT_VERSION < 0x50000
         OFFSET_TEST(QObjectPrivate, objectName) << 28 << 48;  // sizeof(QObjectData)
@@ -251,11 +289,16 @@ void tst_offsets::offsets_data()
         OFFSET_TEST(QDateTimePrivate, m_timeZone) << 20 << 24;
         OFFSET_TEST(QDateTimePrivate, m_status) << 24 << 32;
 #   endif
-#else
+#elif QT_VERSION < 0x50e00
         OFFSET_TEST(QDateTimePrivate, m_msecs) << 0 << 0;
         OFFSET_TEST(QDateTimePrivate, m_status) << 8 << 8;
         OFFSET_TEST(QDateTimePrivate, m_offsetFromUtc) << 12 << 12;
         OFFSET_TEST(QDateTimePrivate, m_timeZone) << 20 << 24;
+#else
+        OFFSET_TEST(QDateTimePrivate, m_msecs) << 8 << 8;
+        OFFSET_TEST(QDateTimePrivate, m_status) << 4 << 4;
+        OFFSET_TEST(QDateTimePrivate, m_offsetFromUtc) << 16 << 16;
+        OFFSET_TEST(QDateTimePrivate, m_timeZone) << 20 << 24;
 #endif
 
 #ifdef HAS_BOOST
