@@ -61,18 +61,22 @@ private slots:
     void importTypes_data();
     void importTypes();
 
+    void moduleMapping_data();
+    void moduleMapping();
+
     void initTestCase();
 private:
     QStringList m_basePaths;
 };
 
-void scanDir(const QString &dir)
+void scanDirectory(const QString &dir)
 {
     QFutureInterface<void> result;
     PathsAndLanguages paths;
     paths.maybeInsert(Utils::FilePath::fromString(dir), Dialect::Qml);
     ModelManagerInterface::importScan(result, ModelManagerInterface::workingCopy(), paths,
                                       ModelManagerInterface::instance(), false);
+    ModelManagerInterface::instance()->test_joinAllThreads();
     ViewerContext vCtx;
     vCtx.paths.append(dir);
     Snapshot snap = ModelManagerInterface::instance()->snapshot();
@@ -123,7 +127,8 @@ void tst_ImportCheck::test_data()
     QTest::addColumn<QStringList>("expectedFiles");
     QTest::newRow("base") << QStringList(QString(TESTSRCDIR "/base"))
                           << QStringList({"QML 1.0", "QtQml 2.2", "QtQml 2.1", "QtQuick 2.0",
-                                          "QtQml 2.0", "QtQuick 2.1", "QtQuick 2.2", "<cpp>"})
+                                          "QtQml 2.0", "QtQuick 2.1", "QtQuick 2.2", "QtQuick 2.14",
+                                          "<cpp>"})
                           << QStringList();
     QTest::newRow("001_flatQmlOnly") << QStringList(QString(TESTSRCDIR "/001_flatQmlOnly"))
                           << QStringList()
@@ -169,7 +174,8 @@ void tst_ImportCheck::test_data()
                               << QString(TESTSRCDIR "/003_packageQmlOnly/QtGraphicalEffects/FastBlur.qml"));
     QTest::newRow("004_cppOnly copy") << QStringList(QString(TESTSRCDIR "/004_cppOnly copy"))
                           << QStringList({ "QML 1.0", "QtQml 2.2", "QtQml 2.1", "QtQuick 2.0",
-                                           "QtQml 2.0", "QtQuick 2.1", "QtQuick 2.2", "<cpp>" })
+                                           "QtQml 2.0", "QtQuick 2.1", "QtQuick 2.2", "QtQuick 2.14",
+                                           "<cpp>" })
                           << QStringList();
 }
 
@@ -185,6 +191,7 @@ void tst_ImportCheck::test()
         lPaths.maybeInsert(Utils::FilePath::fromString(path), Dialect::Qml);
     ModelManagerInterface::importScan(result, ModelManagerInterface::workingCopy(), lPaths,
                                       ModelManagerInterface::instance(), false);
+    ModelManagerInterface::instance()->test_joinAllThreads();
     ViewerContext vCtx;
     vCtx.paths.append(paths);
     Snapshot snap = ModelManagerInterface::instance()->snapshot();
@@ -260,7 +267,7 @@ void tst_ImportCheck::importTypes()
 
     // the default qtQmlPath is based on the Qt version in use otherwise
     ModelManagerInterface::ProjectInfo defaultProject;
-    defaultProject.qtQmlPath = importPath;
+    defaultProject.qtQmlPath = Utils::FilePath::fromString(importPath);
     modelManager->setDefaultProject(defaultProject, nullptr);
     modelManager->activateScan();
 
@@ -295,6 +302,100 @@ void tst_ImportCheck::importTypes()
     QVERIFY(allFound);
 }
 
+typedef QHash<QString, QString> StrStrHash;
+
+void tst_ImportCheck::moduleMapping_data()
+{
+    QTest::addColumn<QString>("qmlFile");
+    QTest::addColumn<QString>("importPath");
+    QTest::addColumn<StrStrHash>("moduleMappings");
+    QTest::addColumn<QStringList>("expectedTypes");
+    QTest::addColumn<bool>("expectedResult");
+
+    QTest::newRow("check for plain QtQuick/Controls")
+            << QString(TESTSRCDIR "/moduleMapping/importQtQuick.qml")
+            << QString(TESTSRCDIR "/moduleMapping")
+            << StrStrHash()
+            << QStringList({ "Item", "Button" })
+            << true;
+    QTest::newRow("check that MyControls is not imported")
+            << QString(TESTSRCDIR "/moduleMapping/importQtQuick.qml")
+            << QString(TESTSRCDIR "/moduleMapping")
+            << StrStrHash()
+            << QStringList({ "Item", "Oblong" })
+            << false;
+    QTest::newRow("check that QtQuick controls cannot be found with a mapping")
+            << QString(TESTSRCDIR "/moduleMapping/importQtQuick.qml")
+            << QString(TESTSRCDIR "/moduleMapping")
+            << StrStrHash({ std::make_pair(QStringLiteral("QtQuick.Controls"), QStringLiteral("MyControls")) })
+            << QStringList({ "Item", "Button" })
+            << false;
+    QTest::newRow("check that custom controls can be found with a mapping")
+            << QString(TESTSRCDIR "/moduleMapping/importQtQuick.qml")
+            << QString(TESTSRCDIR "/moduleMapping")
+            << StrStrHash({ std::make_pair(QStringLiteral("QtQuick.Controls"), QStringLiteral("MyControls")) })
+            << QStringList({ "Item", "Oblong" })  // item is in QtQuick, and should still be found, as only
+                                                  // the QtQuick.Controls are redirected
+            << true;
+}
+
+void tst_ImportCheck::moduleMapping()
+{
+    QFETCH(QString, qmlFile);
+    QFETCH(QString, importPath);
+    QFETCH(StrStrHash, moduleMappings);
+    QFETCH(QStringList, expectedTypes);
+    QFETCH(bool, expectedResult);
+
+    // full reset
+    delete ModelManagerInterface::instance();
+    MyModelManager *modelManager = new MyModelManager;
+
+    ModelManagerInterface::ProjectInfo defaultProject;
+    defaultProject.importPaths = PathsAndLanguages();
+    QString qtQuickImportPath = QString(TESTSRCDIR "/importTypes/imports-QtQuick-qmldir-import");
+    defaultProject.importPaths.maybeInsert(Utils::FilePath::fromString(qtQuickImportPath), Dialect::Qml);
+    defaultProject.importPaths.maybeInsert(Utils::FilePath::fromString(importPath), Dialect::Qml);
+    defaultProject.moduleMappings = moduleMappings;
+    modelManager->setDefaultProject(defaultProject, nullptr);
+    modelManager->activateScan();
+
+    scanDirectory(importPath);
+    scanDirectory(qtQuickImportPath);
+
+    modelManager->updateSourceFiles(QStringList(qmlFile), false);
+    modelManager->test_joinAllThreads();
+
+    Snapshot snapshot = modelManager->newestSnapshot();
+    Document::Ptr doc = snapshot.document(qmlFile);
+    QVERIFY(!doc.isNull());
+
+    // It's unfortunate, but nowadays linking can trigger async module loads,
+    // so do it once to start the process, then do it again for real once the
+    // dependencies are available.
+    const auto getContext = [&]() {
+        Link link(snapshot, modelManager->completeVContext(modelManager->projectVContext(doc->language(), doc),doc),
+                  modelManager->builtins(doc));
+        return link();
+    };
+    getContext();
+    modelManager->test_joinAllThreads();
+    snapshot = modelManager->newestSnapshot();
+    doc = snapshot.document(qmlFile);
+
+    ContextPtr context = getContext();
+
+    bool allFound = true;
+    for (const auto &expected : expectedTypes) {
+        if (!context->lookupType(doc.data(), QStringList(expected))) {
+            allFound = false;
+            qWarning() << "Type '" << expected << "' not found";
+        }
+    }
+    QVERIFY(allFound == expectedResult);
+    delete ModelManagerInterface::instance();
+}
+
 #ifdef MANUAL_IMPORT_SCANNER
 
 int main(int argc, char *argv[])
