@@ -172,6 +172,7 @@ def cleanUpUserFiles(pathsToProFiles=None):
             doneWithoutErrors = False
     return doneWithoutErrors
 
+
 def invokeMenuItem(menu, item, *subItems):
     if platform.system() == "Darwin":
         try:
@@ -189,31 +190,28 @@ def invokeMenuItem(menu, item, *subItems):
     itemObject = waitForObjectItem(objectMap.realName(menuObject), item)
     waitFor("itemObject.enabled", 2000)
     activateItem(itemObject)
-    numberedPrefix = "(&\\d \| )?"
+    numberedPrefix = "%d | "
     for subItem in subItems:
-        sub = itemObject.menu()
-        waitFor("sub.visible", 1000)
         # we might have numbered sub items (e.g. "Recent Files") - these have this special prefix
         if subItem.startswith(numberedPrefix):
-            actions = sub.actions()
             triggered = False
-            for i in range(actions.count()):
-                current = actions.at(i)
-                nonPrefix = subItem[len(numberedPrefix):]
-                matcher = re.match("%s(.*)" % numberedPrefix, str(current.text))
-                if matcher and matcher.group(2) == nonPrefix:
-                    itemObject = current
+            for i in range(1, 10):
+                try:
+                    itemObject = waitForObjectItem(itemObject, subItem % i, 1000)
                     activateItem(itemObject)
                     triggered = True
                     break
+                except:
+                    continue
             if not triggered:
                 test.fail("Could not trigger '%s' - item missing or code wrong?" % subItem,
                           "Function arguments: '%s', '%s', %s" % (menu, item, str(subItems)))
                 break # we failed to trigger - no need to process subItems further
         else:
-            itemObject = waitForObjectItem(sub, subItem)
+            itemObject = waitForObjectItem(itemObject, subItem)
             activateItem(itemObject)
 
+
 def logApplicationOutput():
     # make sure application output is shown
     ensureChecked(":Qt Creator_AppOutput_Core::Internal::OutputPaneToggleButton")
@@ -317,7 +315,7 @@ def addCurrentCreatorDocumentation():
     selectFromFileDialog(docPath)
     try:
         waitForObject("{type='QMessageBox' unnamed='1' visible='1' "
-                      "text?='Unable to register documentation.*'}", 3000)
+                      "text~='Unable to register documentation.*'}", 3000)
         test.passes("Qt Creator's documentation found already registered.")
         clickButton(waitForObject("{type='QPushButton' text='OK' unnamed='1' visible='1' "
                                   "container={name='groupBox' type='QGroupBox' visible='1'}}"))
@@ -367,7 +365,7 @@ def getConfiguredKits():
     # end of internal function for iterateQtVersions
     def __setQtVersionForKit__(kit, kitName, kitsQtVersionName):
         mouseClick(waitForObjectItem(":BuildAndRun_QTreeView", kit))
-        qtVersionStr = str(waitForObject(":Kits_QtVersion_QComboBox").currentText)
+        qtVersionStr = str(waitForObjectExists(":Kits_QtVersion_QComboBox").currentText)
         kitsQtVersionName[kitName] = qtVersionStr
     # end of internal function for iterate kits
 
@@ -579,7 +577,7 @@ def dumpChildren(item):
 
 def writeTestResults(folder):
     if not os.path.exists(folder):
-        print "Skipping writing test results (folder '%s' does not exist)." % folder
+        print("Skipping writing test results (folder '%s' does not exist)." % folder)
         return
     resultFile = open("%s.srf" % os.path.join(folder, os.path.basename(squishinfo.testCase)), "w")
     resultFile.write("suite:%s\n" % os.path.basename(os.path.dirname(squishinfo.testCase)))
@@ -619,8 +617,8 @@ def clickOnTab(tabBarStr, tabText, timeout=5000):
     if not waitFor("object.exists(tabBarStr)", timeout):
         raise LookupError("Could not find QTabBar: %s" % objectMap.realName(tabBarStr))
     tabBar = findObject(tabBarStr)
-    if platform.system() == 'Darwin' and not tabBar.visible:
-        test.log("Using workaround for Mac.")
+    if not (platform.system() == 'Linux' or tabBar.visible):
+        test.log("Using workaround for Mac and Windows.")
         setWindowState(tabBar, WindowState.Normal)
         tabBar = waitForObject(tabBarStr, 2000)
     clickTab(tabBar, tabText)
