@@ -65,15 +65,16 @@ def checkTypeAndProperties(typePropertiesDetails):
 
 def main():
     # prepare example project
-    sourceExample = os.path.join(Qt5Path.examplesPath(Targets.DESKTOP_561_DEFAULT),
+    sourceExample = os.path.join(Qt5Path.examplesPath(Targets.DESKTOP_5_14_1_DEFAULT),
                                  "quick", "animation")
     if not neededFilePresent(sourceExample):
         return
     # open Qt Creator
-    startApplication("qtcreator" + SettingsPath)
+    startQC()
     if not startedWithoutPluginError():
         return
 
+    switchToSubMode('Projects')
     typePropDet = (("QPushButton", "Get Started Now", "Get Started Now button"),
                    ("QTreeView", "Sessions", "Sessions section"),
                    ("SessionModelIndex", ("default", False), "default session listed"),
@@ -98,7 +99,7 @@ def main():
     # select "Open project" and select a project
     examplePath = os.path.join(prepareTemplate(sourceExample), "animation.pro")
     openQmakeProject(examplePath, fromWelcome = True)
-    progressBarWait(30000)
+    waitForProjectParsing()
     test.verify(checkIfObjectExists("{column='0' container=':Qt Creator_Utils::NavigationTreeView'"
                                     " text~='animation( \(.*\))?' type='QModelIndex'}"),
                 "Verifying: The project is opened in 'Edit' mode after configuring.")
