@@ -52,12 +52,12 @@ def main():
                         "Verifying Run and Debug are enabled (Qt5 is available).")
             and test.compare(target, Targets.getStringForTarget(Targets.getDefaultKit()),
                              "Verifying selected Target is Qt5.")
-            and test.compare(exe, "QML Scene", "Verifying selected executable is QML Scene.")):
+            and test.compare(exe, "QML Runtime", "Verifying selected executable is QML Runtime.")):
         earlyExit("Something went wrong opening Qml project - probably missing Qt5.")
         return
     switchViewTo(ViewConstants.PROJECTS)
     switchToBuildOrRunSettingsFor(Targets.getDefaultKit(), ProjectSettings.RUN)
-    ensureChecked("{container=':Qt Creator.scrollArea_QScrollArea' text='Enable QML' "
+    ensureChecked("{container=':Qt Creator_Core::Internal::MainWindow' text='Enable QML' "
                   "type='QCheckBox' unnamed='1' visible='1'}")
     switchViewTo(ViewConstants.EDIT)
     clickButton(fancyDebugButton)
@@ -71,9 +71,13 @@ def main():
         earlyExit("Could not find expected Inspector tree inside Locals and Expressions.")
         return
     # reduce items to outer Rectangle object
-    items = items.getChild("QQmlEngine")
+    items = items.getChild("QQuickView")
     if items == None:
-        earlyExit("Could not find expected QQmlEngine tree inside Locals and Expressions.")
+        earlyExit("Could not find expected QQuickView tree inside Locals and Expressions.")
+        return
+    items = items.getChild("QQuickRootItem")
+    if items == None:
+        earlyExit("Could not find expected QQuickRootItem tree inside Locals and Expressions.")
         return
     items = items.getChild("Rectangle")
     if items == None:
@@ -95,23 +99,28 @@ def main():
     invokeMenuItem("File", "Exit")
 
 def __unfoldTree__():
-    rootIndex = getQModelIndexStr("text='QQmlEngine'",
+    # TODO inspect the qmlengine as well?
+    rootIndex = getQModelIndexStr("text='QQuickView'",
                                   ':Locals and Expressions_Debugger::Internal::WatchTreeView')
-    mainRect = getQModelIndexStr("text='Rectangle'", rootIndex)
-    unfoldQModelIndexIncludingProperties(mainRect)
+    unfoldQModelIndex(rootIndex, False)
+    quickRootItem = getQModelIndexStr("text='QQuickRootItem'", rootIndex)
+    unfoldQModelIndex(quickRootItem, False)
+    mainRect = getQModelIndexStr("text='Rectangle'", quickRootItem)
+    unfoldQModelIndex(mainRect)
     subItems = ["text='Rectangle'", "text='Rectangle' occurrence='2'", "text='Text'"]
     for item in subItems:
-        unfoldQModelIndexIncludingProperties(getQModelIndexStr(item, mainRect))
+        unfoldQModelIndex(getQModelIndexStr(item, mainRect))
 
-def unfoldQModelIndexIncludingProperties(indexStr):
+def unfoldQModelIndex(indexStr, includingProperties=True):
     tv = waitForObject(':Locals and Expressions_Debugger::Internal::WatchTreeView')
     # HACK to avoid failing clicks
     tv.scrollToBottom()
     doubleClick(waitForObject(indexStr))
-    propIndex = getQModelIndexStr("text='Properties'", indexStr)
-    # HACK to avoid failing clicks
-    tv.scrollToBottom()
-    doubleClick(waitForObject(propIndex))
+    if includingProperties:
+        propIndex = getQModelIndexStr("text='Properties'", indexStr)
+        # HACK to avoid failing clicks
+        tv.scrollToBottom()
+        doubleClick(waitForObject(propIndex))
 
 def fetchItems(index, valIndex, treeView):
     tree = Tree()
