@@ -44,23 +44,22 @@ def main():
                            'onTriggered: console.log("Break here")'])
         invokeMenuItem("File", "Save All")
         filesAndLines = [
-                        { "%s.Resources.qml\.qrc./.main\\.qml" % projectName : 'onTriggered.*' },
-                        { "%s.Sources.main\\.cpp" % projectName : "QQmlApplicationEngine engine;" }
+                        { "%s.Sources.main\\.cpp" % projectName : "QQmlApplicationEngine engine;" },
+                        { "%s.Resources.qml\.qrc./.main\\.qml" % projectName : 'onTriggered.*' }
                         ]
         test.log("Setting breakpoints")
-        result = setBreakpointsForCurrentProject(filesAndLines)
-        if result:
-            expectedBreakpointsOrder = [{os.path.join(workingDir, projectName, "main.cpp"):10},
-                                        {os.path.join(workingDir, projectName, "main.qml"):13}]
+        expectedBreakpointsOrder = setBreakpointsForCurrentProject(filesAndLines)
+        if expectedBreakpointsOrder:
             availableConfigs = iterateBuildConfigs("Debug")
-            progressBarWait()
-            if not availableConfigs:
+            if len(availableConfigs) > 1:  # having just one config means no change, no progress bar
+                progressBarWait()
+            elif len(availableConfigs) == 0:
                 test.fatal("Haven't found a suitable Qt version - leaving without debugging.")
             for kit, config in availableConfigs:
                 test.log("Selecting '%s' as build config" % config)
                 verifyBuildConfig(kit, config, True, True, True)
                 # explicitly build before start debugging for adding the executable as allowed program to WinFW
-                invokeMenuItem("Build", "Rebuild All Projects")
+                selectFromLocator("t rebuild", "Rebuild (Rebuild All Projects)")
                 waitForCompile(300000)
                 if not checkCompile():
                     test.fatal("Compile had errors... Skipping current build config")
@@ -72,8 +71,7 @@ def main():
                                             "debug")
                     switchViewTo(ViewConstants.EDIT)
                     allowAppThroughWinFW(buildDir, projectName, None)
-                if not doSimpleDebugging(kit, config,
-                                         len(expectedBreakpointsOrder), expectedBreakpointsOrder):
+                if not doSimpleDebugging(kit, config, expectedBreakpointsOrder):
                     try:
                         stopB = findObject(':Qt Creator.Stop_QToolButton')
                         if stopB.enabled:
