@@ -58,7 +58,7 @@ def main():
         test.log("Testing build configuration: " + config)
         invokeMenuItem("Build", "Run qmake")
         waitForCompile()
-        invokeMenuItem("Build", "Rebuild All")
+        selectFromLocator("t rebuild", "Rebuild (Rebuild All Projects)")
         waitForCompile(300000)
         checkCompile()
         checkLastBuild()
