@@ -70,8 +70,8 @@ def main():
         template = current.values()[0]
         with TestSection("Testing project template %s -> %s" % (category, template)):
             displayedPlatforms = __createProject__(category, template)
-            if template.startswith("Qt Quick Application - "):
-                qtVersionsForQuick = ["5.10", "5.14"]
+            if template == "Qt Quick Application":
+                qtVersionsForQuick = ["5.14"]
                 for counter, qtVersion in enumerate(qtVersionsForQuick):
                     def additionalFunc(displayedPlatforms, qtVersion):
                         requiredQtVersion = __createProjectHandleQtQuickSelection__(qtVersion)
