@@ -149,7 +149,7 @@ def __qtFunc__(it, foundQt, qmakePath):
 
 def __kitFunc__(it, foundQt, foundCompNames):
     global currentSelectedTreeItem, warningOrError
-    qtVersionStr = str(waitForObject(":Kits_QtVersion_QComboBox").currentText)
+    qtVersionStr = str(waitForObjectExists(":Kits_QtVersion_QComboBox").currentText)
     test.compare(it, "Desktop (default)", "Verifying whether default Desktop kit has been created.")
     if foundQt:
         test.compare(qtVersionStr, foundQt, "Verifying if Qt versions match.")
@@ -209,7 +209,8 @@ def __getExpectedCompilers__():
         expected.extend(__getWinCompilers__())
     compilers = ["g++", "gcc"]
     if platform.system() in ('Linux', 'Darwin'):
-        for c in ('clang++', 'clang', 'afl-clang', 'clang-[0-9]', 'clang-[0-9].[0-9]',
+        for c in ('clang++', 'clang', 'afl-clang',
+                  'clang-[0-9]', 'clang-[0-9].[0-9]', 'clang-1[0-9]', 'clang-1[0-9].[0-9]',
                   '*g++*', '*gcc*'):
             compilers.extend(findAllFilesInPATH(c))
     if platform.system() == 'Darwin':
