@@ -34,18 +34,13 @@ def __platformToBeRunToday__():
 # for all machines using the same IP-address like you.
 skipPastingToPastebinCom = platform.system() not in __platformToBeRunToday__()
 
-NAME_KDE = "Paste.KDE.Org"
-NAME_PBCA = "Pastebin.Ca"
 NAME_PBCOM = "Pastebin.Com"
-NAME_PCXYZ = "Pastecode.Xyz"
+NAME_DPCOM = "DPaste.Com"
 
 serverProblems = "Server side problems."
 
 def invalidPasteId(protocol):
-    if protocol == NAME_KDE:
-        return None
-    else:
-        return -1
+    return -1
 
 def closeHTTPStatusAndPasterDialog(protocol, pasterDialog):
     try:
@@ -58,11 +53,11 @@ def closeHTTPStatusAndPasterDialog(protocol, pasterDialog):
         if 'Service Unavailable' in text:
             test.warning(text)
             return True
+        test.log("Closed dialog without expected error.", text)
     except:
         t,v = sys.exc_info()[:2]
         test.warning("An exception occurred in closeHTTPStatusAndPasterDialog(): %s(%s)"
                      % (str(t), str(v)))
-    test.log("Closed dialog without expected error.", text)
     return False
 
 def pasteFile(sourceFile, protocol):
@@ -96,8 +91,8 @@ def pasteFile(sourceFile, protocol):
     clickButton(waitForObject(":Send to Codepaster.Paste_QPushButton"))
     try:
         outputWindow = waitForObject(":Qt Creator_Core::OutputWindow")
-        waitFor("'https://' in str(outputWindow.plainText)", 20000)
-        output = str(outputWindow.plainText).splitlines()[-1]
+        waitFor("re.search('^https://', str(outputWindow.plainText)) is not None", 20000)
+        output = filter(lambda x: len(x), str(outputWindow.plainText).splitlines())[-1]
     except:
         output = ""
         if closeHTTPStatusAndPasterDialog(protocol, ':Send to Codepaster_CodePaster::PasteView'):
@@ -133,17 +128,23 @@ def fetchSnippet(protocol, description, pasteId, skippedPasting):
     except:
         closeHTTPStatusAndPasterDialog(protocol, ':PasteSelectDialog_CodePaster::PasteSelectDialog')
         return -1
-    waitFor("pasteModel.rowCount() > 1", 20000)
-    if (not skippedPasting and not protocol == NAME_PBCA
-        and not any(map(lambda str:pasteId in str, dumpItems(pasteModel)))):
+
+    condition = "pasteModel.rowCount() > 1"
+    if protocol == NAME_DPCOM: # no list support
+        condition = "pasteModel.rowCount() == 1"
+    waitFor(condition, 20000)
+
+    if protocol == NAME_DPCOM:
+        items = dumpItems(pasteModel)
+        test.compare(items, ["This protocol does not support listing"], "Check expected message.")
+    elif (protocol != NAME_PBCOM and not skippedPasting
+          and not any(map(lambda str:pasteId in str, dumpItems(pasteModel)))):
         test.warning("Fetching too fast for server of %s - waiting 3s and trying to refresh." % protocol)
         snooze(3)
         clickButton("{text='Refresh' type='QPushButton' unnamed='1' visible='1' "
             "window=':PasteSelectDialog_CodePaster::PasteSelectDialog'}")
         waitFor("pasteModel.rowCount() == 1", 1000)
         waitFor("pasteModel.rowCount() > 1", 20000)
-    if protocol == NAME_PBCA:
-        description = description[:32]
     if pasteId == -1:
         try:
             pasteLine = filter(lambda str:description in str, dumpItems(pasteModel))[0]
@@ -155,28 +156,44 @@ def fetchSnippet(protocol, description, pasteId, skippedPasting):
     else:
         try:
             pasteLine = filter(lambda str:pasteId in str, dumpItems(pasteModel))[0]
-            if protocol in (NAME_PBCA, NAME_PBCOM):
+            if protocol == NAME_PBCOM:
                 test.verify(description in pasteLine,
                             "Verify that line in list of pastes contains the description")
         except:
-            if protocol == NAME_PBCA:
-                test.xfail("%s does not list the pasted snippet on time" % NAME_PBCA)
-            elif not skippedPasting:
-                test.fail("Could not find id '%s' in list of pastes from %s" % (pasteId, protocol))
+            if not skippedPasting:
+                message = "Could not find id '%s' in list of pastes from %s" % (pasteId, protocol)
+                if protocol == NAME_PBCOM:
+                    test.xfail(message, "pastebin.com does not show pastes in list anymore")
+                elif protocol != NAME_DPCOM: # does not happen now, but kept to avoid forgetting to
+                    test.fail(message)       # re-add when protocols change again
             foundSnippet = False
             replaceEditorContent(waitForObject(":PasteSelectDialog.pasteEdit_QLineEdit"), pasteId)
     if foundSnippet:
         pasteLine = pasteLine.replace(".", "\\.")
-        waitForObjectItem(":PasteSelectDialog.listWidget_QListWidget", pasteLine)
-        clickItem(":PasteSelectDialog.listWidget_QListWidget", pasteLine, 5, 5, 0, Qt.LeftButton)
+        mouseClick(waitForObjectItem(":PasteSelectDialog.listWidget_QListWidget", pasteLine))
     clickButton(waitForObject(":PasteSelectDialog.OK_QPushButton"))
     return pasteId
 
+def checkForMovedUrl():  # protocol may be redirected (HTTP status 30x) - check for permanent moves
+    try:
+        pattern = re.compile(r'HTTP redirect \((\d{3})\) to "(.+)"')
+        outputWindow = waitForObject(":Qt Creator_Core::OutputWindow", 2000)
+        match = [None]
+        def __patternMatches__(matchHack):
+            matchHack[0] = pattern.search(str(outputWindow.plainText))
+            return matchHack[0] is not None
+        waitFor("__patternMatches__(match)", 1000)
+        if match[0] is not None:
+            test.fail("URL has moved permanently.", match[0].group(0))
+    except:
+        t,v,tb = sys.exc_info()
+        test.log(str(t), str(v))
+
 def main():
     startQC()
     if not startedWithoutPluginError():
         return
-    protocolsToTest = [NAME_KDE, NAME_PBCA, NAME_PBCOM, NAME_PCXYZ]
+    protocolsToTest = [NAME_PBCOM, NAME_DPCOM]
     sourceFile = os.path.join(os.getcwd(), "testdata", "main.cpp")
     # make sure General Messages is open
     openGeneralMessages()
@@ -185,10 +202,7 @@ def main():
         with TestSection(protocol):
             skippedPasting = True
             description = "Paste from 2017-05-11"
-            if protocol == NAME_KDE:
-                pasteId = "pysjk6n2i"
-                pastedText = readFile(os.path.join(os.getcwd(), "testdata", "main-prepasted.cpp"))
-            elif skipPastingToPastebinCom and protocol == NAME_PBCOM:
+            if skipPastingToPastebinCom and protocol == NAME_PBCOM:
                 pasteId = "8XHP0ZgH"
                 pastedText = readFile(os.path.join(os.getcwd(), "testdata", "main-prepasted.cpp"))
             else:
@@ -225,9 +239,20 @@ def main():
                 clickButton(waitForObject(":*Qt Creator.Clear_QToolButton"))
                 continue
             test.compare(filenameCombo.currentText, "%s: %s" % (protocol, pasteId), "Verify title of editor")
-            if protocol in (NAME_KDE, NAME_PBCOM) and pastedText.endswith("\n"):
-                pastedText = pastedText[:-1]
-            test.compare(editor.plainText, pastedText, "Verify that pasted and fetched texts are the same")
+            if protocol in (NAME_PBCOM):
+                test.verify(abs(len(str(editor.plainText)) - len(pastedText)) < 2,
+                            "Verify that pasted and fetched texts have similar length")
+                test.compare(str(editor.plainText).rstrip(), pastedText.rstrip(),
+                             "Verify that pasted and fetched texts have the same content")
+            else:
+                if protocol in (NAME_DPCOM) and pastedText.endswith("\n"):
+                    pastedText = pastedText[:-1]
+                test.compare(editor.plainText, pastedText,
+                             "Verify that pasted and fetched texts are the same")
+
+            if protocol == NAME_DPCOM:
+                checkForMovedUrl()
+
             invokeMenuItem("File", "Close All")
     invokeMenuItem("File", "Open File or Project...")
     selectFromFileDialog(sourceFile)
