@@ -26,32 +26,37 @@
 #pragma once
 
 #include <utils/hostosinfo.h>
-#include <utils/smallstring.h>
 
-#include <string>
+#include <filepath.h>
+#include <nativefilepath.h>
 
-// use std::filesystem::path if it is supported by all compilers
-static const char nativeSeparator = Utils::HostOsInfo::isWindowsHost() ? '\\' : '/';
+#include <QCoreApplication>
+#include <QDir>
 
-
-template <std::size_t Size>
-std::string toNativePath(const char (&text)[Size])
+template<std::size_t Size>
+ClangBackEnd::NativeFilePath toNativePath(const char (&text)[Size])
 {
-    std::string path = text;
+    ClangBackEnd::FilePath path = text;
 
-    if (Utils::HostOsInfo::isWindowsHost())
-        std::replace(path.begin(), path.end(), '/', '\\');
-
-    return path;
+    return ClangBackEnd::NativeFilePath{path};
 }
 
-inline
-std::string toNativePath(const QString &qStringPath)
+inline Utils::PathString toNativePath(const QString &text)
 {
-    auto path = qStringPath.toStdString();
+    ClangBackEnd::FilePath path{text};
 
-    if (Utils::HostOsInfo::isWindowsHost())
-        std::replace(path.begin(), path.end(), '/', '\\');
+    return ClangBackEnd::NativeFilePath{path}.path();
+}
 
-    return path;
+inline Utils::PathString toNativePath(Utils::SmallStringView text)
+{
+    ClangBackEnd::FilePath path{text};
+
+    return ClangBackEnd::NativeFilePath{path}.path();
+}
+
+inline QString resourcePath()
+{
+    return QDir::cleanPath(QCoreApplication::applicationDirPath() + '/' + RELATIVE_DATA_PATH
+                           + "/indexing_preincludes");
 }
