@@ -25,7 +25,8 @@
 
 #pragma once
 
-#include <googletest.h>
+#include "googletest.h"
+#include "sqlitedatabasemock.h"
 
 #include <sqlitebasestatement.h>
 
@@ -33,7 +34,14 @@
 class BaseMockSqliteStatement
 {
 public:
-    MOCK_METHOD0(next, bool ());
+    using Database = SqliteDatabaseMock;
+
+    BaseMockSqliteStatement() = default;
+    BaseMockSqliteStatement(SqliteDatabaseMock &databaseMock)
+        : m_databaseMock{&databaseMock}
+    {}
+
+    MOCK_METHOD0(next, bool());
     MOCK_METHOD0(step, void ());
     MOCK_METHOD0(reset, void ());
 
@@ -58,6 +66,14 @@ public:
     MOCK_METHOD1(prepare, void(Utils::SmallStringView sqlStatement));
 
     MOCK_METHOD1(checkColumnCount, void(int));
+    MOCK_METHOD1(checkBindingParameterCount, void(int));
+
+    MOCK_CONST_METHOD0(isReadOnlyStatement, bool());
+
+    SqliteDatabaseMock &database() { return *m_databaseMock; }
+
+private:
+    SqliteDatabaseMock *m_databaseMock = nullptr;
 };
 
 template<>
@@ -96,13 +112,16 @@ Utils::PathString BaseMockSqliteStatement::fetchValue<Utils::PathString>(int col
     return fetchPathStringValue(column);
 }
 
-class MockSqliteStatement : public Sqlite::StatementImplementation<NiceMock<BaseMockSqliteStatement>>
+template<int ResultCount = 1, int BindParameterCount = 0>
+class MockSqliteStatement
+    : public Sqlite::StatementImplementation<BaseMockSqliteStatement, ResultCount, BindParameterCount>
 {
-public:
-    explicit MockSqliteStatement()
-        : Sqlite::StatementImplementation<NiceMock<BaseMockSqliteStatement>>()
-    {}
+    using Base = Sqlite::StatementImplementation<BaseMockSqliteStatement, ResultCount, BindParameterCount>;
 
+public:
+    explicit MockSqliteStatement(SqliteDatabaseMock &databaseMock)
+        : Base{databaseMock}
+    {}
 
 protected:
     void checkIsWritableStatement();
