@@ -55,25 +55,29 @@ class SqliteDatabase : public ::testing::Test
 protected:
     SqliteDatabase()
     {
+        database.lock();
         database.setJournalMode(JournalMode::Memory);
         database.setDatabaseFilePath(databaseFilePath);
-        auto &table = database.addTable();
+        Table table;
         table.setName("test");
         table.addColumn("id", Sqlite::ColumnType::Integer, {Sqlite::PrimaryKey{}});
         table.addColumn("name");
 
         database.open();
+
+        table.initialize(database);
     }
 
     ~SqliteDatabase()
     {
         if (database.isOpen())
             database.close();
+        database.unlock();
     }
 
     std::vector<Utils::SmallString> names() const
     {
-        return Sqlite::ReadStatement("SELECT name FROM test", database).values<Utils::SmallString>(8);
+        return Sqlite::ReadStatement<1>("SELECT name FROM test", database).values<Utils::SmallString>(8);
     }
 
     static void updateHookCallback(
@@ -107,6 +111,39 @@ TEST_F(SqliteDatabase, SetJournalMode)
     ASSERT_THAT(database.journalMode(), JournalMode::Memory);
 }
 
+TEST_F(SqliteDatabase, LockingModeIsByDefaultExlusive)
+{
+    ASSERT_THAT(database.lockingMode(), Sqlite::LockingMode::Exclusive);
+}
+
+TEST_F(SqliteDatabase, CreateDatabaseWithLockingModeNormal)
+{
+    Utils::PathString path{Utils::TemporaryDirectory::masterDirectoryPath()
+                           + "/database_exclusive_locked.db"};
+
+    Sqlite::Database database{path, JournalMode::Wal, Sqlite::LockingMode::Normal};
+
+    ASSERT_THAT(database.lockingMode(), Sqlite::LockingMode::Normal);
+}
+
+TEST_F(SqliteDatabase, ExclusivelyLockedDatabaseIsLockedForSecondConnection)
+{
+    Utils::PathString path{Utils::TemporaryDirectory::masterDirectoryPath()
+                           + "/database_exclusive_locked.db"};
+    Sqlite::Database database{path};
+
+    ASSERT_THROW(Sqlite::Database database2{path}, Sqlite::StatementIsBusy);
+}
+
+TEST_F(SqliteDatabase, NormalLockedDatabaseCanBeReopened)
+{
+    Utils::PathString path{Utils::TemporaryDirectory::masterDirectoryPath()
+                           + "/database_exclusive_locked.db"};
+    Sqlite::Database database{path, JournalMode::Wal, Sqlite::LockingMode::Normal};
+
+    ASSERT_NO_THROW((Sqlite::Database{path, JournalMode::Wal, Sqlite::LockingMode::Normal}));
+}
+
 TEST_F(SqliteDatabase, SetOpenlMode)
 {
     database.setOpenMode(OpenMode::ReadOnly);
@@ -157,16 +194,9 @@ TEST_F(SqliteDatabase, DatabaseIsNotInitializedIfDatabasePathDoesNotExistAtOpeni
     ASSERT_FALSE(database.isInitialized());
 }
 
-TEST_F(SqliteDatabase, AddTable)
-{
-    auto sqliteTable = database.addTable();
-
-    ASSERT_THAT(database.tables(), Contains(sqliteTable));
-}
-
 TEST_F(SqliteDatabase, GetChangesCount)
 {
-    Sqlite::WriteStatement statement("INSERT INTO test(name) VALUES (?)", database);
+    Sqlite::WriteStatement<1> statement("INSERT INTO test(name) VALUES (?)", database);
     statement.write(42);
 
     ASSERT_THAT(database.changesCount(), 1);
@@ -174,7 +204,7 @@ TEST_F(SqliteDatabase, GetChangesCount)
 
 TEST_F(SqliteDatabase, GetTotalChangesCount)
 {
-    Sqlite::WriteStatement statement("INSERT INTO test(name) VALUES (?)", database);
+    Sqlite::WriteStatement<1> statement("INSERT INTO test(name) VALUES (?)", database);
     statement.write(42);
 
     ASSERT_THAT(database.lastInsertedRowId(), 1);
@@ -182,24 +212,12 @@ TEST_F(SqliteDatabase, GetTotalChangesCount)
 
 TEST_F(SqliteDatabase, GetLastInsertedRowId)
 {
-    Sqlite::WriteStatement statement("INSERT INTO test(name) VALUES (?)", database);
+    Sqlite::WriteStatement<1> statement("INSERT INTO test(name) VALUES (?)", database);
     statement.write(42);
 
     ASSERT_THAT(database.lastInsertedRowId(), 1);
 }
 
-TEST_F(SqliteDatabase, TableIsReadyAfterOpenDatabase)
-{
-    database.close();
-    auto &table = database.addTable();
-    table.setName("foo");
-    table.addColumn("name");
-
-    database.open();
-
-    ASSERT_TRUE(table.isReady());
-}
-
 TEST_F(SqliteDatabase, LastRowId)
 {
     database.setLastInsertedRowId(42);
@@ -247,7 +265,7 @@ TEST_F(SqliteDatabase, SetUpdateHookSet)
     database.setUpdateHook(this, updateHookCallback);
 
     EXPECT_CALL(callbackMock, Call(_, _, _, _));
-    Sqlite::WriteStatement("INSERT INTO test(name) VALUES (?)", database).write(42);
+    Sqlite::WriteStatement<1>("INSERT INTO test(name) VALUES (?)", database).write(42);
 }
 
 TEST_F(SqliteDatabase, SetNullUpdateHook)
@@ -257,7 +275,7 @@ TEST_F(SqliteDatabase, SetNullUpdateHook)
     database.setUpdateHook(nullptr, nullptr);
 
     EXPECT_CALL(callbackMock, Call(_, _, _, _)).Times(0);
-    Sqlite::WriteStatement("INSERT INTO test(name) VALUES (?)", database).write(42);
+    Sqlite::WriteStatement<1>("INSERT INTO test(name) VALUES (?)", database).write(42);
 }
 
 TEST_F(SqliteDatabase, ResetUpdateHook)
@@ -267,12 +285,12 @@ TEST_F(SqliteDatabase, ResetUpdateHook)
     database.resetUpdateHook();
 
     EXPECT_CALL(callbackMock, Call(_, _, _, _)).Times(0);
-    Sqlite::WriteStatement("INSERT INTO test(name) VALUES (?)", database).write(42);
+    Sqlite::WriteStatement<1>("INSERT INTO test(name) VALUES (?)", database).write(42);
 }
 
 TEST_F(SqliteDatabase, DeleteUpdateHookCall)
 {
-    Sqlite::WriteStatement("INSERT INTO test(name) VALUES (?)", database).write(42);
+    Sqlite::WriteStatement<1>("INSERT INTO test(name) VALUES (?)", database).write(42);
     database.setUpdateHook(this, updateHookCallback);
 
     EXPECT_CALL(callbackMock, Call(Eq(Sqlite::ChangeType::Delete), _, _, _));
@@ -286,7 +304,7 @@ TEST_F(SqliteDatabase, InsertUpdateHookCall)
 
     EXPECT_CALL(callbackMock, Call(Eq(Sqlite::ChangeType::Insert), _, _, _));
 
-    Sqlite::WriteStatement("INSERT INTO test(name) VALUES (?)", database).write(42);
+    Sqlite::WriteStatement<1>("INSERT INTO test(name) VALUES (?)", database).write(42);
 }
 
 TEST_F(SqliteDatabase, UpdateUpdateHookCall)
@@ -295,7 +313,7 @@ TEST_F(SqliteDatabase, UpdateUpdateHookCall)
 
     EXPECT_CALL(callbackMock, Call(Eq(Sqlite::ChangeType::Insert), _, _, _));
 
-    Sqlite::WriteStatement("INSERT INTO test(name) VALUES (?)", database).write(42);
+    Sqlite::WriteStatement<1>("INSERT INTO test(name) VALUES (?)", database).write(42);
 }
 
 TEST_F(SqliteDatabase, RowIdUpdateHookCall)
@@ -304,7 +322,7 @@ TEST_F(SqliteDatabase, RowIdUpdateHookCall)
 
     EXPECT_CALL(callbackMock, Call(_, _, _, Eq(42)));
 
-    Sqlite::WriteStatement("INSERT INTO test(rowid, name) VALUES (?,?)", database).write(42, "foo");
+    Sqlite::WriteStatement<2>("INSERT INTO test(rowid, name) VALUES (?,?)", database).write(42, "foo");
 }
 
 TEST_F(SqliteDatabase, DatabaseUpdateHookCall)
@@ -313,7 +331,7 @@ TEST_F(SqliteDatabase, DatabaseUpdateHookCall)
 
     EXPECT_CALL(callbackMock, Call(_, StrEq("main"), _, _));
 
-    Sqlite::WriteStatement("INSERT INTO test(name) VALUES (?)", database).write(42);
+    Sqlite::WriteStatement<1>("INSERT INTO test(name) VALUES (?)", database).write(42);
 }
 
 TEST_F(SqliteDatabase, TableUpdateHookCall)
@@ -322,18 +340,21 @@ TEST_F(SqliteDatabase, TableUpdateHookCall)
 
     EXPECT_CALL(callbackMock, Call(_, _, StrEq("test"), _));
 
-    Sqlite::WriteStatement("INSERT INTO test(name) VALUES (?)", database).write(42);
+    Sqlite::WriteStatement<1>("INSERT INTO test(name) VALUES (?)", database).write(42);
 }
 
 TEST_F(SqliteDatabase, SessionsCommit)
 {
     database.setAttachedTables({"test"});
-    Sqlite::WriteStatement("INSERT INTO test(id, name) VALUES (?,?)", database).write(1, "foo");
+    Sqlite::WriteStatement<2>("INSERT INTO test(id, name) VALUES (?,?)", database).write(1, "foo");
+    database.unlock();
 
     Sqlite::ImmediateSessionTransaction transaction{database};
-    Sqlite::WriteStatement("INSERT INTO test(id, name) VALUES (?,?)", database).write(2, "bar");
+    Sqlite::WriteStatement<2>("INSERT INTO test(id, name) VALUES (?,?)", database).write(2, "bar");
     transaction.commit();
-    Sqlite::WriteStatement("INSERT OR REPLACE INTO test(id, name) VALUES (?,?)", database).write(2, "hoo");
+    database.lock();
+    Sqlite::WriteStatement<2>("INSERT OR REPLACE INTO test(id, name) VALUES (?,?)", database)
+        .write(2, "hoo");
     database.applyAndUpdateSessions();
 
     ASSERT_THAT(names(), ElementsAre("foo", "bar"));
@@ -342,13 +363,16 @@ TEST_F(SqliteDatabase, SessionsCommit)
 TEST_F(SqliteDatabase, SessionsRollback)
 {
     database.setAttachedTables({"test"});
-    Sqlite::WriteStatement("INSERT INTO test(id, name) VALUES (?,?)", database).write(1, "foo");
+    Sqlite::WriteStatement<2>("INSERT INTO test(id, name) VALUES (?,?)", database).write(1, "foo");
+    database.unlock();
 
     {
         Sqlite::ImmediateSessionTransaction transaction{database};
-        Sqlite::WriteStatement("INSERT INTO test(id, name) VALUES (?,?)", database).write(2, "bar");
+        Sqlite::WriteStatement<2>("INSERT INTO test(id, name) VALUES (?,?)", database).write(2, "bar");
     }
-    Sqlite::WriteStatement("INSERT OR REPLACE INTO test(id, name) VALUES (?,?)", database).write(2, "hoo");
+    database.lock();
+    Sqlite::WriteStatement<2>("INSERT OR REPLACE INTO test(id, name) VALUES (?,?)", database)
+        .write(2, "hoo");
     database.applyAndUpdateSessions();
 
     ASSERT_THAT(names(), ElementsAre("foo", "hoo"));
